#!/usr/bin/python
# coding: utf-8

import math
import json
import matplotlib
matplotlib.use("TkAgg")
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
from matplotlib.figure import Figure
from tkinter import *
from tkinter.filedialog import askopenfilename, asksaveasfilename, askdirectory
from tkinter import messagebox
import tkinter.ttk as ttk
from PIL import ImageTk
import numpy as np


__version__ = "1.0"
__author__  = "MEDEAS Project"
__date__    = "12/2017"

##############################  Drift_tool Class ##########################
class Plot_tool(Frame):
    
    def __init__(self, master=None):
        Frame.__init__(self, master) # esto define self.master           
        self.master = master
        self.master.option_add('*tearOff', 'FALSE') # menus no detaching
        self.master.title("MEDEAS Plotting tool")
        
        self.eje = None     # axis canvas
        self.figura = None  # canvas
        
        #  Datos
        self.py_dict = {}
        self.scenario_info = 'no scenario info'
        self.time_step_info = 'no time step info'       
        self.return_timestep_info = '0.125'       
        self.py_dict1 = {}
        self.scenario_info1 = 'no scenario info'
        self.time_step_info1 = 'no time step info'       

        # generate funcname: (unit, legend) dictionary from file
        self.legend_by_name = {}
        try:
            with open('legend_by_name.txt', 'r', encoding='utf-8') as f:
                legend_by_name_list = f.readlines()
            for line in legend_by_name_list:
                funcname, unit, legend = [x.strip() for x in line.split('@@')]
                self.legend_by_name[funcname] = (unit, legend)
        except:
            print('legend_by_name.txt cannot be read correctly\n No legends and units generated')
            raise 

        # Default timestep and default time array
        self.time_step = 0.125
        self.return_timestep = 0.125
        self.time = np.arange(start=1995, stop=2050.0001, step=self.time_step)
        
        self.init_window()

    def init_window(self):
                
        self.pack(fill=BOTH, expand=1)
        
        # Variables
        #logo = "medeas.png"
        #img = ImageTk.PhotoImage(file=logo)
        
        with open('./medeas_var.txt', encoding='utf-8') as df: 
            d_keys = [line.strip() for line in df.readlines()]
        # FIN Variables

        menubar = Menu(self.master)
        self.master.config(menu = menubar)
        
        filemenu = Menu(menubar)
        filemenu.add_command(label = "Load", command = self.open_file)
        filemenu.add_command(label = "Save")
        filemenu.add_command(label = "Exit", command = lambda: on_closing(self))
        menubar.add_cascade(label = "File", menu = filemenu)
    
        #tab 1: Experiment configuration
        #panel = Label(self.master, image = img)
        #panel.pack(side = "top",fill = "both", expand = "yes")
        
        group1 = LabelFrame(self, text="Variables", labelanchor="n", font=("Arial",10,"bold"), padx=5, pady=5)
        group1.pack(side="left", fill=Y)
            
        scrollbar = Scrollbar(group1)
        scrollbar.pack(side=RIGHT,fill=Y)
        self.imeis = Listbox(group1, yscrollcommand=scrollbar.set)
        self.imeis.pack(side=LEFT, fill=Y)
        scrollbar.config(command = self.imeis.yview)
        self.imeis.insert(0, *d_keys)
        self.imeis.bind('<<ListboxSelect>>', self.dibuja)

        #Tabulador Gráfico
        group3 = Frame(self)
        group3.pack(side="right")

        fig = Figure(figsize=(200,100))
        plt.rc('legend', fontsize='small')
        self.eje = fig.add_subplot(111)

        self.figura = FigureCanvasTkAgg(fig, master=group3)     # Definimos el canvas y le pasamos fig y el master
        toolbar = NavigationToolbar2TkAgg(self.figura, group3)  # Definimos la barra-widget de navegacion
        self.figura.get_tk_widget().pack()                      # Pack la figura
        toolbar.update()                                        # actualizamos la barra

    def replace_let_by_x(self, stri):
        for let in '\"\'-+\%?&\/=\(\)': stri = stri.replace(let, 'x')
        return stri
        
    def open_file(self):
        ext = (("log","*.*log"),("json","*.*json"),("all files","*.*"))
        filename = askopenfilename(initialdir = "./",title = "Open file", filetypes = ext)
        if filename.split('.')[-1] == "log":
            print("Reading log file...",filename)
            ### If file is vensim like format
            with open(filename, 'r', encoding='utf-8') as f:
                time_data_given = False            
                for line in f.readlines():
                    if line.startswith('#'):  
                        data_info = line[1:].split(',')
                        self.scenario_info, self.time_step_info = data_info[0:2]
                        try:
                            self.return_timestep_info = data_info[2]
                        except:
                            # if no data in header (e.g. in older dataset)
                            self.return_timestep_info = self.time_step_info
                    elif line.lower().startswith('time\t'):
                        self.time_updated = np.array([float(x) for x in line.split('\t')[1:]])
                        time_data_given = True
                    else:
                        s = line.split('\t')
                        try:
                            first_value = float(s[1])
                            py_index = self.replace_let_by_x(s[0].strip().lower())
                            py_index1 = py_index.split('[')
                            py_index1[0] = py_index1[0].replace(' ', '_')
                            py_index = '['.join(py_index1)
                            self.py_dict[py_index] = np.array([float(x) for x in s[1:]])
                        except:
                            #raise
                            continue

                else:
                    for py_index in self.py_dict:
                        time_points = len(self.py_dict[py_index])
                        break
            # compute time array and time_step from the number of points in data
            try:
                self.time_step = float(self.time_step_info)
                self.return_timestep = float(self.return_timestep_info)
            except:
                print('Could not read time step in *log file header, taken default time step = ', self.time_step)
            # for older datasets without time
            if not time_data_given:
                self.time_updated = np.linspace(start=1995, stop=1995+self.return_timestep*time_points, endpoint=True, num=time_points)


        elif filename.split('.')[-1] == "json":
            ### If file is json format
            print("Reading json file...",filename)
            json_data = open(filename).read()
            self.py_dict = json.loads(json_data)
        else:
            print("No file selected")
                        
        return

    dict_correct = {
        'year':'Year',
        'ej':'EJ',
        'tw':'TW',
        'mdollar':'M Dollar',
        'pj':'PJ', 'gj':'GJ',
        'beTWeen':'between',
        'DegreesC':u'Temperature (\u00B0C)',
        'degrees Celsius':u'\u00B0C',
        'degree C':u'\u00B0C',
        'degrees C':u'\u00B0C',
        'yr':'Year',
        'Dmnl':'Dimensionless'
        }

    def define_units(self, varname):
        def adjust_format(name):
            for wrong in self.dict_correct:
                name = name.replace(wrong, self.dict_correct[wrong])
            return name
        dicti = {'ej':'EJ', 'mtoe':'mtoe', 'tw':'TW', 'twh':'TWh', 'mt':'Mt', 'DegreesC': u'Temperature (\u00B0C)'}
        clean_name = varname.split('[')[0].strip(' x')
        ret = None
        for unit in dicti:
            if clean_name.endswith(unit):
                ret = dicti[unit]
                break
        else:
            for name in self.legend_by_name:
                if varname.startswith(name):
                    ret = adjust_format(self.legend_by_name[name][0])
        return ret

    def define_title(self, varname):
        # specific update of the legend
        def adjust_format(name):
            for wrong in self.dict_correct:
                name = name.replace(wrong, self.dict_correct[wrong])
            name = name.replace('\\', '\n').replace('\t', ' ')
            return name.split('.')[0].strip()
        dicti = {}
        clean_name = varname.split('[')[0].strip(' x')
        ret = ''
        for unit in dicti:
            if clean_name.startswith(unit):
                ret = dicti[unit]
                break
        else:
            for name in self.legend_by_name:
                if varname.startswith(name):
                    ret = adjust_format(self.legend_by_name[name][1].split('[')[0]) #.split('(')[0])
        return ret

        
    def dibuja(self, event):
        # If py_dict is empty does nothing
        if not bool(self.py_dict): return
        self.eje.clear()
        column = self.imeis.get(int(self.imeis.curselection()[0]))
        title = self.define_title(column)
        title = title + '\n' + column if title else column
        self.eje.set_title(title)
        self.eje.set_xlabel('Year')
        ylabel = self.define_units(column)
        if ylabel:
            self.eje.set_ylabel(ylabel, rotation='vertical')        
        label_hist, label_proj = ('scenario: ' + self.scenario_info + ', time step: ' + self.time_step_info + ' historical',
            'scenario: ' + self.scenario_info + ', time step: ' + self.time_step_info  + ' projected')
        nt = len(np.where(self.time_updated < 2014)[0])
        try:
            self.eje.plot(self.time_updated[0:nt], self.py_dict[column][0:nt], label=label_hist)
            self.eje.plot(self.time_updated[nt-1:], self.py_dict[column][nt-1:],'--', label=label_proj)
            self.eje.legend()
            self.figura.show()
        except KeyError:
            if not column.lower().startswith('-'):  
                print('No data for variable \"{}\"'.format(column))
        return

######################################  FIN Drift_tool ##########################           
def on_closing(root):
    if messagebox.askokcancel("Quit", "Do you want to quit?"):
        root.destroy()
        exit()
        
def main():
    root = Tk()
    root.geometry("1200x600")
    app = Plot_tool(root)
    root.protocol("WM_DELETE_WINDOW", lambda: on_closing(root))
    root.mainloop()
    
if __name__ == '__main__':
    main()
