"""
Python model model.py
Translated using PySD version 0.8.2
"""
from __future__ import division
import numpy as np
from pysd import utils
import xarray as xr
import inspect


from pysd.py_backend.functions import cache
from pysd.py_backend import functions

from debugutils import interrupt, round_sig, single_precision, less_thr_func, test_nans


equality_threshold = 1e-4

_subscript_dict = {
    'households vehicles':
    ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels'],
    'vehiclet': [
        'hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq',
        'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec'
    ],
    'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
    'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
    'water': ['xcleanx pumped waterx', 'xdistilledx deionized waterx'],
    'materials': [
        'adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement',
        'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx',
        'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass',
        'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime',
        'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment',
        'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel',
        'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead', 'plastics', 'polypropylene',
        'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver',
        'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium',
        'titanium dioxide', 'vanadium', 'wires', 'zinc'
    ],
    'res heat': ['xsolarxheatx', 'xgeotxheatx', 'xsolid bioexheatx'],
    'sectors1': [
        'agriculture hunting forestry and fishing', 'mining and quarrying',
        'food beverages and tobacco', 'textiles and textile products',
        'leather leather and footwear', 'wood and products of woood and cork',
        'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
        'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral',
        'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment',
        'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply',
        'construction',
        'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
        'wholesale trade and commissions trade except of motor vehicles and motorcycles',
        'retail trade except of motor vehicles and motorcycles repair of household goods',
        'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
        'other supporting and auxiliary transport activities activities of travel agencies',
        'post and telecommunications', 'financial intermedation', 'real estate activities',
        'renting od meq and other business activities',
        'public admin and defence compulsory social security', 'education',
        'health and social work', 'other community social and persona services',
        'private households with employed persons'
    ],
    'sectors': [
        'agriculture hunting forestry and fishing', 'mining and quarrying',
        'food beverages and tobacco', 'textiles and textile products',
        'leather leather and footwear', 'wood and products of woood and cork',
        'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
        'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral',
        'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment',
        'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply',
        'construction',
        'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
        'wholesale trade and commissions trade except of motor vehicles and motorcycles',
        'retail trade except of motor vehicles and motorcycles repair of household goods',
        'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
        'other supporting and auxiliary transport activities activities of travel agencies',
        'post and telecommunications', 'financial intermedation', 'real estate activities',
        'renting od meq and other business activities',
        'public admin and defence compulsory social security', 'education',
        'health and social work', 'other community social and persona services',
        'private households with employed persons'
    ],
    'res elec': [
        'hydro', 'xgeotxelecx', 'xsolid bioexelecx', 'oceanic', 'wind onshore', 'wind offshore',
        'solar pv', 'csp'
    ],
    'gtco2 ipcc emissions': ['gtco2e a1', 'gtco2e a2', 'gtco2e b1', 'gtco2e b2'],
    'gtc cumulated': ['gtc a2', 'gtc a1g', 'gtc b1']
}

_namespace = {
    'TIME':
    'time',
    'Time':
    'time',
    'Leontief Matrix':
    'leontief_matrix',
    'IA Matrix':
    'ia_matrix',
    'IA matrix 2002':
    'ia_matrix_2002',
    'IA matrix 2003':
    'ia_matrix_2003',
    'IA matrix 2004':
    'ia_matrix_2004',
    'IA matrix 2005':
    'ia_matrix_2005',
    'IA matrix 2006':
    'ia_matrix_2006',
    'IA matrix 2007':
    'ia_matrix_2007',
    'IA matrix 2008':
    'ia_matrix_2008',
    'IA matrix for Python':
    'ia_matrix_for_python',
    'IA matrix 1995':
    'ia_matrix_1995',
    'IA matrix 1996':
    'ia_matrix_1996',
    'IA matrix 1997':
    'ia_matrix_1997',
    'IA matrix 1998':
    'ia_matrix_1998',
    'IA matrix 1999':
    'ia_matrix_1999',
    'IA matrix 2000':
    'ia_matrix_2000',
    'IA matrix 2001':
    'ia_matrix_2001',
    'IA matrix 2009':
    'ia_matrix_2009',
    'A matrix 2002':
    'a_matrix_2002',
    'A matrix 2003':
    'a_matrix_2003',
    'A matrix 2004':
    'a_matrix_2004',
    'A matrix 2005':
    'a_matrix_2005',
    'A matrix 2006':
    'a_matrix_2006',
    'A matrix 2007':
    'a_matrix_2007',
    'A matrix 2008':
    'a_matrix_2008',
    'A matrix 1995':
    'a_matrix_1995',
    'A matrix 1996':
    'a_matrix_1996',
    'A matrix 1997':
    'a_matrix_1997',
    'A matrix 1998':
    'a_matrix_1998',
    'A matrix 1999':
    'a_matrix_1999',
    'A matrix 2000':
    'a_matrix_2000',
    'A matrix 2001':
    'a_matrix_2001',
    'A matrix 2009':
    'a_matrix_2009',
    'Leontief matrix for Python':
    'leontief_matrix_for_python',
    'Leontief matrix 1996':
    'leontief_matrix_1996',
    'Leontief matrix 1997':
    'leontief_matrix_1997',
    'Leontief matrix 1998':
    'leontief_matrix_1998',
    'Leontief matrix 1999':
    'leontief_matrix_1999',
    'Leontief matrix 2000':
    'leontief_matrix_2000',
    'Leontief matrix 2001':
    'leontief_matrix_2001',
    'Leontief matrix 2002':
    'leontief_matrix_2002',
    'Leontief matrix 2003':
    'leontief_matrix_2003',
    'Leontief matrix 2004':
    'leontief_matrix_2004',
    'Leontief matrix 2005':
    'leontief_matrix_2005',
    'Leontief matrix 2006':
    'leontief_matrix_2006',
    'Leontief matrix 2007':
    'leontief_matrix_2007',
    'Leontief matrix 2008':
    'leontief_matrix_2008',
    'Leontief matrix 2009':
    'leontief_matrix_2009',
    'Leontief matrix 1995':
    'leontief_matrix_1995',
    'replacement batteries':
    'replacement_batteries',
    'share of electric light vehicles':
    'share_of_electric_light_vehicles',
    'Xshare elecXhyb light vehiclesX':
    'xshare_elecxhyb_light_vehiclesx',
    'aux P inland transp':
    'aux_p_inland_transp',
    'T fin inlandT':
    't_fin_inlandt',
    'H elec initial growth':
    'h_elec_initial_growth',
    'T fin H veh':
    't_fin_h_veh',
    'H gas initial growth':
    'h_gas_initial_growth',
    'T ini H veh':
    't_ini_h_veh',
    'H hyb initial growth':
    'h_hyb_initial_growth',
    'policy 2wheels':
    'policy_2wheels',
    'H 2w initial growth':
    'h_2w_initial_growth',
    'var percents H vehicles':
    'var_percents_h_vehicles',
    'increase Households energy final demand for Transp':
    'increase_households_energy_final_demand_for_transp',
    'adapt growth RES elec':
    'adapt_growth_res_elec',
    'GDPpc annual growth SSP2':
    'gdppc_annual_growth_ssp2',
    'Annual GDPpc growth rate':
    'annual_gdppc_growth_rate',
    'variation pop SSP4':
    'variation_pop_ssp4',
    'variation pop SSP5':
    'variation_pop_ssp5',
    'start year P recycling minerals alt techn':
    'start_year_p_recycling_minerals_alt_techn',
    'start year P recycling minerals Rest':
    'start_year_p_recycling_minerals_rest',
    'select pop SSPs':
    'select_pop_ssps',
    'variation pop SSP1':
    'variation_pop_ssp1',
    'variation pop SSP2':
    'variation_pop_ssp2',
    'variation pop SSP3':
    'variation_pop_ssp3',
    'Annual population growth rate':
    'annual_population_growth_rate',
    'improvement recycling rates minerals Rest':
    'improvement_recycling_rates_minerals_rest',
    'variation pop SSPs':
    'variation_pop_ssps',
    'Pop SSP2':
    'pop_ssp2',
    'Pop SSP4':
    'pop_ssp4',
    'Pop SSP1':
    'pop_ssp1',
    'improvement recycling rates minerals alt techn':
    'improvement_recycling_rates_minerals_alt_techn',
    'Pop SSP5':
    'pop_ssp5',
    'Pop SSP3':
    'pop_ssp3',
    'EROImet':
    'eroimet',
    'EROImet FC tot from 2015 1':
    'eroimet_fc_tot_from_2015_1',
    'XActivate EROImet methodXX':
    'xactivate_eroimet_methodxx',
    'EROI FC tot from 2015':
    'eroi_fc_tot_from_2015',
    'XDesired GDP growth affected by real GDPXX':
    'xdesired_gdp_growth_affected_by_real_gdpxx',
    'Dynamic low range FEC good standard of living':
    'dynamic_low_range_fec_good_standard_of_living',
    'Cp EV batteries for elec storage':
    'cp_ev_batteries_for_elec_storage',
    'Dynamic threshold Xhigh developmentX':
    'dynamic_threshold_xhigh_developmentx',
    'max Cp EV batteries for elec storage':
    'max_cp_ev_batteries_for_elec_storage',
    'TFEC intensity until 2009':
    'tfec_intensity_until_2009',
    'Desired annual GDP growth rate':
    'desired_annual_gdp_growth_rate',
    'auxiliary metabolic EROI':
    'auxiliary_metabolic_eroi',
    'Cumulative TFEC intensity change from 2009':
    'cumulative_tfec_intensity_change_from_2009',
    'lifetime EV batteries':
    'lifetime_ev_batteries',
    'Dynamic high range FEC good standard of living':
    'dynamic_high_range_fec_good_standard_of_living',
    'EROI FC tot from 2015 1':
    'eroi_fc_tot_from_2015_1',
    'TFEC per capita':
    'tfec_per_capita',
    'aux15':
    'aux15',
    'Cp EV batteries for Transp':
    'cp_ev_batteries_for_transp',
    'annual GDP growth rate delayed 3yr':
    'annual_gdp_growth_rate_delayed_3yr',
    'annual GDP growth rate delayed 1yr':
    'annual_gdp_growth_rate_delayed_1yr',
    'annual GDP growth rate delayed 2yr':
    'annual_gdp_growth_rate_delayed_2yr',
    'P customized cte GDPpc variation':
    'p_customized_cte_gdppc_variation',
    'Annual growth rate electricity generation RES elec tot':
    'annual_growth_rate_electricity_generation_res_elec_tot',
    'Annual growth rate RES for heat':
    'annual_growth_rate_res_for_heat',
    'P customized year pop evolution':
    'p_customized_year_pop_evolution',
    'Implementatio policy to change final energy':
    'implementatio_policy_to_change_final_energy',
    'Implementatio policy to change final energy H':
    'implementatio_policy_to_change_final_energy_h',
    'Implementation policy to improve energy intensity effciency':
    'implementation_policy_to_improve_energy_intensity_effciency',
    'P timeseries GDPpc growth rate':
    'p_timeseries_gdppc_growth_rate',
    'Desired variation GDPpc':
    'desired_variation_gdppc',
    'select GDPpc evolution input':
    'select_gdppc_evolution_input',
    'select Population evolution input':
    'select_population_evolution_input',
    'P customized year GDPpc evolution':
    'p_customized_year_gdppc_evolution',
    'pop variation':
    'pop_variation',
    'Implementation policy to improve energy intensity effciency H':
    'implementation_policy_to_improve_energy_intensity_effciency_h',
    'FE tot generation all RES elec TWh delayed 1yr':
    'fe_tot_generation_all_res_elec_twh_delayed_1yr',
    'P customized cte pop variation':
    'p_customized_cte_pop_variation',
    'FES RES for heat delayed 1yr':
    'fes_res_for_heat_delayed_1yr',
    'exp slow evol change energy':
    'exp_slow_evol_change_energy',
    'Year to finish policy improve efficiency':
    'year_to_finish_policy_improve_efficiency',
    'exp slow evolution improve efficiency':
    'exp_slow_evolution_improve_efficiency',
    'exp rapid evolution improve efficiency':
    'exp_rapid_evolution_improve_efficiency',
    'exp rapid evol change energy':
    'exp_rapid_evol_change_energy',
    'Year to finish policy change energy':
    'year_to_finish_policy_change_energy',
    'Year to finish policy improve efficiency H':
    'year_to_finish_policy_improve_efficiency_h',
    'Year to finish policy change energy H':
    'year_to_finish_policy_change_energy_h',
    'exp rapid evolution change energy H':
    'exp_rapid_evolution_change_energy_h',
    'Pressure to improve energy intensity efficiency H':
    'pressure_to_improve_energy_intensity_efficiency_h',
    'exp slow evolution change energy H':
    'exp_slow_evolution_change_energy_h',
    'exp rapid evolution improve efficiency H':
    'exp_rapid_evolution_improve_efficiency_h',
    'exp slow evolution improve efficiency H':
    'exp_slow_evolution_improve_efficiency_h',
    'Desired GDP':
    'desired_gdp',
    'Desired GDP delayed 1yr':
    'desired_gdp_delayed_1yr',
    'Exogenous GDP growth rate corrected':
    'exogenous_gdp_growth_rate_corrected',
    'diff annual GDP growth rate':
    'diff_annual_gdp_growth_rate',
    'variation historic GDPpc':
    'variation_historic_gdppc',
    'GDPpc initial year':
    'gdppc_initial_year',
    'Historic GDPpc':
    'historic_gdppc',
    'Desired GDPpc':
    'desired_gdppc',
    'XSIAF feedbackXX':
    'xsiaf_feedbackxx',
    'Energy intensity of households rest':
    'energy_intensity_of_households_rest',
    'Total Radiative Forcing':
    'total_radiative_forcing',
    'Evol final energy intensity by sector and fuel two':
    'evol_final_energy_intensity_by_sector_and_fuel_two',
    'TPES intensity until 2009':
    'tpes_intensity_until_2009',
    'XshareXE losses CC until 2015X':
    'xsharexe_losses_cc_until_2015x',
    'aux13':
    'aux13',
    'aux14':
    'aux14',
    'Xshare EXlosses CC from 2015X':
    'xshare_exlosses_cc_from_2015x',
    'ELF':
    'elf',
    'Cumulative TPES intensity change from 2009':
    'cumulative_tpes_intensity_change_from_2009',
    'Cumulative TFEC intensity change from 2009 without EROI':
    'cumulative_tfec_intensity_change_from_2009_without_eroi',
    'XELF DietzXStern X2015XX':
    'xelf_dietzxstern_x2015xx',
    'Arctic September free of sea ice':
    'arctic_september_free_of_sea_ice',
    'TFEC intensity until 2009 without EROI':
    'tfec_intensity_until_2009_without_eroi',
    'TFEI sectors':
    'tfei_sectors',
    'Annual TFES intensity growth rate without EROI':
    'annual_tfes_intensity_growth_rate_without_eroi',
    'Final energy intensity by sector and fuel':
    'final_energy_intensity_by_sector_and_fuel',
    'Energy intensity of households':
    'energy_intensity_of_households',
    'TFES intensity without EROI delayed 1yr':
    'tfes_intensity_without_eroi_delayed_1yr',
    'Households total final energy demand':
    'households_total_final_energy_demand',
    'required TFED sectors':
    'required_tfed_sectors',
    'share FED households vs sectors':
    'share_fed_households_vs_sectors',
    'TFES intensity EJ Tdolar without EROI':
    'tfes_intensity_ej_tdolar_without_eroi',
    'GHG emissions 2050 MLT2030':
    'ghg_emissions_2050_mlt2030',
    'GHG emissions 2050 MLT2020':
    'ghg_emissions_2050_mlt2020',
    'Cumulative CO2e GHG emissions':
    'cumulative_co2e_ghg_emissions',
    'Xadapt growth RES for heatXcomX':
    'xadapt_growth_res_for_heatxcomx',
    'Xadapt growth RES for heatXncX':
    'xadapt_growth_res_for_heatxncx',
    'Start year P growth RES elec':
    'start_year_p_growth_res_elec',
    'Start year P growth RES heat':
    'start_year_p_growth_res_heat',
    'Total Ce all GHG':
    'total_ce_all_ghg',
    'Labour share growth':
    'labour_share_growth',
    'Year Initial Labour share':
    'year_initial_labour_share',
    'Year Final Labour share':
    'year_final_labour_share',
    'P labour share 2050':
    'p_labour_share_2050',
    'growth labour share':
    'growth_labour_share',
    'Initial Labour share':
    'initial_labour_share',
    'XTotal GHG emissions OLT MEDEAS D3X2X':
    'xtotal_ghg_emissions_olt_medeas_d3x2x',
    'XTotal GHG emissions BAUXCAT MEDEAS D3X2X':
    'xtotal_ghg_emissions_bauxcat_medeas_d3x2x',
    'XTotal GHG emissions MLT2030 MEDEAS D3X2X':
    'xtotal_ghg_emissions_mlt2030_medeas_d3x2x',
    'XTotal GHG emissions MLT2020 MEDEAS D3X2X':
    'xtotal_ghg_emissions_mlt2020_medeas_d3x2x',
    'Low range FEC good standard of living':
    'low_range_fec_good_standard_of_living',
    'Threshold FEC Xhigh developmentX':
    'threshold_fec_xhigh_developmentx',
    'High range FEC good standard of living':
    'high_range_fec_good_standard_of_living',
    'Decrease of intensity due to energy a technology change TOP DOWN':
    'decrease_of_intensity_due_to_energy_a_technology_change_top_down',
    'Maximun yearly aceleration of intensity improvement pct':
    'maximun_yearly_aceleration_of_intensity_improvement_pct',
    'Policy change energy speed':
    'policy_change_energy_speed',
    'Maximun yearly aceleratuin of intensity improvement':
    'maximun_yearly_aceleratuin_of_intensity_improvement',
    'Policy to improve efficiency speed':
    'policy_to_improve_efficiency_speed',
    'inertial rate energy intensity TOP DOWN':
    'inertial_rate_energy_intensity_top_down',
    'minimum fraction':
    'minimum_fraction',
    'Increase of intensity due to energy a technology change TOP DOWN':
    'increase_of_intensity_due_to_energy_a_technology_change_top_down',
    'Year policy change energy':
    'year_policy_change_energy',
    'Max yearly change':
    'max_yearly_change',
    'efficiency rate of substitution':
    'efficiency_rate_of_substitution',
    'Activate BOTTOM UP method':
    'activate_bottom_up_method',
    'Year policy to improve efficiency':
    'year_policy_to_improve_efficiency',
    'Decrease of intensity due to energy a technology change H TOP DOWN':
    'decrease_of_intensity_due_to_energy_a_technology_change_h_top_down',
    'Increase of intensity due to energy a technology change H TOP DOWN':
    'increase_of_intensity_due_to_energy_a_technology_change_h_top_down',
    'Maximun yearly aceleratuin of intensity improvement H':
    'maximun_yearly_aceleratuin_of_intensity_improvement_h',
    'Evol final energy intensity H':
    'evol_final_energy_intensity_h',
    'inertial rate energy intensity H TOP DOWN':
    'inertial_rate_energy_intensity_h_top_down',
    'XTotal CO2e XGWPX100 yearsXX':
    'xtotal_co2e_xgwpx100_yearsxx',
    'Total CO2e all GHG':
    'total_co2e_all_ghg',
    'Net TFEC per capita':
    'net_tfec_per_capita',
    'GWP 100 years CH4':
    'gwp_100_years_ch4',
    'correction factor all GHGs':
    'correction_factor_all_ghgs',
    'real generation RES elec var TWh':
    'real_generation_res_elec_var_twh',
    'XActivate energy scarcity feedbackXX':
    'xactivate_energy_scarcity_feedbackxx',
    'max potential PHS TWh':
    'max_potential_phs_twh',
    'replacement rate PHS':
    'replacement_rate_phs',
    'Energy scarcity feedback shortage coeff':
    'energy_scarcity_feedback_shortage_coeff',
    'real FE elec stored EV batteries TWh':
    'real_fe_elec_stored_ev_batteries_twh',
    'share RES elec generation stored ENDOGENOUS':
    'share_res_elec_generation_stored_endogenous',
    'share tot FEI RES elec var':
    'share_tot_fei_res_elec_var',
    'Cp EV batteries required':
    'cp_ev_batteries_required',
    'output EV bateries for storage over lifetime':
    'output_ev_bateries_for_storage_over_lifetime',
    'discarded batteries':
    'discarded_batteries',
    'ESOI elec storage':
    'esoi_elec_storage',
    'ESOI EV batteries':
    'esoi_ev_batteries',
    'rt elec storage efficiency':
    'rt_elec_storage_efficiency',
    'XXXabundanceXX storageX':
    'xxxabundancexx_storagex',
    'Used EV batteries for elec storage':
    'used_ev_batteries_for_elec_storage',
    'max Cp EV batteries':
    'max_cp_ev_batteries',
    'Total capacity elec storage TW':
    'total_capacity_elec_storage_tw',
    'adapt growth PHS':
    'adapt_growth_phs',
    'demand EV batteries for elec storage':
    'demand_ev_batteries_for_elec_storage',
    'CED per TW over lifetime PHS':
    'ced_per_tw_over_lifetime_phs',
    'remaining potential constraint on new PHS capacity':
    'remaining_potential_constraint_on_new_phs_capacity',
    'new required PHS capacity':
    'new_required_phs_capacity',
    'share FEI for RES vs TFEC':
    'share_fei_for_res_vs_tfec',
    'FEI EV batteries':
    'fei_ev_batteries',
    'output PHS over lifetime':
    'output_phs_over_lifetime',
    'ESOI static PHS':
    'esoi_static_phs',
    'Total FEI RES':
    'total_fei_res',
    'adapt growth RES elec after allocation':
    'adapt_growth_res_elec_after_allocation',
    'replacement capacity RES elec':
    'replacement_capacity_res_elec',
    'ESOI PHS':
    'esoi_phs',
    'P PHS growth':
    'p_phs_growth',
    'constraint elec storage availability':
    'constraint_elec_storage_availability',
    'EROIst tot':
    'eroist_tot',
    'initial instal cap PHS':
    'initial_instal_cap_phs',
    'CEDtot over lifetime PHS':
    'cedtot_over_lifetime_phs',
    'installed capacity PHS TW':
    'installed_capacity_phs_tw',
    'PHS capacity under construction':
    'phs_capacity_under_construction',
    'past PHS capacity growth':
    'past_phs_capacity_growth',
    'new PHS capacity under planning':
    'new_phs_capacity_under_planning',
    'wear PHS':
    'wear_phs',
    'replacement capacity PHS':
    'replacement_capacity_phs',
    'real FE elec stored PHS TWh':
    'real_fe_elec_stored_phs_twh',
    'PHS planned capacity':
    'phs_planned_capacity',
    'Final energy invested PHS':
    'final_energy_invested_phs',
    'remaining potential PHS':
    'remaining_potential_phs',
    'Historic new required capacity PHS':
    'historic_new_required_capacity_phs',
    'table hist capacity PHS':
    'table_hist_capacity_phs',
    'initial capacity in construction PHS':
    'initial_capacity_in_construction_phs',
    'required capacity PHS':
    'required_capacity_phs',
    'initial required capacity PHS':
    'initial_required_capacity_phs',
    'Total FEI over lifetime RES elec dispatch':
    'total_fei_over_lifetime_res_elec_dispatch',
    'min lifetime EV batteries':
    'min_lifetime_ev_batteries',
    'Total transport FED by fuel':
    'total_transport_fed_by_fuel',
    'Total required final energy transport IOT sectors':
    'total_required_final_energy_transport_iot_sectors',
    'Total final energy invested RES elec var':
    'total_final_energy_invested_res_elec_var',
    'materials required for EV batteries Mt':
    'materials_required_for_ev_batteries_mt',
    'XnewXreplaced batteries TWX':
    'xnewxreplaced_batteries_twx',
    'XEnergy intensity construction EV batteries MJXMWX':
    'xenergy_intensity_construction_ev_batteries_mjxmwx',
    'new batteries':
    'new_batteries',
    'Xbatteries EVXhibX2wEX':
    'xbatteries_evxhibx2wex',
    'Xbatteries EVXhibX2wEX2':
    'xbatteries_evxhibx2wex2',
    'kW per MW':
    'kw_per_mw',
    'Net stored energy EV battery over lifetime':
    'net_stored_energy_ev_battery_over_lifetime',
    'Grid correction factor EV batteries':
    'grid_correction_factor_ev_batteries',
    'Share energy requirements for decom EV batteries':
    'share_energy_requirements_for_decom_ev_batteries',
    'Working hours per year':
    'working_hours_per_year',
    'days per year':
    'days_per_year',
    'Total energy required for material consumption for RES elec':
    'total_energy_required_for_material_consumption_for_res_elec',
    'Total energy required for material consumption per RES elec':
    'total_energy_required_for_material_consumption_per_res_elec',
    'MW in 1 year to MJ':
    'mw_in_1_year_to_mj',
    'Working hours per day':
    'working_hours_per_day',
    'Annual work hours for RES':
    'annual_work_hours_for_res',
    'Hours work per GJ RES delivered':
    'hours_work_per_gj_res_delivered',
    'Total energy required for total material consumption for EV batteries':
    'total_energy_required_for_total_material_consumption_for_ev_batteries',
    'Total energy required for total material consumption for alt techn':
    'total_energy_required_for_total_material_consumption_for_alt_techn',
    'Energy required for material consumption per RES elec':
    'energy_required_for_material_consumption_per_res_elec',
    'Total energy required per material for alt techn':
    'total_energy_required_per_material_for_alt_techn',
    'Total jobs RES elec':
    'total_jobs_res_elec',
    'total jobs biofuels':
    'total_jobs_biofuels',
    'Total jobs RES':
    'total_jobs_res',
    'XTotal DXI jobs RES heat per technX':
    'xtotal_dxi_jobs_res_heat_per_technx',
    'Total jobs RES heat':
    'total_jobs_res_heat',
    'XTotal DXI jobs RES elec per technX':
    'xtotal_dxi_jobs_res_elec_per_technx',
    'Ratio total vs D jobs RES heat':
    'ratio_total_vs_d_jobs_res_heat',
    'Ratio total vs D jobs RES elec':
    'ratio_total_vs_d_jobs_res_elec',
    'Employment factor biofuels':
    'employment_factor_biofuels',
    'installed capacity RES elec delayed 1yr':
    'installed_capacity_res_elec_delayed_1yr',
    'XJobs OXM RES heat per technX':
    'xjobs_oxm_res_heat_per_technx',
    'D jobs new installed RES heat per techn':
    'd_jobs_new_installed_res_heat_per_techn',
    'D jobs fuel supply solids bioE':
    'd_jobs_fuel_supply_solids_bioe',
    'new capacity installed growth rate RES elec':
    'new_capacity_installed_growth_rate_res_elec',
    'Employment factor fuel supply solids bioE':
    'employment_factor_fuel_supply_solids_bioe',
    'XActivate EROI tot FC feedback through intensitiesXX':
    'xactivate_eroi_tot_fc_feedback_through_intensitiesxx',
    'test7':
    'test7',
    'Exogenous growth GTL':
    'exogenous_growth_gtl',
    'replacement GTL':
    'replacement_gtl',
    'XCrash programme GTLXX':
    'xcrash_programme_gtlxx',
    'XActivate EROI tot FC feedbackXX':
    'xactivate_eroi_tot_fc_feedbackxx',
    'Exogenous growth CTL':
    'exogenous_growth_ctl',
    'replacement CTL':
    'replacement_ctl',
    'table max extraction ASPO oil EJ010':
    'table_max_extraction_aspo_oil_ej010',
    'aux12':
    'aux12',
    'coal to leave underground':
    'coal_to_leave_underground',
    'RURR coal start year PLG':
    'rurr_coal_start_year_plg',
    'RURR conv oil until start year PLG':
    'rurr_conv_oil_until_start_year_plg',
    'conv gas to leave underground':
    'conv_gas_to_leave_underground',
    'conv oil to leave underground':
    'conv_oil_to_leave_underground',
    'URR oil ASPO 0':
    'urr_oil_aspo_0',
    'URR oil ASPO 0 0':
    'urr_oil_aspo_0_0',
    'URR oil ASPO 0 1':
    'urr_oil_aspo_0_1',
    'RURR unconv oil until start year PLG':
    'rurr_unconv_oil_until_start_year_plg',
    'RURR unconv gas until start year PLG':
    'rurr_unconv_gas_until_start_year_plg',
    'unconv gas to leave underground':
    'unconv_gas_to_leave_underground',
    'tot agg gas to leave underground':
    'tot_agg_gas_to_leave_underground',
    'overdemand EROIst tot':
    'overdemand_eroist_tot',
    'unconv oil to leave underground':
    'unconv_oil_to_leave_underground',
    'aux11':
    'aux11',
    'tot agg oil to leave underground':
    'tot_agg_oil_to_leave_underground',
    'aux6':
    'aux6',
    'aux7':
    'aux7',
    'aux8':
    'aux8',
    'aux9':
    'aux9',
    'RURR conv gas until start year PLG':
    'rurr_conv_gas_until_start_year_plg',
    'aux10':
    'aux10',
    'test ir restando al EROIst tot':
    'test_ir_restando_al_eroist_tot',
    'table max extraction ASPO oil EJ01':
    'table_max_extraction_aspo_oil_ej01',
    'table max extraction ASPO oil EJ0':
    'table_max_extraction_aspo_oil_ej0',
    'table max extraction ASPO oil EJ00':
    'table_max_extraction_aspo_oil_ej00',
    'RURR tot gas until start year PLG':
    'rurr_tot_gas_until_start_year_plg',
    'RURR tot oil until start year PLG':
    'rurr_tot_oil_until_start_year_plg',
    'overdemand EROIst tot from 2015':
    'overdemand_eroist_tot_from_2015',
    'overdemand EROIst tot until 2015':
    'overdemand_eroist_tot_until_2015',
    'Required FED by fuel before heat correction':
    'required_fed_by_fuel_before_heat_correction',
    'aux5':
    'aux5',
    'EROI FC tot from 2015 2':
    'eroi_fc_tot_from_2015_2',
    'share Elec demand covered by RES':
    'share_elec_demand_covered_by_res',
    'RURR conv gas':
    'rurr_conv_gas',
    'RURR unconv gas':
    'rurr_unconv_gas',
    'RURR conv oil':
    'rurr_conv_oil',
    'Xvariation share transmXdistr losses elecX':
    'xvariation_share_transmxdistr_losses_elecx',
    'real PE demand of liquids for Transp EJ':
    'real_pe_demand_of_liquids_for_transp_ej',
    'Share demand solids in transport':
    'share_demand_solids_in_transport',
    'Share demand gas in transport':
    'share_demand_gas_in_transport',
    'Share demand by fuel in transport':
    'share_demand_by_fuel_in_transport',
    'Share demand electricity in transport':
    'share_demand_electricity_in_transport',
    'demand gas for Transp EJ':
    'demand_gas_for_transp_ej',
    'Share demand heat in transport':
    'share_demand_heat_in_transport',
    'Share demand liquids in transport':
    'share_demand_liquids_in_transport',
    'demand liquids for Transp EJ':
    'demand_liquids_for_transp_ej',
    'CO2 emissions per value added':
    'co2_emissions_per_value_added',
    'liquids per X bus':
    'liquids_per_x_bus',
    'liquids per X HV':
    'liquids_per_x_hv',
    'liquids per X LV':
    'liquids_per_x_lv',
    'energy per X train':
    'energy_per_x_train',
    'adjust energy for transport to inland transport':
    'adjust_energy_for_transport_to_inland_transport',
    'Xeffects shortage elec on EVXhibX':
    'xeffects_shortage_elec_on_evxhibx',
    'hist var percent H':
    'hist_var_percent_h',
    'required number standard batteries':
    'required_number_standard_batteries',
    'adjust exogenous GDP growth rate':
    'adjust_exogenous_gdp_growth_rate',
    'Transport TFED energy intensity':
    'transport_tfed_energy_intensity',
    'total number light vehicles':
    'total_number_light_vehicles',
    'Transport TFED':
    'transport_tfed',
    'Transport households final energy demand':
    'transport_households_final_energy_demand',
    'Liquid demand transport EJ':
    'liquid_demand_transport_ej',
    'PE losses related to Elec Transp':
    'pe_losses_related_to_elec_transp',
    'variation LC':
    'variation_lc',
    'variation CC':
    'variation_cc',
    'Increase Elec demand for Transp EJ':
    'increase_elec_demand_for_transp_ej',
    'Total liquid saved by Elec EJ':
    'total_liquid_saved_by_elec_ej',
    'test':
    'test',
    'real FE consumption liquids EJ':
    'real_fe_consumption_liquids_ej',
    'Oil liquids saved by EVs kbd':
    'oil_liquids_saved_by_evs_kbd',
    'Savings PE Transport EJ':
    'savings_pe_transport_ej',
    'increase Elec demand EVs TWh':
    'increase_elec_demand_evs_twh',
    'increase Elec demand for Transp TWh':
    'increase_elec_demand_for_transp_twh',
    'share oil for Elec':
    'share_oil_for_elec',
    'CO2 emissions unconv gas':
    'co2_emissions_unconv_gas',
    'Xreduce oil share for electricityXX':
    'xreduce_oil_share_for_electricityxx',
    'max solar on land Mha':
    'max_solar_on_land_mha',
    'max solar PV on land MHa':
    'max_solar_pv_on_land_mha',
    'max solar PV on land TWe':
    'max_solar_pv_on_land_twe',
    'Total CH4 emissions fossil fuels':
    'total_ch4_emissions_fossil_fuels',
    'max CSP on land MHa':
    'max_csp_on_land_mha',
    'max CSP TWe':
    'max_csp_twe',
    'gCH4 per MJ GTL':
    'gch4_per_mj_gtl',
    'share liquids fot heat plants':
    'share_liquids_fot_heat_plants',
    'CH4 emissions GTL':
    'ch4_emissions_gtl',
    'CH4 emissions CTL':
    'ch4_emissions_ctl',
    'Xreduce oil share for heatXcomXX':
    'xreduce_oil_share_for_heatxcomxx',
    'gCH4 per MJ CTL':
    'gch4_per_mj_ctl',
    'g per Gt':
    'g_per_gt',
    'BioE CO2 emissions':
    'bioe_co2_emissions',
    'maximum 2w':
    'maximum_2w',
    'P H vehicle':
    'p_h_vehicle',
    'EROIst delayed 1yr':
    'eroist_delayed_1yr',
    'EROIst delayed':
    'eroist_delayed',
    'solid biofuels emissions relevant EJ':
    'solid_biofuels_emissions_relevant_ej',
    'XEROIstX1995XX':
    'xeroistx1995xx',
    'EROIst until 2015':
    'eroist_until_2015',
    'aux4':
    'aux4',
    'remaining potential elec storage by RES techn':
    'remaining_potential_elec_storage_by_res_techn',
    'Cp hydro 2015':
    'cp_hydro_2015',
    'ESOI PHS full potential':
    'esoi_phs_full_potential',
    'max capacity elec storage':
    'max_capacity_elec_storage',
    'CC impacts feedback shortage coeff':
    'cc_impacts_feedback_shortage_coeff',
    'XEOLXRR minerals alt techn RES vsX total economyX':
    'xeolxrr_minerals_alt_techn_res_vsx_total_economyx',
    'XCrash programme CTLXX':
    'xcrash_programme_ctlxx',
    'recycling rates minerals Rest':
    'recycling_rates_minerals_rest',
    'recycling rates minerals alt techn':
    'recycling_rates_minerals_alt_techn',
    'P elec':
    'p_elec',
    'percents H vehicles':
    'percents_h_vehicles',
    'T hist H transp':
    't_hist_h_transp',
    'bat number 2w':
    'bat_number_2w',
    'bateries ratio 2w E':
    'bateries_ratio_2w_e',
    'bat number EV':
    'bat_number_ev',
    'bat number hib':
    'bat_number_hib',
    'bateries ratio bus E':
    'bateries_ratio_bus_e',
    'bateries ratio hib bus':
    'bateries_ratio_hib_bus',
    'bateries ratio hib HV':
    'bateries_ratio_hib_hv',
    'bateries ratio hib LV':
    'bateries_ratio_hib_lv',
    'total number hybrid light vehicles':
    'total_number_hybrid_light_vehicles',
    'total number elec light vehicles':
    'total_number_elec_light_vehicles',
    'total number gas light vehicles':
    'total_number_gas_light_vehicles',
    'EV batteries TW':
    'ev_batteries_tw',
    'var percent T vehicles':
    'var_percent_t_vehicles',
    'P train elec':
    'p_train_elec',
    'P HV gas':
    'p_hv_gas',
    'T ini inlandT':
    't_ini_inlandt',
    'P inlandT':
    'p_inlandt',
    'Activate policy inlandT':
    'activate_policy_inlandt',
    'P LV elec':
    'p_lv_elec',
    'P LV hyb':
    'p_lv_hyb',
    'P bus elec':
    'p_bus_elec',
    'P bus gas':
    'p_bus_gas',
    'P bus hyb':
    'p_bus_hyb',
    'percents bus':
    'percents_bus',
    'percents train':
    'percents_train',
    'P HV hyb':
    'p_hv_hyb',
    'adapt var inlandT':
    'adapt_var_inlandt',
    'percents LV':
    'percents_lv',
    'var I inland Elec':
    'var_i_inland_elec',
    'var I inlandT Gas':
    'var_i_inlandt_gas',
    'var I inlandT liq':
    'var_i_inlandt_liq',
    'shares available T':
    'shares_available_t',
    'vehicles inlandT':
    'vehicles_inlandt',
    'percent T vehicles':
    'percent_t_vehicles',
    'Efects shortage inlandT':
    'efects_shortage_inlandt',
    'P LV gas':
    'p_lv_gas',
    'percents HV':
    'percents_hv',
    'initial Xt inland':
    'initial_xt_inland',
    'NX bus inlandT':
    'nx_bus_inlandt',
    'NX HV inland T':
    'nx_hv_inland_t',
    'NX LV inland T':
    'nx_lv_inland_t',
    'N vehicles inland t 0':
    'n_vehicles_inland_t_0',
    'Energy initial inland transport':
    'energy_initial_inland_transport',
    'energy per X t':
    'energy_per_x_t',
    'NX train inland T':
    'nx_train_inland_t',
    'A2 coef tH':
    'a2_coef_th',
    'H gas adapt growth':
    'h_gas_adapt_growth',
    'H hyb adapt growth':
    'h_hyb_adapt_growth',
    'Activate policy H transp':
    'activate_policy_h_transp',
    'Initial energy intensity of households transport 2009':
    'initial_energy_intensity_of_households_transport_2009',
    'H EV adapt growth':
    'h_ev_adapt_growth',
    'var IH E2':
    'var_ih_e2',
    'var IH gas2':
    'var_ih_gas2',
    'A1 coef tH':
    'a1_coef_th',
    'var IH liq2':
    'var_ih_liq2',
    'percent H vehicles initial':
    'percent_h_vehicles_initial',
    'Energy intensity of households transport':
    'energy_intensity_of_households_transport',
    'H 2wE adapt growth':
    'h_2we_adapt_growth',
    'Liq 4w 0':
    'liq_4w_0',
    'percent 2w liq 0':
    'percent_2w_liq_0',
    'percent 4w liq 0':
    'percent_4w_liq_0',
    'Electricity 2wE 0':
    'electricity_2we_0',
    'N vehicles H 0':
    'n_vehicles_h_0',
    'Demand H 0':
    'demand_h_0',
    'ratio N veh Demand H':
    'ratio_n_veh_demand_h',
    'percents 2w H vehicles':
    'percents_2w_h_vehicles',
    'percents 4w H vehicles':
    'percents_4w_h_vehicles',
    'P 2wE':
    'p_2we',
    'max percent 2 wheels':
    'max_percent_2_wheels',
    'max percent 4 wheels':
    'max_percent_4_wheels',
    'aux hist H':
    'aux_hist_h',
    'P gas':
    'p_gas',
    'P share 2 wheelers':
    'p_share_2_wheelers',
    'P hyb':
    'p_hyb',
    'initial 2w percent':
    'initial_2w_percent',
    'share available 2w':
    'share_available_2w',
    'rate 4w to 2w':
    'rate_4w_to_2w',
    'share FEH over FED by final fuel':
    'share_feh_over_fed_by_final_fuel',
    'share FEH over FED oil':
    'share_feh_over_fed_oil',
    'share FEH over FED coal':
    'share_feh_over_fed_coal',
    'Xshare FEH over FED natX gasX':
    'xshare_feh_over_fed_natx_gasx',
    'XFED coal for heatXncX':
    'xfed_coal_for_heatxncx',
    'PES oil EJ delayed':
    'pes_oil_ej_delayed',
    'Energy required for material consumption for EV batteries':
    'energy_required_for_material_consumption_for_ev_batteries',
    'area solar collectors':
    'area_solar_collectors',
    'Households final energy demand':
    'households_final_energy_demand',
    'change total intensity to rest':
    'change_total_intensity_to_rest',
    'Activate transport H BOTTOM UP method':
    'activate_transport_h_bottom_up_method',
    'variation energy intensity of households transport':
    'variation_energy_intensity_of_households_transport',
    'Number vehicles H':
    'number_vehicles_h',
    'effects shortage elec on EV':
    'effects_shortage_elec_on_ev',
    'effects shortage gas H veh':
    'effects_shortage_gas_h_veh',
    'XXaX extraction projection mineralsX':
    'xxax_extraction_projection_mineralsx',
    'XXbX extraction projection mineralsX':
    'xxbx_extraction_projection_mineralsx',
    'variation minerals extraction Rest':
    'variation_minerals_extraction_rest',
    'Total recycled materials for other Mt':
    'total_recycled_materials_for_other_mt',
    'share minerals consumption alt techn vs total economy':
    'share_minerals_consumption_alt_techn_vs_total_economy',
    'XLabor share cteXX':
    'xlabor_share_ctexx',
    'initial minerals extraction Rest':
    'initial_minerals_extraction_rest',
    'Minerals extraction projection Rest cte rr':
    'minerals_extraction_projection_rest_cte_rr',
    'Materials to extract Rest Mt':
    'materials_to_extract_rest_mt',
    'Historical extraction minerals Rest SUBAdhesive':
    'historical_extraction_minerals_rest_subadhesive',
    'Historical extraction minerals Rest SUBAluminium':
    'historical_extraction_minerals_rest_subaluminium',
    'Historical extraction minerals Rest SUBAluminiummirrors':
    'historical_extraction_minerals_rest_subaluminiummirrors',
    'Historical extraction minerals Rest SUBCadmium':
    'historical_extraction_minerals_rest_subcadmium',
    'Historical extraction minerals Rest SUBCarbonfiber':
    'historical_extraction_minerals_rest_subcarbonfiber',
    'Historical extraction minerals Rest SUBCement':
    'historical_extraction_minerals_rest_subcement',
    'Historical extraction minerals Rest SUBChromium':
    'historical_extraction_minerals_rest_subchromium',
    'Historical extraction minerals Rest SUBCopper':
    'historical_extraction_minerals_rest_subcopper',
    'Historical extraction minerals Rest SUBdiesel':
    'historical_extraction_minerals_rest_subdiesel',
    'Historical extraction minerals Rest SUBDy':
    'historical_extraction_minerals_rest_subdy',
    'Historical extraction minerals Rest SUBXElectricXelectroniccomponentsX':
    'historical_extraction_minerals_rest_subxelectricxelectroniccomponentsx',
    'Historical extraction minerals Rest SUBEvacuationlines':
    'historical_extraction_minerals_rest_subevacuationlines',
    'Historical extraction minerals Rest SUBFiberglass':
    'historical_extraction_minerals_rest_subfiberglass',
    'Historical extraction minerals Rest SUBFoamglass':
    'historical_extraction_minerals_rest_subfoamglass',
    'Historical extraction minerals Rest SUBGalium':
    'historical_extraction_minerals_rest_subgalium',
    'Historical extraction minerals Rest SUBGlass':
    'historical_extraction_minerals_rest_subglass',
    'Historical extraction minerals Rest SUBGlassreinforcingplastic':
    'historical_extraction_minerals_rest_subglassreinforcingplastic',
    'Historical extraction minerals Rest SUBgravel':
    'historical_extraction_minerals_rest_subgravel',
    'Historical extraction minerals Rest SUBIndium':
    'historical_extraction_minerals_rest_subindium',
    'Historical extraction minerals Rest SUBIron':
    'historical_extraction_minerals_rest_subiron',
    'Historical extraction minerals Rest SUBKNO3mined':
    'historical_extraction_minerals_rest_subkno3mined',
    'Historical extraction minerals Rest SUBAsphalt':
    'historical_extraction_minerals_rest_subasphalt',
    'Historical extraction minerals Rest SUBLime':
    'historical_extraction_minerals_rest_sublime',
    'Historical extraction minerals Rest SUBLimestone':
    'historical_extraction_minerals_rest_sublimestone',
    'Historical extraction minerals Rest SUBLithium':
    'historical_extraction_minerals_rest_sublithium',
    'Historical extraction minerals Rest SUBLubricant':
    'historical_extraction_minerals_rest_sublubricant',
    'Historical extraction minerals Rest SUBMagnesium':
    'historical_extraction_minerals_rest_submagnesium',
    'Historical extraction minerals Rest SUBManganese':
    'historical_extraction_minerals_rest_submanganese',
    'Historical extraction minerals Rest SUBHeavyequipment':
    'historical_extraction_minerals_rest_subheavyequipment',
    'Historical extraction minerals Rest SUBConcrete':
    'historical_extraction_minerals_rest_subconcrete',
    'Historical extraction minerals Rest SUBMolybdenum':
    'historical_extraction_minerals_rest_submolybdenum',
    'Historical extraction minerals Rest SUBNaNO3mined':
    'historical_extraction_minerals_rest_subnano3mined',
    'Historical extraction minerals Rest SUBNaNO3synthetic':
    'historical_extraction_minerals_rest_subnano3synthetic',
    'Historical extraction minerals Rest SUBNeodymium':
    'historical_extraction_minerals_rest_subneodymium',
    'Historical extraction minerals Rest SUBNickel':
    'historical_extraction_minerals_rest_subnickel',
    'Historical extraction minerals Rest SUBXOvergridX15XXX':
    'historical_extraction_minerals_rest_subxovergridx15xxx',
    'Historical extraction minerals Rest SUBXOvergridX5XXX':
    'historical_extraction_minerals_rest_subxovergridx5xxx',
    'Historical extraction minerals Rest SUBPaint':
    'historical_extraction_minerals_rest_subpaint',
    'Historical extraction minerals Rest SUBLead':
    'historical_extraction_minerals_rest_sublead',
    'Historical extraction minerals Rest SUBPlastics':
    'historical_extraction_minerals_rest_subplastics',
    'Historical extraction minerals Rest SUBPolypropylene':
    'historical_extraction_minerals_rest_subpolypropylene',
    'Historical extraction minerals Rest SUBRock':
    'historical_extraction_minerals_rest_subrock',
    'Historical extraction minerals Rest SUBRockwool':
    'historical_extraction_minerals_rest_subrockwool',
    'Historical extraction minerals Rest SUBSand':
    'historical_extraction_minerals_rest_subsand',
    'Historical extraction minerals Rest SUBSiliconsand':
    'historical_extraction_minerals_rest_subsiliconsand',
    'Historical extraction minerals Rest SUBSiliconwafermodules':
    'historical_extraction_minerals_rest_subsiliconwafermodules',
    'Historical extraction minerals Rest SUBSilver':
    'historical_extraction_minerals_rest_subsilver',
    'Historical extraction minerals Rest SUBSitepreparation':
    'historical_extraction_minerals_rest_subsitepreparation',
    'Historical extraction minerals Rest SUBTin':
    'historical_extraction_minerals_rest_subtin',
    'Historical extraction minerals Rest SUBsodaash':
    'historical_extraction_minerals_rest_subsodaash',
    'Historical extraction minerals Rest SUBsteel':
    'historical_extraction_minerals_rest_substeel',
    'Historical extraction minerals Rest SUBsyntheticoil':
    'historical_extraction_minerals_rest_subsyntheticoil',
    'Historical extraction minerals Rest SUBtellurium':
    'historical_extraction_minerals_rest_subtellurium',
    'Historical extraction minerals Rest SUBtitanium':
    'historical_extraction_minerals_rest_subtitanium',
    'Historical extraction minerals Rest SUBtitaniumdioxide':
    'historical_extraction_minerals_rest_subtitaniumdioxide',
    'Historical extraction minerals Rest SUBvanadium':
    'historical_extraction_minerals_rest_subvanadium',
    'Historical extraction minerals Rest SUBwires':
    'historical_extraction_minerals_rest_subwires',
    'Historical extraction minerals Rest SUBzinc':
    'historical_extraction_minerals_rest_subzinc',
    'Historical variation minerals extraction Rest':
    'historical_variation_minerals_extraction_rest',
    'Minerals extraction projection Rest with rr':
    'minerals_extraction_projection_rest_with_rr',
    'Minerals consumption estimation Rest cte rr':
    'minerals_consumption_estimation_rest_cte_rr',
    'XTotal materials required for RES elec X EV batteries MtX':
    'xtotal_materials_required_for_res_elec_x_ev_batteries_mtx',
    'Xvariation nonXenergy useX':
    'xvariation_nonxenergy_usex',
    'Number 2w':
    'number_2w',
    'Number 4w':
    'number_4w',
    'Number all':
    'number_all',
    'sum 4w shares':
    'sum_4w_shares',
    'share available 4w':
    'share_available_4w',
    'percent 4w':
    'percent_4w',
    'saving ratio 2wE':
    'saving_ratio_2we',
    'percent all':
    'percent_all',
    'percent 2w':
    'percent_2w',
    'Xshare FED coal vs NRE heatXncX':
    'xshare_fed_coal_vs_nre_heatxncx',
    'Xshare FED gas vs NRE heatXncX':
    'xshare_fed_gas_vs_nre_heatxncx',
    'XFED NRE for heatXncX':
    'xfed_nre_for_heatxncx',
    'Xshare FED liquids vs NRE heatXncX':
    'xshare_fed_liquids_vs_nre_heatxncx',
    'Required FED by fuel':
    'required_fed_by_fuel',
    'BioE gen land marg available':
    'bioe_gen_land_marg_available',
    'Potential PEavail cellulosic biofuel EJ':
    'potential_peavail_cellulosic_biofuel_ej',
    'Potential PEavail total biofuels':
    'potential_peavail_total_biofuels',
    'Biofuels 3gen land compet available':
    'biofuels_3gen_land_compet_available',
    'FES total biofuels production EJ':
    'fes_total_biofuels_production_ej',
    'FES total biofuels production EJ 2':
    'fes_total_biofuels_production_ej_2',
    'XFES total biofuels production MbXdX':
    'xfes_total_biofuels_production_mbxdx',
    'PEavail biofuels 2gen land compet EJ':
    'peavail_biofuels_2gen_land_compet_ej',
    'PEavail biofuels 3gen land compet EJ':
    'peavail_biofuels_3gen_land_compet_ej',
    'PEavail biofuels land marg EJ':
    'peavail_biofuels_land_marg_ej',
    'PE biofuels land marg EJ':
    'pe_biofuels_land_marg_ej',
    'XPE biofuels prod 2genX3gen EJX':
    'xpe_biofuels_prod_2genx3gen_ejx',
    'PE biomass for biofuels production EJ':
    'pe_biomass_for_biofuels_production_ej',
    'Xnew BioE residues for heatXelecX':
    'xnew_bioe_residues_for_heatxelecx',
    'PE cellulosic biofuel EJ':
    'pe_cellulosic_biofuel_ej',
    'new biofuels land marg':
    'new_biofuels_land_marg',
    'Max PEavail biofuels potential':
    'max_peavail_biofuels_potential',
    'start year biofuels land marg':
    'start_year_biofuels_land_marg',
    'Land required biofuels land marg':
    'land_required_biofuels_land_marg',
    'Max PEavail potential biofuels marginal lands':
    'max_peavail_potential_biofuels_marginal_lands',
    'share biofuels overcapacity':
    'share_biofuels_overcapacity',
    'Total land requirements renew Mha':
    'total_land_requirements_renew_mha',
    'Land occupation ratio biofuels marg land':
    'land_occupation_ratio_biofuels_marg_land',
    'new cellulosic biofuels':
    'new_cellulosic_biofuels',
    'PEavail cellulosic biofuel EJ':
    'peavail_cellulosic_biofuel_ej',
    'Potential PEavail biofuels land marg EJ':
    'potential_peavail_biofuels_land_marg_ej',
    'BioE potential NPP marginal lands':
    'bioe_potential_npp_marginal_lands',
    'Oil saved by biofuels Mtoe':
    'oil_saved_by_biofuels_mtoe',
    'start production biofuels':
    'start_production_biofuels',
    'Additional PE production of bioenergy for biofuels':
    'additional_pe_production_of_bioenergy_for_biofuels',
    'Oil liquids saved by biofuels EJ':
    'oil_liquids_saved_by_biofuels_ej',
    'Efficiency bioE residues to cellulosic liquids':
    'efficiency_bioe_residues_to_cellulosic_liquids',
    'Conv efficiency from NPP to biofuels':
    'conv_efficiency_from_npp_to_biofuels',
    'PES heat RES':
    'pes_heat_res',
    'PED NRE Liquids':
    'ped_nre_liquids',
    'XPotential FES CTLXGTL EJX':
    'xpotential_fes_ctlxgtl_ejx',
    'TFEC RES delayed 1yr':
    'tfec_res_delayed_1yr',
    'TFEC RES EJ':
    'tfec_res_ej',
    'XPE supply from RES nonXelec without trad bioE EJX':
    'xpe_supply_from_res_nonxelec_without_trad_bioe_ejx',
    'PES gases':
    'pes_gases',
    'XFES CTLXGTL EJX':
    'xfes_ctlxgtl_ejx',
    'Xactivate ELF all scenXX':
    'xactivate_elf_all_scenxx',
    'TPES RES delayed 1yr':
    'tpes_res_delayed_1yr',
    'Annual TPES RES growth rate':
    'annual_tpes_res_growth_rate',
    'XPED natX gas EJX':
    'xped_natx_gas_ejx',
    'Annual share RES vs TFEC growth rate':
    'annual_share_res_vs_tfec_growth_rate',
    'Annual share RES vs TPES growth rate':
    'annual_share_res_vs_tpes_growth_rate',
    'share RES vs TPES delayed 1yr':
    'share_res_vs_tpes_delayed_1yr',
    'GTL production':
    'gtl_production',
    'PES biogas for TFC':
    'pes_biogas_for_tfc',
    'Xshare CTLXGTL overcapacityX':
    'xshare_ctlxgtl_overcapacityx',
    'share RES vs TFEC delayed 1yr':
    'share_res_vs_tfec_delayed_1yr',
    'Xshare EXlosses CCX':
    'xshare_exlosses_ccx',
    'XPED natX gas for GTL EJX':
    'xped_natx_gas_for_gtl_ejx',
    'max unconv gas growth extraction':
    'max_unconv_gas_growth_extraction',
    'PED coal for CTL EJ':
    'ped_coal_for_ctl_ej',
    'Annual TFEC RES growth rate':
    'annual_tfec_res_growth_rate',
    'PED total oil EJ':
    'ped_total_oil_ej',
    'CTL production':
    'ctl_production',
    'CH4 emissions coal without CTL':
    'ch4_emissions_coal_without_ctl',
    'PES tot RES for heat':
    'pes_tot_res_for_heat',
    'XTotal FE real supply RES for heatXnc EJX':
    'xtotal_fe_real_supply_res_for_heatxnc_ejx',
    'real extraction conv oil emissions relevant EJ':
    'real_extraction_conv_oil_emissions_relevant_ej',
    'Xnew RES capacity for heatXnc TWX':
    'xnew_res_capacity_for_heatxnc_twx',
    'CO2 emissions coal without CTL':
    'co2_emissions_coal_without_ctl',
    'XAnnual variation nonXenergy useX':
    'xannual_variation_nonxenergy_usex',
    'real extraction conv gas emissions relevant EJ':
    'real_extraction_conv_gas_emissions_relevant_ej',
    'XPES RES for heatXnc by technX':
    'xpes_res_for_heatxnc_by_technx',
    'FES NRE for heat':
    'fes_nre_for_heat',
    'Xinstalled capacity RES heatXnc TWX':
    'xinstalled_capacity_res_heatxnc_twx',
    'PES solids bioE EJ':
    'pes_solids_bioe_ej',
    'XFED HeatXcom plants fossil fuels EJX':
    'xfed_heatxcom_plants_fossil_fuels_ejx',
    'real extraction unconv gas emissions relevant EJ':
    'real_extraction_unconv_gas_emissions_relevant_ej',
    'real extraction unconv oil emissions relevant EJ':
    'real_extraction_unconv_oil_emissions_relevant_ej',
    'available PE potential solid bioE for elec EJ':
    'available_pe_potential_solid_bioe_for_elec_ej',
    'remaining potential RES for heat':
    'remaining_potential_res_for_heat',
    'extraction coal emissions relevant EJ':
    'extraction_coal_emissions_relevant_ej',
    'Xwear RES capacity for heatXnc TWX':
    'xwear_res_capacity_for_heatxnc_twx',
    'Xpotential FES RES for heatXnc TWhX':
    'xpotential_fes_res_for_heatxnc_twhx',
    'abundance RES heat2 0':
    'abundance_res_heat2_0',
    'XFE real generation RES heatXnc EJX':
    'xfe_real_generation_res_heatxnc_ejx',
    'XTotal FE real supply RES for heatXcom EJX':
    'xtotal_fe_real_supply_res_for_heatxcom_ejx',
    'Xnew RES capacity for heatXcom TW 0X':
    'xnew_res_capacity_for_heatxcom_tw_0x',
    'initial value RES for heat 0':
    'initial_value_res_for_heat_0',
    'share coal Heat EJ':
    'share_coal_heat_ej',
    'Xpast RES growth for heatXcom 0X':
    'xpast_res_growth_for_heatxcom_0x',
    'PES RES for heat by techn 0':
    'pes_res_for_heat_by_techn_0',
    'installed capacity RES heat TW 0':
    'installed_capacity_res_heat_tw_0',
    'share renewables Heat EJ':
    'share_renewables_heat_ej',
    'XFE real generation RES heatXcom EJ 0X':
    'xfe_real_generation_res_heatxcom_ej_0x',
    'remaining potential constraint on new RES heat capacity 0':
    'remaining_potential_constraint_on_new_res_heat_capacity_0',
    'Efficiency conversion BioE plants to heat 0':
    'efficiency_conversion_bioe_plants_to_heat_0',
    'XFED HeatXcom after priorities EJX':
    'xfed_heatxcom_after_priorities_ejx',
    'FES heat from BioW':
    'fes_heat_from_biow',
    'share fossil fuel CHP EJ':
    'share_fossil_fuel_chp_ej',
    'adapt growth RES for heat 0':
    'adapt_growth_res_for_heat_0',
    'Xabundance RES heatXncX':
    'xabundance_res_heatxncx',
    'remaining potential RES for heat 0':
    'remaining_potential_res_for_heat_0',
    'Efficiency geothermal for heat 0':
    'efficiency_geothermal_for_heat_0',
    'XFES HeatXcom nuclear CHP plants EJX':
    'xfes_heatxcom_nuclear_chp_plants_ejx',
    'XFE real supply RES for heatXnc tot EJX':
    'xfe_real_supply_res_for_heatxnc_tot_ejx',
    'RES heat tot overcapacity 0':
    'res_heat_tot_overcapacity_0',
    'Xinitial value RES for heatXncX':
    'xinitial_value_res_for_heatxncx',
    'share gases Heat EJ':
    'share_gases_heat_ej',
    'Xpast RES growth for heatXncX':
    'xpast_res_growth_for_heatxncx',
    'wear RES capacity for heat TW 0':
    'wear_res_capacity_for_heat_tw_0',
    'P solid bioE for heat 0':
    'p_solid_bioe_for_heat_0',
    'FED heat fossil fuels CHP plants EJ':
    'fed_heat_fossil_fuels_chp_plants_ej',
    'Xreplacement RES for heatXncX':
    'xreplacement_res_for_heatxncx',
    'potential FES RES for heat EJ 0':
    'potential_fes_res_for_heat_ej_0',
    'Efficiency solar panels for heat 0':
    'efficiency_solar_panels_for_heat_0',
    'Cp RES for heat 0':
    'cp_res_for_heat_0',
    'Xreplacement RES for heatXnc TWX':
    'xreplacement_res_for_heatxnc_twx',
    'P RES for heat 0':
    'p_res_for_heat_0',
    'abundance RES heat 0':
    'abundance_res_heat_0',
    'P solar for heat 0':
    'p_solar_for_heat_0',
    'Xabundance RES heatXcomX':
    'xabundance_res_heatxcomx',
    'Xabundance RES heatXnc2X':
    'xabundance_res_heatxnc2x',
    'Xpotential FES tot RES for heatXnc EJX':
    'xpotential_fes_tot_res_for_heatxnc_ejx',
    'life time RES for heat 0':
    'life_time_res_for_heat_0',
    'Efficiency RES heat 0':
    'efficiency_res_heat_0',
    'XHistoric RES capacity for heatXcom 0X SUBXsolarXheatX':
    'xhistoric_res_capacity_for_heatxcom_0x_subxsolarxheatx',
    'XHistoric RES capacity for heatXcom 0X SUBXgeotXheatX':
    'xhistoric_res_capacity_for_heatxcom_0x_subxgeotxheatx',
    'XHistoric RES capacity for heatXcom 0X SUBXsolidbioEXheatX':
    'xhistoric_res_capacity_for_heatxcom_0x_subxsolidbioexheatx',
    'replacement RES for heat 0':
    'replacement_res_for_heat_0',
    'P geothermal for heat 0':
    'p_geothermal_for_heat_0',
    'FES RES for heat EJ':
    'fes_res_for_heat_ej',
    'XFE real supply RES for heatXcom tot EJX':
    'xfe_real_supply_res_for_heatxcom_tot_ejx',
    'share liquids Heat EJ':
    'share_liquids_heat_ej',
    'Losses solar for heat 0':
    'losses_solar_for_heat_0',
    'potential FES tot RES for heat EJ 0':
    'potential_fes_tot_res_for_heat_ej_0',
    'potential FES RES for heat TWh 0':
    'potential_fes_res_for_heat_twh_0',
    'FE real supply RES for heat tot EJ 0':
    'fe_real_supply_res_for_heat_tot_ej_0',
    'share nuclear Heat EJ':
    'share_nuclear_heat_ej',
    'XRES heatXnc tot overcapacityX':
    'xres_heatxnc_tot_overcapacityx',
    'Xpotential FES RES for heatXnc EJX':
    'xpotential_fes_res_for_heatxnc_ejx',
    'XHistoric RES capacity for heatXncX SUBXsolarXheatX':
    'xhistoric_res_capacity_for_heatxncx_subxsolarxheatx',
    'XHistoric RES capacity for heatXncX SUBXgeotXheatX':
    'xhistoric_res_capacity_for_heatxncx_subxgeotxheatx',
    'XHistoric RES capacity for heatXncX SUBXsolidbioEXheatX':
    'xhistoric_res_capacity_for_heatxncx_subxsolidbioexheatx',
    'replacement RES for heat TW 0':
    'replacement_res_for_heat_tw_0',
    'Historic share of transformation losses vs extraction SUBliquids':
    'historic_share_of_transformation_losses_vs_extraction_subliquids',
    'Historic share of transformation losses vs extraction SUBsolids':
    'historic_share_of_transformation_losses_vs_extraction_subsolids',
    'Other solids required':
    'other_solids_required',
    'Ratio gain gas vs lose solids in tranf processes':
    'ratio_gain_gas_vs_lose_solids_in_tranf_processes',
    'Other gases required':
    'other_gases_required',
    'Transformation Losses EJ':
    'transformation_losses_ej',
    'FES Elec fossil fuel CHP plants EJ':
    'fes_elec_fossil_fuel_chp_plants_ej',
    'Historic GDP WIOD':
    'historic_gdp_wiod',
    'check GDP':
    'check_gdp',
    'Total FE Elec demand after priorities TWh':
    'total_fe_elec_demand_after_priorities_twh',
    'Demand Elec NRE TWh':
    'demand_elec_nre_twh',
    'XNonXenergy use demand by final fuel EJX':
    'xnonxenergy_use_demand_by_final_fuel_ejx',
    'share solids for final energy':
    'share_solids_for_final_energy',
    'real FE consumption gases EJ':
    'real_fe_consumption_gases_ej',
    'real FE consumption solids EJ':
    'real_fe_consumption_solids_ej',
    'XTotal real nonXenergy use consumption EJX':
    'xtotal_real_nonxenergy_use_consumption_ejx',
    'share gases for final energy':
    'share_gases_for_final_energy',
    'Xratio FED for heatXnc vs FED for heatXcomX':
    'xratio_fed_for_heatxnc_vs_fed_for_heatxcomx',
    'real FE consumption by fuel before heat correction':
    'real_fe_consumption_by_fuel_before_heat_correction',
    'Required TFED before heat dem corr':
    'required_tfed_before_heat_dem_corr',
    'cum materials to extract for EV batteries':
    'cum_materials_to_extract_for_ev_batteries',
    'share cum dem materials to extract alt techn vs total':
    'share_cum_dem_materials_to_extract_alt_techn_vs_total',
    'Total recycled materials for EV batteries Mt':
    'total_recycled_materials_for_ev_batteries_mt',
    'share RES vs TFEC':
    'share_res_vs_tfec',
    'cum materials to extract for alt techn from 2015':
    'cum_materials_to_extract_for_alt_techn_from_2015',
    'Total materials to extract for EV batteries from 2015 Mt':
    'total_materials_to_extract_for_ev_batteries_from_2015_mt',
    'Xmaterials per new capacity installed X EV batteriesX':
    'xmaterials_per_new_capacity_installed_x_ev_batteriesx',
    'Total materials to extract for EV batteries Mt':
    'total_materials_to_extract_for_ev_batteries_mt',
    'Total materials to extract Mt':
    'total_materials_to_extract_mt',
    'cum materials requirements for EV batteries':
    'cum_materials_requirements_for_ev_batteries',
    'XXstaticX EROIgrid RES elecX':
    'xxstaticx_eroigrid_res_elecx',
    'T per x':
    't_per_x',
    'Total materials required for EV batteries':
    'total_materials_required_for_ev_batteries',
    'TFEC from RES per capita':
    'tfec_from_res_per_capita',
    'share materials cum demand to extract vs reserves for RES elec':
    'share_materials_cum_demand_to_extract_vs_reserves_for_res_elec',
    'rt storage efficiency EV batteries':
    'rt_storage_efficiency_ev_batteries',
    'cum materials to extract for EV batteries from 2015':
    'cum_materials_to_extract_for_ev_batteries_from_2015',
    'total cumulative demand materials to extract from 2015':
    'total_cumulative_demand_materials_to_extract_from_2015',
    'initial cumulated material requirements for EV batteries 1995':
    'initial_cumulated_material_requirements_for_ev_batteries_1995',
    'share materials cum demand to extract vs resources for RES elec':
    'share_materials_cum_demand_to_extract_vs_resources_for_res_elec',
    'FE tot generation all RES elec EJ':
    'fe_tot_generation_all_res_elec_ej',
    'XCarbon footprint tonnesCXpersonX':
    'xcarbon_footprint_tonnescxpersonx',
    'Potential max HDI':
    'potential_max_hdi',
    'kW per battery EV':
    'kw_per_battery_ev',
    'PES fossil fuel extraction delayed':
    'pes_fossil_fuel_extraction_delayed',
    'PE traditional biomass EJ':
    'pe_traditional_biomass_ej',
    'Historic share of losses vs extraction SUBliquids':
    'historic_share_of_losses_vs_extraction_subliquids',
    'Historic share of losses vs extraction SUBsolids':
    'historic_share_of_losses_vs_extraction_subsolids',
    'Historic share of losses vs extraction SUBgases':
    'historic_share_of_losses_vs_extraction_subgases',
    'Losses EJ':
    'losses_ej',
    'PES fossil fuel extraction':
    'pes_fossil_fuel_extraction',
    'modern BioE in households':
    'modern_bioe_in_households',
    'Historic demand by sector SUBAgricultureHuntingForestryandFishing':
    'historic_demand_by_sector_subagriculturehuntingforestryandfishing',
    'Historic demand by sector SUBMiningandQuarrying':
    'historic_demand_by_sector_subminingandquarrying',
    'Historic demand by sector SUBFoodBeveragesandTobacco':
    'historic_demand_by_sector_subfoodbeveragesandtobacco',
    'Historic demand by sector SUBTextilesandTextileProducts':
    'historic_demand_by_sector_subtextilesandtextileproducts',
    'Historic demand by sector SUBLeatherLeatherandFootwear':
    'historic_demand_by_sector_subleatherleatherandfootwear',
    'Historic demand by sector SUBWoodandProductsofWooodandCork':
    'historic_demand_by_sector_subwoodandproductsofwooodandcork',
    'Historic demand by sector SUBPulpPaperPrintingandPublishing':
    'historic_demand_by_sector_subpulppaperprintingandpublishing',
    'Historic demand by sector SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_demand_by_sector_subcokerefinedpetroleumandnuclearfuel',
    'Historic demand by sector SUBChemicalsandChemicalproducts':
    'historic_demand_by_sector_subchemicalsandchemicalproducts',
    'Historic demand by sector SUBRubberandPlastics':
    'historic_demand_by_sector_subrubberandplastics',
    'Historic demand by sector SUBOtherNonMetalicMineral':
    'historic_demand_by_sector_subothernonmetalicmineral',
    'Historic demand by sector SUBBasicMetalsandFabricatedMetal':
    'historic_demand_by_sector_subbasicmetalsandfabricatedmetal',
    'Historic demand by sector SUBMachineryNec':
    'historic_demand_by_sector_submachinerynec',
    'Historic demand by sector SUBElectricalandOpticalEquipment':
    'historic_demand_by_sector_subelectricalandopticalequipment',
    'Historic demand by sector SUBTransportEquipment':
    'historic_demand_by_sector_subtransportequipment',
    'Historic demand by sector SUBManufacturingNecRecycling':
    'historic_demand_by_sector_submanufacturingnecrecycling',
    'Historic demand by sector SUBElectricityGasandWaterSupply':
    'historic_demand_by_sector_subelectricitygasandwatersupply',
    'Historic demand by sector SUBConstruction':
    'historic_demand_by_sector_subconstruction',
    'Historic demand by sector SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_demand_by_sector_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic demand by sector SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_demand_by_sector_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic demand by sector SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_demand_by_sector_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic demand by sector SUBHotelsandRestaurants':
    'historic_demand_by_sector_subhotelsandrestaurants',
    'Historic demand by sector SUBInlandTransport':
    'historic_demand_by_sector_subinlandtransport',
    'Historic demand by sector SUBWaterTransport':
    'historic_demand_by_sector_subwatertransport',
    'Historic demand by sector SUBAirTransport':
    'historic_demand_by_sector_subairtransport',
    'Historic demand by sector SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_demand_by_sector_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic demand by sector SUBPostandTelecommunications':
    'historic_demand_by_sector_subpostandtelecommunications',
    'Historic demand by sector SUBFinancialIntermedation':
    'historic_demand_by_sector_subfinancialintermedation',
    'Historic demand by sector SUBRealEstateActivities':
    'historic_demand_by_sector_subrealestateactivities',
    'Historic demand by sector SUBRentingodMEqandOtherBusinessActivities':
    'historic_demand_by_sector_subrentingodmeqandotherbusinessactivities',
    'Historic demand by sector SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_demand_by_sector_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic demand by sector SUBEducation':
    'historic_demand_by_sector_subeducation',
    'Historic demand by sector SUBHealthandSocialWork':
    'historic_demand_by_sector_subhealthandsocialwork',
    'Historic demand by sector SUBOtherCommunitySocialandPersonaServices':
    'historic_demand_by_sector_subothercommunitysocialandpersonaservices',
    'Historic demand by sector SUBPrivateHouseholdswithEmployedPersons':
    'historic_demand_by_sector_subprivatehouseholdswithemployedpersons',
    'variation demand flow FD':
    'variation_demand_flow_fd',
    'historic variation demand':
    'historic_variation_demand',
    'Initial energy intensity 1995 H':
    'initial_energy_intensity_1995_h',
    'min energy intensity vs intial H':
    'min_energy_intensity_vs_intial_h',
    'minimum fraction H':
    'minimum_fraction_h',
    'Energy market pressure H':
    'energy_market_pressure_h',
    'Pressure to change energy technology H':
    'pressure_to_change_energy_technology_h',
    'Policy to improve efficiency speed H':
    'policy_to_improve_efficiency_speed_h',
    'Max yearly change H':
    'max_yearly_change_h',
    'available improvement efficiency H':
    'available_improvement_efficiency_h',
    'Year policy change energy H':
    'year_policy_change_energy_h',
    'Efficiency energy aceleration H':
    'efficiency_energy_aceleration_h',
    'efficiency rate of substitution H':
    'efficiency_rate_of_substitution_h',
    'Energy cost pressure H':
    'energy_cost_pressure_h',
    'abundance of energy H':
    'abundance_of_energy_h',
    'Global energy intensity H':
    'global_energy_intensity_h',
    'historic rate final energy intensity H':
    'historic_rate_final_energy_intensity_h',
    'Maximun yearly aceleration of intensity improvement pct H':
    'maximun_yearly_aceleration_of_intensity_improvement_pct_h',
    'historical mean rate energy intensity H':
    'historical_mean_rate_energy_intensity_h',
    'relative abundance of energy H':
    'relative_abundance_of_energy_h',
    'Policy change energy speed H':
    'policy_change_energy_speed_h',
    'Historic final energy intensity H SUBelectricity':
    'historic_final_energy_intensity_h_subelectricity',
    'Historic final energy intensity H SUBheat':
    'historic_final_energy_intensity_h_subheat',
    'Historic final energy intensity H SUBliquids':
    'historic_final_energy_intensity_h_subliquids',
    'Historic final energy intensity H SUBsolids':
    'historic_final_energy_intensity_h_subsolids',
    'Historic final energy intensity H SUBgases':
    'historic_final_energy_intensity_h_subgases',
    'Initial global energy intensity 2009 H':
    'initial_global_energy_intensity_2009_h',
    'Year policy to improve efficiency H':
    'year_policy_to_improve_efficiency_h',
    'share conv vs total oil extraction':
    'share_conv_vs_total_oil_extraction',
    'XPES natX gas without GTLX':
    'xpes_natx_gas_without_gtlx',
    'Tot FEH NRE':
    'tot_feh_nre',
    'CO2 emissions conv gas without GTL':
    'co2_emissions_conv_gas_without_gtl',
    'CO2 emissions conv oil':
    'co2_emissions_conv_oil',
    'CO2 emissions GTL':
    'co2_emissions_gtl',
    'PE gas extraction for Transp EJ':
    'pe_gas_extraction_for_transp_ej',
    'CH4 emissions conv gas without GTL':
    'ch4_emissions_conv_gas_without_gtl',
    'CO2 emissions unconv oil':
    'co2_emissions_unconv_oil',
    'XShortage coef without MIN without EXlossesX':
    'xshortage_coef_without_min_without_exlossesx',
    'share conv vs total gas extraction':
    'share_conv_vs_total_gas_extraction',
    'share FEH over PES coal solids':
    'share_feh_over_pes_coal_solids',
    'CH4 emissions unconv gas':
    'ch4_emissions_unconv_gas',
    'FEH coal delayed':
    'feh_coal_delayed',
    'FEH coal':
    'feh_coal',
    'FE heat demand consum after CC impacts EJ':
    'fe_heat_demand_consum_after_cc_impacts_ej',
    'share electricity vs TFES':
    'share_electricity_vs_tfes',
    'CO2 emissions peat':
    'co2_emissions_peat',
    'XTotal EXlosses due to CC impactsX':
    'xtotal_exlosses_due_to_cc_impactsx',
    'Required FED by gas':
    'required_fed_by_gas',
    'XEXlosses by final fuel due to CC impactsX':
    'xexlosses_by_final_fuel_due_to_cc_impactsx',
    'XEXlosses heatXcom due to CC impactsX':
    'xexlosses_heatxcom_due_to_cc_impactsx',
    'Required TFED':
    'required_tfed',
    'Total FE Elec consumption EJ':
    'total_fe_elec_consumption_ej',
    'Total FE Elec consumption TWh':
    'total_fe_elec_consumption_twh',
    'PED gases':
    'ped_gases',
    'Total FE Heat consumption EJ':
    'total_fe_heat_consumption_ej',
    'share solids vs TFES':
    'share_solids_vs_tfes',
    'Total CO2 emissions GTCO2':
    'total_co2_emissions_gtco2',
    'share gases vs TFES':
    'share_gases_vs_tfes',
    'Required FED solids':
    'required_fed_solids',
    'XFED HeatXnc EJX':
    'xfed_heatxnc_ejx',
    'share heat vs TFES':
    'share_heat_vs_tfes',
    'PED solids':
    'ped_solids',
    'scarcity conv oil':
    'scarcity_conv_oil',
    'test2':
    'test2',
    'share liquids vs TFES':
    'share_liquids_vs_tfes',
    'GFCF not covered':
    'gfcf_not_covered',
    'LC not covered':
    'lc_not_covered',
    'CC total not covered':
    'cc_total_not_covered',
    'Household demand not covered':
    'household_demand_not_covered',
    'Real total output inland transport':
    'real_total_output_inland_transport',
    'TFES intensity EJ Tdolar':
    'tfes_intensity_ej_tdolar',
    'TFES intensity EJ Tdolar delayed 1yr':
    'tfes_intensity_ej_tdolar_delayed_1yr',
    'TPES intensity EJ Tdolar delayed 1yr':
    'tpes_intensity_ej_tdolar_delayed_1yr',
    'Annual TPES intensity growth rate':
    'annual_tpes_intensity_growth_rate',
    'Annual TFES intensity growth rate':
    'annual_tfes_intensity_growth_rate',
    'Total D jobs RES elec per techn':
    'total_d_jobs_res_elec_per_techn',
    'Total D jobs RES heat per techn':
    'total_d_jobs_res_heat_per_techn',
    'Employment factors new RES elec':
    'employment_factors_new_res_elec',
    'Employment factors new RES heat':
    'employment_factors_new_res_heat',
    'XEmployment factors OXM RES elecX':
    'xemployment_factors_oxm_res_elecx',
    'XEmployment factors OXM RES heatX':
    'xemployment_factors_oxm_res_heatx',
    'D jobs new installed RES elec per techn':
    'd_jobs_new_installed_res_elec_per_techn',
    'XJobs OXM RES elec per technX':
    'xjobs_oxm_res_elec_per_technx',
    'NX0 vehicles per Xinland T':
    'nx0_vehicles_per_xinland_t',
    'initial percent T vehicles':
    'initial_percent_t_vehicles',
    'saving ratios V':
    'saving_ratios_v',
    'FEH gas delayed':
    'feh_gas_delayed',
    'share FEH over PES gases':
    'share_feh_over_pes_gases',
    'Xshare gases dem for HeatXncX':
    'xshare_gases_dem_for_heatxncx',
    'FEH gas':
    'feh_gas',
    'increase scarcity conv gas':
    'increase_scarcity_conv_gas',
    'scarcity conv gas stock':
    'scarcity_conv_gas_stock',
    'XPES natX gas for HeatXnc plantsX':
    'xpes_natx_gas_for_heatxnc_plantsx',
    'real growth CTL':
    'real_growth_ctl',
    'CTL potential production':
    'ctl_potential_production',
    'abundance liquids GTL':
    'abundance_liquids_gtl',
    'GTL potential production':
    'gtl_potential_production',
    'abundance liquids CTL':
    'abundance_liquids_ctl',
    'wear GTL':
    'wear_gtl',
    'lifetime GTL':
    'lifetime_gtl',
    'lifetime CTL':
    'lifetime_ctl',
    'wear CTL':
    'wear_ctl',
    'abundance unconv oil stock':
    'abundance_unconv_oil_stock',
    'increase abundance unconv oil':
    'increase_abundance_unconv_oil',
    'abundance unconv oil':
    'abundance_unconv_oil',
    'abundance unconv oil delayed 1yr':
    'abundance_unconv_oil_delayed_1yr',
    'abundance unconv oil2':
    'abundance_unconv_oil2',
    'real extraction unconv oil EJ':
    'real_extraction_unconv_oil_ej',
    'scarcity conv oil stock':
    'scarcity_conv_oil_stock',
    'Share variable RES elec generation vs total gen':
    'share_variable_res_elec_generation_vs_total_gen',
    'extra monet invest to cope with variable Elec RES':
    'extra_monet_invest_to_cope_with_variable_elec_res',
    'increase scarcity conv oil':
    'increase_scarcity_conv_oil',
    'share extra monet invest to cope with variable Elec RES':
    'share_extra_monet_invest_to_cope_with_variable_elec_res',
    'initial share variable RES elec gen vs total':
    'initial_share_variable_res_elec_gen_vs_total',
    'XPES oil for HeatXnc plantsX':
    'xpes_oil_for_heatxnc_plantsx',
    'cumulated invest E grid':
    'cumulated_invest_e_grid',
    'Total monet invest RES for elec Tdolar':
    'total_monet_invest_res_for_elec_tdolar',
    'Cp exogenous RES elec var reduction':
    'cp_exogenous_res_elec_var_reduction',
    'Cp exogenous RES elec dispatch reduction':
    'cp_exogenous_res_elec_dispatch_reduction',
    'increase variable RES share elec vs total generation':
    'increase_variable_res_share_elec_vs_total_generation',
    'available improvement efficiency':
    'available_improvement_efficiency',
    'XPES NRE HeatXncX':
    'xpes_nre_heatxncx',
    'share FEH over PES liquids':
    'share_feh_over_pes_liquids',
    'XFED natX gas for heatXncX':
    'xfed_natx_gas_for_heatxncx',
    'TPES heat':
    'tpes_heat',
    'XFED oil for heatXncX':
    'xfed_oil_for_heatxncx',
    'FEH oil':
    'feh_oil',
    'FEH oil delayed':
    'feh_oil_delayed',
    'PES NRE heat':
    'pes_nre_heat',
    'Xshare liquids dem for HeatXncX':
    'xshare_liquids_dem_for_heatxncx',
    'XPES NRE HeatXcomX':
    'xpes_nre_heatxcomx',
    'check TPE':
    'check_tpe',
    'XFED by fuel for heatXncX':
    'xfed_by_fuel_for_heatxncx',
    'XShare FED heatXcom vs total heatX':
    'xshare_fed_heatxcom_vs_total_heatx',
    'check hist Elec':
    'check_hist_elec',
    'Average elec consumption per capita':
    'average_elec_consumption_per_capita',
    'PES solids':
    'pes_solids',
    'XFED solid bioE for heatXncX':
    'xfed_solid_bioe_for_heatxncx',
    'Total FED Heat EJ':
    'total_fed_heat_ej',
    'XHeatXnc distribution lossesX':
    'xheatxnc_distribution_lossesx',
    'XTotal FED HeatXnc EJX':
    'xtotal_fed_heatxnc_ejx',
    'XTotal FED NRE HeatXncX':
    'xtotal_fed_nre_heatxncx',
    'XPED gas HeatXncX':
    'xped_gas_heatxncx',
    'XPED coal HeatXncX':
    'xped_coal_heatxncx',
    'XDeactivate heat dem correctionXX':
    'xdeactivate_heat_dem_correctionxx',
    'XPED liquids HeatXncX':
    'xped_liquids_heatxncx',
    'PED coal EJ':
    'ped_coal_ej',
    'share FEH over FED solid bioE':
    'share_feh_over_fed_solid_bioe',
    'FES Heat from coal':
    'fes_heat_from_coal',
    'XFES Heat from natX gasX':
    'xfes_heat_from_natx_gasx',
    'FES Heat from oil':
    'fes_heat_from_oil',
    'Total distribution losses':
    'total_distribution_losses',
    'Abundance heat':
    'abundance_heat',
    'XPES coal for HeatXnc plantsX':
    'xpes_coal_for_heatxnc_plantsx',
    'Population dependent on trad biomass':
    'population_dependent_on_trad_biomass',
    'required FED sectors by fuel':
    'required_fed_sectors_by_fuel',
    'share trad biomass vs solids in households':
    'share_trad_biomass_vs_solids_in_households',
    'PE traditional biomass EJ delayed 1yr':
    'pe_traditional_biomass_ej_delayed_1yr',
    'Xnew RES capacity for heatXcom TWX':
    'xnew_res_capacity_for_heatxcom_twx',
    'share global pop dependent on trad biomass':
    'share_global_pop_dependent_on_trad_biomass',
    'XHistoric RES capacity for heatXcomX SUBXsolarXheatX':
    'xhistoric_res_capacity_for_heatxcomx_subxsolarxheatx',
    'XHistoric RES capacity for heatXcomX SUBXgeotXheatX':
    'xhistoric_res_capacity_for_heatxcomx_subxgeotxheatx',
    'XHistoric RES capacity for heatXcomX SUBXsolidbioEXheatX':
    'xhistoric_res_capacity_for_heatxcomx_subxsolidbioexheatx',
    'XPES oil for HeatXcom plantsX':
    'xpes_oil_for_heatxcom_plantsx',
    'Xshare coal dem for HeatXncX':
    'xshare_coal_dem_for_heatxncx',
    'XRES heatXcom tot overcapacityX':
    'xres_heatxcom_tot_overcapacityx',
    'XPES natX gas for HeatXcom plantsX':
    'xpes_natx_gas_for_heatxcom_plantsx',
    'XPES coal for HeatXcom plantsX':
    'xpes_coal_for_heatxcom_plantsx',
    'check hist solids bioE':
    'check_hist_solids_bioe',
    'XRequired heatXcom after CC impactsX':
    'xrequired_heatxcom_after_cc_impactsx',
    'variation labour share':
    'variation_labour_share',
    'share inventories next step SUBAgricultureHuntingForestryandFishing':
    'share_inventories_next_step_subagriculturehuntingforestryandfishing',
    'share inventories next step SUBMiningandQuarrying':
    'share_inventories_next_step_subminingandquarrying',
    'share inventories next step SUBFoodBeveragesandTobacco':
    'share_inventories_next_step_subfoodbeveragesandtobacco',
    'share inventories next step SUBTextilesandTextileProducts':
    'share_inventories_next_step_subtextilesandtextileproducts',
    'share inventories next step SUBLeatherLeatherandFootwear':
    'share_inventories_next_step_subleatherleatherandfootwear',
    'share inventories next step SUBWoodandProductsofWooodandCork':
    'share_inventories_next_step_subwoodandproductsofwooodandcork',
    'share inventories next step SUBPulpPaperPrintingandPublishing':
    'share_inventories_next_step_subpulppaperprintingandpublishing',
    'share inventories next step SUBCokeRefinedPetroleumandNuclearFuel':
    'share_inventories_next_step_subcokerefinedpetroleumandnuclearfuel',
    'share inventories next step SUBChemicalsandChemicalproducts':
    'share_inventories_next_step_subchemicalsandchemicalproducts',
    'share inventories next step SUBRubberandPlastics':
    'share_inventories_next_step_subrubberandplastics',
    'share inventories next step SUBOtherNonMetalicMineral':
    'share_inventories_next_step_subothernonmetalicmineral',
    'share inventories next step SUBBasicMetalsandFabricatedMetal':
    'share_inventories_next_step_subbasicmetalsandfabricatedmetal',
    'share inventories next step SUBMachineryNec':
    'share_inventories_next_step_submachinerynec',
    'share inventories next step SUBElectricalandOpticalEquipment':
    'share_inventories_next_step_subelectricalandopticalequipment',
    'share inventories next step SUBTransportEquipment':
    'share_inventories_next_step_subtransportequipment',
    'share inventories next step SUBManufacturingNecRecycling':
    'share_inventories_next_step_submanufacturingnecrecycling',
    'share inventories next step SUBElectricityGasandWaterSupply':
    'share_inventories_next_step_subelectricitygasandwatersupply',
    'share inventories next step SUBConstruction':
    'share_inventories_next_step_subconstruction',
    'share inventories next step SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_inventories_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share inventories next step SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_inventories_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share inventories next step SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_inventories_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share inventories next step SUBHotelsandRestaurants':
    'share_inventories_next_step_subhotelsandrestaurants',
    'share inventories next step SUBInlandTransport':
    'share_inventories_next_step_subinlandtransport',
    'share inventories next step SUBWaterTransport':
    'share_inventories_next_step_subwatertransport',
    'share inventories next step SUBAirTransport':
    'share_inventories_next_step_subairtransport',
    'share inventories next step SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_inventories_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share inventories next step SUBPostandTelecommunications':
    'share_inventories_next_step_subpostandtelecommunications',
    'share inventories next step SUBFinancialIntermedation':
    'share_inventories_next_step_subfinancialintermedation',
    'share inventories next step SUBRealEstateActivities':
    'share_inventories_next_step_subrealestateactivities',
    'share inventories next step SUBRentingodMEqandOtherBusinessActivities':
    'share_inventories_next_step_subrentingodmeqandotherbusinessactivities',
    'share inventories next step SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_inventories_next_step_subpublicadminanddefencecompulsorysocialsecurity',
    'share inventories next step SUBEducation':
    'share_inventories_next_step_subeducation',
    'share inventories next step SUBHealthandSocialWork':
    'share_inventories_next_step_subhealthandsocialwork',
    'share inventories next step SUBOtherCommunitySocialandPersonaServices':
    'share_inventories_next_step_subothercommunitysocialandpersonaservices',
    'share inventories next step SUBPrivateHouseholdswithEmployedPersons':
    'share_inventories_next_step_subprivatehouseholdswithemployedpersons',
    'variation historic demand':
    'variation_historic_demand',
    'share consum goverments SUBAgricultureHuntingForestryandFishing':
    'share_consum_goverments_subagriculturehuntingforestryandfishing',
    'share consum goverments SUBMiningandQuarrying':
    'share_consum_goverments_subminingandquarrying',
    'share consum goverments SUBFoodBeveragesandTobacco':
    'share_consum_goverments_subfoodbeveragesandtobacco',
    'share consum goverments SUBTextilesandTextileProducts':
    'share_consum_goverments_subtextilesandtextileproducts',
    'share consum goverments SUBLeatherLeatherandFootwear':
    'share_consum_goverments_subleatherleatherandfootwear',
    'share consum goverments SUBWoodandProductsofWooodandCork':
    'share_consum_goverments_subwoodandproductsofwooodandcork',
    'share consum goverments SUBPulpPaperPrintingandPublishing':
    'share_consum_goverments_subpulppaperprintingandpublishing',
    'share consum goverments SUBCokeRefinedPetroleumandNuclearFuel':
    'share_consum_goverments_subcokerefinedpetroleumandnuclearfuel',
    'share consum goverments SUBChemicalsandChemicalproducts':
    'share_consum_goverments_subchemicalsandchemicalproducts',
    'share consum goverments SUBRubberandPlastics':
    'share_consum_goverments_subrubberandplastics',
    'share consum goverments SUBOtherNonMetalicMineral':
    'share_consum_goverments_subothernonmetalicmineral',
    'share consum goverments SUBBasicMetalsandFabricatedMetal':
    'share_consum_goverments_subbasicmetalsandfabricatedmetal',
    'share consum goverments SUBMachineryNec':
    'share_consum_goverments_submachinerynec',
    'share consum goverments SUBElectricalandOpticalEquipment':
    'share_consum_goverments_subelectricalandopticalequipment',
    'share consum goverments SUBTransportEquipment':
    'share_consum_goverments_subtransportequipment',
    'share consum goverments SUBManufacturingNecRecycling':
    'share_consum_goverments_submanufacturingnecrecycling',
    'share consum goverments SUBElectricityGasandWaterSupply':
    'share_consum_goverments_subelectricitygasandwatersupply',
    'share consum goverments SUBConstruction':
    'share_consum_goverments_subconstruction',
    'share consum goverments SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_consum_goverments_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share consum goverments SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_consum_goverments_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share consum goverments SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_consum_goverments_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share consum goverments SUBHotelsandRestaurants':
    'share_consum_goverments_subhotelsandrestaurants',
    'share consum goverments SUBInlandTransport':
    'share_consum_goverments_subinlandtransport',
    'share consum goverments SUBWaterTransport':
    'share_consum_goverments_subwatertransport',
    'share consum goverments SUBAirTransport':
    'share_consum_goverments_subairtransport',
    'share consum goverments SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_consum_goverments_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share consum goverments SUBPostandTelecommunications':
    'share_consum_goverments_subpostandtelecommunications',
    'share consum goverments SUBFinancialIntermedation':
    'share_consum_goverments_subfinancialintermedation',
    'share consum goverments SUBRealEstateActivities':
    'share_consum_goverments_subrealestateactivities',
    'share consum goverments SUBRentingodMEqandOtherBusinessActivities':
    'share_consum_goverments_subrentingodmeqandotherbusinessactivities',
    'share consum goverments SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_consum_goverments_subpublicadminanddefencecompulsorysocialsecurity',
    'share consum goverments SUBEducation':
    'share_consum_goverments_subeducation',
    'share consum goverments SUBHealthandSocialWork':
    'share_consum_goverments_subhealthandsocialwork',
    'share consum goverments SUBOtherCommunitySocialandPersonaServices':
    'share_consum_goverments_subothercommunitysocialandpersonaservices',
    'share consum goverments SUBPrivateHouseholdswithEmployedPersons':
    'share_consum_goverments_subprivatehouseholdswithemployedpersons',
    'variation GFCF':
    'variation_gfcf',
    'variation historic GFCF':
    'variation_historic_gfcf',
    'variation household demand':
    'variation_household_demand',
    'labour share':
    'labour_share',
    'share inventories SUBAgricultureHuntingForestryandFishing':
    'share_inventories_subagriculturehuntingforestryandfishing',
    'share inventories SUBMiningandQuarrying':
    'share_inventories_subminingandquarrying',
    'share inventories SUBFoodBeveragesandTobacco':
    'share_inventories_subfoodbeveragesandtobacco',
    'share inventories SUBTextilesandTextileProducts':
    'share_inventories_subtextilesandtextileproducts',
    'share inventories SUBLeatherLeatherandFootwear':
    'share_inventories_subleatherleatherandfootwear',
    'share inventories SUBWoodandProductsofWooodandCork':
    'share_inventories_subwoodandproductsofwooodandcork',
    'share inventories SUBPulpPaperPrintingandPublishing':
    'share_inventories_subpulppaperprintingandpublishing',
    'share inventories SUBCokeRefinedPetroleumandNuclearFuel':
    'share_inventories_subcokerefinedpetroleumandnuclearfuel',
    'share inventories SUBChemicalsandChemicalproducts':
    'share_inventories_subchemicalsandchemicalproducts',
    'share inventories SUBRubberandPlastics':
    'share_inventories_subrubberandplastics',
    'share inventories SUBOtherNonMetalicMineral':
    'share_inventories_subothernonmetalicmineral',
    'share inventories SUBBasicMetalsandFabricatedMetal':
    'share_inventories_subbasicmetalsandfabricatedmetal',
    'share inventories SUBMachineryNec':
    'share_inventories_submachinerynec',
    'share inventories SUBElectricalandOpticalEquipment':
    'share_inventories_subelectricalandopticalequipment',
    'share inventories SUBTransportEquipment':
    'share_inventories_subtransportequipment',
    'share inventories SUBManufacturingNecRecycling':
    'share_inventories_submanufacturingnecrecycling',
    'share inventories SUBElectricityGasandWaterSupply':
    'share_inventories_subelectricitygasandwatersupply',
    'share inventories SUBConstruction':
    'share_inventories_subconstruction',
    'share inventories SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_inventories_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share inventories SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_inventories_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share inventories SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_inventories_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share inventories SUBHotelsandRestaurants':
    'share_inventories_subhotelsandrestaurants',
    'share inventories SUBInlandTransport':
    'share_inventories_subinlandtransport',
    'share inventories SUBWaterTransport':
    'share_inventories_subwatertransport',
    'share inventories SUBAirTransport':
    'share_inventories_subairtransport',
    'share inventories SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_inventories_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share inventories SUBPostandTelecommunications':
    'share_inventories_subpostandtelecommunications',
    'share inventories SUBFinancialIntermedation':
    'share_inventories_subfinancialintermedation',
    'share inventories SUBRealEstateActivities':
    'share_inventories_subrealestateactivities',
    'share inventories SUBRentingodMEqandOtherBusinessActivities':
    'share_inventories_subrentingodmeqandotherbusinessactivities',
    'share inventories SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_inventories_subpublicadminanddefencecompulsorysocialsecurity',
    'share inventories SUBEducation':
    'share_inventories_subeducation',
    'share inventories SUBHealthandSocialWork':
    'share_inventories_subhealthandsocialwork',
    'share inventories SUBOtherCommunitySocialandPersonaServices':
    'share_inventories_subothercommunitysocialandpersonaservices',
    'share inventories SUBPrivateHouseholdswithEmployedPersons':
    'share_inventories_subprivatehouseholdswithemployedpersons',
    'share consum goverments next step SUBAgricultureHuntingForestryandFishing':
    'share_consum_goverments_next_step_subagriculturehuntingforestryandfishing',
    'share consum goverments next step SUBMiningandQuarrying':
    'share_consum_goverments_next_step_subminingandquarrying',
    'share consum goverments next step SUBFoodBeveragesandTobacco':
    'share_consum_goverments_next_step_subfoodbeveragesandtobacco',
    'share consum goverments next step SUBTextilesandTextileProducts':
    'share_consum_goverments_next_step_subtextilesandtextileproducts',
    'share consum goverments next step SUBLeatherLeatherandFootwear':
    'share_consum_goverments_next_step_subleatherleatherandfootwear',
    'share consum goverments next step SUBWoodandProductsofWooodandCork':
    'share_consum_goverments_next_step_subwoodandproductsofwooodandcork',
    'share consum goverments next step SUBPulpPaperPrintingandPublishing':
    'share_consum_goverments_next_step_subpulppaperprintingandpublishing',
    'share consum goverments next step SUBCokeRefinedPetroleumandNuclearFuel':
    'share_consum_goverments_next_step_subcokerefinedpetroleumandnuclearfuel',
    'share consum goverments next step SUBChemicalsandChemicalproducts':
    'share_consum_goverments_next_step_subchemicalsandchemicalproducts',
    'share consum goverments next step SUBRubberandPlastics':
    'share_consum_goverments_next_step_subrubberandplastics',
    'share consum goverments next step SUBOtherNonMetalicMineral':
    'share_consum_goverments_next_step_subothernonmetalicmineral',
    'share consum goverments next step SUBBasicMetalsandFabricatedMetal':
    'share_consum_goverments_next_step_subbasicmetalsandfabricatedmetal',
    'share consum goverments next step SUBMachineryNec':
    'share_consum_goverments_next_step_submachinerynec',
    'share consum goverments next step SUBElectricalandOpticalEquipment':
    'share_consum_goverments_next_step_subelectricalandopticalequipment',
    'share consum goverments next step SUBTransportEquipment':
    'share_consum_goverments_next_step_subtransportequipment',
    'share consum goverments next step SUBManufacturingNecRecycling':
    'share_consum_goverments_next_step_submanufacturingnecrecycling',
    'share consum goverments next step SUBElectricityGasandWaterSupply':
    'share_consum_goverments_next_step_subelectricitygasandwatersupply',
    'share consum goverments next step SUBConstruction':
    'share_consum_goverments_next_step_subconstruction',
    'share consum goverments next step SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_consum_goverments_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share consum goverments next step SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_consum_goverments_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share consum goverments next step SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_consum_goverments_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share consum goverments next step SUBHotelsandRestaurants':
    'share_consum_goverments_next_step_subhotelsandrestaurants',
    'share consum goverments next step SUBInlandTransport':
    'share_consum_goverments_next_step_subinlandtransport',
    'share consum goverments next step SUBWaterTransport':
    'share_consum_goverments_next_step_subwatertransport',
    'share consum goverments next step SUBAirTransport':
    'share_consum_goverments_next_step_subairtransport',
    'share consum goverments next step SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_consum_goverments_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share consum goverments next step SUBPostandTelecommunications':
    'share_consum_goverments_next_step_subpostandtelecommunications',
    'share consum goverments next step SUBFinancialIntermedation':
    'share_consum_goverments_next_step_subfinancialintermedation',
    'share consum goverments next step SUBRealEstateActivities':
    'share_consum_goverments_next_step_subrealestateactivities',
    'share consum goverments next step SUBRentingodMEqandOtherBusinessActivities':
    'share_consum_goverments_next_step_subrentingodmeqandotherbusinessactivities',
    'share consum goverments next step SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_consum_goverments_next_step_subpublicadminanddefencecompulsorysocialsecurity',
    'share consum goverments next step SUBEducation':
    'share_consum_goverments_next_step_subeducation',
    'share consum goverments next step SUBHealthandSocialWork':
    'share_consum_goverments_next_step_subhealthandsocialwork',
    'share consum goverments next step SUBOtherCommunitySocialandPersonaServices':
    'share_consum_goverments_next_step_subothercommunitysocialandpersonaservices',
    'share consum goverments next step SUBPrivateHouseholdswithEmployedPersons':
    'share_consum_goverments_next_step_subprivatehouseholdswithemployedpersons',
    'share consum goverment and inventories':
    'share_consum_goverment_and_inventories',
    'share consum goverments and inventories next step':
    'share_consum_goverments_and_inventories_next_step',
    'capital share':
    'capital_share',
    'historic growth capital share':
    'historic_growth_capital_share',
    'Historic GFCF SUBAgricultureHuntingForestryandFishing':
    'historic_gfcf_subagriculturehuntingforestryandfishing',
    'Historic GFCF SUBMiningandQuarrying':
    'historic_gfcf_subminingandquarrying',
    'Historic GFCF SUBFoodBeveragesandTobacco':
    'historic_gfcf_subfoodbeveragesandtobacco',
    'Historic GFCF SUBTextilesandTextileProducts':
    'historic_gfcf_subtextilesandtextileproducts',
    'Historic GFCF SUBLeatherLeatherandFootwear':
    'historic_gfcf_subleatherleatherandfootwear',
    'Historic GFCF SUBWoodandProductsofWooodandCork':
    'historic_gfcf_subwoodandproductsofwooodandcork',
    'Historic GFCF SUBPulpPaperPrintingandPublishing':
    'historic_gfcf_subpulppaperprintingandpublishing',
    'Historic GFCF SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_gfcf_subcokerefinedpetroleumandnuclearfuel',
    'Historic GFCF SUBChemicalsandChemicalproducts':
    'historic_gfcf_subchemicalsandchemicalproducts',
    'Historic GFCF SUBRubberandPlastics':
    'historic_gfcf_subrubberandplastics',
    'Historic GFCF SUBOtherNonMetalicMineral':
    'historic_gfcf_subothernonmetalicmineral',
    'Historic GFCF SUBBasicMetalsandFabricatedMetal':
    'historic_gfcf_subbasicmetalsandfabricatedmetal',
    'Historic GFCF SUBMachineryNec':
    'historic_gfcf_submachinerynec',
    'Historic GFCF SUBElectricalandOpticalEquipment':
    'historic_gfcf_subelectricalandopticalequipment',
    'Historic GFCF SUBTransportEquipment':
    'historic_gfcf_subtransportequipment',
    'Historic GFCF SUBManufacturingNecRecycling':
    'historic_gfcf_submanufacturingnecrecycling',
    'Historic GFCF SUBElectricityGasandWaterSupply':
    'historic_gfcf_subelectricitygasandwatersupply',
    'Historic GFCF SUBConstruction':
    'historic_gfcf_subconstruction',
    'Historic GFCF SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_gfcf_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic GFCF SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_gfcf_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic GFCF SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_gfcf_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic GFCF SUBHotelsandRestaurants':
    'historic_gfcf_subhotelsandrestaurants',
    'Historic GFCF SUBInlandTransport':
    'historic_gfcf_subinlandtransport',
    'Historic GFCF SUBWaterTransport':
    'historic_gfcf_subwatertransport',
    'Historic GFCF SUBAirTransport':
    'historic_gfcf_subairtransport',
    'Historic GFCF SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_gfcf_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic GFCF SUBPostandTelecommunications':
    'historic_gfcf_subpostandtelecommunications',
    'Historic GFCF SUBFinancialIntermedation':
    'historic_gfcf_subfinancialintermedation',
    'Historic GFCF SUBRealEstateActivities':
    'historic_gfcf_subrealestateactivities',
    'Historic GFCF SUBRentingodMEqandOtherBusinessActivities':
    'historic_gfcf_subrentingodmeqandotherbusinessactivities',
    'Historic GFCF SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_gfcf_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic GFCF SUBEducation':
    'historic_gfcf_subeducation',
    'Historic GFCF SUBHealthandSocialWork':
    'historic_gfcf_subhealthandsocialwork',
    'Historic GFCF SUBOtherCommunitySocialandPersonaServices':
    'historic_gfcf_subothercommunitysocialandpersonaservices',
    'Historic GFCF SUBPrivateHouseholdswithEmployedPersons':
    'historic_gfcf_subprivatehouseholdswithemployedpersons',
    'Demand by sector':
    'demand_by_sector',
    'Historic HD SUBAgricultureHuntingForestryandFishing':
    'historic_hd_subagriculturehuntingforestryandfishing',
    'Historic HD SUBMiningandQuarrying':
    'historic_hd_subminingandquarrying',
    'Historic HD SUBFoodBeveragesandTobacco':
    'historic_hd_subfoodbeveragesandtobacco',
    'Historic HD SUBTextilesandTextileProducts':
    'historic_hd_subtextilesandtextileproducts',
    'Historic HD SUBLeatherLeatherandFootwear':
    'historic_hd_subleatherleatherandfootwear',
    'Historic HD SUBWoodandProductsofWooodandCork':
    'historic_hd_subwoodandproductsofwooodandcork',
    'Historic HD SUBPulpPaperPrintingandPublishing':
    'historic_hd_subpulppaperprintingandpublishing',
    'Historic HD SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_hd_subcokerefinedpetroleumandnuclearfuel',
    'Historic HD SUBChemicalsandChemicalproducts':
    'historic_hd_subchemicalsandchemicalproducts',
    'Historic HD SUBRubberandPlastics':
    'historic_hd_subrubberandplastics',
    'Historic HD SUBOtherNonMetalicMineral':
    'historic_hd_subothernonmetalicmineral',
    'Historic HD SUBBasicMetalsandFabricatedMetal':
    'historic_hd_subbasicmetalsandfabricatedmetal',
    'Historic HD SUBMachineryNec':
    'historic_hd_submachinerynec',
    'Historic HD SUBElectricalandOpticalEquipment':
    'historic_hd_subelectricalandopticalequipment',
    'Historic HD SUBTransportEquipment':
    'historic_hd_subtransportequipment',
    'Historic HD SUBManufacturingNecRecycling':
    'historic_hd_submanufacturingnecrecycling',
    'Historic HD SUBElectricityGasandWaterSupply':
    'historic_hd_subelectricitygasandwatersupply',
    'Historic HD SUBConstruction':
    'historic_hd_subconstruction',
    'Historic HD SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_hd_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic HD SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_hd_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic HD SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_hd_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic HD SUBHotelsandRestaurants':
    'historic_hd_subhotelsandrestaurants',
    'Historic HD SUBInlandTransport':
    'historic_hd_subinlandtransport',
    'Historic HD SUBWaterTransport':
    'historic_hd_subwatertransport',
    'Historic HD SUBAirTransport':
    'historic_hd_subairtransport',
    'Historic HD SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_hd_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic HD SUBPostandTelecommunications':
    'historic_hd_subpostandtelecommunications',
    'Historic HD SUBFinancialIntermedation':
    'historic_hd_subfinancialintermedation',
    'Historic HD SUBRealEstateActivities':
    'historic_hd_subrealestateactivities',
    'Historic HD SUBRentingodMEqandOtherBusinessActivities':
    'historic_hd_subrentingodmeqandotherbusinessactivities',
    'Historic HD SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_hd_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic HD SUBEducation':
    'historic_hd_subeducation',
    'Historic HD SUBHealthandSocialWork':
    'historic_hd_subhealthandsocialwork',
    'Historic HD SUBOtherCommunitySocialandPersonaServices':
    'historic_hd_subothercommunitysocialandpersonaservices',
    'Historic HD SUBPrivateHouseholdswithEmployedPersons':
    'historic_hd_subprivatehouseholdswithemployedpersons',
    'Xinclude materials for overgridsXX':
    'xinclude_materials_for_overgridsxx',
    'XAll minerals virginXX':
    'xall_minerals_virginxx',
    'materials for new RES elec per capacity installed':
    'materials_for_new_res_elec_per_capacity_installed',
    'CEDtot per material RES elec var':
    'cedtot_per_material_res_elec_var',
    'CEDtot per TW RES elec var':
    'cedtot_per_tw_res_elec_var',
    'CEDtot per TW per material RES elec var':
    'cedtot_per_tw_per_material_res_elec_var',
    'Demand by sector FD':
    'demand_by_sector_fd',
    'sum variation':
    'sum_variation',
    'CC sectoral':
    'cc_sectoral',
    'CC sectoral next step':
    'cc_sectoral_next_step',
    'Gross fixed capital formation':
    'gross_fixed_capital_formation',
    'variation CC sectoral':
    'variation_cc_sectoral',
    'share CC next step SUBAgricultureHuntingForestryandFishing':
    'share_cc_next_step_subagriculturehuntingforestryandfishing',
    'share CC next step SUBMiningandQuarrying':
    'share_cc_next_step_subminingandquarrying',
    'share CC next step SUBFoodBeveragesandTobacco':
    'share_cc_next_step_subfoodbeveragesandtobacco',
    'share CC next step SUBTextilesandTextileProducts':
    'share_cc_next_step_subtextilesandtextileproducts',
    'share CC next step SUBLeatherLeatherandFootwear':
    'share_cc_next_step_subleatherleatherandfootwear',
    'share CC next step SUBWoodandProductsofWooodandCork':
    'share_cc_next_step_subwoodandproductsofwooodandcork',
    'share CC next step SUBPulpPaperPrintingandPublishing':
    'share_cc_next_step_subpulppaperprintingandpublishing',
    'share CC next step SUBCokeRefinedPetroleumandNuclearFuel':
    'share_cc_next_step_subcokerefinedpetroleumandnuclearfuel',
    'share CC next step SUBChemicalsandChemicalproducts':
    'share_cc_next_step_subchemicalsandchemicalproducts',
    'share CC next step SUBRubberandPlastics':
    'share_cc_next_step_subrubberandplastics',
    'share CC next step SUBOtherNonMetalicMineral':
    'share_cc_next_step_subothernonmetalicmineral',
    'share CC next step SUBBasicMetalsandFabricatedMetal':
    'share_cc_next_step_subbasicmetalsandfabricatedmetal',
    'share CC next step SUBMachineryNec':
    'share_cc_next_step_submachinerynec',
    'share CC next step SUBElectricalandOpticalEquipment':
    'share_cc_next_step_subelectricalandopticalequipment',
    'share CC next step SUBTransportEquipment':
    'share_cc_next_step_subtransportequipment',
    'share CC next step SUBManufacturingNecRecycling':
    'share_cc_next_step_submanufacturingnecrecycling',
    'share CC next step SUBElectricityGasandWaterSupply':
    'share_cc_next_step_subelectricitygasandwatersupply',
    'share CC next step SUBConstruction':
    'share_cc_next_step_subconstruction',
    'share CC next step SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_cc_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share CC next step SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_cc_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share CC next step SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_cc_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share CC next step SUBHotelsandRestaurants':
    'share_cc_next_step_subhotelsandrestaurants',
    'share CC next step SUBInlandTransport':
    'share_cc_next_step_subinlandtransport',
    'share CC next step SUBWaterTransport':
    'share_cc_next_step_subwatertransport',
    'share CC next step SUBAirTransport':
    'share_cc_next_step_subairtransport',
    'share CC next step SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_cc_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share CC next step SUBPostandTelecommunications':
    'share_cc_next_step_subpostandtelecommunications',
    'share CC next step SUBFinancialIntermedation':
    'share_cc_next_step_subfinancialintermedation',
    'share CC next step SUBRealEstateActivities':
    'share_cc_next_step_subrealestateactivities',
    'share CC next step SUBRentingodMEqandOtherBusinessActivities':
    'share_cc_next_step_subrentingodmeqandotherbusinessactivities',
    'share CC next step SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_cc_next_step_subpublicadminanddefencecompulsorysocialsecurity',
    'share CC next step SUBEducation':
    'share_cc_next_step_subeducation',
    'share CC next step SUBHealthandSocialWork':
    'share_cc_next_step_subhealthandsocialwork',
    'share CC next step SUBOtherCommunitySocialandPersonaServices':
    'share_cc_next_step_subothercommunitysocialandpersonaservices',
    'share CC next step SUBPrivateHouseholdswithEmployedPersons':
    'share_cc_next_step_subprivatehouseholdswithemployedpersons',
    'Xpct GFCF vs GFCFXHDX':
    'xpct_gfcf_vs_gfcfxhdx',
    'Household demand':
    'household_demand',
    'LC next step':
    'lc_next_step',
    'Household demand total':
    'household_demand_total',
    'CEDtot per TW over lifetime RES elec dispatch':
    'cedtot_per_tw_over_lifetime_res_elec_dispatch',
    'quality of electricity 2015':
    'quality_of_electricity_2015',
    'Dynamic quality of electricity':
    'dynamic_quality_of_electricity',
    'quality of electricity':
    'quality_of_electricity',
    'Cp exogenous RES elec reduction':
    'cp_exogenous_res_elec_reduction',
    'Cp RES elec':
    'cp_res_elec',
    'XRES elec variablesXX':
    'xres_elec_variablesxx',
    'aux3':
    'aux3',
    'overdemand elec EROI from 2015':
    'overdemand_elec_eroi_from_2015',
    'FEI RES elec var':
    'fei_res_elec_var',
    'XXstaticX EROI RES elecX':
    'xxstaticx_eroi_res_elecx',
    'materials required for new RES elec Mt':
    'materials_required_for_new_res_elec_mt',
    'EROIst tot elec':
    'eroist_tot_elec',
    'FEI over lifetime RES elec var':
    'fei_over_lifetime_res_elec_var',
    'CED decom RES elec capacity':
    'ced_decom_res_elec_capacity',
    'test ir restando al EROIst tot elec':
    'test_ir_restando_al_eroist_tot_elec',
    'EROIst tot elec 2':
    'eroist_tot_elec_2',
    'XCED OXM per material RES elec varX':
    'xced_oxm_per_material_res_elec_varx',
    'CEDtot new cap RES elec var':
    'cedtot_new_cap_res_elec_var',
    'XCEDtot OXM RES elec varX':
    'xcedtot_oxm_res_elec_varx',
    'overdemand elec tot until 2015':
    'overdemand_elec_tot_until_2015',
    'aux2':
    'aux2',
    'aux1':
    'aux1',
    'overdemand elec tot':
    'overdemand_elec_tot',
    'overdemand elec EROI delayed 1yr':
    'overdemand_elec_eroi_delayed_1yr',
    'XActivate EROI elec feedbackXX':
    'xactivate_eroi_elec_feedbackxx',
    'CED unconv gas for elec':
    'ced_unconv_gas_for_elec',
    'CED uranium for elec':
    'ced_uranium_for_elec',
    'CEDtot over lifetime NRE elec':
    'cedtot_over_lifetime_nre_elec',
    'b lineal regr':
    'b_lineal_regr',
    'EROI uranium':
    'eroi_uranium',
    'EROI coal':
    'eroi_coal',
    'EROI conv gas':
    'eroi_conv_gas',
    'EROI total oil':
    'eroi_total_oil',
    'EROI unconv gas':
    'eroi_unconv_gas',
    'CED coal for elec':
    'ced_coal_for_elec',
    'CED conv gas for elec':
    'ced_conv_gas_for_elec',
    'a lineal regr':
    'a_lineal_regr',
    'CED total oil for elec':
    'ced_total_oil_for_elec',
    'ESOI PHS depleted potential':
    'esoi_phs_depleted_potential',
    'Historic labour share':
    'historic_labour_share',
    'CC total':
    'cc_total',
    'initial household demand':
    'initial_household_demand',
    'total demand':
    'total_demand',
    'demand not covered by sector FD':
    'demand_not_covered_by_sector_fd',
    'initial demand by sectot':
    'initial_demand_by_sectot',
    'share CC sectoral SUBAgricultureHuntingForestryandFishing':
    'share_cc_sectoral_subagriculturehuntingforestryandfishing',
    'share CC sectoral SUBMiningandQuarrying':
    'share_cc_sectoral_subminingandquarrying',
    'share CC sectoral SUBFoodBeveragesandTobacco':
    'share_cc_sectoral_subfoodbeveragesandtobacco',
    'share CC sectoral SUBTextilesandTextileProducts':
    'share_cc_sectoral_subtextilesandtextileproducts',
    'share CC sectoral SUBLeatherLeatherandFootwear':
    'share_cc_sectoral_subleatherleatherandfootwear',
    'share CC sectoral SUBWoodandProductsofWooodandCork':
    'share_cc_sectoral_subwoodandproductsofwooodandcork',
    'share CC sectoral SUBPulpPaperPrintingandPublishing':
    'share_cc_sectoral_subpulppaperprintingandpublishing',
    'share CC sectoral SUBCokeRefinedPetroleumandNuclearFuel':
    'share_cc_sectoral_subcokerefinedpetroleumandnuclearfuel',
    'share CC sectoral SUBChemicalsandChemicalproducts':
    'share_cc_sectoral_subchemicalsandchemicalproducts',
    'share CC sectoral SUBRubberandPlastics':
    'share_cc_sectoral_subrubberandplastics',
    'share CC sectoral SUBOtherNonMetalicMineral':
    'share_cc_sectoral_subothernonmetalicmineral',
    'share CC sectoral SUBBasicMetalsandFabricatedMetal':
    'share_cc_sectoral_subbasicmetalsandfabricatedmetal',
    'share CC sectoral SUBMachineryNec':
    'share_cc_sectoral_submachinerynec',
    'share CC sectoral SUBElectricalandOpticalEquipment':
    'share_cc_sectoral_subelectricalandopticalequipment',
    'share CC sectoral SUBTransportEquipment':
    'share_cc_sectoral_subtransportequipment',
    'share CC sectoral SUBManufacturingNecRecycling':
    'share_cc_sectoral_submanufacturingnecrecycling',
    'share CC sectoral SUBElectricityGasandWaterSupply':
    'share_cc_sectoral_subelectricitygasandwatersupply',
    'share CC sectoral SUBConstruction':
    'share_cc_sectoral_subconstruction',
    'share CC sectoral SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_cc_sectoral_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share CC sectoral SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_cc_sectoral_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share CC sectoral SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_cc_sectoral_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share CC sectoral SUBHotelsandRestaurants':
    'share_cc_sectoral_subhotelsandrestaurants',
    'share CC sectoral SUBInlandTransport':
    'share_cc_sectoral_subinlandtransport',
    'share CC sectoral SUBWaterTransport':
    'share_cc_sectoral_subwatertransport',
    'share CC sectoral SUBAirTransport':
    'share_cc_sectoral_subairtransport',
    'share CC sectoral SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_cc_sectoral_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share CC sectoral SUBPostandTelecommunications':
    'share_cc_sectoral_subpostandtelecommunications',
    'share CC sectoral SUBFinancialIntermedation':
    'share_cc_sectoral_subfinancialintermedation',
    'share CC sectoral SUBRealEstateActivities':
    'share_cc_sectoral_subrealestateactivities',
    'share CC sectoral SUBRentingodMEqandOtherBusinessActivities':
    'share_cc_sectoral_subrentingodmeqandotherbusinessactivities',
    'share CC sectoral SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_cc_sectoral_subpublicadminanddefencecompulsorysocialsecurity',
    'share CC sectoral SUBEducation':
    'share_cc_sectoral_subeducation',
    'share CC sectoral SUBHealthandSocialWork':
    'share_cc_sectoral_subhealthandsocialwork',
    'share CC sectoral SUBOtherCommunitySocialandPersonaServices':
    'share_cc_sectoral_subothercommunitysocialandpersonaservices',
    'share CC sectoral SUBPrivateHouseholdswithEmployedPersons':
    'share_cc_sectoral_subprivatehouseholdswithemployedpersons',
    'Beta 1 lab':
    'beta_1_lab',
    'initial LC total':
    'initial_lc_total',
    'initial GFCF':
    'initial_gfcf',
    'beta 0 cap':
    'beta_0_cap',
    'initial CC total':
    'initial_cc_total',
    'beta 2 cap':
    'beta_2_cap',
    'demand not covered total FD':
    'demand_not_covered_total_fd',
    'beta 1 cap':
    'beta_1_cap',
    'LC':
    'lc',
    'Bet 0 lab':
    'bet_0_lab',
    'TPED by fuel':
    'tped_by_fuel',
    'abundance TPE':
    'abundance_tpe',
    'beta 2 lab':
    'beta_2_lab',
    'historic rate final energy intensity':
    'historic_rate_final_energy_intensity',
    'Initial global energy intensity by sector 2009':
    'initial_global_energy_intensity_by_sector_2009',
    'Efficiency energy aceleration':
    'efficiency_energy_aceleration',
    'available PE potential solid bioE for heat EJ':
    'available_pe_potential_solid_bioe_for_heat_ej',
    'Max PE potential RES for heat':
    'max_pe_potential_res_for_heat',
    'XPES RES for heatXcom by technX':
    'xpes_res_for_heatxcom_by_technx',
    'P RES for heat':
    'p_res_for_heat',
    'XFE real generation RES heatXcom EJX':
    'xfe_real_generation_res_heatxcom_ejx',
    'Xpotential FES tot RES for heatXcom EJX':
    'xpotential_fes_tot_res_for_heatxcom_ejx',
    'XTotal PE solid bioE potential heatXelec EJX':
    'xtotal_pe_solid_bioe_potential_heatxelec_ejx',
    'available potential FE solid bioE for elec EJ':
    'available_potential_fe_solid_bioe_for_elec_ej',
    'Xreplacement RES for heatXcom TWX':
    'xreplacement_res_for_heatxcom_twx',
    'max BioE TWe':
    'max_bioe_twe',
    'remaining potential tot RES heat':
    'remaining_potential_tot_res_heat',
    'XPotential FES HeatXcom nuclear CHP plants EJX':
    'xpotential_fes_heatxcom_nuclear_chp_plants_ejx',
    'share of heat production in CHP plants vs total nucelar elec generation':
    'share_of_heat_production_in_chp_plants_vs_total_nucelar_elec_generation',
    'PES waste for elec plants':
    'pes_waste_for_elec_plants',
    'Losses CHP biogas':
    'losses_chp_biogas',
    'Losses CHP waste':
    'losses_chp_waste',
    'share PES biogas for heat':
    'share_pes_biogas_for_heat',
    'PE losses RES for elec':
    'pe_losses_res_for_elec',
    'PES tot waste for elec':
    'pes_tot_waste_for_elec',
    'XPES tot waste for heatXcomX':
    'xpes_tot_waste_for_heatxcomx',
    'Xabundance RES heatXcom2X':
    'xabundance_res_heatxcom2x',
    'max PE potential biogas for heat':
    'max_pe_potential_biogas_for_heat',
    'share efficiency biogas for elec in CHP plants':
    'share_efficiency_biogas_for_elec_in_chp_plants',
    'share efficiency waste for elec in CHP plants':
    'share_efficiency_waste_for_elec_in_chp_plants',
    'XMax potential PE nonXelectric RESX':
    'xmax_potential_pe_nonxelectric_resx',
    'PES tot biogas for elec':
    'pes_tot_biogas_for_elec',
    'XPES tot biogas for heatXcomX':
    'xpes_tot_biogas_for_heatxcomx',
    'PE losses NRE elec generation':
    'pe_losses_nre_elec_generation',
    'Elec gen related losses EJ':
    'elec_gen_related_losses_ej',
    'PE losses biogas for elec':
    'pe_losses_biogas_for_elec',
    'PE losses waste for elec':
    'pe_losses_waste_for_elec',
    'max PE potential tot RES heat EJ':
    'max_pe_potential_tot_res_heat_ej',
    'share PES biogas for elec':
    'share_pes_biogas_for_elec',
    'P hydro growth':
    'p_hydro_growth',
    'FE real tot generation RES elec TWh':
    'fe_real_tot_generation_res_elec_twh',
    'P solar PV growth':
    'p_solar_pv_growth',
    'remaining potential constraint on new RES heat capacity':
    'remaining_potential_constraint_on_new_res_heat_capacity',
    'Total FE Elec generation TWh':
    'total_fe_elec_generation_twh',
    'FE tot generation all RES elec TWh':
    'fe_tot_generation_all_res_elec_twh',
    'P oceanic growth':
    'p_oceanic_growth',
    'PE Elec generation from RES EJ':
    'pe_elec_generation_from_res_ej',
    'P RES elec growth':
    'p_res_elec_growth',
    'share RES electricity generation':
    'share_res_electricity_generation',
    'XP solid bioEXelec growthX':
    'xp_solid_bioexelec_growthx',
    'P CSP growth':
    'p_csp_growth',
    'remaining potential tot RES elec':
    'remaining_potential_tot_res_elec',
    'P wind offshore growth':
    'p_wind_offshore_growth',
    'P wind onshore growth':
    'p_wind_onshore_growth',
    'XElec generation baseXload from RES TWhX':
    'xelec_generation_basexload_from_res_twhx',
    'FES elec from RES with priority TWh':
    'fes_elec_from_res_with_priority_twh',
    'share Renewables Elec':
    'share_renewables_elec',
    'abundance RES elec':
    'abundance_res_elec',
    'FES elec from BioW':
    'fes_elec_from_biow',
    'P geot growth':
    'p_geot_growth',
    'max potential tot RES elec TWh':
    'max_potential_tot_res_elec_twh',
    'P solid bioE for heat':
    'p_solid_bioe_for_heat',
    'PE real generation RES elec':
    'pe_real_generation_res_elec',
    'share Renewables Elec check':
    'share_renewables_elec_check',
    'share renewables Heat check':
    'share_renewables_heat_check',
    'Efficiency RES heat':
    'efficiency_res_heat',
    'XFED heatXcom by NRE CHP plants EJX':
    'xfed_heatxcom_by_nre_chp_plants_ejx',
    'XConstant final energy intensity sectorsXhouseholds TDXX':
    'xconstant_final_energy_intensity_sectorsxhouseholds_tdxx',
    'Xshare oil dem for HeatXcomX':
    'xshare_oil_dem_for_heatxcomx',
    'FE Elec generation from total oil TWh':
    'fe_elec_generation_from_total_oil_twh',
    'Total FE Heat generation EJ':
    'total_fe_heat_generation_ej',
    'XPE supply RES nonXElec EJX':
    'xpe_supply_res_nonxelec_ejx',
    'share coal dem for Elec':
    'share_coal_dem_for_elec',
    'PE losses oil for Elec EJ':
    'pe_losses_oil_for_elec_ej',
    'share oil dem for Elec':
    'share_oil_dem_for_elec',
    'Xshare coal dem for HeatXcomX':
    'xshare_coal_dem_for_heatxcomx',
    'FE Elec generation from conv gas TWh':
    'fe_elec_generation_from_conv_gas_twh',
    'share RES heat generation':
    'share_res_heat_generation',
    'share gas for Transp':
    'share_gas_for_transp',
    'Xshare natX gas dem for ElecX':
    'xshare_natx_gas_dem_for_elecx',
    'PE losses conv gas for Elec EJ':
    'pe_losses_conv_gas_for_elec_ej',
    'PE losses coal for Elec EJ':
    'pe_losses_coal_for_elec_ej',
    'FE Elec generation from coal TWh':
    'fe_elec_generation_from_coal_twh',
    'Xshare natX gas dem for HeatXcomX':
    'xshare_natx_gas_dem_for_heatxcomx',
    'Year scarcity Heat':
    'year_scarcity_heat',
    'XFES heatXcom from biogas in CHP plantsX':
    'xfes_heatxcom_from_biogas_in_chp_plantsx',
    'efficiency biogas for heat plants':
    'efficiency_biogas_for_heat_plants',
    'FES elec from biogas in CHP plants':
    'fes_elec_from_biogas_in_chp_plants',
    'FES elec from biogas EJ':
    'fes_elec_from_biogas_ej',
    'XPES biogas for heatXcom plantsX':
    'xpes_biogas_for_heatxcom_plantsx',
    'FES elec from biogas in elec plants':
    'fes_elec_from_biogas_in_elec_plants',
    'efficiency biogas for elec plants':
    'efficiency_biogas_for_elec_plants',
    'XFES biogas for heatXcom plantsX':
    'xfes_biogas_for_heatxcom_plantsx',
    'PES biogas for elec plants':
    'pes_biogas_for_elec_plants',
    'share PES biogas TFC':
    'share_pes_biogas_tfc',
    'Potential PES biogas for TFC':
    'potential_pes_biogas_for_tfc',
    'FES elec from biogas TWh':
    'fes_elec_from_biogas_twh',
    'PES biogas for CHP':
    'pes_biogas_for_chp',
    'Xshare PES biogas for heatXcom plantsX':
    'xshare_pes_biogas_for_heatxcom_plantsx',
    'efficiency biogas for elec CHP plants':
    'efficiency_biogas_for_elec_chp_plants',
    'XFES heatXcom from biogas EJX':
    'xfes_heatxcom_from_biogas_ejx',
    'efficiency biogas for heat CHP plants':
    'efficiency_biogas_for_heat_chp_plants',
    'share PES biogas for elec plants':
    'share_pes_biogas_for_elec_plants',
    'share PES biogas for CHP':
    'share_pes_biogas_for_chp',
    'demand Elec plants fossil fuels TWh':
    'demand_elec_plants_fossil_fuels_twh',
    'adapt growth biogas':
    'adapt_growth_biogas',
    'Total extraction NRE EJ':
    'total_extraction_nre_ej',
    'new PES biogas':
    'new_pes_biogas',
    'Energy cost pressure':
    'energy_cost_pressure',
    'Pressure to improve energy intensity efficiency':
    'pressure_to_improve_energy_intensity_efficiency',
    'abundance of energy':
    'abundance_of_energy',
    'Energy market pressure':
    'energy_market_pressure',
    'relative abundance of energy':
    'relative_abundance_of_energy',
    'Pressure to change energy technology':
    'pressure_to_change_energy_technology',
    'remaining recycling rate improv for alt techn per mineral':
    'remaining_recycling_rate_improv_for_alt_techn_per_mineral',
    'remaining recycling rate improv Rest per mineral':
    'remaining_recycling_rate_improv_rest_per_mineral',
    'XCED OXM over lifetime per material RES elec varX':
    'xced_oxm_over_lifetime_per_material_res_elec_varx',
    'Energy cons per unit of material cons for RES elec':
    'energy_cons_per_unit_of_material_cons_for_res_elec',
    'CED new cap per material RES elec var':
    'ced_new_cap_per_material_res_elec_var',
    'Energy required for material consumption for new RES elec':
    'energy_required_for_material_consumption_for_new_res_elec',
    'XEnergy required for material consumption for OXM RES elecX':
    'xenergy_required_for_material_consumption_for_oxm_res_elecx',
    'share other cumulative demand to extract vs resources materials':
    'share_other_cumulative_demand_to_extract_vs_resources_materials',
    'Materials to extract Rest from 2015 Mt':
    'materials_to_extract_rest_from_2015_mt',
    'Total recycled materials for RES elec Mt':
    'total_recycled_materials_for_res_elec_mt',
    'cum materials to extract Rest':
    'cum_materials_to_extract_rest',
    'cum materials to extract Rest from 2015':
    'cum_materials_to_extract_rest_from_2015',
    'share other cumulative demand to extract vs reserves materials':
    'share_other_cumulative_demand_to_extract_vs_reserves_materials',
    'Xmaterials availability Xannual extractionXX':
    'xmaterials_availability_xannual_extractionxx',
    'Mt per tonne':
    'mt_per_tonne',
    'initial cumulated material requirements for Rest 1995':
    'initial_cumulated_material_requirements_for_rest_1995',
    'P recycling minerals Rest':
    'p_recycling_minerals_rest',
    'cum materials to extract for RES elec from 2015':
    'cum_materials_to_extract_for_res_elec_from_2015',
    'Total materials to extract for RES elec Mt':
    'total_materials_to_extract_for_res_elec_mt',
    'cum materials to extract for RES elec':
    'cum_materials_to_extract_for_res_elec',
    'Total materials to extract for RES elec from 2015 Mt':
    'total_materials_to_extract_for_res_elec_from_2015_mt',
    'Max recycling rates minerals':
    'max_recycling_rates_minerals',
    'P recycling minerals alt techn':
    'p_recycling_minerals_alt_techn',
    'current recycling rates minerals':
    'current_recycling_rates_minerals',
    'Historic improvement recycling rates minerals':
    'historic_improvement_recycling_rates_minerals',
    'min energy intensity vs intial':
    'min_energy_intensity_vs_intial',
    'Global energy intensity by sector':
    'global_energy_intensity_by_sector',
    'historical mean rate energy intensity':
    'historical_mean_rate_energy_intensity',
    'XInitial energy cons per unit of material cons XrecycledXX':
    'xinitial_energy_cons_per_unit_of_material_cons_xrecycledxx',
    'Xmaterials required for OXM RES elec MtX':
    'xmaterials_required_for_oxm_res_elec_mtx',
    'Initial energy intensity by fuel and sector 1995':
    'initial_energy_intensity_by_fuel_and_sector_1995',
    'share tot cum dem vs reserves materials':
    'share_tot_cum_dem_vs_reserves_materials',
    'share tot cum dem vs resources materials':
    'share_tot_cum_dem_vs_resources_materials',
    'maximum annual extraction materials':
    'maximum_annual_extraction_materials',
    'Xmaterials availability XresourcesXX':
    'xmaterials_availability_xresourcesxx',
    'Xmaterials availability XreservesXX':
    'xmaterials_availability_xreservesxx',
    'TPES EJ':
    'tpes_ej',
    'XFES heatXcom from waste EJX':
    'xfes_heatxcom_from_waste_ejx',
    'XFES waste for heatXcom plantsX':
    'xfes_waste_for_heatxcom_plantsx',
    'FES elec from waste in CHP plants':
    'fes_elec_from_waste_in_chp_plants',
    'FES elec from waste in elec plants':
    'fes_elec_from_waste_in_elec_plants',
    'XFES heatXcom from waste in CHP plantsX':
    'xfes_heatxcom_from_waste_in_chp_plantsx',
    'XPES waste for heatXcom plantsX':
    'xpes_waste_for_heatxcom_plantsx',
    'Xshare PES waste for heatXcom plantsX':
    'xshare_pes_waste_for_heatxcom_plantsx',
    'share PES waste TFC':
    'share_pes_waste_tfc',
    'share PES waste for CHP':
    'share_pes_waste_for_chp',
    'share PES waste for elec plants':
    'share_pes_waste_for_elec_plants',
    'new waste supply EJ':
    'new_waste_supply_ej',
    'waste change':
    'waste_change',
    'FES elec from waste EJ':
    'fes_elec_from_waste_ej',
    'FES elec from waste TWh':
    'fes_elec_from_waste_twh',
    'efficiency waste for elec plants':
    'efficiency_waste_for_elec_plants',
    'PES waste EJ':
    'pes_waste_ej',
    'PES waste for CHP plants':
    'pes_waste_for_chp_plants',
    'efficiency waste for elec CHP plants':
    'efficiency_waste_for_elec_chp_plants',
    'initial PES waste':
    'initial_pes_waste',
    'PES waste for TFC':
    'pes_waste_for_tfc',
    'efficiency waste for heat CHP plants':
    'efficiency_waste_for_heat_chp_plants',
    'adapt growth waste':
    'adapt_growth_waste',
    'P waste change':
    'p_waste_change',
    'efficiency waste for heat plants':
    'efficiency_waste_for_heat_plants',
    'max waste':
    'max_waste',
    'XPES solids bioE X waste EJX':
    'xpes_solids_bioe_x_waste_ejx',
    'Past waste growth':
    'past_waste_growth',
    'Historic PES waste EJ':
    'historic_pes_waste_ej',
    'CH4 emissions oil':
    'ch4_emissions_oil',
    'gCH4 per MJ coal':
    'gch4_per_mj_coal',
    'gCH4 per MJ oil':
    'gch4_per_mj_oil',
    'FE Elec generation from fossil fuels TWh':
    'fe_elec_generation_from_fossil_fuels_twh',
    'PE demand uranium EJ':
    'pe_demand_uranium_ej',
    'FES Elec fossil fuel CHP plants TWh':
    'fes_elec_fossil_fuel_chp_plants_twh',
    'XFES heatXcom fossil fuels CHP plants EJX':
    'xfes_heatxcom_fossil_fuels_chp_plants_ejx',
    'potential generation nuclear elec TWh':
    'potential_generation_nuclear_elec_twh',
    'threshold remaining potential new capacity':
    'threshold_remaining_potential_new_capacity',
    'installed capacity RES elec TW':
    'installed_capacity_res_elec_tw',
    'Xtotal time planXconstr RES elecX':
    'xtotal_time_planxconstr_res_elecx',
    'Historic new required capacity RES elec':
    'historic_new_required_capacity_res_elec',
    'new required capacity RES elec':
    'new_required_capacity_res_elec',
    'table hist capacity RES elec SUBhydro':
    'table_hist_capacity_res_elec_subhydro',
    'table hist capacity RES elec SUBXgeotXelecX':
    'table_hist_capacity_res_elec_subxgeotxelecx',
    'table hist capacity RES elec SUBXsolidbioEXelecX':
    'table_hist_capacity_res_elec_subxsolidbioexelecx',
    'table hist capacity RES elec SUBoceanic':
    'table_hist_capacity_res_elec_suboceanic',
    'table hist capacity RES elec SUBwindonshore':
    'table_hist_capacity_res_elec_subwindonshore',
    'table hist capacity RES elec SUBwindoffshore':
    'table_hist_capacity_res_elec_subwindoffshore',
    'table hist capacity RES elec SUBsolarPV':
    'table_hist_capacity_res_elec_subsolarpv',
    'table hist capacity RES elec SUBCSP':
    'table_hist_capacity_res_elec_subcsp',
    'initial capacity in construction RES elec':
    'initial_capacity_in_construction_res_elec',
    'remaining potential constraint on new RES elec capacity':
    'remaining_potential_constraint_on_new_res_elec_capacity',
    'required capacity RES elec TW':
    'required_capacity_res_elec_tw',
    'RES elec planned capacity TW':
    'res_elec_planned_capacity_tw',
    'XInitial energy cons per unit of material cons XrecycledX X dataX':
    'xinitial_energy_cons_per_unit_of_material_cons_xrecycledx_x_datax',
    'Cp PHS':
    'cp_phs',
    'max potential PHS TWe':
    'max_potential_phs_twe',
    'max capacity potential PHS':
    'max_capacity_potential_phs',
    'Xmaterials per new RES elec capacity installed X HVDCsX':
    'xmaterials_per_new_res_elec_capacity_installed_x_hvdcsx',
    'Xmaterials per new RES elec capacity installed X material overgrid high powerX':
    'xmaterials_per_new_res_elec_capacity_installed_x_material_overgrid_high_powerx',
    'real growth GTL':
    'real_growth_gtl',
    'scarcity conv gas delayed 1yr':
    'scarcity_conv_gas_delayed_1yr',
    'scarcity conv oil delayed 1yr':
    'scarcity_conv_oil_delayed_1yr',
    'XPES natX gasX':
    'xpes_natx_gasx',
    'FE Elec generation from unconv gas TWh':
    'fe_elec_generation_from_unconv_gas_twh',
    'exponent availability conv oil':
    'exponent_availability_conv_oil',
    'PE losses uncon gas for Elec EJ':
    'pe_losses_uncon_gas_for_elec_ej',
    'Xabundance total natX gasX':
    'xabundance_total_natx_gasx',
    'exponent availability conv gas':
    'exponent_availability_conv_gas',
    'scarcity conv gas':
    'scarcity_conv_gas',
    'XYear scarcity total natX gasX':
    'xyear_scarcity_total_natx_gasx',
    'share unconv gas vs tot agg in 2050':
    'share_unconv_gas_vs_tot_agg_in_2050',
    'share unconv gas vs tot agg in 2050 Laherrere2010':
    'share_unconv_gas_vs_tot_agg_in_2050_laherrere2010',
    'share unconv gas vs tot agg in 2050 Mohr12 BG':
    'share_unconv_gas_vs_tot_agg_in_2050_mohr12_bg',
    'share unconv gas vs tot agg in 2050 User defined':
    'share_unconv_gas_vs_tot_agg_in_2050_user_defined',
    'cumulated tot agg gas extraction to 1995':
    'cumulated_tot_agg_gas_extraction_to_1995',
    'share conv gas vs tot agg':
    'share_conv_gas_vs_tot_agg',
    'share RURR tot agg gas to leave underground':
    'share_rurr_tot_agg_gas_to_leave_underground',
    'share unconv gas vs tot agg':
    'share_unconv_gas_vs_tot_agg',
    'RURR tot agg gas':
    'rurr_tot_agg_gas',
    'Flow tot agg gas left in ground':
    'flow_tot_agg_gas_left_in_ground',
    'Tot RURR tot agg gas':
    'tot_rurr_tot_agg_gas',
    'URR total agg gas unlimited':
    'urr_total_agg_gas_unlimited',
    'Start policy leave in ground tot agg gas':
    'start_policy_leave_in_ground_tot_agg_gas',
    'Xextraction unconv gas X tot aggX':
    'xextraction_unconv_gas_x_tot_aggx',
    'Total agg gas left in ground':
    'total_agg_gas_left_in_ground',
    'evolution share unconv gas vs tot agg':
    'evolution_share_unconv_gas_vs_tot_agg',
    'real extraction conv gas EJ':
    'real_extraction_conv_gas_ej',
    'URR tot agg gas':
    'urr_tot_agg_gas',
    'cumulated tot agg gas extraction':
    'cumulated_tot_agg_gas_extraction',
    'Xextraction conv gas X tot aggX':
    'xextraction_conv_gas_x_tot_aggx',
    'real extraction unconv gas EJ':
    'real_extraction_unconv_gas_ej',
    'max extraction tot agg gas EJ':
    'max_extraction_tot_agg_gas_ej',
    'extraction tot agg gas EJ':
    'extraction_tot_agg_gas_ej',
    'Xreal extraction conv oil MbXdX':
    'xreal_extraction_conv_oil_mbxdx',
    'share unconv oil vs tot agg':
    'share_unconv_oil_vs_tot_agg',
    'evolution share unconv oil vs tot agg':
    'evolution_share_unconv_oil_vs_tot_agg',
    'share unconv oil vs tot agg in 2050':
    'share_unconv_oil_vs_tot_agg_in_2050',
    'share unconv oil vs tot agg in 2050 Laherrere2006':
    'share_unconv_oil_vs_tot_agg_in_2050_laherrere2006',
    'share unconv oil vs tot agg in 2050 User defined':
    'share_unconv_oil_vs_tot_agg_in_2050_user_defined',
    'Xextraction conv oil X tot aggX':
    'xextraction_conv_oil_x_tot_aggx',
    'Xextraction unconv oil X tot aggX':
    'xextraction_unconv_oil_x_tot_aggx',
    'share conv oil vs tot agg':
    'share_conv_oil_vs_tot_agg',
    'RURR tot agg oil':
    'rurr_tot_agg_oil',
    'PES oil EJ':
    'pes_oil_ej',
    'check hist crude oil':
    'check_hist_crude_oil',
    'extraction tot agg oil EJ':
    'extraction_tot_agg_oil_ej',
    'real extraction conv oil EJ':
    'real_extraction_conv_oil_ej',
    'max extraction tot agg oil EJ':
    'max_extraction_tot_agg_oil_ej',
    'gCH4 per MJ conv gas':
    'gch4_per_mj_conv_gas',
    'Total agg oil left in ground':
    'total_agg_oil_left_in_ground',
    'cumulated tot agg oil extraction':
    'cumulated_tot_agg_oil_extraction',
    'cumulated tot agg extraction to 1995':
    'cumulated_tot_agg_extraction_to_1995',
    'URR tot agg oil':
    'urr_tot_agg_oil',
    'URR tot agg oil unlimited':
    'urr_tot_agg_oil_unlimited',
    'Tot RURR tot agg oil':
    'tot_rurr_tot_agg_oil',
    'Start policy leave in ground tot agg oil':
    'start_policy_leave_in_ground_tot_agg_oil',
    'Flow tot agg oil left in ground':
    'flow_tot_agg_oil_left_in_ground',
    'g per Mt':
    'g_per_mt',
    'share RURR tot agg oil to leave underground':
    'share_rurr_tot_agg_oil_to_leave_underground',
    'gCH4 per MJ unconv gas':
    'gch4_per_mj_unconv_gas',
    'Tot RURR unconv gas':
    'tot_rurr_unconv_gas',
    'Start policy leave in ground coal':
    'start_policy_leave_in_ground_coal',
    'Total unconv gas left in ground':
    'total_unconv_gas_left_in_ground',
    'Flow coal left in ground':
    'flow_coal_left_in_ground',
    'Flow conv gas left in ground':
    'flow_conv_gas_left_in_ground',
    'Flow conv oil left in ground':
    'flow_conv_oil_left_in_ground',
    'Flow unconv oil left in ground':
    'flow_unconv_oil_left_in_ground',
    'share RURR unconv gas to leave underground':
    'share_rurr_unconv_gas_to_leave_underground',
    'share RURR unconv oil to leave underground':
    'share_rurr_unconv_oil_to_leave_underground',
    'extraction unconv gas EJ':
    'extraction_unconv_gas_ej',
    'Tot RURR coal':
    'tot_rurr_coal',
    'Tot RURR conv gas':
    'tot_rurr_conv_gas',
    'Tot RURR conv oil':
    'tot_rurr_conv_oil',
    'share RURR conv oil to leave underground':
    'share_rurr_conv_oil_to_leave_underground',
    'Tot RURR unconv oil':
    'tot_rurr_unconv_oil',
    'Total unconv oil left in ground':
    'total_unconv_oil_left_in_ground',
    'Total coal left in ground':
    'total_coal_left_in_ground',
    'Total conv gas left in ground':
    'total_conv_gas_left_in_ground',
    'Total conv oil left in ground':
    'total_conv_oil_left_in_ground',
    'extraction conv oil EJ':
    'extraction_conv_oil_ej',
    'extraction coal EJ':
    'extraction_coal_ej',
    'RURR coal':
    'rurr_coal',
    'max extraction conv oil EJ':
    'max_extraction_conv_oil_ej',
    'Start policy leave in ground unconv gas':
    'start_policy_leave_in_ground_unconv_gas',
    'Flow unconv gas left in ground':
    'flow_unconv_gas_left_in_ground',
    'Start policy leave in ground unconv oil':
    'start_policy_leave_in_ground_unconv_oil',
    'Start policy leave in ground conv oil':
    'start_policy_leave_in_ground_conv_oil',
    'share RURR coal to leave underground':
    'share_rurr_coal_to_leave_underground',
    'XPES oil MbXdX':
    'xpes_oil_mbxdx',
    'current mineral reserves Mt':
    'current_mineral_reserves_mt',
    'Losses in charcoal plants EJ':
    'losses_in_charcoal_plants_ej',
    'check hist TPES':
    'check_hist_tpes',
    'share RURR conv gas to leave underground':
    'share_rurr_conv_gas_to_leave_underground',
    'Start policy leave in ground conv gas':
    'start_policy_leave_in_ground_conv_gas',
    'extraction conv gas EJ':
    'extraction_conv_gas_ej',
    'check gases':
    'check_gases',
    'abundance gases':
    'abundance_gases',
    'Demand conv gas':
    'demand_conv_gas',
    'XPED natX gas without GTLX':
    'xped_natx_gas_without_gtlx',
    'abundance solids':
    'abundance_solids',
    'GDP':
    'gdp',
    'demand Elec gas and coal TWh':
    'demand_elec_gas_and_coal_twh',
    'FE demand coal Elec plants TWh':
    'fe_demand_coal_elec_plants_twh',
    'FE demand gas Elec plants TWh':
    'fe_demand_gas_elec_plants_twh',
    'XFED Heat gasXcoal EJX':
    'xfed_heat_gasxcoal_ejx',
    'PE demand oil Elec plants EJ':
    'pe_demand_oil_elec_plants_ej',
    'FE demand oil Elec plants TWh':
    'fe_demand_oil_elec_plants_twh',
    'share coal for Elec':
    'share_coal_for_elec',
    'share liquids Elec':
    'share_liquids_elec',
    'current mineral resources Mt':
    'current_mineral_resources_mt',
    'share CHP plants oil':
    'share_chp_plants_oil',
    'historic share CHP plants gas':
    'historic_share_chp_plants_gas',
    'share CHP plants coal':
    'share_chp_plants_coal',
    'Historic share liquids for heat plants':
    'historic_share_liquids_for_heat_plants',
    'Xshare coalXcoalXgasX for heat plantsX':
    'xshare_coalxcoalxgasx_for_heat_plantsx',
    'historic share CHP plants oil':
    'historic_share_chp_plants_oil',
    'output elec over lifetime RES elec for allocation2':
    'output_elec_over_lifetime_res_elec_for_allocation2',
    'real generation RES elec TWh':
    'real_generation_res_elec_twh',
    'demand storage capacity':
    'demand_storage_capacity',
    'Xshare capacity storageXRES elec varX':
    'xshare_capacity_storagexres_elec_varx',
    'XstaticXdynamic quality of electricityXX':
    'xstaticxdynamic_quality_of_electricityxx',
    'XXstaticX EROIgrid totXeffective for allocation RES elecX':
    'xxstaticx_eroigrid_totxeffective_for_allocation_res_elecx',
    'Xratio EROIgrid vs EROI XstaticXX':
    'xratio_eroigrid_vs_eroi_xstaticxx',
    'Xratio EROI per techn vs EROItot XstaticXX':
    'xratio_eroi_per_techn_vs_eroitot_xstaticxx',
    'Total installed capacity RES elec var':
    'total_installed_capacity_res_elec_var',
    'share RES elec generation stored':
    'share_res_elec_generation_stored',
    'rt storage efficiency PHS':
    'rt_storage_efficiency_phs',
    'Cp baseload reduction':
    'cp_baseload_reduction',
    'Xratio X 1X':
    'xratio_x_1x',
    'CEDtot over lifetime RES elec':
    'cedtot_over_lifetime_res_elec',
    'real Cp RES elec':
    'real_cp_res_elec',
    'CEDtot over lifetime RES elec for allocation':
    'cedtot_over_lifetime_res_elec_for_allocation',
    'remaining potential RES elec after intermitt':
    'remaining_potential_res_elec_after_intermitt',
    'remaining potential RES elec switch':
    'remaining_potential_res_elec_switch',
    'output elec over lifetime RES elec for allocation':
    'output_elec_over_lifetime_res_elec_for_allocation',
    'potential RES elec after intermitt TWh':
    'potential_res_elec_after_intermitt_twh',
    'replacement rate RES elec':
    'replacement_rate_res_elec',
    'XXstaticX EROItot RES elecX':
    'xxstaticx_eroitot_res_elecx',
    'XXstaticX EROItotXeffective for allocation RES elecX':
    'xxstaticx_eroitotxeffective_for_allocation_res_elecx',
    'new capacity installed onshore wind TW':
    'new_capacity_installed_onshore_wind_tw',
    'XCED OXM over lifetime per water RES elec varX':
    'xced_oxm_over_lifetime_per_water_res_elec_varx',
    'XselfXelectricity consumption RES elecX':
    'xselfxelectricity_consumption_res_elecx',
    'share energy requirements for decom RES elec':
    'share_energy_requirements_for_decom_res_elec',
    'XCED OXM over lifetime RES elec varX':
    'xced_oxm_over_lifetime_res_elec_varx',
    'Grid correction factor RES elec':
    'grid_correction_factor_res_elec',
    'Xmaterials for OXM per capacity installed X CSPX':
    'xmaterials_for_oxm_per_capacity_installed_x_cspx',
    'XTotal water for OXM required by RES elecX':
    'xtotal_water_for_oxm_required_by_res_elecx',
    'XWater for OXM required for RES elecX':
    'xwater_for_oxm_required_for_res_elecx',
    'Xmaterials for OXM per capacity installed RES elecX':
    'xmaterials_for_oxm_per_capacity_installed_res_elecx',
    'XTotal energy requirements OXM for water consumption RES elecX':
    'xtotal_energy_requirements_oxm_for_water_consumption_res_elecx',
    'XEnergy requirements for OXM for water consumption RES elecX':
    'xenergy_requirements_for_oxm_for_water_consumption_res_elecx',
    'Xwater for OXM X RES elecX':
    'xwater_for_oxm_x_res_elecx',
    'Energy requirements per unit of water consumption':
    'energy_requirements_per_unit_of_water_consumption',
    'Xunlimited NREXX':
    'xunlimited_nrexx',
    'extraction uranium EJ':
    'extraction_uranium_ej',
    'URR uranium':
    'urr_uranium',
    'URR conv oil':
    'urr_conv_oil',
    'Historic biogas PES':
    'historic_biogas_pes',
    'growth biogas':
    'growth_biogas',
    'past biogas growth':
    'past_biogas_growth',
    'PES Biogas EJ':
    'pes_biogas_ej',
    'biogas evol':
    'biogas_evol',
    'max biogas EJ':
    'max_biogas_ej',
    'Historic TPES primary solids biofuels EJ':
    'historic_tpes_primary_solids_biofuels_ej',
    'Historic PES peat EJ':
    'historic_pes_peat_ej',
    'PES peat EJ':
    'pes_peat_ej',
    'FEI over lifetime RES elec dispatch':
    'fei_over_lifetime_res_elec_dispatch',
    'activate EROI allocation rule':
    'activate_eroi_allocation_rule',
    'EROI allocation rule per RES elec':
    'eroi_allocation_rule_per_res_elec',
    'XXdynamicX EROI RES elec varX':
    'xxdynamicx_eroi_res_elec_varx',
    'XEROIXini RES elec dispatchX':
    'xeroixini_res_elec_dispatchx',
    'share renewables fuel CHP IEA EJ':
    'share_renewables_fuel_chp_iea_ej',
    'share renewables Heat IEA EJ':
    'share_renewables_heat_iea_ej',
    'share coal Heat check':
    'share_coal_heat_check',
    'share nuclear Heat check':
    'share_nuclear_heat_check',
    'share nuclear Heat IEA EJ':
    'share_nuclear_heat_iea_ej',
    'share gases Heat IEA EJ':
    'share_gases_heat_iea_ej',
    'share coal Heat IEA EJ':
    'share_coal_heat_iea_ej',
    'share liquids Heat IEA EJ':
    'share_liquids_heat_iea_ej',
    'share fossil fuel CHP check':
    'share_fossil_fuel_chp_check',
    'share gases Heat check':
    'share_gases_heat_check',
    'share fossil fuel CHP IEA EJ':
    'share_fossil_fuel_chp_iea_ej',
    'share liquids Heat check':
    'share_liquids_heat_check',
    'Cp historic CSP':
    'cp_historic_csp',
    'RES elec capacity under construction TW':
    'res_elec_capacity_under_construction_tw',
    'share liquids Elec IEA':
    'share_liquids_elec_iea',
    'share Nuclear Elec':
    'share_nuclear_elec',
    'share Nuclear Elec check':
    'share_nuclear_elec_check',
    'share Nuclear Elec IEA':
    'share_nuclear_elec_iea',
    'share renewables Elec CHP IEA':
    'share_renewables_elec_chp_iea',
    'share Renewables Elec IEA':
    'share_renewables_elec_iea',
    'PE gas for CHP plants IEA EJ':
    'pe_gas_for_chp_plants_iea_ej',
    'check coal primary CHP plants':
    'check_coal_primary_chp_plants',
    'PE demand coal for Heat plants IEA EJ':
    'pe_demand_coal_for_heat_plants_iea_ej',
    'PE demand for liquids Heat plants IEA EJ':
    'pe_demand_for_liquids_heat_plants_iea_ej',
    'PE liquids for CHP plants IEA EJ':
    'pe_liquids_for_chp_plants_iea_ej',
    'check gas primary CHP plants':
    'check_gas_primary_chp_plants',
    'check primary liquids Elec plants':
    'check_primary_liquids_elec_plants',
    'PE demand liquids Elec plants IEA EJ':
    'pe_demand_liquids_elec_plants_iea_ej',
    'check primary liquids Heat plants':
    'check_primary_liquids_heat_plants',
    'share fossil fuel Elec CHP IEA':
    'share_fossil_fuel_elec_chp_iea',
    'PE coal for CHP plants IEA EJ':
    'pe_coal_for_chp_plants_iea_ej',
    'check primary gas Elec gas':
    'check_primary_gas_elec_gas',
    'share coal Elec':
    'share_coal_elec',
    'share coal Elec check':
    'share_coal_elec_check',
    'check liquids CHP plants':
    'check_liquids_chp_plants',
    'PE demand coal Elec plants IEA EJ':
    'pe_demand_coal_elec_plants_iea_ej',
    'check primary coal Elec plants':
    'check_primary_coal_elec_plants',
    'PE demand gases for Heat plants IEA EJ':
    'pe_demand_gases_for_heat_plants_iea_ej',
    'share gases Elec':
    'share_gases_elec',
    'share gases Elec check':
    'share_gases_elec_check',
    'PE demand gas Elec plants IEA EJ':
    'pe_demand_gas_elec_plants_iea_ej',
    'share liquids Elec check':
    'share_liquids_elec_check',
    'Xcheck primary liquids for ElecX Heat and CHP plantsX':
    'xcheck_primary_liquids_for_elecx_heat_and_chp_plantsx',
    'Xcheck primary gas for ElecX Heat and CHP plantsX':
    'xcheck_primary_gas_for_elecx_heat_and_chp_plantsx',
    'Xcheck primary coal for ElecX Heat and CHP plantsX':
    'xcheck_primary_coal_for_elecx_heat_and_chp_plantsx',
    'check primary coal for Heat plants':
    'check_primary_coal_for_heat_plants',
    'share gases Elec IEA':
    'share_gases_elec_iea',
    'share Coal Elec IEA':
    'share_coal_elec_iea',
    'share fossil fuel CHP Elec check':
    'share_fossil_fuel_chp_elec_check',
    'check primary gases Heat plants':
    'check_primary_gases_heat_plants',
    'share fossil fuel CHP Elec':
    'share_fossil_fuel_chp_elec',
    'real generation solar PV EJ':
    'real_generation_solar_pv_ej',
    'materials required for new CSP Mt':
    'materials_required_for_new_csp_mt',
    'materials required for new PV Mt':
    'materials_required_for_new_pv_mt',
    'materials required for new wind offshore Mt':
    'materials_required_for_new_wind_offshore_mt',
    'materials required for new wind onshore Mt':
    'materials_required_for_new_wind_onshore_mt',
    'Xmaterials required for OXM CSP MtX':
    'xmaterials_required_for_oxm_csp_mtx',
    'Xmaterials required for OXM PV MtX':
    'xmaterials_required_for_oxm_pv_mtx',
    'Xmaterials required for OXM wind offshore MtX':
    'xmaterials_required_for_oxm_wind_offshore_mtx',
    'Xmaterials required for OXM wind onshore MtX':
    'xmaterials_required_for_oxm_wind_onshore_mtx',
    'real generation RES elec EJ':
    'real_generation_res_elec_ej',
    'Xenergy ownXuse constant 830 Mtoe in 2013X':
    'xenergy_ownxuse_constant_830_mtoe_in_2013x',
    'Xpipeline transport constant 2X6 EJ in 2014X':
    'xpipeline_transport_constant_2x6_ej_in_2014x',
    'CEDtot solar PV':
    'cedtot_solar_pv',
    'Total materials required for new RES elec Mt':
    'total_materials_required_for_new_res_elec_mt',
    'XTotal materials required for OXM RES elec MtX':
    'xtotal_materials_required_for_oxm_res_elec_mtx',
    'XEROIX1X':
    'xeroix1x',
    'Electrical distribution losses TWh':
    'electrical_distribution_losses_twh',
    'XMax share transmXdistr elec lossesX':
    'xmax_share_transmxdistr_elec_lossesx',
    'XHeatXcom distribution lossesX':
    'xheatxcom_distribution_lossesx',
    'potential generation RES elec TWh':
    'potential_generation_res_elec_twh',
    'output elec over lifetime RES elec':
    'output_elec_over_lifetime_res_elec',
    'Xvariation share transmXdistr elec lossesX':
    'xvariation_share_transmxdistr_elec_lossesx',
    'Xremaining share transmXdistr elec lossesX':
    'xremaining_share_transmxdistr_elec_lossesx',
    'Xshare transmXdistr elec lossesX':
    'xshare_transmxdistr_elec_lossesx',
    'Total gen losses demand for Elec plants EJ':
    'total_gen_losses_demand_for_elec_plants_ej',
    'PE losses BioE for Elec EJ':
    'pe_losses_bioe_for_elec_ej',
    'Xwater for OXM X wind offshoreX':
    'xwater_for_oxm_x_wind_offshorex',
    'Total materials required for RES elec Mt':
    'total_materials_required_for_res_elec_mt',
    'share energy for material consumption for alt techn vs TFEC':
    'share_energy_for_material_consumption_for_alt_techn_vs_tfec',
    'Xwater for OXM X PVX':
    'xwater_for_oxm_x_pvx',
    'Xwater for OXM X wind onshoreX':
    'xwater_for_oxm_x_wind_onshorex',
    'Xmaterials for OXM per capacity installed X wind offshoreX':
    'xmaterials_for_oxm_per_capacity_installed_x_wind_offshorex',
    'Xmaterials for OXM per capacity installed X wind onshoreX':
    'xmaterials_for_oxm_per_capacity_installed_x_wind_onshorex',
    'Xmaterials for OXM per capacity installed X PVX':
    'xmaterials_for_oxm_per_capacity_installed_x_pvx',
    'Xwater for OXM X CSPX':
    'xwater_for_oxm_x_cspx',
    'Xmaterials per capacity installed X PV 0X':
    'xmaterials_per_capacity_installed_x_pv_0x',
    'cum materials requirements for RES elec':
    'cum_materials_requirements_for_res_elec',
    'MJ per EJ':
    'mj_per_ej',
    'XInitial energy cons per unit of material cons XvirginXX':
    'xinitial_energy_cons_per_unit_of_material_cons_xvirginxx',
    'Xmaterials per new capacity installed X CSPX':
    'xmaterials_per_new_capacity_installed_x_cspx',
    'Xmaterials per new capacity installed X wind offshoreX':
    'xmaterials_per_new_capacity_installed_x_wind_offshorex',
    'Xmaterials per new capacity installed X wind onshoreX':
    'xmaterials_per_new_capacity_installed_x_wind_onshorex',
    'initial cumulated material requirements for RES elec 1995':
    'initial_cumulated_material_requirements_for_res_elec_1995',
    'Xmaterials per new capacity installed X PVX':
    'xmaterials_per_new_capacity_installed_x_pvx',
    'kg per Mt':
    'kg_per_mt',
    'M per T':
    'm_per_t',
    'Elec generation variable from RES TWh':
    'elec_generation_variable_from_res_twh',
    'copper for solar thermal':
    'copper_for_solar_thermal',
    'Copper demand for solar thermal':
    'copper_demand_for_solar_thermal',
    'Total gen losses demand for Heat plants EJ':
    'total_gen_losses_demand_for_heat_plants_ej',
    'Total gen losses demand for CHP plants EJ':
    'total_gen_losses_demand_for_chp_plants_ej',
    'new required capacity nuclear':
    'new_required_capacity_nuclear',
    'Xreplacement RES for heatXcomX':
    'xreplacement_res_for_heatxcomx',
    'Losses solar for heat':
    'losses_solar_for_heat',
    'Xpotential FES RES for heatXcom EJX':
    'xpotential_fes_res_for_heatxcom_ejx',
    'Xpotential FES RES for heatXcom TWhX':
    'xpotential_fes_res_for_heatxcom_twhx',
    'Cp RES for heat':
    'cp_res_for_heat',
    'Efficiency geothermal for heat':
    'efficiency_geothermal_for_heat',
    'Efficiency solar panels for heat':
    'efficiency_solar_panels_for_heat',
    'Max FE potential RES for heat':
    'max_fe_potential_res_for_heat',
    'P geothermal for heat':
    'p_geothermal_for_heat',
    'Xpast RES growth for heatXcomX':
    'xpast_res_growth_for_heatxcomx',
    'Xinstalled capacity RES heatXcom TWX':
    'xinstalled_capacity_res_heatxcom_twx',
    'Xinitial value RES for heatXcomX':
    'xinitial_value_res_for_heatxcomx',
    'P solar for heat':
    'p_solar_for_heat',
    'life time RES for heat':
    'life_time_res_for_heat',
    'Xwear RES capacity for heatXcom TWX':
    'xwear_res_capacity_for_heatxcom_twx',
    'Potential FE gen Elec fossil fuel CHP plants EJ':
    'potential_fe_gen_elec_fossil_fuel_chp_plants_ej',
    'efficiency Heat oil CHP plants':
    'efficiency_heat_oil_chp_plants',
    'FED heat liquids CHP plants EJ':
    'fed_heat_liquids_chp_plants_ej',
    'Potential FE gen Elec liquids CHP plants EJ':
    'potential_fe_gen_elec_liquids_chp_plants_ej',
    'efficiency Elec coal CHP plants':
    'efficiency_elec_coal_chp_plants',
    'efficiency Elec gas CHP plants':
    'efficiency_elec_gas_chp_plants',
    'efficiency Elec oil CHP plants':
    'efficiency_elec_oil_chp_plants',
    'Potential FE gen Elec coal CHP plants EJ':
    'potential_fe_gen_elec_coal_chp_plants_ej',
    'Potential FE gen Elec gas CHP plants EJ':
    'potential_fe_gen_elec_gas_chp_plants_ej',
    'FED heat gas CHP plants EJ':
    'fed_heat_gas_chp_plants_ej',
    'efficiency Heat gas CHP plants':
    'efficiency_heat_gas_chp_plants',
    'PED oil for CHP plants EJ':
    'ped_oil_for_chp_plants_ej',
    'XShare heatXcom CHP plants NRE vs NRE tot heatXcom generationX':
    'xshare_heatxcom_chp_plants_nre_vs_nre_tot_heatxcom_generationx',
    'efficiency Heat coal CHP plants':
    'efficiency_heat_coal_chp_plants',
    'FED heat coal CHP plants EJ':
    'fed_heat_coal_chp_plants_ej',
    'PED gas for CHP plants EJ':
    'ped_gas_for_chp_plants_ej',
    'PED coal for CHP plants EJ':
    'ped_coal_for_chp_plants_ej',
    'remaining potential CSP':
    'remaining_potential_csp',
    'PE CSP for Elec generation EJ':
    'pe_csp_for_elec_generation_ej',
    'Invest RES for Elec':
    'invest_res_for_elec',
    'power density CSP':
    'power_density_csp',
    'max potential RES elec TWe':
    'max_potential_res_elec_twe',
    'surface CSP Mha':
    'surface_csp_mha',
    'FE Elec generation from CSP TWh':
    'fe_elec_generation_from_csp_twh',
    'invest CSP Tdolar':
    'invest_csp_tdolar',
    'Cp limit nuclear':
    'cp_limit_nuclear',
    'invest nuclear Tdolar':
    'invest_nuclear_tdolar',
    'Cp nuclear':
    'cp_nuclear',
    'installed capacity nuclear TW':
    'installed_capacity_nuclear_tw',
    'Xnuclear capacity phaseXoutX':
    'xnuclear_capacity_phasexoutx',
    'min Cp nuclear':
    'min_cp_nuclear',
    'replacement nuclear capacity':
    'replacement_nuclear_capacity',
    'min Cp baseload RES':
    'min_cp_baseload_res',
    'abundance uranium':
    'abundance_uranium',
    'nuclear overcapacity':
    'nuclear_overcapacity',
    'XP nuclear scen3X4X':
    'xp_nuclear_scen3x4x',
    'Xstart year nuclear growth scen3X4X':
    'xstart_year_nuclear_growth_scen3x4x',
    'P nuclear elec gen':
    'p_nuclear_elec_gen',
    'required capacity nuclear TW':
    'required_capacity_nuclear_tw',
    'time planification nuclear':
    'time_planification_nuclear',
    'time construction nuclear':
    'time_construction_nuclear',
    'initial capacity installed nuclear':
    'initial_capacity_installed_nuclear',
    'Nuclear capacity under construction':
    'nuclear_capacity_under_construction',
    'new nuclear capacity under planning':
    'new_nuclear_capacity_under_planning',
    'selection of nuclear scenario':
    'selection_of_nuclear_scenario',
    'initial gen nuclear':
    'initial_gen_nuclear',
    'initial capacity in construction nuclear':
    'initial_capacity_in_construction_nuclear',
    'initial required capacity nuclear':
    'initial_required_capacity_nuclear',
    'Planned nuclear capacity TW':
    'planned_nuclear_capacity_tw',
    'wear nuclear':
    'wear_nuclear',
    'effects shortage uranium':
    'effects_shortage_uranium',
    'TPES intensity EJ Tdolar':
    'tpes_intensity_ej_tdolar',
    'real PED intensity of Electricity':
    'real_ped_intensity_of_electricity',
    'Annual GDP growth rate':
    'annual_gdp_growth_rate',
    'Historic energy intensity':
    'historic_energy_intensity',
    'GDP delayed 1yr':
    'gdp_delayed_1yr',
    'share tot monet invest Elec RES vs GDP':
    'share_tot_monet_invest_elec_res_vs_gdp',
    'real PED intensity of Transport':
    'real_ped_intensity_of_transport',
    'Climate Feedback Param':
    'climate_feedback_param',
    'Feedback Cooling':
    'feedback_cooling',
    'GDPpc':
    'gdppc',
    'Xunlimited coalXX':
    'xunlimited_coalxx',
    'Real TFEC':
    'real_tfec',
    'Xunlimited uraniumXX':
    'xunlimited_uraniumxx',
    'URR coal unlimited':
    'urr_coal_unlimited',
    'URR uranium unlimited':
    'urr_uranium_unlimited',
    'URR conv gas unlimited':
    'urr_conv_gas_unlimited',
    'Xunlimited gasXX':
    'xunlimited_gasxx',
    'Xunlimited oilXX':
    'xunlimited_oilxx',
    'ELF concentrations logistic':
    'elf_concentrations_logistic',
    'URR conv oil unlimited':
    'urr_conv_oil_unlimited',
    'b logistic':
    'b_logistic',
    'a logistic':
    'a_logistic',
    'selection shape ELF':
    'selection_shape_elf',
    'Xactivate ELF by scenXX':
    'xactivate_elf_by_scenxx',
    'Method CC impacts':
    'method_cc_impacts',
    'XTotal demand liquids mbXdX':
    'xtotal_demand_liquids_mbxdx',
    'PED coal without CTL':
    'ped_coal_without_ctl',
    'share liquids for Transp':
    'share_liquids_for_transp',
    'PES for Transp EJ':
    'pes_for_transp_ej',
    'surface hydro Mha':
    'surface_hydro_mha',
    'remaining potential hydro':
    'remaining_potential_hydro',
    'Grid reinforcement costs Tdollar':
    'grid_reinforcement_costs_tdollar',
    'remaining potential onshore wind':
    'remaining_potential_onshore_wind',
    'remaining potential offshore wind':
    'remaining_potential_offshore_wind',
    'remaining potential BioE':
    'remaining_potential_bioe',
    'remaining potential oceanic':
    'remaining_potential_oceanic',
    'Xremaining potential geotXelecX':
    'xremaining_potential_geotxelecx',
    'Xremaining potential solarXelec PVX':
    'xremaining_potential_solarxelec_pvx',
    'XHistoric nonXenergy useX SUBliquids':
    'xhistoric_nonxenergy_usex_subliquids',
    'XHistoric nonXenergy useX SUBsolids':
    'xhistoric_nonxenergy_usex_subsolids',
    'XHistoric nonXenergy useX SUBgases':
    'xhistoric_nonxenergy_usex_subgases',
    'XHistoric nonXenergy useX SUBelectricity':
    'xhistoric_nonxenergy_usex_subelectricity',
    'XHistoric nonXenergy useX SUBheat':
    'xhistoric_nonxenergy_usex_subheat',
    'Xinitial nonXenergy useX':
    'xinitial_nonxenergy_usex',
    'Real total output':
    'real_total_output',
    'Real demand':
    'real_demand',
    'max potential RES elec TWh':
    'max_potential_res_elec_twh',
    'PE onshore wind for Elec generation EJ':
    'pe_onshore_wind_for_elec_generation_ej',
    'FE Elec generation from solar PV TWh':
    'fe_elec_generation_from_solar_pv_twh',
    'power density solar PV':
    'power_density_solar_pv',
    'PE hydro for Elec generation EJ':
    'pe_hydro_for_elec_generation_ej',
    'invest bioW Tdolar':
    'invest_biow_tdolar',
    'FE Elec generation from hydro TWh':
    'fe_elec_generation_from_hydro_twh',
    'Xinvest geotXelec TdolarX':
    'xinvest_geotxelec_tdolarx',
    'invest hydro Tdolar':
    'invest_hydro_tdolar',
    'FE Elec generation from onshore wind TWh':
    'fe_elec_generation_from_onshore_wind_twh',
    'invest oceanic Tdolar':
    'invest_oceanic_tdolar',
    'invest onshore wind Tdolar':
    'invest_onshore_wind_tdolar',
    'invest offshore wind Tdolar':
    'invest_offshore_wind_tdolar',
    'XFE Elec generation from geotXelec TWhX':
    'xfe_elec_generation_from_geotxelec_twhx',
    'XPE geotXelec for Elec generation EJX':
    'xpe_geotxelec_for_elec_generation_ejx',
    'PE solar PV for Elec generation EJ':
    'pe_solar_pv_for_elec_generation_ej',
    'FE Elec generation from bioE TWh':
    'fe_elec_generation_from_bioe_twh',
    'PE oceanic for Elec generation EJ':
    'pe_oceanic_for_elec_generation_ej',
    'PE bioE for Elec generation EJ':
    'pe_bioe_for_elec_generation_ej',
    'FE Elec generation from offshore wind TWh':
    'fe_elec_generation_from_offshore_wind_twh',
    'FE Elec generation from oceanic TWh':
    'fe_elec_generation_from_oceanic_twh',
    'PE offshore wind for Elec generation EJ':
    'pe_offshore_wind_for_elec_generation_ej',
    'invest solar Tdolar':
    'invest_solar_tdolar',
    'initial value land compet biofuels 2gen Mha':
    'initial_value_land_compet_biofuels_2gen_mha',
    'initial value land compet biofuels 2gen ktoe':
    'initial_value_land_compet_biofuels_2gen_ktoe',
    'abundance coal':
    'abundance_coal',
    'abundance liquids':
    'abundance_liquids',
    'check liquids':
    'check_liquids',
    'efficiency liquids for heat plants':
    'efficiency_liquids_for_heat_plants',
    'Xshare gasXXcoalXgasX for heat plantsX':
    'xshare_gasxxcoalxgasx_for_heat_plantsx',
    'Share heat distribution losses':
    'share_heat_distribution_losses',
    'efficiency coal for heat plants':
    'efficiency_coal_for_heat_plants',
    'efficiency gases for heat plants':
    'efficiency_gases_for_heat_plants',
    'potential tot RES elec after intermitt':
    'potential_tot_res_elec_after_intermitt',
    'invest RES elec Tdolar':
    'invest_res_elec_tdolar',
    'PE BioW for Elec generation Mtoe':
    'pe_biow_for_elec_generation_mtoe',
    'remaining potential tot RES elec after intermitt':
    'remaining_potential_tot_res_elec_after_intermitt',
    'invest cost RES elec SUBhydro':
    'invest_cost_res_elec_subhydro',
    'invest cost RES elec SUBXgeotXelecX':
    'invest_cost_res_elec_subxgeotxelecx',
    'invest cost RES elec SUBXsolidbioEXelecX':
    'invest_cost_res_elec_subxsolidbioexelecx',
    'invest cost RES elec SUBoceanic':
    'invest_cost_res_elec_suboceanic',
    'invest cost RES elec SUBwindonshore':
    'invest_cost_res_elec_subwindonshore',
    'invest cost RES elec SUBwindoffshore':
    'invest_cost_res_elec_subwindoffshore',
    'invest cost RES elec SUBsolarPV':
    'invest_cost_res_elec_subsolarpv',
    'invest cost RES elec SUBCSP':
    'invest_cost_res_elec_subcsp',
    'Xpower density RES elec TWeXMhaX':
    'xpower_density_res_elec_twexmhax',
    'Xpower density RES elec TWXMhaX':
    'xpower_density_res_elec_twxmhax',
    'Time 95pc TS potential RES elec':
    'time_95pc_ts_potential_res_elec',
    'surface RES elec':
    'surface_res_elec',
    'Xmax geotXelec TWeX':
    'xmax_geotxelec_twex',
    'Xmax PE geotXelec TWthX':
    'xmax_pe_geotxelec_twthx',
    'Efficiency conversion geot PE to Elec':
    'efficiency_conversion_geot_pe_to_elec',
    'Geot PE potential for heat EJ':
    'geot_pe_potential_for_heat_ej',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subairtransport',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivatehouseholdswithemployedpersons',
    'Historic final energy intensity by sector and fuel SUBheat SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBheat SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBheat SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBheat SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBheat SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBheat SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBheat SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBheat SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBheat SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBheat SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBheat SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBheat SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBheat SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBheat SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBheat SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBheat SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBheat SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBheat SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction',
    'Historic final energy intensity by sector and fuel SUBheat SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBheat SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBheat SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBheat SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBheat SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBheat SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBheat SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subairtransport',
    'Historic final energy intensity by sector and fuel SUBheat SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBheat SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBheat SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBheat SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBheat SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBheat SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBheat SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation',
    'Historic final energy intensity by sector and fuel SUBheat SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBheat SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBheat SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivatehouseholdswithemployedpersons',
    'Historic final energy intensity by sector and fuel SUBliquids SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBliquids SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBliquids SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBliquids SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBliquids SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBliquids SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBliquids SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBliquids SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBliquids SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBliquids SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBliquids SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBliquids SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBliquids SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBliquids SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBliquids SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBliquids SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBliquids SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBliquids SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction',
    'Historic final energy intensity by sector and fuel SUBliquids SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBliquids SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBliquids SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBliquids SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBliquids SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBliquids SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBliquids SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subairtransport',
    'Historic final energy intensity by sector and fuel SUBliquids SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBliquids SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBliquids SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBliquids SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBliquids SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBliquids SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBliquids SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation',
    'Historic final energy intensity by sector and fuel SUBliquids SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBliquids SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBliquids SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivatehouseholdswithemployedpersons',
    'Historic final energy intensity by sector and fuel SUBgases SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBgases SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBgases SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBgases SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBgases SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBgases SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBgases SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBgases SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBgases SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBgases SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBgases SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBgases SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBgases SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBgases SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBgases SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBgases SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBgases SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBgases SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction',
    'Historic final energy intensity by sector and fuel SUBgases SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBgases SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBgases SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBgases SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBgases SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBgases SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBgases SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subairtransport',
    'Historic final energy intensity by sector and fuel SUBgases SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBgases SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBgases SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBgases SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBgases SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBgases SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBgases SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation',
    'Historic final energy intensity by sector and fuel SUBgases SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBgases SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBgases SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivatehouseholdswithemployedpersons',
    'Historic final energy intensity by sector and fuel SUBsolids SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBsolids SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBsolids SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBsolids SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBsolids SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBsolids SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBsolids SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBsolids SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBsolids SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBsolids SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBsolids SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBsolids SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBsolids SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBsolids SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBsolids SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBsolids SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBsolids SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBsolids SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction',
    'Historic final energy intensity by sector and fuel SUBsolids SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBsolids SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBsolids SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBsolids SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBsolids SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBsolids SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBsolids SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subairtransport',
    'Historic final energy intensity by sector and fuel SUBsolids SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBsolids SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBsolids SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBsolids SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBsolids SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBsolids SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBsolids SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation',
    'Historic final energy intensity by sector and fuel SUBsolids SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBsolids SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBsolids SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivatehouseholdswithemployedpersons',
    'initial instal cap RES elec':
    'initial_instal_cap_res_elec',
    'abundance RES elec2':
    'abundance_res_elec2',
    'potential tot generation RES elec TWh':
    'potential_tot_generation_res_elec_twh',
    'RES elec tot overcapacity':
    'res_elec_tot_overcapacity',
    'wear RES elec':
    'wear_res_elec',
    'XCpXini RES elecX SUBhydro':
    'xcpxini_res_elecx_subhydro',
    'XCpXini RES elecX SUBXgeotXelecX':
    'xcpxini_res_elecx_subxgeotxelecx',
    'XCpXini RES elecX SUBXsolidbioEXelecX':
    'xcpxini_res_elecx_subxsolidbioexelecx',
    'XCpXini RES elecX SUBoceanic':
    'xcpxini_res_elecx_suboceanic',
    'XCpXini RES elecX SUBwindonshore':
    'xcpxini_res_elecx_subwindonshore',
    'XCpXini RES elecX SUBwindoffshore':
    'xcpxini_res_elecx_subwindoffshore',
    'XCpXini RES elecX SUBsolarPV':
    'xcpxini_res_elecx_subsolarpv',
    'XCpXini RES elecX SUBCSP':
    'xcpxini_res_elecx_subcsp',
    'G per T':
    'g_per_t',
    'time construction RES elec':
    'time_construction_res_elec',
    'time planification RES elec':
    'time_planification_res_elec',
    'initial required capacity RES elec':
    'initial_required_capacity_res_elec',
    'new RES elec capacity under planning':
    'new_res_elec_capacity_under_planning',
    'lifetime RES elec':
    'lifetime_res_elec',
    'past RES elec capacity growth':
    'past_res_elec_capacity_growth',
    'Afforestation program 2020 GtCO2':
    'afforestation_program_2020_gtco2',
    'Mt per Gt':
    'mt_per_gt',
    'GTL efficiency':
    'gtl_efficiency',
    'CTL efficiency':
    'ctl_efficiency',
    'XCTLXGTL GbX':
    'xctlxgtl_gbx',
    'XMbXd per EJXyearX':
    'xmbxd_per_ejxyearx',
    'max extraction uranium EJ':
    'max_extraction_uranium_ej',
    'Hist growth CTL':
    'hist_growth_ctl',
    'Gboe per EJ':
    'gboe_per_ej',
    'table max extraction uranium user defined':
    'table_max_extraction_uranium_user_defined',
    'XUserXdefined extraction growth unconv gasX':
    'xuserxdefined_extraction_growth_unconv_gasx',
    'XUserXdefined extraction growth unconv oilX':
    'xuserxdefined_extraction_growth_unconv_oilx',
    'Hist growth GTL':
    'hist_growth_gtl',
    'URR uranium User defined':
    'urr_uranium_user_defined',
    'replacement rate nuclear':
    'replacement_rate_nuclear',
    'demand gas for oil refinery gains':
    'demand_gas_for_oil_refinery_gains',
    'share RES vs TPES':
    'share_res_vs_tpes',
    'TPES Mtoe':
    'tpes_mtoe',
    'FE Elec generation from NRE TWh':
    'fe_elec_generation_from_nre_twh',
    'check Transp':
    'check_transp',
    'PEpc consumption people depending on trad biomass':
    'pepc_consumption_people_depending_on_trad_biomass',
    'share gas for oil refinery gains':
    'share_gas_for_oil_refinery_gains',
    'People relying trad biomass ref':
    'people_relying_trad_biomass_ref',
    'PE consumption trad biomass ref':
    'pe_consumption_trad_biomass_ref',
    'TPEFpc threshold high development':
    'tpefpc_threshold_high_development',
    'TPED acceptable standard living':
    'tped_acceptable_standard_living',
    'Pop not dependent on trad biomass':
    'pop_not_dependent_on_trad_biomass',
    'XCarbon footprint tCO2XpersonX':
    'xcarbon_footprint_tco2xpersonx',
    'XAverage TPESpc Xwithout trad biomassXX':
    'xaverage_tpespc_xwithout_trad_biomassxx',
    'XTPES Xwithout trad biomassXX':
    'xtpes_xwithout_trad_biomassxx',
    'Historic electricity consumption':
    'historic_electricity_consumption',
    'Max NPP potential BioE residues for heat and elec':
    'max_npp_potential_bioe_residues_for_heat_and_elec',
    'Max NPP potential BioE residues for cellulosic biofuels':
    'max_npp_potential_bioe_residues_for_cellulosic_biofuels',
    'Total Carbon Emisions':
    'total_carbon_emisions',
    'init CO2 in Atmos ppm':
    'init_co2_in_atmos_ppm',
    'Transfer coefficient for lower level':
    'transfer_coefficient_for_lower_level',
    'Total cumulative emissions GtCO2':
    'total_cumulative_emissions_gtco2',
    'A UO Heat Cap':
    'a_uo_heat_cap',
    'init CO2 in Atmos':
    'init_co2_in_atmos',
    'GtCO2 historic emissions RCPs':
    'gtco2_historic_emissions_rcps',
    'Heat Trans Coeff':
    'heat_trans_coeff',
    'Carbon emissions GtC':
    'carbon_emissions_gtc',
    'Climate equation coefficient for upper level':
    'climate_equation_coefficient_for_upper_level',
    'Historic unconv oil':
    'historic_unconv_oil',
    'gCO2 per MJ unconv gas':
    'gco2_per_mj_unconv_gas',
    'gCO2 per MJ unconv oil':
    'gco2_per_mj_unconv_oil',
    'CO2 emissions CTL':
    'co2_emissions_ctl',
    'gCO2 per MJ conv gas':
    'gco2_per_mj_conv_gas',
    'gCO2 per MJ CTL':
    'gco2_per_mj_ctl',
    'gCO2 per MJ GTL':
    'gco2_per_mj_gtl',
    'Historic unconv gas':
    'historic_unconv_gas',
    'gCO2 per MJ coal':
    'gco2_per_mj_coal',
    'gCO2 per MJ conv oil':
    'gco2_per_mj_conv_oil',
    'gCO2 per MJ shale oil':
    'gco2_per_mj_shale_oil',
    'technological max pct of change':
    'technological_max_pct_of_change',
    'K liquid Elec BEV TWh EJ':
    'k_liquid_elec_bev_twh_ej',
    'HEV fraction of share available':
    'hev_fraction_of_share_available',
    'BEV fraction of share available':
    'bev_fraction_of_share_available',
    'dollars to Tdollars':
    'dollars_to_tdollars',
    'variation historic pop':
    'variation_historic_pop',
    'Historic pop':
    'historic_pop',
    'land compet 2gen vs total land compet':
    'land_compet_2gen_vs_total_land_compet',
    'GJ per EJ':
    'gj_per_ej',
    'Land shifted to biofuels 3gen':
    'land_shifted_to_biofuels_3gen',
    'Average TPES per capita':
    'average_tpes_per_capita',
    'Geot PE potential for heat TWth':
    'geot_pe_potential_for_heat_twth',
    'PE losses uranium for Elec EJ':
    'pe_losses_uranium_for_elec_ej',
    'PE demand Transport after policies Mtoe':
    'pe_demand_transport_after_policies_mtoe',
    'share RES for Elec vs TPE RES':
    'share_res_for_elec_vs_tpe_res',
    'XTPES de Castro PhD X Scen XmadcoalX EJX':
    'xtpes_de_castro_phd_x_scen_xmadcoalx_ejx',
    'extraction coal Mtoe':
    'extraction_coal_mtoe',
    'max extraction coal Mtoe':
    'max_extraction_coal_mtoe',
    'TPE from RES EJ':
    'tpe_from_res_ej',
    'TPE from RES Mtoe':
    'tpe_from_res_mtoe',
    'XTPES de Castro PhD X Scen II EJX':
    'xtpes_de_castro_phd_x_scen_ii_ejx',
    'Electrical distribution losses EJ':
    'electrical_distribution_losses_ej',
    'Efficiency conversion BioE plants to heat':
    'efficiency_conversion_bioe_plants_to_heat',
    'Efficiency improvement biofuels 3gen':
    'efficiency_improvement_biofuels_3gen',
    'Potential PEavail biofuels 2gen land compet EJ':
    'potential_peavail_biofuels_2gen_land_compet_ej',
    'Annual additional historic land use biofuels 2gen':
    'annual_additional_historic_land_use_biofuels_2gen',
    'Max NPP potential bioE residues':
    'max_npp_potential_bioe_residues',
    'Grid reinforcement costs':
    'grid_reinforcement_costs',
    'Balancing costs':
    'balancing_costs',
    'Historic land compet available for biofuels 2gen':
    'historic_land_compet_available_for_biofuels_2gen',
    'Land productivity biofuels 2gen EJ MHa':
    'land_productivity_biofuels_2gen_ej_mha',
    'XMax PEavail potential biofuels 2X3genX':
    'xmax_peavail_potential_biofuels_2x3genx',
    'Balancing costs ref':
    'balancing_costs_ref',
    'Land compet biofuels 2gen Mha':
    'land_compet_biofuels_2gen_mha',
    'efficiency conversion bioE to Elec':
    'efficiency_conversion_bioe_to_elec',
    'Potential PEavail biofuels prod 3gen EJ':
    'potential_peavail_biofuels_prod_3gen_ej',
    'Max PEavail potential bioE residues for cellulosic biofuels':
    'max_peavail_potential_bioe_residues_for_cellulosic_biofuels',
    'P biofuels 3gen':
    'p_biofuels_3gen',
    'Xconstrain gas exogenous growthX delayed 1yrX':
    'xconstrain_gas_exogenous_growthx_delayed_1yrx',
    'Xconstrain gas exogenous growthXX':
    'xconstrain_gas_exogenous_growthxx',
    'Xconstrain liquids exogenous growthX delayed 1yrX':
    'xconstrain_liquids_exogenous_growthx_delayed_1yrx',
    'check liquids delayed 1yr':
    'check_liquids_delayed_1yr',
    'check gas delayed 1yr':
    'check_gas_delayed_1yr',
    'Xconstrain liquids exogenous growthXX':
    'xconstrain_liquids_exogenous_growthxx',
    'check hist RES':
    'check_hist_res',
    'Historic total RES extraction':
    'historic_total_res_extraction',
    'Max potential NPP bioE conventional for heat':
    'max_potential_npp_bioe_conventional_for_heat',
    'Share variable RES elec vs total generation delayed 1yr':
    'share_variable_res_elec_vs_total_generation_delayed_1yr',
    'Xovercapacity vsX intermittent RES penetration 0X':
    'xovercapacity_vsx_intermittent_res_penetration_0x',
    'XTotal GHG Emissions Excluding LandXUse Change and ForestryX':
    'xtotal_ghg_emissions_excluding_landxuse_change_and_forestryx',
    'XTotal GHG Emissions Including LandXUse Change and ForestryX':
    'xtotal_ghg_emissions_including_landxuse_change_and_forestryx',
    'check hist emissions':
    'check_hist_emissions',
    'check hist liquids':
    'check_hist_liquids',
    'check hist natural gas':
    'check_hist_natural_gas',
    'Total CO2 emissions from fossil fuel consumption and cement production':
    'total_co2_emissions_from_fossil_fuel_consumption_and_cement_production',
    'check hist coal':
    'check_hist_coal',
    'Total electrical losses EJ':
    'total_electrical_losses_ej',
    'Gen losses vs PE for elec':
    'gen_losses_vs_pe_for_elec',
    'initial GTL production':
    'initial_gtl_production',
    'variation CTL':
    'variation_ctl',
    'variation GTL':
    'variation_gtl',
    'Historic GTL production':
    'historic_gtl_production',
    'initial CTL production':
    'initial_ctl_production',
    'Historic CTL production':
    'historic_ctl_production',
    'improvement efficiency gas for electricity':
    'improvement_efficiency_gas_for_electricity',
    'PE demand gas Elec plants EJ':
    'pe_demand_gas_elec_plants_ej',
    'PE demand coal Elec plants EJ':
    'pe_demand_coal_elec_plants_ej',
    'Max efficiency gas power plants':
    'max_efficiency_gas_power_plants',
    'Total PE for electricity consumption EJ':
    'total_pe_for_electricity_consumption_ej',
    'initial efficiency gas for electricity':
    'initial_efficiency_gas_for_electricity',
    'remaining efficiency improv gas for electricity':
    'remaining_efficiency_improv_gas_for_electricity',
    'Efficiency improv gas for electricity':
    'efficiency_improv_gas_for_electricity',
    'efficiency gas for electricity':
    'efficiency_gas_for_electricity',
    'percent to share':
    'percent_to_share',
    'XAdditional PE production of CTLXGTL for liquidsX':
    'xadditional_pe_production_of_ctlxgtl_for_liquidsx',
    'Annual additional historic product biofuels 2gen':
    'annual_additional_historic_product_biofuels_2gen',
    'Historic produc biofuels 2gen':
    'historic_produc_biofuels_2gen',
    'NGV adapt growth':
    'ngv_adapt_growth',
    'Historical share NGVs':
    'historical_share_ngvs',
    'variation share NGV':
    'variation_share_ngv',
    'HEV adapt growth':
    'hev_adapt_growth',
    'BEV adapt growth':
    'bev_adapt_growth',
    'carbon budget':
    'carbon_budget',
    'Adapt CO2 emissions unconv gas':
    'adapt_co2_emissions_unconv_gas',
    'Cumulative emissions to 1995':
    'cumulative_emissions_to_1995',
    'variation share BEV':
    'variation_share_bev',
    'variation share HEV':
    'variation_share_hev',
    'XHistorical share BEVXHEVX':
    'xhistorical_share_bevxhevx',
    'FE solar potential for heat':
    'fe_solar_potential_for_heat',
    'Xstart year BioE residues for heatXelecX':
    'xstart_year_bioe_residues_for_heatxelecx',
    'CO2 fossil fuel emissions':
    'co2_fossil_fuel_emissions',
    't per Gt':
    't_per_gt',
    'Additional increase PE demand for Transp Elec EJ':
    'additional_increase_pe_demand_for_transp_elec_ej',
    'Temp change 2C':
    'temp_change_2c',
    'PE demand Transport after policies EJ':
    'pe_demand_transport_after_policies_ej',
    'PE liquids supply for Transp EJ':
    'pe_liquids_supply_for_transp_ej',
    'Share land compet biofuels':
    'share_land_compet_biofuels',
    'share land total RES vs arable':
    'share_land_total_res_vs_arable',
    'share land RES land compet vs arable':
    'share_land_res_land_compet_vs_arable',
    'Year scarcity oil':
    'year_scarcity_oil',
    'CO2 Net Emiss':
    'co2_net_emiss',
    'Xshare gasXXcoal XgasX for ElecX':
    'xshare_gasxxcoal_xgasx_for_elecx',
    'Xfuture share gasXXcoalXgasX for ElecX':
    'xfuture_share_gasxxcoalxgasx_for_elecx',
    'cumulated conv gas extraction':
    'cumulated_conv_gas_extraction',
    'cumulated conv gas extraction to 1995':
    'cumulated_conv_gas_extraction_to_1995',
    'Cumulated unconv gas extraction':
    'cumulated_unconv_gas_extraction',
    'cumulated unconv gas extraction to 1995':
    'cumulated_unconv_gas_extraction_to_1995',
    'Marginal Atmos Retention':
    'marginal_atmos_retention',
    'RURR uranium':
    'rurr_uranium',
    'cumulated uranium extraction to 1995':
    'cumulated_uranium_extraction_to_1995',
    'cumulated conv oil extraction':
    'cumulated_conv_oil_extraction',
    'cumulated conv oil extraction to 1995':
    'cumulated_conv_oil_extraction_to_1995',
    'cumulated unconv oil extraction':
    'cumulated_unconv_oil_extraction',
    'Cumulated coal extraction':
    'cumulated_coal_extraction',
    'cumulated coal extraction to 1995':
    'cumulated_coal_extraction_to_1995',
    'cumulated unconv oil extraction to 1995':
    'cumulated_unconv_oil_extraction_to_1995',
    'Cumulated uranium extraction':
    'cumulated_uranium_extraction',
    'Oil refinery gains EJ':
    'oil_refinery_gains_ej',
    'Oil refinery gains share':
    'oil_refinery_gains_share',
    'Efficiency gas for oil refinery gains':
    'efficiency_gas_for_oil_refinery_gains',
    'PES Liquids EJ':
    'pes_liquids_ej',
    'XCO2 landXuse change emissionsX':
    'xco2_landxuse_change_emissionsx',
    'Temperature change':
    'temperature_change',
    'Temp Diff':
    'temp_diff',
    'CO2 Rad Forcing':
    'co2_rad_forcing',
    'Atmos UOcean Temp':
    'atmos_uocean_temp',
    'Chg A UO Temp':
    'chg_a_uo_temp',
    'Heat Transfer':
    'heat_transfer',
    'Deep Ocean Temp':
    'deep_ocean_temp',
    'Chg DO Temp':
    'chg_do_temp',
    'Climate Sensitivity':
    'climate_sensitivity',
    'DO Heat Cap':
    'do_heat_cap',
    'init Atmos UOcean Temp':
    'init_atmos_uocean_temp',
    'CO2 Rad Force Coeff':
    'co2_rad_force_coeff',
    'Heat Capacity Ratio':
    'heat_capacity_ratio',
    'Other GHG Rad Forcing':
    'other_ghg_rad_forcing',
    'one year':
    'one_year',
    'init Deep Ocean Temp':
    'init_deep_ocean_temp',
    'CO2 Storage':
    'co2_storage',
    'Preindustrial CO2':
    'preindustrial_co2',
    'Average Atmos Retention':
    'average_atmos_retention',
    'CO2 in Atmos':
    'co2_in_atmos',
    'CO2 ppm concentrations':
    'co2_ppm_concentrations',
    'Rate of CO2 Transfer':
    'rate_of_co2_transfer',
    'Land compet required dedicated crops for biofuels':
    'land_compet_required_dedicated_crops_for_biofuels',
    'new biofuels 2gen land compet':
    'new_biofuels_2gen_land_compet',
    'Annual shift from 2gen to 3gen':
    'annual_shift_from_2gen_to_3gen',
    'Land compet biofuels 3gen Mha':
    'land_compet_biofuels_3gen_mha',
    'XP bioE residues for heatXelecX':
    'xp_bioe_residues_for_heatxelecx',
    'Potential PE cellulosic biofuel EJ':
    'potential_pe_cellulosic_biofuel_ej',
    'Cellulosic biofuels available':
    'cellulosic_biofuels_available',
    'XBioE residues for heatXelec availableX':
    'xbioe_residues_for_heatxelec_availablex',
    'XPE bioE residues for heatXelec EJX':
    'xpe_bioe_residues_for_heatxelec_ejx',
    'P cellulosic biofuels':
    'p_cellulosic_biofuels',
    'share cellulosic biofuels vs BioE residues':
    'share_cellulosic_biofuels_vs_bioe_residues',
    'start year cellulosic biofuels':
    'start_year_cellulosic_biofuels',
    'share land total RES vs urban surface':
    'share_land_total_res_vs_urban_surface',
    'Max land compet biofuels 2gen':
    'max_land_compet_biofuels_2gen',
    'P biofuels 2gen':
    'p_biofuels_2gen',
    'past biofuels 2gen':
    'past_biofuels_2gen',
    'adapt growth biofuels 2gen':
    'adapt_growth_biofuels_2gen',
    'EJ per ktoe':
    'ej_per_ktoe',
    'abundance total oil':
    'abundance_total_oil',
    'Cp nuclear initial':
    'cp_nuclear_initial',
    'extraction coal without CTL EJ':
    'extraction_coal_without_ctl_ej',
    'extraction coal for CTL EJ':
    'extraction_coal_for_ctl_ej',
    'Other liquids supply EJ':
    'other_liquids_supply_ej',
    'max offshore wind TWe':
    'max_offshore_wind_twe',
    'URR unconv oil High Mohr15':
    'urr_unconv_oil_high_mohr15',
    'URR unconv oil Low Mohr15':
    'urr_unconv_oil_low_mohr15',
    'URR unconv oil User defined':
    'urr_unconv_oil_user_defined',
    'URR uranium Zittel12':
    'urr_uranium_zittel12',
    'URR conv oil Maggio12 High':
    'urr_conv_oil_maggio12_high',
    'URR conv oil Maggio12 Low':
    'urr_conv_oil_maggio12_low',
    'table max extraction tot agg oil Laherrere 2006':
    'table_max_extraction_tot_agg_oil_laherrere_2006',
    'table max extraction Maggio12 High conv oil EJ':
    'table_max_extraction_maggio12_high_conv_oil_ej',
    'table max extraction Maggio12 Low con oil EJ':
    'table_max_extraction_maggio12_low_con_oil_ej',
    'table max extraction Maggio12middle conv oil EJ':
    'table_max_extraction_maggio12middle_conv_oil_ej',
    'Xseparate conv and unconv gasXX':
    'xseparate_conv_and_unconv_gasxx',
    'URR tot agg oil Laherrere 2006':
    'urr_tot_agg_oil_laherrere_2006',
    'table max extraction unconv oil User defined':
    'table_max_extraction_unconv_oil_user_defined',
    'table max extraction uranium Zittel12':
    'table_max_extraction_uranium_zittel12',
    'URR tot agg oil User defined':
    'urr_tot_agg_oil_user_defined',
    'choose extraction curve tot agg oil':
    'choose_extraction_curve_tot_agg_oil',
    'URR conv oil Maggio12 middle':
    'urr_conv_oil_maggio12_middle',
    'URR conv oil User defined':
    'urr_conv_oil_user_defined',
    'choose extraction tot agg gas curve':
    'choose_extraction_tot_agg_gas_curve',
    'table max extraction unconv oil Low Mohr15':
    'table_max_extraction_unconv_oil_low_mohr15',
    'URR unconv oil BG Mohr15':
    'urr_unconv_oil_bg_mohr15',
    'table max extraction unconv oil High Mohr15':
    'table_max_extraction_unconv_oil_high_mohr15',
    'table max extraction tot agg oil User defined':
    'table_max_extraction_tot_agg_oil_user_defined',
    'table max conv oil extraction User defined':
    'table_max_conv_oil_extraction_user_defined',
    'Xseparate conv and unconv oilXX':
    'xseparate_conv_and_unconv_oilxx',
    'URR uranium EWG13':
    'urr_uranium_ewg13',
    'table max extraction uranium EWG13 EJ':
    'table_max_extraction_uranium_ewg13_ej',
    'kt uranium per EJ':
    'kt_uranium_per_ej',
    'life time nuclear':
    'life_time_nuclear',
    'surface solar PV Mha':
    'surface_solar_pv_mha',
    'Hist share oil Elec':
    'hist_share_oil_elec',
    'Cumulated uranium extraction kt':
    'cumulated_uranium_extraction_kt',
    'extraction uranium kt':
    'extraction_uranium_kt',
    'efficiency uranium for electricity':
    'efficiency_uranium_for_electricity',
    'XHist share gasXXcoal XgasX ElecX':
    'xhist_share_gasxxcoal_xgasx_elecx',
    'Historic nuclear generation TWh':
    'historic_nuclear_generation_twh',
    'invest cost nuclear':
    'invest_cost_nuclear',
    'FE nuclear Elec generation TWh':
    'fe_nuclear_elec_generation_twh',
    'kWh per TWh':
    'kwh_per_twh',
    'Total FE Elec demand EJ':
    'total_fe_elec_demand_ej',
    'gCO2e per KWh biomass':
    'gco2e_per_kwh_biomass',
    'MToe per EJ':
    'mtoe_per_ej',
    'Xshare transmXdistr elec losses initialX':
    'xshare_transmxdistr_elec_losses_initialx',
    'EJ per TWh':
    'ej_per_twh',
    'surface onshore wind Mha':
    'surface_onshore_wind_mha',
    'TWe per TWh':
    'twe_per_twh',
    'initial population':
    'initial_population',
    'Population':
    'population',
    'VE objetive UE2020 extrap':
    've_objetive_ue2020_extrap',
    'VE objetive UE2020':
    've_objetive_ue2020',
    'Choose extraction uranium curve':
    'choose_extraction_uranium_curve',
    'P NGV growth':
    'p_ngv_growth',
    'NGV past share growth':
    'ngv_past_share_growth',
    'XBEVXHEV past share growthX':
    'xbevxhev_past_share_growthx',
    'NGV share':
    'ngv_share',
    'HEV share':
    'hev_share',
    'BEV share':
    'bev_share',
    'Year scarcity TPE':
    'year_scarcity_tpe',
    'Year scarcity uranium':
    'year_scarcity_uranium',
    'Year scarcity coal':
    'year_scarcity_coal',
    'abundance transport':
    'abundance_transport',
    'Year scarcity Transp':
    'year_scarcity_transp',
    'Year scarcity liquids':
    'year_scarcity_liquids',
    'Xeffects shortage elec on BEVXHEVX':
    'xeffects_shortage_elec_on_bevxhevx',
    'effects shortage gas':
    'effects_shortage_gas',
    'NGV fraction of change available':
    'ngv_fraction_of_change_available',
    'Year scarcity gases':
    'year_scarcity_gases',
    'P BEV growth':
    'p_bev_growth',
    'max NGV percent of change':
    'max_ngv_percent_of_change',
    'max percent of change':
    'max_percent_of_change',
    'choose extraction coal curve':
    'choose_extraction_coal_curve',
    'Selection constraint extraction unconv gas':
    'selection_constraint_extraction_unconv_gas',
    'Year scarcity Elec':
    'year_scarcity_elec',
    'extraction unconv gas delayed':
    'extraction_unconv_gas_delayed',
    'max unconv gas growth extraction EJ':
    'max_unconv_gas_growth_extraction_ej',
    'P constraint growth extraction unconv gas':
    'p_constraint_growth_extraction_unconv_gas',
    'choose extraction curve unconv gas':
    'choose_extraction_curve_unconv_gas',
    'choose extraction curve unconv oil':
    'choose_extraction_curve_unconv_oil',
    'total share EV':
    'total_share_ev',
    'P HEV growth':
    'p_hev_growth',
    'Biofuels land compet available':
    'biofuels_land_compet_available',
    'URR oil ASPO':
    'urr_oil_aspo',
    'URR gas Mohr High2013':
    'urr_gas_mohr_high2013',
    'URR gas Leherrere2010':
    'urr_gas_leherrere2010',
    'URR gas Mohr BG2012':
    'urr_gas_mohr_bg2012',
    'RES to fossil accounting':
    'res_to_fossil_accounting',
    'new C GtC':
    'new_c_gtc',
    'year adjust':
    'year_adjust',
    'XTPES de Castro PhD X Scen XmadcoalXX':
    'xtpes_de_castro_phd_x_scen_xmadcoalxx',
    'urban surface 2008':
    'urban_surface_2008',
    'pre industrial value ppm':
    'pre_industrial_value_ppm',
    'Historic TPES':
    'historic_tpes',
    'C per CO2':
    'c_per_co2',
    'Historic coal extraction':
    'historic_coal_extraction',
    'Historic natural gas extraction':
    'historic_natural_gas_extraction',
    'Historic liquids extraction':
    'historic_liquids_extraction',
    'Historic crude oil extraction':
    'historic_crude_oil_extraction',
    'GtC historic emissions RCPs':
    'gtc_historic_emissions_rcps',
    'activate Affores program':
    'activate_affores_program',
    'Adapt emissions shale oil':
    'adapt_emissions_shale_oil',
    'Afforestation program 2020':
    'afforestation_program_2020',
    'Global arable land':
    'global_arable_land',
    'choose extraction curve conv oil':
    'choose_extraction_curve_conv_oil',
    'choose extraction conv gas curve':
    'choose_extraction_conv_gas_curve',
    'Cumulated total monet invest RES for Elec':
    'cumulated_total_monet_invest_res_for_elec',
    'efficiency coal for electricity':
    'efficiency_coal_for_electricity',
    'Historic efficiency gas for electricity':
    'historic_efficiency_gas_for_electricity',
    'efficiency liquids for electricity':
    'efficiency_liquids_for_electricity',
    'Additional land compet available for biofuels':
    'additional_land_compet_available_for_biofuels',
    'max hydro TWe':
    'max_hydro_twe',
    'max oceanic TWe':
    'max_oceanic_twe',
    'max onshore wind TWe':
    'max_onshore_wind_twe',
    'P timeseries pop growth rate':
    'p_timeseries_pop_growth_rate',
    'P CTL':
    'p_ctl',
    'P GTL':
    'p_gtl',
    'ppm per GtC':
    'ppm_per_gtc',
    'start year 3gen':
    'start_year_3gen',
    'Time dmnl':
    'time_dmnl',
    'Share variable RES elec generation vs total':
    'share_variable_res_elec_generation_vs_total',
    'table max extraction ASPO oil EJ':
    'table_max_extraction_aspo_oil_ej',
    'table max extraction gas Laherrere2010':
    'table_max_extraction_gas_laherrere2010',
    'table max extraction gas Mohr High2012':
    'table_max_extraction_gas_mohr_high2012',
    'table max extraction gas Mohr BG2012':
    'table_max_extraction_gas_mohr_bg2012',
    'Total cumulative emissions GtC':
    'total_cumulative_emissions_gtc',
    'XTNES de Castro PhD X Scen IX':
    'xtnes_de_castro_phd_x_scen_ix',
    'XNet oil extraction de Castro PhD X Scen IX':
    'xnet_oil_extraction_de_castro_phd_x_scen_ix',
    'XPE oil extraction de Castro PhD X Scen IIX':
    'xpe_oil_extraction_de_castro_phd_x_scen_iix',
    'XNet oil extraction de Castro PhD X Scen IIIX':
    'xnet_oil_extraction_de_castro_phd_x_scen_iiix',
    'XPE coal extraction de Castro PhD X Scen IIX':
    'xpe_coal_extraction_de_castro_phd_x_scen_iix',
    'XNet gas extraction de Castro PhD X Scen IX':
    'xnet_gas_extraction_de_castro_phd_x_scen_ix',
    'XPE gas extraction de Castro PhD X Scen IIX':
    'xpe_gas_extraction_de_castro_phd_x_scen_iix',
    'XNet gas extraction de Castro PhD X Scen IIIX':
    'xnet_gas_extraction_de_castro_phd_x_scen_iiix',
    'XNet coal extraction de Castro PhD X Scen IIIX':
    'xnet_coal_extraction_de_castro_phd_x_scen_iiix',
    'XNet coal extraction de Castro PhD X Scen IX':
    'xnet_coal_extraction_de_castro_phd_x_scen_ix',
    'XTNES de Castro PhD X Scen IIIX':
    'xtnes_de_castro_phd_x_scen_iiix',
    'XPrimary coal extraction de Castro PhD X Scen XmadcoalXX':
    'xprimary_coal_extraction_de_castro_phd_x_scen_xmadcoalxx',
    'XTPES de Castro PhD X Scen IIX':
    'xtpes_de_castro_phd_x_scen_iix',
    'P nuclear scen 1':
    'p_nuclear_scen_1',
    'XP nuclear 2X3X':
    'xp_nuclear_2x3x',
    'gCO2e per GTCO2e 4':
    'gco2e_per_gtco2e_4',
    'gCO2e per GTCO2e 3':
    'gco2e_per_gtco2e_3',
    'gCO2e per GTCO2e 0':
    'gco2e_per_gtco2e_0',
    'gCO2e per GTCO2e 1':
    'gco2e_per_gtco2e_1',
    'TWh per gCO2e 0':
    'twh_per_gco2e_0',
    'Real demand by sector':
    'real_demand_by_sector',
    'Required total output by sector':
    'required_total_output_by_sector',
    'rate change intensity BOTTOM UP':
    'rate_change_intensity_bottom_up',
    'Evol final energy intensity by sector and fuel':
    'evol_final_energy_intensity_by_sector_and_fuel',
    'RURR unconv oil EJ':
    'rurr_unconv_oil_ej',
    'PE Elec for Transp EJ':
    'pe_elec_for_transp_ej',
    'Oil liquids saved by EVs EJ':
    'oil_liquids_saved_by_evs_ej',
    'Oil liquids saved by NGVs':
    'oil_liquids_saved_by_ngvs',
    'Total FE Elec demand TWh':
    'total_fe_elec_demand_twh',
    'share liquids for final energy':
    'share_liquids_for_final_energy',
    'PED liquids EJ':
    'ped_liquids_ej',
    'FED Heat liquids plants EJ':
    'fed_heat_liquids_plants_ej',
    'Real final energy by sector and fuel':
    'real_final_energy_by_sector_and_fuel',
    'hist var inlandT':
    'hist_var_inlandt',
    'inland transport variation intensity':
    'inland_transport_variation_intensity',
    'max unconv oil growth extraction':
    'max_unconv_oil_growth_extraction',
    'XFED HeatXcom NRE EJX':
    'xfed_heatxcom_nre_ejx',
    'real FE consumption by fuel':
    'real_fe_consumption_by_fuel',
    'Other liquids required EJ':
    'other_liquids_required_ej',
    'FE Elec demand consum after CC impacts EJ':
    'fe_elec_demand_consum_after_cc_impacts_ej',
    'Required FED by liquids EJ':
    'required_fed_by_liquids_ej',
    'Real GFCF':
    'real_gfcf',
    'Real Household demand':
    'real_household_demand',
    'extraction unconv oil EJ':
    'extraction_unconv_oil_ej',
    'Demand conv oil EJ':
    'demand_conv_oil_ej',
    'max unconv oil growth extraction EJ':
    'max_unconv_oil_growth_extraction_ej',
    'XFED HeatXcom EJX':
    'xfed_heatxcom_ejx',
    'FE demand Elec consum TWh':
    'fe_demand_elec_consum_twh',
    'XTotal FED HeatXcom EJX':
    'xtotal_fed_heatxcom_ejx',
    'Real total output by fuel and sector':
    'real_total_output_by_fuel_and_sector',
    'Required final energy by sector and fuel':
    'required_final_energy_by_sector_and_fuel',
    'URR unconv oil':
    'urr_unconv_oil',
    'max extraction unconv gas':
    'max_extraction_unconv_gas',
    'max extraction unconv oil':
    'max_extraction_unconv_oil',
    'max extraction coal EJ':
    'max_extraction_coal_ej',
    'max extraction conv gas EJ':
    'max_extraction_conv_gas_ej',
    'FED Heat gas plants EJ':
    'fed_heat_gas_plants_ej',
    'FED Heat coal plants EJ':
    'fed_heat_coal_plants_ej',
    'URR coal':
    'urr_coal',
    'URR conv gas':
    'urr_conv_gas',
    'Real demand by sector delayed':
    'real_demand_by_sector_delayed',
    'Required final energy other transport':
    'required_final_energy_other_transport',
    'Current trends demand Elec for transport':
    'current_trends_demand_elec_for_transport',
    'PED gases for Heat plants EJ':
    'ped_gases_for_heat_plants_ej',
    'PED oil for Heat plants EJ':
    'ped_oil_for_heat_plants_ej',
    'PED coal for Heat plants EJ':
    'ped_coal_for_heat_plants_ej',
    'Real total output by sector':
    'real_total_output_by_sector',
    'Required final energy air transport':
    'required_final_energy_air_transport',
    'Required final energy inland transport':
    'required_final_energy_inland_transport',
    'Required final energy water transport':
    'required_final_energy_water_transport',
    'table max extraction conv gas Low Mohr15':
    'table_max_extraction_conv_gas_low_mohr15',
    'table max extraction conv gas User defined':
    'table_max_extraction_conv_gas_user_defined',
    'table max extraction total gas BG Mohr12':
    'table_max_extraction_total_gas_bg_mohr12',
    'table max extraction total gas Laherrere10':
    'table_max_extraction_total_gas_laherrere10',
    'table max extraction total gas User defined':
    'table_max_extraction_total_gas_user_defined',
    'table max extraction unconv gas BG Mohr15':
    'table_max_extraction_unconv_gas_bg_mohr15',
    'table max extraction unconv gas High Mohr15':
    'table_max_extraction_unconv_gas_high_mohr15',
    'table max extraction unconv gas Low Mohr15':
    'table_max_extraction_unconv_gas_low_mohr15',
    'table max extraction unconv gas User defined':
    'table_max_extraction_unconv_gas_user_defined',
    'table max extraction coal BG Mohr15':
    'table_max_extraction_coal_bg_mohr15',
    'table max extraction coal High Mohr15':
    'table_max_extraction_coal_high_mohr15',
    'URR coal BG Mohr15':
    'urr_coal_bg_mohr15',
    'URR coal User defined EJ':
    'urr_coal_user_defined_ej',
    'URR conv gas BG Mohr15':
    'urr_conv_gas_bg_mohr15',
    'URR conv gas High Mohr15':
    'urr_conv_gas_high_mohr15',
    'URR conv gas Low Mohr15':
    'urr_conv_gas_low_mohr15',
    'URR conv gas User defined':
    'urr_conv_gas_user_defined',
    'URR total gas Laherrere10':
    'urr_total_gas_laherrere10',
    'URR total gas Mohr12 BG':
    'urr_total_gas_mohr12_bg',
    'URR total gas User defined':
    'urr_total_gas_user_defined',
    'URR unconv gas':
    'urr_unconv_gas',
    'URR unconv gas BG Mohr15':
    'urr_unconv_gas_bg_mohr15',
    'table max extraction coal Low Mohr15':
    'table_max_extraction_coal_low_mohr15',
    'table max extraction coal Mohr2012 EJ':
    'table_max_extraction_coal_mohr2012_ej',
    'table max extraction coal User defined':
    'table_max_extraction_coal_user_defined',
    'table max extraction conv gas BG Mohr15':
    'table_max_extraction_conv_gas_bg_mohr15',
    'table max extraction conv gas High Mohr15':
    'table_max_extraction_conv_gas_high_mohr15',
    'URR unconv gas Low Mohr15':
    'urr_unconv_gas_low_mohr15',
    'URR coal High15':
    'urr_coal_high15',
    'URR coal Low Mohr15':
    'urr_coal_low_mohr15',
    'URR unconv gas High Mohr15':
    'urr_unconv_gas_high_mohr15',
    'URR unconv gas User defined':
    'urr_unconv_gas_user_defined',
    'URR coal Mohr2012 EJ':
    'urr_coal_mohr2012_ej',
    'table max extraction unconv oil BG Mohr15':
    'table_max_extraction_unconv_oil_bg_mohr15',
    'Selection constraint extraction unconv oil':
    'selection_constraint_extraction_unconv_oil',
    'Abundance electricity':
    'abundance_electricity',
    'P constraint growth extraction unconv oil':
    'p_constraint_growth_extraction_unconv_oil',
    'extraction unconv oil delayed':
    'extraction_unconv_oil_delayed',
    'FINAL TIME':
    'final_time',
    'INITIAL TIME':
    'initial_time',
    'SAVEPER':
    'saveper',
    'TIME STEP':
    'time_step'
}

functions1 = set()

# ECON- Final demands by sector, households and fuel
@cache('step')
def leontief_matrix():
    """
    Leontief Matrix



    component


    """

    ret =  leontief_matrix_for_python()
    name = 0
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def ia_matrix():
    """
    IA Matrix



    component


    """

    ret =  ia_matrix_for_python()
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2002():
    """
    IA matrix 2002



    constant


    """

    ret =  xr.DataArray(
        data=[[0.131315829, 0.000933669, 0.25764361, 0.039438198, 0.036633454,
            0.129627905, 0.012123226, 0.001562256, 0.004955206, 0.0140601,
            0.000623449, 0.000271843, 0.000276016, 0.000188636, 0.000149733,
            0.009527471, 0.000510374, 0.010256609, 0.000455947, 0.003133054,
            0.002645024, 0.038678427, 0.001338086, 0.001279996, 0.000307578,
            0.003939451, 0.000252928, 0.000141304, 0.000371861, 0.000627869,
            0.001906118, 0.00201888, 0.002151984, 0.002334622, 0.015372458
        ], [0.00352914, 0.073187387, 0.002615202, 0.00201922, 0.000851768,
            0.003653669, 0.004388637, 0.442566481, 0.03182023, 0.002937557,
            0.064238016, 0.043989098, 0.00294532, 0.001278056, 0.001237646,
            0.006005501, 0.106270138, 0.017527625, 0.000449917, 0.000674029,
            0.00107797, 0.001571003, 0.003845481, 0.002403354, 0.002420279,
            0.001567511, 0.000495052, 0.000269923, 0.001652317, 0.000823667,
            0.002749329, 0.002997115, 0.000869258, 0.002451498, 0.061735655
        ], [0.054180465, 0.000643059, 0.147039624, 0.002677164, 0.082385461,
            0.001299553, 0.00274504, 0.000747645, 0.009352539, 0.00254585,
            0.001451857, 0.00091834, 0.001064716, 0.000907012, 0.000808241,
            0.002026328, 0.000706891, 0.000896595, 0.001586893, 0.004663312,
            0.004478962, 0.146896488, 0.001229752, 0.006262548, 0.003064506,
            0.001617524, 0.001461036, 0.000689845, 0.000232331, 0.001572909,
            0.00631706, 0.007565279, 0.012874017, 0.006387422, 0.003288181
        ], [0.002006843, 0.001075707, 0.00153823, 0.301508047, 0.048633377,
            0.003539674, 0.005304043, 0.000726834, 0.003197631, 0.01114956,
            0.004151758, 0.001602376, 0.00276374, 0.001945256, 0.005590248,
            0.020854177, 0.000514172, 0.002187176, 0.001393908, 0.002436637,
            0.003107608, 0.002557516, 0.001552732, 0.001847724, 0.001133339,
            0.001496355, 0.000826729, 0.000491484, 0.000252858, 0.001061092,
            0.003324369, 0.000956161, 0.002773383, 0.006158342, 0.000142113
        ], [0.000133399, 9.47579e-05, 0.000146871, 0.0059124, 0.212521324,
            0.000424063, 0.000571795, 4.85136e-05, 0.000218821, 0.001417522,
            0.000286707, 0.000192086, 0.000245768, 0.000236819, 0.001239475,
            0.003996125, 0.000123783, 0.000126771, 0.000400562, 0.000417897,
            0.000322356, 0.00012783, 0.000186509, 0.00014257, 0.000120078,
            0.000173915, 0.000254114, 3.41829e-05, 1.26275e-05, 0.000109905,
            0.000491493, 0.000107976, 0.000107733, 0.000387677, 4.19055e-06
        ], [0.002133086, 0.001495755, 0.001646649, 0.001304958, 0.001724277,
            0.212660584, 0.010161514, 0.000272584, 0.001479282, 0.003369931,
            0.006131361, 0.00293257, 0.00283651, 0.001545278, 0.00284319,
            0.067946024, 0.000833441, 0.032440652, 0.000683174, 0.00150209,
            0.001300395, 0.002357106, 0.001180237, 0.000488472, 0.000338699,
            0.002996641, 0.00070673, 0.00037427, 0.001994877, 0.000882198,
            0.001489793, 0.002024278, 0.000962912, 0.002695246, 0.000101196
        ], [0.002589472, 0.002647316, 0.024810946, 0.010579562, 0.013304633,
            0.012413578, 0.228332827, 0.001575063, 0.016718117, 0.016799853,
            0.019601511, 0.005408542, 0.008688682, 0.011274008, 0.004795946,
            0.020092213, 0.003487569, 0.004273543, 0.007700171, 0.013508198,
            0.012545455, 0.008726088, 0.005948049, 0.00264805, 0.005336394,
            0.013457586, 0.011521106, 0.013253254, 0.002216097, 0.021337905,
            0.018057439, 0.017970807, 0.009095412, 0.025262104, 0.000207308
        ], [0.013188805, 0.010337618, 0.003380349, 0.005318086, 0.003695765,
            0.005135618, 0.004469915, 0.048645053, 0.031818904, 0.008338689,
            0.017995637, 0.009827114, 0.003798291, 0.002278187, 0.001942407,
            0.00397617, 0.019513334, 0.013115523, 0.004127438, 0.005721513,
            0.004139236, 0.003609597, 0.051707005, 0.079859624, 0.062432525,
            0.014097638, 0.003212907, 0.001171542, 0.001028213, 0.004434038,
            0.008783316, 0.002709726, 0.002990072, 0.005028242, 4.72208e-05
        ], [0.038038787, 0.010494326, 0.010542373, 0.069201063, 0.037512199,
            0.032441715, 0.035154803, 0.015084121, 0.230863668, 0.201669822,
            0.033013469, 0.015696104, 0.011875281, 0.017289655, 0.012014637,
            0.025783089, 0.006304014, 0.010539087, 0.006320973, 0.003150346,
            0.002327944, 0.004525379, 0.002967288, 0.003552077, 0.002456911,
            0.003427711, 0.001414667, 0.000853278, 0.001295686, 0.004743442,
            0.009944684, 0.003799831, 0.064560953, 0.010008993, 0.000155806
        ], [0.005155392, 0.006697623, 0.017206739, 0.010084883, 0.034818906,
            0.009343379, 0.015614152, 0.002886492, 0.018411757, 0.12764402,
            0.010722736, 0.00722221, 0.02828255, 0.025412126, 0.035398338,
            0.031084568, 0.002493156, 0.018120907, 0.016526316, 0.004779719,
            0.005159003, 0.004192222, 0.009947341, 0.003045405, 0.002977222,
            0.005932192, 0.00337561, 0.000602723, 0.001006119, 0.002764542,
            0.003281995, 0.000949348, 0.00507268, 0.003762522, 3.2924e-05
        ], [0.002396536, 0.004368691, 0.006030805, 0.00243471, 0.00179705,
            0.008053233, 0.001276634, 0.002183591, 0.005989635, 0.006687103,
            0.115017351, 0.010776589, 0.005540109, 0.011947162, 0.007085699,
            0.006542728, 0.001938776, 0.076606134, 0.002918356, 0.001292705,
            0.001075744, 0.003529057, 0.001276917, 0.000637478, 0.000508203,
            0.001414303, 0.001257923, 0.000184129, 0.00188767, 0.001080058,
            0.001340791, 0.0016994, 0.00203615, 0.002001259, 2.79889e-05
        ], [0.0046742, 0.021764386, 0.013962697, 0.005845585, 0.010056668,
            0.025067902, 0.009136161, 0.006784498, 0.014490576, 0.025233814,
            0.031855507, 0.315711044, 0.163871829, 0.062887224, 0.094565947,
            0.079638509, 0.007354554, 0.088202428, 0.017421614, 0.00414447,
            0.003692428, 0.004548669, 0.007027096, 0.007306819, 0.004110258,
            0.006487789, 0.004056849, 0.000666126, 0.002243252, 0.003680472,
            0.004662256, 0.001765171, 0.002201141, 0.004452414, 0.000200665
        ], [0.006917628, 0.01469079, 0.003871259, 0.008247428, 0.005135918,
            0.008080653, 0.006325508, 0.005471052, 0.006625737, 0.010811618,
            0.016704878, 0.016071554, 0.124263321, 0.014441467, 0.030336265,
            0.009413847, 0.009714777, 0.017215457, 0.020611496, 0.003886039,
            0.002512237, 0.002016781, 0.005645719, 0.009282112, 0.00457942,
            0.004498216, 0.002867317, 0.001011411, 0.001390545, 0.003027114,
            0.005868231, 0.002137076, 0.002237175, 0.003357562, 7.7125e-05
        ], [0.001877178, 0.006296258, 0.003313542, 0.005926711, 0.00482406,
            0.006734389, 0.009780987, 0.005128298, 0.009527336, 0.013783266,
            0.010840997, 0.016199855, 0.087766598, 0.302390201, 0.053805994,
            0.020846592, 0.016422371, 0.030305774, 0.026565345, 0.0090525,
            0.007423475, 0.004117447, 0.006596852, 0.004456161, 0.004472434,
            0.007696366, 0.038518424, 0.00386178, 0.001900298, 0.015009342,
            0.013651272, 0.006249813, 0.021173282, 0.009780842, 0.000584822
        ], [0.004508286, 0.005984424, 0.002255963, 0.00281451, 0.002519828,
            0.003542356, 0.002424391, 0.001298755, 0.002240703, 0.004421946,
            0.003691893, 0.00536205, 0.014137101, 0.005033421, 0.299788181,
            0.004766489, 0.002030794, 0.004494497, 0.089111882, 0.006049897,
            0.004352661, 0.00137874, 0.035515817, 0.041684978, 0.052458657,
            0.011671399, 0.004005547, 0.001197916, 0.00099148, 0.00430604,
            0.016230307, 0.0016056, 0.001331653, 0.005405423, 0.000187691
        ], [0.000704125, 0.001024989, 0.001095159, 0.004848134, 0.002878729,
            0.004729457, 0.003719757, 0.000559699, 0.001701483, 0.002562333,
            0.002841472, 0.009193948, 0.003868115, 0.002768392, 0.004981248,
            0.041167933, 0.001280015, 0.005069501, 0.002250941, 0.001312929,
            0.001508634, 0.002487744, 0.001870558, 0.000749417, 0.001409049,
            0.0016613, 0.001023635, 0.000920886, 0.001642237, 0.001895402,
            0.003911703, 0.003098075, 0.00282035, 0.005510608, 0.002037634
        ], [0.010832112, 0.027029295, 0.013078252, 0.018076281, 0.008341925,
            0.01710373, 0.023047459, 0.014004261, 0.026404744, 0.021623949,
            0.038275368, 0.028261452, 0.011612254, 0.00831688, 0.008678897,
            0.009080691, 0.096198498, 0.005810995, 0.010177762, 0.006381617,
            0.01362693, 0.023383014, 0.014869382, 0.003420036, 0.004386254,
            0.01361205, 0.009081572, 0.004158661, 0.009801946, 0.005803496,
            0.015419372, 0.019929468, 0.012696252, 0.015632167, 0.000194963
        ], [0.003755614, 0.008670104, 0.002477636, 0.002847382, 0.001721063,
            0.003357262, 0.003554503, 0.002732729, 0.003488391, 0.003037052,
            0.007222754, 0.004310593, 0.003365902, 0.002410334, 0.002064966,
            0.002473739, 0.022626514, 0.047648854, 0.00429858, 0.003994041,
            0.004944986, 0.006065174, 0.009947986, 0.002778254, 0.002782399,
            0.012673361, 0.011157627, 0.005063832, 0.041032153, 0.004947183,
            0.018107835, 0.015345834, 0.005986122, 0.010314783, 0.000709895
        ], [0.005038356, 0.004036658, 0.006805591, 0.005794962, 0.007679881,
            0.005913912, 0.005001134, 0.002609576, 0.005993286, 0.006360608,
            0.007421439, 0.006058722, 0.005267681, 0.004094162, 0.004848233,
            0.006164589, 0.008248447, 0.007082693, 0.010495515, 0.004171643,
            0.004209183, 0.004972883, 0.016737413, 0.002131401, 0.003748984,
            0.007964137, 0.003824251, 0.002533164, 0.001859976, 0.005798092,
            0.004411696, 0.002904285, 0.003654867, 0.006243272, 2.7702e-05
        ], [0.033658131, 0.024860485, 0.064319325, 0.052507964, 0.066813014,
            0.055335402, 0.050915385, 0.042257127, 0.049422047, 0.049962994,
            0.04372396, 0.046442674, 0.048364542, 0.042919412, 0.044481862,
            0.046721575, 0.023178318, 0.041097734, 0.035250123, 0.025670042,
            0.012928765, 0.044664564, 0.022034111, 0.019362354, 0.020078516,
            0.019788358, 0.010504079, 0.004210213, 0.003561806, 0.012971992,
            0.018281368, 0.010410566, 0.026402114, 0.016235167, 0.000190568
        ], [0.017078117, 0.009528735, 0.030997619, 0.027867638, 0.040671149,
            0.020986893, 0.01891515, 0.010972035, 0.021839969, 0.020535067,
            0.019468396, 0.016721449, 0.017551506, 0.014105077, 0.019023927,
            0.024605082, 0.0084466, 0.029488114, 0.015946428, 0.00685054,
            0.007712061, 0.025444294, 0.013533109, 0.007826447, 0.009285106,
            0.008765155, 0.008022418, 0.004266029, 0.002967375, 0.009120574,
            0.008657242, 0.00573249, 0.011573878, 0.010731821, 0.000326523
        ], [0.001317146, 0.003071325, 0.003784409, 0.004372122, 0.004148086,
            0.0048277, 0.006341092, 0.001322568, 0.004397792, 0.00576705,
            0.005519552, 0.004586708, 0.006163653, 0.005005242, 0.002751197,
            0.003786206, 0.005565546, 0.004431264, 0.005748586, 0.008436445,
            0.005767769, 0.010409226, 0.007553707, 0.004019906, 0.021137734,
            0.016788923, 0.009437895, 0.009475019, 0.002505475, 0.014147087,
            0.010758874, 0.006145657, 0.008496282, 0.011799152, 6.66582e-05
        ], [0.01271717, 0.02123563, 0.023201699, 0.019654646, 0.020667414,
            0.023279171, 0.019215018, 0.033684508, 0.017987141, 0.017593649,
            0.033691294, 0.018004623, 0.012766915, 0.008642762, 0.011452431,
            0.01708272, 0.022007312, 0.015573729, 0.014156592, 0.018578658,
            0.011709029, 0.009315747, 0.045272612, 0.043697576, 0.013862012,
            0.056873737, 0.00865676, 0.00525938, 0.001766715, 0.00576086,
            0.010797988, 0.010036528, 0.006887915, 0.009189366, 0.000297054
        ], [0.003784235, 0.005943942, 0.004815802, 0.004713467, 0.005917339,
            0.008272535, 0.004113686, 0.008059746, 0.003807861, 0.00408709,
            0.010874711, 0.007033898, 0.003705054, 0.002582534, 0.002236166,
            0.00311404, 0.00355268, 0.004738248, 0.00113362, 0.003166628,
            0.001644553, 0.001743891, 0.003195572, 0.104316382, 0.002708299,
            0.005661555, 0.000821042, 0.000511302, 0.000594771, 0.001056717,
            0.001445726, 0.001106583, 0.000532451, 0.001304674, 2.10223e-05
        ], [0.000663915, 0.001984984, 0.00135336, 0.001809963, 0.002646226,
            0.001802611, 0.002294011, 0.000616803, 0.001690192, 0.001930331,
            0.001678322, 0.001353008, 0.00219909, 0.001946648, 0.001447868,
            0.001293905, 0.000861576, 0.001009128, 0.001560043, 0.003078255,
            0.001396263, 0.001088807, 0.002305829, 0.006682735, 0.03376888,
            0.017439828, 0.003391373, 0.002727331, 0.000475161, 0.003902916,
            0.003759776, 0.001915599, 0.001147294, 0.003928198, 1.25732e-05
        ], [0.003548878, 0.005385149, 0.006341204, 0.00389294, 0.004735976,
            0.006792094, 0.009356652, 0.007254411, 0.005163527, 0.005352864,
            0.008321831, 0.00527485, 0.004741647, 0.003397221, 0.004415223,
            0.004437167, 0.00418651, 0.00406452, 0.010392378, 0.020401853,
            0.010588811, 0.007614471, 0.04028783, 0.112198366, 0.099561519,
            0.105403727, 0.004505714, 0.003217237, 0.000921526, 0.004893775,
            0.004822986, 0.002370077, 0.003027406, 0.005829079, 2.53379e-05
        ], [0.00267326, 0.006046831, 0.004881497, 0.006061926, 0.007122241,
            0.006191287, 0.013551176, 0.003004596, 0.008571623, 0.006873279,
            0.007471771, 0.006505156, 0.008566734, 0.007855753, 0.00458603,
            0.007512976, 0.006609298, 0.012539845, 0.015437579, 0.021230725,
            0.021990342, 0.013921273, 0.016418696, 0.0171028, 0.024744516,
            0.025287559, 0.126539105, 0.028266106, 0.005003756, 0.02628928,
            0.022343507, 0.009935523, 0.012723362, 0.022793085, 0.000436304
        ], [0.021014135, 0.029112578, 0.015348867, 0.021930902, 0.015446604,
            0.017589285, 0.023459934, 0.011706735, 0.017744791, 0.017995941,
            0.024917726, 0.018424448, 0.018604038, 0.014317684, 0.013276364,
            0.021925042, 0.023585037, 0.02117435, 0.021982514, 0.037529053,
            0.03540501, 0.026002547, 0.04953412, 0.043510381, 0.038633029,
            0.026891134, 0.02218497, 0.198924344, 0.073615227, 0.03742236,
            0.024928295, 0.012284724, 0.023709171, 0.043160776, 0.007388415
        ], [0.005105609, 0.005507224, 0.004821338, 0.005102359, 0.004709797,
            0.006401513, 0.010022301, 0.002267374, 0.004776342, 0.00637849,
            0.005129417, 0.005487742, 0.006275611, 0.00478982, 0.004646709,
            0.006552791, 0.005147569, 0.010209385, 0.02351519, 0.02263123,
            0.040905683, 0.029578722, 0.009267827, 0.01840852, 0.011392795,
            0.022291094, 0.016721603, 0.022487585, 0.032763844, 0.024110513,
            0.014600158, 0.019972918, 0.034048647, 0.02601125, 5.12455e-05
        ], [0.011226175, 0.028792825, 0.038844232, 0.026958729, 0.021987935,
            0.026819914, 0.063883444, 0.0151799, 0.067815323, 0.039201764,
            0.034966205, 0.031177829, 0.045463156, 0.050907587, 0.040024918,
            0.030330928, 0.03822413, 0.05319742, 0.051005171, 0.054722434,
            0.06033958, 0.04217551, 0.043907266, 0.034090874, 0.085639322,
            0.066377094, 0.070377626, 0.09359434, 0.032713261, 0.142743032,
            0.077569369, 0.032031287, 0.05735968, 0.075443284, 0.002393294
        ], [0.001423908, 0.003199932, 0.001828886, 0.001110611, 0.000744897,
            0.003188369, 0.003222972, 0.001037814, 0.00275221, 0.00203998,
            0.002164539, 0.002446004, 0.001117675, 0.00109976, 0.001144437,
            0.001849908, 0.004714965, 0.001478522, 0.002427735, 0.001460771,
            0.002668983, 0.003629998, 0.003050622, 0.005961288, 0.009442094,
            0.00334172, 0.002940545, 0.001469701, 0.00306816, 0.002690037,
            0.007276602, 0.004439867, 0.00445018, 0.008165743, 6.27059e-06
        ], [0.000507793, 0.000450761, 0.000464532, 0.000442401, 0.00047921,
            0.000397113, 0.000677882, 0.000398894, 0.000996413, 0.000669651,
            0.000691382, 0.000632005, 0.000895125, 0.000828707, 0.000715183,
            0.000445345, 0.000806414, 0.000489027, 0.000811212, 0.000741528,
            0.001033935, 0.000585263, 0.001142297, 0.000375702, 0.00188586,
            0.001182294, 0.001522828, 0.001195731, 0.000298592, 0.002327972,
            0.004128042, 0.011932592, 0.001087342, 0.001585374, 5.05288e-06
        ], [0.001490484, 0.000381636, 0.00051539, 0.000412807, 0.001086843,
            0.00040325, 0.000431896, 0.000194739, 0.000526594, 0.000405704,
            0.000427699, 0.000373029, 0.000535757, 0.000372755, 0.000340403,
            0.000499627, 0.00034353, 0.000434422, 0.000567715, 0.000611904,
            0.000464216, 0.001083578, 0.00066296, 0.000340282, 0.000456104,
            0.00062634, 0.000687475, 0.00053238, 9.4814e-05, 0.00072693,
            0.004750227, 0.001505716, 0.024875541, 0.001146753, 1.4577e-07
        ], [0.002568154, 0.006476888, 0.005635973, 0.005233612, 0.006526335,
            0.005509546, 0.013450617, 0.002862943, 0.007963984, 0.00581574,
            0.005990283, 0.006362449, 0.005840321, 0.004482119, 0.004260952,
            0.0052654, 0.005961785, 0.00603579, 0.008277212, 0.008217871,
            0.008637449, 0.013156738, 0.009226786, 0.005185778, 0.008003777,
            0.010217908, 0.020649187, 0.009870351, 0.007256217, 0.026227887,
            0.015921883, 0.010531638, 0.013694503, 0.079103247, 0.001872351
        ], [1.7234e-05, 2.0392e-05, 2.61728e-05, 5.94183e-05, 2.38557e-05,
            3.22037e-05, 3.81192e-05, 1.34552e-05, 3.20485e-05, 3.92075e-05,
            2.20377e-05, 2.06752e-05, 2.66707e-05, 1.55686e-05, 4.20794e-05,
            1.41105e-05, 9.84367e-06, 6.57959e-05, 5.45386e-05, 1.32667e-05,
            3.83636e-05, 5.16637e-05, 7.15444e-05, 1.04754e-05, 2.32562e-05,
            1.88441e-05, 3.94316e-05, 4.90602e-05, 0.000242861, 6.11539e-05,
            7.05908e-05, 3.89425e-05, 6.60497e-05, 0.000110022, 9.5447e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2002():
    """
    IA matrix 2002



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2002())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2003():
    """
    IA matrix 2003



    constant


    """

    ret =  xr.DataArray(
        data=[[0.128251478, 0.000802303, 0.25668027, 0.0426759, 0.042483353,
            0.124470047, 0.012868491, 0.001945821, 0.005331104, 0.013203244,
            0.000564509, 0.000252567, 0.000265907, 0.000172078, 0.00015033,
            0.009713017, 0.0004726, 0.008136106, 0.000495083, 0.002971511,
            0.002569215, 0.03879115, 0.001616409, 0.00117875, 0.00028913,
            0.004031851, 0.000262692, 0.000139846, 0.000360426, 0.00069594,
            0.002176008, 0.002262956, 0.002164395, 0.00243113, 0.017557447
        ], [0.00332194, 0.082234206, 0.002670743, 0.002439711, 0.001262039,
            0.004132039, 0.004698109, 0.466175612, 0.03344912, 0.003323698,
            0.066959454, 0.047533929, 0.003017259, 0.001228493, 0.001244808,
            0.005449232, 0.113529653, 0.017622988, 0.000490317, 0.000658781,
            0.001180912, 0.001762693, 0.004448997, 0.002728161, 0.002365127,
            0.001747299, 0.000560834, 0.000264762, 0.001288543, 0.000837147,
            0.003088539, 0.002892219, 0.001092913, 0.002738598, 0.060288018
        ], [0.056511477, 0.000762079, 0.150645341, 0.003342352, 0.087238539,
            0.001557323, 0.002963784, 0.001081702, 0.010117854, 0.002951535,
            0.001954631, 0.001204645, 0.001238839, 0.001061213, 0.00094456,
            0.002330368, 0.000813332, 0.001036601, 0.001614635, 0.004789023,
            0.004534735, 0.150432337, 0.001355379, 0.00608859, 0.003407222,
            0.001628173, 0.001419675, 0.000682969, 0.000236472, 0.001670901,
            0.007216702, 0.008046876, 0.013267067, 0.006678732, 0.002991253
        ], [0.001983404, 0.001025394, 0.0015834, 0.30916073, 0.052197798,
            0.003449903, 0.005135896, 0.000517824, 0.003297155, 0.011811925,
            0.003773857, 0.001652437, 0.002455685, 0.001873604, 0.005404231,
            0.020867902, 0.000580324, 0.002242908, 0.001377832, 0.002428184,
            0.002946335, 0.002670154, 0.001668378, 0.002215427, 0.001334515,
            0.001493407, 0.000761255, 0.00045439, 0.000248974, 0.001172875,
            0.003009414, 0.000938677, 0.002879041, 0.005473805, 0.000104462
        ], [0.000137004, 8.0512e-05, 0.000126554, 0.007747518, 0.212491624,
            0.000458159, 0.000510193, 4.43231e-05, 0.000216748, 0.001171981,
            0.000316792, 0.00018041, 0.000217237, 0.000249284, 0.00144764,
            0.004225441, 0.000110859, 0.000120735, 0.000396394, 0.000405119,
            0.000302006, 0.00012289, 0.000190731, 0.000131819, 0.000126042,
            0.000150889, 0.000244135, 3.16055e-05, 1.06295e-05, 0.000108215,
            0.000417681, 0.000100702, 0.000104411, 0.000385726, 1.74238e-06
        ], [0.001945471, 0.00153071, 0.001662131, 0.001230411, 0.001711124,
            0.222046166, 0.010422765, 0.000255988, 0.00148333, 0.003340744,
            0.006493991, 0.003129366, 0.002930763, 0.001560112, 0.002783478,
            0.070792356, 0.000867246, 0.032476846, 0.000778512, 0.001448268,
            0.001239425, 0.002313394, 0.001177105, 0.000554803, 0.000407349,
            0.002849459, 0.000693898, 0.000346182, 0.00187363, 0.000913146,
            0.00159978, 0.002195714, 0.001072394, 0.002720118, 9.64006e-05
        ], [0.002523376, 0.002732761, 0.024086259, 0.011069723, 0.013868743,
            0.012996626, 0.232589289, 0.001263196, 0.016122287, 0.016568935,
            0.019518014, 0.005287922, 0.008139905, 0.010479749, 0.004704772,
            0.019933712, 0.003128496, 0.004333032, 0.007545084, 0.012949745,
            0.011981613, 0.008414549, 0.006047874, 0.002953761, 0.005307327,
            0.013182838, 0.011138533, 0.013026447, 0.002118701, 0.020701533,
            0.01859817, 0.017899793, 0.008688492, 0.024604834, 0.00017388
        ], [0.012157304, 0.009105314, 0.003046978, 0.005075613, 0.00374425,
            0.004769227, 0.004357972, 0.038201527, 0.033984057, 0.008179414,
            0.017138403, 0.010282934, 0.00371982, 0.002072222, 0.00181411,
            0.003556651, 0.018548895, 0.012068389, 0.003968001, 0.004995774,
            0.003780569, 0.003355491, 0.049210487, 0.072476589, 0.063956523,
            0.015786418, 0.003179908, 0.001114105, 0.000921764, 0.004414947,
            0.008350643, 0.002943738, 0.002816652, 0.005074399, 3.48911e-05
        ], [0.038336424, 0.010522534, 0.00999355, 0.06989513, 0.03756249,
            0.03339286, 0.035411889, 0.013367075, 0.237431472, 0.205155668,
            0.034899927, 0.01593319, 0.011440306, 0.017295443, 0.011889592,
            0.025687381, 0.006463133, 0.01082411, 0.006282547, 0.003166391,
            0.00222327, 0.004502576, 0.00323309, 0.004408262, 0.003280927,
            0.003832926, 0.001401995, 0.000807658, 0.001303588, 0.00475064,
            0.01000891, 0.003800542, 0.063292692, 0.009948183, 0.000143276
        ], [0.005042721, 0.006340169, 0.017682938, 0.010367541, 0.03607421,
            0.009098169, 0.015880439, 0.002579758, 0.018146469, 0.132888594,
            0.010518357, 0.007110502, 0.026991326, 0.024544322, 0.036219078,
            0.032803921, 0.002419652, 0.018433674, 0.016572821, 0.004543999,
            0.005117875, 0.004005218, 0.010238811, 0.003059916, 0.00314877,
            0.006061738, 0.003402524, 0.000598153, 0.00099406, 0.002789733,
            0.003292333, 0.001051123, 0.005054476, 0.003819217, 2.64243e-05
        ], [0.001958636, 0.004559002, 0.005938244, 0.002276877, 0.001652786,
            0.007617142, 0.001196441, 0.001799817, 0.005729342, 0.006104292,
            0.123152641, 0.010485917, 0.005228727, 0.010634366, 0.006934643,
            0.00661289, 0.00180079, 0.080126343, 0.003036674, 0.001203535,
            0.001012624, 0.00331612, 0.001289334, 0.000608874, 0.000566356,
            0.001312971, 0.001187236, 0.000151955, 0.001535074, 0.000970465,
            0.001337749, 0.001660461, 0.001936022, 0.00187401, 2.18944e-05
        ], [0.004453483, 0.021162388, 0.012831161, 0.005681158, 0.010133515,
            0.024429949, 0.008703717, 0.005613018, 0.013668006, 0.024378988,
            0.032033465, 0.316731698, 0.161803666, 0.062886785, 0.092960337,
            0.075951364, 0.007035315, 0.088743303, 0.017419472, 0.003939792,
            0.003539939, 0.004265107, 0.00670576, 0.006471886, 0.004739435,
            0.006937642, 0.003886206, 0.000636467, 0.002026148, 0.003636434,
            0.004629102, 0.001736183, 0.002178748, 0.004323377, 0.000183784
        ], [0.006803999, 0.015652379, 0.003876108, 0.00834418, 0.005874878,
            0.008449998, 0.006497715, 0.00565669, 0.006961599, 0.011514151,
            0.017344218, 0.018304355, 0.131186983, 0.014874972, 0.032739768,
            0.009438204, 0.008696244, 0.017476087, 0.02222622, 0.003691171,
            0.002535441, 0.002076496, 0.005551581, 0.009475857, 0.00530898,
            0.005363047, 0.002881838, 0.000936743, 0.001369029, 0.002927123,
            0.005936722, 0.00206732, 0.00259423, 0.003499695, 9.85335e-05
        ], [0.001784929, 0.006391531, 0.003424846, 0.006252446, 0.004821027,
            0.006904126, 0.010011154, 0.003893562, 0.009551814, 0.013621744,
            0.01062083, 0.016740134, 0.092464574, 0.315180363, 0.05790102,
            0.023195584, 0.018000115, 0.032209473, 0.029627641, 0.008852423,
            0.007698879, 0.004310672, 0.006803285, 0.00421233, 0.004791701,
            0.008630943, 0.03910451, 0.003666423, 0.001970763, 0.015413772,
            0.0148755, 0.007390009, 0.022605878, 0.010744906, 0.000651832
        ], [0.004412382, 0.005724947, 0.002061476, 0.002793273, 0.002519205,
            0.00348837, 0.00256182, 0.001160989, 0.002334902, 0.004621448,
            0.00433671, 0.005819901, 0.014953003, 0.004881743, 0.305031982,
            0.005238227, 0.002343755, 0.004796834, 0.087558061, 0.005578868,
            0.003981161, 0.001469669, 0.036391917, 0.040763823, 0.050506006,
            0.012038651, 0.004063601, 0.001095536, 0.000887212, 0.004248509,
            0.017119608, 0.001826159, 0.001318945, 0.006473424, 0.000192978
        ], [0.000821098, 0.000854983, 0.000976825, 0.004350566, 0.002706388,
            0.004479079, 0.003662124, 0.00051502, 0.001656101, 0.00257464,
            0.002905776, 0.009181243, 0.003747354, 0.002572561, 0.004906835,
            0.044367213, 0.001167016, 0.005007996, 0.001943785, 0.00121626,
            0.001428443, 0.002322826, 0.001933283, 0.000694995, 0.001534928,
            0.001474917, 0.000969247, 0.000858368, 0.001188181, 0.001778617,
            0.00375793, 0.002919186, 0.002876406, 0.004992274, 0.002110646
        ], [0.010822679, 0.029965175, 0.013154647, 0.018092651, 0.008332114,
            0.017132446, 0.023161227, 0.01388678, 0.025837972, 0.020827446,
            0.039016336, 0.028101005, 0.011398281, 0.008039791, 0.008696588,
            0.00900837, 0.103541713, 0.00621072, 0.010118242, 0.006443083,
            0.013631286, 0.023112413, 0.014916733, 0.003595012, 0.005215312,
            0.014299748, 0.009310089, 0.004229395, 0.008486968, 0.005722823,
            0.015320751, 0.020341505, 0.012160238, 0.015577836, 0.000201872
        ], [0.003634883, 0.007738657, 0.002423251, 0.002785751, 0.001652996,
            0.003273321, 0.003506142, 0.002279493, 0.0033573, 0.002900607,
            0.007040227, 0.004173032, 0.003146405, 0.002227792, 0.002013348,
            0.002513542, 0.019784261, 0.048705828, 0.004411403, 0.003947675,
            0.004881287, 0.005887801, 0.009566505, 0.002594567, 0.003299916,
            0.012588656, 0.010831968, 0.004985452, 0.039675555, 0.004828728,
            0.018485539, 0.013954289, 0.006260958, 0.009788323, 0.000458219
        ], [0.004917101, 0.003639499, 0.006516618, 0.005480613, 0.00680912,
            0.006118083, 0.004940025, 0.002354659, 0.005870542, 0.00616409,
            0.007202421, 0.005783303, 0.004971916, 0.003743028, 0.004820822,
            0.006139936, 0.008117898, 0.007068392, 0.010621508, 0.004054621,
            0.004060383, 0.004853991, 0.016203501, 0.002142538, 0.004083179,
            0.007277926, 0.003673284, 0.002515703, 0.001815554, 0.005638359,
            0.004448877, 0.002833361, 0.003560522, 0.006057583, 2.19605e-05
        ], [0.033195497, 0.022445743, 0.064699828, 0.052223238, 0.062931043,
            0.055045618, 0.050399243, 0.041648171, 0.047226552, 0.04842297,
            0.041490856, 0.045104693, 0.046125518, 0.038753564, 0.043995129,
            0.044094728, 0.023218615, 0.0411012, 0.034217009, 0.024609832,
            0.01278873, 0.046474086, 0.021304286, 0.020053482, 0.020295909,
            0.019414648, 0.010531959, 0.003876464, 0.003337537, 0.012799982,
            0.018901137, 0.010775193, 0.025945813, 0.016353218, 0.000153094
        ], [0.017079169, 0.008898434, 0.030689487, 0.026934384, 0.037137109,
            0.020938484, 0.01881166, 0.010909876, 0.021674813, 0.020066508,
            0.019160264, 0.01638824, 0.016613344, 0.012734824, 0.018519627,
            0.024786006, 0.00860239, 0.031257298, 0.015558038, 0.006502507,
            0.007381489, 0.025244345, 0.013970991, 0.007868675, 0.009423839,
            0.008691178, 0.008108113, 0.004352269, 0.002895564, 0.008947672,
            0.008653562, 0.005908956, 0.011628319, 0.010682686, 0.000251602
        ], [0.001359482, 0.003204752, 0.003766752, 0.004235921, 0.004262058,
            0.004769139, 0.0060352, 0.001037038, 0.004200233, 0.005457483,
            0.005274976, 0.004464899, 0.00568673, 0.004596963, 0.00273689,
            0.003762765, 0.004022706, 0.004602309, 0.005693056, 0.008257769,
            0.005709773, 0.010463119, 0.007907126, 0.004317084, 0.022471256,
            0.017459807, 0.009683465, 0.009768567, 0.002460581, 0.014141207,
            0.011345095, 0.006796557, 0.008668551, 0.011662387, 6.9635e-05
        ], [0.013614935, 0.020554784, 0.023297841, 0.019103039, 0.0201975,
            0.023806147, 0.019492301, 0.035494387, 0.017876104, 0.017164366,
            0.033478243, 0.01749663, 0.01231433, 0.008244264, 0.011226775,
            0.017737657, 0.019562909, 0.016599592, 0.01389763, 0.019397003,
            0.012021264, 0.009261147, 0.045209385, 0.034029369, 0.012590718,
            0.055330065, 0.008793747, 0.005225503, 0.001594578, 0.005684926,
            0.011114205, 0.010318264, 0.006940498, 0.009032651, 0.000218592
        ], [0.003489304, 0.005946937, 0.004854798, 0.004806605, 0.005799893,
            0.00780699, 0.003891591, 0.00551181, 0.003507662, 0.003834389,
            0.009490828, 0.006306403, 0.003507389, 0.002385968, 0.002221571,
            0.003189623, 0.003221971, 0.004431649, 0.001207688, 0.003944835,
            0.001981896, 0.00169936, 0.00329176, 0.119214576, 0.002864003,
            0.005300579, 0.000836337, 0.000495629, 0.000907016, 0.001087643,
            0.001353159, 0.001061648, 0.000514627, 0.001322151, 1.86795e-05
        ], [0.000699556, 0.001882203, 0.001388224, 0.001750691, 0.002627767,
            0.001884025, 0.002490538, 0.000576455, 0.001673053, 0.00189862,
            0.00163891, 0.001358245, 0.002056494, 0.001790097, 0.001391078,
            0.001335811, 0.000799612, 0.001097338, 0.001720253, 0.002989521,
            0.001487667, 0.001250308, 0.002794519, 0.007427011, 0.031378409,
            0.017131387, 0.003663525, 0.002961861, 0.000456989, 0.004200666,
            0.004595828, 0.002169292, 0.00125995, 0.004013249, 1.58223e-05
        ], [0.003308566, 0.005353437, 0.006646383, 0.00393591, 0.004637737,
            0.006909593, 0.00941968, 0.006479274, 0.004943185, 0.00525752,
            0.008283731, 0.005156287, 0.004503252, 0.003187606, 0.004135203,
            0.00435577, 0.004015592, 0.005501037, 0.010673851, 0.02012254,
            0.011235215, 0.007598594, 0.040302102, 0.113733278, 0.095103768,
            0.103086489, 0.004424784, 0.003306431, 0.000902015, 0.005016414,
            0.004956665, 0.002428776, 0.003130738, 0.005790388, 1.69145e-05
        ], [0.002919933, 0.00560112, 0.004779369, 0.006193433, 0.00702229,
            0.006165715, 0.013145267, 0.002637917, 0.008193697, 0.006383487,
            0.007094393, 0.006509947, 0.007640094, 0.007526468, 0.00453772,
            0.008279227, 0.006189937, 0.012446661, 0.015714896, 0.020764446,
            0.021839114, 0.014434768, 0.016702133, 0.017352601, 0.025623596,
            0.025180735, 0.12760305, 0.028302711, 0.004981967, 0.026012977,
            0.022952396, 0.010393603, 0.01288272, 0.022343714, 0.000525984
        ], [0.019515889, 0.02695759, 0.015138035, 0.021081135, 0.015152556,
            0.017084191, 0.0230849, 0.011128046, 0.01754497, 0.016856046,
            0.023094578, 0.017981311, 0.017686167, 0.013764257, 0.013242801,
            0.025046665, 0.022287392, 0.021658674, 0.021566421, 0.035188902,
            0.0360044, 0.02557575, 0.047936667, 0.037261657, 0.037576235,
            0.028882821, 0.022305637, 0.202100374, 0.076805255, 0.038034358,
            0.021711323, 0.013825994, 0.024210764, 0.042821941, 0.004686462
        ], [0.004586971, 0.00631436, 0.004621509, 0.005198486, 0.005149948,
            0.006117705, 0.009707608, 0.001941458, 0.004603443, 0.006144591,
            0.005107521, 0.0053335, 0.005890284, 0.004451898, 0.004713052,
            0.00667042, 0.004694924, 0.010316405, 0.023394655, 0.022124817,
            0.041159757, 0.028926775, 0.00930485, 0.017532155, 0.013257446,
            0.022765048, 0.016315458, 0.023025509, 0.029400489, 0.024686465,
            0.014704042, 0.01982276, 0.034247703, 0.026420105, 1.11853e-06
        ], [0.011480006, 0.029206128, 0.037855939, 0.026901291, 0.021975413,
            0.026630525, 0.062815969, 0.012045042, 0.066218809, 0.038061237,
            0.034434791, 0.03032378, 0.043936583, 0.048289338, 0.039172057,
            0.029784621, 0.03317107, 0.053255409, 0.051307037, 0.05485969,
            0.060200136, 0.04234011, 0.04441587, 0.032366755, 0.085697652,
            0.066481424, 0.068871654, 0.09734752, 0.033096768, 0.141389397,
            0.080551063, 0.032856771, 0.058159064, 0.075751939, 0.002617846
        ], [0.001498021, 0.0033279, 0.001788463, 0.001129164, 0.00077522,
            0.003125284, 0.003217116, 0.000725439, 0.00272031, 0.001985674,
            0.002363793, 0.002486756, 0.001129604, 0.00105827, 0.001180686,
            0.001850203, 0.004958206, 0.001545098, 0.002398689, 0.001478138,
            0.002667042, 0.003654368, 0.003080082, 0.005953301, 0.009617785,
            0.003468509, 0.002968703, 0.001520107, 0.00299225, 0.002752024,
            0.007450796, 0.004422996, 0.004539823, 0.008387935, 4.47473e-06
        ], [0.00051311, 0.000425779, 0.000473136, 0.000424244, 0.000488117,
            0.000395908, 0.000684568, 0.000371784, 0.00097442, 0.000678854,
            0.00070275, 0.000634745, 0.000831845, 0.000763514, 0.000670291,
            0.000455483, 0.000771864, 0.000506516, 0.000813492, 0.000751655,
            0.000960005, 0.000596017, 0.0011136, 0.000390144, 0.002004045,
            0.001223123, 0.001631231, 0.001210716, 0.000308708, 0.002285399,
            0.004283269, 0.011610116, 0.001144506, 0.001574455, 3.27316e-06
        ], [0.001343679, 0.000374161, 0.000475534, 0.000402072, 0.000992738,
            0.000431161, 0.000430067, 0.000167401, 0.000523693, 0.000398995,
            0.000483237, 0.000386851, 0.00058304, 0.000370009, 0.000361021,
            0.000509022, 0.000345452, 0.000419761, 0.000561167, 0.00062427,
            0.000453612, 0.001055666, 0.000655319, 0.000343884, 0.00041124,
            0.000603165, 0.000707389, 0.000489352, 9.62737e-05, 0.000712889,
            0.004803472, 0.00146999, 0.025610829, 0.001137175, 1.22023e-07
        ], [0.002622548, 0.006096264, 0.005305623, 0.00489638, 0.006315198,
            0.005412929, 0.012846991, 0.002252594, 0.007518848, 0.005454905,
            0.005770328, 0.005859296, 0.005273652, 0.004054461, 0.004262336,
            0.005023497, 0.005474505, 0.006026262, 0.007662987, 0.007884059,
            0.008512625, 0.013408505, 0.010294731, 0.005207711, 0.008010522,
            0.011131229, 0.020123275, 0.00983748, 0.007369146, 0.025759873,
            0.016762229, 0.010887018, 0.013684347, 0.078356155, 0.002180873
        ], [1.40602e-05, 2.24296e-05, 2.4764e-05, 5.75753e-05, 1.70001e-05,
            3.04476e-05, 3.74455e-05, 8.13903e-06, 1.92898e-05, 2.43695e-05,
            2.01898e-05, 1.81431e-05, 2.73016e-05, 1.53573e-05, 7.15171e-05,
            1.18537e-05, 5.84632e-06, 7.03975e-05, 5.89372e-05, 1.16449e-05,
            3.91367e-05, 5.29344e-05, 7.64027e-05, 2.63495e-05, 2.58013e-05,
            1.91726e-05, 4.07749e-05, 5.50902e-05, 0.000232072, 6.80072e-05,
            7.22623e-05, 3.80176e-05, 5.4143e-05, 0.00011798, 0.000113511
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 3
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2003():
    """
    IA matrix 2003



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2003())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2004():
    """
    IA matrix 2004



    constant


    """

    ret =  xr.DataArray(
        data=[[0.130100085, 0.000774275, 0.261175907, 0.044378224, 0.048799803,
            0.122809367, 0.013681837, 0.001495636, 0.005502175, 0.013091638,
            0.000576532, 0.000236829, 0.000230561, 0.000122512, 0.000136909,
            0.00878868, 0.000496432, 0.006825071, 0.00076407, 0.002784804,
            0.00259458, 0.039493995, 0.001643775, 0.001063565, 0.000326902,
            0.005206937, 0.000272803, 0.000142262, 0.000376732, 0.000792102,
            0.002071086, 0.002496966, 0.002194474, 0.002580587, 0.019727466
        ], [0.002927428, 0.084749104, 0.002704673, 0.002514319, 0.001710314,
            0.004218515, 0.004701284, 0.458014421, 0.033834244, 0.00372342,
            0.070870708, 0.050991115, 0.0030674, 0.001267689, 0.001479835,
            0.005684053, 0.111050254, 0.017370495, 0.000556769, 0.000627479,
            0.001080869, 0.001967873, 0.004885749, 0.003125174, 0.002766012,
            0.002130135, 0.000573123, 0.000284015, 0.001680688, 0.000909418,
            0.00342853, 0.002523107, 0.001311563, 0.002973793, 0.061327604
        ], [0.057344177, 0.000791808, 0.152765089, 0.003728871, 0.088074829,
            0.001795647, 0.002843695, 0.001322181, 0.010091889, 0.003049233,
            0.001912909, 0.00131039, 0.001329704, 0.001126259, 0.000986803,
            0.002351378, 0.000868037, 0.001118167, 0.001762366, 0.004590917,
            0.004590848, 0.150576988, 0.001496483, 0.005231772, 0.003411173,
            0.001747169, 0.001496069, 0.000654823, 0.000253407, 0.001732843,
            0.007046713, 0.00845133, 0.013216773, 0.006846735, 0.002761675
        ], [0.001897313, 0.001045731, 0.00139734, 0.311457989, 0.053736668,
            0.003219958, 0.00529225, 0.000448678, 0.003212089, 0.0121132,
            0.003491497, 0.001575904, 0.002286973, 0.001848243, 0.005247906,
            0.020591671, 0.000622831, 0.002363009, 0.001503512, 0.0023457,
            0.003062053, 0.002773218, 0.001749569, 0.002378083, 0.001500644,
            0.001540204, 0.000710122, 0.000407962, 0.000334025, 0.001274027,
            0.002694171, 0.000874733, 0.003081362, 0.005162812, 9.82311e-05
        ], [0.000136653, 6.72078e-05, 0.000126619, 0.008067477, 0.199198724,
            0.00039361, 0.000521965, 4.45521e-05, 0.000205855, 0.00089347,
            0.000237038, 0.000166558, 0.000202861, 0.000249043, 0.001509296,
            0.004004204, 9.4825e-05, 0.000112269, 0.000382759, 0.000356891,
            0.000286177, 0.000119914, 0.0002041, 0.000114343, 0.000121599,
            0.000139781, 0.000245406, 2.94447e-05, 9.67592e-06, 9.63656e-05,
            0.000333309, 9.0276e-05, 0.000104453, 0.000379359, 2.89658e-06
        ], [0.001881734, 0.001662022, 0.001631544, 0.001292122, 0.001833802,
            0.233437574, 0.010614514, 0.000280412, 0.001472493, 0.003075293,
            0.006678829, 0.003080113, 0.002835166, 0.001587603, 0.002791814,
            0.070031485, 0.000837597, 0.032062101, 0.001102025, 0.00144501,
            0.001272933, 0.002350868, 0.001157398, 0.000556006, 0.00038399,
            0.002729611, 0.000670007, 0.000336158, 0.002466981, 0.000928017,
            0.001733622, 0.002447311, 0.001227321, 0.002711387, 5.12038e-05
        ], [0.002541162, 0.002715439, 0.024457268, 0.011441347, 0.015020156,
            0.013326228, 0.23967352, 0.00135901, 0.015534743, 0.016141678,
            0.019957518, 0.004964665, 0.007739373, 0.009723184, 0.00450702,
            0.019273364, 0.00302021, 0.004263747, 0.0082401, 0.012548486,
            0.011943324, 0.008440651, 0.006152933, 0.003087269, 0.005138115,
            0.013547949, 0.010791429, 0.012975103, 0.002237177, 0.020916258,
            0.0185996, 0.018189611, 0.008604033, 0.024803221, 0.000185744
        ], [0.012506041, 0.008368225, 0.003113629, 0.005608643, 0.004374865,
            0.004943446, 0.004342524, 0.042323611, 0.037454136, 0.008952235,
            0.017866928, 0.011682576, 0.004101647, 0.002181675, 0.001944825,
            0.003574706, 0.019687549, 0.011824954, 0.004534089, 0.004899396,
            0.003782111, 0.003505087, 0.050840906, 0.072493436, 0.072619933,
            0.018462388, 0.00314673, 0.001158037, 0.00110134, 0.004613193,
            0.008521612, 0.003148212, 0.002880161, 0.005174979, 2.90503e-05
        ], [0.038681445, 0.010151798, 0.009609941, 0.069929539, 0.039537267,
            0.032838518, 0.034942814, 0.013518979, 0.238655964, 0.20607921,
            0.03483578, 0.015470734, 0.011108044, 0.017583471, 0.011595541,
            0.024743038, 0.006515647, 0.010674283, 0.006358005, 0.003146102,
            0.002314915, 0.00443074, 0.003342108, 0.004399896, 0.003633445,
            0.004101937, 0.001399853, 0.000756893, 0.00157367, 0.004838083,
            0.009621279, 0.003826033, 0.061918883, 0.009951816, 0.000159609
        ], [0.004918485, 0.006300126, 0.017685305, 0.010393988, 0.038043564,
            0.009113335, 0.015877425, 0.002391045, 0.017149573, 0.133932756,
            0.01015247, 0.006718907, 0.02539514, 0.023684611, 0.035612379,
            0.031716214, 0.002304259, 0.018406544, 0.01671564, 0.004506754,
            0.005338268, 0.004051705, 0.010261791, 0.002767497, 0.003296251,
            0.006370878, 0.003355011, 0.000614463, 0.001216974, 0.002909605,
            0.003388747, 0.001123733, 0.005390725, 0.004329468, 3.94199e-05
        ], [0.001834927, 0.004734064, 0.005585484, 0.002190064, 0.001642327,
            0.007752586, 0.001186738, 0.001664156, 0.005479113, 0.005771393,
            0.125462151, 0.010012938, 0.005127048, 0.01032467, 0.006967562,
            0.006423405, 0.001693166, 0.083343163, 0.003402947, 0.001214874,
            0.001057284, 0.003279575, 0.001358045, 0.000523046, 0.000602087,
            0.001359664, 0.001169622, 0.000134833, 0.001717374, 0.000999411,
            0.001359908, 0.00163521, 0.001998698, 0.001902704, 2.61914e-05
        ], [0.004261421, 0.020362028, 0.012386326, 0.005272169, 0.010199828,
            0.023280842, 0.008570557, 0.005671894, 0.01239771, 0.023555356,
            0.031333724, 0.321518246, 0.159678259, 0.063751555, 0.093853037,
            0.074937804, 0.006613513, 0.086532224, 0.015888973, 0.00362584,
            0.00343342, 0.004047003, 0.006377229, 0.005282795, 0.004324035,
            0.007171268, 0.003621124, 0.000578018, 0.002441464, 0.003655531,
            0.004361188, 0.00175067, 0.002113966, 0.004330731, 0.000154949
        ], [0.007166829, 0.016347588, 0.003809717, 0.008639319, 0.006728445,
            0.008894996, 0.00671229, 0.006249665, 0.007104845, 0.011952329,
            0.018151625, 0.019719631, 0.141812451, 0.015956102, 0.035506258,
            0.011054115, 0.008453808, 0.018114996, 0.024208671, 0.003407595,
            0.00260478, 0.002202963, 0.005718248, 0.00926745, 0.006337766,
            0.006280329, 0.003007989, 0.000895135, 0.001632823, 0.002967213,
            0.006239441, 0.002185663, 0.00305097, 0.00390165, 0.000177651
        ], [0.001922095, 0.007057038, 0.003626999, 0.006119527, 0.005091534,
            0.007179131, 0.010308781, 0.003635923, 0.009855022, 0.013683245,
            0.010784588, 0.016824309, 0.097852884, 0.329431132, 0.060316567,
            0.02400519, 0.020611597, 0.034008027, 0.031898234, 0.008773378,
            0.008319927, 0.004751849, 0.007069864, 0.004027193, 0.005168274,
            0.00967168, 0.040675902, 0.003916807, 0.002334241, 0.016270503,
            0.016336087, 0.008471238, 0.023887902, 0.011778101, 0.000735315
        ], [0.004320812, 0.005753344, 0.002009984, 0.002739141, 0.002566015,
            0.003445567, 0.002714257, 0.001106465, 0.002296997, 0.004546809,
            0.004575048, 0.00600268, 0.015288036, 0.004585536, 0.305775298,
            0.005902207, 0.002613062, 0.004988514, 0.088045704, 0.004907881,
            0.004025062, 0.001656171, 0.036445814, 0.035195115, 0.048640529,
            0.01314878, 0.003809902, 0.001064828, 0.000978667, 0.004213627,
            0.016924261, 0.001920796, 0.001383488, 0.007221647, 0.000211328
        ], [0.000579394, 0.000778085, 0.000924919, 0.004512342, 0.002867608,
            0.004230448, 0.003559956, 0.00054411, 0.001663058, 0.002545392,
            0.002799404, 0.008731866, 0.003730334, 0.002388684, 0.005031552,
            0.045432723, 0.001266639, 0.005135101, 0.001958754, 0.001227386,
            0.001504788, 0.002332128, 0.002313717, 0.000733862, 0.001846307,
            0.001535596, 0.000974208, 0.000752306, 0.001378941, 0.001733414,
            0.003678171, 0.002878137, 0.002852043, 0.00494621, 0.002095997
        ], [0.010895078, 0.031801106, 0.013475023, 0.018403715, 0.009014682,
            0.017199958, 0.022725537, 0.016537861, 0.025514741, 0.020469341,
            0.040864561, 0.028707507, 0.012024776, 0.007943381, 0.008674537,
            0.009461703, 0.111596382, 0.006345553, 0.011090114, 0.006444749,
            0.014120193, 0.023013965, 0.015664185, 0.00358496, 0.00540354,
            0.014621115, 0.009379055, 0.004058993, 0.009643195, 0.005655694,
            0.014948637, 0.02060814, 0.01206655, 0.015876136, 0.000201539
        ], [0.003470342, 0.007120332, 0.002378891, 0.002658841, 0.001653622,
            0.003141269, 0.00320737, 0.00254911, 0.003152373, 0.00270067,
            0.006624926, 0.003860614, 0.00295116, 0.002077376, 0.001898853,
            0.002554001, 0.017918272, 0.048182347, 0.00483784, 0.003782974,
            0.004807617, 0.005722697, 0.009611398, 0.00217123, 0.003098111,
            0.012565108, 0.010237291, 0.00491485, 0.033120721, 0.004727129,
            0.018353923, 0.012602767, 0.006405615, 0.009396977, 0.00070441
        ], [0.005282912, 0.003598247, 0.006856476, 0.005421049, 0.006929249,
            0.006356651, 0.004948231, 0.002506859, 0.005916747, 0.006265038,
            0.00720621, 0.005758477, 0.005025926, 0.003595842, 0.004945267,
            0.006536695, 0.008211002, 0.007043383, 0.010966473, 0.004269748,
            0.004187072, 0.004964631, 0.016345993, 0.002096372, 0.004924442,
            0.00761504, 0.003731438, 0.002739897, 0.001978044, 0.005658818,
            0.004444153, 0.00286377, 0.003694978, 0.006251856, 2.80576e-05
        ], [0.032777521, 0.02110555, 0.06368301, 0.049249078, 0.061469469,
            0.055447874, 0.050694872, 0.044464836, 0.046490333, 0.048282926,
            0.040489202, 0.045149444, 0.046478473, 0.036923667, 0.044896431,
            0.043236626, 0.022001604, 0.04118116, 0.035062924, 0.025866408,
            0.013186039, 0.047030414, 0.022747238, 0.020284125, 0.021777647,
            0.019242452, 0.011022951, 0.003980189, 0.004104011, 0.01318716,
            0.019526111, 0.011142819, 0.02662328, 0.017040682, 0.000183597
        ], [0.016754845, 0.007888115, 0.030577793, 0.02609504, 0.035923866,
            0.02066193, 0.018359288, 0.011646203, 0.021803469, 0.020076638,
            0.018311236, 0.016560416, 0.016621381, 0.012165789, 0.018842711,
            0.025864797, 0.007757562, 0.031729372, 0.01599211, 0.006318014,
            0.007484716, 0.024150824, 0.014064197, 0.00734233, 0.009427026,
            0.00864514, 0.007992048, 0.004415165, 0.003285148, 0.00862798,
            0.008338142, 0.005856782, 0.011874497, 0.010747663, 0.000283483
        ], [0.001403886, 0.003422119, 0.003813873, 0.004138982, 0.004454108,
            0.00449465, 0.005704123, 0.001035525, 0.003972577, 0.00508278,
            0.005065331, 0.004093555, 0.005430523, 0.004219379, 0.002666909,
            0.003592803, 0.003202749, 0.004708313, 0.005596664, 0.00802711,
            0.00640856, 0.010555608, 0.007873096, 0.00408579, 0.01884247,
            0.019485369, 0.010323601, 0.010469929, 0.002966388, 0.014174288,
            0.011814844, 0.00800883, 0.00902257, 0.011807014, 7.71145e-05
        ], [0.012868066, 0.020908268, 0.023998675, 0.019841946, 0.020937168,
            0.023998349, 0.019281698, 0.037086284, 0.01793774, 0.017015159,
            0.033245421, 0.017233814, 0.012426117, 0.008170377, 0.011131894,
            0.018259774, 0.018530842, 0.017958603, 0.015589194, 0.02013986,
            0.012155245, 0.00945805, 0.045309272, 0.025170108, 0.011419987,
            0.054886753, 0.008705922, 0.004918931, 0.001903711, 0.005645683,
            0.010982664, 0.010052844, 0.007356567, 0.009257093, 0.000345466
        ], [0.003273646, 0.006687834, 0.004795575, 0.004830774, 0.0053747,
            0.007238595, 0.003742332, 0.004478196, 0.003146049, 0.003670991,
            0.00891332, 0.005745904, 0.003470105, 0.002281168, 0.002216544,
            0.002909818, 0.003217791, 0.003891566, 0.001702622, 0.004752125,
            0.002422808, 0.00165016, 0.003299112, 0.130220089, 0.002988739,
            0.005146187, 0.000930586, 0.000576917, 0.001464546, 0.001304162,
            0.00131489, 0.001059093, 0.000558266, 0.001290716, 2.86248e-05
        ], [0.000743863, 0.001748947, 0.00142468, 0.001703018, 0.002410709,
            0.001828296, 0.00235236, 0.000579938, 0.001654753, 0.001735193,
            0.001587115, 0.001216616, 0.001995105, 0.001723807, 0.001409796,
            0.001318862, 0.000716554, 0.001050993, 0.002017029, 0.002921268,
            0.001526036, 0.001310198, 0.002660803, 0.006747063, 0.032383603,
            0.015666111, 0.003797469, 0.003032389, 0.00054147, 0.004068414,
            0.004437001, 0.002515988, 0.001302166, 0.004044595, 1.86978e-05
        ], [0.003273243, 0.004370424, 0.00658271, 0.003984328, 0.004735921,
            0.006914874, 0.00912793, 0.006729886, 0.004838507, 0.005233187,
            0.008267752, 0.004913782, 0.004385128, 0.003091349, 0.004115351,
            0.004464656, 0.003887284, 0.006755184, 0.011249819, 0.020574149,
            0.011900001, 0.007561361, 0.040058647, 0.114880954, 0.089047813,
            0.10307933, 0.004463321, 0.003422451, 0.000987415, 0.005035405,
            0.004834355, 0.00266042, 0.003214743, 0.005942764, 2.15542e-05
        ], [0.003260901, 0.005380721, 0.005039436, 0.006079035, 0.006958574,
            0.006146597, 0.012615371, 0.00285334, 0.007761855, 0.006030183,
            0.006846955, 0.006293022, 0.007155132, 0.007341786, 0.004461788,
            0.008795784, 0.006124619, 0.011879435, 0.01682986, 0.020109124,
            0.022020881, 0.014754838, 0.016821131, 0.018133909, 0.023216889,
            0.024964194, 0.120821798, 0.028248006, 0.005592943, 0.026471825,
            0.022731015, 0.010791448, 0.013578332, 0.022050936, 0.000616615
        ], [0.01854221, 0.023522861, 0.014869532, 0.020924875, 0.015155447,
            0.015988004, 0.020402987, 0.010693993, 0.016366496, 0.014651417,
            0.0207937, 0.016167776, 0.016281046, 0.012640241, 0.012446209,
            0.026963373, 0.020435395, 0.020624369, 0.019076463, 0.031963436,
            0.034566329, 0.023097302, 0.044417657, 0.030357965, 0.031960473,
            0.029946642, 0.019765189, 0.206639527, 0.079588141, 0.035836585,
            0.023363868, 0.012613115, 0.022090894, 0.041001945, 0.004490453
        ], [0.004046214, 0.00497516, 0.004779299, 0.004975625, 0.005563769,
            0.006029424, 0.009822958, 0.00175156, 0.004328436, 0.005948212,
            0.004885014, 0.004754142, 0.005484765, 0.004130493, 0.004523066,
            0.006691064, 0.004756888, 0.010234063, 0.022839331, 0.021554805,
            0.043988229, 0.029466194, 0.009152896, 0.015479464, 0.01217382,
            0.022525154, 0.016185175, 0.024066165, 0.031416737, 0.025261265,
            0.014136595, 0.01943379, 0.035447507, 0.027245168, 9.70666e-07
        ], [0.011684419, 0.028127627, 0.037885943, 0.024481547, 0.021841685,
            0.025891898, 0.060064581, 0.011148973, 0.062561236, 0.036350826,
            0.033398437, 0.027727248, 0.040990695, 0.043932444, 0.038110794,
            0.02791529, 0.03033085, 0.052292345, 0.050925038, 0.05580592,
            0.061327817, 0.043013596, 0.043281937, 0.030174422, 0.077756938,
            0.065657216, 0.065950401, 0.099220276, 0.037784607, 0.142798953,
            0.081697621, 0.033662221, 0.059977876, 0.076517609, 0.002813168
        ], [0.001488437, 0.002912525, 0.001850409, 0.000994792, 0.000773519,
            0.003272953, 0.003077746, 0.000636403, 0.002532328, 0.001811901,
            0.002351759, 0.00244771, 0.001073634, 0.000977694, 0.001123275,
            0.001970133, 0.004378165, 0.001506495, 0.002682509, 0.001541966,
            0.002750109, 0.003701325, 0.003479578, 0.005614576, 0.009029847,
            0.00327008, 0.002961007, 0.001584726, 0.003814791, 0.00301024,
            0.007360619, 0.004435898, 0.004581914, 0.008651159, 3.20839e-06
        ], [0.000556463, 0.000447731, 0.00047205, 0.000400693, 0.000501891,
            0.000379925, 0.000642913, 0.00034118, 0.00092247, 0.000659945,
            0.000607519, 0.000599086, 0.000818485, 0.000710904, 0.000716141,
            0.000445602, 0.000777906, 0.000519213, 0.000801209, 0.000853002,
            0.000955051, 0.000617786, 0.00113914, 0.000351224, 0.001951004,
            0.001260372, 0.001689898, 0.001208885, 0.000317854, 0.002431538,
            0.004526127, 0.013493987, 0.001216405, 0.001655838, 1.01413e-05
        ], [0.001310278, 0.000419133, 0.000443122, 0.000394217, 0.000933898,
            0.000435614, 0.000429299, 0.000151702, 0.000510766, 0.000383228,
            0.000510215, 0.000377838, 0.000648206, 0.000364316, 0.000377947,
            0.000498168, 0.000362422, 0.000394267, 0.000543517, 0.000566984,
            0.000438814, 0.000933967, 0.000635268, 0.000297199, 0.000408273,
            0.000563743, 0.000685173, 0.000487641, 8.93916e-05, 0.000677312,
            0.004856382, 0.0015125, 0.026453025, 0.001228184, 1.97896e-07
        ], [0.002742041, 0.006223616, 0.005319393, 0.00466787, 0.006791016,
            0.005368081, 0.012590739, 0.002020276, 0.007134488, 0.005226321,
            0.005806376, 0.005362873, 0.004995379, 0.003744881, 0.004303695,
            0.004936699, 0.005403125, 0.006186484, 0.007847876, 0.007806793,
            0.008860767, 0.013414861, 0.011141821, 0.005432603, 0.007862597,
            0.012501183, 0.019738331, 0.01003502, 0.008133371, 0.026067278,
            0.017187588, 0.011851185, 0.014127885, 0.078559943, 0.00235424
        ], [1.40514e-05, 2.34077e-05, 2.46892e-05, 6.78197e-05, 2.22322e-05,
            2.95985e-05, 3.5102e-05, 8.62592e-06, 1.84598e-05, 2.19341e-05,
            1.9998e-05, 1.61258e-05, 2.82623e-05, 1.47014e-05, 7.49342e-05,
            1.17246e-05, 5.32583e-06, 6.75919e-05, 5.31389e-05, 1.13143e-05,
            4.09037e-05, 5.39261e-05, 9.46746e-05, 1.75429e-05, 2.16006e-05,
            1.80988e-05, 3.9339e-05, 6.18009e-05, 0.000234087, 6.63067e-05,
            6.98526e-05, 3.79116e-05, 5.46024e-05, 0.000118257, 0.000129145
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 4
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2004():
    """
    IA matrix 2004



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2004())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2005():
    """
    IA matrix 2005



    constant


    """

    ret =  xr.DataArray(
        data=[[0.129407266, 0.000650647, 0.260644047, 0.048211664, 0.053440631,
            0.130996855, 0.015431955, 0.001465404, 0.006002795, 0.01322785,
            0.00058427, 0.00023433, 0.000223745, 0.000115862, 0.000138397,
            0.009046737, 0.000518845, 0.005759286, 0.000839906, 0.002675122,
            0.002489806, 0.039642949, 0.001804847, 0.000990375, 0.000338074,
            0.006384407, 0.000254595, 0.000141305, 0.000366732, 0.000831057,
            0.00222532, 0.002597003, 0.002095426, 0.002587768, 0.021122947
        ], [0.002905819, 0.087044665, 0.00295081, 0.002774118, 0.002097788,
            0.004659292, 0.004997145, 0.452335484, 0.031240469, 0.004076666,
            0.071436255, 0.052543655, 0.003233216, 0.001375669, 0.001834267,
            0.005416069, 0.113475558, 0.019015329, 0.000556346, 0.000549434,
            0.001068008, 0.002151481, 0.005343436, 0.003533462, 0.002977951,
            0.002661985, 0.000553977, 0.000285576, 0.001581843, 0.000868381,
            0.003296319, 0.00238784, 0.001520085, 0.00300242, 0.062085974
        ], [0.059353322, 0.000789966, 0.154417972, 0.00433457, 0.095987014,
            0.002092524, 0.002966256, 0.001483261, 0.010775261, 0.003426031,
            0.002245228, 0.001538847, 0.001529925, 0.001294792, 0.001034797,
            0.002471672, 0.00096098, 0.001282255, 0.001831223, 0.004565153,
            0.004774488, 0.151746417, 0.001689988, 0.005136835, 0.003557514,
            0.001815434, 0.001553298, 0.000666602, 0.000277455, 0.00179361,
            0.007547669, 0.008378467, 0.013037927, 0.006927793, 0.002980137
        ], [0.001911529, 0.000949837, 0.0014826, 0.32962188, 0.055317329,
            0.003348604, 0.005644213, 0.000447171, 0.003395753, 0.012894443,
            0.003633707, 0.001680942, 0.002280196, 0.001893926, 0.004968493,
            0.021355538, 0.00071414, 0.002454791, 0.001592572, 0.002290409,
            0.003280742, 0.00288068, 0.001877443, 0.002637305, 0.00166522,
            0.001701006, 0.000724024, 0.000417828, 0.000353352, 0.001353727,
            0.002788647, 0.000988379, 0.003349283, 0.005018983, 9.82303e-05
        ], [0.000134632, 5.6637e-05, 0.000120508, 0.009526298, 0.204427973,
            0.000399627, 0.000603808, 4.06085e-05, 0.000207013, 0.000862661,
            0.000234157, 0.000155928, 0.000189402, 0.000255821, 0.001621714,
            0.004166728, 8.59049e-05, 0.000106069, 0.000413266, 0.000333671,
            0.000260546, 0.000120863, 0.000217261, 0.000114866, 0.000126583,
            0.000134258, 0.000242413, 2.97885e-05, 8.53178e-06, 9.18227e-05,
            0.000451424, 9.75274e-05, 0.000115549, 0.000400131, 3.81655e-06
        ], [0.001753273, 0.001537498, 0.00157046, 0.001307133, 0.001812787,
            0.238789302, 0.010882151, 0.000311857, 0.001452111, 0.003044933,
            0.007060202, 0.003284452, 0.00292554, 0.00160589, 0.002723321,
            0.068520398, 0.000829455, 0.031753511, 0.001227788, 0.001404978,
            0.00125994, 0.002382208, 0.001212336, 0.000574753, 0.000400727,
            0.002657192, 0.000621036, 0.000324807, 0.002585614, 0.000898688,
            0.001819411, 0.002586634, 0.001283893, 0.002679751, 2.84157e-05
        ], [0.002485134, 0.002777199, 0.024640069, 0.011733946, 0.015208491,
            0.013680784, 0.241153492, 0.001622717, 0.015498448, 0.016417142,
            0.020986016, 0.005089012, 0.0074738, 0.009250575, 0.004323103,
            0.01900323, 0.003156423, 0.004264177, 0.008212674, 0.011614958,
            0.012137881, 0.008442088, 0.006311314, 0.00336299, 0.004935213,
            0.013308584, 0.01042477, 0.01309468, 0.002237279, 0.020445682,
            0.018823458, 0.018217811, 0.009015974, 0.024551474, 0.00015857
        ], [0.01195871, 0.006858945, 0.003052102, 0.005250283, 0.004225808,
            0.004732767, 0.004329475, 0.066841371, 0.039968608, 0.008736922,
            0.01805823, 0.011680831, 0.003881836, 0.00199121, 0.001887179,
            0.003499319, 0.021126394, 0.011118151, 0.004593705, 0.004832789,
            0.003647674, 0.003481677, 0.050220688, 0.067572962, 0.083563197,
            0.019560937, 0.003147508, 0.001128488, 0.001074639, 0.0045051,
            0.008799313, 0.002952595, 0.002822244, 0.005008333, 2.64063e-05
        ], [0.036713987, 0.009568343, 0.0092383, 0.067289477, 0.03747888,
            0.032121129, 0.033997279, 0.015856932, 0.238214696, 0.208101049,
            0.034991816, 0.015218858, 0.010570919, 0.017043193, 0.010840196,
            0.024095929, 0.006302418, 0.010443617, 0.005781488, 0.002734012,
            0.002137858, 0.004383812, 0.003400623, 0.004457148, 0.004094321,
            0.004213712, 0.001301131, 0.000695778, 0.001691946, 0.004678638,
            0.009808997, 0.003682126, 0.06131502, 0.009554097, 0.000165512
        ], [0.004680617, 0.005657506, 0.018239521, 0.010710911, 0.037484487,
            0.008708587, 0.015914837, 0.002537139, 0.017460022, 0.139599043,
            0.010688006, 0.006820016, 0.024635428, 0.023416767, 0.034788563,
            0.032043921, 0.002321393, 0.018583004, 0.016366331, 0.004294528,
            0.005219752, 0.004124881, 0.010682757, 0.002754372, 0.003424567,
            0.006286069, 0.003295565, 0.0006124, 0.001323934, 0.00284299,
            0.003316317, 0.001177295, 0.00523657, 0.00439089, 6.19045e-05
        ], [0.0016503, 0.004562077, 0.005487055, 0.002187115, 0.001610261,
            0.007218209, 0.001194984, 0.001873157, 0.005596031, 0.005631218,
            0.132810306, 0.010230044, 0.005095781, 0.010079401, 0.006813321,
            0.006524235, 0.001800051, 0.086563317, 0.003570485, 0.001014787,
            0.000993044, 0.003225937, 0.00139243, 0.000505353, 0.000633514,
            0.001433525, 0.001149184, 0.000121686, 0.001742069, 0.00095721,
            0.001382664, 0.001645526, 0.002024845, 0.00187696, 3.2515e-05
        ], [0.004212891, 0.019269736, 0.011596165, 0.00527505, 0.009936104,
            0.022201836, 0.008349381, 0.005635519, 0.012425235, 0.023137856,
            0.031583391, 0.321053303, 0.158598335, 0.064321645, 0.089955216,
            0.073279479, 0.006616673, 0.086789933, 0.015504323, 0.003361444,
            0.003024948, 0.003930694, 0.0061908, 0.004268114, 0.003714365,
            0.007872932, 0.003441299, 0.000553337, 0.002487059, 0.003560773,
            0.004230895, 0.001804121, 0.002171703, 0.004386367, 0.000148229
        ], [0.007097314, 0.014448857, 0.004019873, 0.008921373, 0.006945824,
            0.009086034, 0.006947801, 0.006995411, 0.007538004, 0.012359628,
            0.019290812, 0.020762091, 0.147256993, 0.016488768, 0.036528415,
            0.012319617, 0.008398131, 0.018261535, 0.024191178, 0.002985659,
            0.002740448, 0.002233125, 0.005724275, 0.008887788, 0.006753992,
            0.007162733, 0.002906877, 0.000869142, 0.001656431, 0.002851762,
            0.006300919, 0.002112502, 0.003311009, 0.003947009, 0.00028829
        ], [0.001931121, 0.006575571, 0.00381891, 0.006627012, 0.005074295,
            0.007581105, 0.010586795, 0.003861996, 0.010379376, 0.014053659,
            0.011425792, 0.017930596, 0.100950522, 0.348713732, 0.062356742,
            0.026496409, 0.022567543, 0.036100144, 0.03272861, 0.008693774,
            0.008822988, 0.004953857, 0.007712306, 0.003797037, 0.005282437,
            0.010253764, 0.042060695, 0.004008104, 0.002472122, 0.016919341,
            0.016552604, 0.009339832, 0.024449015, 0.012624321, 0.000871221
        ], [0.004426218, 0.005342384, 0.00200826, 0.002849518, 0.002541097,
            0.003750743, 0.002986716, 0.00118084, 0.002404322, 0.004586649,
            0.005187802, 0.006238083, 0.015956918, 0.004728054, 0.31007272,
            0.006427711, 0.002916423, 0.005244733, 0.085463362, 0.004279998,
            0.004527306, 0.001781101, 0.0367579, 0.035624065, 0.050449192,
            0.014320342, 0.003717044, 0.001051878, 0.000983324, 0.004205479,
            0.017287341, 0.002112031, 0.001561384, 0.007756631, 0.000224268
        ], [0.000558685, 0.000733802, 0.000876293, 0.004789633, 0.00282577,
            0.004113194, 0.003351386, 0.00059164, 0.001728613, 0.002604131,
            0.002842458, 0.008378705, 0.003730086, 0.002419918, 0.005030859,
            0.046198151, 0.001459098, 0.005146446, 0.001861217, 0.001144558,
            0.001479379, 0.00225576, 0.002780176, 0.000711091, 0.001960501,
            0.0014536, 0.001058371, 0.000724389, 0.001359112, 0.001678876,
            0.003578828, 0.003010564, 0.002912156, 0.005085208, 0.002137575
        ], [0.011182154, 0.030935854, 0.014082416, 0.01857037, 0.008867421,
            0.017680845, 0.02332652, 0.023474905, 0.026782159, 0.021110639,
            0.043401052, 0.030497109, 0.012270802, 0.007893999, 0.008529122,
            0.00946265, 0.121503738, 0.006792746, 0.011227907, 0.006410963,
            0.013983072, 0.024164619, 0.015981197, 0.003637463, 0.005556291,
            0.015423508, 0.009788389, 0.004124636, 0.010397561, 0.005795413,
            0.014749937, 0.021319208, 0.012475232, 0.016102295, 0.000186442
        ], [0.003264861, 0.006702271, 0.002165694, 0.002396654, 0.001571745,
            0.002921596, 0.002958466, 0.003621247, 0.003050628, 0.002538199,
            0.00604943, 0.003835207, 0.002815161, 0.001785025, 0.001672813,
            0.002474762, 0.016736522, 0.049153588, 0.004957492, 0.003427298,
            0.004325169, 0.00540782, 0.009056831, 0.001994665, 0.002649261,
            0.012023383, 0.01027796, 0.00459414, 0.032910001, 0.004456108,
            0.017806667, 0.011849047, 0.006240185, 0.008922026, 0.000637504
        ], [0.005077194, 0.003244889, 0.006812394, 0.005103968, 0.00653888,
            0.006191643, 0.004855712, 0.002633317, 0.005846966, 0.006208917,
            0.006840217, 0.005776848, 0.004845495, 0.003380496, 0.004861112,
            0.006451408, 0.007581974, 0.007122095, 0.011562275, 0.004438694,
            0.004145376, 0.005212598, 0.01617198, 0.002091893, 0.005098273,
            0.007602102, 0.003763763, 0.002831702, 0.002005178, 0.005601884,
            0.004419625, 0.002879966, 0.003940925, 0.006284869, 2.38989e-05
        ], [0.030851912, 0.017975124, 0.062220143, 0.04551815, 0.054866815,
            0.052000957, 0.047336616, 0.045950968, 0.04455119, 0.045607138,
            0.037132218, 0.041890623, 0.043186982, 0.033395172, 0.042604976,
            0.041179635, 0.02021959, 0.039643832, 0.034451659, 0.02435927,
            0.012831319, 0.046487739, 0.022128022, 0.018102298, 0.020555998,
            0.018879508, 0.010778854, 0.003906073, 0.00413166, 0.012500573,
            0.020345829, 0.011028527, 0.027063508, 0.016680146, 0.000174201
        ], [0.017830226, 0.007940905, 0.031000143, 0.025750075, 0.036179793,
            0.021047431, 0.018930128, 0.013413707, 0.022071761, 0.021251528,
            0.019447514, 0.017297004, 0.017734983, 0.013301858, 0.019000799,
            0.025903341, 0.007700226, 0.032832903, 0.015485717, 0.006320329,
            0.007240135, 0.025049326, 0.014649677, 0.007966671, 0.009991881,
            0.008750792, 0.008750579, 0.004562202, 0.003334578, 0.008530004,
            0.008313508, 0.006217786, 0.013076211, 0.011321933, 0.000249546
        ], [0.001444178, 0.003152691, 0.003863286, 0.004176122, 0.004642994,
            0.004196205, 0.005433718, 0.001093906, 0.003998873, 0.004973607,
            0.005178576, 0.004195739, 0.005260058, 0.003936277, 0.002626398,
            0.003462772, 0.003526231, 0.00485914, 0.005397463, 0.007594002,
            0.00679631, 0.010369621, 0.007953486, 0.004458948, 0.016647972,
            0.021141741, 0.009974747, 0.010536226, 0.00303208, 0.013480371,
            0.011587132, 0.008650888, 0.008828916, 0.012054338, 7.08649e-05
        ], [0.013093933, 0.019158568, 0.024082448, 0.019974762, 0.020430924,
            0.02382665, 0.01889971, 0.038988471, 0.018141791, 0.016955931,
            0.033648246, 0.017616278, 0.012632947, 0.008039097, 0.010833538,
            0.01760249, 0.017792234, 0.018742037, 0.016435906, 0.020739318,
            0.012938506, 0.009387936, 0.046356797, 0.018366772, 0.010305754,
            0.055520955, 0.008694981, 0.004856224, 0.001826828, 0.005620762,
            0.010591761, 0.009869813, 0.007388449, 0.009366565, 0.000375522
        ], [0.002831936, 0.006161648, 0.004636015, 0.004522933, 0.004722345,
            0.006701431, 0.0035552, 0.004025319, 0.003112518, 0.003374563,
            0.008866352, 0.005725165, 0.003286935, 0.002054876, 0.002305215,
            0.002479978, 0.002873978, 0.00324491, 0.001849056, 0.00469775,
            0.00352184, 0.001597879, 0.003410306, 0.147968734, 0.002669404,
            0.005143492, 0.001004655, 0.000618611, 0.001969239, 0.001313919,
            0.001213239, 0.000940972, 0.000587853, 0.001278146, 3.04805e-05
        ], [0.000730591, 0.001465195, 0.001348596, 0.001642844, 0.002366445,
            0.001722058, 0.002164973, 0.000602314, 0.001660427, 0.001646754,
            0.001550728, 0.001179379, 0.002011179, 0.001664866, 0.001326833,
            0.001230259, 0.000716295, 0.001059312, 0.00198424, 0.002778076,
            0.001598163, 0.001247832, 0.002452365, 0.005748731, 0.033483196,
            0.014492036, 0.00381425, 0.002860106, 0.000557478, 0.003700057,
            0.004089981, 0.002589643, 0.001199663, 0.004038537, 1.60473e-05
        ], [0.003374788, 0.004543229, 0.006718621, 0.004029747, 0.004764401,
            0.007082587, 0.008994979, 0.00668994, 0.00494354, 0.005294583,
            0.008365154, 0.005078262, 0.004333409, 0.002959532, 0.003961708,
            0.004415264, 0.003737409, 0.008729278, 0.012053968, 0.021286785,
            0.012245301, 0.007712058, 0.039855524, 0.108508255, 0.091956826,
            0.100178513, 0.004479325, 0.003460178, 0.001023461, 0.005190858,
            0.004884529, 0.002780311, 0.003287201, 0.006215502, 1.84633e-05
        ], [0.00343404, 0.005240851, 0.005072714, 0.006126263, 0.006610965,
            0.006008481, 0.012404776, 0.003015145, 0.007634687, 0.005835263,
            0.006564583, 0.006500211, 0.006797141, 0.00722554, 0.004222,
            0.009202505, 0.006560175, 0.011853921, 0.01684637, 0.020089933,
            0.021834229, 0.015009406, 0.017229866, 0.017647614, 0.021808944,
            0.023158257, 0.124185672, 0.028763578, 0.005900286, 0.025204805,
            0.022928371, 0.011017239, 0.013902988, 0.022118251, 0.000627489
        ], [0.017347131, 0.022630324, 0.014508319, 0.020478015, 0.014988824,
            0.015162996, 0.019887103, 0.011017624, 0.016733802, 0.014579519,
            0.020571794, 0.016797995, 0.016246581, 0.012272165, 0.012103828,
            0.026188187, 0.021332481, 0.020759179, 0.019553436, 0.031340061,
            0.03465936, 0.023415087, 0.043419579, 0.026724835, 0.029898973,
            0.031359054, 0.019741806, 0.216110234, 0.083556938, 0.034965726,
            0.023407048, 0.012767685, 0.022039024, 0.042146929, 0.005907651
        ], [0.003499047, 0.005352966, 0.004660143, 0.005054754, 0.005835402,
            0.005865633, 0.010070676, 0.001733642, 0.004450206, 0.005971127,
            0.004988462, 0.004825793, 0.005384872, 0.004009437, 0.004633981,
            0.006647411, 0.004433356, 0.010194526, 0.022642875, 0.021205073,
            0.044618445, 0.030755298, 0.009316862, 0.013844826, 0.010379309,
            0.021176507, 0.016789021, 0.025945886, 0.029472539, 0.025544885,
            0.013702936, 0.020262007, 0.036581015, 0.027758378, 1.08714e-06
        ], [0.011765759, 0.02982723, 0.038387458, 0.023631422, 0.020820219,
            0.025485048, 0.05953744, 0.012054902, 0.063972124, 0.036031848,
            0.032487434, 0.028107663, 0.04043785, 0.041192404, 0.038742298,
            0.027655476, 0.031621594, 0.053992843, 0.050788475, 0.056764743,
            0.061299459, 0.04425481, 0.042748595, 0.026931687, 0.076527458,
            0.062529108, 0.064918349, 0.10309422, 0.04020853, 0.143759349,
            0.083512264, 0.035343418, 0.062340311, 0.078390097, 0.002788265
        ], [0.001502843, 0.002744611, 0.001866032, 0.000964354, 0.000755469,
            0.003270834, 0.003290283, 0.000875671, 0.002633277, 0.001874419,
            0.002496498, 0.002792265, 0.001158077, 0.000929848, 0.001130445,
            0.001943336, 0.00440282, 0.001543761, 0.002731223, 0.001567762,
            0.002669173, 0.003690991, 0.003734814, 0.004992963, 0.008940156,
            0.003212153, 0.002869577, 0.001603891, 0.004072869, 0.00301024,
            0.007531503, 0.004519783, 0.004299027, 0.008697472, 4.0572e-06
        ], [0.000735356, 0.000410946, 0.000440117, 0.000369504, 0.000445293,
            0.000353449, 0.000554251, 0.000311603, 0.000913317, 0.000597087,
            0.000545281, 0.000550824, 0.000759887, 0.000648529, 0.000703221,
            0.000421326, 0.000712778, 0.000481851, 0.000784288, 0.000844198,
            0.001040811, 0.000615682, 0.001082461, 0.000317147, 0.001906748,
            0.001176885, 0.001459695, 0.001236012, 0.000296414, 0.002505376,
            0.004620743, 0.013781721, 0.001440354, 0.001599843, 1.20743e-05
        ], [0.001278327, 0.000412609, 0.000479785, 0.000400545, 0.00097212,
            0.000470736, 0.000460941, 0.000164704, 0.000523372, 0.000403503,
            0.000609777, 0.000426443, 0.000748771, 0.000382807, 0.000421159,
            0.000500744, 0.000383126, 0.000402224, 0.000553848, 0.000579074,
            0.00044152, 0.000986869, 0.000636852, 0.000307647, 0.000414647,
            0.000522371, 0.000671283, 0.000556354, 9.28391e-05, 0.000637537,
            0.005102913, 0.001630615, 0.034919459, 0.001290866, 3.40349e-07
        ], [0.002860947, 0.006296248, 0.005406703, 0.004564652, 0.006561396,
            0.00509183, 0.012285627, 0.001971398, 0.007101557, 0.00507267,
            0.005724065, 0.005404879, 0.004738551, 0.003411511, 0.004148151,
            0.00461282, 0.005593763, 0.006218522, 0.00752703, 0.007576472,
            0.009056544, 0.013762758, 0.011928935, 0.005016571, 0.007522326,
            0.012761927, 0.019301146, 0.010177614, 0.008355319, 0.025270632,
            0.017366, 0.012370527, 0.014076732, 0.078238306, 0.001972603
        ], [1.43478e-05, 2.44395e-05, 2.3741e-05, 7.7755e-05, 2.5745e-05,
            2.88407e-05, 3.09734e-05, 1.10871e-05, 1.95301e-05, 2.14994e-05,
            2.08148e-05, 1.65598e-05, 3.07705e-05, 1.46546e-05, 7.64555e-05,
            1.25023e-05, 6.2727e-06, 6.72929e-05, 4.57973e-05, 1.13875e-05,
            3.80354e-05, 5.28132e-05, 0.000109889, 2.57128e-05, 1.60338e-05,
            1.49606e-05, 3.75788e-05, 6.07456e-05, 0.000224378, 6.42811e-05,
            6.58038e-05, 4.05495e-05, 5.21875e-05, 0.000110644, 0.000116148
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 5
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2005():
    """
    IA matrix 2005



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2005())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2006():
    """
    IA matrix 2006



    constant


    """

    ret =  xr.DataArray(
        data=[[0.126353608, 0.000578661, 0.259175245, 0.049574397, 0.055531638,
            0.127181871, 0.015353242, 0.001613421, 0.006442175, 0.013314001,
            0.000579479, 0.00021205, 0.00019473, 0.000101236, 0.000126664,
            0.01062746, 0.000540087, 0.004593932, 0.000869679, 0.002499365,
            0.002258542, 0.039466377, 0.001760788, 0.000977682, 0.000375788,
            0.006138668, 0.000228343, 0.000133382, 0.00038566, 0.000819952,
            0.002162223, 0.002705486, 0.002159961, 0.002489017, 0.0250289
        ], [0.002354299, 0.07702691, 0.002808492, 0.002783506, 0.002320969,
            0.004249805, 0.004827237, 0.471836678, 0.030358879, 0.003719742,
            0.070027467, 0.052002984, 0.002911538, 0.001128618, 0.001584064,
            0.005099451, 0.109191662, 0.017907317, 0.000577412, 0.000520574,
            0.00106199, 0.002062987, 0.004997847, 0.003395164, 0.003148286,
            0.002852291, 0.000465793, 0.000266682, 0.001610161, 0.000807234,
            0.003065207, 0.001796453, 0.001515867, 0.002702593, 0.063078972
        ], [0.063383379, 0.000859104, 0.159297479, 0.004964434, 0.101407016,
            0.002528716, 0.003195579, 0.001425139, 0.011520503, 0.003851979,
            0.002606184, 0.001848427, 0.001694569, 0.001380885, 0.001119431,
            0.002785363, 0.001101275, 0.001486819, 0.00167385, 0.004626085,
            0.004745071, 0.153086001, 0.001801392, 0.005774814, 0.00420067,
            0.001790164, 0.0014338, 0.000679566, 0.000294701, 0.001861802,
            0.007427219, 0.009023462, 0.012685396, 0.007437019, 0.003189991
        ], [0.001856181, 0.000988801, 0.001584207, 0.348676124, 0.0584062,
            0.003400542, 0.005926682, 0.000424688, 0.003533135, 0.013880935,
            0.003906671, 0.001923456, 0.002316114, 0.001851559, 0.004641489,
            0.023008788, 0.00082431, 0.002531016, 0.001644294, 0.002420335,
            0.003309733, 0.002916812, 0.001975634, 0.00307261, 0.00194817,
            0.00184277, 0.000707605, 0.000425935, 0.000392042, 0.001443461,
            0.002838944, 0.001039656, 0.003524866, 0.004868624, 0.00010012
        ], [0.000155707, 5.65087e-05, 0.000131264, 0.011271419, 0.214473673,
            0.000475366, 0.000732601, 3.65431e-05, 0.000231047, 0.000932479,
            0.000245296, 0.000150184, 0.000190717, 0.000280806, 0.001761344,
            0.004974638, 8.66439e-05, 0.000109017, 0.000388298, 0.000338559,
            0.000264887, 0.000130199, 0.000215471, 0.000107699, 0.000128098,
            0.000152806, 0.000234722, 3.0259e-05, 9.10679e-06, 9.0971e-05,
            0.000576995, 9.75224e-05, 0.000124782, 0.000426651, 1.59646e-06
        ], [0.001732719, 0.001539717, 0.001685894, 0.0014246, 0.001929074,
            0.252826435, 0.011376995, 0.000327804, 0.001477083, 0.003148788,
            0.007663424, 0.003593639, 0.003010579, 0.001597147, 0.002680533,
            0.072130628, 0.000867512, 0.031057058, 0.001208337, 0.0013551,
            0.001232685, 0.002269838, 0.001238074, 0.000614261, 0.000392538,
            0.00246283, 0.000598313, 0.00030345, 0.002666698, 0.000895028,
            0.001907578, 0.002708949, 0.001359781, 0.002675287, 2.93231e-05
        ], [0.002445378, 0.002771827, 0.024390171, 0.011672574, 0.014894388,
            0.013947968, 0.245662876, 0.001536014, 0.015010131, 0.016360368,
            0.021076278, 0.004968636, 0.006996969, 0.00844742, 0.004151895,
            0.018835228, 0.003150511, 0.004198331, 0.008139855, 0.011292064,
            0.011487543, 0.008296405, 0.006376676, 0.003332919, 0.005099926,
            0.013357856, 0.00969479, 0.012300274, 0.002147534, 0.019902836,
            0.018365682, 0.0182563, 0.008984075, 0.023450349, 0.00017142
        ], [0.012296689, 0.006197514, 0.003189736, 0.005246624, 0.004106444,
            0.004728704, 0.004425816, 0.11046223, 0.04634872, 0.008888002,
            0.019029249, 0.012473151, 0.003615087, 0.001892582, 0.00184719,
            0.003787901, 0.024448984, 0.011062565, 0.004656503, 0.005251188,
            0.003510072, 0.003518332, 0.053029794, 0.070701164, 0.091970687,
            0.020130834, 0.003169807, 0.001126254, 0.001097387, 0.004455482,
            0.008233978, 0.003079025, 0.00278392, 0.005078632, 4.14333e-05
        ], [0.03602533, 0.009273487, 0.009188741, 0.067188262, 0.037316062,
            0.03356572, 0.034323852, 0.016677758, 0.245179556, 0.215229144,
            0.036547373, 0.01500592, 0.010218115, 0.017176345, 0.010607278,
            0.025662563, 0.006441504, 0.01063748, 0.005501374, 0.002673826,
            0.002114347, 0.004450969, 0.003471967, 0.004767379, 0.004103877,
            0.004356969, 0.001233569, 0.000646906, 0.001872376, 0.00467927,
            0.009912151, 0.003740773, 0.062186174, 0.009771937, 0.000197471
        ], [0.004644235, 0.005480078, 0.018536192, 0.010864814, 0.038709975,
            0.00856205, 0.016297988, 0.003067999, 0.017542598, 0.14743025,
            0.011448123, 0.00691939, 0.023621231, 0.023009403, 0.034019138,
            0.03295762, 0.002383339, 0.01865682, 0.015705708, 0.004335611,
            0.005003571, 0.004143101, 0.010780879, 0.002977905, 0.003681965,
            0.006338039, 0.003175753, 0.000588139, 0.001346187, 0.00283659,
            0.0033388, 0.00131373, 0.005312801, 0.004582117, 6.24787e-05
        ], [0.00157033, 0.004404432, 0.005257702, 0.002114211, 0.001609134,
            0.00733382, 0.001225667, 0.001662871, 0.005542927, 0.005522422,
            0.136896554, 0.010126491, 0.004965878, 0.009646621, 0.006521259,
            0.006846732, 0.001761509, 0.089645408, 0.0034635, 0.000989584,
            0.000993221, 0.003207204, 0.001430171, 0.000467887, 0.000629122,
            0.001448196, 0.001107853, 0.000116744, 0.001744046, 0.000973874,
            0.00149653, 0.00163641, 0.001938726, 0.001850372, 2.93181e-05
        ], [0.004298805, 0.018532625, 0.011037826, 0.005196941, 0.010001769,
            0.02210544, 0.008688894, 0.005449553, 0.012605592, 0.023370394,
            0.032989008, 0.329064501, 0.156950122, 0.064640228, 0.088379728,
            0.075703719, 0.006679437, 0.088129126, 0.014808776, 0.003273522,
            0.002921488, 0.003978874, 0.006166259, 0.004621099, 0.003772319,
            0.007940782, 0.003207529, 0.00052676, 0.002495091, 0.003567813,
            0.004286483, 0.001838006, 0.002175256, 0.004456677, 0.000154051
        ], [0.007342646, 0.015193346, 0.004106389, 0.009878374, 0.007650599,
            0.010156154, 0.007532403, 0.006940622, 0.008224958, 0.013489368,
            0.021669771, 0.023872946, 0.160076155, 0.017544115, 0.038851903,
            0.013713839, 0.008695801, 0.019172433, 0.023674846, 0.002897618,
            0.002680144, 0.002394384, 0.005923745, 0.01015585, 0.008025963,
            0.008009666, 0.002867663, 0.00088181, 0.001673401, 0.002868994,
            0.006439316, 0.002229013, 0.003959571, 0.00405124, 0.000348229
        ], [0.002053102, 0.007361563, 0.003973277, 0.007027596, 0.005417426,
            0.008147683, 0.010799407, 0.003693025, 0.010790754, 0.014753359,
            0.012393513, 0.018687795, 0.106936267, 0.372716684, 0.06447194,
            0.030904504, 0.025080097, 0.039767809, 0.03262901, 0.008864797,
            0.008923722, 0.005509744, 0.008352088, 0.004023282, 0.005734903,
            0.010879778, 0.043372302, 0.004023293, 0.002667619, 0.01836518,
            0.017894274, 0.011188772, 0.025698506, 0.01379586, 0.001113317
        ], [0.004819752, 0.005134243, 0.002018334, 0.00289466, 0.002652225,
            0.004043986, 0.003365276, 0.001149512, 0.002507153, 0.004816006,
            0.005934844, 0.007050706, 0.016272446, 0.004626887, 0.3135085,
            0.007187691, 0.003186131, 0.005573834, 0.080824593, 0.004359528,
            0.004482199, 0.001946385, 0.038389081, 0.038811554, 0.05317244,
            0.016418411, 0.003469227, 0.001051764, 0.001031971, 0.004225539,
            0.017153164, 0.002373263, 0.001573407, 0.00794159, 0.000257602
        ], [0.000631183, 0.000859341, 0.000952911, 0.004404741, 0.00265629,
            0.003915351, 0.004148457, 0.000563763, 0.001678165, 0.002850847,
            0.00317582, 0.009570334, 0.003644354, 0.002250148, 0.00477563,
            0.046316113, 0.001378461, 0.004944191, 0.0018085, 0.001129128,
            0.001413674, 0.00225629, 0.002510778, 0.000726874, 0.00180856,
            0.001538275, 0.000996998, 0.000713683, 0.001414116, 0.001625988,
            0.003659498, 0.003442683, 0.002962013, 0.005106365, 0.001797644
        ], [0.010996393, 0.028920677, 0.013430082, 0.017294183, 0.008394988,
            0.016795926, 0.022969261, 0.029357197, 0.025382219, 0.0201155,
            0.04229779, 0.028819696, 0.011175707, 0.007078915, 0.007956198,
            0.009266464, 0.131068064, 0.006612782, 0.011193987, 0.00618844,
            0.013082027, 0.023179374, 0.015606356, 0.003447795, 0.005744213,
            0.016378467, 0.009390932, 0.003752513, 0.009671813, 0.005526464,
            0.014153357, 0.020661475, 0.0121669, 0.015732368, 0.000197298
        ], [0.003224017, 0.0065843, 0.00211131, 0.002255046, 0.00143721,
            0.002908867, 0.002992306, 0.005059807, 0.003022039, 0.002488718,
            0.005962045, 0.00368772, 0.00263285, 0.001603869, 0.001658413,
            0.002494621, 0.016096534, 0.050464011, 0.00528668, 0.00348044,
            0.004319689, 0.005456524, 0.008816144, 0.002371367, 0.002743177,
            0.012263442, 0.010091441, 0.004363786, 0.032428437, 0.004395339,
            0.017793549, 0.011594343, 0.00663999, 0.008734113, 0.00064014
        ], [0.005109723, 0.003065619, 0.006652342, 0.004662776, 0.006057692,
            0.00592349, 0.004891465, 0.002491038, 0.005577527, 0.00599639,
            0.006599039, 0.005525325, 0.004547937, 0.003041629, 0.004734044,
            0.006270058, 0.007487648, 0.007139005, 0.011659732, 0.004417067,
            0.004136877, 0.00522445, 0.015617894, 0.002077319, 0.005119586,
            0.007925073, 0.003718809, 0.002783021, 0.002052192, 0.005485195,
            0.004534937, 0.003086931, 0.004036234, 0.006331891, 4.30297e-05
        ], [0.031361435, 0.017111392, 0.060645971, 0.041044026, 0.049545299,
            0.050666132, 0.046795872, 0.05628698, 0.044395024, 0.044618612,
            0.037218248, 0.041193581, 0.041391133, 0.031123726, 0.041370889,
            0.041488618, 0.021414674, 0.039602397, 0.033361823, 0.024831995,
            0.012585901, 0.045541027, 0.022499915, 0.017967485, 0.020740013,
            0.019802708, 0.010773432, 0.003740184, 0.004399552, 0.012474767,
            0.019985627, 0.011323097, 0.026497566, 0.016511191, 0.000308406
        ], [0.017489645, 0.007314404, 0.030157995, 0.02350981, 0.032514527,
            0.020234677, 0.018784986, 0.013211113, 0.021543771, 0.020527795,
            0.018937572, 0.016531858, 0.017060349, 0.012419254, 0.01854132,
            0.026187782, 0.007597318, 0.032488709, 0.014698324, 0.006181922,
            0.007073794, 0.024792385, 0.014342693, 0.007815009, 0.009718276,
            0.008887965, 0.008512434, 0.004393048, 0.003430171, 0.008199643,
            0.008273764, 0.006309853, 0.013290386, 0.011360562, 0.000434081
        ], [0.001481562, 0.003256549, 0.00395929, 0.003988257, 0.004605606,
            0.004164762, 0.005454509, 0.001035597, 0.004019246, 0.004921571,
            0.00530914, 0.004268034, 0.005169252, 0.00371863, 0.00258534,
            0.003495401, 0.003255024, 0.005043125, 0.005230877, 0.00758799,
            0.006664657, 0.010421743, 0.007725276, 0.00426455, 0.016171268,
            0.020941349, 0.009594448, 0.010182118, 0.002884409, 0.013322508,
            0.011401596, 0.009158963, 0.008974341, 0.012271616, 6.50288e-05
        ], [0.013257914, 0.018495114, 0.023635836, 0.01970764, 0.019294701,
            0.023602699, 0.018757135, 0.043683302, 0.018302248, 0.016877316,
            0.034204362, 0.017285308, 0.012421717, 0.007597096, 0.010751247,
            0.019064381, 0.019337281, 0.019348753, 0.017122833, 0.021026012,
            0.012882647, 0.009433152, 0.046053117, 0.012743327, 0.00963078,
            0.056104046, 0.008640463, 0.004766334, 0.001902888, 0.00547094,
            0.010269602, 0.009679115, 0.007341483, 0.009308278, 0.000612149
        ], [0.002588989, 0.005991101, 0.004837445, 0.004330916, 0.004274225,
            0.006562853, 0.003391363, 0.003023307, 0.003059925, 0.003252943,
            0.008269947, 0.005500133, 0.003006648, 0.001831731, 0.002245794,
            0.002673019, 0.002663237, 0.003009831, 0.001962524, 0.004798005,
            0.003567612, 0.001576041, 0.003559711, 0.153072299, 0.002659155,
            0.005014136, 0.001008674, 0.000579947, 0.00216247, 0.001298224,
            0.001126343, 0.000887266, 0.00057512, 0.001260979, 4.36263e-05
        ], [0.000780924, 0.001373483, 0.001380056, 0.001581687, 0.002320516,
            0.001763463, 0.002218018, 0.000526607, 0.001618008, 0.00162599,
            0.001602792, 0.001131232, 0.002008156, 0.00160973, 0.001325819,
            0.001324502, 0.000714399, 0.001092643, 0.002055718, 0.002874218,
            0.001636877, 0.001294861, 0.002561291, 0.005596515, 0.031173291,
            0.013879002, 0.003778595, 0.00278904, 0.00053405, 0.003737713,
            0.004078987, 0.002861156, 0.001219174, 0.004182051, 2.10266e-05
        ], [0.003458555, 0.00444868, 0.006749557, 0.003837543, 0.004564367,
            0.007121573, 0.009488392, 0.006860927, 0.004858666, 0.005208397,
            0.008300317, 0.004971479, 0.004210415, 0.002798686, 0.003947773,
            0.004307502, 0.003867538, 0.009465547, 0.013154011, 0.021603311,
            0.012248606, 0.007612696, 0.03954573, 0.111721654, 0.091732256,
            0.100805269, 0.004326838, 0.003260327, 0.001069015, 0.005164077,
            0.004701212, 0.002794818, 0.003272946, 0.006319297, 3.45637e-05
        ], [0.003728366, 0.004929574, 0.005015694, 0.00602011, 0.006176909,
            0.005998608, 0.012721932, 0.003154015, 0.007502961, 0.005576619,
            0.0064724, 0.006672132, 0.006370327, 0.006923667, 0.004082264,
            0.010443441, 0.006813774, 0.011679325, 0.017054147, 0.020220127,
            0.021689835, 0.01514749, 0.01756699, 0.018668709, 0.02152144,
            0.023011905, 0.120945798, 0.028046004, 0.005692523, 0.024968857,
            0.02299641, 0.011384171, 0.013939344, 0.022076509, 0.000734218
        ], [0.017451472, 0.02379366, 0.015045045, 0.02128574, 0.015643892,
            0.015845878, 0.021279998, 0.011738292, 0.017025533, 0.014789825,
            0.021975425, 0.017227932, 0.016141732, 0.012100409, 0.012596042,
            0.026852016, 0.022433158, 0.021310001, 0.019687106, 0.031504701,
            0.035092052, 0.024074896, 0.045489405, 0.025020996, 0.031298895,
            0.032015168, 0.019925231, 0.218201687, 0.082882166, 0.036192309,
            0.023210718, 0.013902709, 0.023330954, 0.044453646, 0.006569115
        ], [0.002833145, 0.005253147, 0.004554051, 0.004893906, 0.006103899,
            0.005738999, 0.010024356, 0.001752782, 0.004225659, 0.005799811,
            0.004920918, 0.004497364, 0.00514329, 0.003655787, 0.004605026,
            0.006722693, 0.004430922, 0.010388028, 0.022309652, 0.021951953,
            0.042319148, 0.030067754, 0.009103979, 0.013746292, 0.009890012,
            0.021067641, 0.015283139, 0.023849948, 0.030996662, 0.024586798,
            0.0138359, 0.019892438, 0.036113663, 0.027195219, 1.1579e-06
        ], [0.012193135, 0.029070664, 0.037637061, 0.02208352, 0.019745269,
            0.025426865, 0.060919331, 0.01296895, 0.060078486, 0.035061071,
            0.031854943, 0.026859028, 0.038671882, 0.038241612, 0.037231141,
            0.027231223, 0.030746983, 0.053521472, 0.050710686, 0.058523345,
            0.061863148, 0.045161392, 0.042771448, 0.025229405, 0.077408878,
            0.061887368, 0.063595167, 0.100316901, 0.039395931, 0.147537659,
            0.085933956, 0.036249566, 0.063846183, 0.079891563, 0.002852615
        ], [0.001389659, 0.002468824, 0.001768633, 0.000834706, 0.000717963,
            0.003100964, 0.003302415, 0.001025719, 0.002489143, 0.001847682,
            0.00255243, 0.002657181, 0.001149827, 0.000840598, 0.001087887,
            0.00202233, 0.004438474, 0.001529819, 0.00267413, 0.001626862,
            0.002751407, 0.003648307, 0.003641027, 0.005182592, 0.009223939,
            0.003187284, 0.002666186, 0.001509297, 0.003946056, 0.002955589,
            0.007340312, 0.004545075, 0.004250105, 0.008598609, 1.16376e-05
        ], [0.000523124, 0.000405677, 0.000423461, 0.000327745, 0.000397983,
            0.000345694, 0.000556251, 0.0003493, 0.000849717, 0.000564303,
            0.000526547, 0.000507992, 0.000743153, 0.000600637, 0.000728553,
            0.000404849, 0.000658764, 0.000483712, 0.000803955, 0.00075821,
            0.001290035, 0.000601973, 0.001019238, 0.000312864, 0.001986475,
            0.001149024, 0.001457711, 0.001200922, 0.000295953, 0.002259999,
            0.004259452, 0.014106793, 0.001440362, 0.001602184, 8.02746e-06
        ], [0.00126252, 0.000444055, 0.00046472, 0.000385805, 0.000966525,
            0.000508451, 0.000483276, 0.000160569, 0.000520592, 0.000390897,
            0.000703365, 0.00047864, 0.000836386, 0.000395394, 0.00044869,
            0.000480971, 0.000391574, 0.000391191, 0.000525232, 0.000576669,
            0.00041377, 0.000969334, 0.000600947, 0.000319742, 0.000414631,
            0.000495014, 0.000619392, 0.000556063, 9.32945e-05, 0.000594428,
            0.005323035, 0.001642846, 0.03422822, 0.001320904, 6.75229e-07
        ], [0.002932232, 0.006217507, 0.005229628, 0.004339667, 0.006204246,
            0.00499114, 0.012447215, 0.001889622, 0.006878352, 0.004859957,
            0.005637293, 0.005206326, 0.004467287, 0.003049825, 0.004051294,
            0.004435933, 0.005535019, 0.006099617, 0.007266226, 0.007522137,
            0.009019608, 0.013860493, 0.012471796, 0.004718108, 0.007740699,
            0.012895544, 0.017437973, 0.009891738, 0.008364198, 0.024566686,
            0.017403866, 0.012489531, 0.014026314, 0.078922515, 0.00186571
        ], [1.39759e-05, 2.52242e-05, 2.32951e-05, 8.84825e-05, 2.75316e-05,
            3.06553e-05, 3.17017e-05, 1.02875e-05, 1.83767e-05, 2.11395e-05,
            2.14217e-05, 1.54637e-05, 3.06786e-05, 1.3416e-05, 4.76845e-05,
            1.32694e-05, 6.86315e-06, 6.39855e-05, 3.79468e-05, 1.21205e-05,
            3.77271e-05, 5.20396e-05, 0.000112481, 5.17696e-06, 1.29305e-05,
            1.21912e-05, 3.39461e-05, 5.8201e-05, 0.000222058, 6.16742e-05,
            6.59491e-05, 4.0116e-05, 5.4152e-05, 0.000115509, 0.000121974
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 6
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2006():
    """
    IA matrix 2006



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2006())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2007():
    """
    IA matrix 2007



    constant


    """

    ret =  xr.DataArray(
        data=[[0.119057336, 0.000558434, 0.255589334, 0.049230418, 0.054939699,
            0.120320652, 0.015975383, 0.001889974, 0.00655901, 0.012307622,
            0.000553514, 0.000193687, 0.000172585, 8.7438e-05, 0.000114068,
            0.010462317, 0.000499416, 0.003305317, 0.000896418, 0.002297029,
            0.002283445, 0.038487253, 0.001715266, 0.000864287, 0.000361627,
            0.006509085, 0.000206132, 0.000147712, 0.000377055, 0.000830846,
            0.002106896, 0.0026346, 0.002109457, 0.002476042, 0.028660158
        ], [0.002195204, 0.084218277, 0.00278563, 0.002946155, 0.002386478,
            0.004165866, 0.004708937, 0.443794268, 0.029621479, 0.003678071,
            0.068806616, 0.050207243, 0.002533004, 0.00101534, 0.001484548,
            0.005088815, 0.106609079, 0.018722503, 0.000552766, 0.000502553,
            0.001020612, 0.002052725, 0.004923196, 0.003156485, 0.003603614,
            0.002910779, 0.000440509, 0.000258582, 0.00144268, 0.000758565,
            0.003168821, 0.001544114, 0.001587811, 0.002682714, 0.063468852
        ], [0.067025997, 0.000934072, 0.164023771, 0.005467563, 0.104246785,
            0.002843698, 0.003471727, 0.001790394, 0.012353721, 0.004057437,
            0.002823068, 0.00199329, 0.001786182, 0.001438201, 0.001186805,
            0.002921176, 0.00117666, 0.001607159, 0.001674205, 0.004702569,
            0.00474087, 0.154833857, 0.001849239, 0.005842608, 0.004438143,
            0.001814136, 0.00140711, 0.000715459, 0.00030874, 0.00191057,
            0.007402537, 0.00901568, 0.012652709, 0.007316499, 0.002849029
        ], [0.001769008, 0.001095153, 0.001649055, 0.361716286, 0.060245806,
            0.003160779, 0.006388496, 0.000518073, 0.003415939, 0.014586259,
            0.004041883, 0.001944882, 0.002300866, 0.001790356, 0.004230988,
            0.023680158, 0.000953238, 0.002434633, 0.001615749, 0.002549472,
            0.003015608, 0.002843163, 0.002039278, 0.003103408, 0.002122255,
            0.001954075, 0.000667888, 0.00045021, 0.000401018, 0.001501775,
            0.002891901, 0.001108129, 0.003665785, 0.004500576, 7.05294e-05
        ], [0.000154658, 5.82186e-05, 0.000130786, 0.012323581, 0.220471081,
            0.000479752, 0.000664946, 4.10119e-05, 0.000241362, 0.000927583,
            0.000248218, 0.000144156, 0.000184082, 0.00028486, 0.001892014,
            0.005367518, 8.63831e-05, 0.000105979, 0.000371955, 0.00032227,
            0.000269953, 0.000123719, 0.000244854, 9.54716e-05, 0.000137984,
            0.00016401, 0.000232944, 3.35435e-05, 9.72326e-06, 9.41628e-05,
            0.000732372, 0.000107599, 0.000239632, 0.000439559, 1.28632e-06
        ], [0.001672384, 0.001668492, 0.001752618, 0.001542478, 0.002018751,
            0.262682893, 0.01234069, 0.000366759, 0.001533569, 0.003137585,
            0.00827143, 0.003789223, 0.003079825, 0.001613553, 0.00267203,
            0.075342943, 0.000812325, 0.030224373, 0.001309344, 0.001300046,
            0.001234571, 0.002302044, 0.001258695, 0.000614343, 0.000400009,
            0.002293822, 0.000553077, 0.000302388, 0.002252078, 0.000890219,
            0.002071847, 0.002915716, 0.001452856, 0.002677121, 2.286e-05
        ], [0.002405062, 0.002944551, 0.024648043, 0.0120444, 0.015124824,
            0.014441215, 0.250215873, 0.001809326, 0.014852935, 0.015997056,
            0.021665426, 0.005068361, 0.006660093, 0.008105658, 0.004091493,
            0.01939646, 0.003119925, 0.004203785, 0.008315386, 0.011210954,
            0.010929731, 0.008098959, 0.006546251, 0.003326959, 0.004657873,
            0.013315976, 0.009415052, 0.012694426, 0.002047419, 0.019983,
            0.018117461, 0.018761266, 0.00930171, 0.023538108, 0.000149384
        ], [0.011996286, 0.006272476, 0.00309441, 0.005015804, 0.004073623,
            0.004437576, 0.00427055, 0.098090694, 0.04275612, 0.00852248,
            0.018518099, 0.012249103, 0.003473786, 0.001775192, 0.001734584,
            0.00375568, 0.026610578, 0.010249672, 0.005049386, 0.004525894,
            0.003489515, 0.003368801, 0.053960519, 0.058361064, 0.091847558,
            0.020969707, 0.003220886, 0.001165034, 0.001061316, 0.004426697,
            0.008204659, 0.002933321, 0.002678651, 0.00482234, 2.80191e-05
        ], [0.037577704, 0.010466514, 0.009461619, 0.067317109, 0.038327549,
            0.035531611, 0.036057025, 0.018543612, 0.251223004, 0.221470999,
            0.038545379, 0.015513545, 0.010267434, 0.017697861, 0.01060629,
            0.026704111, 0.006829969, 0.010900191, 0.005760062, 0.00278443,
            0.002190316, 0.00467709, 0.00339193, 0.004543469, 0.003830403,
            0.004505024, 0.00122271, 0.000692513, 0.002085608, 0.004851038,
            0.009669065, 0.003977429, 0.064078931, 0.010477862, 0.000160148
        ], [0.004670572, 0.005862232, 0.01921605, 0.011109047, 0.039382704,
            0.008354608, 0.016948607, 0.003230502, 0.017616236, 0.152158698,
            0.012113035, 0.006912985, 0.022978318, 0.022951402, 0.03331125,
            0.033419047, 0.002441716, 0.01861943, 0.015614811, 0.004341448,
            0.004913815, 0.004203323, 0.011373599, 0.002772077, 0.003869552,
            0.006490456, 0.003077427, 0.000624394, 0.0013048, 0.002860597,
            0.00331687, 0.001464327, 0.005192018, 0.004586962, 5.35151e-05
        ], [0.001514718, 0.004503401, 0.005344907, 0.002123106, 0.001656298,
            0.007125934, 0.001255802, 0.002067483, 0.005633043, 0.005427945,
            0.143950521, 0.010597201, 0.004966021, 0.009790723, 0.006355513,
            0.007304057, 0.001794795, 0.094188202, 0.003601868, 0.000985001,
            0.001018191, 0.003291457, 0.001473936, 0.000435769, 0.000665638,
            0.00147678, 0.001087399, 0.000118165, 0.00171914, 0.000982791,
            0.001625162, 0.001704231, 0.001939275, 0.001887088, 2.58891e-05
        ], [0.004373924, 0.019275632, 0.010338281, 0.005099874, 0.009717035,
            0.021824164, 0.009084842, 0.005925924, 0.012698204, 0.023184613,
            0.03452721, 0.334097532, 0.157428779, 0.064043817, 0.085231984,
            0.076334156, 0.006976738, 0.089242326, 0.014778995, 0.003201341,
            0.002864174, 0.004079796, 0.006366525, 0.004931614, 0.004260231,
            0.008317289, 0.003195396, 0.000540176, 0.00244478, 0.003626196,
            0.00429447, 0.001853691, 0.002192743, 0.00460348, 0.000121105
        ], [0.007607879, 0.01662017, 0.004271887, 0.010944036, 0.008448391,
            0.011147691, 0.008087824, 0.008382081, 0.009117883, 0.014504033,
            0.024184046, 0.026327346, 0.173367825, 0.018526878, 0.040877859,
            0.0153419, 0.009697125, 0.020143518, 0.024554656, 0.00284828,
            0.002595083, 0.002479866, 0.006121525, 0.009686411, 0.008628142,
            0.009170068, 0.002931513, 0.000997167, 0.00166487, 0.003097736,
            0.007019183, 0.00241279, 0.004414478, 0.0042075, 0.000336107
        ], [0.002050758, 0.008569024, 0.00401303, 0.006747605, 0.005400788,
            0.007845858, 0.010705379, 0.004226264, 0.011148142, 0.01360452,
            0.012311146, 0.018641995, 0.108693092, 0.389464478, 0.066302181,
            0.032720707, 0.029964276, 0.042753965, 0.033294219, 0.009073931,
            0.008392747, 0.005845536, 0.008939044, 0.003915237, 0.006188393,
            0.011450689, 0.045407433, 0.004662465, 0.002750035, 0.021107981,
            0.020313661, 0.012974529, 0.027801261, 0.01624049, 0.000877418
        ], [0.005327125, 0.005557984, 0.002055866, 0.003148111, 0.00288233,
            0.004731999, 0.0041332, 0.001487186, 0.002674144, 0.005265855,
            0.007323167, 0.00802768, 0.018088124, 0.004700597, 0.3228334,
            0.008060157, 0.004023945, 0.006180063, 0.080473383, 0.005244601,
            0.004114125, 0.00209946, 0.041071848, 0.040011798, 0.058944702,
            0.01863065, 0.003550846, 0.001123172, 0.001208024, 0.004474224,
            0.019662989, 0.002766115, 0.001617538, 0.008493442, 0.000182814
        ], [0.000639972, 0.000911154, 0.000980984, 0.004394358, 0.002843523,
            0.003880432, 0.004421014, 0.000670311, 0.00168933, 0.002943323,
            0.003526369, 0.009897221, 0.003733949, 0.002202029, 0.004734954,
            0.050432598, 0.001496604, 0.004819844, 0.001696544, 0.001119889,
            0.001409504, 0.002332754, 0.002625857, 0.000711492, 0.001924337,
            0.001521093, 0.001070804, 0.000829159, 0.001370697, 0.001685984,
            0.003617968, 0.003784142, 0.003058047, 0.005249855, 0.00146207
        ], [0.010732026, 0.030828612, 0.013537093, 0.017374586, 0.008010364,
            0.017207633, 0.023348993, 0.031387055, 0.026179163, 0.020464094,
            0.042609532, 0.029168917, 0.011064229, 0.007139903, 0.007647859,
            0.009358165, 0.142471309, 0.006982991, 0.01104774, 0.006151616,
            0.012918034, 0.023096068, 0.015460861, 0.003320923, 0.005826293,
            0.015751179, 0.009439714, 0.003955649, 0.007950912, 0.005477731,
            0.014726937, 0.021119519, 0.012040387, 0.015531502, 0.000160901
        ], [0.003009863, 0.00635594, 0.002029623, 0.002112663, 0.001346476,
            0.002886683, 0.002974156, 0.004900142, 0.003037809, 0.002343282,
            0.005835424, 0.003652734, 0.002476361, 0.00151322, 0.001592041,
            0.002405229, 0.01564844, 0.049456754, 0.005378161, 0.003285543,
            0.004330364, 0.005354527, 0.008475206, 0.00162424, 0.00267512,
            0.011965491, 0.010242361, 0.004379269, 0.031699146, 0.004269731,
            0.018068934, 0.010767451, 0.006936197, 0.008730331, 0.000507318
        ], [0.005159413, 0.003141148, 0.006596889, 0.004397274, 0.005740736,
            0.005871752, 0.004749009, 0.002867762, 0.005357705, 0.00573129,
            0.006414211, 0.005373108, 0.004248213, 0.002919055, 0.004401169,
            0.00626269, 0.007523175, 0.006945958, 0.011576917, 0.004157887,
            0.004147927, 0.005178946, 0.015231042, 0.002008476, 0.00501022,
            0.007954323, 0.003612479, 0.002794935, 0.002133071, 0.005275521,
            0.004361181, 0.002810855, 0.003959693, 0.006131935, 3.86579e-05
        ], [0.034239955, 0.018620157, 0.062922228, 0.040726397, 0.049027192,
            0.050668632, 0.046118385, 0.058903088, 0.043738458, 0.044538734,
            0.039069222, 0.041162938, 0.041176101, 0.030576156, 0.040066371,
            0.04164824, 0.02198545, 0.041299512, 0.032490577, 0.024169684,
            0.012825342, 0.047147687, 0.022687028, 0.018492185, 0.022590665,
            0.020394492, 0.011432304, 0.003947556, 0.004573041, 0.013229581,
            0.019326111, 0.012099975, 0.027900376, 0.017341663, 0.000257006
        ], [0.016133125, 0.006713248, 0.027997005, 0.020883189, 0.02775486,
            0.018010695, 0.017106066, 0.014761809, 0.02012713, 0.018568453,
            0.016515168, 0.015307704, 0.014832494, 0.010625781, 0.016704468,
            0.025282368, 0.007204752, 0.0280189, 0.014620505, 0.005786474,
            0.006879379, 0.023560084, 0.013821276, 0.006411804, 0.009511873,
            0.008586153, 0.007894086, 0.004375328, 0.003285971, 0.007895453,
            0.008131604, 0.005796185, 0.012392852, 0.01072452, 0.000337962
        ], [0.001486232, 0.003281197, 0.003922423, 0.003817409, 0.004410235,
            0.004004632, 0.005194425, 0.001095035, 0.003916902, 0.004471621,
            0.005322132, 0.004201729, 0.004872192, 0.003444666, 0.002510409,
            0.00342581, 0.003042643, 0.005038773, 0.005066954, 0.007416819,
            0.005844249, 0.009903041, 0.007545045, 0.004048928, 0.017915333,
            0.021727926, 0.008933081, 0.009992374, 0.002562156, 0.012551516,
            0.01129675, 0.009313177, 0.008459784, 0.011994694, 5.24579e-05
        ], [0.013066882, 0.018634282, 0.023206649, 0.019226599, 0.01871731,
            0.022622925, 0.018546512, 0.044852978, 0.017465782, 0.01619444,
            0.033634177, 0.016824736, 0.011839632, 0.007121826, 0.010299719,
            0.019081864, 0.018933301, 0.019567261, 0.017107808, 0.021460536,
            0.012189862, 0.009392043, 0.046152565, 0.008037147, 0.009266696,
            0.058073509, 0.008706425, 0.00493827, 0.001765979, 0.005469324,
            0.010852094, 0.009617312, 0.007268767, 0.0090875, 0.000504099
        ], [0.002384574, 0.006272222, 0.005111568, 0.004485995, 0.004050504,
            0.006563199, 0.003097904, 0.0032444, 0.003041828, 0.003128733,
            0.007908117, 0.005213675, 0.002837513, 0.001683453, 0.002411409,
            0.002685204, 0.002664106, 0.002877424, 0.002065685, 0.00580558,
            0.003115807, 0.001663845, 0.003823733, 0.150616412, 0.002992557,
            0.00545203, 0.001129738, 0.000657072, 0.00258329, 0.001409687,
            0.001209942, 0.000911299, 0.000586839, 0.001363744, 4.58498e-05
        ], [0.000823373, 0.001524798, 0.00142343, 0.001627393, 0.002361853,
            0.001805604, 0.002180073, 0.000580558, 0.001626331, 0.001519908,
            0.001630251, 0.001109294, 0.001872585, 0.001454943, 0.001248328,
            0.001348897, 0.00073964, 0.001149796, 0.002260213, 0.002972042,
            0.001559941, 0.001330349, 0.002596775, 0.005329325, 0.03027022,
            0.01398012, 0.003765175, 0.00288781, 0.000535426, 0.003717411,
            0.004207271, 0.003284026, 0.001226141, 0.004325911, 1.6433e-05
        ], [0.003613751, 0.004554492, 0.0066244, 0.00401275, 0.004357308,
            0.006966464, 0.009201204, 0.00719875, 0.004802135, 0.00513572,
            0.008239442, 0.004961442, 0.004143515, 0.002693017, 0.003844329,
            0.004332556, 0.003813523, 0.01189111, 0.013643601, 0.020720014,
            0.012260069, 0.007434578, 0.039283638, 0.114736797, 0.093992166,
            0.102283674, 0.004267009, 0.003396986, 0.001074324, 0.005240638,
            0.004876061, 0.002885642, 0.003256593, 0.006306472, 2.65925e-05
        ], [0.003979242, 0.004930375, 0.005013867, 0.006007658, 0.005704094,
            0.005915291, 0.012251242, 0.003677582, 0.007424378, 0.00509162,
            0.006436318, 0.00679453, 0.006009702, 0.006692261, 0.004121236,
            0.011643854, 0.007068184, 0.01142259, 0.017553887, 0.019193788,
            0.021752341, 0.0149751, 0.017955279, 0.017638252, 0.021770093,
            0.022844892, 0.121383686, 0.029291293, 0.005082404, 0.024772805,
            0.023052324, 0.011606363, 0.013901955, 0.022125082, 0.000664909
        ], [0.017901345, 0.024989623, 0.015634674, 0.022320459, 0.016648148,
            0.01662907, 0.021888916, 0.013250217, 0.017935121, 0.014448503,
            0.023252725, 0.017832903, 0.015975543, 0.011959044, 0.013097047,
            0.028249403, 0.023994054, 0.021963972, 0.020487444, 0.031325587,
            0.034723953, 0.024673236, 0.048011086, 0.023218139, 0.033359243,
            0.035664725, 0.019855899, 0.212878905, 0.078084812, 0.03599279,
            0.023490742, 0.015407282, 0.024152033, 0.045409322, 0.007728698
        ], [0.002096568, 0.005047989, 0.004463664, 0.004809827, 0.006104338,
            0.005855326, 0.009736082, 0.001767392, 0.004083622, 0.005422638,
            0.00482709, 0.004467202, 0.004954824, 0.003334403, 0.004591127,
            0.006777097, 0.004357405, 0.010191368, 0.022430846, 0.01999581,
            0.043054267, 0.027861659, 0.008871093, 0.012816672, 0.010139274,
            0.020369796, 0.014891714, 0.023931664, 0.034962119, 0.023369544,
            0.013726236, 0.020402409, 0.035431763, 0.026627461, 1.02562e-06
        ], [0.012860613, 0.030044324, 0.036730904, 0.020713346, 0.018642208,
            0.025495306, 0.06171373, 0.014047344, 0.06123506, 0.033093085,
            0.032210698, 0.027221017, 0.037796063, 0.037185652, 0.038444601,
            0.027469718, 0.031261359, 0.05418069, 0.051842798, 0.058274048,
            0.061240173, 0.044612909, 0.042979536, 0.024765007, 0.06550855,
            0.063362632, 0.064147247, 0.105368509, 0.034386968, 0.149589504,
            0.085321456, 0.037389614, 0.065087895, 0.082174631, 0.002703582
        ], [0.001359427, 0.00252266, 0.001737172, 0.000720481, 0.000616526,
            0.002753027, 0.003404703, 0.001105849, 0.002562926, 0.001827679,
            0.002440789, 0.002526465, 0.001155061, 0.000839458, 0.001036018,
            0.002033649, 0.004352171, 0.001482326, 0.002615237, 0.001527703,
            0.002577151, 0.003317276, 0.003666807, 0.005096696, 0.008713054,
            0.00309914, 0.002725106, 0.001576497, 0.003671472, 0.002916512,
            0.007897431, 0.004641136, 0.004196141, 0.008363876, 9.7559e-06
        ], [0.000520827, 0.000388092, 0.000406248, 0.000306738, 0.000356518,
            0.000337344, 0.000517489, 0.00036825, 0.000855582, 0.00053344,
            0.000488106, 0.000478327, 0.000690522, 0.000543115, 0.000672776,
            0.000384285, 0.0006524, 0.00046283, 0.000785045, 0.000728236,
            0.001023462, 0.000576307, 0.000965735, 0.00027147, 0.001960902,
            0.001186481, 0.001398258, 0.001210821, 0.000310617, 0.002319082,
            0.004034179, 0.01520081, 0.001405825, 0.001588225, 1.18414e-05
        ], [0.001258267, 0.000444989, 0.000446316, 0.000380262, 0.000832332,
            0.000507128, 0.000481664, 0.00017625, 0.000509277, 0.000380639,
            0.000729546, 0.000471071, 0.000808864, 0.00038593, 0.000436742,
            0.000465957, 0.000412578, 0.000406625, 0.000532685, 0.000505027,
            0.000440676, 0.000957922, 0.000577831, 0.0003081, 0.000414969,
            0.000480771, 0.000612237, 0.00057797, 9.34326e-05, 0.000581793,
            0.005588021, 0.001650708, 0.033464545, 0.001336462, 4.97349e-07
        ], [0.002982701, 0.006233054, 0.004965234, 0.004024591, 0.005866752,
            0.004817333, 0.012168947, 0.001935913, 0.006613866, 0.004408571,
            0.005583885, 0.005052392, 0.004143899, 0.002737262, 0.003931804,
            0.004370863, 0.00558076, 0.005853047, 0.007026512, 0.007504968,
            0.008499519, 0.013401788, 0.01249137, 0.004779131, 0.006904193,
            0.013913267, 0.017725138, 0.01002198, 0.007872466, 0.023771891,
            0.017660526, 0.012179327, 0.013631719, 0.077900253, 0.001422569
        ], [1.38338e-05, 2.65096e-05, 2.33817e-05, 8.20845e-05, 2.93447e-05,
            3.27502e-05, 3.13764e-05, 1.22082e-05, 1.95113e-05, 1.89003e-05,
            2.25061e-05, 1.66082e-05, 3.06067e-05, 1.16841e-05, 4.44892e-05,
            1.30882e-05, 6.68618e-06, 5.79481e-05, 3.69515e-05, 1.23283e-05,
            3.80263e-05, 5.14698e-05, 0.00011366, 4.91111e-06, 1.25213e-05,
            1.19771e-05, 3.4846e-05, 6.31349e-05, 0.000213945, 6.39272e-05,
            7.2228e-05, 4.11406e-05, 5.56648e-05, 0.000110319, 9.8223e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 7
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2007():
    """
    IA matrix 2007



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2007())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2008():
    """
    IA matrix 2008



    constant


    """

    ret =  xr.DataArray(
        data=[[0.116933167, 0.000461251, 0.259949765, 0.051115578, 0.05500655,
            0.119948473, 0.016662548, 0.001515679, 0.006843401, 0.012539149,
            0.000552577, 0.000199458, 0.000175244, 8.86027e-05, 0.000122722,
            0.011132714, 0.000506196, 0.003287389, 0.001010253, 0.002327645,
            0.002313368, 0.039138607, 0.001609733, 0.00083866, 0.000397172,
            0.006874846, 0.00021503, 0.000177389, 0.000376736, 0.000837432,
            0.001950958, 0.002649568, 0.002128874, 0.002573434, 0.030064079
        ], [0.002393234, 0.076278758, 0.003141118, 0.003898023, 0.00287269,
            0.004747886, 0.005270747, 0.474645435, 0.028951097, 0.003886816,
            0.070068508, 0.052358049, 0.002923065, 0.001107468, 0.001501986,
            0.00559058, 0.114341392, 0.020438471, 0.000539608, 0.000523278,
            0.000864394, 0.002324688, 0.005798186, 0.003825665, 0.004143811,
            0.003550805, 0.000468452, 0.00027079, 0.001306737, 0.000847985,
            0.003116566, 0.001764287, 0.001876974, 0.002836588, 0.065758121
        ], [0.066582893, 0.00074526, 0.165427465, 0.005604587, 0.105215897,
            0.003089718, 0.00353709, 0.00142654, 0.012415787, 0.004117957,
            0.002924866, 0.00206675, 0.001879969, 0.001526012, 0.001279509,
            0.003116142, 0.001133208, 0.001655639, 0.001677488, 0.004651168,
            0.004694989, 0.151651187, 0.00184416, 0.005720748, 0.004565617,
            0.001833292, 0.001344328, 0.0007615, 0.000292801, 0.001916561,
            0.007214735, 0.00854507, 0.011985727, 0.007320964, 0.003259645
        ], [0.001707384, 0.000769116, 0.001640265, 0.382004258, 0.062694873,
            0.003382242, 0.006705502, 0.000527862, 0.003710538, 0.015746745,
            0.00443434, 0.002251872, 0.002492484, 0.001923737, 0.004204267,
            0.02521907, 0.001035346, 0.002403193, 0.001689941, 0.002578758,
            0.002809304, 0.002853182, 0.002048819, 0.003085339, 0.002456519,
            0.001969587, 0.000631169, 0.000470528, 0.000406803, 0.001488798,
            0.002777961, 0.00112787, 0.003561734, 0.004218842, 6.1595e-05
        ], [0.000157735, 4.92665e-05, 0.000124929, 0.013426817, 0.231416114,
            0.000525368, 0.000645853, 3.64237e-05, 0.00024683, 0.001057242,
            0.000246083, 0.000142569, 0.000181057, 0.000301046, 0.002098825,
            0.00582707, 7.62748e-05, 0.000106714, 0.000380078, 0.00029044,
            0.000232897, 0.000120488, 0.000224672, 9.09419e-05, 0.000136775,
            0.000185752, 0.000191092, 3.5758e-05, 9.94814e-06, 9.28969e-05,
            0.000736368, 0.000115953, 0.000277723, 0.000422013, 1.62332e-06
        ], [0.001613114, 0.001294385, 0.001675289, 0.001530329, 0.001954059,
            0.270061985, 0.012583822, 0.000245325, 0.00149331, 0.003079763,
            0.008362719, 0.003849265, 0.003054613, 0.001612431, 0.00249878,
            0.077259936, 0.000719147, 0.029051703, 0.001345699, 0.001275474,
            0.001221726, 0.002226872, 0.001205438, 0.000595997, 0.000412795,
            0.002130288, 0.000525836, 0.000287069, 0.001491699, 0.000932256,
            0.002032824, 0.00296074, 0.001446804, 0.002604774, 2.13907e-05
        ], [0.002347897, 0.002402983, 0.02446087, 0.012050243, 0.01493386,
            0.014655172, 0.252681943, 0.001558614, 0.014603231, 0.016236792,
            0.02169169, 0.004962176, 0.006355081, 0.007990147, 0.003915717,
            0.020339282, 0.002825317, 0.004078449, 0.008292997, 0.01122629,
            0.010575063, 0.007904987, 0.006562643, 0.003260238, 0.004757781,
            0.012848332, 0.009047158, 0.012751039, 0.001783151, 0.01965659,
            0.017656725, 0.018951624, 0.008971498, 0.023180985, 0.000105017
        ], [0.012903025, 0.005168669, 0.00338048, 0.005079605, 0.004403037,
            0.004703286, 0.004705797, 0.17475577, 0.05385856, 0.009521228,
            0.021596017, 0.014852987, 0.004046637, 0.002060373, 0.002009099,
            0.004162941, 0.035675995, 0.01041413, 0.005569073, 0.004690331,
            0.003707433, 0.003558388, 0.055232112, 0.063627507, 0.111863795,
            0.023204191, 0.00346552, 0.001219765, 0.001109523, 0.00466339,
            0.008406853, 0.003131539, 0.002723827, 0.005253082, 1.9964e-05
        ], [0.038093078, 0.008802879, 0.009420899, 0.061406788, 0.036750124,
            0.036041073, 0.036078106, 0.01634648, 0.252064824, 0.219871974,
            0.038229431, 0.015168373, 0.009850142, 0.017548455, 0.010529052,
            0.026733045, 0.006645235, 0.010768078, 0.005699406, 0.002710839,
            0.002057522, 0.00463111, 0.003328833, 0.004281792, 0.003931692,
            0.004514476, 0.001167984, 0.000696455, 0.001718561, 0.004738503,
            0.009046684, 0.003925785, 0.061532646, 0.010279086, 0.000145392
        ], [0.004583817, 0.00473405, 0.019002238, 0.01105915, 0.039783462,
            0.007946052, 0.016913454, 0.003124288, 0.017716409, 0.155690207,
            0.012146958, 0.006778711, 0.021940416, 0.022981666, 0.033163141,
            0.034265609, 0.002292744, 0.018162987, 0.014806301, 0.004399474,
            0.004790636, 0.004219364, 0.011196807, 0.002715704, 0.003860152,
            0.006586714, 0.003092539, 0.000627683, 0.001170567, 0.002804907,
            0.003076307, 0.00146913, 0.005079801, 0.004535347, 6.23224e-05
        ], [0.001500662, 0.003515705, 0.005270513, 0.002084452, 0.001618103,
            0.006650338, 0.001248417, 0.001648036, 0.005607308, 0.005411849,
            0.14564545, 0.010795907, 0.004971774, 0.009919456, 0.006440327,
            0.007524489, 0.001634151, 0.093740208, 0.003514457, 0.00095072,
            0.000988566, 0.003282571, 0.001454533, 0.000414784, 0.000694666,
            0.001490907, 0.001062147, 0.000120308, 0.001609551, 0.000983769,
            0.001493022, 0.001715711, 0.001921749, 0.00187824, 2.8084e-05
        ], [0.004336521, 0.016485225, 0.010194269, 0.004908737, 0.009173866,
            0.021084942, 0.009331124, 0.005249702, 0.01266054, 0.023074626,
            0.03442273, 0.327097896, 0.152374823, 0.063684382, 0.085307337,
            0.075941268, 0.006483766, 0.090265448, 0.014426577, 0.003192677,
            0.002865317, 0.004313625, 0.006402572, 0.004120545, 0.0043721,
            0.008474436, 0.003293518, 0.000575075, 0.002199501, 0.003771877,
            0.00417679, 0.001806352, 0.002215985, 0.004707392, 0.000142877
        ], [0.007963676, 0.013899418, 0.004359889, 0.010970394, 0.008495771,
            0.011636156, 0.008233052, 0.007884052, 0.009122631, 0.015203,
            0.024850548, 0.026917793, 0.175987644, 0.018890287, 0.043751646,
            0.016126966, 0.009330079, 0.020846595, 0.025100152, 0.002906527,
            0.002611826, 0.002610432, 0.006293045, 0.009629099, 0.009111387,
            0.009662975, 0.002950526, 0.001112787, 0.001617582, 0.003167967,
            0.007240132, 0.002477907, 0.004785046, 0.004297406, 0.000358567
        ], [0.00214387, 0.00745467, 0.004226065, 0.00702667, 0.005492984,
            0.007884248, 0.010960725, 0.003647827, 0.011320183, 0.013874865,
            0.012882551, 0.018502715, 0.109169596, 0.400829518, 0.069859949,
            0.033636588, 0.030322264, 0.044731989, 0.033640675, 0.009551852,
            0.008082482, 0.00621385, 0.009651954, 0.003932446, 0.006588793,
            0.012161828, 0.044581973, 0.004920693, 0.002598326, 0.022078824,
            0.021791146, 0.014094387, 0.029122745, 0.017694174, 0.000953278
        ], [0.005369257, 0.004520617, 0.002031266, 0.003224289, 0.0028152,
            0.004921502, 0.004370982, 0.001225644, 0.002675204, 0.005465012,
            0.00750656, 0.008288927, 0.017907836, 0.004602134, 0.326714961,
            0.008498111, 0.003975368, 0.006459087, 0.075061724, 0.005546829,
            0.004129093, 0.002166, 0.040858253, 0.039167355, 0.062872625,
            0.02154021, 0.003632457, 0.001183082, 0.001387942, 0.004442382,
            0.019173956, 0.002800944, 0.001578171, 0.008683282, 0.000175499
        ], [0.000717495, 0.00072647, 0.000971834, 0.004867637, 0.002934769,
            0.003735308, 0.004200811, 0.000613319, 0.001716299, 0.00288804,
            0.003699271, 0.009633449, 0.003678183, 0.002183246, 0.005004083,
            0.054837399, 0.00165845, 0.004549565, 0.001556164, 0.001113457,
            0.001396111, 0.002385377, 0.003388021, 0.00067984, 0.002017877,
            0.001473507, 0.001333957, 0.000825184, 0.001227515, 0.001690549,
            0.003409503, 0.003687714, 0.003120203, 0.005170379, 0.001697087
        ], [0.011167819, 0.027944126, 0.014619931, 0.0182518, 0.0080034,
            0.018389821, 0.02484546, 0.046623395, 0.027864626, 0.021840589,
            0.044984948, 0.030540785, 0.011760523, 0.007689493, 0.008318939,
            0.010538893, 0.147411047, 0.007664393, 0.011281822, 0.006659617,
            0.01363611, 0.025021193, 0.016410701, 0.00335993, 0.00699522,
            0.017284492, 0.010078704, 0.004412416, 0.007695805, 0.005845057,
            0.014518957, 0.022283432, 0.012545613, 0.016356693, 0.000120179
        ], [0.003034392, 0.005816464, 0.002116221, 0.002116411, 0.001267225,
            0.002766644, 0.002937288, 0.005898464, 0.002888708, 0.002191548,
            0.005671311, 0.003521057, 0.002379065, 0.001454471, 0.001576298,
            0.00246934, 0.013959986, 0.051020112, 0.005634533, 0.003319059,
            0.004455793, 0.005724391, 0.008730393, 0.001498546, 0.002867782,
            0.012127941, 0.010116918, 0.004919834, 0.030962164, 0.004334668,
            0.017649077, 0.010836817, 0.007289666, 0.00898154, 0.00041569
        ], [0.005121679, 0.002610618, 0.006453783, 0.003848487, 0.005114499,
            0.005566305, 0.004645679, 0.002488477, 0.005222426, 0.005461503,
            0.006009024, 0.005158073, 0.004028923, 0.002784147, 0.004427816,
            0.006183625, 0.006556208, 0.006642176, 0.011503436, 0.004013726,
            0.004054795, 0.005023117, 0.014319223, 0.001808211, 0.005028604,
            0.008155351, 0.00335508, 0.002879377, 0.002003363, 0.005219068,
            0.004010447, 0.002579323, 0.00377922, 0.005822863, 1.81871e-05
        ], [0.032699239, 0.01611326, 0.062247575, 0.037259035, 0.047678878,
            0.049309807, 0.046136045, 0.061544515, 0.044498995, 0.043886289,
            0.038279378, 0.041960879, 0.040257628, 0.030926399, 0.041369847,
            0.04133872, 0.019804685, 0.041298437, 0.031505187, 0.023369029,
            0.012297876, 0.047573261, 0.021711006, 0.017511424, 0.021996426,
            0.020539267, 0.011614428, 0.004164881, 0.004236661, 0.013233617,
            0.020134299, 0.012254652, 0.027486097, 0.017319852, 0.000120548
        ], [0.016171249, 0.005937669, 0.028074365, 0.019688226, 0.026027414,
            0.017860231, 0.017099409, 0.013022477, 0.020436484, 0.018693695,
            0.016434736, 0.015614709, 0.014920342, 0.010721517, 0.017203276,
            0.024638087, 0.006945777, 0.025602704, 0.014759319, 0.005843301,
            0.006877265, 0.023683466, 0.013772966, 0.005891801, 0.009360104,
            0.009037396, 0.007782133, 0.004735335, 0.002863558, 0.007844648,
            0.007837229, 0.005731264, 0.012403489, 0.010668591, 0.000163198
        ], [0.001487984, 0.002657068, 0.00391283, 0.003789712, 0.00444957,
            0.00376272, 0.004944887, 0.000861543, 0.003812219, 0.004258741,
            0.005196167, 0.004013493, 0.00471161, 0.003456311, 0.002538247,
            0.003465655, 0.002651726, 0.004953341, 0.005073468, 0.007481113,
            0.005670227, 0.00986331, 0.007283999, 0.003820003, 0.01668363,
            0.02227749, 0.0080321, 0.010669167, 0.002013743, 0.012380898,
            0.010982929, 0.009412161, 0.008103904, 0.011879683, 3.41955e-05
        ], [0.012814957, 0.015566525, 0.023797197, 0.018463554, 0.018586307,
            0.022525977, 0.018584236, 0.051264356, 0.018250525, 0.016505482,
            0.033688421, 0.017168289, 0.01201672, 0.007335872, 0.010663124,
            0.01840666, 0.018107115, 0.019865211, 0.016791474, 0.02112726,
            0.012145439, 0.009545962, 0.045956703, 0.007911504, 0.009450786,
            0.057433011, 0.008635227, 0.005226879, 0.001583825, 0.005668427,
            0.010423137, 0.009625314, 0.007181877, 0.009230232, 0.000297154
        ], [0.002283334, 0.005028394, 0.005125892, 0.004700224, 0.004221084,
            0.007282444, 0.003231864, 0.002265431, 0.003126092, 0.003382655,
            0.008382742, 0.005332349, 0.00312133, 0.001835573, 0.002561106,
            0.00272722, 0.002440771, 0.002978595, 0.001653207, 0.006053763,
            0.003106632, 0.001682922, 0.003453609, 0.148819931, 0.003073373,
            0.00573803, 0.001052656, 0.000637242, 0.002762188, 0.001365665,
            0.001141496, 0.000848277, 0.000512857, 0.001240053, 1.43427e-05
        ], [0.000805283, 0.001290366, 0.001420911, 0.001540716, 0.002211534,
            0.00181482, 0.002109697, 0.00045717, 0.001591038, 0.001535323,
            0.001564642, 0.001069999, 0.001870844, 0.001467328, 0.001219152,
            0.001295759, 0.000692783, 0.001170454, 0.002355372, 0.00305556,
            0.001624274, 0.001351079, 0.002656529, 0.004916401, 0.030425988,
            0.013534259, 0.003983541, 0.003156147, 0.000447514, 0.003739759,
            0.004039149, 0.003387591, 0.001230895, 0.004426367, 7.86761e-06
        ], [0.003705683, 0.004028341, 0.006702041, 0.003903129, 0.004073977,
            0.00667041, 0.00895392, 0.005875314, 0.004842968, 0.005148014,
            0.008053937, 0.004956764, 0.00407154, 0.002670861, 0.003860365,
            0.004375227, 0.003482785, 0.01256258, 0.014415972, 0.020882714,
            0.012306805, 0.007571998, 0.037997432, 0.114025202, 0.100829449,
            0.100773622, 0.00414388, 0.003390627, 0.001120032, 0.005150012,
            0.004689621, 0.002905835, 0.003293637, 0.006368514, 1.22365e-05
        ], [0.004257025, 0.004495071, 0.005440132, 0.006319407, 0.006222708,
            0.005972687, 0.012162865, 0.003464361, 0.007636393, 0.00520897,
            0.006552771, 0.007107266, 0.005995486, 0.007119543, 0.004156565,
            0.011994123, 0.006840191, 0.011983306, 0.018273452, 0.019600403,
            0.022015634, 0.015632367, 0.019077348, 0.018763236, 0.022001334,
            0.022765869, 0.120476774, 0.030227853, 0.004644135, 0.025191472,
            0.023451091, 0.01224052, 0.014397699, 0.022506369, 0.000591323
        ], [0.017084669, 0.019870163, 0.016020252, 0.022148976, 0.016690278,
            0.016155701, 0.021409887, 0.012556969, 0.017541334, 0.014796726,
            0.022992193, 0.01676336, 0.015434116, 0.012134299, 0.013062841,
            0.027413706, 0.022155339, 0.021152864, 0.020655551, 0.030893885,
            0.033435769, 0.024788005, 0.046262557, 0.021309999, 0.032177106,
            0.033759519, 0.019107727, 0.202998718, 0.081138403, 0.03455482,
            0.022154498, 0.015773684, 0.023008985, 0.041261593, 0.008579669
        ], [0.002547566, 0.004803331, 0.004478148, 0.004569459, 0.005770104,
            0.005603393, 0.009143292, 0.001782537, 0.003944811, 0.005230502,
            0.004530715, 0.004372559, 0.004763986, 0.003181958, 0.004458053,
            0.006798967, 0.004019419, 0.010282345, 0.021893653, 0.019466688,
            0.042313321, 0.027359355, 0.008501869, 0.011638082, 0.009278509,
            0.020086987, 0.013495662, 0.023066431, 0.034037702, 0.021679467,
            0.014494773, 0.020400803, 0.033965995, 0.026232158, 9.48375e-07
        ], [0.013130136, 0.026878061, 0.03728047, 0.020058328, 0.017625827,
            0.023656721, 0.061809247, 0.01263344, 0.060999989, 0.033539289,
            0.030825876, 0.026698988, 0.036531853, 0.037355543, 0.037293526,
            0.028122209, 0.029069641, 0.054509579, 0.053377056, 0.061419461,
            0.062080334, 0.047855786, 0.042803259, 0.02331633, 0.0660444,
            0.062744709, 0.062036701, 0.110465483, 0.029193631, 0.154122067,
            0.088127442, 0.0381976, 0.068625813, 0.085218175, 0.001879258
        ], [0.001363579, 0.002255038, 0.001880627, 0.000687351, 0.000589046,
            0.002657276, 0.003796439, 0.00109176, 0.002681637, 0.001960127,
            0.002572166, 0.003260108, 0.001184991, 0.000838597, 0.00102503,
            0.002278417, 0.003963637, 0.001477366, 0.002855039, 0.00160049,
            0.002712516, 0.003679163, 0.003511789, 0.004661511, 0.008558092,
            0.003174156, 0.002673534, 0.001686242, 0.003121738, 0.002895619,
            0.007456962, 0.004526936, 0.004316554, 0.008148121, 3.79376e-06
        ], [0.00054135, 0.000335151, 0.000401494, 0.000300798, 0.000344699,
            0.000328907, 0.000508096, 0.000337824, 0.000844288, 0.000536318,
            0.00048254, 0.000472144, 0.000684372, 0.000509286, 0.000676231,
            0.000380958, 0.000588282, 0.000457967, 0.000765797, 0.000733248,
            0.001095293, 0.000561342, 0.000928639, 0.000263037, 0.002056454,
            0.001139964, 0.001257687, 0.001213918, 0.000286844, 0.002172085,
            0.004056209, 0.015063277, 0.001371115, 0.001658622, 6.70971e-06
        ], [0.001261777, 0.000379093, 0.000450631, 0.00038554, 0.000915347,
            0.000546368, 0.000504663, 0.000140682, 0.000506592, 0.000364872,
            0.000800419, 0.000495891, 0.000892746, 0.00040652, 0.000486336,
            0.000458898, 0.000389192, 0.000385055, 0.000522866, 0.000466226,
            0.000427195, 0.000855483, 0.000551576, 0.000297208, 0.00042782,
            0.000455112, 0.000538112, 0.000559514, 8.87142e-05, 0.00056777,
            0.005709985, 0.001644097, 0.032488878, 0.001343209, 2.60735e-07
        ], [0.002983998, 0.005332036, 0.005017302, 0.004018995, 0.005771622,
            0.004572854, 0.012087739, 0.001517787, 0.006552142, 0.004357351,
            0.005458368, 0.005097927, 0.003946228, 0.002666698, 0.003861679,
            0.004390676, 0.005149944, 0.005606704, 0.006992206, 0.007700545,
            0.008564416, 0.014206769, 0.012744358, 0.004510437, 0.007086292,
            0.014481749, 0.016733324, 0.010568599, 0.006694122, 0.023885932,
            0.01729728, 0.012066863, 0.013592285, 0.076563012, 0.001047606
        ], [1.31904e-05, 2.25995e-05, 2.36995e-05, 8.45047e-05, 2.70827e-05,
            2.88484e-05, 2.97422e-05, 1.11246e-05, 1.73989e-05, 1.79229e-05,
            1.98974e-05, 1.43748e-05, 2.90354e-05, 1.13338e-05, 4.37457e-05,
            1.18856e-05, 6.18323e-06, 5.23174e-05, 2.98652e-05, 1.34951e-05,
            3.75016e-05, 5.31313e-05, 0.000119306, 4.89536e-06, 1.31267e-05,
            1.0709e-05, 3.37309e-05, 6.97744e-05, 0.000206878, 6.25151e-05,
            6.82776e-05, 4.17579e-05, 5.77957e-05, 0.000103009, 7.3583e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 8
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2008():
    """
    IA matrix 2008



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2008())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('step')
def ia_matrix_for_python():
    """
    IA matrix for Python



    component


    """

    if time() < 1996:
        ret = ia_matrix_1995()
    elif time() < 1997:
        ret = ia_matrix_1996()
    elif time() < 1998:
        ret = ia_matrix_1997()
    elif time() < 1999:
        ret = ia_matrix_1998()
    elif time() < 2000:
        ret = ia_matrix_1999()
    elif time() < 2001:
        ret = ia_matrix_2000()
    elif time() < 2002:
        ret = ia_matrix_2001()
    elif time() < 2003:
        ret = ia_matrix_2002()
    elif time() < 2004:
        ret = ia_matrix_2003()
    elif time() < 2005:
        ret = ia_matrix_2004()
    elif time() < 2006:
        ret = ia_matrix_2005()
    elif time() < 2007:
        ret = ia_matrix_2006()
    elif time() < 2008:
        ret = ia_matrix_2007()
    elif time() < 2009:
        ret = ia_matrix_2008()
    else:
        ret = ia_matrix_2009()

    name = 9
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_1995():
    """
    IA matrix 1995



    constant


    """

    ret =  xr.DataArray(
        data=[[0.133347852, 0.002994061, 0.269202856, 0.040645879, 0.029550007,
            0.140634738, 0.012512885, 0.003105442, 0.006638955, 0.013810624,
            0.001215523, 0.000333429, 0.000335808, 0.000265041, 0.000198707,
            0.013078404, 0.000500613, 0.003330773, 0.000528276, 0.002336107,
            0.002529064, 0.036759151, 0.000712228, 0.00199406, 0.000448338,
            0.001104142, 0.000188924, 0.000241248, 0.000534767, 0.001135283,
            0.001746518, 0.001856458, 0.002285006, 0.002241517, 0.015596745
        ], [0.003351171, 0.082069897, 0.003162907, 0.002161771, 0.001491261,
            0.00343566, 0.005067435, 0.426548063, 0.033085251, 0.003346967,
            0.080387783, 0.044756116, 0.003369836, 0.002327251, 0.001548245,
            0.006607311, 0.09811338, 0.019798052, 0.000712524, 0.001692356,
            0.000799184, 0.002077368, 0.006644347, 0.002753419, 0.002789598,
            0.003950777, 0.000558722, 0.000671999, 0.002221224, 0.001019724,
            0.00256993, 0.002842379, 0.001355669, 0.004444483, 0.065589054
        ], [0.05301342, 0.000822757, 0.137250507, 0.002389905, 0.084176065,
            0.0011654, 0.002815945, 0.001118286, 0.009819289, 0.002486386,
            0.001724154, 0.000981881, 0.001162617, 0.001178837, 0.000819823,
            0.00168515, 0.00084638, 0.001064266, 0.001731463, 0.004980866,
            0.004542241, 0.165189944, 0.001621234, 0.00553345, 0.004485957,
            0.002301173, 0.001325177, 0.000999116, 0.000244173, 0.00264925,
            0.008171633, 0.00896135, 0.014609106, 0.005027631, 0.001201309
        ], [0.002520864, 0.001517015, 0.001459438, 0.282760359, 0.052193065,
            0.00547208, 0.007127388, 0.000543699, 0.003977012, 0.018561753,
            0.004741438, 0.001618937, 0.00232742, 0.002247316, 0.006027474,
            0.025187307, 0.000702411, 0.002575631, 0.00172704, 0.003187149,
            0.003729183, 0.003325705, 0.001706654, 0.002936723, 0.00127145,
            0.001735365, 0.002943266, 0.000759909, 0.000267241, 0.001400025,
            0.003463281, 0.00074832, 0.003636633, 0.004952687, 0.000523151
        ], [0.000233707, 0.000299946, 0.000196779, 0.006393776, 0.186462421,
            0.00090658, 0.001397612, 0.000104402, 0.000262644, 0.000931782,
            0.000564356, 0.000375085, 0.000410098, 0.000397849, 0.001203648,
            0.003732673, 0.000189194, 0.000182299, 0.000418842, 0.000542536,
            0.000556168, 0.000221703, 0.000205248, 0.000216585, 0.000146464,
            0.000161638, 0.000640394, 7.67576e-05, 1.99478e-05, 0.000159219,
            0.000910199, 0.000133144, 0.00016204, 0.000689726, 1.52209e-05
        ], [0.001585412, 0.001937267, 0.001510969, 0.001470413, 0.00226386,
            0.187662464, 0.011285935, 0.000402701, 0.001526078, 0.002694099,
            0.006473, 0.002439937, 0.002869877, 0.00195204, 0.003237308,
            0.073104967, 0.00079078, 0.034793219, 0.000831092, 0.001891518,
            0.001677709, 0.0024467, 0.001436695, 0.000611308, 0.000425251,
            0.004945164, 0.000770322, 0.00066278, 0.002159461, 0.000881898,
            0.001422057, 0.001084348, 0.001080988, 0.002905033, 0.000305355
        ], [0.002911216, 0.003411313, 0.023979095, 0.010746901, 0.012843957,
            0.012598547, 0.240983614, 0.001899342, 0.020172977, 0.018583995,
            0.022345174, 0.0059523, 0.01022551, 0.015889058, 0.00602285,
            0.022473881, 0.00353113, 0.005270496, 0.00787651, 0.016312639,
            0.015569031, 0.011557581, 0.006264402, 0.004725943, 0.008400447,
            0.017134527, 0.012269684, 0.016620049, 0.002643433, 0.02276076,
            0.016695535, 0.015409039, 0.010080023, 0.025340169, 0.000747692
        ], [0.012888979, 0.013762998, 0.003726943, 0.005117927, 0.003525075,
            0.005076612, 0.004589267, 0.046095575, 0.02541595, 0.008663735,
            0.015680525, 0.008952182, 0.003753396, 0.003260413, 0.002065759,
            0.004454121, 0.020977464, 0.01246268, 0.00440259, 0.006470645,
            0.005546146, 0.004765646, 0.051308651, 0.049730818, 0.060966438,
            0.014707458, 0.003725228, 0.001388466, 0.001190051, 0.00465098,
            0.007335931, 0.002963448, 0.003470942, 0.00668675, 8.60444e-05
        ], [0.041441193, 0.013741621, 0.0104493, 0.069953141, 0.035489123,
            0.025268363, 0.034205664, 0.015073879, 0.230167842, 0.205193359,
            0.028911692, 0.015003274, 0.012510929, 0.019132899, 0.01258376,
            0.032681715, 0.005776388, 0.009701744, 0.007149789, 0.003388408,
            0.002429838, 0.004611147, 0.0031629, 0.002794505, 0.002169833,
            0.003394421, 0.001390871, 0.000914331, 0.00149026, 0.004489364,
            0.007904692, 0.00333409, 0.06211447, 0.009038951, 0.000140874
        ], [0.004412177, 0.008102499, 0.014321997, 0.00822424, 0.032064839,
            0.00951537, 0.013600865, 0.001845946, 0.017849802, 0.104133983,
            0.009842796, 0.007197463, 0.025960397, 0.025689808, 0.035011553,
            0.040115087, 0.002359315, 0.017663283, 0.019871472, 0.004906684,
            0.005468816, 0.004347275, 0.010839934, 0.002285759, 0.002521484,
            0.006257391, 0.003759938, 0.00078845, 0.000939062, 0.002503527,
            0.003010668, 0.000798776, 0.005389462, 0.003791215, 9.239e-05
        ], [0.001999312, 0.005268112, 0.005607437, 0.001869078, 0.001481817,
            0.008744957, 0.001330509, 0.002165035, 0.006337518, 0.00597473,
            0.106238128, 0.008920492, 0.005515605, 0.011861662, 0.00678703,
            0.006938045, 0.001934753, 0.079392006, 0.002889658, 0.00165925,
            0.001090828, 0.003542703, 0.001666867, 0.000668844, 0.000558392,
            0.001597686, 0.001071238, 0.000321869, 0.00208714, 0.000931003,
            0.001467126, 0.001514212, 0.002068889, 0.002147849, 6.80936e-05
        ], [0.004978944, 0.024696071, 0.015131154, 0.005473694, 0.010707126,
            0.026518803, 0.009096813, 0.006678399, 0.017810027, 0.023668338,
            0.031360423, 0.31833471, 0.163482217, 0.078759541, 0.100378673,
            0.111146266, 0.007507727, 0.083932535, 0.019970524, 0.005326277,
            0.004269733, 0.004752142, 0.007864305, 0.011018146, 0.008301511,
            0.00685303, 0.00386015, 0.000882027, 0.002562505, 0.003197067,
            0.005166809, 0.001777099, 0.002541266, 0.004939988, 0.000341272
        ], [0.007159062, 0.019789372, 0.003930784, 0.006364623, 0.0050476,
            0.008810413, 0.007261906, 0.005046781, 0.007725644, 0.010614027,
            0.012167068, 0.016049186, 0.117655766, 0.016133822, 0.029147291,
            0.009521586, 0.00746765, 0.016148045, 0.021855496, 0.003891829,
            0.00338663, 0.002333065, 0.006360712, 0.005006228, 0.003946696,
            0.004346591, 0.00293233, 0.001237178, 0.001440462, 0.003055873,
            0.006251066, 0.002305264, 0.00207539, 0.003548678, 0.000183255
        ], [0.001521142, 0.007118386, 0.002958593, 0.004299439, 0.003459132,
            0.005696365, 0.008593064, 0.003034116, 0.008242393, 0.011356089,
            0.007727212, 0.014178045, 0.068250934, 0.250167657, 0.047661184,
            0.018886632, 0.011926089, 0.024755017, 0.023343849, 0.008515586,
            0.007507126, 0.003820196, 0.005664323, 0.003027849, 0.003375896,
            0.005730915, 0.026054658, 0.003056245, 0.00134764, 0.011517959,
            0.011038882, 0.005125374, 0.016906983, 0.006999914, 0.000704334
        ], [0.00363293, 0.005707457, 0.001682498, 0.002479075, 0.002182076,
            0.002856222, 0.001824274, 0.00120983, 0.001951592, 0.003157048,
            0.003476767, 0.004850754, 0.013438791, 0.00476323, 0.288768114,
            0.004479062, 0.00145511, 0.003851702, 0.08913319, 0.005505144,
            0.004435684, 0.00123027, 0.027101224, 0.035323587, 0.046673994,
            0.009472666, 0.003030542, 0.000961046, 0.00047933, 0.003128078,
            0.018118091, 0.000952987, 0.001323522, 0.003389285, 0.000149565
        ], [0.000914527, 0.001254225, 0.001037695, 0.005656422, 0.003555405,
            0.006029762, 0.003038281, 0.000648539, 0.001831787, 0.002527967,
            0.003214511, 0.00687942, 0.003817621, 0.003041233, 0.004940953,
            0.034330385, 0.000899634, 0.005142408, 0.002194172, 0.001750773,
            0.001798092, 0.003140429, 0.001297914, 0.001122601, 0.001168328,
            0.001137779, 0.000879809, 0.001223265, 0.000986836, 0.002487224,
            0.003884737, 0.003352244, 0.002925107, 0.00599562, 0.004112183
        ], [0.009217599, 0.023330287, 0.011731217, 0.016846113, 0.00787993,
            0.015656338, 0.020699571, 0.015130216, 0.028442408, 0.021594053,
            0.036862585, 0.026643744, 0.011037302, 0.010097413, 0.009065559,
            0.010549761, 0.092374909, 0.005097125, 0.010102673, 0.006860541,
            0.013897756, 0.0236364, 0.015779659, 0.004498713, 0.004024655,
            0.012914092, 0.008285639, 0.004716048, 0.008347197, 0.006382473,
            0.014655065, 0.018322379, 0.012915816, 0.016442752, 0.000244617
        ], [0.00367014, 0.008299959, 0.00269237, 0.002645593, 0.00216941,
            0.004607808, 0.00406959, 0.003750316, 0.004565105, 0.003867116,
            0.007570545, 0.0049534, 0.003546695, 0.003577989, 0.0024623,
            0.003546917, 0.026681251, 0.036422233, 0.00467155, 0.004464915,
            0.005037726, 0.005870339, 0.009753506, 0.003669304, 0.002536949,
            0.01408434, 0.012637068, 0.005740893, 0.04380817, 0.005773888,
            0.021268423, 0.013505525, 0.006848819, 0.013474952, 0.003149753
        ], [0.00516213, 0.004442358, 0.006289323, 0.005128861, 0.007237688,
            0.005944246, 0.004410727, 0.003799484, 0.005990582, 0.006026155,
            0.007374344, 0.005888385, 0.005047066, 0.004898098, 0.004642263,
            0.006093283, 0.009561681, 0.007827622, 0.007927839, 0.005565194,
            0.005145376, 0.005333971, 0.018503691, 0.003574527, 0.002841854,
            0.007155219, 0.004688793, 0.002437929, 0.001841069, 0.005914035,
            0.003871522, 0.003040329, 0.003374765, 0.006422952, 4.56524e-05
        ], [0.031974175, 0.025217604, 0.058265228, 0.052564937, 0.069470325,
            0.053855518, 0.047939478, 0.045124369, 0.047782097, 0.048488653,
            0.043553831, 0.04236367, 0.04653396, 0.052188975, 0.041118511,
            0.04916961, 0.021195102, 0.042788951, 0.03729795, 0.033167639,
            0.013991331, 0.044271956, 0.023719271, 0.017846567, 0.018125298,
            0.021558184, 0.008946241, 0.005718082, 0.00336068, 0.012122767,
            0.019027198, 0.009386302, 0.026990522, 0.015001791, 0.000333733
        ], [0.017168074, 0.010441708, 0.029450978, 0.025934335, 0.039862633,
            0.021088652, 0.018190105, 0.015786886, 0.022335372, 0.022177467,
            0.019526002, 0.016188626, 0.017167447, 0.017295903, 0.018025661,
            0.026315077, 0.00863216, 0.027407905, 0.01749944, 0.009727614,
            0.009031615, 0.02652719, 0.015832043, 0.009399961, 0.008605703,
            0.009537785, 0.009804605, 0.004203319, 0.003245725, 0.009588325,
            0.007862266, 0.005786237, 0.011732619, 0.012557207, 0.00063707
        ], [0.001354729, 0.003521309, 0.004014537, 0.004295829, 0.003997177,
            0.004930179, 0.005688269, 0.002347842, 0.005375778, 0.005729974,
            0.006012603, 0.005099619, 0.00616424, 0.006707545, 0.003225026,
            0.004622629, 0.004974446, 0.005370422, 0.006803611, 0.009852612,
            0.006389633, 0.009658253, 0.007766124, 0.00665935, 0.023744876,
            0.019341349, 0.004909034, 0.010106225, 0.002353281, 0.013233358,
            0.008865266, 0.006078937, 0.008268914, 0.010868643, 0.00011014
        ], [0.015013346, 0.028619864, 0.021879232, 0.017814717, 0.021355013,
            0.024763582, 0.018547712, 0.044260726, 0.02008352, 0.017202703,
            0.035206394, 0.01824284, 0.012582979, 0.010493793, 0.011947319,
            0.018787034, 0.026684866, 0.018504906, 0.013387591, 0.021758678,
            0.011594397, 0.01060534, 0.041615696, 0.017631452, 0.012113819,
            0.031244461, 0.011344005, 0.007018231, 0.001914119, 0.007327376,
            0.01185732, 0.00975037, 0.007989693, 0.01103205, 0.000724509
        ], [0.002714565, 0.00443304, 0.003235549, 0.002567415, 0.003360837,
            0.006193569, 0.001510039, 0.005810413, 0.002989803, 0.002047615,
            0.005374527, 0.003930689, 0.00162025, 0.001329474, 0.001624248,
            0.001815074, 0.002433631, 0.00242854, 0.001091555, 0.003036474,
            0.001459858, 0.001442538, 0.00344821, 0.09843665, 0.002881142,
            0.004996921, 0.001549332, 0.000696928, 0.000151062, 0.000954653,
            0.001250115, 0.001145841, 0.00059991, 0.001024454, 6.41201e-05
        ], [0.000801535, 0.002175193, 0.00149861, 0.00171061, 0.002866039,
            0.001385565, 0.002160402, 0.001119779, 0.001976968, 0.001877297,
            0.001500485, 0.001457172, 0.001675004, 0.002132724, 0.001677883,
            0.001629079, 0.000837357, 0.00111559, 0.001951979, 0.004512365,
            0.001678387, 0.001203168, 0.002039266, 0.004505215, 0.030411192,
            0.016659574, 0.005267493, 0.003156067, 0.000454596, 0.004302847,
            0.00437377, 0.002169488, 0.001307306, 0.003950904, 2.49721e-05
        ], [0.003679386, 0.008346574, 0.006340813, 0.004005496, 0.005020741,
            0.006352971, 0.007000116, 0.00699518, 0.005595951, 0.005245015,
            0.009662936, 0.005593231, 0.004879594, 0.004345888, 0.004204175,
            0.00475323, 0.005037746, 0.003495278, 0.007393265, 0.021112383,
            0.0098722, 0.007806847, 0.035918811, 0.137260249, 0.097638604,
            0.106313209, 0.003465195, 0.003846226, 0.000683581, 0.004759578,
            0.003265175, 0.002366209, 0.003014671, 0.005662676, 6.42988e-05
        ], [0.001846168, 0.004818312, 0.003311597, 0.00353185, 0.003565491,
            0.003556592, 0.009254447, 0.002294633, 0.005430306, 0.004641414,
            0.004180855, 0.004096593, 0.005526469, 0.005694055, 0.003080344,
            0.005778211, 0.004311982, 0.006130631, 0.009985015, 0.017744071,
            0.017153845, 0.010620077, 0.012756258, 0.018452193, 0.028594519,
            0.018715459, 0.070664296, 0.020415558, 0.003165123, 0.018607671,
            0.015543939, 0.006704324, 0.009401403, 0.018295651, 0.000161843
        ], [0.020134025, 0.033392406, 0.016127439, 0.022414464, 0.019639247,
            0.020974024, 0.021216977, 0.015061877, 0.021600799, 0.01926038,
            0.02261346, 0.021002261, 0.020102974, 0.018392578, 0.014256886,
            0.022349422, 0.022920391, 0.019438775, 0.026058774, 0.045987837,
            0.035677655, 0.023882495, 0.041066199, 0.041877908, 0.038340858,
            0.028073732, 0.024286915, 0.163976882, 0.047113185, 0.035145063,
            0.023720506, 0.013063927, 0.021267311, 0.032630254, 0.004238545
        ], [0.007184519, 0.005642618, 0.004212491, 0.005778727, 0.006715496,
            0.005292456, 0.008481112, 0.002829833, 0.004995433, 0.006528465,
            0.005876424, 0.005457423, 0.006487941, 0.005833624, 0.004038792,
            0.009164501, 0.006457893, 0.012535312, 0.02511011, 0.033607611,
            0.044620069, 0.03309295, 0.011610724, 0.02858551, 0.017566097,
            0.022019362, 0.013045371, 0.026428642, 0.029708905, 0.023568542,
            0.011124627, 0.018138449, 0.030203817, 0.027770553, 0.000747191
        ], [0.011600035, 0.029660874, 0.034242057, 0.026984689, 0.025116522,
            0.021122514, 0.053852474, 0.016565408, 0.058126245, 0.033632447,
            0.031519076, 0.026654247, 0.040329016, 0.056234267, 0.035471021,
            0.036919872, 0.035813826, 0.051179976, 0.042818161, 0.061676155,
            0.063101571, 0.032651439, 0.038029627, 0.054276218, 0.092860957,
            0.061661035, 0.061760097, 0.08125575, 0.031595984, 0.122460445,
            0.053406226, 0.025561353, 0.047682309, 0.062273473, 0.00021864
        ], [0.002494378, 0.003788251, 0.00229677, 0.001771868, 0.001230638,
            0.005392863, 0.004083556, 0.001830852, 0.003933964, 0.002711408,
            0.003000564, 0.002999021, 0.002156533, 0.002089519, 0.001876432,
            0.003259391, 0.00759944, 0.002008365, 0.003250308, 0.003196896,
            0.004834993, 0.004076747, 0.003805062, 0.010065801, 0.011596379,
            0.004685751, 0.002819225, 0.002360883, 0.004467577, 0.004758735,
            0.008926699, 0.003912761, 0.005673581, 0.010693085, 8.64271e-06
        ], [0.000826024, 0.000454732, 0.000278524, 0.000298782, 0.000320412,
            0.000210336, 0.000540029, 0.000357722, 0.000663323, 0.00037073,
            0.000452345, 0.000411344, 0.000695833, 0.000900181, 0.000359644,
            0.000274731, 0.000698912, 0.000396204, 0.000352169, 0.000966145,
            0.00192897, 0.000370008, 0.000877586, 0.000289613, 0.0015428,
            0.000858598, 0.001984948, 0.001046057, 0.000133984, 0.00188756,
            0.004353998, 0.005427549, 0.0008975, 0.001213415, 7.16322e-07
        ], [0.001855911, 0.000416151, 0.00034455, 0.000396891, 0.000374634,
            0.00033575, 0.000372334, 0.000183688, 0.000406592, 0.000357481,
            0.000416871, 0.000329254, 0.000459991, 0.000436183, 0.000249237,
            0.0004375, 0.000326708, 0.000467006, 0.000458031, 0.00064654,
            0.000449927, 0.000971287, 0.000658392, 0.000830341, 0.000502966,
            0.000560183, 0.000488003, 0.000535, 8.40707e-05, 0.000614651,
            0.006165844, 0.001379857, 0.020395456, 0.001393603, 6.53207e-08
        ], [0.002856823, 0.006544769, 0.006782452, 0.00532237, 0.006464885,
            0.006189543, 0.012909914, 0.003773928, 0.008795235, 0.00714829,
            0.006074222, 0.007356634, 0.00652029, 0.007190238, 0.004452248,
            0.007137458, 0.005266932, 0.005889579, 0.009397455, 0.009991303,
            0.011455976, 0.013406109, 0.010212682, 0.007988312, 0.009696813,
            0.011329832, 0.017865794, 0.010978716, 0.007419254, 0.029323745,
            0.017013238, 0.013213901, 0.015717723, 0.071457097, 0.00060386
        ], [1.09748e-05, 1.62163e-05, 3.3644e-05, 3.99474e-05, 2.78826e-05,
            3.61057e-05, 4.10564e-05, 1.53774e-05, 2.99816e-05, 3.54221e-05,
            2.40688e-05, 3.54234e-05, 3.18596e-05, 2.17691e-05, 4.58056e-05,
            4.05146e-05, 1.61634e-05, 8.94356e-05, 0.000155791, 5.69875e-05,
            7.37625e-05, 7.50137e-05, 0.000173589, 2.76345e-05, 4.22151e-05,
            3.28437e-05, 9.18283e-05, 6.75954e-05, 0.000289781, 7.64379e-05,
            0.000100741, 5.50673e-05, 0.000126119, 0.000127429, 2.7927e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_1995():
    """
    IA matrix 1995



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_1995())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_1996():
    """
    IA matrix 1996



    constant


    """

    ret =  xr.DataArray(
        data=[[0.129659953, 0.002896558, 0.272639635, 0.040355209, 0.029482267,
            0.13798054, 0.01174434, 0.002886797, 0.007030273, 0.012918151,
            0.001117297, 0.000317862, 0.000317932, 0.000241614, 0.000194006,
            0.011710805, 0.000507344, 0.003251737, 0.000554363, 0.002365078,
            0.002581156, 0.037733888, 0.000712811, 0.002053852, 0.00043843,
            0.001171137, 0.000210216, 0.000172263, 0.000525149, 0.001111424,
            0.001837966, 0.001979651, 0.002429268, 0.002357345, 0.01522777
        ], [0.003718556, 0.097313326, 0.003589887, 0.002286383, 0.001371051,
            0.003845417, 0.005392146, 0.436414845, 0.033047605, 0.003804956,
            0.078392589, 0.045200189, 0.003651401, 0.002426435, 0.001587427,
            0.007572281, 0.098991993, 0.020119298, 0.000720529, 0.001786557,
            0.000808648, 0.002659206, 0.007095667, 0.002996645, 0.002964435,
            0.004142248, 0.00054152, 0.000489003, 0.00206372, 0.001091195,
            0.002940563, 0.003783993, 0.001558111, 0.003523086, 0.064144424
        ], [0.053373775, 0.000804982, 0.139009929, 0.002311541, 0.083437503,
            0.001168624, 0.002777428, 0.000953989, 0.009711607, 0.002368331,
            0.00185279, 0.000930163, 0.001123387, 0.001103943, 0.00079455,
            0.001586519, 0.000736104, 0.001000468, 0.001621667, 0.004618256,
            0.004404405, 0.161299227, 0.001606493, 0.006795716, 0.004322816,
            0.002453884, 0.001516641, 0.000803836, 0.000219753, 0.002291536,
            0.008042272, 0.009032607, 0.01432422, 0.005318456, 0.001260682
        ], [0.002418462, 0.001546426, 0.001485782, 0.285331153, 0.054213432,
            0.005885993, 0.00726785, 0.000534526, 0.00417253, 0.019840587,
            0.005144909, 0.001656997, 0.002464897, 0.002275891, 0.006170536,
            0.024927239, 0.000559012, 0.002732464, 0.001804958, 0.003010449,
            0.003675476, 0.003251781, 0.001744693, 0.003118092, 0.00128741,
            0.001793263, 0.002869942, 0.000665858, 0.000263621, 0.001369937,
            0.003295811, 0.000810695, 0.003564451, 0.00493945, 0.000508203
        ], [0.000246407, 0.000328695, 0.000207026, 0.006667822, 0.188859961,
            0.001129236, 0.00133403, 0.00010904, 0.000291481, 0.001040978,
            0.000652049, 0.000435987, 0.0004472, 0.000410281, 0.001229401,
            0.003686108, 0.000213053, 0.00018186, 0.000422446, 0.000506424,
            0.000535474, 0.000234643, 0.000217696, 0.00021196, 0.0001482,
            0.000173049, 0.000677218, 7.91158e-05, 2.14835e-05, 0.000162299,
            0.000889472, 0.000150069, 0.000172596, 0.000734594, 1.40127e-05
        ], [0.001626495, 0.001987625, 0.001512154, 0.001534348, 0.002220867,
            0.187641378, 0.011115458, 0.000421848, 0.001573424, 0.00268443,
            0.006665337, 0.002525898, 0.002897142, 0.001873194, 0.003111175,
            0.072952191, 0.000831569, 0.035583319, 0.00081667, 0.001716794,
            0.001532057, 0.002306702, 0.00137679, 0.000595805, 0.000430067,
            0.004343589, 0.000730557, 0.000625581, 0.002238382, 0.000936208,
            0.001386559, 0.001144534, 0.000940626, 0.00283003, 0.000302171
        ], [0.002833596, 0.003384351, 0.024069479, 0.010189354, 0.012707345,
            0.012676982, 0.235126423, 0.001676006, 0.019752092, 0.018432981,
            0.023227918, 0.006051781, 0.01004571, 0.01554481, 0.005946846,
            0.02228959, 0.003479407, 0.005326579, 0.007975653, 0.015146909,
            0.015011443, 0.011058055, 0.006205521, 0.004393825, 0.007654884,
            0.01630114, 0.011481405, 0.01622772, 0.002677819, 0.022737593,
            0.016393056, 0.016299789, 0.009919717, 0.02477997, 0.000694923
        ], [0.012170818, 0.013557568, 0.003621048, 0.004816411, 0.003239794,
            0.004760866, 0.004570627, 0.045210174, 0.026259445, 0.00858899,
            0.016002131, 0.008929759, 0.003671932, 0.003063619, 0.002048297,
            0.004445444, 0.019498404, 0.01277302, 0.00442553, 0.006393718,
            0.005409681, 0.003936843, 0.051077611, 0.051253268, 0.056298441,
            0.014720402, 0.00345236, 0.001269984, 0.00117175, 0.004488909,
            0.007479284, 0.003003096, 0.003410036, 0.006299345, 7.94193e-05
        ], [0.040208979, 0.013823937, 0.010654472, 0.070872245, 0.034149838,
            0.026394382, 0.033956304, 0.01417111, 0.23048735, 0.199881221,
            0.03078287, 0.01541163, 0.012646484, 0.01922002, 0.012851326,
            0.033005512, 0.005836453, 0.009844464, 0.007055643, 0.003457274,
            0.002537663, 0.004534092, 0.00335039, 0.003060051, 0.002184908,
            0.003455218, 0.001420891, 0.000885157, 0.001533712, 0.00441479,
            0.008237753, 0.003670525, 0.061457857, 0.009124386, 0.000140615
        ], [0.004386203, 0.008642069, 0.01466963, 0.0082164, 0.033516619,
            0.009579604, 0.014228549, 0.001765105, 0.018004024, 0.109304431,
            0.010713306, 0.007493388, 0.026082133, 0.026243118, 0.035030357,
            0.038208723, 0.002278868, 0.017770297, 0.018572637, 0.004720599,
            0.005303473, 0.004225894, 0.01150188, 0.002284638, 0.002430127,
            0.006217904, 0.003771197, 0.000793289, 0.000977558, 0.002560747,
            0.003001298, 0.000863652, 0.005386125, 0.003387666, 9.77402e-05
        ], [0.001879147, 0.005110698, 0.00541338, 0.001935287, 0.001606484,
            0.009368053, 0.001397057, 0.001983846, 0.006527225, 0.006064084,
            0.105754353, 0.00944097, 0.005619356, 0.012020798, 0.006917653,
            0.006899962, 0.001806799, 0.079809576, 0.002868988, 0.001554843,
            0.00105188, 0.003588981, 0.001441719, 0.000542678, 0.000527728,
            0.001364588, 0.001075682, 0.000253746, 0.002121816, 0.000967686,
            0.00148335, 0.001640104, 0.002004889, 0.001871838, 7.9846e-05
        ], [0.004941888, 0.025825578, 0.014944872, 0.005586658, 0.010425594,
            0.027157616, 0.009127469, 0.006037003, 0.017653091, 0.023560774,
            0.032070218, 0.314610222, 0.160905187, 0.076686615, 0.100230074,
            0.108767456, 0.007478182, 0.085053701, 0.019838284, 0.005090378,
            0.004137133, 0.004671204, 0.008009105, 0.01082883, 0.008008196,
            0.006763962, 0.003874451, 0.000910141, 0.002586022, 0.003383619,
            0.005027124, 0.001886568, 0.00268755, 0.004501, 0.00032462
        ], [0.007029481, 0.01923891, 0.00385439, 0.006365394, 0.004904916,
            0.008670165, 0.007058031, 0.004445044, 0.007646384, 0.010558349,
            0.01242983, 0.016089831, 0.118761352, 0.015214806, 0.028773903,
            0.009395459, 0.007472459, 0.016047227, 0.023756651, 0.003927314,
            0.00320322, 0.002276602, 0.006623273, 0.005488368, 0.004207802,
            0.004652988, 0.002962229, 0.001224105, 0.001425384, 0.003078968,
            0.006414735, 0.002142068, 0.002195866, 0.003582416, 0.000179988
        ], [0.001523221, 0.006888726, 0.003171475, 0.004635356, 0.003488842,
            0.006302293, 0.009164046, 0.002886191, 0.008765745, 0.011921307,
            0.008518083, 0.015551074, 0.074059822, 0.253847039, 0.049145,
            0.02066588, 0.012282066, 0.025782228, 0.024469191, 0.008971577,
            0.007817687, 0.004089424, 0.00620242, 0.003198938, 0.003615492,
            0.006552346, 0.027716868, 0.003555386, 0.001493359, 0.012475503,
            0.011685467, 0.005544197, 0.018021459, 0.007660016, 0.000658983
        ], [0.003738531, 0.005697815, 0.00173061, 0.002535993, 0.002261395,
            0.002880743, 0.00186032, 0.001018175, 0.002045716, 0.003305158,
            0.003525109, 0.004730073, 0.013542158, 0.004750981, 0.291389919,
            0.004349003, 0.001487442, 0.003842435, 0.089299966, 0.005340474,
            0.004423612, 0.001262065, 0.029036697, 0.039742167, 0.044933338,
            0.01019274, 0.002944393, 0.000866372, 0.000463256, 0.003055301,
            0.017995727, 0.000989197, 0.001364836, 0.003260064, 0.000126873
        ], [0.00090564, 0.001433162, 0.00113957, 0.005557363, 0.003549243,
            0.006302017, 0.003144455, 0.00055826, 0.001829998, 0.00247004,
            0.003452535, 0.007022994, 0.003884864, 0.003182882, 0.005116658,
            0.035689444, 0.001037762, 0.004973725, 0.002236005, 0.001605519,
            0.001677469, 0.002876154, 0.001430335, 0.001037038, 0.001173157,
            0.00140445, 0.000935553, 0.001213167, 0.001065742, 0.002290995,
            0.003929907, 0.003630153, 0.002904066, 0.005956575, 0.003628516
        ], [0.00935334, 0.024953287, 0.011624547, 0.016508077, 0.007632128,
            0.015739575, 0.0216684, 0.014253046, 0.029109013, 0.021731954,
            0.03689868, 0.027440574, 0.011296118, 0.010399644, 0.009009223,
            0.01123895, 0.093436672, 0.005134802, 0.010553314, 0.006873082,
            0.014128567, 0.023480476, 0.015731208, 0.004175145, 0.004301106,
            0.013618374, 0.008289305, 0.004641714, 0.008356713, 0.006563709,
            0.01517517, 0.019492222, 0.013391149, 0.016075851, 0.000236234
        ], [0.003630175, 0.008060589, 0.002530738, 0.002583948, 0.001981903,
            0.004494708, 0.004020089, 0.003187252, 0.004487882, 0.003787548,
            0.00781841, 0.005081449, 0.003445833, 0.00353371, 0.002371838,
            0.003320686, 0.026504201, 0.035165018, 0.004340221, 0.004394033,
            0.005008878, 0.005562734, 0.009900164, 0.003539277, 0.002337233,
            0.015323027, 0.011790893, 0.005623986, 0.04499341, 0.005621917,
            0.020800632, 0.014486971, 0.007044238, 0.013082205, 0.002656123
        ], [0.005015224, 0.004214347, 0.006316257, 0.005154449, 0.007230774,
            0.006159215, 0.00448428, 0.003141375, 0.006204552, 0.006244127,
            0.007700042, 0.006287001, 0.005280846, 0.005287031, 0.004660051,
            0.006191913, 0.009150275, 0.00771991, 0.008364177, 0.004524282,
            0.004626992, 0.00539558, 0.018330724, 0.003618616, 0.002822565,
            0.007495218, 0.00468928, 0.002495579, 0.001865348, 0.006160049,
            0.003999537, 0.003272766, 0.003328324, 0.006596827, 4.79614e-05
        ], [0.032807845, 0.026458446, 0.06011085, 0.051595512, 0.069941101,
            0.053173717, 0.048084546, 0.038012049, 0.048184347, 0.049985769,
            0.04400818, 0.0423373, 0.048040301, 0.052282962, 0.042145573,
            0.051592844, 0.022715486, 0.043379664, 0.038860774, 0.033898679,
            0.013853401, 0.04419724, 0.022708819, 0.017850577, 0.017863763,
            0.01953925, 0.009205329, 0.004962998, 0.00323827, 0.01196931,
            0.020059066, 0.009908768, 0.027434745, 0.014133438, 0.000324585
        ], [0.016765728, 0.010786513, 0.030014655, 0.025958016, 0.039747264,
            0.021160773, 0.018054319, 0.013671897, 0.022463456, 0.022035253,
            0.019525414, 0.016026579, 0.017320541, 0.016811105, 0.018478044,
            0.026278659, 0.008201988, 0.028008878, 0.017672923, 0.009210181,
            0.008698601, 0.027856995, 0.015342132, 0.009091474, 0.007966631,
            0.009620698, 0.010077705, 0.004104142, 0.003332258, 0.009555507,
            0.007923946, 0.005917002, 0.011729119, 0.012660572, 0.000634183
        ], [0.001322761, 0.003604137, 0.003742064, 0.004210668, 0.003967247,
            0.005193135, 0.005535949, 0.001694249, 0.005161368, 0.00577181,
            0.005938872, 0.005128625, 0.006272458, 0.006654445, 0.003168711,
            0.004616917, 0.00460066, 0.005089127, 0.006542828, 0.009533433,
            0.006237207, 0.009867032, 0.007967218, 0.006224255, 0.022643732,
            0.020280492, 0.007926118, 0.010205138, 0.00234562, 0.013618902,
            0.009220855, 0.006854413, 0.008199615, 0.011087529, 0.00010152
        ], [0.014085238, 0.028348268, 0.021751976, 0.018012995, 0.0207858,
            0.024492694, 0.018201743, 0.034511489, 0.019670382, 0.017302614,
            0.035508517, 0.018200429, 0.012442895, 0.010059933, 0.011796501,
            0.018467947, 0.025318441, 0.017791995, 0.013866345, 0.019203814,
            0.010743422, 0.010300668, 0.041911319, 0.018771251, 0.011453667,
            0.030931305, 0.009788831, 0.006445852, 0.001747833, 0.006969296,
            0.011683431, 0.010080719, 0.007733085, 0.009962734, 0.000734395
        ], [0.002671329, 0.004154138, 0.003630672, 0.002870637, 0.003643534,
            0.006989385, 0.001611896, 0.004468169, 0.002941716, 0.002182204,
            0.005132709, 0.003981254, 0.001568304, 0.001429265, 0.001619189,
            0.002121944, 0.00202299, 0.002463304, 0.001113541, 0.00255951,
            0.001368675, 0.001590588, 0.003327489, 0.100876363, 0.002905321,
            0.005738249, 0.001169673, 0.000655583, 0.000128525, 0.001003928,
            0.00142711, 0.001523248, 0.000569985, 0.000918254, 6.69162e-05
        ], [0.000826701, 0.002495197, 0.001582592, 0.001769715, 0.002837799,
            0.001436944, 0.00221276, 0.000896282, 0.00206443, 0.001918824,
            0.001583501, 0.001532236, 0.001881227, 0.002256744, 0.001842026,
            0.001697241, 0.000843277, 0.001230614, 0.002154012, 0.004372731,
            0.001748113, 0.001249445, 0.002159687, 0.00460322, 0.032272524,
            0.019285552, 0.00525849, 0.003452573, 0.000505297, 0.004480612,
            0.004660763, 0.002212813, 0.001352623, 0.004043798, 2.17695e-05
        ], [0.003354935, 0.007001744, 0.006026764, 0.003899478, 0.004751476,
            0.00648427, 0.006958103, 0.006685782, 0.005436352, 0.005323635,
            0.009480709, 0.005732585, 0.004785506, 0.004183739, 0.004169435,
            0.004917823, 0.005024451, 0.003428707, 0.007523643, 0.019635319,
            0.009595464, 0.008149735, 0.036993556, 0.140839805, 0.096491224,
            0.112976377, 0.003485176, 0.00381434, 0.00065391, 0.004896482,
            0.004473183, 0.002428982, 0.002996075, 0.005205421, 6.89827e-05
        ], [0.001915898, 0.005293483, 0.003278108, 0.003613865, 0.003716905,
            0.003802884, 0.009604827, 0.002089672, 0.005729468, 0.004725244,
            0.004604787, 0.004498042, 0.005882693, 0.005891715, 0.003179963,
            0.006031274, 0.004463411, 0.007515475, 0.010699817, 0.017751845,
            0.017613017, 0.010729429, 0.013328643, 0.0194581, 0.028162341,
            0.020787732, 0.080223196, 0.021313619, 0.00349919, 0.019326475,
            0.016187518, 0.007357376, 0.009702709, 0.019581197, 0.000143597
        ], [0.018827924, 0.032038634, 0.015649731, 0.021425027, 0.018448076,
            0.021638854, 0.021134303, 0.012432975, 0.021298681, 0.019013375,
            0.023621409, 0.021374434, 0.020074517, 0.018188244, 0.014049433,
            0.021410756, 0.022699886, 0.019833826, 0.026059769, 0.043039273,
            0.035727943, 0.023355066, 0.0412109, 0.040356106, 0.037151368,
            0.027369063, 0.022981751, 0.170027638, 0.04894328, 0.035220222,
            0.022848493, 0.01302161, 0.021476878, 0.031677141, 0.00365031
        ], [0.006684765, 0.005490927, 0.004102099, 0.005428336, 0.005779265,
            0.00530694, 0.008283073, 0.002479349, 0.004783331, 0.006184813,
            0.005427958, 0.005272766, 0.006281318, 0.005440651, 0.003947825,
            0.008744539, 0.005701008, 0.0126676, 0.024642928, 0.02917499,
            0.042942112, 0.028748869, 0.010698391, 0.027688521, 0.016390828,
            0.021282013, 0.012024102, 0.025073302, 0.030091031, 0.023699282,
            0.010945195, 0.017505179, 0.029367114, 0.027820913, 0.000625306
        ], [0.011785102, 0.029876139, 0.035117301, 0.02658489, 0.024916627,
            0.022550123, 0.055070342, 0.015113738, 0.058179342, 0.033619318,
            0.032456389, 0.027255963, 0.041876573, 0.05822971, 0.035528359,
            0.036525502, 0.034643397, 0.051082826, 0.042972511, 0.058696763,
            0.063159265, 0.032679028, 0.038765315, 0.053489901, 0.091549879,
            0.063705287, 0.062943137, 0.084957975, 0.033375472, 0.126588745,
            0.054343021, 0.025619829, 0.049482031, 0.063367569, 0.000214075
        ], [0.001713452, 0.003661173, 0.001852937, 0.001228964, 0.000933599,
            0.003181617, 0.003200312, 0.001154109, 0.003146554, 0.002207286,
            0.002448835, 0.00246777, 0.001376364, 0.00163094, 0.001210223,
            0.002480236, 0.005460999, 0.00149161, 0.002349584, 0.001993061,
            0.003737264, 0.003668003, 0.003249963, 0.008844854, 0.009023738,
            0.003893647, 0.002655547, 0.001683987, 0.003626269, 0.002828651,
            0.007774777, 0.003906226, 0.005287227, 0.007794869, 2.87498e-06
        ], [0.000807044, 0.000475993, 0.000340113, 0.00036319, 0.000366291,
            0.000259773, 0.000621323, 0.000425021, 0.000817771, 0.000466198,
            0.000503941, 0.000476328, 0.000783265, 0.000951313, 0.000469793,
            0.000341829, 0.001018193, 0.000481609, 0.000427017, 0.000960798,
            0.00193564, 0.00041641, 0.0010732, 0.00034602, 0.001747993,
            0.001103616, 0.001616266, 0.001259784, 0.000172764, 0.002141622,
            0.004438646, 0.006405385, 0.001053067, 0.001402682, 8.32451e-07
        ], [0.002028694, 0.000393451, 0.000387978, 0.000427931, 0.000393002,
            0.000370815, 0.000421959, 0.00019374, 0.000433264, 0.000385758,
            0.000435331, 0.000362063, 0.00051512, 0.000451149, 0.00029597,
            0.000508409, 0.000411754, 0.000520905, 0.000521361, 0.00066402,
            0.000471231, 0.0010167, 0.000688203, 0.000910047, 0.000562589,
            0.0006852, 0.000775819, 0.000604241, 0.000100222, 0.00078511,
            0.006170869, 0.001425334, 0.02229047, 0.001643834, 1.02927e-07
        ], [0.002415733, 0.006081182, 0.006286526, 0.005388501, 0.006305516,
            0.005573682, 0.013008788, 0.003527436, 0.008283769, 0.006000501,
            0.006050614, 0.007119907, 0.006642093, 0.006250376, 0.004253276,
            0.006803615, 0.006245322, 0.0064516, 0.010099807, 0.009901243,
            0.011146824, 0.013951506, 0.009815249, 0.00706116, 0.009623645,
            0.011473728, 0.017902088, 0.011089716, 0.007686089, 0.029408526,
            0.015523231, 0.010390065, 0.015551125, 0.078652589, 0.000493677
        ], [1.06019e-05, 1.51742e-05, 3.21763e-05, 3.9354e-05, 2.83863e-05,
            3.55654e-05, 3.7168e-05, 1.39332e-05, 3.03032e-05, 3.47757e-05,
            2.49402e-05, 3.42023e-05, 3.1417e-05, 2.10951e-05, 4.03277e-05,
            3.57644e-05, 1.65735e-05, 8.50735e-05, 0.000138434, 4.47814e-05,
            6.21993e-05, 6.92429e-05, 0.000165706, 2.85625e-05, 3.80083e-05,
            3.43312e-05, 8.41192e-05, 6.59996e-05, 0.000283537, 7.4358e-05,
            9.01757e-05, 5.44688e-05, 0.000128192, 0.000123181, 2.5096e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_1996():
    """
    IA matrix 1996



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_1996())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_1997():
    """
    IA matrix 1997



    constant


    """

    ret =  xr.DataArray(
        data=[[0.128397899, 0.00288089, 0.268910601, 0.038690202, 0.029829602,
            0.132857977, 0.01160687, 0.003129731, 0.007089067, 0.012891835,
            0.001018114, 0.000303137, 0.000303133, 0.000228253, 0.000181488,
            0.01197033, 0.000478186, 0.00304915, 0.000493111, 0.002196271,
            0.002381494, 0.037065143, 0.000682396, 0.001976864, 0.000442483,
            0.001157398, 0.000199273, 0.000176779, 0.000518675, 0.001105391,
            0.001787072, 0.001899819, 0.00234746, 0.002342852, 0.013028469
        ], [0.003445926, 0.086453084, 0.003434352, 0.002065805, 0.001316679,
            0.003564083, 0.005243907, 0.426213558, 0.032364468, 0.003544437,
            0.077534427, 0.041681288, 0.003370927, 0.002036728, 0.001482677,
            0.006823633, 0.095104284, 0.018098265, 0.000635543, 0.001486543,
            0.000857415, 0.00224075, 0.006121246, 0.00291212, 0.00273783,
            0.00356269, 0.000509548, 0.000410769, 0.001963592, 0.001037098,
            0.002720399, 0.003802139, 0.001501783, 0.003096856, 0.054880235
        ], [0.054333335, 0.00080214, 0.138645782, 0.002306033, 0.085071037,
            0.001262288, 0.002790004, 0.001019496, 0.009663612, 0.002459512,
            0.001908523, 0.000967204, 0.001171888, 0.001158918, 0.000817001,
            0.001553905, 0.000782009, 0.001051083, 0.001693718, 0.00455339,
            0.004337245, 0.159100006, 0.001613403, 0.007337838, 0.004585508,
            0.002442863, 0.001399848, 0.000830922, 0.000238143, 0.002145204,
            0.007870599, 0.008911694, 0.014155102, 0.005673216, 0.001078606
        ], [0.002473605, 0.001420914, 0.001499851, 0.280278856, 0.055668182,
            0.005816545, 0.007126467, 0.000533107, 0.004204301, 0.019874235,
            0.004939412, 0.001582393, 0.002345834, 0.002130142, 0.006278643,
            0.024187483, 0.00061174, 0.00257056, 0.001770152, 0.002845096,
            0.00347016, 0.003061283, 0.001703161, 0.003111789, 0.001134529,
            0.001703517, 0.00304825, 0.000666469, 0.000263628, 0.001279284,
            0.003354706, 0.00086113, 0.003570367, 0.00494858, 0.000434805
        ], [0.00023783, 0.000296835, 0.000223715, 0.00725957, 0.192516745,
            0.001144255, 0.001365843, 0.000120687, 0.00028856, 0.001095371,
            0.000648439, 0.000436689, 0.000439169, 0.000398534, 0.001287995,
            0.003431772, 0.00019162, 0.000177943, 0.000451412, 0.000561101,
            0.000551029, 0.00022863, 0.000218574, 0.000215384, 0.000147444,
            0.000166611, 0.000715269, 7.04103e-05, 2.16715e-05, 0.000160175,
            0.00092219, 0.000151426, 0.000169642, 0.000727576, 1.19889e-05
        ], [0.001627333, 0.00205764, 0.001540806, 0.001571586, 0.002224155,
            0.192609041, 0.010919704, 0.000444378, 0.00158412, 0.002700988,
            0.006705705, 0.002618593, 0.002862238, 0.001855208, 0.003046499,
            0.071301105, 0.000795723, 0.034417352, 0.000783804, 0.00161832,
            0.001487162, 0.002255255, 0.001311397, 0.000528664, 0.000385784,
            0.004200154, 0.000761861, 0.000603439, 0.002170557, 0.000915428,
            0.001407477, 0.001169798, 0.001016073, 0.002793157, 0.000258529
        ], [0.002838087, 0.003396032, 0.024574465, 0.010577829, 0.01280794,
            0.012407544, 0.236306211, 0.001706744, 0.019419192, 0.018586357,
            0.02285275, 0.005956292, 0.010096435, 0.015177124, 0.006093516,
            0.021515879, 0.003577087, 0.005355286, 0.007873129, 0.014750322,
            0.01437704, 0.010696779, 0.00605459, 0.004151066, 0.007162355,
            0.016119069, 0.012007772, 0.015756354, 0.0027113, 0.022619183,
            0.016547894, 0.016716418, 0.010063716, 0.025448184, 0.000594557
        ], [0.01244241, 0.013149222, 0.003560142, 0.004637733, 0.003136847,
            0.004619719, 0.004457872, 0.044649796, 0.02702986, 0.008805529,
            0.016214626, 0.009012176, 0.003704907, 0.002940384, 0.001987779,
            0.004287006, 0.019851968, 0.012765196, 0.004378675, 0.005936027,
            0.005073976, 0.003729197, 0.050632972, 0.050735036, 0.055367906,
            0.013851237, 0.00354921, 0.001252782, 0.001213375, 0.004464067,
            0.007579684, 0.002972122, 0.003537552, 0.006629857, 6.7949e-05
        ], [0.04076455, 0.014035824, 0.010941392, 0.070927562, 0.033314252,
            0.0259778, 0.034081478, 0.014377292, 0.232855006, 0.20156022,
            0.03022867, 0.015575275, 0.01277011, 0.019134101, 0.012837534,
            0.032099737, 0.005876455, 0.010093958, 0.006929657, 0.003429806,
            0.00252187, 0.004386534, 0.003251196, 0.002620751, 0.002154963,
            0.003478068, 0.001472106, 0.000906522, 0.001544161, 0.004309803,
            0.008234431, 0.003759918, 0.06395785, 0.009267209, 0.000120306
        ], [0.004535885, 0.009081804, 0.014974628, 0.008371675, 0.034825962,
            0.009533684, 0.014009274, 0.001899495, 0.018186269, 0.110750319,
            0.01084037, 0.007539671, 0.026360725, 0.025993118, 0.036109617,
            0.038337318, 0.002287962, 0.018123184, 0.018976729, 0.004604474,
            0.005157652, 0.004175149, 0.011206529, 0.002245139, 0.002443335,
            0.006133873, 0.003775129, 0.000741477, 0.000987898, 0.002514711,
            0.003045665, 0.000914915, 0.005555565, 0.003587204, 8.36239e-05
        ], [0.001894733, 0.005025044, 0.005549772, 0.002005759, 0.001720439,
            0.009155057, 0.001413417, 0.002161185, 0.006470184, 0.006102992,
            0.109164138, 0.00964538, 0.005608354, 0.011773615, 0.007125075,
            0.006699374, 0.001899667, 0.080091904, 0.002923031, 0.001479653,
            0.001011819, 0.003627024, 0.001323619, 0.00050546, 0.000481673,
            0.001338526, 0.001124919, 0.00017339, 0.002232675, 0.001003708,
            0.001512774, 0.001694997, 0.002042592, 0.001856075, 6.83141e-05
        ], [0.004987218, 0.026546575, 0.015063825, 0.005737678, 0.010202629,
            0.026921353, 0.009161499, 0.006249603, 0.017486699, 0.023621302,
            0.031861546, 0.317750453, 0.1614944, 0.075218629, 0.100445259,
            0.111083954, 0.007567227, 0.084790318, 0.019729037, 0.004785278,
            0.003944033, 0.004447344, 0.007770311, 0.010434269, 0.00775112,
            0.006544628, 0.004046097, 0.000801088, 0.002532247, 0.003342702,
            0.005238668, 0.002022584, 0.002752956, 0.00460606, 0.000277736
        ], [0.007245221, 0.019524401, 0.003838685, 0.006380607, 0.004843375,
            0.008619064, 0.007044935, 0.004768416, 0.007593853, 0.01035345,
            0.012985006, 0.016108925, 0.118031183, 0.014966191, 0.028692186,
            0.008895094, 0.007682139, 0.015899787, 0.022930146, 0.00395428,
            0.003043654, 0.00216301, 0.00646984, 0.005914392, 0.004144648,
            0.004744429, 0.003228868, 0.001188217, 0.001433416, 0.003020028,
            0.00652847, 0.002462522, 0.002434378, 0.003673577, 0.000153993
        ], [0.001624521, 0.007541803, 0.003275378, 0.004795688, 0.003534703,
            0.006509492, 0.009411145, 0.003076498, 0.008897523, 0.012197487,
            0.008340076, 0.0154988, 0.073373935, 0.255319734, 0.050641062,
            0.019680936, 0.012659562, 0.026380252, 0.024824281, 0.009016472,
            0.00772095, 0.004034077, 0.006138558, 0.003157992, 0.003615904,
            0.006433764, 0.029225892, 0.003469164, 0.001605106, 0.013030849,
            0.012094828, 0.005997799, 0.018637749, 0.007832027, 0.000563808
        ], [0.00410204, 0.005685969, 0.001781085, 0.002564258, 0.002246405,
            0.002932554, 0.001908562, 0.001001334, 0.002079093, 0.00330905,
            0.003469826, 0.004907434, 0.013748013, 0.004934165, 0.292086383,
            0.004519906, 0.001567387, 0.004124969, 0.085582923, 0.00506541,
            0.004265398, 0.001240564, 0.030257955, 0.040978322, 0.045861866,
            0.01030899, 0.003038009, 0.000834248, 0.000506621, 0.002975456,
            0.018382085, 0.001002617, 0.001431781, 0.003274886, 0.000108549
        ], [0.00087132, 0.001374437, 0.001146736, 0.005600694, 0.003318185,
            0.006306254, 0.003235269, 0.000600324, 0.001911196, 0.002536373,
            0.003464874, 0.00708407, 0.003853859, 0.002976786, 0.005140121,
            0.035130306, 0.001062125, 0.005211929, 0.002254064, 0.001546394,
            0.001747089, 0.002784495, 0.001482724, 0.00104417, 0.001046911,
            0.001347194, 0.000934863, 0.001208574, 0.001145707, 0.002453572,
            0.004021484, 0.003529101, 0.003033489, 0.005775989, 0.00310446
        ], [0.009081911, 0.022670392, 0.011478242, 0.017089786, 0.007334252,
            0.01534235, 0.020543639, 0.013318101, 0.027812875, 0.020993118,
            0.034978898, 0.026428447, 0.010666821, 0.00961521, 0.008915457,
            0.010478783, 0.095282856, 0.005058031, 0.01010572, 0.006155833,
            0.013097701, 0.022755283, 0.014858419, 0.003903552, 0.003902925,
            0.012160405, 0.008326691, 0.004490955, 0.008137596, 0.006163745,
            0.014610227, 0.018664248, 0.012785158, 0.015524624, 0.000202115
        ], [0.003522435, 0.008076591, 0.002493561, 0.002580586, 0.002004588,
            0.004263754, 0.00390482, 0.003026737, 0.004213893, 0.003552174,
            0.00711151, 0.004786255, 0.003328099, 0.003215595, 0.002377941,
            0.003207739, 0.025819233, 0.036313307, 0.004288705, 0.003948899,
            0.004708541, 0.005168582, 0.009545882, 0.003223335, 0.00223004,
            0.014653895, 0.011819248, 0.00524021, 0.04270701, 0.005441151,
            0.019319997, 0.013172837, 0.00664315, 0.012191179, 0.002272507
        ], [0.004999214, 0.004314665, 0.006290482, 0.005361151, 0.007526875,
            0.006009924, 0.004442567, 0.003208364, 0.006064364, 0.006050055,
            0.007328912, 0.006000279, 0.005246263, 0.004886112, 0.004684286,
            0.006072456, 0.009050393, 0.007751687, 0.008704785, 0.005041349,
            0.00475228, 0.005282938, 0.01830954, 0.003365995, 0.003095348,
            0.007273249, 0.004702153, 0.002445483, 0.001938898, 0.006049782,
            0.003957759, 0.003168133, 0.003344806, 0.006572041, 4.10344e-05
        ], [0.033274109, 0.025437731, 0.06083138, 0.052412422, 0.07063672,
            0.054182547, 0.048007081, 0.037857409, 0.048440358, 0.048659518,
            0.042084866, 0.041921086, 0.046935042, 0.050923297, 0.042809042,
            0.050211865, 0.022335918, 0.042745532, 0.03849672, 0.03174035,
            0.013770085, 0.044347796, 0.022478296, 0.017795362, 0.017897362,
            0.019237705, 0.00949573, 0.004901321, 0.00338654, 0.012210227,
            0.020413977, 0.010137603, 0.027901013, 0.014408255, 0.000277706
        ], [0.016866456, 0.010363615, 0.029730564, 0.026418014, 0.039812136,
            0.020983239, 0.017683634, 0.012941699, 0.022572319, 0.021459859,
            0.018936085, 0.015885551, 0.016805312, 0.015968788, 0.018183019,
            0.025595169, 0.008175453, 0.02774605, 0.017307877, 0.008626256,
            0.008406207, 0.026550852, 0.015287208, 0.008963839, 0.008172032,
            0.009182348, 0.010189648, 0.003987212, 0.003309402, 0.009465008,
            0.007833328, 0.00579732, 0.011636146, 0.012499261, 0.00054259
        ], [0.001393313, 0.003668845, 0.004107609, 0.004443766, 0.004055101,
            0.005295087, 0.005667479, 0.001818609, 0.005202447, 0.005778724,
            0.006172649, 0.005146335, 0.006480891, 0.006734341, 0.003392431,
            0.004549126, 0.004802814, 0.00532428, 0.006680327, 0.009499701,
            0.006312435, 0.01017269, 0.007916998, 0.006566557, 0.022888783,
            0.021706419, 0.006753897, 0.010056031, 0.00244553, 0.013811902,
            0.009623849, 0.007192433, 0.008105535, 0.011381746, 8.68578e-05
        ], [0.014278562, 0.026867795, 0.021946654, 0.018822604, 0.020670428,
            0.024645721, 0.018275756, 0.033057328, 0.019756661, 0.01722616,
            0.035024281, 0.018104643, 0.012171856, 0.009749733, 0.011806681,
            0.018251597, 0.025850311, 0.017263857, 0.013871964, 0.018282187,
            0.010509661, 0.009953625, 0.041359346, 0.018809557, 0.010495543,
            0.030631601, 0.010356497, 0.006308048, 0.001776255, 0.006802003,
            0.01147471, 0.010367814, 0.007684637, 0.009794841, 0.000628328
        ], [0.002883333, 0.004189358, 0.003979834, 0.003057905, 0.003934542,
            0.007419927, 0.001713474, 0.004444114, 0.003136824, 0.00224653,
            0.005909739, 0.004022565, 0.001599061, 0.001507568, 0.001652038,
            0.002293199, 0.002184121, 0.002649169, 0.001197928, 0.002145616,
            0.001271663, 0.001730743, 0.0039945, 0.099135037, 0.003089098,
            0.006081809, 0.001138594, 0.000684809, 0.000137419, 0.001071062,
            0.001481332, 0.001904858, 0.000618081, 0.001039925, 5.72517e-05
        ], [0.000827348, 0.002377983, 0.001600487, 0.001790211, 0.002842036,
            0.001386863, 0.00220556, 0.000876691, 0.002058567, 0.001911665,
            0.001499468, 0.001485093, 0.001949874, 0.002264823, 0.00177818,
            0.001651479, 0.000851503, 0.001254234, 0.002175293, 0.004139558,
            0.001730284, 0.001251201, 0.002128905, 0.004231082, 0.033264232,
            0.020289123, 0.005642899, 0.003480932, 0.00052436, 0.004609185,
            0.004776945, 0.002203612, 0.001350245, 0.004166, 1.86254e-05
        ], [0.003326416, 0.006454247, 0.006221039, 0.003887793, 0.004678195,
            0.006761236, 0.007003225, 0.00638065, 0.005359105, 0.005077236,
            0.00915084, 0.0054031, 0.004541723, 0.003988224, 0.00412748,
            0.004691563, 0.004862287, 0.003303466, 0.00769254, 0.019023196,
            0.009816826, 0.008199347, 0.038802566, 0.145834168, 0.096861729,
            0.114858205, 0.003534392, 0.003824913, 0.000700313, 0.004921654,
            0.003814497, 0.002490025, 0.003039649, 0.005548926, 5.90197e-05
        ], [0.001987285, 0.005530397, 0.003545457, 0.003993172, 0.004071984,
            0.004018786, 0.00996192, 0.002290319, 0.00587807, 0.005061083,
            0.004813647, 0.004856203, 0.006219177, 0.006102185, 0.003413534,
            0.006233362, 0.005033002, 0.00760662, 0.011328117, 0.017865638,
            0.017775498, 0.011042391, 0.014058638, 0.020099096, 0.028673402,
            0.021670755, 0.088687939, 0.022680563, 0.003693534, 0.020539613,
            0.016996708, 0.00811023, 0.010152895, 0.021007572, 0.000122858
        ], [0.018406954, 0.032599027, 0.015680715, 0.021906494, 0.017901536,
            0.022088079, 0.021089324, 0.013449048, 0.021183596, 0.01852471,
            0.02300697, 0.021175464, 0.01972351, 0.017521746, 0.014241936,
            0.021525524, 0.023097485, 0.02036015, 0.026317889, 0.041569572,
            0.034394057, 0.023251401, 0.041686042, 0.039651655, 0.038262683,
            0.025837524, 0.02295942, 0.173416046, 0.050208355, 0.036231189,
            0.023216783, 0.013666533, 0.021762876, 0.032639848, 0.003123107
        ], [0.006686241, 0.005457159, 0.004030942, 0.005626681, 0.005620819,
            0.00513275, 0.008233864, 0.002364425, 0.004722252, 0.006057503,
            0.005572079, 0.005247938, 0.006222115, 0.005338876, 0.003931709,
            0.00850786, 0.005912998, 0.012161981, 0.024304526, 0.028650961,
            0.041987186, 0.027777052, 0.010571172, 0.026519595, 0.016025324,
            0.021236661, 0.012711779, 0.023855012, 0.029480607, 0.023604575,
            0.010910272, 0.017507472, 0.029537078, 0.028324816, 0.000534995
        ], [0.012445677, 0.031603851, 0.036338496, 0.027974801, 0.025554967,
            0.022390712, 0.056988383, 0.015142399, 0.059664451, 0.034056444,
            0.031632838, 0.027880791, 0.042974998, 0.059493691, 0.037081434,
            0.036528935, 0.036938806, 0.054802151, 0.044494748, 0.056207701,
            0.062690773, 0.033720448, 0.040031105, 0.051890466, 0.093765158,
            0.066087373, 0.068654391, 0.086551613, 0.033273918, 0.132068361,
            0.055921887, 0.026620288, 0.050590639, 0.066227706, 0.000183157
        ], [0.001686115, 0.003374494, 0.001798045, 0.001256502, 0.000886649,
            0.003119748, 0.003033108, 0.001086152, 0.003083207, 0.002136673,
            0.002248369, 0.002184622, 0.0012899, 0.001570612, 0.001206398,
            0.002380187, 0.005708817, 0.00141947, 0.002184772, 0.001725663,
            0.003472243, 0.00354475, 0.003141373, 0.009411979, 0.010320488,
            0.003511018, 0.00277585, 0.001600838, 0.003374508, 0.002778785,
            0.007556404, 0.0038514, 0.005132847, 0.007587826, 2.45976e-06
        ], [0.000837405, 0.000521545, 0.000334798, 0.000359483, 0.000378462,
            0.000261623, 0.000596198, 0.000439618, 0.000791672, 0.000472913,
            0.000512544, 0.000475034, 0.000787729, 0.000956195, 0.000460961,
            0.000304941, 0.001043432, 0.000455238, 0.000453062, 0.000877146,
            0.001812122, 0.000399979, 0.001086015, 0.000350085, 0.001638162,
            0.001071646, 0.001721341, 0.001246139, 0.000173368, 0.0020601,
            0.004451031, 0.006589461, 0.001029746, 0.001391578, 7.12222e-07
        ], [0.002061076, 0.000413464, 0.000402877, 0.000441949, 0.000412586,
            0.000364425, 0.000442435, 0.000211542, 0.000444978, 0.000389162,
            0.00043821, 0.000369547, 0.00055914, 0.000453355, 0.000299884,
            0.000488012, 0.000434559, 0.000544137, 0.000587822, 0.000697499,
            0.00049138, 0.001038035, 0.000778636, 0.000593938, 0.000567302,
            0.000752563, 0.00075713, 0.000621843, 0.000110096, 0.000794063,
            0.00632777, 0.001499277, 0.022380758, 0.001676006, 8.80619e-08
        ], [0.002581933, 0.005951961, 0.006306084, 0.00528491, 0.006158931,
            0.005984194, 0.013368338, 0.003506261, 0.008385579, 0.006427541,
            0.006054525, 0.00721608, 0.006575539, 0.00654436, 0.004254584,
            0.006902248, 0.006359514, 0.0062187, 0.009622925, 0.00892464,
            0.010530982, 0.013329119, 0.009399214, 0.007334549, 0.008970976,
            0.011576019, 0.018476624, 0.010530984, 0.007737447, 0.028776904,
            0.015829437, 0.011552978, 0.014960421, 0.079210345, 0.000422377
        ], [1.12306e-05, 1.357e-05, 3.13485e-05, 4.09308e-05, 2.699e-05,
            3.45605e-05, 3.58591e-05, 1.16021e-05, 3.16394e-05, 3.45246e-05,
            2.3004e-05, 3.24389e-05, 2.75784e-05, 2.02194e-05, 3.70828e-05,
            3.63046e-05, 1.58638e-05, 8.57338e-05, 0.000126296, 3.62699e-05,
            5.40855e-05, 6.78024e-05, 0.000157967, 2.62135e-05, 3.57885e-05,
            3.18437e-05, 8.5624e-05, 6.33893e-05, 0.000268572, 7.47766e-05,
            8.93421e-05, 5.27179e-05, 0.000132325, 0.000119849, 2.1472e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_1997():
    """
    IA matrix 1997



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_1997())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_1998():
    """
    IA matrix 1998



    constant


    """

    ret =  xr.DataArray(
        data=[[0.1299642, 0.002804855, 0.267820482, 0.039799397, 0.030843153,
            0.135661811, 0.011784677, 0.003481402, 0.007091672, 0.014106851,
            0.001043886, 0.000303774, 0.00031383, 0.000221246, 0.000181211,
            0.011167844, 0.00053267, 0.00480767, 0.000492935, 0.00236285,
            0.002495076, 0.038593368, 0.000725217, 0.001927537, 0.000466472,
            0.002139816, 0.000215559, 0.000180183, 0.000536462, 0.000978037,
            0.001876873, 0.002030631, 0.002400363, 0.002396953, 0.015968863
        ], [0.003201955, 0.082756959, 0.003023835, 0.001926037, 0.001159831,
            0.003509954, 0.004949395, 0.408844307, 0.037420344, 0.003585594,
            0.076021532, 0.041252505, 0.00311976, 0.001795111, 0.001436004,
            0.006305932, 0.094836026, 0.019218167, 0.000568977, 0.00128875,
            0.000882285, 0.001992548, 0.005291007, 0.003446641, 0.00249122,
            0.002961295, 0.000482434, 0.00034738, 0.00225567, 0.000899979,
            0.002405409, 0.003282405, 0.001277066, 0.002454793, 0.067266147
        ], [0.053945559, 0.000780719, 0.140266603, 0.002283975, 0.083867797,
            0.001240097, 0.002730816, 0.00116458, 0.009708145, 0.002454855,
            0.00188054, 0.000962687, 0.001185194, 0.001108984, 0.00082807,
            0.001579964, 0.000786752, 0.001051018, 0.001683322, 0.004611152,
            0.004259451, 0.157726913, 0.001640345, 0.007861478, 0.004674062,
            0.002416126, 0.001425036, 0.000841402, 0.000240109, 0.001940329,
            0.007981599, 0.009073252, 0.014269226, 0.005840659, 0.001322036
        ], [0.002364778, 0.001626461, 0.001484416, 0.275626109, 0.054897349,
            0.005470347, 0.007047931, 0.000635456, 0.004163119, 0.019222342,
            0.00510942, 0.001605478, 0.00247594, 0.002051176, 0.006177075,
            0.026137879, 0.000598463, 0.002603699, 0.001757203, 0.00284516,
            0.003474486, 0.002910035, 0.001690845, 0.002772505, 0.001180663,
            0.001767312, 0.003123462, 0.000652776, 0.000286592, 0.001252896,
            0.003593158, 0.000912808, 0.003600436, 0.005037282, 0.000532936
        ], [0.000226326, 0.000269802, 0.000212412, 0.006609095, 0.190282435,
            0.000968683, 0.001313351, 0.000127019, 0.000285108, 0.001170884,
            0.000655225, 0.000423792, 0.000448146, 0.000372801, 0.001303479,
            0.003367539, 0.000181817, 0.000181278, 0.000442559, 0.000507054,
            0.000529929, 0.00021716, 0.000202074, 0.000195974, 0.000141969,
            0.00016946, 0.000681859, 6.66719e-05, 1.7912e-05, 0.000146136,
            0.0009385, 0.000150543, 0.000175482, 0.000705979, 1.46947e-05
        ], [0.001691278, 0.002149282, 0.001589249, 0.001486382, 0.002014724,
            0.196335919, 0.010755123, 0.000431923, 0.001516646, 0.002752336,
            0.006416038, 0.002703648, 0.002958343, 0.00180006, 0.002879603,
            0.073616692, 0.00081001, 0.03311746, 0.000790533, 0.001638322,
            0.001556927, 0.002149602, 0.001312851, 0.000536127, 0.000401538,
            0.003944241, 0.000734393, 0.000552113, 0.002342862, 0.000893368,
            0.001426014, 0.00153421, 0.001026749, 0.002673, 0.000316876
        ], [0.002852114, 0.003319801, 0.024340169, 0.010439175, 0.012900988,
            0.012538151, 0.233060427, 0.002085919, 0.019116589, 0.018659144,
            0.022974273, 0.006101402, 0.009841157, 0.014602322, 0.006116385,
            0.02279696, 0.003618252, 0.005322291, 0.007891466, 0.014571169,
            0.0141591, 0.010211865, 0.006250729, 0.003894828, 0.007127261,
            0.015800665, 0.012176193, 0.015358073, 0.002852131, 0.02227272,
            0.017035994, 0.017291088, 0.010153875, 0.025819484, 0.000728743
        ], [0.011861662, 0.013558624, 0.003598825, 0.004237167, 0.00299286,
            0.004568395, 0.004364292, 0.046429292, 0.026103148, 0.007969418,
            0.01566838, 0.00915013, 0.003605183, 0.002744819, 0.001932136,
            0.004287694, 0.018666843, 0.012973967, 0.004338122, 0.005732043,
            0.004850306, 0.0034919, 0.05074551, 0.053976846, 0.054147507,
            0.014539026, 0.003562089, 0.001271854, 0.001434467, 0.004397017,
            0.007545161, 0.002917094, 0.003334198, 0.006371888, 8.32844e-05
        ], [0.040480086, 0.013988896, 0.011109749, 0.06989172, 0.034040612,
            0.026885318, 0.03450225, 0.015756109, 0.228564438, 0.199015311,
            0.030525524, 0.01575755, 0.012816435, 0.018672579, 0.013089615,
            0.03370671, 0.006256753, 0.010582359, 0.006987183, 0.003563942,
            0.002602077, 0.004406673, 0.003415609, 0.00280186, 0.002311384,
            0.003779357, 0.001495084, 0.000954481, 0.001805514, 0.004461218,
            0.008578153, 0.00368278, 0.064307353, 0.009427967, 0.000147458
        ], [0.004679359, 0.008991199, 0.015223745, 0.008568781, 0.033845041,
            0.009730358, 0.014457338, 0.002134146, 0.018497946, 0.112324166,
            0.01118561, 0.00770515, 0.026642186, 0.026184421, 0.035878184,
            0.041267397, 0.002378119, 0.018287715, 0.018149361, 0.004628435,
            0.005236266, 0.004174329, 0.011229933, 0.002701384, 0.002599334,
            0.006350341, 0.003775668, 0.000728513, 0.001132509, 0.002542562,
            0.003138204, 0.000924553, 0.005831525, 0.00340814, 0.000102497
        ], [0.00192553, 0.005132844, 0.005403305, 0.001972581, 0.001678155,
            0.008977378, 0.001374487, 0.002092161, 0.00637819, 0.006069414,
            0.109690724, 0.00980534, 0.005624362, 0.011363228, 0.006866145,
            0.006952334, 0.001984199, 0.078074119, 0.002878311, 0.001443852,
            0.001013216, 0.003555375, 0.0012834, 0.000574259, 0.000516091,
            0.001385454, 0.001087272, 0.000176631, 0.002299052, 0.000958448,
            0.001466184, 0.001635488, 0.002034832, 0.001803794, 8.37319e-05
        ], [0.004814641, 0.025156514, 0.014837982, 0.005617969, 0.009700872,
            0.026866734, 0.009263716, 0.006672975, 0.017580003, 0.023900599,
            0.032233052, 0.31442599, 0.159608119, 0.072836617, 0.100655544,
            0.118647079, 0.00785064, 0.085121908, 0.019392239, 0.004654537,
            0.003973162, 0.004412057, 0.007712548, 0.009144893, 0.007968593,
            0.006643226, 0.004125657, 0.000788282, 0.002933552, 0.003300922,
            0.005252349, 0.001908137, 0.002725887, 0.004474591, 0.000340419
        ], [0.007019778, 0.018800052, 0.003744205, 0.006266292, 0.004775775,
            0.008481583, 0.006860367, 0.004676099, 0.007477688, 0.010071357,
            0.012843856, 0.016006902, 0.112517577, 0.014377997, 0.027951035,
            0.009431845, 0.008047205, 0.015831558, 0.021962933, 0.004016728,
            0.003094056, 0.002088713, 0.006630286, 0.00570243, 0.004071114,
            0.004915082, 0.003081131, 0.001263958, 0.001623854, 0.002866813,
            0.006417023, 0.002349612, 0.002476192, 0.003455461, 0.000188747
        ], [0.001645612, 0.007751086, 0.003391781, 0.00488126, 0.003641823,
            0.00712488, 0.00949439, 0.003414719, 0.009395332, 0.012805782,
            0.008776859, 0.016064279, 0.074276367, 0.255953404, 0.050897917,
            0.021563467, 0.012873267, 0.027448286, 0.024128029, 0.009013368,
            0.00810441, 0.004116885, 0.006319876, 0.003415003, 0.003790219,
            0.00667178, 0.031504144, 0.003554979, 0.001819348, 0.013365909,
            0.012654495, 0.006048213, 0.019621682, 0.008062308, 0.000691054
        ], [0.004106505, 0.006341971, 0.001884375, 0.002684828, 0.002346626,
            0.003274587, 0.002093531, 0.001039615, 0.002095928, 0.003376296,
            0.003495854, 0.005236786, 0.013669909, 0.005093164, 0.282462617,
            0.004908932, 0.001776241, 0.004477804, 0.086666917, 0.005230151,
            0.004181105, 0.001396871, 0.031956189, 0.037112735, 0.046472379,
            0.012697813, 0.003437436, 0.000936111, 0.000921986, 0.003290989,
            0.017927172, 0.001133408, 0.001540408, 0.003501512, 0.000133047
        ], [0.000717457, 0.001349366, 0.001124023, 0.005401247, 0.003075696,
            0.006401692, 0.003375367, 0.000575076, 0.001896573, 0.002528754,
            0.003460181, 0.007542789, 0.003882614, 0.002878118, 0.005169254,
            0.037936883, 0.001061449, 0.005211098, 0.002354643, 0.001445557,
            0.001731891, 0.002628993, 0.001567281, 0.000917782, 0.001004072,
            0.001420088, 0.00091274, 0.001198153, 0.001436634, 0.002436826,
            0.003996186, 0.003480916, 0.003191566, 0.005634699, 0.003805105
        ], [0.008986002, 0.021608463, 0.011429073, 0.017070929, 0.007326413,
            0.015671104, 0.0204239, 0.014007392, 0.028031147, 0.020980006,
            0.035227004, 0.026928819, 0.01099356, 0.009350879, 0.008593652,
            0.010988203, 0.098424406, 0.005053512, 0.010013889, 0.006076087,
            0.013232452, 0.02238681, 0.01481631, 0.003606868, 0.003902109,
            0.012077517, 0.008106637, 0.004533004, 0.00933474, 0.006127708,
            0.015012196, 0.018517444, 0.012743084, 0.015213833, 0.000247731
        ], [0.00343829, 0.007141151, 0.002478985, 0.00249837, 0.001973516,
            0.004166198, 0.003752261, 0.002908727, 0.004123667, 0.003473593,
            0.006971659, 0.004773349, 0.003279453, 0.003082369, 0.002305134,
            0.003276412, 0.025936663, 0.036342844, 0.00423831, 0.003822657,
            0.004635304, 0.005187773, 0.009887249, 0.002988445, 0.002306958,
            0.014976557, 0.011832172, 0.005343001, 0.043783265, 0.005320877,
            0.01944321, 0.013908146, 0.00670216, 0.011719273, 0.002785389
        ], [0.004900998, 0.004307471, 0.006399251, 0.005608109, 0.00776566,
            0.005921491, 0.00461606, 0.003723017, 0.006149924, 0.006005636,
            0.007435852, 0.006017545, 0.005340648, 0.004720388, 0.004706182,
            0.006355302, 0.008901061, 0.007492413, 0.00888375, 0.004805877,
            0.004580912, 0.005211366, 0.018285127, 0.003255221, 0.003375898,
            0.007383064, 0.004685574, 0.002463544, 0.002142891, 0.005953411,
            0.004064743, 0.003193238, 0.003394498, 0.006418447, 5.02955e-05
        ], [0.033766808, 0.026668891, 0.061314818, 0.054844562, 0.071254826,
            0.054829951, 0.050022259, 0.039127233, 0.049674804, 0.049048759,
            0.043474094, 0.043797577, 0.047105215, 0.049990448, 0.043523676,
            0.054160549, 0.022567489, 0.042705813, 0.037692937, 0.030812764,
            0.013960754, 0.044362981, 0.023185256, 0.019099299, 0.020016042,
            0.020581673, 0.010254926, 0.004914923, 0.004092284, 0.012686319,
            0.021229762, 0.010326216, 0.028915814, 0.015010834, 0.000340382
        ], [0.016625475, 0.010600615, 0.02960988, 0.027290798, 0.040154723,
            0.021311575, 0.018166249, 0.014527106, 0.022301445, 0.021322466,
            0.019455418, 0.016375654, 0.017111605, 0.015793118, 0.018744023,
            0.027064019, 0.008377069, 0.02871964, 0.017276623, 0.008465406,
            0.008425797, 0.025796807, 0.015622377, 0.009095491, 0.009111555,
            0.009789275, 0.010413508, 0.004000889, 0.00365569, 0.009354347,
            0.007989901, 0.005849618, 0.011920946, 0.01248297, 0.000665047
        ], [0.00136628, 0.003890129, 0.00401894, 0.004433961, 0.004078326,
            0.004836694, 0.005730589, 0.001845298, 0.005139008, 0.005646684,
            0.005984413, 0.005079814, 0.006468767, 0.006389675, 0.003162416,
            0.004731216, 0.004960001, 0.005097605, 0.006428552, 0.009421452,
            0.006358219, 0.010276976, 0.008100681, 0.005892554, 0.023136761,
            0.022437641, 0.007216054, 0.010275618, 0.002778201, 0.013808084,
            0.010082382, 0.007542413, 0.008526324, 0.011531445, 0.000106461
        ], [0.0141143, 0.026584943, 0.022092062, 0.019609881, 0.021015049,
            0.024845087, 0.018394117, 0.03092132, 0.019550125, 0.017704736,
            0.033908696, 0.018309214, 0.012472325, 0.009419447, 0.01179078,
            0.020086075, 0.027062816, 0.017339328, 0.01441384, 0.01801726,
            0.01073407, 0.009879213, 0.043937237, 0.0313193, 0.012133258,
            0.03226129, 0.010379195, 0.006278793, 0.0021543, 0.006649145,
            0.011766732, 0.010648033, 0.007958936, 0.00972868, 0.000770135
        ], [0.003032121, 0.004905033, 0.004055243, 0.003910839, 0.00508746,
            0.007741398, 0.00245684, 0.005982009, 0.003183714, 0.002701202,
            0.005846614, 0.005261714, 0.001994133, 0.00171764, 0.001850297,
            0.002226551, 0.002624781, 0.00313362, 0.001241362, 0.00236542,
            0.001380916, 0.001678118, 0.003937502, 0.117105862, 0.003262737,
            0.005062485, 0.001090347, 0.000589197, 0.00019972, 0.000952526,
            0.001294617, 0.001472462, 0.000643155, 0.001129217, 7.01728e-05
        ], [0.00079111, 0.002192545, 0.001587017, 0.001807129, 0.002747396,
            0.001455693, 0.002149955, 0.000864679, 0.002009765, 0.001955237,
            0.001499301, 0.001490366, 0.001890553, 0.002126256, 0.001842143,
            0.001796502, 0.000865702, 0.001215176, 0.002211901, 0.003840422,
            0.001690828, 0.001218822, 0.002165843, 0.003757161, 0.034751543,
            0.020261022, 0.005304338, 0.003422258, 0.000595722, 0.004449607,
            0.004652291, 0.00223972, 0.001365929, 0.004129967, 2.2829e-05
        ], [0.00349483, 0.006590472, 0.00629611, 0.004228593, 0.004933253,
            0.006787764, 0.00707649, 0.00715555, 0.005280985, 0.005035066,
            0.008503691, 0.005238443, 0.00453043, 0.003813695, 0.004375781,
            0.004934576, 0.004821624, 0.003626529, 0.008076051, 0.018469164,
            0.010424402, 0.008253714, 0.038243859, 0.130828902, 0.098778579,
            0.118460708, 0.003590375, 0.004010771, 0.000825356, 0.005022458,
            0.004294475, 0.002591938, 0.003240342, 0.005653393, 7.23399e-05
        ], [0.002065095, 0.005655112, 0.003677656, 0.004189877, 0.004423665,
            0.004382725, 0.010411482, 0.00286913, 0.006167171, 0.005240273,
            0.005024633, 0.005138874, 0.006532945, 0.006275775, 0.003544833,
            0.007036077, 0.00538801, 0.008623587, 0.011575557, 0.018121975,
            0.018534498, 0.011226504, 0.014625376, 0.017985289, 0.029783804,
            0.022790111, 0.098592366, 0.024017228, 0.004072194, 0.02091191,
            0.017773896, 0.008530833, 0.010570785, 0.021570449, 0.000150585
        ], [0.018664974, 0.031264259, 0.015958882, 0.02211903, 0.017349108,
            0.020786449, 0.021366716, 0.014031802, 0.020558102, 0.018571757,
            0.022955544, 0.020561485, 0.020026377, 0.016849073, 0.014135529,
            0.022071444, 0.023607666, 0.02034212, 0.026675236, 0.040187171,
            0.035353172, 0.023770488, 0.043750763, 0.03930639, 0.040091146,
            0.026537479, 0.023295876, 0.181591874, 0.054759001, 0.036558903,
            0.02444581, 0.014064538, 0.022635416, 0.033079775, 0.003827961
        ], [0.006651146, 0.005359352, 0.004027504, 0.005540728, 0.005557145,
            0.005320393, 0.008191602, 0.003373304, 0.004677726, 0.005923463,
            0.005363063, 0.005282835, 0.006229759, 0.005149701, 0.004085059,
            0.008709417, 0.006023288, 0.01199449, 0.024422923, 0.02718411,
            0.042301436, 0.026680548, 0.010682719, 0.02213804, 0.015980158,
            0.021316481, 0.012847677, 0.023765228, 0.030990705, 0.023189384,
            0.01211294, 0.017836663, 0.030567088, 0.028403724, 0.000655738
        ], [0.011948467, 0.031434385, 0.03739083, 0.027801674, 0.024726126,
            0.024020162, 0.058452463, 0.017984434, 0.062204618, 0.035062471,
            0.032734556, 0.029162407, 0.043475857, 0.0594115, 0.038082399,
            0.039760778, 0.039068001, 0.055710949, 0.046409551, 0.055469492,
            0.064891542, 0.034918342, 0.041652566, 0.046205885, 0.09968995,
            0.068490323, 0.072343932, 0.09093698, 0.036535689, 0.135628172,
            0.060090354, 0.027963799, 0.054293631, 0.067979011, 0.000224493
        ], [0.001661477, 0.003225474, 0.001760607, 0.001144862, 0.000822183,
            0.003247578, 0.003026699, 0.001134418, 0.003079891, 0.002112096,
            0.002247238, 0.002456452, 0.001236789, 0.001462977, 0.00118826,
            0.002281431, 0.00608338, 0.001383944, 0.002179857, 0.001608627,
            0.003439525, 0.003453042, 0.003038069, 0.007854674, 0.009952492,
            0.00346351, 0.002828672, 0.001600068, 0.003308115, 0.002672866,
            0.007713981, 0.003843752, 0.005136953, 0.007495475, 3.0149e-06
        ], [0.000820751, 0.000532727, 0.00033998, 0.000358169, 0.000387841,
            0.000252363, 0.000596689, 0.000474332, 0.000788635, 0.000483537,
            0.000493862, 0.000467633, 0.000749497, 0.00094067, 0.0004717,
            0.00031621, 0.001097649, 0.00043932, 0.000488801, 0.000839621,
            0.001868357, 0.000414147, 0.001089087, 0.000326702, 0.001680031,
            0.001046009, 0.001565753, 0.001271178, 0.000178569, 0.002071902,
            0.004411678, 0.00642203, 0.001014187, 0.001366186, 8.72964e-07
        ], [0.001942051, 0.000429249, 0.000415162, 0.000437552, 0.000689506,
            0.000379451, 0.000436995, 0.000245603, 0.000449492, 0.000372917,
            0.000426141, 0.000379665, 0.000549105, 0.000435666, 0.000308307,
            0.000481052, 0.000424076, 0.000521606, 0.000572193, 0.000656224,
            0.000472679, 0.001024809, 0.000777496, 0.000468097, 0.000548396,
            0.000706767, 0.000679086, 0.000616829, 0.000105267, 0.000784117,
            0.00628859, 0.001465961, 0.022532962, 0.00162432, 1.07937e-07
        ], [0.0025481, 0.006360264, 0.006191316, 0.005249262, 0.006394617,
            0.005884263, 0.013752537, 0.004199323, 0.008497514, 0.006133368,
            0.006088919, 0.007343559, 0.006650614, 0.006070977, 0.004161207,
            0.00695932, 0.006587944, 0.00645767, 0.009689125, 0.00886167,
            0.010507227, 0.013125444, 0.009302436, 0.006557816, 0.009322574,
            0.01154705, 0.019442463, 0.010722838, 0.008163929, 0.028298134,
            0.015941843, 0.011323799, 0.015349652, 0.078683171, 0.000517703
        ], [1.27881e-05, 1.30555e-05, 3.35149e-05, 4.02804e-05, 2.82574e-05,
            4.02956e-05, 3.99982e-05, 1.18184e-05, 3.64898e-05, 4.02854e-05,
            2.55138e-05, 3.3757e-05, 3.18059e-05, 2.05234e-05, 3.75568e-05,
            4.47019e-05, 1.65868e-05, 0.000102034, 0.000133615, 3.5009e-05,
            5.74162e-05, 7.38739e-05, 0.00016912, 2.45854e-05, 3.62652e-05,
            3.53916e-05, 9.24426e-05, 7.23952e-05, 0.00029994, 8.41283e-05,
            0.000102276, 5.30281e-05, 0.000156425, 0.000131215, 2.6318e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_1998():
    """
    IA matrix 1998



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_1998())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_1999():
    """
    IA matrix 1999



    constant


    """

    ret =  xr.DataArray(
        data=[[0.13131576, 0.002422978, 0.266148851, 0.039706639, 0.032568493,
            0.145684008, 0.011879775, 0.003177235, 0.006468798, 0.013628408,
            0.000920189, 0.000294182, 0.000315697, 0.000207127, 0.000176158,
            0.010425039, 0.000449511, 0.006336493, 0.000485853, 0.002308301,
            0.002397182, 0.039736853, 0.001014055, 0.001726476, 0.000411803,
            0.003039494, 0.00024328, 0.000172042, 0.000524739, 0.000853588,
            0.002015882, 0.002114409, 0.002470741, 0.002499208, 0.016852826
        ], [0.003151141, 0.082474407, 0.002854371, 0.001992512, 0.001056922,
            0.003647769, 0.00483207, 0.43158197, 0.038782292, 0.003643518,
            0.073128155, 0.042690499, 0.003248186, 0.001530718, 0.001357677,
            0.005119018, 0.102404978, 0.019390701, 0.000497901, 0.001409056,
            0.000946596, 0.001629574, 0.004579222, 0.003555351, 0.002382606,
            0.002237031, 0.000461939, 0.000275993, 0.001792877, 0.000931225,
            0.002083802, 0.003827771, 0.001280763, 0.002135247, 0.070989699
        ], [0.052496142, 0.000765765, 0.140407773, 0.002350521, 0.082344181,
            0.001176139, 0.002574647, 0.000939005, 0.00963344, 0.002404445,
            0.001678376, 0.000917534, 0.001151476, 0.0009884, 0.000790248,
            0.001742233, 0.000741705, 0.00100416, 0.001602155, 0.004317486,
            0.004175175, 0.152825653, 0.001375524, 0.008526348, 0.004196389,
            0.002262066, 0.001427074, 0.000778865, 0.000244373, 0.001782684,
            0.007463952, 0.008678963, 0.013763415, 0.006171733, 0.001395218
        ], [0.002240573, 0.001511685, 0.00136223, 0.282105707, 0.055637578,
            0.004422546, 0.006868879, 0.00058343, 0.003868531, 0.016878723,
            0.004743302, 0.001500796, 0.002478095, 0.001889546, 0.006254817,
            0.022019348, 0.000544414, 0.002349946, 0.001651987, 0.002652549,
            0.003312372, 0.00278305, 0.001598742, 0.002380586, 0.001124309,
            0.001680342, 0.002732023, 0.000562164, 0.000273708, 0.001183355,
            0.003800907, 0.000939315, 0.003396957, 0.005478776, 0.000562437
        ], [0.000193652, 0.000201338, 0.000175272, 0.006932976, 0.189120471,
            0.000721434, 0.001250283, 8.83567e-05, 0.000248687, 0.001079023,
            0.000550777, 0.000338904, 0.000374978, 0.000293151, 0.001293671,
            0.003292928, 0.00015953, 0.000156532, 0.000420527, 0.000435189,
            0.000450351, 0.000187716, 0.000192788, 0.000185642, 0.00012995,
            0.000160444, 0.000573804, 5.53705e-05, 1.63002e-05, 0.000130733,
            0.000890536, 0.000141477, 0.000156025, 0.000642795, 1.55081e-05
        ], [0.001801437, 0.001974031, 0.001544593, 0.001330357, 0.001822094,
            0.196946659, 0.010764676, 0.000306143, 0.001459382, 0.002917515,
            0.005943131, 0.002704111, 0.002901966, 0.001635572, 0.002886607,
            0.070537413, 0.000815221, 0.032803897, 0.000720681, 0.001555564,
            0.001457956, 0.002128871, 0.001291496, 0.000510364, 0.000370179,
            0.003737864, 0.000786722, 0.000479013, 0.002251869, 0.000875686,
            0.00143104, 0.001809739, 0.000987688, 0.002729736, 0.000334417
        ], [0.002729565, 0.00302382, 0.024118618, 0.010808795, 0.013283147,
            0.011708403, 0.233894801, 0.002008949, 0.018596276, 0.017948586,
            0.022200372, 0.00585038, 0.009731894, 0.013791709, 0.005826223,
            0.020144277, 0.003911737, 0.00491548, 0.00777845, 0.014563766,
            0.013989259, 0.009610934, 0.006058, 0.003509662, 0.006652435,
            0.015101433, 0.01218922, 0.015043167, 0.002748101, 0.022372112,
            0.017166638, 0.018504204, 0.009921667, 0.025114867, 0.000769083
        ], [0.012371484, 0.012807208, 0.003576585, 0.004989592, 0.003549025,
            0.004502047, 0.00432534, 0.049234053, 0.026409005, 0.007610298,
            0.016105739, 0.008943101, 0.003551042, 0.002371305, 0.001870106,
            0.003932683, 0.019089047, 0.012645191, 0.004365997, 0.005804893,
            0.004774766, 0.003567283, 0.051810653, 0.061323142, 0.061649505,
            0.014807648, 0.003689611, 0.001278657, 0.001361884, 0.004489228,
            0.007817202, 0.002892583, 0.003195387, 0.006003905, 8.78946e-05
        ], [0.039345205, 0.012600573, 0.010674637, 0.070592943, 0.035429934,
            0.026377276, 0.034172807, 0.01442869, 0.227685575, 0.199714635,
            0.030490888, 0.015388657, 0.012974845, 0.017871988, 0.013212768,
            0.028578264, 0.006468231, 0.010241988, 0.006817072, 0.003507506,
            0.002596857, 0.004358491, 0.003290473, 0.003213826, 0.002611982,
            0.003766281, 0.001494363, 0.000965085, 0.001693054, 0.004637513,
            0.008496446, 0.00368394, 0.064482402, 0.009751292, 0.000155621
        ], [0.004696511, 0.008568353, 0.015902069, 0.009180297, 0.03655397,
            0.009015713, 0.014611494, 0.002078699, 0.018889754, 0.11506496,
            0.010744919, 0.007433717, 0.027270611, 0.024831733, 0.036476198,
            0.035583807, 0.002554477, 0.018051293, 0.018162192, 0.004772815,
            0.005352052, 0.004158337, 0.010595833, 0.002764626, 0.002801545,
            0.006164515, 0.003496706, 0.000703611, 0.001192181, 0.00261796,
            0.003261037, 0.000943673, 0.005812426, 0.003440856, 0.000108171
        ], [0.001993716, 0.004859017, 0.005661397, 0.002108459, 0.001806252,
            0.008552701, 0.001361031, 0.002201018, 0.006465772, 0.006213993,
            0.112241443, 0.009893374, 0.005942293, 0.010795697, 0.007022764,
            0.00640301, 0.002082493, 0.077337405, 0.002928238, 0.001375823,
            0.001046925, 0.003509472, 0.001305908, 0.0006048, 0.000534368,
            0.001409119, 0.001147686, 0.000183189, 0.002368646, 0.001019165,
            0.00143148, 0.001784455, 0.002075791, 0.001883763, 8.8367e-05
        ], [0.004615596, 0.024719109, 0.013961172, 0.005633374, 0.009968187,
            0.024902919, 0.008870446, 0.006528964, 0.017164329, 0.023810264,
            0.031237036, 0.308529701, 0.161601634, 0.067462708, 0.099836065,
            0.093984101, 0.007693571, 0.085401307, 0.019215387, 0.004413279,
            0.003854611, 0.004355447, 0.007409121, 0.008136299, 0.006875439,
            0.006400083, 0.004134878, 0.000767182, 0.002823487, 0.003484078,
            0.005154646, 0.001906868, 0.002616662, 0.004535119, 0.000359263
        ], [0.006568706, 0.016706603, 0.003781086, 0.006704226, 0.005061306,
            0.007758745, 0.006432132, 0.004778204, 0.007225077, 0.010412543,
            0.013476782, 0.015837436, 0.113401635, 0.013648291, 0.028901697,
            0.008592006, 0.007542953, 0.015928959, 0.020948511, 0.003913829,
            0.002906871, 0.002010524, 0.005943313, 0.006392967, 0.003870062,
            0.004312362, 0.002948073, 0.001038308, 0.001443528, 0.00289074,
            0.00604026, 0.002207974, 0.002497342, 0.003423049, 0.000199195
        ], [0.001712738, 0.007157825, 0.003416146, 0.005416592, 0.004056317,
            0.006971136, 0.009856869, 0.003663594, 0.009793276, 0.013745159,
            0.009319777, 0.016628937, 0.081220795, 0.271655148, 0.05327892,
            0.020554866, 0.013841979, 0.028017087, 0.025350844, 0.009243499,
            0.008567782, 0.004260198, 0.006409645, 0.003595688, 0.003883919,
            0.006765666, 0.036543225, 0.003761541, 0.001870468, 0.014293018,
            0.012760494, 0.006138881, 0.019908418, 0.00889766, 0.000729307
        ], [0.003928664, 0.00642433, 0.001911735, 0.002518168, 0.002558642,
            0.003073153, 0.00209243, 0.00098064, 0.002057822, 0.003516483,
            0.003354182, 0.004957052, 0.013376785, 0.004805795, 0.286212039,
            0.004512759, 0.001686097, 0.004133972, 0.090151692, 0.005370127,
            0.004301236, 0.001306301, 0.032056586, 0.038413526, 0.046818153,
            0.010898697, 0.00354497, 0.000986397, 0.000985704, 0.003645954,
            0.016982616, 0.001269725, 0.001473078, 0.00342693, 0.000140412
        ], [0.000726135, 0.001364259, 0.001161013, 0.005167087, 0.003078674,
            0.005732523, 0.003457143, 0.000538794, 0.001816359, 0.002565755,
            0.003239782, 0.007876947, 0.003915406, 0.00279741, 0.005132058,
            0.037706544, 0.001145094, 0.005114432, 0.002367421, 0.001379769,
            0.001646856, 0.002625083, 0.001637601, 0.000833065, 0.001053812,
            0.001475851, 0.00095476, 0.001120562, 0.001681743, 0.0021013,
            0.003910385, 0.003447664, 0.002931543, 0.005342124, 0.004015738
        ], [0.009491348, 0.023568488, 0.011478601, 0.017359866, 0.00751317,
            0.015585394, 0.021156927, 0.01416287, 0.027870169, 0.021677368,
            0.0347626, 0.026874914, 0.011230629, 0.009277203, 0.008399548,
            0.009514847, 0.08602396, 0.005346391, 0.009985061, 0.006309141,
            0.014253167, 0.023705661, 0.014531924, 0.003683507, 0.003899524,
            0.012241688, 0.008537407, 0.004731248, 0.01064226, 0.00667372,
            0.01581509, 0.018739583, 0.012973433, 0.015033122, 0.000261444
        ], [0.003660979, 0.007435305, 0.002355501, 0.00261207, 0.002018871,
            0.003694621, 0.003592571, 0.002540474, 0.00392684, 0.003339557,
            0.007294906, 0.004651405, 0.003571973, 0.00306693, 0.002334854,
            0.002849619, 0.025930049, 0.037715177, 0.004178476, 0.003900724,
            0.004629644, 0.005290658, 0.009556152, 0.002791392, 0.002078897,
            0.014613154, 0.011600899, 0.005319802, 0.042927675, 0.005173722,
            0.019329046, 0.015132102, 0.006615422, 0.01113384, 0.002939576
        ], [0.004816068, 0.004350688, 0.006406809, 0.005590272, 0.00761905,
            0.005774123, 0.004704957, 0.003300909, 0.006075868, 0.006037701,
            0.007499543, 0.005930264, 0.005406508, 0.00443041, 0.004757859,
            0.006234524, 0.009137427, 0.007348919, 0.009121777, 0.004687047,
            0.004488962, 0.005062069, 0.018365111, 0.003025759, 0.003196792,
            0.007456742, 0.00478761, 0.002602046, 0.00218551, 0.006018211,
            0.004390192, 0.003222954, 0.00347045, 0.006530263, 5.30796e-05
        ], [0.033008384, 0.028119342, 0.062827403, 0.0574064, 0.072746854,
            0.053660228, 0.050007541, 0.037451666, 0.049274407, 0.049191457,
            0.043758661, 0.045162817, 0.048644267, 0.047578898, 0.04464188,
            0.048451247, 0.022332322, 0.042533633, 0.037712706, 0.029887595,
            0.013618365, 0.04481227, 0.022728488, 0.019617032, 0.020915875,
            0.020316892, 0.010719887, 0.004719168, 0.00412515, 0.012807038,
            0.019712963, 0.010593118, 0.028355743, 0.015686446, 0.000359224
        ], [0.016454583, 0.010904076, 0.030223659, 0.027513185, 0.039587829,
            0.021241008, 0.018344726, 0.012098251, 0.022163668, 0.020921467,
            0.019515112, 0.016469928, 0.017393444, 0.015068486, 0.019240771,
            0.025742859, 0.008217516, 0.028322319, 0.017103443, 0.008010862,
            0.008312885, 0.02549122, 0.015393549, 0.008846898, 0.009088035,
            0.009516722, 0.00975149, 0.003972931, 0.00345884, 0.009518497,
            0.008073788, 0.005919688, 0.011818399, 0.01214297, 0.000701861
        ], [0.001359145, 0.00393028, 0.003895884, 0.004368525, 0.004124031,
            0.00451956, 0.005664363, 0.001802222, 0.004916472, 0.005590188,
            0.005583551, 0.004772362, 0.006434444, 0.005676242, 0.00294055,
            0.004144802, 0.006309574, 0.004846661, 0.006047598, 0.009449051,
            0.006371545, 0.010351516, 0.008199259, 0.005335943, 0.021096262,
            0.020130373, 0.008478637, 0.01003116, 0.002653704, 0.014195948,
            0.010841255, 0.007292983, 0.008336877, 0.011750373, 0.000112354
        ], [0.0143204, 0.024507223, 0.022921547, 0.020287825, 0.021545998,
            0.024148004, 0.018591644, 0.028617656, 0.019152104, 0.017587813,
            0.033249624, 0.017865267, 0.012873924, 0.009150998, 0.012079075,
            0.018108536, 0.027066753, 0.017323951, 0.01375531, 0.017404751,
            0.010864394, 0.010018431, 0.045729489, 0.040201435, 0.013545024,
            0.033764134, 0.01006233, 0.005910716, 0.002060157, 0.006545321,
            0.011593616, 0.011310636, 0.007945808, 0.009577027, 0.000812767
        ], [0.0031918, 0.005607059, 0.004545108, 0.003836537, 0.005093193,
            0.007398435, 0.003023906, 0.006322837, 0.003391956, 0.003123076,
            0.006657385, 0.005791585, 0.002552188, 0.001985498, 0.001990975,
            0.002045315, 0.00297546, 0.003737831, 0.00121579, 0.002600289,
            0.001548628, 0.001871034, 0.003195004, 0.107974188, 0.003320837,
            0.005201481, 0.000977011, 0.000520044, 0.000239504, 0.000976758,
            0.001381021, 0.001475263, 0.000603755, 0.001207352, 7.40572e-05
        ], [0.000702038, 0.001990969, 0.001508115, 0.00187596, 0.002847466,
            0.00146425, 0.002144736, 0.000820355, 0.001975233, 0.001991429,
            0.001546291, 0.001530388, 0.002117579, 0.002117836, 0.001825891,
            0.001453972, 0.001095555, 0.001162288, 0.002045051, 0.003844891,
            0.001719966, 0.001169289, 0.001993198, 0.003688218, 0.034537809,
            0.019361991, 0.00482251, 0.003395662, 0.000596694, 0.004600292,
            0.004765726, 0.002322534, 0.001287957, 0.004354714, 2.40927e-05
        ], [0.003586651, 0.006109723, 0.006327773, 0.004129042, 0.004927502,
            0.006438424, 0.007274459, 0.00647336, 0.00517715, 0.004992206,
            0.008322965, 0.005084897, 0.004638852, 0.003553461, 0.004516221,
            0.004534569, 0.004364414, 0.003956684, 0.008201585, 0.018181297,
            0.010689847, 0.007930214, 0.038143028, 0.127819027, 0.101403199,
            0.122967032, 0.003383804, 0.004061705, 0.000943906, 0.005077667,
            0.00477306, 0.002639118, 0.003160926, 0.00577943, 7.63443e-05
        ], [0.002160598, 0.006018339, 0.003917687, 0.004680217, 0.00514034,
            0.004568754, 0.011345453, 0.003000651, 0.006812751, 0.005706122,
            0.005571173, 0.005586704, 0.007371669, 0.00693209, 0.003824731,
            0.006477515, 0.006004849, 0.009703254, 0.012087888, 0.01949354,
            0.019824558, 0.011774804, 0.014801536, 0.017582503, 0.028651731,
            0.023887064, 0.112298552, 0.026024727, 0.004358079, 0.022580771,
            0.019392052, 0.008834888, 0.011087668, 0.022671191, 0.000158921
        ], [0.019959783, 0.032125031, 0.016444095, 0.022302706, 0.017907511,
            0.019541581, 0.022115725, 0.012654113, 0.020417381, 0.018930988,
            0.023483577, 0.0200652, 0.020659417, 0.016399991, 0.014348462,
            0.021785228, 0.0263697, 0.021177924, 0.026118988, 0.041034942,
            0.037125637, 0.024728272, 0.046209961, 0.041881685, 0.039249089,
            0.027440257, 0.024444017, 0.191079452, 0.05689964, 0.038968484,
            0.02570234, 0.014822548, 0.023199123, 0.036088737, 0.004039859
        ], [0.005348257, 0.005018313, 0.00403606, 0.005164903, 0.005067002,
            0.005232004, 0.007991484, 0.002871217, 0.00456021, 0.005907667,
            0.005088801, 0.005184892, 0.006142828, 0.00473146, 0.003985968,
            0.00777913, 0.006122333, 0.011220728, 0.023392409, 0.026334285,
            0.041257724, 0.025856815, 0.010358001, 0.020345521, 0.013968908,
            0.020962879, 0.013357677, 0.023745804, 0.031501229, 0.022943518,
            0.013529371, 0.018050098, 0.031940505, 0.026500544, 0.000692036
        ], [0.01179411, 0.030452205, 0.037587864, 0.028657173, 0.025671714,
            0.024434601, 0.057898585, 0.017390344, 0.063500237, 0.035617036,
            0.033238995, 0.029526561, 0.044398403, 0.057622263, 0.037788266,
            0.034203261, 0.040816242, 0.053176001, 0.046874442, 0.055413293,
            0.065262435, 0.035786941, 0.041588892, 0.042344057, 0.094353129,
            0.066517901, 0.072653539, 0.093879512, 0.035541318, 0.137843764,
            0.06342829, 0.028187556, 0.054368675, 0.070260036, 0.00023692
        ], [0.001602263, 0.003182718, 0.001719201, 0.001156496, 0.000860151,
            0.003154856, 0.002965288, 0.001250087, 0.002945492, 0.002118008,
            0.002173675, 0.002517976, 0.00125847, 0.001409948, 0.001162291,
            0.001917157, 0.006330303, 0.001383409, 0.002266494, 0.001544169,
            0.00337966, 0.00345991, 0.002953074, 0.006991875, 0.009382534,
            0.003425019, 0.002927728, 0.001655485, 0.003416012, 0.002808392,
            0.007354038, 0.004035221, 0.005167385, 0.007623932, 3.18179e-06
        ], [0.000751384, 0.000562102, 0.000379733, 0.000390997, 0.000453497,
            0.000269972, 0.000649448, 0.000480915, 0.000870227, 0.000540525,
            0.000541719, 0.000490996, 0.000808118, 0.000955307, 0.00050819,
            0.000358058, 0.000943264, 0.000425663, 0.000507651, 0.000818727,
            0.001766404, 0.000461574, 0.001125791, 0.000323779, 0.001697908,
            0.001070289, 0.001590221, 0.001240917, 0.000194457, 0.00218891,
            0.004403239, 0.006630546, 0.001083332, 0.001466106, 9.21287e-07
        ], [0.002275739, 0.000506621, 0.000464903, 0.000443249, 0.000905399,
            0.000379965, 0.000444153, 0.000217847, 0.000472835, 0.00037232,
            0.000431527, 0.000386659, 0.000552906, 0.000411579, 0.000307064,
            0.000485693, 0.000377709, 0.000534072, 0.000532213, 0.000660599,
            0.000467571, 0.001070501, 0.0007859, 0.000421653, 0.000522167,
            0.000677184, 0.000680221, 0.000581831, 0.000101037, 0.000793382,
            0.005808533, 0.001492574, 0.022880427, 0.001527685, 1.13911e-07
        ], [0.002551096, 0.006431627, 0.00603766, 0.005464629, 0.006734415,
            0.005417083, 0.01347034, 0.003785317, 0.008311885, 0.005880101,
            0.005967347, 0.006972883, 0.006564173, 0.005516132, 0.004089948,
            0.00618577, 0.00651725, 0.006162777, 0.009387989, 0.008941165,
            0.01031438, 0.01306448, 0.008833695, 0.00606707, 0.008738931,
            0.011290936, 0.019481998, 0.010851542, 0.007704902, 0.027874952,
            0.016065024, 0.010920029, 0.015187165, 0.080014251, 0.000546361
        ], [1.41814e-05, 1.80947e-05, 2.88465e-05, 4.4699e-05, 2.71306e-05,
            3.28999e-05, 3.63865e-05, 1.28306e-05, 3.22773e-05, 3.68142e-05,
            2.2339e-05, 2.6635e-05, 2.92626e-05, 2.18194e-05, 4.24215e-05,
            2.4948e-05, 1.76084e-05, 7.91595e-05, 0.000104801, 2.43351e-05,
            5.63432e-05, 6.02808e-05, 0.000131893, 1.94472e-05, 3.33877e-05,
            2.9561e-05, 7.32409e-05, 6.91712e-05, 0.000289134, 7.74291e-05,
            8.90481e-05, 4.60337e-05, 0.000140025, 0.000128978, 2.7775e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_1999():
    """
    IA matrix 1999



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_1999())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2000():
    """
    IA matrix 2000



    constant


    """

    ret =  xr.DataArray(
        data=[[0.130354083, 0.001710566, 0.259683952, 0.037935074, 0.0339944,
            0.128116338, 0.011511103, 0.002606412, 0.005224617, 0.012848384,
            0.000809544, 0.000273754, 0.000269196, 0.000175765, 0.000162405,
            0.00967625, 0.000433682, 0.007131263, 0.000490607, 0.00261283,
            0.002208166, 0.037614687, 0.001115813, 0.001695555, 0.000334468,
            0.003548096, 0.000222814, 0.000125807, 0.000426838, 0.000680882,
            0.001932798, 0.002023288, 0.002364005, 0.002478442, 0.014964669
        ], [0.00373729, 0.089461689, 0.003095401, 0.0021825, 0.001077691,
            0.004330272, 0.004769322, 0.453979031, 0.035082082, 0.003827989,
            0.074070919, 0.046599797, 0.003570269, 0.001371692, 0.00137701,
            0.005648079, 0.117370365, 0.020445783, 0.000426999, 0.000804239,
            0.000935464, 0.001677337, 0.00485951, 0.003700759, 0.002023982,
            0.001749564, 0.000482607, 0.000261793, 0.001660307, 0.000966788,
            0.002132421, 0.004479643, 0.001322639, 0.002484215, 0.063987774
        ], [0.053852889, 0.000755421, 0.142565282, 0.002667858, 0.084065723,
            0.001315534, 0.002720479, 0.000866592, 0.008977142, 0.002686442,
            0.00158525, 0.000924906, 0.001095139, 0.000955136, 0.000901956,
            0.002049197, 0.000697721, 0.000955748, 0.001632768, 0.004329347,
            0.004057413, 0.148954922, 0.001263898, 0.009680501, 0.003172434,
            0.002039819, 0.001452069, 0.000683158, 0.000250011, 0.001559082,
            0.007972772, 0.008413354, 0.013846687, 0.006510515, 0.002705627
        ], [0.002161473, 0.001260192, 0.001337143, 0.288619501, 0.053837594,
            0.004143529, 0.005682837, 0.000508027, 0.003516958, 0.014552355,
            0.004323005, 0.00141732, 0.002416476, 0.001756945, 0.006035756,
            0.023839323, 0.000456864, 0.002332106, 0.001475507, 0.002442292,
            0.003018926, 0.002783323, 0.001556526, 0.002113379, 0.001078091,
            0.001560803, 0.001053248, 0.000488914, 0.000266701, 0.001085641,
            0.003503572, 0.000969164, 0.003374199, 0.00542066, 0.000285036
        ], [0.00017007, 0.000138642, 0.000153691, 0.006195601, 0.192150862,
            0.000623692, 0.000807469, 7.29396e-05, 0.000217703, 0.001113181,
            0.000461377, 0.000279405, 0.000321979, 0.000246887, 0.001393561,
            0.003529967, 0.000137971, 0.000150238, 0.000410145, 0.0004375,
            0.000365036, 0.000155886, 0.000191843, 0.000170271, 0.000124055,
            0.000169529, 0.000332256, 4.48662e-05, 1.44184e-05, 0.000113028,
            0.000708735, 0.000133359, 0.000160926, 0.000484902, 1.40049e-05
        ], [0.001931524, 0.001767253, 0.001668896, 0.001350035, 0.001896109,
            0.206574137, 0.010207595, 0.000292846, 0.001436947, 0.002961098,
            0.005887819, 0.002686429, 0.002740301, 0.001454992, 0.002703221,
            0.074100754, 0.000764796, 0.032144207, 0.00068001, 0.001561822,
            0.001393849, 0.002202813, 0.00122546, 0.000514211, 0.000346707,
            0.003279456, 0.000739832, 0.000404184, 0.00216931, 0.000852307,
            0.001412658, 0.001923043, 0.000976874, 0.00279566, 0.000168483
        ], [0.002652525, 0.002990064, 0.024842481, 0.010766974, 0.01304137,
            0.012676652, 0.231923018, 0.001400261, 0.017722309, 0.017682564,
            0.021436504, 0.005596549, 0.009098881, 0.012312621, 0.00520278,
            0.022237714, 0.003958866, 0.00482237, 0.007789539, 0.013979286,
            0.013618728, 0.009602441, 0.005971786, 0.003127268, 0.006187022,
            0.013971999, 0.01186725, 0.014241495, 0.00256564, 0.022525117,
            0.016729541, 0.018203663, 0.009521897, 0.025080444, 0.000307766
        ], [0.012511611, 0.011247901, 0.00334331, 0.005175636, 0.003647384,
            0.00484613, 0.004170003, 0.043112108, 0.02854164, 0.007586741,
            0.01568427, 0.008296985, 0.003396971, 0.002062294, 0.00178102,
            0.003841028, 0.018576713, 0.012001053, 0.004156988, 0.00569818,
            0.004436052, 0.003579157, 0.051071578, 0.067531526, 0.064627684,
            0.014017344, 0.003282895, 0.00122427, 0.001092856, 0.004408282,
            0.007565186, 0.002804597, 0.003061705, 0.005377341, 5.83827e-05
        ], [0.038559691, 0.011291836, 0.010590941, 0.069946446, 0.035516251,
            0.030424706, 0.033662153, 0.013380215, 0.231425501, 0.200300407,
            0.0308316, 0.014989968, 0.011476786, 0.016690757, 0.01190117,
            0.029492529, 0.006392355, 0.010236565, 0.006069428, 0.003273622,
            0.00242368, 0.004675139, 0.003418116, 0.003705959, 0.003788388,
            0.003728467, 0.001496481, 0.000922535, 0.001705073, 0.004741424,
            0.008982179, 0.003703134, 0.062229314, 0.009771838, 0.000148825
        ], [0.004891556, 0.007681575, 0.0165467, 0.009475001, 0.036322914,
            0.009511043, 0.014633932, 0.0022543, 0.018479949, 0.117094188,
            0.01059319, 0.007311853, 0.026680166, 0.023187933, 0.034912694,
            0.037332995, 0.00258391, 0.018335766, 0.017311777, 0.004726564,
            0.005226466, 0.004164549, 0.010168908, 0.002647842, 0.002759092,
            0.006079389, 0.003421307, 0.000648649, 0.001147123, 0.002603758,
            0.003185038, 0.000891421, 0.005569823, 0.003424423, 6.04898e-05
        ], [0.002175633, 0.004542201, 0.005982077, 0.00221567, 0.001961104,
            0.008599315, 0.001327389, 0.00180754, 0.00616592, 0.006381092,
            0.113431264, 0.009962176, 0.005841215, 0.010454864, 0.006943089,
            0.006668347, 0.001933931, 0.076670624, 0.002903773, 0.001550683,
            0.001091269, 0.003700671, 0.001256959, 0.00064365, 0.000560002,
            0.001481047, 0.001226429, 0.000189855, 0.002125649, 0.001040873,
            0.001424876, 0.001800086, 0.002155609, 0.002028004, 5.40018e-05
        ], [0.004530422, 0.022981485, 0.013930755, 0.005607246, 0.010105973,
            0.025847623, 0.008924888, 0.006145159, 0.01519096, 0.024631467,
            0.031226399, 0.310315029, 0.158319387, 0.061867274, 0.097137198,
            0.096694242, 0.007084852, 0.085623633, 0.017579105, 0.004411011,
            0.00380714, 0.004441831, 0.006785961, 0.006617637, 0.006200916,
            0.00653812, 0.004056447, 0.000693859, 0.002626683, 0.003444324,
            0.004800619, 0.001771161, 0.002498964, 0.004576573, 0.000226787
        ], [0.006770139, 0.015228868, 0.004101776, 0.007178651, 0.005072661,
            0.00793243, 0.006550496, 0.005374559, 0.006614667, 0.010675713,
            0.014243506, 0.015547565, 0.117557175, 0.013287773, 0.029770886,
            0.009347867, 0.007705637, 0.016465053, 0.021897203, 0.00416853,
            0.002810715, 0.002137144, 0.005947953, 0.007673243, 0.004031112,
            0.004274141, 0.002971856, 0.000987612, 0.001413801, 0.002960047,
            0.006384187, 0.002099925, 0.002546695, 0.00351346, 0.00012437
        ], [0.001877252, 0.006076424, 0.003706754, 0.006358592, 0.004586636,
            0.007353623, 0.01074961, 0.003734394, 0.010083569, 0.014827704,
            0.01047172, 0.017888631, 0.085214763, 0.29265839, 0.055096687,
            0.025504389, 0.014943852, 0.030257737, 0.027280743, 0.010188717,
            0.00884022, 0.004794633, 0.006549079, 0.004025573, 0.004239013,
            0.007422991, 0.04263863, 0.003895946, 0.001930147, 0.015042948,
            0.013892476, 0.00600073, 0.020233699, 0.009756028, 0.0006211
        ], [0.004067005, 0.006144199, 0.002116347, 0.002708408, 0.002406845,
            0.003274945, 0.002223282, 0.000863733, 0.002357585, 0.003913269,
            0.003479215, 0.004936387, 0.012961113, 0.004516312, 0.292612106,
            0.004519661, 0.001765435, 0.004228487, 0.09103741, 0.00583836,
            0.004694295, 0.001334352, 0.032625608, 0.039038355, 0.048760152,
            0.011957665, 0.00352842, 0.001027053, 0.000926581, 0.003934431,
            0.017176931, 0.001385645, 0.00142901, 0.003526462, 0.000202191
        ], [0.000719088, 0.001272131, 0.001164121, 0.005356125, 0.003078926,
            0.005052377, 0.003733073, 0.000672229, 0.001801208, 0.002619195,
            0.003045773, 0.009556453, 0.004006144, 0.002670032, 0.004661016,
            0.042113437, 0.001218219, 0.005058078, 0.002415312, 0.001352641,
            0.001572537, 0.002800749, 0.00174379, 0.000825923, 0.001241566,
            0.001654145, 0.001042422, 0.001013849, 0.001470048, 0.001995508,
            0.003876465, 0.003416249, 0.002719726, 0.00556783, 0.00231479
        ], [0.010182541, 0.025605561, 0.012554344, 0.018064911, 0.007987338,
            0.017941501, 0.022614451, 0.012216401, 0.027999478, 0.022380194,
            0.036899269, 0.027256711, 0.01119504, 0.008692746, 0.008808072,
            0.010087798, 0.088965908, 0.00564666, 0.010141297, 0.006447736,
            0.014587111, 0.024406125, 0.014870307, 0.003633548, 0.003954712,
            0.012684053, 0.009007322, 0.00469853, 0.011770873, 0.006789635,
            0.015999379, 0.019802042, 0.013741095, 0.015776519, 0.000201186
        ], [0.003746011, 0.006600872, 0.002330476, 0.002612773, 0.001895578,
            0.003673591, 0.003355228, 0.00211629, 0.003455754, 0.003037489,
            0.006904626, 0.004277681, 0.003280628, 0.002612049, 0.002104804,
            0.002743673, 0.025246062, 0.041993494, 0.004328582, 0.003855376,
            0.004825718, 0.00553301, 0.009369909, 0.002734772, 0.002132635,
            0.012641295, 0.011288527, 0.004903124, 0.041814623, 0.004863535,
            0.019098485, 0.015213399, 0.006612658, 0.010437507, 0.001471134
        ], [0.004980271, 0.004060675, 0.006695835, 0.005879624, 0.007940115,
            0.006076214, 0.0048461, 0.002982445, 0.006191139, 0.006244668,
            0.007371836, 0.006008975, 0.005155484, 0.004188691, 0.004935259,
            0.006347288, 0.00840825, 0.007238473, 0.010582895, 0.004120539,
            0.004202268, 0.005031161, 0.016906179, 0.002403559, 0.003155224,
            0.007087273, 0.004195686, 0.002457513, 0.002014727, 0.005814215,
            0.004423591, 0.003050251, 0.003543111, 0.00661314, 3.38226e-05
        ], [0.033477839, 0.027795683, 0.063976181, 0.055181872, 0.071628159,
            0.055462827, 0.049491752, 0.037753232, 0.049172392, 0.048807107,
            0.043640044, 0.045403617, 0.047541282, 0.044491178, 0.044456142,
            0.053641216, 0.022820516, 0.041387853, 0.037104292, 0.028785504,
            0.013911796, 0.04384858, 0.022427352, 0.019699368, 0.020825315,
            0.020236305, 0.010684611, 0.004288078, 0.003642399, 0.013126225,
            0.020215756, 0.010527525, 0.027138191, 0.015578475, 0.000226252
        ], [0.016690894, 0.01003831, 0.030352023, 0.027175415, 0.038807741,
            0.020954413, 0.018682167, 0.01079575, 0.022262255, 0.020663066,
            0.018622712, 0.01584945, 0.016629768, 0.013939002, 0.018888521,
            0.025657343, 0.007952335, 0.028305094, 0.016316014, 0.007482293,
            0.007973616, 0.025554227, 0.014222415, 0.008408913, 0.008867199,
            0.009134812, 0.009025147, 0.004013412, 0.003138078, 0.009154337,
            0.00817889, 0.005755753, 0.011628861, 0.011802339, 0.000421444
        ], [0.001305091, 0.003256478, 0.003741488, 0.004122006, 0.004010067,
            0.0047156, 0.005683949, 0.001169449, 0.004362403, 0.005428348,
            0.005233311, 0.004451451, 0.005971618, 0.005039096, 0.002844322,
            0.004190111, 0.006698402, 0.004467125, 0.005890576, 0.008700015,
            0.006007868, 0.010101562, 0.007908036, 0.004197747, 0.021369964,
            0.018732512, 0.008645393, 0.009373842, 0.002603457, 0.014047896,
            0.01053255, 0.006713616, 0.00826005, 0.011826296, 7.63822e-05
        ], [0.013385862, 0.021335017, 0.023331604, 0.020032494, 0.021770772,
            0.024124636, 0.019023047, 0.025764294, 0.01834256, 0.01721626,
            0.033540468, 0.017288961, 0.012507672, 0.00840373, 0.012120768,
            0.018921462, 0.023719163, 0.016084619, 0.013885095, 0.01922384,
            0.011487702, 0.00966068, 0.045815375, 0.042854073, 0.013112192,
            0.05982024, 0.009461706, 0.005473052, 0.001801417, 0.006185105,
            0.011210005, 0.010284256, 0.007418487, 0.009456373, 0.000461871
        ], [0.003575216, 0.005600322, 0.004924941, 0.004381176, 0.006067157,
            0.008201228, 0.003557715, 0.005397178, 0.003357095, 0.003402873,
            0.007779025, 0.006121833, 0.002829897, 0.002138638, 0.002123352,
            0.002576855, 0.002846827, 0.004025485, 0.001175938, 0.002747787,
            0.001531758, 0.002084885, 0.003212101, 0.098974636, 0.00316431,
            0.006181132, 0.000879505, 0.000472539, 0.000240324, 0.000927105,
            0.001242932, 0.001405737, 0.000597107, 0.001223642, 1.89214e-05
        ], [0.000702209, 0.001892338, 0.001452776, 0.001865935, 0.002914277,
            0.001594124, 0.002390709, 0.000628888, 0.001899828, 0.002027349,
            0.001601236, 0.001477255, 0.002264893, 0.002139342, 0.001855418,
            0.00160814, 0.001154649, 0.001175167, 0.001850003, 0.003572363,
            0.001617629, 0.001202962, 0.001863003, 0.003804682, 0.035050826,
            0.019796979, 0.004322481, 0.003313822, 0.000600806, 0.004586532,
            0.004361269, 0.002213337, 0.00126459, 0.004512733, 1.2495e-05
        ], [0.003565293, 0.005463372, 0.006195394, 0.004107239, 0.00524867,
            0.007225893, 0.008637692, 0.005642941, 0.005080597, 0.005101675,
            0.008389127, 0.005069496, 0.004698726, 0.003522482, 0.00452169,
            0.004730621, 0.004303488, 0.003952386, 0.009404336, 0.021794413,
            0.010705285, 0.007896693, 0.041190755, 0.132571599, 0.095729739,
            0.106366387, 0.004654205, 0.003832873, 0.000852613, 0.00509527,
            0.004810028, 0.002584075, 0.002997603, 0.005943505, 4.06301e-05
        ], [0.002292364, 0.00581364, 0.004267257, 0.00498984, 0.005686138,
            0.005231434, 0.012292466, 0.00239728, 0.007363241, 0.006035842,
            0.006145348, 0.005656996, 0.007565538, 0.006894407, 0.004378927,
            0.007150076, 0.006308346, 0.010717943, 0.01330199, 0.020391589,
            0.020911212, 0.012710047, 0.015020946, 0.017260208, 0.026345901,
            0.023581968, 0.121616902, 0.027800764, 0.004808089, 0.024334157,
            0.020451809, 0.009411391, 0.011850026, 0.023661875, 0.000315848
        ], [0.020382194, 0.03051973, 0.015691093, 0.021686111, 0.016930213,
            0.019964525, 0.023575404, 0.010900605, 0.01869441, 0.018041057,
            0.022844961, 0.018561136, 0.018742333, 0.014737489, 0.014185445,
            0.022449677, 0.025749384, 0.021091221, 0.02422489, 0.038647808,
            0.035705398, 0.025320368, 0.047156083, 0.040962899, 0.038152371,
            0.025781445, 0.023061477, 0.197738133, 0.059710253, 0.038291845,
            0.024981871, 0.012693254, 0.022641284, 0.037334106, 0.005609512
        ], [0.005290936, 0.005088943, 0.004378207, 0.005042622, 0.004709688,
            0.005670214, 0.008963877, 0.002054618, 0.004523163, 0.006089763,
            0.004885097, 0.005145961, 0.005977954, 0.004429928, 0.004464422,
            0.007459804, 0.005732214, 0.009335729, 0.023381951, 0.024435617,
            0.042301232, 0.027979734, 0.009588686, 0.019855621, 0.013647265,
            0.020761801, 0.016321755, 0.024040298, 0.034709689, 0.024000106,
            0.013964443, 0.018454958, 0.032902579, 0.026122185, 0.000217713
        ], [0.01172017, 0.030233301, 0.038736256, 0.02816326, 0.024547287,
            0.025414171, 0.063485129, 0.012877257, 0.064507881, 0.036892835,
            0.032998453, 0.029476982, 0.043874547, 0.052407853, 0.039344521,
            0.036305761, 0.042380547, 0.05523486, 0.048160813, 0.056139333,
            0.064143593, 0.038911902, 0.042538881, 0.037724226, 0.094575123,
            0.066092046, 0.074016694, 0.096786024, 0.033130665, 0.139887858,
            0.064797592, 0.029342806, 0.054936933, 0.074979245, 0.001732383
        ], [0.001533296, 0.003523526, 0.001808982, 0.001161812, 0.000859841,
            0.003389197, 0.002910064, 0.000780683, 0.002691586, 0.002081909,
            0.002166236, 0.002173236, 0.001179778, 0.001242269, 0.001198618,
            0.002031523, 0.004793115, 0.001612837, 0.002196556, 0.001582075,
            0.003201191, 0.003581103, 0.002828429, 0.006526755, 0.008136968,
            0.003134306, 0.002964324, 0.001567508, 0.003063478, 0.002654689,
            0.007245444, 0.004050034, 0.004473014, 0.007726968, 7.21756e-06
        ], [0.000649592, 0.000473713, 0.000453634, 0.000420937, 0.000430526,
            0.000348009, 0.00071703, 0.000352765, 0.000942392, 0.000621962,
            0.000599509, 0.000616173, 0.000807938, 0.000847838, 0.00076192,
            0.000442849, 0.000982237, 0.000488133, 0.000727229, 0.00083637,
            0.001756277, 0.000533268, 0.001185197, 0.000371192, 0.001659058,
            0.001148242, 0.001531557, 0.001248114, 0.00028927, 0.002347225,
            0.004462656, 0.009564, 0.001092856, 0.001595629, 3.04567e-06
        ], [0.001543175, 0.000408586, 0.000474207, 0.000422906, 0.000898024,
            0.000397119, 0.000423144, 0.000162654, 0.000494802, 0.000397268,
            0.000406935, 0.000360302, 0.000525006, 0.000377479, 0.000343459,
            0.000467281, 0.000349448, 0.000479681, 0.000590122, 0.000627712,
            0.00048499, 0.001132345, 0.000744808, 0.000384915, 0.000487678,
            0.000613436, 0.000719737, 0.000564745, 9.9939e-05, 0.000792345,
            0.005594708, 0.00147453, 0.025009392, 0.001310793, 1.56567e-07
        ], [0.00249435, 0.006251734, 0.005872977, 0.005276953, 0.006265161,
            0.005465523, 0.013335568, 0.002720734, 0.007801795, 0.005577944,
            0.005567367, 0.006402912, 0.006033137, 0.004834256, 0.004360008,
            0.006431063, 0.006317696, 0.006186282, 0.008975105, 0.008451136,
            0.009339259, 0.013645916, 0.008046869, 0.005486954, 0.008599329,
            0.011223575, 0.019853825, 0.010630322, 0.007664937, 0.027810753,
            0.015539418, 0.010569588, 0.015153115, 0.079679529, 0.001331396
        ], [1.6336e-05, 1.97956e-05, 2.85695e-05, 5.32798e-05, 2.79486e-05,
            3.83786e-05, 4.07698e-05, 1.24051e-05, 3.46268e-05, 4.08863e-05,
            2.2841e-05, 2.36591e-05, 2.75941e-05, 2.03107e-05, 3.96587e-05,
            2.71574e-05, 1.66392e-05, 7.72898e-05, 0.000105669, 2.16105e-05,
            5.31545e-05, 6.02959e-05, 0.000104877, 1.55566e-05, 2.71625e-05,
            2.45078e-05, 6.14635e-05, 6.45951e-05, 0.000285292, 7.3198e-05,
            8.2384e-05, 4.19294e-05, 0.000112235, 0.000135981, 7.585e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2000():
    """
    IA matrix 2000



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2000())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2001():
    """
    IA matrix 2001



    constant


    """

    ret =  xr.DataArray(
        data=[[0.128990658, 0.00130797, 0.25641065, 0.038414818, 0.035569611,
            0.125822581, 0.011695, 0.001741439, 0.005024294, 0.013502866,
            0.000705558, 0.000257418, 0.000264066, 0.000188959, 0.000153783,
            0.009660785, 0.000437962, 0.008648063, 0.000472555, 0.002897472,
            0.002372607, 0.037817046, 0.001314873, 0.001505526, 0.000314297,
            0.003890489, 0.000245394, 0.00013934, 0.000385154, 0.000650331,
            0.002018589, 0.002140676, 0.002236508, 0.002391716, 0.015891051
        ], [0.003231349, 0.075437585, 0.002580875, 0.001943225, 0.000849562,
            0.003703303, 0.004120634, 0.434928026, 0.033294368, 0.003054123,
            0.066567454, 0.043787224, 0.002981953, 0.001250254, 0.001194567,
            0.005644336, 0.109148246, 0.017724304, 0.000443248, 0.000717295,
            0.000985758, 0.001530257, 0.003946111, 0.002658342, 0.001992056,
            0.001564707, 0.000484474, 0.000261638, 0.001803935, 0.000828771,
            0.001859956, 0.003390831, 0.001008301, 0.002454635, 0.062407836
        ], [0.054453981, 0.000718921, 0.146113538, 0.002782729, 0.082932958,
            0.001301409, 0.002673998, 0.000820696, 0.009605069, 0.002854879,
            0.001513816, 0.00094353, 0.001111951, 0.000990502, 0.000899276,
            0.002083445, 0.000701955, 0.0009584, 0.001700753, 0.004591647,
            0.004390947, 0.14828038, 0.001262997, 0.008076718, 0.003228536,
            0.001855182, 0.001612447, 0.000716764, 0.000241357, 0.00155686,
            0.008261021, 0.007722549, 0.013391848, 0.006250972, 0.003162994
        ], [0.002041249, 0.001135117, 0.001320705, 0.292978908, 0.05022159,
            0.003530146, 0.005291103, 0.000569802, 0.003260532, 0.012262637,
            0.004198338, 0.001436498, 0.002520761, 0.001842111, 0.005650828,
            0.023325841, 0.000463625, 0.00218027, 0.001442481, 0.002375882,
            0.002802589, 0.002587306, 0.001487502, 0.001982298, 0.001090993,
            0.001493909, 0.000938267, 0.000479772, 0.000252415, 0.001030985,
            0.003420472, 0.000920227, 0.002922983, 0.00564666, 0.0002047
        ], [0.000154005, 0.000106551, 0.000148194, 0.006257462, 0.203319334,
            0.000551543, 0.000668556, 7.02552e-05, 0.000230984, 0.001285762,
            0.000400703, 0.000246287, 0.000296748, 0.000256744, 0.001356302,
            0.003835132, 0.000133012, 0.000141174, 0.000426617, 0.000426954,
            0.000336744, 0.000144461, 0.000195533, 0.000152498, 0.0001287,
            0.000176916, 0.000318897, 4.09079e-05, 1.21218e-05, 0.000109059,
            0.000617432, 0.000121628, 0.000136797, 0.0004378, 7.87636e-06
        ], [0.002015468, 0.001644085, 0.001614212, 0.001315027, 0.001832574,
            0.211210902, 0.010044516, 0.000325046, 0.001534897, 0.003184429,
            0.006009824, 0.002815463, 0.002761252, 0.001511309, 0.00270877,
            0.073157591, 0.000883134, 0.031852038, 0.00069188, 0.001494847,
            0.001317321, 0.002158079, 0.00117419, 0.000496645, 0.00035487,
            0.003106756, 0.000703157, 0.000385638, 0.001975221, 0.000855742,
            0.001390893, 0.002004033, 0.000945602, 0.002752617, 0.000143899
        ], [0.002617515, 0.002920667, 0.02445819, 0.010694158, 0.012831759,
            0.012486336, 0.226319821, 0.00140137, 0.017177409, 0.017323386,
            0.020378345, 0.005493695, 0.008797141, 0.012112747, 0.005014773,
            0.021523677, 0.004552866, 0.00458042, 0.007734807, 0.013196077,
            0.012673072, 0.008694611, 0.005864279, 0.002826302, 0.005923661,
            0.013767352, 0.011884616, 0.01341445, 0.002355962, 0.02165984,
            0.017252852, 0.018151513, 0.00901752, 0.025211722, 0.000237953
        ], [0.012669351, 0.010987213, 0.003339118, 0.005388715, 0.003696622,
            0.005024828, 0.004327241, 0.045740437, 0.029946909, 0.00801612,
            0.0168267, 0.008958074, 0.003618318, 0.002252943, 0.001906525,
            0.004032913, 0.020358186, 0.01248846, 0.004226546, 0.005776087,
            0.004304134, 0.003541385, 0.051205819, 0.069479216, 0.063962234,
            0.0139503, 0.003297682, 0.001210981, 0.001122056, 0.004381679,
            0.007887059, 0.002787956, 0.002991292, 0.005263668, 5.23389e-05
        ], [0.038459614, 0.011203503, 0.010411666, 0.068915471, 0.03562473,
            0.031295855, 0.034100637, 0.014178596, 0.227451244, 0.198434015,
            0.031565318, 0.015238937, 0.01153714, 0.01687117, 0.011963407,
            0.028616763, 0.006203985, 0.010302071, 0.006258106, 0.00319812,
            0.002416044, 0.004736017, 0.003155974, 0.003483663, 0.003218783,
            0.003613624, 0.001479944, 0.00089555, 0.00193523, 0.004718596,
            0.009238611, 0.003765227, 0.062981982, 0.010024668, 0.000144636
        ], [0.004956749, 0.007209875, 0.016514787, 0.00967522, 0.034941631,
            0.009250782, 0.014906878, 0.002835085, 0.018275062, 0.123160401,
            0.010618957, 0.007275059, 0.026960095, 0.02424074, 0.034731551,
            0.035530798, 0.002756742, 0.017779925, 0.016513829, 0.004562698,
            0.004987632, 0.004046385, 0.009867926, 0.002875949, 0.002935882,
            0.005868081, 0.003412718, 0.00061019, 0.00106058, 0.002647119,
            0.00311271, 0.000905112, 0.005225996, 0.003588339, 4.32137e-05
        ], [0.002268991, 0.004554566, 0.005881846, 0.002318836, 0.001922327,
            0.008129758, 0.001315945, 0.001968636, 0.006145996, 0.006580131,
            0.115489616, 0.010290057, 0.005774869, 0.01112299, 0.006990083,
            0.006568663, 0.002096595, 0.075855209, 0.002903729, 0.001381507,
            0.001056544, 0.003535551, 0.001276583, 0.000639461, 0.000555258,
            0.001380776, 0.001182812, 0.000187299, 0.002023066, 0.00101768,
            0.001342169, 0.001726747, 0.002079469, 0.002017692, 4.08388e-05
        ], [0.004560348, 0.022994076, 0.013797071, 0.005747364, 0.010377935,
            0.025243831, 0.008708123, 0.006965946, 0.014639147, 0.025015051,
            0.031797343, 0.312200696, 0.159792443, 0.062791749, 0.094599653,
            0.092904061, 0.007781144, 0.087054428, 0.017926084, 0.004203362,
            0.003714246, 0.004303335, 0.006633184, 0.006496566, 0.005373179,
            0.006385152, 0.003955828, 0.000668918, 0.002523664, 0.003536043,
            0.004591198, 0.001771378, 0.002351261, 0.004521007, 0.000205571
        ], [0.006858738, 0.01553685, 0.00394727, 0.007892632, 0.005240528,
            0.008051417, 0.006450906, 0.00583449, 0.006562553, 0.010853824,
            0.016140286, 0.016048995, 0.121984292, 0.014158786, 0.030117702,
            0.009683159, 0.008087413, 0.016935004, 0.021623769, 0.003958743,
            0.002672284, 0.002106767, 0.005580365, 0.008448155, 0.00431436,
            0.004673684, 0.002891118, 0.001004686, 0.001412352, 0.00298392,
            0.005897205, 0.002156878, 0.002391337, 0.003432937, 8.52735e-05
        ], [0.001867645, 0.00657285, 0.003507978, 0.00614547, 0.005030507,
            0.006562771, 0.00974451, 0.004499262, 0.009369238, 0.013849262,
            0.010280152, 0.016554866, 0.084004925, 0.294022084, 0.053506997,
            0.023279233, 0.016017206, 0.030917238, 0.025719113, 0.009313879,
            0.007648534, 0.004394613, 0.006602659, 0.004253671, 0.004388795,
            0.007728219, 0.039748179, 0.003812155, 0.00186657, 0.015239965,
            0.01362345, 0.006284856, 0.02091665, 0.009784784, 0.00053865
        ], [0.004278625, 0.006144322, 0.002198183, 0.002773068, 0.00240931,
            0.003314516, 0.002297476, 0.001059345, 0.002183232, 0.004071458,
            0.003574817, 0.005082676, 0.013197038, 0.004919513, 0.294858634,
            0.00471445, 0.001948236, 0.004235539, 0.085669022, 0.005766233,
            0.004051716, 0.001328201, 0.033411681, 0.043136662, 0.051092603,
            0.011604336, 0.003556572, 0.001128144, 0.001008316, 0.004002709,
            0.015814335, 0.001518775, 0.001327068, 0.00474632, 0.000183717
        ], [0.000717957, 0.001253746, 0.001148122, 0.0052736, 0.002989318,
            0.004743271, 0.003743364, 0.000624477, 0.00178747, 0.00262821,
            0.002992023, 0.00912002, 0.003897169, 0.002773255, 0.004834615,
            0.043813041, 0.00126427, 0.005009716, 0.002184313, 0.00131782,
            0.001494333, 0.002573135, 0.001848784, 0.000782346, 0.00133153,
            0.001642549, 0.001018666, 0.000948025, 0.001496562, 0.001935589,
            0.003915362, 0.003136449, 0.002656246, 0.00575861, 0.002329361
        ], [0.011000135, 0.027532869, 0.013701964, 0.018661715, 0.008252857,
            0.018541218, 0.024418589, 0.013883548, 0.028595763, 0.024092667,
            0.03949536, 0.029021346, 0.011775532, 0.009269415, 0.009295448,
            0.010418221, 0.088262604, 0.006181439, 0.010732033, 0.006412796,
            0.014873492, 0.026121091, 0.014980133, 0.003672851, 0.004459509,
            0.013761068, 0.009511722, 0.004627057, 0.012798428, 0.00696837,
            0.016530515, 0.020589404, 0.014291573, 0.016783858, 0.000180232
        ], [0.003818489, 0.007547106, 0.002377358, 0.002798649, 0.001852211,
            0.003327174, 0.00339003, 0.002610824, 0.003455132, 0.003027493,
            0.007099156, 0.004236684, 0.003406525, 0.002887474, 0.002198511,
            0.002683431, 0.026705733, 0.04411472, 0.004394135, 0.003919606,
            0.004776192, 0.005762955, 0.009717779, 0.002737014, 0.002630356,
            0.012772098, 0.011419782, 0.004950035, 0.040841201, 0.004888769,
            0.018191884, 0.015665138, 0.006219454, 0.010334447, 0.00100998
        ], [0.00488606, 0.004035789, 0.006771006, 0.005777508, 0.00781704,
            0.005922209, 0.004866756, 0.002755547, 0.006091522, 0.006284694,
            0.007314218, 0.005946675, 0.00521299, 0.004313447, 0.004953229,
            0.006276774, 0.008016673, 0.00697247, 0.010067236, 0.003944703,
            0.004015245, 0.004940304, 0.016346618, 0.002216962, 0.003518303,
            0.007151101, 0.003923619, 0.002426467, 0.001872301, 0.005698243,
            0.004129509, 0.002872279, 0.003492647, 0.006279551, 2.75203e-05
        ], [0.033217712, 0.026920957, 0.063965487, 0.055941554, 0.069249969,
            0.056024676, 0.049631063, 0.040472308, 0.050728747, 0.05022076,
            0.043771528, 0.045512924, 0.048248241, 0.045448777, 0.044336958,
            0.052371773, 0.022731923, 0.0417049, 0.036274084, 0.026631144,
            0.013136843, 0.043912674, 0.021627687, 0.018903908, 0.021003149,
            0.019618638, 0.010369469, 0.004200014, 0.003943586, 0.013035198,
            0.020673158, 0.010364842, 0.026794661, 0.016154055, 0.000191677
        ], [0.016671194, 0.010188135, 0.030911461, 0.027663071, 0.041245561,
            0.02122943, 0.019003239, 0.011653281, 0.022347813, 0.020832584,
            0.01952484, 0.016450141, 0.017307127, 0.014753783, 0.01920413,
            0.025915665, 0.008488605, 0.029312039, 0.016164845, 0.007130474,
            0.007744719, 0.025788904, 0.013898824, 0.008059791, 0.009710794,
            0.009092784, 0.00864205, 0.004174081, 0.002900749, 0.009227787,
            0.008382634, 0.005816368, 0.011624605, 0.011486389, 0.000340523
        ], [0.001297634, 0.003271942, 0.003674186, 0.004237697, 0.004017872,
            0.004701979, 0.005963943, 0.001222989, 0.004428825, 0.005636089,
            0.005383845, 0.004554, 0.006075621, 0.005193398, 0.002857179,
            0.004174748, 0.009265071, 0.004432048, 0.005870148, 0.008468236,
            0.005787023, 0.009960963, 0.007934149, 0.004007511, 0.023574098,
            0.017835499, 0.008941468, 0.009271043, 0.002481592, 0.013541387,
            0.010473248, 0.006310026, 0.008002183, 0.011776789, 7.13389e-05
        ], [0.012872262, 0.021730778, 0.022924055, 0.019599751, 0.020638618,
            0.023960135, 0.018905482, 0.031313919, 0.018182196, 0.01747746,
            0.033386871, 0.017427055, 0.012502847, 0.00877581, 0.011895779,
            0.018699511, 0.023524757, 0.015971631, 0.014110145, 0.019051755,
            0.011506397, 0.009416907, 0.045889056, 0.044153199, 0.014305955,
            0.059293136, 0.009043701, 0.00531541, 0.00189793, 0.005900498,
            0.010911821, 0.010219689, 0.007144718, 0.009391508, 0.000333653
        ], [0.003599337, 0.005843252, 0.004829616, 0.004727485, 0.006019853,
            0.008645347, 0.003968466, 0.006698786, 0.003635877, 0.003890319,
            0.009207763, 0.006676939, 0.003215691, 0.002428396, 0.002173755,
            0.002854894, 0.003074334, 0.004449843, 0.001140895, 0.002969041,
            0.001521159, 0.00185321, 0.003190298, 0.105105532, 0.002973596,
            0.006341478, 0.000808859, 0.000486273, 0.000420691, 0.001019307,
            0.001443195, 0.00127214, 0.000577043, 0.001332256, 2.24734e-05
        ], [0.000649311, 0.001905043, 0.001328073, 0.001788344, 0.002730879,
            0.001678943, 0.00232686, 0.000608131, 0.001755267, 0.001941992,
            0.001612535, 0.001409866, 0.002195247, 0.002075415, 0.001686393,
            0.001489919, 0.001402897, 0.001076876, 0.001687527, 0.003142944,
            0.001412355, 0.001088181, 0.002090981, 0.005279876, 0.035127204,
            0.019060553, 0.003652741, 0.00293696, 0.000536496, 0.004141851,
            0.003888861, 0.001944755, 0.001175904, 0.00404695, 1.0273e-05
        ], [0.00358025, 0.005514369, 0.006155069, 0.00403627, 0.004963675,
            0.006740132, 0.009241314, 0.00674967, 0.005097918, 0.005244281,
            0.008312919, 0.005165765, 0.00467059, 0.003570018, 0.004535437,
            0.004723412, 0.004459775, 0.00413622, 0.009934307, 0.02082621,
            0.010195508, 0.007717457, 0.040462594, 0.121261142, 0.099458321,
            0.107182057, 0.004622185, 0.003444688, 0.000937338, 0.005022814,
            0.004755739, 0.002483856, 0.003033936, 0.006055191, 2.93923e-05
        ], [0.002501192, 0.006238801, 0.004635441, 0.005619544, 0.006463389,
            0.005670015, 0.01333907, 0.002725458, 0.008024302, 0.006608753,
            0.006836475, 0.006319596, 0.00809418, 0.007795981, 0.004471182,
            0.007922836, 0.007605216, 0.012044321, 0.014583751, 0.020324131,
            0.020989635, 0.01321183, 0.015966607, 0.017811116, 0.027333583,
            0.024326234, 0.12702912, 0.027671626, 0.004866399, 0.025813582,
            0.021648923, 0.009743622, 0.012511801, 0.023565825, 0.000364199
        ], [0.020507185, 0.030739745, 0.015348359, 0.02188145, 0.015917441,
            0.019120217, 0.024089633, 0.011654149, 0.018309943, 0.018080649,
            0.023585442, 0.019086412, 0.018399199, 0.014969609, 0.013843343,
            0.023471227, 0.030179119, 0.021032647, 0.023124205, 0.037068132,
            0.034613194, 0.02553761, 0.046851617, 0.040307429, 0.03928175,
            0.025865198, 0.022263501, 0.204475943, 0.065695935, 0.037758961,
            0.024038483, 0.011837414, 0.022749981, 0.040923532, 0.00776897
        ], [0.005455381, 0.005794982, 0.004645068, 0.005095725, 0.004584883,
            0.006138845, 0.009714716, 0.002312515, 0.004725053, 0.006333867,
            0.005084079, 0.00549077, 0.006174181, 0.00486759, 0.004657973,
            0.007554123, 0.007233132, 0.010053168, 0.02258489, 0.02358628,
            0.039505593, 0.028892637, 0.009595417, 0.019239737, 0.013468972,
            0.021572838, 0.017040141, 0.022706529, 0.035303478, 0.024073015,
            0.015383105, 0.01948701, 0.033300887, 0.026569553, 0.000122766
        ], [0.011385567, 0.031026575, 0.039002404, 0.02811605, 0.022887243,
            0.025436226, 0.065060116, 0.014034772, 0.066761793, 0.038681927,
            0.033894253, 0.031011171, 0.045131509, 0.054874694, 0.04098146,
            0.036600057, 0.054175754, 0.055242258, 0.050444485, 0.053548775,
            0.059930856, 0.039931347, 0.043200087, 0.034189465, 0.093284807,
            0.06652978, 0.072877445, 0.093595103, 0.032515704, 0.142718686,
            0.070404848, 0.030903932, 0.056149609, 0.075894201, 0.002219784
        ], [0.001502947, 0.003851298, 0.001863446, 0.00114776, 0.000784077,
            0.003257023, 0.003094049, 0.000910514, 0.002750791, 0.002189272,
            0.002185744, 0.002354933, 0.001192293, 0.001248479, 0.001235067,
            0.002039017, 0.005079163, 0.001672876, 0.002299865, 0.001479129,
            0.002909189, 0.003647175, 0.00297314, 0.00619151, 0.009039992,
            0.003292474, 0.002914486, 0.001490543, 0.003207629, 0.002733989,
            0.007318879, 0.004221287, 0.004510136, 0.008061367, 5.96045e-06
        ], [0.000587064, 0.000470894, 0.000473913, 0.000456108, 0.000480337,
            0.000386598, 0.000772204, 0.000400876, 0.00091005, 0.000670137,
            0.000620913, 0.000642119, 0.00085499, 0.000886816, 0.000715297,
            0.000477143, 0.001090799, 0.000509598, 0.000800971, 0.000778768,
            0.001465193, 0.000566704, 0.001143238, 0.00043978, 0.001882858,
            0.001171412, 0.001616978, 0.001277496, 0.000296894, 0.002288518,
            0.004383454, 0.010675692, 0.001135985, 0.00158405, 4.00135e-06
        ], [0.001491814, 0.000415041, 0.000508749, 0.000422707, 0.001028419,
            0.000410861, 0.00043634, 0.000189798, 0.000518675, 0.000412513,
            0.000421351, 0.000381125, 0.00054704, 0.000405702, 0.000361286,
            0.000493901, 0.000342344, 0.000461085, 0.000578658, 0.000616261,
            0.000467646, 0.001114325, 0.00071247, 0.000360557, 0.000462823,
            0.000643993, 0.00069855, 0.000539821, 9.42046e-05, 0.000770266,
            0.005254157, 0.001527145, 0.024992787, 0.001226622, 9.29981e-08
        ], [0.002548267, 0.00674309, 0.005729933, 0.005379013, 0.006544487,
            0.005392896, 0.013777546, 0.002794296, 0.007765519, 0.005746772,
            0.005738315, 0.006320995, 0.005831657, 0.004862391, 0.004345819,
            0.00613716, 0.007118284, 0.006161883, 0.008620044, 0.008139943,
            0.008672772, 0.013131125, 0.009187843, 0.005288448, 0.008492492,
            0.010658929, 0.019788541, 0.009888818, 0.007264161, 0.027075469,
            0.015566205, 0.010378109, 0.014274135, 0.079995065, 0.001688324
        ], [1.7347e-05, 2.28642e-05, 2.6553e-05, 5.6109e-05, 2.62342e-05,
            3.41391e-05, 4.1307e-05, 1.33692e-05, 3.36486e-05, 4.08335e-05,
            2.24359e-05, 2.30104e-05, 2.61862e-05, 1.94134e-05, 4.09892e-05,
            2.43773e-05, 1.94098e-05, 7.32694e-05, 6.68004e-05, 1.56339e-05,
            4.1984e-05, 5.36295e-05, 8.86582e-05, 1.30926e-05, 2.73819e-05,
            2.13976e-05, 5.13832e-05, 5.63822e-05, 0.000261832, 6.71376e-05,
            7.32823e-05, 4.11591e-05, 8.69702e-05, 0.000124841, 8.90569999999e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2001():
    """
    IA matrix 2001



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2001())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('run')
def a_matrix_2009():
    """
    IA matrix 2009



    constant


    """

    ret =  xr.DataArray(
        data=[[0.120888701, 0.000581938, 0.266267579, 0.051740741, 0.057534653,
            0.136416322, 0.018138489, 0.002408409, 0.007666947, 0.012876032,
            0.000557553, 0.000185118, 0.000182636, 8.44346e-05, 0.000127455,
            0.011699683, 0.000594999, 0.003503522, 0.000875825, 0.002442128,
            0.002277436, 0.041516933, 0.001679149, 0.000842528, 0.000434503,
            0.007513973, 0.000193071, 0.000174063, 0.000390308, 0.000868088,
            0.001744895, 0.002794591, 0.002242827, 0.002671932, 0.026632933
        ], [0.002516134, 0.095426858, 0.003344696, 0.004224709, 0.003166993,
            0.005463546, 0.005890823, 0.38890673, 0.032171439, 0.004587065,
            0.076014821, 0.060129665, 0.003958356, 0.001213883, 0.001646343,
            0.006248299, 0.106990704, 0.023242137, 0.00052141, 0.000570472,
            0.001222553, 0.002777597, 0.006370548, 0.004653555, 0.004644945,
            0.004577285, 0.000493351, 0.000285769, 0.001880296, 0.000886143,
            0.00331124, 0.001957449, 0.001965331, 0.003133667, 0.060101324
        ], [0.065542748, 0.000830643, 0.164829985, 0.005417614, 0.103532923,
            0.003221501, 0.003573617, 0.002030333, 0.012756078, 0.004105474,
            0.003052673, 0.002321784, 0.00210338, 0.001587049, 0.00139888,
            0.003118466, 0.00116346, 0.001763652, 0.001636915, 0.004908239,
            0.004767341, 0.150665483, 0.001945067, 0.005958816, 0.004868597,
            0.001824221, 0.001245711, 0.000724307, 0.000291239, 0.001943444,
            0.006214673, 0.007735489, 0.010854932, 0.00722962, 0.003456451
        ], [0.001643387, 0.000962437, 0.001610717, 0.392390012, 0.06630692,
            0.003395911, 0.006730605, 0.000695214, 0.003896811, 0.016245994,
            0.004858499, 0.002634979, 0.002799783, 0.002041497, 0.004152301,
            0.025793647, 0.001151146, 0.002452531, 0.001500945, 0.002712389,
            0.002713542, 0.002748356, 0.002193889, 0.003389504, 0.002737699,
            0.002118105, 0.00059748, 0.000465139, 0.000424981, 0.001608053,
            0.002498953, 0.001105698, 0.003182434, 0.00423507, 7.63043e-05
        ], [0.000155303, 5.53333e-05, 0.000120728, 0.0140659, 0.2371921,
            0.000558476, 0.000705806, 4.68754e-05, 0.000276589, 0.001172357,
            0.000257708, 0.000129412, 0.000189738, 0.000320787, 0.002487057,
            0.005966764, 7.1953e-05, 0.000105511, 0.000346567, 0.000265196,
            0.000214776, 0.000108468, 0.000247034, 9.28678e-05, 0.000142616,
            0.000192127, 0.000156027, 3.18536e-05, 9.49538e-06, 9.33202e-05,
            0.000622743, 0.000110526, 0.000253237, 0.00038771, 1.91532e-06
        ], [0.00167302, 0.001476841, 0.001590974, 0.001506566, 0.001855102,
            0.272681475, 0.013010751, 0.000313161, 0.001507165, 0.003046165,
            0.008591048, 0.004187618, 0.003261763, 0.001658191, 0.00266532,
            0.073463463, 0.000759106, 0.028324087, 0.001008915, 0.00109799,
            0.001145331, 0.002240422, 0.001226123, 0.000607341, 0.000396911,
            0.002164099, 0.000463643, 0.000257006, 0.001352525, 0.000975165,
            0.001992096, 0.003018019, 0.001422382, 0.00254386, 2.41936e-05
        ], [0.002271173, 0.002033861, 0.023955809, 0.011521887, 0.014846848,
            0.014362371, 0.250162516, 0.002110678, 0.014283708, 0.015885531,
            0.022236487, 0.004897852, 0.006162193, 0.007827576, 0.003900227,
            0.018865842, 0.002845496, 0.00383846, 0.007431204, 0.010693051,
            0.009968063, 0.007523698, 0.006557358, 0.003512257, 0.004772941,
            0.012646187, 0.008464967, 0.011292242, 0.001681255, 0.019197002,
            0.016856023, 0.018197035, 0.007977775, 0.022097123, 0.000108888
        ], [0.012816824, 0.006037935, 0.003430349, 0.004838133, 0.004155043,
            0.004619381, 0.004569765, 0.10510685, 0.04352231, 0.009232793,
            0.019742004, 0.013998609, 0.003972585, 0.001950279, 0.002089922,
            0.004133269, 0.025513918, 0.010766549, 0.005505045, 0.004912679,
            0.003701262, 0.003532218, 0.056491763, 0.065836371, 0.090560827,
            0.025468383, 0.003304255, 0.001233687, 0.001161405, 0.004773583,
            0.008851635, 0.003166728, 0.002660227, 0.005290161, 2.18733e-05
        ], [0.036903224, 0.008494161, 0.009320295, 0.059667027, 0.036023876,
            0.036887746, 0.036539066, 0.019519659, 0.243782129, 0.221976294,
            0.039574174, 0.015313869, 0.010066712, 0.018262833, 0.010760263,
            0.026779904, 0.006394529, 0.011257586, 0.005578649, 0.002627866,
            0.001890423, 0.004367338, 0.003445508, 0.004738474, 0.004034261,
            0.005026175, 0.001086994, 0.00065948, 0.001604313, 0.004716661,
            0.00759195, 0.003844708, 0.059385415, 0.01013981, 0.000150674
        ], [0.004586781, 0.004801018, 0.019109114, 0.010798407, 0.039539044,
            0.007530765, 0.01718239, 0.006099653, 0.018241933, 0.166125196,
            0.01269518, 0.006999667, 0.022803574, 0.024334942, 0.034033008,
            0.033804263, 0.00243792, 0.017601911, 0.014606404, 0.004109645,
            0.004503645, 0.004165213, 0.011512129, 0.00304469, 0.003969331,
            0.006868594, 0.002927619, 0.000575032, 0.001127375, 0.002817948,
            0.002825145, 0.001421755, 0.004678495, 0.004667707, 7.38282e-05
        ], [0.001367181, 0.003532917, 0.004719499, 0.001843963, 0.00149634,
            0.005586625, 0.001167129, 0.002119255, 0.005293097, 0.004891491,
            0.139761828, 0.010589933, 0.004860504, 0.009475931, 0.00591678,
            0.007393483, 0.001527621, 0.090156819, 0.003083961, 0.000845736,
            0.000858787, 0.002930787, 0.001362391, 0.000389106, 0.000622785,
            0.001346365, 0.000865694, 0.000101145, 0.001409438, 0.000881753,
            0.001287628, 0.00158889, 0.001653393, 0.001667731, 2.97486e-05
        ], [0.00419571, 0.01654448, 0.010277129, 0.004852253, 0.009182844,
            0.020163865, 0.009463752, 0.007053585, 0.012741075, 0.023001731,
            0.036531268, 0.33901594, 0.162989368, 0.067501377, 0.089442777,
            0.073888841, 0.006638796, 0.096896995, 0.013674036, 0.002850876,
            0.002634208, 0.004171701, 0.006389847, 0.004340424, 0.003876954,
            0.009422913, 0.00283683, 0.000488432, 0.002189806, 0.00372264,
            0.004097113, 0.00188014, 0.00208447, 0.004507807, 0.000151494
        ], [0.006985887, 0.014421577, 0.003867992, 0.009943277, 0.007724057,
            0.010314999, 0.007571775, 0.00793316, 0.008592427, 0.0137906,
            0.023366594, 0.026605983, 0.167267113, 0.01724183, 0.042802494,
            0.016601788, 0.007852326, 0.018857925, 0.021131806, 0.002434577,
            0.002194756, 0.002263226, 0.00542435, 0.009465864, 0.007460831,
            0.00826833, 0.002457226, 0.000777152, 0.001243092, 0.002850838,
            0.006222059, 0.002193488, 0.004390029, 0.003654881, 0.000366523
        ], [0.002076254, 0.008607423, 0.004157373, 0.006986534, 0.005350386,
            0.007641833, 0.010797926, 0.005646377, 0.011493637, 0.013464256,
            0.012848553, 0.018963135, 0.112463223, 0.410120085, 0.073587762,
            0.039961627, 0.031271109, 0.044916505, 0.032677075, 0.008905444,
            0.008038353, 0.006086614, 0.009667943, 0.004077445, 0.006685305,
            0.011861367, 0.043522874, 0.004237932, 0.00251306, 0.022895611,
            0.020959558, 0.013882705, 0.027666056, 0.017874436, 0.00108315
        ], [0.004053406, 0.004229143, 0.001847293, 0.003069773, 0.002564866,
            0.004309366, 0.004549726, 0.001654116, 0.002602849, 0.005193957,
            0.007001474, 0.008373815, 0.017135723, 0.004237159, 0.31531448,
            0.009837426, 0.003804162, 0.005231898, 0.068974527, 0.004894827,
            0.00359654, 0.002116657, 0.039337586, 0.041770377, 0.05826115,
            0.014674402, 0.003348504, 0.001029631, 0.000896676, 0.004364901,
            0.018523581, 0.002733143, 0.001418298, 0.008441612, 0.000273222
        ], [0.000620695, 0.00071955, 0.000967854, 0.005285527, 0.003257754,
            0.003538314, 0.004427354, 0.000941862, 0.001784646, 0.00283243,
            0.003681335, 0.008376299, 0.003945532, 0.002140651, 0.005569767,
            0.066611812, 0.001965872, 0.004189325, 0.001377658, 0.000998719,
            0.00132995, 0.002310054, 0.004081826, 0.000686082, 0.002305388,
            0.001457625, 0.00155272, 0.000830639, 0.00115312, 0.001697734,
            0.003084815, 0.003539207, 0.002787596, 0.004824092, 0.001900479
        ], [0.010536546, 0.027720116, 0.01389976, 0.017513145, 0.007439277,
            0.017472857, 0.023211699, 0.027016877, 0.026456895, 0.021033585,
            0.043981387, 0.030518698, 0.012147684, 0.007192453, 0.008420487,
            0.009539437, 0.145688849, 0.007451522, 0.01052203, 0.00634622,
            0.012605662, 0.023391663, 0.016570829, 0.003307458, 0.006899956,
            0.018085514, 0.009572582, 0.004028663, 0.008220846, 0.00576708,
            0.014454138, 0.020677628, 0.011712985, 0.015464777, 0.000134556
        ], [0.003284402, 0.006243117, 0.002238115, 0.002245176, 0.00122265,
            0.002794405, 0.00294276, 0.005220382, 0.002879404, 0.002266691,
            0.005886103, 0.003399518, 0.002493677, 0.00143435, 0.001730842,
            0.002730649, 0.015887261, 0.05271935, 0.005246605, 0.003341565,
            0.004647027, 0.006126866, 0.009643094, 0.00155895, 0.00334374,
            0.012715452, 0.010298021, 0.005047622, 0.031831582, 0.004709722,
            0.018772899, 0.011707454, 0.007529419, 0.009262424, 0.000529321
        ], [0.004661299, 0.002611811, 0.005853969, 0.00346753, 0.004656699,
            0.004778564, 0.004297519, 0.0035358, 0.004810483, 0.005156134,
            0.005520048, 0.004569508, 0.003804168, 0.002447495, 0.004329164,
            0.005758454, 0.00636979, 0.005940684, 0.011074758, 0.003591167,
            0.003745981, 0.00478142, 0.013966782, 0.001789563, 0.004889592,
            0.007562288, 0.003041772, 0.002508543, 0.001841165, 0.005291228,
            0.004003688, 0.002463013, 0.003544851, 0.005421874, 1.63446e-05
        ], [0.032385705, 0.015560077, 0.062376655, 0.036605304, 0.044916502,
            0.047616101, 0.045050886, 0.058517782, 0.044043666, 0.043283113,
            0.037531726, 0.038255333, 0.03881281, 0.031148402, 0.041179804,
            0.039443214, 0.018633716, 0.03990171, 0.03061926, 0.021794099,
            0.012738413, 0.045422008, 0.022083264, 0.017596152, 0.023168866,
            0.022459183, 0.011924235, 0.003923637, 0.003979559, 0.013444606,
            0.01864345, 0.011209958, 0.025957599, 0.016607425, 0.000102972
        ], [0.016027539, 0.00613409, 0.027178162, 0.018723791, 0.024238537,
            0.017564029, 0.017439356, 0.019087152, 0.019734299, 0.01855704,
            0.01659739, 0.015801628, 0.015570651, 0.011188319, 0.018384411,
            0.024130112, 0.007528115, 0.024035867, 0.015075497, 0.005574273,
            0.006756991, 0.02324804, 0.014256953, 0.006649544, 0.010164347,
            0.009789237, 0.00786404, 0.004412306, 0.002727395, 0.008075477,
            0.008017691, 0.005727689, 0.012315184, 0.010605491, 0.000127396
        ], [0.001469052, 0.003101963, 0.003894186, 0.003739392, 0.004314738,
            0.003653675, 0.004640373, 0.001101323, 0.003753093, 0.004220695,
            0.005350753, 0.003994775, 0.004712508, 0.003297883, 0.002498412,
            0.00332137, 0.00288861, 0.005058, 0.00458188, 0.007218638,
            0.005394406, 0.009812475, 0.007591032, 0.003927349, 0.015571168,
            0.022699421, 0.006871309, 0.010044071, 0.00192382, 0.012362754,
            0.010701779, 0.0093195, 0.007781858, 0.011192181, 3.80526e-05
        ], [0.012242553, 0.016374229, 0.02258429, 0.017755532, 0.017434512,
            0.021034705, 0.017866967, 0.043651822, 0.017565233, 0.016017448,
            0.03241889, 0.016709448, 0.012322748, 0.007279222, 0.010819275,
            0.018573651, 0.018312104, 0.019750916, 0.015998232, 0.021462293,
            0.012339042, 0.009127237, 0.046220212, 0.007575869, 0.009477216,
            0.056406597, 0.008597819, 0.005138129, 0.001612239, 0.005793638,
            0.00988756, 0.009275825, 0.006725135, 0.009094088, 0.00027994
        ], [0.002193438, 0.006069633, 0.004981452, 0.004506297, 0.004041947,
            0.006986484, 0.003434382, 0.003417472, 0.003410069, 0.003507934,
            0.009126311, 0.006187933, 0.003532993, 0.001985228, 0.002724187,
            0.002696377, 0.002707484, 0.003016066, 0.001649856, 0.006284285,
            0.002922504, 0.001507135, 0.00293385, 0.156727877, 0.002919115,
            0.005412028, 0.000909763, 0.000524271, 0.001738181, 0.001174336,
            0.001096204, 0.000766948, 0.000505851, 0.00113478, 1.26804e-05
        ], [0.00074432, 0.001301232, 0.001336568, 0.001445579, 0.002076667,
            0.001581792, 0.001905431, 0.000720588, 0.0015054, 0.001415125,
            0.001497593, 0.000986284, 0.001823306, 0.001407448, 0.001226949,
            0.001236609, 0.000721091, 0.001112405, 0.001955805, 0.002893464,
            0.001584309, 0.001403258, 0.002410063, 0.004291434, 0.031327185,
            0.012506033, 0.003633111, 0.002915912, 0.000421928, 0.003648055,
            0.003631351, 0.003554034, 0.001122089, 0.004235387, 7.30692e-06
        ], [0.003772519, 0.004429411, 0.006621263, 0.003653301, 0.00382203,
            0.006588179, 0.008683254, 0.009488712, 0.004676526, 0.005135828,
            0.007812329, 0.004778929, 0.003893505, 0.002603338, 0.003800707,
            0.004045541, 0.00382904, 0.013501268, 0.013878233, 0.019633701,
            0.012896749, 0.007593553, 0.037161657, 0.113707308, 0.102474447,
            0.097599154, 0.004008962, 0.003467267, 0.001137746, 0.005170474,
            0.004898505, 0.002831283, 0.003375436, 0.0062951, 9.73793e-06
        ], [0.004463967, 0.004240239, 0.005744442, 0.006862455, 0.006101215,
            0.006536256, 0.012070503, 0.004444499, 0.007856529, 0.005403036,
            0.006894629, 0.00778954, 0.006098697, 0.007862385, 0.004760726,
            0.016394232, 0.007787672, 0.012637345, 0.017245795, 0.018315466,
            0.022191358, 0.016440301, 0.021328135, 0.020081345, 0.023905376,
            0.02446745, 0.119975515, 0.029791255, 0.004709601, 0.025799657,
            0.023963141, 0.012410648, 0.01442908, 0.022420202, 0.000736682
        ], [0.018861197, 0.019865506, 0.017098438, 0.022453566, 0.017239811,
            0.01657294, 0.022817505, 0.01502997, 0.018729197, 0.015503224,
            0.024854271, 0.018033014, 0.016257468, 0.013214217, 0.014860692,
            0.030604933, 0.024631939, 0.021446218, 0.021458849, 0.029726579,
            0.034583679, 0.028164597, 0.050257549, 0.022862106, 0.033156531,
            0.036385442, 0.019360401, 0.210688319, 0.081711046, 0.038806342,
            0.024621158, 0.017092974, 0.025150996, 0.044576053, 0.01025043
        ], [0.002643269, 0.005282047, 0.004394853, 0.00448442, 0.005664317,
            0.005354039, 0.008880479, 0.002057561, 0.003743045, 0.005053642,
            0.004425126, 0.004066567, 0.004572572, 0.003201016, 0.004366973,
            0.006462553, 0.004468598, 0.010359687, 0.021685975, 0.018811996,
            0.041548949, 0.028601815, 0.008657137, 0.011726885, 0.008684949,
            0.021509266, 0.01291889, 0.021855438, 0.032746164, 0.021876848,
            0.014137156, 0.021178032, 0.033477006, 0.02590052, 1.06513e-06
        ], [0.01351312, 0.022230347, 0.039075584, 0.019547625, 0.017381564,
            0.023968257, 0.061889045, 0.020030073, 0.063345625, 0.034615764,
            0.030884869, 0.02678479, 0.036446852, 0.037989992, 0.039338319,
            0.027602531, 0.033605461, 0.054509417, 0.054771051, 0.057991,
            0.062618174, 0.050864757, 0.044632029, 0.024778681, 0.070304599,
            0.066050742, 0.058506804, 0.105897677, 0.029086824, 0.162016883,
            0.090850799, 0.038606316, 0.069687667, 0.086237134, 0.002250642
        ], [0.001356968, 0.001830687, 0.001959396, 0.000639838, 0.000541984,
            0.002741022, 0.003811216, 0.001132107, 0.002623977, 0.001940376,
            0.002474912, 0.003168719, 0.00118331, 0.000822726, 0.001077832,
            0.001987481, 0.004346593, 0.001438077, 0.00261513, 0.001440683,
            0.002838534, 0.003869299, 0.003660466, 0.005015839, 0.010468846,
            0.003541833, 0.002546227, 0.001626767, 0.003282131, 0.002946671,
            0.007726874, 0.004648745, 0.004566067, 0.00872816, 4.61683e-06
        ], [0.000547815, 0.00040566, 0.00038745, 0.000281477, 0.000321371,
            0.000310157, 0.000516227, 0.000436186, 0.000787919, 0.000498952,
            0.00046994, 0.000446195, 0.00070943, 0.000495287, 0.000622777,
            0.000353625, 0.000598958, 0.000473916, 0.000778217, 0.00073707,
            0.001244991, 0.000570168, 0.000918027, 0.000281241, 0.002102718,
            0.001078897, 0.001180857, 0.001193541, 0.000271294, 0.002193221,
            0.004295471, 0.016088321, 0.001298496, 0.001593386, 9.54745e-06
        ], [0.001266497, 0.000492357, 0.000468988, 0.000387123, 0.000996002,
            0.000590477, 0.000543451, 0.000220118, 0.000536451, 0.000380023,
            0.000916234, 0.00058674, 0.001091166, 0.000446769, 0.000564677,
            0.000481039, 0.000427029, 0.000398325, 0.000567782, 0.000469579,
            0.000438077, 0.00089232, 0.000601365, 0.00032889, 0.000450356,
            0.000455659, 0.000517206, 0.000549961, 8.36447e-05, 0.000617285,
            0.005883929, 0.001703813, 0.033417432, 0.00147311, 4.13318e-07
        ], [0.003036903, 0.005453801, 0.005182608, 0.004214113, 0.005712603,
            0.004635995, 0.012095707, 0.002292434, 0.006621708, 0.004554864,
            0.00570093, 0.005022149, 0.004044902, 0.002749303, 0.004232972,
            0.004323051, 0.00561415, 0.005705004, 0.006916814, 0.007656581,
            0.008825032, 0.014488451, 0.014030592, 0.004779447, 0.007674732,
            0.015471682, 0.015524187, 0.010356837, 0.006910271, 0.026061011,
            0.017430804, 0.011789667, 0.013719849, 0.07680589, 0.001263074
        ], [1.42671e-05, 2.40286e-05, 2.47281e-05, 0.000100001, 3.23713e-05,
            3.04051e-05, 2.71023e-05, 1.27161e-05, 1.66471e-05, 1.8645e-05,
            1.94115e-05, 1.32096e-05, 3.37777e-05, 1.22646e-05, 6.18991e-05,
            1.52228e-05, 7.79118e-06, 4.4963e-05, 2.5485e-05, 8.75894e-06,
            3.42579e-05, 5.08565e-05, 0.000134128, 5.77412e-06, 1.45253e-05,
            1.28275e-05, 2.9525e-05, 5.80156e-05, 0.000161812, 5.93361e-05,
            6.40354e-05, 4.01316e-05, 5.07878e-05, 9.4449e-05, 9.0189e-05
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def ia_matrix_2009():
    """
    IA matrix 2009



    constant


    """

    ret =  functions.make_ia_matrix(a_matrix_2009())
    return ret

# ECON- Final demands by sector, households and fuel
@cache('step')
def leontief_matrix_for_python():
    """
    Leontief matrix for Python



    component


    """

    ret =  functions.invert_matrix(ia_matrix())
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_1996():
    """
    Leontief matrix 1996



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17575305e+00, 8.32659202e-03, 3.77221238e-01, 7.52160716e-02, 9.28578382e-02,
            2.06052968e-01, 2.85525958e-02, 9.94415558e-03, 2.21734806e-02, 2.92395654e-02,
            1.04966512e-02, 7.78537906e-03, 8.36514893e-03, 8.44549620e-03, 9.24354804e-03,
            3.89121816e-02, 5.30670320e-03, 1.77448005e-02, 6.40871057e-03, 9.07401780e-03,
            8.59456056e-03, 1.09667171e-01, 6.56618636e-03, 1.14147746e-02, 9.27584662e-03,
            9.95894183e-03, 4.84832243e-03, 4.51526682e-03, 3.37078235e-03, 7.28950337e-03,
            1.00822916e-02, 8.90046873e-03, 1.36371101e-02, 1.10712001e-02, 1.92964894e-02
        ], [
            2.36521725e-02, 1.13063555e+00, 2.61051069e-02, 2.77620299e-02, 2.32798820e-02,
            2.96108526e-02, 2.80583313e-02, 5.25149073e-01, 8.35126525e-02, 4.20722582e-02,
            1.28658091e-01, 9.86101641e-02, 3.71188365e-02, 2.97818406e-02, 3.28015029e-02,
            3.82664186e-02, 1.42457997e-01, 5.88467486e-02, 1.68244647e-02, 1.34081842e-02,
            1.15912435e-02, 1.94389090e-02, 4.65253373e-02, 4.63119380e-02, 4.45391050e-02,
            2.61817527e-02, 1.01747769e-02, 6.99250284e-03, 9.35709613e-03, 1.13304048e-02,
            1.75049791e-02, 1.35243252e-02, 1.64673177e-02, 1.82778877e-02, 7.35197697e-02
        ], [
            7.61004883e-02, 5.16755677e-03, 1.18984480e+00, 1.57263805e-02, 1.31615973e-01,
            2.01041379e-02, 1.18570724e-02, 6.07340918e-03, 2.14208310e-02, 1.37153385e-02,
            8.75045786e-03, 7.54115905e-03, 8.21245721e-03, 8.61502385e-03, 8.01807904e-03,
            1.09492980e-02, 4.95333459e-03, 7.74952879e-03, 6.99787695e-03, 1.04290314e-02,
            9.16160298e-03, 1.99304453e-01, 7.06144157e-03, 1.53629727e-02, 1.42904222e-02,
            1.14328528e-02, 6.19036350e-03, 5.76495921e-03, 2.30948326e-03, 8.68766190e-03,
            1.46174288e-02, 1.40197480e-02, 2.28849543e-02, 1.25472888e-02, 3.16076006e-03
        ], [
            6.48889338e-03, 5.14168534e-03, 7.70022571e-03, 1.40473261e+00, 9.94155158e-02,
            1.50563888e-02, 1.74722472e-02, 4.61735333e-03, 1.19629941e-02, 3.67305962e-02,
            1.24272641e-02, 7.58274965e-03, 9.53959703e-03, 9.46119075e-03, 1.85501817e-02,
            4.28521321e-02, 3.43704738e-03, 9.29826010e-03, 7.17011922e-03, 6.65185465e-03,
            7.31987016e-03, 8.23200872e-03, 5.80551649e-03, 8.73618769e-03, 5.60471726e-03,
            6.02371500e-03, 6.53023428e-03, 2.93801147e-03, 1.67790720e-03, 4.55977326e-03,
            7.48352708e-03, 2.81733373e-03, 8.01186040e-03, 1.03741925e-02, 1.40293576e-03
        ], [
            6.94736498e-04, 8.21390334e-04, 9.40973359e-04, 1.20558078e-02, 1.23425639e+00,
            2.38758119e-03, 2.74593756e-03, 7.13339394e-04, 1.08632409e-03, 2.33622305e-03,
            1.54227127e-03, 1.35772122e-03, 1.40866722e-03, 1.35145805e-03, 3.00140272e-03,
            5.81959430e-03, 6.47916232e-04, 9.73964566e-04, 1.17271712e-03, 9.77999235e-04,
            9.86566318e-04, 7.96940512e-04, 7.32499773e-04, 8.62805237e-04, 7.24331139e-04,
            6.86940437e-04, 1.21886908e-03, 3.76052122e-04, 1.94022907e-04, 5.72028830e-04,
            1.51618198e-03, 4.23087077e-04, 6.10627631e-04, 1.42532892e-03, 1.21130076e-04
        ], [
            4.36684012e-03, 5.26038452e-03, 6.10351413e-03, 6.44536438e-03, 7.55708364e-03,
            1.23525967e+00, 2.09893385e-02, 4.24654743e-03, 6.20586228e-03, 7.91826524e-03,
            1.32468443e-02, 8.67674671e-03, 8.72228659e-03, 7.25405572e-03, 1.02833140e-02,
            9.72816793e-02, 4.58714989e-03, 4.98055556e-02, 4.39541103e-03, 4.46318632e-03,
            4.09305084e-03, 6.21865591e-03, 4.75141640e-03, 4.77650314e-03, 4.00492206e-03,
            9.34749179e-03, 3.30780119e-03, 2.95585277e-03, 5.92554577e-03, 3.79128229e-03,
            5.04455486e-03, 3.75988948e-03, 3.84273015e-03, 7.22612785e-03, 1.33154546e-03
        ], [
            1.58106752e-02, 1.56386981e-02, 5.25627664e-02, 3.56904447e-02, 4.28486957e-02,
            3.62645506e-02, 1.32355451e+00, 1.57245350e-02, 4.94643252e-02, 4.93343376e-02,
            4.90305679e-02, 2.66605805e-02, 3.43410674e-02, 4.48000908e-02, 3.14405949e-02,
            5.00881674e-02, 1.52979233e-02, 2.63167433e-02, 2.44602092e-02, 3.05714148e-02,
            2.91645704e-02, 3.24582836e-02, 2.06767636e-02, 2.38297764e-02, 2.68900370e-02,
            3.66869667e-02, 2.59195130e-02, 3.41080835e-02, 9.85195647e-03, 4.25783449e-02,
            3.22451156e-02, 2.77227363e-02, 2.58276287e-02, 4.62473152e-02, 2.79947967e-03
        ], [
            2.20112315e-02, 2.37864261e-02, 1.87000655e-02, 1.97987169e-02, 1.74799691e-02,
            1.95259393e-02, 1.64164451e-02, 1.06390305e+00, 4.62912440e-02, 2.77426619e-02,
            3.17420729e-02, 2.45029561e-02, 1.62999064e-02, 1.50298078e-02, 1.53682653e-02,
            1.79416909e-02, 3.08681266e-02, 2.57439285e-02, 1.20883869e-02, 1.26236969e-02,
            1.04394737e-02, 1.30133064e-02, 6.25847506e-02, 7.04675053e-02, 6.97117249e-02,
            2.66093798e-02, 8.71281888e-03, 5.48774224e-03, 4.34004191e-03, 9.98800863e-03,
            1.39125151e-02, 7.26636670e-03, 1.10109037e-02, 1.32823770e-02, 2.27134904e-03
        ], [
            7.12781541e-02, 3.23406089e-02, 5.29518282e-02, 1.47961331e-01, 9.41342329e-02,
            6.99498946e-02, 7.63147022e-02, 3.92468787e-02, 1.32227052e+00, 3.10962492e-01,
            6.47097093e-02, 4.68069720e-02, 4.89483695e-02, 5.97174739e-02, 5.96307481e-02,
            8.14768013e-02, 2.01570011e-02, 4.00833985e-02, 2.92029646e-02, 1.42062516e-02,
            1.25081769e-02, 2.54264940e-02, 1.93312974e-02, 1.95381991e-02, 1.70697430e-02,
            1.81522545e-02, 1.13638138e-02, 7.91269977e-03, 7.09862413e-03, 1.55582564e-02,
            2.21792162e-02, 1.14451920e-02, 9.26443836e-02, 2.44391873e-02, 4.16390548e-03
        ], [
            1.24370112e-02, 1.72885491e-02, 2.88376537e-02, 2.32712215e-02, 5.85116954e-02,
            2.32369162e-02, 2.90517226e-02, 1.35038289e-02, 3.48988070e-02, 1.13733355e+00,
            2.36709474e-02, 2.20570335e-02, 4.69008346e-02, 4.91354667e-02, 6.92555293e-02,
            5.68846850e-02, 9.96793365e-03, 3.19787271e-02, 3.34088460e-02, 1.04169761e-02,
            1.03907042e-02, 1.40437829e-02, 2.11777072e-02, 1.29798210e-02, 1.21663586e-02,
            1.48988717e-02, 9.70643574e-03, 4.59649664e-03, 4.22187736e-03, 8.12650745e-03,
            1.00829610e-02, 4.68868171e-03, 1.29949896e-02, 1.01527154e-02, 1.90242475e-03
        ], [
            5.66489001e-03, 1.01944711e-02, 1.16504460e-02, 7.61343708e-03, 7.66038977e-03,
            1.78742761e-02, 6.42428628e-03, 8.76287956e-03, 1.42131462e-02, 1.42028848e-02,
            1.12397875e+00, 2.03391057e-02, 1.55640227e-02, 2.36629051e-02, 1.88925922e-02,
            1.54778577e-02, 8.14224143e-03, 9.83782688e-02, 8.40658917e-03, 4.47303787e-03,
            3.80525159e-03, 8.63204937e-03, 5.93372845e-03, 5.35217871e-03, 4.78226773e-03,
            6.35536158e-03, 4.75568123e-03, 2.60273163e-03, 7.81196334e-03, 4.00500062e-03,
            6.24141358e-03, 4.76697244e-03, 6.06659398e-03, 6.17852108e-03, 1.23875647e-03
        ], [
            2.42209236e-02, 6.24446239e-02, 4.69192583e-02, 3.49102103e-02, 4.48204250e-02,
            7.33819694e-02, 3.90320856e-02, 4.65737462e-02, 5.77711731e-02, 6.97492831e-02,
            8.07715425e-02, 1.49101445e+00, 3.02119657e-01, 1.75703119e-01, 2.49247407e-01,
            1.95602451e-01, 3.57780813e-02, 1.61908747e-01, 7.23544773e-02, 2.08935660e-02,
            1.80553163e-02, 2.62120682e-02, 3.63232670e-02, 4.64822551e-02, 3.91697989e-02,
            3.12808013e-02, 2.15065616e-02, 1.05574991e-02, 1.56769196e-02, 1.86418877e-02,
            2.77530588e-02, 1.32859073e-02, 2.05248552e-02, 2.34278778e-02, 6.48466094e-03
        ], [
            1.41212739e-02, 3.00014601e-02, 1.47403431e-02, 1.78879937e-02, 1.62327147e-02,
            2.13132188e-02, 1.74493173e-02, 2.19236372e-02, 2.00637127e-02, 2.39520001e-02,
            2.60502100e-02, 3.53941274e-02, 1.14899249e+00, 3.28270605e-02, 5.88493491e-02,
            2.33335432e-02, 1.72270903e-02, 3.03448137e-02, 3.74234317e-02, 8.80392497e-03,
            7.51583475e-03, 9.26351098e-03, 1.53869223e-02, 1.61728769e-02, 1.35260530e-02,
            1.22936044e-02, 7.89019675e-03, 4.64623928e-03, 4.61592126e-03, 7.92242170e-03,
            1.28628085e-02, 5.58634185e-03, 7.65238769e-03, 9.44516541e-03, 2.66297287e-03
        ], [
            1.11752146e-02, 2.22783784e-02, 1.82636799e-02, 2.24849204e-02, 2.12028429e-02,
            2.47702519e-02, 2.93456408e-02, 1.99193282e-02, 3.01209812e-02, 3.57654753e-02,
            2.88331004e-02, 4.59420621e-02, 1.32173316e-01, 1.35874304e+00, 1.15151430e-01,
            4.75502215e-02, 2.91951991e-02, 5.36580034e-02, 5.43995761e-02, 2.10649668e-02,
            1.86689123e-02, 1.64398460e-02, 2.25252672e-02, 2.24880198e-02, 2.21854770e-02,
            2.24664342e-02, 4.78720575e-02, 1.30968261e-02, 8.00896404e-03, 2.67090711e-02,
            2.70254382e-02, 1.33677419e-02, 3.37985480e-02, 2.13810943e-02, 3.06327350e-03
        ], [
            1.16781719e-02, 1.58578697e-02, 1.32095137e-02, 1.36658592e-02, 1.44100638e-02,
            1.51760344e-02, 1.13526237e-02, 1.35196308e-02, 1.29941073e-02, 1.49634855e-02,
            1.59612433e-02, 1.91466014e-02, 3.19632747e-02, 1.80526936e-02, 1.42240037e+00,
            1.67772695e-02, 1.01733620e-02, 1.56384578e-02, 1.33381693e-01, 1.31675101e-02,
            1.07225191e-02, 8.98112878e-03, 5.07009596e-02, 7.24831266e-02, 7.36943201e-02,
            2.50959630e-02, 9.27803369e-03, 5.19425816e-03, 3.18086439e-03, 9.60918107e-03,
            3.07844255e-02, 4.89134273e-03, 6.93402074e-03, 1.07322628e-02, 1.63966011e-03
        ], [
            2.49487283e-03, 3.49741895e-03, 3.84971662e-03, 1.03106098e-02, 7.62020548e-03,
            1.06092419e-02, 6.55441783e-03, 3.09923388e-03, 4.96818679e-03, 6.00274424e-03,
            6.69680342e-03, 1.29054124e-02, 9.07613762e-03, 7.76562911e-03, 1.17732848e-02,
            1.04136703e+00, 2.98630109e-03, 9.01295699e-03, 5.15948675e-03, 3.16673017e-03,
            3.11402638e-03, 4.99781452e-03, 3.53803508e-03, 3.78085685e-03, 3.71954763e-03,
            3.66290448e-03, 2.53002130e-03, 2.70497738e-03, 2.17417007e-03, 4.13447177e-03,
            5.86470143e-03, 4.79757401e-03, 4.72995350e-03, 8.36560746e-03, 4.12173269e-03
        ], [
            2.17211904e-02, 4.05202320e-02, 3.16730674e-02, 4.11586572e-02, 2.94931527e-02,
            3.77164742e-02, 4.43816069e-02, 4.03332449e-02, 5.68357241e-02, 5.01096678e-02,
            6.27474563e-02, 5.84721434e-02, 3.63296186e-02, 3.34308359e-02, 3.59345855e-02,
            3.51389106e-02, 1.11491704e+00, 2.69811063e-02, 2.42998815e-02, 1.56923037e-02,
            2.27464139e-02, 3.89042014e-02, 2.92040935e-02, 2.01675725e-02, 1.87341798e-02,
            2.76827000e-02, 1.69243981e-02, 1.21166394e-02, 1.37290338e-02, 1.58524035e-02,
            2.55629891e-02, 2.70253777e-02, 2.55635071e-02, 2.83475531e-02, 3.69174424e-03
        ], [
            8.97191308e-03, 1.49177163e-02, 1.10733195e-02, 1.11836836e-02, 1.06640501e-02,
            1.35483281e-02, 1.29121334e-02, 1.35709106e-02, 1.42931727e-02, 1.33564460e-02,
            1.77164648e-02, 1.56706787e-02, 1.27381094e-02, 1.26842336e-02, 1.20756272e-02,
            1.23910937e-02, 3.58750410e-02, 1.04499252e+00, 1.14007141e-02, 1.06063947e-02,
            1.14655107e-02, 1.36113372e-02, 1.72513519e-02, 1.47033020e-02, 1.14245509e-02,
            2.48820248e-02, 1.77384091e-02, 1.19705594e-02, 5.06262470e-02, 1.20103969e-02,
            2.69035838e-02, 1.90993894e-02, 1.37254219e-02, 2.11215051e-02, 4.09787074e-03
        ], [
            9.58115487e-03, 8.91650730e-03, 1.43701277e-02, 1.31764268e-02, 1.63199048e-02,
            1.41106634e-02, 1.13222265e-02, 1.00016903e-02, 1.40692465e-02, 1.44618089e-02,
            1.50160680e-02, 1.48383569e-02, 1.33697877e-02, 1.33304796e-02, 1.40510920e-02,
            1.39737745e-02, 1.43541371e-02, 1.48555008e-02, 1.01341363e+00, 8.09532671e-03,
            7.75353980e-03, 1.11356805e-02, 2.34538737e-02, 1.03220529e-02, 8.58302024e-03,
            1.32039302e-02, 8.17501257e-03, 5.70236050e-03, 3.95485412e-03, 9.92716152e-03,
            7.74312558e-03, 5.67648203e-03, 7.29620624e-03, 1.09574153e-02, 9.80087544e-04
        ], [
            5.93665927e-02, 4.92552199e-02, 1.07920694e-01, 1.03870207e-01, 1.30377154e-01,
            1.00161144e-01, 8.94843156e-02, 7.29871126e-02, 9.23759748e-02, 9.89408562e-02,
            8.10581337e-02, 9.00216049e-02, 9.73270881e-02, 1.03405122e-01, 1.02984212e-01,
            9.78036124e-02, 4.57566717e-02, 8.24062111e-02, 6.73699235e-02, 1.04975481e+00,
            2.76014748e-02, 7.76585957e-02, 4.61148614e-02, 4.80313060e-02, 4.48292131e-02,
            4.35437059e-02, 2.46139114e-02, 1.69850169e-02, 1.24931215e-02, 2.81219049e-02,
            3.87837708e-02, 2.13363699e-02, 4.84536151e-02, 3.36787329e-02, 5.60071950e-03
        ], [
            2.93677308e-02, 2.10886312e-02, 5.28982101e-02, 5.10653383e-02, 6.97797826e-02,
            4.21765239e-02, 3.61318822e-02, 2.89471277e-02, 4.27313284e-02, 4.43092348e-02,
            3.61270254e-02, 3.60689123e-02, 3.76703618e-02, 3.73979056e-02, 4.45165223e-02,
            4.66722118e-02, 1.86949037e-02, 4.54483133e-02, 3.04105412e-02, 1.70082222e-02,
            1.01561955e+00, 4.41255896e-02, 2.63833370e-02, 2.35920298e-02, 2.10551748e-02,
            2.15261439e-02, 1.80292713e-02, 1.08763938e-02, 8.33153575e-03, 1.79798262e-02,
            1.69842870e-02, 1.15075256e-02, 2.17465552e-02, 2.27478407e-02, 3.03134335e-03
        ], [
            5.87006517e-03, 9.19411572e-03, 1.15097346e-02, 1.29939256e-02, 1.33069235e-02,
            1.34736432e-02, 1.43567473e-02, 9.07894807e-03, 1.43221543e-02, 1.51166713e-02,
            1.40861242e-02, 1.45259430e-02, 1.60356481e-02, 1.69895741e-02, 1.35687782e-02,
            1.35036114e-02, 1.02491296e-02, 1.33483862e-02, 1.29559309e-02, 1.49624261e-02,
            1.08773227e-02, 1.01625786e+00, 1.45466846e-02, 1.71610118e-02, 3.24576452e-02,
            2.94540140e-02, 1.31563734e-02, 1.65452478e-02, 5.38874120e-03, 1.97430930e-02,
            1.43144130e-02, 9.90885442e-03, 1.35327041e-02, 1.74049782e-02, 1.03588425e-03
        ], [
            2.78268225e-02, 4.34258592e-02, 4.54417359e-02, 4.24062103e-02, 4.75187895e-02,
            4.91020083e-02, 3.88318894e-02, 6.32268352e-02, 4.38296983e-02, 4.16177918e-02,
            5.97644035e-02, 4.38437736e-02, 3.46303856e-02, 3.10895027e-02, 3.74084987e-02,
            4.15377451e-02, 4.18240526e-02, 3.95989230e-02, 2.72617969e-02, 2.87117321e-02,
            1.86834073e-02, 2.70184351e-02, 1.05744017e+00, 4.10321582e-02, 2.92495194e-02,
            4.75930098e-02, 1.81675634e-02, 1.38569437e-02, 6.94028073e-03, 1.57208273e-02,
            2.16448161e-02, 1.65959884e-02, 1.84204753e-02, 2.08862412e-02, 4.54195926e-03
        ], [
            5.19117046e-03, 6.87217592e-03, 7.95585015e-03, 7.04263692e-03, 8.40996891e-03,
            1.25323523e-02, 4.62867423e-03, 9.32617953e-03, 6.91930874e-03, 6.12384053e-03,
            9.34965886e-03, 8.95354634e-03, 5.46218479e-03, 4.95663496e-03, 5.97319126e-03,
            6.40827466e-03, 4.55431570e-03, 6.38534651e-03, 3.27379223e-03, 4.23890483e-03,
            2.66550069e-03, 4.43509555e-03, 5.95282574e-03, 1.11547883e+00, 6.10082166e-03,
            9.02001998e-03, 2.51072933e-03, 1.75203564e-03, 9.16921640e-04, 2.37880925e-03,
            3.04866479e-03, 2.61582574e-03, 2.21540209e-03, 2.57663027e-03, 6.85436039e-04
        ], [
            2.76958665e-03, 4.92062577e-03, 4.92796835e-03, 5.44052896e-03, 7.05445857e-03,
            4.77998039e-03, 5.84165003e-03, 4.51882750e-03, 5.84010454e-03, 5.70176512e-03,
            4.94198534e-03, 5.20572367e-03, 5.62219774e-03, 6.24114421e-03, 6.16008691e-03,
            5.20165682e-03, 3.15486242e-03, 4.40302315e-03, 4.82659631e-03, 6.83654639e-03,
            3.67619469e-03, 3.93383090e-03, 5.29495431e-03, 1.13856542e-02, 1.03824757e+00,
            2.48198931e-02, 7.57271366e-03, 5.92177649e-03, 1.61170443e-03, 6.93737047e-03,
            6.79264226e-03, 3.45130826e-03, 3.43401851e-03, 6.61508540e-03, 4.76071350e-04
        ], [
            1.12374476e-02, 1.65980031e-02, 1.96402992e-02, 1.68723852e-02, 1.98449138e-02,
            2.09842651e-02, 1.96339640e-02, 2.12470284e-02, 1.89038415e-02, 1.88174260e-02,
            2.37601155e-02, 2.01095673e-02, 1.77677975e-02, 1.69133650e-02, 1.86720178e-02,
            1.80247599e-02, 1.39389951e-02, 1.54924838e-02, 1.65299077e-02, 2.86787053e-02,
            1.57020348e-02, 1.82283037e-02, 5.06836583e-02, 1.85656547e-01, 1.20776566e-01,
            1.13829769e+00, 9.47668797e-03, 9.39179831e-03, 3.46910651e-03, 1.13862953e-02,
            1.12520285e-02, 6.71859342e-03, 9.44540856e-03, 1.25304611e-02, 1.62978050e-03
        ], [
            9.33691098e-03, 1.45261453e-02, 1.59161067e-02, 1.70140210e-02, 1.83062521e-02,
            1.64764606e-02, 2.48080431e-02, 1.41535808e-02, 2.02553559e-02, 1.89790948e-02,
            1.73306796e-02, 1.83150311e-02, 2.01480812e-02, 2.09092505e-02, 1.81008905e-02,
            1.96530464e-02, 1.36331113e-02, 2.03515439e-02, 2.12940626e-02, 2.78377899e-02,
            2.64174598e-02, 2.16618073e-02, 2.46344808e-02, 3.79330110e-02, 4.46292775e-02,
            3.56904370e-02, 1.09442061e+00, 3.42223123e-02, 8.91279934e-03, 3.06601618e-02,
            2.54264826e-02, 1.27779641e-02, 1.87970407e-02, 3.14440680e-02, 1.64952217e-03
        ], [
            4.53859959e-02, 6.35674434e-02, 5.86577378e-02, 6.75338041e-02, 6.59375285e-02,
            6.56467993e-02, 6.26532342e-02, 5.79038628e-02, 6.61416561e-02, 6.37873380e-02,
            6.54957466e-02, 6.82078619e-02, 6.48297434e-02, 6.28211232e-02, 6.19955060e-02,
            6.52369877e-02, 5.36732742e-02, 6.02577730e-02, 5.80607429e-02, 7.21142437e-02,
            6.14151318e-02, 5.76165418e-02, 7.49913540e-02, 8.72187058e-02, 7.67371534e-02,
            6.30327414e-02, 4.68914149e-02, 1.22167500e+00, 7.02905284e-02, 6.45728991e-02,
            4.73952143e-02, 2.85347743e-02, 4.81668241e-02, 6.27031493e-02, 1.01523717e-02
        ], [
            1.70105852e-02, 1.64002017e-02, 2.20121918e-02, 2.30832392e-02, 2.55902838e-02,
            2.23298794e-02, 2.56888142e-02, 1.67592389e-02, 2.21148882e-02, 2.35853589e-02,
            2.09245473e-02, 2.22428894e-02, 2.36498151e-02, 2.34028497e-02, 2.25521124e-02,
            2.59809365e-02, 1.67568776e-02, 2.88932198e-02, 3.77717897e-02, 4.07483141e-02,
            5.33149858e-02, 4.33757173e-02, 2.37105814e-02, 4.85924943e-02, 3.34442324e-02,
            3.74772424e-02, 2.23922623e-02, 3.91759974e-02, 1.03730688e+00, 3.65893621e-02,
            2.13513373e-02, 2.41506545e-02, 4.10244349e-02, 4.17115125e-02, 2.48926377e-03
        ], [
            4.73707499e-02, 7.11564215e-02, 9.51780515e-02, 9.26142264e-02, 9.44545709e-02,
            8.04353967e-02, 1.25739173e-01, 7.04320405e-02, 1.32969188e-01, 1.07544261e-01,
            9.15001520e-02, 9.22898400e-02, 1.12502451e-01, 1.37673371e-01, 1.16295767e-01,
            1.00663368e-01, 7.72469760e-02, 1.11162226e-01, 9.22675163e-02, 9.87935520e-02,
            9.93659743e-02, 8.20798741e-02, 8.49152077e-02, 1.22537660e-01, 1.55205870e-01,
            1.20516687e-01, 1.03562556e-01, 1.36299977e-01, 5.82954726e-02, 1.17526955e+00,
            9.33261872e-02, 4.89112911e-02, 9.19520348e-02, 1.10984361e-01, 7.48811386e-03
        ], [
            3.75609017e-03, 6.05567550e-03, 5.33074870e-03, 4.61625365e-03, 4.48070658e-03,
            6.93185857e-03, 6.90503099e-03, 5.15321449e-03, 7.11869963e-03, 6.14415430e-03,
            5.94662001e-03, 6.47456165e-03, 4.94559296e-03, 5.19653632e-03, 5.09264707e-03,
            6.12581271e-03, 8.16458391e-03, 4.73828678e-03, 4.69353327e-03, 3.86990263e-03,
            5.46841001e-03, 6.38274370e-03, 5.64729305e-03, 1.30600089e-02, 1.22563668e-02,
            6.91786445e-03, 4.51796485e-03, 3.55539884e-03, 4.72706472e-03, 4.97248703e-03,
            1.00972721e+00, 5.20412233e-03, 7.48232305e-03, 1.04794961e-02, 5.45051936e-04
        ], [
            1.53096123e-03, 1.22847263e-03, 1.59467640e-03, 1.53042602e-03, 1.60375554e-03,
            1.38343993e-03, 1.80034932e-03, 1.42187577e-03, 2.11935021e-03, 1.73657004e-03,
            1.56042409e-03, 1.64750928e-03, 2.03549444e-03, 2.33187207e-03, 1.83637930e-03,
            1.47123713e-03, 1.82752779e-03, 1.53930950e-03, 1.28367760e-03, 1.66696739e-03,
            2.58279399e-03, 1.32584575e-03, 1.91612182e-03, 1.52998291e-03, 2.91354915e-03,
            2.12428979e-03, 2.38497998e-03, 2.12466181e-03, 5.86339779e-04, 3.00767356e-03,
            5.17186705e-03, 1.00686285e+00, 1.80950013e-03, 2.28675933e-03, 1.35016301e-04
        ], [
            2.79177799e-03, 8.81247867e-04, 1.76697414e-03, 1.32133761e-03, 1.35250233e-03,
            1.44467599e-03, 1.18988785e-03, 8.65198022e-04, 1.23032466e-03, 1.19687224e-03,
            1.12182063e-03, 1.12924160e-03, 1.29597065e-03, 1.25501765e-03, 1.14076503e-03,
            1.30028950e-03, 9.23851404e-04, 1.21332060e-03, 1.04989223e-03, 1.09990068e-03,
            8.65302268e-04, 1.79191993e-03, 1.22025549e-03, 1.75071850e-03, 1.27692620e-03,
            1.33009264e-03, 1.23196667e-03, 1.08321698e-03, 3.60844578e-04, 1.29816155e-03,
            6.77230039e-03, 1.74588266e-03, 1.02326055e+00, 2.29201583e-03, 1.21566362e-04
        ], [
            9.74263658e-03, 1.50785046e-02, 1.92441004e-02, 1.92417218e-02, 2.13349738e-02,
            1.82186492e-02, 2.94912150e-02, 1.54075623e-02, 2.38128194e-02, 2.10069309e-02,
            1.88098926e-02, 2.18577734e-02, 2.15708897e-02, 2.18557871e-02, 2.00835171e-02,
            2.08659665e-02, 1.55580608e-02, 1.95458276e-02, 2.07285716e-02, 1.91676062e-02,
            1.98120387e-02, 2.52881145e-02, 2.01620866e-02, 2.16205071e-02, 2.38277603e-02,
            2.43873652e-02, 2.82599078e-02, 2.22857085e-02, 1.32456747e-02, 4.23728685e-02,
            2.49740428e-02, 1.61663126e-02, 2.55940348e-02, 1.09457767e+00, 2.02732839e-03
        ], [
            4.80985248e-05, 5.83062239e-05, 9.41535010e-05, 1.10374700e-04, 1.02333785e-04,
            9.98930715e-05, 1.04199249e-04, 6.73697321e-05, 9.72789540e-05, 1.03461985e-04,
            8.57559870e-05, 1.03574045e-04, 9.99745458e-05, 8.81312500e-05, 1.21661532e-04,
            1.03802489e-04, 6.09055197e-05, 1.49703133e-04, 1.93925924e-04, 9.18345830e-05,
            1.08285004e-04, 1.26712162e-04, 2.19469169e-04, 9.98065386e-05, 1.00857829e-04,
            9.60586634e-05, 1.28294613e-04, 1.19029245e-04, 3.16445850e-04, 1.25758585e-04,
            1.32734920e-04, 8.34228095e-05, 1.77193186e-04, 1.82126670e-04, 1.00003186e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_1997():
    """
    Leontief matrix 1997



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17407328e+00, 8.10385233e-03, 3.71432733e-01, 7.19222826e-02, 9.36765402e-02,
            1.99469376e-01, 2.80716229e-02, 1.00181436e-02, 2.20700237e-02, 2.90689181e-02,
            1.01448740e-02, 7.60187355e-03, 8.16722562e-03, 8.18065116e-03, 9.15418334e-03,
            3.79584574e-02, 5.19253818e-03, 1.69371708e-02, 6.21794016e-03, 8.55791050e-03,
            8.13237937e-03, 1.06900106e-01, 6.39411535e-03, 1.14033693e-02, 9.23629672e-03,
            9.85914401e-03, 4.71719680e-03, 4.40465908e-03, 3.25152658e-03, 7.14765507e-03,
            9.85025930e-03, 8.69969438e-03, 1.33607056e-02, 1.10859202e-02, 1.64594222e-02
        ], [
            2.25628180e-02, 1.11545863e+00, 2.46494568e-02, 2.61150722e-02, 2.20460426e-02,
            2.76086130e-02, 2.64361394e-02, 5.05352591e-01, 8.06563636e-02, 4.01773810e-02,
            1.24812373e-01, 9.09206676e-02, 3.43415011e-02, 2.69965339e-02, 3.07134910e-02,
            3.51405621e-02, 1.35594559e-01, 5.44221007e-02, 1.55120028e-02, 1.17736978e-02,
            1.05878293e-02, 1.76229757e-02, 4.31933673e-02, 4.36920434e-02, 4.17992295e-02,
            2.36056441e-02, 9.90890810e-03, 6.41798463e-03, 8.65537074e-03, 1.06537965e-02,
            1.63945197e-02, 1.29049458e-02, 1.58606247e-02, 1.71236341e-02, 6.20163012e-02
        ], [
            7.73270143e-02, 5.10096830e-03, 1.18951224e+00, 1.57447633e-02, 1.34702144e-01,
            2.00997226e-02, 1.19262283e-02, 6.10251897e-03, 2.14243724e-02, 1.38864298e-02,
            8.80628412e-03, 7.58252979e-03, 8.28730404e-03, 8.63621607e-03, 8.20332498e-03,
            1.07858252e-02, 5.06184634e-03, 7.84274002e-03, 7.08080199e-03, 1.01917782e-02,
            9.01134343e-03, 1.96619455e-01, 7.07142657e-03, 1.61689318e-02, 1.46616765e-02,
            1.16970217e-02, 5.94724309e-03, 5.74533846e-03, 2.33753170e-03, 8.56246771e-03,
            1.44933777e-02, 1.39682306e-02, 2.27036368e-02, 1.30886604e-02, 2.71593185e-03
        ], [
            6.52589025e-03, 4.80705078e-03, 7.64597296e-03, 1.39492399e+00, 1.01673007e-01,
            1.48177474e-02, 1.70651919e-02, 4.36365302e-03, 1.18872756e-02, 3.65414816e-02,
            1.19171921e-02, 7.28489317e-03, 9.15019218e-03, 8.96462333e-03, 1.86282211e-02,
            4.12817286e-02, 3.42437309e-03, 8.89631591e-03, 6.97357414e-03, 6.24583592e-03,
            6.90409414e-03, 7.80406060e-03, 5.66224182e-03, 8.62166285e-03, 5.31080588e-03,
            5.78799219e-03, 6.84054978e-03, 2.88315002e-03, 1.62509797e-03, 4.38291685e-03,
            7.49716141e-03, 2.86755588e-03, 7.97349488e-03, 1.03117244e-02, 1.17082399e-03
        ], [
            7.00393012e-04, 7.76053170e-04, 9.84397924e-04, 1.30608667e-02, 1.23996191e+00,
            2.44439981e-03, 2.82983224e-03, 7.05068554e-04, 1.10354013e-03, 2.45768177e-03,
            1.54773489e-03, 1.36934896e-03, 1.41229048e-03, 1.34203140e-03, 3.15013037e-03,
            5.53675075e-03, 6.23831391e-04, 9.82314721e-04, 1.22180218e-03, 1.04458950e-03,
            1.00916838e-03, 7.96995157e-04, 7.48514298e-04, 8.89011023e-04, 7.40717692e-04,
            6.89352645e-04, 1.30616331e-03, 3.70138676e-04, 1.95138723e-04, 5.81802264e-04,
            1.57768156e-03, 4.34831218e-04, 6.24002586e-04, 1.43894871e-03, 1.01045614e-04
        ], [
            4.36165609e-03, 5.25531524e-03, 6.12127741e-03, 6.46665885e-03, 7.57404121e-03,
            1.24275301e+00, 2.07667381e-02, 4.18297051e-03, 6.18472133e-03, 7.92438348e-03,
            1.32674297e-02, 8.79066883e-03, 8.63962602e-03, 7.10998442e-03, 1.02203668e-02,
            9.55884730e-02, 4.46563771e-03, 4.86339578e-02, 4.26939428e-03, 4.20923615e-03,
            3.95415602e-03, 6.05212233e-03, 4.62020953e-03, 4.62782469e-03, 3.88215681e-03,
            9.09284655e-03, 3.40713487e-03, 2.86273635e-03, 5.68570044e-03, 3.75485137e-03,
            4.97754376e-03, 3.71789421e-03, 3.92158756e-03, 7.12469319e-03, 1.13196975e-03
        ], [
            1.59504566e-02, 1.54514057e-02, 5.34099459e-02, 3.63042562e-02, 4.34652934e-02,
            3.59116521e-02, 1.32562155e+00, 1.54668709e-02, 4.90908959e-02, 4.95716127e-02,
            4.83592059e-02, 2.63981774e-02, 3.42063652e-02, 4.39676374e-02, 3.19079143e-02,
            4.86715872e-02, 1.55305285e-02, 2.63502959e-02, 2.42527400e-02, 2.96306272e-02,
            2.81760603e-02, 3.18744151e-02, 2.05283927e-02, 2.35613729e-02, 2.64319125e-02,
            3.65484902e-02, 2.73382998e-02, 3.36120619e-02, 9.88253050e-03, 4.27718370e-02,
            3.25973568e-02, 2.84755303e-02, 2.62143319e-02, 4.75179985e-02, 2.38237628e-03
        ], [
            2.23616804e-02, 2.28106693e-02, 1.86382108e-02, 1.95450494e-02, 1.74224005e-02,
            1.92801508e-02, 1.62114553e-02, 1.06242938e+00, 4.72745701e-02, 2.81887353e-02,
            3.17512381e-02, 2.43996355e-02, 1.61718533e-02, 1.46124897e-02, 1.53050602e-02,
            1.75152605e-02, 3.11321669e-02, 2.56044861e-02, 1.19046479e-02, 1.17797037e-02,
            9.91727053e-03, 1.25758108e-02, 6.19272776e-02, 6.94977843e-02, 6.85122800e-02,
            2.55207719e-02, 9.03773956e-03, 5.42963508e-03, 4.31673156e-03, 9.96751235e-03,
            1.39474786e-02, 7.25456385e-03, 1.12843114e-02, 1.36675635e-02, 1.89101430e-03
        ], [
            7.24890007e-02, 3.25142858e-02, 5.37937824e-02, 1.47714766e-01, 9.45470630e-02,
            6.95663938e-02, 7.68255278e-02, 3.93734741e-02, 1.32654219e+00, 3.14955323e-01,
            6.41860724e-02, 4.72932934e-02, 4.93924613e-02, 5.95734359e-02, 6.07072527e-02,
            8.01984924e-02, 2.03194764e-02, 4.06152480e-02, 2.91342694e-02, 1.38748310e-02,
            1.22857129e-02, 2.51120844e-02, 1.92157849e-02, 1.90329585e-02, 1.71406298e-02,
            1.81799538e-02, 1.19117866e-02, 7.91549705e-03, 7.08791650e-03, 1.55535019e-02,
            2.23787473e-02, 1.17249504e-02, 9.64691338e-02, 2.49409974e-02, 3.58710370e-03
        ], [
            1.28015166e-02, 1.76768964e-02, 2.93912779e-02, 2.35994794e-02, 6.08724150e-02,
            2.33205016e-02, 2.88689759e-02, 1.36462519e-02, 3.53384467e-02, 1.13934384e+00,
            2.38619664e-02, 2.21976271e-02, 4.72718739e-02, 4.87914452e-02, 7.13261958e-02,
            5.69336230e-02, 1.00620462e-02, 3.25221083e-02, 3.38174657e-02, 1.01383514e-02,
            1.01553172e-02, 1.39375817e-02, 2.09918856e-02, 1.31481765e-02, 1.23601034e-02,
            1.48404602e-02, 1.00222099e-02, 4.52163326e-03, 4.21088432e-03, 8.15724368e-03,
            1.02368866e-02, 4.81417574e-03, 1.33832870e-02, 1.04980366e-02, 1.65582364e-03
        ], [
            5.72049347e-03, 1.00053646e-02, 1.18571192e-02, 7.70756388e-03, 7.91363446e-03,
            1.76618033e-02, 6.43468235e-03, 8.75707281e-03, 1.41575422e-02, 1.42778262e-02,
            1.12812590e+00, 2.06896152e-02, 1.55696798e-02, 2.33003324e-02, 1.93828896e-02,
            1.51970747e-02, 8.19573419e-03, 9.91520379e-02, 8.43217502e-03, 4.26240507e-03,
            3.68190631e-03, 8.60450475e-03, 5.77659381e-03, 5.29765569e-03, 4.74900017e-03,
            6.25780417e-03, 4.95220225e-03, 2.45043347e-03, 7.74594821e-03, 4.06018383e-03,
            6.18194047e-03, 4.74884541e-03, 6.13059584e-03, 6.12395025e-03, 1.05079900e-03
        ], [
            2.46021107e-02, 6.31347754e-02, 4.73327839e-02, 3.52597876e-02, 4.48984931e-02,
            7.34252598e-02, 3.92142604e-02, 4.65809266e-02, 5.77105253e-02, 7.00784607e-02,
            8.07309571e-02, 1.49751515e+00, 3.03706764e-01, 1.73363273e-01, 2.51272469e-01,
            1.99050736e-01, 3.59882000e-02, 1.62266432e-01, 7.13591499e-02, 1.99806092e-02,
            1.74727199e-02, 2.55560450e-02, 3.61548655e-02, 4.63013574e-02, 3.90949348e-02,
            3.08578479e-02, 2.25539484e-02, 1.02771985e-02, 1.53000951e-02, 1.87349351e-02,
            2.81472531e-02, 1.35564994e-02, 2.09239560e-02, 2.36877765e-02, 5.60254057e-03
        ], [
            1.44206885e-02, 2.99378415e-02, 1.47640322e-02, 1.78277290e-02, 1.62505467e-02,
            2.12478425e-02, 1.73964275e-02, 2.19096074e-02, 1.99671701e-02, 2.36713531e-02,
            2.66284591e-02, 3.53273897e-02, 1.14792875e+00, 3.22622445e-02, 5.87874397e-02,
            2.25584963e-02, 1.73952096e-02, 3.01492761e-02, 3.62038145e-02, 8.63858391e-03,
            7.21451220e-03, 8.99567364e-03, 1.51721427e-02, 1.67302559e-02, 1.34869724e-02,
            1.23230647e-02, 8.41012931e-03, 4.56967205e-03, 4.54688477e-03, 7.87014160e-03,
            1.29748103e-02, 5.95537253e-03, 7.97318326e-03, 9.56958895e-03, 2.27484607e-03
        ], [
            1.16482258e-02, 2.31896737e-02, 1.88404019e-02, 2.31343111e-02, 2.18363769e-02,
            2.54504984e-02, 3.01864682e-02, 2.04548014e-02, 3.07901170e-02, 3.65946430e-02,
            2.88216823e-02, 4.62315709e-02, 1.31549865e-01, 1.36143565e+00, 1.18695264e-01,
            4.63516339e-02, 3.01711683e-02, 5.49120277e-02, 5.49388109e-02, 2.10576808e-02,
            1.86132804e-02, 1.64871003e-02, 2.29094657e-02, 2.30637705e-02, 2.28826699e-02,
            2.26955776e-02, 5.11401210e-02, 1.33275929e-02, 8.22352365e-03, 2.79931113e-02,
            2.79922032e-02, 1.42453813e-02, 3.50863974e-02, 2.21294764e-02, 2.68209337e-03
        ], [
            1.23487045e-02, 1.56324590e-02, 1.35285207e-02, 1.38437481e-02, 1.46330589e-02,
            1.54414174e-02, 1.14944461e-02, 1.32006009e-02, 1.30901669e-02, 1.49937424e-02,
            1.58380284e-02, 1.94314522e-02, 3.22979088e-02, 1.82404801e-02, 1.42394952e+00,
            1.69596113e-02, 1.03206014e-02, 1.60605808e-02, 1.28255338e-01, 1.26370337e-02,
            1.04441398e-02, 8.90167713e-03, 5.24911117e-02, 7.45036804e-02, 7.52892506e-02,
            2.54489504e-02, 9.71787696e-03, 5.16219137e-03, 3.23978274e-03, 9.55299839e-03,
            3.13914522e-02, 5.00148325e-03, 7.10928430e-03, 1.08459162e-02, 1.40253012e-03
        ], [
            2.48690781e-03, 3.41192960e-03, 3.88246701e-03, 1.03541497e-02, 7.41219904e-03,
            1.06698303e-02, 6.71195399e-03, 3.07549970e-03, 5.11037072e-03, 6.13495842e-03,
            6.71100415e-03, 1.30309827e-02, 9.04538335e-03, 7.47710731e-03, 1.18759553e-02,
            1.04076428e+00, 3.03312989e-03, 9.29599950e-03, 5.14882903e-03, 3.05832967e-03,
            3.16941439e-03, 4.88138656e-03, 3.61175517e-03, 3.81498679e-03, 3.62426942e-03,
            3.61972188e-03, 2.62393124e-03, 2.72526274e-03, 2.26214658e-03, 4.35188231e-03,
            5.98798869e-03, 4.72290618e-03, 4.90671921e-03, 8.21584030e-03, 3.52061547e-03
        ], [
            2.10338925e-02, 3.68408665e-02, 3.07955670e-02, 4.12029806e-02, 2.86535557e-02,
            3.64967734e-02, 4.22571257e-02, 3.68980746e-02, 5.43601520e-02, 4.83303223e-02,
            5.94766435e-02, 5.61138245e-02, 3.44894461e-02, 3.12246214e-02, 3.50381597e-02,
            3.31204630e-02, 1.11635308e+00, 2.58411830e-02, 2.31440466e-02, 1.42147785e-02,
            2.11434176e-02, 3.74036316e-02, 2.75851698e-02, 1.89997103e-02, 1.75405766e-02,
            2.53904891e-02, 1.70287938e-02, 1.16569683e-02, 1.32694779e-02, 1.51179366e-02,
            2.46188291e-02, 2.60022014e-02, 2.46140415e-02, 2.74598724e-02, 2.92851546e-03
        ], [
            8.65066663e-03, 1.44096470e-02, 1.06756408e-02, 1.08659405e-02, 1.03893032e-02,
            1.28962223e-02, 1.24083553e-02, 1.27993531e-02, 1.35527878e-02, 1.26418026e-02,
            1.64307071e-02, 1.47909767e-02, 1.20813387e-02, 1.17360587e-02, 1.17173595e-02,
            1.16999684e-02, 3.49624292e-02, 1.04574130e+00, 1.09685474e-02, 9.65322590e-03,
            1.07289343e-02, 1.27171616e-02, 1.65815825e-02, 1.38942370e-02, 1.09550498e-02,
            2.38072863e-02, 1.79012413e-02, 1.12797235e-02, 4.80793566e-02, 1.16215152e-02,
            2.51644949e-02, 1.76212307e-02, 1.30536597e-02, 1.99665304e-02, 3.46570687e-03
        ], [
            9.59518497e-03, 8.84410603e-03, 1.43496236e-02, 1.34576262e-02, 1.68472533e-02,
            1.39286359e-02, 1.12736480e-02, 9.90440844e-03, 1.39013397e-02, 1.42222444e-02,
            1.45103895e-02, 1.43846917e-02, 1.31613340e-02, 1.26567852e-02, 1.40655011e-02,
            1.36744834e-02, 1.42659668e-02, 1.48011333e-02, 1.01368070e+00, 8.45873093e-03,
            7.79314597e-03, 1.09317256e-02, 2.34097589e-02, 1.00210665e-02, 8.84008384e-03,
            1.29444865e-02, 8.35785310e-03, 5.64439255e-03, 3.99289363e-03, 9.85031462e-03,
            7.69483216e-03, 5.59490146e-03, 7.34118400e-03, 1.09763908e-02, 8.33000625e-04
        ], [
            5.99879630e-02, 4.72438668e-02, 1.08591154e-01, 1.04347239e-01, 1.32057945e-01,
            1.01291119e-01, 8.92071730e-02, 7.11857977e-02, 9.24821131e-02, 9.74054016e-02,
            7.82250655e-02, 8.89210605e-02, 9.52475675e-02, 1.00714472e-01, 1.03828013e-01,
            9.53726949e-02, 4.49800377e-02, 8.11804567e-02, 6.63740135e-02, 1.04683629e+00,
            2.70785102e-02, 7.72009263e-02, 4.55853210e-02, 4.78466227e-02, 4.47779116e-02,
            4.29979893e-02, 2.54969667e-02, 1.68437550e-02, 1.24123969e-02, 2.84572877e-02,
            3.90741797e-02, 2.16070508e-02, 4.91249289e-02, 3.41100054e-02, 4.67593970e-03
        ], [
            2.94342208e-02, 2.01820073e-02, 5.23723785e-02, 5.13206474e-02, 7.01555897e-02,
            4.17859926e-02, 3.55031588e-02, 2.74316460e-02, 4.26858360e-02, 4.35728678e-02,
            3.50833636e-02, 3.55675228e-02, 3.66449312e-02, 3.58207465e-02, 4.39775193e-02,
            4.53951835e-02, 1.84576722e-02, 4.48719308e-02, 2.96457259e-02, 1.59826913e-02,
            1.01504691e+00, 4.23394966e-02, 2.60744766e-02, 2.31876136e-02, 2.10282595e-02,
            2.08571879e-02, 1.84018659e-02, 1.06515085e-02, 8.15366036e-03, 1.78507410e-02,
            1.67692150e-02, 1.13580845e-02, 2.16548551e-02, 2.25629365e-02, 2.53500708e-03
        ], [
            6.08516208e-03, 9.21543889e-03, 1.21210276e-02, 1.34524373e-02, 1.37145827e-02,
            1.37786654e-02, 1.46888614e-02, 9.14526464e-03, 1.45475153e-02, 1.52707384e-02,
            1.43739241e-02, 1.46332859e-02, 1.63327761e-02, 1.71185666e-02, 1.41285668e-02,
            1.34344967e-02, 1.06043714e-02, 1.37561683e-02, 1.31776745e-02, 1.47920350e-02,
            1.09293794e-02, 1.01665348e+00, 1.46898159e-02, 1.79812174e-02, 3.30735777e-02,
            3.12490982e-02, 1.22802094e-02, 1.65046409e-02, 5.52892595e-03, 2.01638895e-02,
            1.48381056e-02, 1.03797784e-02, 1.35907742e-02, 1.79356906e-02, 8.92863249e-04
        ], [
            2.79843641e-02, 4.09035286e-02, 4.54936352e-02, 4.32050780e-02, 4.75979270e-02,
            4.91552516e-02, 3.87143272e-02, 5.98662566e-02, 4.36140402e-02, 4.13407335e-02,
            5.86523453e-02, 4.31685972e-02, 3.38395439e-02, 3.00787502e-02, 3.72922647e-02,
            4.07180922e-02, 4.19469525e-02, 3.86007717e-02, 2.69192877e-02, 2.72271103e-02,
            1.81418018e-02, 2.62575524e-02, 1.05651491e+00, 4.07917164e-02, 2.78975474e-02,
            4.69759254e-02, 1.90767072e-02, 1.36271689e-02, 6.81089925e-03, 1.55091032e-02,
            2.12799407e-02, 1.68641500e-02, 1.83775238e-02, 2.07014371e-02, 3.74166599e-03
        ], [
            5.53848001e-03, 6.86703253e-03, 8.53217967e-03, 7.38383906e-03, 8.99822674e-03,
            1.32575568e-02, 4.84483529e-03, 9.23611369e-03, 7.27258398e-03, 6.33695051e-03,
            1.03709420e-02, 9.06646009e-03, 5.55030349e-03, 5.08478239e-03, 6.13143008e-03,
            6.67483962e-03, 4.79024255e-03, 6.73028171e-03, 3.39243530e-03, 3.76251814e-03,
            2.57326679e-03, 4.67368930e-03, 6.76388633e-03, 1.11348796e+00, 6.40196205e-03,
            9.51584622e-03, 2.58119329e-03, 1.83166345e-03, 9.51541619e-04, 2.51981235e-03,
            3.16124638e-03, 3.08987965e-03, 2.35428957e-03, 2.80126265e-03, 5.94362649e-04
        ], [
            2.80167786e-03, 4.74140843e-03, 4.99533279e-03, 5.50010151e-03, 7.14523590e-03,
            4.76176012e-03, 5.89523314e-03, 4.36662666e-03, 5.87788747e-03, 5.72183760e-03,
            4.82113800e-03, 5.12833018e-03, 5.68948960e-03, 6.23825219e-03, 6.13104324e-03,
            5.11382544e-03, 3.18437103e-03, 4.44446776e-03, 4.86475395e-03, 6.56185351e-03,
            3.67561753e-03, 3.94549827e-03, 5.38405439e-03, 1.13052705e-02, 1.03951443e+00,
            2.61143127e-02, 8.19223069e-03, 6.04862981e-03, 1.65406124e-03, 7.18900545e-03,
            6.96034165e-03, 3.49878147e-03, 3.49425526e-03, 6.86182092e-03, 3.98543773e-04
        ], [
            1.13563870e-02, 1.56681211e-02, 2.00764721e-02, 1.69883699e-02, 2.00789631e-02,
            2.16072197e-02, 1.97988344e-02, 2.02957593e-02, 1.88799572e-02, 1.85276742e-02,
            2.33907280e-02, 1.95161768e-02, 1.72666174e-02, 1.64139346e-02, 1.86282143e-02,
            1.76073410e-02, 1.37567833e-02, 1.52999520e-02, 1.66654443e-02, 2.77427293e-02,
            1.58929847e-02, 1.82789628e-02, 5.29624246e-02, 1.91961885e-01, 1.21616732e-01,
            1.14098359e+00, 9.87538329e-03, 9.49892868e-03, 3.51566180e-03, 1.15766547e-02,
            1.05905436e-02, 6.95518784e-03, 9.59916227e-03, 1.31172857e-02, 1.34578512e-03
        ], [
            9.91157940e-03, 1.51555546e-02, 1.70175818e-02, 1.83568537e-02, 1.97509070e-02,
            1.75260268e-02, 2.61995440e-02, 1.47748731e-02, 2.13447336e-02, 2.01756930e-02,
            1.81457310e-02, 1.95230042e-02, 2.13064852e-02, 2.18632329e-02, 1.94574264e-02,
            2.05033834e-02, 1.49378626e-02, 2.12991127e-02, 2.26324216e-02, 2.82868127e-02,
            2.70861966e-02, 2.25803610e-02, 2.62696538e-02, 3.98573794e-02, 4.64955935e-02,
            3.77427003e-02, 1.10538585e+00, 3.68641212e-02, 9.50576863e-03, 3.30598773e-02,
            2.70360844e-02, 1.41120462e-02, 1.99962331e-02, 3.40992341e-02, 1.47618168e-03
        ], [
            4.51281865e-02, 6.36409322e-02, 5.87479659e-02, 6.84679195e-02, 6.58400017e-02,
            6.64772390e-02, 6.30025158e-02, 5.84275786e-02, 6.64408072e-02, 6.33769858e-02,
            6.45633661e-02, 6.78903746e-02, 6.41725336e-02, 6.14751938e-02, 6.27880973e-02,
            6.49608915e-02, 5.45856341e-02, 6.10300642e-02, 5.83897347e-02, 6.98696842e-02,
            5.98246480e-02, 5.72241650e-02, 7.59369489e-02, 8.65090957e-02, 7.86794974e-02,
            6.15565880e-02, 4.83685680e-02, 1.22693591e+00, 7.21205337e-02, 6.67343715e-02,
            4.82813499e-02, 2.97556287e-02, 4.90915527e-02, 6.48069408e-02, 8.70573057e-03
        ], [
            1.69514514e-02, 1.59778282e-02, 2.17924475e-02, 2.32363563e-02, 2.53977535e-02,
            2.19719672e-02, 2.55417530e-02, 1.61259088e-02, 2.19254423e-02, 2.32088646e-02,
            2.06715427e-02, 2.19259282e-02, 2.32409577e-02, 2.28416553e-02, 2.24996826e-02,
            2.52542968e-02, 1.69286134e-02, 2.82256918e-02, 3.72035380e-02, 3.96191267e-02,
            5.20273312e-02, 4.20139097e-02, 2.34836248e-02, 4.70975263e-02, 3.30708163e-02,
            3.73693327e-02, 2.36012001e-02, 3.77967623e-02, 1.03654448e+00, 3.66370605e-02,
            2.13003740e-02, 2.42346008e-02, 4.11912054e-02, 4.23870134e-02, 2.09544126e-03
        ], [
            4.94170352e-02, 7.34141674e-02, 9.85392623e-02, 9.63478279e-02, 9.78372441e-02,
            8.18779526e-02, 1.30435538e-01, 7.11765820e-02, 1.37195707e-01, 1.10268645e-01,
            9.15219210e-02, 9.45260259e-02, 1.15216962e-01, 1.40544994e-01, 1.21261800e-01,
            1.01392962e-01, 8.16185896e-02, 1.17011432e-01, 9.51043763e-02, 9.60421801e-02,
            9.94820741e-02, 8.40460616e-02, 8.80600418e-02, 1.23090651e-01, 1.60608811e-01,
            1.25412424e-01, 1.13824047e-01, 1.40467772e-01, 5.90838558e-02, 1.18396570e+00,
            9.67558399e-02, 5.13382655e-02, 9.51353556e-02, 1.16784365e-01, 6.60679080e-03
        ], [
            3.67253890e-03, 5.57840577e-03, 5.16198610e-03, 4.56032880e-03, 4.34256388e-03,
            6.76169017e-03, 6.60666103e-03, 4.74706935e-03, 6.92966385e-03, 5.94966941e-03,
            5.55771970e-03, 5.92778255e-03, 4.64787217e-03, 4.93544491e-03, 4.94975644e-03,
            5.81298868e-03, 8.34401198e-03, 4.49953018e-03, 4.41675221e-03, 3.45844137e-03,
            5.10562456e-03, 6.12154351e-03, 5.44513500e-03, 1.35326963e-02, 1.34991318e-02,
            6.45739574e-03, 4.70597257e-03, 3.42546981e-03, 4.42692587e-03, 4.89248843e-03,
            1.00946330e+00, 5.14001895e-03, 7.27747066e-03, 1.02340431e-02, 4.35576245e-04
        ], [
            1.56006926e-03, 1.25203508e-03, 1.58095241e-03, 1.51016457e-03, 1.60977517e-03,
            1.37239289e-03, 1.75636253e-03, 1.42293204e-03, 2.07673787e-03, 1.72253856e-03,
            1.54425939e-03, 1.62802143e-03, 2.01611147e-03, 2.30737676e-03, 1.81976615e-03,
            1.39837460e-03, 1.86131833e-03, 1.50282277e-03, 1.29174928e-03, 1.54383912e-03,
            2.43723462e-03, 1.28445766e-03, 1.92404904e-03, 1.52429237e-03, 2.80276474e-03,
            2.08135935e-03, 2.54102931e-03, 2.11225546e-03, 5.80025922e-04, 2.93199945e-03,
            5.18182826e-03, 1.00705390e+00, 1.78317576e-03, 2.28255418e-03, 1.16647262e-04
        ], [
            2.84386212e-03, 9.03636180e-04, 1.80732821e-03, 1.35733038e-03, 1.41531417e-03,
            1.45159086e-03, 1.23841650e-03, 8.92528891e-04, 1.27011380e-03, 1.22609651e-03,
            1.13799525e-03, 1.15423513e-03, 1.36108706e-03, 1.26957635e-03, 1.17661604e-03,
            1.28744542e-03, 9.70673068e-04, 1.25622284e-03, 1.12942612e-03, 1.12839746e-03,
            8.87205285e-04, 1.82215209e-03, 1.33352404e-03, 1.42867399e-03, 1.31841458e-03,
            1.42460130e-03, 1.24784330e-03, 1.11782744e-03, 3.76248386e-04, 1.32865192e-03,
            6.94865901e-03, 1.83686192e-03, 1.02337217e+00, 2.35063440e-03, 1.06346641e-04
        ], [
            9.93694492e-03, 1.46711935e-02, 1.92680423e-02, 1.90215866e-02, 2.11787796e-02,
            1.87019046e-02, 2.99936711e-02, 1.49278038e-02, 2.39026536e-02, 2.14571036e-02,
            1.85368283e-02, 2.18452105e-02, 2.13729406e-02, 2.20351257e-02, 2.01954229e-02,
            2.07366029e-02, 1.56888188e-02, 1.92910662e-02, 2.01127181e-02, 1.77035340e-02,
            1.89492747e-02, 2.44218583e-02, 1.96718043e-02, 2.18716004e-02, 2.31677938e-02,
            2.45194442e-02, 2.94984350e-02, 2.16663696e-02, 1.32271609e-02, 4.18944399e-02,
            2.53415995e-02, 1.75182513e-02, 2.50198567e-02, 1.09540360e+00, 1.71137070e-03
        ], [
            4.74615746e-05, 5.37539349e-05, 9.09681683e-05, 1.10333315e-04, 9.84966309e-05,
            9.63668086e-05, 1.00559311e-04, 6.05835635e-05, 9.65572748e-05, 1.00966204e-04,
            8.00894129e-05, 9.81431229e-05, 9.22401543e-05, 8.34860139e-05, 1.14712388e-04,
            1.00535148e-04, 5.86191133e-05, 1.47459673e-04, 1.78676789e-04, 7.97945275e-05,
            9.74895078e-05, 1.21827895e-04, 2.09251559e-04, 9.38929724e-05, 9.65045852e-05,
            9.12672177e-05, 1.31305289e-04, 1.14934948e-04, 3.00071265e-04, 1.25449383e-04,
            1.30693957e-04, 8.11277864e-05, 1.80203452e-04, 1.77801090e-04, 1.00002693e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_1998():
    """
    Leontief matrix 1998



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17610288e+00, 8.15580396e-03, 3.71439005e-01, 7.34348506e-02, 9.46030156e-02,
            2.04951637e-01, 2.85184471e-02, 1.06335145e-02, 2.22395797e-02, 3.08540870e-02,
            1.03476559e-02, 7.89188645e-03, 8.39276860e-03, 8.17743569e-03, 9.22773982e-03,
            3.87795393e-02, 5.51588011e-03, 1.91936747e-02, 6.32414150e-03, 8.85957779e-03,
            8.40883775e-03, 1.08345701e-01, 6.77118074e-03, 1.18834931e-02, 9.77285521e-03,
            1.15349869e-02, 5.03350774e-03, 4.59739931e-03, 3.63654695e-03, 7.03211476e-03,
            1.03023863e-02, 9.16830208e-03, 1.37347173e-02, 1.13859420e-02, 2.02250296e-02
        ], [
            2.16113439e-02, 1.11040090e+00, 2.35899462e-02, 2.55476001e-02, 2.14299018e-02,
            2.74811163e-02, 2.58051465e-02, 4.84111069e-01, 8.56942499e-02, 4.03213470e-02,
            1.21823897e-01, 8.94856413e-02, 3.31349534e-02, 2.56043331e-02, 2.96537905e-02,
            3.57762146e-02, 1.34219812e-01, 5.48593740e-02, 1.50206497e-02, 1.10857860e-02,
            1.03875275e-02, 1.67263973e-02, 4.12172195e-02, 4.52222033e-02, 3.98178835e-02,
            2.30090939e-02, 9.84389233e-03, 6.35321111e-03, 9.50619699e-03, 1.02106143e-02,
            1.59179080e-02, 1.21705716e-02, 1.57894179e-02, 1.59016675e-02, 7.56532803e-02
        ], [
            7.70565013e-02, 5.11248552e-03, 1.19158081e+00, 1.56342493e-02, 1.32896439e-01,
            2.03304446e-02, 1.19126478e-02, 6.40607691e-03, 2.14617749e-02, 1.39551107e-02,
            8.78414542e-03, 7.65763806e-03, 8.27088130e-03, 8.37208136e-03, 8.07547538e-03,
            1.12088501e-02, 5.20771030e-03, 7.97476749e-03, 7.01956088e-03, 1.02246555e-02,
            8.99546633e-03, 1.95432315e-01, 7.25081062e-03, 1.69024442e-02, 1.50255539e-02,
            1.20465808e-02, 6.21209640e-03, 5.91144815e-03, 2.55046392e-03, 8.31619002e-03,
            1.48297226e-02, 1.42913024e-02, 2.30562681e-02, 1.33640351e-02, 3.33094456e-03
        ], [
            6.30386033e-03, 5.07275782e-03, 7.52838563e-03, 1.38586020e+00, 9.93475762e-02,
            1.42429026e-02, 1.68144002e-02, 4.63732325e-03, 1.17455489e-02, 3.53485358e-02,
            1.21657229e-02, 7.41077720e-03, 9.25205953e-03, 8.68236655e-03, 1.80580380e-02,
            4.42211827e-02, 3.49282772e-03, 8.94543262e-03, 6.86192894e-03, 6.18446620e-03,
            6.90326380e-03, 7.52251912e-03, 5.70115222e-03, 8.10664131e-03, 5.48598294e-03,
            5.99916547e-03, 7.05123207e-03, 2.91828503e-03, 1.77281572e-03, 4.34084470e-03,
            7.85153982e-03, 2.96807240e-03, 8.05129957e-03, 1.04028242e-02, 1.45530549e-03
        ], [
            6.63817616e-04, 7.24022744e-04, 9.38415987e-04, 1.18136744e-02, 1.23641801e+00,
            2.15224956e-03, 2.70291984e-03, 6.82291956e-04, 1.06863188e-03, 2.50073787e-03,
            1.52842312e-03, 1.32838674e-03, 1.38660211e-03, 1.25992202e-03, 3.09189592e-03,
            5.45283117e-03, 6.01963153e-04, 9.56435023e-04, 1.18999537e-03, 9.56288319e-04,
            9.69986000e-04, 7.52507428e-04, 7.23193718e-04, 8.32276564e-04, 7.32286723e-04,
            6.94149063e-04, 1.26644492e-03, 3.61030447e-04, 1.99095676e-04, 5.48364625e-04,
            1.58682406e-03, 4.27919161e-04, 6.22339422e-04, 1.38790339e-03, 1.18162181e-04
        ], [
            4.44112703e-03, 5.31146671e-03, 6.22787485e-03, 6.29660996e-03, 7.22896348e-03,
            1.24859670e+00, 2.05504246e-02, 4.17179903e-03, 6.09439964e-03, 8.00044157e-03,
            1.29205399e-02, 9.06714442e-03, 8.77641308e-03, 6.95692948e-03, 9.87411990e-03,
            9.95469164e-02, 4.51656609e-03, 4.71972335e-02, 4.27024444e-03, 4.19405361e-03,
            4.07210357e-03, 5.92463410e-03, 4.67726400e-03, 4.46526366e-03, 3.95330307e-03,
            8.86956431e-03, 3.44144946e-03, 2.84093449e-03, 6.01716170e-03, 3.72225025e-03,
            5.04799325e-03, 4.22738496e-03, 4.01291243e-03, 6.96058666e-03, 1.40536738e-03
        ], [
            1.58303223e-02, 1.51977209e-02, 5.29752805e-02, 3.57327401e-02, 4.31519931e-02,
            3.63577635e-02, 1.32023862e+00, 1.60573919e-02, 4.83883714e-02, 4.93831944e-02,
            4.84607991e-02, 2.67778336e-02, 3.35472064e-02, 4.24942120e-02, 3.14227425e-02,
            5.15467355e-02, 1.58354825e-02, 2.63671423e-02, 2.41663390e-02, 2.91297442e-02,
            2.80142766e-02, 3.10495258e-02, 2.10533358e-02, 2.27548848e-02, 2.69730026e-02,
            3.65554799e-02, 2.81330162e-02, 3.35022208e-02, 1.06004832e-02, 4.22913151e-02,
            3.34988284e-02, 2.92687021e-02, 2.65999554e-02, 4.79722380e-02, 2.90663072e-03
        ], [
            2.16024949e-02, 2.31455969e-02, 1.84529153e-02, 1.88058266e-02, 1.70970287e-02,
            1.93743603e-02, 1.60826727e-02, 1.06435518e+00, 4.59385048e-02, 2.68857920e-02,
            3.10696599e-02, 2.47469228e-02, 1.59434283e-02, 1.40355660e-02, 1.49486645e-02,
            1.82162496e-02, 3.01411371e-02, 2.58953224e-02, 1.18184528e-02, 1.15096198e-02,
            9.77546276e-03, 1.22179266e-02, 6.23986199e-02, 7.52731736e-02, 6.78087262e-02,
            2.66689213e-02, 9.21182100e-03, 5.57573838e-03, 4.82171604e-03, 9.87993159e-03,
            1.40206453e-02, 7.22213612e-03, 1.11027909e-02, 1.33595648e-02, 2.33148487e-03
        ], [
            7.18350621e-02, 3.22029341e-02, 5.38654555e-02, 1.44392152e-01, 9.43628966e-02,
            7.14620505e-02, 7.72264126e-02, 4.09287050e-02, 1.31954947e+00, 3.10213429e-01,
            6.46094056e-02, 4.77036110e-02, 4.90646260e-02, 5.82624001e-02, 5.99081954e-02,
            8.45736213e-02, 2.11181753e-02, 4.14660846e-02, 2.88229643e-02, 1.40198046e-02,
            1.25589964e-02, 2.51232953e-02, 1.97873040e-02, 1.96086035e-02, 1.79308012e-02,
            1.92775437e-02, 1.23875898e-02, 8.25663898e-03, 7.94186715e-03, 1.58388875e-02,
            2.31055399e-02, 1.18193910e-02, 9.68588753e-02, 2.51767750e-02, 4.38437513e-03
        ], [
            1.30643488e-02, 1.75901879e-02, 2.99993388e-02, 2.39459936e-02, 5.96375301e-02,
            2.40897627e-02, 2.97040880e-02, 1.38463909e-02, 3.59405791e-02, 1.14156456e+00,
            2.44969963e-02, 2.27578225e-02, 4.76353753e-02, 4.91349660e-02, 7.03389839e-02,
            6.15711502e-02, 1.04131519e-02, 3.30794552e-02, 3.29590687e-02, 1.02175803e-02,
            1.04055900e-02, 1.40879072e-02, 2.13636354e-02, 1.38104061e-02, 1.29308141e-02,
            1.56581558e-02, 1.04338915e-02, 4.71400765e-03, 4.70207975e-03, 8.32913263e-03,
            1.05625207e-02, 4.98444326e-03, 1.40201733e-02, 1.04415280e-02, 2.05146348e-03
        ], [
            5.69235663e-03, 9.94182718e-03, 1.16557875e-02, 7.55993270e-03, 7.74091090e-03,
            1.75273295e-02, 6.33728150e-03, 8.55480589e-03, 1.39816693e-02, 1.41663156e-02,
            1.12873672e+00, 2.08878377e-02, 1.54264412e-02, 2.25471746e-02, 1.86616299e-02,
            1.59198932e-02, 8.29469087e-03, 9.68763310e-02, 8.26605207e-03, 4.15447859e-03,
            3.68864547e-03, 8.44903789e-03, 5.77123244e-03, 5.24594285e-03, 4.84281806e-03,
            6.42544436e-03, 4.98965136e-03, 2.49998491e-03, 7.93728953e-03, 3.97236722e-03,
            6.12731530e-03, 4.73238839e-03, 6.15820266e-03, 5.97139610e-03, 1.27880387e-03
        ], [
            2.39433128e-02, 6.00461667e-02, 4.66366045e-02, 3.46579924e-02, 4.35001449e-02,
            7.36254307e-02, 3.92506014e-02, 4.51458888e-02, 5.75733838e-02, 7.01497816e-02,
            8.06576396e-02, 1.49046477e+00, 2.97069533e-01, 1.67388582e-01, 2.46484798e-01,
            2.11790184e-01, 3.63795506e-02, 1.62081357e-01, 6.99404608e-02, 1.95876186e-02,
            1.76270011e-02, 2.52372356e-02, 3.64649728e-02, 4.35519070e-02, 3.97197233e-02,
            3.20238474e-02, 2.32841089e-02, 1.05921468e-02, 1.66813243e-02, 1.86640475e-02,
            2.81724318e-02, 1.35398855e-02, 2.12317423e-02, 2.33640082e-02, 6.68909997e-03
        ], [
            1.39519958e-02, 2.86504738e-02, 1.44030082e-02, 1.74019431e-02, 1.58776916e-02,
            2.10419409e-02, 1.69772425e-02, 2.08558853e-02, 1.96460848e-02, 2.30827387e-02,
            2.61287692e-02, 3.48110445e-02, 1.14047873e+00, 3.08761621e-02, 5.63104061e-02,
            2.36685051e-02, 1.77031654e-02, 2.98159796e-02, 3.46722088e-02, 8.56606171e-03,
            7.24654126e-03, 8.76007932e-03, 1.52923734e-02, 1.62715561e-02, 1.33800021e-02,
            1.27021895e-02, 8.34435120e-03, 4.72389362e-03, 4.95616052e-03, 7.61023186e-03,
            1.27744532e-02, 5.81460951e-03, 8.04030732e-03, 9.19165684e-03, 2.69394459e-03
        ], [
            1.19038461e-02, 2.36495959e-02, 1.95141485e-02, 2.37565175e-02, 2.24888593e-02,
            2.73006855e-02, 3.09647243e-02, 2.13104095e-02, 3.22596757e-02, 3.82171161e-02,
            3.00596305e-02, 4.79448970e-02, 1.32830835e-01, 1.36278129e+00, 1.18288560e-01,
            5.10665376e-02, 3.12294641e-02, 5.72430534e-02, 5.43966337e-02, 2.13267740e-02,
            1.96371440e-02, 1.70002487e-02, 2.40063927e-02, 2.35467106e-02, 2.42652115e-02,
            2.42901984e-02, 5.56595650e-02, 1.42490695e-02, 9.30143831e-03, 2.89906810e-02,
            2.93816207e-02, 1.46973808e-02, 3.71488407e-02, 2.29646148e-02, 3.35919268e-03
        ], [
            1.24089900e-02, 1.66522522e-02, 1.39487498e-02, 1.44046162e-02, 1.51438763e-02,
            1.64298757e-02, 1.22208208e-02, 1.37998642e-02, 1.34910456e-02, 1.54191269e-02,
            1.61664572e-02, 2.04427265e-02, 3.21332581e-02, 1.85306430e-02, 1.40524702e+00,
            1.85336129e-02, 1.11141532e-02, 1.69089541e-02, 1.28414085e-01, 1.29349688e-02,
            1.05379839e-02, 9.32248854e-03, 5.48950995e-02, 7.00037860e-02, 7.63450237e-02,
            2.95714771e-02, 1.06588577e-02, 5.64436356e-03, 4.18151756e-03, 1.01749092e-02,
            3.07468206e-02, 5.34642107e-03, 7.58633881e-03, 1.13599734e-02, 1.80218031e-03
        ], [
            2.30747709e-03, 3.38884157e-03, 3.83772539e-03, 1.00430193e-02, 7.06683623e-03,
            1.09016016e-02, 6.93834043e-03, 3.05794455e-03, 5.13663186e-03, 6.16701874e-03,
            6.77434803e-03, 1.37602783e-02, 9.15945818e-03, 7.36049718e-03, 1.18900795e-02,
            1.04416696e+00, 3.10292126e-03, 9.41181983e-03, 5.29447270e-03, 2.96239434e-03,
            3.20971994e-03, 4.72525421e-03, 3.78702426e-03, 3.66887902e-03, 3.69914428e-03,
            3.82838821e-03, 2.70048694e-03, 2.79729256e-03, 2.67338457e-03, 4.37452478e-03,
            6.03744557e-03, 4.72236073e-03, 5.15231829e-03, 8.11034605e-03, 4.32491740e-03
        ], [
            2.09605048e-02, 3.55220666e-02, 3.08669058e-02, 4.11353576e-02, 2.86305753e-02,
            3.74683239e-02, 4.22815086e-02, 3.69461723e-02, 5.47970729e-02, 4.85257917e-02,
            5.99537909e-02, 5.70411396e-02, 3.48604942e-02, 3.06459678e-02, 3.43057698e-02,
            3.53097015e-02, 1.12031737e+00, 2.61179225e-02, 2.30879055e-02, 1.41261522e-02,
            2.15497579e-02, 3.71016184e-02, 2.78660526e-02, 1.87735404e-02, 1.79821656e-02,
            2.58659953e-02, 1.72447889e-02, 1.20456261e-02, 1.50801254e-02, 1.52011662e-02,
            2.53887931e-02, 2.60468997e-02, 2.49507960e-02, 2.72693320e-02, 3.51314815e-03
        ], [
            8.53118905e-03, 1.32563694e-02, 1.06449546e-02, 1.07319540e-02, 1.03075797e-02,
            1.29199071e-02, 1.22527927e-02, 1.21840032e-02, 1.34234428e-02, 1.25234638e-02,
            1.61761324e-02, 1.47853925e-02, 1.19450467e-02, 1.13756460e-02, 1.14475228e-02,
            1.22522576e-02, 3.52101604e-02, 1.04579173e+00, 1.09469847e-02, 9.44378949e-03,
            1.08200202e-02, 1.26966114e-02, 1.70771689e-02, 1.33980637e-02, 1.13382256e-02,
            2.44912299e-02, 1.82404526e-02, 1.16619391e-02, 4.95499647e-02, 1.15360083e-02,
            2.55046707e-02, 1.84664663e-02, 1.33193358e-02, 1.95077165e-02, 4.17269857e-03
        ], [
            9.48822556e-03, 8.78421761e-03, 1.44792230e-02, 1.37752608e-02, 1.71194194e-02,
            1.39691031e-02, 1.15416306e-02, 1.03878988e-02, 1.40408739e-02, 1.42308823e-02,
            1.46510529e-02, 1.44656603e-02, 1.31928672e-02, 1.23175479e-02, 1.39144174e-02,
            1.44688259e-02, 1.42433162e-02, 1.45991951e-02, 1.01386781e+00, 8.19582156e-03,
            7.70334143e-03, 1.08644159e-02, 2.35607169e-02, 1.01425874e-02, 9.40325670e-03,
            1.33220434e-02, 8.53316402e-03, 5.81239275e-03, 4.37767175e-03, 9.79317373e-03,
            7.92238248e-03, 5.67872691e-03, 7.51885879e-03, 1.08534695e-02, 1.01899263e-03
        ], [
            6.08727074e-02, 4.87303218e-02, 1.10035032e-01, 1.07761584e-01, 1.33158302e-01,
            1.03759133e-01, 9.24050583e-02, 7.33045943e-02, 9.46691183e-02, 9.88351572e-02,
            8.06632293e-02, 9.24138302e-02, 9.57674406e-02, 9.95778906e-02, 1.04278957e-01,
            1.03365858e-01, 4.61745733e-02, 8.22793608e-02, 6.58927639e-02, 1.04612214e+00,
            2.78522567e-02, 7.76399098e-02, 4.73709269e-02, 5.04910674e-02, 4.84974134e-02,
            4.60739568e-02, 2.74600706e-02, 1.76372465e-02, 1.41131689e-02, 2.94671147e-02,
            4.07607788e-02, 2.23258698e-02, 5.11210500e-02, 3.53397533e-02, 5.89492030e-03
        ], [
            2.92137385e-02, 2.04700774e-02, 5.23851531e-02, 5.23433846e-02, 7.04823630e-02,
            4.27386843e-02, 3.63083432e-02, 2.92816551e-02, 4.25224956e-02, 4.35557504e-02,
            3.59453596e-02, 3.65781853e-02, 3.70442384e-02, 3.54979396e-02, 4.44170213e-02,
            4.84457458e-02, 1.90969239e-02, 4.62895729e-02, 2.97694536e-02, 1.58697029e-02,
            1.01530464e+00, 4.16275643e-02, 2.69436412e-02, 2.38865089e-02, 2.27251138e-02,
            2.22302764e-02, 1.91968630e-02, 1.10247129e-02, 9.00684247e-03, 1.79028328e-02,
            1.73091219e-02, 1.16370337e-02, 2.22844614e-02, 2.27471820e-02, 3.14376707e-03
        ], [
            6.08700069e-03, 9.46313779e-03, 1.21364442e-02, 1.35194292e-02, 1.37718426e-02,
            1.34400567e-02, 1.49402785e-02, 9.39323342e-03, 1.46219030e-02, 1.52308362e-02,
            1.42957504e-02, 1.47181933e-02, 1.62753742e-02, 1.65723792e-02, 1.36937405e-02,
            1.42323805e-02, 1.10224901e-02, 1.36380436e-02, 1.29865119e-02, 1.47249468e-02,
            1.11861706e-02, 1.01683625e+00, 1.51495677e-02, 1.72639756e-02, 3.39294063e-02,
            3.25199145e-02, 1.31502969e-02, 1.71671678e-02, 6.19860297e-03, 2.03343704e-02,
            1.55600584e-02, 1.08564779e-02, 1.42806935e-02, 1.82566677e-02, 1.11750403e-03
        ], [
            2.79186244e-02, 4.06421273e-02, 4.59835304e-02, 4.44503830e-02, 4.83128951e-02,
            5.02433166e-02, 3.92077880e-02, 5.74635226e-02, 4.36245858e-02, 4.21231042e-02,
            5.77261710e-02, 4.39238917e-02, 3.42582698e-02, 2.95058684e-02, 3.70970836e-02,
            4.43809659e-02, 4.37341005e-02, 3.89652511e-02, 2.76356407e-02, 2.70487985e-02,
            1.86772982e-02, 2.63548307e-02, 1.05966925e+00, 5.60511013e-02, 3.03651345e-02,
            4.97562369e-02, 1.96535944e-02, 1.39897032e-02, 7.71931336e-03, 1.55242853e-02,
            2.19811920e-02, 1.73979901e-02, 1.90333269e-02, 2.08405990e-02, 4.59089900e-03
        ], [
            5.99162709e-03, 8.05018736e-03, 9.09929342e-03, 9.05912930e-03, 1.10995122e-02,
            1.43743231e-02, 6.24849113e-03, 1.17120553e-02, 7.84100570e-03, 7.37661837e-03,
            1.08765782e-02, 1.15612873e-02, 6.72257189e-03, 5.82774205e-03, 7.03831322e-03,
            7.55174089e-03, 5.73775977e-03, 7.86344868e-03, 3.74327875e-03, 4.15634468e-03,
            2.86419607e-03, 4.87476753e-03, 7.09065894e-03, 1.13623159e+00, 6.94511141e-03,
            8.69470293e-03, 2.75116824e-03, 1.84106102e-03, 1.19422894e-03, 2.54002827e-03,
            3.20451433e-03, 2.77784628e-03, 2.60349517e-03, 3.11471499e-03, 8.29809135e-04
        ], [
            2.72960073e-03, 4.49097136e-03, 4.93996786e-03, 5.48256878e-03, 6.96506335e-03,
            4.86004878e-03, 5.81291744e-03, 4.24805264e-03, 5.77864569e-03, 5.72651470e-03,
            4.76267305e-03, 5.12219291e-03, 5.54580205e-03, 5.94362194e-03, 6.10837675e-03,
            5.44876863e-03, 3.20701467e-03, 4.39991985e-03, 4.88976745e-03, 6.19350600e-03,
            3.66831033e-03, 3.87572903e-03, 5.44845035e-03, 1.05079589e-02, 1.04123699e+00,
            2.62642957e-02, 7.93980673e-03, 6.07286352e-03, 1.82053897e-03, 7.02122864e-03,
            6.88463973e-03, 3.54871030e-03, 3.55004213e-03, 6.82289111e-03, 4.71286614e-04
        ], [
            1.16148619e-02, 1.59059703e-02, 2.03476023e-02, 1.78124594e-02, 2.07266766e-02,
            2.19574180e-02, 2.02338195e-02, 2.14357026e-02, 1.89187111e-02, 1.86780140e-02,
            2.26227962e-02, 1.96939531e-02, 1.72984731e-02, 1.60360416e-02, 1.89120734e-02,
            1.87734657e-02, 1.40685192e-02, 1.58295105e-02, 1.72307106e-02, 2.71744848e-02,
            1.67941805e-02, 1.84275436e-02, 5.28421849e-02, 1.77904886e-01, 1.24993687e-01,
            1.14583343e+00, 1.02416604e-02, 1.00100893e-02, 4.00959558e-03, 1.18154212e-02,
            1.13513412e-02, 7.12622590e-03, 1.00810784e-02, 1.34259957e-02, 1.67912093e-03
        ], [
            1.04433129e-02, 1.57849095e-02, 1.79856295e-02, 1.94708792e-02, 2.10214810e-02,
            1.89786391e-02, 2.78125067e-02, 1.61772753e-02, 2.26528097e-02, 2.13188465e-02,
            1.92151478e-02, 2.09069065e-02, 2.24699463e-02, 2.26592058e-02, 2.03188420e-02,
            2.32169986e-02, 1.62209044e-02, 2.34195695e-02, 2.36632950e-02, 2.91268719e-02,
            2.87844162e-02, 2.35062151e-02, 2.80257139e-02, 3.86136169e-02, 4.96174920e-02,
            4.05557294e-02, 1.11827254e+00, 3.99580398e-02, 1.08756339e-02, 3.44034718e-02,
            2.89115136e-02, 1.51037503e-02, 2.14114357e-02, 3.56934804e-02, 1.90104848e-03
        ], [
            4.60770610e-02, 6.24824667e-02, 6.02950076e-02, 6.97239532e-02, 6.59223040e-02,
            6.63638187e-02, 6.47016892e-02, 5.90001546e-02, 6.67170074e-02, 6.43284555e-02,
            6.54033614e-02, 6.81253046e-02, 6.49432883e-02, 6.05393702e-02, 6.26295729e-02,
            6.88977261e-02, 5.65499789e-02, 6.21242660e-02, 5.97471334e-02, 6.89853285e-02,
            6.24712398e-02, 5.88336809e-02, 8.02571443e-02, 8.83882167e-02, 8.36053663e-02,
            6.48205461e-02, 5.07840597e-02, 1.24032611e+00, 7.97124724e-02, 6.84878350e-02,
            5.12908714e-02, 3.10904532e-02, 5.17386954e-02, 6.66916136e-02, 1.06935368e-02
        ], [
            1.68883635e-02, 1.58125228e-02, 2.17904384e-02, 2.31640871e-02, 2.52033587e-02,
            2.24179887e-02, 2.56737900e-02, 1.73254801e-02, 2.19587702e-02, 2.30871420e-02,
            2.05046410e-02, 2.21777594e-02, 2.31382974e-02, 2.22286387e-02, 2.24590389e-02,
            2.65133335e-02, 1.73539754e-02, 2.82154620e-02, 3.73889862e-02, 3.79781082e-02,
            5.26408534e-02, 4.08878963e-02, 2.40078880e-02, 4.24297869e-02, 3.38467889e-02,
            3.80576495e-02, 2.43574536e-02, 3.83739628e-02, 1.03872715e+00, 3.63708384e-02,
            2.29280928e-02, 2.47412863e-02, 4.26731931e-02, 4.26557350e-02, 2.56474556e-03
        ], [
            5.00791767e-02, 7.43168269e-02, 1.01941945e-01, 9.81710930e-02, 9.87253565e-02,
            8.70228926e-02, 1.35124520e-01, 7.60300809e-02, 1.43108976e-01, 1.14415988e-01,
            9.52235824e-02, 9.92031096e-02, 1.17765896e-01, 1.41694601e-01, 1.23804359e-01,
            1.11649507e-01, 8.67619386e-02, 1.21049243e-01, 9.91554938e-02, 9.63950787e-02,
            1.04291277e-01, 8.71986335e-02, 9.31667345e-02, 1.19044008e-01, 1.72435237e-01,
            1.32620588e-01, 1.22163191e-01, 1.50082107e-01, 6.59545995e-02, 1.19039056e+00,
            1.04224115e-01, 5.43646634e-02, 1.02312789e-01, 1.21222527e-01, 8.29113744e-03
        ], [
            3.62310438e-03, 5.38543234e-03, 5.09872284e-03, 4.38458370e-03, 4.21757767e-03,
            6.97324147e-03, 6.60507999e-03, 4.69786257e-03, 6.92068084e-03, 5.91817130e-03,
            5.54620496e-03, 6.33832070e-03, 4.61193221e-03, 4.74750527e-03, 4.89259565e-03,
            5.95198738e-03, 8.79570110e-03, 4.50532768e-03, 4.40530991e-03, 3.29780414e-03,
            5.09494106e-03, 5.99849663e-03, 5.36991446e-03, 1.18923182e-02, 1.32018548e-02,
            6.45664266e-03, 4.84512558e-03, 3.48099203e-03, 4.45035492e-03, 4.77357322e-03,
            1.00966595e+00, 5.14479613e-03, 7.32552624e-03, 1.01331927e-02, 5.20558433e-04
        ], [
            1.54416028e-03, 1.26066400e-03, 1.59337024e-03, 1.51554215e-03, 1.62396428e-03,
            1.38754024e-03, 1.77422378e-03, 1.47482057e-03, 2.08954217e-03, 1.74615887e-03,
            1.53816118e-03, 1.63906835e-03, 1.96982163e-03, 2.27385259e-03, 1.82001572e-03,
            1.48615798e-03, 1.95140598e-03, 1.50542686e-03, 1.33606363e-03, 1.50040223e-03,
            2.51346359e-03, 1.30531460e-03, 1.95754727e-03, 1.48742493e-03, 2.89781646e-03,
            2.09751418e-03, 2.42486521e-03, 2.18770744e-03, 6.27016725e-04, 2.96009067e-03,
            5.17126161e-03, 1.00689732e+00, 1.79671247e-03, 2.26834669e-03, 1.44049227e-04
        ], [
            2.70048183e-03, 9.09966724e-04, 1.77150555e-03, 1.33778456e-03, 1.74236543e-03,
            1.46133346e-03, 1.22668075e-03, 9.29163792e-04, 1.27067701e-03, 1.20295683e-03,
            1.12015174e-03, 1.17064642e-03, 1.33605666e-03, 1.22405756e-03, 1.16604203e-03,
            1.31718991e-03, 9.70416127e-04, 1.23433289e-03, 1.10729159e-03, 1.07335989e-03,
            8.70030042e-04, 1.79202995e-03, 1.33835876e-03, 1.26692975e-03, 1.31118048e-03,
            1.38779288e-03, 1.18122134e-03, 1.12715282e-03, 3.88979572e-04, 1.31366476e-03,
            6.91749839e-03, 1.80368707e-03, 1.02353856e+00, 2.28969718e-03, 1.28399836e-04
        ], [
            9.93781123e-03, 1.51402823e-02, 1.92848093e-02, 1.90544488e-02, 2.14918251e-02,
            1.89281524e-02, 3.07148868e-02, 1.60643917e-02, 2.42980927e-02, 2.13152828e-02,
            1.88129847e-02, 2.23168068e-02, 2.14535299e-02, 2.12364876e-02, 1.99264741e-02,
            2.18008556e-02, 1.63487757e-02, 1.98095348e-02, 2.03001487e-02, 1.76102584e-02,
            1.92055164e-02, 2.42875543e-02, 1.99686616e-02, 2.09425554e-02, 2.42823725e-02,
            2.50578590e-02, 3.12836673e-02, 2.24870486e-02, 1.42108696e-02, 4.15888204e-02,
            2.58811299e-02, 1.74647328e-02, 2.58475161e-02, 1.09502944e+00, 2.13939336e-03
        ], [
            5.24861716e-05, 5.66493563e-05, 9.92561096e-05, 1.14745717e-04, 1.05629029e-04,
            1.09931762e-04, 1.12072400e-04, 6.50933172e-05, 1.08469806e-04, 1.14017737e-04,
            8.81173032e-05, 1.05686449e-04, 1.02103658e-04, 8.81892415e-05, 1.19873006e-04,
            1.19868502e-04, 6.48732117e-05, 1.70632249e-04, 1.91259816e-04, 8.22296070e-05,
            1.06593076e-04, 1.32987374e-04, 2.27116244e-04, 9.88017422e-05, 1.05932713e-04,
            1.03438127e-04, 1.45725660e-04, 1.32576924e-04, 3.38133028e-04, 1.40874956e-04,
            1.49868442e-04, 8.52774376e-05, 2.11510328e-04, 1.95882925e-04, 1.00003360e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_1999():
    """
    Leontief matrix 1999



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17714229e+00, 7.53359026e-03, 3.69508268e-01, 7.40367861e-02, 9.63099284e-02,
            2.19506266e-01, 2.87281512e-02, 9.94342628e-03, 2.10753079e-02, 2.99165162e-02,
            9.86152204e-03, 7.67112252e-03, 8.42826106e-03, 7.74064612e-03, 9.20129837e-03,
            3.70771672e-02, 5.56107316e-03, 2.12676459e-02, 6.20104617e-03, 8.65934694e-03,
            8.25433361e-03, 1.07622113e-01, 6.98278287e-03, 1.18025533e-02, 9.34458262e-03,
            1.23639110e-02, 5.31136908e-03, 4.57926352e-03, 3.69530583e-03, 6.89782932e-03,
            1.04057681e-02, 9.26027079e-03, 1.34749399e-02, 1.17555181e-02, 2.13192958e-02
        ], [
            2.26136455e-02, 1.11049499e+00, 2.42037458e-02, 2.77917955e-02, 2.30367433e-02,
            2.81675761e-02, 2.65812238e-02, 5.12177607e-01, 8.92431379e-02, 4.20117373e-02,
            1.19930683e-01, 9.18189101e-02, 3.47912693e-02, 2.51113559e-02, 3.05792933e-02,
            3.11419524e-02, 1.42760850e-01, 5.56445276e-02, 1.55084603e-02, 1.16497153e-02,
            1.10006416e-02, 1.70250566e-02, 4.26981139e-02, 5.14033227e-02, 4.54445295e-02,
            2.32244761e-02, 1.05916919e-02, 6.64893251e-03, 9.30872687e-03, 1.08926108e-02,
            1.64075736e-02, 1.33267723e-02, 1.63146331e-02, 1.59707448e-02, 7.98615534e-02
        ], [
            7.50135849e-02, 4.86269404e-03, 1.19083923e+00, 1.56872233e-02, 1.30466786e-01,
            2.03877004e-02, 1.14563202e-02, 5.90473038e-03, 2.09715605e-02, 1.35638645e-02,
            8.18711076e-03, 7.22450653e-03, 8.08284395e-03, 7.69725352e-03, 7.79453171e-03,
            1.02228676e-02, 5.28583844e-03, 7.71286781e-03, 6.68478724e-03, 9.69829833e-03,
            8.77757335e-03, 1.89368142e-01, 6.74413622e-03, 1.72150095e-02, 1.36857874e-02,
            1.11283440e-02, 6.45680910e-03, 5.72677996e-03, 2.46383163e-03, 8.07859090e-03,
            1.42186990e-02, 1.37218815e-02, 2.22352034e-02, 1.37218428e-02, 3.45398644e-03
        ], [
            5.96820245e-03, 4.69864747e-03, 7.08543090e-03, 1.39823135e+00, 1.01128494e-01,
            1.21067999e-02, 1.63686874e-02, 4.36474143e-03, 1.10042004e-02, 3.17305640e-02,
            1.13703534e-02, 6.88668997e-03, 9.05625222e-03, 8.02634069e-03, 1.80298204e-02,
            3.73900122e-02, 3.31294890e-03, 8.20135968e-03, 6.60630612e-03, 5.83050030e-03,
            6.62226332e-03, 7.15844765e-03, 5.39982038e-03, 7.35268627e-03, 5.26511705e-03,
            5.67019541e-03, 6.52965675e-03, 2.74412226e-03, 1.67700054e-03, 4.19309906e-03,
            8.06014821e-03, 2.97255984e-03, 7.62598551e-03, 1.09980098e-02, 1.47671174e-03
        ], [
            5.79504111e-04, 5.87049905e-04, 8.19583698e-04, 1.24145731e-02, 1.23464927e+00,
            1.68422739e-03, 2.55158422e-03, 5.57760989e-04, 9.45451773e-04, 2.29142170e-03,
            1.31393253e-03, 1.10777049e-03, 1.21220299e-03, 1.05338168e-03, 3.00863150e-03,
            5.12280899e-03, 5.30942902e-04, 8.27513252e-04, 1.12504900e-03, 8.37182111e-04,
            8.46533327e-04, 6.61954933e-04, 6.67961403e-04, 7.68195196e-04, 6.69684334e-04,
            6.31832122e-04, 1.10840251e-03, 3.26456358e-04, 1.78856786e-04, 5.01953832e-04,
            1.49632400e-03, 4.01756143e-04, 5.57787410e-04, 1.27897496e-03, 1.10910720e-04
        ], [
            4.53845398e-03, 4.98075418e-03, 6.13927150e-03, 6.02691198e-03, 6.91933585e-03,
            1.24930112e+00, 2.05189261e-02, 3.88187001e-03, 5.87105658e-03, 8.12503059e-03,
            1.21385541e-02, 8.88868957e-03, 8.70521997e-03, 6.54379961e-03, 9.84187282e-03,
            9.49594470e-02, 4.45639199e-03, 4.66956654e-02, 4.13787587e-03, 4.03039811e-03,
            3.90245101e-03, 5.80799203e-03, 4.55238243e-03, 4.27636817e-03, 3.80120322e-03,
            8.48746434e-03, 3.55391413e-03, 2.72945945e-03, 5.84110077e-03, 3.65010998e-03,
            5.00199757e-03, 4.62069180e-03, 3.86552915e-03, 6.92632285e-03, 1.43990901e-03
        ], [
            1.54107765e-02, 1.44514822e-02, 5.25396084e-02, 3.69082906e-02, 4.40175328e-02,
            3.44882612e-02, 1.32153253e+00, 1.55726836e-02, 4.73640716e-02, 4.82360024e-02,
            4.71904349e-02, 2.58849796e-02, 3.36590949e-02, 4.09742337e-02, 3.09667252e-02,
            4.52651363e-02, 1.62759115e-02, 2.53506107e-02, 2.39431853e-02, 2.90504867e-02,
            2.78706379e-02, 2.99600060e-02, 2.06316739e-02, 2.18030157e-02, 2.59556770e-02,
            3.53544188e-02, 2.87978133e-02, 3.36025790e-02, 1.04215345e-02, 4.26990614e-02,
            3.37849572e-02, 3.09384674e-02, 2.61515411e-02, 4.73045413e-02, 2.97724243e-03
        ], [
            2.23907373e-02, 2.22968526e-02, 1.89124386e-02, 2.06274636e-02, 1.85502749e-02,
            1.95830543e-02, 1.63491174e-02, 1.06751933e+00, 4.65512658e-02, 2.69242922e-02,
            3.17731230e-02, 2.44542202e-02, 1.62941405e-02, 1.34671936e-02, 1.51517713e-02,
            1.62648590e-02, 3.06198491e-02, 2.57634374e-02, 1.19507426e-02, 1.17226899e-02,
            9.88761842e-03, 1.24887079e-02, 6.38614896e-02, 8.42486487e-02, 7.65091746e-02,
            2.74830396e-02, 9.65866130e-03, 5.78667694e-03, 4.78870224e-03, 1.02289750e-02,
            1.45267115e-02, 7.42618566e-03, 1.10707024e-02, 1.32000046e-02, 2.41067800e-03
        ], [
            6.99233073e-02, 2.96586548e-02, 5.26117079e-02, 1.47121324e-01, 9.77831650e-02,
            7.00132456e-02, 7.65807668e-02, 3.85089219e-02, 1.31776782e+00, 3.11307834e-01,
            6.38653845e-02, 4.61178617e-02, 4.97044523e-02, 5.64438591e-02, 6.05306745e-02,
            7.24640446e-02, 2.11919939e-02, 4.04924068e-02, 2.87127599e-02, 1.38529163e-02,
            1.25301485e-02, 2.45056794e-02, 1.92002187e-02, 2.01181200e-02, 1.82847412e-02,
            1.88854032e-02, 1.27020997e-02, 8.35899048e-03, 7.71119253e-03, 1.61872226e-02,
            2.29294007e-02, 1.19273469e-02, 9.68201366e-02, 2.56288038e-02, 4.35952167e-03
        ], [
            1.30971660e-02, 1.68568478e-02, 3.09767756e-02, 2.54541191e-02, 6.39461668e-02,
            2.28863214e-02, 3.00331221e-02, 1.37105877e-02, 3.65071615e-02, 1.14526125e+00,
            2.38850550e-02, 2.20749867e-02, 4.88692235e-02, 4.76995145e-02, 7.19165162e-02,
            5.34085200e-02, 1.06424094e-02, 3.26805396e-02, 3.33866858e-02, 1.03818541e-02,
            1.05969538e-02, 1.40972513e-02, 2.06651598e-02, 1.40968700e-02, 1.32664575e-02,
            1.52942005e-02, 1.05054113e-02, 4.78344653e-03, 4.73875673e-03, 8.56254340e-03,
            1.07001226e-02, 5.11377808e-03, 1.40256239e-02, 1.05585090e-02, 2.08144031e-03
        ], [
            5.81081154e-03, 9.58347734e-03, 1.20448235e-02, 7.98305649e-03, 8.13991512e-03,
            1.68684658e-02, 6.33056182e-03, 8.69003382e-03, 1.41152817e-02, 1.44727854e-02,
            1.13193927e+00, 2.08830664e-02, 1.61222811e-02, 2.19443857e-02, 1.91339702e-02,
            1.42634471e-02, 8.40525198e-03, 9.63830218e-02, 8.43869915e-03, 4.09655806e-03,
            3.76189078e-03, 8.44291055e-03, 5.78296587e-03, 5.34859126e-03, 4.89851264e-03,
            6.38928372e-03, 5.25794942e-03, 2.57637505e-03, 7.93634184e-03, 4.11331749e-03,
            6.11447043e-03, 5.05369825e-03, 6.22341579e-03, 6.06661152e-03, 1.31924287e-03
        ], [
            2.28681020e-02, 5.76990158e-02, 4.42848146e-02, 3.45912319e-02, 4.36420219e-02,
            6.79925995e-02, 3.76237840e-02, 4.47348215e-02, 5.57431740e-02, 6.91997131e-02,
            7.79043666e-02, 1.47666759e+00, 2.98334067e-01, 1.57260497e-01, 2.43914643e-01,
            1.68802437e-01, 3.55683513e-02, 1.60270739e-01, 6.93730825e-02, 1.88186626e-02,
            1.71660517e-02, 2.41496120e-02, 3.51702476e-02, 4.17382177e-02, 3.76367032e-02,
            3.03213568e-02, 2.38972871e-02, 1.04936285e-02, 1.61329934e-02, 1.88225562e-02,
            2.72110492e-02, 1.34697439e-02, 2.03662032e-02, 2.29017963e-02, 6.67388425e-03
        ], [
            1.32170392e-02, 2.58500480e-02, 1.40784677e-02, 1.81597695e-02, 1.63352341e-02,
            1.96098971e-02, 1.61858200e-02, 2.02136764e-02, 1.89580845e-02, 2.33043476e-02,
            2.64270803e-02, 3.39494372e-02, 1.14168077e+00, 2.97388689e-02, 5.79733663e-02,
            2.07051478e-02, 1.67584232e-02, 2.96834774e-02, 3.37652232e-02, 8.33722122e-03,
            6.97804211e-03, 8.43695740e-03, 1.43377950e-02, 1.70262405e-02, 1.30896070e-02,
            1.16799658e-02, 8.37606298e-03, 4.44223075e-03, 4.67469264e-03, 7.65020085e-03,
            1.22631449e-02, 5.66356742e-03, 7.90709114e-03, 9.06859283e-03, 2.61490946e-03
        ], [
            1.26996420e-02, 2.36866178e-02, 2.08549270e-02, 2.67987145e-02, 2.52195817e-02,
            2.80416310e-02, 3.32718564e-02, 2.26344032e-02, 3.45542634e-02, 4.18177089e-02,
            3.24378954e-02, 5.06172609e-02, 1.48072540e-01, 1.39275828e+00, 1.27738880e-01,
            4.89242588e-02, 3.38118589e-02, 6.03536178e-02, 5.89985023e-02, 2.28263558e-02,
            2.14889117e-02, 1.82470570e-02, 2.55217526e-02, 2.55953942e-02, 2.60907837e-02,
            2.56445975e-02, 6.62318633e-02, 1.60447484e-02, 9.96589912e-03, 3.20918452e-02,
            3.10724247e-02, 1.57583712e-02, 3.90340815e-02, 2.57042571e-02, 3.61169591e-03
        ], [
            1.22151617e-02, 1.67001201e-02, 1.41549901e-02, 1.45683388e-02, 1.58907371e-02,
            1.60072872e-02, 1.24455362e-02, 1.39557176e-02, 1.35577282e-02, 1.58540898e-02,
            1.60868923e-02, 1.99163917e-02, 3.21637344e-02, 1.80924557e-02, 1.41281265e+00,
            1.67023065e-02, 1.12204121e-02, 1.65222653e-02, 1.34047487e-01, 1.31968748e-02,
            1.08624841e-02, 9.25987011e-03, 5.53777387e-02, 7.18711579e-02, 7.71732571e-02,
            2.70668553e-02, 1.11802103e-02, 5.97016303e-03, 4.31845247e-03, 1.09683697e-02,
            2.97254010e-02, 5.70947189e-03, 7.57783338e-03, 1.14991980e-02, 1.89738693e-03
        ], [
            2.27824022e-03, 3.33767710e-03, 3.83641962e-03, 9.79075198e-03, 7.07514140e-03,
            9.92323809e-03, 6.97872194e-03, 3.01701451e-03, 4.96231905e-03, 6.14765125e-03,
            6.46848681e-03, 1.40856148e-02, 9.32208018e-03, 7.18436902e-03, 1.19178267e-02,
            1.04320761e+00, 3.17661042e-03, 9.22928941e-03, 5.31346005e-03, 2.85689977e-03,
            3.09823182e-03, 4.67715779e-03, 3.81655632e-03, 3.53036888e-03, 3.67334106e-03,
            3.80686958e-03, 2.78520136e-03, 2.70050902e-03, 2.89311106e-03, 3.99347321e-03,
            5.91724398e-03, 4.69855535e-03, 4.82823167e-03, 7.77144530e-03, 4.55475157e-03
        ], [
            2.14266385e-02, 3.72775643e-02, 3.10131074e-02, 4.20385651e-02, 2.94072247e-02,
            3.69495265e-02, 4.31168979e-02, 3.83886321e-02, 5.42788053e-02, 4.91599487e-02,
            5.89817473e-02, 5.61068256e-02, 3.54496103e-02, 3.01278007e-02, 3.41620324e-02,
            3.04719832e-02, 1.10590845e+00, 2.63078671e-02, 2.30730913e-02, 1.44207037e-02,
            2.26930170e-02, 3.82674151e-02, 2.74911722e-02, 1.92486205e-02, 1.82966853e-02,
            2.59664180e-02, 1.82456770e-02, 1.26298746e-02, 1.64755914e-02, 1.61147916e-02,
            2.62779445e-02, 2.62744608e-02, 2.51279235e-02, 2.71458830e-02, 3.82249768e-03
        ], [
            8.69707969e-03, 1.35027230e-02, 1.05305197e-02, 1.10557487e-02, 1.04572175e-02,
            1.21892708e-02, 1.20202009e-02, 1.20361522e-02, 1.30957284e-02, 1.23258462e-02,
            1.64595353e-02, 1.44485874e-02, 1.24032772e-02, 1.11867150e-02, 1.15269662e-02,
            1.06213474e-02, 3.49720588e-02, 1.04716761e+00, 1.08164282e-02, 9.44238344e-03,
            1.07872590e-02, 1.27228307e-02, 1.66655310e-02, 1.29572289e-02, 1.09305329e-02,
            2.40540813e-02, 1.83375620e-02, 1.17979565e-02, 4.87535234e-02, 1.14420453e-02,
            2.55273149e-02, 1.98052797e-02, 1.32319139e-02, 1.88294865e-02, 4.41997387e-03
        ], [
            9.37158570e-03, 8.74047759e-03, 1.44880060e-02, 1.40075505e-02, 1.70928478e-02,
            1.36944532e-02, 1.16992217e-02, 1.00175507e-02, 1.39295664e-02, 1.42918463e-02,
            1.47001908e-02, 1.42197521e-02, 1.34213643e-02, 1.18828137e-02, 1.40935707e-02,
            1.33531298e-02, 1.44573291e-02, 1.44191276e-02, 1.01415468e+00, 8.08647281e-03,
            7.67680217e-03, 1.06921058e-02, 2.36726765e-02, 1.00493106e-02, 9.27437659e-03,
            1.34346893e-02, 8.86958844e-03, 6.12429657e-03, 4.44310654e-03, 9.98001678e-03,
            8.32101510e-03, 5.79077765e-03, 7.60132192e-03, 1.10729474e-02, 1.06672636e-03
        ], [
            5.98175822e-02, 4.97607046e-02, 1.11417047e-01, 1.12832497e-01, 1.36082637e-01,
            1.01686085e-01, 9.24112855e-02, 7.27604296e-02, 9.40017589e-02, 9.90922588e-02,
            8.08870446e-02, 9.35412380e-02, 9.88408686e-02, 9.67274957e-02, 1.06913756e-01,
            9.16571608e-02, 4.61574291e-02, 8.20532299e-02, 6.64072777e-02, 1.04513021e+00,
            2.76261783e-02, 7.77808123e-02, 4.68302167e-02, 5.14402772e-02, 4.97185531e-02,
            4.54677631e-02, 2.89800499e-02, 1.77595697e-02, 1.41059274e-02, 3.00061030e-02,
            3.93561968e-02, 2.29064259e-02, 5.03908692e-02, 3.63481335e-02, 6.23281125e-03
        ], [
            2.88584606e-02, 2.04482718e-02, 5.28925104e-02, 5.32577504e-02, 7.00506415e-02,
            4.23165177e-02, 3.64456948e-02, 2.69360756e-02, 4.20941341e-02, 4.29874438e-02,
            3.57967438e-02, 3.62306048e-02, 3.76743664e-02, 3.43875267e-02, 4.53282367e-02,
            4.44472315e-02, 1.89132930e-02, 4.56934875e-02, 2.96813600e-02, 1.53149947e-02,
            1.01518353e+00, 4.11013051e-02, 2.64943830e-02, 2.34862524e-02, 2.25137047e-02,
            2.16490639e-02, 1.89034847e-02, 1.11575303e-02, 8.74294846e-03, 1.81955320e-02,
            1.74045097e-02, 1.18075525e-02, 2.20490588e-02, 2.24542338e-02, 3.29197924e-03
        ], [
            6.05144049e-03, 9.42776773e-03, 1.20445243e-02, 1.37400713e-02, 1.40125410e-02,
            1.28739741e-02, 1.49487301e-02, 9.40084033e-03, 1.43951828e-02, 1.52037353e-02,
            1.38603400e-02, 1.41885257e-02, 1.63821565e-02, 1.55705075e-02, 1.34408234e-02,
            1.24349226e-02, 1.25649635e-02, 1.32645138e-02, 1.26067937e-02, 1.47527370e-02,
            1.13135413e-02, 1.01692579e+00, 1.51969290e-02, 1.61955720e-02, 3.16115652e-02,
            2.99596166e-02, 1.48964380e-02, 1.72636234e-02, 6.11346721e-03, 2.09740442e-02,
            1.64808802e-02, 1.07232498e-02, 1.41184310e-02, 1.87214697e-02, 1.16796179e-03
        ], [
            2.81332218e-02, 3.81070074e-02, 4.71018433e-02, 4.61549333e-02, 4.95996666e-02,
            4.91944798e-02, 3.95273305e-02, 5.47106992e-02, 4.28551974e-02, 4.19177057e-02,
            5.67015996e-02, 4.27774345e-02, 3.50118941e-02, 2.87510739e-02, 3.76865718e-02,
            3.92885672e-02, 4.33696671e-02, 3.86650373e-02, 2.70458843e-02, 2.64273983e-02,
            1.89070874e-02, 2.65074899e-02, 1.06146212e+00, 6.64418899e-02, 3.23016306e-02,
            5.16385403e-02, 1.97652396e-02, 1.37694284e-02, 7.58780810e-03, 1.55987136e-02,
            2.18845928e-02, 1.82660667e-02, 1.89100830e-02, 2.07884996e-02, 4.64985132e-03
        ], [
            6.27625196e-03, 8.89996575e-03, 9.88434894e-03, 9.19539009e-03, 1.13376616e-02,
            1.39443167e-02, 7.17347528e-03, 1.26300957e-02, 8.31043400e-03, 8.12335196e-03,
            1.20273405e-02, 1.24350322e-02, 7.82063118e-03, 6.37780276e-03, 7.60887747e-03,
            6.89311466e-03, 6.33291413e-03, 8.79772005e-03, 3.88049334e-03, 4.45364695e-03,
            3.11869811e-03, 5.24688958e-03, 6.32976090e-03, 1.12487772e+00, 7.21440482e-03,
            8.91080383e-03, 2.76556915e-03, 1.82702828e-03, 1.30482153e-03, 2.67888309e-03,
            3.41076236e-03, 2.87445045e-03, 2.65996631e-03, 3.32314564e-03, 9.43287379e-04
        ], [
            2.60845608e-03, 4.23160867e-03, 4.83118683e-03, 5.69341352e-03, 7.15962015e-03,
            4.78667436e-03, 5.83922727e-03, 4.11835579e-03, 5.73484558e-03, 5.79235720e-03,
            4.79600463e-03, 5.14118742e-03, 5.93323809e-03, 5.94173585e-03, 6.18571736e-03,
            4.67580369e-03, 3.47384520e-03, 4.35068963e-03, 4.75061186e-03, 6.18508066e-03,
            3.73211106e-03, 3.79525780e-03, 5.22637970e-03, 1.01561814e-02, 1.04095254e+00,
            2.53000923e-02, 7.56188593e-03, 6.16192001e-03, 1.84123907e-03, 7.26296792e-03,
            7.05640508e-03, 3.66929593e-03, 3.47623480e-03, 7.14231478e-03, 4.74077975e-04
        ], [
            1.17388577e-02, 1.52601358e-02, 2.06432301e-02, 1.81013697e-02, 2.10495742e-02,
            2.12685946e-02, 2.07365761e-02, 2.05420770e-02, 1.87516811e-02, 1.87434974e-02,
            2.24594380e-02, 1.93859078e-02, 1.78422546e-02, 1.55771424e-02, 1.93897561e-02,
            1.67505046e-02, 1.36085727e-02, 1.62329319e-02, 1.74707864e-02, 2.69720634e-02,
            1.72597263e-02, 1.81370679e-02, 5.28538745e-02, 1.73723438e-01, 1.28782080e-01,
            1.15164715e+00, 1.02302112e-02, 1.02933752e-02, 4.19750110e-03, 1.21097541e-02,
            1.20230873e-02, 7.33781409e-03, 9.98630736e-03, 1.38030078e-02, 1.72362900e-03
        ], [
            1.13290274e-02, 1.70504846e-02, 1.96847421e-02, 2.20883235e-02, 2.38617878e-02,
            2.02147301e-02, 3.07045003e-02, 1.75247956e-02, 2.49993250e-02, 2.35541008e-02,
            2.11899645e-02, 2.27691195e-02, 2.55059873e-02, 2.49240391e-02, 2.25705861e-02,
            2.22103416e-02, 1.81621586e-02, 2.60445080e-02, 2.55230228e-02, 3.18057997e-02,
            3.14674078e-02, 2.53439258e-02, 2.95695852e-02, 3.96143997e-02, 5.01303752e-02,
            4.34260603e-02, 1.13659951e+00, 4.45788851e-02, 1.19772988e-02, 3.80100602e-02,
            3.21087070e-02, 1.62832807e-02, 2.31192790e-02, 3.87017953e-02, 2.15546895e-03
        ], [
            4.89352041e-02, 6.47386763e-02, 6.35097565e-02, 7.34904911e-02, 6.98201201e-02,
            6.62101230e-02, 6.80428609e-02, 6.00379756e-02, 6.85890007e-02, 6.69515544e-02,
            6.79586008e-02, 6.87830391e-02, 6.85759542e-02, 6.12091599e-02, 6.54577866e-02,
            6.51766340e-02, 6.19942135e-02, 6.47548183e-02, 6.08299087e-02, 7.15853649e-02,
            6.64028233e-02, 6.18275998e-02, 8.51603189e-02, 9.39892835e-02, 8.47483291e-02,
            6.78526079e-02, 5.50652155e-02, 1.25639870e+00, 8.38720633e-02, 7.39604314e-02,
            5.47783111e-02, 3.33819771e-02, 5.40330891e-02, 7.28242389e-02, 1.15818727e-02
        ], [
            1.51597354e-02, 1.51977014e-02, 2.11710363e-02, 2.28101103e-02, 2.44683782e-02,
            2.15345942e-02, 2.52303685e-02, 1.63653814e-02, 2.14741644e-02, 2.28115136e-02,
            1.99047054e-02, 2.16029050e-02, 2.31640461e-02, 2.11589290e-02, 2.22778299e-02,
            2.32814390e-02, 1.74220602e-02, 2.70172582e-02, 3.61805032e-02, 3.69677492e-02,
            5.16000227e-02, 3.97255778e-02, 2.34715082e-02, 3.98176034e-02, 3.13466380e-02,
            3.73733508e-02, 2.53402279e-02, 3.88822550e-02, 1.03922110e+00, 3.62576730e-02,
            2.44344632e-02, 2.50086012e-02, 4.39632481e-02, 4.07366987e-02, 2.61394904e-03
        ], [
            5.02684929e-02, 7.32381448e-02, 1.03194927e-01, 1.02590416e-01, 1.02514175e-01,
            8.71712266e-02, 1.35769746e-01, 7.58255872e-02, 1.45705940e-01, 1.16840333e-01,
            9.65042458e-02, 9.97026943e-02, 1.21838767e-01, 1.40445661e-01, 1.25719042e-01,
            9.84522956e-02, 8.97602832e-02, 1.18935884e-01, 1.00741474e-01, 9.70675580e-02,
            1.05923562e-01, 8.87285628e-02, 9.38971758e-02, 1.14611734e-01, 1.67015587e-01,
            1.30829744e-01, 1.26108417e-01, 1.57083920e-01, 6.56319381e-02, 1.19505013e+00,
            1.09272786e-01, 5.56463244e-02, 1.03289202e-01, 1.25805528e-01, 8.65820327e-03
        ], [
            3.54106070e-03, 5.32658101e-03, 5.03766581e-03, 4.48452210e-03, 4.32078111e-03,
            6.79944597e-03, 6.55794132e-03, 4.87079211e-03, 6.76284442e-03, 5.92653093e-03,
            5.45331531e-03, 6.38090350e-03, 4.74136327e-03, 4.65741109e-03, 4.91759848e-03,
            5.13709399e-03, 9.04054270e-03, 4.49918215e-03, 4.51410695e-03, 3.25082193e-03,
            5.08071773e-03, 6.00014164e-03, 5.29539650e-03, 1.08620047e-02, 1.26321729e-02,
            6.41038409e-03, 5.08568628e-03, 3.64783360e-03, 4.58687975e-03, 4.99392501e-03,
            1.00935999e+00, 5.37715801e-03, 7.36996560e-03, 1.03339841e-02, 5.41221146e-04
        ], [
            1.47944698e-03, 1.29864566e-03, 1.64319710e-03, 1.62385475e-03, 1.76294382e-03,
            1.41164937e-03, 1.87865536e-03, 1.51740733e-03, 2.22351962e-03, 1.86588116e-03,
            1.61708034e-03, 1.68478933e-03, 2.11009799e-03, 2.33313418e-03, 1.93398606e-03,
            1.42630164e-03, 1.80952003e-03, 1.52220366e-03, 1.39216621e-03, 1.50262826e-03,
            2.44233228e-03, 1.37038038e-03, 2.02239823e-03, 1.51677256e-03, 2.94804644e-03,
            2.15399329e-03, 2.53228588e-03, 2.22229265e-03, 6.57233374e-04, 3.13917956e-03,
            5.20411092e-03, 1.00713151e+00, 1.89774036e-03, 2.42569793e-03, 1.54098078e-04
        ], [
            3.10625031e-03, 9.87354554e-04, 1.95565472e-03, 1.40111562e-03, 2.07342262e-03,
            1.54988846e-03, 1.24727891e-03, 9.44950759e-04, 1.30641456e-03, 1.22055700e-03,
            1.13151735e-03, 1.17531841e-03, 1.36260622e-03, 1.18944460e-03, 1.17985137e-03,
            1.24432959e-03, 9.32050353e-04, 1.25787726e-03, 1.06923459e-03, 1.07846924e-03,
            8.68516645e-04, 1.87853362e-03, 1.34621313e-03, 1.20289979e-03, 1.27265734e-03,
            1.35441479e-03, 1.20916228e-03, 1.10477101e-03, 3.83813109e-04, 1.33427805e-03,
            6.43414917e-03, 1.83963283e-03, 1.02390493e+00, 2.19779972e-03, 1.47813639e-04
        ], [
            9.84007538e-03, 1.50178298e-02, 1.90532006e-02, 1.97247784e-02, 2.21537642e-02,
            1.80061348e-02, 3.03182138e-02, 1.56432374e-02, 2.39555276e-02, 2.09598554e-02,
            1.85317195e-02, 2.14878018e-02, 2.14750244e-02, 2.01708544e-02, 1.97973789e-02,
            1.90396890e-02, 1.63290266e-02, 1.92082906e-02, 1.99341112e-02, 1.76626277e-02,
            1.90527495e-02, 2.41336364e-02, 1.93867769e-02, 2.00715542e-02, 2.33405921e-02,
            2.45895508e-02, 3.19266986e-02, 2.30824880e-02, 1.37054325e-02, 4.13665153e-02,
            2.61831224e-02, 1.71246868e-02, 2.56287428e-02, 1.09674250e+00, 2.23515158e-03
        ], [
            4.96034370e-05, 5.67035561e-05, 8.82714140e-05, 1.16518615e-04, 9.84335417e-05,
            9.32906428e-05, 1.01435484e-04, 6.25010945e-05, 9.68207649e-05, 1.03233533e-04,
            7.80715174e-05, 8.86568090e-05, 9.33352776e-05, 8.23259364e-05, 1.20141787e-04,
            8.35753750e-05, 6.19002465e-05, 1.39427913e-04, 1.57776609e-04, 6.69176356e-05,
            1.01670296e-04, 1.13410260e-04, 1.83976653e-04, 8.62207502e-05, 9.56493183e-05,
            8.99906946e-05, 1.23440560e-04, 1.26852812e-04, 3.24179480e-04, 1.30448814e-04,
            1.33176976e-04, 7.58603296e-05, 1.90681825e-04, 1.89017683e-04, 1.00003504e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2000():
    """
    Leontief matrix 2000



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17557246e+00, 6.02766542e-03, 3.60851853e-01, 7.11276783e-02, 9.77218124e-02,
            1.95749199e-01, 2.71552215e-02, 8.21416733e-03, 1.81265028e-02, 2.78203334e-02,
            8.88535993e-03, 7.09810986e-03, 7.59843642e-03, 6.84687835e-03, 8.48959009e-03,
            3.53615683e-02, 5.18248592e-03, 2.08856845e-02, 5.81357865e-03, 8.62341007e-03,
            7.58130127e-03, 1.02120942e-01, 6.61982658e-03, 1.15551032e-02, 8.32112824e-03,
            1.20438054e-02, 4.94042578e-03, 4.10081408e-03, 3.32518556e-03, 6.23844967e-03,
            1.00165229e-02, 8.72673770e-03, 1.27506795e-02, 1.14279885e-02, 1.92144286e-02
        ], [
            2.44486476e-02, 1.11904765e+00, 2.60637113e-02, 2.96537470e-02, 2.45236916e-02,
            3.19472187e-02, 2.79688034e-02, 5.38574025e-01, 8.79340087e-02, 4.36147870e-02,
            1.24100089e-01, 1.00017030e-01, 3.68916980e-02, 2.52624745e-02, 3.22660922e-02,
            3.47663253e-02, 1.62904819e-01, 5.88962193e-02, 1.61257418e-02, 1.16720816e-02,
            1.15300848e-02, 1.84119587e-02, 4.47906963e-02, 5.73584987e-02, 4.88731484e-02,
            2.42933981e-02, 1.12836470e-02, 6.97947723e-03, 9.60291156e-03, 1.15402608e-02,
            1.72862850e-02, 1.48804780e-02, 1.67386638e-02, 1.71568773e-02, 7.24939234e-02
        ], [
            7.68158023e-02, 4.41120574e-03, 1.19372836e+00, 1.58400576e-02, 1.33772807e-01,
            1.95806777e-02, 1.14618897e-02, 5.26252949e-03, 1.95842204e-02, 1.34885970e-02,
            7.79096027e-03, 6.97778224e-03, 7.63078404e-03, 7.17829174e-03, 7.78335322e-03,
            1.07902732e-02, 5.16906365e-03, 7.48999462e-03, 6.56951748e-03, 9.49333204e-03,
            8.42071986e-03, 1.84998061e-01, 6.38984014e-03, 1.81442472e-02, 1.21239227e-02,
            1.03190525e-02, 6.44437351e-03, 5.34821520e-03, 2.36087904e-03, 7.63064707e-03,
            1.46339614e-02, 1.32655911e-02, 2.21430229e-02, 1.40721472e-02, 4.81377800e-03
        ], [
            5.69178389e-03, 4.01296624e-03, 6.72574980e-03, 1.41056731e+00, 9.87694376e-02,
            1.15041591e-02, 1.39589237e-02, 3.79491843e-03, 9.98516376e-03, 2.79201949e-02,
            1.04118339e-02, 6.54648159e-03, 8.54356230e-03, 7.33786114e-03, 1.73101437e-02,
            4.03566395e-02, 2.99538299e-03, 7.87988778e-03, 6.07164337e-03, 5.36074302e-03,
            6.01438762e-03, 6.95837494e-03, 5.07504924e-03, 6.62802252e-03, 4.87897529e-03,
            5.23640930e-03, 3.79260301e-03, 2.40873123e-03, 1.54460737e-03, 3.83220989e-03,
            7.44926939e-03, 2.87996728e-03, 7.35896025e-03, 1.07475626e-02, 9.33365873e-04
        ], [
            5.13770009e-04, 4.52609289e-04, 7.21760505e-04, 1.12311473e-02, 1.23911337e+00,
            1.48915399e-03, 1.77184317e-03, 4.51795259e-04, 8.11767148e-04, 2.22464470e-03,
            1.11312791e-03, 9.62495980e-04, 1.05960675e-03, 9.02295779e-04, 3.12367906e-03,
            5.41583948e-03, 4.59653306e-04, 7.49819860e-04, 1.08408057e-03, 7.98307294e-04,
            6.94302063e-04, 5.69021480e-04, 6.27372678e-04, 6.98612761e-04, 6.13945634e-04,
            5.93762538e-04, 7.21981933e-04, 2.62521137e-04, 1.50964170e-04, 4.22517039e-04,
            1.22252879e-03, 3.56313537e-04, 5.10512680e-04, 1.01232479e-03, 8.02873079e-05
        ], [
            4.76573133e-03, 4.59040082e-03, 6.42248540e-03, 6.16809201e-03, 7.12331952e-03,
            1.26445104e+00, 1.98437812e-02, 3.64598179e-03, 5.71851200e-03, 8.21106792e-03,
            1.20837365e-02, 9.16099925e-03, 8.50880053e-03, 6.17273194e-03, 9.57370456e-03,
            1.01390690e-01, 4.37090311e-03, 4.65680987e-02, 4.07874637e-03, 4.02969111e-03,
            3.80283832e-03, 6.00894043e-03, 4.42734918e-03, 4.13683758e-03, 3.63231150e-03,
            7.73334593e-03, 3.54532469e-03, 2.57166063e-03, 5.68758903e-03, 3.60593877e-03,
            4.97863016e-03, 4.78993817e-03, 3.81456520e-03, 7.07699316e-03, 9.56802660e-04
        ], [
            1.51568484e-02, 1.36520700e-02, 5.34359329e-02, 3.65393198e-02, 4.33823018e-02,
            3.63312264e-02, 1.31826908e+00, 1.37783015e-02, 4.53323843e-02, 4.72271448e-02,
            4.56585454e-02, 2.50718402e-02, 3.19152180e-02, 3.78284019e-02, 2.93535377e-02,
            4.89996819e-02, 1.61868152e-02, 2.49983547e-02, 2.35469113e-02, 2.80623899e-02,
            2.70317934e-02, 2.99068496e-02, 2.01092911e-02, 2.04161122e-02, 2.45000442e-02,
            3.31548359e-02, 2.85587070e-02, 3.25324701e-02, 9.99263933e-03, 4.27739876e-02,
            3.30398200e-02, 3.04102152e-02, 2.51889212e-02, 4.73170030e-02, 2.23667799e-03
        ], [
            2.24627373e-02, 1.99897892e-02, 1.86174260e-02, 2.11300393e-02, 1.89080298e-02,
            2.03232182e-02, 1.62711715e-02, 1.05951780e+00, 4.88139097e-02, 2.73194910e-02,
            3.11263139e-02, 2.32984923e-02, 1.56292837e-02, 1.25609767e-02, 1.47214134e-02,
            1.67720060e-02, 2.96506608e-02, 2.48396015e-02, 1.15338471e-02, 1.16486311e-02,
            9.38803358e-03, 1.23338014e-02, 6.25732774e-02, 9.03118927e-02, 7.90125436e-02,
            2.76902071e-02, 9.11889477e-03, 5.60014540e-03, 4.34714834e-03, 1.00110224e-02,
            1.40695868e-02, 7.19064479e-03, 1.08047609e-02, 1.24566005e-02, 1.96222197e-03
        ], [
            6.89262173e-02, 2.70314264e-02, 5.22514148e-02, 1.47677197e-01, 9.83216553e-02,
            7.65030128e-02, 7.57523073e-02, 3.58856549e-02, 1.32317961e+00, 3.13711646e-01,
            6.42334212e-02, 4.55095424e-02, 4.71162177e-02, 5.42128925e-02, 5.77248004e-02,
            7.65141550e-02, 2.10980289e-02, 4.09391008e-02, 2.72846736e-02, 1.35237907e-02,
            1.21089376e-02, 2.46519640e-02, 1.89340103e-02, 2.05136644e-02, 1.94980685e-02,
            1.87630000e-02, 1.27634179e-02, 8.22311662e-03, 7.61166708e-03, 1.63377138e-02,
            2.34452832e-02, 1.19060306e-02, 9.41412665e-02, 2.58020046e-02, 3.64283375e-03
        ], [
            1.33980403e-02, 1.53675151e-02, 3.20457396e-02, 2.60922694e-02, 6.41150224e-02,
            2.39771937e-02, 3.01014690e-02, 1.31899044e-02, 3.57588861e-02, 1.14775923e+00,
            2.36263705e-02, 2.19577632e-02, 4.81041335e-02, 4.58508220e-02, 6.99586269e-02,
            5.65522685e-02, 1.06552160e-02, 3.32030442e-02, 3.23915736e-02, 1.03887504e-02,
            1.04178084e-02, 1.42320714e-02, 2.00605722e-02, 1.39296342e-02, 1.31361431e-02,
            1.53814227e-02, 1.06667645e-02, 4.68956057e-03, 4.62088315e-03, 8.56964849e-03,
            1.06317995e-02, 5.02860910e-03, 1.36036941e-02, 1.06376103e-02, 1.68360411e-03
        ], [
            6.08559917e-03, 8.98408090e-03, 1.26184245e-02, 8.22571799e-03, 8.49182192e-03,
            1.72593775e-02, 6.31830557e-03, 7.94387946e-03, 1.35403663e-02, 1.46378243e-02,
            1.13339882e+00, 2.10527994e-02, 1.60059054e-02, 2.16704347e-02, 1.91240563e-02,
            1.50607977e-02, 8.21826517e-03, 9.62082124e-02, 8.45827403e-03, 4.31154541e-03,
            3.81658623e-03, 8.78981207e-03, 5.65876084e-03, 5.35591708e-03, 4.88883822e-03,
            6.30457912e-03, 5.52654369e-03, 2.55981522e-03, 7.54896043e-03, 4.15104878e-03,
            6.12039199e-03, 5.08624320e-03, 6.26635417e-03, 6.23482029e-03, 1.01406139e-03
        ], [
            2.26599736e-02, 5.37841802e-02, 4.44865669e-02, 3.47111102e-02, 4.39601945e-02,
            7.06096222e-02, 3.77939806e-02, 4.27144690e-02, 5.12664976e-02, 6.99185938e-02,
            7.78640897e-02, 1.48045305e+00, 2.94299344e-01, 1.49154050e-01, 2.40571412e-01,
            1.75822791e-01, 3.48258050e-02, 1.61745839e-01, 6.72045921e-02, 1.89850406e-02,
            1.70016584e-02, 2.43695898e-02, 3.39493247e-02, 3.94971526e-02, 3.67252690e-02,
            3.06274229e-02, 2.45584871e-02, 1.02857321e-02, 1.55998653e-02, 1.87831149e-02,
            2.67637725e-02, 1.31961795e-02, 1.96199525e-02, 2.31149188e-02, 5.28569735e-03
        ], [
            1.36497671e-02, 2.40635496e-02, 1.48452844e-02, 1.92386011e-02, 1.67595820e-02,
            2.03878045e-02, 1.66111689e-02, 2.03577266e-02, 1.80386846e-02, 2.37638183e-02,
            2.75754929e-02, 3.38943015e-02, 1.14702701e+00, 2.96171218e-02, 6.00950473e-02,
            2.25021085e-02, 1.72474543e-02, 3.07912381e-02, 3.53232786e-02, 8.79909059e-03,
            6.96812238e-03, 8.83312288e-03, 1.45343910e-02, 1.88934767e-02, 1.35791985e-02,
            1.19836876e-02, 8.74833518e-03, 4.47505457e-03, 4.67263172e-03, 7.89793568e-03,
            1.28581190e-02, 5.64540228e-03, 7.97351397e-03, 9.41192871e-03, 2.15690767e-03
        ], [
            1.41639463e-02, 2.30231788e-02, 2.35416870e-02, 3.10666774e-02, 2.85843972e-02,
            3.16828832e-02, 3.76129388e-02, 2.30612302e-02, 3.70515117e-02, 4.64532497e-02,
            3.66096467e-02, 5.60913468e-02, 1.60834464e-01, 1.43480613e+00, 1.38206853e-01,
            6.06637417e-02, 3.74834646e-02, 6.75561310e-02, 6.54425817e-02, 2.59079855e-02,
            2.33888910e-02, 2.08935005e-02, 2.76641755e-02, 2.84982474e-02, 2.88649912e-02,
            2.88626930e-02, 7.95497137e-02, 1.79908215e-02, 1.08730501e-02, 3.54448328e-02,
            3.48773106e-02, 1.67342828e-02, 4.15446293e-02, 2.92562086e-02, 3.24954977e-03
        ], [
            1.27888283e-02, 1.62214191e-02, 1.51671285e-02, 1.56063842e-02, 1.64840620e-02,
            1.71913056e-02, 1.33572695e-02, 1.35559231e-02, 1.42914987e-02, 1.70891086e-02,
            1.67996772e-02, 2.03833724e-02, 3.20546233e-02, 1.78978906e-02, 1.42598516e+00,
            1.78974564e-02, 1.14677422e-02, 1.71758263e-02, 1.36960042e-01, 1.42864158e-02,
            1.16295065e-02, 9.71563537e-03, 5.68301355e-02, 7.36180773e-02, 8.09791799e-02,
            3.03644810e-02, 1.14897599e-02, 6.24354558e-03, 4.27751444e-03, 1.16785571e-02,
            3.04352195e-02, 5.96682165e-03, 7.70140468e-03, 1.20847231e-02, 1.78365967e-03
        ], [
            2.31646753e-03, 3.24205541e-03, 3.96258899e-03, 1.02458484e-02, 7.21484200e-03,
            9.36063772e-03, 7.45314529e-03, 3.11796429e-03, 4.97919185e-03, 6.36049232e-03,
            6.40410619e-03, 1.67585474e-02, 9.92580904e-03, 7.24465990e-03, 1.17434520e-02,
            1.04849235e+00, 3.33250961e-03, 9.49223885e-03, 5.44515837e-03, 2.87443194e-03,
            3.03775062e-03, 4.94190516e-03, 3.97662327e-03, 3.57865451e-03, 3.92291897e-03,
            4.08876855e-03, 2.99095532e-03, 2.60440687e-03, 2.67699860e-03, 3.94067638e-03,
            5.94520359e-03, 4.71502351e-03, 4.63210188e-03, 8.12482737e-03, 2.75362778e-03
        ], [
            2.26960004e-02, 3.96953222e-02, 3.33055782e-02, 4.40041410e-02, 3.09873475e-02,
            4.15207669e-02, 4.58244266e-02, 3.76067559e-02, 5.48874858e-02, 5.07850375e-02,
            6.24771453e-02, 5.76419520e-02, 3.57648844e-02, 2.94412996e-02, 3.53706298e-02,
            3.32269089e-02, 1.11041089e+00, 2.76724181e-02, 2.36446150e-02, 1.49088594e-02,
            2.33292330e-02, 3.98192952e-02, 2.81699033e-02, 1.97170640e-02, 1.86711494e-02,
            2.70659160e-02, 1.94313099e-02, 1.29417678e-02, 1.79205228e-02, 1.66826806e-02,
            2.68994987e-02, 2.77999705e-02, 2.62436386e-02, 2.86237636e-02, 3.58659677e-03
        ], [
            8.68738957e-03, 1.23122179e-02, 1.04377268e-02, 1.08863718e-02, 1.01191657e-02,
            1.22633873e-02, 1.17170010e-02, 1.07820149e-02, 1.20667355e-02, 1.16514952e-02,
            1.57858390e-02, 1.36696811e-02, 1.16082243e-02, 1.01059036e-02, 1.09360131e-02,
            1.06702073e-02, 3.42217361e-02, 1.05148520e+00, 1.07791550e-02, 9.20744943e-03,
            1.08547716e-02, 1.29864539e-02, 1.62024115e-02, 1.22823416e-02, 1.03206586e-02,
            2.15826315e-02, 1.82625561e-02, 1.12875111e-02, 4.78246239e-02, 1.10466160e-02,
            2.52426430e-02, 1.99248000e-02, 1.30767204e-02, 1.80258872e-02, 2.69028616e-03
        ], [
            9.48925821e-03, 8.10031075e-03, 1.47774529e-02, 1.43425121e-02, 1.74750372e-02,
            1.41236925e-02, 1.18177039e-02, 9.15353911e-03, 1.37700132e-02, 1.44058465e-02,
            1.43739200e-02, 1.41653116e-02, 1.29245596e-02, 1.13283088e-02, 1.42460437e-02,
            1.38209882e-02, 1.35408174e-02, 1.42271466e-02, 1.01555223e+00, 7.46226114e-03,
            7.27092285e-03, 1.05991659e-02, 2.20344370e-02, 9.15646299e-03, 8.97511895e-03,
            1.31845897e-02, 8.24158211e-03, 5.89893144e-03, 4.18591277e-03, 9.69902042e-03,
            8.26565092e-03, 5.53773617e-03, 7.54200025e-03, 1.11381269e-02, 9.04562509e-04
        ], [
            6.05024279e-02, 4.83706316e-02, 1.13131187e-01, 1.10479493e-01, 1.35194495e-01,
            1.05138315e-01, 9.16962886e-02, 7.21003189e-02, 9.32622064e-02, 9.86330692e-02,
            8.07611231e-02, 9.42033516e-02, 9.72042527e-02, 9.28196939e-02, 1.06819192e-01,
            9.99567311e-02, 4.70584924e-02, 8.13703716e-02, 6.57616229e-02, 1.04402420e+00,
            2.76882422e-02, 7.67830188e-02, 4.62904303e-02, 5.15770227e-02, 4.94798683e-02,
            4.54104078e-02, 2.93957810e-02, 1.71795099e-02, 1.33557412e-02, 3.03639364e-02,
            3.98553967e-02, 2.27663513e-02, 4.87782281e-02, 3.64743392e-02, 5.37889676e-03
        ], [
            2.89958172e-02, 1.87989918e-02, 5.29431141e-02, 5.28675841e-02, 6.90770311e-02,
            4.20828957e-02, 3.66274644e-02, 2.44636374e-02, 4.15667743e-02, 4.23939426e-02,
            3.43841988e-02, 3.50636048e-02, 3.61218528e-02, 3.24214929e-02, 4.44580792e-02,
            4.51370890e-02, 1.84159235e-02, 4.54166197e-02, 2.86088746e-02, 1.46269052e-02,
            1.01458466e+00, 4.08984780e-02, 2.48820954e-02, 2.25048031e-02, 2.17752976e-02,
            2.10422552e-02, 1.81405344e-02, 1.10805492e-02, 8.23621521e-03, 1.76637545e-02,
            1.73186093e-02, 1.14946815e-02, 2.15464342e-02, 2.20284397e-02, 2.62107750e-03
        ], [
            5.79772211e-03, 8.28526593e-03, 1.16644390e-02, 1.31243151e-02, 1.35238169e-02,
            1.31306333e-02, 1.49430938e-02, 7.87655815e-03, 1.32472005e-02, 1.46120928e-02,
            1.31442814e-02, 1.34161868e-02, 1.53624842e-02, 1.42567123e-02, 1.29686147e-02,
            1.27048741e-02, 1.27791784e-02, 1.26248961e-02, 1.22462375e-02, 1.38866110e-02,
            1.07595607e-02, 1.01653970e+00, 1.46362301e-02, 1.43179750e-02, 3.12545159e-02,
            2.79865768e-02, 1.51708104e-02, 1.65470088e-02, 5.93920866e-03, 2.07427883e-02,
            1.60316179e-02, 1.00482273e-02, 1.38042105e-02, 1.88061800e-02, 9.84622411e-04
        ], [
            2.72756321e-02, 3.42961139e-02, 4.79210786e-02, 4.63451524e-02, 5.06690375e-02,
            5.01550064e-02, 4.06089804e-02, 5.00342784e-02, 4.14853621e-02, 4.15277752e-02,
            5.71822776e-02, 4.22119576e-02, 3.44836509e-02, 2.75731807e-02, 3.80094867e-02,
            4.16821748e-02, 3.97451748e-02, 3.76503614e-02, 2.75379291e-02, 2.91314771e-02,
            1.97391279e-02, 2.64244141e-02, 1.06262404e+00, 7.35433360e-02, 3.43281596e-02,
            8.15265702e-02, 1.94546149e-02, 1.33572566e-02, 7.15064769e-03, 1.53842978e-02,
            2.15817749e-02, 1.72375473e-02, 1.82302211e-02, 2.09479028e-02, 3.63191043e-03
        ], [
            6.80651960e-03, 8.80120908e-03, 1.05841799e-02, 1.01561422e-02, 1.29023593e-02,
            1.53124755e-02, 8.00532861e-03, 1.14939993e-02, 8.21361824e-03, 8.54491073e-03,
            1.34560885e-02, 1.30260090e-02, 8.30229888e-03, 6.66648533e-03, 8.00248442e-03,
            8.03028854e-03, 6.28397171e-03, 9.38920596e-03, 3.93297088e-03, 4.68102863e-03,
            3.11683110e-03, 5.61638611e-03, 6.35717616e-03, 1.11391169e+00, 7.06519392e-03,
            1.00846313e-02, 2.73671937e-03, 1.78463526e-03, 1.31934371e-03, 2.66702055e-03,
            3.30870394e-03, 2.83130726e-03, 2.66948816e-03, 3.42453141e-03, 7.96212110e-04
        ], [
            2.59476687e-03, 4.02406364e-03, 4.77982572e-03, 5.67861933e-03, 7.23970813e-03,
            5.06717529e-03, 6.25484788e-03, 3.73266021e-03, 5.56341586e-03, 5.81010746e-03,
            4.85184195e-03, 5.05400499e-03, 6.05577343e-03, 5.91449505e-03, 6.25671461e-03,
            5.05233662e-03, 3.54918495e-03, 4.39240466e-03, 4.57976162e-03, 5.97360528e-03,
            3.60042075e-03, 3.83329668e-03, 5.12927373e-03, 1.02882259e-02, 1.04131446e+00,
            2.54462322e-02, 7.11637480e-03, 6.11415334e-03, 1.83369971e-03, 7.27420068e-03,
            6.64290148e-03, 3.55556363e-03, 3.41379425e-03, 7.37284258e-03, 4.20498934e-04
        ], [
            1.19278867e-02, 1.41975970e-02, 2.10360002e-02, 1.85933145e-02, 2.22294073e-02,
            2.32167505e-02, 2.31761935e-02, 1.88303215e-02, 1.86272010e-02, 1.91923257e-02,
            2.29350722e-02, 1.96789772e-02, 1.81384577e-02, 1.56181603e-02, 1.98130472e-02,
            1.82574151e-02, 1.35516320e-02, 1.66028339e-02, 1.89291553e-02, 3.09343138e-02,
            1.71838004e-02, 1.82643764e-02, 5.56853391e-02, 1.75524115e-01, 1.20231734e-01,
            1.13211339e+00, 1.19190106e-02, 1.00226947e-02, 4.06078070e-03, 1.21934337e-02,
            1.20890761e-02, 7.28095338e-03, 9.82166756e-03, 1.41831268e-02, 1.45056665e-03
        ], [
            1.21442431e-02, 1.70657208e-02, 2.13491907e-02, 2.37048111e-02, 2.58366253e-02,
            2.26888141e-02, 3.36861819e-02, 1.67941302e-02, 2.66046927e-02, 2.51887837e-02,
            2.28430572e-02, 2.37960485e-02, 2.65847877e-02, 2.55129445e-02, 2.46617533e-02,
            2.51230733e-02, 1.93751923e-02, 2.85145858e-02, 2.78974336e-02, 3.37388702e-02,
            3.34598966e-02, 2.75796169e-02, 3.07901608e-02, 3.99047105e-02, 4.83684144e-02,
            4.40256258e-02, 1.14948827e+00, 4.85305460e-02, 1.31183253e-02, 4.13147451e-02,
            3.42964068e-02, 1.74413038e-02, 2.47898521e-02, 4.13476502e-02, 2.30144681e-03
        ], [
            4.94499751e-02, 6.18653155e-02, 6.28215125e-02, 7.25189462e-02, 6.81004308e-02,
            6.77987130e-02, 7.10005494e-02, 5.59872905e-02, 6.50152751e-02, 6.50416973e-02,
            6.68985485e-02, 6.61170817e-02, 6.47376218e-02, 5.72098945e-02, 6.46724952e-02,
            6.80800974e-02, 6.16739350e-02, 6.44925395e-02, 5.85121540e-02, 6.90372199e-02,
            6.49833298e-02, 6.29108741e-02, 8.63514412e-02, 9.21870339e-02, 8.28810811e-02,
            6.66802883e-02, 5.43452256e-02, 1.26699918e+00, 8.80348849e-02, 7.38303999e-02,
            5.41334319e-02, 3.09558388e-02, 5.32193511e-02, 7.53582677e-02, 1.27489555e-02
        ], [
            1.51953277e-02, 1.49483910e-02, 2.18266701e-02, 2.27585903e-02, 2.41448000e-02,
            2.26095832e-02, 2.71463631e-02, 1.49282383e-02, 2.13549623e-02, 2.31561835e-02,
            1.97390137e-02, 2.15834762e-02, 2.28276538e-02, 2.03974984e-02, 2.31895271e-02,
            2.40817362e-02, 1.71893661e-02, 2.53476090e-02, 3.64773096e-02, 3.52913814e-02,
            5.29324789e-02, 4.23487112e-02, 2.28197363e-02, 3.90659285e-02, 3.10699442e-02,
            3.72338098e-02, 2.93899891e-02, 4.01173826e-02, 1.04269959e+00, 3.79452073e-02,
            2.51911079e-02, 2.55859472e-02, 4.52306312e-02, 4.10144925e-02, 2.02008568e-03
        ], [
            5.10373249e-02, 7.21048532e-02, 1.06502535e-01, 1.03580074e-01, 1.02613051e-01,
            9.18674693e-02, 1.46178689e-01, 6.93477202e-02, 1.47447519e-01, 1.20022068e-01,
            9.74302929e-02, 1.00598290e-01, 1.21183237e-01, 1.33627341e-01, 1.29548949e-01,
            1.06158544e-01, 9.29983673e-02, 1.22982504e-01, 1.03280716e-01, 9.87633615e-02,
            1.05205819e-01, 9.38760148e-02, 9.58000914e-02, 1.09045102e-01, 1.67233340e-01,
            1.31024373e-01, 1.30563167e-01, 1.63465336e-01, 6.39350931e-02, 1.19909946e+00,
            1.12015739e-01, 5.74623120e-02, 1.04487954e-01, 1.33601150e-01, 9.74823181e-03
        ], [
            3.38486729e-03, 5.53066856e-03, 5.02481717e-03, 4.35463508e-03, 4.20837770e-03,
            7.06454439e-03, 6.40207745e-03, 4.37174841e-03, 6.23588305e-03, 5.67436989e-03,
            5.31541895e-03, 5.77109549e-03, 4.38839968e-03, 4.20124169e-03, 4.77552570e-03,
            5.30261358e-03, 7.39426008e-03, 4.62941163e-03, 4.33488779e-03, 3.21115577e-03,
            4.79472086e-03, 6.04404941e-03, 5.03640603e-03, 1.00936853e-02, 1.11510117e-02,
            5.95849489e-03, 5.12001904e-03, 3.50182089e-03, 4.18528018e-03, 4.76800179e-03,
            1.00915828e+00, 5.33361049e-03, 6.54061520e-03, 1.03830081e-02, 5.08269627e-04
        ], [
            1.39949023e-03, 1.21538075e-03, 1.76150751e-03, 1.72711399e-03, 1.80585969e-03,
            1.57618533e-03, 2.06206668e-03, 1.33579790e-03, 2.35966817e-03, 2.04119613e-03,
            1.73686833e-03, 1.91303906e-03, 2.18309650e-03, 2.24679792e-03, 2.38996924e-03,
            1.66351857e-03, 1.89097489e-03, 1.67575407e-03, 1.70165359e-03, 1.56850037e-03,
            2.47098430e-03, 1.50510385e-03, 2.13979362e-03, 1.62349134e-03, 2.96402449e-03,
            2.30854320e-03, 2.55223780e-03, 2.30904113e-03, 7.80134914e-04, 3.37565852e-03,
            5.33120329e-03, 1.01013945e+00, 1.95106838e-03, 2.64696850e-03, 1.43968621e-04
        ], [
            2.21893507e-03, 8.44021432e-04, 1.67353553e-03, 1.30729511e-03, 1.98914799e-03,
            1.38421448e-03, 1.19395171e-03, 7.93176235e-04, 1.28128143e-03, 1.21342630e-03,
            1.07028401e-03, 1.10701437e-03, 1.29312452e-03, 1.10789773e-03, 1.21021178e-03,
            1.22353183e-03, 8.69958694e-04, 1.17327567e-03, 1.11891241e-03, 1.03209487e-03,
            8.66191798e-04, 1.85088778e-03, 1.28142688e-03, 1.11448531e-03, 1.19086679e-03,
            1.26277993e-03, 1.25642077e-03, 1.08427480e-03, 3.67305029e-04, 1.32012943e-03,
            6.20813904e-03, 1.81008525e-03, 1.02611005e+00, 1.95319767e-03, 1.12310889e-04
        ], [
            9.59666534e-03, 1.43480583e-02, 1.87216369e-02, 1.92143684e-02, 2.12048434e-02,
            1.82308216e-02, 3.02738275e-02, 1.37114949e-02, 2.28267004e-02, 2.02390863e-02,
            1.77600622e-02, 2.03644597e-02, 2.02152497e-02, 1.85127198e-02, 1.98732847e-02,
            1.97611383e-02, 1.60664833e-02, 1.90520969e-02, 1.93835862e-02, 1.70688357e-02,
            1.78500777e-02, 2.47387604e-02, 1.83345228e-02, 1.88758653e-02, 2.27732987e-02,
            2.42714666e-02, 3.27132355e-02, 2.30719661e-02, 1.35829092e-02, 4.13591257e-02,
            2.55738114e-02, 1.67063049e-02, 2.54006445e-02, 1.09654793e+00, 2.93535090e-03
        ], [
            5.06102790e-05, 5.48363849e-05, 8.69482458e-05, 1.27030203e-04, 9.79591082e-05,
            1.00017290e-04, 1.06515454e-04, 5.77389070e-05, 9.65771423e-05, 1.06302007e-04,
            7.59709688e-05, 8.18325335e-05, 8.74762934e-05, 7.63573474e-05, 1.14372246e-04,
            8.76402563e-05, 5.86599044e-05, 1.35360430e-04, 1.56570950e-04, 6.18247403e-05,
            9.67392517e-05, 1.12884641e-04, 1.53412802e-04, 7.69541331e-05, 8.51381589e-05,
            8.33448999e-05, 1.11021881e-04, 1.20971841e-04, 3.20042011e-04, 1.25109237e-04,
            1.24867624e-04, 7.06738158e-05, 1.61133600e-04, 1.95592991e-04, 1.00008239e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2001():
    """
    Leontief matrix 2001



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17376324e+00, 5.42692320e-03, 3.57302102e-01, 7.23802767e-02, 1.00172097e-01,
            1.93117710e-01, 2.72004328e-02, 6.97740870e-03, 1.79681832e-02, 2.88294478e-02,
            8.75474334e-03, 7.14435505e-03, 7.75673501e-03, 7.14127789e-03, 8.53506619e-03,
            3.49588758e-02, 5.61866050e-03, 2.26215053e-02, 5.77760178e-03, 8.89303282e-03,
            7.70944611e-03, 1.01477301e-01, 6.81076529e-03, 1.04984726e-02, 8.56473815e-03,
            1.22629266e-02, 5.05326739e-03, 4.05856738e-03, 3.28621564e-03, 6.11431748e-03,
            1.02110598e-02, 8.59071056e-03, 1.22960926e-02, 1.12621339e-02, 2.03713870e-02
        ], [
            2.27107367e-02, 1.10103012e+00, 2.39487917e-02, 2.80103369e-02, 2.28312283e-02,
            2.94758750e-02, 2.59220461e-02, 5.09889282e-01, 8.28524776e-02, 4.09382639e-02,
            1.13007496e-01, 9.39857982e-02, 3.47025140e-02, 2.43625708e-02, 2.99968432e-02,
            3.24294883e-02, 1.50921292e-01, 5.35653491e-02, 1.50624474e-02, 1.06908577e-02,
            1.06243747e-02, 1.71884065e-02, 4.14822093e-02, 5.43547499e-02, 4.60897178e-02,
            2.26970539e-02, 1.05833018e-02, 6.41495695e-03, 9.29579172e-03, 1.06938025e-02,
            1.61598379e-02, 1.30363692e-02, 1.54012781e-02, 1.63300377e-02, 6.95446830e-02
        ], [
            7.78999990e-02, 4.35897632e-03, 1.19876344e+00, 1.64883093e-02, 1.34718669e-01,
            1.96987748e-02, 1.15794859e-02, 5.27263066e-03, 2.06201348e-02, 1.42512470e-02,
            7.90218161e-03, 7.19702363e-03, 7.89934270e-03, 7.54974595e-03, 7.93797933e-03,
            1.09454957e-02, 5.89882869e-03, 7.74764063e-03, 6.67906219e-03, 9.73278721e-03,
            8.72650722e-03, 1.85006323e-01, 6.44936256e-03, 1.60054067e-02, 1.27100117e-02,
            9.97361307e-03, 6.74851265e-03, 5.36505750e-03, 2.37783796e-03, 7.56613612e-03,
            1.50810080e-02, 1.24630015e-02, 2.16929862e-02, 1.38695519e-02, 5.46433930e-03
        ], [
            5.39785760e-03, 3.69288608e-03, 6.44316714e-03, 1.41902705e+00, 9.39018449e-02,
            1.02453040e-02, 1.30335091e-02, 3.73964202e-03, 9.30048519e-03, 2.43366272e-02,
            1.01065508e-02, 6.45235315e-03, 8.59030498e-03, 7.44376416e-03, 1.63606477e-02,
            3.94310592e-02, 3.01654150e-03, 7.47618667e-03, 5.70741384e-03, 5.12820467e-03,
            5.56258859e-03, 6.53912294e-03, 4.85489583e-03, 6.32986381e-03, 4.85062490e-03,
            4.99424587e-03, 3.52975382e-03, 2.29866623e-03, 1.47802218e-03, 3.66411502e-03,
            7.24111640e-03, 2.74882513e-03, 6.58600379e-03, 1.10724048e-02, 7.96172939e-04
        ], [
            4.89937894e-04, 4.02188375e-04, 7.02755468e-04, 1.15620148e-02, 1.25644823e+00,
            1.37513930e-03, 1.55202250e-03, 4.32529827e-04, 8.23461311e-04, 2.48075040e-03,
            1.03059929e-03, 9.10350900e-04, 1.03724794e-03, 9.40900414e-04, 3.09838835e-03,
            5.86613488e-03, 4.58770114e-04, 7.28768537e-04, 1.08838105e-03, 7.78917757e-04,
            6.47826556e-04, 5.43724326e-04, 6.33026739e-04, 6.79715701e-04, 6.29048374e-04,
            5.98224058e-04, 7.02756382e-04, 2.46550663e-04, 1.44941942e-04, 4.09371506e-04,
            1.11176500e-03, 3.36967314e-04, 4.72408325e-04, 9.61816282e-04, 6.94891963e-05
        ], [
            4.90243192e-03, 4.44144788e-03, 6.38092521e-03, 6.18189695e-03, 7.06914392e-03,
            1.27180543e+00, 1.95697291e-02, 3.65672269e-03, 5.84900382e-03, 8.64963430e-03,
            1.23409717e-02, 9.40058902e-03, 8.68584160e-03, 6.44660547e-03, 9.69302468e-03,
            1.00831919e-01, 4.69597744e-03, 4.65785339e-02, 4.02793954e-03, 3.86734582e-03,
            3.60297016e-03, 5.91928253e-03, 4.38303974e-03, 4.02323224e-03, 3.71755861e-03,
            7.50484787e-03, 3.49429736e-03, 2.48491486e-03, 5.41613382e-03, 3.58163383e-03,
            4.93542656e-03, 4.89763953e-03, 3.73613214e-03, 7.07456188e-03, 9.02443832e-04
        ], [
            1.46909271e-02, 1.31531553e-02, 5.21316193e-02, 3.59242093e-02, 4.23254728e-02,
            3.53582992e-02, 1.30838415e+00, 1.36317730e-02, 4.35965344e-02, 4.61206927e-02,
            4.37377553e-02, 2.45380451e-02, 3.11082850e-02, 3.73542113e-02, 2.83393569e-02,
            4.69765239e-02, 1.75240887e-02, 2.40964835e-02, 2.27634819e-02, 2.61885873e-02,
            2.49373672e-02, 2.80764023e-02, 1.95263271e-02, 1.90649986e-02, 2.38759535e-02,
            3.22529760e-02, 2.80465854e-02, 3.06852653e-02, 9.58580386e-03, 4.10385151e-02,
            3.33757021e-02, 3.00178211e-02, 2.39824179e-02, 4.70743308e-02, 2.16920890e-03
        ], [
            2.28651976e-02, 1.98025841e-02, 1.88946509e-02, 2.20396259e-02, 1.93825453e-02,
            2.10323023e-02, 1.68674647e-02, 1.06282561e+00, 5.08659705e-02, 2.87861179e-02,
            3.31552624e-02, 2.48718386e-02, 1.66522713e-02, 1.35818862e-02, 1.53988662e-02,
            1.72594846e-02, 3.20682175e-02, 2.60119233e-02, 1.17878686e-02, 1.16877758e-02,
            9.21605150e-03, 1.24741862e-02, 6.30445635e-02, 9.34455288e-02, 7.88908594e-02,
            2.78046494e-02, 9.20300813e-03, 5.55895278e-03, 4.54376105e-03, 1.00740384e-02,
            1.46098754e-02, 7.29692372e-03, 1.09636875e-02, 1.25939256e-02, 1.94753930e-03
        ], [
            6.83114649e-02, 2.63129650e-02, 5.14611931e-02, 1.46054420e-01, 9.79558148e-02,
            7.73238864e-02, 7.56013783e-02, 3.66297986e-02, 1.31608454e+00, 3.11206531e-01,
            6.50821325e-02, 4.58123344e-02, 4.75155231e-02, 5.52437612e-02, 5.74535357e-02,
            7.40969498e-02, 2.11939990e-02, 4.09901938e-02, 2.67786708e-02, 1.30199739e-02,
            1.15929598e-02, 2.43238042e-02, 1.84270506e-02, 2.02084871e-02, 1.89941934e-02,
            1.83763707e-02, 1.25607195e-02, 7.97526443e-03, 7.83938111e-03, 1.61989747e-02,
            2.36029632e-02, 1.19300079e-02, 9.43884575e-02, 2.62297361e-02, 3.59471813e-03
        ], [
            1.35275635e-02, 1.47458071e-02, 3.22214330e-02, 2.66846816e-02, 6.32251536e-02,
            2.37047311e-02, 3.04608987e-02, 1.37083309e-02, 3.54462440e-02, 1.15562085e+00,
            2.38734033e-02, 2.20308746e-02, 4.91917134e-02, 4.82137446e-02, 7.03276249e-02,
            5.45826806e-02, 1.11181487e-02, 3.28938606e-02, 3.12519041e-02, 1.00760505e-02,
            9.96895635e-03, 1.40835464e-02, 1.98290186e-02, 1.44600567e-02, 1.37209060e-02,
            1.51739135e-02, 1.06737851e-02, 4.59789132e-03, 4.51723929e-03, 8.68454388e-03,
            1.05248246e-02, 5.06413870e-03, 1.32401678e-02, 1.10208978e-02, 1.61349217e-03
        ], [
            6.22975544e-03, 9.05001083e-03, 1.25820042e-02, 8.51493780e-03, 8.52133269e-03,
            1.66739635e-02, 6.31411379e-03, 8.22324800e-03, 1.35000882e-02, 1.50481643e-02,
            1.13615070e+00, 2.17275078e-02, 1.63254050e-02, 2.30714908e-02, 1.94068294e-02,
            1.48692920e-02, 8.71419091e-03, 9.58148461e-02, 8.40613728e-03, 4.05758057e-03,
            3.67480281e-03, 8.62076109e-03, 5.74827110e-03, 5.40810046e-03, 5.05325188e-03,
            6.21731554e-03, 5.50234122e-03, 2.52945417e-03, 7.34923130e-03, 4.15355237e-03,
            5.96472537e-03, 5.06439871e-03, 6.16443677e-03, 6.28335070e-03, 9.61236422e-04
        ], [
            2.27926547e-02, 5.36462678e-02, 4.43609713e-02, 3.55162839e-02, 4.48167857e-02,
            6.97054063e-02, 3.72262227e-02, 4.39443782e-02, 5.00418604e-02, 7.09845625e-02,
            7.97569039e-02, 1.48463394e+00, 2.99271662e-01, 1.52841887e-01, 2.36940329e-01,
            1.70256987e-01, 3.68437223e-02, 1.65053640e-01, 6.60300500e-02, 1.82660980e-02,
            1.62690742e-02, 2.40915705e-02, 3.38252399e-02, 4.04461573e-02, 3.64180875e-02,
            3.05230778e-02, 2.42095675e-02, 1.01645436e-02, 1.55413655e-02, 1.90354537e-02,
            2.60439133e-02, 1.33564576e-02, 1.93281927e-02, 2.34894314e-02, 5.11765676e-03
        ], [
            1.38630763e-02, 2.43884399e-02, 1.47970881e-02, 2.07082292e-02, 1.72845820e-02,
            2.07736038e-02, 1.65724039e-02, 2.10143049e-02, 1.80499824e-02, 2.43979727e-02,
            3.03397104e-02, 3.51407878e-02, 1.15338840e+00, 3.16857925e-02, 6.13566232e-02,
            2.30180512e-02, 1.80631615e-02, 3.20063094e-02, 3.50037800e-02, 8.52095267e-03,
            6.71194086e-03, 8.85034151e-03, 1.43179975e-02, 2.03909997e-02, 1.43961518e-02,
            1.26198128e-02, 8.74045480e-03, 4.51017467e-03, 4.77237761e-03, 8.03671430e-03,
            1.23917014e-02, 5.79983406e-03, 7.86583844e-03, 9.55738192e-03, 2.11389750e-03
        ], [
            1.39361207e-02, 2.35374653e-02, 2.28205291e-02, 3.06370851e-02, 2.88661332e-02,
            2.99050474e-02, 3.54900588e-02, 2.46256747e-02, 3.54413845e-02, 4.47822222e-02,
            3.66437715e-02, 5.36549387e-02, 1.59485735e-01, 1.43784764e+00, 1.34885804e-01,
            5.64851042e-02, 3.99961908e-02, 6.84275782e-02, 6.20142074e-02, 2.40887010e-02,
            2.10668021e-02, 2.00430805e-02, 2.75879030e-02, 2.91239748e-02, 2.94855047e-02,
            2.91710361e-02, 7.52254632e-02, 1.75438596e-02, 1.08168605e-02, 3.56761899e-02,
            3.42535831e-02, 1.71574846e-02, 4.22854322e-02, 2.93593894e-02, 3.14374564e-03
        ], [
            1.31367902e-02, 1.62369296e-02, 1.54329074e-02, 1.59579514e-02, 1.65880131e-02,
            1.74551613e-02, 1.36307297e-02, 1.42610541e-02, 1.40408929e-02, 1.76069473e-02,
            1.73021404e-02, 2.09998720e-02, 3.30311333e-02, 1.92002727e-02, 1.43068146e+00,
            1.81999159e-02, 1.20324969e-02, 1.74589811e-02, 1.29681510e-01, 1.40821351e-02,
            1.05918835e-02, 9.73487874e-03, 5.80651711e-02, 8.07213764e-02, 8.50310430e-02,
            3.00674524e-02, 1.15399495e-02, 6.39399934e-03, 4.46560792e-03, 1.18619796e-02,
            2.86379811e-02, 6.18367349e-03, 7.56743638e-03, 1.40264959e-02, 1.76509887e-03
        ], [
            2.29544480e-03, 3.17952480e-03, 3.90850095e-03, 1.01867069e-02, 7.09663489e-03,
            8.93421838e-03, 7.40944635e-03, 3.06993047e-03, 4.89625969e-03, 6.36772636e-03,
            6.35191924e-03, 1.61627808e-02, 9.78273224e-03, 7.43828944e-03, 1.18884171e-02,
            1.05013737e+00, 3.47977980e-03, 9.40833665e-03, 5.12401911e-03, 2.76897632e-03,
            2.87303144e-03, 4.67247009e-03, 4.07871801e-03, 3.52594935e-03, 4.06001362e-03,
            4.05602588e-03, 2.93997204e-03, 2.48685046e-03, 2.69514314e-03, 3.86206345e-03,
            5.97195977e-03, 4.42956063e-03, 4.53726108e-03, 8.36436505e-03, 2.76778759e-03
        ], [
            2.41368982e-02, 4.18794246e-02, 3.56431963e-02, 4.58694971e-02, 3.24008373e-02,
            4.31721821e-02, 4.87311972e-02, 4.05538981e-02, 5.61645206e-02, 5.40214691e-02,
            6.65845061e-02, 6.13043518e-02, 3.80684144e-02, 3.17270794e-02, 3.71872004e-02,
            3.42126595e-02, 1.11063838e+00, 2.95489592e-02, 2.46839930e-02, 1.49589674e-02,
            2.36473025e-02, 4.23362320e-02, 2.88648787e-02, 2.04960393e-02, 2.03607621e-02,
            2.88839734e-02, 2.04604477e-02, 1.30668082e-02, 1.93362130e-02, 1.73091978e-02,
            2.79886094e-02, 2.89727832e-02, 2.72417799e-02, 3.04551641e-02, 3.72970520e-03
        ], [
            8.90978695e-03, 1.35083763e-02, 1.07208438e-02, 1.14743997e-02, 1.02839488e-02,
            1.20469890e-02, 1.20611991e-02, 1.20544999e-02, 1.23429815e-02, 1.20548823e-02,
            1.64980127e-02, 1.40835356e-02, 1.21531114e-02, 1.09391423e-02, 1.13327392e-02,
            1.06894345e-02, 3.65089621e-02, 1.05411429e+00, 1.09209151e-02, 9.17828604e-03,
            1.06080718e-02, 1.34467714e-02, 1.67925293e-02, 1.23234953e-02, 1.12805420e-02,
            2.20053765e-02, 1.86379639e-02, 1.13301929e-02, 4.70516365e-02, 1.11905120e-02,
            2.45793150e-02, 2.06046091e-02, 1.27767264e-02, 1.81793830e-02, 2.30701901e-03
        ], [
            9.30106304e-03, 7.94036937e-03, 1.47265898e-02, 1.41754240e-02, 1.72897098e-02,
            1.38000081e-02, 1.17091505e-02, 8.92124989e-03, 1.34724706e-02, 1.44255222e-02,
            1.42725497e-02, 1.40585348e-02, 1.30368983e-02, 1.16192224e-02, 1.41885213e-02,
            1.35179437e-02, 1.32319673e-02, 1.39107710e-02, 1.01488623e+00, 7.10559904e-03,
            6.89012293e-03, 1.04052658e-02, 2.13663882e-02, 8.77740841e-03, 9.40259572e-03,
            1.31345375e-02, 7.85827319e-03, 5.75683213e-03, 4.01699294e-03, 9.50641487e-03,
            7.92000271e-03, 5.31435025e-03, 7.39036386e-03, 1.07608974e-02, 8.89834126e-04
        ], [
            6.01643546e-02, 4.68935814e-02, 1.12948810e-01, 1.12304600e-01, 1.33190682e-01,
            1.05813730e-01, 9.13279640e-02, 7.44413348e-02, 9.47090067e-02, 1.01050876e-01,
            8.12092915e-02, 9.45740815e-02, 9.89146107e-02, 9.52813795e-02, 1.06723311e-01,
            9.79200352e-02, 4.77401287e-02, 8.20995289e-02, 6.41877387e-02, 1.04125190e+00,
            2.62508947e-02, 7.65021679e-02, 4.54457976e-02, 5.07821127e-02, 5.03222020e-02,
            4.45612524e-02, 2.88794266e-02, 1.68227836e-02, 1.36954729e-02, 3.02105692e-02,
            4.02720759e-02, 2.25667474e-02, 4.83309947e-02, 3.73678155e-02, 5.25227829e-03
        ], [
            2.90750544e-02, 1.89262768e-02, 5.37491970e-02, 5.40872192e-02, 7.29263174e-02,
            4.25980080e-02, 3.70111539e-02, 2.56138828e-02, 4.16752024e-02, 4.30223120e-02,
            3.57922984e-02, 3.62911456e-02, 3.76393545e-02, 3.42798982e-02, 4.53142986e-02,
            4.54189743e-02, 1.96852458e-02, 4.69922759e-02, 2.83763951e-02, 1.41045391e-02,
            1.01413347e+00, 4.12374268e-02, 2.47229983e-02, 2.22903592e-02, 2.31616143e-02,
            2.10812310e-02, 1.78125894e-02, 1.12416419e-02, 8.12015253e-03, 1.78204057e-02,
            1.76701340e-02, 1.16402762e-02, 2.15651925e-02, 2.19559989e-02, 2.57792736e-03
        ], [
            5.81020914e-03, 8.35151072e-03, 1.16222065e-02, 1.34802797e-02, 1.35806419e-02,
            1.31864302e-02, 1.53883249e-02, 8.15271571e-03, 1.34178793e-02, 1.51307781e-02,
            1.35898219e-02, 1.38227142e-02, 1.57662877e-02, 1.48039055e-02, 1.31149286e-02,
            1.26693009e-02, 1.60485437e-02, 1.27396748e-02, 1.22035523e-02, 1.34341091e-02,
            1.03347361e-02, 1.01643755e+00, 1.46827252e-02, 1.37670162e-02, 3.36607328e-02,
            2.70576747e-02, 1.55073299e-02, 1.63749264e-02, 5.92425221e-03, 2.02318738e-02,
            1.60983178e-02, 9.72473874e-03, 1.35695114e-02, 1.89125352e-02, 1.01808317e-03
        ], [
            2.67080884e-02, 3.44482176e-02, 4.73164915e-02, 4.61024369e-02, 4.94135372e-02,
            4.99834204e-02, 4.03845236e-02, 5.62860442e-02, 4.13676418e-02, 4.22198252e-02,
            5.74410990e-02, 4.28003435e-02, 3.50204376e-02, 2.87273781e-02, 3.78093002e-02,
            4.11790449e-02, 4.00629061e-02, 3.78446792e-02, 2.75818694e-02, 2.86297295e-02,
            1.94331367e-02, 2.59844742e-02, 1.06292394e+00, 7.50545937e-02, 3.64184226e-02,
            8.10604714e-02, 1.89592472e-02, 1.30146127e-02, 7.31690052e-03, 1.50440200e-02,
            2.13248384e-02, 1.71559003e-02, 1.78555914e-02, 2.10409365e-02, 3.49136617e-03
        ], [
            7.01663877e-03, 9.20841271e-03, 1.07526448e-02, 1.10845218e-02, 1.32876845e-02,
            1.63218559e-02, 8.82192991e-03, 1.33129848e-02, 8.90679077e-03, 9.62346018e-03,
            1.56565529e-02, 1.43115209e-02, 9.36003955e-03, 7.58724333e-03, 8.53967074e-03,
            8.70425087e-03, 6.86559765e-03, 1.04440262e-02, 4.07535487e-03, 5.01005758e-03,
            3.16971737e-03, 5.50984771e-03, 6.58514467e-03, 1.12176506e+00, 7.22464845e-03,
            1.05084630e-02, 2.78517840e-03, 1.87399994e-03, 1.62904300e-03, 2.90723312e-03,
            3.69738538e-03, 2.78969230e-03, 2.79659571e-03, 3.75406014e-03, 8.30515550e-04
        ], [
            2.41832699e-03, 3.90802118e-03, 4.43559921e-03, 5.42673284e-03, 6.80367775e-03,
            5.00276018e-03, 5.98512676e-03, 3.66283896e-03, 5.18599843e-03, 5.55829127e-03,
            4.76944139e-03, 4.85339027e-03, 5.84143499e-03, 5.74198262e-03, 5.81563333e-03,
            4.70676714e-03, 3.84988404e-03, 4.15012471e-03, 4.21196183e-03, 5.29630016e-03,
            3.16937373e-03, 3.52566274e-03, 5.18333982e-03, 1.14713758e-02, 1.04124628e+00,
            2.44654786e-02, 6.19071822e-03, 5.46621345e-03, 1.70435761e-03, 6.62250025e-03,
            6.03344487e-03, 3.20360061e-03, 3.16786872e-03, 6.73219982e-03, 4.09490155e-04
        ], [
            1.17807790e-02, 1.40351108e-02, 2.07243998e-02, 1.84783061e-02, 2.15570641e-02,
            2.24092175e-02, 2.38595635e-02, 2.04494704e-02, 1.85481355e-02, 1.95102158e-02,
            2.30103762e-02, 1.98673857e-02, 1.82751332e-02, 1.59635849e-02, 1.97056104e-02,
            1.79248881e-02, 1.40070539e-02, 1.68183967e-02, 1.93004249e-02, 2.95578106e-02,
            1.63509836e-02, 1.78293201e-02, 5.48583762e-02, 1.62827032e-01, 1.24818446e-01,
            1.13294060e+00, 1.17208113e-02, 9.37699072e-03, 4.18889102e-03, 1.20260382e-02,
            1.20056867e-02, 7.09024571e-03, 9.74065499e-03, 1.43386138e-02, 1.42452232e-03
        ], [
            1.28726430e-02, 1.80868793e-02, 2.26683294e-02, 2.58220762e-02, 2.79228849e-02,
            2.41619175e-02, 3.61692903e-02, 1.83316447e-02, 2.85111452e-02, 2.73584886e-02,
            2.49295092e-02, 2.60361730e-02, 2.87737564e-02, 2.84288567e-02, 2.60353172e-02,
            2.69035927e-02, 2.27071044e-02, 3.13159819e-02, 2.99828847e-02, 3.38532765e-02,
            3.37078066e-02, 2.88996232e-02, 3.26798400e-02, 4.12733286e-02, 5.10173951e-02,
            4.59128941e-02, 1.15700904e+00, 4.92093606e-02, 1.38567745e-02, 4.39106434e-02,
            3.65989521e-02, 1.83196259e-02, 2.62355092e-02, 4.23998033e-02, 2.56571316e-03
        ], [
            5.00235058e-02, 6.24281309e-02, 6.29856225e-02, 7.42471838e-02, 6.73172772e-02,
            6.73669809e-02, 7.27294165e-02, 5.78898469e-02, 6.51852115e-02, 6.65457091e-02,
            6.93175812e-02, 6.85176542e-02, 6.60528674e-02, 5.95927084e-02, 6.51073297e-02,
            6.96696681e-02, 6.98521198e-02, 6.57186567e-02, 5.75294322e-02, 6.71019176e-02,
            6.36117667e-02, 6.40647845e-02, 8.69717676e-02, 9.22123987e-02, 8.60280119e-02,
            6.77472620e-02, 5.40553366e-02, 1.27751418e+00, 9.68361843e-02, 7.42511278e-02,
            5.41608055e-02, 3.06408199e-02, 5.41434525e-02, 8.16266183e-02, 1.55917359e-02
        ], [
            1.53692007e-02, 1.57376885e-02, 2.21770621e-02, 2.30931303e-02, 2.40382427e-02,
            2.32903252e-02, 2.82938662e-02, 1.58779684e-02, 2.17878970e-02, 2.39173013e-02,
            2.04333057e-02, 2.25120189e-02, 2.36140625e-02, 2.16721987e-02, 2.37249987e-02,
            2.41913455e-02, 1.99780654e-02, 2.64609070e-02, 3.56291488e-02, 3.40711356e-02,
            4.97135798e-02, 4.33704247e-02, 2.29314728e-02, 3.83131060e-02, 3.15362264e-02,
            3.82843262e-02, 3.03403768e-02, 3.85899449e-02, 1.04354587e+00, 3.81677744e-02,
            2.69703754e-02, 2.68201022e-02, 4.57373343e-02, 4.18829638e-02, 2.05908742e-03
        ], [
            5.10934386e-02, 7.35069080e-02, 1.07488442e-01, 1.05392899e-01, 1.01245086e-01,
            9.27459906e-02, 1.49381458e-01, 7.26075642e-02, 1.51459308e-01, 1.25075440e-01,
            1.00943121e-01, 1.05216916e-01, 1.25810515e-01, 1.40680365e-01, 1.33936104e-01,
            1.06884278e-01, 1.10868912e-01, 1.24991267e-01, 1.06034453e-01, 9.49357691e-02,
            9.96418631e-02, 9.56477638e-02, 9.74785287e-02, 1.04972834e-01, 1.68394182e-01,
            1.32491467e-01, 1.30049913e-01, 1.60214146e-01, 6.44847949e-02, 1.20320848e+00,
            1.19472598e-01, 5.99651760e-02, 1.06718504e-01, 1.36659290e-01, 1.06796098e-02
        ], [
            3.38074155e-03, 5.88822378e-03, 5.12835839e-03, 4.42625447e-03, 4.17692762e-03,
            6.96859832e-03, 6.70338567e-03, 4.70098277e-03, 6.37754124e-03, 5.93931160e-03,
            5.47543595e-03, 6.15874212e-03, 4.56991915e-03, 4.37953490e-03, 4.93021594e-03,
            5.33835353e-03, 7.89823567e-03, 4.79852520e-03, 4.46960918e-03, 3.08816651e-03,
            4.47149097e-03, 6.15724506e-03, 5.25842216e-03, 9.82398671e-03, 1.22397422e-02,
            6.21440802e-03, 5.11857309e-03, 3.42536155e-03, 4.37692655e-03, 4.90471711e-03,
            1.00931026e+00, 5.55421512e-03, 6.61911745e-03, 1.08342055e-02, 5.35465875e-04
        ], [
            1.32204381e-03, 1.21438054e-03, 1.75625348e-03, 1.78171678e-03, 1.86197449e-03,
            1.61056387e-03, 2.13699013e-03, 1.40608839e-03, 2.31763305e-03, 2.11252318e-03,
            1.78816941e-03, 1.97792180e-03, 2.27694981e-03, 2.34840616e-03, 2.34230880e-03,
            1.69236426e-03, 2.08390673e-03, 1.71818313e-03, 1.76620837e-03, 1.48650593e-03,
            2.15029430e-03, 1.53547025e-03, 2.10761213e-03, 1.69095166e-03, 3.23284017e-03,
            2.34569983e-03, 2.65531255e-03, 2.34567502e-03, 8.06321188e-04, 3.32709774e-03,
            5.27641571e-03, 1.01128365e+00, 1.99717805e-03, 2.66097384e-03, 1.49670104e-04
        ], [
            2.15443549e-03, 8.42269407e-04, 1.68785007e-03, 1.31471365e-03, 2.16823038e-03,
            1.38488461e-03, 1.20562317e-03, 8.26596616e-04, 1.30521877e-03, 1.24662235e-03,
            1.09313411e-03, 1.14506363e-03, 1.33895993e-03, 1.16948824e-03, 1.24369731e-03,
            1.24463562e-03, 8.89530951e-04, 1.16365143e-03, 1.10135380e-03, 1.00148912e-03,
            8.25908669e-04, 1.82482752e-03, 1.24702806e-03, 1.07494820e-03, 1.18127143e-03,
            1.28883769e-03, 1.22690081e-03, 1.04318445e-03, 3.60997944e-04, 1.28925810e-03,
            5.85823710e-03, 1.86306022e-03, 1.02608642e+00, 1.86233953e-03, 1.14558507e-04
        ], [
            9.58544228e-03, 1.48069787e-02, 1.84905619e-02, 1.94635821e-02, 2.15182386e-02,
            1.80909547e-02, 3.07799548e-02, 1.41581620e-02, 2.27369139e-02, 2.06224556e-02,
            1.81660216e-02, 2.04456836e-02, 2.01556799e-02, 1.88916819e-02, 1.98330961e-02,
            1.92301869e-02, 1.77955222e-02, 1.90858066e-02, 1.88552872e-02, 1.63237129e-02,
            1.66672109e-02, 2.40822634e-02, 1.95282962e-02, 1.82646946e-02, 2.27131497e-02,
            2.35727657e-02, 3.25979481e-02, 2.18774673e-02, 1.31612569e-02, 4.04860565e-02,
            2.57430895e-02, 1.65476667e-02, 2.43578332e-02, 1.09698303e+00, 3.38644611e-03
        ], [
            4.87081991e-05, 5.51288856e-05, 8.01461508e-05, 1.27091389e-04, 9.04820185e-05,
            9.03063014e-05, 1.02452658e-04, 5.78432215e-05, 9.07233764e-05, 1.02409679e-04,
            7.21083238e-05, 7.76195984e-05, 8.23543889e-05, 7.27279048e-05, 1.11684991e-04,
            7.91787551e-05, 6.15820626e-05, 1.27170575e-04, 1.12694728e-04, 5.06765822e-05,
            7.95970486e-05, 1.01608096e-04, 1.32607515e-04, 6.82763419e-05, 8.16108091e-05,
            7.45916628e-05, 9.61986155e-05, 1.06657104e-04, 2.94175618e-04, 1.14430936e-04,
            1.12792260e-04, 6.78900950e-05, 1.31197609e-04, 1.80070916e-04, 1.00009558e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2002():
    """
    Leontief matrix 2002



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17709342e+00, 4.80351142e-03, 3.60603400e-01, 7.51606409e-02, 1.03332048e-01,
            1.99793526e-01, 2.83219925e-02, 6.64419487e-03, 1.79185656e-02, 2.98087705e-02,
            8.78426692e-03, 7.34206896e-03, 8.11876687e-03, 7.25519862e-03, 8.74946518e-03,
            3.36189737e-02, 5.04378544e-03, 2.51359621e-02, 5.83194536e-03, 9.30840923e-03,
            8.18164730e-03, 1.02788382e-01, 6.91441094e-03, 9.47717184e-03, 8.17388216e-03,
            1.21635401e-02, 5.09308048e-03, 4.11306191e-03, 3.39724424e-03, 6.24396722e-03,
            9.62452291e-03, 8.49373137e-03, 1.21866168e-02, 1.13937382e-02, 1.97991282e-02
        ], [
            2.35432358e-02, 1.09763368e+00, 2.43588107e-02, 2.83125767e-02, 2.32578670e-02,
            2.95876673e-02, 2.63884462e-02, 5.19069381e-01, 8.20305377e-02, 4.09808157e-02,
            1.10753054e-01, 9.52880418e-02, 3.57654922e-02, 2.46446525e-02, 3.03403920e-02,
            3.02686338e-02, 1.47415295e-01, 5.40867643e-02, 1.50329732e-02, 1.06366108e-02,
            1.04821968e-02, 1.69103242e-02, 4.21761053e-02, 6.05284260e-02, 4.59009114e-02,
            2.26009873e-02, 1.03636784e-02, 6.21406070e-03, 8.51384470e-03, 1.05204672e-02,
            1.75479219e-02, 1.23951301e-02, 1.49863841e-02, 1.59420139e-02, 6.85648350e-02
        ], [
            7.77654407e-02, 3.96040646e-03, 1.20012660e+00, 1.64205646e-02, 1.35596576e-01,
            1.99896356e-02, 1.17689521e-02, 5.08340477e-03, 2.01810486e-02, 1.38726059e-02,
            7.73837799e-03, 7.09042643e-03, 7.87931987e-03, 7.28227609e-03, 7.67309806e-03,
            1.01442286e-02, 4.82718082e-03, 7.75735236e-03, 6.44866830e-03, 9.77903546e-03,
            8.80884749e-03, 1.83635427e-01, 6.27105277e-03, 1.33638919e-02, 1.17206931e-02,
            9.25864083e-03, 6.53422955e-03, 5.29025543e-03, 2.33836282e-03, 7.62630099e-03,
            1.27673391e-02, 1.22048136e-02, 2.10932817e-02, 1.39390361e-02, 5.53921466e-03
        ], [
            5.47123676e-03, 3.55602374e-03, 6.94279727e-03, 1.43631231e+00, 9.32568012e-02,
            1.04275949e-02, 1.32787272e-02, 4.04501893e-03, 9.32334346e-03, 2.29884861e-02,
            1.01792833e-02, 6.90994142e-03, 9.28842478e-03, 7.80763034e-03, 1.65619785e-02,
            3.56960463e-02, 2.97064577e-03, 7.66124582e-03, 5.76365947e-03, 5.29852767e-03,
            6.08556987e-03, 6.65728827e-03, 5.07591350e-03, 6.19937427e-03, 4.92030981e-03,
            5.03312616e-03, 3.39999329e-03, 2.34198898e-03, 1.50116351e-03, 3.77384106e-03,
            7.23149895e-03, 2.83978136e-03, 6.46095747e-03, 1.19709607e-02, 6.81865664e-04
        ], [
            4.54503060e-04, 3.60151543e-04, 6.87112880e-04, 1.11843839e-02, 1.27107124e+00,
            1.16133137e-03, 1.39731996e-03, 3.90817389e-04, 7.88519582e-04, 2.66363451e-03,
            8.51229939e-04, 8.08164336e-04, 9.59224308e-04, 8.99459778e-04, 2.91338908e-03,
            6.00047646e-03, 4.16058183e-04, 6.77759084e-04, 1.04356570e-03, 7.62951411e-04,
            6.27483452e-04, 5.09273312e-04, 6.13804752e-04, 6.34228458e-04, 5.86268032e-04,
            5.73337312e-04, 5.97479171e-04, 2.23030289e-04, 1.36780398e-04, 3.95686915e-04,
            9.45180052e-04, 3.10674062e-04, 4.22290029e-04, 8.85035506e-04, 5.83178125e-05
        ], [
            5.10203631e-03, 4.18137845e-03, 6.53993781e-03, 6.13245975e-03, 6.96331368e-03,
            1.27417091e+00, 1.98445731e-02, 3.55662668e-03, 5.73864115e-03, 8.93919760e-03,
            1.24980025e-02, 9.61198909e-03, 8.95638462e-03, 6.53178767e-03, 1.00266183e-02,
            9.34974816e-02, 4.28642352e-03, 4.76472416e-02, 4.08117336e-03, 3.88880652e-03,
            3.60525998e-03, 6.20410535e-03, 4.44208326e-03, 3.95153953e-03, 3.61399339e-03,
            7.32842272e-03, 3.47644085e-03, 2.45557545e-03, 5.47857120e-03, 3.60962693e-03,
            5.09559400e-03, 4.90946277e-03, 3.75578240e-03, 6.94516391e-03, 7.73024951e-04
        ], [
            1.46575658e-02, 1.21093124e-02, 5.27747455e-02, 3.57723099e-02, 4.33625863e-02,
            3.52444680e-02, 1.31161079e+00, 1.37053790e-02, 4.27548699e-02, 4.53866427e-02,
            4.25529708e-02, 2.42950471e-02, 3.11500822e-02, 3.57703098e-02, 2.77638254e-02,
            4.30460541e-02, 1.45812082e-02, 2.34991508e-02, 2.25890947e-02, 2.65401520e-02,
            2.47285179e-02, 2.81271980e-02, 1.96458886e-02, 1.84060222e-02, 2.22427955e-02,
            3.14712327e-02, 2.71642608e-02, 3.01174020e-02, 9.39902511e-03, 4.04456731e-02,
            3.45209953e-02, 2.97770788e-02, 2.39715419e-02, 4.69720323e-02, 2.02090642e-03
        ], [
            2.39050914e-02, 1.90283979e-02, 1.96237987e-02, 2.27191983e-02, 2.01614583e-02,
            2.18520680e-02, 1.75622476e-02, 1.06648515e+00, 5.38379363e-02, 3.03650183e-02,
            3.51994437e-02, 2.67940444e-02, 1.78545308e-02, 1.41351185e-02, 1.60550670e-02,
            1.64456080e-02, 3.09333525e-02, 2.74823263e-02, 1.19268969e-02, 1.17499068e-02,
            9.15252386e-03, 1.27063485e-02, 6.39615375e-02, 1.05899743e-01, 7.72023620e-02,
            2.77676641e-02, 9.07862883e-03, 5.47331494e-03, 4.41092801e-03, 1.01642668e-02,
            1.57674481e-02, 7.24471003e-03, 1.12229711e-02, 1.23885683e-02, 1.87548992e-03
        ], [
            6.84975995e-02, 2.50157918e-02, 5.26664065e-02, 1.49402220e-01, 1.03031439e-01,
            8.02946261e-02, 7.85726949e-02, 3.80134082e-02, 1.32230736e+00, 3.19251050e-01,
            6.78099623e-02, 4.74253044e-02, 5.01929933e-02, 5.77915210e-02, 5.93177997e-02,
            6.76102055e-02, 2.07187859e-02, 4.26380325e-02, 2.75614707e-02, 1.32573497e-02,
            1.17520232e-02, 2.44287459e-02, 1.87546341e-02, 2.07769893e-02, 1.79297018e-02,
            1.82184130e-02, 1.25444291e-02, 7.94048082e-03, 7.02631137e-03, 1.64494284e-02,
            2.49519217e-02, 1.21048971e-02, 9.69747229e-02, 2.65485640e-02, 3.43680437e-03
        ], [
            1.41103674e-02, 1.40403538e-02, 3.37346249e-02, 2.79586347e-02, 6.44754475e-02,
            2.43414327e-02, 3.20190724e-02, 1.39026251e-02, 3.60935448e-02, 1.16203756e+00,
            2.43899087e-02, 2.23947655e-02, 5.21676829e-02, 5.10728894e-02, 7.26979680e-02,
            4.86150363e-02, 1.05571461e-02, 3.39248865e-02, 3.20297181e-02, 1.05207340e-02,
            1.03525955e-02, 1.45877834e-02, 2.04029390e-02, 1.48691718e-02, 1.39141152e-02,
            1.53858154e-02, 1.07796731e-02, 4.67543591e-03, 4.49758132e-03, 9.01902379e-03,
            1.10183033e-02, 5.20275056e-03, 1.33212228e-02, 1.14243963e-02, 1.52730438e-03
        ], [
            6.46045186e-03, 8.83135455e-03, 1.29410724e-02, 8.83023424e-03, 8.51348518e-03,
            1.67253555e-02, 6.39132346e-03, 8.56961007e-03, 1.33671172e-02, 1.53375743e-02,
            1.13566279e+00, 2.27122163e-02, 1.66937431e-02, 2.47067536e-02, 1.99763829e-02,
            1.42290235e-02, 8.04774282e-03, 9.72369503e-02, 8.61057306e-03, 4.00823489e-03,
            3.75877031e-03, 8.70691793e-03, 5.91681593e-03, 5.52007729e-03, 4.98857401e-03,
            6.28118620e-03, 5.63082377e-03, 2.55445623e-03, 7.23338408e-03, 4.28630301e-03,
            6.05969608e-03, 5.03635301e-03, 6.14729496e-03, 6.29753150e-03, 8.98601612e-04
        ], [
            2.34422758e-02, 5.14576945e-02, 4.53451437e-02, 3.63836699e-02, 4.50729483e-02,
            7.02274553e-02, 3.85973388e-02, 4.37711472e-02, 5.01982139e-02, 7.23779780e-02,
            8.06770406e-02, 1.49257997e+00, 3.09931781e-01, 1.55865311e-01, 2.40437160e-01,
            1.48359531e-01, 3.55519866e-02, 1.68741515e-01, 6.67394991e-02, 1.84288876e-02,
            1.64581104e-02, 2.46525886e-02, 3.54081910e-02, 4.22608318e-02, 3.48101245e-02,
            3.07616143e-02, 2.44085465e-02, 1.02351532e-02, 1.52077469e-02, 1.94701908e-02,
            2.66483419e-02, 1.33737158e-02, 1.92373762e-02, 2.35214127e-02, 4.81334273e-03
        ], [
            1.41068607e-02, 2.32509463e-02, 1.49357079e-02, 2.16480033e-02, 1.74423958e-02,
            2.10875944e-02, 1.66561255e-02, 2.05420266e-02, 1.82381270e-02, 2.46988199e-02,
            3.12448622e-02, 3.56057170e-02, 1.15707008e+00, 3.26447713e-02, 6.24797197e-02,
            2.16966648e-02, 1.97200624e-02, 3.27819759e-02, 3.42944820e-02, 8.51672458e-03,
            6.62176936e-03, 8.83111620e-03, 1.46580218e-02, 2.15338211e-02, 1.46138475e-02,
            1.24256972e-02, 8.72995708e-03, 4.52299512e-03, 4.74942188e-03, 8.13833941e-03,
            1.25571722e-02, 5.82468627e-03, 7.75490439e-03, 9.53814535e-03, 1.99988084e-03
        ], [
            1.43309274e-02, 2.28789405e-02, 2.29923442e-02, 3.09412450e-02, 2.90896114e-02,
            3.08186170e-02, 3.61630459e-02, 2.59986133e-02, 3.62400482e-02, 4.56665054e-02,
            3.83252916e-02, 5.40984788e-02, 1.68581229e-01, 1.45515893e+00, 1.38267637e-01,
            5.13601714e-02, 4.02121241e-02, 6.89110655e-02, 6.45651887e-02, 2.40850245e-02,
            2.10578230e-02, 1.98781163e-02, 2.84811070e-02, 2.98615914e-02, 2.94789429e-02,
            2.93296100e-02, 7.39184661e-02, 1.76347714e-02, 1.09063807e-02, 3.56866010e-02,
            3.50054058e-02, 1.72994511e-02, 4.31285513e-02, 2.95509331e-02, 3.12440030e-03
        ], [
            1.39863522e-02, 1.61355302e-02, 1.63431467e-02, 1.67757988e-02, 1.75742534e-02,
            1.85468180e-02, 1.45540474e-02, 1.53611807e-02, 1.46952259e-02, 1.89478363e-02,
            1.83011096e-02, 2.24138496e-02, 3.57996715e-02, 2.01102890e-02, 1.44142028e+00,
            1.78979622e-02, 1.21290993e-02, 1.86522126e-02, 1.35992743e-01, 1.48699493e-02,
            1.13925259e-02, 1.02596802e-02, 6.20921338e-02, 7.93736846e-02, 8.76932255e-02,
            3.06029137e-02, 1.24922780e-02, 6.73637555e-03, 4.60100633e-03, 1.26693341e-02,
            2.98424935e-02, 6.52190979e-03, 7.89302463e-03, 1.53876680e-02, 1.76599652e-03
        ], [
            2.27273750e-03, 2.82399161e-03, 3.84207050e-03, 9.60565921e-03, 6.92499869e-03,
            8.90052931e-03, 7.36989680e-03, 2.89720870e-03, 4.73501837e-03, 6.26357499e-03,
            6.14304327e-03, 1.62938035e-02, 9.90880242e-03, 7.45930256e-03, 1.21571904e-02,
            1.04676592e+00, 3.28792314e-03, 9.49291807e-03, 5.23042837e-03, 2.75332858e-03,
            2.88282438e-03, 4.55860062e-03, 4.12765009e-03, 3.46310245e-03, 4.04185927e-03,
            4.02797324e-03, 2.90517560e-03, 2.41647107e-03, 2.82982654e-03, 3.78523735e-03,
            5.97347589e-03, 4.37274715e-03, 4.67812332e-03, 8.03959880e-03, 2.42500827e-03
        ], [
            2.36871254e-02, 4.06077875e-02, 3.44910409e-02, 4.48331395e-02, 3.19805032e-02,
            4.08782619e-02, 4.65605511e-02, 4.06824002e-02, 5.28037492e-02, 5.03154296e-02,
            6.48245641e-02, 6.02490005e-02, 3.77449526e-02, 2.98406511e-02, 3.56290145e-02,
            2.96246815e-02, 1.11879922e+00, 2.85789766e-02, 2.35637705e-02, 1.45004763e-02,
            2.19306189e-02, 3.88862233e-02, 2.84910257e-02, 1.97524356e-02, 1.90836617e-02,
            2.79602489e-02, 1.93110806e-02, 1.18284187e-02, 1.56681594e-02, 1.53423842e-02,
            2.64939893e-02, 2.80494392e-02, 2.48576723e-02, 2.84902359e-02, 3.56665242e-03
        ], [
            8.83860583e-03, 1.44306590e-02, 1.08243816e-02, 1.15077291e-02, 1.01628506e-02,
            1.20112986e-02, 1.21913084e-02, 1.28344872e-02, 1.22531568e-02, 1.19600168e-02,
            1.65809575e-02, 1.41819329e-02, 1.22060726e-02, 1.01531272e-02, 1.10549626e-02,
            9.63683810e-03, 3.17006553e-02, 1.05803473e+00, 1.08034729e-02, 9.20432502e-03,
            1.07900412e-02, 1.36672582e-02, 1.70817983e-02, 1.22619446e-02, 1.10994330e-02,
            2.18044635e-02, 1.81986962e-02, 1.13575772e-02, 4.72103136e-02, 1.11774164e-02,
            2.45029126e-02, 2.02611812e-02, 1.24655306e-02, 1.80166936e-02, 2.02757481e-03
        ], [
            9.57870761e-03, 7.83519752e-03, 1.50041203e-02, 1.44292571e-02, 1.74222196e-02,
            1.39764736e-02, 1.20414669e-02, 8.98416979e-03, 1.34484686e-02, 1.46809848e-02,
            1.45336068e-02, 1.44234464e-02, 1.34268355e-02, 1.14582452e-02, 1.42298692e-02,
            1.28106761e-02, 1.32371971e-02, 1.42221760e-02, 1.01541941e+00, 7.40031111e-03,
            7.15108170e-03, 1.05343366e-02, 2.19210178e-02, 8.79963639e-03, 9.59379283e-03,
            1.40189066e-02, 7.71926619e-03, 5.87636892e-03, 4.01680647e-03, 9.63821827e-03,
            8.31346363e-03, 5.37795197e-03, 7.59143705e-03, 1.07514294e-02, 8.71922909e-04
        ], [
            6.08662527e-02, 4.37198251e-02, 1.13920853e-01, 1.08290088e-01, 1.31103859e-01,
            1.05387469e-01, 9.33043216e-02, 7.58601438e-02, 9.30653923e-02, 1.00963867e-01,
            8.11346198e-02, 9.62635998e-02, 1.00442376e-01, 9.25355201e-02, 1.07496977e-01,
            8.75564664e-02, 4.65265582e-02, 8.19706292e-02, 6.33706647e-02, 1.04027004e+00,
            2.60347486e-02, 7.72220323e-02, 4.62320778e-02, 5.15449247e-02, 4.85333884e-02,
            4.42791297e-02, 2.85716362e-02, 1.66430643e-02, 1.30695110e-02, 3.00218291e-02,
            3.79289337e-02, 2.24767601e-02, 4.76849236e-02, 3.72346691e-02, 4.93467194e-03
        ], [
            2.95750324e-02, 1.77455882e-02, 5.40824161e-02, 5.47851877e-02, 7.28805332e-02,
            4.24571928e-02, 3.69472023e-02, 2.47236873e-02, 4.09056230e-02, 4.27157861e-02,
            3.56204427e-02, 3.67460167e-02, 3.83930465e-02, 3.35224685e-02, 4.52358656e-02,
            4.21375334e-02, 1.86355600e-02, 4.73541177e-02, 2.81705872e-02, 1.37546608e-02,
            1.01406657e+00, 4.08428381e-02, 2.43437586e-02, 2.17483661e-02, 2.20537959e-02,
            2.03466935e-02, 1.68519490e-02, 1.11714072e-02, 8.12085567e-03, 1.75645940e-02,
            1.79244000e-02, 1.14605560e-02, 2.13827583e-02, 2.09873792e-02, 2.43730885e-03
        ], [
            5.81029900e-03, 7.75589622e-03, 1.17474414e-02, 1.36079024e-02, 1.37779320e-02,
            1.32634234e-02, 1.58091482e-02, 8.12958614e-03, 1.32381409e-02, 1.52207256e-02,
            1.35813147e-02, 1.37530114e-02, 1.60195488e-02, 1.44750837e-02, 1.29240707e-02,
            1.13706034e-02, 1.13079954e-02, 1.27530437e-02, 1.20579933e-02, 1.33969709e-02,
            1.03220045e-02, 1.01684121e+00, 1.42812343e-02, 1.34426716e-02, 3.06764623e-02,
            2.56414087e-02, 1.59470623e-02, 1.65600236e-02, 5.98506160e-03, 2.08704240e-02,
            1.64925197e-02, 9.50226618e-03, 1.40551839e-02, 1.88747546e-02, 9.60763648e-04
        ], [
            2.66275702e-02, 3.32896974e-02, 4.76952860e-02, 4.63884127e-02, 4.97750119e-02,
            4.90684088e-02, 4.08121063e-02, 5.90035878e-02, 4.10507334e-02, 4.24406169e-02,
            5.77525690e-02, 4.38312964e-02, 3.59821788e-02, 2.86393933e-02, 3.73843841e-02,
            3.73315471e-02, 3.76042804e-02, 3.77035877e-02, 2.75758962e-02, 2.79770718e-02,
            1.95533502e-02, 2.57031582e-02, 1.06230885e+00, 7.39512226e-02, 3.51531251e-02,
            7.77352589e-02, 1.82385683e-02, 1.26974144e-02, 6.99986013e-03, 1.47130728e-02,
            2.11624758e-02, 1.68106535e-02, 1.74130853e-02, 2.05721671e-02, 3.30936753e-03
        ], [
            7.41273584e-03, 9.34099565e-03, 1.10219149e-02, 1.13665717e-02, 1.34654485e-02,
            1.60752790e-02, 9.23166549e-03, 1.52001686e-02, 9.39580089e-03, 1.02015425e-02,
            1.79726290e-02, 1.52096933e-02, 1.04634454e-02, 8.13033197e-03, 9.01361281e-03,
            8.66298208e-03, 7.44994910e-03, 1.12521047e-02, 4.23958919e-03, 5.28743733e-03,
            3.38144190e-03, 5.51529876e-03, 6.78678640e-03, 1.12097389e+00, 6.93565887e-03,
            9.71940211e-03, 2.85694594e-03, 1.93721433e-03, 1.86944847e-03, 3.02036849e-03,
            3.82871547e-03, 2.66166844e-03, 2.85047466e-03, 3.80613705e-03, 8.31136542e-04
        ], [
            2.38946276e-03, 3.81904656e-03, 4.39139121e-03, 5.36237943e-03, 6.63166720e-03,
            5.07178413e-03, 5.82152246e-03, 3.69262576e-03, 4.97553671e-03, 5.43658737e-03,
            4.76266847e-03, 4.68980814e-03, 5.78820966e-03, 5.43209409e-03, 5.34371859e-03,
            4.09517777e-03, 2.95343648e-03, 3.99353130e-03, 3.95617757e-03, 5.11030910e-03,
            3.06862688e-03, 3.42929921e-03, 5.26040419e-03, 1.25292938e-02, 1.03935935e+00,
            2.23853715e-02, 5.73850129e-03, 5.05322862e-03, 1.58812833e-03, 6.22787355e-03,
            5.83125853e-03, 3.10832892e-03, 3.04100874e-03, 6.45578245e-03, 3.92815930e-04
        ], [
            1.17408317e-02, 1.34281678e-02, 2.08925211e-02, 1.81134491e-02, 2.11381141e-02,
            2.22087622e-02, 2.39751710e-02, 2.12230384e-02, 1.84612133e-02, 1.96225820e-02,
            2.30776460e-02, 2.00493870e-02, 1.85962623e-02, 1.55508940e-02, 1.94287634e-02,
            1.63195301e-02, 1.30059103e-02, 1.67229177e-02, 1.96827380e-02, 2.89045775e-02,
            1.67059373e-02, 1.75954133e-02, 5.45436549e-02, 1.51199483e-01, 1.24033657e-01,
            1.12999436e+00, 1.13216522e-02, 8.86716535e-03, 4.08926220e-03, 1.16959333e-02,
            1.19865304e-02, 6.83837769e-03, 9.60322772e-03, 1.38626880e-02, 1.34619468e-03
        ], [
            1.34401867e-02, 1.74446712e-02, 2.35600011e-02, 2.70046881e-02, 2.95199430e-02,
            2.53180344e-02, 3.68948763e-02, 1.90283765e-02, 2.95651367e-02, 2.83503472e-02,
            2.61698695e-02, 2.67749341e-02, 3.02432779e-02, 2.87167459e-02, 2.66976140e-02,
            2.50680707e-02, 2.01474302e-02, 3.24344528e-02, 3.12505500e-02, 3.50849799e-02,
            3.50622115e-02, 3.00880658e-02, 3.37004115e-02, 4.06220046e-02, 4.76100121e-02,
            4.70161074e-02, 1.15629629e+00, 4.97936587e-02, 1.43864260e-02, 4.45525242e-02,
            3.78637333e-02, 1.86590021e-02, 2.67538686e-02, 4.16077939e-02, 2.58182721e-03
        ], [
            5.06455689e-02, 5.85884120e-02, 6.32266481e-02, 7.42422051e-02, 6.68009428e-02,
            6.49405212e-02, 7.15267080e-02, 5.73987131e-02, 6.35322781e-02, 6.60074212e-02,
            7.05031573e-02, 6.71154629e-02, 6.66854483e-02, 5.78637138e-02, 6.37388510e-02,
            6.30767978e-02, 5.78688893e-02, 6.58326920e-02, 5.60073495e-02, 6.74278622e-02,
            6.46594071e-02, 6.45767604e-02, 9.00874293e-02, 9.62337427e-02, 8.32904786e-02,
            6.84144936e-02, 5.32389796e-02, 1.26868287e+00, 1.05912144e-01, 7.33868757e-02,
            5.53046196e-02, 3.10422206e-02, 5.51964353e-02, 8.38495979e-02, 1.47115289e-02
        ], [
            1.49364663e-02, 1.47142949e-02, 2.22109221e-02, 2.29043484e-02, 2.40890473e-02,
            2.33820564e-02, 2.85152940e-02, 1.56460854e-02, 2.15279533e-02, 2.38387329e-02,
            2.03228735e-02, 2.23535592e-02, 2.38922681e-02, 2.11533720e-02, 2.35280873e-02,
            2.13707335e-02, 1.60853024e-02, 2.65439630e-02, 3.64166006e-02, 3.29372716e-02,
            5.10011697e-02, 4.39083663e-02, 2.26025721e-02, 3.70077580e-02, 2.84839446e-02,
            3.85985797e-02, 2.95967080e-02, 3.78814246e-02, 1.04091696e+00, 3.79569161e-02,
            2.62215191e-02, 2.71946316e-02, 4.63273219e-02, 4.09466270e-02, 1.86658932e-03
        ], [
            5.08646954e-02, 6.81591405e-02, 1.07208279e-01, 1.03488018e-01, 1.00193057e-01,
            9.41579396e-02, 1.47366419e-01, 7.28955233e-02, 1.51858077e-01, 1.25811565e-01,
            1.01387062e-01, 1.04739420e-01, 1.27066336e-01, 1.34328794e-01, 1.32052401e-01,
            9.27692358e-02, 8.66557670e-02, 1.22663147e-01, 1.06209865e-01, 9.58652470e-02,
            9.97676823e-02, 9.74275131e-02, 9.84136102e-02, 1.03958021e-01, 1.56842334e-01,
            1.30650148e-01, 1.25437985e-01, 1.58563286e-01, 6.47885127e-02, 1.20235023e+00,
            1.27360311e-01, 6.06032435e-02, 1.07676166e-01, 1.35143491e-01, 1.03180195e-02
        ], [
            3.26640182e-03, 5.05455133e-03, 5.03711470e-03, 4.33749758e-03, 4.09227595e-03,
            6.83441761e-03, 6.83081323e-03, 4.53777067e-03, 6.28398636e-03, 5.73511749e-03,
            5.35875532e-03, 6.22214022e-03, 4.51716494e-03, 4.12167458e-03, 4.75510822e-03,
            4.76880867e-03, 7.21016070e-03, 4.55400231e-03, 4.55501398e-03, 3.04551716e-03,
            4.20314798e-03, 6.07531631e-03, 5.31377154e-03, 9.50467728e-03, 1.24825400e-02,
            6.17266160e-03, 5.08118015e-03, 3.34420911e-03, 4.18515954e-03, 4.80906878e-03,
            1.00925970e+00, 5.74599189e-03, 6.50917043e-03, 1.08753396e-02, 4.72341683e-04
        ], [
            1.21235466e-03, 1.12682853e-03, 1.68326374e-03, 1.72823683e-03, 1.81740277e-03,
            1.57939523e-03, 1.98302361e-03, 1.37935165e-03, 2.38741376e-03, 2.10387717e-03,
            1.83292443e-03, 1.93246532e-03, 2.31545154e-03, 2.23459960e-03, 2.30959762e-03,
            1.49998241e-03, 1.64980316e-03, 1.66856463e-03, 1.75605906e-03, 1.43217965e-03,
            1.69689468e-03, 1.51031204e-03, 2.09173851e-03, 1.58433951e-03, 3.15949024e-03,
            2.31344319e-03, 2.50887858e-03, 2.21329378e-03, 7.95351055e-04, 3.34264227e-03,
            5.02044860e-03, 1.01254990e+00, 1.93056846e-03, 2.62565014e-03, 1.39283202e-04
        ], [
            2.15112604e-03, 7.68736326e-04, 1.68851702e-03, 1.29608863e-03, 2.25058920e-03,
            1.36968403e-03, 1.18597950e-03, 8.09482752e-04, 1.29439833e-03, 1.23004051e-03,
            1.08347349e-03, 1.12081899e-03, 1.32169644e-03, 1.10361107e-03, 1.19552419e-03,
            1.16459302e-03, 8.20829828e-04, 1.12313769e-03, 1.07450072e-03, 9.85205295e-04,
            8.10351000e-04, 1.78219890e-03, 1.18358134e-03, 1.01984739e-03, 1.12301004e-03,
            1.23736733e-03, 1.18945042e-03, 1.01113007e-03, 3.53231220e-04, 1.22174819e-03,
            5.32904076e-03, 1.83218628e-03, 1.02594835e+00, 1.75139355e-03, 1.06927287e-04
        ], [
            9.57326921e-03, 1.39606686e-02, 1.83039931e-02, 1.91687742e-02, 2.14891937e-02,
            1.81797514e-02, 3.01802071e-02, 1.41275699e-02, 2.27719736e-02, 2.06770704e-02,
            1.83185576e-02, 2.03984549e-02, 2.02713470e-02, 1.80115777e-02, 1.95276133e-02,
            1.68324441e-02, 1.51071418e-02, 1.88423004e-02, 1.83746955e-02, 1.63163627e-02,
            1.65414592e-02, 2.40009002e-02, 1.95235051e-02, 1.78415633e-02, 2.12990123e-02,
            2.26990942e-02, 3.32349927e-02, 2.15205963e-02, 1.30961893e-02, 3.92801408e-02,
            2.62196246e-02, 1.66205461e-02, 2.36097590e-02, 1.09567758e+00, 3.49391590e-03
        ], [
            4.58467061e-05, 4.75028257e-05, 7.53905398e-05, 1.27820513e-04, 8.32554601e-05,
            8.33302557e-05, 9.33268660e-05, 5.33530714e-05, 8.34624202e-05, 9.56290362e-05,
            6.65491873e-05, 6.95182511e-05, 7.84071469e-05, 6.23296142e-05, 1.08004044e-04,
            5.83851201e-05, 4.21672796e-05, 1.14663652e-04, 9.66790345e-05, 4.44163857e-05,
            7.25774561e-05, 9.53917837e-05, 1.11039068e-04, 5.93617191e-05, 6.97524535e-05,
            6.54592746e-05, 7.79636427e-05, 9.29698689e-05, 2.71633778e-04, 1.03284840e-04,
            1.06473216e-04, 6.29505123e-05, 1.05777359e-04, 1.58961571e-04, 1.00010105e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2003():
    """
    Leontief matrix 2003



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17385679e+00, 4.68124904e-03, 3.59831224e-01, 8.21291166e-02, 1.14865729e-01,
            1.94114745e-01, 2.96927432e-02, 7.04004375e-03, 1.89509653e-02, 2.93029282e-02,
            9.07055229e-03, 7.53843659e-03, 8.10745511e-03, 7.22186382e-03, 9.01630099e-03,
            3.45332104e-02, 4.75513695e-03, 2.26015499e-02, 5.94722435e-03, 9.11522601e-03,
            8.09402509e-03, 1.03972322e-01, 7.40725627e-03, 9.49584439e-03, 8.55467288e-03,
            1.23774237e-02, 5.10992095e-03, 4.21255093e-03, 3.20070297e-03, 6.41410477e-03,
            1.04350089e-02, 9.04998811e-03, 1.24771761e-02, 1.16023940e-02, 2.21779895e-02
        ], [
            2.36881404e-02, 1.10923628e+00, 2.50156082e-02, 3.09489553e-02, 2.55971311e-02,
            3.16080478e-02, 2.82653143e-02, 5.45896216e-01, 8.88604857e-02, 4.43078242e-02,
            1.17726775e-01, 1.03706192e-01, 3.81032618e-02, 2.59315960e-02, 3.26021189e-02,
            3.11320078e-02, 1.58888890e-01, 5.68197907e-02, 1.60070242e-02, 1.07259577e-02,
            1.09234466e-02, 1.77435037e-02, 4.37028983e-02, 6.01970053e-02, 4.92352211e-02,
            2.51055207e-02, 1.09994837e-02, 6.55633927e-03, 8.07025019e-03, 1.10300162e-02,
            1.87943803e-02, 1.30853967e-02, 1.59674841e-02, 1.70545052e-02, 6.77092457e-02
        ], [
            8.12607679e-02, 4.28664302e-03, 1.20635460e+00, 1.91477915e-02, 1.44946293e-01,
            2.09803814e-02, 1.25611954e-02, 5.63320649e-03, 2.18548189e-02, 1.50610781e-02,
            8.78254446e-03, 7.86328336e-03, 8.38887400e-03, 7.72996490e-03, 8.46158473e-03,
            1.11759729e-02, 4.81696869e-03, 8.33105142e-03, 6.75381420e-03, 1.00546676e-02,
            9.03495988e-03, 1.89158293e-01, 6.79996694e-03, 1.36398602e-02, 1.27736779e-02,
            9.76673069e-03, 6.73126766e-03, 5.57476437e-03, 2.41953417e-03, 7.98415285e-03,
            1.43377130e-02, 1.31550580e-02, 2.19899461e-02, 1.45894434e-02, 5.44373377e-03
        ], [
            5.47863943e-03, 3.50094046e-03, 7.07728954e-03, 1.45262453e+00, 1.00997703e-01,
            1.04662599e-02, 1.31938779e-02, 3.68035165e-03, 9.64822462e-03, 2.44977461e-02,
            9.77885117e-03, 7.10399192e-03, 8.91999032e-03, 7.76767265e-03, 1.66301740e-02,
            3.63859494e-02, 3.04555587e-03, 7.89072493e-03, 5.78478940e-03, 5.29519405e-03,
            5.89538574e-03, 6.90303954e-03, 5.31437923e-03, 6.83147500e-03, 5.23451560e-03,
            5.10737740e-03, 3.29668787e-03, 2.31081965e-03, 1.46728928e-03, 3.93473386e-03,
            6.90380902e-03, 2.85779422e-03, 6.70310115e-03, 1.10149103e-02, 6.28867306e-04
        ], [
            4.63767298e-04, 3.41057705e-04, 6.57551238e-04, 1.46837395e-02, 1.27129412e+00,
            1.24071927e-03, 1.32100949e-03, 3.69594419e-04, 7.93723254e-04, 2.37646320e-03,
            9.09886382e-04, 8.02773518e-04, 9.24311467e-04, 9.20704449e-04, 3.33386234e-03,
            6.40675020e-03, 3.96401691e-04, 6.92462831e-04, 1.07116327e-03, 7.49239888e-04,
            6.06674178e-04, 5.09740780e-04, 6.41470521e-04, 6.43500332e-04, 6.14230192e-04,
            5.52052742e-04, 5.85561326e-04, 2.20734158e-04, 1.30309601e-04, 3.96160789e-04,
            8.71997263e-04, 3.06904671e-04, 4.29881600e-04, 8.94601402e-04, 5.52201887e-05
        ], [
            4.86735529e-03, 4.26422173e-03, 6.53902039e-03, 6.13255676e-03, 7.09228096e-03,
            1.28952822e+00, 2.06247360e-02, 3.55173143e-03, 5.83545115e-03, 9.06138881e-03,
            1.33422210e-02, 1.01966960e-02, 9.30575815e-03, 6.66949952e-03, 1.02462544e-02,
            9.88264776e-02, 4.24534605e-03, 4.85280849e-02, 4.26537109e-03, 3.81585013e-03,
            3.54646203e-03, 6.19517641e-03, 4.50292706e-03, 4.03522794e-03, 3.76754084e-03,
            7.21530496e-03, 3.47931908e-03, 2.45759554e-03, 5.26249602e-03, 3.67490152e-03,
            5.39148436e-03, 5.15819883e-03, 3.98016599e-03, 7.01258631e-03, 7.74578126e-04
        ], [
            1.43277909e-02, 1.20126625e-02, 5.16104123e-02, 3.69831226e-02, 4.44557994e-02,
            3.63267140e-02, 1.31847067e+00, 1.28114441e-02, 4.17275639e-02, 4.49113299e-02,
            4.26399502e-02, 2.38533504e-02, 2.99167185e-02, 3.40865029e-02, 2.75765658e-02,
            4.31962157e-02, 1.36435987e-02, 2.37534714e-02, 2.22365028e-02, 2.55441306e-02,
            2.38968304e-02, 2.76167758e-02, 1.96525927e-02, 1.83941201e-02, 2.19189520e-02,
            3.10651597e-02, 2.64020995e-02, 2.99503052e-02, 9.18291613e-03, 3.94195906e-02,
            3.53982491e-02, 2.98733566e-02, 2.33204502e-02, 4.59914739e-02, 1.89115960e-03
        ], [
            2.22438808e-02, 1.73611181e-02, 1.83505523e-02, 2.24454487e-02, 2.00211128e-02,
            2.08508139e-02, 1.71892225e-02, 1.05388741e+00, 5.61885106e-02, 3.04806740e-02,
            3.37449196e-02, 2.68196483e-02, 1.73256443e-02, 1.34638021e-02, 1.57048013e-02,
            1.58082623e-02, 2.91102581e-02, 2.60522319e-02, 1.15372414e-02, 1.07641864e-02,
            8.59931931e-03, 1.20049592e-02, 6.05019452e-02, 9.70281123e-02, 7.77309568e-02,
            2.89987064e-02, 8.82979273e-03, 5.33225710e-03, 4.05864046e-03, 9.93229929e-03,
            1.52045862e-02, 7.36205205e-03, 1.09509862e-02, 1.21529306e-02, 1.71956283e-03
        ], [
            6.93017038e-02, 2.54427651e-02, 5.27367629e-02, 1.54687454e-01, 1.06617336e-01,
            8.33190264e-02, 8.05595102e-02, 3.58976480e-02, 1.33401586e+00, 3.29401606e-01,
            7.20895685e-02, 4.88432336e-02, 5.04433655e-02, 5.89580208e-02, 6.14272355e-02,
            6.98271401e-02, 2.11956726e-02, 4.45808986e-02, 2.82448076e-02, 1.32566506e-02,
            1.17188065e-02, 2.47792126e-02, 1.95542265e-02, 2.21220280e-02, 1.94286134e-02,
            1.93584825e-02, 1.27305523e-02, 8.04855499e-03, 7.01255573e-03, 1.66568637e-02,
            2.57101461e-02, 1.24680950e-02, 9.64136509e-02, 2.67856919e-02, 3.54543608e-03
        ], [
            1.40593691e-02, 1.37343299e-02, 3.45612029e-02, 2.91727226e-02, 6.72061980e-02,
            2.43703050e-02, 3.28058558e-02, 1.33459161e-02, 3.61696455e-02, 1.16919059e+00,
            2.45786113e-02, 2.25875876e-02, 5.12440464e-02, 5.05839672e-02, 7.53603407e-02,
            5.13523139e-02, 1.04242157e-02, 3.48043778e-02, 3.25375567e-02, 1.02207864e-02,
            1.03452006e-02, 1.46679853e-02, 2.10109828e-02, 1.49381290e-02, 1.42676869e-02,
            1.57954337e-02, 1.08767783e-02, 4.73870460e-03, 4.42586300e-03, 9.08991363e-03,
            1.14003391e-02, 5.46597354e-03, 1.34543627e-02, 1.16571288e-02, 1.49835864e-03
        ], [
            5.82263503e-03, 9.05684166e-03, 1.26190572e-02, 8.61514089e-03, 8.29488948e-03,
            1.61621269e-02, 6.20160928e-03, 8.16233157e-03, 1.30405705e-02, 1.45638030e-02,
            1.14615755e+00, 2.24251875e-02, 1.61683660e-02, 2.29146342e-02, 1.98365290e-02,
            1.43126026e-02, 7.72656536e-03, 1.02340247e-01, 8.72999567e-03, 3.81010950e-03,
            3.63047251e-03, 8.41332986e-03, 5.88411379e-03, 5.30145179e-03, 5.05027502e-03,
            6.20351271e-03, 5.45574640e-03, 2.48942214e-03, 6.83627170e-03, 4.08119620e-03,
            6.19782816e-03, 4.95103408e-03, 6.02673421e-03, 6.09780273e-03, 8.62835011e-04
        ], [
            2.26223035e-02, 5.09843431e-02, 4.30328249e-02, 3.63839714e-02, 4.54832130e-02,
            6.96351208e-02, 3.79549419e-02, 4.18337247e-02, 4.89744626e-02, 7.12732520e-02,
            8.20415734e-02, 1.49598031e+00, 3.10182041e-01, 1.58316420e-01, 2.41224242e-01,
            1.44074771e-01, 3.47698021e-02, 1.70952335e-01, 6.74112288e-02, 1.77385817e-02,
            1.60800061e-02, 2.39244649e-02, 3.50190979e-02, 4.06000031e-02, 3.56777071e-02,
            3.19174617e-02, 2.42213590e-02, 1.01329537e-02, 1.44836115e-02, 1.92832427e-02,
            2.72340567e-02, 1.33704013e-02, 1.94312607e-02, 2.34748321e-02, 4.65573841e-03
        ], [
            1.41667910e-02, 2.50298464e-02, 1.52122109e-02, 2.26343725e-02, 1.92598662e-02,
            2.22932108e-02, 1.74135981e-02, 2.18801043e-02, 1.93815207e-02, 2.64498962e-02,
            3.32023661e-02, 4.02295511e-02, 1.16758802e+00, 3.45729273e-02, 6.85339140e-02,
            2.27291127e-02, 1.91251274e-02, 3.44713732e-02, 3.71936372e-02, 8.43008429e-03,
            6.79309923e-03, 9.13577868e-03, 1.51751945e-02, 2.25779421e-02, 1.61584773e-02,
            1.40000695e-02, 9.01742932e-03, 4.56648677e-03, 4.74028615e-03, 8.22380766e-03,
            1.31839653e-02, 5.95062461e-03, 8.46969725e-03, 1.00324582e-02, 2.12016947e-03
        ], [
            1.47214817e-02, 2.42244370e-02, 2.39631917e-02, 3.33209205e-02, 3.06392171e-02,
            3.27033556e-02, 3.80910852e-02, 2.51072446e-02, 3.77329828e-02, 4.72992790e-02,
            4.00797923e-02, 5.78266366e-02, 1.81688452e-01, 1.48298840e+00, 1.52279777e-01,
            5.70310769e-02, 4.38030573e-02, 7.45564161e-02, 7.19537223e-02, 2.43921265e-02,
            2.21595910e-02, 2.11063045e-02, 3.03398518e-02, 3.09115882e-02, 3.16037812e-02,
            3.23478445e-02, 7.67416006e-02, 1.81740753e-02, 1.13550248e-02, 3.73290648e-02,
            3.85911988e-02, 1.97837499e-02, 4.66221130e-02, 3.22465094e-02, 3.31510095e-03
        ], [
            1.38616626e-02, 1.59277059e-02, 1.60360231e-02, 1.70624663e-02, 1.76832004e-02,
            1.87748236e-02, 1.50283680e-02, 1.50357372e-02, 1.50013275e-02, 1.94855628e-02,
            1.96008800e-02, 2.35912289e-02, 3.76456314e-02, 2.00273238e-02, 1.45271059e+00,
            1.89912902e-02, 1.25190301e-02, 1.96899917e-02, 1.34957693e-01, 1.43077094e-02,
            1.09881551e-02, 1.04628389e-02, 6.38545757e-02, 7.91480415e-02, 8.52718474e-02,
            3.12570182e-02, 1.27164781e-02, 6.71053943e-03, 4.46664674e-03, 1.26827063e-02,
            3.16219139e-02, 7.01985902e-03, 8.00583543e-03, 1.71530932e-02, 1.74960014e-03
        ], [
            2.33687881e-03, 2.58138337e-03, 3.62789689e-03, 8.91960171e-03, 6.64254129e-03,
            8.60487592e-03, 7.23250421e-03, 2.65727417e-03, 4.58166349e-03, 6.19610825e-03,
            6.19326211e-03, 1.62733440e-02, 9.71430145e-03, 7.15495245e-03, 1.20896178e-02,
            1.05017044e+00, 3.04145980e-03, 9.43931056e-03, 4.84861236e-03, 2.55784931e-03,
            2.71926244e-03, 4.30101983e-03, 4.12719670e-03, 3.25264677e-03, 4.07264395e-03,
            3.76428770e-03, 2.75410876e-03, 2.28888298e-03, 2.28119664e-03, 3.56150484e-03,
            5.79851167e-03, 4.15831940e-03, 4.67839982e-03, 7.38529052e-03, 2.48451153e-03
        ], [
            2.38020873e-02, 4.48104800e-02, 3.48289827e-02, 4.59775606e-02, 3.28168943e-02,
            4.16697309e-02, 4.73878917e-02, 4.29207345e-02, 5.31241744e-02, 5.00660277e-02,
            6.74176091e-02, 6.11703510e-02, 3.78211529e-02, 2.96898671e-02, 3.63624647e-02,
            3.00614365e-02, 1.12850970e+00, 3.00421708e-02, 2.37737439e-02, 1.45330547e-02,
            2.20427696e-02, 3.90060194e-02, 2.89838279e-02, 2.01020525e-02, 2.05229570e-02,
            2.92577890e-02, 1.97257119e-02, 1.20978144e-02, 1.42059456e-02, 1.53107144e-02,
            2.68986069e-02, 2.88900911e-02, 2.44524081e-02, 2.86458821e-02, 3.78784376e-03
        ], [
            8.44025876e-03, 1.33134886e-02, 1.04068984e-02, 1.12704963e-02, 9.89644976e-03,
            1.16477978e-02, 1.18328972e-02, 1.16969731e-02, 1.17253935e-02, 1.14074134e-02,
            1.60934482e-02, 1.36219155e-02, 1.15302590e-02, 9.52775347e-03, 1.07539018e-02,
            9.50572764e-03, 2.82722506e-02, 1.05906519e+00, 1.06996941e-02, 8.89532877e-03,
            1.05248915e-02, 1.31969094e-02, 1.64419375e-02, 1.15887346e-02, 1.14464970e-02,
            2.15102187e-02, 1.76183508e-02, 1.12197941e-02, 4.55047603e-02, 1.08502027e-02,
            2.48079870e-02, 1.86932870e-02, 1.25614019e-02, 1.71990081e-02, 1.64975232e-03
        ], [
            9.30403991e-03, 7.32990653e-03, 1.44618017e-02, 1.40263320e-02, 1.62493515e-02,
            1.41637129e-02, 1.18602901e-02, 8.36382728e-03, 1.31612442e-02, 1.43038762e-02,
            1.42201740e-02, 1.38853728e-02, 1.28235336e-02, 1.07768379e-02, 1.40880251e-02,
            1.27301384e-02, 1.28964241e-02, 1.41603402e-02, 1.01543030e+00, 7.13450378e-03,
            6.90842424e-03, 1.02882994e-02, 2.12294231e-02, 8.36102538e-03, 9.69053181e-03,
            1.31182474e-02, 7.43186625e-03, 5.83647581e-03, 3.88186984e-03, 9.34763010e-03,
            8.34466599e-03, 5.29359418e-03, 7.40972072e-03, 1.04323974e-02, 8.15481780e-04
        ], [
            5.97719516e-02, 4.08562551e-02, 1.13664446e-01, 1.08900254e-01, 1.27020258e-01,
            1.05115925e-01, 9.25099914e-02, 7.29951925e-02, 9.01529549e-02, 9.86091822e-02,
            7.85941972e-02, 9.39988002e-02, 9.68825272e-02, 8.64201019e-02, 1.06790098e-01,
            8.46970912e-02, 4.57018330e-02, 8.16493835e-02, 6.19220004e-02, 1.03858239e+00,
            2.55056842e-02, 7.90492713e-02, 4.50221712e-02, 5.13814490e-02, 4.84098812e-02,
            4.37398095e-02, 2.81078302e-02, 1.61753119e-02, 1.23906056e-02, 2.94184281e-02,
            3.87522812e-02, 2.28963843e-02, 4.68315189e-02, 3.69787830e-02, 4.67299302e-03
        ], [
            2.94317361e-02, 1.70342413e-02, 5.36253005e-02, 5.42100814e-02, 6.89052921e-02,
            4.25834978e-02, 3.68703343e-02, 2.40793487e-02, 4.07920084e-02, 4.22559807e-02,
            3.54751697e-02, 3.62607535e-02, 3.70337576e-02, 3.15154047e-02, 4.46717777e-02,
            4.24711994e-02, 1.85019617e-02, 4.93194418e-02, 2.76455225e-02, 1.31979273e-02,
            1.01362816e+00, 4.06788531e-02, 2.47292740e-02, 2.15034418e-02, 2.21114944e-02,
            2.03142243e-02, 1.68014490e-02, 1.13266526e-02, 7.96864820e-03, 1.72450814e-02,
            1.81081197e-02, 1.17225317e-02, 2.14072904e-02, 2.08238336e-02, 2.30430626e-03
        ], [
            5.76098506e-03, 7.82221425e-03, 1.16220441e-02, 1.34816363e-02, 1.38962121e-02,
            1.32149235e-02, 1.53628595e-02, 7.70873598e-03, 1.28618406e-02, 1.46979853e-02,
            1.32521241e-02, 1.34682267e-02, 1.52552817e-02, 1.37169067e-02, 1.28591466e-02,
            1.13628625e-02, 9.45136372e-03, 1.30047385e-02, 1.19647433e-02, 1.31258623e-02,
            1.02527699e-02, 1.01688311e+00, 1.46597143e-02, 1.37878522e-02, 3.19872183e-02,
            2.64085116e-02, 1.61644887e-02, 1.70807327e-02, 5.95278841e-03, 2.08181959e-02,
            1.71838474e-02, 1.02147139e-02, 1.42189455e-02, 1.86828752e-02, 9.27013162e-04
        ], [
            2.74942101e-02, 3.24979102e-02, 4.80099224e-02, 4.62481284e-02, 4.96489046e-02,
            5.01012686e-02, 4.12392381e-02, 6.00829285e-02, 4.10661103e-02, 4.19679999e-02,
            5.76736152e-02, 4.29875363e-02, 3.50746529e-02, 2.77568941e-02, 3.71401228e-02,
            3.81991728e-02, 3.47585115e-02, 3.91199455e-02, 2.72255744e-02, 2.85538443e-02,
            1.97699757e-02, 2.58022126e-02, 1.06205974e+00, 6.28964562e-02, 3.34226140e-02,
            7.57986421e-02, 1.82822732e-02, 1.26932825e-02, 6.67524721e-03, 1.45166153e-02,
            2.16616441e-02, 1.71720534e-02, 1.74457428e-02, 2.02836428e-02, 3.15377228e-03
        ], [
            7.04319025e-03, 9.45940961e-03, 1.10902443e-02, 1.18122222e-02, 1.36265440e-02,
            1.56524883e-02, 9.04878825e-03, 1.23966020e-02, 9.02129540e-03, 9.88758439e-03,
            1.65201009e-02, 1.42502229e-02, 1.01019709e-02, 7.79531697e-03, 9.00089863e-03,
            8.75890567e-03, 7.08297727e-03, 1.08614425e-02, 4.34400374e-03, 6.18105081e-03,
            3.77917670e-03, 5.55127582e-03, 6.79106821e-03, 1.13947091e+00, 6.96894202e-03,
            9.36149203e-03, 2.86517648e-03, 1.94607014e-03, 2.17365261e-03, 3.06002578e-03,
            3.77727818e-03, 2.64103760e-03, 2.83737103e-03, 3.83152249e-03, 8.21670045e-04
        ], [
            2.45028474e-03, 3.75348628e-03, 4.51106859e-03, 5.42425655e-03, 6.71311180e-03,
            5.29342076e-03, 6.17663109e-03, 3.59919146e-03, 5.02479065e-03, 5.46426218e-03,
            4.79937526e-03, 4.75298123e-03, 5.64870411e-03, 5.24052207e-03, 5.35613617e-03,
            4.25082584e-03, 2.88685698e-03, 4.23425286e-03, 4.17915095e-03, 5.04376629e-03,
            3.23526358e-03, 3.68698731e-03, 5.83482913e-03, 1.35288155e-02, 1.03677154e+00,
            2.20623490e-02, 6.10216101e-03, 5.47545498e-03, 1.63064957e-03, 6.61944990e-03,
            6.80972506e-03, 3.44628698e-03, 3.23288962e-03, 6.62749740e-03, 3.84417310e-04
        ], [
            1.13407461e-02, 1.32964229e-02, 2.11436673e-02, 1.83392492e-02, 2.10194276e-02,
            2.23869982e-02, 2.40731991e-02, 1.98158677e-02, 1.80349978e-02, 1.93044847e-02,
            2.28713671e-02, 1.96512491e-02, 1.79840642e-02, 1.48896143e-02, 1.89794534e-02,
            1.62768042e-02, 1.25583870e-02, 1.85081863e-02, 1.98988378e-02, 2.85315676e-02,
            1.74287364e-02, 1.76820539e-02, 5.43984473e-02, 1.54398156e-01, 1.18210880e-01,
            1.12688195e+00, 1.12017116e-02, 9.07355201e-03, 4.11961604e-03, 1.17988119e-02,
            1.23320251e-02, 7.01303872e-03, 9.70684304e-03, 1.37731199e-02, 1.29637888e-03
        ], [
            1.34933698e-02, 1.66927197e-02, 2.32071957e-02, 2.73200293e-02, 2.92099973e-02,
            2.53063651e-02, 3.61963278e-02, 1.78621599e-02, 2.87336825e-02, 2.73288133e-02,
            2.54768081e-02, 2.64998989e-02, 2.86171873e-02, 2.77243112e-02, 2.65582837e-02,
            2.60573583e-02, 1.91007591e-02, 3.24806899e-02, 3.14439195e-02, 3.42741812e-02,
            3.48527561e-02, 3.06422379e-02, 3.39225144e-02, 4.06744621e-02, 4.83263419e-02,
            4.68956690e-02, 1.15750081e+00, 5.02051629e-02, 1.43702519e-02, 4.40880795e-02,
            3.86984497e-02, 1.93305809e-02, 2.69064875e-02, 4.09452133e-02, 2.51091801e-03
        ], [
            4.80220217e-02, 5.58255598e-02, 6.17085177e-02, 7.33658154e-02, 6.61972242e-02,
            6.37543667e-02, 7.08839283e-02, 5.49353805e-02, 6.28694902e-02, 6.39562343e-02,
            6.78309752e-02, 6.59255096e-02, 6.45933251e-02, 5.60973424e-02, 6.37425914e-02,
            6.70777156e-02, 5.53525396e-02, 6.66633472e-02, 5.54921143e-02, 6.43405728e-02,
            6.56404125e-02, 6.39917301e-02, 8.82580982e-02, 8.76719835e-02, 8.22186550e-02,
            7.12118768e-02, 5.33704935e-02, 1.27426550e+00, 1.09877563e-01, 7.43263786e-02,
            5.20091148e-02, 3.32929236e-02, 5.60691196e-02, 8.35631671e-02, 1.11182317e-02
        ], [
            1.40991815e-02, 1.54551671e-02, 2.15531363e-02, 2.31206024e-02, 2.43820631e-02,
            2.29000615e-02, 2.80043598e-02, 1.53339852e-02, 2.11566156e-02, 2.32853977e-02,
            2.02212240e-02, 2.19802503e-02, 2.30062855e-02, 2.01905253e-02, 2.35876832e-02,
            2.15363633e-02, 1.53088156e-02, 2.68502643e-02, 3.61495249e-02, 3.21323039e-02,
            5.11162889e-02, 4.30820159e-02, 2.26475322e-02, 3.60042030e-02, 3.03769391e-02,
            3.91225683e-02, 2.90110260e-02, 3.88204130e-02, 1.03743246e+00, 3.84367688e-02,
            2.64899181e-02, 2.71282031e-02, 4.64723357e-02, 4.12521433e-02, 1.76049532e-03
        ], [
            5.04154740e-02, 6.82102266e-02, 1.05257142e-01, 1.04221963e-01, 1.00253842e-01,
            9.41480589e-02, 1.45801799e-01, 6.83300755e-02, 1.49409202e-01, 1.23694697e-01,
            1.00668170e-01, 1.03018099e-01, 1.24003154e-01, 1.29694478e-01, 1.31286870e-01,
            9.27736111e-02, 7.96738193e-02, 1.23517669e-01, 1.06274350e-01, 9.50236567e-02,
            9.94369040e-02, 9.74900967e-02, 9.87575650e-02, 1.00623526e-01, 1.56122938e-01,
            1.30892435e-01, 1.23148012e-01, 1.64413517e-01, 6.54058977e-02, 1.20039560e+00,
            1.31064264e-01, 6.19765615e-02, 1.08492328e-01, 1.35166878e-01, 1.01479433e-02
        ], [
            3.34826122e-03, 5.25862046e-03, 5.02929300e-03, 4.46748598e-03, 4.21158128e-03,
            6.85917793e-03, 6.88692858e-03, 4.31402995e-03, 6.30441155e-03, 5.71850371e-03,
            5.68471138e-03, 6.34780537e-03, 4.54601615e-03, 4.06940821e-03, 4.89137609e-03,
            4.84249309e-03, 7.53099456e-03, 4.74228610e-03, 4.56100278e-03, 3.05829212e-03,
            4.22023213e-03, 6.14423561e-03, 5.38866115e-03, 9.58416889e-03, 1.26826291e-02,
            6.36327746e-03, 5.13271019e-03, 3.46618981e-03, 4.10811421e-03, 4.89191539e-03,
            1.00952113e+00, 5.77922439e-03, 6.63395681e-03, 1.11369103e-02, 4.78007443e-04
        ], [
            1.20642734e-03, 1.09409386e-03, 1.67928131e-03, 1.71590636e-03, 1.83349192e-03,
            1.58000723e-03, 1.98687821e-03, 1.31057892e-03, 2.35069025e-03, 2.10042307e-03,
            1.84545305e-03, 1.92708860e-03, 2.22240598e-03, 2.12750982e-03, 2.25181419e-03,
            1.51925565e-03, 1.58507065e-03, 1.70078383e-03, 1.75027024e-03, 1.42812867e-03,
            1.61959411e-03, 1.52441769e-03, 2.05955166e-03, 1.57661358e-03, 3.27056037e-03,
            2.36239776e-03, 2.62221449e-03, 2.25301544e-03, 8.05750448e-04, 3.28802013e-03,
            5.19246523e-03, 1.01223137e+00, 1.99004094e-03, 2.61046640e-03, 1.30957818e-04
        ], [
            1.96402946e-03, 7.60135250e-04, 1.58041523e-03, 1.29207012e-03, 2.12830306e-03,
            1.37692737e-03, 1.18037295e-03, 7.67657207e-04, 1.28814132e-03, 1.21682297e-03,
            1.15378975e-03, 1.14331546e-03, 1.37716259e-03, 1.09646201e-03, 1.24221687e-03,
            1.18284904e-03, 8.12230611e-04, 1.12110715e-03, 1.07138861e-03, 9.86033433e-04,
            7.93841259e-04, 1.73468393e-03, 1.17233083e-03, 1.00741132e-03, 1.06794839e-03,
            1.20933469e-03, 1.20767851e-03, 9.62253504e-04, 3.47436455e-04, 1.19895682e-03,
            5.39483230e-03, 1.79972823e-03, 1.02671997e+00, 1.73506873e-03, 1.03300044e-04
        ], [
            9.41069123e-03, 1.33929912e-02, 1.75924935e-02, 1.86387044e-02, 2.09952069e-02,
            1.79525127e-02, 2.91536667e-02, 1.29552217e-02, 2.18519877e-02, 1.98120082e-02,
            1.78606309e-02, 1.93542951e-02, 1.89946991e-02, 1.68587553e-02, 1.92462184e-02,
            1.64111060e-02, 1.41027811e-02, 1.87134215e-02, 1.75255294e-02, 1.57122668e-02,
            1.62664932e-02, 2.40955553e-02, 2.05679993e-02, 1.76299260e-02, 2.11598280e-02,
            2.37032152e-02, 3.23697518e-02, 2.15720963e-02, 1.31131813e-02, 3.84763338e-02,
            2.71425223e-02, 1.70239287e-02, 2.34541931e-02, 1.09459326e+00, 3.72551716e-03
        ], [
            4.14836825e-05, 5.04402773e-05, 7.24732522e-05, 1.25555858e-04, 7.40513874e-05,
            8.10343775e-05, 9.27068266e-05, 4.89527299e-05, 6.75650743e-05, 7.56403707e-05,
            6.52045624e-05, 6.66500844e-05, 7.87462294e-05, 6.12163049e-05, 1.50439658e-04,
            5.61195168e-05, 3.77631614e-05, 1.20517918e-04, 1.04933170e-04, 4.31650698e-05,
            7.40060377e-05, 9.65913864e-05, 1.18625981e-04, 7.85697339e-05, 7.61095210e-05,
            6.81317256e-05, 8.03661094e-05, 1.02595340e-04, 2.60948310e-04, 1.11813636e-04,
            1.10415299e-04, 6.27998295e-05, 9.33406825e-05, 1.68534011e-04, 1.00011908e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2004():
    """
    Leontief matrix 2004



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17725946e+00, 4.70979270e-03, 3.68109222e-01, 8.60168424e-02, 1.24470513e-01,
            1.95156060e-01, 3.13576462e-02, 6.69016059e-03, 1.93278549e-02, 2.93976764e-02,
            9.20834979e-03, 7.55312145e-03, 8.15624037e-03, 7.23932161e-03, 9.18402919e-03,
            3.35242687e-02, 4.69029300e-03, 2.11568587e-02, 6.62282583e-03, 8.96028719e-03,
            8.45022854e-03, 1.06356276e-01, 7.71100607e-03, 9.25456380e-03, 8.44727149e-03,
            1.44263803e-02, 5.25037132e-03, 4.46782876e-03, 3.40237684e-03, 6.77913302e-03,
            1.04930680e-02, 9.82753149e-03, 1.28617904e-02, 1.21306042e-02, 2.47451793e-02
        ], [
            2.37460738e-02, 1.11243893e+00, 2.57055433e-02, 3.23143775e-02, 2.77943773e-02,
            3.27820766e-02, 2.88452239e-02, 5.41087803e-01, 9.19562368e-02, 4.63584813e-02,
            1.24481620e-01, 1.11717152e-01, 4.07062624e-02, 2.77401707e-02, 3.51558440e-02,
            3.24770914e-02, 1.58296214e-01, 5.83311133e-02, 1.75218633e-02, 1.09109743e-02,
            1.13014807e-02, 1.84041746e-02, 4.55323381e-02, 6.08595369e-02, 5.45017966e-02,
            2.78940523e-02, 1.11766064e-02, 6.87491324e-03, 8.97052173e-03, 1.16098336e-02,
            1.95487849e-02, 1.31032069e-02, 1.67495093e-02, 1.79352720e-02, 6.91498577e-02
        ], [
            8.28748340e-02, 4.35727906e-03, 1.21033259e+00, 2.02292587e-02, 1.45282887e-01,
            2.16801677e-02, 1.25370759e-02, 5.98751073e-03, 2.18486990e-02, 1.51372252e-02,
            8.73834774e-03, 7.98949688e-03, 8.58456424e-03, 7.83346987e-03, 8.66645169e-03,
            1.11750330e-02, 4.73716991e-03, 8.46964101e-03, 7.12453987e-03, 9.83390209e-03,
            9.37644365e-03, 1.90123245e-01, 7.12770128e-03, 1.25725976e-02, 1.21621259e-02,
            1.06205850e-02, 6.95193752e-03, 5.83084430e-03, 2.69134651e-03, 8.20126763e-03,
            1.43295143e-02, 1.40376771e-02, 2.21671268e-02, 1.50059206e-02, 5.40807514e-03
        ], [
            5.32625848e-03, 3.52287627e-03, 6.79283451e-03, 1.45752158e+00, 1.02608580e-01,
            1.01481690e-02, 1.36217987e-02, 3.60806586e-03, 9.47121682e-03, 2.50046296e-02,
            9.35564240e-03, 6.94894577e-03, 8.71192485e-03, 7.79290611e-03, 1.64267812e-02,
            3.60562082e-02, 3.10748005e-03, 8.07936702e-03, 6.06703186e-03, 5.18862732e-03,
            6.14529853e-03, 7.04348502e-03, 5.50298316e-03, 7.00323228e-03, 5.43341671e-03,
            5.32218184e-03, 3.19590650e-03, 2.29462013e-03, 1.65649769e-03, 4.14415025e-03,
            6.48781627e-03, 2.82276078e-03, 7.06765184e-03, 1.06382580e-02, 6.37806732e-04
        ], [
            4.42528670e-04, 3.04714560e-04, 6.24163322e-04, 1.50436750e-02, 1.25021604e+00,
            1.10477211e-03, 1.30889071e-03, 3.43716991e-04, 7.33179894e-04, 1.92490149e-03,
            7.58971575e-04, 7.38769795e-04, 8.62897970e-04, 8.81650041e-04, 3.35643646e-03,
            5.99076584e-03, 3.56014982e-04, 6.38552779e-04, 1.04029459e-03, 6.66800930e-04,
            5.77360484e-04, 4.87849536e-04, 6.45413093e-04, 5.80012439e-04, 5.84691409e-04,
            5.30094273e-04, 5.63095427e-04, 2.10375163e-04, 1.29219343e-04, 3.70882924e-04,
            7.41023319e-04, 2.86146753e-04, 4.19412056e-04, 8.65465885e-04, 5.44663426e-05
        ], [
            4.81650492e-03, 4.45658678e-03, 6.59895441e-03, 6.36447591e-03, 7.43623569e-03,
            1.30867855e+00, 2.13885824e-02, 3.72145638e-03, 5.85325384e-03, 8.75513864e-03,
            1.38416297e-02, 1.02301406e-02, 9.35789592e-03, 6.84437731e-03, 1.04671178e-02,
            9.93496486e-02, 4.19440523e-03, 4.87314535e-02, 4.86631774e-03, 3.85839652e-03,
            3.72083227e-03, 6.35059320e-03, 4.61825473e-03, 3.97420782e-03, 3.73558953e-03,
            7.25755397e-03, 3.44019941e-03, 2.52681646e-03, 5.90707626e-03, 3.79850545e-03,
            5.66145311e-03, 5.55867397e-03, 4.31066376e-03, 7.15397928e-03, 7.59333385e-04
        ], [
            1.44686883e-02, 1.18218684e-02, 5.27459990e-02, 3.77728812e-02, 4.64618072e-02,
            3.74765318e-02, 1.33045218e+00, 1.29261588e-02, 4.07064423e-02, 4.41326788e-02,
            4.34975769e-02, 2.31748451e-02, 2.93915237e-02, 3.29706681e-02, 2.72838204e-02,
            4.25378301e-02, 1.33440738e-02, 2.38838197e-02, 2.34917880e-02, 2.51719363e-02,
            2.42556571e-02, 2.79916740e-02, 1.99916204e-02, 1.82265768e-02, 2.12261948e-02,
            3.21442911e-02, 2.57158598e-02, 3.04852484e-02, 9.86228579e-03, 4.01775329e-02,
            3.58222862e-02, 3.07367065e-02, 2.34931124e-02, 4.68209153e-02, 1.96576992e-03
        ], [
            2.32432235e-02, 1.70656546e-02, 1.94017945e-02, 2.44764643e-02, 2.21015347e-02,
            2.21363523e-02, 1.79711976e-02, 1.05859477e+00, 6.15673118e-02, 3.31420366e-02,
            3.55510077e-02, 2.96843032e-02, 1.89897024e-02, 1.45805049e-02, 1.70886546e-02,
            1.66375930e-02, 3.08429953e-02, 2.68059228e-02, 1.30605036e-02, 1.11332066e-02,
            9.08320247e-03, 1.27106169e-02, 6.31031013e-02, 9.86805581e-02, 8.78854182e-02,
            3.29069877e-02, 9.04601051e-03, 5.71399079e-03, 4.57355814e-03, 1.05825403e-02,
            1.58242523e-02, 7.93012889e-03, 1.17100319e-02, 1.28300439e-02, 1.81526715e-03
        ], [
            7.00559239e-02, 2.49858685e-02, 5.29666424e-02, 1.55831285e-01, 1.10506152e-01,
            8.36546196e-02, 8.07104525e-02, 3.59308314e-02, 1.33572413e+00, 3.31546925e-01,
            7.22619752e-02, 4.82072604e-02, 5.02268832e-02, 6.02243833e-02, 6.12997389e-02,
            6.83113454e-02, 2.13265666e-02, 4.46590336e-02, 2.90202847e-02, 1.32012053e-02,
            1.21987527e-02, 2.49620876e-02, 2.00243724e-02, 2.16648502e-02, 2.00377555e-02,
            2.05262580e-02, 1.27102099e-02, 8.18768351e-03, 7.63777275e-03, 1.70943667e-02,
            2.54018669e-02, 1.27879277e-02, 9.51995398e-02, 2.72795561e-02, 3.75190870e-03
        ], [
            1.38811015e-02, 1.36460746e-02, 3.46833212e-02, 2.92198216e-02, 6.93963390e-02,
            2.45849655e-02, 3.30039753e-02, 1.31042821e-02, 3.46344556e-02, 1.17015730e+00,
            2.41226577e-02, 2.20106403e-02, 4.97535639e-02, 4.99712389e-02, 7.46602124e-02,
            5.01107505e-02, 1.02802632e-02, 3.48195379e-02, 3.30138185e-02, 1.01285072e-02,
            1.07551767e-02, 1.48195057e-02, 2.11561357e-02, 1.39366904e-02, 1.42908222e-02,
            1.65374882e-02, 1.07550500e-02, 4.88495331e-03, 4.73401795e-03, 9.36533750e-03,
            1.15718375e-02, 5.66993929e-03, 1.39183142e-02, 1.24865467e-02, 1.56595825e-03
        ], [
            5.63142980e-03, 9.23748737e-03, 1.21544212e-02, 8.43975767e-03, 8.24316615e-03,
            1.65216367e-02, 6.16804775e-03, 8.12417392e-03, 1.26287864e-02, 1.40103794e-02,
            1.14917594e+00, 2.18368362e-02, 1.61469684e-02, 2.27921200e-02, 1.99746641e-02,
            1.40810391e-02, 7.55016512e-03, 1.06368846e-01, 9.33086097e-03, 3.79342249e-03,
            3.75380173e-03, 8.32941799e-03, 6.06519369e-03, 4.97015800e-03, 5.04966386e-03,
            6.45996790e-03, 5.38417192e-03, 2.52896080e-03, 6.63874867e-03, 4.17084073e-03,
            6.31565063e-03, 4.90158211e-03, 6.17356466e-03, 6.20754134e-03, 9.26472582e-04
        ], [
            2.23627373e-02, 5.02977026e-02, 4.25370906e-02, 3.56405205e-02, 4.59170501e-02,
            6.87783203e-02, 3.80865479e-02, 4.17465259e-02, 4.67313728e-02, 6.98741547e-02,
            8.19101500e-02, 1.50725458e+00, 3.13943867e-01, 1.64593009e-01, 2.47253368e-01,
            1.44209140e-01, 3.43298540e-02, 1.69447449e-01, 6.75935874e-02, 1.71583693e-02,
            1.63822402e-02, 2.37714001e-02, 3.52389580e-02, 3.73808881e-02, 3.54532933e-02,
            3.36108888e-02, 2.40447199e-02, 1.02790772e-02, 1.46479912e-02, 1.97641969e-02,
            2.73642780e-02, 1.36188501e-02, 1.99064161e-02, 2.42386168e-02, 4.76287449e-03
        ], [
            1.50920573e-02, 2.65565823e-02, 1.59333480e-02, 2.39151789e-02, 2.13245541e-02,
            2.40220437e-02, 1.84606752e-02, 2.35661438e-02, 2.02498287e-02, 2.78667636e-02,
            3.54486140e-02, 4.40344429e-02, 1.18355937e+00, 3.80897065e-02, 7.53498282e-02,
            2.57252822e-02, 1.95769512e-02, 3.66425003e-02, 4.11299268e-02, 8.41203686e-03,
            7.30878678e-03, 9.75940046e-03, 1.62900175e-02, 2.29111952e-02, 1.82677078e-02,
            1.61220792e-02, 9.56152458e-03, 4.83233438e-03, 5.29915421e-03, 8.75520695e-03,
            1.41469342e-02, 6.43601878e-03, 9.53955528e-03, 1.11667681e-02, 2.41952730e-03
        ], [
            1.60082208e-02, 2.66643039e-02, 2.59816552e-02, 3.48311052e-02, 3.31063826e-02,
            3.54377708e-02, 4.06337398e-02, 2.68437150e-02, 3.99166513e-02, 4.95107361e-02,
            4.28405208e-02, 6.12165335e-02, 1.98180870e-01, 1.51587110e+00, 1.63335465e-01,
            6.09582617e-02, 4.99371744e-02, 8.03913090e-02, 7.92822035e-02, 2.52475796e-02,
            2.44703911e-02, 2.32165022e-02, 3.27628348e-02, 3.13892690e-02, 3.36197423e-02,
            3.64536416e-02, 8.09273547e-02, 1.99247695e-02, 1.28830206e-02, 4.04224793e-02,
            4.26642716e-02, 2.25750043e-02, 5.06554856e-02, 3.58380952e-02, 3.80178641e-03
        ], [
            1.37137909e-02, 1.59827767e-02, 1.60514085e-02, 1.69393528e-02, 1.77353835e-02,
            1.88345924e-02, 1.52969080e-02, 1.50127363e-02, 1.48077573e-02, 1.92528287e-02,
            2.00116455e-02, 2.40647112e-02, 3.86755443e-02, 1.96644018e-02, 1.45455151e+00,
            2.00915181e-02, 1.28648011e-02, 2.01508647e-02, 1.36272747e-01, 1.34527336e-02,
            1.12482555e-02, 1.07999381e-02, 6.41752283e-02, 7.03172004e-02, 8.26468358e-02,
            3.31117014e-02, 1.22303478e-02, 6.80489653e-03, 4.84498961e-03, 1.27522535e-02,
            3.14323122e-02, 7.29234695e-03, 8.27274038e-03, 1.84974371e-02, 1.85069963e-03
        ], [
            2.03901830e-03, 2.46830526e-03, 3.48715492e-03, 9.16701750e-03, 6.82674429e-03,
            8.31167644e-03, 7.12634944e-03, 2.66053005e-03, 4.52551062e-03, 6.11057538e-03,
            6.06052236e-03, 1.56918541e-02, 9.65992285e-03, 6.93006345e-03, 1.22713772e-02,
            1.05126403e+00, 3.12571889e-03, 9.50429984e-03, 4.92700573e-03, 2.55554022e-03,
            2.83507123e-03, 4.29145609e-03, 4.56348097e-03, 3.19286126e-03, 4.35303001e-03,
            3.92633319e-03, 2.71216641e-03, 2.18161837e-03, 2.49653892e-03, 3.52594442e-03,
            5.72908467e-03, 4.14668053e-03, 4.68597415e-03, 7.36827342e-03, 2.47039998e-03
        ], [
            2.44009696e-02, 4.76733237e-02, 3.60904428e-02, 4.73582046e-02, 3.47121216e-02,
            4.29896912e-02, 4.77398012e-02, 4.76658836e-02, 5.36169920e-02, 5.03295324e-02,
            7.12808601e-02, 6.37722805e-02, 3.98948333e-02, 3.05891280e-02, 3.75565084e-02,
            3.12322341e-02, 1.13913815e+00, 3.14045648e-02, 2.57567149e-02, 1.47779719e-02,
            2.31795051e-02, 3.96468900e-02, 3.07400152e-02, 2.04316538e-02, 2.13625924e-02,
            3.07659095e-02, 1.99659312e-02, 1.22760643e-02, 1.60225071e-02, 1.56782359e-02,
            2.70277010e-02, 2.98030284e-02, 2.49274683e-02, 2.96778246e-02, 4.10658992e-03
        ], [
            7.99748340e-03, 1.22519695e-02, 1.00209392e-02, 1.06517388e-02, 9.54908651e-03,
            1.11536443e-02, 1.09736933e-02, 1.13835037e-02, 1.09307933e-02, 1.05932447e-02,
            1.51865341e-02, 1.27258912e-02, 1.08770430e-02, 8.94117073e-03, 1.01766696e-02,
            9.17442598e-03, 2.58626298e-02, 1.05806273e+00, 1.08554918e-02, 8.35229006e-03,
            1.01614530e-02, 1.25634363e-02, 1.62184077e-02, 1.03902857e-02, 1.05625446e-02,
            2.11574220e-02, 1.63966920e-02, 1.09011006e-02, 3.85310030e-02, 1.03968309e-02,
            2.43299423e-02, 1.70275342e-02, 1.23556659e-02, 1.64001613e-02, 1.86547611e-03
        ], [
            9.84349284e-03, 7.32844176e-03, 1.52093134e-02, 1.41312375e-02, 1.66009551e-02,
            1.48215774e-02, 1.20263723e-02, 8.63591969e-03, 1.32680058e-02, 1.44976879e-02,
            1.43739319e-02, 1.40154027e-02, 1.31053669e-02, 1.07593483e-02, 1.44784993e-02,
            1.32871187e-02, 1.30653748e-02, 1.43155180e-02, 1.01603656e+00, 7.41875517e-03,
            7.18258630e-03, 1.06268224e-02, 2.15427126e-02, 8.18188768e-03, 1.05457518e-02,
            1.37346827e-02, 7.46863670e-03, 6.24437947e-03, 4.19369838e-03, 9.48951725e-03,
            8.45267617e-03, 5.43700230e-03, 7.69908973e-03, 1.08036756e-02, 8.75280859e-04
        ], [
            5.96261456e-02, 3.95514778e-02, 1.13297652e-01, 1.05253589e-01, 1.24752910e-01,
            1.07189478e-01, 9.38232950e-02, 7.57757929e-02, 8.93926587e-02, 9.84467984e-02,
            7.80030104e-02, 9.49421223e-02, 9.87460186e-02, 8.54047859e-02, 1.09222219e-01,
            8.40354938e-02, 4.45024260e-02, 8.22579408e-02, 6.40656866e-02, 1.03995342e+00,
            2.64857401e-02, 7.99275934e-02, 4.73212885e-02, 5.13409858e-02, 5.03551808e-02,
            4.49673092e-02, 2.86066231e-02, 1.68254209e-02, 1.35809755e-02, 3.03642311e-02,
            3.98114235e-02, 2.37837368e-02, 4.80265787e-02, 3.83979782e-02, 4.82811099e-03
        ], [
            2.91065771e-02, 1.58198505e-02, 5.35936972e-02, 5.30984191e-02, 6.68532733e-02,
            4.25724362e-02, 3.63678511e-02, 2.43207916e-02, 4.06842647e-02, 4.20917054e-02,
            3.44210889e-02, 3.65132651e-02, 3.73504207e-02, 3.10264204e-02, 4.53491025e-02,
            4.34746620e-02, 1.73269473e-02, 4.97506812e-02, 2.84342537e-02, 1.28920135e-02,
            1.01385430e+00, 3.95629856e-02, 2.49331288e-02, 2.04060909e-02, 2.18247107e-02,
            2.05893648e-02, 1.64167154e-02, 1.14911348e-02, 8.42117681e-03, 1.69402194e-02,
            1.77859185e-02, 1.17612016e-02, 2.17530677e-02, 2.09748075e-02, 2.35378367e-03
        ], [
            5.81870183e-03, 7.92611433e-03, 1.17529736e-02, 1.32585382e-02, 1.40823333e-02,
            1.29660383e-02, 1.48945903e-02, 7.79480176e-03, 1.23909455e-02, 1.40356494e-02,
            1.29480674e-02, 1.28601351e-02, 1.48685220e-02, 1.31161996e-02, 1.26888270e-02,
            1.11114264e-02, 8.46827386e-03, 1.30946011e-02, 1.20172716e-02, 1.29485143e-02,
            1.11403015e-02, 1.01705391e+00, 1.47536162e-02, 1.35780111e-02, 2.80832319e-02,
            2.87615374e-02, 1.66844728e-02, 1.81963926e-02, 6.78120106e-03, 2.09667149e-02,
            1.77509834e-02, 1.15482570e-02, 1.46989403e-02, 1.89650890e-02, 9.79258188e-04
        ], [
            2.67785340e-02, 3.28080032e-02, 4.89235500e-02, 4.75545758e-02, 5.07060211e-02,
            5.08677451e-02, 4.12248623e-02, 6.21384984e-02, 4.12979751e-02, 4.18622115e-02,
            5.76857739e-02, 4.30014878e-02, 3.56264323e-02, 2.80008049e-02, 3.74195757e-02,
            3.88437766e-02, 3.37749184e-02, 4.08755100e-02, 2.95250119e-02, 2.93417646e-02,
            2.01554511e-02, 2.62300805e-02, 1.06251455e+00, 5.26633565e-02, 3.21637200e-02,
            7.57883761e-02, 1.80557414e-02, 1.24954467e-02, 7.24431779e-03, 1.46450912e-02,
            2.16718257e-02, 1.71102079e-02, 1.81427081e-02, 2.07829161e-02, 3.40338431e-03
        ], [
            6.89115196e-03, 1.05220859e-02, 1.12293641e-02, 1.20877789e-02, 1.32173266e-02,
            1.52273463e-02, 9.08115383e-03, 1.18410343e-02, 8.68920703e-03, 9.74443414e-03,
            1.61493252e-02, 1.37571817e-02, 1.02285203e-02, 7.83397855e-03, 9.18141378e-03,
            8.47683685e-03, 7.30571172e-03, 1.03472904e-02, 5.13232335e-03, 7.23953144e-03,
            4.44744053e-03, 5.66715983e-03, 6.96229284e-03, 1.15375556e+00, 7.22601676e-03,
            9.44570960e-03, 3.04195005e-03, 2.19123712e-03, 2.90305089e-03, 3.46257937e-03,
            3.84586610e-03, 2.73751497e-03, 3.01618937e-03, 3.95632152e-03, 9.26533662e-04
        ], [
            2.46539955e-03, 3.51302671e-03, 4.51757115e-03, 5.26944043e-03, 6.32465241e-03,
            5.19354054e-03, 5.90829415e-03, 3.45017558e-03, 4.86113505e-03, 5.14239731e-03,
            4.62625880e-03, 4.43961940e-03, 5.49322911e-03, 5.07916632e-03, 5.30072230e-03,
            4.14490503e-03, 2.68889209e-03, 4.11255591e-03, 4.49570941e-03, 4.91899522e-03,
            3.28248393e-03, 3.71895057e-03, 5.59644575e-03, 1.25021450e-02, 1.03743835e+00,
            2.03897703e-02, 6.14822465e-03, 5.59818437e-03, 1.78939328e-03, 6.46654869e-03,
            6.62871705e-03, 3.81827930e-03, 3.27596448e-03, 6.65102808e-03, 3.85568630e-04
        ], [
            1.12875169e-02, 1.21620724e-02, 2.12291399e-02, 1.84622309e-02, 2.11046521e-02,
            2.26092529e-02, 2.37584317e-02, 1.96329476e-02, 1.77347707e-02, 1.91373113e-02,
            2.27353197e-02, 1.92044959e-02, 1.79464073e-02, 1.47779472e-02, 1.90981161e-02,
            1.63802332e-02, 1.22032339e-02, 2.00012920e-02, 2.09569795e-02, 2.91893193e-02,
            1.83744107e-02, 1.77473599e-02, 5.42401878e-02, 1.56859581e-01, 1.11255574e-01,
            1.12680269e+00, 1.11694617e-02, 9.36913319e-03, 4.53527977e-03, 1.19192798e-02,
            1.22502715e-02, 7.40610205e-03, 9.96143829e-03, 1.40862605e-02, 1.28937666e-03
        ], [
            1.37408644e-02, 1.59281646e-02, 2.34927571e-02, 2.66025670e-02, 2.86818977e-02,
            2.51949444e-02, 3.49507191e-02, 1.77058837e-02, 2.74161701e-02, 2.60773687e-02,
            2.46629045e-02, 2.56478552e-02, 2.76569431e-02, 2.70333072e-02, 2.61427285e-02,
            2.63367337e-02, 1.84878065e-02, 3.14576537e-02, 3.26601876e-02, 3.31851391e-02,
            3.50186264e-02, 3.07943498e-02, 3.37428712e-02, 4.07314746e-02, 4.42814273e-02,
            4.64801931e-02, 1.14823205e+00, 5.02593189e-02, 1.54460330e-02, 4.43746041e-02,
            3.82918914e-02, 1.97730699e-02, 2.76241917e-02, 4.03951394e-02, 2.63349165e-03
        ], [
            4.56855315e-02, 4.99160559e-02, 5.96353539e-02, 7.11758054e-02, 6.41693737e-02,
            6.06424520e-02, 6.50332203e-02, 5.09400059e-02, 5.84982040e-02, 5.81905133e-02,
            6.25140343e-02, 6.08076783e-02, 6.05944987e-02, 5.24133532e-02, 6.03592480e-02,
            6.76846158e-02, 5.10299323e-02, 6.32912125e-02, 5.18189436e-02, 5.96171428e-02,
            6.40824864e-02, 5.99439930e-02, 8.29984636e-02, 7.58432132e-02, 7.28083163e-02,
            7.18237023e-02, 4.83069305e-02, 1.28107605e+00, 1.14366798e-01, 7.09572898e-02,
            5.31281881e-02, 3.14839540e-02, 5.30354847e-02, 8.07350880e-02, 1.06755269e-02
        ], [
            1.34717020e-02, 1.36928398e-02, 2.17444583e-02, 2.26096159e-02, 2.48269249e-02,
            2.28504489e-02, 2.81196818e-02, 1.43658480e-02, 2.04426149e-02, 2.26912159e-02,
            1.96689097e-02, 2.08763724e-02, 2.23855757e-02, 1.95315852e-02, 2.32988077e-02,
            2.15572202e-02, 1.49110031e-02, 2.67275026e-02, 3.58770075e-02, 3.16004499e-02,
            5.44137190e-02, 4.37753314e-02, 2.25473278e-02, 3.32075754e-02, 2.84880242e-02,
            3.92415554e-02, 2.85819411e-02, 4.07564740e-02, 1.04016739e+00, 3.93493574e-02,
            2.61739472e-02, 2.70045768e-02, 4.80770794e-02, 4.24776685e-02, 1.72151699e-03
        ], [
            5.01497283e-02, 6.54816733e-02, 1.05237963e-01, 9.90369818e-02, 9.90272609e-02,
            9.30275509e-02, 1.41215116e-01, 6.59605003e-02, 1.42452895e-01, 1.18634428e-01,
            9.79325196e-02, 9.77102375e-02, 1.19166001e-01, 1.22626567e-01, 1.28469265e-01,
            8.95634596e-02, 7.49584747e-02, 1.21374975e-01, 1.06198747e-01, 9.54186389e-02,
            1.01363671e-01, 9.80709380e-02, 9.70088380e-02, 9.47917391e-02, 1.44231594e-01,
            1.30459927e-01, 1.17622335e-01, 1.68433299e-01, 7.22129519e-02, 1.20215446e+00,
            1.32434339e-01, 6.31866909e-02, 1.10603048e-01, 1.36306274e-01, 1.04433229e-02
        ], [
            3.34460390e-03, 4.79184614e-03, 5.15658690e-03, 4.26334626e-03, 4.24280832e-03,
            7.13005873e-03, 6.74255433e-03, 4.01173141e-03, 6.00635688e-03, 5.43257497e-03,
            5.64362639e-03, 6.27665277e-03, 4.49761112e-03, 3.97408687e-03, 4.83930093e-03,
            4.97970475e-03, 6.85079266e-03, 4.73257119e-03, 4.93933121e-03, 3.16002833e-03,
            4.41005488e-03, 6.25846235e-03, 5.83145243e-03, 9.16196019e-03, 1.20007008e-02,
            6.24822364e-03, 5.10714328e-03, 3.66682256e-03, 5.05919246e-03, 5.25308914e-03,
            1.00948210e+00, 5.85864468e-03, 6.75536908e-03, 1.15026693e-02, 4.68372931e-04
        ], [
            1.27175422e-03, 1.11756966e-03, 1.73139687e-03, 1.69576564e-03, 1.87919614e-03,
            1.60166262e-03, 1.95709786e-03, 1.30672415e-03, 2.29126203e-03, 2.07149339e-03,
            1.75399870e-03, 1.89412597e-03, 2.23189372e-03, 2.07796381e-03, 2.34615335e-03,
            1.52304344e-03, 1.59956807e-03, 1.72969647e-03, 1.78835888e-03, 1.54945666e-03,
            1.65271057e-03, 1.58152642e-03, 2.11609021e-03, 1.51664028e-03, 3.20524488e-03,
            2.45269733e-03, 2.68047507e-03, 2.31060622e-03, 8.66645168e-04, 3.48746156e-03,
            5.48485926e-03, 1.01418931e+00, 2.10077547e-03, 2.74243737e-03, 1.47434699e-04
        ], [
            1.92121394e-03, 8.00101189e-04, 1.53209371e-03, 1.26618493e-03, 2.02732230e-03,
            1.37885927e-03, 1.16780673e-03, 7.66263262e-04, 1.25160803e-03, 1.17475014e-03,
            1.18116576e-03, 1.12690629e-03, 1.45355985e-03, 1.08463226e-03, 1.26605816e-03,
            1.16047073e-03, 8.24048199e-04, 1.08872295e-03, 1.06348017e-03, 9.17372251e-04,
            7.80653601e-04, 1.59694235e-03, 1.14957200e-03, 9.20151420e-04, 1.02778616e-03,
            1.16891251e-03, 1.16328633e-03, 9.63538211e-04, 3.54450518e-04, 1.15975393e-03,
            5.45086098e-03, 1.85160648e-03, 1.02760624e+00, 1.83566623e-03, 1.10673495e-04
        ], [
            9.58457660e-03, 1.34147103e-02, 1.78166190e-02, 1.81987733e-02, 2.16090414e-02,
            1.80937764e-02, 2.88416718e-02, 1.27833185e-02, 2.10971297e-02, 1.92245612e-02,
            1.78939930e-02, 1.85371912e-02, 1.85197638e-02, 1.62515148e-02, 1.92553447e-02,
            1.62441082e-02, 1.38965036e-02, 1.88884105e-02, 1.79630422e-02, 1.56835592e-02,
            1.69054182e-02, 2.42438668e-02, 2.16311917e-02, 1.77724098e-02, 2.05980656e-02,
            2.56291327e-02, 3.15528260e-02, 2.21892295e-02, 1.43818632e-02, 3.90034899e-02,
            2.77591575e-02, 1.82576627e-02, 2.41388104e-02, 1.09502154e+00, 3.97294274e-03
        ], [
            4.18050290e-05, 5.11311920e-05, 7.34884595e-05, 1.41068179e-04, 8.24330625e-05,
            8.12994538e-05, 9.01467836e-05, 5.06044363e-05, 6.60644332e-05, 7.24717068e-05,
            6.55169027e-05, 6.38918461e-05, 7.98806002e-05, 6.00109393e-05, 1.55716111e-04,
            5.67188124e-05, 3.69342234e-05, 1.18009057e-04, 1.00623762e-04, 4.30226451e-05,
            7.74985710e-05, 9.83081175e-05, 1.38314965e-04, 6.67200986e-05, 7.01950550e-05,
            6.91866975e-05, 7.77059523e-05, 1.12341898e-04, 2.65073014e-04, 1.10736972e-04,
            1.08588850e-04, 6.33057301e-05, 9.49631205e-05, 1.69949584e-04, 1.00013505e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2005():
    """
    Leontief matrix 2005



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17727159e+00, 4.50981961e-03, 3.68368415e-01, 9.61223075e-02, 1.36885580e-01,
            2.09523354e-01, 3.46974862e-02, 7.07646538e-03, 2.07705540e-02, 3.07556175e-02,
            1.00495108e-02, 8.07923837e-03, 8.60996687e-03, 7.70957320e-03, 9.59164862e-03,
            3.52297143e-02, 4.96273144e-03, 2.07761690e-02, 6.96527950e-03, 8.89496146e-03,
            8.65829406e-03, 1.07208064e-01, 8.32200388e-03, 9.50081219e-03, 8.75286422e-03,
            1.63893295e-02, 5.36740709e-03, 4.71628353e-03, 3.57282938e-03, 6.93394497e-03,
            1.11323439e-02, 1.02343656e-02, 1.31137171e-02, 1.25184522e-02, 2.64683703e-02
        ], [
            2.35435901e-02, 1.11416277e+00, 2.63473979e-02, 3.36459447e-02, 2.88900290e-02,
            3.39347223e-02, 2.98132523e-02, 5.51364699e-01, 9.14553010e-02, 4.77262105e-02,
            1.28070114e-01, 1.15642590e-01, 4.21312950e-02, 2.91226622e-02, 3.61336281e-02,
            3.26958232e-02, 1.64680639e-01, 6.16852415e-02, 1.80041340e-02, 1.09019025e-02,
            1.15828766e-02, 1.92784285e-02, 4.68344140e-02, 6.00093622e-02, 6.22328632e-02,
            3.01584781e-02, 1.16142685e-02, 7.21909226e-03, 9.35278239e-03, 1.17212204e-02,
            2.00032389e-02, 1.33589200e-02, 1.74962415e-02, 1.83831759e-02, 7.01531577e-02
        ], [
            8.59104074e-02, 4.30245808e-03, 1.21375282e+00, 2.32711551e-02, 1.59734173e-01,
            2.37974674e-02, 1.31849758e-02, 6.59604937e-03, 2.32998624e-02, 1.64098166e-02,
            9.63576783e-03, 8.69636087e-03, 9.20728469e-03, 8.45797077e-03, 9.12537932e-03,
            1.18642678e-02, 5.13286213e-03, 9.11185249e-03, 7.37426792e-03, 9.80780254e-03,
            9.83098433e-03, 1.92322078e-01, 7.65308386e-03, 1.27733928e-02, 1.22739385e-02,
            1.13735037e-02, 7.11709092e-03, 6.08837803e-03, 2.89177112e-03, 8.25605176e-03,
            1.51642088e-02, 1.42745834e-02, 2.24122113e-02, 1.54137382e-02, 5.87188301e-03
        ], [
            5.56000206e-03, 3.43853228e-03, 7.30645229e-03, 1.49762605e+00, 1.09331251e-01,
            1.09094725e-02, 1.48540542e-02, 3.87179496e-03, 1.02752759e-02, 2.74681560e-02,
            1.01570342e-02, 7.53316268e-03, 9.21419134e-03, 8.46992913e-03, 1.65987466e-02,
            3.85107555e-02, 3.46370485e-03, 8.75279164e-03, 6.41524263e-03, 5.27082797e-03,
            6.73796551e-03, 7.55357368e-03, 6.00760103e-03, 7.74582636e-03, 5.99845157e-03,
            5.87544608e-03, 3.40489347e-03, 2.50354742e-03, 1.82212099e-03, 4.43874905e-03,
            6.93923081e-03, 3.18047162e-03, 7.89062726e-03, 1.08478587e-02, 6.66986402e-04
        ], [
            4.54009228e-04, 2.89836506e-04, 6.40658585e-04, 1.83107967e-02, 1.25870582e+00,
            1.15991558e-03, 1.49461652e-03, 3.57490265e-04, 7.72640723e-04, 1.98672564e-03,
            8.02791172e-04, 7.50361554e-04, 8.82299535e-04, 9.42815786e-04, 3.63068052e-03,
            6.34924088e-03, 3.63659438e-04, 6.68367386e-04, 1.11401997e-03, 6.51026812e-04,
            5.74382512e-04, 5.14885703e-04, 7.00947824e-04, 6.14718863e-04, 6.35210871e-04,
            5.55070209e-04, 5.79022963e-04, 2.27667163e-04, 1.39157888e-04, 3.82302672e-04,
            9.23714781e-04, 3.15940283e-04, 4.70562223e-04, 9.34630856e-04, 5.79749013e-05
        ], [
            4.61778422e-03, 4.22281590e-03, 6.48839764e-03, 6.63565828e-03, 7.52571839e-03,
            1.31786825e+00, 2.19837491e-02, 3.89438484e-03, 5.89860242e-03, 8.86633367e-03,
            1.46804340e-02, 1.06786410e-02, 9.68931449e-03, 7.10811367e-03, 1.04674527e-02,
            9.80871128e-02, 4.22624501e-03, 4.88904072e-02, 5.05381128e-03, 3.76561993e-03,
            3.73497592e-03, 6.41697156e-03, 4.77325893e-03, 3.92942213e-03, 3.80754751e-03,
            7.19074218e-03, 3.42120391e-03, 2.57480088e-03, 6.11312493e-03, 3.74879480e-03,
            5.80130205e-03, 5.79851087e-03, 4.49783409e-03, 7.17753366e-03, 7.23740277e-04
        ], [
            1.43449720e-02, 1.16196931e-02, 5.30993052e-02, 3.93036960e-02, 4.74797667e-02,
            3.82432414e-02, 1.33285340e+00, 1.39195841e-02, 4.08650611e-02, 4.50025652e-02,
            4.56012693e-02, 2.35892270e-02, 2.91318488e-02, 3.26690342e-02, 2.69141282e-02,
            4.23100320e-02, 1.38253145e-02, 2.44977252e-02, 2.33593407e-02, 2.38183882e-02,
            2.46000375e-02, 2.82200418e-02, 2.03414645e-02, 1.81226451e-02, 2.10034065e-02,
            3.17629665e-02, 2.52255865e-02, 3.12380576e-02, 1.02023228e-02, 3.94158733e-02,
            3.62678292e-02, 3.10046927e-02, 2.44316298e-02, 4.66764214e-02, 1.99065971e-03
        ], [
            2.31373144e-02, 1.54013235e-02, 1.96863517e-02, 2.54649233e-02, 2.27681907e-02,
            2.26478513e-02, 1.85685410e-02, 1.08635781e+00, 6.68211814e-02, 3.50146633e-02,
            3.72775037e-02, 3.06411007e-02, 1.93870676e-02, 1.50468319e-02, 1.74127296e-02,
            1.69954782e-02, 3.34827180e-02, 2.70952069e-02, 1.35573683e-02, 1.13313518e-02,
            9.39297933e-03, 1.31176550e-02, 6.42872168e-02, 9.63684219e-02, 1.02693599e-01,
            3.52064105e-02, 9.40076870e-03, 5.99368742e-03, 4.82180164e-03, 1.07296205e-02,
            1.66263783e-02, 8.05263125e-03, 1.23745682e-02, 1.31367347e-02, 1.77236121e-03
        ], [
            6.68543742e-02, 2.35175837e-02, 5.16757294e-02, 1.55349166e-01, 1.08364265e-01,
            8.28957117e-02, 7.93829188e-02, 3.98089497e-02, 1.33550058e+00, 3.36867365e-01,
            7.35931498e-02, 4.80297034e-02, 4.96252441e-02, 6.06712064e-02, 5.98264288e-02,
            6.76683393e-02, 2.13873726e-02, 4.49853867e-02, 2.80008910e-02, 1.23447275e-02,
            1.20259122e-02, 2.47350805e-02, 2.05437469e-02, 2.17613667e-02, 2.14091767e-02,
            2.09359899e-02, 1.25976334e-02, 8.23836214e-03, 7.95236934e-03, 1.67074584e-02,
            2.57109520e-02, 1.27338399e-02, 9.53085074e-02, 2.67695018e-02, 3.74062342e-03
        ], [
            1.36872880e-02, 1.26216850e-02, 3.56704060e-02, 3.09653782e-02, 6.99875875e-02,
            2.43966756e-02, 3.33901653e-02, 1.35124249e-02, 3.54873757e-02, 1.17830582e+00,
            2.54294951e-02, 2.25401726e-02, 4.96948039e-02, 5.12720012e-02, 7.43699121e-02,
            5.10915824e-02, 1.05266568e-02, 3.57458047e-02, 3.26801869e-02, 9.83691541e-03,
            1.08193865e-02, 1.52331571e-02, 2.19349339e-02, 1.38479647e-02, 1.48299235e-02,
            1.67546788e-02, 1.09043517e-02, 5.05852718e-03, 5.00594832e-03, 9.34083975e-03,
            1.16891620e-02, 5.90459071e-03, 1.41551974e-02, 1.28043642e-02, 1.58514568e-03
        ], [
            5.39940698e-03, 8.97608617e-03, 1.20346725e-02, 8.73634081e-03, 8.37431308e-03,
            1.59402871e-02, 6.22764077e-03, 8.73835690e-03, 1.29866086e-02, 1.41456943e-02,
            1.15904440e+00, 2.25080140e-02, 1.64852272e-02, 2.32107218e-02, 1.99898157e-02,
            1.43424974e-02, 7.84537284e-03, 1.11434957e-01, 9.60749970e-03, 3.53043870e-03,
            3.72279647e-03, 8.33759620e-03, 6.22101176e-03, 4.91933534e-03, 5.25185173e-03,
            6.68765265e-03, 5.51501140e-03, 2.59135812e-03, 6.87739181e-03, 4.14090611e-03,
            6.44909621e-03, 4.98887490e-03, 6.39060918e-03, 6.27742772e-03, 9.34394107e-04
        ], [
            2.19888176e-02, 4.73591028e-02, 4.11388435e-02, 3.68385676e-02, 4.58369026e-02,
            6.72663314e-02, 3.78364017e-02, 4.21465424e-02, 4.71292897e-02, 7.00665570e-02,
            8.36650527e-02, 1.50715080e+00, 3.15172990e-01, 1.70632375e-01, 2.41486355e-01,
            1.42498459e-01, 3.48844275e-02, 1.71154790e-01, 6.62963641e-02, 1.63377244e-02,
            1.60519299e-02, 2.35732035e-02, 3.51078105e-02, 3.53001849e-02, 3.53982072e-02,
            3.53039878e-02, 2.42896302e-02, 1.04740218e-02, 1.49348418e-02, 1.96029298e-02,
            2.72999955e-02, 1.39217831e-02, 2.06418411e-02, 2.46553557e-02, 4.69922430e-03
        ], [
            1.51457814e-02, 2.41617790e-02, 1.64443094e-02, 2.54877918e-02, 2.23396296e-02,
            2.48884971e-02, 1.91723579e-02, 2.43086946e-02, 2.12509482e-02, 2.92460269e-02,
            3.77836349e-02, 4.63830224e-02, 1.19208474e+00, 4.06858847e-02, 7.84735673e-02,
            2.79094601e-02, 1.98261069e-02, 3.79592797e-02, 4.16703008e-02, 7.96604257e-03,
            7.69380653e-03, 1.00865185e-02, 1.67639471e-02, 2.29059367e-02, 1.96571917e-02,
            1.77202628e-02, 9.75415029e-03, 5.00626737e-03, 5.51997204e-03, 8.76008870e-03,
            1.45427574e-02, 6.56565513e-03, 1.02838376e-02, 1.15554022e-02, 2.46971785e-03
        ], [
            1.69832372e-02, 2.65280156e-02, 2.80200652e-02, 3.90481733e-02, 3.54464343e-02,
            3.85367897e-02, 4.34481030e-02, 2.96582766e-02, 4.34666491e-02, 5.35559124e-02,
            4.73239715e-02, 6.72540993e-02, 2.12502649e-01, 1.56254710e+00, 1.75300256e-01,
            6.82656415e-02, 5.60346249e-02, 8.82424723e-02, 8.39911250e-02, 2.61407960e-02,
            2.67117519e-02, 2.52687117e-02, 3.60115640e-02, 3.25809399e-02, 3.65703219e-02,
            3.97341252e-02, 8.66516239e-02, 2.18118162e-02, 1.43106612e-02, 4.31523615e-02,
            4.53413563e-02, 2.53290025e-02, 5.45786297e-02, 3.93899704e-02, 4.21165690e-03
        ], [
            1.39504162e-02, 1.51264675e-02, 1.62860285e-02, 1.78430760e-02, 1.80446756e-02,
            1.97753140e-02, 1.59843519e-02, 1.55246100e-02, 1.53020675e-02, 1.97971521e-02,
            2.15585059e-02, 2.50163104e-02, 4.05595691e-02, 2.06750163e-02, 1.46387179e+00,
            2.11927700e-02, 1.34860648e-02, 2.12683457e-02, 1.33650198e-01, 1.27063659e-02,
            1.22906004e-02, 1.12515190e-02, 6.52803856e-02, 7.22065444e-02, 8.63974222e-02,
            3.53109806e-02, 1.23494414e-02, 7.06212086e-03, 5.09226445e-03, 1.28447710e-02,
            3.22877206e-02, 7.77759656e-03, 8.89451317e-03, 1.96482008e-02, 1.87774441e-03
        ], [
            2.00815802e-03, 2.34885481e-03, 3.41847823e-03, 9.88049246e-03, 6.88351029e-03,
            8.19824909e-03, 6.87010056e-03, 2.81980185e-03, 4.65741930e-03, 6.27691772e-03,
            6.22250624e-03, 1.52215361e-02, 9.66289361e-03, 7.12432474e-03, 1.22397492e-02,
            1.05208297e+00, 3.40722488e-03, 9.57787350e-03, 4.80506467e-03, 2.44530132e-03,
            2.83103965e-03, 4.23036462e-03, 5.10348305e-03, 3.11101047e-03, 4.51680471e-03,
            3.89069286e-03, 2.83274944e-03, 2.19386544e-03, 2.50460309e-03, 3.45100973e-03,
            5.64990311e-03, 4.32242973e-03, 4.83985302e-03, 7.55393505e-03, 2.51516952e-03
        ], [
            2.54395688e-02, 4.71677167e-02, 3.80254973e-02, 5.02713131e-02, 3.63549802e-02,
            4.52814154e-02, 4.98493339e-02, 5.79098639e-02, 5.72024139e-02, 5.34963669e-02,
            7.70767851e-02, 6.83810782e-02, 4.21083059e-02, 3.24334213e-02, 3.87083294e-02,
            3.24621065e-02, 1.15277416e+00, 3.41328860e-02, 2.66686301e-02, 1.50758535e-02,
            2.37097562e-02, 4.21343451e-02, 3.24881273e-02, 2.14261892e-02, 2.35939816e-02,
            3.29836820e-02, 2.11473600e-02, 1.30782349e-02, 1.74582983e-02, 1.62933086e-02,
            2.76727878e-02, 3.13836010e-02, 2.65861336e-02, 3.08547340e-02, 4.19934417e-03
        ], [
            7.56980382e-03, 1.15070438e-02, 9.45899531e-03, 1.02654795e-02, 9.22620741e-03,
            1.06540245e-02, 1.03994750e-02, 1.26579514e-02, 1.06862138e-02, 1.02685858e-02,
            1.44736455e-02, 1.25720977e-02, 1.05110125e-02, 8.42384004e-03, 9.54416792e-03,
            8.83224084e-03, 2.46439945e-02, 1.05904294e+00, 1.07743621e-02, 7.79916872e-03,
            9.57844509e-03, 1.21105624e-02, 1.55538529e-02, 9.67950191e-03, 9.98971914e-03,
            2.02889107e-02, 1.63896698e-02, 1.06419019e-02, 3.82922798e-02, 9.90541562e-03,
            2.36302995e-02, 1.62256718e-02, 1.22384539e-02, 1.57727659e-02, 1.76567113e-03
        ], [
            9.56807445e-03, 6.75821379e-03, 1.50911446e-02, 1.40190336e-02, 1.62680961e-02,
            1.46734855e-02, 1.18790271e-02, 8.91873852e-03, 1.32212733e-02, 1.45376362e-02,
            1.40908469e-02, 1.40732761e-02, 1.29480370e-02, 1.06353509e-02, 1.43226029e-02,
            1.31574117e-02, 1.24652380e-02, 1.45144974e-02, 1.01661032e+00, 7.54785247e-03,
            7.17585783e-03, 1.09096609e-02, 2.14261487e-02, 7.92431938e-03, 1.08296517e-02,
            1.37361305e-02, 7.52727437e-03, 6.49369483e-03, 4.30697746e-03, 9.39747440e-03,
            8.45256781e-03, 5.49442303e-03, 8.07510998e-03, 1.08825316e-02, 8.60043056e-04
        ], [
            5.64340856e-02, 3.44647303e-02, 1.09812399e-01, 1.01205744e-01, 1.16212919e-01,
            1.01932109e-01, 8.83264770e-02, 7.72157731e-02, 8.60977866e-02, 9.44515839e-02,
            7.36639761e-02, 8.91010605e-02, 9.30417553e-02, 8.04969841e-02, 1.03648202e-01,
            8.00002682e-02, 4.20671971e-02, 7.93045253e-02, 6.18777934e-02, 1.03750362e+00,
            2.57580178e-02, 7.84492664e-02, 4.61194901e-02, 4.75656102e-02, 4.92119997e-02,
            4.40308087e-02, 2.77249022e-02, 1.66211765e-02, 1.34970542e-02, 2.87592471e-02,
            3.99801409e-02, 2.33762553e-02, 4.82872044e-02, 3.73487239e-02, 4.55874589e-03
        ], [
            3.05972458e-02, 1.57895589e-02, 5.49286493e-02, 5.47496459e-02, 6.88802922e-02,
            4.41734771e-02, 3.76157269e-02, 2.74154049e-02, 4.17672318e-02, 4.45182048e-02,
            3.67729366e-02, 3.82980869e-02, 3.98077121e-02, 3.40158402e-02, 4.65382721e-02,
            4.42268953e-02, 1.79007368e-02, 5.20620233e-02, 2.83381065e-02, 1.29817408e-02,
            1.01393084e+00, 4.10803642e-02, 2.61225911e-02, 2.14400797e-02, 2.32527359e-02,
            2.13128329e-02, 1.76773282e-02, 1.21196751e-02, 8.81056392e-03, 1.70353108e-02,
            1.82132597e-02, 1.24636451e-02, 2.36507861e-02, 2.20276486e-02, 2.44731958e-03
        ], [
            5.80566175e-03, 7.46371110e-03, 1.17709517e-02, 1.35739493e-02, 1.43672035e-02,
            1.25664280e-02, 1.44175071e-02, 8.02610308e-03, 1.24187325e-02, 1.39815226e-02,
            1.31879754e-02, 1.30404287e-02, 1.46624737e-02, 1.28142149e-02, 1.25286115e-02,
            1.08875775e-02, 8.90909129e-03, 1.34151587e-02, 1.17553088e-02, 1.24395839e-02,
            1.15459829e-02, 1.01692507e+00, 1.48969727e-02, 1.38333511e-02, 2.59624540e-02,
            3.04076583e-02, 1.62646213e-02, 1.85031508e-02, 6.99736574e-03, 2.00775388e-02,
            1.74972363e-02, 1.22602536e-02, 1.46697538e-02, 1.92436973e-02, 9.80507247e-04
        ], [
            2.70620989e-02, 3.04060427e-02, 4.92921218e-02, 4.92443289e-02, 5.09919078e-02,
            5.11963505e-02, 4.08544221e-02, 6.52433097e-02, 4.20103227e-02, 4.24835062e-02,
            5.88834341e-02, 4.37954351e-02, 3.62801112e-02, 2.85150406e-02, 3.71521711e-02,
            3.82977724e-02, 3.33065288e-02, 4.25103258e-02, 3.05113468e-02, 2.98897246e-02,
            2.11281795e-02, 2.64763105e-02, 1.06396457e+00, 4.45107647e-02, 3.21920758e-02,
            7.66735642e-02, 1.82073756e-02, 1.26985246e-02, 7.39365607e-03, 1.45800689e-02,
            2.14672909e-02, 1.70789811e-02, 1.86079690e-02, 2.10813256e-02, 3.38310080e-03
        ], [
            6.37972134e-03, 9.92960110e-03, 1.10580808e-02, 1.20833336e-02, 1.25946711e-02,
            1.47124539e-02, 8.94754141e-03, 1.15242117e-02, 8.77995453e-03, 9.58411129e-03,
            1.65127191e-02, 1.39719842e-02, 1.01807663e-02, 7.73855778e-03, 9.42597653e-03,
            8.01648226e-03, 6.99697025e-03, 9.84634948e-03, 5.39682886e-03, 7.28332142e-03,
            5.85552922e-03, 5.69274455e-03, 7.23253327e-03, 1.17757258e+00, 7.09334186e-03,
            9.63913699e-03, 3.20447856e-03, 2.36170535e-03, 3.57750893e-03, 3.52660255e-03,
            3.79611659e-03, 2.65843983e-03, 3.17486000e-03, 4.03068514e-03, 9.04808833e-04
        ], [
            2.35764851e-03, 3.06067103e-03, 4.29339345e-03, 5.18016854e-03, 6.16191404e-03,
            4.93618512e-03, 5.50600333e-03, 3.33914636e-03, 4.75191502e-03, 4.95679024e-03,
            4.48490618e-03, 4.27769432e-03, 5.40177476e-03, 4.95473477e-03, 5.04238229e-03,
            3.91929978e-03, 2.61382140e-03, 4.06893303e-03, 4.35896193e-03, 4.65808962e-03,
            3.29857876e-03, 3.57197673e-03, 5.24159547e-03, 1.10005118e-02, 1.03843044e+00,
            1.88850160e-02, 6.10928653e-03, 5.39060648e-03, 1.80293984e-03, 5.95182997e-03,
            6.19645964e-03, 3.87046180e-03, 3.14735945e-03, 6.57130689e-03, 3.61253467e-04
        ], [
            1.13355310e-02, 1.19284208e-02, 2.14413891e-02, 1.89506249e-02, 2.12435122e-02,
            2.28620636e-02, 2.34870410e-02, 2.02614812e-02, 1.80443524e-02, 1.94144552e-02,
            2.31120888e-02, 1.95540863e-02, 1.79946604e-02, 1.47940682e-02, 1.88801756e-02,
            1.62670926e-02, 1.21450174e-02, 2.25332937e-02, 2.18912033e-02, 2.98446322e-02,
            1.89936944e-02, 1.80769829e-02, 5.40446743e-02, 1.50509462e-01, 1.14557070e-01,
            1.12326771e+00, 1.13126636e-02, 9.65349379e-03, 4.86025661e-03, 1.20568002e-02,
            1.24046489e-02, 7.65102162e-03, 1.03218674e-02, 1.45471668e-02, 1.31827712e-03
        ], [
            1.38601642e-02, 1.54313724e-02, 2.35670876e-02, 2.72508523e-02, 2.83308037e-02,
            2.50555956e-02, 3.45210313e-02, 1.86056510e-02, 2.73835318e-02, 2.60369324e-02,
            2.46176373e-02, 2.61976199e-02, 2.73662724e-02, 2.72058631e-02, 2.57125340e-02,
            2.67809506e-02, 1.93501660e-02, 3.18451965e-02, 3.26701895e-02, 3.30661974e-02,
            3.49926079e-02, 3.13487599e-02, 3.43143264e-02, 3.95056567e-02, 4.26042007e-02,
            4.42134939e-02, 1.15256284e+00, 5.18607685e-02, 1.62845526e-02, 4.27770045e-02,
            3.86698350e-02, 2.02680785e-02, 2.85118143e-02, 4.07487850e-02, 2.72295701e-03
        ], [
            4.42502295e-02, 4.86653385e-02, 5.92428073e-02, 7.29577476e-02, 6.50755004e-02,
            6.01092066e-02, 6.49047026e-02, 5.33307469e-02, 6.01293058e-02, 5.95382410e-02,
            6.37941699e-02, 6.30782551e-02, 6.17753599e-02, 5.32984515e-02, 6.05833332e-02,
            6.73933551e-02, 5.35425120e-02, 6.51371029e-02, 5.30684265e-02, 5.93622713e-02,
            6.54665022e-02, 6.14859247e-02, 8.31633872e-02, 7.09335371e-02, 7.15713059e-02,
            7.44968820e-02, 4.91811038e-02, 1.29754106e+00, 1.21249998e-01, 7.05823205e-02,
            5.40614996e-02, 3.25896342e-02, 5.48323627e-02, 8.37374941e-02, 1.25806713e-02
        ], [
            1.28413406e-02, 1.39333018e-02, 2.15498885e-02, 2.33218477e-02, 2.53766326e-02,
            2.26878875e-02, 2.84970461e-02, 1.53039670e-02, 2.09390043e-02, 2.31321332e-02,
            2.02257092e-02, 2.13214479e-02, 2.25439895e-02, 1.97341148e-02, 2.36021256e-02,
            2.16006899e-02, 1.49701685e-02, 2.72477889e-02, 3.57696970e-02, 3.12336686e-02,
            5.52519913e-02, 4.53342477e-02, 2.29911143e-02, 3.09402931e-02, 2.67339797e-02,
            3.78202404e-02, 2.95089682e-02, 4.40491726e-02, 1.03875310e+00, 3.97132789e-02,
            2.60339493e-02, 2.81371842e-02, 5.00665698e-02, 4.34444563e-02, 1.82349953e-03
        ], [
            5.04032469e-02, 6.67758624e-02, 1.06665178e-01, 1.00842646e-01, 9.92844217e-02,
            9.35033765e-02, 1.41141787e-01, 7.13927188e-02, 1.46082208e-01, 1.20809287e-01,
            9.92298031e-02, 1.00105179e-01, 1.19899213e-01, 1.21285407e-01, 1.30304649e-01,
            8.99574698e-02, 7.86765771e-02, 1.25611399e-01, 1.06621469e-01, 9.64595027e-02,
            1.02412086e-01, 1.01033492e-01, 9.76659277e-02, 8.93230748e-02, 1.44118035e-01,
            1.27458337e-01, 1.17377737e-01, 1.77164961e-01, 7.70962949e-02, 1.20354829e+00,
            1.35616298e-01, 6.61990661e-02, 1.15853042e-01, 1.40107869e-01, 1.09231534e-02
        ], [
            3.38304868e-03, 4.59135053e-03, 5.24333426e-03, 4.39087849e-03, 4.33709008e-03,
            7.24149680e-03, 7.07355169e-03, 4.39184878e-03, 6.23164369e-03, 5.66311280e-03,
            5.96680623e-03, 6.87641074e-03, 4.76426555e-03, 4.06495006e-03, 4.96721491e-03,
            5.03085697e-03, 6.99469731e-03, 4.95669518e-03, 5.04208665e-03, 3.19340104e-03,
            4.38961956e-03, 6.33457612e-03, 6.17518204e-03, 8.49049671e-03, 1.19999803e-02,
            6.23484099e-03, 5.05412942e-03, 3.80407042e-03, 5.39035763e-03, 5.26432060e-03,
            1.00971013e+00, 6.01220961e-03, 6.60600815e-03, 1.16255855e-02, 4.70471583e-04
        ], [
            1.47034277e-03, 1.04710265e-03, 1.74951543e-03, 1.69157554e-03, 1.82423184e-03,
            1.59732115e-03, 1.83086693e-03, 1.30082028e-03, 2.28492301e-03, 2.00843485e-03,
            1.68534287e-03, 1.82336709e-03, 2.14555966e-03, 2.00028358e-03, 2.30347325e-03,
            1.48071508e-03, 1.54053502e-03, 1.69719105e-03, 1.75396122e-03, 1.52704033e-03,
            1.73987236e-03, 1.59910694e-03, 2.05232149e-03, 1.40484477e-03, 3.15320031e-03,
            2.33817701e-03, 2.41977858e-03, 2.39040431e-03, 8.71499741e-04, 3.55384287e-03,
            5.58643501e-03, 1.01449358e+00, 2.36535433e-03, 2.69093714e-03, 1.54976796e-04
        ], [
            1.91127620e-03, 8.00153098e-04, 1.59766062e-03, 1.34816594e-03, 2.14826922e-03,
            1.47573491e-03, 1.24026229e-03, 8.29689234e-04, 1.30581698e-03, 1.24792659e-03,
            1.35142092e-03, 1.24499500e-03, 1.63588060e-03, 1.17002416e-03, 1.38299296e-03,
            1.20482436e-03, 8.83400805e-04, 1.16048553e-03, 1.10740698e-03, 9.41794871e-04,
            8.04029386e-04, 1.69082790e-03, 1.18650982e-03, 9.36250702e-04, 1.06603659e-03,
            1.15249004e-03, 1.17274978e-03, 1.08484712e-03, 3.86558927e-04, 1.13384649e-03,
            5.77544066e-03, 2.00958747e-03, 1.03664513e+00, 1.94416119e-03, 1.16594626e-04
        ], [
            9.66776380e-03, 1.33031724e-02, 1.79380616e-02, 1.84167668e-02, 2.14240112e-02,
            1.77524284e-02, 2.82927184e-02, 1.33159916e-02, 2.11170400e-02, 1.91626825e-02,
            1.80118751e-02, 1.87173888e-02, 1.81709573e-02, 1.58108920e-02, 1.88902753e-02,
            1.57791876e-02, 1.43644341e-02, 1.91399723e-02, 1.75049088e-02, 1.53220128e-02,
            1.71381492e-02, 2.47493980e-02, 2.25425444e-02, 1.68463003e-02, 2.02095537e-02,
            2.57908784e-02, 3.10208686e-02, 2.27557377e-02, 1.48453923e-02, 3.79091216e-02,
            2.79637185e-02, 1.89433502e-02, 2.44079488e-02, 1.09471687e+00, 3.60946758e-03
        ], [
            4.20195217e-05, 5.17537231e-05, 7.25789601e-05, 1.60044070e-04, 8.91216616e-05,
            8.11456494e-05, 8.47801249e-05, 5.66028325e-05, 6.82793313e-05, 7.35491413e-05,
            6.82386433e-05, 6.58147240e-05, 8.36941618e-05, 6.09342495e-05, 1.58950870e-04,
            5.80088732e-05, 3.90206393e-05, 1.19189352e-04, 9.30829306e-05, 4.24544077e-05,
            7.45454608e-05, 9.73732987e-05, 1.54469031e-04, 7.42679567e-05, 6.41988043e-05,
            6.58564541e-05, 7.55172317e-05, 1.12621055e-04, 2.55330081e-04, 1.07381403e-04,
            1.04375434e-04, 6.63176702e-05, 9.36354609e-05, 1.61794229e-04, 1.00012229e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2006():
    """
    Leontief matrix 2006



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.17488806e+00, 4.40977486e-03, 3.67799437e-01, 1.02134026e-01, 1.44956970e-01,
            2.07452363e-01, 3.52882800e-02, 7.90503803e-03, 2.21322191e-02, 3.20947309e-02,
            1.07100321e-02, 8.63220712e-03, 8.90836498e-03, 7.99040041e-03, 9.82198665e-03,
            3.86932708e-02, 5.16174804e-03, 1.95243678e-02, 6.92797053e-03, 8.75656775e-03,
            8.34807987e-03, 1.07327108e-01, 8.41499188e-03, 1.00131459e-02, 9.21375712e-03,
            1.61393571e-02, 5.15066436e-03, 4.58213820e-03, 3.56290356e-03, 6.97438619e-03,
            1.10830511e-02, 1.07611485e-02, 1.32737574e-02, 1.26868714e-02, 3.10875447e-02
        ], [
            2.41138494e-02, 1.10172077e+00, 2.68566691e-02, 3.57238048e-02, 3.08218051e-02,
            3.49109214e-02, 3.09271579e-02, 5.98205733e-01, 9.81126255e-02, 5.08950101e-02,
            1.28616545e-01, 1.16956261e-01, 4.25788159e-02, 2.99984137e-02, 3.63213635e-02,
            3.44587157e-02, 1.63318821e-01, 6.20679673e-02, 1.83400810e-02, 1.16619714e-02,
            1.16519081e-02, 1.95339448e-02, 5.09085652e-02, 6.67476096e-02, 7.21276568e-02,
            3.27179250e-02, 1.16225725e-02, 7.17469972e-03, 9.40300890e-03, 1.20568189e-02,
            2.00309275e-02, 1.31053221e-02, 1.83116966e-02, 1.86322590e-02, 7.06364243e-02
        ], [
            9.20543911e-02, 4.49649618e-03, 1.22276539e+00, 2.67151341e-02, 1.72531316e-01,
            2.59675262e-02, 1.43062008e-02, 7.43919690e-03, 2.52697878e-02, 1.82834822e-02,
            1.07928865e-02, 9.77695845e-03, 1.00290897e-02, 9.16330642e-03, 9.84507491e-03,
            1.35956437e-02, 5.59816908e-03, 9.94285725e-03, 7.36994684e-03, 1.01094563e-02,
            9.93919333e-03, 1.95740947e-01, 8.08218275e-03, 1.41244664e-02, 1.34443034e-02,
            1.17210512e-02, 6.97891182e-03, 6.11198635e-03, 2.99262386e-03, 8.56287839e-03,
            1.53401577e-02, 1.54904708e-02, 2.24851407e-02, 1.64976244e-02, 6.67917881e-03
        ], [
            5.78551814e-03, 3.65587530e-03, 7.93622071e-03, 1.54223794e+00, 1.20429016e-01,
            1.17885485e-02, 1.63235664e-02, 4.53839928e-03, 1.12179322e-02, 3.07597456e-02,
            1.13913011e-02, 8.76028211e-03, 1.00601912e-02, 9.16330961e-03, 1.68752708e-02,
            4.30004220e-02, 3.96858213e-03, 9.59508557e-03, 6.71414922e-03, 5.74714861e-03,
            7.05402388e-03, 8.01680594e-03, 6.54165856e-03, 9.07414603e-03, 6.89951059e-03,
            6.54145640e-03, 3.51024803e-03, 2.62257146e-03, 2.00387330e-03, 4.83281306e-03,
            7.38541057e-03, 3.51696069e-03, 8.59429884e-03, 1.11087588e-02, 7.31706164e-04
        ], [
            5.26981074e-04, 3.13075269e-04, 7.30676472e-04, 2.25557992e-02, 1.27525499e+00,
            1.39822736e-03, 1.83728133e-03, 4.25435061e-04, 8.95236595e-04, 2.29180414e-03,
            9.20910730e-04, 8.49550703e-04, 9.87741749e-04, 1.08872322e-03, 4.03743656e-03,
            7.72562268e-03, 4.10248433e-04, 7.62783817e-04, 1.14188853e-03, 7.00799789e-04,
            6.19597602e-04, 5.83929871e-04, 7.67507425e-04, 7.02564222e-04, 7.25565598e-04,
            6.50702452e-04, 5.95983471e-04, 2.46886111e-04, 1.58473872e-04, 4.22208569e-04,
            1.14613085e-03, 3.57860987e-04, 5.42819915e-04, 1.04267261e-03, 6.38120226e-05
        ], [
            4.75724550e-03, 4.30029055e-03, 6.90621167e-03, 7.25405978e-03, 8.15815704e-03,
            1.34282804e+00, 2.36629084e-02, 4.49221298e-03, 6.24072191e-03, 9.54129136e-03,
            1.62339389e-02, 1.19933917e-02, 1.04274874e-02, 7.58795885e-03, 1.08799865e-02,
            1.05371734e-01, 4.45538623e-03, 4.92896330e-02, 5.15893258e-03, 3.82295537e-03,
            3.77097763e-03, 6.48807795e-03, 4.97565588e-03, 4.25173105e-03, 4.02893188e-03,
            7.19760246e-03, 3.41450345e-03, 2.54362745e-03, 6.32721417e-03, 3.86519415e-03,
            6.11193416e-03, 6.20129216e-03, 4.80883220e-03, 7.39837867e-03, 7.48893736e-04
        ], [
            1.44225860e-02, 1.13089318e-02, 5.30216271e-02, 4.01379183e-02, 4.78950694e-02,
            3.94811331e-02, 1.34099260e+00, 1.51049510e-02, 4.04035376e-02, 4.56357698e-02,
            4.63819732e-02, 2.36750433e-02, 2.85810693e-02, 3.17657583e-02, 2.64615599e-02,
            4.29856825e-02, 1.40343277e-02, 2.47392568e-02, 2.29285899e-02, 2.34590446e-02,
            2.36381445e-02, 2.80484083e-02, 2.05378503e-02, 1.83468529e-02, 2.15191958e-02,
            3.20019474e-02, 2.37437337e-02, 2.96540307e-02, 9.88828223e-03, 3.87558912e-02,
            3.57303343e-02, 3.13605447e-02, 2.44379512e-02, 4.53500413e-02, 2.08666425e-03
        ], [
            2.55369389e-02, 1.55671858e-02, 2.19314298e-02, 2.92496866e-02, 2.56758091e-02,
            2.55673235e-02, 2.11324358e-02, 1.14192703e+00, 8.11254249e-02, 4.12372047e-02,
            4.21698217e-02, 3.47983288e-02, 2.13419934e-02, 1.70163538e-02, 1.93982416e-02,
            2.01312689e-02, 4.03395474e-02, 2.98338364e-02, 1.48570885e-02, 1.28474645e-02,
            1.00082863e-02, 1.44947542e-02, 7.14606842e-02, 1.06559654e-01, 1.18189832e-01,
            3.86921307e-02, 1.01152761e-02, 6.38981683e-03, 5.31221128e-03, 1.16060101e-02,
            1.73431117e-02, 9.08392180e-03, 1.41055838e-02, 1.45170045e-02, 2.03779031e-03
        ], [
            6.68404885e-02, 2.32708862e-02, 5.27567157e-02, 1.62313127e-01, 1.13516719e-01,
            8.81091560e-02, 8.25563925e-02, 4.45730233e-02, 1.34937661e+00, 3.55521927e-01,
            7.86359328e-02, 4.98943152e-02, 5.13896813e-02, 6.43493804e-02, 6.14756410e-02,
            7.37300633e-02, 2.27285590e-02, 4.75959018e-02, 2.80097215e-02, 1.27754221e-02,
            1.22451245e-02, 2.55277577e-02, 2.18351896e-02, 2.39727857e-02, 2.32002155e-02,
            2.22842385e-02, 1.27600154e-02, 8.21804330e-03, 8.46931403e-03, 1.72934051e-02,
            2.66284623e-02, 1.35328061e-02, 9.80262170e-02, 2.79311576e-02, 4.10927155e-03
        ], [
            1.40837888e-02, 1.24539707e-02, 3.67804538e-02, 3.28898638e-02, 7.40734439e-02,
            2.52271943e-02, 3.49762750e-02, 1.56083474e-02, 3.66916605e-02, 1.19013638e+00,
            2.74940002e-02, 2.38109205e-02, 5.01683288e-02, 5.30530096e-02, 7.46844044e-02,
            5.38525453e-02, 1.11624420e-02, 3.70255558e-02, 3.19268311e-02, 1.01298438e-02,
            1.07101606e-02, 1.57093687e-02, 2.26400857e-02, 1.50550770e-02, 1.59930840e-02,
            1.75124064e-02, 1.08822909e-02, 5.02883977e-03, 5.15085143e-03, 9.61830006e-03,
            1.20496177e-02, 6.44065998e-03, 1.47176294e-02, 1.34289303e-02, 1.69032799e-03
        ], [
            5.37939297e-03, 8.73374110e-03, 1.18381812e-02, 8.94882480e-03, 8.64020596e-03,
            1.65727221e-02, 6.51676931e-03, 9.34908212e-03, 1.32269657e-02, 1.44393035e-02,
            1.16479786e+00, 2.28659627e-02, 1.67625073e-02, 2.33940017e-02, 1.98041889e-02,
            1.52702603e-02, 7.99988082e-03, 1.16121345e-01, 9.46892843e-03, 3.58646679e-03,
            3.74845916e-03, 8.39318468e-03, 6.42552158e-03, 5.20545773e-03, 5.54219475e-03,
            6.97685157e-03, 5.48357424e-03, 2.55520605e-03, 7.01632994e-03, 4.25228654e-03,
            6.74040418e-03, 5.12420728e-03, 6.48074092e-03, 6.37725817e-03, 9.61046022e-04
        ], [
            2.30409757e-02, 4.68009843e-02, 4.15286644e-02, 3.93846902e-02, 4.85190230e-02,
            7.05555442e-02, 4.07334386e-02, 4.61253692e-02, 4.98426506e-02, 7.41550176e-02,
            8.98808898e-02, 1.52849723e+00, 3.23774653e-01, 1.80905328e-01, 2.45309056e-01,
            1.51464755e-01, 3.68770253e-02, 1.78438360e-01, 6.55255088e-02, 1.68857722e-02,
            1.63142916e-02, 2.44973306e-02, 3.67736983e-02, 3.90306101e-02, 3.83106523e-02,
            3.77090722e-02, 2.47054256e-02, 1.05852124e-02, 1.54324476e-02, 2.05877174e-02,
            2.85573616e-02, 1.50753940e-02, 2.21129664e-02, 2.59541955e-02, 4.93895004e-03
        ], [
            1.62301994e-02, 2.56372842e-02, 1.76254753e-02, 2.92358809e-02, 2.53153207e-02,
            2.84175357e-02, 2.14914161e-02, 2.76816414e-02, 2.38998055e-02, 3.31002317e-02,
            4.32566780e-02, 5.42920779e-02, 1.21305715e+00, 4.60034397e-02, 8.60436404e-02,
            3.23417012e-02, 2.15861164e-02, 4.19449052e-02, 4.24183223e-02, 8.41391051e-03,
            8.04125005e-03, 1.09941779e-02, 1.83416645e-02, 2.67127310e-02, 2.31117131e-02,
            2.01906420e-02, 1.02623599e-02, 5.24976107e-03, 5.90004840e-03, 9.40622352e-03,
            1.55791344e-02, 7.30579124e-03, 1.19176757e-02, 1.25084069e-02, 2.80471578e-03
        ], [
            1.90921841e-02, 2.96267965e-02, 3.09445336e-02, 4.44395689e-02, 4.00637577e-02,
            4.40769817e-02, 4.82580744e-02, 3.57792807e-02, 4.84476978e-02, 6.03171263e-02,
            5.44994645e-02, 7.58579446e-02, 2.37530123e-01, 1.62521369e+00, 1.91184957e-01,
            8.18064921e-02, 6.50445046e-02, 1.01077131e-01, 8.81931819e-02, 2.84759990e-02,
            2.86920547e-02, 2.85111083e-02, 4.06440514e-02, 3.76789253e-02, 4.20499909e-02,
            4.49617498e-02, 9.27023783e-02, 2.32577734e-02, 1.59147991e-02, 4.84829293e-02,
            5.08046118e-02, 3.04768274e-02, 6.02672845e-02, 4.44909842e-02, 5.15304352e-03
        ], [
            1.49609843e-02, 1.48598391e-02, 1.69720123e-02, 1.88710214e-02, 1.90290478e-02,
            2.13512271e-02, 1.74772821e-02, 1.73340472e-02, 1.61958417e-02, 2.11620004e-02,
            2.37869774e-02, 2.77137875e-02, 4.26090319e-02, 2.15958738e-02, 1.47201379e+00,
            2.36402366e-02, 1.45145679e-02, 2.29012382e-02, 1.27857649e-01, 1.32402831e-02,
            1.24560159e-02, 1.18947428e-02, 6.84798909e-02, 7.92319927e-02, 9.16712043e-02,
            3.95389548e-02, 1.20523153e-02, 7.08517014e-03, 5.32142943e-03, 1.31455740e-02,
            3.25278729e-02, 8.52111205e-03, 9.28535302e-03, 2.03665883e-02, 2.05581930e-03
        ], [
            2.15892526e-03, 2.50920091e-03, 3.64203283e-03, 9.63041549e-03, 6.89708022e-03,
            8.23135181e-03, 8.13772277e-03, 3.18036579e-03, 4.76323589e-03, 6.83224748e-03,
            6.91326100e-03, 1.73988831e-02, 1.01583055e-02, 7.35592214e-03, 1.22722663e-02,
            1.05263326e+00, 3.45134606e-03, 9.76049662e-03, 4.75631270e-03, 2.47721857e-03,
            2.78315470e-03, 4.31555266e-03, 4.91590836e-03, 3.29905392e-03, 4.51321408e-03,
            4.10674781e-03, 2.75674707e-03, 2.17199051e-03, 2.58121279e-03, 3.45552258e-03,
            5.80418926e-03, 4.85887731e-03, 4.96948386e-03, 7.66445943e-03, 2.19167852e-03
        ], [
            2.54348163e-02, 4.43567127e-02, 3.73373341e-02, 4.96949891e-02, 3.63718321e-02,
            4.49901764e-02, 5.02039427e-02, 6.87467525e-02, 5.68514850e-02, 5.33938909e-02,
            7.69713385e-02, 6.69800478e-02, 4.08260399e-02, 3.17210099e-02, 3.75413578e-02,
            3.31253183e-02, 1.16549989e+00, 3.42948482e-02, 2.63027233e-02, 1.49744234e-02,
            2.26663196e-02, 4.11821766e-02, 3.29942529e-02, 2.27354453e-02, 2.55749144e-02,
            3.48215024e-02, 2.03943940e-02, 1.22851883e-02, 1.66526786e-02, 1.59713479e-02,
            2.71198991e-02, 3.10312481e-02, 2.63945748e-02, 3.06110056e-02, 4.20994762e-03
        ], [
            7.53463647e-03, 1.11337875e-02, 9.35380727e-03, 1.02108929e-02, 9.14227315e-03,
            1.07769879e-02, 1.05826260e-02, 1.53919208e-02, 1.08565712e-02, 1.03963540e-02,
            1.45245129e-02, 1.24474732e-02, 1.02829560e-02, 8.21160551e-03, 9.45232600e-03,
            9.09561523e-03, 2.42359114e-02, 1.06059479e+00, 1.10268397e-02, 7.90127250e-03,
            9.41505194e-03, 1.20924501e-02, 1.54768416e-02, 1.05066325e-02, 1.04839418e-02,
            2.06966435e-02, 1.59381527e-02, 1.01197379e-02, 3.77967481e-02, 9.78047827e-03,
            2.36156774e-02, 1.59904524e-02, 1.26686146e-02, 1.55562976e-02, 1.79932187e-03
        ], [
            9.60895486e-03, 6.38372287e-03, 1.48987261e-02, 1.36260039e-02, 1.58290210e-02,
            1.45136660e-02, 1.20448375e-02, 9.42926581e-03, 1.29773691e-02, 1.44383151e-02,
            1.39308729e-02, 1.38319868e-02, 1.26049700e-02, 1.02725093e-02, 1.40425583e-02,
            1.31903608e-02, 1.25032724e-02, 1.45733412e-02, 1.01656167e+00, 7.53796879e-03,
            7.10861498e-03, 1.08939663e-02, 2.09055636e-02, 8.02929770e-03, 1.10239151e-02,
            1.41476144e-02, 7.33978892e-03, 6.32876361e-03, 4.32329659e-03, 9.27102268e-03,
            8.54517359e-03, 5.75304097e-03, 8.17739447e-03, 1.09452177e-02, 9.13760212e-04
        ], [
            5.77453637e-02, 3.32136457e-02, 1.09044458e-01, 9.80057570e-02, 1.12057687e-01,
            1.02755535e-01, 8.92147226e-02, 9.41685567e-02, 8.85433385e-02, 9.60859922e-02,
            7.58040147e-02, 9.03188069e-02, 9.26234422e-02, 7.97758480e-02, 1.02636612e-01,
            8.26634729e-02, 4.48371123e-02, 8.07100512e-02, 6.02664796e-02, 1.03834196e+00,
            2.55106666e-02, 7.78263901e-02, 4.80346573e-02, 5.02747675e-02, 5.23708149e-02,
            4.62875331e-02, 2.74593909e-02, 1.61688609e-02, 1.38455599e-02, 2.90102946e-02,
            3.99508329e-02, 2.42903757e-02, 4.82742826e-02, 3.76136352e-02, 4.97056068e-03
        ], [
            3.02402838e-02, 1.47495690e-02, 5.39157374e-02, 5.27889244e-02, 6.52052505e-02,
            4.37187178e-02, 3.78005736e-02, 2.87952301e-02, 4.15523314e-02, 4.43615834e-02,
            3.66114432e-02, 3.77226863e-02, 3.92772789e-02, 3.34400106e-02, 4.57855958e-02,
            4.52051295e-02, 1.79855656e-02, 5.18753532e-02, 2.70877761e-02, 1.28304656e-02,
            1.01359506e+00, 4.06707812e-02, 2.59332867e-02, 2.17461771e-02, 2.34355911e-02,
            2.16160425e-02, 1.70831998e-02, 1.16337963e-02, 8.81051538e-03, 1.66392645e-02,
            1.80996470e-02, 1.27010572e-02, 2.39024404e-02, 2.20596989e-02, 2.71625803e-03
        ], [
            5.89711495e-03, 7.41043945e-03, 1.18915409e-02, 1.35390902e-02, 1.44778272e-02,
            1.27540057e-02, 1.46542775e-02, 8.80268243e-03, 1.25261782e-02, 1.41541528e-02,
            1.35320787e-02, 1.33101464e-02, 1.47061174e-02, 1.27218555e-02, 1.24849174e-02,
            1.12441750e-02, 8.76693424e-03, 1.37908964e-02, 1.14848385e-02, 1.24554044e-02,
            1.13372399e-02, 1.01699450e+00, 1.47311360e-02, 1.37642023e-02, 2.55921950e-02,
            3.02444914e-02, 1.56336365e-02, 1.78938028e-02, 6.77606811e-03, 1.99369529e-02,
            1.73187734e-02, 1.28658058e-02, 1.48718623e-02, 1.95274261e-02, 1.02605118e-03
        ], [
            2.76959980e-02, 2.93260236e-02, 4.92702203e-02, 5.05804253e-02, 5.08429837e-02,
            5.22048704e-02, 4.15768583e-02, 7.50120542e-02, 4.38529894e-02, 4.39366307e-02,
            6.06965393e-02, 4.43975538e-02, 3.67126553e-02, 2.87619418e-02, 3.74234552e-02,
            4.12061218e-02, 3.60347433e-02, 4.41049061e-02, 3.12485077e-02, 3.04266377e-02,
            2.10977189e-02, 2.67691132e-02, 1.06461183e+00, 3.95881334e-02, 3.33189802e-02,
            7.80687457e-02, 1.80427778e-02, 1.24628623e-02, 7.55004102e-03, 1.45835639e-02,
            2.13449290e-02, 1.72215920e-02, 1.89354746e-02, 2.13267087e-02, 3.75685438e-03
        ], [
            6.11510414e-03, 9.58147418e-03, 1.13142573e-02, 1.20780496e-02, 1.22041752e-02,
            1.47603101e-02, 8.83767939e-03, 1.10211384e-02, 8.82885750e-03, 9.61030846e-03,
            1.58957081e-02, 1.37990833e-02, 9.86404672e-03, 7.51252279e-03, 9.30008289e-03,
            8.48815192e-03, 6.78652088e-03, 9.59952146e-03, 5.45315026e-03, 7.45202088e-03,
            5.90459788e-03, 5.71517148e-03, 7.44690315e-03, 1.18470770e+00, 7.17652547e-03,
            9.56896839e-03, 3.14866484e-03, 2.26127014e-03, 3.80524321e-03, 3.51575835e-03,
            3.68495244e-03, 2.63192502e-03, 3.19377685e-03, 4.02541139e-03, 9.36124964e-04
        ], [
            2.44402078e-03, 2.90374929e-03, 4.37369977e-03, 5.20972661e-03, 6.19716894e-03,
            5.10557018e-03, 5.67610217e-03, 3.51934283e-03, 4.75047349e-03, 5.02587232e-03,
            4.61611138e-03, 4.27313094e-03, 5.46081078e-03, 4.97908799e-03, 5.06539722e-03,
            4.16072322e-03, 2.66259041e-03, 4.18432844e-03, 4.41826188e-03, 4.76930307e-03,
            3.32461521e-03, 3.65029279e-03, 5.37160899e-03, 1.08656553e-02, 1.03598236e+00,
            1.82115781e-02, 6.00935177e-03, 5.25948544e-03, 1.77107562e-03, 6.01496010e-03,
            6.19862344e-03, 4.19368336e-03, 3.20327072e-03, 6.76049599e-03, 3.79974271e-04
        ], [
            1.16166284e-02, 1.16164707e-02, 2.17376393e-02, 1.90858085e-02, 2.12471911e-02,
            2.34829389e-02, 2.46086477e-02, 2.22620486e-02, 1.84167962e-02, 1.98270227e-02,
            2.34695415e-02, 1.97652708e-02, 1.80667745e-02, 1.48292385e-02, 1.89845691e-02,
            1.68484259e-02, 1.26988692e-02, 2.37469229e-02, 2.31188636e-02, 3.03772255e-02,
            1.90302299e-02, 1.81273458e-02, 5.40491796e-02, 1.55807816e-01, 1.14632024e-01,
            1.12427897e+00, 1.10405853e-02, 9.29049249e-03, 4.97720003e-03, 1.21245378e-02,
            1.22861878e-02, 7.86405150e-03, 1.04610362e-02, 1.48350279e-02, 1.41766646e-03
        ], [
            1.43205334e-02, 1.47358003e-02, 2.36057536e-02, 2.75159649e-02, 2.79362616e-02,
            2.55584692e-02, 3.54628028e-02, 2.04920145e-02, 2.73992725e-02, 2.61441245e-02,
            2.49219298e-02, 2.67931807e-02, 2.70676549e-02, 2.71330782e-02, 2.55030584e-02,
            2.89269905e-02, 1.99970131e-02, 3.19576551e-02, 3.26304347e-02, 3.32378601e-02,
            3.46295209e-02, 3.15047842e-02, 3.48749851e-02, 4.11322714e-02, 4.25790897e-02,
            4.41815241e-02, 1.14804094e+00, 5.04344125e-02, 1.58579900e-02, 4.24932578e-02,
            3.87021590e-02, 2.08809463e-02, 2.86428795e-02, 4.07874855e-02, 2.93508516e-03
        ], [
            4.52599117e-02, 5.00480658e-02, 6.11927885e-02, 7.73256884e-02, 6.84794470e-02,
            6.34148367e-02, 6.93780434e-02, 6.10959515e-02, 6.27215562e-02, 6.24436630e-02,
            6.81891694e-02, 6.59997361e-02, 6.36200195e-02, 5.50840484e-02, 6.29265521e-02,
            7.11872574e-02, 5.70380429e-02, 6.81359571e-02, 5.37804335e-02, 6.05933347e-02,
            6.63133285e-02, 6.33567082e-02, 8.75010058e-02, 7.06155309e-02, 7.57806524e-02,
            7.71909364e-02, 4.93836723e-02, 1.30077538e+00, 1.21129940e-01, 7.32034915e-02,
            5.48962907e-02, 3.49900133e-02, 5.76202786e-02, 8.81045329e-02, 1.38921157e-02
        ], [
            1.19226554e-02, 1.33443456e-02, 2.08538843e-02, 2.29268491e-02, 2.53197719e-02,
            2.24516240e-02, 2.85237154e-02, 1.66292903e-02, 2.04360538e-02, 2.28800792e-02,
            2.01536578e-02, 2.07664563e-02, 2.19561832e-02, 1.90392579e-02, 2.31270870e-02,
            2.18698221e-02, 1.50157435e-02, 2.72994595e-02, 3.49657986e-02, 3.18700462e-02,
            5.25929180e-02, 4.43208830e-02, 2.26451571e-02, 3.08960705e-02, 2.62404942e-02,
            3.75603431e-02, 2.71529127e-02, 4.08298457e-02, 1.03995455e+00, 3.84403534e-02,
            2.59258832e-02, 2.78151222e-02, 4.94501728e-02, 4.26661703e-02, 1.85256645e-03
        ], [
            5.13951945e-02, 6.49059830e-02, 1.06651734e-01, 1.00478741e-01, 9.93424930e-02,
            9.57235634e-02, 1.45665361e-01, 7.93629745e-02, 1.42803110e-01, 1.21535908e-01,
            1.00506262e-01, 9.99457696e-02, 1.19048956e-01, 1.19454974e-01, 1.28503054e-01,
            9.24282895e-02, 7.93338100e-02, 1.26819483e-01, 1.05842381e-01, 9.91359684e-02,
            1.02994922e-01, 1.02669107e-01, 9.90928575e-02, 8.89972900e-02, 1.47495653e-01,
            1.28248625e-01, 1.14904625e-01, 1.73342750e-01, 7.60513300e-02, 1.20890180e+00,
            1.39025862e-01, 6.83615012e-02, 1.18400171e-01, 1.43113588e-01, 1.13876701e-02
        ], [
            3.23900285e-03, 4.18895651e-03, 5.07074343e-03, 4.23033341e-03, 4.28280181e-03,
            7.08918849e-03, 7.14259109e-03, 4.78210752e-03, 6.07704237e-03, 5.66997506e-03,
            6.05726072e-03, 6.71528468e-03, 4.72489551e-03, 3.95545474e-03, 4.85231490e-03,
            5.19162691e-03, 7.07240420e-03, 4.95296235e-03, 4.91613254e-03, 3.24736915e-03,
            4.41882777e-03, 6.24424603e-03, 6.09544862e-03, 8.78458820e-03, 1.23359676e-02,
            6.21583830e-03, 4.72280465e-03, 3.59863548e-03, 5.23171703e-03, 5.17646439e-03,
            1.00949194e+00, 6.05140205e-03, 6.53631312e-03, 1.15071600e-02, 4.70837667e-04
        ], [
            1.20553193e-03, 9.95919317e-04, 1.61999534e-03, 1.60148617e-03, 1.71940193e-03,
            1.53590363e-03, 1.81404600e-03, 1.43058070e-03, 2.17457437e-03, 1.94671956e-03,
            1.65526238e-03, 1.74866847e-03, 2.09740618e-03, 1.92229836e-03, 2.29478388e-03,
            1.46906950e-03, 1.47945791e-03, 1.67730503e-03, 1.72833843e-03, 1.41879044e-03,
            1.95778586e-03, 1.53593110e-03, 1.97654825e-03, 1.39615978e-03, 3.22798786e-03,
            2.28308441e-03, 2.35956898e-03, 2.28328289e-03, 8.41001991e-04, 3.26316056e-03,
            5.18717465e-03, 1.01482050e+00, 2.33739129e-03, 2.66451172e-03, 1.49196073e-04
        ], [
            1.89452117e-03, 8.26491140e-04, 1.57690610e-03, 1.36532120e-03, 2.17760654e-03,
            1.55331221e-03, 1.29284621e-03, 9.26398286e-04, 1.32420578e-03, 1.26816940e-03,
            1.49942524e-03, 1.35972324e-03, 1.79315732e-03, 1.23492800e-03, 1.45768602e-03,
            1.23722926e-03, 9.19208082e-04, 1.19048388e-03, 1.07925256e-03, 9.40715852e-04,
            7.71284106e-04, 1.67099151e-03, 1.16218824e-03, 9.73875660e-04, 1.09194313e-03,
            1.13274951e-03, 1.09965805e-03, 1.06908139e-03, 3.85415536e-04, 1.08776617e-03,
            6.00292314e-03, 2.03266429e-03, 1.03591156e+00, 1.98451302e-03, 1.28198121e-04
        ], [
            9.75154428e-03, 1.28864118e-02, 1.76587672e-02, 1.82741006e-02, 2.10460758e-02,
            1.78497340e-02, 2.87628852e-02, 1.44004545e-02, 2.08074100e-02, 1.90588197e-02,
            1.80703656e-02, 1.85014857e-02, 1.77378932e-02, 1.52546057e-02, 1.85177430e-02,
            1.59019027e-02, 1.44265063e-02, 1.90415922e-02, 1.69457532e-02, 1.52501188e-02,
            1.69085754e-02, 2.47642878e-02, 2.31790139e-02, 1.65980032e-02, 2.06067809e-02,
            2.59752312e-02, 2.83570049e-02, 2.19412853e-02, 1.47007959e-02, 3.70607410e-02,
            2.79166381e-02, 1.91537340e-02, 2.43231208e-02, 1.09545784e+00, 3.54746127e-03
        ], [
            4.09813241e-05, 5.08327020e-05, 7.10203250e-05, 1.79548024e-04, 9.32441450e-05,
            8.38762906e-05, 8.61808660e-05, 6.06552435e-05, 6.65736994e-05, 7.34613056e-05,
            6.91100235e-05, 6.39661425e-05, 8.23825513e-05, 5.87214984e-05, 1.16549081e-04,
            6.04265675e-05, 3.98016510e-05, 1.15575184e-04, 8.00872989e-05, 4.27037700e-05,
            7.25828728e-05, 9.57662633e-05, 1.55263387e-04, 4.77758566e-05, 5.89560056e-05,
            6.18651463e-05, 6.91487208e-05, 1.07268549e-04, 2.52275887e-04, 1.03659153e-04,
            1.03244033e-04, 6.58487158e-05, 9.52198862e-05, 1.66200355e-04, 1.00012835e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2007():
    """
    Leontief matrix 2007



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.16627557e+00, 4.57397835e-03, 3.62216629e-01, 1.03410891e-01, 1.45429358e-01,
            1.98016500e-01, 3.65721220e-02, 8.41306471e-03, 2.26597856e-02, 3.10510025e-02,
            1.10934062e-02, 8.77802331e-03, 8.92311671e-03, 8.03232300e-03, 9.76085720e-03,
            3.88149366e-02, 5.24380161e-03, 1.75692568e-02, 6.92400843e-03, 8.42703075e-03,
            8.12406837e-03, 1.05489493e-01, 8.40933335e-03, 9.70093577e-03, 9.52692147e-03,
            1.66861673e-02, 4.97876695e-03, 4.62211585e-03, 3.26160900e-03, 6.91180531e-03,
            1.09909619e-02, 1.06971784e-02, 1.31596217e-02, 1.24976567e-02, 3.49457700e-02
        ], [
            2.28566761e-02, 1.11067029e+00, 2.59519508e-02, 3.53945706e-02, 3.02532510e-02,
            3.40244627e-02, 3.05754203e-02, 5.60844689e-01, 9.30796851e-02, 4.95335820e-02,
            1.27741309e-01, 1.14630218e-01, 4.17493508e-02, 2.96443735e-02, 3.51836775e-02,
            3.44511732e-02, 1.63740328e-01, 6.25670784e-02, 1.81163246e-02, 1.08312172e-02,
            1.10974624e-02, 1.89700123e-02, 4.92755450e-02, 5.44720163e-02, 6.90216387e-02,
            3.24859407e-02, 1.14168720e-02, 7.19474162e-03, 8.68991919e-03, 1.17332361e-02,
            1.99592227e-02, 1.27545165e-02, 1.80235618e-02, 1.81937937e-02, 7.16010182e-02
        ], [
            9.72134324e-02, 4.93053655e-03, 1.23105299e+00, 2.93623123e-02, 1.80165908e-01,
            2.71226146e-02, 1.53955091e-02, 8.31333438e-03, 2.72167203e-02, 1.94220510e-02,
            1.17410797e-02, 1.04658322e-02, 1.05982723e-02, 9.69495465e-03, 1.04335767e-02,
            1.46397913e-02, 6.05071831e-03, 1.04596235e-02, 7.60214869e-03, 1.03080821e-02,
            9.87986343e-03, 1.99287857e-01, 8.43441443e-03, 1.42859038e-02, 1.44551297e-02,
            1.24390238e-02, 6.97759789e-03, 6.30402764e-03, 2.92833289e-03, 8.69012456e-03,
            1.56463842e-02, 1.58118145e-02, 2.28094160e-02, 1.66180809e-02, 6.78420701e-03
        ], [
            5.83673238e-03, 4.12261822e-03, 8.36702210e-03, 1.57429565e+00, 1.27787834e-01,
            1.17566656e-02, 1.79223215e-02, 5.02816152e-03, 1.14625812e-02, 3.30055991e-02,
            1.22936369e-02, 9.29469843e-03, 1.05751257e-02, 9.57051476e-03, 1.67064431e-02,
            4.55475915e-02, 4.54959914e-03, 9.90184607e-03, 6.87761927e-03, 6.12939584e-03,
            6.74876572e-03, 8.16189508e-03, 6.95960946e-03, 9.28293960e-03, 7.51309669e-03,
            7.10804302e-03, 3.59156880e-03, 2.83639846e-03, 2.03671544e-03, 5.13145130e-03,
            7.79576723e-03, 3.83371663e-03, 9.13206112e-03, 1.08675560e-02, 7.22102362e-04
        ], [
            5.50928284e-04, 3.52815685e-04, 7.66391156e-04, 2.53445961e-02, 1.28536725e+00,
            1.46348676e-03, 1.82109334e-03, 4.70840339e-04, 9.57212361e-04, 2.41188591e-03,
            1.00385935e-03, 9.07109547e-04, 1.05210379e-03, 1.16060959e-03, 4.40491501e-03,
            8.47713115e-03, 4.58586792e-04, 8.12742705e-04, 1.17630473e-03, 7.08684566e-04,
            6.37963988e-04, 6.03855044e-04, 8.67247490e-04, 7.27203777e-04, 8.19848547e-04,
            7.26575525e-04, 6.16376100e-04, 2.72424198e-04, 1.65345203e-04, 4.53869826e-04,
            1.40090791e-03, 4.00022652e-04, 7.35737631e-04, 1.09992922e-03, 6.59120489e-05
        ], [
            4.79194163e-03, 4.72867908e-03, 7.23112099e-03, 7.91704629e-03, 8.72091407e-03,
            1.36095453e+00, 2.60476963e-02, 4.75880371e-03, 6.61505696e-03, 9.91408044e-03,
            1.78868361e-02, 1.29564069e-02, 1.11295453e-02, 8.08017906e-03, 1.13700207e-02,
            1.12140044e-01, 4.65504792e-03, 4.91448353e-02, 5.48482816e-03, 3.81376198e-03,
            3.81326754e-03, 6.68314828e-03, 5.20564657e-03, 4.21666514e-03, 4.24188215e-03,
            7.21602924e-03, 3.47503031e-03, 2.67636331e-03, 5.77211543e-03, 4.01100576e-03,
            6.57430244e-03, 6.69384625e-03, 5.13576524e-03, 7.65787698e-03, 7.50807001e-04
        ], [
            1.48494108e-02, 1.22054847e-02, 5.41982158e-02, 4.21789456e-02, 4.94538795e-02,
            4.12550614e-02, 1.34964734e+00, 1.60895135e-02, 4.09628760e-02, 4.59018376e-02,
            4.86005030e-02, 2.46266432e-02, 2.88426209e-02, 3.20831435e-02, 2.70031085e-02,
            4.50505536e-02, 1.48320043e-02, 2.56714095e-02, 2.35435639e-02, 2.35089269e-02,
            2.29637812e-02, 2.82070065e-02, 2.13160263e-02, 1.82060742e-02, 2.11914038e-02,
            3.28229890e-02, 2.36537343e-02, 3.07357202e-02, 9.53299313e-03, 3.93476234e-02,
            3.59536787e-02, 3.25778485e-02, 2.53426018e-02, 4.60774997e-02, 2.15319749e-03
        ], [
            2.45720007e-02, 1.59174679e-02, 2.12212193e-02, 2.84525773e-02, 2.50626134e-02,
            2.45610321e-02, 2.07407975e-02, 1.12639741e+00, 7.52744832e-02, 3.95743581e-02,
            4.14188756e-02, 3.41940932e-02, 2.09137686e-02, 1.66519077e-02, 1.87711992e-02,
            2.00079886e-02, 4.32533367e-02, 2.86821556e-02, 1.51251970e-02, 1.18689649e-02,
            9.65647934e-03, 1.40500407e-02, 7.16982982e-02, 8.82049893e-02, 1.16534781e-01,
            3.96193630e-02, 1.00832202e-02, 6.50077670e-03, 5.01119888e-03, 1.14521478e-02,
            1.72130522e-02, 8.98587413e-03, 1.36821583e-02, 1.40989616e-02, 2.06668784e-03
        ], [
            7.01389375e-02, 2.65908498e-02, 5.56862036e-02, 1.68948859e-01, 1.20016465e-01,
            9.43208546e-02, 8.87359886e-02, 4.89967387e-02, 1.36184895e+00, 3.71519131e-01,
            8.53163520e-02, 5.33620184e-02, 5.45300933e-02, 6.90321141e-02, 6.44313613e-02,
            7.91767072e-02, 2.54882715e-02, 5.08728322e-02, 2.97330715e-02, 1.35169928e-02,
            1.27061334e-02, 2.70621799e-02, 2.34517644e-02, 2.38809658e-02, 2.45644621e-02,
            2.42386782e-02, 1.36149420e-02, 9.07362630e-03, 8.94386985e-03, 1.85752288e-02,
            2.77506326e-02, 1.48444354e-02, 1.02363418e-01, 3.01808623e-02, 4.56598129e-03
        ], [
            1.46185575e-02, 1.36528605e-02, 3.84886858e-02, 3.47882200e-02, 7.69072124e-02,
            2.57804133e-02, 3.69839634e-02, 1.64922026e-02, 3.77247088e-02, 1.19754915e+00,
            2.96926394e-02, 2.48497676e-02, 5.10751578e-02, 5.49927237e-02, 7.54983325e-02,
            5.59099857e-02, 1.22767615e-02, 3.81198753e-02, 3.23594913e-02, 1.03950926e-02,
            1.06927920e-02, 1.63335465e-02, 2.40525872e-02, 1.48161433e-02, 1.71139507e-02,
            1.85936224e-02, 1.11708845e-02, 5.42301430e-03, 5.13809741e-03, 1.01103279e-02,
            1.27327105e-02, 7.04131276e-03, 1.51597465e-02, 1.39979140e-02, 1.77942715e-03
        ], [
            5.49191317e-03, 9.28468778e-03, 1.22342606e-02, 9.42989080e-03, 9.11236164e-03,
            1.67937622e-02, 6.95234589e-03, 1.00579719e-02, 1.38076891e-02, 1.48290813e-02,
            1.17490257e+00, 2.43920571e-02, 1.77079040e-02, 2.47283868e-02, 2.03340513e-02,
            1.65062262e-02, 8.64513711e-03, 1.23019943e-01, 9.96641814e-03, 3.67901594e-03,
            3.87206947e-03, 8.72045408e-03, 6.80395238e-03, 5.11428531e-03, 5.97834852e-03,
            7.43189005e-03, 5.76958350e-03, 2.75029681e-03, 7.16796791e-03, 4.49649366e-03,
            7.34477248e-03, 5.39371114e-03, 6.83428738e-03, 6.74782140e-03, 9.93750526e-04
        ], [
            2.40472076e-02, 5.06045076e-02, 4.17203055e-02, 4.18404445e-02, 5.01625029e-02,
            7.28762317e-02, 4.37932214e-02, 4.86821595e-02, 5.22072802e-02, 7.65287235e-02,
            9.70645931e-02, 1.54336627e+00, 3.34716608e-01, 1.87261420e-01, 2.46552715e-01,
            1.57172768e-01, 4.09327672e-02, 1.84734819e-01, 6.71835903e-02, 1.74701801e-02,
            1.64663483e-02, 2.54350749e-02, 3.90610342e-02, 3.97468506e-02, 4.19312059e-02,
            4.09371573e-02, 2.60528481e-02, 1.15676869e-02, 1.55960151e-02, 2.21078507e-02,
            3.08832481e-02, 1.62545061e-02, 2.37451336e-02, 2.78390648e-02, 5.13072124e-03
        ], [
            1.74669156e-02, 2.91467038e-02, 1.91301055e-02, 3.34807880e-02, 2.85657117e-02,
            3.19368760e-02, 2.41082455e-02, 3.12994326e-02, 2.69817960e-02, 3.68706099e-02,
            4.97140568e-02, 6.15067564e-02, 1.23551943e+00, 5.12621860e-02, 9.40610994e-02,
            3.71999683e-02, 2.51037402e-02, 4.63492731e-02, 4.55050063e-02, 8.99297844e-03,
            8.39153520e-03, 1.19860898e-02, 2.03021964e-02, 2.72196563e-02, 2.61457041e-02,
            2.34831921e-02, 1.12706103e-02, 6.04116443e-03, 6.22569880e-03, 1.05915860e-02,
            1.77199254e-02, 8.29441043e-03, 1.35804331e-02, 1.38469999e-02, 3.11021357e-03
        ], [
            2.11020493e-02, 3.53849674e-02, 3.38213941e-02, 4.83851135e-02, 4.35813303e-02,
            4.76140717e-02, 5.25745984e-02, 4.13077455e-02, 5.36111974e-02, 6.30745433e-02,
            6.05708811e-02, 8.23985445e-02, 2.54139811e-01, 1.67276579e+00, 2.06417616e-01,
            9.08073262e-02, 7.97582061e-02, 1.12278495e-01, 9.43881429e-02, 3.08076876e-02,
            2.96464237e-02, 3.15357309e-02, 4.57480376e-02, 3.98731248e-02, 4.76726296e-02,
            5.08162988e-02, 1.00653643e-01, 2.72984614e-02, 1.70517210e-02, 5.64995173e-02,
            5.90292877e-02, 3.59067706e-02, 6.74812063e-02, 5.23112151e-02, 5.33441119e-03
        ], [
            1.65995101e-02, 1.68524414e-02, 1.85045015e-02, 2.11895536e-02, 2.10391374e-02,
            2.43503254e-02, 2.04062674e-02, 1.95146484e-02, 1.78316814e-02, 2.34746849e-02,
            2.82413244e-02, 3.17578215e-02, 4.84864192e-02, 2.36534085e-02, 1.49393357e+00,
            2.70551819e-02, 1.74002933e-02, 2.60199366e-02, 1.30099014e-01, 1.53286636e-02,
            1.24248789e-02, 1.30554188e-02, 7.45280357e-02, 8.28954791e-02, 1.02971911e-01,
            4.53308038e-02, 1.29725581e-02, 7.90850262e-03, 5.87909946e-03, 1.43485625e-02,
            3.75882906e-02, 9.83966302e-03, 1.02084409e-02, 2.23152871e-02, 2.17726513e-03
        ], [
            2.25814753e-03, 2.75874978e-03, 3.80914654e-03, 9.98793111e-03, 7.39531949e-03,
            8.44753594e-03, 8.76500675e-03, 3.47086648e-03, 4.97051986e-03, 7.17396594e-03,
            7.70002572e-03, 1.83114677e-02, 1.07523670e-02, 7.65247243e-03, 1.26032072e-02,
            1.05750787e+00, 3.83175646e-03, 9.98251661e-03, 4.78590580e-03, 2.53214369e-03,
            2.81448570e-03, 4.49418577e-03, 5.21789952e-03, 3.31501158e-03, 4.83761520e-03,
            4.31774528e-03, 2.94300470e-03, 2.42434049e-03, 2.55244907e-03, 3.63934463e-03,
            5.93981323e-03, 5.34136473e-03, 5.20417031e-03, 7.98019043e-03, 1.86823459e-03
        ], [
            2.57040685e-02, 4.83423962e-02, 3.82530301e-02, 5.19591418e-02, 3.71941844e-02,
            4.69711604e-02, 5.23760240e-02, 7.23503471e-02, 5.95185586e-02, 5.58272119e-02,
            8.01964353e-02, 6.97276926e-02, 4.24879413e-02, 3.33944668e-02, 3.83182885e-02,
            3.49979001e-02, 1.18222675e+00, 3.64110006e-02, 2.68142180e-02, 1.50974278e-02,
            2.26337244e-02, 4.17407568e-02, 3.38322645e-02, 2.18510240e-02, 2.66389476e-02,
            3.52653089e-02, 2.09231048e-02, 1.29428166e-02, 1.47515922e-02, 1.63275706e-02,
            2.86034576e-02, 3.23126956e-02, 2.69965564e-02, 3.10700360e-02, 4.49963490e-03
        ], [
            7.17638804e-03, 1.10351065e-02, 9.05138684e-03, 1.00105277e-02, 8.87788117e-03,
            1.06453870e-02, 1.05302040e-02, 1.47591961e-02, 1.07579741e-02, 1.01248937e-02,
            1.44408535e-02, 1.23852916e-02, 1.00770013e-02, 8.07645340e-03, 9.27645217e-03,
            9.08877188e-03, 2.40011428e-02, 1.05949216e+00, 1.10657751e-02, 7.49444896e-03,
            9.28969541e-03, 1.17397530e-02, 1.50523901e-02, 9.05804031e-03, 1.02849357e-02,
            2.03806599e-02, 1.60466156e-02, 1.01209648e-02, 3.69180307e-02, 9.53616853e-03,
            2.38895375e-02, 1.51440630e-02, 1.29074093e-02, 1.54637069e-02, 1.65786878e-03
        ], [
            9.64715532e-03, 6.62762659e-03, 1.48189026e-02, 1.34041896e-02, 1.54704645e-02,
            1.44895319e-02, 1.19350499e-02, 9.81345470e-03, 1.27274663e-02, 1.41085735e-02,
            1.39182088e-02, 1.37288035e-02, 1.23293604e-02, 1.01951369e-02, 1.35721365e-02,
            1.33396065e-02, 1.28154929e-02, 1.44281039e-02, 1.01642855e+00, 7.21263143e-03,
            7.02119713e-03, 1.07990323e-02, 2.05525310e-02, 7.66381948e-03, 1.09371958e-02,
            1.43127716e-02, 7.20444899e-03, 6.36062160e-03, 4.27635940e-03, 9.03523650e-03,
            8.40548429e-03, 5.51831443e-03, 8.10257401e-03, 1.07271518e-02, 9.41226419e-04
        ], [
            6.19168154e-02, 3.64132393e-02, 1.13920523e-01, 1.00712492e-01, 1.14406451e-01,
            1.05083024e-01, 9.02805165e-02, 9.77612386e-02, 8.91717623e-02, 9.76670498e-02,
            8.05944528e-02, 9.24506301e-02, 9.49351494e-02, 8.15693722e-02, 1.02845875e-01,
            8.53539393e-02, 4.78832270e-02, 8.44857550e-02, 6.01442585e-02, 1.03802444e+00,
            2.58424364e-02, 8.08425615e-02, 4.95152151e-02, 4.99982713e-02, 5.59289081e-02,
            4.87350177e-02, 2.88680855e-02, 1.72310353e-02, 1.40129642e-02, 3.06216886e-02,
            4.05181209e-02, 2.59357108e-02, 5.07065503e-02, 3.94709016e-02, 5.37064458e-03
        ], [
            2.81930283e-02, 1.41608607e-02, 5.04143141e-02, 4.88459927e-02, 5.83392049e-02,
            4.01130368e-02, 3.53495821e-02, 2.95284364e-02, 3.93528616e-02, 4.13191903e-02,
            3.37040469e-02, 3.56587317e-02, 3.60010384e-02, 3.04783679e-02, 4.23455168e-02,
            4.37833552e-02, 1.76086144e-02, 4.63223228e-02, 2.63078403e-02, 1.20467951e-02,
            1.01289434e+00, 3.85046990e-02, 2.51238728e-02, 1.89760309e-02, 2.29682290e-02,
            2.11153578e-02, 1.59857161e-02, 1.14317622e-02, 8.11516292e-03, 1.59838684e-02,
            1.76222993e-02, 1.19801198e-02, 2.25682770e-02, 2.09611572e-02, 2.54712708e-03
        ], [
            5.87173347e-03, 7.57473424e-03, 1.17140335e-02, 1.33354213e-02, 1.41048105e-02,
            1.24960715e-02, 1.42451741e-02, 8.78200853e-03, 1.22859550e-02, 1.34659940e-02,
            1.36543374e-02, 1.32196362e-02, 1.43129176e-02, 1.23134901e-02, 1.22579208e-02,
            1.12265814e-02, 8.69566768e-03, 1.37848901e-02, 1.12213694e-02, 1.21041908e-02,
            1.02845011e-02, 1.01628350e+00, 1.45378692e-02, 1.32854579e-02, 2.72803812e-02,
            3.12351095e-02, 1.47745954e-02, 1.75245361e-02, 6.13644560e-03, 1.89742665e-02,
            1.71284216e-02, 1.30419534e-02, 1.42357977e-02, 1.91153852e-02, 1.02708330e-03
        ], [
            2.75248843e-02, 3.01366097e-02, 4.88834862e-02, 5.07707097e-02, 5.04977856e-02,
            5.12368977e-02, 4.17556682e-02, 7.54854716e-02, 4.28175492e-02, 4.32327116e-02,
            6.09183542e-02, 4.42231289e-02, 3.64249671e-02, 2.85013306e-02, 3.68937750e-02,
            4.18431050e-02, 3.65522560e-02, 4.48355488e-02, 3.13472005e-02, 3.08008127e-02,
            2.02518730e-02, 2.67263638e-02, 1.06509395e+00, 3.30421080e-02, 3.36010249e-02,
            8.09379839e-02, 1.82093172e-02, 1.28756480e-02, 7.20924627e-03, 1.47237508e-02,
            2.22251737e-02, 1.73772024e-02, 1.90110588e-02, 2.12427410e-02, 3.74490589e-03
        ], [
            5.98117401e-03, 1.01677245e-02, 1.18126519e-02, 1.27380859e-02, 1.22027635e-02,
            1.49983934e-02, 8.64836338e-03, 1.14137482e-02, 8.99230932e-03, 9.65885353e-03,
            1.57742989e-02, 1.35990866e-02, 9.83116667e-03, 7.46635485e-03, 9.72078219e-03,
            8.78715740e-03, 7.09870847e-03, 9.63581565e-03, 5.72522984e-03, 8.70611068e-03,
            5.42416891e-03, 6.00708936e-03, 7.96544886e-03, 1.18131541e+00, 7.86983948e-03,
            1.03887496e-02, 3.39406824e-03, 2.49345200e-03, 4.31215225e-03, 3.77171889e-03,
            3.94435332e-03, 2.77221860e-03, 3.36902854e-03, 4.29553299e-03, 9.89002789e-04
        ], [
            2.54688634e-03, 3.17939341e-03, 4.49710836e-03, 5.43461628e-03, 6.35373642e-03,
            5.25681588e-03, 5.71341454e-03, 3.70068344e-03, 4.83816272e-03, 4.96509219e-03,
            4.79423029e-03, 4.34104619e-03, 5.37197710e-03, 4.83611025e-03, 5.01225269e-03,
            4.32045451e-03, 2.83693945e-03, 4.37345261e-03, 4.66619120e-03, 4.86810044e-03,
            3.23157553e-03, 3.72822082e-03, 5.48797964e-03, 1.05143848e-02, 1.03510180e+00,
            1.84634648e-02, 6.01963621e-03, 5.42868126e-03, 1.73355677e-03, 6.03351319e-03,
            6.39028747e-03, 4.69413646e-03, 3.26603871e-03, 6.97356408e-03, 4.02536451e-04
        ], [
            1.19282611e-02, 1.22247550e-02, 2.18472557e-02, 1.98503298e-02, 2.11752804e-02,
            2.35435231e-02, 2.44502980e-02, 2.27993077e-02, 1.84595588e-02, 1.98647260e-02,
            2.38864518e-02, 2.00237141e-02, 1.82301517e-02, 1.48854761e-02, 1.90113843e-02,
            1.73115327e-02, 1.31602983e-02, 2.68750737e-02, 2.38353802e-02, 2.96254638e-02,
            1.89621180e-02, 1.80559594e-02, 5.41388928e-02, 1.59025134e-01, 1.17605154e-01,
            1.12671426e+00, 1.11327689e-02, 9.69569272e-03, 5.05365366e-03, 1.23902295e-02,
            1.27661680e-02, 8.20513453e-03, 1.06301610e-02, 1.50498597e-02, 1.47703634e-03
        ], [
            1.48296906e-02, 1.53431496e-02, 2.38125234e-02, 2.80735056e-02, 2.74651300e-02,
            2.57622363e-02, 3.51743691e-02, 2.13787546e-02, 2.75780508e-02, 2.56399305e-02,
            2.55581327e-02, 2.74572245e-02, 2.69963705e-02, 2.72122629e-02, 2.58638055e-02,
            3.10116363e-02, 2.11003494e-02, 3.20967251e-02, 3.34028704e-02, 3.20418330e-02,
            3.45671662e-02, 3.13534774e-02, 3.57828778e-02, 3.92650418e-02, 4.30040978e-02,
            4.47837758e-02, 1.14872993e+00, 5.22875364e-02, 1.48033591e-02, 4.24516231e-02,
            3.90410135e-02, 2.14573084e-02, 2.88289052e-02, 4.11498895e-02, 2.97437818e-03
        ], [
            4.61729569e-02, 5.29744226e-02, 6.25864684e-02, 8.11275635e-02, 7.11830544e-02,
            6.56046087e-02, 7.13205791e-02, 6.37931169e-02, 6.49295695e-02, 6.29912503e-02,
            7.19574272e-02, 6.83931267e-02, 6.47269919e-02, 5.60531697e-02, 6.48519834e-02,
            7.47630050e-02, 6.11012401e-02, 7.02954939e-02, 5.52158753e-02, 5.99193488e-02,
            6.52129621e-02, 6.39254464e-02, 9.14328839e-02, 6.71098639e-02, 7.91419507e-02,
            8.34999086e-02, 4.93641596e-02, 1.29265218e+00, 1.14015221e-01, 7.27792730e-02,
            5.57656536e-02, 3.74204010e-02, 5.88671209e-02, 8.93937818e-02, 1.55906806e-02
        ], [
            1.09522164e-02, 1.33816824e-02, 2.01605351e-02, 2.26956789e-02, 2.48175800e-02,
            2.22757552e-02, 2.79776112e-02, 1.63803158e-02, 1.99746864e-02, 2.20081641e-02,
            2.01249308e-02, 2.06390473e-02, 2.15094256e-02, 1.83737712e-02, 2.28614525e-02,
            2.20823415e-02, 1.52046107e-02, 2.69185185e-02, 3.50104768e-02, 2.95854820e-02,
            5.31392338e-02, 4.17169970e-02, 2.23868327e-02, 2.90035609e-02, 2.61520279e-02,
            3.70719265e-02, 2.65614123e-02, 4.08467994e-02, 1.04361962e+00, 3.69834676e-02,
            2.57180046e-02, 2.84735665e-02, 4.86814884e-02, 4.19891677e-02, 1.84881967e-03
        ], [
            5.35142221e-02, 6.88838205e-02, 1.07333088e-01, 1.01575220e-01, 9.97015670e-02,
            9.82772197e-02, 1.49623905e-01, 8.24540798e-02, 1.46893629e-01, 1.21663673e-01,
            1.04835474e-01, 1.03360082e-01, 1.20986750e-01, 1.21150227e-01, 1.33228142e-01,
            9.62266910e-02, 8.37332088e-02, 1.30334898e-01, 1.08679328e-01, 9.91636240e-02,
            1.02281836e-01, 1.02743251e-01, 1.01793939e-01, 8.74832713e-02, 1.36101864e-01,
            1.33307526e-01, 1.16635121e-01, 1.81076612e-01, 6.98221571e-02, 1.21260099e+00,
            1.40244881e-01, 7.11575111e-02, 1.21414631e-01, 1.47548927e-01, 1.17821236e-02
        ], [
            3.20307149e-03, 4.33946848e-03, 5.01273444e-03, 4.11382296e-03, 4.13993627e-03,
            6.66149768e-03, 7.32186951e-03, 4.84277538e-03, 6.19592983e-03, 5.68342388e-03,
            6.02564556e-03, 6.58055471e-03, 4.74150123e-03, 3.98536806e-03, 4.77247912e-03,
            5.24998845e-03, 7.12199197e-03, 4.90600313e-03, 4.85220516e-03, 3.11362801e-03,
            4.18998558e-03, 5.86016113e-03, 6.14266802e-03, 8.52502071e-03, 1.17843931e-02,
            6.17881568e-03, 4.78119504e-03, 3.69611816e-03, 4.89554879e-03, 5.11928631e-03,
            1.01007236e+00, 6.17807505e-03, 6.48353316e-03, 1.12532899e-02, 4.80836896e-04
        ], [
            1.19954173e-03, 1.00536472e-03, 1.58496034e-03, 1.57950466e-03, 1.65042411e-03,
            1.52003962e-03, 1.77671134e-03, 1.43674514e-03, 2.19056767e-03, 1.90972806e-03,
            1.63302550e-03, 1.71401453e-03, 2.03547454e-03, 1.85115136e-03, 2.21531791e-03,
            1.46271581e-03, 1.50917937e-03, 1.65388475e-03, 1.70455258e-03, 1.38042740e-03,
            1.67517791e-03, 1.49282863e-03, 1.93208875e-03, 1.30514108e-03, 3.18647624e-03,
            2.35568384e-03, 2.29551994e-03, 2.31614905e-03, 8.24706472e-04, 3.33723771e-03,
            4.97388391e-03, 1.01595943e+00, 2.31034533e-03, 2.65945246e-03, 1.56337728e-04
        ], [
            1.88120804e-03, 8.51317051e-04, 1.54717965e-03, 1.37738436e-03, 2.00991582e-03,
            1.54968139e-03, 1.30551700e-03, 9.38771483e-04, 1.31844441e-03, 1.25928713e-03,
            1.56010075e-03, 1.36655132e-03, 1.78389290e-03, 1.24274587e-03, 1.45233142e-03,
            1.24504413e-03, 9.73583693e-04, 1.22371489e-03, 1.08971031e-03, 8.61417547e-04,
            7.90911893e-04, 1.65016374e-03, 1.14911396e-03, 9.35150449e-04, 1.09648060e-03,
            1.13878179e-03, 1.09568440e-03, 1.10230840e-03, 3.74359227e-04, 1.07808245e-03,
            6.28681996e-03, 2.05226294e-03, 1.03510033e+00, 2.00798663e-03, 1.34670997e-04
        ], [
            9.80446417e-03, 1.31881245e-02, 1.72522254e-02, 1.79102596e-02, 2.04751095e-02,
            1.76508244e-02, 2.84721657e-02, 1.43617477e-02, 2.04308361e-02, 1.83347895e-02,
            1.82645552e-02, 1.83830326e-02, 1.73013602e-02, 1.47674402e-02, 1.82926062e-02,
            1.60284192e-02, 1.48143322e-02, 1.88153177e-02, 1.66493686e-02, 1.50582723e-02,
            1.61156231e-02, 2.40529066e-02, 2.32965235e-02, 1.63229224e-02, 1.95016262e-02,
            2.74066868e-02, 2.85970487e-02, 2.21242443e-02, 1.37826030e-02, 3.60554490e-02,
            2.81473843e-02, 1.89000789e-02, 2.38207990e-02, 1.09424874e+00, 3.10768800e-03
        ], [
            4.12296599e-05, 5.38749624e-05, 7.15885581e-05, 1.73707091e-04, 9.63313297e-05,
            8.79420542e-05, 8.71913214e-05, 6.37106527e-05, 6.90883477e-05, 7.18726737e-05,
            7.26015393e-05, 6.72082108e-05, 8.38505018e-05, 5.73492495e-05, 1.13702603e-04,
            6.25068447e-05, 4.15424501e-05, 1.10745979e-04, 7.93810687e-05, 4.25622990e-05,
            7.25164545e-05, 9.47337370e-05, 1.57661303e-04, 4.55323247e-05, 5.91659899e-05,
            6.36502078e-05, 7.03044570e-05, 1.13935863e-04, 2.43965365e-04, 1.06175772e-04,
            1.10414759e-04, 6.76185800e-05, 9.71599326e-05, 1.61167013e-04, 1.00010492e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 3
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2008():
    """
    Leontief matrix 2008



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.16382123e+00, 3.81765699e-03, 3.68272895e-01, 1.10566088e-01, 1.49689545e-01,
            1.99304999e-01, 3.80603006e-02, 8.26174063e-03, 2.34987198e-02, 3.20772025e-02,
            1.14001215e-02, 8.93466443e-03, 9.08286363e-03, 8.47023620e-03, 1.02475815e-02,
            4.10925771e-02, 5.15057172e-03, 1.75290884e-02, 7.15821808e-03, 8.58418330e-03,
            8.16980659e-03, 1.06041737e-01, 8.37081251e-03, 9.70470156e-03, 1.00543458e-02,
            1.73284214e-02, 4.87131728e-03, 4.87341073e-03, 2.97916148e-03, 7.04272395e-03,
            1.07978171e-02, 1.07134868e-02, 1.30494513e-02, 1.27311970e-02, 3.66370802e-02
        ], [
            2.70992488e-02, 1.10073744e+00, 3.07087131e-02, 4.25957177e-02, 3.60689657e-02,
            3.98730238e-02, 3.60501870e-02, 6.54241984e-01, 1.09802266e-01, 5.83271841e-02,
            1.37490437e-01, 1.23817824e-01, 4.64850347e-02, 3.43061806e-02, 4.03080496e-02,
            3.99110920e-02, 1.84022238e-01, 6.95217172e-02, 2.08340210e-02, 1.28738080e-02,
            1.27089783e-02, 2.24611460e-02, 5.83623038e-02, 6.74599190e-02, 9.47681188e-02,
            4.00952068e-02, 1.33219654e-02, 8.68288451e-03, 9.17647207e-03, 1.39370091e-02,
            2.23849779e-02, 1.50611077e-02, 2.08550541e-02, 2.12178691e-02, 7.36947008e-02
        ], [
            9.65554339e-02, 4.05313401e-03, 1.23342933e+00, 3.09480967e-02, 1.84696585e-01,
            2.75887418e-02, 1.56226958e-02, 8.12154782e-03, 2.74995016e-02, 1.97178723e-02,
            1.18750384e-02, 1.04679671e-02, 1.06787058e-02, 1.01082128e-02, 1.09197512e-02,
            1.54131868e-02, 5.79688772e-03, 1.05205760e-02, 7.60818060e-03, 1.02804143e-02,
            9.75387135e-03, 1.95794691e-01, 8.34421883e-03, 1.39918120e-02, 1.46734240e-02,
            1.25883258e-02, 6.60064447e-03, 6.51342830e-03, 2.67904121e-03, 8.66334320e-03,
            1.52972817e-02, 1.52653596e-02, 2.18578062e-02, 1.65740871e-02, 7.35493725e-03
        ], [
            6.01387468e-03, 3.38505087e-03, 8.82537848e-03, 1.62676352e+00, 1.39355509e-01,
            1.29231274e-02, 1.94897354e-02, 5.46470844e-03, 1.27535103e-02, 3.68730669e-02,
            1.36883968e-02, 1.04371910e-02, 1.16344907e-02, 1.07711068e-02, 1.79401409e-02,
            5.04966673e-02, 4.85272894e-03, 1.05291662e-02, 7.31573165e-03, 6.48124221e-03,
            6.68407269e-03, 8.57080710e-03, 7.36947260e-03, 9.61750899e-03, 8.72766017e-03,
            7.59395393e-03, 3.68982339e-03, 3.06393863e-03, 2.06180213e-03, 5.39111130e-03,
            7.95009534e-03, 4.09259905e-03, 9.38858760e-03, 1.08642219e-02, 7.04179830e-04
        ], [
            5.79327831e-04, 3.08548755e-04, 7.99884876e-04, 2.89069848e-02, 1.30408052e+00,
            1.62329692e-03, 1.88775749e-03, 5.12094169e-04, 1.03608462e-03, 2.78401254e-03,
            1.07342568e-03, 9.62574012e-04, 1.11813763e-03, 1.28239817e-03, 4.97507768e-03,
            9.45331849e-03, 4.63851101e-04, 8.68539735e-04, 1.23891632e-03, 7.00760221e-04,
            6.10473599e-04, 6.33104205e-04, 9.01472608e-04, 7.72819281e-04, 9.33862434e-04,
            8.17406138e-04, 5.75431518e-04, 2.92593347e-04, 1.66548131e-04, 4.77649875e-04,
            1.44843453e-03, 4.33377077e-04, 8.21019612e-04, 1.11725004e-03, 6.93599452e-05
        ], [
            4.71845868e-03, 3.82252890e-03, 7.17140610e-03, 8.22703406e-03, 8.78888178e-03,
            1.37466705e+00, 2.67963191e-02, 4.71661072e-03, 6.62798476e-03, 9.95321447e-03,
            1.81938501e-02, 1.30110334e-02, 1.11055648e-02, 8.29702793e-03, 1.13374076e-02,
            1.16649524e-01, 4.35404501e-03, 4.79991486e-02, 5.47546866e-03, 3.80412855e-03,
            3.75625836e-03, 6.61078095e-03, 5.22374539e-03, 4.09414213e-03, 4.46346979e-03,
            7.02769538e-03, 3.40492237e-03, 2.68925331e-03, 4.57859430e-03, 4.09528145e-03,
            6.48214760e-03, 6.80197056e-03, 5.12867664e-03, 7.59077586e-03, 7.29698018e-04
        ], [
            1.46801625e-02, 1.01025481e-02, 5.41640240e-02, 4.31479349e-02, 4.99164264e-02,
            4.18724996e-02, 1.35410089e+00, 1.63857315e-02, 4.08354578e-02, 4.65622692e-02,
            4.86656501e-02, 2.41337160e-02, 2.81160441e-02, 3.24884341e-02, 2.71100916e-02,
            4.70271891e-02, 1.40007675e-02, 2.54517883e-02, 2.34089712e-02, 2.36575598e-02,
            2.24114794e-02, 2.79932210e-02, 2.12556341e-02, 1.77506066e-02, 2.19671803e-02,
            3.22151129e-02, 2.28304744e-02, 3.08263661e-02, 8.79572831e-03, 3.90663891e-02,
            3.53203420e-02, 3.29464527e-02, 2.48280302e-02, 4.55671111e-02, 1.99460577e-03
        ], [
            2.98811544e-02, 1.57406367e-02, 2.63435149e-02, 3.53244250e-02, 3.14784875e-02,
            3.06711729e-02, 2.63195296e-02, 1.23529645e+00, 1.02804410e-01, 5.17208268e-02,
            5.29063400e-02, 4.41445547e-02, 2.65280249e-02, 2.18422680e-02, 2.44415264e-02,
            2.55857128e-02, 6.11923579e-02, 3.45363423e-02, 1.86170541e-02, 1.41916360e-02,
            1.16890315e-02, 1.73955838e-02, 8.15469922e-02, 1.05412619e-01, 1.55277974e-01,
            4.83541611e-02, 1.22735773e-02, 8.09164827e-03, 5.85694871e-03, 1.39275348e-02,
            2.02477517e-02, 1.11796773e-02, 1.70252186e-02, 1.74694270e-02, 2.32891952e-03
        ], [
            7.08281680e-02, 2.21074636e-02, 5.62307918e-02, 1.62059085e-01, 1.19219343e-01,
            9.60769450e-02, 8.93436907e-02, 4.85603048e-02, 1.36401097e+00, 3.71327972e-01,
            8.50328752e-02, 5.21388631e-02, 5.31494696e-02, 6.99821477e-02, 6.50843308e-02,
            8.06804959e-02, 2.50303874e-02, 5.05222250e-02, 2.91029179e-02, 1.35388759e-02,
            1.23467380e-02, 2.71243434e-02, 2.33643161e-02, 2.34036715e-02, 2.62830456e-02,
            2.46578164e-02, 1.33255424e-02, 9.23024630e-03, 8.00823883e-03, 1.85155781e-02,
            2.67288493e-02, 1.48977191e-02, 9.88234214e-02, 2.98987745e-02, 4.44506681e-03
        ], [
            1.45583155e-02, 1.12040623e-02, 3.84961952e-02, 3.58290875e-02, 7.90076258e-02,
            2.54721628e-02, 3.73257552e-02, 1.70910288e-02, 3.83280684e-02, 1.20285288e+00,
            2.98805902e-02, 2.44623947e-02, 4.98468323e-02, 5.63186729e-02, 7.65740327e-02,
            5.78136622e-02, 1.19170196e-02, 3.78574024e-02, 3.12278797e-02, 1.06072652e-02,
            1.05497740e-02, 1.64056112e-02, 2.40347343e-02, 1.47776355e-02, 1.81875931e-02,
            1.91524313e-02, 1.11523654e-02, 5.57884530e-03, 4.79595479e-03, 1.02029869e-02,
            1.24802072e-02, 7.16504693e-03, 1.51040609e-02, 1.40921601e-02, 1.69270605e-03
        ], [
            5.49009266e-03, 7.50087108e-03, 1.22323474e-02, 9.58083019e-03, 9.18772403e-03,
            1.61968037e-02, 7.01514000e-03, 9.92430102e-03, 1.38876943e-02, 1.49311277e-02,
            1.17716168e+00, 2.44781350e-02, 1.77373903e-02, 2.55098771e-02, 2.09458747e-02,
            1.70104940e-02, 8.13417121e-03, 1.22974084e-01, 9.87668470e-03, 3.69589846e-03,
            3.84063171e-03, 8.78128789e-03, 6.86697680e-03, 5.07352701e-03, 6.45884244e-03,
            7.62494569e-03, 5.70438711e-03, 2.87973780e-03, 6.87053103e-03, 4.57409959e-03,
            7.17280801e-03, 5.46381176e-03, 6.87473881e-03, 6.82837017e-03, 9.02660718e-04
        ], [
            2.39526811e-02, 4.23186705e-02, 4.15498055e-02, 4.21503843e-02, 4.94792593e-02,
            7.15363801e-02, 4.44147517e-02, 4.92151171e-02, 5.23591231e-02, 7.64977697e-02,
            9.61292211e-02, 1.52693391e+00, 3.23318410e-01, 1.88074342e-01, 2.47960259e-01,
            1.56722227e-01, 3.91780247e-02, 1.85267996e-01, 6.52114953e-02, 1.77171846e-02,
            1.63553655e-02, 2.59500030e-02, 3.93157471e-02, 3.80888754e-02, 4.48922009e-02,
            4.22986179e-02, 2.59126281e-02, 1.20721139e-02, 1.47076990e-02, 2.26424200e-02,
            3.06428098e-02, 1.64498035e-02, 2.40216171e-02, 2.83203278e-02, 4.77956915e-03
        ], [
            1.82554546e-02, 2.45234069e-02, 1.99689464e-02, 3.49786171e-02, 2.96980813e-02,
            3.35561573e-02, 2.50291080e-02, 3.24221630e-02, 2.78350788e-02, 3.87537039e-02,
            5.11841799e-02, 6.24749244e-02, 1.23985790e+00, 5.35518531e-02, 1.01132948e-01,
            3.94255310e-02, 2.47391005e-02, 4.81539646e-02, 4.66021905e-02, 9.42378251e-03,
            8.59992089e-03, 1.26165377e-02, 2.11921008e-02, 2.77997004e-02, 2.89881154e-02,
            2.51562319e-02, 1.14960211e-02, 6.53310268e-03, 6.12618411e-03, 1.10776716e-02,
            1.83459694e-02, 8.68216720e-03, 1.43685942e-02, 1.44233719e-02, 2.96301352e-03
        ], [
            2.24671120e-02, 3.13142763e-02, 3.63040146e-02, 5.22393796e-02, 4.64116891e-02,
            5.01064567e-02, 5.57067215e-02, 4.51892295e-02, 5.68461091e-02, 6.68261462e-02,
            6.41696022e-02, 8.42011060e-02, 2.61128011e-01, 1.70632610e+00, 2.22855292e-01,
            9.64035133e-02, 8.22835701e-02, 1.19459089e-01, 9.74855298e-02, 3.31603274e-02,
            3.02660136e-02, 3.41313982e-02, 4.93184903e-02, 4.18454895e-02, 5.36797389e-02,
            5.53282300e-02, 1.01324313e-01, 2.94282574e-02, 1.69535293e-02, 6.03846326e-02,
            6.35273427e-02, 3.92703023e-02, 7.19327474e-02, 5.69775353e-02, 5.38757293e-03
        ], [
            1.67941282e-02, 1.39864065e-02, 1.88933573e-02, 2.20069741e-02, 2.14185696e-02,
            2.51963049e-02, 2.13127077e-02, 2.04590154e-02, 1.84078279e-02, 2.43867037e-02,
            2.88069649e-02, 3.23124153e-02, 4.85843822e-02, 2.42366331e-02, 1.50327655e+00,
            2.84238153e-02, 1.70485698e-02, 2.69046900e-02, 1.22879090e-01, 1.60905524e-02,
            1.26014916e-02, 1.34711451e-02, 7.47132290e-02, 8.23980160e-02, 1.11146644e-01,
            5.06140998e-02, 1.31497828e-02, 8.31661862e-03, 6.01159756e-03, 1.45823458e-02,
            3.70911236e-02, 1.00755401e-02, 1.03090876e-02, 2.28732541e-02, 2.02865299e-03
        ], [
            2.39412958e-03, 2.30011968e-03, 3.91093383e-03, 1.11880520e-02, 7.79979539e-03,
            8.39294828e-03, 8.60316707e-03, 3.67979379e-03, 5.13339134e-03, 7.27014839e-03,
            7.99851642e-03, 1.78536225e-02, 1.05863426e-02, 7.77939211e-03, 1.32171090e-02,
            1.06254760e+00, 4.01440354e-03, 9.77367953e-03, 4.65414493e-03, 2.58978518e-03,
            2.82429167e-03, 4.63699693e-03, 6.13121215e-03, 3.30666943e-03, 5.21670099e-03,
            4.42466426e-03, 3.24706576e-03, 2.47250923e-03, 2.34560317e-03, 3.69808172e-03,
            5.76199573e-03, 5.29465454e-03, 5.31882504e-03, 7.95546368e-03, 2.10473636e-03
        ], [
            2.78655002e-02, 4.39569074e-02, 4.20071061e-02, 5.72222337e-02, 4.03960371e-02,
            5.13928618e-02, 5.70030989e-02, 1.01266833e-01, 6.67190067e-02, 6.15046734e-02,
            8.63425626e-02, 7.38705814e-02, 4.54587940e-02, 3.70151879e-02, 4.24436116e-02,
            3.93355583e-02, 1.19119001e+00, 3.98558661e-02, 2.84219847e-02, 1.67203124e-02,
            2.42699395e-02, 4.57087187e-02, 3.78883898e-02, 2.54881331e-02, 3.47475000e-02,
            3.97976568e-02, 2.25230904e-02, 1.44540841e-02, 1.46698120e-02, 1.79062359e-02,
            2.95954725e-02, 3.46989857e-02, 2.88378993e-02, 3.34467546e-02, 4.41707835e-03
        ], [
            7.30668109e-03, 9.82652450e-03, 9.34986142e-03, 1.02936206e-02, 8.97698203e-03,
            1.05997784e-02, 1.05908272e-02, 1.74954396e-02, 1.09910180e-02, 1.01348937e-02,
            1.43257098e-02, 1.21671726e-02, 9.89455187e-03, 8.17973113e-03, 9.43783616e-03,
            9.35301613e-03, 2.21531716e-02, 1.06128802e+00, 1.13512329e-02, 7.61415644e-03,
            9.40775312e-03, 1.22635410e-02, 1.55336625e-02, 9.10277860e-03, 1.13769149e-02,
            2.07779279e-02, 1.58137858e-02, 1.08277994e-02, 3.60809154e-02, 9.65196780e-03,
            2.35145141e-02, 1.52822233e-02, 1.32692599e-02, 1.57784430e-02, 1.52115962e-03
        ], [
            9.49209795e-03, 5.42993596e-03, 1.45833300e-02, 1.26352609e-02, 1.46213602e-02,
            1.39907147e-02, 1.17093546e-02, 9.90050635e-03, 1.25563098e-02, 1.37097611e-02,
            1.32325271e-02, 1.30986535e-02, 1.17566475e-02, 1.00106955e-02, 1.35884105e-02,
            1.32079611e-02, 1.15162657e-02, 1.38916146e-02, 1.01618891e+00, 7.02828724e-03,
            6.83446196e-03, 1.05446884e-02, 1.95089615e-02, 7.32567797e-03, 1.12863589e-02,
            1.44357853e-02, 6.76129303e-03, 6.43533414e-03, 3.98437570e-03, 8.93809400e-03,
            7.91711867e-03, 5.22802925e-03, 7.79732696e-03, 1.02947251e-02, 8.57038066e-04
        ], [
            6.03622332e-02, 3.11094966e-02, 1.13356235e-01, 9.80710713e-02, 1.14244876e-01,
            1.04204741e-01, 9.11489683e-02, 1.08829191e-01, 9.25542513e-02, 9.85890617e-02,
            8.04686190e-02, 9.33304490e-02, 9.40863150e-02, 8.41296571e-02, 1.06815735e-01,
            8.65519582e-02, 4.61210192e-02, 8.50565802e-02, 5.91631721e-02, 1.03757254e+00,
            2.53351206e-02, 8.12153009e-02, 4.94908136e-02, 5.00388697e-02, 5.99664425e-02,
            5.00958982e-02, 2.89862455e-02, 1.79423902e-02, 1.32051582e-02, 3.10643540e-02,
            4.14722910e-02, 2.63726935e-02, 5.04265155e-02, 3.98040512e-02, 5.02726214e-03
        ], [
            2.82988708e-02, 1.22146262e-02, 5.08920221e-02, 4.84008360e-02, 5.71356424e-02,
            4.03760361e-02, 3.56834332e-02, 2.97901830e-02, 4.03420331e-02, 4.20658879e-02,
            3.38277741e-02, 3.59196101e-02, 3.61725557e-02, 3.14049597e-02, 4.39201179e-02,
            4.38418690e-02, 1.72785099e-02, 4.40275209e-02, 2.64371963e-02, 1.22378132e-02,
            1.01286858e+00, 3.87392013e-02, 2.52320182e-02, 1.84929721e-02, 2.40356445e-02,
            2.19395584e-02, 1.57398172e-02, 1.19950263e-02, 7.39896556e-03, 1.60681841e-02,
            1.72879329e-02, 1.19912560e-02, 2.25858462e-02, 2.09763683e-02, 2.32964880e-03
        ], [
            5.82663825e-03, 6.20792997e-03, 1.17228994e-02, 1.34774413e-02, 1.42504277e-02,
            1.21211665e-02, 1.38994731e-02, 8.82423425e-03, 1.21985230e-02, 1.32400728e-02,
            1.33582491e-02, 1.27119256e-02, 1.39101350e-02, 1.24927193e-02, 1.24015770e-02,
            1.13233786e-02, 8.00566033e-03, 1.35981140e-02, 1.11606116e-02, 1.22002231e-02,
            1.00295887e-02, 1.01628003e+00, 1.41975563e-02, 1.28958838e-02, 2.64885990e-02,
            3.17478484e-02, 1.35853116e-02, 1.82569795e-02, 5.44406701e-03, 1.88024340e-02,
            1.67293794e-02, 1.31350966e-02, 1.38096398e-02, 1.89029887e-02, 9.37152407e-04
        ], [
            2.77610238e-02, 2.53651232e-02, 5.02841221e-02, 5.14753015e-02, 5.17769723e-02,
            5.20531802e-02, 4.26250442e-02, 8.95883164e-02, 4.63121591e-02, 4.52472090e-02,
            6.19145767e-02, 4.50977568e-02, 3.70943422e-02, 3.00071402e-02, 3.87348507e-02,
            4.23159873e-02, 3.65333271e-02, 4.58771447e-02, 3.13501085e-02, 3.07910374e-02,
            2.03771290e-02, 2.73796001e-02, 1.06594335e+00, 3.42668587e-02, 3.80755357e-02,
            8.11965361e-02, 1.82669710e-02, 1.35455366e-02, 6.89576007e-03, 1.53350242e-02,
            2.20710220e-02, 1.76928461e-02, 1.92749297e-02, 2.17973489e-02, 3.34503799e-03
        ], [
            5.83842745e-03, 8.17896957e-03, 1.18681619e-02, 1.34859283e-02, 1.27296367e-02,
            1.62675443e-02, 8.96131421e-03, 1.04051990e-02, 9.20622226e-03, 1.01636580e-02,
            1.63653488e-02, 1.36450494e-02, 1.02703911e-02, 7.99018242e-03, 1.02797741e-02,
            9.16056354e-03, 6.65544518e-03, 9.89789212e-03, 5.27130635e-03, 9.01599866e-03,
            5.40624739e-03, 6.05765530e-03, 7.49190954e-03, 1.17883434e+00, 8.29362539e-03,
            1.07940360e-02, 3.28379316e-03, 2.49756643e-03, 4.45920696e-03, 3.75344871e-03,
            3.87045871e-03, 2.73065266e-03, 3.30231232e-03, 4.17466478e-03, 8.48841151e-04
        ], [
            2.53584526e-03, 2.68503228e-03, 4.54500231e-03, 5.42678484e-03, 6.25479471e-03,
            5.30940488e-03, 5.67335561e-03, 3.74680037e-03, 4.87334387e-03, 5.04858435e-03,
            4.71663606e-03, 4.25824069e-03, 5.36268556e-03, 4.98087682e-03, 5.08259505e-03,
            4.35574583e-03, 2.73455787e-03, 4.42479558e-03, 4.78212339e-03, 4.99402496e-03,
            3.30689632e-03, 3.80913470e-03, 5.54105806e-03, 9.91642733e-03, 1.03548669e+00,
            1.79888253e-02, 6.25129297e-03, 5.79676545e-03, 1.62314209e-03, 6.11578799e-03,
            6.23906554e-03, 4.83425735e-03, 3.29922401e-03, 7.10979995e-03, 3.71604117e-04
        ], [
            1.19802816e-02, 1.04141172e-02, 2.20762889e-02, 1.99850852e-02, 2.10515043e-02,
            2.33515835e-02, 2.42309595e-02, 2.27654447e-02, 1.88912840e-02, 2.01591967e-02,
            2.36525499e-02, 1.98681068e-02, 1.80793161e-02, 1.52303897e-02, 1.94460245e-02,
            1.75767897e-02, 1.25324083e-02, 2.76429632e-02, 2.45587780e-02, 2.98345020e-02,
            1.89568664e-02, 1.83027526e-02, 5.25406407e-02, 1.57489640e-01, 1.25980627e-01,
            1.12498705e+00, 1.09184639e-02, 9.80516592e-03, 4.96566268e-03, 1.23704219e-02,
            1.25156061e-02, 8.29023256e-03, 1.06614267e-02, 1.51475331e-02, 1.36675989e-03
        ], [
            1.54156231e-02, 1.34947847e-02, 2.50744094e-02, 2.95925854e-02, 2.91162337e-02,
            2.63694308e-02, 3.56229351e-02, 2.30822392e-02, 2.87174659e-02, 2.66182092e-02,
            2.61057756e-02, 2.81091585e-02, 2.73428138e-02, 2.89847232e-02, 2.69720321e-02,
            3.22702698e-02, 2.07745790e-02, 3.32485774e-02, 3.45021994e-02, 3.28583429e-02,
            3.49642835e-02, 3.27193084e-02, 3.73372962e-02, 4.07301661e-02, 4.46327299e-02,
            4.51446528e-02, 1.14754955e+00, 5.35708578e-02, 1.41966833e-02, 4.33493084e-02,
            3.97420162e-02, 2.24676896e-02, 2.97011602e-02, 4.17990942e-02, 2.85102785e-03
        ], [
            4.40983665e-02, 4.19752538e-02, 6.18791331e-02, 8.07927644e-02, 7.06165160e-02,
            6.34005614e-02, 6.92477998e-02, 6.42111176e-02, 6.36904863e-02, 6.25639880e-02,
            6.95251379e-02, 6.40687010e-02, 6.17549886e-02, 5.61037374e-02, 6.41769273e-02,
            7.29243414e-02, 5.64790859e-02, 6.74644273e-02, 5.40537947e-02, 5.84687249e-02,
            6.23755135e-02, 6.31488829e-02, 8.74677441e-02, 6.24142648e-02, 7.82681930e-02,
            7.95045155e-02, 4.66853070e-02, 1.27661469e+00, 1.15387838e-01, 6.96601631e-02,
            5.28588317e-02, 3.72882584e-02, 5.61648444e-02, 8.22968119e-02, 1.58206878e-02
        ], [
            1.11831407e-02, 1.16067303e-02, 1.99908177e-02, 2.21481882e-02, 2.40958908e-02,
            2.15533267e-02, 2.67309210e-02, 1.71497485e-02, 1.95828018e-02, 2.14783480e-02,
            1.92284856e-02, 1.98426272e-02, 2.05538862e-02, 1.80314508e-02, 2.24696542e-02,
            2.18595425e-02, 1.41840411e-02, 2.63660020e-02, 3.39626534e-02, 2.87600266e-02,
            5.18972303e-02, 4.09691845e-02, 2.14725304e-02, 2.69307730e-02, 2.54886215e-02,
            3.62637022e-02, 2.42761293e-02, 3.91020893e-02, 1.04204730e+00, 3.47203228e-02,
            2.59963591e-02, 2.82573390e-02, 4.66427802e-02, 4.09090803e-02, 1.73617575e-03
        ], [
            5.45108831e-02, 6.01402490e-02, 1.10182019e-01, 1.03163494e-01, 1.00779372e-01,
            9.70940936e-02, 1.51873920e-01, 8.71678919e-02, 1.49434137e-01, 1.24605121e-01,
            1.03948752e-01, 1.02525555e-01, 1.19765629e-01, 1.25044948e-01, 1.34795995e-01,
            9.92984088e-02, 8.07390535e-02, 1.31803237e-01, 1.10851845e-01, 1.03867741e-01,
            1.03544932e-01, 1.08148782e-01, 1.02522929e-01, 8.55605140e-02, 1.41571177e-01,
            1.34178445e-01, 1.13982392e-01, 1.88211305e-01, 6.38775333e-02, 1.21945468e+00,
            1.44253119e-01, 7.30635015e-02, 1.26198144e-01, 1.51913446e-01, 1.06493441e-02
        ], [
            3.26273294e-03, 3.81359312e-03, 5.30090590e-03, 4.21058892e-03, 4.26479001e-03,
            6.66698911e-03, 7.93724400e-03, 5.24040066e-03, 6.51224357e-03, 6.02988649e-03,
            6.27503455e-03, 7.66500040e-03, 5.00331372e-03, 4.24392027e-03, 5.05663832e-03,
            5.73734104e-03, 6.69525071e-03, 5.07937224e-03, 5.13905560e-03, 3.22624424e-03,
            4.31762513e-03, 6.31096217e-03, 6.04335386e-03, 8.03952162e-03, 1.18957319e-02,
            6.33096593e-03, 4.69863756e-03, 3.85585291e-03, 4.27845143e-03, 5.14002123e-03,
            1.00965182e+00, 6.09247695e-03, 6.62603643e-03, 1.10461364e-02, 4.53953058e-04
        ], [
            1.21339964e-03, 8.43665046e-04, 1.58483431e-03, 1.57726673e-03, 1.63885916e-03,
            1.49682628e-03, 1.75056524e-03, 1.47714805e-03, 2.17699877e-03, 1.91004358e-03,
            1.60133329e-03, 1.67042732e-03, 1.98854858e-03, 1.81220538e-03, 2.22607022e-03,
            1.46381228e-03, 1.39892106e-03, 1.62559173e-03, 1.66124009e-03, 1.37747767e-03,
            1.72487297e-03, 1.47850839e-03, 1.87169956e-03, 1.25858656e-03, 3.32139605e-03,
            2.28515517e-03, 2.09618875e-03, 2.29027782e-03, 7.62579221e-04, 3.16587564e-03,
            4.97021113e-03, 1.01581101e+00, 2.25492780e-03, 2.70841800e-03, 1.42026503e-04
        ], [
            1.87865618e-03, 7.18183196e-04, 1.56278184e-03, 1.42005817e-03, 2.14848559e-03,
            1.61180212e-03, 1.34379203e-03, 9.40226323e-04, 1.32524061e-03, 1.25294638e-03,
            1.64440516e-03, 1.39651452e-03, 1.89258080e-03, 1.30759486e-03, 1.56561692e-03,
            1.26745670e-03, 9.30223751e-04, 1.21391420e-03, 1.08280514e-03, 8.20729529e-04,
            7.70232774e-04, 1.54716184e-03, 1.11694357e-03, 9.09569749e-04, 1.14321497e-03,
            1.11206404e-03, 9.97411693e-04, 1.07216194e-03, 3.50261538e-04, 1.06188072e-03,
            6.40060673e-03, 2.04444695e-03, 1.03405401e+00, 2.00725565e-03, 1.29438844e-04
        ], [
            9.82260511e-03, 1.11820311e-02, 1.75029446e-02, 1.82427079e-02, 2.06326524e-02,
            1.73533760e-02, 2.84912776e-02, 1.46734451e-02, 2.05842504e-02, 1.84723780e-02,
            1.80314861e-02, 1.82445933e-02, 1.69084066e-02, 1.49857327e-02, 1.84607516e-02,
            1.62981812e-02, 1.40307061e-02, 1.85469284e-02, 1.65811931e-02, 1.53954699e-02,
            1.61082878e-02, 2.50761302e-02, 2.35687384e-02, 1.59034942e-02, 2.03959238e-02,
            2.81213476e-02, 2.71601903e-02, 2.28669812e-02, 1.22277319e-02, 3.62882074e-02,
            2.77384304e-02, 1.88193883e-02, 2.37640694e-02, 1.09270489e+00, 2.59937601e-03
        ], [
            4.04163728e-05, 4.53579620e-05, 7.21476339e-05, 1.81893494e-04, 9.51072002e-05,
            8.24970967e-05, 8.48368538e-05, 6.65472371e-05, 6.70178312e-05, 7.07291008e-05,
            6.88593507e-05, 6.28055814e-05, 8.04250093e-05, 5.72138987e-05, 1.12985835e-04,
            6.15178390e-05, 3.95943034e-05, 1.03712344e-04, 7.12278202e-05, 4.37928622e-05,
            7.12318026e-05, 9.64611358e-05, 1.63071438e-04, 4.44092752e-05, 6.18283338e-05,
            6.23793650e-05, 6.74247531e-05, 1.21016152e-04, 2.35664874e-04, 1.04120345e-04,
            1.05818892e-04, 6.80542736e-05, 9.83366199e-05, 1.52442597e-04, 1.00007983e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 3
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_2009():
    """
    Leontief matrix 2009



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.16963337e+00, 4.46228395e-03, 3.79008126e-01, 1.14867590e-01, 1.56998544e-01,
            2.27894740e-01, 4.14547398e-02, 9.57824267e-03, 2.53612530e-02, 3.41504227e-02,
            1.25157962e-02, 1.00098975e-02, 1.01417261e-02, 9.40939862e-03, 1.12997795e-02,
            4.45508194e-02, 5.69120001e-03, 1.91333144e-02, 7.04078033e-03, 8.95905282e-03,
            8.37311766e-03, 1.10574755e-01, 9.11846284e-03, 1.06843084e-02, 1.06594976e-02,
            1.87913047e-02, 4.80596043e-03, 4.90944297e-03, 3.11206255e-03, 7.59117410e-03,
            1.05138959e-02, 1.09815029e-02, 1.30661855e-02, 1.32167933e-02, 3.29691110e-02
        ], [
            2.33323085e-02, 1.12290405e+00, 2.79416797e-02, 3.87241659e-02, 3.28951250e-02,
            3.76569067e-02, 3.35291866e-02, 5.03034646e-01, 9.49260436e-02, 5.33560577e-02,
            1.39509033e-01, 1.35126045e-01, 4.98773520e-02, 3.51675332e-02, 4.11731397e-02,
            3.96545621e-02, 1.64604507e-01, 7.25139135e-02, 1.84140978e-02, 1.11994472e-02,
            1.15555207e-02, 2.06327991e-02, 5.03590771e-02, 5.85598810e-02, 6.60981360e-02,
            3.70425171e-02, 1.16119497e-02, 7.56324060e-03, 9.40983308e-03, 1.28002277e-02,
            2.04280525e-02, 1.36885271e-02, 1.83735476e-02, 1.94256867e-02, 6.86362233e-02
        ], [
            9.54704553e-02, 4.43080396e-03, 1.23296788e+00, 3.11901994e-02, 1.83631263e-01,
            2.98669982e-02, 1.58604724e-02, 8.57648438e-03, 2.78673612e-02, 2.02135237e-02,
            1.23050846e-02, 1.12873953e-02, 1.14192374e-02, 1.06651248e-02, 1.15496790e-02,
            1.59248838e-02, 5.98342902e-03, 1.09592600e-02, 7.34670651e-03, 1.04476688e-02,
            9.77526028e-03, 1.94698504e-01, 8.70848800e-03, 1.47329003e-02, 1.48487073e-02,
            1.27905946e-02, 6.11696901e-03, 6.23467326e-03, 2.63854686e-03, 8.86392432e-03,
            1.39403840e-02, 1.42521770e-02, 2.02902964e-02, 1.63185866e-02, 7.26209850e-03
        ], [
            6.03027012e-03, 4.00667670e-03, 9.04268739e-03, 1.65517472e+00, 1.50809867e-01,
            1.34113043e-02, 2.00652967e-02, 5.72045014e-03, 1.34217874e-02, 3.92542068e-02,
            1.50148732e-02, 1.19762270e-02, 1.30860236e-02, 1.19547506e-02, 1.87185596e-02,
            5.34267141e-02, 5.33892518e-03, 1.11782339e-02, 7.06534071e-03, 6.78937044e-03,
            6.68157444e-03, 8.62198940e-03, 7.96756239e-03, 1.07517436e-02, 9.39491430e-03,
            8.08298194e-03, 3.70055983e-03, 3.09595897e-03, 2.14950874e-03, 5.89614633e-03,
            7.65735732e-03, 4.16703886e-03, 8.91339712e-03, 1.11540102e-02, 7.66576485e-04
        ], [
            5.81988712e-04, 3.46949317e-04, 8.17427954e-04, 3.10345555e-02, 1.31430340e+00,
            1.72743067e-03, 2.04652840e-03, 5.17744759e-04, 1.11736731e-03, 3.10697396e-03,
            1.15212110e-03, 1.00356224e-03, 1.21716032e-03, 1.40834108e-03, 5.75728622e-03,
            9.95929560e-03, 4.85066620e-04, 9.03958627e-04, 1.22095290e-03, 6.79012236e-04,
            5.97127273e-04, 6.30364687e-04, 9.92262204e-04, 8.71654878e-04, 9.89390126e-04,
            8.34886674e-04, 5.31636127e-04, 2.86385863e-04, 1.67344885e-04, 5.09943132e-04,
            1.31895450e-03, 4.36383486e-04, 7.89533403e-04, 1.09775967e-03, 7.43118023e-05
        ], [
            4.76856977e-03, 4.23927436e-03, 7.06960308e-03, 8.30952936e-03, 8.69530213e-03,
            1.37961253e+00, 2.75923769e-02, 4.39110456e-03, 6.56798420e-03, 1.00272710e-02,
            1.86355039e-02, 1.38032813e-02, 1.18000667e-02, 8.71230292e-03, 1.19118981e-02,
            1.13044914e-01, 4.55653608e-03, 4.72606725e-02, 4.85256643e-03, 3.47756399e-03,
            3.62590586e-03, 6.61225437e-03, 5.39718526e-03, 4.26475973e-03, 4.39173676e-03,
            7.08523064e-03, 3.26616991e-03, 2.57001658e-03, 4.36931836e-03, 4.25768768e-03,
            6.40311566e-03, 6.88883109e-03, 4.98164070e-03, 7.44577954e-03, 7.55936409e-04
        ], [
            1.40488245e-02, 9.48398008e-03, 5.27230104e-02, 4.17284445e-02, 4.90884474e-02,
            4.09009207e-02, 1.34905261e+00, 1.51921536e-02, 3.91931657e-02, 4.57832518e-02,
            4.87552527e-02, 2.39254390e-02, 2.76856911e-02, 3.24746577e-02, 2.68621738e-02,
            4.50142305e-02, 1.37747425e-02, 2.45969450e-02, 2.14235777e-02, 2.22203366e-02,
            2.12497656e-02, 2.69582118e-02, 2.10694440e-02, 1.82407207e-02, 2.13793058e-02,
            3.15259480e-02, 2.12543709e-02, 2.79299458e-02, 8.26596242e-03, 3.85367550e-02,
            3.37346706e-02, 3.15626233e-02, 2.27604299e-02, 4.34972618e-02, 1.90257036e-03
        ], [
            2.60444932e-02, 1.52729421e-02, 2.30514435e-02, 2.94198712e-02, 2.65795080e-02,
            2.65576290e-02, 2.21460551e-02, 1.13486963e+00, 7.70261485e-02, 4.25080622e-02,
            4.43372116e-02, 3.87148260e-02, 2.35860513e-02, 1.91568928e-02, 2.15215953e-02,
            2.22562860e-02, 4.25253533e-02, 3.10692960e-02, 1.58609368e-02, 1.27509410e-02,
            1.02501817e-02, 1.50306045e-02, 7.58948548e-02, 1.00597658e-01, 1.17703440e-01,
            4.60556617e-02, 1.03088856e-02, 6.91077598e-03, 5.08017144e-03, 1.26102985e-02,
            1.81538694e-02, 9.58333172e-03, 1.36684774e-02, 1.51542677e-02, 1.99668439e-03
        ], [
            6.83434374e-02, 2.22653463e-02, 5.55455729e-02, 1.59033689e-01, 1.18277898e-01,
            9.77366080e-02, 8.94349368e-02, 4.87433089e-02, 1.34890775e+00, 3.75269974e-01,
            8.65163196e-02, 5.34943171e-02, 5.50429148e-02, 7.36023548e-02, 6.64018903e-02,
            8.16775933e-02, 2.42893246e-02, 5.13113614e-02, 2.80356473e-02, 1.29809917e-02,
            1.18514698e-02, 2.64888489e-02, 2.39036521e-02, 2.51108771e-02, 2.54254588e-02,
            2.52837834e-02, 1.28433168e-02, 8.75874521e-03, 7.73934421e-03, 1.87993696e-02,
            2.43292131e-02, 1.45906865e-02, 9.44452513e-02, 2.94311634e-02, 4.11235800e-03
        ], [
            1.45912395e-02, 1.19182539e-02, 3.91094368e-02, 3.63191729e-02, 8.02176943e-02,
            2.53301915e-02, 3.82449732e-02, 1.95261293e-02, 3.92999477e-02, 1.21838656e+00,
            3.12190849e-02, 2.59784139e-02, 5.25904714e-02, 6.09964742e-02, 7.91299631e-02,
            5.93634724e-02, 1.24469226e-02, 3.80259554e-02, 3.07828355e-02, 1.02184607e-02,
            1.03087635e-02, 1.65454739e-02, 2.50219690e-02, 1.62838781e-02, 1.82966185e-02,
            1.94531979e-02, 1.10243953e-02, 5.40767885e-03, 4.74472547e-03, 1.06910961e-02,
            1.22519457e-02, 7.20174898e-03, 1.46217516e-02, 1.44535275e-02, 1.70773548e-03
        ], [
            5.02100703e-03, 7.61262606e-03, 1.11035866e-02, 8.82813479e-03, 8.50432905e-03,
            1.41613913e-02, 6.55192980e-03, 8.96730004e-03, 1.28288749e-02, 1.38680851e-02,
            1.16893556e+00, 2.42140167e-02, 1.74835768e-02, 2.48113902e-02, 1.96902301e-02,
            1.66862845e-02, 7.88764383e-03, 1.17803725e-01, 8.73437694e-03, 3.31945985e-03,
            3.51812047e-03, 8.03430225e-03, 6.58175211e-03, 4.93314732e-03, 5.80059289e-03,
            7.11261743e-03, 5.14873479e-03, 2.64122820e-03, 6.46773013e-03, 4.39123597e-03,
            6.69667984e-03, 5.20040208e-03, 6.19467766e-03, 6.31175336e-03, 8.58909772e-04
        ], [
            2.34896201e-02, 4.51662933e-02, 4.21226636e-02, 4.30213158e-02, 5.04080486e-02,
            7.07920172e-02, 4.55492744e-02, 4.69610459e-02, 5.26374266e-02, 7.85162975e-02,
            1.01705305e-01, 1.55680092e+00, 3.48761600e-01, 2.04617443e-01, 2.61647277e-01,
            1.60726361e-01, 4.05247978e-02, 2.00049817e-01, 6.33336354e-02, 1.69976702e-02,
            1.62230125e-02, 2.62233293e-02, 4.04198035e-02, 4.11723502e-02, 4.31734470e-02,
            4.31771165e-02, 2.56963623e-02, 1.17843577e-02, 1.52149622e-02, 2.39095847e-02,
            3.12327896e-02, 1.71281957e-02, 2.39956290e-02, 2.86855290e-02, 4.83613537e-03
        ], [
            1.58543465e-02, 2.51178207e-02, 1.76640273e-02, 3.17300113e-02, 2.68576510e-02,
            2.99085125e-02, 2.26479634e-02, 2.73391696e-02, 2.50210630e-02, 3.53561696e-02,
            4.75146349e-02, 6.16704862e-02, 1.22661586e+00, 5.01016181e-02, 9.65486270e-02,
            3.91640175e-02, 2.14665140e-02, 4.41860607e-02, 3.94631089e-02, 7.95135443e-03,
            7.39244445e-03, 1.10906347e-02, 1.88008922e-02, 2.67252283e-02, 2.41966911e-02,
            2.14434550e-02, 9.84865714e-03, 5.31788596e-03, 5.17592984e-03, 1.01406381e-02,
            1.60404918e-02, 7.70163515e-03, 1.27411095e-02, 1.26080028e-02, 2.75698681e-03
        ], [
            2.18976141e-02, 3.48392049e-02, 3.62675437e-02, 5.29320301e-02, 4.63789133e-02,
            4.98234989e-02, 5.58994495e-02, 4.43119027e-02, 5.70651844e-02, 6.74859385e-02,
            6.51556297e-02, 8.89554918e-02, 2.71425788e-01, 1.73438282e+00, 2.33820532e-01,
            1.11459106e-01, 8.54080221e-02, 1.21932722e-01, 9.50568608e-02, 3.16408625e-02,
            3.04169385e-02, 3.43552203e-02, 5.05018106e-02, 4.44627573e-02, 5.32524343e-02,
            5.44550938e-02, 1.00182165e-01, 2.80129108e-02, 1.68796428e-02, 6.37730214e-02,
            6.30803065e-02, 3.94713330e-02, 7.01039086e-02, 5.80609010e-02, 5.77830879e-03
        ], [
            1.35267293e-02, 1.33475893e-02, 1.65038913e-02, 2.01357961e-02, 1.91312669e-02,
            2.21325430e-02, 2.01625483e-02, 1.71163297e-02, 1.65415886e-02, 2.25949577e-02,
            2.63739376e-02, 3.16501446e-02, 4.58365433e-02, 2.27657754e-02, 1.47756597e+00,
            2.94174628e-02, 1.56543769e-02, 2.35379298e-02, 1.10562773e-01, 1.39654252e-02,
            1.10224478e-02, 1.21553743e-02, 7.00023676e-02, 8.41495695e-02, 1.00124241e-01,
            3.73851376e-02, 1.16876758e-02, 7.26312404e-03, 4.76839005e-03, 1.39181029e-02,
            3.47901988e-02, 9.32678649e-03, 9.09482932e-03, 2.13489501e-02, 1.91821571e-03
        ], [
            2.26330816e-03, 2.37720520e-03, 3.89604843e-03, 1.22104793e-02, 8.45104168e-03,
            8.15251727e-03, 8.99617595e-03, 3.71697893e-03, 5.19475480e-03, 7.35546125e-03,
            8.05601959e-03, 1.64216687e-02, 1.09200422e-02, 7.85976809e-03, 1.40355452e-02,
            1.07590234e+00, 4.43436135e-03, 9.39349569e-03, 4.36832925e-03, 2.44915517e-03,
            2.77716254e-03, 4.59335619e-03, 7.02267954e-03, 3.50386072e-03, 5.52713890e-03,
            4.44736085e-03, 3.46931172e-03, 2.47394271e-03, 2.26388881e-03, 3.81976898e-03,
            5.46246535e-03, 5.19610161e-03, 4.95983030e-03, 7.64637200e-03, 2.33893311e-03
        ], [
            2.54675861e-02, 4.40223143e-02, 3.93177135e-02, 5.42804827e-02, 3.76679628e-02,
            4.85150021e-02, 5.28821840e-02, 6.35081554e-02, 5.95792996e-02, 5.81505764e-02,
            8.26850941e-02, 7.36933204e-02, 4.58083344e-02, 3.61309904e-02, 4.18183974e-02,
            3.71594126e-02, 1.18599004e+00, 3.83304941e-02, 2.59445599e-02, 1.53648833e-02,
            2.23900300e-02, 4.25970740e-02, 3.55678091e-02, 2.28672308e-02, 2.83724773e-02,
            3.89993085e-02, 2.08281863e-02, 1.31557960e-02, 1.48846134e-02, 1.73719907e-02,
            2.83276494e-02, 3.20880950e-02, 2.63912341e-02, 3.12499155e-02, 4.06415621e-03
        ], [
            7.68585660e-03, 1.07334735e-02, 9.82089286e-03, 1.08055137e-02, 9.17400215e-03,
            1.09828244e-02, 1.08085854e-02, 1.48454268e-02, 1.08137291e-02, 1.04924950e-02,
            1.48868561e-02, 1.25824233e-02, 1.04622916e-02, 8.60891760e-03, 1.00556201e-02,
            1.00523707e-02, 2.45629876e-02, 1.06351099e+00, 1.09685230e-02, 7.64584656e-03,
            9.78443152e-03, 1.30493818e-02, 1.67873695e-02, 9.52878729e-03, 1.17120987e-02,
            2.18499800e-02, 1.60535736e-02, 1.11264853e-02, 3.71520662e-02, 1.04889025e-02,
            2.49395574e-02, 1.64105492e-02, 1.36527509e-02, 1.63451071e-02, 1.67078151e-03
        ], [
            8.67979367e-03, 5.42145712e-03, 1.34013552e-02, 1.16762128e-02, 1.34956573e-02,
            1.25881351e-02, 1.08748740e-02, 9.57211823e-03, 1.14949938e-02, 1.29903971e-02,
            1.22845342e-02, 1.21683465e-02, 1.11678671e-02, 9.36263881e-03, 1.30188961e-02,
            1.24576864e-02, 1.10641892e-02, 1.27408577e-02, 1.01529927e+00, 6.33713773e-03,
            6.36272146e-03, 9.93263673e-03, 1.89744772e-02, 7.20588979e-03, 1.06753090e-02,
            1.34891303e-02, 6.13521932e-03, 5.79679393e-03, 3.67817593e-03, 8.98732143e-03,
            7.67096797e-03, 4.93567506e-03, 7.22901624e-03, 9.62517608e-03, 7.81072769e-04
        ], [
            5.89052269e-02, 3.08696396e-02, 1.12508312e-01, 9.66351411e-02, 1.10050024e-01,
            1.01573342e-01, 8.84820922e-02, 9.40516304e-02, 8.81253854e-02, 9.70596637e-02,
            7.79369142e-02, 8.83757938e-02, 9.17196303e-02, 8.51705802e-02, 1.04955436e-01,
            8.44020984e-02, 4.28084791e-02, 8.23774609e-02, 5.60175566e-02, 1.03505736e+00,
            2.52223535e-02, 7.83014194e-02, 4.88805489e-02, 5.01469636e-02, 5.69043972e-02,
            5.08759102e-02, 2.83120607e-02, 1.68853857e-02, 1.25274973e-02, 3.13777102e-02,
            3.89010726e-02, 2.46936709e-02, 4.74497851e-02, 3.81808557e-02, 4.67777968e-03
        ], [
            2.79925001e-02, 1.29125961e-02, 4.99018337e-02, 4.73293807e-02, 5.49613604e-02,
            4.04895076e-02, 3.61421206e-02, 3.38238830e-02, 3.90521736e-02, 4.22607928e-02,
            3.42766051e-02, 3.72294912e-02, 3.78374027e-02, 3.31892438e-02, 4.60179788e-02,
            4.41304200e-02, 1.80450252e-02, 4.28339768e-02, 2.64017832e-02, 1.18145309e-02,
            1.01275000e+00, 3.82057501e-02, 2.63254422e-02, 2.05319850e-02, 2.48520568e-02,
            2.29390776e-02, 1.56660130e-02, 1.14940308e-02, 7.18770132e-03, 1.67184403e-02,
            1.74414468e-02, 1.19844595e-02, 2.21954055e-02, 2.09284902e-02, 2.22445568e-03
        ], [
            5.70650109e-03, 6.77956833e-03, 1.16072781e-02, 1.33490376e-02, 1.39443161e-02,
            1.18964785e-02, 1.33354985e-02, 8.33194307e-03, 1.18584859e-02, 1.32142063e-02,
            1.35034570e-02, 1.29067394e-02, 1.39892555e-02, 1.24941519e-02, 1.23970119e-02,
            1.12746528e-02, 8.33772357e-03, 1.37045483e-02, 1.03619106e-02, 1.16529529e-02,
            9.68779956e-03, 1.01619168e+00, 1.45486649e-02, 1.32391030e-02, 2.52344709e-02,
            3.21019443e-02, 1.20198250e-02, 1.73563105e-02, 5.25363927e-03, 1.89665474e-02,
            1.63646579e-02, 1.29788556e-02, 1.33155170e-02, 1.80715869e-02, 9.58083346e-04
        ], [
            2.62282936e-02, 2.66870585e-02, 4.79472081e-02, 4.97749568e-02, 4.92862380e-02,
            4.94368192e-02, 4.08273723e-02, 7.18963844e-02, 4.24256819e-02, 4.36694411e-02,
            5.92815646e-02, 4.45914061e-02, 3.74743108e-02, 3.02055194e-02, 3.86014999e-02,
            4.22170044e-02, 3.51955408e-02, 4.50852553e-02, 2.94783827e-02, 3.04816039e-02,
            2.02739042e-02, 2.62013604e-02, 1.06514311e+00, 3.28958597e-02, 3.42761632e-02,
            7.91072947e-02, 1.76748790e-02, 1.30775429e-02, 6.74810532e-03, 1.54703263e-02,
            2.09456019e-02, 1.69506921e-02, 1.79594368e-02, 2.11295921e-02, 3.17921664e-03
        ], [
            5.75895785e-03, 9.89023088e-03, 1.17742788e-02, 1.34488776e-02, 1.26423085e-02,
            1.61010410e-02, 9.37178036e-03, 1.12031521e-02, 9.67756828e-03, 1.07465072e-02,
            1.78034018e-02, 1.58516688e-02, 1.16154845e-02, 8.88773415e-03, 1.11103198e-02,
            9.50265266e-03, 7.19455593e-03, 1.04554831e-02, 5.22037902e-03, 9.25118982e-03,
            5.15643132e-03, 5.82398167e-03, 7.02810794e-03, 1.19005824e+00, 8.01345625e-03,
            1.04452910e-02, 3.05984347e-03, 2.22535669e-03, 3.23220756e-03, 3.57794647e-03,
            3.81248011e-03, 2.62389363e-03, 3.22576006e-03, 4.02667491e-03, 8.95327306e-04
        ], [
            2.36626683e-03, 2.69605067e-03, 4.28903003e-03, 5.14690250e-03, 5.89923394e-03,
            4.84614235e-03, 5.23361256e-03, 3.55504652e-03, 4.55879943e-03, 4.78872435e-03,
            4.50079248e-03, 4.10186901e-03, 5.21466850e-03, 4.88494179e-03, 4.98531580e-03,
            4.20987150e-03, 2.69968556e-03, 4.22725005e-03, 4.17417100e-03, 4.64983248e-03,
            3.19016968e-03, 3.75700588e-03, 5.17741899e-03, 9.06269271e-03, 1.03620074e+00,
            1.66806414e-02, 5.71554194e-03, 5.40446586e-03, 1.52547813e-03, 6.02679900e-03,
            5.73094525e-03, 4.94531621e-03, 3.06271513e-03, 6.80079906e-03, 3.54278232e-04
        ], [
            1.18244335e-02, 1.12998861e-02, 2.16472987e-02, 1.93123600e-02, 2.02436692e-02,
            2.29319052e-02, 2.34971565e-02, 2.47225377e-02, 1.81616639e-02, 2.00301860e-02,
            2.32588164e-02, 1.99602221e-02, 1.79887468e-02, 1.53791159e-02, 1.92818690e-02,
            1.71743668e-02, 1.29137787e-02, 2.85627540e-02, 2.33938582e-02, 2.81225607e-02,
            1.94626802e-02, 1.80749817e-02, 5.15231610e-02, 1.58223170e-01, 1.27208473e-01,
            1.12087013e+00, 1.04587366e-02, 9.69141137e-03, 4.82120888e-03, 1.25013626e-02,
            1.25358841e-02, 8.10159466e-03, 1.04280245e-02, 1.48735278e-02, 1.34894760e-03
        ], [
            1.58479723e-02, 1.37145736e-02, 2.59548708e-02, 3.11883413e-02, 2.94544708e-02,
            2.77979690e-02, 3.58633267e-02, 2.22311352e-02, 2.89688460e-02, 2.76739029e-02,
            2.71585279e-02, 3.03377191e-02, 2.88377929e-02, 3.17440868e-02, 2.92178799e-02,
            3.88089979e-02, 2.24786368e-02, 3.48357110e-02, 3.33818104e-02, 3.12533430e-02,
            3.54668699e-02, 3.41875016e-02, 4.07901848e-02, 4.38772867e-02, 4.73304090e-02,
            4.80134491e-02, 1.14686439e+00, 5.33078531e-02, 1.43817519e-02, 4.51967769e-02,
            4.07842951e-02, 2.29329132e-02, 2.99333024e-02, 4.21987818e-02, 3.07293048e-03
        ], [
            4.78231186e-02, 4.43270368e-02, 6.65282418e-02, 8.48291285e-02, 7.46091353e-02,
            6.72922498e-02, 7.39424529e-02, 6.16039512e-02, 6.66422952e-02, 6.70172633e-02,
            7.48109112e-02, 7.02413124e-02, 6.71461886e-02, 6.24836963e-02, 7.12211677e-02,
            8.11380100e-02, 6.15456791e-02, 7.12083219e-02, 5.62464861e-02, 5.79554016e-02,
            6.54846665e-02, 7.00707673e-02, 9.55785921e-02, 6.83357813e-02, 8.12679131e-02,
            8.62160333e-02, 4.80000721e-02, 1.28986318e+00, 1.17705305e-01, 7.88349679e-02,
            5.80156975e-02, 4.03274363e-02, 6.04487229e-02, 8.91736963e-02, 1.81216218e-02
        ], [
            1.11412040e-02, 1.23183436e-02, 1.98000109e-02, 2.18981855e-02, 2.36511406e-02,
            2.11289040e-02, 2.61854873e-02, 1.60076982e-02, 1.88722020e-02, 2.12768701e-02,
            1.90968285e-02, 1.97138496e-02, 2.04768365e-02, 1.84947984e-02, 2.24744366e-02,
            2.16783627e-02, 1.48507465e-02, 2.63405501e-02, 3.33090157e-02, 2.76519331e-02,
            5.10395373e-02, 4.22463572e-02, 2.18926176e-02, 2.77166124e-02, 2.48083844e-02,
            3.79777434e-02, 2.32284525e-02, 3.75010638e-02, 1.04051962e+00, 3.54796729e-02,
            2.56394427e-02, 2.90268776e-02, 4.59201815e-02, 4.04940085e-02, 1.76245333e-03
        ], [
            5.54176205e-02, 5.64759885e-02, 1.13962352e-01, 1.03905657e-01, 1.01723667e-01,
            9.91796353e-02, 1.53269080e-01, 8.49449679e-02, 1.51940061e-01, 1.29346974e-01,
            1.05332997e-01, 1.05861665e-01, 1.22930926e-01, 1.31003940e-01, 1.40841869e-01,
            1.02271282e-01, 8.70155275e-02, 1.33647262e-01, 1.11845782e-01, 9.94897809e-02,
            1.05139685e-01, 1.13506570e-01, 1.07237251e-01, 9.13627058e-02, 1.47351098e-01,
            1.40286067e-01, 1.09482166e-01, 1.83913015e-01, 6.39076130e-02, 1.23275783e+00,
            1.48857937e-01, 7.43825370e-02, 1.28115763e-01, 1.54925392e-01, 1.08489795e-02
        ], [
            3.23523654e-03, 3.45426334e-03, 5.40833640e-03, 4.13897846e-03, 4.20288681e-03,
            6.82183484e-03, 7.95460466e-03, 4.40106937e-03, 6.28859511e-03, 6.03780910e-03,
            6.15318824e-03, 7.67880505e-03, 5.11934618e-03, 4.36175865e-03, 5.20854581e-03,
            5.51193546e-03, 7.05764764e-03, 5.06826070e-03, 4.83499329e-03, 3.03361287e-03,
            4.46714878e-03, 6.55838283e-03, 6.25142506e-03, 8.62766096e-03, 1.38092709e-02,
            6.80914244e-03, 4.51575596e-03, 3.78174398e-03, 4.44221862e-03, 5.33459992e-03,
            1.00994872e+00, 6.24105502e-03, 6.86350125e-03, 1.16981417e-02, 4.15663021e-04
        ], [
            1.21743074e-03, 9.34742566e-04, 1.58361158e-03, 1.54843739e-03, 1.60326517e-03,
            1.48855462e-03, 1.75969712e-03, 1.47441187e-03, 2.08197246e-03, 1.87312196e-03,
            1.59684025e-03, 1.67988219e-03, 2.04028357e-03, 1.83966383e-03, 2.16581973e-03,
            1.46627884e-03, 1.43378052e-03, 1.65218452e-03, 1.64085990e-03, 1.35395802e-03,
            1.88057622e-03, 1.50564142e-03, 1.88428922e-03, 1.31310538e-03, 3.36419346e-03,
            2.23112624e-03, 1.98615832e-03, 2.26181748e-03, 7.46788744e-04, 3.24220858e-03,
            5.22520350e-03, 1.01687110e+00, 2.17215085e-03, 2.65060564e-03, 1.48492086e-04
        ], [
            1.91052355e-03, 8.89385435e-04, 1.63635352e-03, 1.48391258e-03, 2.32075697e-03,
            1.75681394e-03, 1.44083142e-03, 1.00449903e-03, 1.39554056e-03, 1.34395662e-03,
            1.84950593e-03, 1.63444235e-03, 2.24048280e-03, 1.47397018e-03, 1.78133332e-03,
            1.38277643e-03, 1.02380698e-03, 1.31545211e-03, 1.15058399e-03, 8.31160685e-04,
            8.02097169e-04, 1.62756417e-03, 1.22300586e-03, 1.01600444e-03, 1.21062445e-03,
            1.16089525e-03, 9.85578236e-04, 1.07903480e-03, 3.58364795e-04, 1.17034440e-03,
            6.62092261e-03, 2.13085659e-03, 1.03506984e+00, 2.18969307e-03, 1.35631287e-04
        ], [
            1.00686668e-02, 1.17083099e-02, 1.82070873e-02, 1.90789700e-02, 2.10315496e-02,
            1.79461276e-02, 2.89295045e-02, 1.43021953e-02, 2.08949911e-02, 1.94175718e-02,
            1.88293586e-02, 1.89835791e-02, 1.78045294e-02, 1.60649932e-02, 1.98177794e-02,
            1.69705296e-02, 1.50259447e-02, 1.92787977e-02, 1.66344933e-02, 1.53583907e-02,
            1.67149153e-02, 2.59255856e-02, 2.56407103e-02, 1.71407847e-02, 2.15587971e-02,
            2.99532200e-02, 2.57547874e-02, 2.29169789e-02, 1.26107092e-02, 3.97369938e-02,
            2.83759363e-02, 1.87698052e-02, 2.42013302e-02, 1.09347280e+00, 2.85986433e-03
        ], [
            4.04895638e-05, 4.71993336e-05, 7.20084273e-05, 2.08043750e-04, 1.03358389e-04,
            8.35898297e-05, 7.91585223e-05, 6.00524519e-05, 6.30610739e-05, 7.08493614e-05,
            6.74483163e-05, 6.16791325e-05, 8.58808464e-05, 5.89878193e-05, 1.38261001e-04,
            6.57925250e-05, 4.07162921e-05, 9.45149603e-05, 6.49793764e-05, 3.61462622e-05,
            6.47226923e-05, 9.21098548e-05, 1.78163731e-04, 4.64990597e-05, 6.14599494e-05,
            6.33246009e-05, 5.96884892e-05, 1.03204885e-04, 1.86698998e-04, 9.96143461e-05,
            9.97043849e-05, 6.46479979e-05, 8.74744069e-05, 1.40406788e-04, 1.00009630e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 3
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('run')
def leontief_matrix_1995():
    """
    Leontief matrix 1995



    constant


    """

    ret =  xr.DataArray(
        data=[[
            1.18047535e+00, 8.35773041e-03, 3.73351437e-01, 7.57840203e-02, 9.28131090e-02,
            2.10652131e-01, 3.02072845e-02, 1.06100688e-02, 2.17755101e-02, 3.03903773e-02,
            1.05098394e-02, 7.87583112e-03, 8.43834730e-03, 8.66736477e-03, 9.39775549e-03,
            4.12123614e-02, 5.47831896e-03, 1.79706275e-02, 6.52210075e-03, 9.48589583e-03,
            8.75995967e-03, 1.09648210e-01, 6.59886817e-03, 1.08023703e-02, 9.67370345e-03,
            9.79247304e-03, 4.44970881e-03, 4.72262254e-03, 3.37763614e-03, 7.45688147e-03,
            1.00516862e-02, 8.57633240e-03, 1.35758412e-02, 1.08985522e-02, 1.98424787e-02
        ], [
            2.32033833e-02, 1.11101577e+00, 2.48720641e-02, 2.69139312e-02, 2.30183022e-02,
            2.82449786e-02, 2.69429477e-02, 5.06060749e-01, 8.09541836e-02, 4.05753956e-02,
            1.27978039e-01, 9.60459711e-02, 3.58116965e-02, 2.91127225e-02, 3.17185000e-02,
            3.63251549e-02, 1.39403729e-01, 5.69431742e-02, 1.61973227e-02, 1.33764672e-02,
            1.14427609e-02, 1.88286435e-02, 4.45460968e-02, 4.32047749e-02, 4.53996269e-02,
            2.46728331e-02, 1.00334744e-02, 7.10996504e-03, 9.21370822e-03, 1.09771950e-02,
            1.64553143e-02, 1.17785460e-02, 1.57490392e-02, 1.92213429e-02, 7.39074825e-02
        ], [
            7.59279909e-02, 5.17925041e-03, 1.18721723e+00, 1.59690530e-02, 1.32222369e-01,
            2.03036923e-02, 1.22640201e-02, 6.75013650e-03, 2.17360933e-02, 1.41384365e-02,
            8.63369745e-03, 7.73498807e-03, 8.33575902e-03, 8.91981203e-03, 8.19588943e-03,
            1.13632058e-02, 5.30536893e-03, 7.97099629e-03, 7.26404524e-03, 1.11672969e-02,
            9.47703403e-03, 2.03503804e-01, 7.11989254e-03, 1.37394763e-02, 1.49823673e-02,
            1.10335521e-02, 5.33528661e-03, 6.05800699e-03, 2.36238083e-03, 9.13621706e-03,
            1.47701047e-02, 1.37615076e-02, 2.32589480e-02, 1.22054279e-02, 3.14376112e-03
        ], [
            6.65890399e-03, 4.91987921e-03, 7.57248636e-03, 1.39958000e+00, 9.51712374e-02,
            1.42176333e-02, 1.71936313e-02, 4.69153485e-03, 1.14910482e-02, 3.43465419e-02,
            1.15628574e-02, 7.37726747e-03, 9.07702632e-03, 9.22311364e-03, 1.79263281e-02,
            4.28891046e-02, 3.61234135e-03, 8.83776612e-03, 6.91965532e-03, 6.97426579e-03,
            7.39731609e-03, 8.33457098e-03, 5.60561818e-03, 8.23482377e-03, 5.63581937e-03,
            5.76574765e-03, 6.50612123e-03, 3.05171151e-03, 1.62721955e-03, 4.55633822e-03,
            7.65334885e-03, 2.65051337e-03, 8.02117270e-03, 1.03072999e-02, 1.44681899e-03
        ], [
            6.71835288e-04, 7.47348868e-04, 9.01035830e-04, 1.14965709e-02, 1.23053574e+00,
            2.01564608e-03, 2.83203186e-03, 6.90238230e-04, 1.01444268e-03, 2.12113283e-03,
            1.37506967e-03, 1.22208018e-03, 1.29966123e-03, 1.29534693e-03, 2.88583488e-03,
            5.79207288e-03, 6.01634082e-04, 9.13729108e-04, 1.13535540e-03, 1.02783679e-03,
            1.00683039e-03, 7.71192618e-04, 6.89618068e-04, 8.24953271e-04, 7.19701316e-04,
            6.42070905e-04, 1.14535296e-03, 3.65053934e-04, 1.81426706e-04, 5.57517973e-04,
            1.52699540e-03, 3.87319554e-04, 5.81145210e-04, 1.34918300e-03, 1.21945173e-04
        ], [
            4.38264258e-03, 5.11262425e-03, 6.11220909e-03, 6.39489170e-03, 7.67764656e-03,
            1.23528213e+00, 2.14372507e-02, 4.33529849e-03, 6.17162127e-03, 7.95177676e-03,
            1.29115169e-02, 8.51718329e-03, 8.64667165e-03, 7.39617803e-03, 1.04492888e-02,
            9.74005388e-02, 4.52308929e-03, 4.88282410e-02, 4.42833512e-03, 4.84718213e-03,
            4.33948765e-03, 6.50606892e-03, 4.81652027e-03, 4.84014622e-03, 4.20416568e-03,
            1.00055623e-02, 3.37112626e-03, 3.01313318e-03, 5.70384009e-03, 3.74769212e-03,
            5.10400086e-03, 3.57641382e-03, 4.01027201e-03, 7.35033290e-03, 1.41004466e-03
        ], [
            1.65127169e-02, 1.57189890e-02, 5.30117402e-02, 3.73223823e-02, 4.38256755e-02,
            3.66334681e-02, 1.33412027e+00, 1.71332451e-02, 5.08281713e-02, 5.03737650e-02,
            4.80499098e-02, 2.68219660e-02, 3.47137739e-02, 4.58527001e-02, 3.18779672e-02,
            5.10637515e-02, 1.57298970e-02, 2.63761919e-02, 2.45740608e-02, 3.30634901e-02,
            3.03857309e-02, 3.38122611e-02, 2.10001227e-02, 2.45088099e-02, 2.88485097e-02,
            3.78446973e-02, 2.69607542e-02, 3.46877184e-02, 9.73673031e-03, 4.28915124e-02,
            3.29881339e-02, 2.67176202e-02, 2.63104929e-02, 4.73338758e-02, 2.99935309e-03
        ], [
            2.32256460e-02, 2.37360355e-02, 1.91493534e-02, 2.02848437e-02, 1.81132063e-02,
            2.01548954e-02, 1.67228855e-02, 1.06579011e+00, 4.53839581e-02, 2.78879958e-02,
            3.13842327e-02, 2.46784969e-02, 1.64609056e-02, 1.54612689e-02, 1.54646409e-02,
            1.81892546e-02, 3.27919098e-02, 2.54944245e-02, 1.21048410e-02, 1.32239102e-02,
            1.08162368e-02, 1.42238026e-02, 6.29302071e-02, 6.84205256e-02, 7.51086488e-02,
            2.63255171e-02, 9.15307296e-03, 5.72698292e-03, 4.35361112e-03, 1.02747756e-02,
            1.38264676e-02, 7.13843261e-03, 1.11198926e-02, 1.39530247e-02, 2.36903669e-03
        ], [
            7.35229860e-02, 3.14672398e-02, 5.28462251e-02, 1.45956070e-01, 9.53981415e-02,
            6.84319314e-02, 7.70950313e-02, 4.04670994e-02, 1.32164841e+00, 3.16636982e-01,
            6.13053822e-02, 4.57709351e-02, 4.81955274e-02, 5.92686305e-02, 5.87968297e-02,
            8.16230086e-02, 2.00775598e-02, 3.93106250e-02, 2.94764125e-02, 1.45822545e-02,
            1.25788549e-02, 2.58585172e-02, 1.87394501e-02, 1.85787492e-02, 1.75460061e-02,
            1.77226283e-02, 1.11184825e-02, 7.91777846e-03, 6.87913702e-03, 1.55580068e-02,
            2.17087374e-02, 1.07158084e-02, 9.32437345e-02, 2.44046729e-02, 4.28530450e-03
        ], [
            1.24586416e-02, 1.61585763e-02, 2.80355448e-02, 2.29526299e-02, 5.59543441e-02,
            2.28171757e-02, 2.80744266e-02, 1.33978427e-02, 3.42836103e-02, 1.13064836e+00,
            2.20207094e-02, 2.12115492e-02, 4.58433132e-02, 4.77458134e-02, 6.82295162e-02,
            5.84810651e-02, 9.87418787e-03, 3.13259709e-02, 3.43562879e-02, 1.07396353e-02,
            1.05447073e-02, 1.41085781e-02, 2.00204887e-02, 1.22617809e-02, 1.23619707e-02,
            1.44419000e-02, 9.41957159e-03, 4.48059889e-03, 4.04233206e-03, 7.90172892e-03,
            9.91136703e-03, 4.38431651e-03, 1.27770877e-02, 1.05065759e-02, 1.90332651e-03
        ], [
            5.89987586e-03, 1.01967328e-02, 1.19271779e-02, 7.48164942e-03, 7.53392815e-03,
            1.70051653e-02, 6.33597903e-03, 9.15387800e-03, 1.39277578e-02, 1.40068011e-02,
            1.12443736e+00, 1.94891876e-02, 1.51147088e-02, 2.32917893e-02, 1.84244680e-02,
            1.53824031e-02, 8.32225539e-03, 9.78689929e-02, 8.33557271e-03, 4.71012002e-03,
            3.88225801e-03, 8.74485654e-03, 6.11050478e-03, 5.36935478e-03, 4.96363075e-03,
            6.38905433e-03, 4.72809003e-03, 2.67544568e-03, 7.61271600e-03, 3.93493990e-03,
            6.22218851e-03, 4.47449815e-03, 6.06838446e-03, 6.52293824e-03, 1.30522811e-03
        ], [
            2.47480060e-02, 6.01033488e-02, 4.73096959e-02, 3.47772329e-02, 4.55979356e-02,
            7.24269356e-02, 3.93438165e-02, 4.73407141e-02, 5.81049464e-02, 7.01145337e-02,
            7.92725066e-02, 1.49859679e+00, 3.06590639e-01, 1.80189631e-01, 2.49982843e-01,
            1.99845226e-01, 3.57854948e-02, 1.60800482e-01, 7.22086765e-02, 2.18490371e-02,
            1.85367891e-02, 2.68104121e-02, 3.54555861e-02, 4.51250546e-02, 4.04694656e-02,
            3.03967575e-02, 2.12903741e-02, 1.04198389e-02, 1.53332280e-02, 1.82906551e-02,
            2.79383639e-02, 1.27001818e-02, 2.01712312e-02, 2.42584211e-02, 6.68350124e-03
        ], [
            1.44186714e-02, 3.01064456e-02, 1.47976589e-02, 1.78362871e-02, 1.64070077e-02,
            2.14510891e-02, 1.78228612e-02, 2.28234072e-02, 2.01313788e-02, 2.39595468e-02,
            2.55756522e-02, 3.53217517e-02, 1.14746210e+00, 3.40957090e-02, 5.91580274e-02,
            2.35292235e-02, 1.72693533e-02, 3.03364345e-02, 3.51915513e-02, 8.96686273e-03,
            7.79131306e-03, 9.41973195e-03, 1.48911606e-02, 1.50785060e-02, 1.35025676e-02,
            1.16072162e-02, 7.78064750e-03, 4.62275757e-03, 4.56563061e-03, 7.84059733e-03,
            1.26463212e-02, 5.61892561e-03, 7.46959562e-03, 9.44980107e-03, 2.76257009e-03
        ], [
            1.07363330e-02, 2.13214324e-02, 1.69728159e-02, 2.08729912e-02, 2.00846785e-02,
            2.26780093e-02, 2.74865879e-02, 1.98861755e-02, 2.81200568e-02, 3.35394824e-02,
            2.62642223e-02, 4.21084076e-02, 1.21143536e-01, 1.35091737e+00, 1.09810355e-01,
            4.36486506e-02, 2.79737125e-02, 5.06095835e-02, 5.11029437e-02, 2.01228599e-02,
            1.77349347e-02, 1.54238737e-02, 2.05445015e-02, 2.01946345e-02, 2.11265104e-02,
            1.97742798e-02, 4.42828685e-02, 1.15661737e-02, 7.24477585e-03, 2.44848690e-02,
            2.52746398e-02, 1.21732228e-02, 3.13788235e-02, 1.96334053e-02, 3.10494245e-03
        ], [
            1.15767859e-02, 1.54796478e-02, 1.28616637e-02, 1.33595598e-02, 1.40907462e-02,
            1.48811583e-02, 1.12499476e-02, 1.42920322e-02, 1.26809895e-02, 1.44941424e-02,
            1.55220593e-02, 1.91327968e-02, 3.14711606e-02, 1.79213531e-02, 1.41701749e+00,
            1.68166479e-02, 1.01857706e-02, 1.54761538e-02, 1.32429653e-01, 1.37335943e-02,
            1.08367410e-02, 8.88461870e-03, 4.75807029e-02, 6.46565059e-02, 7.59149603e-02,
            2.32539022e-02, 9.32190849e-03, 5.28031705e-03, 3.16279128e-03, 9.63198628e-03,
            3.07491068e-02, 4.64324608e-03, 6.79292377e-03, 1.09417587e-02, 1.68992774e-03
        ], [
            2.53174709e-03, 3.21243658e-03, 3.71884036e-03, 1.04179804e-02, 7.60398886e-03,
            1.02291750e-02, 6.44641061e-03, 3.16912543e-03, 4.95535956e-03, 6.04310846e-03,
            6.32211921e-03, 1.26766407e-02, 8.89611229e-03, 7.56101180e-03, 1.14131493e-02,
            1.03988970e+00, 2.81309430e-03, 9.09088082e-03, 5.04572537e-03, 3.38687599e-03,
            3.26024134e-03, 5.27376844e-03, 3.33854066e-03, 3.73725406e-03, 3.75416573e-03,
            3.27905143e-03, 2.43354397e-03, 2.69938356e-03, 2.06842144e-03, 4.32837062e-03,
            5.81087140e-03, 4.47455627e-03, 4.72673265e-03, 8.36820400e-03, 4.61519640e-03
        ], [
            2.16696159e-02, 3.76591070e-02, 3.14120212e-02, 4.12443542e-02, 2.95642728e-02,
            3.71984550e-02, 4.30204001e-02, 4.04112135e-02, 5.54606779e-02, 4.94281160e-02,
            6.18794875e-02, 5.68365584e-02, 3.53106068e-02, 3.26421596e-02, 3.53314867e-02,
            3.40556221e-02, 1.11332954e+00, 2.63269770e-02, 2.34727853e-02, 1.59609193e-02,
            2.25054244e-02, 3.90755293e-02, 2.88835466e-02, 1.98099439e-02, 1.86230525e-02,
            2.62402835e-02, 1.65574573e-02, 1.20169457e-02, 1.35358693e-02, 1.54145492e-02,
            2.47690251e-02, 2.54065765e-02, 2.47597101e-02, 2.85696674e-02, 3.61488031e-03
        ], [
            9.25820241e-03, 1.49834963e-02, 1.13694883e-02, 1.14349454e-02, 1.11341019e-02,
            1.38177504e-02, 1.31535963e-02, 1.46236039e-02, 1.45264143e-02, 1.36042652e-02,
            1.74682902e-02, 1.55596425e-02, 1.28308423e-02, 1.28555760e-02, 1.22339888e-02,
            1.27705344e-02, 3.62646263e-02, 1.04631173e+00, 1.17485932e-02, 1.11605056e-02,
            1.16614439e-02, 1.42853435e-02, 1.71486403e-02, 1.45861449e-02, 1.19256437e-02,
            2.32783499e-02, 1.85983184e-02, 1.21199160e-02, 4.93882078e-02, 1.21917886e-02,
            2.74621869e-02, 1.80514298e-02, 1.35539775e-02, 2.15987512e-02, 4.67811194e-03
        ], [
            9.91077198e-03, 9.01685545e-03, 1.44229760e-02, 1.31957221e-02, 1.64185424e-02,
            1.39168995e-02, 1.13311239e-02, 1.11760189e-02, 1.38605607e-02, 1.42065992e-02,
            1.46075674e-02, 1.43036186e-02, 1.29039705e-02, 1.27818102e-02, 1.38631330e-02,
            1.38594895e-02, 1.48895611e-02, 1.48725087e-02, 1.01289409e+00, 9.34051784e-03,
            8.31926309e-03, 1.12186620e-02, 2.36040630e-02, 1.00565687e-02, 8.75132380e-03,
            1.26403875e-02, 8.06497345e-03, 5.55172451e-03, 3.86848838e-03, 9.59395958e-03,
            7.58661324e-03, 5.36903031e-03, 7.30341016e-03, 1.07547199e-02, 1.02529661e-03
        ], [
            5.87852180e-02, 4.69900709e-02, 1.04976547e-01, 1.04740222e-01, 1.29071853e-01,
            1.00587825e-01, 8.97782188e-02, 8.06317122e-02, 9.16765519e-02, 9.68825796e-02,
            7.97043932e-02, 8.97572744e-02, 9.46941579e-02, 1.02906934e-01, 1.00633769e-01,
            9.52971690e-02, 4.41520129e-02, 8.13220045e-02, 6.52001127e-02, 1.04980900e+00,
            2.80195703e-02, 7.77285851e-02, 4.70645693e-02, 4.75444914e-02, 4.64827141e-02,
            4.50642412e-02, 2.38819288e-02, 1.77776023e-02, 1.23896368e-02, 2.81444800e-02,
            3.75358695e-02, 2.02667437e-02, 4.77078367e-02, 3.46368743e-02, 5.64516734e-03
        ], [
            3.01410017e-02, 2.04180345e-02, 5.23180891e-02, 5.10115137e-02, 6.98214932e-02,
            4.22108401e-02, 3.66363652e-02, 3.15411516e-02, 4.26606740e-02, 4.44927024e-02,
            3.59087598e-02, 3.63353630e-02, 3.73684501e-02, 3.80820524e-02, 4.37807893e-02,
            4.69202256e-02, 1.92785546e-02, 4.48021209e-02, 3.01121292e-02, 1.79542547e-02,
            1.01610384e+00, 4.30294637e-02, 2.68262267e-02, 2.35381214e-02, 2.21901017e-02,
            2.10575417e-02, 1.74924365e-02, 1.08890922e-02, 8.10180842e-03, 1.79305769e-02,
            1.68905513e-02, 1.11775748e-02, 2.16953076e-02, 2.26194353e-02, 3.10200555e-03
        ], [
            6.03207389e-03, 8.91043511e-03, 1.17638998e-02, 1.31104346e-02, 1.33737519e-02,
            1.30809519e-02, 1.45356877e-02, 1.00305739e-02, 1.45422860e-02, 1.50620115e-02,
            1.39847857e-02, 1.43873671e-02, 1.56723628e-02, 1.69321666e-02, 1.34770132e-02,
            1.34428248e-02, 1.06410030e-02, 1.34761124e-02, 1.30406786e-02, 1.54218800e-02,
            1.09698808e-02, 1.01609399e+00, 1.41600276e-02, 1.71114279e-02, 3.34802636e-02,
            2.78870022e-02, 9.68508791e-03, 1.60856829e-02, 5.25946314e-03, 1.90923909e-02,
            1.38110197e-02, 9.01210255e-03, 1.34717409e-02, 1.69959224e-02, 1.06877245e-03
        ], [
            2.98023675e-02, 4.35309050e-02, 4.64968821e-02, 4.29120806e-02, 4.90574045e-02,
            5.02898681e-02, 4.02632946e-02, 7.47113610e-02, 4.52835687e-02, 4.23901402e-02,
            5.99643291e-02, 4.46108183e-02, 3.52846429e-02, 3.23895151e-02, 3.81265662e-02,
            4.26906895e-02, 4.40532136e-02, 4.08798896e-02, 2.71553695e-02, 3.22691664e-02,
            2.01053628e-02, 2.82838270e-02, 1.05794868e+00, 4.01903249e-02, 3.16865337e-02,
            4.80134698e-02, 2.00469983e-02, 1.47837213e-02, 7.23707258e-03, 1.64763640e-02,
            2.21960007e-02, 1.62701773e-02, 1.91028116e-02, 2.26444666e-02, 4.72039546e-03
        ], [
            5.32189104e-03, 7.13662645e-03, 7.48851966e-03, 6.63054130e-03, 7.99909661e-03,
            1.14925474e-02, 4.55729774e-03, 1.11298441e-02, 7.08161543e-03, 6.03458009e-03,
            9.69447440e-03, 8.96043715e-03, 5.52877980e-03, 4.86889749e-03, 5.97532290e-03,
            6.02735762e-03, 5.15229318e-03, 6.35770385e-03, 3.25921614e-03, 4.85075855e-03,
            2.80587042e-03, 4.27854813e-03, 6.14169370e-03, 1.11234450e+00, 6.18380693e-03,
            8.03542012e-03, 2.94397894e-03, 1.80599436e-03, 9.36031928e-04, 2.32863333e-03,
            2.85920219e-03, 2.16866160e-03, 2.27027872e-03, 2.74444337e-03, 7.19744709e-04
        ], [
            2.69400994e-03, 4.37646958e-03, 4.66593917e-03, 5.21980684e-03, 6.92716352e-03,
            4.54787283e-03, 5.65122156e-03, 4.58982211e-03, 5.56150738e-03, 5.47096849e-03,
            4.60909813e-03, 4.90257427e-03, 5.14420075e-03, 5.89963472e-03, 5.69932112e-03,
            4.95640867e-03, 3.01527545e-03, 4.08251487e-03, 4.42605019e-03, 6.93136380e-03,
            3.51346637e-03, 3.76488781e-03, 4.89408551e-03, 1.04821572e-02, 1.03591307e+00,
            2.13947947e-02, 7.39629621e-03, 5.40276746e-03, 1.46098903e-03, 6.59086251e-03,
            6.34923681e-03, 3.30903269e-03, 3.26160327e-03, 6.37584872e-03, 4.53901847e-04
        ], [
            1.18733730e-02, 1.78727789e-02, 2.00247429e-02, 1.70534682e-02, 2.02929665e-02,
            2.06843066e-02, 1.98171149e-02, 2.31821546e-02, 1.92203979e-02, 1.87454496e-02,
            2.39338332e-02, 1.99276439e-02, 1.77257926e-02, 1.71559595e-02, 1.85851512e-02,
            1.77826229e-02, 1.42182986e-02, 1.55308739e-02, 1.62157407e-02, 3.05650360e-02,
            1.60389652e-02, 1.80024706e-02, 4.91490311e-02, 1.79173802e-01, 1.21340557e-01,
            1.12926546e+00, 9.41718800e-03, 9.30383120e-03, 3.44213045e-03, 1.11235961e-02,
            9.78021144e-03, 6.44053028e-03, 9.46130638e-03, 1.30983082e-02, 1.77025343e-03
        ], [
            9.08424894e-03, 1.32642411e-02, 1.53016450e-02, 1.63454993e-02, 1.75292314e-02,
            1.55021533e-02, 2.37660273e-02, 1.42560571e-02, 1.91787183e-02, 1.81562769e-02,
            1.59542403e-02, 1.69840316e-02, 1.87381014e-02, 1.98740054e-02, 1.70843935e-02,
            1.86517799e-02, 1.29885961e-02, 1.79779080e-02, 1.97799384e-02, 2.76092077e-02,
            2.54318404e-02, 2.10411320e-02, 2.32241655e-02, 3.52498032e-02, 4.42472446e-02,
            3.19622983e-02, 1.08276508e+00, 3.21236552e-02, 8.03197038e-03, 2.90293907e-02,
            2.40900204e-02, 1.16362248e-02, 1.78433967e-02, 2.91925255e-02, 1.62287304e-03
        ], [
            4.77891210e-02, 6.40274131e-02, 5.96816698e-02, 6.93529064e-02, 6.81434279e-02,
            6.49707903e-02, 6.32158026e-02, 6.31083213e-02, 6.67297219e-02, 6.42759639e-02,
            6.38381936e-02, 6.75641942e-02, 6.42224089e-02, 6.30699620e-02, 6.19210769e-02,
            6.64413270e-02, 5.41998658e-02, 5.93106625e-02, 5.75982461e-02, 7.65063829e-02,
            6.13969088e-02, 5.89105749e-02, 7.45764773e-02, 8.82854526e-02, 7.90786822e-02,
            6.28562484e-02, 4.78447015e-02, 1.21273666e+00, 6.73137902e-02, 6.38855531e-02,
            4.81749791e-02, 2.81561231e-02, 4.76344260e-02, 6.35438185e-02, 1.10987308e-02
        ], [
            1.83835290e-02, 1.68339135e-02, 2.30608946e-02, 2.45854394e-02, 2.79328126e-02,
            2.32251868e-02, 2.68857188e-02, 1.88029296e-02, 2.32505067e-02, 2.48832982e-02,
            2.20066772e-02, 2.32720836e-02, 2.44910495e-02, 2.47753789e-02, 2.34199953e-02,
            2.73405439e-02, 1.81728770e-02, 2.94003685e-02, 3.86789853e-02, 4.63646630e-02,
            5.55233998e-02, 4.86551035e-02, 2.51884956e-02, 4.99710338e-02, 3.57847020e-02,
            3.83742328e-02, 2.35095134e-02, 4.06827142e-02, 1.03690442e+00, 3.66421734e-02,
            2.18972364e-02, 2.49134196e-02, 4.22047375e-02, 4.19248404e-02, 2.77798766e-03
        ], [
            4.75183071e-02, 6.90212706e-02, 9.30110997e-02, 9.25801194e-02, 9.41652839e-02,
            7.75485523e-02, 1.23828898e-01, 7.35173775e-02, 1.31899514e-01, 1.06762412e-01,
            8.85258885e-02, 9.01614805e-02, 1.08156353e-01, 1.33446341e-01, 1.14168169e-01,
            1.00047438e-01, 7.81111854e-02, 1.09478705e-01, 9.05049693e-02, 1.02795777e-01,
            9.87580230e-02, 8.17276730e-02, 8.27431000e-02, 1.20504253e-01, 1.56412069e-01,
            1.15119096e-01, 1.00169893e-01, 1.29374552e-01, 5.50203625e-02, 1.16894586e+00,
            9.12638241e-02, 4.77923722e-02, 8.87311652e-02, 1.08394399e-01, 7.64094059e-03
        ], [
            5.30590828e-03, 6.74168603e-03, 6.98114403e-03, 6.36907058e-03, 6.00281728e-03,
            1.06815827e-02, 9.09250124e-03, 6.75586161e-03, 9.15910377e-03, 7.91086306e-03,
            7.47037651e-03, 8.15026785e-03, 6.85342848e-03, 6.84527294e-03, 7.10706148e-03,
            8.20575953e-03, 1.11344715e-02, 6.29164682e-03, 6.39119388e-03, 5.86721423e-03,
            7.21627383e-03, 7.75319723e-03, 6.95067719e-03, 1.53078580e-02, 1.59691794e-02,
            8.50009373e-03, 5.23233582e-03, 4.89021760e-03, 5.92755198e-03, 7.68617115e-03,
            1.01153733e+00, 5.60065679e-03, 8.55343430e-03, 1.42622098e-02, 6.75848654e-04
        ], [
            1.50596807e-03, 1.10248973e-03, 1.43201987e-03, 1.33891049e-03, 1.43887322e-03,
            1.23683407e-03, 1.60115552e-03, 1.29788211e-03, 1.80725872e-03, 1.48769111e-03,
            1.37372510e-03, 1.44458904e-03, 1.79599344e-03, 2.14422335e-03, 1.54722096e-03,
            1.28751492e-03, 1.41428453e-03, 1.33466080e-03, 1.10442574e-03, 1.63235709e-03,
            2.51743916e-03, 1.20007167e-03, 1.61469698e-03, 1.31166388e-03, 2.60735534e-03,
            1.72512634e-03, 2.68498683e-03, 1.78916544e-03, 4.88421699e-04, 2.65555016e-03,
            5.00923819e-03, 1.00581957e+00, 1.56166356e-03, 2.00500030e-03, 1.27525204e-04
        ], [
            2.56972884e-03, 8.49572533e-04, 1.59119151e-03, 1.21511897e-03, 1.25248322e-03,
            1.33452373e-03, 1.07233427e-03, 8.49078357e-04, 1.13463892e-03, 1.10088267e-03,
            1.03919618e-03, 1.02767141e-03, 1.15912511e-03, 1.17002667e-03, 1.00466506e-03,
            1.16422106e-03, 8.05848958e-04, 1.09156134e-03, 9.26733545e-04, 1.05435381e-03,
            8.06307070e-04, 1.67561769e-03, 1.13176743e-03, 1.56569496e-03, 1.15892864e-03,
            1.10941582e-03, 8.58015257e-04, 9.41899049e-04, 3.13803806e-04, 1.06114288e-03,
            6.71021406e-03, 1.66094218e-03, 1.02123300e+00, 1.96737608e-03, 1.17971112e-04
        ], [
            1.05020459e-02, 1.52878883e-02, 1.99030214e-02, 1.91889248e-02, 2.16702420e-02,
            1.90219963e-02, 2.93820193e-02, 1.63419984e-02, 2.44370881e-02, 2.24434302e-02,
            1.85976007e-02, 2.20951836e-02, 2.12859640e-02, 2.30647661e-02, 2.03577627e-02,
            2.13835052e-02, 1.44944551e-02, 1.88319954e-02, 1.98105920e-02, 1.95727156e-02,
            2.00823614e-02, 2.48016522e-02, 2.03785544e-02, 2.23218479e-02, 2.40853641e-02,
            2.36258151e-02, 2.76844157e-02, 2.16382274e-02, 1.26585224e-02, 4.17495846e-02,
            2.63453713e-02, 1.89270060e-02, 2.55225284e-02, 1.08612722e+00, 2.23051979e-03
        ], [
            5.22319300e-05, 6.10758103e-05, 9.99656982e-05, 1.15348340e-04, 1.07210683e-04,
            1.04563337e-04, 1.13696035e-04, 7.67287665e-05, 1.01159198e-04, 1.08246490e-04,
            8.77381585e-05, 1.09008406e-04, 1.03844219e-04, 9.32394370e-05, 1.32965273e-04,
            1.13429182e-04, 6.40546165e-05, 1.57512103e-04, 2.14461103e-04, 1.10296425e-04,
            1.23227418e-04, 1.37986484e-04, 2.30563938e-04, 1.01137752e-04, 1.10740065e-04,
            9.61642076e-05, 1.37519465e-04, 1.22094045e-04, 3.23264127e-04, 1.29800422e-04,
            1.45869651e-04, 8.52453077e-05, 1.77231453e-04, 1.88710579e-04, 1.00003545e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    name = 3
    return round_sig(x=ret, name=name)

# Transportation
# TRANSP - Electric batteries
@cache('step')
def replacement_batteries():
    """
    replacement batteries



    component


    """

    ret =  discarded_batteries()
    name = 3
    return round_sig(x=ret, name=name)

# Transportation
# TRANSP - Total number vehicles
@cache('step')
def share_of_electric_light_vehicles():
    """
    share of electric light vehicles



    component


    """

    ret =  total_number_elec_light_vehicles() / total_number_light_vehicles()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def xshare_elecxhyb_light_vehiclesx():
    """
    Xshare elecXhyb light vehiclesX



    component


    """

    ret =  (total_number_hybrid_light_vehicles() + total_number_elec_light_vehicles()
            ) / total_number_light_vehicles()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def aux_p_inland_transp():
    """
    aux P inland transp



    component


    """

    ret =  functions.if_then_else(
        time() < t_fin_inlandt(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_inlandt() - 1) < equality_threshold, time() > t_ini_inlandt()),
                               (p_inlandt() - initial_percent_t_vehicles()) /
                               (t_fin_inlandt() - t_ini_inlandt()), hist_var_inlandt()), 0)
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def t_fin_inlandt():
    """
    T fin inlandT



    constant


    """

    ret =  2050
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def h_elec_initial_growth():
    """
    H elec initial growth



    component


    """

    ret =  functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold, time() > t_ini_h_veh()),
                               (p_h_vehicle().loc[{'households vehicles':'elec 4wheels'}].values - percent_h_vehicles_initial().loc[{'households vehicles':'elec 4wheels'}].values) /
                               (t_fin_h_veh() - t_ini_h_veh()), aux_hist_h().loc[{'households vehicles':'elec 4wheels'}].values), 0)
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
@cache('run')
def t_fin_h_veh():
    """
    T fin H veh



    constant


    """

    ret =  2050
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def h_gas_initial_growth():
    """
    H gas initial growth



    component


    """

    ret =  functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold, time() > t_ini_h_veh()),
                               (p_h_vehicle().loc[{'households vehicles':'gas 4wheels'}].values - percent_h_vehicles_initial().loc[{'households vehicles':'gas 4wheels'}].values) /
                               (t_fin_h_veh() - t_ini_h_veh()), aux_hist_h().loc[{'households vehicles':'gas 4wheels'}].values), 0)
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
# TRANSP - Households transport
@cache('run')
def t_ini_h_veh():
    """
    T ini H veh



    constant


    """

    ret =  2020
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def h_hyb_initial_growth():
    """
    H hyb initial growth



    component


    """

    ret =  functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold, time() > t_ini_h_veh()),
                               (p_h_vehicle().loc[{'households vehicles':'hib 4wheels'}].values - percent_h_vehicles_initial().loc[{'households vehicles':'hib 4wheels'}].values) /
                               (t_fin_h_veh() - t_ini_h_veh()), aux_hist_h().loc[{'households vehicles':'hib 4wheels'}].values), 0)
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def policy_2wheels():
    """
    policy 2wheels



    component


    """

    ret =  functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold, time() > t_ini_h_veh()),
                               (p_share_2_wheelers() - initial_2w_percent()) /
                               (t_fin_h_veh() - t_ini_h_veh()), 0), 0)
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def h_2w_initial_growth():
    """
    H 2w initial growth



    component


    """

    ret =  functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold, time() > t_ini_h_veh()),
                               (p_h_vehicle().loc[{'households vehicles':'elec 2wheels'}].values - percent_h_vehicles_initial().loc[{'households vehicles':'elec 2wheels'}].values) /
                               (t_fin_h_veh() - t_ini_h_veh()), aux_hist_h().loc[{'households vehicles':'elec 2wheels'}].values), 0)
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def var_percents_h_vehicles():
    """
    var percents H vehicles



    component


    """

    ret =  xr.DataArray(
        data=[functions.if_then_else(
            time() < t_ini_h_veh(),
            aux_hist_h().loc[{'households vehicles':'liq 4wheels'}].values,
            -h_ev_adapt_growth() - h_hyb_adapt_growth() - h_gas_adapt_growth() - rate_4w_to_2w()),

        functions.if_then_else(time() < t_ini_h_veh(), aux_hist_h().loc[{'households vehicles':'elec 4wheels'}].values, h_ev_adapt_growth()),

        functions.if_then_else(time() < t_ini_h_veh(), aux_hist_h().loc[{'households vehicles':'hib 4wheels'}].values, h_hyb_adapt_growth()),

        functions.if_then_else(time() < t_ini_h_veh(), aux_hist_h().loc[{'households vehicles':'gas 4wheels'}].values, h_gas_adapt_growth()),

        functions.if_then_else(time() < t_ini_h_veh(),
                               aux_hist_h().loc[{'households vehicles':'liq 2wheels'}].values, -h_2we_adapt_growth() + rate_4w_to_2w()),

        functions.if_then_else(time() < t_ini_h_veh(), aux_hist_h().loc[{'households vehicles':'elec 2wheels'}].values, h_2we_adapt_growth())],
        coords={'households vehicles':['liq 4wheels','elec 4wheels','hib 4wheels','gas 4wheels','liq 2wheels','elec 2wheels']},
        dims=['households vehicles'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def increase_households_energy_final_demand_for_transp():
    """
    increase Households energy final demand for Transp



    component


    """

    ret =  (energy_intensity_of_households_transport() -
            initial_energy_intensity_of_households_transport_2009()
            ) * household_demand_total() / 1e+006
    name = 4
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def adapt_growth_res_elec():
    """
    adapt growth RES elec



    component


    """

    ret =  xr.DataArray(
        data=[functions.if_then_else(
            time() < 2015,
            past_res_elec_capacity_growth().loc[{'res elec':'hydro'}].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc[{'res elec':'hydro'}].values +
                                   (p_res_elec_growth().loc[{'res elec':'hydro'}].values - past_res_elec_capacity_growth().loc[{'res elec':'hydro'}].values) *
                                   (time_dmnl(time()) - 2015) /
                                   (start_year_p_growth_res_elec() - 2015), p_res_elec_growth().loc[{'res elec':'hydro'}].values)),

        functions.if_then_else(
            time() < 2013,
            past_res_elec_capacity_growth().loc[{'res elec':'xgeotxelecx'}].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc[{'res elec':'xgeotxelecx'}].values +
                                   (p_res_elec_growth().loc[{'res elec':'xgeotxelecx'}].values - past_res_elec_capacity_growth().loc[{'res elec':'xgeotxelecx'}].values) *
                                   (time_dmnl(time()) - 2013) /
                                   (start_year_p_growth_res_elec() - 2013), p_res_elec_growth().loc[{'res elec':'xgeotxelecx'}].values)),

        functions.if_then_else(
            time() < 2013,
            past_res_elec_capacity_growth().loc[{'res elec':'xsolid bioexelecx'}].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc[{'res elec':'xsolid bioexelecx'}].values +
                                   (p_res_elec_growth().loc[{'res elec':'xsolid bioexelecx'}].values - past_res_elec_capacity_growth().loc[{'res elec':'xsolid bioexelecx'}].values) *
                                   (time_dmnl(time()) - 2013) /
                                   (start_year_p_growth_res_elec() - 2013), p_res_elec_growth().loc[{'res elec':'xsolid bioexelecx'}].values)),

        functions.if_then_else(
            time() < 2014,
            past_res_elec_capacity_growth().loc[{'res elec':'oceanic'}].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc[{'res elec':'oceanic'}].values +
                                   (p_res_elec_growth().loc[{'res elec':'oceanic'}].values - past_res_elec_capacity_growth().loc[{'res elec':'oceanic'}].values) *
                                   (time_dmnl(time()) - 2014) /
                                   (start_year_p_growth_res_elec() - 2014), p_res_elec_growth().loc[{'res elec':'oceanic'}].values)),

        functions.if_then_else(
            time() < 2015,
            past_res_elec_capacity_growth().loc[{'res elec':'wind onshore'}].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc[{'res elec':'wind onshore'}].values +
                                   (p_res_elec_growth().loc[{'res elec':'wind onshore'}].values - past_res_elec_capacity_growth().loc[{'res elec':'wind onshore'}].values) *
                                   (time_dmnl(time()) - 2015) /
                                   (start_year_p_growth_res_elec() - 2015), p_res_elec_growth().loc[{'res elec':'wind onshore'}].values)),

        functions.if_then_else(
            time() < 2014,
            past_res_elec_capacity_growth().loc[{'res elec':'wind offshore'}].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc[{'res elec':'wind offshore'}].values +
                                   (p_res_elec_growth().loc[{'res elec':'wind offshore'}].values - past_res_elec_capacity_growth().loc[{'res elec':'wind offshore'}].values) *
                                   (time_dmnl(time()) - 2014) /
                                   (start_year_p_growth_res_elec() - 2014), p_res_elec_growth().loc[{'res elec':'wind offshore'}].values)),

        functions.if_then_else(
            time() < 2014,
            past_res_elec_capacity_growth().loc[{'res elec':'solar pv'}].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc[{'res elec':'solar pv'}].values +
                                   (p_res_elec_growth().loc[{'res elec':'solar pv'}].values - past_res_elec_capacity_growth().loc[{'res elec':'solar pv'}].values) *
                                   (time_dmnl(time()) - 2014) /
                                   (start_year_p_growth_res_elec() - 2014), p_res_elec_growth().loc[{'res elec':'solar pv'}].values)),

        functions.if_then_else(
            time() < 2014,
            past_res_elec_capacity_growth().loc[{'res elec':'csp'}].values,
            functions.if_then_else(
                time() < start_year_p_growth_res_elec(),
                past_res_elec_capacity_growth().loc[{'res elec':'csp'}].values +
                (p_res_elec_growth().loc[{'res elec':'csp'}].values - past_res_elec_capacity_growth().loc[{'res elec':'csp'}].values) *
                (time_dmnl(time()) - 2014) / (start_year_p_growth_res_elec() - 2014),
                p_res_elec_growth().loc[{'res elec':'csp'}].values))],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 4
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# ECON: final energy demand by...
def gdppc_annual_growth_ssp2(x):
    """
    GDPpc annual growth SSP2



    lookup


    """

    ret =  functions.lookup(x, [
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        0.03857969744680689, 0.03857969744680689, 0.03857969744680689, 0.03857969744680689,
        0.03857969744680689, 0.03857969744680689, 0.035236454832179964, 0.035236454832179964,
        0.035236454832179964, 0.035236454832179964, 0.035236454832179964, 0.035236454832179964,
        0.035236454832179964, 0.035236454832179964, 0.035236454832179964, 0.035236454832179964,
        0.026075635629570826, 0.026075635629570826, 0.026075635629570826, 0.026075635629570826,
        0.026075635629570826, 0.026075635629570826, 0.026075635629570826, 0.026075635629570826,
        0.026075635629570826, 0.026075635629570826, 0.022743789939873027, 0.022743789939873027,
        0.022743789939873027, 0.022743789939873027, 0.022743789939873027, 0.022743789939873027,
        0.022743789939873027, 0.022743789939873027, 0.022743789939873027, 0.022743789939873027,
        0.021843416743548616, 0.021843416743548616, 0.021843416743548616, 0.021843416743548616,
        0.021843416743548616, 0.021843416743548616, 0.021843416743548616, 0.021843416743548616,
        0.021843416743548616, 0.021843416743548616, 0.022653169341066315, 0.022653169341066315,
        0.022653169341066315, 0.022653169341066315, 0.022653169341066315, 0.022653169341066315,
        0.022653169341066315, 0.022653169341066315, 0.022653169341066315, 0.022653169341066315,
        0.022024412912428737, 0.022024412912428737, 0.022024412912428737, 0.022024412912428737,
        0.022024412912428737, 0.022024412912428737, 0.022024412912428737, 0.022024412912428737,
        0.022024412912428737, 0.022024412912428737, 0.021875392173132502, 0.021875392173132502,
        0.021875392173132502, 0.021875392173132502, 0.021875392173132502, 0.021875392173132502,
        0.021875392173132502, 0.021875392173132502, 0.021875392173132502, 0.021875392173132502,
        0.021284366520702536, 0.021284366520702536, 0.021284366520702536, 0.021284366520702536,
        0.021284366520702536, 0.021284366520702536, 0.021284366520702536, 0.021284366520702536,
        0.021284366520702536, 0.021284366520702536
    ])
    name = 4
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def annual_gdppc_growth_rate():
    """
    Annual GDPpc growth rate



    component


    """

    ret =  functions.if_then_else(
        abs(select_gdppc_evolution_input() ) < equality_threshold,
        gdppc_annual_growth_ssp2(time()),
        functions.if_then_else(abs(select_gdppc_evolution_input() - 1) < equality_threshold,
                               p_timeseries_gdppc_growth_rate(time()),
                               functions.if_then_else(time() < p_customized_year_gdppc_evolution(),
                                                      p_timeseries_gdppc_growth_rate(time()),
                                                      p_customized_cte_gdppc_variation())))
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def variation_pop_ssp4():
    """
    variation pop SSP4



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, -1 + pop_ssp4(time() + 1) / pop_ssp4(time()))
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def variation_pop_ssp5():
    """
    variation pop SSP5



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, -1 + pop_ssp5(time() + 1) / pop_ssp5(time()))
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('run')
def start_year_p_recycling_minerals_alt_techn():
    """
    start year P recycling minerals alt techn



    constant


    """

    ret =  2020
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('run')
def start_year_p_recycling_minerals_rest():
    """
    start year P recycling minerals Rest



    constant


    """

    ret =  2020
    name = 5
    return round_sig(x=ret, name=name)

@cache('run')
def select_pop_ssps():
    """
    select pop SSPs



    constant


    """

    ret =  2
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def variation_pop_ssp1():
    """
    variation pop SSP1



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, -1 + pop_ssp1(time() + 1) / pop_ssp1(time()))
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def variation_pop_ssp2():
    """
    variation pop SSP2



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, -1 + pop_ssp2(time() + 1) / pop_ssp2(time()))
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def variation_pop_ssp3():
    """
    variation pop SSP3



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, -1 + pop_ssp3(time() + 1) / pop_ssp3(time()))
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def annual_population_growth_rate():
    """
    Annual population growth rate



    component


    """

    ret =  functions.if_then_else(
        abs(select_population_evolution_input() ) < equality_threshold,
        variation_pop_ssps(),
        functions.if_then_else(abs(select_population_evolution_input() - 1) < equality_threshold,
                               p_timeseries_pop_growth_rate(time()),
                               functions.if_then_else(time() < p_customized_year_pop_evolution(),
                                                      p_timeseries_pop_growth_rate(time()),
                                                      p_customized_cte_pop_variation())))
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('step')
def improvement_recycling_rates_minerals_rest():
    """
    improvement recycling rates minerals Rest



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        historic_improvement_recycling_rates_minerals(),
        functions.if_then_else(
            time() < start_year_p_recycling_minerals_rest(),
            historic_improvement_recycling_rates_minerals(),
            recycling_rates_minerals_rest() * remaining_recycling_rate_improv_rest_per_mineral() *
            p_recycling_minerals_rest()))
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def variation_pop_ssps():
    """
    variation pop SSPs



    component


    """

    ret =  functions.if_then_else(abs(select_pop_ssps() - 1) < equality_threshold,
                                  variation_pop_ssp1(),
                                  functions.if_then_else(abs(select_pop_ssps() - 2) < equality_threshold,
                                                         variation_pop_ssp2(),
                                                         functions.if_then_else(
                                                             abs(select_pop_ssps() - 3) < equality_threshold,
                                                             variation_pop_ssp3(),
                                                             functions.if_then_else(
                                                                 abs(select_pop_ssps() - 4) < equality_threshold,
                                                                 variation_pop_ssp4(),
                                                                 variation_pop_ssp5()))))
    name = 6
    return round_sig(x=ret, name=name)

def pop_ssp2(x):
    """
    Pop SSP2



    lookup


    """

    ret =  functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [
                                6867.39, 7611.25, 8261.99, 8787.12, 9169.11, 9384.7, 9456.88,
                                9407.26, 9253.95, 9032.42, 9032.42
                            ])
    name = 6
    return round_sig(x=ret, name=name)

def pop_ssp4(x):
    """
    Pop SSP4



    lookup


    """

    ret =  functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [
                                6895.882, 7626.353, 8258.565, 8765.68, 9146.527, 9377.45, 9472.529,
                                9471.44, 9401.586, 9292.446, 9292.446
                            ])
    name = 6
    return round_sig(x=ret, name=name)

def pop_ssp1(x):
    """
    Pop SSP1



    lookup


    """

    ret =  functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [
                                6921.798, 7576.105, 8061.938, 8388.763, 8530.5, 8492.176, 8298.95,
                                7967.387, 7510.454, 6957.989, 6957.989
                            ])
    name = 6
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('step')
def improvement_recycling_rates_minerals_alt_techn():
    """
    improvement recycling rates minerals alt techn



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        historic_improvement_recycling_rates_minerals(),
        functions.if_then_else(time() < start_year_p_recycling_minerals_alt_techn(),
                               historic_improvement_recycling_rates_minerals(),
                               recycling_rates_minerals_alt_techn() *
                               remaining_recycling_rate_improv_for_alt_techn_per_mineral() *
                               p_recycling_minerals_alt_techn()))
    name = 6
    return round_sig(x=ret, name=name)

def pop_ssp5(x):
    """
    Pop SSP5



    lookup


    """

    ret =  functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [6894, 7552, 8054, 8403, 8579, 8589, 8457, 8200, 7831, 7375, 7375])
    name = 6
    return round_sig(x=ret, name=name)

def pop_ssp3(x):
    """
    Pop SSP3



    lookup


    """

    ret =  functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [
                                6879.59, 7697.854, 8514.307, 9257.22, 9957.131, 10574.362,
                                11117.376, 11633.415, 12134.327, 12620.136, 12620.136
                            ])
    name = 6
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def eroimet():
    """
    EROImet



    component


    """

    ret =  eroist_tot() * auxiliary_metabolic_eroi()
    name = 6
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def eroimet_fc_tot_from_2015_1():
    """
    EROImet FC tot from 2015 1



    component


    """

    ret =  functions.if_then_else(
        time() > 2016,
        np.maximum(0, (eroist_delayed_1yr() * auxiliary_metabolic_eroi() /
                       (eroist_delayed_1yr() * auxiliary_metabolic_eroi() - 1)) *
                   ((eroist_until_2015() * auxiliary_metabolic_eroi() - 1) /
                    (eroist_until_2015() * auxiliary_metabolic_eroi()))), 1)
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def xactivate_eroimet_methodxx():
    """
    XActivate EROImet methodXX



    constant


    """

    ret =  0
    name = 7
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
# ECON- Final demands by sector, households and fuel
# EROI system
# INDICATORS - Energy
@cache('step')
def eroi_fc_tot_from_2015():
    """
    EROI FC tot from 2015



    component


    """

    ret =  functions.if_then_else(
        abs(xactivate_eroi_tot_fc_feedbackxx() ) < equality_threshold, 1,
        functions.if_then_else(functions.logical_and(abs(xactivate_eroi_tot_fc_feedbackxx() - 1) < equality_threshold
                              , abs(xactivate_eroimet_methodxx() ) < equality_threshold),
                               eroi_fc_tot_from_2015_1(),
                               functions.if_then_else(functions.logical_and(abs(xactivate_eroi_tot_fc_feedbackxx() - 1) < equality_threshold
                                                     , abs(xactivate_eroimet_methodxx() - 1) < equality_threshold),
                                                      eroimet_fc_tot_from_2015_1(),
                                                      functions.if_then_else(
                                                          abs(xactivate_eroi_tot_fc_feedbackxx() - 2) < equality_threshold,
                                                          eroi_fc_tot_from_2015_2(), 1))))
    name = 7
    return round_sig(x=ret, name=name)

@cache('run')
def xdesired_gdp_growth_affected_by_real_gdpxx():
    """
    XDesired GDP growth affected by real GDPXX



    constant


    """

    ret =  0
    name = 7
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def dynamic_low_range_fec_good_standard_of_living():
    """
    Dynamic low range FEC good standard of living



    component


    """

    ret =  low_range_fec_good_standard_of_living() * (
        1 + cumulative_tfec_intensity_change_from_2009())
    name = 7
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# ESOI - EV batteries
@cache('step')
def cp_ev_batteries_for_elec_storage():
    """
    Cp EV batteries for elec storage



    component


    """

    ret =  np.minimum(cp_ev_batteries_required(), max_cp_ev_batteries_for_elec_storage())
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def dynamic_threshold_xhigh_developmentx():
    """
    Dynamic threshold Xhigh developmentX



    component


    """

    ret =  threshold_fec_xhigh_developmentx() * (1 + cumulative_tfec_intensity_change_from_2009())
    name = 7
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# ESOI - EV batteries
@cache('step')
def max_cp_ev_batteries_for_elec_storage():
    """
    max Cp EV batteries for elec storage



    component


    """

    ret =  cp_ev_batteries_for_transp()
    name = 7
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def tfec_intensity_until_2009():
    """
    TFEC intensity until 2009



    component


    """

    ret =  functions.if_then_else(time() < 2009, tfes_intensity_ej_tdolar(), aux15())
    name = 7
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# ECON- Final demands by sector, households and fuel
@cache('step')
def desired_annual_gdp_growth_rate():
    """
    Desired annual GDP growth rate



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (-1 + desired_gdp() / desired_gdp_delayed_1yr()),
        functions.if_then_else(abs(xdesired_gdp_growth_affected_by_real_gdpxx() ) < equality_threshold,
                               (-1 + desired_gdp() / desired_gdp_delayed_1yr()),
                               (0.3 * annual_gdp_growth_rate_delayed_1yr() +
                                0.25 * annual_gdp_growth_rate_delayed_2yr() +
                                0.2 * annual_gdp_growth_rate_delayed_3yr() + 0.25 *
                                (-1 + desired_gdp() / desired_gdp_delayed_1yr()))))
    name = 7
    return round_sig(x=ret, name=name)

# EROI system
@cache('run')
def auxiliary_metabolic_eroi():
    """
    auxiliary metabolic EROI



    constant


    """

    ret =  1
    name = 7
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
# INDICATORS - Energy
@cache('step')
def cumulative_tfec_intensity_change_from_2009():
    """
    Cumulative TFEC intensity change from 2009



    component


    """

    ret =  functions.if_then_else(time() < 2009, 0,
                                  -1 + (tfes_intensity_ej_tdolar() / tfec_intensity_until_2009()))
    name = 8
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
# ESOI - EV batteries
@cache('step')
def lifetime_ev_batteries():
    """
    lifetime EV batteries



    component


    """

    ret =  functions.zidz(net_stored_energy_ev_battery_over_lifetime(),
                          ((cp_ev_batteries_for_elec_storage() + cp_ev_batteries_for_transp()) *
                           mw_in_1_year_to_mj() * (kw_per_battery_ev() / kw_per_mw())))
    name = 8
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def dynamic_high_range_fec_good_standard_of_living():
    """
    Dynamic high range FEC good standard of living



    component


    """

    ret =  high_range_fec_good_standard_of_living() * (
        1 + cumulative_tfec_intensity_change_from_2009())
    name = 8
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def eroi_fc_tot_from_2015_1():
    """
    EROI FC tot from 2015 1



    component


    """

    ret =  functions.if_then_else(time() > 2016,
                                  np.maximum(0, (eroist_delayed_1yr() /
                                                 (eroist_delayed_1yr() - 1)) *
                                             ((eroist_until_2015() - 1) / eroist_until_2015())), 1)
    name = 8
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def tfec_per_capita():
    """
    TFEC per capita



    component


    """

    ret =  functions.zidz(real_tfec() * gj_per_ej(), population())
    name = 8
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def aux15():
    """
    aux15



    component


    """

    ret =  delay_tfec_intensity_until_2009_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 8
    return round_sig(x=ret, name=name)

# ESOI - EV batteries
@cache('run')
def cp_ev_batteries_for_transp():
    """
    Cp EV batteries for Transp



    constant


    """

    ret =  0.0055
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def annual_gdp_growth_rate_delayed_3yr():
    """
    annual GDP growth rate delayed 3yr



    component


    """

    ret =  delay_annual_gdp_growth_rate_round3__time_step___time_step_0_3__time_step()
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def annual_gdp_growth_rate_delayed_1yr():
    """
    annual GDP growth rate delayed 1yr



    component


    """

    ret =  delay_annual_gdp_growth_rate_round1__time_step___time_step_0_1__time_step()
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def annual_gdp_growth_rate_delayed_2yr():
    """
    annual GDP growth rate delayed 2yr



    component


    """

    ret =  delay_annual_gdp_growth_rate_round2__time_step___time_step_0_2__time_step()
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def p_customized_cte_gdppc_variation():
    """
    P customized cte GDPpc variation



    constant


    """

    ret =  0.02
    name = 9
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
@cache('step')
def annual_growth_rate_electricity_generation_res_elec_tot():
    """
    Annual growth rate electricity generation RES elec tot



    component


    """

    ret =  -1 + fe_tot_generation_all_res_elec_twh(
    ) / fe_tot_generation_all_res_elec_twh_delayed_1yr()
    name = 9
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
@cache('step')
def annual_growth_rate_res_for_heat():
    """
    Annual growth rate RES for heat



    component


    """

    ret =  -1 + fes_res_for_heat_ej() / fes_res_for_heat_delayed_1yr()
    name = 9
    return round_sig(x=ret, name=name)

@cache('run')
def p_customized_year_pop_evolution():
    """
    P customized year pop evolution



    constant


    """

    ret =  2020
    name = 9
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def implementatio_policy_to_change_final_energy():
    """
    Implementatio policy to change final energy



    component


    """

    ret =  functions.if_then_else(functions.logical_or(
        year_policy_change_energy() < 2015
       , year_policy_change_energy() > year_to_finish_policy_change_energy()), 0,
        functions.if_then_else(
            time() < year_policy_change_energy(), 0,
            functions.if_then_else(
                time() > year_to_finish_policy_change_energy(), 1,
                functions.if_then_else(
                    abs(policy_change_energy_speed() - 1) < equality_threshold,
                    ((time() - year_policy_change_energy()) /
                     (year_to_finish_policy_change_energy() - year_policy_change_energy()))
                    **(exp_rapid_evol_change_energy()),
                    functions.if_then_else(
                        abs(policy_change_energy_speed() - 2) < equality_threshold,
                        ((time() - year_policy_change_energy()) /
                         (year_to_finish_policy_change_energy() - year_policy_change_energy())),
                        functions.if_then_else(
                            abs(policy_change_energy_speed() - 3) < equality_threshold,
                            ((time() - year_policy_change_energy()) /
                             (year_to_finish_policy_change_energy() - year_policy_change_energy()))
                            **exp_slow_evol_change_energy(), 0))))))
    name = 9
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def implementatio_policy_to_change_final_energy_h():
    """
    Implementatio policy to change final energy H



    component


    """

    ret =  functions.if_then_else(functions.logical_or(
        year_policy_change_energy_h() < 2015
       , year_policy_change_energy_h() > year_to_finish_policy_change_energy_h()), 0,
        functions.if_then_else(
            time() < year_policy_change_energy_h(), 0,
            functions.if_then_else(
                time() > year_to_finish_policy_change_energy_h(), 1,
                functions.if_then_else(
                    abs(policy_change_energy_speed_h() - 1) < equality_threshold,
                    ((time() - year_policy_change_energy_h()) /
                     (year_to_finish_policy_change_energy_h() - year_policy_change_energy_h()))
                    **(exp_rapid_evolution_change_energy_h()),
                    functions.if_then_else(abs(policy_change_energy_speed_h() - 2) < equality_threshold, (
                        (time() - year_policy_change_energy_h()) /
                        (year_to_finish_policy_change_energy_h() - year_policy_change_energy_h())),
                                           functions.if_then_else(
                                               abs(policy_change_energy_speed_h() - 3) < equality_threshold,
                                               ((time() - year_policy_change_energy_h()) /
                                                (year_to_finish_policy_change_energy_h() -
                                                 year_policy_change_energy_h()))
                                               **exp_slow_evolution_change_energy_h(), 0))))))
    name = 9
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def implementation_policy_to_improve_energy_intensity_effciency():
    """
    Implementation policy to improve energy intensity effciency



    component


    """

    ret =  functions.if_then_else(functions.logical_or(
        year_policy_to_improve_efficiency() < 2015
       , year_policy_to_improve_efficiency() > year_to_finish_policy_improve_efficiency()), 0,
        functions.if_then_else(
            time() < year_policy_to_improve_efficiency(), 0,
            functions.if_then_else(
                time() > year_to_finish_policy_improve_efficiency(), 1,
                functions.if_then_else(abs(policy_to_improve_efficiency_speed() - 1) < equality_threshold,
                                       ((time() - year_policy_to_improve_efficiency()) /
                                        (year_to_finish_policy_improve_efficiency() -
                                         year_policy_to_improve_efficiency()))
                                       **(exp_rapid_evolution_improve_efficiency()),
                                       functions.if_then_else(
                                           abs(policy_to_improve_efficiency_speed() - 2) < equality_threshold,
                                           ((time() - year_policy_to_improve_efficiency()) /
                                            (year_to_finish_policy_improve_efficiency() -
                                             year_policy_to_improve_efficiency())),
                                           functions.if_then_else(
                                               abs(policy_to_improve_efficiency_speed() - 3) < equality_threshold,
                                               ((time() - year_policy_to_improve_efficiency()) /
                                                (year_to_finish_policy_improve_efficiency() -
                                                 year_policy_to_improve_efficiency()))
                                               **exp_slow_evolution_improve_efficiency(), 0))))))
    name = 9
    return round_sig(x=ret, name=name)
# ECON DEM - WIOD sectors and households

def p_timeseries_gdppc_growth_rate(x):
    """
    P timeseries GDPpc growth rate



    lookup


    """

    ret =  functions.lookup(x, [
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019
    ])
    name = 9
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def desired_variation_gdppc():
    """
    Desired variation GDPpc



    component


    """

    ret =  functions.if_then_else(time() < 2013,
                                  variation_historic_gdppc(),
                                  desired_gdppc() * annual_gdppc_growth_rate())
    name = 9
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def select_gdppc_evolution_input():
    """
    select GDPpc evolution input



    constant


    """

    ret =  1
    name = 9
    return round_sig(x=ret, name=name)

@cache('run')
def select_population_evolution_input():
    """
    select Population evolution input



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def p_customized_year_gdppc_evolution():
    """
    P customized year GDPpc evolution



    constant


    """

    ret =  2020
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def pop_variation():
    """
    pop variation



    component


    """

    ret =  functions.if_then_else(time() < 2014,
                                  variation_historic_pop(),
                                  population() * annual_population_growth_rate())
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def implementation_policy_to_improve_energy_intensity_effciency_h():
    """
    Implementation policy to improve energy intensity effciency H



    component


    """

    ret =  functions.if_then_else(functions.logical_or(
        year_policy_to_improve_efficiency_h() < 2015
       , year_policy_to_improve_efficiency_h() > year_to_finish_policy_improve_efficiency_h()), 0,
        functions.if_then_else(
            time() < year_policy_to_improve_efficiency_h(), 0,
            functions.if_then_else(
                time() > year_to_finish_policy_improve_efficiency_h(), 1,
                functions.if_then_else(abs(policy_to_improve_efficiency_speed_h() - 1) < equality_threshold,
                                       ((time() - year_policy_to_improve_efficiency_h()) /
                                        (year_to_finish_policy_improve_efficiency_h() -
                                         year_policy_to_improve_efficiency_h()))
                                       **(exp_rapid_evolution_improve_efficiency_h()),
                                       functions.if_then_else(
                                           abs(policy_to_improve_efficiency_speed_h() - 2) < equality_threshold,
                                           ((time() - year_policy_to_improve_efficiency_h()) /
                                            (year_to_finish_policy_improve_efficiency_h() -
                                             year_policy_to_improve_efficiency_h())),
                                           functions.if_then_else(
                                               abs(policy_to_improve_efficiency_speed_h() - 3) < equality_threshold,
                                               ((time() - year_policy_to_improve_efficiency_h()) /
                                                (year_to_finish_policy_improve_efficiency_h() -
                                                 year_policy_to_improve_efficiency_h()))
                                               **exp_slow_evolution_improve_efficiency_h(), 0))))))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
@cache('step')
def fe_tot_generation_all_res_elec_twh_delayed_1yr():
    """
    FE tot generation all RES elec TWh delayed 1yr



    component


    """

    ret =  delay_fe_tot_generation_all_res_elec_twh_round1__time_step___time_step_2464_1__time_step(
    )
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def p_customized_cte_pop_variation():
    """
    P customized cte pop variation



    constant


    """

    ret =  0.02
    name = 1
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
@cache('step')
def fes_res_for_heat_delayed_1yr():
    """
    FES RES for heat delayed 1yr



    component


    """

    ret =  delay_fes_res_for_heat_ej_round1__time_step___time_step_3488_1__time_step()
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def exp_slow_evol_change_energy():
    """
    exp slow evol change energy



    constant


    """

    ret =  2
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def year_to_finish_policy_improve_efficiency():
    """
    Year to finish policy improve efficiency



    constant


    """

    ret =  2050
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def exp_slow_evolution_improve_efficiency():
    """
    exp slow evolution improve efficiency



    constant


    """

    ret =  2
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def exp_rapid_evolution_improve_efficiency():
    """
    exp rapid evolution improve efficiency



    constant


    """

    ret =  1 / 2
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def exp_rapid_evol_change_energy():
    """
    exp rapid evol change energy



    constant


    """

    ret =  1 / 2
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def year_to_finish_policy_change_energy():
    """
    Year to finish policy change energy



    constant


    """

    ret =  2050
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def year_to_finish_policy_improve_efficiency_h():
    """
    Year to finish policy improve efficiency H



    constant


    """

    ret =  2050
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def year_to_finish_policy_change_energy_h():
    """
    Year to finish policy change energy H



    constant


    """

    ret =  2050
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def exp_rapid_evolution_change_energy_h():
    """
    exp rapid evolution change energy H



    constant


    """

    ret =  1 / 2
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def pressure_to_improve_energy_intensity_efficiency_h():
    """
    Pressure to improve energy intensity efficiency H



    component


    """

    ret =  energy_cost_pressure_h(
    ) + implementation_policy_to_improve_energy_intensity_effciency_h()
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def exp_slow_evolution_change_energy_h():
    """
    exp slow evolution change energy H



    constant


    """

    ret =  2
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def exp_rapid_evolution_improve_efficiency_h():
    """
    exp rapid evolution improve efficiency H



    constant


    """

    ret =  1 / 2
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def exp_slow_evolution_improve_efficiency_h():
    """
    exp slow evolution improve efficiency H



    constant


    """

    ret =  2
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# ECON- Final demands by sector, households and fuel
@cache('step')
def desired_gdp():
    """
    Desired GDP



    component


    """

    ret =  desired_gdppc() * population() / dollars_to_tdollars()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def desired_gdp_delayed_1yr():
    """
    Desired GDP delayed 1yr



    component


    """

    ret =  delay_desired_gdp_round1__time_step___time_step_2916_1__time_step()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def exogenous_gdp_growth_rate_corrected():
    """
    Exogenous GDP growth rate corrected



    component


    """

    ret =  functions.if_then_else(functions.logical_and(time() > 2009, time() < 2014),
                                  desired_annual_gdp_growth_rate() *
                                  (1 + adjust_exogenous_gdp_growth_rate()),
                                  desired_annual_gdp_growth_rate())
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def diff_annual_gdp_growth_rate():
    """
    diff annual GDP growth rate



    component


    """

    ret =  functions.zidz((annual_gdp_growth_rate() - desired_annual_gdp_growth_rate()),
                          desired_annual_gdp_growth_rate())
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_historic_gdppc():
    """
    variation historic GDPpc



    component


    """

    ret =  functions.if_then_else(time() < 2013,
                                  historic_gdppc(time() + 1) - historic_gdppc(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def gdppc_initial_year():
    """
    GDPpc initial year



    constant


    """

    ret =  5110.298114414277
    name = 1
    return round_sig(x=ret, name=name)
# ECON DEM - WIOD sectors and households

def historic_gdppc(x):
    """
    Historic GDPpc



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        5110.298114414277, 5178.447116478765, 5265.609479336191, 5331.613722454502,
        5489.112017137853, 5617.04027591463, 5624.020226414094, 5618.914766428531,
        5719.399720398113, 5875.090818786368, 6004.810219353785, 6163.484700715218,
        6319.322702473521, 6357.557811858254, 6115.968557332449, 6231.248173984241,
        6305.811229139167, 6390.382840381044, 6485.390211714519, 6548.837589031854
    ])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def desired_gdppc():
    """
    Desired GDPpc



    component


    """


    ret =  integ_desired_gdppc()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xsiaf_feedbackxx():
    """
    XSIAF feedbackXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def energy_intensity_of_households_rest():
    """
    Energy intensity of households rest



    component


    """

    ret =  xr.DataArray(
        data=[functions.if_then_else(abs(activate_transport_h_bottom_up_method() - 1) < equality_threshold,
                               evol_final_energy_intensity_h().loc[{'final sources':'liquids'}].values * change_total_intensity_to_rest().loc[{'final sources':'liquids'}].values,
                               functions.if_then_else(
                                   abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                                   historic_final_energy_intensity_h_subliquids(time()),
                                   evol_final_energy_intensity_h().loc[{'final sources':'liquids'}].values)),

        functions.if_then_else(abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                               historic_final_energy_intensity_h_subsolids(time()),
                               evol_final_energy_intensity_h().loc[{'final sources':'solids'}].values),

        functions.if_then_else(abs(activate_transport_h_bottom_up_method() - 1) < equality_threshold,
                               evol_final_energy_intensity_h().loc[{'final sources':'gases'}].values * change_total_intensity_to_rest().loc[{'final sources':'gases'}].values,
                               functions.if_then_else(
                                   abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                                   historic_final_energy_intensity_h_subgases(time()),
                                   evol_final_energy_intensity_h().loc[{'final sources':'gases'}].values)),

        functions.if_then_else(abs(activate_transport_h_bottom_up_method() - 1) < equality_threshold,
                               evol_final_energy_intensity_h().loc[{'final sources':'electricity'}].values * change_total_intensity_to_rest().loc[{'final sources':'electricity'}].values,
                               functions.if_then_else(
                                   abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                                   historic_final_energy_intensity_h_subelectricity(time()),
                                   evol_final_energy_intensity_h().loc[{'final sources':'electricity'}].values)),

        functions.if_then_else(abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                               historic_final_energy_intensity_h_subheat(time()),
                               evol_final_energy_intensity_h().loc[{'final sources':'heat'}].values)],
        coords={'final sources':['liquids','solids','gases','electricity','heat']},
        dims=['final sources'])
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def total_radiative_forcing():
    """
    Total Radiative Forcing



    component


    """

    ret =  co2_rad_forcing() + other_ghg_rad_forcing(
        time()) + arctic_september_free_of_sea_ice(time()) * xsiaf_feedbackxx()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def evol_final_energy_intensity_by_sector_and_fuel_two():
    """
    Evol final energy intensity by sector and fuel two



    component


    """

    ret =  xr.DataArray(
        data=np.zeros((5, 35)),
        coords={'final sources':['electricity','heat','liquids','gases','solids'], 'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['final sources','sectors'])
    def row(final_source):
        return [functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'agriculture hunting forestry and fishing'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'agriculture hunting forestry and fishing','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculturehuntingforestryandfishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'agriculture hunting forestry and fishing','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'mining and quarrying'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'mining and quarrying','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subminingandquarrying(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'mining and quarrying','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'food beverages and tobacco'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'food beverages and tobacco','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfoodbeveragesandtobacco(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'food beverages and tobacco','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'textiles and textile products'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'textiles and textile products','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextilesandtextileproducts(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'textiles and textile products','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'leather leather and footwear'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'leather leather and footwear','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleatherleatherandfootwear(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'leather leather and footwear','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'wood and products of woood and cork'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'wood and products of woood and cork','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwoodandproductsofwooodandcork(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'wood and products of woood and cork','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'pulp paper printing and publishing'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'pulp paper printing and publishing','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulppaperprintingandpublishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'pulp paper printing and publishing','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'coke refined petroleum and nuclear fuel'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'coke refined petroleum and nuclear fuel','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcokerefinedpetroleumandnuclearfuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'coke refined petroleum and nuclear fuel','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'chemicals and chemical products'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'chemicals and chemical products','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicalsandchemicalproducts(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'chemicals and chemical products','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'rubber and plastics'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'rubber and plastics','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubberandplastics(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'rubber and plastics','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'other non metalic mineral'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'other non metalic mineral','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothernonmetalicmineral(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'other non metalic mineral','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'basic metals and fabricated metal'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'basic metals and fabricated metal','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasicmetalsandfabricatedmetal(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'basic metals and fabricated metal','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'machinery nec'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'machinery nec','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinerynec(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'machinery nec','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'electrical and optical equipment'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'electrical and optical equipment','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricalandopticalequipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'electrical and optical equipment','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'transport equipment'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'transport equipment','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransportequipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'transport equipment','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'manufacturing nec recycling'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'manufacturing nec recycling','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturingnecrecycling(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'manufacturing nec recycling','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'electricity gas and water supply'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'electricity gas and water supply','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricitygasandwatersupply(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'electricity gas and water supply','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'construction'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'construction','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'construction','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'wholesale trade and commissions trade except of motor vehicles and motorcycles'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'wholesale trade and commissions trade except of motor vehicles and motorcycles','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'wholesale trade and commissions trade except of motor vehicles and motorcycles','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'retail trade except of motor vehicles and motorcycles repair of household goods'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'retail trade except of motor vehicles and motorcycles repair of household goods','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'retail trade except of motor vehicles and motorcycles repair of household goods','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'hotels and restaurants'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'hotels and restaurants','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotelsandrestaurants(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'hotels and restaurants','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'inland transport'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'inland transport','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinlandtransport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'inland transport','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'water transport'}].values - 1) < equality_threshold, evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'water transport','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwatertransport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'water transport','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'air transport'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'air transport','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subairtransport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'air transport','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'other supporting and auxiliary transport activities activities of travel agencies'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'other supporting and auxiliary transport activities activities of travel agencies','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'other supporting and auxiliary transport activities activities of travel agencies','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'post and telecommunications'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'post and telecommunications','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpostandtelecommunications(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'post and telecommunications','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'financial intermedation'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'financial intermedation','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancialintermedation(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'financial intermedation','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'real estate activities'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'real estate activities','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrealestateactivities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'real estate activities','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'renting od meq and other business activities'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'renting od meq and other business activities','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrentingodmeqandotherbusinessactivities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'renting od meq and other business activities','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'public admin and defence compulsory social security'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'public admin and defence compulsory social security','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublicadminanddefencecompulsorysocialsecurity(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'public admin and defence compulsory social security','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'education'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'education','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'education','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'health and social work'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'health and social work','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealthandsocialwork(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'health and social work','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'other community social and persona services'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'other community social and persona services','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothercommunitysocialandpersonaservices(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'other community social and persona services','final sources':final_source}].values)),
        functions.if_then_else(
            abs(activate_bottom_up_method().loc[{'sectors':'private households with employed persons'}].values - 1) < equality_threshold,
            evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'private households with employed persons','final sources':final_source}].values,
            functions.if_then_else(
                abs(xconstant_final_energy_intensity_sectorsxhouseholds_tdxx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivatehouseholdswithemployedpersons(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc[{'sectors':'private households with employed persons','final sources':final_source}].values))]
    final_sources = ['electricity','heat','liquids','gases','solids']    
    data = [row(s) for s in final_sources]
    ret.values = list(map(lambda x: x if type(x) != xr.DataArray else xr.align(ret, x, join='left')[1], data))
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def tpes_intensity_until_2009():
    """
    TPES intensity until 2009



    component


    """

    ret =  functions.if_then_else(time() < 2009, tpes_intensity_ej_tdolar(), aux14())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xsharexe_losses_cc_until_2015x():
    """
    XshareXE losses CC until 2015X



    component


    """

    ret =  functions.if_then_else(time() < 2015, elf(), aux1())
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def aux13():
    """
    aux13



    component


    """

    ret =  delay_tfec_intensity_until_2009_without_eroi_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def aux14():
    """
    aux14



    component


    """

    ret =  delay_tpes_intensity_until_2009_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_exlosses_cc_from_2015x():
    """
    Xshare EXlosses CC from 2015X



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0,
                                  np.maximum(0,
                                             np.minimum(1,
                                                        elf() - xsharexe_losses_cc_until_2015x())))
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS CC - Energy losses function
@cache('step')
def elf():
    """
    ELF



    component


    """

    ret =  functions.if_then_else(abs(selection_shape_elf() ) < equality_threshold,
                                  elf_concentrations_logistic(),
                                  functions.if_then_else(abs(selection_shape_elf() - 1) < equality_threshold,
                                                         xelf_dietzxstern_x2015xx(), 0))
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def cumulative_tpes_intensity_change_from_2009():
    """
    Cumulative TPES intensity change from 2009



    component


    """

    ret =  functions.if_then_else(time() < 2009, 0,
                                  -1 + (tpes_intensity_ej_tdolar() / tpes_intensity_until_2009()))
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def cumulative_tfec_intensity_change_from_2009_without_eroi():
    """
    Cumulative TFEC intensity change from 2009 without EROI



    component


    """

    ret =  functions.if_then_else(
        time() < 2009, 0,
        -1 + (tfes_intensity_ej_tdolar_without_eroi() / tfec_intensity_until_2009_without_eroi()))
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xelf_dietzxstern_x2015xx():
    """
    XELF DietzXStern X2015XX



    component


    """

    ret =  1 - 1 / (1 + (2.84 / 1000) * (temperature_change())**2 + (8.19 / 100000) *
                    (temperature_change())**6.754)
    name = 1
    return round_sig(x=ret, name=name)
# IMPACTS - Carbon cycle & climate

def arctic_september_free_of_sea_ice(x):
    """
    Arctic September free of sea ice



    lookup


    """

    ret =  functions.lookup(x, [1995, 2010, 2050, 2100], [0.0, 0.11, 0.29, 0.515])
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def tfec_intensity_until_2009_without_eroi():
    """
    TFEC intensity until 2009 without EROI



    component


    """

    ret =  functions.if_then_else(time() < 2009, tfes_intensity_ej_tdolar_without_eroi(), aux13())
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def tfei_sectors():
    """
    TFEI sectors



    component


    """

    ret =  functions.sum(final_energy_intensity_by_sector_and_fuel())
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def annual_tfes_intensity_growth_rate_without_eroi():
    """
    Annual TFES intensity growth rate without EROI



    component


    """

    ret =  -1 + tfes_intensity_ej_tdolar_without_eroi() / tfes_intensity_without_eroi_delayed_1yr()
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def final_energy_intensity_by_sector_and_fuel():
    """
    Final energy intensity by sector and fuel



    component


    """

    ret =  evol_final_energy_intensity_by_sector_and_fuel_two() * (
        1 + xactivate_eroi_tot_fc_feedback_through_intensitiesxx() *
        (eroi_fc_tot_from_2015() * 1 - 1))
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def energy_intensity_of_households():
    """
    Energy intensity of households



    component


    """

    ret =  functions.if_then_else(
        time() < 2009,
        energy_intensity_of_households_rest(),
        functions.if_then_else(
            abs(activate_transport_h_bottom_up_method() ) < equality_threshold,
            energy_intensity_of_households_rest(),
            energy_intensity_of_households_transport() + energy_intensity_of_households_rest()))
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def tfes_intensity_without_eroi_delayed_1yr():
    """
    TFES intensity without EROI delayed 1yr



    component


    """

    ret =  delay_tfes_intensity_ej_tdolar_without_eroi_round1__time_step___time_step_8827_1__time_step(
    )
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def households_total_final_energy_demand():
    """
    Households total final energy demand



    component


    """

    ret =  functions.sum(households_final_energy_demand())
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def required_tfed_sectors():
    """
    required TFED sectors



    component


    """

    ret =  functions.sum(required_fed_sectors_by_fuel())
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def share_fed_households_vs_sectors():
    """
    share FED households vs sectors



    component


    """

    ret =  households_total_final_energy_demand() / required_tfed_sectors()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def tfes_intensity_ej_tdolar_without_eroi():
    """
    TFES intensity EJ Tdolar without EROI



    component


    """

    ret =  functions.zidz(real_tfec() / eroi_fc_tot_from_2015(), gdp())
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def ghg_emissions_2050_mlt2030():
    """
    GHG emissions 2050 MLT2030



    constant


    """

    ret =  7.958767677327897
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def ghg_emissions_2050_mlt2020():
    """
    GHG emissions 2050 MLT2020



    constant


    """

    ret =  17.232523345825705
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def cumulative_co2e_ghg_emissions():
    """
    Cumulative CO2e GHG emissions



    component


    """


    ret =  integ_cumulative_co2e_ghg_emissions()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xadapt_growth_res_for_heatxcomx():
    """
    Xadapt growth RES for heatXcomX



    component


    """

    ret =  functions.if_then_else(
        time() < 2014,
        xpast_res_growth_for_heatxcomx(),
        functions.if_then_else(time() < start_year_p_growth_res_heat(),
                               xpast_res_growth_for_heatxcomx() +
                               (p_res_for_heat() - xpast_res_growth_for_heatxcomx()) *
                               (time_dmnl(time()) - 2014) /
                               (start_year_p_growth_res_heat() - 2014), p_res_for_heat()))
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xadapt_growth_res_for_heatxncx():
    """
    Xadapt growth RES for heatXncX



    component


    """

    ret =  functions.if_then_else(
        time() < 2014,
        xpast_res_growth_for_heatxncx(),
        functions.if_then_else(time() < start_year_p_growth_res_heat(),
                               xpast_res_growth_for_heatxncx() +
                               (p_res_for_heat() - xpast_res_growth_for_heatxncx()) *
                               (time_dmnl(time()) - 2014) /
                               (start_year_p_growth_res_heat() - 2014),
                               p_res_for_heat())) * xdeactivate_heat_dem_correctionxx()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def start_year_p_growth_res_elec():
    """
    Start year P growth RES elec



    constant


    """

    ret =  2020
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
@cache('run')
def start_year_p_growth_res_heat():
    """
    Start year P growth RES heat



    constant


    """

    ret =  2020
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def total_ce_all_ghg():
    """
    Total Ce all GHG



    component


    """

    ret =  xtotal_co2e_xgwpx100_yearsxx() * correction_factor_all_ghgs() * c_per_co2()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def labour_share_growth():
    """
    Labour share growth



    component


    """

    ret =  ((p_labour_share_2050() / initial_labour_share())**
            (1 / (year_final_labour_share() - year_initial_labour_share()))) - 1
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def year_initial_labour_share():
    """
    Year Initial Labour share



    constant


    """

    ret =  2014
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def year_final_labour_share():
    """
    Year Final Labour share



    constant


    """

    ret =  2050
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def p_labour_share_2050():
    """
    P labour share 2050



    constant


    """

    ret =  0.52
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def growth_labour_share():
    """
    growth labour share



    component


    """

    ret =  functions.if_then_else(time() > 2014,
                                  functions.if_then_else(
                                      time() > 2050, 0,
                                      labour_share_growth() * xlabor_share_ctexx()),
                                  historic_labour_share(time()))
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def initial_labour_share():
    """
    Initial Labour share



    constant


    """

    ret =  0.5621
    name = 1
    return round_sig(x=ret, name=name)

def xtotal_ghg_emissions_olt_medeas_d3x2x(x):
    """
    XTotal GHG emissions OLT MEDEAS D3X2X



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        35.759413148286896, 35.9654102916019, 35.24924015949415, 35.39397812433697,
        35.518166161269896, 36.24446809126249, 36.95717178883806, 37.17157881794709,
        37.401771576560655, 37.62916715402579, 38.88923353257618, 39.1672436132021,
        39.70029697776367, 40.962653787287586, 42.43164699720583, 43.4585485141285,
        44.73882999735216, 45.98817373194996, 46.41034844291112, 45.85923565238791,
        47.554580971711395, 48.42974943441521, 47.42876601959408, 47.09505948151982,
        46.76135294344556, 46.4276464053713, 46.09393986729704, 45.76023332922278,
        45.42652679114852, 45.09282025307426, 44.759113715, 44.05724267735, 43.355371639699996,
        42.65350060205, 41.9516295644, 41.24975852675, 40.5478874891, 39.84601645145,
        39.144145413800004, 38.44227437615, 37.7404033385, 36.99498139958053, 36.24955946066106,
        35.50413752174159, 34.75871558282212, 34.01329364390265, 33.26787170498318,
        32.52244976606371, 31.77702782714424, 31.03160588822477, 30.2861839493053,
        29.55797931044127, 28.82977467157724, 28.101570032713212, 27.37336539384918,
        26.645160754985152, 25.91695611612112, 25.188751477257092, 24.46054683839306,
        23.73234219952903, 23.004137560665, 22.37502643707183, 21.745915313478662,
        21.11680418988549, 20.48769306629232, 19.85858194269915, 19.22947081910598,
        18.600359695512807, 17.971248571919638, 17.34213744832647, 16.7130263247333,
        16.21015507735497, 15.707283829976639, 15.204412582598309, 14.701541335219979,
        14.198670087841649, 13.695798840463318, 13.19292759308499, 12.69005634570666,
        12.18718509832833, 11.68431385095, 11.265439517733332, 10.846565184516665, 10.4276908513,
        10.008816518083332, 9.589942184866665, 9.171067851649997, 8.75219351843333,
        8.333319185216663, 7.914444851999997, 7.49557051878333, 7.215631669871664,
        6.935692820959998, 6.655753972048332, 6.375815123136666, 6.095876274225001,
        5.815937425313334, 5.535998576401668, 5.256059727490002, 4.976120878578336,
        4.69618202966667, 4.523135116700003, 4.350088203733336, 4.177041290766669,
        4.0039943778000024, 3.8309474648333355, 3.657900551866668, 3.484853638900001,
        3.311806725933334, 3.0, 3.0
    ])
    name = 1
    return round_sig(x=ret, name=name)

def xtotal_ghg_emissions_bauxcat_medeas_d3x2x(x):
    """
    XTotal GHG emissions BAUXCAT MEDEAS D3X2X



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        35.759413148286896, 35.9654102916019, 35.24924015949415, 35.39397812433697,
        35.518166161269896, 36.24446809126249, 36.95717178883806, 37.17157881794709,
        37.401771576560655, 37.62916715402579, 38.88923353257618, 39.1672436132021,
        39.70029697776367, 40.962653787287586, 42.43164699720583, 43.4585485141285,
        44.73882999735216, 45.98817373194996, 46.41034844291112, 45.85923565238791,
        47.554580971711395, 48.42974943441521, 52.46089897560433, 54.21177696260433,
        55.96265494960433, 57.71353293660433, 59.464410923604326, 61.21528891060433,
        62.96616689760433, 64.71704488460433, 66.46792287160433, 68.00663966160434,
        69.54535645160432, 71.08407324160433, 72.62279003160432, 74.16150682160433,
        75.70022361160433, 77.23894040160432, 78.77765719160433, 80.31637398160433,
        81.85509077160432, 83.05051741060433, 84.24594404960432, 85.44137068860432,
        86.63679732760431, 87.83222396660433, 89.02765060560434, 90.22307724460433,
        91.41850388360433, 92.61393052260432, 93.80935716160432, 95.46841207750431,
        97.12746699340433, 98.78652190930431, 100.44557682520431, 102.10463174110431,
        103.7636866570043, 105.42274157290431, 107.0817964888043, 108.74085140470432,
        110.3999063206043, 111.83580096570431, 113.27169561080431, 114.70759025590432,
        116.14348490100431, 117.57937954610432, 119.01527419120433, 120.45116883630432,
        121.88706348140433, 123.32295812650432, 124.75885277160434, 126.37919332160433,
        127.99953387160433, 129.61987442160432, 131.24021497160436, 132.86055552160434,
        134.48089607160435, 136.10123662160433, 137.72157717160434, 139.34191772160435,
        140.96225827160433, 141.74471066160433, 142.52716305160436, 143.30961544160434,
        144.09206783160434, 144.8745202216043, 145.65697261160435, 146.43942500160435,
        147.22187739160432, 148.00432978160435, 148.78678217160433, 150.03883602160434,
        151.29088987160432, 152.54294372160433, 153.7949975716043, 155.04705142160435,
        156.29910527160436, 157.55115912160434, 158.80321297160435, 160.05526682160433,
        161.30732067160434, 162.69081727160432, 164.07431387160435, 165.45781047160435,
        166.84130707160432, 168.22480367160432, 169.60830027160432, 170.99179687160435,
        172.37529347160432, 173.7587900716043, 175.14228667160432
    ])
    name = 1
    return round_sig(x=ret, name=name)

def xtotal_ghg_emissions_mlt2030_medeas_d3x2x(x):
    """
    XTotal GHG emissions MLT2030 MEDEAS D3X2X



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        35.759413148286896, 35.9654102916019, 35.24924015949415, 35.39397812433697,
        35.518166161269896, 36.24446809126249, 36.95717178883806, 37.17157881794709,
        37.401771576560655, 37.62916715402579, 38.88923353257618, 39.1672436132021,
        39.70029697776367, 40.962653787287586, 42.43164699720583, 43.4585485141285,
        44.73882999735216, 45.98817373194996, 46.41034844291112, 45.85923565238791,
        47.554580971711395, 48.42974943441521, 52.460899, 54.211777, 55.9626549, 57.7135329,
        59.4644109, 61.2152889, 62.9661669, 64.7170449, 66.4679229, 68.0066397, 69.5453565,
        71.0840732, 72.62279, 74.1615068, 75.7002236, 77.2389404, 78.7776572, 80.316374,
        81.8550908, 72.851030812, 64.83741742268, 57.70530150618521, 51.35771834050483,
        45.7083693230493, 40.680448697513874, 36.205599340787344, 32.22298341330074,
        28.678455237837657, 25.523825161675514, 22.716204393891207, 20.217421910563175,
        17.993505500401227, 16.01421989535709, 14.252655706867811, 12.684863579112353,
        11.289528585409993, 10.047680441014894, 8.942435592503255, 7.958767677327897,
        7.083303232821828, 6.304139877211427, 5.61068449071817, 4.993509196739171,
        4.444223185097862, 3.9553586347370975, 3.5202691849160166, 3.1330395745752546,
        2.7884052213719768, 2.4816806470210593, 2.2086957758487427, 1.965739240505381,
        1.749507924049789, 1.5570620524043122, 1.3857852266398378, 1.2333488517094557,
        1.0976804780214156, 0.9769356254390599, 0.8694727066407633, 0.7738307089102793,
        0.6887093309301485, 0.6129513045278322, 0.5455266610297707, 0.4855187283164959,
        0.43211166820168134, 0.3845793846994964, 0.34227565238255175, 0.30462533062047104,
        0.2711165442522192, 0.2412937243844751, 0.21475141470218284, 0.19112875908494273,
        0.17010459558559904, 0.15139309007118315, 0.134739850163353, 0.11991846664538416,
        0.10672743531439191, 0.0949874174298088, 0.08453880151252983, 0.07523953334615155,
        0.06696318467807488, 0.05959723436348664, 0.05304153858350311, 0.04720696933931777,
        0.04201420271199281, 0.0373926404136736, 0.033279449968169506, 0.02961871047167086,
        0.026360652319787067, 0.02346098056461049
    ])
    name = 1
    return round_sig(x=ret, name=name)

def xtotal_ghg_emissions_mlt2020_medeas_d3x2x(x):
    """
    XTotal GHG emissions MLT2020 MEDEAS D3X2X



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        35.759413148286896, 35.9654102916019, 35.24924015949415, 35.39397812433697,
        35.518166161269896, 36.24446809126249, 36.95717178883806, 37.17157881794709,
        37.401771576560655, 37.62916715402579, 38.88923353257618, 39.1672436132021,
        39.70029697776367, 40.962653787287586, 42.43164699720583, 43.4585485141285,
        44.73882999735216, 45.98817373194996, 46.41034844291112, 45.85923565238791,
        47.554580971711395, 48.42974943441521, 52.46089897560433, 54.21177696260433,
        55.96265494960433, 57.71353293660433, 59.464410923604326, 61.21528891060433,
        62.96616689760433, 64.71704488460433, 66.46792287160433, 63.54333426525374,
        60.747427557582576, 58.07454074504894, 55.51926095226679, 53.07641347036705,
        50.7410512776709, 48.508445021453376, 46.37407344050943, 44.33361420912701,
        42.382935183925426, 40.51808603583271, 38.73529025025607, 37.030937479244805,
        35.401576230158035, 33.843906876031085, 32.35477497348572, 30.931164874652346,
        29.570193620167643, 28.269105100880267, 27.025264476441535, 25.836152839478107,
        24.69936211454107, 23.612590181501265, 22.573636213515208, 21.580396220120537,
        20.630858786435233, 19.723100999832084, 18.85528455583947, 18.025652035382535,
        17.232523345825705, 16.474292318609372, 15.74942345659056, 15.056448824500574,
        14.39396507622255, 13.760630612868757, 13.155162865902533, 12.57633569980282,
        12.022976929011497, 11.493965944134992, 10.988231442593053, 10.504749259118958,
        10.042540291717724, 9.600668518882145, 9.17823910405133, 8.774396583473072,
        8.388323133800256, 8.019236915913044, 7.66639049161287, 7.329069309981904, 7.0065902603427,
        6.698300288887622, 6.403575076176566, 6.121817772824797, 5.852457790820506,
        5.594949648024404, 5.34877186351133, 5.113425901516831, 4.888435161850091,
        4.673344014728687, 4.4677168780806245, 4.271137335445077, 4.083207292685493,
        3.903546171807332, 3.731790140247809, 3.5675913740769056, 3.4106173536175217,
        3.2605501900583507, 3.117085981695783, 2.979934198501169, 2.8488170937671176,
        2.7234691416413646, 2.6036364994091445, 2.4890764934351424, 2.379557127723996,
        2.27485661410414, 2.174762923083558, 2.0790733544678814, 1.9875941268712947,
        1.9001399852889578, 1.8165338259362436
    ])
    name = 1
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('run')
def low_range_fec_good_standard_of_living():
    """
    Low range FEC good standard of living



    constant


    """

    ret =  30
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def threshold_fec_xhigh_developmentx():
    """
    Threshold FEC Xhigh developmentX



    constant


    """

    ret =  75
    name = 1
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('run')
def high_range_fec_good_standard_of_living():
    """
    High range FEC good standard of living



    constant


    """

    ret =  40
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def decrease_of_intensity_due_to_energy_a_technology_change_top_down():
    """
    Decrease of intensity due to energy a technology change TOP DOWN



    component


    """

    ret =  functions.if_then_else(
        abs(activate_bottom_up_method() ) < equality_threshold,
        functions.if_then_else(
            functions.zidz(evol_final_energy_intensity_by_sector_and_fuel(),
                            global_energy_intensity_by_sector()) >= minimum_fraction(),
            max_yearly_change() * evol_final_energy_intensity_by_sector_and_fuel() *
            pressure_to_change_energy_technology(), 0), 0)
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def maximun_yearly_aceleration_of_intensity_improvement_pct():
    """
    Maximun yearly aceleration of intensity improvement pct



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01, 0.01, 0.01, 0.01, 0.01]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def policy_change_energy_speed():
    """
    Policy change energy speed



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def maximun_yearly_aceleratuin_of_intensity_improvement():
    """
    Maximun yearly aceleratuin of intensity improvement



    component


    """

    ret =  -maximun_yearly_aceleration_of_intensity_improvement_pct(
    ) * initial_energy_intensity_by_fuel_and_sector_1995()
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def policy_to_improve_efficiency_speed():
    """
    Policy to improve efficiency speed



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def inertial_rate_energy_intensity_top_down():
    """
    inertial rate energy intensity TOP DOWN



    component


    """

    ret =  functions.if_then_else(
        time() > 2008,
        functions.if_then_else(functions.logical_or(
            abs(activate_bottom_up_method() ) < equality_threshold, abs(rate_change_intensity_bottom_up() ) < equality_threshold),
            functions.if_then_else(
                (historical_mean_rate_energy_intensity() + efficiency_energy_aceleration()) < 0,
                evol_final_energy_intensity_by_sector_and_fuel() *
                (historical_mean_rate_energy_intensity() + efficiency_energy_aceleration()) *
                available_improvement_efficiency(),
                initial_energy_intensity_by_fuel_and_sector_1995() *
                (historical_mean_rate_energy_intensity() + efficiency_energy_aceleration())), 0),
        historic_rate_final_energy_intensity())
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def minimum_fraction():
    """
    minimum fraction



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def increase_of_intensity_due_to_energy_a_technology_change_top_down():
    """
    Increase of intensity due to energy a technology change TOP DOWN



    component


    """

    ret =  xr.DataArray(
        data=np.zeros((5, 35)),
        coords={'final sources':['liquids','gases','solids','electricity','heat'], 'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['final sources','sectors'])
    data = [decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'solids'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'liquids',
        'final sources1':'solids'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'gases'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'liquids',
        'final sources1':'gases'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'electricity'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'liquids',
        'final sources1':'electricity'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'heat'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'liquids',
        'final sources1':'heat'}].drop('final sources').drop('final sources1'),

        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'solids'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'gases',
        'final sources1':'solids'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'electricity'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'gases',
        'final sources1':'electricity'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'heat'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'gases',
        'final sources1':'heat'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'liquids'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'gases',
        'final sources1':'liquids'}].drop('final sources').drop('final sources1'),

        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'gases'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'solids',
        'final sources1':'gases'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'electricity'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'solids',
        'final sources1':'electricity'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'heat'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'solids',
        'final sources1':'heat'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'liquids'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'solids',
        'final sources1':'liquids'}].drop('final sources').drop('final sources1'),

        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'solids'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'electricity',
        'final sources1':'solids'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'gases'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'electricity',
        'final sources1':'gases'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'heat'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'electricity',
        'final sources1':'heat'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'liquids'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'electricity',
        'final sources1':'liquids'}].drop('final sources').drop('final sources1'),

        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'solids'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'heat',
        'final sources1':'solids'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'gases'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'heat',
        'final sources1':'gases'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'electricity'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'heat',
        'final sources1':'electricity'}].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[{'final sources':'liquids'}].drop('final sources') *
        efficiency_rate_of_substitution().loc[{'final sources':'heat',
        'final sources1':'liquids'}].drop('final sources').drop('final sources1')]
    ret.values = list(map(lambda x: x if type(x) != xr.DataArray else xr.align(ret, x, join='left')[1], data))
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def year_policy_change_energy():
    """
    Year policy change energy



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def max_yearly_change():
    """
    Max yearly change



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.05, 0.05, 0.05, 0.05, 0.05]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def efficiency_rate_of_substitution():
    """
    efficiency rate of substitution



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
        xr.DataArray(
            data=[[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
                   [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
                'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources', 'final sources1']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
# FE INTENSITIES by sector and fuel
@cache('run')
def activate_bottom_up_method():
    """
    Activate BOTTOM UP method



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['agriculture hunting forestry and fishing']},
            dims=['sectors']),
        xr.DataArray(data=[0.], coords={'sectors': ['mining and quarrying']}, dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['food beverages and tobacco']}, dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['textiles and textile products']}, dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['leather leather and footwear']}, dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['wood and products of woood and cork']},
            dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['pulp paper printing and publishing']},
            dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['coke refined petroleum and nuclear fuel']},
            dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['chemicals and chemical products']},
            dims=['sectors']),
        xr.DataArray(data=[0.], coords={'sectors': ['rubber and plastics']}, dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['other non metalic mineral']}, dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['basic metals and fabricated metal']},
            dims=['sectors']),
        xr.DataArray(data=[0.], coords={'sectors': ['machinery nec']}, dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['electrical and optical equipment']},
            dims=['sectors']),
        xr.DataArray(data=[0.], coords={'sectors': ['transport equipment']}, dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['manufacturing nec recycling']}, dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['electricity gas and water supply']},
            dims=['sectors']),
        xr.DataArray(data=[0.], coords={'sectors': ['construction']}, dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ]
            },
            dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles']
            },
            dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ]
            },
            dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['hotels and restaurants']}, dims=['sectors']),
        xr.DataArray(data=[1.], coords={'sectors': ['inland transport']}, dims=['sectors']),
        xr.DataArray(data=[0.], coords={'sectors': ['water transport']}, dims=['sectors']),
        xr.DataArray(data=[0.], coords={'sectors': ['air transport']}, dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ]
            },
            dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['post and telecommunications']}, dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['financial intermedation']}, dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['real estate activities']}, dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['renting od meq and other business activities']},
            dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['public admin and defence compulsory social security']},
            dims=['sectors']),
        xr.DataArray(data=[0.], coords={'sectors': ['education']}, dims=['sectors']),
        xr.DataArray(
            data=[0.], coords={'sectors': ['health and social work']}, dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['other community social and persona services']},
            dims=['sectors']),
        xr.DataArray(
            data=[0.],
            coords={'sectors': ['private households with employed persons']},
            dims=['sectors']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def year_policy_to_improve_efficiency():
    """
    Year policy to improve efficiency



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def decrease_of_intensity_due_to_energy_a_technology_change_h_top_down():
    """
    Decrease of intensity due to energy a technology change H TOP DOWN



    component


    """

    ret =  functions.if_then_else((functions.zidz(
        evol_final_energy_intensity_h(), global_energy_intensity_h())) >= minimum_fraction_h(),
                                  max_yearly_change_h() * evol_final_energy_intensity_h() *
                                  pressure_to_change_energy_technology_h(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def increase_of_intensity_due_to_energy_a_technology_change_h_top_down():
    """
    Increase of intensity due to energy a technology change H TOP DOWN



    component


    """

    ret =  xr.DataArray(
        data=[decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'solids'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'liquids',
        'final sources1':'solids'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'gases'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'liquids',
        'final sources1':'gases'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'electricity'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'liquids',
        'final sources1':'electricity'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'heat'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'liquids',
        'final sources1':'heat'}].values,

        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'solids'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'gases',
        'final sources1':'solids'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'electricity'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'gases',
        'final sources1':'electricity'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'heat'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'gases',
        'final sources1':'heat'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'liquids'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'gases',
        'final sources1':'liquids'}].values,

        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'gases'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'solids',
        'final sources1':'gases'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'electricity'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'solids',
        'final sources1':'electricity'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'heat'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'solids',
        'final sources1':'heat'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'liquids'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'solids',
        'final sources1':'liquids'}].values,

        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'solids'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'electricity',
        'final sources1':'solids'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'gases'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'electricity',
        'final sources1':'gases'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'heat'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'electricity',
        'final sources1':'heat'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'liquids'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'electricity',
        'final sources1':'liquids'}].values,

        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'solids'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'heat',
        'final sources1':'solids'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'gases'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'heat',
        'final sources1':'gases'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'electricity'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'heat',
        'final sources1':'electricity'}].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc[{'final sources':'liquids'}].values *
        efficiency_rate_of_substitution_h().loc[{'final sources':'heat',
        'final sources1':'liquids'}].values],
        coords={'final sources':['liquids','gases','solids','electricity','heat']},
        dims=['final sources'])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def maximun_yearly_aceleratuin_of_intensity_improvement_h():
    """
    Maximun yearly aceleratuin of intensity improvement H



    component


    """

    ret =  -maximun_yearly_aceleration_of_intensity_improvement_pct_h(
    ) * initial_energy_intensity_1995_h()
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def evol_final_energy_intensity_h():
    """
    Evol final energy intensity H



    component


    """


    ret =  integ_evol_final_energy_intensity_h()
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def inertial_rate_energy_intensity_h_top_down():
    """
    inertial rate energy intensity H TOP DOWN



    component


    """

    ret =  functions.if_then_else(
        time() > 2008,
        functions.if_then_else(
            (historical_mean_rate_energy_intensity_h() + efficiency_energy_aceleration_h()) < 0,
            evol_final_energy_intensity_h() *
            (historical_mean_rate_energy_intensity_h() + efficiency_energy_aceleration_h()) *
            available_improvement_efficiency_h(),
            initial_energy_intensity_1995_h() *
            (historical_mean_rate_energy_intensity_h() + efficiency_energy_aceleration_h())),
        historic_rate_final_energy_intensity_h())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_co2e_xgwpx100_yearsxx():
    """
    XTotal CO2e XGWPX100 yearsXX



    component


    """

    ret =  total_ch4_emissions_fossil_fuels() * gwp_100_years_ch4(
    ) / 1000 + total_co2_emissions_gtco2()
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def total_co2e_all_ghg():
    """
    Total CO2e all GHG



    component


    """

    ret =  xtotal_co2e_xgwpx100_yearsxx() * correction_factor_all_ghgs()
    name = 1
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
# INDICATORS - Social and Environmental impacts
@cache('step')
def net_tfec_per_capita():
    """
    Net TFEC per capita



    component


    """

    ret =  functions.zidz(real_tfec() * gj_per_ej() / eroi_fc_tot_from_2015(), population())
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gwp_100_years_ch4():
    """
    GWP 100 years CH4



    constant


    """

    ret =  34
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def correction_factor_all_ghgs():
    """
    correction factor all GHGs



    constant


    """

    ret =  1.22
    name = 1
    return round_sig(x=ret, name=name)

# EROIgrid - Allocation RES elec
@cache('step')
def real_generation_res_elec_var_twh():
    """
    real generation RES elec var TWh



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'wind onshore'}].values + real_generation_res_elec_twh().loc[{'res elec':'wind offshore'}].values + real_generation_res_elec_twh().loc[{'res elec':'solar pv'}].values + real_generation_res_elec_twh().loc[{'res elec':'csp'}].values
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xactivate_energy_scarcity_feedbackxx():
    """
    XActivate energy scarcity feedbackXX



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ELEC - PHS
@cache('step')
def max_potential_phs_twh():
    """
    max potential PHS TWh



    component


    """

    ret =  max_capacity_potential_phs() * cp_phs() / twe_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def replacement_rate_phs():
    """
    replacement rate PHS



    component


    """

    ret =  functions.if_then_else(real_fe_elec_stored_phs_twh() < max_potential_phs_twh(), 1, 0.8)
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def energy_scarcity_feedback_shortage_coeff():
    """
    Energy scarcity feedback shortage coeff



    component


    """

    ret =  functions.if_then_else(abs(xactivate_energy_scarcity_feedbackxx() - 1) < equality_threshold,
                                  np.minimum(1,
                                             real_fe_consumption_by_fuel_before_heat_correction() /
                                             required_fed_by_fuel_before_heat_correction()), 1)
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# EROIgrid - Allocation RES elec
@cache('step')
def real_fe_elec_stored_ev_batteries_twh():
    """
    real FE elec stored EV batteries TWh



    component


    """

    ret =  used_ev_batteries_for_elec_storage() / twe_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

# EROIgrid - Allocation RES elec
@cache('step')
def share_res_elec_generation_stored_endogenous():
    """
    share RES elec generation stored ENDOGENOUS



    component


    """

    ret =  functions.zidz((real_fe_elec_stored_ev_batteries_twh() + real_fe_elec_stored_phs_twh()),
                          real_generation_res_elec_var_twh())
    name = 2
    return round_sig(x=ret, name=name)

# EROI - Required final energy invested RES
@cache('step')
def share_tot_fei_res_elec_var():
    """
    share tot FEI RES elec var



    component


    """

    ret =  total_final_energy_invested_res_elec_var() / total_fei_res()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('step')
def cp_ev_batteries_required():
    """
    Cp EV batteries required



    component


    """

    ret =  np.maximum(0, demand_ev_batteries_for_elec_storage() / ev_batteries_tw())
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - EV batteries
@cache('step')
def output_ev_bateries_for_storage_over_lifetime():
    """
    output EV bateries for storage over lifetime



    component


    """

    ret =  cp_ev_batteries_for_elec_storage() * xnewxreplaced_batteries_twx() * (
        1 / twe_per_twh()) * lifetime_ev_batteries() * ej_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('step')
def discarded_batteries():
    """
    discarded batteries



    component


    """

    ret =  np.maximum(0, functions.zidz(xbatteries_evxhibx2wex(), lifetime_ev_batteries()))
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# EROIgrid - Allocation RES elec
@cache('step')
def esoi_elec_storage():
    """
    ESOI elec storage



    component


    """

    ret =  (esoi_phs() * installed_capacity_phs_tw() + esoi_ev_batteries(
    ) * used_ev_batteries_for_elec_storage()) / total_capacity_elec_storage_tw()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# ESOI - EV batteries
@cache('step')
def esoi_ev_batteries():
    """
    ESOI EV batteries



    component


    """

    ret =  lifetime_ev_batteries() * cp_ev_batteries_for_elec_storage() * mw_in_1_year_to_mj() / (
        quality_of_electricity() * xenergy_intensity_construction_ev_batteries_mjxmwx() *
        (1 + share_energy_requirements_for_decom_ev_batteries(
        ) + grid_correction_factor_ev_batteries()))
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# EROIgrid - Allocation RES elec
@cache('step')
def rt_elec_storage_efficiency():
    """
    rt elec storage efficiency



    component


    """

    ret =  (rt_storage_efficiency_phs() * installed_capacity_phs_tw() +
            rt_storage_efficiency_ev_batteries() * used_ev_batteries_for_elec_storage()
            ) / total_capacity_elec_storage_tw()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xxxabundancexx_storagex():
    """
    XXXabundanceXX storageX



    component


    """

    ret =  1 - functions.if_then_else(single_precision(total_capacity_elec_storage_tw()) > single_precision(demand_storage_capacity()) - less_thr_func(total_capacity_elec_storage_tw(), demand_storage_capacity()), 1,
        np.maximum(0, 1 - (demand_storage_capacity() - total_capacity_elec_storage_tw()) /
                   total_capacity_elec_storage_tw()))
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('step')
def used_ev_batteries_for_elec_storage():
    """
    Used EV batteries for elec storage



    component


    """

    ret =  ev_batteries_tw() * cp_ev_batteries_for_elec_storage()
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - EV batteries
@cache('step')
def max_cp_ev_batteries():
    """
    max Cp EV batteries



    component


    """

    ret =  net_stored_energy_ev_battery_over_lifetime() / (
        min_lifetime_ev_batteries() * mw_in_1_year_to_mj() * (kw_per_battery_ev() / kw_per_mw()))
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('step')
def total_capacity_elec_storage_tw():
    """
    Total capacity elec storage TW



    component


    """

    ret =  installed_capacity_phs_tw() + used_ev_batteries_for_elec_storage()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def adapt_growth_phs():
    """
    adapt growth PHS



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        past_phs_capacity_growth(),
        functions.if_then_else(time() < 2020,
                               past_phs_capacity_growth() +
                               (p_phs_growth() - past_phs_capacity_growth()) *
                               (time_dmnl(time()) - 2015) / 5, p_phs_growth()) *
        (1 + xxxabundancexx_storagex()))

    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('step')
def demand_ev_batteries_for_elec_storage():
    """
    demand EV batteries for elec storage



    component


    """

    ret =  np.maximum(0, demand_storage_capacity() - installed_capacity_phs_tw())
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - PHS
@cache('step')
def ced_per_tw_over_lifetime_phs():
    """
    CED per TW over lifetime PHS



    component


    """

    ret =  functions.zidz((cp_phs() * lifetime_res_elec().loc[{'res elec':'hydro'}].values * ej_per_twh() / twe_per_twh()),
                          esoi_static_phs() * quality_of_electricity_2015())
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def remaining_potential_constraint_on_new_phs_capacity():
    """
    remaining potential constraint on new PHS capacity



    component


    """

    ret =  functions.if_then_else(single_precision(remaining_potential_phs()) > single_precision(threshold_remaining_potential_new_capacity()) + less_thr_func(remaining_potential_phs(), threshold_remaining_potential_new_capacity()), 1,
        remaining_potential_phs() * (1 / threshold_remaining_potential_new_capacity()))
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def new_required_phs_capacity():
    """
    new required PHS capacity



    component


    """

    ret =  functions.if_then_else(time() < (2014 - xtotal_time_planxconstr_res_elecx().loc[{'res elec':'hydro'}].values),
                                  historic_new_required_capacity_phs(),
                                  installed_capacity_phs_tw() * adapt_growth_phs() *
                                  remaining_potential_constraint_on_new_phs_capacity())
    name = 2
    return round_sig(x=ret, name=name)

# EROI - Required final energy invested RES
@cache('step')
def share_fei_for_res_vs_tfec():
    """
    share FEI for RES vs TFEC



    component


    """

    ret =  total_fei_res() / real_tfec()
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - EV batteries
# EROI - Required final energy invested RES
@cache('step')
def fei_ev_batteries():
    """
    FEI EV batteries



    component


    """

    ret =  functions.zidz(output_ev_bateries_for_storage_over_lifetime(), esoi_ev_batteries())
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ESOI - PHS
@cache('step')
def output_phs_over_lifetime():
    """
    output PHS over lifetime



    component


    """

    ret =  cp_phs() * phs_capacity_under_construction() * (
        1 / twe_per_twh()) * lifetime_res_elec().loc[{'res elec':'hydro'}].values * ej_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - PHS
@cache('step')
def esoi_static_phs():
    """
    ESOI static PHS



    component


    """

    ret =  a_lineal_regr() * installed_capacity_phs_tw() + b_lineal_regr()
    name = 2
    return round_sig(x=ret, name=name)

# EROI - Required final energy invested RES
# EROI system
@cache('step')
def total_fei_res():
    """
    Total FEI RES



    component


    """

    ret =  total_final_energy_invested_res_elec_var() + fei_ev_batteries(
    ) + total_fei_over_lifetime_res_elec_dispatch() + final_energy_invested_phs()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def adapt_growth_res_elec_after_allocation():
    """
    adapt growth RES elec after allocation



    component


    """

    ret =  functions.if_then_else(abs(activate_eroi_allocation_rule(
    ) ) < equality_threshold, adapt_growth_res_elec(), adapt_growth_res_elec() * eroi_allocation_rule_per_res_elec()
                                  ) * constraint_elec_storage_availability()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def replacement_capacity_res_elec():
    """
    replacement capacity RES elec



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, 0,
        replacement_rate_res_elec() * wear_res_elec() *
        (1 - res_elec_tot_overcapacity())) * constraint_elec_storage_availability()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# ESOI - PHS
@cache('step')
def esoi_phs():
    """
    ESOI PHS



    component


    """

    ret =  functions.zidz(output_phs_over_lifetime(),
                          cedtot_over_lifetime_phs() * quality_of_electricity())
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('run')
def p_phs_growth():
    """
    P PHS growth



    constant


    """

    ret =  0.023
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# ELEC RES - Capacities and generation all techn
@cache('step')
def constraint_elec_storage_availability():
    """
    constraint elec storage availability



    component


    """

    ret =  functions.if_then_else(
        abs(xres_elec_variablesxx() ) < equality_threshold, 1,
        functions.if_then_else(single_precision(total_capacity_elec_storage_tw()) > single_precision(demand_storage_capacity()) - less_thr_func(total_capacity_elec_storage_tw(), demand_storage_capacity()), 1,
            np.maximum(0, 1 - (demand_storage_capacity() - total_capacity_elec_storage_tw()) /
                       total_capacity_elec_storage_tw())))
    name = 2
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def eroist_tot():
    """
    EROIst tot



    component


    """

    ret =  real_tfec() / (((xenergy_ownxuse_constant_830_mtoe_in_2013x() / mtoe_per_ej()) / 375.5
                           ) * real_tfec() + total_fei_res()) - test_ir_restando_al_eroist_tot()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('run')
def initial_instal_cap_phs():
    """
    initial instal cap PHS



    constant


    """

    ret =  0.0916300043623243
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - PHS
@cache('step')
def cedtot_over_lifetime_phs():
    """
    CEDtot over lifetime PHS



    component


    """

    ret =  phs_capacity_under_construction() * ced_per_tw_over_lifetime_phs()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# ELEC - PHS
# ESOI - PHS
@cache('step')
def installed_capacity_phs_tw():
    """
    installed capacity PHS TW



    component


    """


    ret =  integ_installed_capacity_phs_tw()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ESOI - PHS
@cache('step')
def phs_capacity_under_construction():
    """
    PHS capacity under construction



    component


    """

    ret =  phs_planned_capacity() / time_construction_res_elec().loc[{'res elec':'hydro'}].values
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('run')
def past_phs_capacity_growth():
    """
    past PHS capacity growth



    constant


    """

    ret =  0.023
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def new_phs_capacity_under_planning():
    """
    new PHS capacity under planning



    component


    """

    ret =  required_capacity_phs() / time_planification_res_elec().loc[{'res elec':'hydro'}].values
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def wear_phs():
    """
    wear PHS



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0,
                                  installed_capacity_phs_tw() / lifetime_res_elec().loc[{'res elec':'hydro'}].values)
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def replacement_capacity_phs():
    """
    replacement capacity PHS



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, replacement_rate_phs() * wear_phs())
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ELEC - PHS
# ESOI - PHS
# EROIgrid - Allocation RES elec
@cache('step')
def real_fe_elec_stored_phs_twh():
    """
    real FE elec stored PHS TWh



    component


    """

    ret =  installed_capacity_phs_tw() * cp_phs() / twe_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def phs_planned_capacity():
    """
    PHS planned capacity



    component


    """


    ret =  integ_phs_planned_capacity()
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - PHS
# EROI - Required final energy invested RES
@cache('step')
def final_energy_invested_phs():
    """
    Final energy invested PHS



    component


    """

    ret =  real_fe_elec_stored_phs_twh() * ej_per_twh() / esoi_phs()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def remaining_potential_phs():
    """
    remaining potential PHS



    component


    """

    ret =  functions.if_then_else(single_precision(max_capacity_potential_phs()) > single_precision(installed_capacity_phs_tw()) + less_thr_func(max_capacity_potential_phs(), installed_capacity_phs_tw()), (
        max_capacity_potential_phs() - installed_capacity_phs_tw()) / max_capacity_potential_phs(),
                                  0)
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def historic_new_required_capacity_phs():
    """
    Historic new required capacity PHS



    component


    """

    ret =  table_hist_capacity_phs(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'hydro'}].values + 1) - table_hist_capacity_phs(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'hydro'}].values)
    name = 2
    return round_sig(x=ret, name=name)
# ELEC - PHS

def table_hist_capacity_phs(x):
    """
    table hist capacity PHS



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.0916300043623243, 0.0933800056375377, 0.0915700044762995, 0.095060004532393,
        0.0963800032561458, 0.0979700041995104, 0.099380003724601, 0.0988500056353863,
        0.0984100053188484, 0.102080005003214, 0.103850004442362, 0.10771000355781,
        0.112910003772387, 0.117070004867921, 0.12167000599687, 0.123550007094676,
        0.130110007937634, 0.132230006731795, 0.1332792609425959, 0.13455522053152233,
        0.1379890354996751
    ])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def initial_capacity_in_construction_phs():
    """
    initial capacity in construction PHS



    component


    """

    ret =  initial_required_capacity_phs()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('step')
def required_capacity_phs():
    """
    required capacity PHS



    component


    """


    ret =  integ_required_capacity_phs()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
@cache('run')
def initial_required_capacity_phs():
    """
    initial required capacity PHS



    constant


    """

    ret =  table_hist_capacity_phs(1996) - table_hist_capacity_phs(1995)
    name = 2
    return round_sig(x=ret, name=name)

# EROI - Required final energy invested RES
@cache('step')
def total_fei_over_lifetime_res_elec_dispatch():
    """
    Total FEI over lifetime RES elec dispatch



    component


    """

    ret =  functions.sum(fei_over_lifetime_res_elec_dispatch())
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - EV batteries
@cache('run')
def min_lifetime_ev_batteries():
    """
    min lifetime EV batteries



    constant


    """

    ret =  5
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def total_transport_fed_by_fuel():
    """
    Total transport FED by fuel



    component


    """

    ret =  total_required_final_energy_transport_iot_sectors(
    ) + transport_households_final_energy_demand()
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def total_required_final_energy_transport_iot_sectors():
    """
    Total required final energy transport IOT sectors



    component


    """

    ret =  required_final_energy_air_transport() + required_final_energy_inland_transport(
    ) + required_final_energy_other_transport() + required_final_energy_water_transport()
    name = 2
    return round_sig(x=ret, name=name)

# EROI - RES elec
# EROI - Required final energy invested RES
@cache('step')
def total_final_energy_invested_res_elec_var():
    """
    Total final energy invested RES elec var



    component


    """

    ret =  functions.sum(fei_res_elec_var())
    name = 2
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for EV batteries
# MATERIALS - Energy consumption
@cache('step')
def materials_required_for_ev_batteries_mt():
    """
    materials required for EV batteries Mt



    component


    """

    ret =  xnewxreplaced_batteries_twx() * xmaterials_per_new_capacity_installed_x_ev_batteriesx(
    ) * m_per_t() / kg_per_mt()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xnewxreplaced_batteries_twx():
    """
    XnewXreplaced batteries TWX



    component


    """
    ret =  (new_batteries() + replacement_batteries()) * kw_per_battery_ev() / kwh_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xenergy_intensity_construction_ev_batteries_mjxmwx():
    """
    XEnergy intensity construction EV batteries MJXMWX



    component


    """

    ret =  functions.zidz(
        total_energy_required_for_total_material_consumption_for_ev_batteries() * mj_per_ej(),
        xnewxreplaced_batteries_twx() * m_per_t())
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('step')
def new_batteries():
    """
    new batteries



    component


    """
    #ret =  5 * xbatteries_evxhibx2wex() * (
    #    1 - (xbatteries_evxhibx2wex() / required_number_standard_batteries()))
    ret = required_number_standard_batteries() - xbatteries_evxhibx2wex()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xbatteries_evxhibx2wex():
    """
    Xbatteries EVXhibX2wEX



    component


    """
    ret =  integ_xbatteries_evxhibx2wex() + 0*xbatteries_evxhibx2wex2()
    #ret =  xbatteries_evxhibx2wex2()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xbatteries_evxhibx2wex2():
    """
    Xbatteries EVXhibX2wEX2



    component


    """

    ret =  np.exp(5*(time()-initial_time()))/(1+5*integ_xbatteries_evxhibx2wex2())
    name = 2
    return round_sig(x=ret, name=name)


# ESOI - EV batteries
@cache('run')
def kw_per_mw():
    """
    kW per MW



    constant


    """

    ret =  1000
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - EV batteries
@cache('run')
def net_stored_energy_ev_battery_over_lifetime():
    """
    Net stored energy EV battery over lifetime



    constant


    """

    ret =  138024
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - EV batteries
@cache('run')
def grid_correction_factor_ev_batteries():
    """
    Grid correction factor EV batteries



    constant


    """

    ret =  0.1
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - EV batteries
@cache('run')
def share_energy_requirements_for_decom_ev_batteries():
    """
    Share energy requirements for decom EV batteries



    constant


    """

    ret =  0.1
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Social and Environmental impacts
@cache('step')
def working_hours_per_year():
    """
    Working hours per year



    component


    """

    ret =  working_hours_per_day() * days_per_year()
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Social and Environmental impacts
@cache('run')
def days_per_year():
    """
    days per year



    constant


    """

    ret =  365
    name = 2
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
@cache('step')
def total_energy_required_for_material_consumption_for_res_elec():
    """
    Total energy required for material consumption for RES elec



    component


    """

    ret =  functions.sum(energy_required_for_material_consumption_per_res_elec())
    name = 2
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
@cache('step')
def total_energy_required_for_material_consumption_per_res_elec():
    """
    Total energy required for material consumption per RES elec



    component


    """

    ret =  functions.sum(dim='materials', data=energy_required_for_material_consumption_per_res_elec())
    name = 2
    return round_sig(x=ret, name=name)

# ESOI - EV batteries
@cache('run')
def mw_in_1_year_to_mj():
    """
    MW in 1 year to MJ



    constant


    """

    ret =  31536000
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Social and Environmental impacts
@cache('run')
def working_hours_per_day():
    """
    Working hours per day



    constant


    """

    ret =  8
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Social and Environmental impacts
@cache('step')
def annual_work_hours_for_res():
    """
    Annual work hours for RES



    component


    """

    ret =  total_jobs_res() * working_hours_per_year()
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Social and Environmental impacts
@cache('step')
def hours_work_per_gj_res_delivered():
    """
    Hours work per GJ RES delivered



    component


    """

    ret =  annual_work_hours_for_res() / (tfec_res_ej() * gj_per_ej())
    name = 2
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
# ESOI - EV batteries
@cache('step')
def total_energy_required_for_total_material_consumption_for_ev_batteries():
    """
    Total energy required for total material consumption for EV batteries



    component


    """

    ret =  functions.sum(energy_required_for_material_consumption_for_ev_batteries())
    name = 2
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
# EROI - Required final energy invested RES
@cache('step')
def total_energy_required_for_total_material_consumption_for_alt_techn():
    """
    Total energy required for total material consumption for alt techn



    component


    """

    ret =  total_energy_required_for_material_consumption_for_res_elec(
    ) + total_energy_required_for_total_material_consumption_for_ev_batteries()
    name = 2
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
@cache('step')
def energy_required_for_material_consumption_per_res_elec():
    """
    Energy required for material consumption per RES elec



    component


    """

    ret =  xenergy_required_for_material_consumption_for_oxm_res_elecx(
    ) + energy_required_for_material_consumption_for_new_res_elec()
    name = 2
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
@cache('step')
def total_energy_required_per_material_for_alt_techn():
    """
    Total energy required per material for alt techn



    component


    """

    ret =  functions.sum(dim='res elec', data=energy_required_for_material_consumption_per_res_elec()
                          ) + energy_required_for_material_consumption_for_ev_batteries()
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('step')
def total_jobs_res_elec():
    """
    Total jobs RES elec



    component


    """

    ret =  functions.sum(xtotal_dxi_jobs_res_elec_per_technx())
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('step')
def total_jobs_biofuels():
    """
    total jobs biofuels



    component


    """

    ret =  employment_factor_biofuels() * fes_total_biofuels_production_ej()
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
# INDICATORS - Social and Environmental impacts
@cache('step')
def total_jobs_res():
    """
    Total jobs RES



    component


    """

    ret =  total_jobs_res_elec() + total_jobs_res_heat() + d_jobs_fuel_supply_solids_bioe(
    ) + total_jobs_biofuels()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_dxi_jobs_res_heat_per_technx():
    """
    XTotal DXI jobs RES heat per technX



    component


    """

    ret =  total_d_jobs_res_heat_per_techn() * ratio_total_vs_d_jobs_res_heat()
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('step')
def total_jobs_res_heat():
    """
    Total jobs RES heat



    component


    """

    ret =  functions.sum(xtotal_dxi_jobs_res_heat_per_technx())
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_dxi_jobs_res_elec_per_technx():
    """
    XTotal DXI jobs RES elec per technX



    component


    """

    ret =  total_d_jobs_res_elec_per_techn() * ratio_total_vs_d_jobs_res_elec()
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('run')
def ratio_total_vs_d_jobs_res_heat():
    """
    Ratio total vs D jobs RES heat



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[1.], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[1.9], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[1.], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('run')
def ratio_total_vs_d_jobs_res_elec():
    """
    Ratio total vs D jobs RES elec



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[2.6], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[1.9], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[1.], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[1.], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[1.25], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[1.25], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[1.5], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[1.7], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('run')
def employment_factor_biofuels():
    """
    Employment factor biofuels



    constant


    """

    ret =  740231.8591670244
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def installed_capacity_res_elec_delayed_1yr():
    """
    installed capacity RES elec delayed 1yr



    component


    """

    data = [delay_installed_capacity_res_elec_tw_hydro(),
        delay_installed_capacity_res_elec_tw_xgeotxelecx(),
        delay_installed_capacity_res_elec_tw_xsolid_bioexelecx(),
        delay_installed_capacity_res_elec_tw_oceanic(),
        delay_installed_capacity_res_elec_tw_wind_onshore(),
        delay_installed_capacity_res_elec_tw_wind_offshore(),
        delay_installed_capacity_res_elec_tw_solar_pv(),
        delay_installed_capacity_res_elec_tw_csp()]
    ret = xr.DataArray(data=data, coords={'res elec': ['hydro', 'xgeotxelecx', 'xsolid bioexelecx', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']}, dims=['res elec'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xjobs_oxm_res_heat_per_technx():
    """
    XJobs OXM RES heat per technX



    component


    """

    ret =  (xinstalled_capacity_res_heatxcom_twx() + xinstalled_capacity_res_heatxnc_twx()
            ) * xemployment_factors_oxm_res_heatx() * m_per_t()
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('step')
def d_jobs_new_installed_res_heat_per_techn():
    """
    D jobs new installed RES heat per techn



    component


    """

    ret =  (xnew_res_capacity_for_heatxcom_twx() + xreplacement_res_for_heatxcom_twx() +
            xnew_res_capacity_for_heatxnc_twx() + xreplacement_res_for_heatxnc_twx()
            ) * employment_factors_new_res_heat() * m_per_t()
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('step')
def d_jobs_fuel_supply_solids_bioe():
    """
    D jobs fuel supply solids bioE



    component


    """

    ret =  pes_solids_bioe_ej() * employment_factor_fuel_supply_solids_bioe() * 1000
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def new_capacity_installed_growth_rate_res_elec():
    """
    new capacity installed growth rate RES elec



    component


    """

    ret =  -1 + functions.zidz(installed_capacity_res_elec_tw(),
                               installed_capacity_res_elec_delayed_1yr())
    name = 2
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('run')
def employment_factor_fuel_supply_solids_bioe():
    """
    Employment factor fuel supply solids bioE



    constant


    """

    ret =  0.029
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xactivate_eroi_tot_fc_feedback_through_intensitiesxx():
    """
    XActivate EROI tot FC feedback through intensitiesXX



    constant


    """

    ret =  0
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def test7():
    """
    test7



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_gases()) > single_precision(abundance_liquids()) - less_thr_func(abundance_gases(), abundance_liquids()), 1, 0)
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def exogenous_growth_gtl():
    """
    Exogenous growth GTL



    component


    """

    ret =  functions.if_then_else(time() < 2015,
                                  hist_growth_gtl(),
                                  functions.if_then_else(abs(xcrash_programme_gtlxx() ) < equality_threshold,
                                                         p_gtl(),
                                                         functions.if_then_else(functions.logical_and(
                                                             abs(xcrash_programme_gtlxx() - 1) < equality_threshold
                                                            , abundance_liquids() >= 1),
                                                             hist_growth_gtl(), p_gtl())))
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def replacement_gtl():
    """
    replacement GTL



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0,
                                  functions.if_then_else(
                                      abs(xcrash_programme_gtlxx() ) < equality_threshold, 0,
                                      functions.if_then_else(
                                          check_liquids() > equality_threshold,
                                          xconstrain_liquids_exogenous_growthxx() * wear_gtl(),
                                          wear_gtl()))) * scarcity_conv_oil()
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xcrash_programme_gtlxx():
    """
    XCrash programme GTLXX



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xactivate_eroi_tot_fc_feedbackxx():
    """
    XActivate EROI tot FC feedbackXX



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def exogenous_growth_ctl():
    """
    Exogenous growth CTL



    component


    """

    ret =  functions.if_then_else(time() < 2015,
                                  hist_growth_ctl(),
                                  functions.if_then_else(abs(xcrash_programme_ctlxx() ) < equality_threshold,
                                                         p_ctl(),
                                                         functions.if_then_else(functions.logical_and(
                                                             abs(xcrash_programme_ctlxx() - 1) < equality_threshold
                                                            , abundance_liquids() >= 1),
                                                             hist_growth_ctl(), p_ctl())))
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def replacement_ctl():
    """
    replacement CTL



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0,
                                  functions.if_then_else(
                                      abs(xcrash_programme_ctlxx() ) < equality_threshold, 0,
                                      functions.if_then_else(
                                          check_liquids() > equality_threshold,
                                          xconstrain_liquids_exogenous_growthxx() * wear_ctl(),
                                          wear_ctl()))) * scarcity_conv_oil()
    name = 2
    return round_sig(x=ret, name=name)

def table_max_extraction_aspo_oil_ej010(x):
    """
    table max extraction ASPO oil EJ010



    lookup


    """

    ret =  functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0, 29.9783, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537, 137.018,
        145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def aux12():
    """
    aux12



    component


    """

    ret =  delay_rurr_coal_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def coal_to_leave_underground():
    """
    coal to leave underground



    component


    """

    ret =  functions.if_then_else(
        time() < start_policy_leave_in_ground_coal(), 0,
        share_rurr_coal_to_leave_underground() * rurr_coal_start_year_plg())
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def rurr_coal_start_year_plg():
    """
    RURR coal start year PLG



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_coal(),
                                  rurr_coal(), aux12())
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def rurr_conv_oil_until_start_year_plg():
    """
    RURR conv oil until start year PLG



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_conv_oil(),
                                  rurr_conv_oil(), aux6())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def conv_gas_to_leave_underground():
    """
    conv gas to leave underground



    component


    """

    ret =  functions.if_then_else(
        time() < start_policy_leave_in_ground_conv_gas(), 0,
        rurr_conv_gas_until_start_year_plg() * share_rurr_conv_gas_to_leave_underground())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def conv_oil_to_leave_underground():
    """
    conv oil to leave underground



    component


    """

    ret =  functions.if_then_else(
        time() < start_policy_leave_in_ground_conv_oil(), 0,
        rurr_conv_oil_until_start_year_plg() * share_rurr_conv_oil_to_leave_underground())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_oil_aspo_0():
    """
    URR oil ASPO 0



    constant


    """

    ret =  12800
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_oil_aspo_0_0():
    """
    URR oil ASPO 0 0



    constant


    """

    ret =  12800
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_oil_aspo_0_1():
    """
    URR oil ASPO 0 1



    constant


    """

    ret =  12800
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def rurr_unconv_oil_until_start_year_plg():
    """
    RURR unconv oil until start year PLG



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_unconv_oil(),
                                  rurr_unconv_oil_ej(), aux7())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def rurr_unconv_gas_until_start_year_plg():
    """
    RURR unconv gas until start year PLG



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_unconv_gas(),
                                  rurr_unconv_gas(), aux10())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def unconv_gas_to_leave_underground():
    """
    unconv gas to leave underground



    component


    """

    ret =  functions.if_then_else(
        time() < start_policy_leave_in_ground_unconv_gas(), 0,
        rurr_unconv_gas_until_start_year_plg() * share_rurr_unconv_gas_to_leave_underground())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def tot_agg_gas_to_leave_underground():
    """
    tot agg gas to leave underground



    component


    """

    ret =  functions.if_then_else(
        time() < start_policy_leave_in_ground_tot_agg_gas(), 0,
        rurr_tot_gas_until_start_year_plg() * share_rurr_tot_agg_gas_to_leave_underground())
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def overdemand_eroist_tot():
    """
    overdemand EROIst tot



    component


    """

    ret =  functions.if_then_else(single_precision(eroist_delayed()) > single_precision(1.1) + less_thr_func(eroist_delayed(), 1.1), 1 / (eroist_delayed() - 1), 5)
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def unconv_oil_to_leave_underground():
    """
    unconv oil to leave underground



    component


    """

    ret =  functions.if_then_else(
        time() < start_policy_leave_in_ground_unconv_oil(), 0,
        rurr_unconv_oil_until_start_year_plg() * share_rurr_unconv_oil_to_leave_underground())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def aux11():
    """
    aux11



    component


    """

    ret =  delay_rurr_tot_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def tot_agg_oil_to_leave_underground():
    """
    tot agg oil to leave underground



    component


    """

    ret =  functions.if_then_else(
        time() < start_policy_leave_in_ground_tot_agg_oil(), 0,
        rurr_tot_oil_until_start_year_plg() * share_rurr_tot_agg_oil_to_leave_underground())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def aux6():
    """
    aux6



    component


    """

    ret =  delay_rurr_conv_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def aux7():
    """
    aux7



    component


    """

    ret =  delay_rurr_unconv_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def aux8():
    """
    aux8



    component


    """

    ret =  delay_rurr_tot_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def aux9():
    """
    aux9



    component


    """

    ret =  delay_rurr_conv_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def rurr_conv_gas_until_start_year_plg():
    """
    RURR conv gas until start year PLG



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_conv_gas(),
                                  rurr_conv_gas(), aux9())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def aux10():
    """
    aux10



    component


    """

    ret =  delay_rurr_unconv_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('run')
def test_ir_restando_al_eroist_tot():
    """
    test ir restando al EROIst tot



    constant


    """

    ret =  0
    name = 3
    return round_sig(x=ret, name=name)

def table_max_extraction_aspo_oil_ej01(x):
    """
    table max extraction ASPO oil EJ01



    lookup


    """

    ret =  functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0, 29.97831, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537,
        137.018, 145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044, 171.044
    ])
    name = 3
    return round_sig(x=ret, name=name)

def table_max_extraction_aspo_oil_ej0(x):
    """
    table max extraction ASPO oil EJ0



    lookup


    """

    ret =  functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0, 29.97832, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537,
        137.018, 145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044, 171.044
    ])
    name = 3
    return round_sig(x=ret, name=name)

def table_max_extraction_aspo_oil_ej00(x):
    """
    table max extraction ASPO oil EJ00



    lookup


    """

    ret =  functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0, 29.97832, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537,
        137.018, 145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044, 171.044
    ])
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def rurr_tot_gas_until_start_year_plg():
    """
    RURR tot gas until start year PLG



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_tot_agg_gas(),
                                  rurr_tot_agg_gas(), aux11())
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def rurr_tot_oil_until_start_year_plg():
    """
    RURR tot oil until start year PLG



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_tot_agg_oil(),
                                  rurr_tot_agg_oil(), aux8())
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def overdemand_eroist_tot_from_2015():
    """
    overdemand EROIst tot from 2015



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0,
                                  overdemand_eroist_tot() - overdemand_eroist_tot_until_2015())
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def overdemand_eroist_tot_until_2015():
    """
    overdemand EROIst tot until 2015



    component


    """

    ret =  functions.if_then_else(time() < 2015, overdemand_eroist_tot(), aux5())
    name = 3
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
# DEM - Adjustment non-commercial heat demand
# ECON- Final demands by sector, households and fuel
# IMPACTS CC - Energy losses function
@cache('step')
def required_fed_by_fuel_before_heat_correction():
    """
    Required FED by fuel before heat correction



    component


    """

    ret =  (required_fed_sectors_by_fuel() + households_final_energy_demand()) * (
        1 + overdemand_elec_eroi_delayed_1yr()) * eroi_fc_tot_from_2015()
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def aux5():
    """
    aux5



    component


    """

    ret =  delay_overdemand_eroist_tot_until_2015_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def eroi_fc_tot_from_2015_2():
    """
    EROI FC tot from 2015 2



    component


    """

    ret =  1 + overdemand_eroist_tot_from_2015()
    name = 3
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC - Storage demand and supply
@cache('step')
def share_elec_demand_covered_by_res():
    """
    share Elec demand covered by RES



    component


    """

    ret =  functions.if_then_else(
        total_fe_elec_demand_twh() > 0,
        np.minimum(1, fe_tot_generation_all_res_elec_twh() / total_fe_elec_demand_twh()), 0.5)
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def rurr_conv_gas():
    """
    RURR conv gas



    component


    """


    ret =  integ_rurr_conv_gas()
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def rurr_unconv_gas():
    """
    RURR unconv gas



    component


    """


    ret =  integ_rurr_unconv_gas()
    name = 3
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def rurr_conv_oil():
    """
    RURR conv oil



    component


    """


    ret =  integ_rurr_conv_oil()
    name = 3
    return round_sig(x=ret, name=name)

@cache('step')
def xvariation_share_transmxdistr_losses_elecx():
    """
    Xvariation share transmXdistr losses elecX



    component


    """

    ret =  xshare_transmxdistr_elec_losses_initialx() * (
        0.0115 * np.exp(4.2297 * share_res_electricity_generation()) - 0.00251)
    name = 3
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
# TRANSP - Transport energy demand
# TRANSP - Alternative transport
@cache('step')
def real_pe_demand_of_liquids_for_transp_ej():
    """
    real PE demand of liquids for Transp EJ



    component


    """

    ret =  np.maximum(demand_liquids_for_transp_ej() - oil_liquids_saved_by_evs_ej() -
                      oil_liquids_saved_by_ngvs(), 0)
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def share_demand_solids_in_transport():
    """
    Share demand solids in transport



    component


    """

    ret =  share_demand_by_fuel_in_transport().loc[{'final sources':'solids'}].values
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def share_demand_gas_in_transport():
    """
    Share demand gas in transport



    component


    """

    ret =  share_demand_by_fuel_in_transport().loc[{'final sources':'gases'}].values
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def share_demand_by_fuel_in_transport():
    """
    Share demand by fuel in transport



    component


    """

    ret =  total_transport_fed_by_fuel() / transport_tfed()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def share_demand_electricity_in_transport():
    """
    Share demand electricity in transport



    component


    """

    ret =  share_demand_by_fuel_in_transport().loc[{'final sources':'electricity'}].values
    name = 3
    return round_sig(x=ret, name=name)

# GASES - PED, PES & FES
# TRANSP - Transport energy demand
@cache('step')
def demand_gas_for_transp_ej():
    """
    demand gas for Transp EJ



    component


    """

    ret =  total_transport_fed_by_fuel().loc[{'final sources':'gases'}].values + oil_liquids_saved_by_ngvs()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def share_demand_heat_in_transport():
    """
    Share demand heat in transport



    component


    """

    ret =  share_demand_by_fuel_in_transport().loc[{'final sources':'heat'}].values
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def share_demand_liquids_in_transport():
    """
    Share demand liquids in transport



    component


    """

    ret =  share_demand_by_fuel_in_transport().loc[{'final sources':'liquids'}].values
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def demand_liquids_for_transp_ej():
    """
    demand liquids for Transp EJ



    component


    """

    ret =  total_transport_fed_by_fuel().loc[{'final sources':'liquids'}].values
    name = 3
    return round_sig(x=ret, name=name)

# INDICATORS - Social and Environmental impacts
@cache('step')
def co2_emissions_per_value_added():
    """
    CO2 emissions per value added



    component


    """

    ret =  total_co2_emissions_gtco2() / gdp()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def liquids_per_x_bus():
    """
    liquids per X bus



    component


    """

    ret =  energy_initial_inland_transport().loc[{'vehiclet':'bus liq'}].values * adjust_energy_for_transport_to_inland_transport(
    ) / initial_xt_inland()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def liquids_per_x_hv():
    """
    liquids per X HV



    component


    """

    ret =  energy_initial_inland_transport().loc[{'vehiclet':'hv liq'}].values * adjust_energy_for_transport_to_inland_transport(
    ) / initial_xt_inland()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def liquids_per_x_lv():
    """
    liquids per X LV



    component


    """

    ret =  energy_initial_inland_transport().loc[{'vehiclet':'lv liq'}].values * adjust_energy_for_transport_to_inland_transport(
    ) / initial_xt_inland()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def energy_per_x_train():
    """
    energy per X train



    component


    """

    ret =  energy_initial_inland_transport().loc[{'vehiclet':'train liq'}].values * adjust_energy_for_transport_to_inland_transport(
    ) / initial_xt_inland()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def adjust_energy_for_transport_to_inland_transport():
    """
    adjust energy for transport to inland transport



    constant


    """

    ret =  0.3767
    name = 3
    return round_sig(x=ret, name=name)

@cache('step')
def xeffects_shortage_elec_on_evxhibx():
    """
    Xeffects shortage elec on EVXhibX



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_electricity()) > single_precision(0.8) + less_thr_func(abundance_electricity(), 0.8),
                                  ((abundance_electricity() - 0.8) * 5)**2, 0)
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def hist_var_percent_h():
    """
    hist var percent H



    constant


    """

    ret =  xr.DataArray(
        data=[0.0,

        functions.if_then_else(time() < 2005, 0,
                               (percent_h_vehicles_initial().loc[{'households vehicles':'hib 4wheels'}].values) / (t_hist_h_transp() - 2005)),

        functions.if_then_else(time() < 2005, 0,
                               (percent_h_vehicles_initial().loc[{'households vehicles':'elec 4wheels'}].values) / (t_hist_h_transp() - 2005)),

        functions.if_then_else(time() < 2005, 0,
                               (percent_h_vehicles_initial().loc[{'households vehicles':'gas 4wheels'}].values) / (t_hist_h_transp() - 2005)),
        0.0,

        functions.if_then_else(time() < 2005, 0,
                               (percent_h_vehicles_initial().loc[{'households vehicles':'elec 2wheels'}].values) / (t_hist_h_transp() - 2005))],
        coords={'households vehicles':['liq 4wheels','hib 4wheels','elec 4wheels','gas 4wheels','liq 2wheels','elec 2wheels']},
        dims=['households vehicles'])
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('step')
def required_number_standard_batteries():
    """
    required number standard batteries



    component


    """

    ret =  bat_number_2w() + bat_number_ev() + bat_number_hib() + 1
    name = 3
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def adjust_exogenous_gdp_growth_rate():
    """
    adjust exogenous GDP growth rate



    constant


    """

    ret =  0.4
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def transport_tfed_energy_intensity():
    """
    Transport TFED energy intensity



    component


    """

    ret =  transport_tfed() / gdp()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Total number vehicles
@cache('step')
def total_number_light_vehicles():
    """
    total number light vehicles



    component


    """

    ret =  number_vehicles_h().loc[{'households vehicles':'liq 4wheels'}].values + number_vehicles_h().loc[{'households vehicles':'hib 4wheels'}].values + number_vehicles_h().loc[{'households vehicles':'elec 4wheels'}].values + number_vehicles_h().loc[{'households vehicles':'gas 4wheels'}].values + vehicles_inlandt().loc[{'vehiclet':'lv liq'}].values + vehicles_inlandt().loc[{'vehiclet':'lv elec'}].values + vehicles_inlandt().loc[{'vehiclet':'lv hib'}].values + vehicles_inlandt().loc[{'vehiclet':'lv gas'}].values
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def transport_tfed():
    """
    Transport TFED



    component


    """

    ret =  functions.sum(total_transport_fed_by_fuel())
    name = 3
    return round_sig(x=ret, name=name)

# FE INTENSITY households
# TRANSP - Transport energy demand
@cache('step')
def transport_households_final_energy_demand():
    """
    Transport households final energy demand



    component


    """

    ret =  energy_intensity_of_households_transport() * household_demand_total() / 1e+006
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
# TRANSP - Alternative transport
# TRANSP - Alternative transport
@cache('step')
def liquid_demand_transport_ej():
    """
    Liquid demand transport EJ



    component


    """

    ret =  transport_households_final_energy_demand().loc[{'final sources':'liquids'}].values + required_final_energy_air_transport().loc[{'final sources':'liquids'}].values + required_final_energy_inland_transport().loc[{'final sources':'liquids'}].values + required_final_energy_other_transport().loc[{'final sources':'liquids'}].values + required_final_energy_water_transport().loc[{'final sources':'liquids'}].values
    name = 3
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
# TRANSP - Alternative transport
@cache('step')
def pe_losses_related_to_elec_transp():
    """
    PE losses related to Elec Transp



    component


    """

    ret =  functions.zidz(
        increase_elec_demand_for_transp_twh() * ej_per_twh() * total_electrical_losses_ej(),
        total_fe_elec_demand_ej())
    name = 3
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_lc():
    """
    variation LC



    component


    """

    ret =  real_demand() * labour_share() * (
        exogenous_gdp_growth_rate_corrected() + growth_labour_share() +
        exogenous_gdp_growth_rate_corrected() * growth_labour_share())
    name = 3
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_cc():
    """
    variation CC



    component


    """

    ret =  capital_share() * (
        exogenous_gdp_growth_rate_corrected() + historic_growth_capital_share() +
        exogenous_gdp_growth_rate_corrected() * historic_growth_capital_share()) * real_demand()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
# ABUND - Abundances by sector
@cache('step')
def increase_elec_demand_for_transp_ej():
    """
    Increase Elec demand for Transp EJ



    component


    """

    ret =  increase_elec_demand_for_transp_twh() * ej_per_twh()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def total_liquid_saved_by_elec_ej():
    """
    Total liquid saved by Elec EJ



    component


    """

    ret =  oil_liquids_saved_by_evs_ej()
    name = 3
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
@cache('step')
def test():
    """
    test



    component


    """
    ret =  (total_fe_elec_generation_twh() - increase_elec_demand_for_transp_twh() *
            (1 + xshare_transmxdistr_elec_lossesx())) / (1 + xshare_transmxdistr_elec_lossesx())
    name = 3
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# LIQUIDS - PED, PES & FES
@cache('step')
def real_fe_consumption_liquids_ej():
    """
    real FE consumption liquids EJ



    component


    """

    ret =  (pes_liquids_ej() + oil_liquids_saved_by_evs_ej() -
            xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'liquids'}].values - other_liquids_required_ej()
            ) * share_liquids_for_final_energy()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def oil_liquids_saved_by_evs_kbd():
    """
    Oil liquids saved by EVs kbd



    component


    """

    ret =  oil_liquids_saved_by_evs_ej() * xmbxd_per_ejxyearx() * 1000
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def savings_pe_transport_ej():
    """
    Savings PE Transport EJ



    component


    """

    ret =  total_liquid_saved_by_elec_ej() - additional_increase_pe_demand_for_transp_elec_ej()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def increase_elec_demand_evs_twh():
    """
    increase Elec demand EVs TWh



    component


    """

    ret =  oil_liquids_saved_by_evs_ej() * k_liquid_elec_bev_twh_ej()
    name = 3
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
# ELEC - Total FE electricity generation
# TRANSP - Alternative transport
@cache('step')
def increase_elec_demand_for_transp_twh():
    """
    increase Elec demand for Transp TWh



    component


    """

    ret =  increase_elec_demand_evs_twh()
    name = 3
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('step')
def share_oil_for_elec():
    """
    share oil for Elec



    component


    """

    ret =  functions.if_then_else(time() < 2010,
                                  hist_share_oil_elec(time()),
                                  functions.if_then_else(
                                      abs(xreduce_oil_share_for_electricityxx() - 1) < equality_threshold,
                                      np.maximum(-0.0029251 * time_dmnl(time()) + 5.91871, 0),
                                      hist_share_oil_elec(time())))
    name = 3
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def co2_emissions_unconv_gas():
    """
    CO2 emissions unconv gas



    component


    """

    ret =  real_extraction_unconv_gas_emissions_relevant_ej() * gco2_per_mj_unconv_gas(
    ) * mj_per_ej() / g_per_gt()
    name = 3
    return round_sig(x=ret, name=name)

@cache('run')
def xreduce_oil_share_for_electricityxx():
    """
    Xreduce oil share for electricityXX



    constant


    """

    ret =  1
    name = 3
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('run')
def max_solar_on_land_mha():
    """
    max solar on land Mha



    constant


    """

    ret =  100
    name = 3
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def max_solar_pv_on_land_mha():
    """
    max solar PV on land MHa



    component


    """

    ret =  max_solar_on_land_mha() - surface_csp_mha()
    name = 3
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES - Potential
@cache('step')
def max_solar_pv_on_land_twe():
    """
    max solar PV on land TWe



    component


    """

    ret =  max_solar_pv_on_land_mha() * power_density_solar_pv()
    name = 3
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def total_ch4_emissions_fossil_fuels():
    """
    Total CH4 emissions fossil fuels



    component


    """

    ret =  ch4_emissions_conv_gas_without_gtl() + ch4_emissions_unconv_gas(
    ) + ch4_emissions_coal_without_ctl() + ch4_emissions_oil() + ch4_emissions_ctl(
    ) + ch4_emissions_gtl()
    name = 3
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def max_csp_on_land_mha():
    """
    max CSP on land MHa



    component


    """

    ret =  max_solar_on_land_mha() - surface_solar_pv_mha()
    name = 3
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def max_csp_twe():
    """
    max CSP TWe



    component


    """

    ret =  max_csp_on_land_mha() * power_density_csp()
    name = 3
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gch4_per_mj_gtl():
    """
    gCH4 per MJ GTL



    constant


    """

    ret =  0.094
    name = 3
    return round_sig(x=ret, name=name)

# HEAT COM- Heat-com demand covered by fossil fuels
@cache('step')
def share_liquids_fot_heat_plants():
    """
    share liquids fot heat plants



    component


    """

    ret =  functions.if_then_else(time() < 2014,
                                  historic_share_liquids_for_heat_plants(time()),
                                  functions.if_then_else(
                                      abs(xreduce_oil_share_for_heatxcomxx() - 1) < equality_threshold,
                                      np.maximum(0, -0.005936 * (time()) + 12.0211),
                                      historic_share_liquids_for_heat_plants(time())))
    name = 3
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def ch4_emissions_gtl():
    """
    CH4 emissions GTL



    component


    """

    ret =  xped_natx_gas_for_gtl_ejx() * gch4_per_mj_gtl() * mj_per_ej() / g_per_mt()
    name = 3
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def ch4_emissions_ctl():
    """
    CH4 emissions CTL



    component


    """

    ret =  extraction_coal_for_ctl_ej() * gch4_per_mj_ctl() * mj_per_ej() / g_per_mt()
    name = 3
    return round_sig(x=ret, name=name)

@cache('run')
def xreduce_oil_share_for_heatxcomxx():
    """
    Xreduce oil share for heatXcomXX



    constant


    """

    ret =  0
    name = 3
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gch4_per_mj_ctl():
    """
    gCH4 per MJ CTL



    constant


    """

    ret =  0.094
    name = 3
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def g_per_gt():
    """
    g per Gt



    constant


    """

    ret =  1000000000000000
    name = 3
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def bioe_co2_emissions():
    """
    BioE CO2 emissions



    component


    """

    ret =  gco2_per_mj_conv_gas() * (
        oil_liquids_saved_by_biofuels_ej() + solid_biofuels_emissions_relevant_ej() +
        xpes_tot_biogas_for_heatxcomx()) * mj_per_ej() / g_per_gt()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def maximum_2w():
    """
    maximum 2w



    component


    """

    ret =  p_share_2_wheelers()
    name = 3
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def p_h_vehicle():
    """
    P H vehicle



    component


    """

    ret =  xr.DataArray(
        data=[(1 - p_share_2_wheelers()) * (-p_elec() - p_gas() - p_hyb()),

        p_elec() * (1 - p_share_2_wheelers()),

        p_hyb() * (1 - p_share_2_wheelers()),

        p_gas() * (1 - p_share_2_wheelers()),

        p_share_2_wheelers() * (1 - p_2we()),

        p_share_2_wheelers() * p_2we()],
        coords={'households vehicles':['liq 4wheels','elec 4wheels','hib 4wheels','gas 4wheels','liq 2wheels','elec 2wheels']},
        dims=['households vehicles'])
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def eroist_delayed_1yr():
    """
    EROIst delayed 1yr



    component


    """

    ret =  delay_eroist_tot_round1__time_step___time_step_xeroistx1995xx_1__time_step()
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def eroist_delayed():
    """
    EROIst delayed



    component


    """

    ret =  delay_eroist_tot_roundtime_step__time_step___time_step_xeroistx1995xx_time_step__time_step(
    )
    name = 3
    return round_sig(x=ret, name=name)

# PES solids biofuels & waste
# IMPACTS - Total GHG emissions
@cache('step')
def solid_biofuels_emissions_relevant_ej():
    """
    solid biofuels emissions relevant EJ



    component


    """

    ret =  pe_bioe_for_elec_generation_ej() + xpes_res_for_heatxcom_by_technx().loc[{'res heat':'xsolid bioexheatx'}].values + xpes_res_for_heatxnc_by_technx().loc[{'res heat':'xsolid bioexheatx'}].values
    name = 3
    return round_sig(x=ret, name=name)

@cache('run')
def xeroistx1995xx():
    """
    XEROIstX1995XX



    constant


    """

    ret =  10.79
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def eroist_until_2015():
    """
    EROIst until 2015



    component


    """

    ret =  functions.if_then_else(time() < 2015, eroist_delayed(), aux4())
    name = 3
    return round_sig(x=ret, name=name)

# EROI system
@cache('step')
def aux4():
    """
    aux4



    component


    """

    ret =  delay_eroist_until_2015_roundtime_step__time_step___time_step_xeroistx1995xx_time_step__time_step(
    )
    name = 3
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('step')
def remaining_potential_elec_storage_by_res_techn():
    """
    remaining potential elec storage by RES techn



    component


    """

    ret =  functions.if_then_else(single_precision(max_capacity_elec_storage()) > single_precision(demand_storage_capacity()) - less_thr_func(max_capacity_elec_storage(), demand_storage_capacity()),
        (max_capacity_elec_storage() - demand_storage_capacity()) / max_capacity_elec_storage(), 0)
    name = 3
    return round_sig(x=ret, name=name)

# ESOI - PHS
@cache('run')
def cp_hydro_2015():
    """
    Cp hydro 2015



    constant


    """

    ret =  0.42302360615086493
    name = 3
    return round_sig(x=ret, name=name)

# ESOI - PHS
@cache('step')
def esoi_phs_full_potential():
    """
    ESOI PHS full potential



    component


    """

    ret =  xeroixini_res_elec_dispatchx().loc[{'res elec':'hydro'}].values * (cp_phs() / cp_hydro_2015())
    name = 3
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('step')
def max_capacity_elec_storage():
    """
    max capacity elec storage



    component


    """

    ret =  max_capacity_potential_phs() + used_ev_batteries_for_elec_storage()
    name = 3
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def cc_impacts_feedback_shortage_coeff():
    """
    CC impacts feedback shortage coeff



    component


    """

    ret =  (1 - xshare_exlosses_ccx())
    name = 4
    return round_sig(x=ret, name=name)

@cache('run')
def xeolxrr_minerals_alt_techn_res_vsx_total_economyx():
    """
    XEOLXRR minerals alt techn RES vsX total economyX



    constant


    """

    ret =  0.3333333333333333
    name = 4
    return round_sig(x=ret, name=name)

@cache('run')
def xcrash_programme_ctlxx():
    """
    XCrash programme CTLXX



    constant


    """

    ret =  1
    name = 4
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
# MATERIALS - Rest demand
@cache('step')
def recycling_rates_minerals_rest():
    """
    recycling rates minerals Rest



    component


    """


    ret =  integ_recycling_rates_minerals_rest()
    name = 4
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
# MATERIALS - Demand for RES ELEC
# MATERIALS - Demand for EV batteries
# MATERIALS - Energy consumption
@cache('step')
def recycling_rates_minerals_alt_techn():
    """
    recycling rates minerals alt techn



    component


    """


    ret =  integ_recycling_rates_minerals_alt_techn()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def p_elec():
    """
    P elec



    constant


    """

    ret =  0.0064
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
# TRANSP - Households transport
@cache('step')
def percents_h_vehicles():
    """
    percents H vehicles



    component


    """


    ret =  integ_percents_h_vehicles()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def t_hist_h_transp():
    """
    T hist H transp



    constant


    """

    ret =  2015
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('step')
def bat_number_2w():
    """
    bat number 2w



    component


    """

    ret =  number_vehicles_h().loc[{'households vehicles':'elec 2wheels'}].values * bateries_ratio_2w_e()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('run')
def bateries_ratio_2w_e():
    """
    bateries ratio 2w E



    constant


    """

    ret =  0.29780564263322884
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('step')
def bat_number_ev():
    """
    bat number EV



    component


    """

    ret =  number_vehicles_h().loc[{'households vehicles':'elec 4wheels'}].values + vehicles_inlandt().loc[{'vehiclet':'lv elec'}].values + vehicles_inlandt().loc[{'vehiclet':'bus elec'}].values * bateries_ratio_bus_e()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('step')
def bat_number_hib():
    """
    bat number hib



    component


    """

    ret =  vehicles_inlandt().loc[{'vehiclet':'lv hib'}].values * bateries_ratio_hib_lv(
    ) + vehicles_inlandt().loc[{'vehiclet':'hv hib'}].values * bateries_ratio_hib_hv() + vehicles_inlandt().loc[{'vehiclet':'bus hib'}].values * bateries_ratio_hib_bus(
    ) + number_vehicles_h().loc[{'households vehicles':'hib 4wheels'}].values * bateries_ratio_hib_lv()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('run')
def bateries_ratio_bus_e():
    """
    bateries ratio bus E



    constant


    """

    ret =  9.801724137931034
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('run')
def bateries_ratio_hib_bus():
    """
    bateries ratio hib bus



    constant


    """

    ret =  0.6573926316519808
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('run')
def bateries_ratio_hib_hv():
    """
    bateries ratio hib HV



    constant


    """

    ret =  0.8382058284923753
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
@cache('run')
def bateries_ratio_hib_lv():
    """
    bateries ratio hib LV



    constant


    """

    ret =  0.10223304298097888
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Total number vehicles
@cache('step')
def total_number_hybrid_light_vehicles():
    """
    total number hybrid light vehicles



    component


    """

    ret =  number_vehicles_h().loc[{'households vehicles':'hib 4wheels'}].values + vehicles_inlandt().loc[{'vehiclet':'lv hib'}].values
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Total number vehicles
@cache('step')
def total_number_elec_light_vehicles():
    """
    total number elec light vehicles



    component


    """

    ret =  number_vehicles_h().loc[{'households vehicles':'elec 4wheels'}].values + vehicles_inlandt().loc[{'vehiclet':'lv elec'}].values
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Total number vehicles
@cache('step')
def total_number_gas_light_vehicles():
    """
    total number gas light vehicles



    component


    """

    ret =  number_vehicles_h().loc[{'households vehicles':'gas 4wheels'}].values + vehicles_inlandt().loc[{'vehiclet':'lv gas'}].values
    name = 4
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# TRANSP - Electric batteries
@cache('step')
def ev_batteries_tw():
    """
    EV batteries TW



    component


    """

    ret =  xbatteries_evxhibx2wex() * kw_per_battery_ev() / kwh_per_twh()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def var_percent_t_vehicles():
    """
    var percent T vehicles



    component


    """

    ret =  xr.DataArray(
        data=[-adapt_var_inlandt().loc[{'vehiclet':'hv hib'}].values - adapt_var_inlandt().loc[{'vehiclet':'hv gas'}].values,

        adapt_var_inlandt().loc[{'vehiclet':'hv hib'}].values,

        adapt_var_inlandt().loc[{'vehiclet':'hv gas'}].values,

        -adapt_var_inlandt().loc[{'vehiclet':'lv hib'}].values - adapt_var_inlandt().loc[{'vehiclet':'lv elec'}].values - adapt_var_inlandt().loc[{'vehiclet':'lv gas'}].values,

        adapt_var_inlandt().loc[{'vehiclet':'lv elec'}].values,

        adapt_var_inlandt().loc[{'vehiclet':'lv hib'}].values,

        adapt_var_inlandt().loc[{'vehiclet':'lv gas'}].values,

        -adapt_var_inlandt().loc[{'vehiclet':'bus elec'}].values - adapt_var_inlandt().loc[{'vehiclet':'bus hib'}].values - adapt_var_inlandt().loc[{'vehiclet':'bus gas'}].values,

        adapt_var_inlandt().loc[{'vehiclet':'bus hib'}].values,

        adapt_var_inlandt().loc[{'vehiclet':'bus gas'}].values,

        -adapt_var_inlandt().loc[{'vehiclet':'train elec'}].values,

        adapt_var_inlandt().loc[{'vehiclet':'train elec'}].values,

        adapt_var_inlandt().loc[{'vehiclet':'bus elec'}].values],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv hib','lv gas','bus liq','bus hib','bus gas','train liq','train elec','bus elec']},
        dims=['vehiclet'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def p_train_elec():
    """
    P train elec



    constant


    """

    ret =  0.2
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def p_hv_gas():
    """
    P HV gas



    constant


    """

    ret =  0.00045
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def t_ini_inlandt():
    """
    T ini inlandT



    constant


    """

    ret =  2020
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def p_inlandt():
    """
    P inlandT



    component


    """

    ret =  xr.DataArray(
        data=[-p_hv_gas() - p_hv_hyb(),

        p_hv_hyb(),

        p_hv_gas(),

        -p_lv_elec() - p_lv_hyb() - p_lv_gas(),

        p_lv_elec(),

        p_lv_gas(),

        -p_bus_hyb() - p_bus_gas() - p_bus_elec(),

        p_bus_elec(),

        p_bus_hyb(),

        p_bus_gas(),

        -p_train_elec(),

        p_train_elec(),

        p_lv_hyb()],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv gas','bus liq','bus elec','bus hib','bus gas','train liq','train elec','lv hib']},
        dims=['vehiclet'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def activate_policy_inlandt():
    """
    Activate policy inlandT



    constant


    """

    ret =  1
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def p_lv_elec():
    """
    P LV elec



    constant


    """

    ret =  0.00074
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def p_lv_hyb():
    """
    P LV hyb



    constant


    """

    ret =  0.00036
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def p_bus_elec():
    """
    P bus elec



    constant


    """

    ret =  0
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def p_bus_gas():
    """
    P bus gas



    constant


    """

    ret =  0
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def p_bus_hyb():
    """
    P bus hyb



    constant


    """

    ret =  0
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def percents_bus():
    """
    percents bus



    constant


    """

    ret =  xr.DataArray(
        data=[0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,

        percent_t_vehicles().loc[{'vehiclet':'bus liq'}].values,

        percent_t_vehicles().loc[{'vehiclet':'bus elec'}].values,

        percent_t_vehicles().loc[{'vehiclet':'bus hib'}].values,

        percent_t_vehicles().loc[{'vehiclet':'bus gas'}].values,
        0.0,
        0.0,

        percent_t_vehicles().loc[{'vehiclet':'lv hib'}].values],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv gas','bus liq','bus elec','bus hib','bus gas','train liq','train elec','lv hib']},
        dims=['vehiclet'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def percents_train():
    """
    percents train



    constant


    """

    ret =  xr.DataArray(
        data=[0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,

        percent_t_vehicles().loc[{'vehiclet':'train liq'}].values,

        percent_t_vehicles().loc[{'vehiclet':'train elec'}].values,

        percent_t_vehicles().loc[{'vehiclet':'lv hib'}].values],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv gas','bus liq','bus elec','bus hib','bus gas','train liq','train elec','lv hib']},
        dims=['vehiclet'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def p_hv_hyb():
    """
    P HV hyb



    constant


    """

    ret =  0.00045
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
# TRANSP - Inland transport sector
@cache('step')
def adapt_var_inlandt():
    """
    adapt var inlandT



    component


    """

    ret =  aux_p_inland_transp() * efects_shortage_inlandt()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def percents_lv():
    """
    percents LV



    constant


    """

    ret =  xr.DataArray(
        data=[0.0,
        0.0,
        0.0,

        percent_t_vehicles().loc[{'vehiclet':'lv liq'}].values,

        percent_t_vehicles().loc[{'vehiclet':'lv elec'}].values,

        percent_t_vehicles().loc[{'vehiclet':'lv gas'}].values,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,

        percent_t_vehicles().loc[{'vehiclet':'lv hib'}].values],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv gas','bus liq','bus elec','bus hib','bus gas','train liq','train elec','lv hib']},
        dims=['vehiclet'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def var_i_inland_elec():
    """
    var I inland Elec



    component


    """

    ret =  energy_per_x_t().loc[{'vehiclet':'lv elec'}].values * var_percent_t_vehicles().loc[{'vehiclet':'lv elec'}].values + energy_per_x_t().loc[{'vehiclet':'train elec'}].values * var_percent_t_vehicles().loc[{'vehiclet':'train elec'}].values + energy_per_x_t().loc[{'vehiclet':'bus elec'}].values * var_percent_t_vehicles().loc[{'vehiclet':'bus elec'}].values
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def var_i_inlandt_gas():
    """
    var I inlandT Gas



    component


    """

    ret =  energy_per_x_t().loc[{'vehiclet':'hv gas'}].values * var_percent_t_vehicles().loc[{'vehiclet':'hv gas'}].values + energy_per_x_t().loc[{'vehiclet':'bus gas'}].values * var_percent_t_vehicles().loc[{'vehiclet':'bus gas'}].values + energy_per_x_t().loc[{'vehiclet':'lv gas'}].values * var_percent_t_vehicles().loc[{'vehiclet':'lv gas'}].values
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def var_i_inlandt_liq():
    """
    var I inlandT liq



    component


    """

    ret =  energy_per_x_t().loc[{'vehiclet':'hv liq'}].values * var_percent_t_vehicles().loc[{'vehiclet':'hv liq'}].values + energy_per_x_t().loc[{'vehiclet':'lv liq'}].values * var_percent_t_vehicles().loc[{'vehiclet':'lv liq'}].values + energy_per_x_t().loc[{'vehiclet':'bus liq'}].values * var_percent_t_vehicles().loc[{'vehiclet':'bus liq'}].values + energy_per_x_t().loc[{'vehiclet':'hv liq'}].values * var_percent_t_vehicles().loc[{'vehiclet':'hv hib'}].values + energy_per_x_t().loc[{'vehiclet':'lv liq'}].values * var_percent_t_vehicles().loc[{'vehiclet':'lv hib'}].values + energy_per_x_t().loc[{'vehiclet':'bus liq'}].values * var_percent_t_vehicles().loc[{'vehiclet':'bus hib'}].values + energy_per_x_t().loc[{'vehiclet':'train liq'}].values * var_percent_t_vehicles().loc[{'vehiclet':'train liq'}].values
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def shares_available_t():
    """
    shares available T



    component


    """

    ret =  xr.DataArray(
        data=[1 - (percent_t_vehicles().loc[{'vehiclet':'hv hib'}].values + percent_t_vehicles().loc[{'vehiclet':'hv gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'hv hib'}].values + percent_t_vehicles().loc[{'vehiclet':'hv gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'hv hib'}].values + percent_t_vehicles().loc[{'vehiclet':'hv gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'lv elec'}].values + percent_t_vehicles().loc[{'vehiclet':'lv hib'}].values + percent_t_vehicles().loc[{'vehiclet':'lv gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'lv elec'}].values + percent_t_vehicles().loc[{'vehiclet':'lv hib'}].values + percent_t_vehicles().loc[{'vehiclet':'lv gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'lv elec'}].values + percent_t_vehicles().loc[{'vehiclet':'lv hib'}].values + percent_t_vehicles().loc[{'vehiclet':'lv gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'lv elec'}].values + percent_t_vehicles().loc[{'vehiclet':'lv hib'}].values + percent_t_vehicles().loc[{'vehiclet':'lv gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'bus elec'}].values + percent_t_vehicles().loc[{'vehiclet':'bus hib'}].values + percent_t_vehicles().loc[{'vehiclet':'bus gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'bus elec'}].values + percent_t_vehicles().loc[{'vehiclet':'bus hib'}].values + percent_t_vehicles().loc[{'vehiclet':'bus gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'bus elec'}].values + percent_t_vehicles().loc[{'vehiclet':'bus hib'}].values + percent_t_vehicles().loc[{'vehiclet':'bus gas'}].values),

        1 - (percent_t_vehicles().loc[{'vehiclet':'bus elec'}].values + percent_t_vehicles().loc[{'vehiclet':'bus hib'}].values + percent_t_vehicles().loc[{'vehiclet':'bus gas'}].values),

        1 - percent_t_vehicles().loc[{'vehiclet':'train elec'}].values,

        1 - percent_t_vehicles().loc[{'vehiclet':'train elec'}].values],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv gas','lv hib','bus liq','bus elec','bus hib','bus gas','train liq','train elec']},
        dims=['vehiclet'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
# TRANSP - Total number vehicles
# TRANSP - Electric batteries
@cache('step')
def vehicles_inlandt():
    """
    vehicles inlandT



    component


    """

    ret =  percent_t_vehicles() * real_total_output_inland_transport(
    ) * nx0_vehicles_per_xinland_t()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
# TRANSP - Inland transport sector
@cache('step')
def percent_t_vehicles():
    """
    percent T vehicles



    component


    """

    ret =  integ_percent_t_vehicles()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def efects_shortage_inlandt():
    """
    Efects shortage inlandT



    constant


    """

    ret =  xr.DataArray(
        data=[1.0,
        1.0,

        effects_shortage_gas(),
        1.0,

        effects_shortage_elec_on_ev(),

        effects_shortage_gas(),
        1.0,

        effects_shortage_elec_on_ev(),
        1.0,

        effects_shortage_gas(),
        1.0,

        effects_shortage_elec_on_ev(),
        1.0],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv gas','bus liq','bus elec','bus hib','bus gas','train liq','train elec','lv hib']},
        dims=['vehiclet'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def p_lv_gas():
    """
    P LV gas



    constant


    """

    ret =  0.01597
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def percents_hv():
    """
    percents HV



    component


    """

    ret =  xr.DataArray(
        data=[percent_t_vehicles().loc[{'vehiclet':'hv liq'}].values,

        percent_t_vehicles().loc[{'vehiclet':'hv hib'}].values,

        percent_t_vehicles().loc[{'vehiclet':'hv gas'}].values,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0,
        0.0],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv gas','bus liq','bus elec','bus hib','bus gas','train liq','train elec','lv hib']},
        dims=['vehiclet'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
# TRANSP - Inland transport sector
@cache('run')
def initial_xt_inland():
    """
    initial Xt inland



    constant


    """

    ret =  2.221228
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def nx_bus_inlandt():
    """
    NX bus inlandT



    component


    """

    ret =  (n_vehicles_inland_t_0().loc[{'vehiclet':'bus liq'}].values + n_vehicles_inland_t_0().loc[{'vehiclet':'bus hib'}].values + n_vehicles_inland_t_0().loc[{'vehiclet':'bus gas'}].values +
            n_vehicles_inland_t_0().loc[{'vehiclet':'bus elec'}].values) / initial_xt_inland()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def nx_hv_inland_t():
    """
    NX HV inland T



    component


    """

    ret =  (n_vehicles_inland_t_0().loc[{'vehiclet':'hv liq'}].values + n_vehicles_inland_t_0().loc[{'vehiclet':'hv hib'}].values + n_vehicles_inland_t_0().loc[{'vehiclet':'hv gas'}].values
            ) / initial_xt_inland()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def nx_lv_inland_t():
    """
    NX LV inland T



    component


    """

    ret =  (n_vehicles_inland_t_0().loc[{'vehiclet':'lv liq'}].values + n_vehicles_inland_t_0().loc[{'vehiclet':'lv elec'}].values + n_vehicles_inland_t_0().loc[{'vehiclet':'lv hib'}].values +
            n_vehicles_inland_t_0().loc[{'vehiclet':'lv gas'}].values) / initial_xt_inland()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def n_vehicles_inland_t_0():
    """
    N vehicles inland t 0



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[58453889.6747974], coords={'vehiclet': ['hv liq']}, dims=['vehiclet']),
        xr.DataArray(data=[26420.04832386], coords={'vehiclet': ['hv hib']}, dims=['vehiclet']),
        xr.DataArray(data=[26420.04832386], coords={'vehiclet': ['hv gas']}, dims=['vehiclet']),
        xr.DataArray(data=[2.09702761e+08], coords={'vehiclet': ['lv liq']}, dims=['vehiclet']),
        xr.DataArray(data=[159229.97493238], coords={'vehiclet': ['lv elec']}, dims=['vehiclet']),
        xr.DataArray(data=[76996.06181609], coords={'vehiclet': ['lv hib']}, dims=['vehiclet']),
        xr.DataArray(data=[3406478.98166824], coords={'vehiclet': ['lv gas']}, dims=['vehiclet']),
        xr.DataArray(data=[15600000.], coords={'vehiclet': ['bus liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['bus elec']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['bus hib']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['bus gas']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['train liq']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['train elec']}, dims=['vehiclet']),
    ])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def energy_initial_inland_transport():
    """
    Energy initial inland transport



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[22.28147355], coords={'vehiclet': ['hv liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['hv hib']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['hv gas']}, dims=['vehiclet']),
        xr.DataArray(data=[8.10305879], coords={'vehiclet': ['lv liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['lv hib']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['lv elec']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['lv gas']}, dims=['vehiclet']),
        xr.DataArray(data=[2.66129086], coords={'vehiclet': ['bus liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['bus elec']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['bus hib']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['bus gas']}, dims=['vehiclet']),
        xr.DataArray(data=[1.30366657], coords={'vehiclet': ['train liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.32591664], coords={'vehiclet': ['train elec']}, dims=['vehiclet']),
    ])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def energy_per_x_t():
    """
    energy per X t



    component


    """

    ret =  xr.DataArray(
        data=[liquids_per_x_hv() * saving_ratios_v().loc[{'vehiclet':'hv liq'}].values,

        liquids_per_x_hv() * saving_ratios_v().loc[{'vehiclet':'hv hib'}].values,

        liquids_per_x_hv() * saving_ratios_v().loc[{'vehiclet':'hv gas'}].values,

        liquids_per_x_lv() * saving_ratios_v().loc[{'vehiclet':'lv liq'}].values,

        liquids_per_x_lv() * saving_ratios_v().loc[{'vehiclet':'lv elec'}].values,

        liquids_per_x_lv() * saving_ratios_v().loc[{'vehiclet':'lv hib'}].values,

        liquids_per_x_lv() * saving_ratios_v().loc[{'vehiclet':'lv gas'}].values,

        liquids_per_x_bus() * saving_ratios_v().loc[{'vehiclet':'bus liq'}].values,

        liquids_per_x_bus() * saving_ratios_v().loc[{'vehiclet':'bus hib'}].values,

        liquids_per_x_bus() * saving_ratios_v().loc[{'vehiclet':'bus gas'}].values,

        energy_per_x_train() * 0.8,

        energy_per_x_train() * 0.2,

        liquids_per_x_bus() * saving_ratios_v().loc[{'vehiclet':'bus elec'}].values],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv hib','lv gas','bus liq','bus hib','bus gas','train liq','train elec','bus elec']},
        dims=['vehiclet'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def nx_train_inland_t():
    """
    NX train inland T



    component


    """

    ret =  1 / initial_xt_inland()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def a2_coef_th():
    """
    A2 coef tH



    component


    """

    ret =  electricity_2we_0() / (demand_h_0() * percent_h_vehicles_initial().loc[{'households vehicles':'elec 2wheels'}].values * saving_ratio_2we())
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def h_gas_adapt_growth():
    """
    H gas adapt growth



    component


    """

    ret =  h_gas_initial_growth() * effects_shortage_gas_h_veh()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def h_hyb_adapt_growth():
    """
    H hyb adapt growth



    component


    """

    ret =  h_hyb_initial_growth()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
# TRANSP - Households transport
@cache('run')
def activate_policy_h_transp():
    """
    Activate policy H transp



    constant


    """

    ret =  1
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def initial_energy_intensity_of_households_transport_2009():
    """
    Initial energy intensity of households transport 2009



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[1.245], coords={'final sources': ['liquids']}, dims=['final sources']),
        xr.DataArray(data=[0.], coords={'final sources': ['solids']}, dims=['final sources']),
        xr.DataArray(data=[0.01711], coords={'final sources': ['gases']}, dims=['final sources']),
        xr.DataArray(
            data=[0.00472], coords={'final sources': ['electricity']}, dims=['final sources']),
        xr.DataArray(data=[0.], coords={'final sources': ['heat']}, dims=['final sources']),
    ])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def h_ev_adapt_growth():
    """
    H EV adapt growth



    component


    """

    ret =  h_elec_initial_growth() * effects_shortage_elec_on_ev()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def var_ih_e2():
    """
    var IH E2



    component


    """

    ret =  a1_coef_th() * var_percents_h_vehicles().loc[{'households vehicles':'elec 4wheels'}].values * saving_ratios_v().loc[{'vehiclet':'lv elec'}].values + a2_coef_th(
    ) * var_percents_h_vehicles().loc[{'households vehicles':'elec 2wheels'}].values * saving_ratio_2we()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def var_ih_gas2():
    """
    var IH gas2



    component


    """

    ret =  a1_coef_th() * var_percents_h_vehicles().loc[{'households vehicles':'gas 4wheels'}].values * saving_ratios_v().loc[{'vehiclet':'lv gas'}].values
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
@cache('step')
def a1_coef_th():
    """
    A1 coef tH



    component


    """

    ret =  ((liq_4w_0() / demand_h_0()) - (a2_coef_th() * percent_2w_liq_0())) / percent_4w_liq_0()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def var_ih_liq2():
    """
    var IH liq2



    component


    """

    ret =  a1_coef_th() * var_percents_h_vehicles().loc[{'households vehicles':'liq 4wheels'}].values + a1_coef_th() * var_percents_h_vehicles().loc[{'households vehicles':'hib 4wheels'}].values + a2_coef_th() * var_percents_h_vehicles().loc[{'households vehicles':'liq 2wheels'}].values
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
# TRANSP - Households transport
@cache('run')
def percent_h_vehicles_initial():
    """
    percent H vehicles initial



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[0.64285241],
            coords={'households vehicles': ['liq 4wheels']},
            dims=['households vehicles']),
        xr.DataArray(
            data=[0.00095159],
            coords={'households vehicles': ['elec 4wheels']},
            dims=['households vehicles']),
        xr.DataArray(
            data=[0.00161276],
            coords={'households vehicles': ['hib 4wheels']},
            dims=['households vehicles']),
        xr.DataArray(
            data=[0.02209406],
            coords={'households vehicles': ['gas 4wheels']},
            dims=['households vehicles']),
        xr.DataArray(
            data=[0.26410155],
            coords={'households vehicles': ['liq 2wheels']},
            dims=['households vehicles']),
        xr.DataArray(
            data=[0.06838764],
            coords={'households vehicles': ['elec 2wheels']},
            dims=['households vehicles']),
    ])
    name = 4
    return round_sig(x=ret, name=name)

# FE INTENSITY households
# TRANSP - Households transport
@cache('step')
def energy_intensity_of_households_transport():
    """
    Energy intensity of households transport



    component


    """


    ret =  integ_energy_intensity_of_households_transport()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def h_2we_adapt_growth():
    """
    H 2wE adapt growth



    component


    """

    ret =  h_2w_initial_growth()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def liq_4w_0():
    """
    Liq 4w 0



    constant


    """

    ret =  41.97111631434505
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def percent_2w_liq_0():
    """
    percent 2w liq 0



    constant


    """

    ret =  0.2641015512843279
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def percent_4w_liq_0():
    """
    percent 4w liq 0



    constant


    """

    ret =  0.642852406005333
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def electricity_2we_0():
    """
    Electricity 2wE 0



    constant


    """

    ret =  0.3420237795619384
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def n_vehicles_h_0():
    """
    N vehicles H 0



    constant


    """

    ret =  2534633971.758305
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def demand_h_0():
    """
    Demand H 0



    constant


    """

    ret =  30.3
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def ratio_n_veh_demand_h():
    """
    ratio N veh Demand H



    component


    """

    ret =  n_vehicles_h_0() / demand_h_0()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def percents_2w_h_vehicles():
    """
    percents 2w H vehicles



    component


    """

    ret =  xr.DataArray(
        data=[percents_h_vehicles().loc[{'households vehicles':'liq 2wheels'}].values / (percents_h_vehicles().loc[{'households vehicles':'elec 2wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'liq 2wheels'}].values),

        percents_h_vehicles().loc[{'households vehicles':'elec 2wheels'}].values / (percents_h_vehicles().loc[{'households vehicles':'elec 2wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'liq 2wheels'}].values),
        0.0,
        0.0,
        0.0,
        0.0],
        coords={'households vehicles':['liq 2wheels','elec 2wheels','liq 4wheels','elec 4wheels','gas 4wheels','hib 4wheels']},
        dims=['households vehicles'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def percents_4w_h_vehicles():
    """
    percents 4w H vehicles



    component


    """

    ret =  xr.DataArray(
        data=[percents_h_vehicles().loc[{'households vehicles':'liq 4wheels'}].values / (percents_h_vehicles().loc[{'households vehicles':'elec 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'hib 4wheels'}].values +
                                 percents_h_vehicles().loc[{'households vehicles':'gas 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'liq 4wheels'}].values),

        percents_h_vehicles().loc[{'households vehicles':'elec 4wheels'}].values / (percents_h_vehicles().loc[{'households vehicles':'elec 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'hib 4wheels'}].values +
                                 percents_h_vehicles().loc[{'households vehicles':'gas 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'liq 4wheels'}].values),

        percents_h_vehicles().loc[{'households vehicles':'hib 4wheels'}].values / (percents_h_vehicles().loc[{'households vehicles':'elec 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'hib 4wheels'}].values +
                                 percents_h_vehicles().loc[{'households vehicles':'gas 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'liq 4wheels'}].values),

        percents_h_vehicles().loc[{'households vehicles':'gas 4wheels'}].values / (percents_h_vehicles().loc[{'households vehicles':'elec 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'hib 4wheels'}].values +
                                 percents_h_vehicles().loc[{'households vehicles':'gas 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'liq 4wheels'}].values),
        0.0,
        0.0],
        coords={'households vehicles':['liq 4wheels','elec 4wheels','hib 4wheels','gas 4wheels','liq 2wheels','elec 2wheels']},
        dims=['households vehicles'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def p_2we():
    """
    P 2wE



    constant


    """

    ret =  0.9254
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
@cache('step')
def max_percent_2_wheels():
    """
    max percent 2 wheels



    component


    """


    ret =  integ_max_percent_2_wheels()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
@cache('step')
def max_percent_4_wheels():
    """
    max percent 4 wheels



    component


    """


    ret =  integ_max_percent_4_wheels()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
# TRANSP - Households transport
@cache('step')
def aux_hist_h():
    """
    aux hist H



    component


    """

    ret =  xr.DataArray(
        data=[-hist_var_percent_h().loc[{'households vehicles':'hib 4wheels'}].values - hist_var_percent_h().loc[{'households vehicles':'elec 4wheels'}].values - hist_var_percent_h().loc[{'households vehicles':'gas 4wheels'}].values,

        hist_var_percent_h().loc[{'households vehicles':'hib 4wheels'}].values,

        hist_var_percent_h().loc[{'households vehicles':'elec 4wheels'}].values,

        -hist_var_percent_h().loc[{'households vehicles':'elec 2wheels'}].values,

        hist_var_percent_h().loc[{'households vehicles':'elec 2wheels'}].values,

        hist_var_percent_h().loc[{'households vehicles':'gas 4wheels'}].values],
        coords={'households vehicles':['liq 4wheels','hib 4wheels','elec 4wheels','liq 2wheels','elec 2wheels','gas 4wheels']},
        dims=['households vehicles'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def p_gas():
    """
    P gas



    constant


    """

    ret =  0.1489
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
@cache('run')
def p_share_2_wheelers():
    """
    P share 2 wheelers



    constant


    """

    ret =  0.3325
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def p_hyb():
    """
    P hyb



    constant


    """

    ret =  0.0108
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('run')
def initial_2w_percent():
    """
    initial 2w percent



    constant


    """

    ret =  0.33248919227982615
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def share_available_2w():
    """
    share available 2w



    component


    """

    ret =  (max_percent_2_wheels() - percents_h_vehicles().loc[{'households vehicles':'elec 2wheels'}].values) / max_percent_2_wheels()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def rate_4w_to_2w():
    """
    rate 4w to 2w



    component


    """

    ret =  policy_2wheels()
    name = 4
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
# ECON- Final demands by sector, households and fuel
@cache('step')
def share_feh_over_fed_by_final_fuel():
    """
    share FEH over FED by final fuel



    component


    """

    ret =  xr.DataArray(
        data=[0 * xdeactivate_heat_dem_correctionxx(),

        0 * xdeactivate_heat_dem_correctionxx(),

        share_feh_over_fed_oil() * xdeactivate_heat_dem_correctionxx(),

        xshare_feh_over_fed_natx_gasx() * xdeactivate_heat_dem_correctionxx(),

        (share_feh_over_fed_coal() + share_feh_over_fed_solid_bioe()) *
        xdeactivate_heat_dem_correctionxx()],
        coords={'final sources':['electricity','heat','liquids','gases','solids']},
        dims=['final sources'])
    name = 4
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('run')
def share_feh_over_fed_oil():
    """
    share FEH over FED oil



    constant


    """

    ret =  0.24447829398324447
    name = 4
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('run')
def share_feh_over_fed_coal():
    """
    share FEH over FED coal



    constant


    """

    ret =  0.39271002371738867
    name = 4
    return round_sig(x=ret, name=name)

@cache('run')
def xshare_feh_over_fed_natx_gasx():
    """
    Xshare FEH over FED natX gasX



    constant


    """

    ret =  0.768348623853211
    name = 4
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_coal_for_heatxncx():
    """
    XFED coal for heatXncX



    component


    """

    ret =  required_fed_by_fuel_before_heat_correction().loc[{'final sources':'solids'}].values * (
        share_feh_over_fed_by_final_fuel().loc[{'final sources':'solids'}].values - share_feh_over_fed_solid_bioe()
    ) * efficiency_coal_for_heat_plants(time()) / (1 + share_heat_distribution_losses())
    name = 4
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def pes_oil_ej_delayed():
    """
    PES oil EJ delayed



    component


    """

    ret =  delay_pes_oil_ej_round01__time_step___time_step_1395_01__time_step()
    name = 4
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
@cache('step')
def energy_required_for_material_consumption_for_ev_batteries():
    """
    Energy required for material consumption for EV batteries



    component


    """

    ret =  materials_required_for_ev_batteries_mt(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    name = 4
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def area_solar_collectors():
    """
    area solar collectors



    component


    """

    ret =  (xinstalled_capacity_res_heatxcom_twx().loc[{'res heat':'xsolarxheatx'}].values + xinstalled_capacity_res_heatxnc_twx().loc[{'res heat':'xsolarxheatx'}].values
            ) * 1e+009 / 0.7
    name = 4
    return round_sig(x=ret, name=name)

# FE INTENSITY households
# ECON- Final demands by sector, households and fuel
# RES BIOE - Traditional biomass
@cache('step')
def households_final_energy_demand():
    """
    Households final energy demand



    component


    """

    ret =  household_demand_total() * energy_intensity_of_households() / 1e+006
    name = 4
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def change_total_intensity_to_rest():
    """
    change total intensity to rest



    component


    """

    ret =  xr.DataArray(data=[1-functions.step(0.78,2009),1-functions.step(0.025,2009),1-functions.step(0.007,2009),], coords={'final sources': ['liquids','gases','electricity']}, dims=['final sources'])
    name = 4
    return round_sig(x=ret, name=name)

# FE INTENSITY households
# TRANSP - Alternative transport
@cache('run')
def activate_transport_h_bottom_up_method():
    """
    Activate transport H BOTTOM UP method



    constant


    """

    ret =  1
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def variation_energy_intensity_of_households_transport():
    """
    variation energy intensity of households transport



    component


    """

    ret =  xr.DataArray(
        data=[functions.if_then_else(time() < 2009, 0, var_ih_liq2()),
        0.0,

        functions.if_then_else(time() > 2009, var_ih_gas2(), 0),

        functions.if_then_else(time() > 2009, var_ih_e2(), 0),
        0.0],
        coords={'final sources':['liquids','solids','gases','electricity','heat']},
        dims=['final sources'])
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Total number vehicles
# TRANSP - Electric batteries
# TRANSP - Electric batteries
@cache('step')
def number_vehicles_h():
    """
    Number vehicles H



    component


    """
    ret =  ratio_n_veh_demand_h() * household_demand_total() * 1e-006 * percents_h_vehicles()
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
# TRANSP - Inland transport sector
@cache('step')
def effects_shortage_elec_on_ev():
    """
    effects shortage elec on EV



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_electricity()) > single_precision(0.8) + less_thr_func(abundance_electricity(), 0.8),
                                  ((abundance_electricity() - 0.8) * 5)**2, 0)
    name = 4
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def effects_shortage_gas_h_veh():
    """
    effects shortage gas H veh



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_gases()) > single_precision(0.8) + less_thr_func(abundance_gases(), 0.8), ((abundance_gases() - 0.8) * 5)**2, 0)
    name = 4
    return round_sig(x=ret, name=name)

@cache('run')
def xxax_extraction_projection_mineralsx():
    """
    XXaX extraction projection mineralsX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(
            data=[1006851.44431989], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[119.69287194], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(
            data=[561172.97577378], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[230741.48718745], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[10.11098981], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[12.05290211], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[80787194.16497986], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[804.07748379], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(
            data=[302664.46064236], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(
            data=[343350.03530151], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(
            data=[5023.5108697], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[40510.95804217], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[73765.76663225], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[257.68585968], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[2274.62093105], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(
            data=[102968.10545833], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[1274.7444251], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[207576.31076445], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 5
    return round_sig(x=ret, name=name)

@cache('run')
def xxbx_extraction_projection_mineralsx():
    """
    XXbX extraction projection mineralsX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(
            data=[-24747508.84046872], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[12862.99276901], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(
            data=[-13481662.07152069], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(
            data=[1147112.83580538], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[-434.34489379], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[-286.28398042], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[-2.77769017e+09], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(
            data=[-25209.40506256], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(
            data=[-11844018.97356096], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(
            data=[-8586601.90070709], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(
            data=[-107292.91153227], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(
            data=[-772331.36137761], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[-554097.26168084], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[4731.27412551], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[111474.4915326], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(
            data=[-448351.45343147], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(
            data=[-18947.84206194], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[-1819702.6572439], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('step')
def variation_minerals_extraction_rest():
    """
    variation minerals extraction Rest



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        historical_variation_minerals_extraction_rest(),
        functions.if_then_else(single_precision(minerals_extraction_projection_rest_cte_rr()) > single_precision(0.01) + less_thr_func(minerals_extraction_projection_rest_cte_rr(), 0.01),
                               (xxax_extraction_projection_mineralsx() *
                                (gdp() - gdp_delayed_1yr())), 0)) * mt_per_tonne()
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('step')
def total_recycled_materials_for_other_mt():
    """
    Total recycled materials for other Mt



    component


    """

    ret =  minerals_consumption_estimation_rest_cte_rr(
    ) - minerals_extraction_projection_rest_with_rr()
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('step')
def share_minerals_consumption_alt_techn_vs_total_economy():
    """
    share minerals consumption alt techn vs total economy



    component


    """

    ret =  functions.zidz(xtotal_materials_required_for_res_elec_x_ev_batteries_mtx(),
                          (minerals_consumption_estimation_rest_cte_rr() +
                           xtotal_materials_required_for_res_elec_x_ev_batteries_mtx()))
    name = 5
    return round_sig(x=ret, name=name)

@cache('run')
def xlabor_share_ctexx():
    """
    XLabor share cteXX



    constant


    """

    ret =  1
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('run')
def initial_minerals_extraction_rest():
    """
    initial minerals extraction Rest



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[19393200.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[18500.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[11999960.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[9992170.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[35.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[239.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[9.99928230e+08], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[6300.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[2640000.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[7579970.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[126000.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[1039680.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[2710000.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[14600.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[188970.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[3814000.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[33700.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[7119990.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('step')
def minerals_extraction_projection_rest_cte_rr():
    """
    Minerals extraction projection Rest cte rr



    component


    """


    ret =  integ_minerals_extraction_projection_rest_cte_rr()
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def materials_to_extract_rest_mt():
    """
    Materials to extract Rest Mt



    component


    """

    ret =  minerals_extraction_projection_rest_with_rr()
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subadhesive(x):
    """
    Historical extraction minerals Rest SUBAdhesive



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subaluminium(x):
    """
    Historical extraction minerals Rest SUBAluminium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        19100000, 19393200, 20694690, 21392970, 22089700, 23084960, 23982470, 24378590, 25872770,
        27662610, 29751750, 31838150, 33607000, 37851990, 38789070, 36941740, 40260150, 43814260,
        45233790, 46864630, 49355110, 55928650, 55520790
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subaluminiummirrors(x):
    """
    Historical extraction minerals Rest SUBAluminiummirrors



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subcadmium(x):
    """
    Historical extraction minerals Rest SUBCadmium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        18100, 18500, 18900, 20000, 19600, 19100, 19700, 18200, 15800, 16890, 18790, 19390, 19280,
        20370, 19550, 18690, 20930, 22010, 22780, 21770, 22020, 22680, 22320
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subcarbonfiber(x):
    """
    Historical extraction minerals Rest SUBCarbonfiber



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subcement(x):
    """
    Historical extraction minerals Rest SUBCement



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subchromium(x):
    """
    Historical extraction minerals Rest SUBChromium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        9570000, 11999960, 12189960, 12499930, 12699880, 13499820, 14399830, 12099800, 13499670,
        15499420, 17499230, 19298990, 19598220, 21496500, 23794450, 19289490, 23683310, 23281570,
        25578710, 28777200, 26364160, 30350300, 30335910
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subcopper(x):
    """
    Historical extraction minerals Rest SUBCopper



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        9430000, 9992170, 10994390, 11392880, 12190320, 12586080, 13182150, 13677330, 13573660,
        13569730, 14561920, 14949450, 15032000, 15309050, 15285180, 15750700, 15707320, 15891470,
        16672810, 18037090, 18126070, 18615350, 18700270
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subdiesel(x):
    """
    Historical extraction minerals Rest SUBdiesel



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subdy(x):
    """
    Historical extraction minerals Rest SUBDy



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subxelectricxelectroniccomponentsx(x):
    """
    Historical extraction minerals Rest SUBXElectricXelectroniccomponentsX



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subevacuationlines(x):
    """
    Historical extraction minerals Rest SUBEvacuationlines



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subfiberglass(x):
    """
    Historical extraction minerals Rest SUBFiberglass



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subfoamglass(x):
    """
    Historical extraction minerals Rest SUBFoamglass



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subgalium(x):
    """
    Historical extraction minerals Rest SUBGalium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        35, 35, 63, 54, 60, 75, 100, 75, 64, 69, 60, 69, 80, 80, 111, 69, 172, 282, 373, 340, 415,
        440, 345
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subglass(x):
    """
    Historical extraction minerals Rest SUBGlass



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subglassreinforcingplastic(x):
    """
    Historical extraction minerals Rest SUBGlassreinforcingplastic



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subgravel(x):
    """
    Historical extraction minerals Rest SUBgravel



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subindium(x):
    """
    Historical extraction minerals Rest SUBIndium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        145, 239, 200, 230, 230, 215, 335, 345, 335, 370, 395, 490, 570, 533, 530, 466, 479, 522,
        622, 629, 564, 369, 155
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subiron(x):
    """
    Historical extraction minerals Rest SUBIron



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        1000000000, 999928230, 1019944840, 1039927180, 1019893920, 993844440, 1059821110,
        1059791350, 1079735210, 1159623830, 1339511300, 1539358150, 1799023770, 1998410780,
        2217726110, 2236160950, 2584196110, 2933714900, 2922833710, 3102160850, 3408008320,
        3663619920, 3568372660
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subkno3mined(x):
    """
    Historical extraction minerals Rest SUBKNO3mined



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subasphalt(x):
    """
    Historical extraction minerals Rest SUBAsphalt



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_sublime(x):
    """
    Historical extraction minerals Rest SUBLime



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_sublimestone(x):
    """
    Historical extraction minerals Rest SUBLimestone



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_sublithium(x):
    """
    Historical extraction minerals Rest SUBLithium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        6100, 6300, 11000, 14000, 15000, 14000, 14000, 15100, 14200, 15100, 20200, 20600, 23500,
        25800, 25400, 18800, 28100, 34100, 35000, 34000, 31700, 31500, 35000
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_sublubricant(x):
    """
    Historical extraction minerals Rest SUBLubricant



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_submagnesium(x):
    """
    Historical extraction minerals Rest SUBMagnesium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        2460000, 2640000, 2639990, 3029990, 3089990, 3089980, 3089980, 3169980, 3319970, 3459940,
        4269920, 4209830, 4059670, 4389210, 5428700, 5507890, 5756730, 5926210, 6345590, 6905580,
        8413980, 27692050, 27689810
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_submanganese(x):
    """
    Historical extraction minerals Rest SUBManganese



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        7190000, 7579970, 7729960, 7679940, 7039890, 6989840, 7279850, 7599810, 8099700, 8199470,
        9349300, 10499090, 11898380, 12596810, 13294950, 10790450, 13884820, 15983240, 15780650,
        16879270, 17767420, 17454820, 15941730
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subheavyequipment(x):
    """
    Historical extraction minerals Rest SUBHeavyequipment



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subconcrete(x):
    """
    Historical extraction minerals Rest SUBConcrete



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_submolybdenum(x):
    """
    Historical extraction minerals Rest SUBMolybdenum



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        104000, 126000, 128000, 139990, 134990, 121980, 128980, 129980, 122970, 124950, 140930,
        184910, 183840, 204680, 217500, 220040, 240480, 262320, 257060, 255930, 277740, 230480,
        221170
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subnano3mined(x):
    """
    Historical extraction minerals Rest SUBNaNO3mined



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subnano3synthetic(x):
    """
    Historical extraction minerals Rest SUBNaNO3synthetic



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subneodymium(x):
    """
    Historical extraction minerals Rest SUBNeodymium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subnickel(x):
    """
    Historical extraction minerals Rest SUBNickel



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        906000, 1039680, 1079770, 1119700, 1139590, 1119400, 1249270, 1329150, 1338990, 1398710,
        1398340, 1487810, 1576940, 1655660, 1564350, 1391710, 1578370, 1927720, 2206300, 2614500,
        2427300, 2249800, 2208450
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subxovergridx15xxx(x):
    """
    Historical extraction minerals Rest SUBXOvergridX15XXX



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subxovergridx5xxx(x):
    """
    Historical extraction minerals Rest SUBXOvergridX5XXX



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subpaint(x):
    """
    Historical extraction minerals Rest SUBPaint



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_sublead(x):
    """
    Historical extraction minerals Rest SUBLead



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        2800000, 2710000, 2920000, 3010000, 3100000, 3019990, 3099990, 3099990, 2909990, 2949980,
        3149970, 3269960, 3469940, 3769880, 3839810, 3859630, 4139410, 4699350, 5169250, 5489190,
        4868700, 4948180, 4817650
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subplastics(x):
    """
    Historical extraction minerals Rest SUBPlastics



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subpolypropylene(x):
    """
    Historical extraction minerals Rest SUBPolypropylene



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subrock(x):
    """
    Historical extraction minerals Rest SUBRock



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subrockwool(x):
    """
    Historical extraction minerals Rest SUBRockwool



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subsand(x):
    """
    Historical extraction minerals Rest SUBSand



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subsiliconsand(x):
    """
    Historical extraction minerals Rest SUBSiliconsand



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subsiliconwafermodules(x):
    """
    Historical extraction minerals Rest SUBSiliconwafermodules



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subsilver(x):
    """
    Historical extraction minerals Rest SUBSilver



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        13900, 14600, 15200, 16390, 16390, 17680, 17690, 18680, 19970, 18750, 19630, 19220, 20060,
        20530, 20870, 20970, 21780, 21860, 23830, 24190, 23910, 21060, 21790
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subsitepreparation(x):
    """
    Historical extraction minerals Rest SUBSitepreparation



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subtin(x):
    """
    Historical extraction minerals Rest SUBTin



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        184000, 188970, 195960, 210950, 205900, 197850, 237860, 221830, 248720, 206510, 263350,
        289200, 300610, 317350, 294830, 251880, 252040, 229800, 223630, 276260, 257510, 249220,
        228660
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subsodaash(x):
    """
    Historical extraction minerals Rest SUBsodaash



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_substeel(x):
    """
    Historical extraction minerals Rest SUBsteel



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subsyntheticoil(x):
    """
    Historical extraction minerals Rest SUBsyntheticoil



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subtellurium(x):
    """
    Historical extraction minerals Rest SUBtellurium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subtitanium(x):
    """
    Historical extraction minerals Rest SUBtitanium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        3600000, 3814000, 4015000, 4055000, 4260000, 4500000, 4700000, 5000000, 4800000, 5299990,
        4999990, 5199990, 5799980, 6289960, 6389940, 5799880, 6399810, 6699790, 7229760, 7399740,
        6039590, 6939440, 6599270
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subtitaniumdioxide(x):
    """
    Historical extraction minerals Rest SUBtitaniumdioxide



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subvanadium(x):
    """
    Historical extraction minerals Rest SUBvanadium



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        33900, 33700, 35100, 37100, 41800, 42800, 43000, 58000, 60000, 41000, 40200, 58200, 55700,
        58500, 55500, 53490, 57590, 62380, 73980, 78980, 82670, 77760, 75940
    ])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subwires(x):
    """
    Historical extraction minerals Rest SUBwires



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 5
    return round_sig(x=ret, name=name)

def historical_extraction_minerals_rest_subzinc(x):
    """
    Historical extraction minerals Rest SUBzinc



    lookup


    """

    ret =  functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        6810000, 7119990, 7439990, 7459980, 7549970, 8039950, 8729950, 8849940, 8359900, 9009830,
        9599770, 9799700, 9999480, 10898960, 11598360, 11196900, 11995070, 12794550, 13493710,
        13393260, 13289410, 12785320, 11881060
    ])
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('step')
def historical_variation_minerals_extraction_rest():
    """
    Historical variation minerals extraction Rest



    component


    """

    ret =  xr.DataArray(
        data=[historical_extraction_minerals_rest_subadhesive(time() + 1) -
        historical_extraction_minerals_rest_subadhesive(time()),

        historical_extraction_minerals_rest_subaluminium(time() + 1) -
        historical_extraction_minerals_rest_subaluminium(time()),

        historical_extraction_minerals_rest_subaluminiummirrors(time() + 1) -
        historical_extraction_minerals_rest_subaluminiummirrors(time()),

        historical_extraction_minerals_rest_subcadmium(time() + 1) -
        historical_extraction_minerals_rest_subcadmium(time()),

        historical_extraction_minerals_rest_subcarbonfiber(time() + 1) -
        historical_extraction_minerals_rest_subcarbonfiber(time()),

        historical_extraction_minerals_rest_subcement(time() + 1) -
        historical_extraction_minerals_rest_subcement(time()),

        historical_extraction_minerals_rest_subchromium(time() + 1) -
        historical_extraction_minerals_rest_subchromium(time()),

        historical_extraction_minerals_rest_subcopper(time() + 1) -
        historical_extraction_minerals_rest_subcopper(time()),

        historical_extraction_minerals_rest_subdiesel(time() + 1) -
        historical_extraction_minerals_rest_subdiesel(time()),

        historical_extraction_minerals_rest_subdy(time() + 1) -
        historical_extraction_minerals_rest_subdy(time()),

        historical_extraction_minerals_rest_subxelectricxelectroniccomponentsx(time() + 1) -
        historical_extraction_minerals_rest_subxelectricxelectroniccomponentsx(time()),

        historical_extraction_minerals_rest_subevacuationlines(time() + 1) -
        historical_extraction_minerals_rest_subevacuationlines(time()),

        historical_extraction_minerals_rest_subfiberglass(time() + 1) -
        historical_extraction_minerals_rest_subfiberglass(time()),

        historical_extraction_minerals_rest_subfoamglass(time() + 1) -
        historical_extraction_minerals_rest_subfoamglass(time()),

        historical_extraction_minerals_rest_subgalium(time() + 1) -
        historical_extraction_minerals_rest_subgalium(time()),

        historical_extraction_minerals_rest_subglass(time() + 1) -
        historical_extraction_minerals_rest_subglass(time()),

        historical_extraction_minerals_rest_subglassreinforcingplastic(time() + 1) -
        historical_extraction_minerals_rest_subglassreinforcingplastic(time()),

        historical_extraction_minerals_rest_subgravel(time() + 1) -
        historical_extraction_minerals_rest_subgravel(time()),

        historical_extraction_minerals_rest_subindium(time() + 1) -
        historical_extraction_minerals_rest_subindium(time()),

        historical_extraction_minerals_rest_subiron(time() + 1) -
        historical_extraction_minerals_rest_subiron(time()),

        historical_extraction_minerals_rest_subkno3mined(time() + 1) -
        historical_extraction_minerals_rest_subkno3mined(time()),

        historical_extraction_minerals_rest_subasphalt(time() + 1) -
        historical_extraction_minerals_rest_subasphalt(time()),

        historical_extraction_minerals_rest_sublime(time() + 1) -
        historical_extraction_minerals_rest_sublime(time()),

        historical_extraction_minerals_rest_sublimestone(time() + 1) -
        historical_extraction_minerals_rest_sublimestone(time()),

        historical_extraction_minerals_rest_sublithium(time() + 1) -
        historical_extraction_minerals_rest_sublithium(time()),

        historical_extraction_minerals_rest_sublubricant(time() + 1) -
        historical_extraction_minerals_rest_sublubricant(time()),

        historical_extraction_minerals_rest_submagnesium(time() + 1) -
        historical_extraction_minerals_rest_submagnesium(time()),

        historical_extraction_minerals_rest_submanganese(time() + 1) -
        historical_extraction_minerals_rest_submanganese(time()),

        historical_extraction_minerals_rest_subheavyequipment(time() + 1) -
        historical_extraction_minerals_rest_subheavyequipment(time()),

        historical_extraction_minerals_rest_subconcrete(time() + 1) -
        historical_extraction_minerals_rest_subconcrete(time()),

        historical_extraction_minerals_rest_submolybdenum(time() + 1) -
        historical_extraction_minerals_rest_submolybdenum(time()),

        historical_extraction_minerals_rest_subnano3mined(time() + 1) -
        historical_extraction_minerals_rest_subnano3mined(time()),

        historical_extraction_minerals_rest_subnano3synthetic(time() + 1) -
        historical_extraction_minerals_rest_subnano3synthetic(time()),

        historical_extraction_minerals_rest_subneodymium(time() + 1) -
        historical_extraction_minerals_rest_subneodymium(time()),

        historical_extraction_minerals_rest_subnickel(time() + 1) -
        historical_extraction_minerals_rest_subnickel(time()),

        historical_extraction_minerals_rest_subxovergridx15xxx(time() + 1) -
        historical_extraction_minerals_rest_subxovergridx15xxx(time()),

        historical_extraction_minerals_rest_subxovergridx5xxx(time() + 1) -
        historical_extraction_minerals_rest_subxovergridx5xxx(time()),

        historical_extraction_minerals_rest_subpaint(time() + 1) -
        historical_extraction_minerals_rest_subpaint(time()),

        historical_extraction_minerals_rest_sublead(time() + 1) -
        historical_extraction_minerals_rest_sublead(time()),

        historical_extraction_minerals_rest_subplastics(time() + 1) -
        historical_extraction_minerals_rest_subplastics(time()),

        historical_extraction_minerals_rest_subpolypropylene(time() + 1) -
        historical_extraction_minerals_rest_subpolypropylene(time()),

        historical_extraction_minerals_rest_subrock(time() + 1) -
        historical_extraction_minerals_rest_subrock(time()),

        historical_extraction_minerals_rest_subrockwool(time() + 1) -
        historical_extraction_minerals_rest_subrockwool(time()),

        historical_extraction_minerals_rest_subsand(time() + 1) -
        historical_extraction_minerals_rest_subsand(time()),

        historical_extraction_minerals_rest_subsiliconsand(time() + 1) -
        historical_extraction_minerals_rest_subsiliconsand(time()),

        historical_extraction_minerals_rest_subsiliconwafermodules(time() + 1) -
        historical_extraction_minerals_rest_subsiliconwafermodules(time()),

        historical_extraction_minerals_rest_subsilver(time() + 1) -
        historical_extraction_minerals_rest_subsilver(time()),

        historical_extraction_minerals_rest_subsitepreparation(time() + 1) -
        historical_extraction_minerals_rest_subsitepreparation(time()),

        historical_extraction_minerals_rest_subtin(time() + 1) -
        historical_extraction_minerals_rest_subtin(time()),

        historical_extraction_minerals_rest_subsodaash(time() + 1) -
        historical_extraction_minerals_rest_subsodaash(time()),

        historical_extraction_minerals_rest_substeel(time() + 1) -
        historical_extraction_minerals_rest_substeel(time()),

        historical_extraction_minerals_rest_subsyntheticoil(time() + 1) -
        historical_extraction_minerals_rest_subsyntheticoil(time()),

        historical_extraction_minerals_rest_subtellurium(time() + 1) -
        historical_extraction_minerals_rest_subtellurium(time()),

        historical_extraction_minerals_rest_subtitanium(time() + 1) -
        historical_extraction_minerals_rest_subtitanium(time()),

        historical_extraction_minerals_rest_subtitaniumdioxide(time() + 1) -
        historical_extraction_minerals_rest_subtitaniumdioxide(time()),

        historical_extraction_minerals_rest_subvanadium(time() + 1) -
        historical_extraction_minerals_rest_subvanadium(time()),

        historical_extraction_minerals_rest_subwires(time() + 1) -
        historical_extraction_minerals_rest_subwires(time()),

        historical_extraction_minerals_rest_subzinc(time() + 1) -
        historical_extraction_minerals_rest_subzinc(time())],
        coords={'materials':['adhesive','aluminium','aluminium mirrors','cadmium','carbon fiber','cement','chromium','copper','diesel','dy','xelectricxelectronic componentsx','evacuation lines','fiberglass','foam glass','galium','glass','glass reinforcing plastic','gravel','indium','iron','kno3 mined','asphalt','lime','limestone','lithium','lubricant','magnesium','manganese','heavy equipment','concrete','molybdenum','nano3 mined','nano3 synthetic','neodymium','nickel','xover grid x15xxx','xover grid x5xxx','paint','lead','plastics','polypropylene','rock','rock wool','sand','silicon sand','silicon wafer modules','silver','site preparation','tin','soda ash','steel','synthetic oil','tellurium','titanium','titanium dioxide','vanadium','wires','zinc']},
        dims=['materials'])
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('step')
def minerals_extraction_projection_rest_with_rr():
    """
    Minerals extraction projection Rest with rr



    component


    """

    ret =  minerals_consumption_estimation_rest_cte_rr() * (1 - recycling_rates_minerals_rest())
    name = 5
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('step')
def minerals_consumption_estimation_rest_cte_rr():
    """
    Minerals consumption estimation Rest cte rr



    component


    """

    ret =  minerals_extraction_projection_rest_cte_rr() / (1 - current_recycling_rates_minerals())
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_materials_required_for_res_elec_x_ev_batteries_mtx():
    """
    XTotal materials required for RES elec X EV batteries MtX



    component


    """

    ret =  total_materials_required_for_ev_batteries() + total_materials_required_for_res_elec_mt()
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def xvariation_nonxenergy_usex():
    """
    Xvariation nonXenergy useX



    constant


    """

    ret =  xr.DataArray(
        data=[0.0,
        0.0,

        functions.if_then_else(xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'liquids'}].values > 0.01,
                               0.461414 * (gdp() - gdp_delayed_1yr()), 0),

        functions.if_then_else(xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'gases'}].values > 0.01,
                               0.123925 * (gdp() - gdp_delayed_1yr()), 0),

        functions.if_then_else(xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'solids'}].values > 0.01,
                               0.0797511 * (gdp() - gdp_delayed_1yr()), 0)],
        coords={'final sources':['electricity','heat','liquids','gases','solids']},
        dims=['final sources'])
    name = 5
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def number_2w():
    """
    Number 2w



    component


    """

    ret =  number_vehicles_h().loc[{'households vehicles':'liq 2wheels'}].values + number_vehicles_h().loc[{'households vehicles':'elec 2wheels'}].values
    name = 5
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def number_4w():
    """
    Number 4w



    component


    """

    ret =  number_vehicles_h().loc[{'households vehicles':'liq 4wheels'}].values + number_vehicles_h().loc[{'households vehicles':'hib 4wheels'}].values + number_vehicles_h().loc[{'households vehicles':'elec 4wheels'}].values + number_vehicles_h().loc[{'households vehicles':'gas 4wheels'}].values
    name = 5
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def number_all():
    """
    Number all



    component


    """

    ret =  number_2w() + number_4w()
    name = 5
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def sum_4w_shares():
    """
    sum 4w shares



    component


    """

    ret =  percents_h_vehicles().loc[{'households vehicles':'hib 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'elec 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'gas 4wheels'}].values
    name = 5
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def share_available_4w():
    """
    share available 4w



    component


    """

    ret =  (max_percent_4_wheels() - sum_4w_shares()) / max_percent_4_wheels()
    name = 5
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def percent_4w():
    """
    percent 4w



    component


    """

    ret =  percents_h_vehicles().loc[{'households vehicles':'liq 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'hib 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'elec 4wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'gas 4wheels'}].values
    name = 5
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Households transport
@cache('run')
def saving_ratio_2we():
    """
    saving ratio 2wE



    constant


    """

    ret =  0.3
    name = 5
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def percent_all():
    """
    percent all



    component


    """

    ret =  percent_2w() + percent_4w()
    name = 5
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
@cache('step')
def percent_2w():
    """
    percent 2w



    component


    """

    ret =  percents_h_vehicles().loc[{'households vehicles':'liq 2wheels'}].values + percents_h_vehicles().loc[{'households vehicles':'elec 2wheels'}].values
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_fed_coal_vs_nre_heatxncx():
    """
    Xshare FED coal vs NRE heatXncX



    component


    """

    ret =  functions.zidz(xfed_coal_for_heatxncx(), xfed_nre_for_heatxncx())
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_fed_gas_vs_nre_heatxncx():
    """
    Xshare FED gas vs NRE heatXncX



    component


    """

    ret =  functions.zidz(xfed_natx_gas_for_heatxncx(), xfed_nre_for_heatxncx())
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_nre_for_heatxncx():
    """
    XFED NRE for heatXncX



    component


    """

    ret =  xfed_coal_for_heatxncx() + xfed_natx_gas_for_heatxncx() + xfed_oil_for_heatxncx()
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_fed_liquids_vs_nre_heatxncx():
    """
    Xshare FED liquids vs NRE heatXncX



    component


    """

    ret =  functions.zidz(xfed_oil_for_heatxncx(), xfed_nre_for_heatxncx())
    name = 5
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# ELEC - Electricity demand
# HEAT - Heat demand
# LIQUIDS - PED, PES & FES
# GASES - PED, PES & FES
# SOLIDS - PED, PES & FES
# IMPACTS CC - Energy losses function
@cache('step')
def required_fed_by_fuel():
    """
    Required FED by fuel



    component


    """

    ret =  xr.DataArray(
        data=[required_fed_by_fuel_before_heat_correction().loc[{'final sources':'electricity'}].values,

        required_fed_by_fuel_before_heat_correction().loc[{'final sources':'heat'}].values *
        (1 + xratio_fed_for_heatxnc_vs_fed_for_heatxcomx()),

        required_fed_by_fuel_before_heat_correction().loc[{'final sources':'liquids'}].values * (1 - share_feh_over_fed_by_final_fuel().loc[{'final sources':'liquids'}].values),

        required_fed_by_fuel_before_heat_correction().loc[{'final sources':'gases'}].values * (1 - share_feh_over_fed_by_final_fuel().loc[{'final sources':'gases'}].values),

        required_fed_by_fuel_before_heat_correction().loc[{'final sources':'solids'}].values * (1 - share_feh_over_fed_by_final_fuel().loc[{'final sources':'solids'}].values)],
        coords={'final sources':['electricity','heat','liquids','gases','solids']},
        dims=['final sources'])
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
@cache('step')
def bioe_gen_land_marg_available():
    """
    BioE gen land marg available



    component


    """

    ret =  (max_peavail_potential_biofuels_marginal_lands() -
            potential_peavail_biofuels_land_marg_ej()
            ) / max_peavail_potential_biofuels_marginal_lands()
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
# RES BIOE - Total biofuels liquids
@cache('step')
def potential_peavail_cellulosic_biofuel_ej():
    """
    Potential PEavail cellulosic biofuel EJ



    component


    """

    ret =  potential_pe_cellulosic_biofuel_ej() * conv_efficiency_from_npp_to_biofuels()
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Total biofuels liquids
@cache('step')
def potential_peavail_total_biofuels():
    """
    Potential PEavail total biofuels



    component


    """

    ret =  potential_peavail_biofuels_2gen_land_compet_ej(
    ) + potential_peavail_biofuels_prod_3gen_ej() + potential_peavail_biofuels_land_marg_ej(
    ) + potential_peavail_cellulosic_biofuel_ej()
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def biofuels_3gen_land_compet_available():
    """
    Biofuels 3gen land compet available



    component


    """

    ret =  (max_land_compet_biofuels_2gen() - land_compet_biofuels_3gen_mha()
            ) / max_land_compet_biofuels_2gen()
    name = 5
    return round_sig(x=ret, name=name)

# RES - Employment
# RES BIOE - Total biofuels liquids
# RES - PED from non-electric RES
# LIQUIDS - PED, PES & FES
# TFES by fuel
@cache('step')
def fes_total_biofuels_production_ej():
    """
    FES total biofuels production EJ



    component


    """

    ret =  peavail_biofuels_2gen_land_compet_ej() + peavail_biofuels_3gen_land_compet_ej(
    ) + peavail_biofuels_land_marg_ej() + peavail_cellulosic_biofuel_ej()
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Total biofuels liquids
@cache('step')
def fes_total_biofuels_production_ej_2():
    """
    FES total biofuels production EJ 2



    component


    """

    ret =  np.minimum(ped_liquids_ej(), potential_peavail_total_biofuels())
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_total_biofuels_production_mbxdx():
    """
    XFES total biofuels production MbXdX



    component


    """

    ret =  fes_total_biofuels_production_ej() * xmbxd_per_ejxyearx()
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Total biofuels liquids
@cache('step')
def peavail_biofuels_2gen_land_compet_ej():
    """
    PEavail biofuels 2gen land compet EJ



    component


    """

    ret =  potential_peavail_biofuels_2gen_land_compet_ej() * (1 - share_biofuels_overcapacity())
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Total biofuels liquids
@cache('step')
def peavail_biofuels_3gen_land_compet_ej():
    """
    PEavail biofuels 3gen land compet EJ



    component


    """

    ret =  potential_peavail_biofuels_prod_3gen_ej() * (1 - share_biofuels_overcapacity())
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
# RES BIOE - Total biofuels liquids
@cache('step')
def peavail_biofuels_land_marg_ej():
    """
    PEavail biofuels land marg EJ



    component


    """

    ret =  potential_peavail_biofuels_land_marg_ej() * (1 - share_biofuels_overcapacity())
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
# RES BIOE - Total biofuels liquids
@cache('step')
def pe_biofuels_land_marg_ej():
    """
    PE biofuels land marg EJ



    component


    """

    ret =  peavail_biofuels_land_marg_ej() / conv_efficiency_from_npp_to_biofuels()
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def xpe_biofuels_prod_2genx3gen_ejx():
    """
    XPE biofuels prod 2genX3gen EJX



    component


    """

    ret =  (peavail_biofuels_2gen_land_compet_ej() + peavail_biofuels_3gen_land_compet_ej()
            ) / conv_efficiency_from_npp_to_biofuels()
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Total biofuels liquids
@cache('step')
def pe_biomass_for_biofuels_production_ej():
    """
    PE biomass for biofuels production EJ



    component


    """

    ret =  pe_biofuels_land_marg_ej() + pe_cellulosic_biofuel_ej(
    ) + xpe_biofuels_prod_2genx3gen_ejx()
    name = 5
    return round_sig(x=ret, name=name)

@cache('step')
def xnew_bioe_residues_for_heatxelecx():
    """
    Xnew BioE residues for heatXelecX



    component


    """

    ret =  functions.if_then_else(
        time() < xstart_year_bioe_residues_for_heatxelecx(), 0,
        functions.if_then_else(
            time() < xstart_year_bioe_residues_for_heatxelecx() + 5,
            start_production_biofuels(time() - xstart_year_bioe_residues_for_heatxelecx()) *
            ej_per_ktoe(),
            xp_bioe_residues_for_heatxelecx() * xpe_bioe_residues_for_heatxelec_ejx() *
            xbioe_residues_for_heatxelec_availablex()))
    name = 5
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
# RES BIOE - Total biofuels liquids
@cache('step')
def pe_cellulosic_biofuel_ej():
    """
    PE cellulosic biofuel EJ



    component


    """

    ret =  potential_pe_cellulosic_biofuel_ej() * (1 - share_biofuels_overcapacity())
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
@cache('step')
def new_biofuels_land_marg():
    """
    new biofuels land marg



    component


    """

    ret =  functions.if_then_else(
        time() < start_year_biofuels_land_marg(), 0,
        functions.if_then_else(
            time() < start_year_biofuels_land_marg() + 5,
            start_production_biofuels(time() - start_year_biofuels_land_marg()) * ej_per_ktoe(),
            functions.if_then_else(check_liquids() > equality_threshold,
                                   xconstrain_liquids_exogenous_growthxx() *
                                   potential_peavail_biofuels_land_marg_ej(),
                                   adapt_growth_biofuels_2gen() * bioe_gen_land_marg_available() *
                                   potential_peavail_biofuels_land_marg_ej())))
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Total biofuels liquids
# RES - PED from non-electric RES
@cache('step')
def max_peavail_biofuels_potential():
    """
    Max PEavail biofuels potential



    component


    """

    ret =  max_peavail_potential_bioe_residues_for_cellulosic_biofuels(
    ) + xmax_peavail_potential_biofuels_2x3genx() + max_peavail_potential_biofuels_marginal_lands(
    )
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
@cache('run')
def start_year_biofuels_land_marg():
    """
    start year biofuels land marg



    constant


    """

    ret =  2020
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
# IMPACTS - RES Land-use
@cache('step')
def land_required_biofuels_land_marg():
    """
    Land required biofuels land marg



    component


    """

    ret =  potential_peavail_biofuels_land_marg_ej() * land_occupation_ratio_biofuels_marg_land(
    ) / conv_efficiency_from_npp_to_biofuels()
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
# RES BIOE - Total biofuels liquids
@cache('step')
def max_peavail_potential_biofuels_marginal_lands():
    """
    Max PEavail potential biofuels marginal lands



    component


    """

    ret =  bioe_potential_npp_marginal_lands() * conv_efficiency_from_npp_to_biofuels()
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Dedicated crops for biofuels (marg)
# RES BIOE - Crop and forest residues
# RES BIOE - Total biofuels liquids
@cache('step')
def share_biofuels_overcapacity():
    """
    share biofuels overcapacity



    component


    """

    ret =  functions.zidz(
        (potential_peavail_total_biofuels() - fes_total_biofuels_production_ej_2()),
        potential_peavail_total_biofuels())
    name = 6
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('step')
def total_land_requirements_renew_mha():
    """
    Total land requirements renew Mha



    component


    """

    ret =  surface_solar_pv_mha() + surface_csp_mha() + surface_hydro_mha(
    ) + land_compet_required_dedicated_crops_for_biofuels() + land_required_biofuels_land_marg(
    ) + surface_onshore_wind_mha() * 0
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
@cache('run')
def land_occupation_ratio_biofuels_marg_land():
    """
    Land occupation ratio biofuels marg land



    constant


    """

    ret =  14.3
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('step')
def new_cellulosic_biofuels():
    """
    new cellulosic biofuels



    component


    """

    ret =  functions.if_then_else(
        time() < start_year_cellulosic_biofuels(), 0,
        functions.if_then_else(
            time() < start_year_cellulosic_biofuels() + 5,
            start_production_biofuels(time() - start_year_cellulosic_biofuels()) * ej_per_ktoe(),
            functions.if_then_else(
                check_liquids() > equality_threshold,
                xconstrain_liquids_exogenous_growthxx() * potential_pe_cellulosic_biofuel_ej(),
                p_cellulosic_biofuels() * potential_pe_cellulosic_biofuel_ej() *
                cellulosic_biofuels_available())))
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
# RES BIOE - Total biofuels liquids
@cache('step')
def peavail_cellulosic_biofuel_ej():
    """
    PEavail cellulosic biofuel EJ



    component


    """

    ret =  pe_cellulosic_biofuel_ej() * efficiency_bioe_residues_to_cellulosic_liquids()
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
# RES BIOE - Total biofuels liquids
@cache('step')
def potential_peavail_biofuels_land_marg_ej():
    """
    Potential PEavail biofuels land marg EJ



    component


    """


    ret =  integ_potential_peavail_biofuels_land_marg_ej()
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (marg)
@cache('run')
def bioe_potential_npp_marginal_lands():
    """
    BioE potential NPP marginal lands



    constant


    """

    ret =  27
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Total biofuels liquids
@cache('step')
def oil_saved_by_biofuels_mtoe():
    """
    Oil saved by biofuels Mtoe



    component


    """

    ret =  oil_liquids_saved_by_biofuels_ej() * mtoe_per_ej()
    name = 6
    return round_sig(x=ret, name=name)
# RES BIOE - Dedicated crops for biofuels (marg)
# RES BIOE - Crop and forest residues
# RES BIOE - Crop and forest residues

def start_production_biofuels(x):
    """
    start production biofuels



    lookup


    """

    ret =  functions.lookup(x, [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14], [
        169.31651889742935, 362.06079369838915, 570.29158956473, 351.80209377924075,
        640.998751176415, 1203.1283705538146, 1807.035671689667, 2350.5205842214987,
        1090.18831543025, 1539.9222813996662, 311.6224461287653, 223.13339104718762,
        996.4005646830498, 1389.4084328043944, 0.0
    ])
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Total biofuels liquids
@cache('step')
def additional_pe_production_of_bioenergy_for_biofuels():
    """
    Additional PE production of bioenergy for biofuels



    component


    """

    ret =  pe_biomass_for_biofuels_production_ej() - oil_liquids_saved_by_biofuels_ej()
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Total biofuels liquids
# IMPACTS - Total GHG emissions
@cache('step')
def oil_liquids_saved_by_biofuels_ej():
    """
    Oil liquids saved by biofuels EJ



    component


    """

    ret =  fes_total_biofuels_production_ej()
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('step')
def efficiency_bioe_residues_to_cellulosic_liquids():
    """
    Efficiency bioE residues to cellulosic liquids



    component


    """

    ret =  conv_efficiency_from_npp_to_biofuels()
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Dedicated crops for biofuels (marg)
# RES BIOE - Crop and forest residues
@cache('run')
def conv_efficiency_from_npp_to_biofuels():
    """
    Conv efficiency from NPP to biofuels



    constant


    """

    ret =  0.15
    name = 6
    return round_sig(x=ret, name=name)

# HEAT - Total PES
@cache('step')
def pes_heat_res():
    """
    PES heat RES



    component


    """

    ret =  xpes_tot_biogas_for_heatxcomx() + functions.sum(xpes_res_for_heatxcom_by_technx()) + functions.sum(
        xpes_res_for_heatxnc_by_technx())
    name = 6
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
# RESOURCES - CTL & GTL supply
@cache('step')
def ped_nre_liquids():
    """
    PED NRE Liquids



    component


    """

    ret =  np.maximum(0, ped_liquids_ej() - fes_total_biofuels_production_ej())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xpotential_fes_ctlxgtl_ejx():
    """
    XPotential FES CTLXGTL EJX



    component


    """

    ret =  ctl_potential_production() + gtl_potential_production()
    name = 6
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def tfec_res_delayed_1yr():
    """
    TFEC RES delayed 1yr



    component


    """

    ret =  delay_tfec_res_ej_round1__time_step___time_step_43_1__time_step()
    name = 6
    return round_sig(x=ret, name=name)

# TFES by fuel
# INDICATORS - Energy
# INDICATORS - Energy
# INDICATORS - Social and Environmental impacts
@cache('step')
def tfec_res_ej():
    """
    TFEC RES EJ



    component


    """

    ret =  fe_tot_generation_all_res_elec_ej() + fes_res_for_heat_ej() + pe_traditional_biomass_ej(
    ) + fes_total_biofuels_production_ej() + pes_biogas_for_tfc()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xpe_supply_from_res_nonxelec_without_trad_bioe_ejx():
    """
    XPE supply from RES nonXelec without trad bioE EJX



    component


    """

    ret =  pes_tot_res_for_heat() + fes_total_biofuels_production_ej() + pes_biogas_for_tfc()
    name = 6
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
# HEAT - Total FE heat generation
# GASES - PED, PES & FES
# GASES - PED, PES & FES
@cache('step')
def pes_gases():
    """
    PES gases



    component


    """

    ret =  xpes_natx_gasx() + pes_biogas_for_tfc()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_ctlxgtl_ejx():
    """
    XFES CTLXGTL EJX



    component


    """

    ret =  np.minimum(ped_nre_liquids(), xpotential_fes_ctlxgtl_ejx())
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def xactivate_elf_all_scenxx():
    """
    Xactivate ELF all scenXX



    constant


    """

    ret =  1
    name = 6
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def tpes_res_delayed_1yr():
    """
    TPES RES delayed 1yr



    component


    """

    ret =  delay_tpe_from_res_ej_round1__time_step___time_step_4803_1__time_step()
    name = 6
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def annual_tpes_res_growth_rate():
    """
    Annual TPES RES growth rate



    component


    """

    ret =  -1 + tpe_from_res_ej() / tpes_res_delayed_1yr()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xped_natx_gas_ejx():
    """
    XPED natX gas EJX



    component


    """

    ret =  np.maximum(0, ped_gases() - pes_biogas_for_tfc())
    name = 6
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def annual_share_res_vs_tfec_growth_rate():
    """
    Annual share RES vs TFEC growth rate



    component


    """

    ret =  -1 + share_res_vs_tfec() / share_res_vs_tfec_delayed_1yr()
    name = 6
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def annual_share_res_vs_tpes_growth_rate():
    """
    Annual share RES vs TPES growth rate



    component


    """

    ret =  -1 + share_res_vs_tpes() / share_res_vs_tpes_delayed_1yr()
    name = 6
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def share_res_vs_tpes_delayed_1yr():
    """
    share RES vs TPES delayed 1yr



    component


    """

    ret =  delay_share_res_vs_tpes_round1__time_step___time_step_0123_1__time_step()
    name = 6
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def gtl_production():
    """
    GTL production



    component


    """

    ret =  gtl_potential_production() * (1 - xshare_ctlxgtl_overcapacityx())
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
# RES - PED from non-electric RES
# GASES - PED, PES & FES
# TFES by fuel
@cache('step')
def pes_biogas_for_tfc():
    """
    PES biogas for TFC



    component


    """

    ret =  np.minimum(ped_gases(), potential_pes_biogas_for_tfc())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_ctlxgtl_overcapacityx():
    """
    Xshare CTLXGTL overcapacityX



    component


    """

    ret =  functions.zidz((xpotential_fes_ctlxgtl_ejx() - xfes_ctlxgtl_ejx()),
                          xpotential_fes_ctlxgtl_ejx())
    name = 6
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def share_res_vs_tfec_delayed_1yr():
    """
    share RES vs TFEC delayed 1yr



    component


    """

    ret =  delay_share_res_vs_tfec_round1__time_step___time_step_01614_1__time_step()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_exlosses_ccx():
    """
    Xshare EXlosses CCX



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0,
                                  functions.if_then_else(abs(xactivate_elf_all_scenxx() ) < equality_threshold, 0,
                                                         functions.if_then_else(
                                                             abs(xactivate_elf_by_scenxx() ) < equality_threshold, 0,
                                                             xshare_exlosses_cc_from_2015x())))
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xped_natx_gas_for_gtl_ejx():
    """
    XPED natX gas for GTL EJX



    component


    """

    ret =  gtl_production() / gtl_efficiency()
    name = 6
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def max_unconv_gas_growth_extraction():
    """
    max unconv gas growth extraction



    component


    """

    ret =  1 + functions.if_then_else(
        abs(selection_constraint_extraction_unconv_gas() - 1) < equality_threshold,
        (p_constraint_growth_extraction_unconv_gas()) * time_step() * scarcity_conv_gas_stock(),
        (xuserxdefined_extraction_growth_unconv_gasx(time())) * time_step())
    name = 6
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
# SOLIDS - PED, PES & FES
# RESOURCES - Coal extraction
# RESOURCES - CTL & GTL supply
@cache('step')
def ped_coal_for_ctl_ej():
    """
    PED coal for CTL EJ



    component


    """

    ret =  ctl_production() / ctl_efficiency()
    name = 6
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def annual_tfec_res_growth_rate():
    """
    Annual TFEC RES growth rate



    component


    """

    ret =  -1 + tfec_res_ej() / tfec_res_delayed_1yr()
    name = 6
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
# LIQUIDS - PED, PES & FES
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# TPED by fuel
@cache('step')
def ped_total_oil_ej():
    """
    PED total oil EJ



    component


    """

    ret =  np.maximum(0, ped_nre_liquids() - xfes_ctlxgtl_ejx() - oil_refinery_gains_ej())
    name = 6
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def ctl_production():
    """
    CTL production



    component


    """

    ret =  ctl_potential_production() * (1 - xshare_ctlxgtl_overcapacityx())
    name = 6
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def ch4_emissions_coal_without_ctl():
    """
    CH4 emissions coal without CTL



    component


    """

    ret =  extraction_coal_emissions_relevant_ej() * gch4_per_mj_coal() * mj_per_ej() / g_per_mt()
    name = 6
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
# RES - PED from non-electric RES
@cache('step')
def pes_tot_res_for_heat():
    """
    PES tot RES for heat



    component


    """

    ret =  functions.sum(xpes_res_for_heatxcom_by_technx()) + functions.sum(
        xpes_res_for_heatxnc_by_technx()) + xpes_tot_biogas_for_heatxcomx()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_fe_real_supply_res_for_heatxnc_ejx():
    """
    XTotal FE real supply RES for heatXnc EJX



    component


    """

    ret =  functions.sum(xfe_real_generation_res_heatxnc_ejx()) * xdeactivate_heat_dem_correctionxx()
    name = 6
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
# IMPACTS - Total GHG emissions
@cache('step')
def real_extraction_conv_oil_emissions_relevant_ej():
    """
    real extraction conv oil emissions relevant EJ



    component


    """

    ret =  np.maximum(
        0,
        real_extraction_conv_oil_ej() -
        (xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'liquids'}].values) * share_conv_vs_total_oil_extraction())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xnew_res_capacity_for_heatxnc_twx():
    """
    Xnew RES capacity for heatXnc TWX



    component


    """

    ret =  xr.DataArray(
        data=[functions.if_then_else(
            time() < 2013,
            xhistoric_res_capacity_for_heatxncx_subxsolarxheatx(time() + 1) -
            xhistoric_res_capacity_for_heatxncx_subxsolarxheatx(time()),
            xadapt_growth_res_for_heatxncx().loc[{'res heat':'xsolarxheatx'}].values * xinstalled_capacity_res_heatxnc_twx().loc[{'res heat':'xsolarxheatx'}].values * remaining_potential_constraint_on_new_res_heat_capacity().loc[{'res heat':'xsolarxheatx'}].values) *
        xabundance_res_heatxnc2x() * xdeactivate_heat_dem_correctionxx(),

        functions.if_then_else(
            time() < 2013,
            xhistoric_res_capacity_for_heatxncx_subxgeotxheatx(time() + 1) -
            xhistoric_res_capacity_for_heatxncx_subxgeotxheatx(time()),
            xadapt_growth_res_for_heatxncx().loc[{'res heat':'xgeotxheatx'}].values * xinstalled_capacity_res_heatxnc_twx().loc[{'res heat':'xgeotxheatx'}].values * remaining_potential_constraint_on_new_res_heat_capacity().loc[{'res heat':'xgeotxheatx'}].values) *
        xabundance_res_heatxnc2x() * xdeactivate_heat_dem_correctionxx(),

        functions.if_then_else(
            time() < 2013,
            xhistoric_res_capacity_for_heatxncx_subxsolidbioexheatx(time() + 1) -
            xhistoric_res_capacity_for_heatxncx_subxsolidbioexheatx(time()),
            xadapt_growth_res_for_heatxncx().loc[{'res heat':'xsolid bioexheatx'}].values * xinstalled_capacity_res_heatxnc_twx().loc[{'res heat':'xsolid bioexheatx'}].values * remaining_potential_constraint_on_new_res_heat_capacity().loc[{'res heat':'xsolid bioexheatx'}].values) *
        xabundance_res_heatxnc2x() * xdeactivate_heat_dem_correctionxx()],
        coords={'res heat':['xsolarxheatx','xgeotxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 6
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def co2_emissions_coal_without_ctl():
    """
    CO2 emissions coal without CTL



    component


    """

    ret =  extraction_coal_emissions_relevant_ej() * gco2_per_mj_coal() * mj_per_ej() / g_per_gt()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xannual_variation_nonxenergy_usex():
    """
    XAnnual variation nonXenergy useX



    component


    """

    ret =  xr.DataArray(
        data=[functions.if_then_else(time() > 2009,
                               xvariation_nonxenergy_usex().loc[{'final sources':'electricity'}].values,
                               xhistoric_nonxenergy_usex_subelectricity(time() + 1) -
                               xhistoric_nonxenergy_usex_subelectricity(time())),

        functions.if_then_else(time() > 2009,
                               xvariation_nonxenergy_usex().loc[{'final sources':'heat'}].values,
                               xhistoric_nonxenergy_usex_subheat(time() + 1) -
                               xhistoric_nonxenergy_usex_subheat(time())),

        functions.if_then_else(time() > 2009,
                               xvariation_nonxenergy_usex().loc[{'final sources':'liquids'}].values,
                               xhistoric_nonxenergy_usex_subliquids(time() + 1) -
                               xhistoric_nonxenergy_usex_subliquids(time())),

        functions.if_then_else(time() > 2009,
                               xvariation_nonxenergy_usex().loc[{'final sources':'gases'}].values,
                               xhistoric_nonxenergy_usex_subgases(time() + 1) -
                               xhistoric_nonxenergy_usex_subgases(time())),

        functions.if_then_else(time() > 2009,
                               xvariation_nonxenergy_usex().loc[{'final sources':'solids'}].values,
                               xhistoric_nonxenergy_usex_subsolids(time() + 1) -
                               xhistoric_nonxenergy_usex_subsolids(time()))],
        coords={'final sources':['electricity','heat','liquids','gases','solids']},
        dims=['final sources'])
    name = 6
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
# IMPACTS - Total GHG emissions
@cache('step')
def real_extraction_conv_gas_emissions_relevant_ej():
    """
    real extraction conv gas emissions relevant EJ



    component


    """

    ret =  np.maximum(0,
                      real_extraction_conv_gas_ej() -
                      (xped_natx_gas_for_gtl_ejx() + xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'gases'}].values) *
                      share_conv_vs_total_gas_extraction())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_res_for_heatxnc_by_technx():
    """
    XPES RES for heatXnc by technX



    component


    """

    ret =  xr.DataArray(
        data=[xfe_real_generation_res_heatxnc_ejx().loc[{'res heat':'xgeotxheatx'}].values / efficiency_res_heat().loc[{'res heat':'xgeotxheatx'}].values,

        xfe_real_generation_res_heatxnc_ejx().loc[{'res heat':'xsolarxheatx'}].values * efficiency_solar_panels_for_heat() /
        efficiency_res_heat().loc[{'res heat':'xsolarxheatx'}].values,

        xfe_real_generation_res_heatxnc_ejx().loc[{'res heat':'xsolid bioexheatx'}].values / efficiency_res_heat().loc[{'res heat':'xsolid bioexheatx'}].values],
        coords={'res heat':['xgeotxheatx','xsolarxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 6
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
@cache('step')
def fes_nre_for_heat():
    """
    FES NRE for heat



    component


    """

    ret =  xfes_heatxcom_fossil_fuels_chp_plants_ejx() + fes_heat_from_coal(
    ) + xfes_heat_from_natx_gasx() + fes_heat_from_oil() + xfes_heatxcom_nuclear_chp_plants_ejx()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xinstalled_capacity_res_heatxnc_twx():
    """
    Xinstalled capacity RES heatXnc TWX



    component


    """


    ret =  integ_xinstalled_capacity_res_heatxnc_twx()
    name = 6
    return round_sig(x=ret, name=name)

# RES - Employment
# PES solids biofuels & waste
# COMP - Historical data
@cache('step')
def pes_solids_bioe_ej():
    """
    PES solids bioE EJ



    component


    """

    ret =  losses_in_charcoal_plants_ej(
        time()) + pe_bioe_for_elec_generation_ej() + pe_traditional_biomass_ej_delayed_1yr(
        ) + xpes_res_for_heatxcom_by_technx().loc[{'res heat':'xsolid bioexheatx'}].values + xpes_res_for_heatxnc_by_technx().loc[{'res heat':'xsolid bioexheatx'}].values
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_heatxcom_plants_fossil_fuels_ejx():
    """
    XFED HeatXcom plants fossil fuels EJX



    component


    """

    ret =  np.maximum(xfed_heatxcom_nre_ejx() - xfes_heatxcom_fossil_fuels_chp_plants_ejx() -
                      xfes_heatxcom_nuclear_chp_plants_ejx(), 0)
    name = 6
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
# IMPACTS - Total GHG emissions
@cache('step')
def real_extraction_unconv_gas_emissions_relevant_ej():
    """
    real extraction unconv gas emissions relevant EJ



    component


    """

    ret =  np.maximum(0,
                      real_extraction_unconv_gas_ej() -
                      (xped_natx_gas_for_gtl_ejx() + xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'gases'}].values) *
                      (1 - share_conv_vs_total_gas_extraction()))
    name = 6
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
# IMPACTS - Total GHG emissions
@cache('step')
def real_extraction_unconv_oil_emissions_relevant_ej():
    """
    real extraction unconv oil emissions relevant EJ



    component


    """

    ret =  np.maximum(0,
                      real_extraction_unconv_oil_ej() -
                      (xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'liquids'}].values) *
                      (1 - share_conv_vs_total_oil_extraction()))
    name = 6
    return round_sig(x=ret, name=name)

# RES BIOE - Elec&Heat potential
@cache('step')
def available_pe_potential_solid_bioe_for_elec_ej():
    """
    available PE potential solid bioE for elec EJ



    component


    """

    ret =  np.maximum(0,
                      xtotal_pe_solid_bioe_potential_heatxelec_ejx() -
                      xpes_res_for_heatxcom_by_technx().loc[{'res heat':'xsolid bioexheatx'}].values - xpes_res_for_heatxnc_by_technx().loc[{'res heat':'xsolid bioexheatx'}].values)
    name = 6
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
@cache('step')
def remaining_potential_res_for_heat():
    """
    remaining potential RES for heat



    component


    """

    ret =  (max_fe_potential_res_for_heat() - xpotential_fes_res_for_heatxcom_ejx() -
            xpotential_fes_res_for_heatxnc_ejx()) / max_fe_potential_res_for_heat()
    name = 6
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
# IMPACTS - Total GHG emissions
@cache('step')
def extraction_coal_emissions_relevant_ej():
    """
    extraction coal emissions relevant EJ



    component


    """

    ret =  np.maximum(
        0, extraction_coal_without_ctl_ej() - xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'solids'}].values)
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xwear_res_capacity_for_heatxnc_twx():
    """
    Xwear RES capacity for heatXnc TWX



    component


    """

    ret =  xinstalled_capacity_res_heatxnc_twx() / life_time_res_for_heat()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xpotential_fes_res_for_heatxnc_twhx():
    """
    Xpotential FES RES for heatXnc TWhX



    component


    """

    ret =  xinstalled_capacity_res_heatxnc_twx() * efficiency_res_heat() * cp_res_for_heat(
    ) / twe_per_twh()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def abundance_res_heat2_0():
    """
    abundance RES heat2 0



    component


    """

    ret =  np.sqrt(abundance_res_heat_0())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xfe_real_generation_res_heatxnc_ejx():
    """
    XFE real generation RES heatXnc EJX



    component


    """

    ret =  xpotential_fes_res_for_heatxnc_ejx() * (1 - xres_heatxnc_tot_overcapacityx())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_fe_real_supply_res_for_heatxcom_ejx():
    """
    XTotal FE real supply RES for heatXcom EJX



    component


    """

    ret =  functions.sum(xfe_real_generation_res_heatxcom_ejx())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xnew_res_capacity_for_heatxcom_tw_0x():
    """
    Xnew RES capacity for heatXcom TW 0X



    component


    """

    ret =  xr.DataArray(
        data=[functions.if_then_else(time() < 2013,
                               xhistoric_res_capacity_for_heatxcom_0x_subxsolarxheatx(time() + 1) -
                               xhistoric_res_capacity_for_heatxcom_0x_subxsolarxheatx(time()),
                               adapt_growth_res_for_heat_0().loc[{'res heat':'xsolarxheatx'}].values * installed_capacity_res_heat_tw_0().loc[{'res heat':'xsolarxheatx'}].values * remaining_potential_constraint_on_new_res_heat_capacity_0().loc[{'res heat':'xsolarxheatx'}].values) *
        abundance_res_heat2_0(),

        functions.if_then_else(time() < 2013,
                               xhistoric_res_capacity_for_heatxcom_0x_subxgeotxheatx(time() + 1) -
                               xhistoric_res_capacity_for_heatxcom_0x_subxgeotxheatx(time()),
                               adapt_growth_res_for_heat_0().loc[{'res heat':'xgeotxheatx'}].values * installed_capacity_res_heat_tw_0().loc[{'res heat':'xgeotxheatx'}].values * remaining_potential_constraint_on_new_res_heat_capacity_0().loc[{'res heat':'xgeotxheatx'}].values) *
        abundance_res_heat2_0(),

        functions.if_then_else(
            time() < 2013,
            xhistoric_res_capacity_for_heatxcom_0x_subxsolidbioexheatx(time() + 1) -
            xhistoric_res_capacity_for_heatxcom_0x_subxsolidbioexheatx(time()),
            adapt_growth_res_for_heat_0().loc[{'res heat':'xsolid bioexheatx'}].values * installed_capacity_res_heat_tw_0().loc[{'res heat':'xsolid bioexheatx'}].values *
            remaining_potential_constraint_on_new_res_heat_capacity_0().loc[{'res heat':'xsolid bioexheatx'}].values) * abundance_res_heat2_0()],
        coords={'res heat':['xsolarxheatx','xgeotxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def initial_value_res_for_heat_0():
    """
    initial value RES for heat 0



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[2.03033268e-06], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.00199927], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(
            data=[0.01167364], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 6
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_coal_heat_ej():
    """
    share coal Heat EJ



    component


    """

    ret =  fed_heat_coal_plants_ej() / total_fed_heat_ej()
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def xpast_res_growth_for_heatxcom_0x():
    """
    Xpast RES growth for heatXcom 0X



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.2771226], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.03571556], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(
            data=[0.06278667], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def pes_res_for_heat_by_techn_0():
    """
    PES RES for heat by techn 0



    component


    """

    ret =  xr.DataArray(
        data=[xfe_real_generation_res_heatxcom_ej_0x().loc[{'res heat':'xgeotxheatx'}].values / efficiency_res_heat_0().loc[{'res heat':'xgeotxheatx'}].values,

        xfe_real_generation_res_heatxcom_ej_0x().loc[{'res heat':'xsolarxheatx'}].values * efficiency_solar_panels_for_heat_0() /
        efficiency_res_heat_0().loc[{'res heat':'xsolarxheatx'}].values,

        xfe_real_generation_res_heatxcom_ej_0x().loc[{'res heat':'xsolid bioexheatx'}].values / efficiency_res_heat_0().loc[{'res heat':'xsolid bioexheatx'}].values],
        coords={'res heat':['xgeotxheatx','xsolarxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def installed_capacity_res_heat_tw_0():
    """
    installed capacity RES heat TW 0



    component


    """


    ret =  integ_installed_capacity_res_heat_tw_0()
    name = 6
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_renewables_heat_ej():
    """
    share renewables Heat EJ



    component


    """

    ret =  (xfe_real_supply_res_for_heatxcom_tot_ejx() + xfe_real_supply_res_for_heatxnc_tot_ejx()
            ) / total_fed_heat_ej()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xfe_real_generation_res_heatxcom_ej_0x():
    """
    XFE real generation RES heatXcom EJ 0X



    component


    """

    ret =  potential_fes_res_for_heat_ej_0() * (1 - res_heat_tot_overcapacity_0())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def remaining_potential_constraint_on_new_res_heat_capacity_0():
    """
    remaining potential constraint on new RES heat capacity 0



    component


    """

    ret =  functions.if_then_else(
        remaining_potential_res_for_heat_0() > threshold_remaining_potential_new_capacity(), 1,
        remaining_potential_res_for_heat_0() * (1 / threshold_remaining_potential_new_capacity()))
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def efficiency_conversion_bioe_plants_to_heat_0():
    """
    Efficiency conversion BioE plants to heat 0



    constant


    """

    ret =  0.55
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_heatxcom_after_priorities_ejx():
    """
    XFED HeatXcom after priorities EJX



    component


    """

    ret =  np.maximum(0,
                      xtotal_fed_heatxcom_ejx() - xfes_heatxcom_from_waste_ejx() -
                      xfes_heatxcom_from_biogas_ejx())
    name = 6
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
@cache('step')
def fes_heat_from_biow():
    """
    FES heat from BioW



    component


    """

    ret =  xfe_real_supply_res_for_heatxcom_tot_ejx() + xfe_real_supply_res_for_heatxnc_tot_ejx(
    ) + xfes_heatxcom_from_biogas_ejx() + xfes_heatxcom_from_waste_ejx()
    name = 6
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_fossil_fuel_chp_ej():
    """
    share fossil fuel CHP EJ



    component


    """

    ret =  (np.maximum(xfes_heatxcom_fossil_fuels_chp_plants_ejx() -
                       xpotential_fes_heatxcom_nuclear_chp_plants_ejx(), 0)) / total_fed_heat_ej()
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def adapt_growth_res_for_heat_0():
    """
    adapt growth RES for heat 0



    component


    """

    ret =  functions.if_then_else(
        time() < 2014,
        xpast_res_growth_for_heatxcom_0x(),
        functions.if_then_else(time() < 2019,
                               xpast_res_growth_for_heatxcom_0x() +
                               (p_res_for_heat_0() - xpast_res_growth_for_heatxcom_0x()) *
                               (time_dmnl(time()) - 2014) / 5, p_res_for_heat_0()))
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xabundance_res_heatxncx():
    """
    Xabundance RES heatXncX



    component


    """

    ret =  functions.if_then_else(
        abs(xtotal_fed_heatxnc_ejx() ) < equality_threshold, 0,
        functions.zidz((xtotal_fed_heatxnc_ejx() - xfe_real_supply_res_for_heatxnc_tot_ejx()),
                       xtotal_fed_heatxnc_ejx()))
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def remaining_potential_res_for_heat_0():
    """
    remaining potential RES for heat 0



    component


    """

    ret =  (max_fe_potential_res_for_heat() - potential_fes_res_for_heat_ej_0()
            ) / max_fe_potential_res_for_heat()
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def efficiency_geothermal_for_heat_0():
    """
    Efficiency geothermal for heat 0



    constant


    """

    ret =  0.6960561017716173
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_heatxcom_nuclear_chp_plants_ejx():
    """
    XFES HeatXcom nuclear CHP plants EJX



    component


    """

    ret =  np.minimum(xpotential_fes_heatxcom_nuclear_chp_plants_ejx(), xfed_heatxcom_nre_ejx())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xfe_real_supply_res_for_heatxnc_tot_ejx():
    """
    XFE real supply RES for heatXnc tot EJX



    component


    """

    ret =  np.minimum(
        np.maximum(xtotal_fed_heatxnc_ejx(), 0), xpotential_fes_tot_res_for_heatxnc_ejx())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def res_heat_tot_overcapacity_0():
    """
    RES heat tot overcapacity 0



    component


    """

    ret =  functions.if_then_else(
        abs(potential_fes_tot_res_for_heat_ej_0() ) < equality_threshold, 0,
        (potential_fes_tot_res_for_heat_ej_0() - fe_real_supply_res_for_heat_tot_ej_0()) /
        potential_fes_tot_res_for_heat_ej_0())
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def xinitial_value_res_for_heatxncx():
    """
    Xinitial value RES for heatXncX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.03252055], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.01718051], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(
            data=[0.37315399], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 6
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_gases_heat_ej():
    """
    share gases Heat EJ



    component


    """

    ret =  fed_heat_gas_plants_ej() / total_fed_heat_ej()
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def xpast_res_growth_for_heatxncx():
    """
    Xpast RES growth for heatXncX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.14412798], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.077], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(
            data=[0.03534992], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def wear_res_capacity_for_heat_tw_0():
    """
    wear RES capacity for heat TW 0



    component


    """

    ret =  installed_capacity_res_heat_tw_0() / life_time_res_for_heat_0()
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def p_solid_bioe_for_heat_0():
    """
    P solid bioE for heat 0



    constant


    """

    ret =  0.063
    name = 6
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def fed_heat_fossil_fuels_chp_plants_ej():
    """
    FED heat fossil fuels CHP plants EJ



    component


    """

    ret =  np.maximum(
        xfed_heatxcom_by_nre_chp_plants_ejx() - xfes_heatxcom_nuclear_chp_plants_ejx(), 0)
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def xreplacement_res_for_heatxncx():
    """
    Xreplacement RES for heatXncX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[1.], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[1.], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[1.], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def potential_fes_res_for_heat_ej_0():
    """
    potential FES RES for heat EJ 0



    component


    """

    ret =  potential_fes_res_for_heat_twh_0() * ej_per_twh()
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def efficiency_solar_panels_for_heat_0():
    """
    Efficiency solar panels for heat 0



    constant


    """

    ret =  0.095
    name = 6
    return round_sig(x=ret, name=name)

@cache('run')
def cp_res_for_heat_0():
    """
    Cp RES for heat 0



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[1.], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.265], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.5], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def xreplacement_res_for_heatxnc_twx():
    """
    Xreplacement RES for heatXnc TWX



    component


    """

    ret =  xwear_res_capacity_for_heatxnc_twx() * xreplacement_res_for_heatxncx() * (
        1 - xres_heatxnc_tot_overcapacityx())
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def p_res_for_heat_0():
    """
    P RES for heat 0



    component


    """

    ret =  xr.DataArray(
        data=[p_solar_for_heat_0(),

        p_geothermal_for_heat_0(),

        p_solid_bioe_for_heat_0()],
        coords={'res heat':['xsolarxheatx','xgeotxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 6
    return round_sig(x=ret, name=name)

@cache('step')
def abundance_res_heat_0():
    """
    abundance RES heat 0



    component


    """

    ret =  functions.if_then_else(
        abs(xfed_heatxcom_after_priorities_ejx() ) < equality_threshold, 0,
        functions.zidz(
            (xfed_heatxcom_after_priorities_ejx() - fe_real_supply_res_for_heat_tot_ej_0()),
            xfed_heatxcom_after_priorities_ejx()))
    name = 7
    return round_sig(x=ret, name=name)

@cache('run')
def p_solar_for_heat_0():
    """
    P solar for heat 0



    constant


    """

    ret =  0.1
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xabundance_res_heatxcomx():
    """
    Xabundance RES heatXcomX



    component


    """

    ret =  functions.if_then_else(
        abs(xfed_heatxcom_after_priorities_ejx() ) < equality_threshold, 0,
        functions.zidz(
            (xfed_heatxcom_after_priorities_ejx() - xfe_real_supply_res_for_heatxcom_tot_ejx()),
            xfed_heatxcom_after_priorities_ejx()))
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xabundance_res_heatxnc2x():
    """
    Xabundance RES heatXnc2X



    component


    """

    ret =  np.sqrt(xabundance_res_heatxncx())
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xpotential_fes_tot_res_for_heatxnc_ejx():
    """
    Xpotential FES tot RES for heatXnc EJX



    component


    """

    ret =  functions.sum(xpotential_fes_res_for_heatxnc_ejx())
    name = 7
    return round_sig(x=ret, name=name)

@cache('run')
def life_time_res_for_heat_0():
    """
    life time RES for heat 0



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[25.], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[25.], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[30.], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def efficiency_res_heat_0():
    """
    Efficiency RES heat 0



    component


    """

    ret =  xr.DataArray(
        data=[efficiency_solar_panels_for_heat_0() * losses_solar_for_heat_0(),

        efficiency_geothermal_for_heat_0(),

        efficiency_conversion_bioe_plants_to_heat_0()],
        coords={'res heat':['xsolarxheatx','xgeotxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 7
    return round_sig(x=ret, name=name)

def xhistoric_res_capacity_for_heatxcom_0x_subxsolarxheatx(x):
    """
    XHistoric RES capacity for heatXcom 0X SUBXsolarXheatX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        2.0303326826418786e-06, 2.0303326826418786e-06, 6.4293868283659495e-06,
        5.414220487045009e-06, 7.782941950127201e-06, 8.121330730567515e-06, 9.136497071888453e-06,
        1.2520384876291584e-05, 1.7257827802455968e-05, 1.7257827802455968e-05,
        1.8611382924217222e-05, 1.658105024157534e-05, 2.0980104387299415e-05,
        2.3348825850381605e-05, 4.7036040481203515e-05, 6.497064584454012e-05,
        9.339530340152642e-05, 0.00014246167656537182, 0.00018137638631600782,
        0.0002493925311845108
    ])
    name = 7
    return round_sig(x=ret, name=name)

def xhistoric_res_capacity_for_heatxcom_0x_subxgeotxheatx(x):
    """
    XHistoric RES capacity for heatXcom 0X SUBXgeotXheatX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.0019992724712776778, 0.0019333400358171797, 0.001957511273499612, 0.0019656481257887483,
        0.0020795640578366507, 0.002191445776812269, 0.0022711390654088053, 0.00233635354331438,
        0.002309190815819764, 0.002360764100181787, 0.002849214556714914, 0.002853282982859482,
        0.002944702911519773, 0.00304868709739123, 0.0030231996041914363, 0.003124551279028173,
        0.003072140377518739, 0.003306194540423882, 0.003420230132064272, 0.0035817705819221166
    ])
    name = 7
    return round_sig(x=ret, name=name)

def xhistoric_res_capacity_for_heatxcom_0x_subxsolidbioexheatx(x):
    """
    XHistoric RES capacity for heatXcom 0X SUBXsolidbioEXheatX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.01167364283024201, 0.014963216653269409, 0.014075786414101827, 0.014111998996576254,
        0.013306506859960276, 0.01325437596189726, 0.014136859473511416, 0.014877727054012784,
        0.016775811784095436, 0.017262303413099544, 0.01806468799009863, 0.01981614664193105,
        0.020802194334247032, 0.02147558347735297, 0.022805745832552054, 0.02708885085880959,
        0.0262413115379895, 0.029897640815394526, 0.030840372931826027, 0.03108885086200959
    ])
    name = 7
    return round_sig(x=ret, name=name)

@cache('run')
def replacement_res_for_heat_0():
    """
    replacement RES for heat 0



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[1.], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[1.], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[1.], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 7
    return round_sig(x=ret, name=name)

@cache('run')
def p_geothermal_for_heat_0():
    """
    P geothermal for heat 0



    constant


    """

    ret =  0.077
    name = 7
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
# TFES by fuel
@cache('step')
def fes_res_for_heat_ej():
    """
    FES RES for heat EJ



    component


    """

    ret =  xfe_real_supply_res_for_heatxcom_tot_ejx() + xfe_real_supply_res_for_heatxnc_tot_ejx(
    ) + xfes_heatxcom_from_biogas_ejx()
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xfe_real_supply_res_for_heatxcom_tot_ejx():
    """
    XFE real supply RES for heatXcom tot EJX



    component


    """

    ret =  np.minimum(
        np.maximum(xfed_heatxcom_after_priorities_ejx(), 0),
        xpotential_fes_tot_res_for_heatxcom_ejx())
    name = 7
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_liquids_heat_ej():
    """
    share liquids Heat EJ



    component


    """

    ret =  fed_heat_liquids_plants_ej() / total_fed_heat_ej()
    name = 7
    return round_sig(x=ret, name=name)

@cache('run')
def losses_solar_for_heat_0():
    """
    Losses solar for heat 0



    constant


    """

    ret =  0.37
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def potential_fes_tot_res_for_heat_ej_0():
    """
    potential FES tot RES for heat EJ 0



    component


    """

    ret =  functions.sum(potential_fes_res_for_heat_ej_0())
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def potential_fes_res_for_heat_twh_0():
    """
    potential FES RES for heat TWh 0



    component


    """

    ret =  installed_capacity_res_heat_tw_0() * efficiency_res_heat_0() * cp_res_for_heat_0(
    ) / twe_per_twh()
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def fe_real_supply_res_for_heat_tot_ej_0():
    """
    FE real supply RES for heat tot EJ 0



    component


    """

    ret =  np.minimum(
        np.maximum(xfed_heatxcom_after_priorities_ejx(), 0), potential_fes_tot_res_for_heat_ej_0())
    name = 7
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_nuclear_heat_ej():
    """
    share nuclear Heat EJ



    component


    """

    ret =  xpotential_fes_heatxcom_nuclear_chp_plants_ejx() / total_fed_heat_ej()
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xres_heatxnc_tot_overcapacityx():
    """
    XRES heatXnc tot overcapacityX



    component


    """

    ret =  functions.if_then_else(
        abs(xpotential_fes_tot_res_for_heatxnc_ejx() ) < equality_threshold, 0,
        (xpotential_fes_tot_res_for_heatxnc_ejx() - xfe_real_supply_res_for_heatxnc_tot_ejx()) /
        xpotential_fes_tot_res_for_heatxnc_ejx())
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xpotential_fes_res_for_heatxnc_ejx():
    """
    Xpotential FES RES for heatXnc EJX



    component


    """

    ret =  xpotential_fes_res_for_heatxnc_twhx() * ej_per_twh()
    name = 7
    return round_sig(x=ret, name=name)

def xhistoric_res_capacity_for_heatxncx_subxsolarxheatx(x):
    """
    XHistoric RES capacity for heatXncX SUBXsolarXheatX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.03252055114522492, 0.037030040661937866, 0.04216484232697614, 0.04801166557416679,
        0.05466924347374749, 0.06225, 0.0685, 0.0756, 0.0849, 0.09559999999999999,
        0.10940000000000001, 0.12459999999999999, 0.1454, 0.1707, 0.2036, 0.24230000000000002,
        0.2863, 0.33080000000000004, 0.3757, 0.41
    ])
    name = 7
    return round_sig(x=ret, name=name)

def xhistoric_res_capacity_for_heatxncx_subxgeotxheatx(x):
    """
    XHistoric RES capacity for heatXncX SUBXgeotXheatX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.0171805111069092, 0.01850341046214121, 0.01992817306772608, 0.021462642393940985,
        0.02311526585827444, 0.024895141329361572, 0.026812067211722412, 0.028876596387025034,
        0.031100094308825963, 0.03349480157060556, 0.036073901291542185, 0.03885159169099093,
        0.04184316425119723, 0.04506508789853942, 0.04853509966672695, 0.05227230234106492,
        0.05629726962132692, 0.060632159382169086, 0.0653008356545961, 0.07032899999999999
    ])
    name = 7
    return round_sig(x=ret, name=name)

def xhistoric_res_capacity_for_heatxncx_subxsolidbioexheatx(x):
    """
    XHistoric RES capacity for heatXncX SUBXsolidbioEXheatX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.3731539881219632, 0.3810181376716334, 0.4133766435321216, 0.4781541017270185,
        0.46389100810830247, 0.4872018890249721, 0.4276920291346447, 0.5048192641024857,
        0.46605798172748863, 0.5476368224827572, 0.5095026881295702, 0.608024772175917,
        0.525800394256257, 0.6167711280936199, 0.4889753847138593, 0.7051383903408738,
        0.5670128296012508, 0.7401071536869618, 0.6308849028385484, 0.7923719533228881
    ])
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def replacement_res_for_heat_tw_0():
    """
    replacement RES for heat TW 0



    component


    """

    ret =  wear_res_capacity_for_heat_tw_0() * replacement_res_for_heat_0() * (
        1 - res_heat_tot_overcapacity_0())
    name = 7
    return round_sig(x=ret, name=name)

def historic_share_of_transformation_losses_vs_extraction_subliquids(x):
    """
    Historic share of transformation losses vs extraction SUBliquids



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.01605706386987077, 0.013375704567402669, 0.01129379949353825, 0.01014342848294583,
        0.011490351286760049, 0.009944630113635408, 0.009105457515074949, 0.010502419167205769,
        0.010548036524274081, 0.009593662119669003, 0.010125399533825435, 0.011405448684794902,
        0.008476172840581088, 0.009926004318436333, 0.01357902103740488, 0.008188000564044163,
        0.009297613425131876, 0.020291121242449806, 0.015035909876251948, 0.01611299762406862
    ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_share_of_transformation_losses_vs_extraction_subsolids(x):
    """
    Historic share of transformation losses vs extraction SUBsolids



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.12808887489468548, 0.12419446569792036, 0.1252050437569366, 0.11935691213136902,
        0.12016837210710733, 0.12425103898164969, 0.11836657177817682, 0.11980088894829675,
        0.12262630759437118, 0.12004068583025358, 0.12182831042908908, 0.1260774488688789,
        0.12589138460016347, 0.12208371603473156, 0.12167722490071634, 0.1253090144654484,
        0.1249926742022905, 0.12795495498588333, 0.1330103435651417, 0.13218471066181914
    ])
    name = 7
    return round_sig(x=ret, name=name)

# SOLIDS - PED, PES & FES
@cache('step')
def other_solids_required():
    """
    Other solids required



    component


    """

    ret =  transformation_losses_ej().loc[{'final sources':'solids'}].values + losses_ej().loc[{'final sources':'solids'}].values + xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'solids'}].values
    name = 7
    return round_sig(x=ret, name=name)
# LOSSES

def ratio_gain_gas_vs_lose_solids_in_tranf_processes(x):
    """
    Ratio gain gas vs lose solids in tranf processes



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        -0.4655436785765454, -0.47767082024331714, -0.4613465490489168, -0.4638943144481499,
        -0.4594885439207112, -0.4471825571722795, -0.4450852126345265, -0.4481817224089238,
        -0.43264382276329605, -0.43190874162897963, -0.4483483475515704, -0.44555528830304947,
        -0.44797554896001957, -0.45012744230484714, -0.43332372797703317, -0.44871221929050276,
        -0.4493572878390711, -0.42914870047197806, -0.4229488604826579, -0.419986716594219
    ])
    name = 7
    return round_sig(x=ret, name=name)

# GASES - PED, PES & FES
@cache('step')
def other_gases_required():
    """
    Other gases required



    component


    """

    ret =  transformation_losses_ej().loc[{'final sources':'gases'}].values + losses_ej().loc[{'final sources':'gases'}].values + xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'gases'}].values
    name = 7
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
# GASES - PED, PES & FES
# SOLIDS - PED, PES & FES
# LOSSES
@cache('step')
def transformation_losses_ej():
    """
    Transformation Losses EJ



    component


    """
    solids = pes_fossil_fuel_extraction_delayed().loc[{'final sources':'solids'}].values * historic_share_of_transformation_losses_vs_extraction_subsolids(time())

    ret =  xr.DataArray(
        data=[pes_fossil_fuel_extraction_delayed().loc[{'final sources':'liquids'}].values *
        historic_share_of_transformation_losses_vs_extraction_subliquids(time()),

        solids,
        0.0,

        solids * ratio_gain_gas_vs_lose_solids_in_tranf_processes(time()),
        0.0],
        coords={'final sources':['liquids','solids','electricity','gases','heat']},
        dims=['final sources'])
    name = 7
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# CHP plants - FES commercial Heat and Elec
@cache('step')
def fes_elec_fossil_fuel_chp_plants_ej():
    """
    FES Elec fossil fuel CHP plants EJ



    component


    """

    ret =  np.minimum(potential_fe_gen_elec_fossil_fuel_chp_plants_ej(),
                      demand_elec_nre_twh() * ej_per_twh())
    name = 7
    return round_sig(x=ret, name=name)
# COMP - GDP

def historic_gdp_wiod(x):
    """
    Historic GDP WIOD



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        29.158559, 29.975939, 30.917286, 31.742682, 33.126306, 34.350697, 34.844121, 35.257196,
        36.340931, 37.798858, 39.115093, 40.646471, 42.186741, 42.966299, 41.83907, 43.1431938119,
        44.18423907858114, 45.304309539223176, 46.53975806035779, 47.54967081026756
    ])
    name = 7
    return round_sig(x=ret, name=name)

# COMP - GDP
@cache('step')
def check_gdp():
    """
    check GDP



    component


    """

    ret =  functions.if_then_else(
        time() < 2009, (gdp() - historic_gdp_wiod(time())) * 100 / historic_gdp_wiod(time()), 0)
    name = 7
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES - Capacities and generation all techn
@cache('step')
def total_fe_elec_demand_after_priorities_twh():
    """
    Total FE Elec demand after priorities TWh



    component


    """

    ret =  np.maximum(total_fe_elec_demand_twh() - fes_elec_from_res_with_priority_twh() -
                      fes_elec_from_waste_twh(), 0)
    name = 7
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC - Electricity demand covered by fossil fuels
# ELEC - Demand of electricity from nuclear
# CHP plants - FES commercial Heat and Elec
@cache('step')
def demand_elec_nre_twh():
    """
    Demand Elec NRE TWh



    component


    """

    ret =  np.maximum(0,
                      total_fe_elec_demand_twh() - fe_tot_generation_all_res_elec_twh() -
                      fes_elec_from_waste_twh())
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xnonxenergy_use_demand_by_final_fuel_ejx():
    """
    XNonXenergy use demand by final fuel EJX



    component


    """


    ret =  integ_xnonxenergy_use_demand_by_final_fuel_ejx()
    name = 7
    return round_sig(x=ret, name=name)

# SOLIDS - PED, PES & FES
@cache('step')
def share_solids_for_final_energy():
    """
    share solids for final energy



    component


    """

    ret =  functions.zidz(required_fed_solids(),
                          (ped_solids() - ped_coal_for_ctl_ej() -
                           xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'solids'}].values - other_solids_required()))
    name = 7
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# GASES - PED, PES & FES
@cache('step')
def real_fe_consumption_gases_ej():
    """
    real FE consumption gases EJ



    component


    """

    ret =  (
        pes_gases() - xped_natx_gas_for_gtl_ejx() - xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'gases'}].values -
        other_gases_required()) * share_gases_for_final_energy()
    name = 7
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# SOLIDS - PED, PES & FES
@cache('step')
def real_fe_consumption_solids_ej():
    """
    real FE consumption solids EJ



    component


    """

    ret =  (extraction_coal_ej() +
            (pe_traditional_biomass_ej_delayed_1yr() + pes_waste_for_tfc() + pes_peat_ej() +
             losses_in_charcoal_plants_ej(time())) - ped_coal_for_ctl_ej() -
            xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'solids'}].values - other_solids_required()
            ) * share_solids_for_final_energy()
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_real_nonxenergy_use_consumption_ejx():
    """
    XTotal real nonXenergy use consumption EJX



    component


    """

    ret =  functions.sum(xnonxenergy_use_demand_by_final_fuel_ejx())
    name = 7
    return round_sig(x=ret, name=name)

# GASES - PED, PES & FES
@cache('step')
def share_gases_for_final_energy():
    """
    share gases for final energy



    component


    """

    ret =  functions.zidz(required_fed_by_gas(),
                          (ped_gases() - xped_natx_gas_for_gtl_ejx() -
                           xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'gases'}].values - other_gases_required()))
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xratio_fed_for_heatxnc_vs_fed_for_heatxcomx():
    """
    Xratio FED for heatXnc vs FED for heatXcomX



    component


    """

    ret =  functions.sum(xfed_by_fuel_for_heatxncx()) * functions.zidz(
        xdeactivate_heat_dem_correctionxx(), required_fed_by_fuel_before_heat_correction().loc[{'final sources':'heat'}].values)
    name = 7
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def real_fe_consumption_by_fuel_before_heat_correction():
    """
    real FE consumption by fuel before heat correction



    component


    """

    ret =  xr.DataArray(
        data=[real_fe_consumption_by_fuel().loc[{'final sources':'electricity'}].values,

        real_fe_consumption_by_fuel().loc[{'final sources':'heat'}].values / (1 + xratio_fed_for_heatxnc_vs_fed_for_heatxcomx()),

        real_fe_consumption_by_fuel().loc[{'final sources':'liquids'}].values / (1 - share_feh_over_fed_by_final_fuel().loc[{'final sources':'liquids'}].values),

        real_fe_consumption_by_fuel().loc[{'final sources':'gases'}].values / (1 - share_feh_over_fed_by_final_fuel().loc[{'final sources':'gases'}].values),

        real_fe_consumption_by_fuel().loc[{'final sources':'solids'}].values / (1 - share_feh_over_fed_by_final_fuel().loc[{'final sources':'solids'}].values)],
        coords={'final sources':['electricity','heat','liquids','gases','solids']},
        dims=['final sources'])
    name = 7
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def required_tfed_before_heat_dem_corr():
    """
    Required TFED before heat dem corr



    component


    """

    ret =  functions.sum(required_fed_by_fuel_before_heat_correction())
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for EV batteries
@cache('step')
def cum_materials_to_extract_for_ev_batteries():
    """
    cum materials to extract for EV batteries



    component


    """


    ret =  integ_cum_materials_to_extract_for_ev_batteries()
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def share_cum_dem_materials_to_extract_alt_techn_vs_total():
    """
    share cum dem materials to extract alt techn vs total



    component


    """

    ret =  functions.if_then_else(total_cumulative_demand_materials_to_extract_from_2015() <= 0, 0,
                                  (cum_materials_to_extract_for_alt_techn_from_2015()) /
                                  total_cumulative_demand_materials_to_extract_from_2015())
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for EV batteries
@cache('step')
def total_recycled_materials_for_ev_batteries_mt():
    """
    Total recycled materials for EV batteries Mt



    component


    """

    ret =  total_materials_required_for_ev_batteries(
    ) - total_materials_to_extract_for_ev_batteries_mt()
    name = 7
    return round_sig(x=ret, name=name)

# TFES by fuel
# INDICATORS - Energy
@cache('step')
def share_res_vs_tfec():
    """
    share RES vs TFEC



    component


    """

    ret =  functions.zidz(tfec_res_ej(), real_tfec())
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def cum_materials_to_extract_for_alt_techn_from_2015():
    """
    cum materials to extract for alt techn from 2015



    component


    """

    ret =  cum_materials_to_extract_for_ev_batteries_from_2015(
    ) + cum_materials_to_extract_for_res_elec_from_2015()
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for EV batteries
@cache('step')
def total_materials_to_extract_for_ev_batteries_from_2015_mt():
    """
    Total materials to extract for EV batteries from 2015 Mt



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0,
                                  total_materials_to_extract_for_ev_batteries_mt())
    name = 7
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_per_new_capacity_installed_x_ev_batteriesx():
    """
    Xmaterials per new capacity installed X EV batteriesX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[500.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[289.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[34.4], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[1631.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[125.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for EV batteries
# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def total_materials_to_extract_for_ev_batteries_mt():
    """
    Total materials to extract for EV batteries Mt



    component


    """

    ret =  total_materials_required_for_ev_batteries() * (1 - recycling_rates_minerals_alt_techn())
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def total_materials_to_extract_mt():
    """
    Total materials to extract Mt



    component


    """

    ret =  materials_to_extract_rest_mt() + total_materials_to_extract_for_res_elec_mt(
    ) + total_materials_to_extract_for_ev_batteries_mt()
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for EV batteries
@cache('step')
def cum_materials_requirements_for_ev_batteries():
    """
    cum materials requirements for EV batteries



    component


    """


    ret =  integ_cum_materials_requirements_for_ev_batteries()
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xxstaticx_eroigrid_res_elecx():
    """
    XXstaticX EROIgrid RES elecX



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(xxstaticx_eroi_res_elecx()) - less_thr_func(0, xxstaticx_eroi_res_elecx()), 0,
        (1 - share_res_elec_generation_stored() +
         share_res_elec_generation_stored() * rt_elec_storage_efficiency()) /
        (1 / xxstaticx_eroi_res_elecx() +
         share_res_elec_generation_stored() * rt_elec_storage_efficiency() / esoi_elec_storage()))
    name = 7
    return round_sig(x=ret, name=name)

@cache('run')
def t_per_x():
    """
    T per x



    constant


    """

    ret =  1000000000000
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for EV batteries
# MATERIALS - Rest demand
@cache('step')
def total_materials_required_for_ev_batteries():
    """
    Total materials required for EV batteries



    component


    """

    ret =  materials_required_for_ev_batteries_mt()
    name = 7
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def tfec_from_res_per_capita():
    """
    TFEC from RES per capita



    component


    """

    ret =  functions.zidz(tfec_res_ej() * gj_per_ej(), population())
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def share_materials_cum_demand_to_extract_vs_reserves_for_res_elec():
    """
    share materials cum demand to extract vs reserves for RES elec



    component


    """

    ret =  functions.if_then_else(
        abs(current_mineral_reserves_mt() ) < equality_threshold, 0,
        cum_materials_to_extract_for_alt_techn_from_2015() / current_mineral_reserves_mt())
    name = 7
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('run')
def rt_storage_efficiency_ev_batteries():
    """
    rt storage efficiency EV batteries



    constant


    """

    ret =  0.9
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for EV batteries
# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def cum_materials_to_extract_for_ev_batteries_from_2015():
    """
    cum materials to extract for EV batteries from 2015



    component


    """


    ret =  integ_cum_materials_to_extract_for_ev_batteries_from_2015()
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def total_cumulative_demand_materials_to_extract_from_2015():
    """
    total cumulative demand materials to extract from 2015



    component


    """

    ret =  cum_materials_to_extract_for_alt_techn_from_2015(
    ) + cum_materials_to_extract_rest_from_2015()
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for EV batteries
@cache('run')
def initial_cumulated_material_requirements_for_ev_batteries_1995():
    """
    initial cumulated material requirements for EV batteries 1995



    constant


    """

    ret =  0
    name = 7
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def share_materials_cum_demand_to_extract_vs_resources_for_res_elec():
    """
    share materials cum demand to extract vs resources for RES elec



    component


    """

    ret =  functions.if_then_else(
        abs(current_mineral_resources_mt() ) < equality_threshold, 0,
        cum_materials_to_extract_for_alt_techn_from_2015() / current_mineral_resources_mt())
    name = 7
    return round_sig(x=ret, name=name)

# TFES by fuel
@cache('step')
def fe_tot_generation_all_res_elec_ej():
    """
    FE tot generation all RES elec EJ



    component


    """

    ret =  fe_tot_generation_all_res_elec_twh() * ej_per_twh()
    name = 7
    return round_sig(x=ret, name=name)

@cache('step')
def xcarbon_footprint_tonnescxpersonx():
    """
    XCarbon footprint tonnesCXpersonX



    component


    """

    ret =  xcarbon_footprint_tco2xpersonx() * c_per_co2()
    name = 7
    return round_sig(x=ret, name=name)

# INDICATORS - Social and Environmental impacts
@cache('step')
def potential_max_hdi():
    """
    Potential max HDI



    component


    """

    ret =  functions.if_then_else(net_tfec_per_capita() <= 0, 0,
                                  np.minimum(1, 0.1395 * np.log(net_tfec_per_capita()) + 0.1508))
    name = 7
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
# ESOI - EV batteries
@cache('run')
def kw_per_battery_ev():
    """
    kW per battery EV



    constant


    """

    ret =  80
    name = 7
    return round_sig(x=ret, name=name)

# LOSSES
@cache('step')
def pes_fossil_fuel_extraction_delayed():
    """
    PES fossil fuel extraction delayed



    component


    """

    #ret =  delay_pes_fossil_fuel_extraction_by_hand()
    ret = xr.DataArray(data=[delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_139_time_step__time_step(), delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_101_time_step__time_step(), delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_79_time_step__time_step(), 0, 0], coords={'final sources': ['liquids','solids','gases','electricity','heat']}, dims=['final sources'])
    name = 7
    return round_sig(x=ret, name=name)

# RES BIOE - Traditional biomass
# TFES by fuel
@cache('step')
def pe_traditional_biomass_ej():
    """
    PE traditional biomass EJ



    component


    """

    ret =  households_final_energy_demand().loc[{'final sources':'solids'}].values * share_trad_biomass_vs_solids_in_households()
    name = 7
    return round_sig(x=ret, name=name)

def historic_share_of_losses_vs_extraction_subliquids(x):
    """
    Historic share of losses vs extraction SUBliquids



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0012739789373926223, 0.0017486800026021427, 0.0013106160395248033,
            0.0015007993499810536, 0.0015448879729707073, 0.0017210004587693457,
            0.0016050651209882875, 0.001608462807127648, 0.0016644010289518145,
            0.0016083865954213087, 0.0017099881829806055, 0.0018255697673978206,
            0.0015685355302594128, 0.0018991216520622383, 0.0017500263224406436
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_share_of_losses_vs_extraction_subsolids(x):
    """
    Historic share of losses vs extraction SUBsolids



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0038436874766076485, 0.003489742533787759, 0.0027225260873578864,
            0.0026049521185396614, 0.0026362712696692743, 0.001098585769199882,
            0.0006980968889870701, 0.0006737200345284238, 0.0008583882245816353,
            0.0009023672127944747, 0.0009568153334068469, 0.0008955097842159121,
            0.0010495767208806045, 0.0008108439599839434, 0.0006342725505040347
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_share_of_losses_vs_extraction_subgases(x):
    """
    Historic share of losses vs extraction SUBgases



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.009422896256783414, 0.010421118505311026, 0.008944163046561274, 0.008947414438666463,
            0.008670592726980444, 0.008904855075100917, 0.007845516574254089, 0.007085866070235719,
            0.006639624981043146, 0.007052502263863183, 0.00934137005114715, 0.009487119248233027,
            0.007689650174055921, 0.008058662926939446, 0.007826223652392038
        ])
    name = 7
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
# GASES - PED, PES & FES
# SOLIDS - PED, PES & FES
# LOSSES
@cache('step')
def losses_ej():
    """
    Losses EJ



    component


    """

    ret =  xr.DataArray(
        data=[pes_fossil_fuel_extraction_delayed().loc[{'final sources':'liquids'}].values * historic_share_of_losses_vs_extraction_subliquids(
            time()),

        pes_fossil_fuel_extraction_delayed().loc[{'final sources':'solids'}].values * historic_share_of_losses_vs_extraction_subsolids(
            time()),

        pes_fossil_fuel_extraction_delayed().loc[{'final sources':'gases'}].values * historic_share_of_losses_vs_extraction_subgases(
            time()),
        0.0,
        0.0],
        coords={'final sources':['liquids','solids','gases','electricity','heat']},
        dims=['final sources'])
    name = 7
    return round_sig(x=ret, name=name)

# LOSSES
@cache('step')
def pes_fossil_fuel_extraction():
    """
    PES fossil fuel extraction



    component


    """

    ret =  xr.DataArray(data=[pes_oil_ej(),extraction_coal_ej(),xpes_natx_gasx(),0,0], coords={'final sources': ['liquids','solids','gases','electricity','heat']}, dims=['final sources'])
    name = 7
    return round_sig(x=ret, name=name)

# RES BIOE - Traditional biomass
@cache('step')
def modern_bioe_in_households():
    """
    modern BioE in households



    component


    """

    ret =  households_final_energy_demand().loc[{'final sources':'solids'}].values - pe_traditional_biomass_ej()
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subagriculturehuntingforestryandfishing(x):
    """
    Historic demand by sector SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            821277.3516440002, 835480.3083830001, 852339.710031, 867561.326752, 888737.4700559999,
            889964.025429, 898181.2951590001, 913098.1065810001, 929168.457933, 940175.14705,
            938211.6497899999, 950724.4604299999, 953269.03026, 1000267.1132700001, 1025427.57278
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subminingandquarrying(x):
    """
    Historic demand by sector SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            106956.558, 101326.61625099998, 111658.141394, 89949.42214700002, 97189.104822,
            113437.34111929999, 118329.25110650001, 115389.89345049998, 114746.10045029999,
            123733.9133598, 140981.9255457, 164113.0972093, 158758.56715390002, 171002.8987663,
            125310.6947264
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subfoodbeveragesandtobacco(x):
    """
    Historic demand by sector SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1761460.089629, 1768733.999874, 1810212.34125, 1861146.10074, 1921397.32094,
            1967617.76054, 1982124.37565, 1977751.3650200001, 2016883.86587, 2061797.69482,
            2122361.6047400003, 2188629.83772, 2273095.40383, 2285147.92467, 2264271.11142
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subtextilesandtextileproducts(x):
    """
    Historic demand by sector SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            500387.93122930004, 496732.7349816, 502418.7662404, 505669.1392506, 516506.8589314,
            543756.6590378999, 544729.2685248, 549154.9920234001, 546981.0669373, 562149.7308112,
            581835.5048554, 616268.9163088, 643652.2674367, 647357.3118739999, 614923.9903501001
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subleatherleatherandfootwear(x):
    """
    Historic demand by sector SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            102249.58580294, 102739.65463260999, 102069.93810505999, 101012.66603034,
            100658.93459827, 106023.70085458002, 107789.0621533, 105896.79562564,
            106633.31462076001, 109273.86423253999, 110422.78371366, 115698.4457478,
            121106.74319760001, 124745.49981770002, 116785.2113091
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subwoodandproductsofwooodandcork(x):
    """
    Historic demand by sector SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            58713.0731415, 56674.16311640001, 62268.855379600005, 58158.672443799995,
            57872.8769209, 58320.3595168, 53786.765614899996, 52005.7191848, 50443.2999891,
            50103.892673500006, 51304.7731625, 51145.0856719, 49934.6358646, 44747.9546273,
            39162.763552
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subpulppaperprintingandpublishing(x):
    """
    Historic demand by sector SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            261905.83585600002, 265565.90128099994, 279257.067462, 279803.98620299995,
            291755.846006, 308452.43508, 304893.28129199997, 298083.50359, 297931.183945,
            308944.077062, 313511.24393999996, 320463.29508, 324466.0879200001, 321599.71193000005,
            295319.46288600005
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subcokerefinedpetroleumandnuclearfuel(x):
    """
    Historic demand by sector SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            224714.8110007, 236897.91061300001, 243059.31450900002, 236224.16534899996,
            247056.70687700002, 256171.347335, 263672.271245, 255718.812736, 253104.36730500002,
            262416.302171, 274366.25466000004, 288918.56216400006, 284291.709385,
            311399.94481899997, 312224.22007199994
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subchemicalsandchemicalproducts(x):
    """
    Historic demand by sector SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            401281.55396000005, 413943.55226, 442258.04358, 446961.19391, 475205.06678,
            488320.40037, 497658.60743000003, 516406.78763999994, 526275.95114, 547978.1543599999,
            543997.78205, 561054.6723000001, 593443.6471099999, 584461.54249, 585171.71828
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subrubberandplastics(x):
    """
    Historic demand by sector SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            113840.58768099999, 113945.8747367, 121180.6431085, 119625.4361529, 124489.8262263,
            128815.7121217, 126288.11073310001, 124339.4912779, 125704.6160781, 134120.678324,
            136741.1405684, 144708.426336, 147552.14972770002, 148650.51235210002, 138539.8729385
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subothernonmetalicmineral(x):
    """
    Historic demand by sector SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            64807.204056099996, 64640.82182840001, 70114.2305891, 66353.5733206, 70736.5639863,
            77060.69853259998, 77541.35509950001, 69471.22983340001, 66587.0727681, 66825.3387091,
            65313.7523083, 66810.5609691, 67500.15473740001, 65682.5872663, 55997.3959377
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subbasicmetalsandfabricatedmetal(x):
    """
    Historic demand by sector SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            259983.9362259, 271267.73239910003, 284850.0447308, 267256.15758, 270201.303311,
            293898.83040549996, 282533.8759788, 267336.84316630004, 274795.9357562, 275358.1269384,
            281170.5760759, 304571.7293547, 295717.7282419, 290339.8805475, 243740.63525680001
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_submachinerynec(x):
    """
    Historic demand by sector SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            777405.4233899999, 801501.20794, 832540.52744, 835703.45149, 819924.72297,
            902042.814358, 879577.6255900001, 849484.3911969999, 920199.3619949999,
            987875.3180239999, 1066212.241217, 1141738.3607909998, 1221521.2313239998,
            1257063.9259199998, 1056073.346949
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subelectricalandopticalequipment(x):
    """
    Historic demand by sector SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            895155.41074, 976698.0374099999, 1094417.91334, 1124876.11626, 1244519.2935199998,
            1454204.57718, 1406121.56144, 1335929.5425900002, 1438594.87697, 1580086.59112,
            1690569.5772499999, 1851443.85137, 1990232.9053299997, 2060699.9134699998,
            1892591.02717
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subtransportequipment(x):
    """
    Historic demand by sector SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1109416.0565499999, 1135802.47235, 1226890.2660299998, 1276699.31241, 1373687.11738,
            1384100.43887, 1397812.84453, 1420318.7065599998, 1501915.9688499998,
            1596650.4719800001, 1691087.47325, 1786616.31244, 1931330.63335, 1876568.46472,
            1617711.04039
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_submanufacturingnecrecycling(x):
    """
    Historic demand by sector SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            323158.75545460003, 335321.0626389, 355650.8314714999, 365768.3166445,
            378992.4530009999, 396661.51933479996, 395347.88420100004, 396660.0778211,
            390690.83852200006, 396934.2083292, 413810.3586759001, 433585.8377551,
            438903.5490424001, 441841.0405447, 408741.12856210006
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subelectricitygasandwatersupply(x):
    """
    Historic demand by sector SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            440415.90923, 458865.99158999993, 459269.43363, 471152.12711999996, 458875.22546000005,
            478247.15768, 488833.95616, 498522.19292, 519711.2229899999, 534402.90054,
            564518.89212, 573584.0885200001, 591804.6821, 635631.9996700002, 608763.13434
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subconstruction(x):
    """
    Historic demand by sector SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3373315.0363289, 3434879.9374897, 3485285.9841774, 3529125.8090693, 3614579.819173,
            3641791.5051483, 3654702.3182296003, 3656187.6076439996, 3760508.5251406003,
            3902283.11202, 4076951.045161, 4285140.697028999, 4449914.955079299, 4452435.567324,
            4336102.287169401
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    Historic demand by sector SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            352813.949813, 364987.526195, 374988.41228399996, 390505.4327470001, 415087.193983,
            426132.658587, 420831.65746899997, 443827.2860779999, 449920.669003, 469539.119757,
            487527.73227100004, 506476.85412399995, 519155.96607500006, 517059.485185,
            470685.328304
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
        x):
    """
    Historic demand by sector SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1263743.815819, 1306432.08967, 1379786.87546, 1434839.818197, 1506643.2353400001,
            1541085.6627800001, 1571980.7765, 1612734.9313400001, 1679155.55503, 1784506.21438,
            1855189.96935, 1904291.2269, 2045268.72965, 2126597.04328, 2061884.04247
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
        x):
    """
    Historic demand by sector SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1598464.786877, 1648586.60758, 1685964.6484180002, 1722990.11125, 1789523.4932309997,
            1850190.8611700002, 1891414.40524, 1923779.9265100001, 1982685.37776, 2043144.18267,
            2131827.02697, 2203047.24762, 2225281.3843300003, 2260433.2593400003, 2157301.38706
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subhotelsandrestaurants(x):
    """
    Historic demand by sector SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1135913.8165600002, 1162855.3384500002, 1172469.28884, 1214919.2694700002,
            1258149.66992, 1302468.91283, 1317318.16966, 1330213.84362, 1355117.6058699999,
            1391521.7102100002, 1442156.8142900004, 1491022.5754500001, 1533115.23302,
            1544786.7201400003, 1489192.91292
        ])
    name = 7
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subinlandtransport(x):
    """
    Historic demand by sector SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            579016.9834, 584911.4735, 600650.3032, 620252.4103999999, 664271.8027000001,
            656405.2264999999, 655483.6779, 668734.9833999999, 686551.086, 709341.2559999999,
            726012.1481, 745568.8351999999, 766743.0387, 795106.8757, 775058.13165
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subwatertransport(x):
    """
    Historic demand by sector SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            49225.027030000005, 52377.94612, 56106.39053, 57596.62124, 63663.95427000001,
            69701.91572, 75626.58343, 74689.24618, 81826.50412000001, 89424.92079999999,
            91514.71831, 97218.65327, 100111.31591, 112992.41015, 105631.90736
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subairtransport(x):
    """
    Historic demand by sector SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            143095.69405000002, 154668.3083, 160504.03436000002, 163488.79459, 167379.92006,
            178105.67977999998, 167901.45130000002, 166335.519676, 171770.97738, 186349.149928,
            189198.01622999998, 198871.62333, 207776.60457999996, 206423.82469, 188968.02946
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    Historic demand by sector SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            165486.95716999998, 165384.27064, 175843.47261000003, 179745.55862000003, 186129.49729,
            180018.40734000003, 185577.71883, 180602.60449000003, 184531.88577, 194745.12383,
            205545.08441, 213653.95099999997, 227410.73386, 240132.33006, 238220.93145
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subpostandtelecommunications(x):
    """
    Historic demand by sector SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            336812.13262999995, 367656.51979, 403243.33547, 441682.99098, 507596.44377,
            579504.20999, 646763.1806800001, 689667.7482999999, 739765.3866699999,
            799182.3432199999, 840815.0947499999, 888244.47724, 941335.00411, 997668.7938999999,
            1005773.2609799999
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subfinancialintermedation(x):
    """
    Historic demand by sector SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1035860.0685400001, 1057752.5364589998, 1137527.325717, 1176056.461432,
            1248309.3130800002, 1371926.4410299999, 1393974.3315, 1399563.49432,
            1445953.6470100002, 1569038.7126999998, 1664819.1774399998, 1796878.4608,
            1952596.1780100001, 1987962.3170099999, 1971211.2656500002
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subrealestateactivities(x):
    """
    Historic demand by sector SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2560435.9584529996, 2654649.5703189997, 2701297.424652, 2787885.699962999,
            2885024.72414, 2953149.4055139995, 3034015.1805149997, 3088175.290212, 3175968.3918589,
            3268139.760941, 3351133.155774, 3420737.17127, 3479278.0990759996, 3513168.39938,
            3544579.12064
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subrentingodmeqandotherbusinessactivities(x):
    """
    Historic demand by sector SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            715097.6881200001, 797501.2814, 835585.9882, 927374.2024399999, 1009120.5656999999,
            1088315.6816, 1127993.1044, 1140984.21059, 1165672.8135, 1214971.802, 1258994.5296,
            1331339.7985, 1419165.8394, 1497408.79, 1435702.263
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subpublicadminanddefencecompulsorysocialsecurity(x):
    """
    Historic demand by sector SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3163684.928, 3243750.0253999997, 3285804.0601, 3371778.8820999996, 3522683.9758,
            3622324.8474, 3719665.8919999995, 3839981.7000999996, 3947516.9477999997, 4051795.136,
            4081013.6649999996, 4137274.7199999993, 4201002.810999999, 4305412.6680000005,
            4451215.852999999
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subeducation(x):
    """
    Historic demand by sector SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1176056.855178, 1187789.1011589998, 1208351.7021030001, 1249417.9983253998,
            1293705.199234, 1273106.580141, 1290728.985381, 1302962.424367, 1328156.208041,
            1350439.9591238, 1354468.59733, 1371169.29075, 1397140.7155360002, 1414788.936597,
            1427681.240051
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subhealthandsocialwork(x):
    """
    Historic demand by sector SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2116985.945408, 2176783.2387836003, 2219207.0180761, 2281499.2523729997,
            2380721.4962919997, 2433973.862812, 2511162.247971, 2600103.371875, 2687871.9925499996,
            2778897.983662, 2904651.5214279997, 2988410.438662, 3089709.4131940003,
            3157580.3929769997, 3215178.3753789994
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subothercommunitysocialandpersonaservices(x):
    """
    Historic demand by sector SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1121312.875, 1129908.562, 1134725.674, 1168176.164, 1225040.244, 1283243.0548,
            1301613.3548, 1340765.8257, 1364030.6045, 1401915.9587000003, 1411694.3254000002,
            1450671.2885999999, 1486387.1959000002, 1509352.3303999999, 1504855.5695999998
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_demand_by_sector_subprivatehouseholdswithemployedpersons(x):
    """
    Historic demand by sector SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            48106.64623337812, 48926.64200190818, 49488.43596377072, 51425.36695231778,
            50869.11033087904, 52155.816324571155, 52146.134999738344, 52317.25933618104,
            53559.402520043535, 54794.767373428214, 55162.71588258705, 56378.119990721425,
            58766.5993487199, 59780.91423515376, 59073.326054017445
        ])
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_demand_flow_fd():
    """
    variation demand flow FD



    component


    """

    ret =  functions.if_then_else(
        time() < 2009,
        historic_variation_demand(),
        (gross_fixed_capital_formation() *
         (1 - ((1 - share_consum_goverments_and_inventories_next_step()) /
               (1 - share_consum_goverment_and_inventories()))) + household_demand() *
         (1 - ((1 - share_consum_goverments_and_inventories_next_step()) /
               (1 - share_consum_goverment_and_inventories()))) + variation_gfcf() +
         variation_household_demand()) / (1 - share_consum_goverments_and_inventories_next_step()))
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def historic_variation_demand():
    """
    historic variation demand



    component


    """

    ret =  xr.DataArray(
        data=[historic_demand_by_sector_subagriculturehuntingforestryandfishing(time() + 1) -
        historic_demand_by_sector_subagriculturehuntingforestryandfishing(time()),

        historic_demand_by_sector_subminingandquarrying(time() + 1) -
        historic_demand_by_sector_subminingandquarrying(time()),

        historic_demand_by_sector_subfoodbeveragesandtobacco(time() + 1) -
        historic_demand_by_sector_subfoodbeveragesandtobacco(time()),

        historic_demand_by_sector_subtextilesandtextileproducts(time() + 1) -
        historic_demand_by_sector_subtextilesandtextileproducts(time()),

        historic_demand_by_sector_subleatherleatherandfootwear(time() + 1) -
        historic_demand_by_sector_subleatherleatherandfootwear(time()),

        historic_demand_by_sector_subwoodandproductsofwooodandcork(time() + 1) -
        historic_demand_by_sector_subwoodandproductsofwooodandcork(time()),

        historic_demand_by_sector_subpulppaperprintingandpublishing(time() + 1) -
        historic_demand_by_sector_subpulppaperprintingandpublishing(time()),

        historic_demand_by_sector_subcokerefinedpetroleumandnuclearfuel(time() + 1) -
        historic_demand_by_sector_subcokerefinedpetroleumandnuclearfuel(time()),

        historic_demand_by_sector_subchemicalsandchemicalproducts(time() + 1) -
        historic_demand_by_sector_subchemicalsandchemicalproducts(time()),

        historic_demand_by_sector_subrubberandplastics(time() + 1) -
        historic_demand_by_sector_subrubberandplastics(time()),

        historic_demand_by_sector_subothernonmetalicmineral(time() + 1) -
        historic_demand_by_sector_subothernonmetalicmineral(time()),

        historic_demand_by_sector_subbasicmetalsandfabricatedmetal(time() + 1) -
        historic_demand_by_sector_subbasicmetalsandfabricatedmetal(time()),

        historic_demand_by_sector_submachinerynec(time() + 1) -
        historic_demand_by_sector_submachinerynec(time()),

        historic_demand_by_sector_subelectricalandopticalequipment(time() + 1) -
        historic_demand_by_sector_subelectricalandopticalequipment(time()),

        historic_demand_by_sector_subtransportequipment(time() + 1) -
        historic_demand_by_sector_subtransportequipment(time()),

        historic_demand_by_sector_submanufacturingnecrecycling(time() + 1) -
        historic_demand_by_sector_submanufacturingnecrecycling(time()),

        historic_demand_by_sector_subelectricitygasandwatersupply(time() + 1) -
        historic_demand_by_sector_subelectricitygasandwatersupply(time()),

        historic_demand_by_sector_subconstruction(time() + 1) -
        historic_demand_by_sector_subconstruction(time()),

        historic_demand_by_sector_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time() + 1) -
        historic_demand_by_sector_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),

        historic_demand_by_sector_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time() + 1) -
        historic_demand_by_sector_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),

        historic_demand_by_sector_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time() + 1) -
        historic_demand_by_sector_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),

        historic_demand_by_sector_subhotelsandrestaurants(time() + 1) -
        historic_demand_by_sector_subhotelsandrestaurants(time()),

        historic_demand_by_sector_subinlandtransport(time() + 1) -
        historic_demand_by_sector_subinlandtransport(time()),

        historic_demand_by_sector_subwatertransport(time() + 1) -
        historic_demand_by_sector_subwatertransport(time()),

        historic_demand_by_sector_subairtransport(time() + 1) -
        historic_demand_by_sector_subairtransport(time()),

        historic_demand_by_sector_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time() + 1) -
        historic_demand_by_sector_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),

        historic_demand_by_sector_subpostandtelecommunications(time() + 1) -
        historic_demand_by_sector_subpostandtelecommunications(time()),

        historic_demand_by_sector_subfinancialintermedation(time() + 1) -
        historic_demand_by_sector_subfinancialintermedation(time()),

        historic_demand_by_sector_subrealestateactivities(time() + 1) -
        historic_demand_by_sector_subrealestateactivities(time()),

        historic_demand_by_sector_subrentingodmeqandotherbusinessactivities(time() + 1) -
        historic_demand_by_sector_subrentingodmeqandotherbusinessactivities(time()),

        historic_demand_by_sector_subpublicadminanddefencecompulsorysocialsecurity(time() + 1) -
        historic_demand_by_sector_subpublicadminanddefencecompulsorysocialsecurity(time()),

        historic_demand_by_sector_subeducation(time() + 1) -
        historic_demand_by_sector_subeducation(time()),

        historic_demand_by_sector_subhealthandsocialwork(time() + 1) -
        historic_demand_by_sector_subhealthandsocialwork(time()),

        historic_demand_by_sector_subothercommunitysocialandpersonaservices(time() + 1) -
        historic_demand_by_sector_subothercommunitysocialandpersonaservices(time()),

        historic_demand_by_sector_subprivatehouseholdswithemployedpersons(time() + 1) -
        historic_demand_by_sector_subprivatehouseholdswithemployedpersons(time())],
        coords={'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['sectors'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
# TRANSP - Households transport
# TRANSP - Households transport
@cache('run')
def initial_energy_intensity_1995_h():
    """
    Initial energy intensity 1995 H



    constant


    """

    ret =  xr.DataArray(
        data=[0.63194525, 0.28551574, 1.568424, 0.80691299, 1.95104725],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def min_energy_intensity_vs_intial_h():
    """
    min energy intensity vs intial H



    constant


    """

    ret =  0.3
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def minimum_fraction_h():
    """
    minimum fraction H



    constant


    """

    ret =  xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def energy_market_pressure_h():
    """
    Energy market pressure H



    component


    """

    ret =  0 * relative_abundance_of_energy_h()
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def pressure_to_change_energy_technology_h():
    """
    Pressure to change energy technology H



    component


    """

    ret =  np.minimum(
        np.maximum(energy_market_pressure_h() + implementatio_policy_to_change_final_energy_h(),
                   0), 1)
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def policy_to_improve_efficiency_speed_h():
    """
    Policy to improve efficiency speed H



    constant


    """

    ret =  xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def max_yearly_change_h():
    """
    Max yearly change H



    constant


    """

    ret =  xr.DataArray(
        data=[0.05, 0.05, 0.05, 0.05, 0.05],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def available_improvement_efficiency_h():
    """
    available improvement efficiency H



    component


    """

    ret =  np.minimum(
        1,
        functions.if_then_else(
            time() > 2009,
            functions.zidz(
                (global_energy_intensity_h() -
                 (min_energy_intensity_vs_intial_h() * initial_global_energy_intensity_2009_h())),
                0.7 * initial_global_energy_intensity_2009_h()), 1))
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def year_policy_change_energy_h():
    """
    Year policy change energy H



    constant


    """

    ret =  xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def efficiency_energy_aceleration_h():
    """
    Efficiency energy aceleration H



    component


    """

    ret =  maximun_yearly_aceleratuin_of_intensity_improvement_h(
    ) * pressure_to_improve_energy_intensity_efficiency_h()
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def efficiency_rate_of_substitution_h():
    """
    efficiency rate of substitution H



    constant


    """

    ret =  xr.DataArray(
        data=[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
              [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]],
        coords={
            'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
            'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
        },
        dims=['final sources', 'final sources1'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def energy_cost_pressure_h():
    """
    Energy cost pressure H



    component


    """

    ret =  1 - abundance_of_energy_h()
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def abundance_of_energy_h():
    """
    abundance of energy H



    component


    """

    ret =  xr.DataArray(
        data=[abundance_liquids(),

        abundance_gases(),

        abundance_solids(),
        1.0,
        1.0],
        coords={'final sources':['liquids','gases','solids','electricity','heat']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def global_energy_intensity_h():
    """
    Global energy intensity H



    component


    """

    ret =  functions.sum(evol_final_energy_intensity_h())
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('step')
def historic_rate_final_energy_intensity_h():
    """
    historic rate final energy intensity H



    component


    """

    ret =  xr.DataArray(
        data=[historic_final_energy_intensity_h_subelectricity(time() + 1) -
        historic_final_energy_intensity_h_subelectricity(time()),

        historic_final_energy_intensity_h_subheat(time() + 1) -
        historic_final_energy_intensity_h_subheat(time()),

        historic_final_energy_intensity_h_subliquids(time() + 1) -
        historic_final_energy_intensity_h_subliquids(time()),

        historic_final_energy_intensity_h_subgases(time() + 1) -
        historic_final_energy_intensity_h_subgases(time()),

        historic_final_energy_intensity_h_subsolids(time() + 1) -
        historic_final_energy_intensity_h_subsolids(time())],
        coords={'final sources':['electricity','heat','liquids','gases','solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def maximun_yearly_aceleration_of_intensity_improvement_pct_h():
    """
    Maximun yearly aceleration of intensity improvement pct H



    constant


    """

    ret =  xr.DataArray(
        data=[0.01, 0.01, 0.01, 0.01, 0.01],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def historical_mean_rate_energy_intensity_h():
    """
    historical mean rate energy intensity H



    constant


    """

    ret =  xr.DataArray(
        data=[0.00481188, -0.04466554, 0.00105756, -0.0108818, -0.02195653],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def relative_abundance_of_energy_h():
    """
    relative abundance of energy H



    constant


    """

    ret =  0
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def policy_change_energy_speed_h():
    """
    Policy change energy speed H



    constant


    """

    ret =  xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_h_subelectricity(x):
    """
    Historic final energy intensity H SUBelectricity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.6319452492878225, 0.6474447373221129, 0.6391440903441314, 0.6425429079164128,
            0.6367531217492081, 0.6381846304893362, 0.6397246612461346, 0.6454008888282491,
            0.6483867554557563, 0.6454292743121216, 0.6541601234210128, 0.6498359966439917,
            0.6468084213534645, 0.6527043158711583, 0.6755273041086106
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_h_subheat(x):
    """
    Historic final energy intensity H SUBheat



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.2855157381546875, 0.25404242100647506, 0.24406615678652396, 0.23610742913107505,
            0.22273851188620555, 0.2209255171420526, 0.2280921535653369, 0.21266637246609502,
            0.20891825736332598, 0.20146970103522027, 0.17173640930783207, 0.17010396189688415,
            0.16529326408979034, 0.16134248582086183, 0.16062564751664934
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_h_subliquids(x):
    """
    Historic final energy intensity H SUBliquids



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.568424004123364, 1.5730823826140428, 1.5751815947459, 1.5215015338090632,
            1.5231645132539757, 1.5686384037861798, 1.5744559913368583, 1.5255667193507039,
            1.5531450950363177, 1.600019681580165, 1.645749849370925, 1.6358124056952272,
            1.6167993069329223, 1.5976487464510023, 1.5959097248509646
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_h_subsolids(x):
    """
    Historic final energy intensity H SUBsolids



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.95104724970324, 1.91801080181749, 1.8449354246520582, 1.7516917434535115,
            1.6942081331809153, 1.6505256393456411, 1.6289997302953656, 1.5913544671283866,
            1.5780902111352826, 1.540341358331322, 1.5113938358165822, 1.4752141383258566,
            1.4422144923500455, 1.4416220292300719, 1.4683310666706337
        ])
    name = 8
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_h_subgases(x):
    """
    Historic final energy intensity H SUBgases



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8069129864373691, 0.8648994945858922, 0.8062956712422991, 0.7601404056051215,
            0.7625889623573991, 0.7699018890737395, 0.7585472995515781, 0.7588815049134393,
            0.778744234679196, 0.7536943195133183, 0.7279680365530793, 0.697784790932038,
            0.6896973704625412, 0.6939916695225792, 0.6975686054927074
        ])
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def initial_global_energy_intensity_2009_h():
    """
    Initial global energy intensity 2009 H



    constant


    """

    ret =  4.597962348639566
    name = 8
    return round_sig(x=ret, name=name)

# FE INTENSITY households
@cache('run')
def year_policy_to_improve_efficiency_h():
    """
    Year policy to improve efficiency H



    constant


    """

    ret =  xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    name = 8
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def share_conv_vs_total_oil_extraction():
    """
    share conv vs total oil extraction



    component


    """

    ret =  functions.zidz(real_extraction_conv_oil_ej(),
                          (real_extraction_conv_oil_ej() + real_extraction_unconv_oil_ej()))
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_natx_gas_without_gtlx():
    """
    XPES natX gas without GTLX



    component


    """

    ret =  xpes_natx_gasx() - xped_natx_gas_for_gtl_ejx()
    name = 8
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('step')
def tot_feh_nre():
    """
    Tot FEH NRE



    component


    """

    ret =  feh_coal() + feh_gas() + feh_oil()
    name = 8
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def co2_emissions_conv_gas_without_gtl():
    """
    CO2 emissions conv gas without GTL



    component


    """

    ret =  functions.if_then_else(
        abs(xseparate_conv_and_unconv_gasxx() - 1) < equality_threshold,
        real_extraction_conv_gas_emissions_relevant_ej() * gco2_per_mj_conv_gas(),
        (1 - adapt_co2_emissions_unconv_gas()
         ) * real_extraction_conv_gas_emissions_relevant_ej() * gco2_per_mj_conv_gas() +
        adapt_co2_emissions_unconv_gas() * real_extraction_conv_gas_emissions_relevant_ej(
        ) * gco2_per_mj_unconv_gas()) * mj_per_ej() / g_per_gt()
    name = 8
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def co2_emissions_conv_oil():
    """
    CO2 emissions conv oil



    component


    """

    ret =  real_extraction_conv_oil_emissions_relevant_ej() * gco2_per_mj_conv_oil() * mj_per_ej(
    ) / g_per_gt()
    name = 8
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def co2_emissions_gtl():
    """
    CO2 emissions GTL



    component


    """

    ret =  xped_natx_gas_for_gtl_ejx() * gco2_per_mj_gtl() * mj_per_ej() / g_per_gt()
    name = 8
    return round_sig(x=ret, name=name)

# ABUND - Abundances by sector
@cache('step')
def pe_gas_extraction_for_transp_ej():
    """
    PE gas extraction for Transp EJ



    component


    """

    ret =  xpes_natx_gas_without_gtlx() * share_gas_for_transp()
    name = 8
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def ch4_emissions_conv_gas_without_gtl():
    """
    CH4 emissions conv gas without GTL



    component


    """

    ret =  functions.if_then_else(
        abs(xseparate_conv_and_unconv_gasxx() - 1) < equality_threshold,
        real_extraction_conv_gas_emissions_relevant_ej() * gch4_per_mj_conv_gas(),
        (1 - adapt_co2_emissions_unconv_gas()
         ) * real_extraction_conv_gas_emissions_relevant_ej() * gch4_per_mj_conv_gas() +
        adapt_co2_emissions_unconv_gas() * real_extraction_conv_gas_emissions_relevant_ej(
        ) * gch4_per_mj_unconv_gas()) * mj_per_ej() / g_per_mt()
    name = 8
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def co2_emissions_unconv_oil():
    """
    CO2 emissions unconv oil



    component


    """

    ret =  (real_extraction_unconv_oil_emissions_relevant_ej() *
            (gco2_per_mj_unconv_oil() +
             (gco2_per_mj_shale_oil() - gco2_per_mj_unconv_oil()) * adapt_emissions_shale_oil())
            ) * mj_per_ej() / g_per_gt()
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def xshortage_coef_without_min_without_exlossesx():
    """
    XShortage coef without MIN without EXlossesX



    component


    """

    ret =  real_fe_consumption_by_fuel_before_heat_correction(
    ) / required_fed_by_fuel_before_heat_correction()
    name = 8
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def share_conv_vs_total_gas_extraction():
    """
    share conv vs total gas extraction



    component


    """

    ret =  functions.zidz(real_extraction_conv_gas_ej(),
                          (real_extraction_conv_gas_ej() + real_extraction_unconv_gas_ej()))
    name = 8
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('run')
def share_feh_over_pes_coal_solids():
    """
    share FEH over PES coal solids



    constant


    """

    ret =  0.2
    name = 8
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def ch4_emissions_unconv_gas():
    """
    CH4 emissions unconv gas



    component


    """

    ret =  real_extraction_unconv_gas_emissions_relevant_ej() * gch4_per_mj_unconv_gas(
    ) * mj_per_ej() / g_per_mt()
    name = 8
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('step')
def feh_coal_delayed():
    """
    FEH coal delayed



    component


    """

    ret =  delay_feh_coal_roundtime_step__time_step___time_step_187_time_step__time_step()
    name = 8
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('step')
def feh_coal():
    """
    FEH coal



    component


    """

    ret =  (extraction_coal_ej() - ped_coal_for_ctl_ej()) * share_feh_over_pes_coal_solids()
    name = 8
    return round_sig(x=ret, name=name)

# HEAT - Heat demand
@cache('step')
def fe_heat_demand_consum_after_cc_impacts_ej():
    """
    FE heat demand consum after CC impacts EJ



    component


    """

    ret =  required_fed_by_fuel().loc[{'final sources':'heat'}].values + xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'heat'}].values
    name = 8
    return round_sig(x=ret, name=name)

# TFES by final fuel
@cache('step')
def share_electricity_vs_tfes():
    """
    share electricity vs TFES



    component


    """

    ret =  real_fe_consumption_by_fuel().loc[{'final sources':'electricity'}].values / functions.sum(real_fe_consumption_by_fuel())
    name = 8
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def co2_emissions_peat():
    """
    CO2 emissions peat



    component


    """

    ret =  pes_peat_ej() * gco2_per_mj_shale_oil() * mj_per_ej() / g_per_gt()
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_exlosses_due_to_cc_impactsx():
    """
    XTotal EXlosses due to CC impactsX



    component


    """

    ret =  functions.sum(xexlosses_by_final_fuel_due_to_cc_impactsx())
    name = 8
    return round_sig(x=ret, name=name)

# GASES - PED, PES & FES
@cache('step')
def required_fed_by_gas():
    """
    Required FED by gas



    component


    """

    ret =  required_fed_by_fuel().loc[{'final sources':'gases'}].values
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def xexlosses_by_final_fuel_due_to_cc_impactsx():
    """
    XEXlosses by final fuel due to CC impactsX



    component


    """

    ret =  required_fed_by_fuel() * xshare_exlosses_ccx() * method_cc_impacts()
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def xexlosses_heatxcom_due_to_cc_impactsx():
    """
    XEXlosses heatXcom due to CC impactsX



    component


    """

    ret =  required_fed_by_fuel_before_heat_correction().loc[{'final sources':'heat'}].values * xshare_exlosses_ccx(
    ) * method_cc_impacts()
    name = 8
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def required_tfed():
    """
    Required TFED



    component


    """

    ret =  functions.sum(required_fed_by_fuel())
    name = 8
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# ELEC - Total FE electricity generation
@cache('step')
def total_fe_elec_consumption_ej():
    """
    Total FE Elec consumption EJ



    component


    """

    ret =  total_fe_elec_consumption_twh() * ej_per_twh()
    name = 8
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# ELEC - Total FE electricity generation
# INDICATORS - Energy
# COMP - Historical data
@cache('step')
def total_fe_elec_consumption_twh():
    """
    Total FE Elec consumption TWh



    component


    """

    ret =  total_fe_elec_generation_twh() / ((1 + xshare_transmxdistr_elec_lossesx()) *
                                             (1 + xshare_exlosses_ccx() * method_cc_impacts()))
    name = 8
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
# GASES - PED, PES & FES
# GASES - PED, PES & FES
# GASES - PED, PES & FES
@cache('step')
def ped_gases():
    """
    PED gases



    component


    """

    ret =  np.maximum(
        0,
        required_fed_by_gas() + xped_natx_gas_for_gtl_ejx() + pe_demand_gas_elec_plants_ej() +
        ped_gases_for_heat_plants_ej() + ped_gas_for_chp_plants_ej() + xped_gas_heatxncx() +
        other_gases_required() + xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'gases'}].values)
    name = 8
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# HEAT - Total FE heat generation
@cache('step')
def total_fe_heat_consumption_ej():
    """
    Total FE Heat consumption EJ



    component


    """

    ret =  total_fe_heat_generation_ej() / ((1 + share_heat_distribution_losses()) *
                                            (1 + xshare_exlosses_ccx() * method_cc_impacts()))
    name = 8
    return round_sig(x=ret, name=name)

# TFES by final fuel
@cache('step')
def share_solids_vs_tfes():
    """
    share solids vs TFES



    component


    """

    ret =  real_fe_consumption_by_fuel().loc[{'final sources':'solids'}].values / functions.sum(real_fe_consumption_by_fuel())
    name = 8
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
# IMPACTS - Carbon cycle & climate
# INDICATORS - Social and Environmental impacts
# COMP - Historical data
@cache('step')
def total_co2_emissions_gtco2():
    """
    Total CO2 emissions GTCO2



    component


    """

    ret =  co2_fossil_fuel_emissions() + xco2_landxuse_change_emissionsx(
        time()) + bioe_co2_emissions() + co2_emissions_peat() - afforestation_program_2020_gtco2()
    name = 8
    return round_sig(x=ret, name=name)

# TFES by final fuel
@cache('step')
def share_gases_vs_tfes():
    """
    share gases vs TFES



    component


    """

    ret =  real_fe_consumption_by_fuel().loc[{'final sources':'gases'}].values / functions.sum(real_fe_consumption_by_fuel())
    name = 8
    return round_sig(x=ret, name=name)

# SOLIDS - PED, PES & FES
@cache('step')
def required_fed_solids():
    """
    Required FED solids



    component


    """

    ret =  required_fed_by_fuel().loc[{'final sources':'solids'}].values
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_heatxnc_ejx():
    """
    XFED HeatXnc EJX



    component


    """

    ret =  fe_heat_demand_consum_after_cc_impacts_ej() - xrequired_heatxcom_after_cc_impactsx()
    name = 8
    return round_sig(x=ret, name=name)

# TFES by final fuel
@cache('step')
def share_heat_vs_tfes():
    """
    share heat vs TFES



    component


    """

    ret =  real_fe_consumption_by_fuel().loc[{'final sources':'heat'}].values / functions.sum(real_fe_consumption_by_fuel())
    name = 8
    return round_sig(x=ret, name=name)

# SOLIDS - PED, PES & FES
# SOLIDS - PED, PES & FES
@cache('step')
def ped_solids():
    """
    PED solids



    component


    """

    ret =  np.maximum(
        0,
        required_fed_solids() + ped_coal_for_ctl_ej() + pe_demand_coal_elec_plants_ej() +
        ped_coal_for_heat_plants_ej() + ped_coal_for_chp_plants_ej() + xped_coal_heatxncx() +
        other_solids_required() + xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'solids'}].values)
    name = 8
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
# RESOURCES - CTL & GTL supply
# RESOURCES - CTL & GTL supply
@cache('step')
def scarcity_conv_oil():
    """
    scarcity conv oil



    component


    """

    ret =  functions.if_then_else(
        abs(max_extraction_conv_oil_ej() ) < equality_threshold, 0,
        functions.if_then_else(single_precision(max_extraction_conv_oil_ej()) > single_precision(extraction_conv_oil_ej()) - less_thr_func(max_extraction_conv_oil_ej(), extraction_conv_oil_ej()), 1 -
                               functions.zidz(max_extraction_conv_oil_ej() - extraction_conv_oil_ej(),
                                max_extraction_conv_oil_ej())**exponent_availability_conv_oil(),
                               0))
    name = 8
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# ELEC - Total FE electricity generation
@cache('step')
def test2():
    """
    test2



    component


    """
    ret =  test() * ej_per_twh() / (1 + (xshare_exlosses_ccx() * method_cc_impacts()))
    name = 8
    return round_sig(x=ret, name=name)

# TFES by final fuel
@cache('step')
def share_liquids_vs_tfes():
    """
    share liquids vs TFES



    component


    """

    ret =  real_fe_consumption_by_fuel().loc[{'final sources':'liquids'}].values / functions.sum(real_fe_consumption_by_fuel())
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def gfcf_not_covered():
    """
    GFCF not covered



    component


    """

    ret =  functions.if_then_else(time() < 2009, 0, gross_fixed_capital_formation() - real_gfcf())
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def lc_not_covered():
    """
    LC not covered



    component


    """

    ret =  demand_not_covered_total_fd() * labour_share()
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def cc_total_not_covered():
    """
    CC total not covered



    component


    """

    ret =  demand_not_covered_total_fd() * capital_share()
    name = 8
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def household_demand_not_covered():
    """
    Household demand not covered



    component


    """

    ret =  functions.if_then_else(time() < 2009, 0, household_demand() - real_household_demand())
    name = 8
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def real_total_output_inland_transport():
    """
    Real total output inland transport



    component


    """

    ret =  real_total_output_by_sector().loc[{'sectors':'inland transport'}].values / 1e+006
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def tfes_intensity_ej_tdolar():
    """
    TFES intensity EJ Tdolar



    component


    """

    ret =  functions.zidz(real_tfec(), gdp())
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def tfes_intensity_ej_tdolar_delayed_1yr():
    """
    TFES intensity EJ Tdolar delayed 1yr



    component


    """

    ret =  delay_tfes_intensity_ej_tdolar_round1__time_step___time_step_8827_1__time_step()
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def tpes_intensity_ej_tdolar_delayed_1yr():
    """
    TPES intensity EJ Tdolar delayed 1yr



    component


    """

    ret =  delay_tpes_intensity_ej_tdolar_round1__time_step___time_step_1314_1__time_step()
    name = 8
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def annual_tpes_intensity_growth_rate():
    """
    Annual TPES intensity growth rate



    component


    """

    ret =  -1 + tpes_intensity_ej_tdolar() / tpes_intensity_ej_tdolar_delayed_1yr()
    name = 8
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
@cache('step')
def annual_tfes_intensity_growth_rate():
    """
    Annual TFES intensity growth rate



    component


    """

    ret =  -1 + tfes_intensity_ej_tdolar() / tfes_intensity_ej_tdolar_delayed_1yr()
    name = 8
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('step')
def total_d_jobs_res_elec_per_techn():
    """
    Total D jobs RES elec per techn



    component


    """

    ret =  d_jobs_new_installed_res_elec_per_techn() + xjobs_oxm_res_elec_per_technx()
    name = 8
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('step')
def total_d_jobs_res_heat_per_techn():
    """
    Total D jobs RES heat per techn



    component


    """

    ret =  d_jobs_new_installed_res_heat_per_techn() + xjobs_oxm_res_heat_per_technx()
    name = 8
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('run')
def employment_factors_new_res_elec():
    """
    Employment factors new RES elec



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[10.9], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[16.6], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[16.9], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[20.4], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[7.9], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[23.6], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[19.7], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[9.3], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 8
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('run')
def employment_factors_new_res_heat():
    """
    Employment factors new RES heat



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[8.4], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[6.9], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[16.9], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 8
    return round_sig(x=ret, name=name)

@cache('run')
def xemployment_factors_oxm_res_elecx():
    """
    XEmployment factors OXM RES elecX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.2], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.4], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[1.5], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.6], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.3], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.2], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.7], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.6], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 8
    return round_sig(x=ret, name=name)

@cache('run')
def xemployment_factors_oxm_res_heatx():
    """
    XEmployment factors OXM RES heatX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[1.5], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 8
    return round_sig(x=ret, name=name)

# RES - Employment
@cache('step')
def d_jobs_new_installed_res_elec_per_techn():
    """
    D jobs new installed RES elec per techn



    component


    """

    ret =  res_elec_capacity_under_construction_tw() * employment_factors_new_res_elec() * m_per_t(
    )
    name = 8
    return round_sig(x=ret, name=name)

@cache('step')
def xjobs_oxm_res_elec_per_technx():
    """
    XJobs OXM RES elec per technX



    component


    """

    ret =  installed_capacity_res_elec_tw() * xemployment_factors_oxm_res_elecx() * m_per_t()
    name = 8
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('step')
def nx0_vehicles_per_xinland_t():
    """
    NX0 vehicles per Xinland T



    component


    """

    ret =  xr.DataArray(
        data=[nx_hv_inland_t(),

        nx_hv_inland_t(),

        nx_hv_inland_t(),

        nx_lv_inland_t(),

        nx_lv_inland_t(),

        nx_lv_inland_t(),

        nx_lv_inland_t(),

        nx_bus_inlandt(),

        nx_bus_inlandt(),

        nx_bus_inlandt(),

        nx_train_inland_t(),

        nx_train_inland_t(),

        nx_bus_inlandt()],
        coords={'vehiclet':['hv liq','hv hib','hv gas','lv liq','lv elec','lv hib','lv gas','bus liq','bus hib','bus gas','train liq','train elec','bus elec']},
        dims=['vehiclet'])
    name = 8
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def initial_percent_t_vehicles():
    """
    initial percent T vehicles



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.99909685], coords={'vehiclet': ['hv liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.00045157], coords={'vehiclet': ['hv hib']}, dims=['vehiclet']),
        xr.DataArray(data=[0.00045157], coords={'vehiclet': ['hv gas']}, dims=['vehiclet']),
        xr.DataArray(data=[0.98292579], coords={'vehiclet': ['lv liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.00074635], coords={'vehiclet': ['lv elec']}, dims=['vehiclet']),
        xr.DataArray(data=[0.0003609], coords={'vehiclet': ['lv hib']}, dims=['vehiclet']),
        xr.DataArray(data=[0.01596696], coords={'vehiclet': ['lv gas']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['bus liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['bus hib']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['bus gas']}, dims=['vehiclet']),
        xr.DataArray(data=[0.8], coords={'vehiclet': ['train liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.2], coords={'vehiclet': ['train elec']}, dims=['vehiclet']),
        xr.DataArray(data=[0.], coords={'vehiclet': ['bus elec']}, dims=['vehiclet']),
    ])
    name = 8
    return round_sig(x=ret, name=name)

# TRANSP - Households transport
# TRANSP - Inland transport sector
@cache('run')
def saving_ratios_v():
    """
    saving ratios V



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[1.], coords={'vehiclet': ['hv liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.66], coords={'vehiclet': ['hv hib']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['hv gas']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['lv liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.33], coords={'vehiclet': ['lv elec']}, dims=['vehiclet']),
        xr.DataArray(data=[0.66], coords={'vehiclet': ['lv hib']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['lv gas']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['bus liq']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['bus hib']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['bus gas']}, dims=['vehiclet']),
        xr.DataArray(data=[1.], coords={'vehiclet': ['train liq']}, dims=['vehiclet']),
        xr.DataArray(data=[0.6], coords={'vehiclet': ['train elec']}, dims=['vehiclet']),
        xr.DataArray(data=[0.5], coords={'vehiclet': ['bus elec']}, dims=['vehiclet']),
    ])
    name = 8
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('step')
def feh_gas_delayed():
    """
    FEH gas delayed



    component


    """

    ret =  delay_feh_gas_roundtime_step__time_step___time_step_3409_time_step__time_step()
    name = 8
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('run')
def share_feh_over_pes_gases():
    """
    share FEH over PES gases



    constant


    """

    ret =  0.45
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_gases_dem_for_heatxncx():
    """
    Xshare gases dem for HeatXncX



    component


    """

    ret =  functions.zidz(xped_gas_heatxncx(), (pes_gases() - xped_natx_gas_for_gtl_ejx()))
    name = 9
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('step')
def feh_gas():
    """
    FEH gas



    component


    """

    ret =  (pes_gases() - xped_natx_gas_for_gtl_ejx()) * share_feh_over_pes_gases()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def increase_scarcity_conv_gas():
    """
    increase scarcity conv gas



    component


    """

    ret =  scarcity_conv_gas_stock() * functions.zidz(
        (scarcity_conv_gas() - scarcity_conv_gas_delayed_1yr()), scarcity_conv_gas())
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def scarcity_conv_gas_stock():
    """
    scarcity conv gas stock



    component


    """


    ret =  integ_scarcity_conv_gas_stock()
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_natx_gas_for_heatxnc_plantsx():
    """
    XPES natX gas for HeatXnc plantsX



    component


    """

    ret =  (pes_gases() - xped_natx_gas_for_gtl_ejx()) * xshare_gases_dem_for_heatxncx()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def real_growth_ctl():
    """
    real growth CTL



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_coal()) > single_precision(abundance_liquids()) - less_thr_func(abundance_coal(), abundance_liquids()),
                                  functions.if_then_else(abs(abundance_coal() - 1) < equality_threshold,
                                                         exogenous_growth_ctl(), 0),
                                  0) * abundance_liquids_ctl() * scarcity_conv_oil()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def ctl_potential_production():
    """
    CTL potential production



    component


    """


    ret =  integ_ctl_potential_production()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def abundance_liquids_gtl():
    """
    abundance liquids GTL



    component


    """

    ret =  np.sqrt(abs((ped_liquids_ej() - gtl_potential_production()) / ped_liquids_ej()))
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def gtl_potential_production():
    """
    GTL potential production



    component


    """


    ret =  integ_gtl_potential_production()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def abundance_liquids_ctl():
    """
    abundance liquids CTL



    component


    """

    ret =  np.sqrt(abs((ped_liquids_ej() - ctl_potential_production()) / ped_liquids_ej()))
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def wear_gtl():
    """
    wear GTL



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, gtl_potential_production() / lifetime_gtl())
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def lifetime_gtl():
    """
    lifetime GTL



    constant


    """

    ret =  40
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def lifetime_ctl():
    """
    lifetime CTL



    constant


    """

    ret =  40
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def wear_ctl():
    """
    wear CTL



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, ctl_potential_production() / lifetime_ctl())
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def abundance_unconv_oil_stock():
    """
    abundance unconv oil stock



    component


    """


    ret =  integ_abundance_unconv_oil_stock()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def increase_abundance_unconv_oil():
    """
    increase abundance unconv oil



    component


    """

    ret =  abundance_unconv_oil() - abundance_unconv_oil_delayed_1yr()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def abundance_unconv_oil():
    """
    abundance unconv oil



    component


    """

    ret =  functions.if_then_else(
        abs(ped_total_oil_ej() ) < equality_threshold, 0,
        functions.if_then_else(single_precision(ped_total_oil_ej()) > single_precision(real_extraction_unconv_oil_ej()) + less_thr_func(ped_total_oil_ej(), real_extraction_unconv_oil_ej()),
            (ped_total_oil_ej() - real_extraction_unconv_oil_ej()) / ped_total_oil_ej(), 0))
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def abundance_unconv_oil_delayed_1yr():
    """
    abundance unconv oil delayed 1yr



    component


    """

    ret =  delay_abundance_unconv_oil_round1__time_step___time_step_1_1__time_step()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def abundance_unconv_oil2():
    """
    abundance unconv oil2



    component


    """

    ret =  abundance_unconv_oil_stock()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# TPES
@cache('step')
def real_extraction_unconv_oil_ej():
    """
    real extraction unconv oil EJ



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_oilxx() - 1) < equality_threshold,
                                  extraction_unconv_oil_ej(), xextraction_unconv_oil_x_tot_aggx())
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def scarcity_conv_oil_stock():
    """
    scarcity conv oil stock



    component


    """


    ret =  integ_scarcity_conv_oil_stock()
    name = 9
    return round_sig(x=ret, name=name)

# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def share_variable_res_elec_generation_vs_total_gen():
    """
    Share variable RES elec generation vs total gen



    component


    """


    ret =  integ_share_variable_res_elec_generation_vs_total_gen()
    name = 9
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def extra_monet_invest_to_cope_with_variable_elec_res():
    """
    extra monet invest to cope with variable Elec RES



    component


    """

    ret =  (fe_elec_generation_from_onshore_wind_twh() + fe_elec_generation_from_offshore_wind_twh(
    )) * balancing_costs() + grid_reinforcement_costs_tdollar()
    name = 9
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def increase_scarcity_conv_oil():
    """
    increase scarcity conv oil



    component


    """

    ret =  scarcity_conv_oil() - scarcity_conv_oil_delayed_1yr()
    name = 9
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def share_extra_monet_invest_to_cope_with_variable_elec_res():
    """
    share extra monet invest to cope with variable Elec RES



    component


    """

    ret =  extra_monet_invest_to_cope_with_variable_elec_res(
    ) / total_monet_invest_res_for_elec_tdolar()
    name = 9
    return round_sig(x=ret, name=name)

# ELEC RES: Overcapacities due to RES variability a
@cache('run')
def initial_share_variable_res_elec_gen_vs_total():
    """
    initial share variable RES elec gen vs total



    constant


    """

    ret =  0.0071
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_oil_for_heatxnc_plantsx():
    """
    XPES oil for HeatXnc plantsX



    component


    """

    ret =  pes_liquids_ej() * xshare_liquids_dem_for_heatxncx()
    name = 9
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def cumulated_invest_e_grid():
    """
    cumulated invest E grid



    component


    """


    ret =  integ_cumulated_invest_e_grid()
    name = 9
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def total_monet_invest_res_for_elec_tdolar():
    """
    Total monet invest RES for elec Tdolar



    component


    """

    ret =  invest_res_for_elec() + extra_monet_invest_to_cope_with_variable_elec_res()
    name = 9
    return round_sig(x=ret, name=name)

# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def cp_exogenous_res_elec_var_reduction():
    """
    Cp exogenous RES elec var reduction



    component


    """

    ret =  1 / (1 + 0.0001 * np.exp(9.85 * share_variable_res_elec_generation_vs_total_gen()))
    name = 9
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def cp_exogenous_res_elec_dispatch_reduction():
    """
    Cp exogenous RES elec dispatch reduction



    component


    """

    ret =  np.minimum(1,
                      -0.6209 * (share_variable_res_elec_generation_vs_total_gen())**2 - 0.3998 *
                      (share_variable_res_elec_generation_vs_total_gen()) + 1.0222)
    name = 9
    return round_sig(x=ret, name=name)

# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def increase_variable_res_share_elec_vs_total_generation():
    """
    increase variable RES share elec vs total generation



    component


    """

    ret =  share_variable_res_elec_generation_vs_total(
    ) - share_variable_res_elec_vs_total_generation_delayed_1yr()
    name = 9
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def available_improvement_efficiency():
    """
    available improvement efficiency



    component


    """

    ret =  np.minimum(1,
                      functions.if_then_else(
                          time() > 2009,
                          functions.zidz((global_energy_intensity_by_sector() -
                                          (min_energy_intensity_vs_intial() *
                                           initial_global_energy_intensity_by_sector_2009())),
                                         0.7 * initial_global_energy_intensity_by_sector_2009()),
                          1))
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_nre_heatxncx():
    """
    XPES NRE HeatXncX



    component


    """

    ret =  xpes_coal_for_heatxnc_plantsx() + xpes_natx_gas_for_heatxnc_plantsx(
    ) + xpes_oil_for_heatxnc_plantsx()
    name = 9
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('run')
def share_feh_over_pes_liquids():
    """
    share FEH over PES liquids



    constant


    """

    ret =  0.2
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_natx_gas_for_heatxncx():
    """
    XFED natX gas for heatXncX



    component


    """

    ret =  required_fed_by_fuel_before_heat_correction().loc[{'final sources':'gases'}].values * share_feh_over_fed_by_final_fuel().loc[{'final sources':'gases'}].values * efficiency_gases_for_heat_plants(time()) / (1 + share_heat_distribution_losses())
    name = 9
    return round_sig(x=ret, name=name)

# HEAT - Total PES
@cache('step')
def tpes_heat():
    """
    TPES heat



    component


    """

    ret =  pes_nre_heat() + pes_heat_res() + xpes_tot_waste_for_heatxcomx()
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_oil_for_heatxncx():
    """
    XFED oil for heatXncX



    component


    """

    ret =  required_fed_by_fuel_before_heat_correction().loc[{'final sources':'liquids'}].values * share_feh_over_fed_by_final_fuel().loc[{'final sources':'liquids'}].values * efficiency_liquids_for_heat_plants(time()) / (1 + share_heat_distribution_losses())
    name = 9
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('step')
def feh_oil():
    """
    FEH oil



    component


    """

    ret =  pes_liquids_ej() * share_feh_over_pes_liquids()
    name = 9
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('step')
def feh_oil_delayed():
    """
    FEH oil delayed



    component


    """

    ret =  delay_feh_oil_roundtime_step__time_step___time_step_3409_time_step__time_step()
    name = 9
    return round_sig(x=ret, name=name)

# HEAT - Total PES
@cache('step')
def pes_nre_heat():
    """
    PES NRE heat



    component


    """

    ret =  xpes_nre_heatxcomx() + xpes_nre_heatxncx()
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_liquids_dem_for_heatxncx():
    """
    Xshare liquids dem for HeatXncX



    component


    """

    ret =  functions.zidz(xped_liquids_heatxncx(), pes_liquids_ej())
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_nre_heatxcomx():
    """
    XPES NRE HeatXcomX



    component


    """

    ret =  xpes_coal_for_heatxcom_plantsx() + xpes_natx_gas_for_heatxcom_plantsx(
    ) + xpes_oil_for_heatxcom_plantsx()
    name = 9
    return round_sig(x=ret, name=name)

# TPED by fuel
@cache('step')
def check_tpe():
    """
    check TPE



    component


    """

    ret =  (tped_by_fuel() - tpes_ej()) * 100 / tpes_ej()
    name = 9
    return round_sig(x=ret, name=name)

@cache('run')
def xfed_by_fuel_for_heatxncx():
    """
    XFED by fuel for heatXncX



    constant


    """

    ret =  xr.DataArray(
        data=[0.0,
        0.0,

        xfed_oil_for_heatxncx(),

        xfed_natx_gas_for_heatxncx(),

        xfed_coal_for_heatxncx() + xfed_solid_bioe_for_heatxncx()],
        coords={'final sources':['electricity','heat','liquids','gases','solids']},
        dims=['final sources'])
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_fed_heatxcom_vs_total_heatx():
    """
    XShare FED heatXcom vs total heatX



    component


    """

    ret =  xtotal_fed_heatxcom_ejx() / (total_fed_heat_ej() + xtotal_fed_heatxcom_ejx())
    name = 9
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def check_hist_elec():
    """
    check hist Elec



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (total_fe_elec_consumption_twh() - historic_electricity_consumption(time()))
        * 100 / historic_electricity_consumption(time()), 0)
    name = 9
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def average_elec_consumption_per_capita():
    """
    Average elec consumption per capita



    component


    """

    ret =  total_fe_elec_consumption_twh() * kwh_per_twh() / population()
    name = 9
    return round_sig(x=ret, name=name)

# SOLIDS - PED, PES & FES
@cache('step')
def pes_solids():
    """
    PES solids



    component


    """

    ret =  extraction_coal_ej() + pe_traditional_biomass_ej_delayed_1yr() + pes_peat_ej(
    ) + pes_waste_for_tfc() + losses_in_charcoal_plants_ej(time())
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_solid_bioe_for_heatxncx():
    """
    XFED solid bioE for heatXncX



    component


    """

    ret =  required_fed_by_fuel_before_heat_correction().loc[{'final sources':'solids'}].values * share_feh_over_fed_solid_bioe(
    ) * efficiency_conversion_bioe_plants_to_heat() / (1 + share_heat_distribution_losses())
    name = 9
    return round_sig(x=ret, name=name)

# HEAT - Heat demand
# HEAT - Total FE heat generation
# COMP- Historical data Heat generation
@cache('step')
def total_fed_heat_ej():
    """
    Total FED Heat EJ



    component


    """

    ret =  xtotal_fed_heatxcom_ejx() + xtotal_fed_heatxnc_ejx()
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xheatxnc_distribution_lossesx():
    """
    XHeatXnc distribution lossesX



    component


    """

    ret =  xtotal_fed_heatxnc_ejx() - xfed_heatxnc_ejx()
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_fed_heatxnc_ejx():
    """
    XTotal FED HeatXnc EJX



    component


    """

    ret =  xfed_heatxnc_ejx() * (1 + share_heat_distribution_losses())
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_fed_nre_heatxncx():
    """
    XTotal FED NRE HeatXncX



    component


    """

    ret =  np.maximum(0, (xtotal_fed_heatxnc_ejx() - xtotal_fe_real_supply_res_for_heatxnc_ejx()))
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xped_gas_heatxncx():
    """
    XPED gas HeatXncX



    component


    """

    ret =  xtotal_fed_nre_heatxncx() * xshare_fed_gas_vs_nre_heatxncx(
    ) / efficiency_gases_for_heat_plants(time())
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xped_coal_heatxncx():
    """
    XPED coal HeatXncX



    component


    """

    ret =  xtotal_fed_nre_heatxncx() * xshare_fed_coal_vs_nre_heatxncx(
    ) / efficiency_coal_for_heat_plants(time())
    name = 9
    return round_sig(x=ret, name=name)

@cache('run')
def xdeactivate_heat_dem_correctionxx():
    """
    XDeactivate heat dem correctionXX



    constant


    """

    ret =  1
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xped_liquids_heatxncx():
    """
    XPED liquids HeatXncX



    component


    """

    ret =  xtotal_fed_nre_heatxncx() * xshare_fed_liquids_vs_nre_heatxncx(
    ) / efficiency_liquids_for_heat_plants(time())
    name = 9
    return round_sig(x=ret, name=name)

# SOLIDS - PED, PES & FES
# SOLIDS - PED, PES & FES
# RESOURCES - Coal extraction
# TPED by fuel
@cache('step')
def ped_coal_ej():
    """
    PED coal EJ



    component


    """

    ret =  np.maximum(0,
                      ped_solids() - (pe_traditional_biomass_ej_delayed_1yr() + pes_peat_ej() +
                                      pes_waste_for_tfc() + losses_in_charcoal_plants_ej(time())))
    name = 9
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
@cache('run')
def share_feh_over_fed_solid_bioe():
    """
    share FEH over FED solid bioE



    constant


    """

    ret =  0.13480214704780927
    name = 9
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
@cache('step')
def fes_heat_from_coal():
    """
    FES Heat from coal



    component


    """

    ret =  (xpes_coal_for_heatxcom_plantsx() + xpes_coal_for_heatxnc_plantsx()
            ) * efficiency_coal_for_heat_plants(time())
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_heat_from_natx_gasx():
    """
    XFES Heat from natX gasX



    component


    """

    ret =  (xpes_natx_gas_for_heatxcom_plantsx() + xpes_natx_gas_for_heatxnc_plantsx()
            ) * efficiency_gases_for_heat_plants(time())
    name = 9
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
@cache('step')
def fes_heat_from_oil():
    """
    FES Heat from oil



    component


    """

    ret =  (xpes_oil_for_heatxcom_plantsx() + xpes_oil_for_heatxnc_plantsx()
            ) * efficiency_liquids_for_heat_plants(time())
    name = 9
    return round_sig(x=ret, name=name)

# ENERGY DISTR LOSSES
# EROI system
@cache('step')
def total_distribution_losses():
    """
    Total distribution losses



    component


    """

    ret =  electrical_distribution_losses_ej() + xheatxcom_distribution_lossesx(
    ) + xheatxnc_distribution_lossesx() + xpipeline_transport_constant_2x6_ej_in_2014x()
    name = 9
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# HEAT - Total FE heat generation
@cache('step')
def abundance_heat():
    """
    Abundance heat



    component


    """

    ret =  functions.if_then_else(
        total_fe_heat_generation_ej() > total_fed_heat_ej(), 1, 1 -
        functions.zidz(total_fed_heat_ej() - total_fe_heat_generation_ej(), total_fed_heat_ej()))
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_coal_for_heatxnc_plantsx():
    """
    XPES coal for HeatXnc plantsX



    component


    """

    ret =  extraction_coal_ej() * xshare_coal_dem_for_heatxncx()
    name = 9
    return round_sig(x=ret, name=name)

# RES BIOE - Traditional biomass
# INDICATORS - Energy
@cache('step')
def population_dependent_on_trad_biomass():
    """
    Population dependent on trad biomass



    component


    """

    ret =  pe_traditional_biomass_ej() / pepc_consumption_people_depending_on_trad_biomass()
    name = 9
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def required_fed_sectors_by_fuel():
    """
    required FED sectors by fuel



    component


    """

    ret =  functions.sum(dim='sectors', data=required_final_energy_by_sector_and_fuel())
    name = 9
    return round_sig(x=ret, name=name)

# RES BIOE - Traditional biomass
@cache('run')
def share_trad_biomass_vs_solids_in_households():
    """
    share trad biomass vs solids in households



    constant


    """

    ret =  0.9
    name = 9
    return round_sig(x=ret, name=name)

# RES BIOE - Traditional biomass
# PES solids biofuels & waste
# RES - PED from non-electric RES
# SOLIDS - PED, PES & FES
# INDICATORS - Energy
@cache('step')
def pe_traditional_biomass_ej_delayed_1yr():
    """
    PE traditional biomass EJ delayed 1yr



    component


    """

    ret =  delay_pe_traditional_biomass_ej_round1__time_step___time_step_30_1__time_step()
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xnew_res_capacity_for_heatxcom_twx():
    """
    Xnew RES capacity for heatXcom TWX



    component


    """

    ret =  xr.DataArray(
        data=[functions.if_then_else(
            time() < 2013,
            xhistoric_res_capacity_for_heatxcomx_subxsolarxheatx(time() + 1) -
            xhistoric_res_capacity_for_heatxcomx_subxsolarxheatx(time()),
            xadapt_growth_res_for_heatxcomx().loc[{'res heat':'xsolarxheatx'}].values * xinstalled_capacity_res_heatxcom_twx().loc[{'res heat':'xsolarxheatx'}].values * remaining_potential_constraint_on_new_res_heat_capacity().loc[{'res heat':'xsolarxheatx'}].values) *
        xabundance_res_heatxcom2x(),

        functions.if_then_else(
            time() < 2013,
            xhistoric_res_capacity_for_heatxcomx_subxgeotxheatx(time() + 1) -
            xhistoric_res_capacity_for_heatxcomx_subxgeotxheatx(time()),
            xadapt_growth_res_for_heatxcomx().loc[{'res heat':'xgeotxheatx'}].values * xinstalled_capacity_res_heatxcom_twx().loc[{'res heat':'xgeotxheatx'}].values * remaining_potential_constraint_on_new_res_heat_capacity().loc[{'res heat':'xgeotxheatx'}].values) *
        xabundance_res_heatxcom2x(),

        functions.if_then_else(
            time() < 2013,
            xhistoric_res_capacity_for_heatxcomx_subxsolidbioexheatx(time() + 1) -
            xhistoric_res_capacity_for_heatxcomx_subxsolidbioexheatx(time()),
            xadapt_growth_res_for_heatxcomx().loc[{'res heat':'xsolid bioexheatx'}].values * xinstalled_capacity_res_heatxcom_twx().loc[{'res heat':'xsolid bioexheatx'}].values * remaining_potential_constraint_on_new_res_heat_capacity().loc[{'res heat':'xsolid bioexheatx'}].values) *
        xabundance_res_heatxcom2x()],
        coords={'res heat':['xsolarxheatx','xgeotxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 9
    return round_sig(x=ret, name=name)

# RES BIOE - Traditional biomass
@cache('step')
def share_global_pop_dependent_on_trad_biomass():
    """
    share global pop dependent on trad biomass



    component


    """

    ret =  population_dependent_on_trad_biomass() / population()
    name = 9
    return round_sig(x=ret, name=name)

def xhistoric_res_capacity_for_heatxcomx_subxsolarxheatx(x):
    """
    XHistoric RES capacity for heatXcomX SUBXsolarXheatX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        2.0303326826418786e-06, 2.0303326826418786e-06, 6.4293868283659495e-06,
        5.414220487045009e-06, 7.782941950127201e-06, 8.121330730567515e-06, 9.136497071888453e-06,
        1.2520384876291584e-05, 1.7257827802455968e-05, 1.7257827802455968e-05,
        1.8611382924217222e-05, 1.658105024157534e-05, 2.0980104387299415e-05,
        2.3348825850381605e-05, 4.7036040481203515e-05, 6.497064584454012e-05,
        9.339530340152642e-05, 0.00014246167656537182, 0.00018137638631600782,
        0.0002493925311845108
    ])
    name = 9
    return round_sig(x=ret, name=name)

def xhistoric_res_capacity_for_heatxcomx_subxgeotxheatx(x):
    """
    XHistoric RES capacity for heatXcomX SUBXgeotXheatX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.0019992724712776778, 0.0019333400358171797, 0.001957511273499612, 0.0019656481257887483,
        0.0020795640578366507, 0.002191445776812269, 0.0022711390654088053, 0.00233635354331438,
        0.002309190815819764, 0.002360764100181787, 0.002849214556714914, 0.002853282982859482,
        0.002944702911519773, 0.00304868709739123, 0.0030231996041914363, 0.003124551279028173,
        0.003072140377518739, 0.003306194540423882, 0.003420230132064272, 0.0035817705819221166
    ])
    name = 9
    return round_sig(x=ret, name=name)

def xhistoric_res_capacity_for_heatxcomx_subxsolidbioexheatx(x):
    """
    XHistoric RES capacity for heatXcomX SUBXsolidbioEXheatX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.01167364283024201, 0.014963216653269409, 0.014075786414101827, 0.014111998996576254,
        0.013306506859960276, 0.01325437596189726, 0.014136859473511416, 0.014877727054012784,
        0.016775811784095436, 0.017262303413099544, 0.01806468799009863, 0.01981614664193105,
        0.020802194334247032, 0.02147558347735297, 0.022805745832552054, 0.02708885085880959,
        0.0262413115379895, 0.029897640815394526, 0.030840372931826027, 0.03108885086200959
    ])
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_oil_for_heatxcom_plantsx():
    """
    XPES oil for HeatXcom plantsX



    component


    """

    ret =  pes_oil_ej() * xshare_oil_dem_for_heatxcomx()
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_coal_dem_for_heatxncx():
    """
    Xshare coal dem for HeatXncX



    component


    """

    ret =  functions.zidz(xped_coal_heatxncx(), ped_coal_ej())
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xres_heatxcom_tot_overcapacityx():
    """
    XRES heatXcom tot overcapacityX



    component


    """

    ret =  functions.if_then_else(
        abs(xpotential_fes_tot_res_for_heatxcom_ejx() ) < equality_threshold, 0,
        (xpotential_fes_tot_res_for_heatxcom_ejx() - xfe_real_supply_res_for_heatxcom_tot_ejx()) /
        xpotential_fes_tot_res_for_heatxcom_ejx())
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_natx_gas_for_heatxcom_plantsx():
    """
    XPES natX gas for HeatXcom plantsX



    component


    """

    ret =  xpes_natx_gasx() * xshare_natx_gas_dem_for_heatxcomx()
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_coal_for_heatxcom_plantsx():
    """
    XPES coal for HeatXcom plantsX



    component


    """

    ret =  extraction_coal_ej() * xshare_coal_dem_for_heatxcomx()
    name = 9
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def check_hist_solids_bioe():
    """
    check hist solids bioE



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (pes_solids_bioe_ej() - historic_tpes_primary_solids_biofuels_ej(time())) *
        100 / historic_tpes_primary_solids_biofuels_ej(time()), 0)
    name = 9
    return round_sig(x=ret, name=name)

@cache('step')
def xrequired_heatxcom_after_cc_impactsx():
    """
    XRequired heatXcom after CC impactsX



    component


    """

    ret =  required_fed_by_fuel_before_heat_correction().loc[{'final sources':'heat'}].values * (
        1 + xshare_exlosses_ccx() * method_cc_impacts())
    name = 9
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_labour_share():
    """
    variation labour share



    component


    """

    ret =  growth_labour_share() * labour_share()
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subagriculturehuntingforestryandfishing(x):
    """
    share inventories next step SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.03461669019581704, 0.041942233336400334, 0.009276799405214435, 0.016075123961072323,
            0.024217754183502624, 0.032446087618470246, 0.013212424725283112, 0.03376795965480616,
            0.03382604411506391, 0.018401487557593552, 0.01982139598204595, 0.018904899276004725,
            0.015809658030545876, 0.010160482589476171
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subminingandquarrying(x):
    """
    share inventories next step SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.09426322868948797, 0.1452938030085441, -0.010522918740413148, 0.015323039580696841,
            0.062066986325035677, 0.04610341694033021, -0.03526634160335441, 0.04212319356415536,
            0.022861810664424075, 0.04151711701584518, 0.10319681541565758, 0.021374515787298973,
            0.0025377049344237237, 0.020923144714220698
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subfoodbeveragesandtobacco(x):
    """
    share inventories next step SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.011229125465680464, 0.014950252179427848, 0.0015067456009418114,
            0.008219784022741013, 0.013112354704970405, 0.015008429524128383, 0.00496232604036188,
            0.010115436166276024, 0.014090630265515118, 0.010472018505405785, 0.011288293056324823,
            0.010285984899975899, 0.011670533759363205, 0.003484397941663512
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subtextilesandtextileproducts(x):
    """
    share inventories next step SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.025922783205493752, 0.025535222531598935, 0.0008444416652216993,
            0.020417391981624455, 0.028992295980142094, 0.025311362536731913, 0.007727580121531834,
            0.002825711516221771, 0.012079303124812086, 0.005519571894805784, 0.013120023882477527,
            0.005518243280871071, 0.007934972086945095, -0.01617110107273339
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subleatherleatherandfootwear(x):
    """
    share inventories next step SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.02741801605303332, 0.03308742478636399, 0.004191804519572037, 0.014882491116945982,
            0.019108017204367233, 0.024126896069485666, -0.0019191225645622243,
            -0.0011903123376724623, 0.000523595101187629, 0.0014322759731371915,
            0.0007313941121081056, -0.0067765566832306955, -0.00849822559971483,
            -0.02726512256395504
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subwoodandproductsofwooodandcork(x):
    """
    share inventories next step SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.05099801463435518, 0.060516673656964955, -0.0006817607991020593, 0.03308155567618923,
            0.04742896859549011, 0.011396171400018095, 0.0012185991078174092,
            -0.0032429105557199417, 0.014559681116070878, 0.010593103068180825,
            -0.012816239554372989, -0.07238100443548619, -0.10795313752850011, -0.1616632516649013
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subpulppaperprintingandpublishing(x):
    """
    share inventories next step SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0017209535478593398, 0.011116633244823542, -0.019023876579578658,
            -0.013690682653597472, 0.01797167429902852, 0.009140803917325045, -0.03402770659174538,
            -0.03710591437128926, -0.03741405276295467, -0.03459212455574808, -0.04266723275308837,
            -0.05746675136230981, -0.06279328074894397, -0.08641675950037707
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subcokerefinedpetroleumandnuclearfuel(x):
    """
    share inventories next step SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.022397163766748884, 0.029657526659950657, -0.009844842065857869,
            -0.0012157576444566558, 0.017875454252156166, 0.017945093269225307,
            -0.016425040672843304, 0.0015011190602734984, 0.012571300154402403,
            0.024019248679712973, 0.01245074623470567, -0.011533595218422517, 0.008062090060610764,
            -0.01979745228789293
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subchemicalsandchemicalproducts(x):
    """
    share inventories next step SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.05005938101189353, 0.06341341306758376, 0.020305971801720977, 0.045376681578992654,
            0.04901889821081202, 0.04136089819946551, 0.009874150034516383, 0.013029559844310389,
            0.031023231245148577, 0.027035191107908304, 0.03542343907150098, 0.03570302269336227,
            0.014621691554917347, 0.002697712399088707
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subrubberandplastics(x):
    """
    share inventories next step SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.05155601037400166, 0.05613935712413145, 0.01993782490332162, 0.014493329733790127,
            0.0326650525055877, 0.022302165133758694, 0.007727464783111726, 0.008855703431832364,
            0.02600911316279692, 0.00811772737440893, 0.02960706717970953, 0.0038264700381658127,
            0.005101709762047021, -0.05411230890422361
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subothernonmetalicmineral(x):
    """
    share inventories next step SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.09206994019660208, 0.10746874545566801, 0.029834516830541346, 0.0323522570370145,
            0.04737016753690253, 0.0459722775727321, -0.06198036957638333, -0.028632375936390057,
            -0.0045939428954693065, 0.007506379019318679, 0.0529179511250499, 0.040814956805921516,
            0.023955408358392986, -0.049341810520510544
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subbasicmetalsandfabricatedmetal(x):
    """
    share inventories next step SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0031870963507300964, 0.009123680856206623, -0.05776728641089819, -0.063023093491151,
            0.028766376131321228, -0.0012881757231380966, -0.05179315666335895,
            -0.02408351048492784, 0.007466442421217998, -7.513578161285693e-05,
            0.039232899341370225, -0.04691660213435318, -0.05954158611418101, -0.1578288206210243
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_submachinerynec(x):
    """
    share inventories next step SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.010574262541391523, 0.00696599493820793, -0.002172547925658203,
            0.0016757710330077133, 0.018576378785219894, 0.013271560872378692,
            -0.011732858194081435, 0.01588751699230089, 0.009124612525020096, 0.006060567821491422,
            0.014060643446260344, 0.006903952206265926, 0.004552560360692592, -0.00599537770581172
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subelectricalandopticalequipment(x):
    """
    share inventories next step SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.023168508723542067, 0.028354704013657167, 0.015847142402905943, 0.02265004660576361,
            0.04296331477731733, 0.0016590409136539952, -0.00859960846267911, 0.011252897017189633,
            0.03358385692165521, 0.02280449294652064, 0.029076321142639805, 0.01526034762999966,
            0.013992211001478148, -0.009340748606651865
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subtransportequipment(x):
    """
    share inventories next step SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.008363827805679104, 0.010886046103550569, 0.0032035392047634697,
            0.021106266218248022, 0.011928490546162383, 0.021600092686336735, 0.011439507854791203,
            0.012926172570669917, 0.010692503650363025, 0.0198993047564401, 0.02190095810026589,
            0.015357859751103917, 0.006712683942431885, -0.006722001475231583
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_submanufacturingnecrecycling(x):
    """
    share inventories next step SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.01857734778416911, 0.022404736598060047, 0.007320140860101643, 0.017631024963925525,
            0.03481081306590101, 0.030668103927010547, 0.014946590623808492, 0.013840143578630438,
            0.021493995531179255, 0.02802975748870618, 0.022896789367939235, 0.007827848982984867,
            0.01601343571723777, -0.006185433819430002
        ])
    name = 9
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subelectricitygasandwatersupply(x):
    """
    share inventories next step SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.00042413317519049144, 0.0013558288107230248, 0.0005431237710083078,
            0.0007282829655163663, 0.0006328974362719102, -0.0002694355176032213,
            -0.0010487328296012264, -0.00033330844580074255, -0.0016884595669077243,
            -0.0005938883617179873, 0.00023858153100637895, -0.0009011798421525196,
            -0.00120160160343804, -0.0021056969906526287
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subconstruction(x):
    """
    share inventories next step SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            -0.00014896785893889322, 0.00032862049346872265, 0.00012345146463194415,
            0.0004437915830468549, 0.0002729355150054159, 0.00034811231920423496,
            2.6341616277743705e-05, -5.133599583922819e-05, -0.00020292513825069224,
            0.00020208833289227384, 0.0003465351793535172, 0.0009605838860180702,
            0.0011973075229034687, 0.0010350447943260576
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    share inventories next step SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0023929746013658833, 0.0031142324982446606, 0.0015004391254643849,
            0.0028785617029874824, 0.006521608339560344, 0.006391678364164279,
            -0.000475560711611828, 0.001976871260373391, 0.004240190894063026,
            0.0031622262651984615, 0.005417379052287837, 0.004613781515618692,
            0.004953710498287375, -0.004117596796533778
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
        x):
    """
    share inventories next step SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.015368793493944032, 0.0176144036678834, 0.0076430350349353925, 0.010562158065515002,
            0.016008740199098148, 0.007243281960070458, 0.0009669664677655769,
            0.0035568537305010403, 0.0022383440684109767, 0.003896461990106976,
            0.005195678507696852, 0.003923266504628535, 0.008047525531025905,
            -0.0021458011259934246
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
        x):
    """
    share inventories next step SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.004783819038525881, 0.006436223327803524, 0.0026383681892997285,
            0.005164286657848896, 0.010710865249582027, 0.00984858735790179, 0.0015008944423482583,
            0.003897921771497273, 0.006130297658989541, 0.004498001375669275, 0.006691703056267292,
            0.0054425175554355726, 0.004830282847275033, -0.000741128990872675
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subhotelsandrestaurants(x):
    """
    share inventories next step SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            -0.00020737488320897337, 0.0001446685483487721, -0.0003572996584286368,
            0.0006549819387167176, 0.0020955879814969908, 0.002911897359610583,
            0.0006233249367963001, 0.0007016828988724827, 0.0005359206216678118,
            0.0004919458570455678, 0.0006475599403373204, 0.0002856755321237399,
            0.00023288616823906658, 0.0004124112092339731
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subinlandtransport(x):
    """
    share inventories next step SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.010176833024630351, 0.012536329641196792, 0.00430214531254968, 0.005348367920419633,
            0.006554338122717555, 0.005378382435539208, 0.0019515962711634627,
            0.0027197909056981668, 0.004730984658814206, 0.003957970135238292,
            0.006311517029460731, 0.003440626216147738, 0.0028761249209255206,
            0.0002411375900309864
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subwatertransport(x):
    """
    share inventories next step SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.03445012707955338, 0.033535203070921914, 0.013131889748329966, 0.027919145462781503,
            0.041024487927804686, 0.08466919183155805, 0.00307740567425285, 0.003488829237790001,
            0.0029639888705386475, 0.003599625678616155, 0.05066574298576478,
            0.0014308799030149516, 0.08611950649678217, 0.08684434873201745
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subairtransport(x):
    """
    share inventories next step SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0021245645834738853, 0.0025850705351703157, 0.0008091388791002279,
            0.0023436047159025035, 0.004759184665233702, 0.010872926861949048,
            -0.00044423394440304637, 0.0012555125044372616, 0.0004702718420441391,
            0.001191508687522141, 0.00350381933999573, 0.0010309466286302909, 0.002977872931688678,
            0.0005918793793829298
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    share inventories next step SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.015631051792266138, 0.02986296802524229, 0.012259844509753592, 0.012338783661043004,
            0.011534383792643545, 0.030505284447352746, 0.004886689383534703, 0.004357898672331982,
            0.005428457355999516, 0.005817028918166772, 0.00976790454953955, 0.005881481394028688,
            0.010246379983008607, 0.0019734244893533683
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subpostandtelecommunications(x):
    """
    share inventories next step SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0009488551439241703, 0.0012396105428931219, 0.0007444440848184912,
            0.0014685834763999532, 0.0020441730699772512, 0.0032325119339692335,
            -0.005068250485275013, 0.00043111420964910284, 0.0009169560942092407,
            0.000570352331915007, 0.0007087983726690691, 0.0005448397305536379,
            0.0009565343787786686, 0.000645581668543594
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subfinancialintermedation(x):
    """
    share inventories next step SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            9.11231272700768e-05, 7.542788209146378e-05, -1.4400616429060148e-05,
            8.229749543927034e-05, 0.0003937912440804006, 0.0009093943635503735,
            -0.007524568226264509, -0.004274426163508445, 8.49992603244964e-05,
            9.833482351623146e-05, 0.00013797126261373458, 8.384317855566425e-05,
            0.0006442841441413284, 0.0012264529642908699
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subrealestateactivities(x):
    """
    share inventories next step SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            2.91907526576806e-05, 1.9879281529659025e-05, 1.4105622407877741e-05,
            8.646807700213472e-05, 2.161401447580686e-05, 7.110088024127258e-06,
            7.479695881646118e-06, 2.0296672084406522e-06, 9.156903678863279e-06,
            2.6108742903649806e-05, 3.0340352036303263e-05, 2.039824756142603e-05,
            7.322588921311033e-05, -2.8778440691593814e-05
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subrentingodmeqandotherbusinessactivities(x):
    """
    share inventories next step SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.000831602174777396, 0.0033847710947051517, 0.0005761788915349635,
            0.0031079738205755606, 0.0021995063936603304, 0.0022211587023244217,
            0.000463289136776625, 0.003827049793333797, 0.0032723048332935715, 0.00282669028048174,
            0.0033238445248807004, 0.0019278144414444816, 0.01864576873493577,
            0.0074378011898418245
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subpublicadminanddefencecompulsorysocialsecurity(x):
    """
    share inventories next step SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            -0.006890673086698083, -0.005679982938310693, -0.0072323360613766275,
            -0.006114828678354021, -0.005356116532156387, -0.0038007332944622442,
            -0.005674782512487631, -0.004853486698943161, -0.004676214705836499,
            -0.004558338326465761, -0.004537527544218771, -0.0053082216326086635,
            -0.005544836892740801, -0.003922579936947399
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subeducation(x):
    """
    share inventories next step SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            1.8730226585082882e-05, 1.3231663407411775e-05, 4.799534989921149e-06,
            3.0446684471332545e-05, 1.098827169556429e-05, 2.5728856619887022e-05,
            1.3380636827243842e-05, 4.505158402132235e-05, 2.248247898388548e-06,
            8.725195270895369e-05, -2.615975302391741e-05, 8.26970101974835e-06,
            1.5833949800231638e-05, 4.7541740478130375e-05
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subhealthandsocialwork(x):
    """
    share inventories next step SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            8.428876000649878e-07, 3.87479673142649e-06, 4.426989397210959e-06,
            1.484503418608409e-05, 2.0269244774470538e-05, 1.2951043297292982e-05,
            -3.1727632790426594e-05, 1.9634547384056006e-05, 1.5997263037852878e-05,
            1.0177684924305919e-05, 1.965940194824921e-05, 1.4489386545164095e-05,
            2.3448960211648786e-05, -9.502931854099198e-06
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subothercommunitysocialandpersonaservices(x):
    """
    share inventories next step SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            -0.010629458350807685, -0.008848671736319593, -0.013448818323945873,
            -0.008946039163755016, -0.004793431904417115, -0.0016249396890407504,
            -0.0022020738024543162, -0.0018956770408739025, -0.001785715673228679,
            -0.0018879388774512668, -0.0017953987374448959, -0.0024802205711734492,
            -0.003448648466737081, -0.0020721433092937003
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_next_step_subprivatehouseholdswithemployedpersons(x):
    """
    share inventories next step SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            9.814582614953874e-05, 9.588248461668407e-06, -7.821361593256881e-05,
            -2.411516128394616e-05, -3.631893686050125e-06, -5.08923087015618e-05,
            -0.00022539143199814182, -0.0002681430771119126, 6.398985647852799e-05,
            4.7633349046714305e-05, 0.0015237584902465407, 0.000990345581418567,
            0.0009393292444326495, 0.0008607609964860263
        ])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_historic_demand():
    """
    variation historic demand



    component


    """

    ret =  xr.DataArray(
        data=[historic_hd_subagriculturehuntingforestryandfishing(time() + 1) -
        historic_hd_subagriculturehuntingforestryandfishing(time()),

        historic_hd_subminingandquarrying(time() + 1) - historic_hd_subminingandquarrying(time()),

        historic_hd_subfoodbeveragesandtobacco(time() + 1) -
        historic_hd_subfoodbeveragesandtobacco(time()),

        historic_hd_subtextilesandtextileproducts(time() + 1) -
        historic_hd_subtextilesandtextileproducts(time()),

        historic_hd_subleatherleatherandfootwear(time() + 1) -
        historic_hd_subleatherleatherandfootwear(time()),

        historic_hd_subwoodandproductsofwooodandcork(time() + 1) -
        historic_hd_subwoodandproductsofwooodandcork(time()),

        historic_hd_subpulppaperprintingandpublishing(time() + 1) -
        historic_hd_subpulppaperprintingandpublishing(time()),

        historic_hd_subcokerefinedpetroleumandnuclearfuel(time() + 1) -
        historic_hd_subcokerefinedpetroleumandnuclearfuel(time()),

        historic_hd_subchemicalsandchemicalproducts(time() + 1) -
        historic_hd_subchemicalsandchemicalproducts(time()),

        historic_hd_subrubberandplastics(time() + 1) - historic_hd_subrubberandplastics(time()),

        historic_hd_subothernonmetalicmineral(time() + 1) - historic_hd_subothernonmetalicmineral(
            time()),

        historic_hd_subbasicmetalsandfabricatedmetal(time() + 1) -
        historic_hd_subbasicmetalsandfabricatedmetal(time()),

        historic_hd_submachinerynec(time() + 1) - historic_hd_submachinerynec(time()),

        historic_hd_subelectricalandopticalequipment(time() + 1) -
        historic_hd_subelectricalandopticalequipment(time()),

        historic_hd_subtransportequipment(time() + 1) - historic_hd_subtransportequipment(time()),

        historic_hd_submanufacturingnecrecycling(time() + 1) -
        historic_hd_submanufacturingnecrecycling(time()),

        historic_hd_subelectricitygasandwatersupply(time() + 1) -
        historic_hd_subelectricitygasandwatersupply(time()),

        historic_hd_subconstruction(time() + 1) - historic_hd_subconstruction(time()),

        historic_hd_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time() + 1) -
        historic_hd_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),

        historic_hd_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time() + 1) -
        historic_hd_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),

        historic_hd_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time() + 1) -
        historic_hd_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),

        historic_hd_subhotelsandrestaurants(time() + 1) - historic_hd_subhotelsandrestaurants(
            time()),

        historic_hd_subinlandtransport(time() + 1) - historic_hd_subinlandtransport(time()),

        historic_hd_subwatertransport(time() + 1) - historic_hd_subwatertransport(time()),

        historic_hd_subairtransport(time() + 1) - historic_hd_subairtransport(time()),

        historic_hd_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time() + 1) -
        historic_hd_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),

        historic_hd_subpostandtelecommunications(time() + 1) -
        historic_hd_subpostandtelecommunications(time()),

        historic_hd_subfinancialintermedation(time() + 1) - historic_hd_subfinancialintermedation(
            time()),

        historic_hd_subrealestateactivities(time() + 1) - historic_hd_subrealestateactivities(
            time()),

        historic_hd_subrentingodmeqandotherbusinessactivities(time() + 1) -
        historic_hd_subrentingodmeqandotherbusinessactivities(time()),

        historic_hd_subpublicadminanddefencecompulsorysocialsecurity(time() + 1) -
        historic_hd_subpublicadminanddefencecompulsorysocialsecurity(time()),

        historic_hd_subeducation(time() + 1) - historic_hd_subeducation(time()),

        historic_hd_subhealthandsocialwork(time() + 1) - historic_hd_subhealthandsocialwork(
            time()),

        historic_hd_subothercommunitysocialandpersonaservices(time() + 1) -
        historic_hd_subothercommunitysocialandpersonaservices(time()),

        historic_hd_subprivatehouseholdswithemployedpersons(time() + 1) -
        historic_hd_subprivatehouseholdswithemployedpersons(time())],
        coords={'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subagriculturehuntingforestryandfishing(x):
    """
    share consum goverments SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.005268868173873272, 0.0067709613778314465, 0.006395238231716023,
            0.007562221479560522, 0.006909511984020509, 0.007982988184916011, 0.008446717428753593,
            0.009326405167881661, 0.009029494951501007, 0.009305637069275475, 0.009311394291421764,
            0.009911989953154088, 0.010233756778334888, 0.008994169837883667, 0.00869869421963916
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subminingandquarrying(x):
    """
    share consum goverments SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.00465464399106785, 0.00436164392290795, 0.003995603763685553, 0.004066047799643348,
            0.006393419006564868, 0.006003153311605073, 0.0042157315738525596,
            0.005643824953173819, 0.005273239069785034, 0.005338694073945094, 0.005231106875192582,
            0.004151188915356074, 0.004717994395060901, 0.00450208701463089, 0.004934708656353205
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subfoodbeveragesandtobacco(x):
    """
    share consum goverments SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.004599538103475525, 0.0038220671963571573, 0.002903073070627261,
            0.002936792709517417, 0.003142168948714033, 0.0031340511473669625,
            0.0030303441468097964, 0.003102005746783019, 0.003076968042145073,
            0.002915823853670982, 0.002931906554520569, 0.002970467773012117, 0.003045614006493215,
            0.002961515281763346, 0.002885066309883363
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subtextilesandtextileproducts(x):
    """
    share consum goverments SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.003187221954138558, 0.003551578898993538, 0.00295957655229884, 0.0026490453856551233,
            0.0028527421747088517, 0.0025817118313251836, 0.0027787410874748823,
            0.0028558425631741735, 0.003004220437100366, 0.0030660841863479906,
            0.0034604581246728532, 0.00402674114226547, 0.004420561138285466, 0.004407476902887508,
            0.005253312524302093
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subleatherleatherandfootwear(x):
    """
    share consum goverments SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0017657285218539121, 0.0015559542279136721, 0.0012399353066103794,
            0.0013267840090444238, 0.00133344367825553, 0.0012313162901100574,
            0.0011201067862366928, 0.0008200600923468703, 0.0007351843303268899,
            0.0007872038350995223, 0.0008524970828855757, 0.000980302019330771,
            0.0012534804090331476, 0.0013308059227996672, 0.001514645887241861
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subwoodandproductsofwooodandcork(x):
    """
    share consum goverments SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.008220627948340928, 0.009005435491861913, 0.00784410468158404, 0.010385380453511185,
            0.005983313227598484, 0.0029442159037194747, 0.003191210291931943,
            0.0038803930252921484, 0.0033353083568353945, 0.0031038702125085094,
            0.002934685814185623, 0.0032009994283761284, 0.0031885473327917984,
            0.003374415685759153, 0.003641151621250122
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subpulppaperprintingandpublishing(x):
    """
    share consum goverments SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.011608853579237062, 0.012152022471383788, 0.010944306361792914, 0.013144143333725558,
            0.01481962044356459, 0.014280346332352903, 0.014584228557471577, 0.01813378209427802,
            0.01800422812064625, 0.017428493050279235, 0.01703528662283678, 0.018253358153044425,
            0.01977514458023117, 0.016156351225622194, 0.016038861285036368
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subcokerefinedpetroleumandnuclearfuel(x):
    """
    share consum goverments SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.014933772211345458, 0.014073012258205416, 0.014138264180255291, 0.01638462176078289,
            0.012559974344452331, 0.009745231174255206, 0.008722834180249866, 0.009133945504476285,
            0.007599833303872037, 0.006750850405801407, 0.005867347651754397, 0.006685369003406569,
            0.007433579067682456, 0.006607524613390546, 0.007784103038001168
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subchemicalsandchemicalproducts(x):
    """
    share consum goverments SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.09588197518751453, 0.1019113905016282, 0.0973261077437338, 0.10451610930994258,
            0.1038905105421696, 0.10684729730821506, 0.11441273827060026, 0.11710461296677933,
            0.11802421308716918, 0.1131680314380918, 0.11811035287289917, 0.1159991266683548,
            0.12291936960693232, 0.13104795855975498, 0.13239560556296268
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subrubberandplastics(x):
    """
    share consum goverments SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.006867200230823096, 0.007733117166691025, 0.007382076353556275, 0.008098756846008068,
            0.008121760071879654, 0.006928724883784163, 0.006611896916921303, 0.006412599503225718,
            0.00627859918453306, 0.006463852038577615, 0.006987327120633945, 0.007095522534506072,
            0.007271046894131369, 0.006823697974194504, 0.0071450764967817045
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subothernonmetalicmineral(x):
    """
    share consum goverments SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.005150147500748169, 0.005601972248454674, 0.004984833707272183,
            0.0050045677027149025, 0.005220994619862051, 0.004906031702269911,
            0.0046695726368900506, 0.004765867407184146, 0.004575710980134348,
            0.004925408480648355, 0.005451801457053173, 0.005476256518325244, 0.005940164753101489,
            0.006012657637842877, 0.006158849250484725
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subbasicmetalsandfabricatedmetal(x):
    """
    share consum goverments SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.003714588731977942, 0.003695607624002559, 0.003264025079857985, 0.003456144989750174,
            0.003909675072085389, 0.003567486126274761, 0.004087484008773003, 0.004580826890509096,
            0.003954657469767477, 0.003832560933406138, 0.003872948283557392,
            0.0037354425586724055, 0.004369197638847987, 0.004504712881791057, 0.005212766425513586
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_submachinerynec(x):
    """
    share consum goverments SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.003462102165770177, 0.0036617822542566983, 0.003605389168536691,
            0.003975012421065428, 0.003576070848771421, 0.0024004574567129543,
            0.002133422389799059, 0.0020732528087047206, 0.0015195162676146234,
            0.0016070917766987168, 0.0017110936542217888, 0.0017042538525656047,
            0.0018089274613794315, 0.00179493990200113, 0.0020036796744404423
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subelectricalandopticalequipment(x):
    """
    share consum goverments SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.005879390033122016, 0.006689280053562139, 0.006260917348372466, 0.006383402577586877,
            0.007626339783897834, 0.008952172345135323, 0.009938402470472538, 0.010584245313257168,
            0.010485426607225824, 0.009528021492371893, 0.009159007241327075, 0.009390892943976928,
            0.009368214117085202, 0.008491380955383702, 0.010526372953268005
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subtransportequipment(x):
    """
    share consum goverments SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.002818274426028272, 0.0030134399979940317, 0.0030013782829294688,
            0.003324582114787668, 0.0033937364928424093, 0.004179287165548907,
            0.004060256365656964, 0.004599388270976094, 0.004521827080112946,
            0.0039045057195699682, 0.003377303061102904, 0.0032156831100202668,
            0.002999720037553041, 0.003025284132570713, 0.0031225730516014757
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_submanufacturingnecrecycling(x):
    """
    share consum goverments SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.02071206794500831, 0.02145589854505419, 0.0214355818274276, 0.02514402773966533,
            0.01774772016365786, 0.013419020602064787, 0.012203537170182726, 0.011759010701610579,
            0.010195116719573927, 0.010562251657894164, 0.010833741123216334, 0.01030096052750391,
            0.010593246762629493, 0.011621782561596393, 0.012719313855909485
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subelectricitygasandwatersupply(x):
    """
    share consum goverments SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.011073682393823454, 0.014062257866705289, 0.013761686141498856, 0.014297671627160625,
            0.012947249209290532, 0.013116308794034108, 0.012724684366983808, 0.01399302698870909,
            0.013964725368533456, 0.014932092232165415, 0.016923049048231383, 0.020237318698904688,
            0.020307088408552488, 0.018222111231047673, 0.019718462441084224
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subconstruction(x):
    """
    share consum goverments SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0038967149105365593, 0.0037926289235951103, 0.003584887454493617,
            0.004030136291386803, 0.0043782604871680005, 0.004424182707116258,
            0.004527167347521452, 0.004951394442164713, 0.0046428441481454205,
            0.003398133507831463, 0.0023546145866514584, 0.001689633412741827,
            0.0016539914075433919, 0.0017930949205848523, 0.0020141898925777795
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    share consum goverments SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0038892410595649297, 0.005403324383601953, 0.005555193258671485,
            0.005806921261116362, 0.0055187734847195, 0.0060804999283362755, 0.006576770190355463,
            0.006599236892085225, 0.006951525047583767, 0.006930155684757487,
            0.0071569467110036466, 0.006965090647827174, 0.006885708406717167,
            0.006969574494336234, 0.008003455543375147
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
        x):
    """
    share consum goverments SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.008215710233384077, 0.01006648267750522, 0.009307515695653029, 0.009444958125729505,
            0.009364679486856759, 0.009765600552568655, 0.010207317570209486, 0.010771962374229452,
            0.01067527004648461, 0.010462899400149748, 0.010587576110538116, 0.0108347645089915,
            0.01077759596108046, 0.010732709364063026, 0.012048869135356076
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
        x):
    """
    share consum goverments SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.004884652051206438, 0.006598553542763822, 0.006508686887531563, 0.006705691997046858,
            0.007025347276833513, 0.00720794339648111, 0.007317693553382741, 0.007707318698817354,
            0.007486817205849609, 0.007496077432961913, 0.007788715871380901, 0.007790913253695477,
            0.00796523132976134, 0.008075132023729639, 0.009141731015561386
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subhotelsandrestaurants(x):
    """
    share consum goverments SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.014358500409294464, 0.009986641171961503, 0.004972707818870327, 0.005180644227287415,
            0.003157808005666468, 0.0026105687179988736, 0.0023619220258709814,
            0.0022802596097973695, 0.002236912343894969, 0.0032285975612425005,
            0.004006717191046085, 0.004771145868031531, 0.005057546903846365, 0.004275366439841901,
            0.004318083939434814
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subinlandtransport(x):
    """
    share consum goverments SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.015458390956758244, 0.020988629145090622, 0.020208389033241396, 0.024190375963752966,
            0.028979965613103086, 0.0340573400355154, 0.03787454338990796, 0.038778817683728796,
            0.042856040285980994, 0.04333737779943819, 0.040877931695314014, 0.040017654428911954,
            0.044561875720359245, 0.042253292012375945, 0.04548291871340436
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subwatertransport(x):
    """
    share consum goverments SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.006638978375793083, 0.007670674964602831, 0.006643243068732121,
            0.0070420634278164475, 0.005625692184953877, 0.005825483070381239,
            0.005755526565640597, 0.0059064747947359724, 0.031040299562048543, 0.0444328959360957,
            0.045343068051017205, 0.0419767170469466, 0.05198184793293863, 0.050442763300947255,
            0.05576891724508192
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subairtransport(x):
    """
    share consum goverments SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.005646296734251731, 0.0056558638910256965, 0.003568218719757792,
            0.003452988698190144, 0.003113214535012367, 0.002600142907132616, 0.00352760744719066,
            0.0035150489272458216, 0.008303301417705423, 0.01003440101939009, 0.007348632019004971,
            0.007740245562564343, 0.009810656999234713, 0.009168958102788652, 0.00981130620506604
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    share consum goverments SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.09733431731089942, 0.09901239662412917, 0.09351013009433082, 0.09307715377473842,
            0.090464559595115, 0.08605821054032661, 0.10652486260006906, 0.11145975472969767,
            0.11357295739188283, 0.11017723359651423, 0.10491172806150008, 0.1011831510665581,
            0.09842241225860136, 0.09838464480853919, 0.09731960940160281
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subpostandtelecommunications(x):
    """
    share consum goverments SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.007425900250296456, 0.007218033564359982, 0.006671966436504588, 0.00755011709325932,
            0.007798984111468217, 0.006194892527289748, 0.005508562494627751, 0.00575293539502172,
            0.005392135225406816, 0.004821274684917958, 0.005148559804682313, 0.00553343108337951,
            0.006297209998691717, 0.006491788496943986, 0.007810889098747097
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subfinancialintermedation(x):
    """
    share consum goverments SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03758331475685865, 0.027581273496790935, 0.025235830692599766, 0.02508840686447435,
            0.024038702335690176, 0.02474875619024354, 0.031203030082480394, 0.03890041017856948,
            0.04298777082414324, 0.04647391196264396, 0.04820648818054139, 0.054020887398685435,
            0.05660365990915811, 0.057191672612287296, 0.05349723889956909
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subrealestateactivities(x):
    """
    share consum goverments SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0053409330371467, 0.00480747520979442, 0.004356879732159147, 0.005051056791957753,
            0.0042817328034103035, 0.003082759572882315, 0.002882345202542986,
            0.0028984962506404615, 0.0027998697728837665, 0.0029336909071597957,
            0.0029228584018284724, 0.0032377155699126986, 0.0032471376757725566,
            0.003313038453281683, 0.0033056974047413435
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subrentingodmeqandotherbusinessactivities(x):
    """
    share consum goverments SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.10777177059907846, 0.12766213970374193, 0.128477992110974, 0.13977631646313407,
            0.14717767633342763, 0.15478196524040605, 0.16957094795516728, 0.17978959576830966,
            0.1755571954925755, 0.16458717780184334, 0.1432232593236837, 0.13416821175274135,
            0.14146388986144026, 0.13442398718655846, 0.13699391236524086
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subpublicadminanddefencecompulsorysocialsecurity(x):
    """
    share consum goverments SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8825512222435824, 0.9019740353263537, 0.9055209153005452, 0.9070353979129336,
            0.9101561542351737, 0.9071215692757418, 0.9039291693459441, 0.9009660123926903,
            0.8991346577949707, 0.8948821888313753, 0.8935084759144025, 0.8922590472793164,
            0.8935680286075393, 0.895349946046101, 0.892671897122667
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subeducation(x):
    """
    share consum goverments SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.7095164543500799, 0.7040856404423688, 0.6993589354235593, 0.698581612534672,
            0.6987685915889159, 0.6958775202480544, 0.6893004419028962, 0.6836379033975156,
            0.6765593042142081, 0.6662693916312913, 0.6630464757694154, 0.660201578395071,
            0.6615962513449365, 0.659828413873094, 0.6593581001080975
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subhealthandsocialwork(x):
    """
    share consum goverments SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.4588957626796008, 0.44707649924014653, 0.43785031864326934, 0.43967250874908564,
            0.44259141677895925, 0.44421483587784627, 0.440685782407788, 0.43726784569342825,
            0.4372577649745116, 0.43796753502845864, 0.4378164095136612, 0.43992755579739684,
            0.4425537217710332, 0.443198533634356, 0.4408943873395209
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subothercommunitysocialandpersonaservices(x):
    """
    share consum goverments SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.13593187360842532, 0.0910574921371381, 0.0873395017587308, 0.08692860129270708,
            0.08999187621790489, 0.090740783333636, 0.08823471238711048, 0.0863032736828504,
            0.08522731060173952, 0.08453682210016202, 0.08167532299694542, 0.0807001909529624,
            0.08463448174674901, 0.0850306568023039, 0.08931954847715241
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_subprivatehouseholdswithemployedpersons(x):
    """
    share consum goverments SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0010228200228570537, 0.0011290416169956153, 0.0009518818302216298,
            0.0009739207509484318, 0.003226765298868626, 0.0045618940852030915,
            0.006250676296558422, 0.007856379237276827, 0.009536129903780428, 0.009469554208776931,
            0.01007163264373241, 0.010290112726275327, 0.010477541610775753, 0.009767631985404314,
            0.012031510285201963
        ])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_gfcf():
    """
    variation GFCF



    component


    """

    ret =  functions.if_then_else(time() < 2009,
                                  variation_historic_gfcf(),
                                  np.exp(beta_0_cap()) * np.exp(beta_1_cap()) *
                                  ((cc_sectoral_next_step())**beta_2_cap() -
                                   (cc_sectoral())**beta_2_cap()))
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_historic_gfcf():
    """
    variation historic GFCF



    component


    """

    ret =  xr.DataArray(
        data=[historic_gfcf_subagriculturehuntingforestryandfishing(time() + 1) -
        historic_gfcf_subagriculturehuntingforestryandfishing(time()),

        historic_gfcf_subminingandquarrying(time() + 1) - historic_gfcf_subminingandquarrying(
            time()),

        historic_gfcf_subfoodbeveragesandtobacco(time() + 1) -
        historic_gfcf_subfoodbeveragesandtobacco(time()),

        historic_gfcf_subtextilesandtextileproducts(time() + 1) -
        historic_gfcf_subtextilesandtextileproducts(time()),

        historic_gfcf_subleatherleatherandfootwear(time() + 1) -
        historic_gfcf_subleatherleatherandfootwear(time()),

        historic_gfcf_subwoodandproductsofwooodandcork(time() + 1) -
        historic_gfcf_subwoodandproductsofwooodandcork(time()),

        historic_gfcf_subpulppaperprintingandpublishing(time() + 1) -
        historic_gfcf_subpulppaperprintingandpublishing(time()),

        historic_gfcf_subcokerefinedpetroleumandnuclearfuel(time() + 1) -
        historic_gfcf_subcokerefinedpetroleumandnuclearfuel(time()),

        historic_gfcf_subchemicalsandchemicalproducts(time() + 1) -
        historic_gfcf_subchemicalsandchemicalproducts(time()),

        historic_gfcf_subrubberandplastics(time() + 1) - historic_gfcf_subrubberandplastics(
            time()),

        historic_gfcf_subothernonmetalicmineral(time() + 1) -
        historic_gfcf_subothernonmetalicmineral(time()),

        historic_gfcf_subbasicmetalsandfabricatedmetal(time() + 1) -
        historic_gfcf_subbasicmetalsandfabricatedmetal(time()),

        historic_gfcf_submachinerynec(time() + 1) - historic_gfcf_submachinerynec(time()),

        historic_gfcf_subelectricalandopticalequipment(time() + 1) -
        historic_gfcf_subelectricalandopticalequipment(time()),

        historic_gfcf_subtransportequipment(time() + 1) - historic_gfcf_subtransportequipment(
            time()),

        historic_gfcf_submanufacturingnecrecycling(time() + 1) -
        historic_gfcf_submanufacturingnecrecycling(time()),

        historic_gfcf_subelectricitygasandwatersupply(time() + 1) -
        historic_gfcf_subelectricitygasandwatersupply(time()),

        historic_gfcf_subconstruction(time() + 1) - historic_gfcf_subconstruction(time()),

        historic_gfcf_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time() + 1) -
        historic_gfcf_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),

        historic_gfcf_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time() + 1) -
        historic_gfcf_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),

        historic_gfcf_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time() + 1) -
        historic_gfcf_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),

        historic_gfcf_subhotelsandrestaurants(time() + 1) - historic_gfcf_subhotelsandrestaurants(
            time()),

        historic_gfcf_subinlandtransport(time() + 1) - historic_gfcf_subinlandtransport(time()),

        historic_gfcf_subwatertransport(time() + 1) - historic_gfcf_subwatertransport(time()),

        historic_gfcf_subairtransport(time() + 1) - historic_gfcf_subairtransport(time()),

        historic_gfcf_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time() + 1) -
        historic_gfcf_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),

        historic_gfcf_subpostandtelecommunications(time() + 1) -
        historic_gfcf_subpostandtelecommunications(time()),

        historic_gfcf_subfinancialintermedation(time() + 1) -
        historic_gfcf_subfinancialintermedation(time()),

        historic_gfcf_subrealestateactivities(time() + 1) - historic_gfcf_subrealestateactivities(
            time()),

        historic_gfcf_subrentingodmeqandotherbusinessactivities(time() + 1) -
        historic_gfcf_subrentingodmeqandotherbusinessactivities(time()),

        historic_gfcf_subpublicadminanddefencecompulsorysocialsecurity(time() + 1) -
        historic_gfcf_subpublicadminanddefencecompulsorysocialsecurity(time()),

        historic_gfcf_subeducation(time() + 1) - historic_gfcf_subeducation(time()),

        historic_gfcf_subhealthandsocialwork(time() + 1) - historic_gfcf_subhealthandsocialwork(
            time()),

        historic_gfcf_subothercommunitysocialandpersonaservices(time() + 1) -
        historic_gfcf_subothercommunitysocialandpersonaservices(time()),

        historic_gfcf_subprivatehouseholdswithemployedpersons(time() + 1) -
        historic_gfcf_subprivatehouseholdswithemployedpersons(time())],
        coords={'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_household_demand():
    """
    variation household demand



    component


    """

    ret =  functions.if_then_else(time() < 2009,
                                  variation_historic_demand(),
                                  np.exp(bet_0_lab()) * np.exp(beta_1_lab()) *
                                  ((lc_next_step())**beta_2_lab() - (lc())**beta_2_lab()))
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def labour_share():
    """
    labour share



    component


    """


    ret =  integ_labour_share()
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subagriculturehuntingforestryandfishing(x):
    """
    share inventories SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.039806446548852466, 0.03461669019581704, 0.041942233336400334, 0.009276799405214435,
            0.016075123961072323, 0.024217754183502624, 0.032446087618470246, 0.013212424725283112,
            0.03376795965480616, 0.03382604411506391, 0.018401487557593552, 0.01982139598204595,
            0.018904899276004725, 0.015809658030545876, 0.010160482589476171
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subminingandquarrying(x):
    """
    share inventories SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.14492615777706683, 0.09426322868948797, 0.1452938030085441, -0.010522918740413148,
            0.015323039580696841, 0.062066986325035677, 0.04610341694033021, -0.03526634160335441,
            0.04212319356415536, 0.022861810664424075, 0.04151711701584518, 0.10319681541565758,
            0.021374515787298973, 0.0025377049344237237, 0.020923144714220698
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subfoodbeveragesandtobacco(x):
    """
    share inventories SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.015018588928445093, 0.011229125465680464, 0.014950252179427848,
            0.0015067456009418114, 0.008219784022741013, 0.013112354704970405,
            0.015008429524128383, 0.00496232604036188, 0.010115436166276024, 0.014090630265515118,
            0.010472018505405785, 0.011288293056324823, 0.010285984899975899, 0.011670533759363205,
            0.003484397941663512
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subtextilesandtextileproducts(x):
    """
    share inventories SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03700800887524616, 0.025922783205493752, 0.025535222531598935, 0.0008444416652216993,
            0.020417391981624455, 0.028992295980142094, 0.025311362536731913, 0.007727580121531834,
            0.002825711516221771, 0.012079303124812086, 0.005519571894805784, 0.013120023882477527,
            0.005518243280871071, 0.007934972086945095, -0.01617110107273339
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subleatherleatherandfootwear(x):
    """
    share inventories SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03913219959356502, 0.02741801605303332, 0.03308742478636399, 0.004191804519572037,
            0.014882491116945982, 0.019108017204367233, 0.024126896069485666,
            -0.0019191225645622243, -0.0011903123376724623, 0.000523595101187629,
            0.0014322759731371915, 0.0007313941121081056, -0.0067765566832306955,
            -0.00849822559971483, -0.02726512256395504
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subwoodandproductsofwooodandcork(x):
    """
    share inventories SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.07544310598978189, 0.05099801463435518, 0.060516673656964955, -0.0006817607991020593,
            0.03308155567618923, 0.04742896859549011, 0.011396171400018095, 0.0012185991078174092,
            -0.0032429105557199417, 0.014559681116070878, 0.010593103068180825,
            -0.012816239554372989, -0.07238100443548619, -0.10795313752850011, -0.1616632516649013
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subpulppaperprintingandpublishing(x):
    """
    share inventories SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.014348233928113558, 0.0017209535478593398, 0.011116633244823542,
            -0.019023876579578658, -0.013690682653597472, 0.01797167429902852,
            0.009140803917325045, -0.03402770659174538, -0.03710591437128926, -0.03741405276295467,
            -0.03459212455574808, -0.04266723275308837, -0.05746675136230981, -0.06279328074894397,
            -0.08641675950037707
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subcokerefinedpetroleumandnuclearfuel(x):
    """
    share inventories SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.009662414285604149, 0.022397163766748884, 0.029657526659950657,
            -0.009844842065857869, -0.0012157576444566558, 0.017875454252156166,
            0.017945093269225307, -0.016425040672843304, 0.0015011190602734984,
            0.012571300154402403, 0.024019248679712973, 0.01245074623470567, -0.011533595218422517,
            0.008062090060610764, -0.01979745228789293
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subchemicalsandchemicalproducts(x):
    """
    share inventories SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.05693764085223191, 0.05005938101189353, 0.06341341306758376, 0.020305971801720977,
            0.045376681578992654, 0.04901889821081202, 0.04136089819946551, 0.009874150034516383,
            0.013029559844310389, 0.031023231245148577, 0.027035191107908304, 0.03542343907150098,
            0.03570302269336227, 0.014621691554917347, 0.002697712399088707
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subrubberandplastics(x):
    """
    share inventories SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0778141986127367, 0.05155601037400166, 0.05613935712413145, 0.01993782490332162,
            0.014493329733790127, 0.0326650525055877, 0.022302165133758694, 0.007727464783111726,
            0.008855703431832364, 0.02600911316279692, 0.00811772737440893, 0.02960706717970953,
            0.0038264700381658127, 0.005101709762047021, -0.05411230890422361
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subothernonmetalicmineral(x):
    """
    share inventories SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.12020984415955097, 0.09206994019660208, 0.10746874545566801, 0.029834516830541346,
            0.0323522570370145, 0.04737016753690253, 0.0459722775727321, -0.06198036957638333,
            -0.028632375936390057, -0.0045939428954693065, 0.007506379019318679,
            0.0529179511250499, 0.040814956805921516, 0.023955408358392986, -0.049341810520510544
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subbasicmetalsandfabricatedmetal(x):
    """
    share inventories SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03598798077997525, 0.0031870963507300964, 0.009123680856206623, -0.05776728641089819,
            -0.063023093491151, 0.028766376131321228, -0.0012881757231380966, -0.05179315666335895,
            -0.02408351048492784, 0.007466442421217998, -7.513578161285693e-05,
            0.039232899341370225, -0.04691660213435318, -0.05954158611418101, -0.1578288206210243
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_submachinerynec(x):
    """
    share inventories SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.01812631038583679, 0.010574262541391523, 0.00696599493820793, -0.002172547925658203,
            0.0016757710330077133, 0.018576378785219894, 0.013271560872378692,
            -0.011732858194081435, 0.01588751699230089, 0.009124612525020096, 0.006060567821491422,
            0.014060643446260344, 0.006903952206265926, 0.004552560360692592, -0.00599537770581172
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subelectricalandopticalequipment(x):
    """
    share inventories SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03195972862002789, 0.023168508723542067, 0.028354704013657167, 0.015847142402905943,
            0.02265004660576361, 0.04296331477731733, 0.0016590409136539952, -0.00859960846267911,
            0.011252897017189633, 0.03358385692165521, 0.02280449294652064, 0.029076321142639805,
            0.01526034762999966, 0.013992211001478148, -0.009340748606651865
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subtransportequipment(x):
    """
    share inventories SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.01872732946069781, 0.008363827805679104, 0.010886046103550569, 0.0032035392047634697,
            0.021106266218248022, 0.011928490546162383, 0.021600092686336735, 0.011439507854791203,
            0.012926172570669917, 0.010692503650363025, 0.0198993047564401, 0.02190095810026589,
            0.015357859751103917, 0.006712683942431885, -0.006722001475231583
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_submanufacturingnecrecycling(x):
    """
    share inventories SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.02326623640267202, 0.01857734778416911, 0.022404736598060047, 0.007320140860101643,
            0.017631024963925525, 0.03481081306590101, 0.030668103927010547, 0.014946590623808492,
            0.013840143578630438, 0.021493995531179255, 0.02802975748870618, 0.022896789367939235,
            0.007827848982984867, 0.01601343571723777, -0.006185433819430002
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subelectricitygasandwatersupply(x):
    """
    share inventories SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0009962481163932318, 0.00042413317519049144, 0.0013558288107230248,
            0.0005431237710083078, 0.0007282829655163663, 0.0006328974362719102,
            -0.0002694355176032213, -0.0010487328296012264, -0.00033330844580074255,
            -0.0016884595669077243, -0.0005938883617179873, 0.00023858153100637895,
            -0.0009011798421525196, -0.00120160160343804, -0.0021056969906526287
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subconstruction(x):
    """
    share inventories SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.00020072745436100465, -0.00014896785893889322, 0.00032862049346872265,
            0.00012345146463194415, 0.0004437915830468549, 0.0002729355150054159,
            0.00034811231920423496, 2.6341616277743705e-05, -5.133599583922819e-05,
            -0.00020292513825069224, 0.00020208833289227384, 0.0003465351793535172,
            0.0009605838860180702, 0.0011973075229034687, 0.0010350447943260576
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(x):
    """
    share inventories SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.004810299878730776, 0.0023929746013658833, 0.0031142324982446606,
            0.0015004391254643849, 0.0028785617029874824, 0.006521608339560344,
            0.006391678364164279, -0.000475560711611828, 0.001976871260373391,
            0.004240190894063026, 0.0031622262651984615, 0.005417379052287837,
            0.004613781515618692, 0.004953710498287375, -0.004117596796533778
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(x):
    """
    share inventories SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.021671426326427642, 0.015368793493944032, 0.0176144036678834, 0.0076430350349353925,
            0.010562158065515002, 0.016008740199098148, 0.007243281960070458,
            0.0009669664677655769, 0.0035568537305010403, 0.0022383440684109767,
            0.003896461990106976, 0.005195678507696852, 0.003923266504628535, 0.008047525531025905,
            -0.0021458011259934246
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(x):
    """
    share inventories SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.008215506595961384, 0.004783819038525881, 0.006436223327803524,
            0.0026383681892997285, 0.005164286657848896, 0.010710865249582027, 0.00984858735790179,
            0.0015008944423482583, 0.003897921771497273, 0.006130297658989541,
            0.004498001375669275, 0.006691703056267292, 0.0054425175554355726,
            0.004830282847275033, -0.000741128990872675
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subhotelsandrestaurants(x):
    """
    share inventories SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            -0.00014589121778786507, -0.00020737488320897337, 0.0001446685483487721,
            -0.0003572996584286368, 0.0006549819387167176, 0.0020955879814969908,
            0.002911897359610583, 0.0006233249367963001, 0.0007016828988724827,
            0.0005359206216678118, 0.0004919458570455678, 0.0006475599403373204,
            0.0002856755321237399, 0.00023288616823906658, 0.0004124112092339731
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subinlandtransport(x):
    """
    share inventories SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.01399853187795113, 0.010176833024630351, 0.012536329641196792, 0.00430214531254968,
            0.005348367920419633, 0.006554338122717555, 0.005378382435539208,
            0.0019515962711634627, 0.0027197909056981668, 0.004730984658814206,
            0.003957970135238292, 0.006311517029460731, 0.003440626216147738,
            0.0028761249209255206, 0.0002411375900309864
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subwatertransport(x):
    """
    share inventories SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.04320323072050123, 0.03445012707955338, 0.033535203070921914, 0.013131889748329966,
            0.027919145462781503, 0.041024487927804686, 0.08466919183155805, 0.00307740567425285,
            0.003488829237790001, 0.0029639888705386475, 0.003599625678616155, 0.05066574298576478,
            0.0014308799030149516, 0.08611950649678217, 0.08684434873201745
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subairtransport(x):
    """
    share inventories SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0031032243349323895, 0.0021245645834738853, 0.0025850705351703157,
            0.0008091388791002279, 0.0023436047159025035, 0.004759184665233702,
            0.010872926861949048, -0.00044423394440304637, 0.0012555125044372616,
            0.0004702718420441391, 0.001191508687522141, 0.00350381933999573,
            0.0010309466286302909, 0.002977872931688678, 0.0005918793793829298
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    share inventories SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.02284442934144017, 0.015631051792266138, 0.02986296802524229, 0.012259844509753592,
            0.012338783661043004, 0.011534383792643545, 0.030505284447352746, 0.004886689383534703,
            0.004357898672331982, 0.005428457355999516, 0.005817028918166772, 0.00976790454953955,
            0.005881481394028688, 0.010246379983008607, 0.0019734244893533683
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subpostandtelecommunications(x):
    """
    share inventories SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0011057850769560624, 0.0009488551439241703, 0.0012396105428931219,
            0.0007444440848184912, 0.0014685834763999532, 0.0020441730699772512,
            0.0032325119339692335, -0.005068250485275013, 0.00043111420964910284,
            0.0009169560942092407, 0.000570352331915007, 0.0007087983726690691,
            0.0005448397305536379, 0.0009565343787786686, 0.000645581668543594
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subfinancialintermedation(x):
    """
    share inventories SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.00018628940902411898, 9.11231272700768e-05, 7.542788209146378e-05,
            -1.4400616429060148e-05, 8.229749543927034e-05, 0.0003937912440804006,
            0.0009093943635503735, -0.007524568226264509, -0.004274426163508445,
            8.49992603244964e-05, 9.833482351623146e-05, 0.00013797126261373458,
            8.384317855566425e-05, 0.0006442841441413284, 0.0012264529642908699
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subrealestateactivities(x):
    """
    share inventories SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3.55572466866178e-05, 2.91907526576806e-05, 1.9879281529659025e-05,
            1.4105622407877741e-05, 8.646807700213472e-05, 2.161401447580686e-05,
            7.110088024127258e-06, 7.479695881646118e-06, 2.0296672084406522e-06,
            9.156903678863279e-06, 2.6108742903649806e-05, 3.0340352036303263e-05,
            2.039824756142603e-05, 7.322588921311033e-05, -2.8778440691593814e-05
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subrentingodmeqandotherbusinessactivities(x):
    """
    share inventories SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0011964287316454426, 0.000831602174777396, 0.0033847710947051517,
            0.0005761788915349635, 0.0031079738205755606, 0.0021995063936603304,
            0.0022211587023244217, 0.000463289136776625, 0.003827049793333797,
            0.0032723048332935715, 0.00282669028048174, 0.0033238445248807004,
            0.0019278144414444816, 0.01864576873493577, 0.0074378011898418245
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subpublicadminanddefencecompulsorysocialsecurity(x):
    """
    share inventories SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            -0.006668793979221436, -0.006890673086698083, -0.005679982938310693,
            -0.0072323360613766275, -0.006114828678354021, -0.005356116532156387,
            -0.0038007332944622442, -0.005674782512487631, -0.004853486698943161,
            -0.004676214705836499, -0.004558338326465761, -0.004537527544218771,
            -0.0053082216326086635, -0.005544836892740801, -0.003922579936947399
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subeducation(x):
    """
    share inventories SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.144403809132422e-05, 1.8730226585082882e-05, 1.3231663407411775e-05,
            4.799534989921149e-06, 3.0446684471332545e-05, 1.098827169556429e-05,
            2.5728856619887022e-05, 1.3380636827243842e-05, 4.505158402132235e-05,
            2.248247898388548e-06, 8.725195270895369e-05, -2.615975302391741e-05,
            8.26970101974835e-06, 1.5833949800231638e-05, 4.7541740478130375e-05
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subhealthandsocialwork(x):
    """
    share inventories SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            5.356250958867206e-06, 8.428876000649878e-07, 3.87479673142649e-06,
            4.426989397210959e-06, 1.484503418608409e-05, 2.0269244774470538e-05,
            1.2951043297292982e-05, -3.1727632790426594e-05, 1.9634547384056006e-05,
            1.5997263037852878e-05, 1.0177684924305919e-05, 1.965940194824921e-05,
            1.4489386545164095e-05, 2.3448960211648786e-05, -9.502931854099198e-06
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subothercommunitysocialandpersonaservices(x):
    """
    share inventories SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            -0.0090127726393938, -0.010629458350807685, -0.008848671736319593,
            -0.013448818323945873, -0.008946039163755016, -0.004793431904417115,
            -0.0016249396890407504, -0.0022020738024543162, -0.0018956770408739025,
            -0.001785715673228679, -0.0018879388774512668, -0.0017953987374448959,
            -0.0024802205711734492, -0.003448648466737081, -0.0020721433092937003
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_inventories_subprivatehouseholdswithemployedpersons(x):
    """
    share inventories SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            7.853351243135922e-05, 9.814582614953874e-05, 9.588248461668407e-06,
            -7.821361593256881e-05, -2.411516128394616e-05, -3.631893686050125e-06,
            -5.08923087015618e-05, -0.00022539143199814182, -0.0002681430771119126,
            6.398985647852799e-05, 4.7633349046714305e-05, 0.0015237584902465407,
            0.000990345581418567, 0.0009393292444326495, 0.0008607609964860263
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subagriculturehuntingforestryandfishing(x):
    """
    share consum goverments next step SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0067709613778314465, 0.006395238231716023, 0.007562221479560522,
            0.006909511984020509, 0.007982988184916011, 0.008446717428753593, 0.009326405167881661,
            0.009029494951501007, 0.009305637069275475, 0.009311394291421764, 0.009911989953154088,
            0.010233756778334888, 0.008994169837883667, 0.00869869421963916
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subminingandquarrying(x):
    """
    share consum goverments next step SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.00436164392290795, 0.003995603763685553, 0.004066047799643348, 0.006393419006564868,
            0.006003153311605073, 0.0042157315738525596, 0.005643824953173819,
            0.005273239069785034, 0.005338694073945094, 0.005231106875192582, 0.004151188915356074,
            0.004717994395060901, 0.00450208701463089, 0.004934708656353205
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subfoodbeveragesandtobacco(x):
    """
    share consum goverments next step SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0038220671963571573, 0.002903073070627261, 0.002936792709517417,
            0.003142168948714033, 0.0031340511473669625, 0.0030303441468097964,
            0.003102005746783019, 0.003076968042145073, 0.002915823853670982, 0.002931906554520569,
            0.002970467773012117, 0.003045614006493215, 0.002961515281763346, 0.002885066309883363
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subtextilesandtextileproducts(x):
    """
    share consum goverments next step SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.003551578898993538, 0.00295957655229884, 0.0026490453856551233,
            0.0028527421747088517, 0.0025817118313251836, 0.0027787410874748823,
            0.0028558425631741735, 0.003004220437100366, 0.0030660841863479906,
            0.0034604581246728532, 0.00402674114226547, 0.004420561138285466, 0.004407476902887508,
            0.005253312524302093
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subleatherleatherandfootwear(x):
    """
    share consum goverments next step SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0015559542279136721, 0.0012399353066103794, 0.0013267840090444238,
            0.00133344367825553, 0.0012313162901100574, 0.0011201067862366928,
            0.0008200600923468703, 0.0007351843303268899, 0.0007872038350995223,
            0.0008524970828855757, 0.000980302019330771, 0.0012534804090331476,
            0.0013308059227996672, 0.001514645887241861
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subwoodandproductsofwooodandcork(x):
    """
    share consum goverments next step SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.009005435491861913, 0.00784410468158404, 0.010385380453511185, 0.005983313227598484,
            0.0029442159037194747, 0.003191210291931943, 0.0038803930252921484,
            0.0033353083568353945, 0.0031038702125085094, 0.002934685814185623,
            0.0032009994283761284, 0.0031885473327917984, 0.003374415685759153,
            0.003641151621250122
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subpulppaperprintingandpublishing(x):
    """
    share consum goverments next step SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.012152022471383788, 0.010944306361792914, 0.013144143333725558, 0.01481962044356459,
            0.014280346332352903, 0.014584228557471577, 0.01813378209427802, 0.01800422812064625,
            0.017428493050279235, 0.01703528662283678, 0.018253358153044425, 0.01977514458023117,
            0.016156351225622194, 0.016038861285036368
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subcokerefinedpetroleumandnuclearfuel(x):
    """
    share consum goverments next step SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.014073012258205416, 0.014138264180255291, 0.01638462176078289, 0.012559974344452331,
            0.009745231174255206, 0.008722834180249866, 0.009133945504476285, 0.007599833303872037,
            0.006750850405801407, 0.005867347651754397, 0.006685369003406569, 0.007433579067682456,
            0.006607524613390546, 0.007784103038001168
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subchemicalsandchemicalproducts(x):
    """
    share consum goverments next step SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.1019113905016282, 0.0973261077437338, 0.10451610930994258, 0.1038905105421696,
            0.10684729730821506, 0.11441273827060026, 0.11710461296677933, 0.11802421308716918,
            0.1131680314380918, 0.11811035287289917, 0.1159991266683548, 0.12291936960693232,
            0.13104795855975498, 0.13239560556296268
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subrubberandplastics(x):
    """
    share consum goverments next step SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007733117166691025, 0.007382076353556275, 0.008098756846008068, 0.008121760071879654,
            0.006928724883784163, 0.006611896916921303, 0.006412599503225718, 0.00627859918453306,
            0.006463852038577615, 0.006987327120633945, 0.007095522534506072, 0.007271046894131369,
            0.006823697974194504, 0.0071450764967817045
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subothernonmetalicmineral(x):
    """
    share consum goverments next step SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.005601972248454674, 0.004984833707272183, 0.0050045677027149025,
            0.005220994619862051, 0.004906031702269911, 0.0046695726368900506,
            0.004765867407184146, 0.004575710980134348, 0.004925408480648355, 0.005451801457053173,
            0.005476256518325244, 0.005940164753101489, 0.006012657637842877, 0.006158849250484725
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subbasicmetalsandfabricatedmetal(x):
    """
    share consum goverments next step SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.003695607624002559, 0.003264025079857985, 0.003456144989750174, 0.003909675072085389,
            0.003567486126274761, 0.004087484008773003, 0.004580826890509096, 0.003954657469767477,
            0.003832560933406138, 0.003872948283557392, 0.0037354425586724055,
            0.004369197638847987, 0.004504712881791057, 0.005212766425513586
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_submachinerynec(x):
    """
    share consum goverments next step SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0036617822542566983, 0.003605389168536691, 0.003975012421065428,
            0.003576070848771421, 0.0024004574567129543, 0.002133422389799059,
            0.0020732528087047206, 0.0015195162676146234, 0.0016070917766987168,
            0.0017110936542217888, 0.0017042538525656047, 0.0018089274613794315,
            0.00179493990200113, 0.0020036796744404423
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subelectricalandopticalequipment(x):
    """
    share consum goverments next step SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.006689280053562139, 0.006260917348372466, 0.006383402577586877, 0.007626339783897834,
            0.008952172345135323, 0.009938402470472538, 0.010584245313257168, 0.010485426607225824,
            0.009528021492371893, 0.009159007241327075, 0.009390892943976928, 0.009368214117085202,
            0.008491380955383702, 0.010526372953268005
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subtransportequipment(x):
    """
    share consum goverments next step SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0030134399979940317, 0.0030013782829294688, 0.003324582114787668,
            0.0033937364928424093, 0.004179287165548907, 0.004060256365656964,
            0.004599388270976094, 0.004521827080112946, 0.0039045057195699682,
            0.003377303061102904, 0.0032156831100202668, 0.002999720037553041,
            0.003025284132570713, 0.0031225730516014757
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_submanufacturingnecrecycling(x):
    """
    share consum goverments next step SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.02145589854505419, 0.0214355818274276, 0.02514402773966533, 0.01774772016365786,
            0.013419020602064787, 0.012203537170182726, 0.011759010701610579, 0.010195116719573927,
            0.010562251657894164, 0.010833741123216334, 0.01030096052750391, 0.010593246762629493,
            0.011621782561596393, 0.012719313855909485
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subelectricitygasandwatersupply(x):
    """
    share consum goverments next step SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.014062257866705289, 0.013761686141498856, 0.014297671627160625, 0.012947249209290532,
            0.013116308794034108, 0.012724684366983808, 0.01399302698870909, 0.013964725368533456,
            0.014932092232165415, 0.016923049048231383, 0.020237318698904688, 0.020307088408552488,
            0.018222111231047673, 0.019718462441084224
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subconstruction(x):
    """
    share consum goverments next step SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0037926289235951103, 0.003584887454493617, 0.004030136291386803,
            0.0043782604871680005, 0.004424182707116258, 0.004527167347521452,
            0.004951394442164713, 0.0046428441481454205, 0.003398133507831463,
            0.0023546145866514584, 0.001689633412741827, 0.0016539914075433919,
            0.0017930949205848523, 0.0020141898925777795
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    share consum goverments next step SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.005403324383601953, 0.005555193258671485, 0.005806921261116362, 0.0055187734847195,
            0.0060804999283362755, 0.006576770190355463, 0.006599236892085225,
            0.006951525047583767, 0.006930155684757487, 0.0071569467110036466,
            0.006965090647827174, 0.006885708406717167, 0.006969574494336234, 0.008003455543375147
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
        x):
    """
    share consum goverments next step SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.01006648267750522, 0.009307515695653029, 0.009444958125729505, 0.009364679486856759,
            0.009765600552568655, 0.010207317570209486, 0.010771962374229452, 0.01067527004648461,
            0.010462899400149748, 0.010587576110538116, 0.0108347645089915, 0.01077759596108046,
            0.010732709364063026, 0.012048869135356076
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
        x):
    """
    share consum goverments next step SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.006598553542763822, 0.006508686887531563, 0.006705691997046858, 0.007025347276833513,
            0.00720794339648111, 0.007317693553382741, 0.007707318698817354, 0.007486817205849609,
            0.007496077432961913, 0.007788715871380901, 0.007790913253695477, 0.00796523132976134,
            0.008075132023729639, 0.009141731015561386
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subhotelsandrestaurants(x):
    """
    share consum goverments next step SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.009986641171961503, 0.004972707818870327, 0.005180644227287415, 0.003157808005666468,
            0.0026105687179988736, 0.0023619220258709814, 0.0022802596097973695,
            0.002236912343894969, 0.0032285975612425005, 0.004006717191046085,
            0.004771145868031531, 0.005057546903846365, 0.004275366439841901, 0.004318083939434814
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subinlandtransport(x):
    """
    share consum goverments next step SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.020988629145090622, 0.020208389033241396, 0.024190375963752966, 0.028979965613103086,
            0.0340573400355154, 0.03787454338990796, 0.038778817683728796, 0.042856040285980994,
            0.04333737779943819, 0.040877931695314014, 0.040017654428911954, 0.044561875720359245,
            0.042253292012375945, 0.04548291871340436
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subwatertransport(x):
    """
    share consum goverments next step SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007670674964602831, 0.006643243068732121, 0.0070420634278164475,
            0.005625692184953877, 0.005825483070381239, 0.005755526565640597,
            0.0059064747947359724, 0.031040299562048543, 0.0444328959360957, 0.045343068051017205,
            0.0419767170469466, 0.05198184793293863, 0.050442763300947255, 0.05576891724508192
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subairtransport(x):
    """
    share consum goverments next step SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0056558638910256965, 0.003568218719757792, 0.003452988698190144,
            0.003113214535012367, 0.002600142907132616, 0.00352760744719066, 0.0035150489272458216,
            0.008303301417705423, 0.01003440101939009, 0.007348632019004971, 0.007740245562564343,
            0.009810656999234713, 0.009168958102788652, 0.00981130620506604
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    share consum goverments next step SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.09901239662412917, 0.09351013009433082, 0.09307715377473842, 0.090464559595115,
            0.08605821054032661, 0.10652486260006906, 0.11145975472969767, 0.11357295739188283,
            0.11017723359651423, 0.10491172806150008, 0.1011831510665581, 0.09842241225860136,
            0.09838464480853919, 0.09731960940160281
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subpostandtelecommunications(x):
    """
    share consum goverments next step SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007218033564359982, 0.006671966436504588, 0.00755011709325932, 0.007798984111468217,
            0.006194892527289748, 0.005508562494627751, 0.00575293539502172, 0.005392135225406816,
            0.004821274684917958, 0.005148559804682313, 0.00553343108337951, 0.006297209998691717,
            0.006491788496943986, 0.007810889098747097
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subfinancialintermedation(x):
    """
    share consum goverments next step SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.027581273496790935, 0.025235830692599766, 0.02508840686447435, 0.024038702335690176,
            0.02474875619024354, 0.031203030082480394, 0.03890041017856948, 0.04298777082414324,
            0.04647391196264396, 0.04820648818054139, 0.054020887398685435, 0.05660365990915811,
            0.057191672612287296, 0.05349723889956909
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subrealestateactivities(x):
    """
    share consum goverments next step SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.00480747520979442, 0.004356879732159147, 0.005051056791957753, 0.0042817328034103035,
            0.003082759572882315, 0.002882345202542986, 0.0028984962506404615,
            0.0027998697728837665, 0.0029336909071597957, 0.0029228584018284724,
            0.0032377155699126986, 0.0032471376757725566, 0.003313038453281683,
            0.0033056974047413435
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subrentingodmeqandotherbusinessactivities(x):
    """
    share consum goverments next step SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.12766213970374193, 0.128477992110974, 0.13977631646313407, 0.14717767633342763,
            0.15478196524040605, 0.16957094795516728, 0.17978959576830966, 0.1755571954925755,
            0.16458717780184334, 0.1432232593236837, 0.13416821175274135, 0.14146388986144026,
            0.13442398718655846, 0.13699391236524086
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subpublicadminanddefencecompulsorysocialsecurity(x):
    """
    share consum goverments next step SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.9019740353263537, 0.9055209153005452, 0.9070353979129336, 0.9101561542351737,
            0.9071215692757418, 0.9039291693459441, 0.9009660123926903, 0.8991346577949707,
            0.8948821888313753, 0.8935084759144025, 0.8922590472793164, 0.8935680286075393,
            0.895349946046101, 0.892671897122667
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subeducation(x):
    """
    share consum goverments next step SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.7040856404423688, 0.6993589354235593, 0.698581612534672, 0.6987685915889159,
            0.6958775202480544, 0.6893004419028962, 0.6836379033975156, 0.6765593042142081,
            0.6662693916312913, 0.6630464757694154, 0.660201578395071, 0.6615962513449365,
            0.659828413873094, 0.6593581001080975
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subhealthandsocialwork(x):
    """
    share consum goverments next step SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.44707649924014653, 0.43785031864326934, 0.43967250874908564, 0.44259141677895925,
            0.44421483587784627, 0.440685782407788, 0.43726784569342825, 0.4372577649745116,
            0.43796753502845864, 0.4378164095136612, 0.43992755579739684, 0.4425537217710332,
            0.443198533634356, 0.4408943873395209
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subothercommunitysocialandpersonaservices(x):
    """
    share consum goverments next step SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0910574921371381, 0.0873395017587308, 0.08692860129270708, 0.08999187621790489,
            0.090740783333636, 0.08823471238711048, 0.0863032736828504, 0.08522731060173952,
            0.08453682210016202, 0.08167532299694542, 0.0807001909529624, 0.08463448174674901,
            0.0850306568023039, 0.08931954847715241
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_consum_goverments_next_step_subprivatehouseholdswithemployedpersons(x):
    """
    share consum goverments next step SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0011290416169956153, 0.0009518818302216298, 0.0009739207509484318,
            0.003226765298868626, 0.0045618940852030915, 0.006250676296558422,
            0.007856379237276827, 0.009536129903780428, 0.009469554208776931, 0.01007163264373241,
            0.010290112726275327, 0.010477541610775753, 0.009767631985404314, 0.012031510285201963
        ])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def share_consum_goverment_and_inventories():
    """
    share consum goverment and inventories



    component


    """

    ret =  xr.DataArray(
        data=[share_inventories_subagriculturehuntingforestryandfishing(
            time()) + share_consum_goverments_subagriculturehuntingforestryandfishing(time()),

        share_inventories_subminingandquarrying(
            time()) + share_consum_goverments_subminingandquarrying(time()),

        share_inventories_subfoodbeveragesandtobacco(
            time()) + share_consum_goverments_subfoodbeveragesandtobacco(time()),

        share_inventories_subtextilesandtextileproducts(
            time()) + share_consum_goverments_subtextilesandtextileproducts(time()),

        share_inventories_subleatherleatherandfootwear(
            time()) + share_consum_goverments_subleatherleatherandfootwear(time()),

        share_inventories_subwoodandproductsofwooodandcork(time()) +
        share_consum_goverments_subwoodandproductsofwooodandcork(time()),

        share_inventories_subpulppaperprintingandpublishing(time()) +
        share_consum_goverments_subpulppaperprintingandpublishing(time()),

        share_inventories_subcokerefinedpetroleumandnuclearfuel(time()) +
        share_consum_goverments_subcokerefinedpetroleumandnuclearfuel(time()),

        share_inventories_subchemicalsandchemicalproducts(time()) +
        share_consum_goverments_subchemicalsandchemicalproducts(time()),

        share_inventories_subrubberandplastics(
            time()) + share_consum_goverments_subrubberandplastics(time()),

        share_inventories_subothernonmetalicmineral(
            time()) + share_consum_goverments_subothernonmetalicmineral(time()),

        share_inventories_subbasicmetalsandfabricatedmetal(
            time()) + share_consum_goverments_subbasicmetalsandfabricatedmetal(time()),

        share_inventories_submachinerynec(time()) + share_consum_goverments_submachinerynec(
            time()),

        share_inventories_subelectricalandopticalequipment(
            time()) + share_consum_goverments_subelectricalandopticalequipment(time()),

        share_inventories_subtransportequipment(
            time()) + share_consum_goverments_subtransportequipment(time()),

        share_inventories_submanufacturingnecrecycling(
            time()) + share_consum_goverments_submanufacturingnecrecycling(time()),

        share_inventories_subelectricitygasandwatersupply(
            time()) + share_consum_goverments_subelectricitygasandwatersupply(time()),

        share_inventories_subconstruction(time()) + share_consum_goverments_subconstruction(
            time()),

        share_inventories_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()) +
        share_consum_goverments_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),

        share_inventories_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()) +
        share_consum_goverments_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),

        share_inventories_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()) +
        share_consum_goverments_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),

        share_inventories_subhotelsandrestaurants(
            time()) + share_consum_goverments_subhotelsandrestaurants(time()),

        share_inventories_subinlandtransport(time()) + share_consum_goverments_subinlandtransport(
            time()),

        share_inventories_subwatertransport(time()) + share_consum_goverments_subwatertransport(time()),

        share_inventories_subairtransport(time()) + share_consum_goverments_subairtransport(
            time()),

        share_inventories_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()) +
        share_consum_goverments_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),

        share_inventories_subpostandtelecommunications(
            time()) + share_consum_goverments_subpostandtelecommunications(time()),

        share_inventories_subfinancialintermedation(
            time()) + share_consum_goverments_subfinancialintermedation(time()),

        share_inventories_subrealestateactivities(
            time()) + share_consum_goverments_subrealestateactivities(time()),

        share_inventories_subrentingodmeqandotherbusinessactivities(
            time()) + share_consum_goverments_subrentingodmeqandotherbusinessactivities(time()),

        share_inventories_subpublicadminanddefencecompulsorysocialsecurity(time()) +
        share_consum_goverments_subpublicadminanddefencecompulsorysocialsecurity(time()),

        share_inventories_subeducation(time()) + share_consum_goverments_subeducation(time()),

        share_inventories_subhealthandsocialwork(
            time()) + share_consum_goverments_subhealthandsocialwork(time()),

        share_inventories_subothercommunitysocialandpersonaservices(
            time()) + share_consum_goverments_subothercommunitysocialandpersonaservices(time()),

        share_inventories_subprivatehouseholdswithemployedpersons(
            time()) + share_consum_goverments_subprivatehouseholdswithemployedpersons(time())],
        coords={'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def share_consum_goverments_and_inventories_next_step():
    """
    share consum goverments and inventories next step



    component


    """

    ret =  xr.DataArray(
        data=[share_consum_goverments_next_step_subagriculturehuntingforestryandfishing(
            time()) + share_inventories_next_step_subagriculturehuntingforestryandfishing(time()),

        share_consum_goverments_next_step_subminingandquarrying(
            time()) + share_inventories_next_step_subminingandquarrying(time()),

        share_consum_goverments_next_step_subfoodbeveragesandtobacco(
            time()) + share_inventories_next_step_subfoodbeveragesandtobacco(time()),

        share_consum_goverments_next_step_subtextilesandtextileproducts(
            time()) + share_inventories_next_step_subtextilesandtextileproducts(time()),

        share_consum_goverments_next_step_subleatherleatherandfootwear(
            time()) + share_inventories_next_step_subleatherleatherandfootwear(time()),

        share_consum_goverments_next_step_subwoodandproductsofwooodandcork(time()) +
        share_inventories_next_step_subwoodandproductsofwooodandcork(time()),

        share_consum_goverments_next_step_subpulppaperprintingandpublishing(time()) +
        share_inventories_next_step_subpulppaperprintingandpublishing(time()),

        share_consum_goverments_next_step_subcokerefinedpetroleumandnuclearfuel(time()) +
        share_inventories_next_step_subcokerefinedpetroleumandnuclearfuel(time()),

        share_consum_goverments_next_step_subchemicalsandchemicalproducts(time()) +
        share_inventories_next_step_subchemicalsandchemicalproducts(time()),

        share_consum_goverments_next_step_subrubberandplastics(
            time()) + share_inventories_next_step_subrubberandplastics(time()),

        share_consum_goverments_next_step_subothernonmetalicmineral(
            time()) + share_inventories_next_step_subothernonmetalicmineral(time()),

        share_consum_goverments_next_step_subbasicmetalsandfabricatedmetal(
            time()) + share_inventories_next_step_subbasicmetalsandfabricatedmetal(time()),

        share_consum_goverments_next_step_submachinerynec(
            time()) + share_inventories_next_step_submachinerynec(time()),

        share_consum_goverments_next_step_subelectricalandopticalequipment(
            time()) + share_inventories_next_step_subelectricalandopticalequipment(time()),

        share_consum_goverments_next_step_subtransportequipment(
            time()) + share_inventories_next_step_subtransportequipment(time()),

        share_consum_goverments_next_step_submanufacturingnecrecycling(
            time()) + share_inventories_next_step_submanufacturingnecrecycling(time()),

        share_consum_goverments_next_step_subelectricitygasandwatersupply(
            time()) + share_inventories_next_step_subelectricitygasandwatersupply(time()),

        share_consum_goverments_next_step_subconstruction(
            time()) + share_inventories_next_step_subconstruction(time()),

        share_consum_goverments_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()) +
        share_inventories_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),

        share_consum_goverments_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()) +
        share_inventories_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),

        share_consum_goverments_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()) +
        share_inventories_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),

        share_consum_goverments_next_step_subhotelsandrestaurants(
            time()) + share_inventories_next_step_subhotelsandrestaurants(time()),

        share_consum_goverments_next_step_subinlandtransport(
            time()) + share_inventories_next_step_subinlandtransport(time()),

        share_consum_goverments_next_step_subwatertransport(
            time()) + share_inventories_next_step_subwatertransport(time()),

        share_consum_goverments_next_step_subairtransport(
            time()) + share_inventories_next_step_subairtransport(time()),

        share_consum_goverments_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()) +
        share_inventories_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),

        share_consum_goverments_next_step_subpostandtelecommunications(
            time()) + share_inventories_next_step_subpostandtelecommunications(time()),

        share_consum_goverments_next_step_subfinancialintermedation(
            time()) + share_inventories_next_step_subfinancialintermedation(time()),

        share_consum_goverments_next_step_subrealestateactivities(
            time()) + share_inventories_next_step_subrealestateactivities(time()),

        share_consum_goverments_next_step_subrentingodmeqandotherbusinessactivities(time()) +
        share_inventories_next_step_subrentingodmeqandotherbusinessactivities(time()),

        share_consum_goverments_next_step_subpublicadminanddefencecompulsorysocialsecurity(time())
        + share_inventories_next_step_subpublicadminanddefencecompulsorysocialsecurity(time()),

        share_consum_goverments_next_step_subeducation(
            time()) + share_inventories_next_step_subeducation(time()),

        share_consum_goverments_next_step_subhealthandsocialwork(
            time()) + share_inventories_next_step_subhealthandsocialwork(time()),

        share_consum_goverments_next_step_subothercommunitysocialandpersonaservices(time()) +
        share_inventories_next_step_subothercommunitysocialandpersonaservices(time()),

        share_consum_goverments_next_step_subprivatehouseholdswithemployedpersons(
            time()) + share_inventories_next_step_subprivatehouseholdswithemployedpersons(time())],
        coords={'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def capital_share():
    """
    capital share



    component


    """

    ret =  1 - labour_share()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def historic_growth_capital_share():
    """
    historic growth capital share



    component


    """

    ret =  -growth_labour_share() * labour_share() / (1 - labour_share())
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subagriculturehuntingforestryandfishing(x):
    """
    Historic GFCF SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            35732.832, 37614.691, 40803.641, 41916.227, 38520.117, 38440.754, 36423.711, 38511.609,
            37743.941, 40307.848, 42430.488, 40835.578, 39981.016, 41155.035, 41396.871
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subminingandquarrying(x):
    """
    Historic GFCF SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            49168.223, 50558.738, 59227.895, 57727.891, 57437.492, 57307.668, 65567.406, 67555.18,
            59670.992, 70731.68, 87666.406, 107134.18, 120398.63, 130683.09, 87922.734
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subfoodbeveragesandtobacco(x):
    """
    Historic GFCF SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3153.3279, 3405.948, 3856.0354, 3880.1582, 3853.2202, 3988.8564, 3816.6501, 3554.2256,
            3559.5305, 3899.083, 4242.4302, 4362.2729, 4699.4136, 4752.7988, 4192.0625
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subtextilesandtextileproducts(x):
    """
    Historic GFCF SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            9805.4414, 10735.85, 11137.633, 10800.096, 10876.121, 11490.052, 11577.753, 12166.8,
            11730.841, 11857.032, 13205.372, 12958.692, 11822.783, 12343.811, 10536.79
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subleatherleatherandfootwear(x):
    """
    Historic GFCF SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            654.27881, 757.39868, 813.76819, 817.67688, 752.20032, 643.75079, 576.78583, 519.88055,
            542.40131, 604.32587, 622.45618, 615.94867, 678.13239, 644.13391, 604.7887
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subwoodandproductsofwooodandcork(x):
    """
    Historic GFCF SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            26489.537, 26581.762, 28397.006, 27623.027, 26920.006, 25842.48, 23475.318, 21862.025,
            20325.885, 18978.408, 20301.361, 20253.908, 20320.295, 18429.826, 15459.762
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subpulppaperprintingandpublishing(x):
    """
    Historic GFCF SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            36798.738, 37032.848, 39961.035, 41669.242, 46405.641, 49903.871, 51752.203, 50975.605,
            50689.711, 54906.703, 56410.535, 57824.684, 59567.02, 58348.961, 52958.492
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subcokerefinedpetroleumandnuclearfuel(x):
    """
    Historic GFCF SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1328.0853, 1227.5522, 1353.3188, 1539.4121, 1476.5179, 1365.4154, 1631.8383, 1701.6514,
            1283.2863, 1285.8644, 1511.526, 1526.3302, 1572.2338, 1773.6797, 1679.7283
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subchemicalsandchemicalproducts(x):
    """
    Historic GFCF SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            10869.645, 12038.465, 13229.726, 13637.507, 12738.396, 11845.654, 11435.997, 10917.669,
            11387.182, 10602.439, 11093.072, 11520.979, 14185.478, 14120.531, 12864.762
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subrubberandplastics(x):
    """
    Historic GFCF SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            12706.039, 14192.511, 14736.633, 15018.491, 15287.824, 14431.255, 14019.702, 14085.24,
            13984.364, 14517.224, 15653.112, 16725.594, 18017.344, 16867.807, 15186.176
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subothernonmetalicmineral(x):
    """
    Historic GFCF SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            5833.2832, 6141.4766, 6658.7974, 6528.9243, 6939.7749, 6940.7358, 6785.4204, 6367.7344,
            6292.3218, 6195.3579, 6629.5493, 6526.0938, 7284.7769, 6779.4404, 5801.4595
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subbasicmetalsandfabricatedmetal(x):
    """
    Historic GFCF SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            151450.72, 168036.17, 175682.53, 172156.44, 176539.11, 174882.11, 172961.69, 172230.45,
            172920.44, 163433.64, 171458.94, 180851.8, 191195.25, 189079.64, 169590.94
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_submachinerynec(x):
    """
    Historic GFCF SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            591070.69, 624630.06, 650666.31, 661738.56, 649393.38, 705485.75, 686770.25, 676821.63,
            711485.5, 772953.69, 844631.38, 901317.69, 980461.25, 1010400.5, 851105.94
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subelectricalandopticalequipment(x):
    """
    Historic GFCF SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            592198.44, 655007.69, 741965.0, 772863.31, 844308.38, 974987.0, 968576.31, 909083.63,
            945232.75, 1011340.2, 1107939.9, 1204886.8, 1314391.4, 1354865.9, 1251572.4
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subtransportequipment(x):
    """
    Historic GFCF SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            525742.88, 560220.13, 599235.56, 630740.38, 681251.81, 683685.88, 656420.25, 667179.38,
            713015.38, 768573.88, 828957.69, 900477.81, 995201.56, 985637.94, 849517.56
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_submanufacturingnecrecycling(x):
    """
    Historic GFCF SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            85966.852, 93761.969, 99366.813, 103879.14, 107439.22, 111866.84, 110534.02, 111077.36,
            113451.95, 118669.55, 131024.1, 131647.16, 136376.63, 142813.61, 132990.0
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subelectricitygasandwatersupply(x):
    """
    Historic GFCF SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            9163.3457, 9879.9082, 9898.4268, 10042.622, 10664.927, 11776.001, 12365.013, 12512.324,
            13303.008, 13730.786, 16534.152, 16654.773, 17834.195, 18707.898, 16229.776
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subconstruction(x):
    """
    Historic GFCF SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3315735.5, 3380354.5, 3427133.5, 3469614.0, 3552832.5, 3576725.5, 3590266.5, 3589436.8,
            3693404.0, 3835621.0, 4009451.3, 4215311.0, 4373414.0, 4370240.5, 4246355.0
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(x):
    """
    Historic GFCF SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            20114.217, 22476.516, 22885.188, 25268.574, 25426.08, 26552.805, 26028.451, 25794.117,
            25518.584, 27205.137, 29690.898, 30987.475, 33514.207, 33058.988, 27617.543
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(x):
    """
    Historic GFCF SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            272784.38, 295626.84, 311371.94, 327195.78, 341950.22, 355568.5, 355673.53, 358418.03,
            358038.75, 385270.97, 408631.41, 413487.31, 427566.13, 435917.31, 387395.69
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(x):
    """
    Historic GFCF SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            134780.13, 145710.55, 152762.44, 161182.28, 164131.47, 166019.34, 166866.67, 170615.55,
            174059.25, 190358.11, 216744.02, 224155.45, 228615.17, 227170.06, 195714.02
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subhotelsandrestaurants(x):
    """
    Historic GFCF SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            6338.0215, 6149.7041, 5875.1206, 4984.3564, 5163.3916, 5408.5967, 5032.4741, 4891.4541,
            4406.771, 4312.0898, 4053.8296, 4194.439, 4313.0947, 4425.0718, 4036.8662
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subinlandtransport(x):
    """
    Historic GFCF SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            49677.738, 50398.684, 53456.406, 54578.48, 58112.934, 52657.016, 50783.926, 52491.391,
            51862.168, 57095.516, 64329.469, 70106.383, 72946.836, 72193.539, 61845.871
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subwatertransport(x):
    """
    Historic GFCF SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2462.5908, 2531.9485, 2810.0728, 4047.5605, 4551.7358, 4567.0215, 4585.7559, 4775.0659,
            4308.2896, 4417.4858, 4655.2271, 4511.1509, 4315.3672, 4326.1084, 4421.8511
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subairtransport(x):
    """
    Historic GFCF SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            4389.7974, 4811.0552, 5240.1558, 5297.9785, 5140.0283, 4839.623, 4492.3311, 4079.4124,
            3658.7437, 4157.6909, 4297.7334, 4729.563, 5259.9507, 5141.3306, 4125.6865
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(x):
    """
    Historic GFCF SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            7163.0146, 6735.1499, 6212.1035, 6414.583, 6624.3755, 6811.5542, 6294.2832, 6947.418,
            7070.0679, 7531.814, 8541.0635, 9228.2764, 9921.543, 9895.9375, 8696.0947
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subpostandtelecommunications(x):
    """
    Historic GFCF SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            9400.0742, 10358.336, 12133.059, 13841.907, 17040.002, 20493.102, 22652.508, 22411.086,
            24883.102, 28778.445, 31190.512, 33387.305, 36844.961, 38329.137, 36114.047
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subfinancialintermedation(x):
    """
    Historic GFCF SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3753.4692, 4015.5376, 4656.2686, 4870.4028, 5495.1387, 5409.3198, 5890.9595, 6514.6733,
            6840.2881, 8639.1191, 9947.8135, 11000.363, 12140.073, 11535.349, 9802.5811
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subrealestateactivities(x):
    """
    Historic GFCF SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            105648.82, 112505.58, 121700.46, 130935.03, 138473.86, 140215.89, 148289.28, 163227.39,
            184642.38, 208041.58, 231293.97, 233717.48, 227724.55, 208201.86, 192484.63
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subrentingodmeqandotherbusinessactivities(x):
    """
    Historic GFCF SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            322190.16, 361983.34, 384212.78, 429207.44, 473706.81, 517232.19, 537414.25, 532837.94,
            542599.75, 577060.56, 617376.75, 667582.25, 703895.94, 725018.06, 672112.25
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subpublicadminanddefencecompulsorysocialsecurity(x):
    """
    Historic GFCF SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            16466.713, 8284.8584, 8695.0361, 8756.8301, 9585.9248, 9790.8984, 9414.084, 9308.3301,
            9819.3438, 10995.426, 12019.087, 12665.439, 13487.202, 12984.448, 10894.959
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subeducation(x):
    """
    Historic GFCF SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            877.41577, 1190.7976, 1051.7266, 1118.6437, 1211.5232, 1361.8879, 1197.3674, 1297.8099,
            1128.9475, 1177.98, 1207.3063, 1207.5222, 1313.7346, 1230.2579, 1138.8306
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subhealthandsocialwork(x):
    """
    Historic GFCF SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2020.5743, 2411.981, 2387.1941, 2317.7742, 2457.7114, 2582.917, 2985.6318, 3204.249,
            3562.0264, 4106.2969, 4548.6128, 4686.6963, 4528.8252, 3930.343, 3988.499
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subothercommunitysocialandpersonaservices(x):
    """
    Historic GFCF SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            19066.77, 23366.191, 23901.691, 25248.57, 25473.307, 27629.49, 27269.873, 26582.359,
            26523.111, 28863.715, 30638.781, 32234.186, 33909.887, 33997.836, 28864.693
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_gfcf_subprivatehouseholdswithemployedpersons(x):
    """
    Historic GFCF SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            133.84763, 131.45456, 128.82707, 147.96854, 122.28409, 104.01704, 80.335487, 71.17453,
            73.229172, 78.620636, 90.77932, 88.329956, 97.497345, 95.166161, 68.236107
        ])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def demand_by_sector():
    """
    Demand by sector



    component


    """

    ret =  demand_by_sector_fd()
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subagriculturehuntingforestryandfishing(x):
    """
    Historic HD SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            748525.1845, 763287.0495, 770336.1225, 811036.2165, 829790.0459, 822865.7691,
            825028.4316, 854006.3347, 851658.472, 859315.9644, 869780.6132, 881620.6251,
            885511.0359, 934301.625, 964691.9819
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subminingandquarrying(x):
    """
    Historic HD SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            41789.6873, 40774.55363, 35760.8687, 32802.32296, 37641.00965, 48407.97747,
            46807.61795, 51252.85249, 49636.55263, 49512.87455, 46724.86493, 39361.70374,
            34217.52762, 39115.98394, 34147.69516
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subfoodbeveragesandtobacco(x):
    """
    Historic HD SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1723750.214, 1738706.496, 1774037.996, 1848995.869, 1895713.275, 1931662.187,
            1942552.633, 1958247.896, 1986716.788, 2022834.744, 2089671.199, 2153060.415,
            2238091.994, 2246958.729, 2245656.855
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subtextilesandtextileproducts(x):
    """
    Historic HD SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            470469.2814, 471356.0045, 476964.8114, 493102.4947, 493611.554, 515098.03, 517850.0139,
            531176.2526, 532061.3635, 541778.7234, 563405.2326, 592743.206, 625432.3504,
            627023.5263, 611100.8105
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subleatherleatherandfootwear(x):
    """
    Historic HD SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            97413.51078, 99005.48025, 97752.3784, 99637.54181, 98274.45556, 103223.4987,
            104490.9256, 105493.3023, 106139.4451, 108526.3023, 109548.0365, 114884.4565,
            121097.4926, 124995.4693, 119187.6975
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subwoodandproductsofwooodandcork(x):
    """
    Historic HD SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            27311.38121, 26691.7558, 29615.10196, 29971.29581, 28692.07457, 29540.09729,
            29526.83953, 29878.51743, 30112.75414, 30240.47199, 30309.37202, 31382.94995,
            33069.44101, 30997.81253, 29891.58369
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subpulppaperprintingandpublishing(x):
    """
    Historic HD SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            218308.7852, 224848.8639, 233135.3592, 239779.917, 245020.8308, 248600.3498,
            245907.4753, 251845.6153, 252932.4609, 260211.7944, 262604.975, 270462.3618,
            277128.7161, 278249.174, 263144.934
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subcokerefinedpetroleumandnuclearfuel(x):
    """
    Historic HD SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            217859.5983, 227030.6499, 231061.0208, 233139.8992, 242777.5242, 247730.3037,
            255008.8399, 255881.6315, 249517.5902, 256059.9905, 264654.8552, 281863.4531,
            283885.0762, 305058.1479, 314295.3504
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subchemicalsandchemicalproducts(x):
    """
    Historic HD SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            329088.216, 338997.7663, 357939.9716, 377533.0605, 391534.1448, 400362.1034,
            408700.5194, 439916.4235, 445918.3201, 458362.0534, 453945.8561, 464577.3553,
            485124.7181, 485202.7031, 493254.1673
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subrubberandplastics(x):
    """
    Historic HD SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            91494.36847, 92997.61224, 98746.44195, 101253.0568, 106386.6536, 109284.1565,
            108616.9065, 108496.0829, 109817.8004, 115248.1582, 119022.5462, 122671.6583,
            127897.3432, 130009.9874, 129860.5313
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subothernonmetalicmineral(x):
    """
    Historic HD SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            50849.6903, 52185.75254, 55570.83701, 57512.95127, 61138.98637, 66091.5223,
            66829.09701, 67078.25726, 61896.61387, 60607.83051, 57837.85562, 56383.1174,
            57059.3999, 56934.76676, 52614.06982
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subbasicmetalsandfabricatedmetal(x):
    """
    Historic HD SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            98211.18593, 101364.5069, 105638.8761, 109614.7046, 109634.716, 109513.8361,
            108781.2866, 107727.9884, 107406.8228, 108813.2145, 108643.8029, 110632.9872, 117104.5,
            117239.6397, 111348.4293
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_submachinerynec(x):
    """
    Historic HD SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            169551.7844, 165460.9408, 173073.1117, 172458.5657, 166225.228, 177635.06, 179257.497,
            180868.4452, 192695.9211, 204320.0422, 213294.6206, 222421.2828, 230417.0138,
            238684.2123, 209182.9328
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subelectricalandopticalequipment(x):
    """
    Historic HD SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            269085.0789, 292528.3037, 314568.9572, 327006.1972, 362531.3665, 403721.8382,
            421237.8362, 424194.5776, 462089.486, 500625.8901, 528593.1563, 575337.1644,
            626824.9313, 659502.0775, 638774.7252
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subtransportequipment(x):
    """
    Historic HD SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            559770.1377, 562660.0135, 610616.3602, 637624.4844, 658779.9693, 678119.7767,
            705524.229, 730358.9824, 762695.1596, 804770.2701, 822767.0034, 841264.7015,
            900674.5172, 872656.5609, 774016.3155
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_submanufacturingnecrecycling(x):
    """
    Historic HD SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            222979.9294, 228135.1029, 240692.1728, 250014.8123, 258144.9556, 265663.7602,
            267864.6516, 274989.6719, 267848.5325, 265540.4372, 266704.1404, 287544.6036,
            294441.8347, 286817.0569, 273080.4631
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subelectricitygasandwatersupply(x):
    """
    Historic HD SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            425936.7741, 442338.7712, 442427.9943, 454117.2328, 441934.9356, 459895.6379,
            470380.3946, 479556.851, 499323.8146, 513594.6788, 538766.6204, 545184.665,
            562485.9796, 606105.3211, 581811.356
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subconstruction(x):
    """
    Historic HD SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            43757.57239, 42009.8992, 44512.78978, 44853.27532, 44317.62707, 47960.07991,
            46618.12233, 48551.27075, 49838.11959, 54193.50436, 57076.19252, 61104.42813,
            64866.31738, 68880.49312, 76525.49367
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(x):
    """
    Historic HD SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            329630.4134, 339665.4583, 348852.2901, 362383.2948, 386175.4877, 394209.6837,
            389345.6728, 415315.3145, 420385.015, 437089.0681, 452805.9513, 469217.9448,
            479671.7303, 477835.4496, 441238.7686
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(x):
    """
    Historic HD SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            953189.7518, 977575.7887, 1031268.424, 1083125.505, 1134670.38, 1145796.696,
            1188875.24, 1235385.121, 1297218.855, 1376569.796, 1419687.917, 1460277.285,
            1587635.385, 1650741.741, 1654069.375
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(x):
    """
    Historic HD SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1442744.515, 1484111.231, 1511377.547, 1545708.108, 1603578.387, 1651018.305,
            1692079.184, 1735449.801, 1786053.772, 1824945.424, 1888889.851, 1946985.91, 1966830.2,
            2004091.37, 1943464.737
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subhotelsandrestaurants(x):
    """
    Historic HD SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1113431.496, 1145333.762, 1160594.202, 1204074.939, 1248189.218, 1290930.693,
            1305338.397, 1321460.001, 1346728.693, 1381971.212, 1431615.207, 1478748.724,
            1520610.363, 1533397.36, 1478111.427
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subinlandtransport(x):
    """
    Historic HD SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            512283.1868, 516283.7531, 527525.772, 548001.3754, 583355.5247, 577090.4927,
            576348.165, 589005.7397, 603398.7816, 618148.8674, 629131.2697, 640920.8658,
            656990.6185, 687030.627, 677773.459
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subwatertransport(x):
    """
    Historic HD SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            44308.95214, 47639.79652, 51042.05014, 52387.1092, 56976.62146, 61869.36149,
            64202.31502, 69243.18102, 74692.81662, 80768.97233, 82380.51438, 83700.92717,
            90448.73024, 93235.80175, 86145.54496
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subairtransport(x):
    """
    Historic HD SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            137453.8779, 148653.8674, 154276.2508, 157494.006, 161326.5298, 171955.3187,
            160991.2496, 161745.3217, 166470.3069, 180233.9222, 183284.5052, 191905.9349,
            200264.0223, 198775.0988, 182876.4735
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(x):
    """
    Historic HD SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            138435.9275, 139688.8976, 147937.0151, 154397.118, 160370.3872, 155638.3897,
            153853.6935, 152642.7157, 155699.8146, 164699.6652, 174244.2692, 180720.5432,
            193769.3659, 204150.5715, 205871.1577
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subpostandtelecommunications(x):
    """
    Historic HD SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            324538.4833, 354295.5739, 387919.9858, 424177.5174, 485852.2574, 554236.5348,
            618457.2676, 666784.4572, 710574.4463, 765818.0055, 804816.0351, 849312.5464,
            898049.3822, 951908.6976, 961153.9218
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subfinancialintermedation(x):
    """
    Historic HD SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            992982.5746, 1024466.451, 1104078.809, 1141697.612, 1212703.706, 1332023.396,
            1343319.477, 1349136.338, 1383135.657, 1487346.859, 1574452.568, 1688561.211,
            1829768.303, 1861451.265, 1853536.727
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subrealestateactivities(x):
    """
    Historic HD SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2441020.979, 2529304.337, 2567774.037, 2642829.576, 2733948.497, 2803765.837,
            2876959.249, 2915973.737, 2982427.268, 3050480.543, 3109956.804, 3175840.531,
            3240184.883, 3293070.023, 3340479.192
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subrentingodmeqandotherbusinessactivities(x):
    """
    Historic HD SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            314984.6207, 333044.0176, 341190.5309, 368007.479, 383757.4154, 400238.0943,
            396798.5427, 402480.575, 413969.7256, 433966.7039, 457741.692, 480708.902, 511773.291,
            543182.732, 556229.075
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subpublicadminanddefencecompulsorysocialsecurity(x):
    """
    Historic HD SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            376202.178, 332038.488, 320408.035, 329085.09, 328446.16, 346046.543, 362074.766,
            392771.431, 407507.525, 433867.474, 441176.919, 451861.479, 455933.663, 461450.031,
            484305.844
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subeducation(x):
    """
    Historic HD SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            340722.53, 350270.806, 362212.427, 375472.918, 388453.727, 385804.453, 399798.349,
            410892.68, 428390.985, 449502.133, 455067.481, 464749.508, 471472.367, 480018.337,
            485121.345
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subhealthandsocialwork(x):
    """
    Historic HD SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1143478.152, 1201180.793, 1245130.725, 1276058.878, 1324541.543, 1350134.311,
            1401510.594, 1460040.018, 1508964.291, 1557680.132, 1628369.246, 1668980.892,
            1717773.42, 1754141.008, 1793666.33
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subothercommunitysocialandpersonaservices(x):
    """
    Historic HD SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            959930.083, 1015666.047, 1021758.423, 1057090.263, 1100282.525, 1145322.223,
            1161611.045, 1201423.452, 1223840.595, 1257042.147, 1268420.147, 1303972.186,
            1330364.267, 1352218.5, 1344696.133
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_hd_subprivatehouseholdswithemployedpersons(x):
    """
    Historic HD SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            47919.81618, 48735.14528, 49312.02724, 51231.33634, 50583.91028, 51814.0594,
            51742.50474, 51846.85244, 52989.78551, 54193.75841, 54513.73037, 55623.74619,
            57995.17327, 59045.67614, 58243.5006
        ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xinclude_materials_for_overgridsxx():
    """
    Xinclude materials for overgridsXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xall_minerals_virginxx():
    """
    XAll minerals virginXX



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
# EROI - RES elec
@cache('step')
def materials_for_new_res_elec_per_capacity_installed():
    """
    materials for new RES elec per capacity installed



    constant


    """
    ret = xr.DataArray(
        data=np.zeros((8, 58)),
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp'], 'materials':['adhesive','aluminium','aluminium mirrors','cadmium','carbon fiber','cement','chromium','copper','diesel','dy','xelectricxelectronic componentsx','evacuation lines','fiberglass','foam glass','galium','glass','glass reinforcing plastic','gravel','indium','iron','kno3 mined','asphalt','lime','limestone','lithium','lubricant','magnesium','manganese','heavy equipment','concrete','molybdenum','nano3 mined','nano3 synthetic','neodymium','nickel','xover grid x15xxx','xover grid x5xxx','paint','lead','plastics','polypropylene','rock','rock wool','sand','silicon sand','silicon wafer modules','silver','site preparation','tin','soda ash','steel','synthetic oil','tellurium','titanium','titanium dioxide','vanadium','wires','zinc']},
        dims=['res elec','materials'])
    data = [[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],

        xmaterials_per_new_capacity_installed_x_wind_onshorex() +
        (xmaterials_per_new_res_elec_capacity_installed_x_hvdcsx() +
         xmaterials_per_new_res_elec_capacity_installed_x_material_overgrid_high_powerx()) *
        xinclude_materials_for_overgridsxx(),

        xmaterials_per_new_capacity_installed_x_wind_offshorex() +
        (xmaterials_per_new_res_elec_capacity_installed_x_hvdcsx() +
         xmaterials_per_new_res_elec_capacity_installed_x_material_overgrid_high_powerx()) *
        xinclude_materials_for_overgridsxx(),

        xmaterials_per_new_capacity_installed_x_pvx() +
        (xmaterials_per_new_res_elec_capacity_installed_x_hvdcsx() +
         xmaterials_per_new_res_elec_capacity_installed_x_material_overgrid_high_powerx()) *
        xinclude_materials_for_overgridsxx(),

        xmaterials_per_new_capacity_installed_x_cspx() +
        (xmaterials_per_new_res_elec_capacity_installed_x_hvdcsx() +
         xmaterials_per_new_res_elec_capacity_installed_x_material_overgrid_high_powerx()) *
        xinclude_materials_for_overgridsxx()]
    ret.values = list(map(lambda x: x if type(x) != xr.DataArray else xr.align(ret, x, join='left')[1], data))
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('step')
def cedtot_per_material_res_elec_var():
    """
    CEDtot per material RES elec var



    component


    """

    ret =  ced_new_cap_per_material_res_elec_var(
    ) + xced_oxm_over_lifetime_per_material_res_elec_varx()
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('step')
def cedtot_per_tw_res_elec_var():
    """
    CEDtot per TW RES elec var



    component


    """

    ret =  functions.sum(dim='materials', data=cedtot_per_tw_per_material_res_elec_var()) * 1e+006
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('step')
def cedtot_per_tw_per_material_res_elec_var():
    """
    CEDtot per TW per material RES elec var



    component


    """

    ret =  functions.zidz(cedtot_per_material_res_elec_var(),
                          res_elec_capacity_under_construction_tw())
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# ECON- Final demands by sector, households and fuel
@cache('step')
def demand_by_sector_fd():
    """
    Demand by sector FD



    component


    """


    ret =  integ_demand_by_sector_fd()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def sum_variation():
    """
    sum variation



    component


    """

    ret =  functions.sum(variation_demand_flow_fd())
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def cc_sectoral():
    """
    CC sectoral



    component


    """
    ret =  xr.DataArray(
        data=[cc_total() * share_cc_sectoral_subagriculturehuntingforestryandfishing(time()),

        cc_total() * share_cc_sectoral_subminingandquarrying(time()),

        cc_total() * share_cc_sectoral_subfoodbeveragesandtobacco(time()),

        cc_total() * share_cc_sectoral_subtextilesandtextileproducts(time()),

        cc_total() * share_cc_sectoral_subleatherleatherandfootwear(time()),

        cc_total() * share_cc_sectoral_subwoodandproductsofwooodandcork(time()),

        cc_total() * share_cc_sectoral_subpulppaperprintingandpublishing(time()),

        cc_total() * share_cc_sectoral_subcokerefinedpetroleumandnuclearfuel(time()),

        cc_total() * share_cc_sectoral_subchemicalsandchemicalproducts(time()),

        cc_total() * share_cc_sectoral_subrubberandplastics(time()),

        cc_total() * share_cc_sectoral_subothernonmetalicmineral(time()),

        cc_total() * share_cc_sectoral_subbasicmetalsandfabricatedmetal(time()),

        cc_total() * share_cc_sectoral_submachinerynec(time()),

        cc_total() * share_cc_sectoral_subelectricalandopticalequipment(time()),

        cc_total() * share_cc_sectoral_subtransportequipment(time()),

        cc_total() * share_cc_sectoral_submanufacturingnecrecycling(time()),

        cc_total() * share_cc_sectoral_subelectricitygasandwatersupply(time()),

        cc_total() * share_cc_sectoral_subconstruction(time()),

        cc_total() *
        share_cc_sectoral_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),

        cc_total() *
        share_cc_sectoral_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),

        cc_total() *
        share_cc_sectoral_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),

        cc_total() * share_cc_sectoral_subhotelsandrestaurants(time()),

        cc_total() * share_cc_sectoral_subinlandtransport(time()),

        cc_total() * share_cc_sectoral_subwatertransport(time()),

        cc_total() * share_cc_sectoral_subairtransport(time()),

        cc_total() *
        share_cc_sectoral_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),

        cc_total() * share_cc_sectoral_subpostandtelecommunications(time()),

        cc_total() * share_cc_sectoral_subfinancialintermedation(time()),

        cc_total() * share_cc_sectoral_subrealestateactivities(time()),

        cc_total() * share_cc_sectoral_subrentingodmeqandotherbusinessactivities(time()),

        cc_total() * share_cc_sectoral_subpublicadminanddefencecompulsorysocialsecurity(time()),

        cc_total() * share_cc_sectoral_subeducation(time()),

        cc_total() * share_cc_sectoral_subhealthandsocialwork(time()),

        cc_total() * share_cc_sectoral_subothercommunitysocialandpersonaservices(time()),

        cc_total() * share_cc_sectoral_subprivatehouseholdswithemployedpersons(time())],
        coords={'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def cc_sectoral_next_step():
    """
    CC sectoral next step



    component


    """

    ret =  cc_sectoral() + variation_cc_sectoral()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def gross_fixed_capital_formation():
    """
    Gross fixed capital formation



    component


    """


    ret =  integ_gross_fixed_capital_formation()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def variation_cc_sectoral():
    """
    variation CC sectoral



    component


    """

    ret =  xr.DataArray(
        data=[((cc_total() + variation_cc()) *
         share_cc_next_step_subagriculturehuntingforestryandfishing(time())) -
        (cc_total() * share_cc_sectoral_subagriculturehuntingforestryandfishing(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subminingandquarrying(time())) -
        (cc_total() * share_cc_sectoral_subminingandquarrying(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subfoodbeveragesandtobacco(time())) -
        (cc_total() * share_cc_sectoral_subfoodbeveragesandtobacco(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subtextilesandtextileproducts(time()))
        - (cc_total() * share_cc_sectoral_subtextilesandtextileproducts(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subleatherleatherandfootwear(time())) -
        (cc_total() * share_cc_sectoral_subleatherleatherandfootwear(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subwoodandproductsofwooodandcork(
            time())) - (cc_total() * share_cc_sectoral_subwoodandproductsofwooodandcork(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subpulppaperprintingandpublishing(
            time())) - (cc_total() * share_cc_sectoral_subpulppaperprintingandpublishing(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subcokerefinedpetroleumandnuclearfuel(
            time())) -
        (cc_total() * share_cc_sectoral_subcokerefinedpetroleumandnuclearfuel(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subchemicalsandchemicalproducts(
            time())) - (cc_total() * share_cc_sectoral_subchemicalsandchemicalproducts(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subrubberandplastics(time())) -
        (cc_total() * share_cc_sectoral_subrubberandplastics(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subothernonmetalicmineral(time())) -
        (cc_total() * share_cc_sectoral_subothernonmetalicmineral(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subbasicmetalsandfabricatedmetal(
            time())) - (cc_total() * share_cc_sectoral_subbasicmetalsandfabricatedmetal(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_submachinerynec(time())) -
        (cc_total() * share_cc_sectoral_submachinerynec(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subelectricalandopticalequipment(
            time())) - (cc_total() * share_cc_sectoral_subelectricalandopticalequipment(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subtransportequipment(time())) -
        (cc_total() * share_cc_sectoral_subtransportequipment(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_submanufacturingnecrecycling(time())) -
        (cc_total() * share_cc_sectoral_submanufacturingnecrecycling(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subelectricitygasandwatersupply(
            time())) - (cc_total() * share_cc_sectoral_subelectricitygasandwatersupply(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subconstruction(time())) -
        (cc_total() * share_cc_sectoral_subconstruction(time())),

        ((cc_total() + variation_cc()) *
         share_cc_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
             time())) -
        (cc_total() *
         share_cc_sectoral_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
             time())),

        ((cc_total() + variation_cc()) *
         share_cc_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
             time())) -
        (cc_total() *
         share_cc_sectoral_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
             time())),

        ((cc_total() + variation_cc()) *
         share_cc_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
             time())) -
        (cc_total() *
         share_cc_sectoral_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
             time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subhotelsandrestaurants(time())) -
        (cc_total() * share_cc_sectoral_subhotelsandrestaurants(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subinlandtransport(time())) -
        (cc_total() * share_cc_sectoral_subinlandtransport(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subwatertransport(time())) -
        (cc_total() * share_cc_sectoral_subwatertransport(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subairtransport(time())) -
        (cc_total() * share_cc_sectoral_subairtransport(time())),

        ((cc_total() + variation_cc()) *
         share_cc_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
             time())) -
        (cc_total() *
         share_cc_sectoral_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
             time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subpostandtelecommunications(time())) -
        (cc_total() * share_cc_sectoral_subpostandtelecommunications(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subfinancialintermedation(time())) -
        (cc_total() * share_cc_sectoral_subfinancialintermedation(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subrealestateactivities(time())) -
        (cc_total() * share_cc_sectoral_subrealestateactivities(time())),

        ((cc_total() + variation_cc()) *
         share_cc_next_step_subrentingodmeqandotherbusinessactivities(time())) -
        (cc_total() * share_cc_sectoral_subrentingodmeqandotherbusinessactivities(time())),

        ((cc_total() + variation_cc()) *
         share_cc_next_step_subpublicadminanddefencecompulsorysocialsecurity(time())) -
        (cc_total() * share_cc_sectoral_subpublicadminanddefencecompulsorysocialsecurity(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subeducation(time())) -
        (cc_total() * share_cc_sectoral_subeducation(time())),

        ((cc_total() + variation_cc()) * share_cc_next_step_subhealthandsocialwork(time())) -
        (cc_total() * share_cc_sectoral_subhealthandsocialwork(time())),

        ((cc_total() + variation_cc()) *
         share_cc_next_step_subothercommunitysocialandpersonaservices(time())) -
        (cc_total() * share_cc_sectoral_subothercommunitysocialandpersonaservices(time())),

        ((cc_total() + variation_cc()) *
         share_cc_next_step_subprivatehouseholdswithemployedpersons(time())) -
        (cc_total() * share_cc_sectoral_subprivatehouseholdswithemployedpersons(time()))],
        coords={'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subagriculturehuntingforestryandfishing(x):
    """
    share CC next step SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.025078382628402493, 0.02464951185425293, 0.024412482955470757, 0.022658014973602505,
            0.02491994295641962, 0.02507741479510443, 0.02302318244876147, 0.02341619403613124,
            0.023769823644028576, 0.02040672872055882, 0.018639964524329856, 0.019132109467672614,
            0.019439319871382504, 0.01655073055008412
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subminingandquarrying(x):
    """
    share CC next step SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.018101740661219327, 0.018332290256034372, 0.01703113414914862, 0.0165157932633882,
            0.017486811309906522, 0.01801189939841712, 0.017721589288219524, 0.016965563499255994,
            0.017484206427070492, 0.01752032040491682, 0.018382928551712276, 0.01856406183756792,
            0.01917091605291903, 0.019668699343532815
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subfoodbeveragesandtobacco(x):
    """
    share CC next step SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.02876498203026079, 0.02814531333997819, 0.027531214667705785, 0.02874565191986845,
            0.02788345999760771, 0.02802430949192551, 0.026795521737131983, 0.02600865546252975,
            0.0238629158502632, 0.02330770485908382, 0.023629186745957292, 0.02356268466844763,
            0.023143221409336112, 0.023638735612115706
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subtextilesandtextileproducts(x):
    """
    share CC next step SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007825063986088282, 0.007651438290957927, 0.0074916774298229, 0.0065129555117770625,
            0.006447661322563113, 0.00606430460937917, 0.005883481200538257, 0.005318456094464006,
            0.005508273168393984, 0.005124370728697409, 0.005360769244134405, 0.005676448056192027,
            0.005897454007932367, 0.005910777608613948
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subleatherleatherandfootwear(x):
    """
    share CC next step SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.001143608757867767, 0.0010819449533878667, 0.0010508849725526389,
            0.0009647149390711795, 0.0009573791517385931, 0.0011008752744973995,
            0.00102773596644666, 0.0009999533655333415, 0.0008843018128007417,
            0.0009008914767220458, 0.0009874610391665518, 0.001067974870512249,
            0.001128960946128135, 0.0011516930123566763
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subwoodandproductsofwooodandcork(x):
    """
    share CC next step SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.003168228187107202, 0.0033681335538219806, 0.0032993092220650784,
            0.003328976784436948, 0.0031833806038531506, 0.003120972846879468,
            0.0029739924497252965, 0.003047707007696271, 0.0030294902999746915,
            0.0029438212130334035, 0.002889431931536702, 0.0029644037920589375,
            0.002754986281307624, 0.0024892388370459616
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subpulppaperprintingandpublishing(x):
    """
    share CC next step SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.015985164553250452, 0.015469435942986349, 0.015498594516144475, 0.016674768573807,
            0.016252638998880308, 0.015313901877892746, 0.01488650915592016, 0.01395986569583055,
            0.014257477691022605, 0.014401523761225027, 0.012986096954139215, 0.0124551897341896,
            0.011948904045551118, 0.011602358740517354
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subcokerefinedpetroleumandnuclearfuel(x):
    """
    share CC next step SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.012746459305556141, 0.013633527821310106, 0.01359401984732537, 0.011047742103384642,
            0.01184315652889293, 0.012897411493201982, 0.010670476406875505, 0.01269276226211265,
            0.01461178252401957, 0.014107919688276434, 0.01221847799976767, 0.011732689862703299,
            0.011403344506130163, 0.012538492970489885
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subchemicalsandchemicalproducts(x):
    """
    share CC next step SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.027479288796231974, 0.02786846743794753, 0.02644383447466471, 0.02724529339079687,
            0.02624198821499827, 0.026181380294071407, 0.027621052672975967, 0.027145511225085295,
            0.027529056528481182, 0.02654123196808494, 0.027374754999941013, 0.02853635359715732,
            0.026591622864745416, 0.026767398009174638
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subrubberandplastics(x):
    """
    share CC next step SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007931836698194594, 0.007929950133355458, 0.008081105585862648, 0.008032743025468464,
            0.007568253072198993, 0.007055808764641412, 0.007271392284354727, 0.007325513751262952,
            0.007354678780301329, 0.00721820304143248, 0.007005938695842652, 0.007367759456408463,
            0.006715065354087769, 0.005981566587978499
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subothernonmetalicmineral(x):
    """
    share CC next step SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.009083085220471966, 0.009359182092185262, 0.008776038855268316, 0.00880951349188728,
            0.008519134385357204, 0.008295573751607464, 0.00811510007249048, 0.008044587776070526,
            0.008372147189461823, 0.00843120193208617, 0.008877056370860618, 0.009110566920957066,
            0.008719093392781017, 0.007857115260071003
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subbasicmetalsandfabricatedmetal(x):
    """
    share CC next step SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.02409241566665906, 0.024628932833239057, 0.022608432090721487, 0.021563891804170753,
            0.02274463504803309, 0.02031556538542781, 0.019502770478668816, 0.020279731161448163,
            0.022241883350801843, 0.023391355493617486, 0.024932615017941605, 0.026498741926502475,
            0.026394927346192133, 0.02322394279397999
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_submachinerynec(x):
    """
    share CC next step SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.013719072372734927, 0.01393565671152066, 0.014096065919260808, 0.012396757811239846,
            0.013555513896937202, 0.012458152299629186, 0.011661210055884422, 0.012266401752284125,
            0.013881604122856654, 0.015158824782683553, 0.016585895022378887, 0.01761172852089743,
            0.01970770333839225, 0.017516772760582607
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subelectricalandopticalequipment(x):
    """
    share CC next step SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0240724320246828, 0.026879516079187753, 0.027491298893010396, 0.030775962613424947,
            0.03820207983468018, 0.027047047895312745, 0.027906514442787748, 0.03409009944486845,
            0.038641933859760995, 0.04129830740111195, 0.043431988576497264, 0.04706087316905279,
            0.04779006980275797, 0.045176727210357176
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subtransportequipment(x):
    """
    share CC next step SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.016071504276576817, 0.016269187914275323, 0.017384791192130742, 0.018521083745308402,
            0.017510845619079896, 0.01750544095862379, 0.019580343726275647, 0.017519592188039654,
            0.017505942568556376, 0.018029153530819162, 0.018884900139347384, 0.02068709274361638,
            0.018734791973192437, 0.017331831349728804
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_submanufacturingnecrecycling(x):
    """
    share CC next step SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.004246578649526875, 0.004325414800652897, 0.004465400376368324,
            0.0042640218851806785, 0.004382284365538603, 0.004177586302183998,
            0.0044474981883766305, 0.0044519849138500095, 0.005381816335691264,
            0.006430083963705986, 0.006026154324366072, 0.0058666091075879, 0.005829000303710761,
            0.005502366671535831
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subelectricitygasandwatersupply(x):
    """
    share CC next step SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.04702297200735588, 0.043770893328872754, 0.043527249921124174, 0.042248197518757455,
            0.04144001516883796, 0.03937928827045648, 0.03954100227559819, 0.03992084800226018,
            0.04122838142863999, 0.04124366752005908, 0.03908474464960975, 0.038426723351002466,
            0.03972970117195154, 0.039340682954113484
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subconstruction(x):
    """
    share CC next step SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.029711743968391505, 0.02784947827139627, 0.028342394795020702, 0.027970268160605542,
            0.025480331282401503, 0.025832810595568145, 0.025271920012836506, 0.02490199946395865,
            0.026099648636237482, 0.026303515770198044, 0.027156725944773412, 0.026060042489757677,
            0.025878577266468014, 0.025758496017789966
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    share CC next step SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.00903374834872022, 0.009173380221024536, 0.009522368180830508, 0.009262104773787541,
            0.008679969036621738, 0.008024579623124155, 0.007668346382009341, 0.007234417099582751,
            0.007793426044455792, 0.0077206884013198435, 0.00787785920922557, 0.007519391350279477,
            0.006816306866714831, 0.00579608344911891
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(x):
    """
    share CC next step SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.06872416518698367, 0.07397466130108132, 0.07759140573182152, 0.0790311572976072,
            0.07895106157594772, 0.08358153667209856, 0.08046215105242577, 0.08057597818003052,
            0.08366266458872629, 0.08459099239634021, 0.08197165939884506, 0.08438846586604647,
            0.0837852429770754, 0.08680986189833101
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(x):
    """
    share CC next step SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.043218689413549206, 0.04418452831060235, 0.04431907265956516, 0.04453363331100283,
            0.04423144754730959, 0.04719982136870136, 0.04802528865820533, 0.046326157202964964,
            0.043729995596854115, 0.04365355366137524, 0.04447043011858734, 0.04115812502638437,
            0.037186956214448466, 0.03600442411878235
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subhotelsandrestaurants(x):
    """
    share CC next step SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.017674065016558355, 0.018149583857362074, 0.018880821695584853, 0.019187506494702862,
            0.01848919255326852, 0.018801788372633084, 0.019375033321432044, 0.01894584982699591,
            0.018775261769920393, 0.018685354294381163, 0.019167548759844704, 0.01817645780586637,
            0.017594543932584812, 0.01729384088477041
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subinlandtransport(x):
    """
    share CC next step SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.01757328059423465, 0.017533477095097345, 0.01708335113402725, 0.016899900875868307,
            0.017573466333478917, 0.0184060737777463, 0.017573394156884907, 0.01752725282216478,
            0.017674037253620118, 0.01775614429616209, 0.01790638924580825, 0.01823776170799223,
            0.018692217055157924, 0.017259316371031952
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subwatertransport(x):
    """
    share CC next step SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0019363175510912095, 0.002174967262959381, 0.002445572534255821,
            0.0025031485102358655, 0.0027778550421596786, 0.003017307520741118,
            0.003093341832727418, 0.0033244710209141787, 0.0040514983108206415,
            0.004701125070493769, 0.004946960268778851, 0.005842127228333952, 0.006485775212538603,
            0.006004459143157512
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subairtransport(x):
    """
    share CC next step SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.004725321799396147, 0.004985152824590062, 0.004397007883431417, 0.003784892111621302,
            0.003282607584440268, 0.0016726602837590257, 0.0021694690555328824,
            0.0030221235882390467, 0.0035400602398324727, 0.0034155250192160567,
            0.0036206051079420323, 0.0033596968339394905, 0.002817936173472771,
            0.0023199328516766635
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    share CC next step SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.006899084212642317, 0.0074114575956901255, 0.0071853424807637525,
            0.006714489302199572, 0.005691227831434559, 0.006036424779551169, 0.006585132573263568,
            0.006975233725391538, 0.007547036142033878, 0.008121570535510506, 0.008817806913968915,
            0.00905327322223547, 0.00899968325699662, 0.007461986150000285
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subpostandtelecommunications(x):
    """
    share CC next step SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.03160549275661185, 0.0322243006026475, 0.03388806015063383, 0.03591291947337805,
            0.03791737172900439, 0.0413838152640967, 0.04456758387526782, 0.04571097051257681,
            0.046742194825149755, 0.04604476002740335, 0.045529587353981896, 0.04443512758226394,
            0.046731061459096244, 0.050155005310443236
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subfinancialintermedation(x):
    """
    share CC next step SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.07263386823827737, 0.07524361994554456, 0.07469807199082727, 0.07674454224960045,
            0.07811355669530828, 0.08063039557217784, 0.0847256373038466, 0.08363368144151041,
            0.07945056175518507, 0.07873728904029753, 0.08477781706234779, 0.08274482610250618,
            0.07932373057747709, 0.08359235909013855
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subrealestateactivities(x):
    """
    share CC next step SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.23937803446193515, 0.23358810685060818, 0.2301581651449089, 0.227948775874283,
            0.2256644107843945, 0.22877553857089394, 0.22367874012980918, 0.22277622638960762,
            0.21590454968628062, 0.2127113153994352, 0.20839896270047154, 0.2057033512412547,
            0.2094927971640791, 0.21569041217322732
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subrentingodmeqandotherbusinessactivities(x):
    """
    share CC next step SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.06905001380524613, 0.06821043531529368, 0.07288181835394007, 0.06817971257110544,
            0.06450830258564538, 0.06454304884231093, 0.06721206423359273, 0.0667362552587201,
            0.06682921530945202, 0.06977231435038858, 0.06861132671855419, 0.07189063738602502,
            0.07522583494958558, 0.06706870740084765
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subpublicadminanddefencecompulsorysocialsecurity(x):
    """
    share CC next step SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.03783224147960273, 0.03701298705578099, 0.03567177066473178, 0.038192407838542546,
            0.038012695658412206, 0.03826439070447431, 0.03718222874310394, 0.03554916941396557,
            0.030954803248098826, 0.029375582837865413, 0.029483320594616752, 0.026471578950167513,
            0.026470257950723577, 0.0318911564104782
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subeducation(x):
    """
    share CC next step SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.005875669601914743, 0.0056678531809458655, 0.006082054444355262,
            0.006127937515178124, 0.005803650449733586, 0.005767438285737494,
            0.0059783876907612225, 0.006127107081716946, 0.005054519325957908,
            0.005130009752766508, 0.004847058955568252, 0.0051597539362335815,
            0.005669951603168591, 0.005755993947517754
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subhealthandsocialwork(x):
    """
    share CC next step SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.029160160403648494, 0.02821173656516283, 0.026615770620869734, 0.0273539727857798,
            0.02662583643780871, 0.028142691231517622, 0.029297619075475182, 0.02941131460970212,
            0.0301810591244524, 0.031490932282237226, 0.031232041065638115, 0.02959403415633184,
            0.02988511373451547, 0.03422943890008811
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subothercommunitysocialandpersonaservices(x):
    """
    share CC next step SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.02841304632967128, 0.027082293603525917, 0.027431107643009052, 0.029323662796849222,
            0.029033478768738467, 0.027866881374139507, 0.028477811794828094, 0.027741634257114605,
            0.02643605715958021, 0.025304071077875273, 0.023855398102559003, 0.02385049848422552,
            0.023811135179053207, 0.024622633133622788
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_next_step_subprivatehouseholdswithemployedpersons(x):
    """
    share CC next step SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            2.224100933757743e-05, 2.3182796720525097e-05, 2.2308822775885723e-05,
            2.2884702082035344e-05, 2.4357628368450084e-05, 2.586345147694192e-05,
            2.6477256965928314e-05, 2.6730466120345426e-05, 2.769540121679224e-05,
            2.994530061908033e-05, 3.0137690959046645e-05, 3.283954762959073e-05,
            3.3795758345553865e-05, 3.676247669895163e-05
        ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpct_gfcf_vs_gfcfxhdx():
    """
    Xpct GFCF vs GFCFXHDX



    component


    """

    ret =  gross_fixed_capital_formation() / (gross_fixed_capital_formation() + household_demand())
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def household_demand():
    """
    Household demand



    component


    """


    ret =  integ_household_demand()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def lc_next_step():
    """
    LC next step



    component


    """

    ret =  lc() + variation_lc()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# FE INTENSITY households
# TRANSP - Households transport
# TRANSP - Households transport
@cache('step')
def household_demand_total():
    """
    Household demand total



    component


    """

    ret =  functions.sum(household_demand())
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('step')
def cedtot_per_tw_over_lifetime_res_elec_dispatch():
    """
    CEDtot per TW over lifetime RES elec dispatch



    component


    """

    ret =  xr.DataArray(
        data=[functions.zidz((1 - xres_elec_variablesxx().loc[{'res elec':'hydro'}].values) * (xcpxini_res_elecx_subhydro(time(
        )) * lifetime_res_elec().loc[{'res elec':'hydro'}].values * ej_per_twh() / twe_per_twh()),
                       xeroixini_res_elec_dispatchx().loc[{'res elec':'hydro'}].values * quality_of_electricity_2015()),

        functions.zidz((1 - xres_elec_variablesxx().loc[{'res elec':'xgeotxelecx'}].values) * (xcpxini_res_elecx_subxgeotxelecx(time(
        )) * lifetime_res_elec().loc[{'res elec':'xgeotxelecx'}].values * ej_per_twh() / twe_per_twh()),
                       xeroixini_res_elec_dispatchx().loc[{'res elec':'xgeotxelecx'}].values * quality_of_electricity_2015()),

        functions.zidz(
            (1 - xres_elec_variablesxx().loc[{'res elec':'xsolid bioexelecx'}].values) * (xcpxini_res_elecx_subxsolidbioexelecx(time(
            )) * lifetime_res_elec().loc[{'res elec':'xsolid bioexelecx'}].values * ej_per_twh() / twe_per_twh()),
            xeroixini_res_elec_dispatchx().loc[{'res elec':'xsolid bioexelecx'}].values * quality_of_electricity_2015()),

        functions.zidz((1 - xres_elec_variablesxx().loc[{'res elec':'oceanic'}].values) * (xcpxini_res_elecx_suboceanic(time(
        )) * lifetime_res_elec().loc[{'res elec':'oceanic'}].values * ej_per_twh() / twe_per_twh()),
                       xeroixini_res_elec_dispatchx().loc[{'res elec':'oceanic'}].values * quality_of_electricity_2015()),

        functions.zidz((1 - xres_elec_variablesxx().loc[{'res elec':'wind onshore'}].values) * (xcpxini_res_elecx_subwindonshore(time(
        )) * lifetime_res_elec().loc[{'res elec':'wind onshore'}].values * ej_per_twh() / twe_per_twh()),
                       xeroixini_res_elec_dispatchx().loc[{'res elec':'wind onshore'}].values * quality_of_electricity_2015()),

        functions.zidz((1 - xres_elec_variablesxx().loc[{'res elec':'wind offshore'}].values) * (xcpxini_res_elecx_subwindoffshore(time(
        )) * lifetime_res_elec().loc[{'res elec':'wind offshore'}].values * ej_per_twh() / twe_per_twh()),
                       xeroixini_res_elec_dispatchx().loc[{'res elec':'wind offshore'}].values * quality_of_electricity_2015()),

        functions.zidz((1 - xres_elec_variablesxx().loc[{'res elec':'solar pv'}].values) * (xcpxini_res_elecx_subsolarpv(time(
        )) * lifetime_res_elec().loc[{'res elec':'solar pv'}].values * ej_per_twh() / twe_per_twh()),
                       xeroixini_res_elec_dispatchx().loc[{'res elec':'solar pv'}].values * quality_of_electricity_2015()),

        functions.zidz((1 - xres_elec_variablesxx().loc[{'res elec':'csp'}].values) * (
            xcpxini_res_elecx_subcsp(time()) * lifetime_res_elec().loc[{'res elec':'csp'}].values * ej_per_twh() / twe_per_twh()),
                       xeroixini_res_elec_dispatchx().loc[{'res elec':'csp'}].values * quality_of_electricity_2015())],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# TPES
# ESOI - PHS
# EROI - RES elec
@cache('step')
def quality_of_electricity_2015():
    """
    quality of electricity 2015



    component


    """

    ret =  functions.if_then_else(time() < 2015, dynamic_quality_of_electricity(), aux3())
    name = 1
    return round_sig(x=ret, name=name)

# TPES
@cache('step')
def dynamic_quality_of_electricity():
    """
    Dynamic quality of electricity



    component


    """

    ret =  real_tfec() / (tpes_ej() - xtotal_real_nonxenergy_use_consumption_ejx())
    name = 1
    return round_sig(x=ret, name=name)

# TPES
# ESOI - PHS
# ESOI - EV batteries
# EROI - RES elec
# EROIgrid - Allocation RES elec
# EROItot - Electricity
@cache('step')
def quality_of_electricity():
    """
    quality of electricity



    component


    """

    ret =  functions.if_then_else(abs(xstaticxdynamic_quality_of_electricityxx() - 1) < equality_threshold,
                                  quality_of_electricity_2015(), dynamic_quality_of_electricity())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def cp_exogenous_res_elec_reduction():
    """
    Cp exogenous RES elec reduction



    component


    """

    ret =  xr.DataArray(
        data=[cp_exogenous_res_elec_dispatch_reduction(),

        cp_exogenous_res_elec_dispatch_reduction(),

        cp_exogenous_res_elec_dispatch_reduction(),

        cp_exogenous_res_elec_dispatch_reduction(),

        cp_exogenous_res_elec_var_reduction(),

        cp_exogenous_res_elec_var_reduction(),

        cp_exogenous_res_elec_var_reduction(),

        cp_exogenous_res_elec_var_reduction()],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def cp_res_elec():
    """
    Cp RES elec



    component


    """

    ret =  xr.DataArray(
        data=[np.maximum(min_cp_baseload_res().loc[{'res elec':'hydro'}].values,
                   xcpxini_res_elecx_subhydro(time()) * cp_exogenous_res_elec_reduction().loc[{'res elec':'hydro'}].values),

        np.maximum(min_cp_baseload_res().loc[{'res elec':'xgeotxelecx'}].values,
                   xcpxini_res_elecx_subxgeotxelecx(time()) * cp_exogenous_res_elec_reduction().loc[{'res elec':'xgeotxelecx'}].values),

        np.maximum(
            min_cp_baseload_res().loc[{'res elec':'xsolid bioexelecx'}].values,
            xcpxini_res_elecx_subxsolidbioexelecx(time()) * cp_exogenous_res_elec_reduction().loc[{'res elec':'xsolid bioexelecx'}].values),

        np.maximum(min_cp_baseload_res().loc[{'res elec':'oceanic'}].values,
                   xcpxini_res_elecx_suboceanic(time()) * cp_exogenous_res_elec_reduction().loc[{'res elec':'oceanic'}].values),

        np.maximum(min_cp_baseload_res().loc[{'res elec':'wind onshore'}].values,
                   xcpxini_res_elecx_subwindonshore(time()) * cp_exogenous_res_elec_reduction().loc[{'res elec':'wind onshore'}].values),

        np.maximum(min_cp_baseload_res().loc[{'res elec':'wind offshore'}].values,
                   xcpxini_res_elecx_subwindoffshore(time()) * cp_exogenous_res_elec_reduction().loc[{'res elec':'wind offshore'}].values),

        np.maximum(min_cp_baseload_res().loc[{'res elec':'solar pv'}].values,
                   xcpxini_res_elecx_subsolarpv(time()) * cp_exogenous_res_elec_reduction().loc[{'res elec':'solar pv'}].values),

        np.maximum(min_cp_baseload_res().loc[{'res elec':'csp'}].values,
                   xcpxini_res_elecx_subcsp(time()) * cp_exogenous_res_elec_reduction().loc[{'res elec':'csp'}].values)],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xres_elec_variablesxx():
    """
    XRES elec variablesXX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[1.], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[1.], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[1.], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[1.], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# TPES
@cache('step')
def aux3():
    """
    aux3



    component


    """

    ret =  delay_quality_of_electricity_2015_round003125__time_step___time_step_075_003125__time_step(
    )
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def overdemand_elec_eroi_from_2015():
    """
    overdemand elec EROI from 2015



    component


    """

    ret =  functions.if_then_else(abs(xactivate_eroi_elec_feedbackxx() ) < equality_threshold, 0,
                                  functions.if_then_else(
                                      time() < 2015, 0,
                                      overdemand_elec_tot() - overdemand_elec_tot_until_2015()))
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
# EROI - RES elec
@cache('step')
def fei_res_elec_var():
    """
    FEI RES elec var



    component


    """

    ret =  (cedtot_new_cap_res_elec_var() * (1 + grid_correction_factor_res_elec()) +
            ced_decom_res_elec_capacity() + xcedtot_oxm_res_elec_varx()) * quality_of_electricity(
            ) + real_generation_res_elec_ej() * xselfxelectricity_consumption_res_elecx()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xxstaticx_eroi_res_elecx():
    """
    XXstaticX EROI RES elecX



    component


    """

    ret =  xr.DataArray(
        data=[functions.if_then_else(abs(fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'hydro'}].values ) < equality_threshold, 0,
                               output_elec_over_lifetime_res_elec().loc[{'res elec':'hydro'}].values /
                               (fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'hydro'}].values * quality_of_electricity())),

        functions.if_then_else(abs(fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'xgeotxelecx'}].values ) < equality_threshold, 0,
                               output_elec_over_lifetime_res_elec().loc[{'res elec':'xgeotxelecx'}].values /
                               (fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'xgeotxelecx'}].values * quality_of_electricity())),

        functions.if_then_else(abs(fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'xsolid bioexelecx'}].values ) < equality_threshold, 0,
                               output_elec_over_lifetime_res_elec().loc[{'res elec':'xsolid bioexelecx'}].values /
                               (fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'xsolid bioexelecx'}].values * quality_of_electricity())),

        functions.if_then_else(abs(fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'oceanic'}].values ) < equality_threshold, 0,
                               output_elec_over_lifetime_res_elec().loc[{'res elec':'oceanic'}].values /
                               (fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'oceanic'}].values * quality_of_electricity())),

        functions.if_then_else(
            abs(fei_over_lifetime_res_elec_var().loc[{'res elec':'wind onshore'}].values ) < equality_threshold, 0,
            output_elec_over_lifetime_res_elec().loc[{'res elec':'wind onshore'}].values / fei_over_lifetime_res_elec_var().loc[{'res elec':'wind onshore'}].values),

        functions.if_then_else(
            abs(fei_over_lifetime_res_elec_var().loc[{'res elec':'wind offshore'}].values ) < equality_threshold, 0,
            output_elec_over_lifetime_res_elec().loc[{'res elec':'wind offshore'}].values / fei_over_lifetime_res_elec_var().loc[{'res elec':'wind offshore'}].values),

        functions.if_then_else(
            abs(fei_over_lifetime_res_elec_var().loc[{'res elec':'solar pv'}].values ) < equality_threshold, 0,
            output_elec_over_lifetime_res_elec().loc[{'res elec':'solar pv'}].values / fei_over_lifetime_res_elec_var().loc[{'res elec':'solar pv'}].values),

        functions.if_then_else(
            abs(fei_over_lifetime_res_elec_var().loc[{'res elec':'csp'}].values ) < equality_threshold, 0,
            functions.zidz(output_elec_over_lifetime_res_elec().loc[{'res elec':'csp'}].values, fei_over_lifetime_res_elec_var().loc[{'res elec':'csp'}].values))],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
# MATERIALS - Energy consumption
# EROI - RES elec
@cache('step')
def materials_required_for_new_res_elec_mt():
    """
    materials required for new RES elec Mt



    component


    """
    ret =  res_elec_capacity_under_construction_tw(
    ) * materials_for_new_res_elec_per_capacity_installed() * m_per_t() / kg_per_mt()

    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def eroist_tot_elec():
    """
    EROIst tot elec



    component


    """

    ret =  (fe_elec_generation_from_nre_twh() * ej_per_twh() +
            functions.sum(output_elec_over_lifetime_res_elec())) / (
                (functions.sum(cedtot_over_lifetime_res_elec()) + cedtot_over_lifetime_nre_elec()
                 ) * quality_of_electricity())
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
# EROIgrid - Allocation RES elec
@cache('step')
def fei_over_lifetime_res_elec_var():
    """
    FEI over lifetime RES elec var



    component


    """

    ret =  (
        cedtot_new_cap_res_elec_var() *
        (1 + share_energy_requirements_for_decom_res_elec() + grid_correction_factor_res_elec()) +
        xced_oxm_over_lifetime_res_elec_varx()) * quality_of_electricity(
        ) + output_elec_over_lifetime_res_elec() * xselfxelectricity_consumption_res_elecx()
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('step')
def ced_decom_res_elec_capacity():
    """
    CED decom RES elec capacity



    component


    """

    ret =  functions.zidz(share_energy_requirements_for_decom_res_elec() *
                          cedtot_new_cap_res_elec_var() * wear_res_elec(),
                          res_elec_capacity_under_construction_tw())
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('run')
def test_ir_restando_al_eroist_tot_elec():
    """
    test ir restando al EROIst tot elec



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def eroist_tot_elec_2():
    """
    EROIst tot elec 2



    component


    """

    ret =  functions.if_then_else(single_precision(eroist_tot_elec() - test_ir_restando_al_eroist_tot_elec()) > single_precision(0) + less_thr_func(eroist_tot_elec() - test_ir_restando_al_eroist_tot_elec(), 0),
                                  eroist_tot_elec() - test_ir_restando_al_eroist_tot_elec(), 0.5)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xced_oxm_per_material_res_elec_varx():
    """
    XCED OXM per material RES elec varX



    component


    """

    ret =  xmaterials_required_for_oxm_res_elec_mtx(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('step')
def cedtot_new_cap_res_elec_var():
    """
    CEDtot new cap RES elec var



    component


    """

    ret =  functions.sum(dim='materials', data=ced_new_cap_per_material_res_elec_var())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xcedtot_oxm_res_elec_varx():
    """
    XCEDtot OXM RES elec varX



    component


    """

    ret =  functions.sum(dim='materials', data=xced_oxm_per_material_res_elec_varx()
                          ) + xtotal_energy_requirements_oxm_for_water_consumption_res_elecx()
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def overdemand_elec_tot_until_2015():
    """
    overdemand elec tot until 2015



    component


    """

    ret =  functions.if_then_else(time() < 2015, overdemand_elec_tot(), aux2())
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def aux2():
    """
    aux2



    component


    """

    ret =  delay_overdemand_elec_tot_until_2015_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS CC - Energy losses function
@cache('step')
def aux1():
    """
    aux1



    component


    """

    ret =  delay_xsharexe_losses_cc_until_2015x_round003125__time_step___time_step_0_003125__time_step(
    )
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def overdemand_elec_tot():
    """
    overdemand elec tot



    component


    """

    ret =  functions.if_then_else(single_precision(eroist_tot_elec_2()) > single_precision(1.1) + less_thr_func(eroist_tot_elec_2(), 1.1), 1 / (eroist_tot_elec_2() - 1), 5)
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# EROItot - Electricity
@cache('step')
def overdemand_elec_eroi_delayed_1yr():
    """
    overdemand elec EROI delayed 1yr



    component


    """

    ret =  delay_overdemand_elec_eroi_from_2015_round1__time_step___time_step_0_1__time_step()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xactivate_eroi_elec_feedbackxx():
    """
    XActivate EROI elec feedbackXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def ced_unconv_gas_for_elec():
    """
    CED unconv gas for elec



    component


    """

    ret =  fe_elec_generation_from_unconv_gas_twh() * ej_per_twh() / eroi_unconv_gas()
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def ced_uranium_for_elec():
    """
    CED uranium for elec



    component


    """

    ret =  fe_nuclear_elec_generation_twh() * ej_per_twh() / eroi_uranium()
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def cedtot_over_lifetime_nre_elec():
    """
    CEDtot over lifetime NRE elec



    component


    """

    ret =  ced_coal_for_elec() + ced_conv_gas_for_elec() + ced_total_oil_for_elec(
    ) + ced_unconv_gas_for_elec() + ced_uranium_for_elec()
    name = 1
    return round_sig(x=ret, name=name)

# ESOI - PHS
@cache('step')
def b_lineal_regr():
    """
    b lineal regr



    component


    """

    ret =  esoi_phs_depleted_potential() - a_lineal_regr() * max_capacity_potential_phs()
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('run')
def eroi_uranium():
    """
    EROI uranium



    constant


    """

    ret =  10
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('run')
def eroi_coal():
    """
    EROI coal



    constant


    """

    ret =  46
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('run')
def eroi_conv_gas():
    """
    EROI conv gas



    constant


    """

    ret =  40
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('run')
def eroi_total_oil():
    """
    EROI total oil



    constant


    """

    ret =  20
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('run')
def eroi_unconv_gas():
    """
    EROI unconv gas



    constant


    """

    ret =  10
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def ced_coal_for_elec():
    """
    CED coal for elec



    component


    """

    ret =  fe_elec_generation_from_coal_twh() * ej_per_twh() / eroi_coal()
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def ced_conv_gas_for_elec():
    """
    CED conv gas for elec



    component


    """

    ret =  fe_elec_generation_from_conv_gas_twh() * ej_per_twh() / eroi_conv_gas()
    name = 1
    return round_sig(x=ret, name=name)

# ESOI - PHS
@cache('step')
def a_lineal_regr():
    """
    a lineal regr



    component


    """

    ret =  (esoi_phs_full_potential() - esoi_phs_depleted_potential()) / (
        0 - max_capacity_potential_phs())
    name = 1
    return round_sig(x=ret, name=name)

# EROItot - Electricity
@cache('step')
def ced_total_oil_for_elec():
    """
    CED total oil for elec



    component


    """

    ret =  fe_elec_generation_from_total_oil_twh() * ej_per_twh() / eroi_total_oil()
    name = 1
    return round_sig(x=ret, name=name)

# ESOI - PHS
@cache('run')
def esoi_phs_depleted_potential():
    """
    ESOI PHS depleted potential



    constant


    """

    ret =  5
    name = 1
    return round_sig(x=ret, name=name)
# ECON DEM - WIOD sectors and households

def historic_labour_share(x):
    """
    Historic labour share



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013
    ], [
        -0.03570221794602446, -0.007511835079606288, -4.9226177098571355e-05,
        -0.0036856044367975516, -0.0018807592046075072, 0.0033245164448738596,
        -0.00916060631432225, -0.0035427091662072474, -0.012338255775489415, -0.00752841386661518,
        -0.0031033781483257306, -0.005033300111722316, -0.00010456283432869196,
        -0.005033300111722316, -0.009419751237859098, -0.009419751237859098, -0.009419751237859098,
        -0.009419751237859098, -0.009419751237859098
    ])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def cc_total():
    """
    CC total



    component


    """


    ret =  integ_cc_total()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def initial_household_demand():
    """
    initial household demand



    constant


    """

    ret =  xr.DataArray(
        data=[
            748525.184544, 41789.6873, 1723750.213929, 470469.2814293, 97413.51078294,
            27311.3812115, 218308.785156, 217859.5983007, 329088.21596, 91494.368471,
            50849.6902961, 98211.1859259, 169551.78439, 269085.07894, 559770.13765, 222979.9293546,
            425936.77411, 43757.5723889, 329630.413413, 953189.751819, 1442744.514577,
            1113431.49591, 512283.1868, 44308.95214, 137453.87786, 138435.92747, 324538.4833,
            992982.57458, 2441020.9794, 314984.6207, 376202.178, 340722.53, 1143478.152,
            959930.083, 47919.81617848
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def total_demand():
    """
    total demand



    component


    """

    ret =  functions.sum(demand_by_sector_fd()) / 1e+006
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def demand_not_covered_by_sector_fd():
    """
    demand not covered by sector FD



    component


    """

    ret =  functions.if_then_else(time() > 2009, demand_by_sector_fd() - real_demand_by_sector(),
                                  0)
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def initial_demand_by_sectot():
    """
    initial demand by sectot



    constant


    """

    ret =  xr.DataArray(
        data=[
            821277.351644, 106956.558, 1761460.089629, 500387.9312293, 102249.58580294,
            58713.0731415, 261905.835856, 224714.8110007, 401281.55396, 113840.587681,
            64807.2040561, 259983.9362259, 777405.42339, 895155.41074, 1109416.05655,
            323158.7554546, 440415.90923, 3373315.0363289, 352813.949813, 1263743.815819,
            1598464.786877, 1135913.81656, 579016.9834, 49225.02703, 143095.69405, 165486.95717,
            336812.13263, 1035860.06854, 2560435.958453, 715097.68812, 3163684.928, 1176056.855178,
            2116985.945408, 1121312.875, 48106.64623338
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subagriculturehuntingforestryandfishing(x):
    """
    share CC sectoral SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.02287729187012649, 0.025078382628402493, 0.02464951185425293, 0.024412482955470757,
            0.022658014973602505, 0.02491994295641962, 0.02507741479510443, 0.02302318244876147,
            0.02341619403613124, 0.023769823644028576, 0.02040672872055882, 0.018639964524329856,
            0.019132109467672614, 0.019439319871382504, 0.01655073055008412
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subminingandquarrying(x):
    """
    share CC sectoral SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.019074894806125776, 0.018101740661219327, 0.018332290256034372, 0.01703113414914862,
            0.0165157932633882, 0.017486811309906522, 0.01801189939841712, 0.017721589288219524,
            0.016965563499255994, 0.017484206427070492, 0.01752032040491682, 0.018382928551712276,
            0.01856406183756792, 0.01917091605291903, 0.019668699343532815
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subfoodbeveragesandtobacco(x):
    """
    share CC sectoral SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0321389816671635, 0.02876498203026079, 0.02814531333997819, 0.027531214667705785,
            0.02874565191986845, 0.02788345999760771, 0.02802430949192551, 0.026795521737131983,
            0.02600865546252975, 0.0238629158502632, 0.02330770485908382, 0.023629186745957292,
            0.02356268466844763, 0.023143221409336112, 0.023638735612115706
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subtextilesandtextileproducts(x):
    """
    share CC sectoral SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.008385489965815875, 0.007825063986088282, 0.007651438290957927, 0.0074916774298229,
            0.0065129555117770625, 0.006447661322563113, 0.00606430460937917, 0.005883481200538257,
            0.005318456094464006, 0.005508273168393984, 0.005124370728697409, 0.005360769244134405,
            0.005676448056192027, 0.005897454007932367, 0.005910777608613948
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subleatherleatherandfootwear(x):
    """
    share CC sectoral SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0012340222421751328, 0.001143608757867767, 0.0010819449533878667,
            0.0010508849725526389, 0.0009647149390711795, 0.0009573791517385931,
            0.0011008752744973995, 0.00102773596644666, 0.0009999533655333415,
            0.0008843018128007417, 0.0009008914767220458, 0.0009874610391665518,
            0.001067974870512249, 0.001128960946128135, 0.0011516930123566763
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subwoodandproductsofwooodandcork(x):
    """
    share CC sectoral SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0036959103200494992, 0.003168228187107202, 0.0033681335538219806,
            0.0032993092220650784, 0.003328976784436948, 0.0031833806038531506,
            0.003120972846879468, 0.0029739924497252965, 0.003047707007696271,
            0.0030294902999746915, 0.0029438212130334035, 0.002889431931536702,
            0.0029644037920589375, 0.002754986281307624, 0.0024892388370459616
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subpulppaperprintingandpublishing(x):
    """
    share CC sectoral SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.016163481867476657, 0.015985164553250452, 0.015469435942986349, 0.015498594516144475,
            0.016674768573807, 0.016252638998880308, 0.015313901877892746, 0.01488650915592016,
            0.01395986569583055, 0.014257477691022605, 0.014401523761225027, 0.012986096954139215,
            0.0124551897341896, 0.011948904045551118, 0.011602358740517354
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subcokerefinedpetroleumandnuclearfuel(x):
    """
    share CC sectoral SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.012911021621846729, 0.012746459305556141, 0.013633527821310106, 0.01359401984732537,
            0.011047742103384642, 0.01184315652889293, 0.012897411493201982, 0.010670476406875505,
            0.01269276226211265, 0.01461178252401957, 0.014107919688276434, 0.01221847799976767,
            0.011732689862703299, 0.011403344506130163, 0.012538492970489885
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subchemicalsandchemicalproducts(x):
    """
    share CC sectoral SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.02819783641479054, 0.027479288796231974, 0.02786846743794753, 0.02644383447466471,
            0.02724529339079687, 0.02624198821499827, 0.026181380294071407, 0.027621052672975967,
            0.027145511225085295, 0.027529056528481182, 0.02654123196808494, 0.027374754999941013,
            0.02853635359715732, 0.026591622864745416, 0.026767398009174638
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subrubberandplastics(x):
    """
    share CC sectoral SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.007458520759641382, 0.007931836698194594, 0.007929950133355458, 0.008081105585862648,
            0.008032743025468464, 0.007568253072198993, 0.007055808764641412, 0.007271392284354727,
            0.007325513751262952, 0.007354678780301329, 0.00721820304143248, 0.007005938695842652,
            0.007367759456408463, 0.006715065354087769, 0.005981566587978499
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subothernonmetalicmineral(x):
    """
    share CC sectoral SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.009909953771210725, 0.009083085220471966, 0.009359182092185262, 0.008776038855268316,
            0.00880951349188728, 0.008519134385357204, 0.008295573751607464, 0.00811510007249048,
            0.008044587776070526, 0.008372147189461823, 0.00843120193208617, 0.008877056370860618,
            0.009110566920957066, 0.008719093392781017, 0.007857115260071003
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subbasicmetalsandfabricatedmetal(x):
    """
    share CC sectoral SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.025876033237234416, 0.02409241566665906, 0.024628932833239057, 0.022608432090721487,
            0.021563891804170753, 0.02274463504803309, 0.02031556538542781, 0.019502770478668816,
            0.020279731161448163, 0.022241883350801843, 0.023391355493617486, 0.024932615017941605,
            0.026498741926502475, 0.026394927346192133, 0.02322394279397999
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_submachinerynec(x):
    """
    share CC sectoral SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.014059658337226175, 0.013719072372734927, 0.01393565671152066, 0.014096065919260808,
            0.012396757811239846, 0.013555513896937202, 0.012458152299629186, 0.011661210055884422,
            0.012266401752284125, 0.013881604122856654, 0.015158824782683553, 0.016585895022378887,
            0.01761172852089743, 0.01970770333839225, 0.017516772760582607
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subelectricalandopticalequipment(x):
    """
    share CC sectoral SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.02211615042750115, 0.0240724320246828, 0.026879516079187753, 0.027491298893010396,
            0.030775962613424947, 0.03820207983468018, 0.027047047895312745, 0.027906514442787748,
            0.03409009944486845, 0.038641933859760995, 0.04129830740111195, 0.043431988576497264,
            0.04706087316905279, 0.04779006980275797, 0.045176727210357176
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subtransportequipment(x):
    """
    share CC sectoral SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.01611580684921055, 0.016071504276576817, 0.016269187914275323, 0.017384791192130742,
            0.018521083745308402, 0.017510845619079896, 0.01750544095862379, 0.019580343726275647,
            0.017519592188039654, 0.017505942568556376, 0.018029153530819162, 0.018884900139347384,
            0.02068709274361638, 0.018734791973192437, 0.017331831349728804
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_submanufacturingnecrecycling(x):
    """
    share CC sectoral SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.004136770188653854, 0.004246578649526875, 0.004325414800652897, 0.004465400376368324,
            0.0042640218851806785, 0.004382284365538603, 0.004177586302183998,
            0.0044474981883766305, 0.0044519849138500095, 0.005381816335691264,
            0.006430083963705986, 0.006026154324366072, 0.0058666091075879, 0.005829000303710761,
            0.005502366671535831
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subelectricitygasandwatersupply(x):
    """
    share CC sectoral SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.047066365445528786, 0.04702297200735588, 0.043770893328872754, 0.043527249921124174,
            0.042248197518757455, 0.04144001516883796, 0.03937928827045648, 0.03954100227559819,
            0.03992084800226018, 0.04122838142863999, 0.04124366752005908, 0.03908474464960975,
            0.038426723351002466, 0.03972970117195154, 0.039340682954113484
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subconstruction(x):
    """
    share CC sectoral SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.032283292539891505, 0.029711743968391505, 0.02784947827139627, 0.028342394795020702,
            0.027970268160605542, 0.025480331282401503, 0.025832810595568145, 0.025271920012836506,
            0.02490199946395865, 0.026099648636237482, 0.026303515770198044, 0.027156725944773412,
            0.026060042489757677, 0.025878577266468014, 0.025758496017789966
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(x):
    """
    share CC sectoral SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.009551079848249643, 0.00903374834872022, 0.009173380221024536, 0.009522368180830508,
            0.009262104773787541, 0.008679969036621738, 0.008024579623124155, 0.007668346382009341,
            0.007234417099582751, 0.007793426044455792, 0.0077206884013198435, 0.00787785920922557,
            0.007519391350279477, 0.006816306866714831, 0.00579608344911891
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(x):
    """
    share CC sectoral SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.06748267878929118, 0.06872416518698367, 0.07397466130108132, 0.07759140573182152,
            0.0790311572976072, 0.07895106157594772, 0.08358153667209856, 0.08046215105242577,
            0.08057597818003052, 0.08366266458872629, 0.08459099239634021, 0.08197165939884506,
            0.08438846586604647, 0.0837852429770754, 0.08680986189833101
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(x):
    """
    share CC sectoral SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03997245896306118, 0.043218689413549206, 0.04418452831060235, 0.04431907265956516,
            0.04453363331100283, 0.04423144754730959, 0.04719982136870136, 0.04802528865820533,
            0.046326157202964964, 0.043729995596854115, 0.04365355366137524, 0.04447043011858734,
            0.04115812502638437, 0.037186956214448466, 0.03600442411878235
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subhotelsandrestaurants(x):
    """
    share CC sectoral SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.01617662855110026, 0.017674065016558355, 0.018149583857362074, 0.018880821695584853,
            0.019187506494702862, 0.01848919255326852, 0.018801788372633084, 0.019375033321432044,
            0.01894584982699591, 0.018775261769920393, 0.018685354294381163, 0.019167548759844704,
            0.01817645780586637, 0.017594543932584812, 0.01729384088477041
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subinlandtransport(x):
    """
    share CC sectoral SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.021253625676560958, 0.01757328059423465, 0.017533477095097345, 0.01708335113402725,
            0.016899900875868307, 0.017573466333478917, 0.0184060737777463, 0.017573394156884907,
            0.01752725282216478, 0.017674037253620118, 0.01775614429616209, 0.01790638924580825,
            0.01823776170799223, 0.018692217055157924, 0.017259316371031952
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subwatertransport(x):
    """
    share CC sectoral SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.00237930634975559, 0.0019363175510912095, 0.002174967262959381, 0.002445572534255821,
            0.0025031485102358655, 0.0027778550421596786, 0.003017307520741118,
            0.003093341832727418, 0.0033244710209141787, 0.0040514983108206415,
            0.004701125070493769, 0.004946960268778851, 0.005842127228333952, 0.006485775212538603,
            0.006004459143157512
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subairtransport(x):
    """
    share CC sectoral SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.004166442318428751, 0.004725321799396147, 0.004985152824590062, 0.004397007883431417,
            0.003784892111621302, 0.003282607584440268, 0.0016726602837590257,
            0.0021694690555328824, 0.0030221235882390467, 0.0035400602398324727,
            0.0034155250192160567, 0.0036206051079420323, 0.0033596968339394905,
            0.002817936173472771, 0.0023199328516766635
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    share CC sectoral SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.006778853572000129, 0.006899084212642317, 0.0074114575956901255,
            0.0071853424807637525, 0.006714489302199572, 0.005691227831434559,
            0.006036424779551169, 0.006585132573263568, 0.006975233725391538, 0.007547036142033878,
            0.008121570535510506, 0.008817806913968915, 0.00905327322223547, 0.00899968325699662,
            0.007461986150000285
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subpostandtelecommunications(x):
    """
    share CC sectoral SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.029057579895015385, 0.03160549275661185, 0.0322243006026475, 0.03388806015063383,
            0.03591291947337805, 0.03791737172900439, 0.0413838152640967, 0.04456758387526782,
            0.04571097051257681, 0.046742194825149755, 0.04604476002740335, 0.045529587353981896,
            0.04443512758226394, 0.046731061459096244, 0.050155005310443236
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subfinancialintermedation(x):
    """
    share CC sectoral SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.07273569540483278, 0.07263386823827737, 0.07524361994554456, 0.07469807199082727,
            0.07674454224960045, 0.07811355669530828, 0.08063039557217784, 0.0847256373038466,
            0.08363368144151041, 0.07945056175518507, 0.07873728904029753, 0.08477781706234779,
            0.08274482610250618, 0.07932373057747709, 0.08359235909013855
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subrealestateactivities(x):
    """
    share CC sectoral SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.24111520591885524, 0.23937803446193515, 0.23358810685060818, 0.2301581651449089,
            0.227948775874283, 0.2256644107843945, 0.22877553857089394, 0.22367874012980918,
            0.22277622638960762, 0.21590454968628062, 0.2127113153994352, 0.20839896270047154,
            0.2057033512412547, 0.2094927971640791, 0.21569041217322732
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subrentingodmeqandotherbusinessactivities(x):
    """
    share CC sectoral SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.06459602118620193, 0.06905001380524613, 0.06821043531529368, 0.07288181835394007,
            0.06817971257110544, 0.06450830258564538, 0.06454304884231093, 0.06721206423359273,
            0.0667362552587201, 0.06682921530945202, 0.06977231435038858, 0.06861132671855419,
            0.07189063738602502, 0.07522583494958558, 0.06706870740084765
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subpublicadminanddefencecompulsorysocialsecurity(x):
    """
    share CC sectoral SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03788162091421682, 0.03783224147960273, 0.03701298705578099, 0.03567177066473178,
            0.038192407838542546, 0.038012695658412206, 0.03826439070447431, 0.03718222874310394,
            0.03554916941396557, 0.030954803248098826, 0.029375582837865413, 0.029483320594616752,
            0.026471578950167513, 0.026470257950723577, 0.0318911564104782
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subeducation(x):
    """
    share CC sectoral SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.006956181200851506, 0.005875669601914743, 0.0056678531809458655,
            0.006082054444355262, 0.006127937515178124, 0.005803650449733586, 0.005767438285737494,
            0.0059783876907612225, 0.006127107081716946, 0.005054519325957908,
            0.005130009752766508, 0.004847058955568252, 0.0051597539362335815,
            0.005669951603168591, 0.005755993947517754
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subhealthandsocialwork(x):
    """
    share CC sectoral SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.027445053332828068, 0.029160160403648494, 0.02821173656516283, 0.026615770620869734,
            0.0273539727857798, 0.02662583643780871, 0.028142691231517622, 0.029297619075475182,
            0.02941131460970212, 0.0301810591244524, 0.031490932282237226, 0.031232041065638115,
            0.02959403415633184, 0.02988511373451547, 0.03422943890008811
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subothercommunitysocialandpersonaservices(x):
    """
    share CC sectoral SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.028727661644533248, 0.02841304632967128, 0.027082293603525917, 0.027431107643009052,
            0.029323662796849222, 0.029033478768738467, 0.027866881374139507, 0.028477811794828094,
            0.027741634257114605, 0.02643605715958021, 0.025304071077875273, 0.023855398102559003,
            0.02385049848422552, 0.023811135179053207, 0.024622633133622788
        ])
    name = 1
    return round_sig(x=ret, name=name)

def share_cc_sectoral_subprivatehouseholdswithemployedpersons(x):
    """
    share CC sectoral SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.242410254852871e-05, 2.224100933757743e-05, 2.3182796720525097e-05,
            2.2308822775885723e-05, 2.2884702082035344e-05, 2.4357628368450084e-05,
            2.586345147694192e-05, 2.6477256965928314e-05, 2.6730466120345426e-05,
            2.769540121679224e-05, 2.994530061908033e-05, 3.0137690959046645e-05,
            3.283954762959073e-05, 3.3795758345553865e-05, 3.676247669895163e-05
        ])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def beta_1_lab():
    """
    Beta 1 lab



    constant


    """

    ret =  xr.DataArray(
        data=[
            0., -3.041303, 0.8496438, -0.4561882, -2.064467, -3.356857, -1.226881, -1.186757,
            -0.712791, -2.035976, -2.704609, -2.061479, -1.488448, -0.675253, -0.1846172, -1.18592,
            -0.5334909, -2.756193, -0.7490777, 0.3986385, 0.7057005, 0.4404486, -0.3547073,
            -2.553138, -1.624648, -1.630129, -0.3472749, 0.4875999, 1.229081, -0.7056298,
            -0.7456611, -0.7185996, 0.5389905, 0.3212347, -2.770384
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def initial_lc_total():
    """
    initial LC total



    constant


    """

    ret =  1.85847e+007
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def initial_gfcf():
    """
    initial GFCF



    constant


    """

    ret =  xr.DataArray(
        data=[
            3.57328320e+04, 4.91682230e+04, 3.15332790e+03, 9.80544140e+03, 6.54278810e+02,
            2.64895370e+04, 3.67987380e+04, 1.32808530e+03, 1.08696450e+04, 1.27060390e+04,
            5.83328320e+03, 1.51450720e+05, 5.91070690e+05, 5.92198440e+05, 5.25742880e+05,
            8.59668520e+04, 9.16334570e+03, 3.31573550e+06, 2.01142170e+04, 2.72784380e+05,
            1.34780130e+05, 6.33802150e+03, 4.96777380e+04, 2.46259080e+03, 4.38979740e+03,
            7.16301460e+03, 9.40007420e+03, 3.75346920e+03, 1.05648820e+05, 3.22190160e+05,
            1.64667130e+04, 8.77415770e+02, 2.02057430e+03, 1.90667700e+04, 1.33847630e+02
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def beta_0_cap():
    """
    beta 0 cap



    constant


    """

    ret =  5.09048
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def initial_cc_total():
    """
    initial CC total



    constant


    """

    ret =  1.05739e+007
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def beta_2_cap():
    """
    beta 2 cap



    constant


    """

    ret =  0.434675
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def demand_not_covered_total_fd():
    """
    demand not covered total FD



    component


    """

    ret =  functions.sum(demand_not_covered_by_sector_fd())
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def beta_1_cap():
    """
    beta 1 cap



    constant


    """

    ret =  xr.DataArray(
        data=[
            0., 0.6549913, -2.409549, -0.7288079, -2.802861, 0.2653398, 0.3796384, -3.044304,
            -1.282831, -0.5025426, -1.428307, 1.431519, 3.098772, 2.975706, 2.985555, 1.702436,
            -1.410645, 4.458449, 0.0193238, 1.628343, 1.206609, -2.001936, 0.4501797, -1.538287,
            -1.335444, -1.169046, -0.8956122, -2.335751, 0.3825687, 2.065982, -1.44898, -2.976849,
            -2.671927, -0.4900947, -3.099085
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def lc():
    """
    LC



    component


    """


    ret =  integ_lc()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def bet_0_lab():
    """
    Bet 0 lab



    constant


    """

    ret =  -5.52042
    name = 1
    return round_sig(x=ret, name=name)

# TPED by fuel
# TPES
@cache('step')
def tped_by_fuel():
    """
    TPED by fuel



    component


    """

    ret =  extraction_uranium_ej() + xpe_supply_res_nonxelec_ejx() + pe_elec_generation_from_res_ej(
    ) + ped_total_oil_ej() + ped_coal_ej() + xped_natx_gas_ejx() + pes_waste_ej()
    name = 1
    return round_sig(x=ret, name=name)

# TPES
@cache('step')
def abundance_tpe():
    """
    abundance TPE



    component


    """

    ret =  functions.if_then_else(single_precision(tpes_ej()) > single_precision(tped_by_fuel()) + less_thr_func(tpes_ej(), tped_by_fuel()), 1,
                                  1 - ((tped_by_fuel() - tpes_ej()) / tped_by_fuel()))
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def beta_2_lab():
    """
    beta 2 lab



    constant


    """

    ret =  1.13545
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def historic_rate_final_energy_intensity():
    """
    historic rate final energy intensity



    component


    """

    ret =  xr.DataArray(
        data=np.zeros((5, 35)),
        coords={'final sources':['electricity','heat','liquids','gases','solids'], 'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons']},
        dims=['final sources','sectors'])
    data = [[historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculturehuntingforestryandfishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculturehuntingforestryandfishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subminingandquarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subminingandquarrying(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfoodbeveragesandtobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfoodbeveragesandtobacco(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextilesandtextileproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextilesandtextileproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleatherleatherandfootwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleatherleatherandfootwear(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwoodandproductsofwooodandcork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwoodandproductsofwooodandcork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulppaperprintingandpublishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulppaperprintingandpublishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcokerefinedpetroleumandnuclearfuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcokerefinedpetroleumandnuclearfuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicalsandchemicalproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicalsandchemicalproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubberandplastics(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubberandplastics(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothernonmetalicmineral(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothernonmetalicmineral(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasicmetalsandfabricatedmetal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasicmetalsandfabricatedmetal(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinerynec(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinerynec(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricalandopticalequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricalandopticalequipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransportequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransportequipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturingnecrecycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturingnecrecycling(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricitygasandwatersupply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricitygasandwatersupply(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotelsandrestaurants(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotelsandrestaurants(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinlandtransport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinlandtransport(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwatertransport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwatertransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subairtransport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subairtransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpostandtelecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpostandtelecommunications(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancialintermedation(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancialintermedation(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrealestateactivities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrealestateactivities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrentingodmeqandotherbusinessactivities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrentingodmeqandotherbusinessactivities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublicadminanddefencecompulsorysocialsecurity(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublicadminanddefencecompulsorysocialsecurity(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation(time() + 1)
        - historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealthandsocialwork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealthandsocialwork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothercommunitysocialandpersonaservices(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothercommunitysocialandpersonaservices(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivatehouseholdswithemployedpersons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivatehouseholdswithemployedpersons(
            time())],
        [
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subminingandquarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subminingandquarrying(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subminingandquarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subminingandquarrying(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subfoodbeveragesandtobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subfoodbeveragesandtobacco(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextilesandtextileproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextilesandtextileproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subleatherleatherandfootwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subleatherleatherandfootwear(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwoodandproductsofwooodandcork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwoodandproductsofwooodandcork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulppaperprintingandpublishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulppaperprintingandpublishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subcokerefinedpetroleumandnuclearfuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subcokerefinedpetroleumandnuclearfuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicalsandchemicalproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicalsandchemicalproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubberandplastics(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubberandplastics(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subothernonmetalicmineral(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subothernonmetalicmineral(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasicmetalsandfabricatedmetal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasicmetalsandfabricatedmetal(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinerynec(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinerynec(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricalandopticalequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricalandopticalequipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransportequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransportequipment(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturingnecrecycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturingnecrecycling(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricitygasandwatersupply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricitygasandwatersupply(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotelsandrestaurants(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotelsandrestaurants(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subinlandtransport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subinlandtransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwatertransport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwatertransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subairtransport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subairtransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpostandtelecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpostandtelecommunications(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancialintermedation(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancialintermedation(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrealestateactivities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrealestateactivities(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrentingodmeqandotherbusinessactivities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrentingodmeqandotherbusinessactivities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublicadminanddefencecompulsorysocialsecurity(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublicadminanddefencecompulsorysocialsecurity(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealthandsocialwork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealthandsocialwork(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subothercommunitysocialandpersonaservices(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subothercommunitysocialandpersonaservices(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivatehouseholdswithemployedpersons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivatehouseholdswithemployedpersons(
            time())],
        [
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subminingandquarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subminingandquarrying(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subminingandquarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subminingandquarrying(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfoodbeveragesandtobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfoodbeveragesandtobacco(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextilesandtextileproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextilesandtextileproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleatherleatherandfootwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleatherleatherandfootwear(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwoodandproductsofwooodandcork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwoodandproductsofwooodandcork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulppaperprintingandpublishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulppaperprintingandpublishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcokerefinedpetroleumandnuclearfuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcokerefinedpetroleumandnuclearfuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicalsandchemicalproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicalsandchemicalproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubberandplastics(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubberandplastics(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothernonmetalicmineral(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothernonmetalicmineral(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasicmetalsandfabricatedmetal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasicmetalsandfabricatedmetal(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinerynec(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinerynec(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricalandopticalequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricalandopticalequipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransportequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransportequipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturingnecrecycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturingnecrecycling(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricitygasandwatersupply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricitygasandwatersupply(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotelsandrestaurants(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotelsandrestaurants(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinlandtransport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinlandtransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwatertransport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwatertransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subairtransport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subairtransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpostandtelecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpostandtelecommunications(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancialintermedation(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancialintermedation(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrealestateactivities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrealestateactivities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrentingodmeqandotherbusinessactivities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrentingodmeqandotherbusinessactivities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublicadminanddefencecompulsorysocialsecurity(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublicadminanddefencecompulsorysocialsecurity(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealthandsocialwork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealthandsocialwork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothercommunitysocialandpersonaservices(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothercommunitysocialandpersonaservices(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivatehouseholdswithemployedpersons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivatehouseholdswithemployedpersons(
            time())],
        [
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subminingandquarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subminingandquarrying(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subminingandquarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subminingandquarrying(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subfoodbeveragesandtobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subfoodbeveragesandtobacco(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextilesandtextileproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextilesandtextileproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subleatherleatherandfootwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subleatherleatherandfootwear(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwoodandproductsofwooodandcork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwoodandproductsofwooodandcork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulppaperprintingandpublishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulppaperprintingandpublishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subcokerefinedpetroleumandnuclearfuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subcokerefinedpetroleumandnuclearfuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicalsandchemicalproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicalsandchemicalproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubberandplastics(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubberandplastics(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subothernonmetalicmineral(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subothernonmetalicmineral(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasicmetalsandfabricatedmetal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasicmetalsandfabricatedmetal(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinerynec(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinerynec(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricalandopticalequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricalandopticalequipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransportequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransportequipment(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturingnecrecycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturingnecrecycling(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricitygasandwatersupply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricitygasandwatersupply(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotelsandrestaurants(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotelsandrestaurants(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subinlandtransport(time() + 1)
        - historic_final_energy_intensity_by_sector_and_fuel_subgases_subinlandtransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwatertransport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwatertransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subairtransport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subairtransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpostandtelecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpostandtelecommunications(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancialintermedation(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancialintermedation(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subrealestateactivities(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subgases_subrealestateactivities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subrentingodmeqandotherbusinessactivities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subrentingodmeqandotherbusinessactivities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublicadminanddefencecompulsorysocialsecurity(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublicadminanddefencecompulsorysocialsecurity(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealthandsocialwork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealthandsocialwork(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subothercommunitysocialandpersonaservices(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subothercommunitysocialandpersonaservices(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivatehouseholdswithemployedpersons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivatehouseholdswithemployedpersons(
            time())],
        [
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subminingandquarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subminingandquarrying(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subminingandquarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subminingandquarrying(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfoodbeveragesandtobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfoodbeveragesandtobacco(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextilesandtextileproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextilesandtextileproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleatherleatherandfootwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleatherleatherandfootwear(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwoodandproductsofwooodandcork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwoodandproductsofwooodandcork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulppaperprintingandpublishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulppaperprintingandpublishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcokerefinedpetroleumandnuclearfuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcokerefinedpetroleumandnuclearfuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicalsandchemicalproducts(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicalsandchemicalproducts(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubberandplastics(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubberandplastics(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothernonmetalicmineral(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothernonmetalicmineral(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasicmetalsandfabricatedmetal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasicmetalsandfabricatedmetal(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinerynec(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinerynec(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricalandopticalequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricalandopticalequipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransportequipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransportequipment(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturingnecrecycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturingnecrecycling(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricitygasandwatersupply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricitygasandwatersupply(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotelsandrestaurants(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotelsandrestaurants(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinlandtransport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinlandtransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwatertransport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwatertransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subairtransport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subairtransport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpostandtelecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpostandtelecommunications(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancialintermedation(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancialintermedation(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrealestateactivities(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrealestateactivities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrentingodmeqandotherbusinessactivities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrentingodmeqandotherbusinessactivities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublicadminanddefencecompulsorysocialsecurity(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublicadminanddefencecompulsorysocialsecurity(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealthandsocialwork(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealthandsocialwork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothercommunitysocialandpersonaservices(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothercommunitysocialandpersonaservices(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivatehouseholdswithemployedpersons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivatehouseholdswithemployedpersons(
            time())]]
    ret.values = list(map(lambda x: x if type(x) != xr.DataArray else xr.align(ret, x, join='left')[1], data))
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def initial_global_energy_intensity_by_sector_2009():
    """
    Initial global energy intensity by sector 2009



    constant


    """

    ret =  xr.DataArray(
        data=[
            3.86995232, 8.59048794, 2.2730697, 2.0108685, 0.93840275, 3.07986057, 3.88216863,
            11.23922135, 5.78914497, 6.90556672, 13.82397954, 6.66828099, 0.8634489, 0.4932426,
            0.66711798, 4.29142682, 8.77651863, 0.93924068, 0.95827484, 0.73265328, 1.63878385,
            2.61806399, 8.12139811, 23.63826497, 25.08570885, 3.1151168, 0.9165701, 0.378893,
            0.51315756, 0.85824151, 2.28456015, 1.98597538, 1.41782258, 1.73390014, 0.
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def efficiency_energy_aceleration():
    """
    Efficiency energy aceleration



    component


    """

    ret =  maximun_yearly_aceleratuin_of_intensity_improvement(
    ) * pressure_to_improve_energy_intensity_efficiency()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Elec&Heat potential
# HEAT RES - Potential
@cache('step')
def available_pe_potential_solid_bioe_for_heat_ej():
    """
    available PE potential solid bioE for heat EJ



    component


    """

    ret =  np.maximum(
        0, xtotal_pe_solid_bioe_potential_heatxelec_ejx() - pe_bioe_for_elec_generation_ej())
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
@cache('step')
def max_pe_potential_res_for_heat():
    """
    Max PE potential RES for heat



    component


    """

    ret =  xr.DataArray(
        data=[fe_solar_potential_for_heat(),

        geot_pe_potential_for_heat_ej(),

        available_pe_potential_solid_bioe_for_heat_ej()],
        coords={'res heat':['xsolarxheatx','xgeotxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_res_for_heatxcom_by_technx():
    """
    XPES RES for heatXcom by technX



    component


    """

    ret =  xr.DataArray(
        data=[xfe_real_generation_res_heatxcom_ejx().loc[{'res heat':'xgeotxheatx'}].values / efficiency_res_heat().loc[{'res heat':'xgeotxheatx'}].values,

        xfe_real_generation_res_heatxcom_ejx().loc[{'res heat':'xsolarxheatx'}].values * efficiency_solar_panels_for_heat() /
        efficiency_res_heat().loc[{'res heat':'xsolarxheatx'}].values,

        xfe_real_generation_res_heatxcom_ejx().loc[{'res heat':'xsolid bioexheatx'}].values / efficiency_res_heat().loc[{'res heat':'xsolid bioexheatx'}].values],
        coords={'res heat':['xgeotxheatx','xsolarxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
@cache('step')
def p_res_for_heat():
    """
    P RES for heat



    component


    """

    ret =  xr.DataArray(
        data=[p_solar_for_heat(),

        p_geothermal_for_heat(),

        p_solid_bioe_for_heat()],
        coords={'res heat':['xsolarxheatx','xgeotxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfe_real_generation_res_heatxcom_ejx():
    """
    XFE real generation RES heatXcom EJX



    component


    """

    ret =  xpotential_fes_res_for_heatxcom_ejx() * (1 - xres_heatxcom_tot_overcapacityx())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpotential_fes_tot_res_for_heatxcom_ejx():
    """
    Xpotential FES tot RES for heatXcom EJX



    component


    """

    ret =  functions.sum(xpotential_fes_res_for_heatxcom_ejx())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_pe_solid_bioe_potential_heatxelec_ejx():
    """
    XTotal PE solid bioE potential heatXelec EJX



    component


    """

    ret =  max_potential_npp_bioe_conventional_for_heat() + xpe_bioe_residues_for_heatxelec_ejx()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
# RES BIOE - Elec&Heat potential
@cache('step')
def available_potential_fe_solid_bioe_for_elec_ej():
    """
    available potential FE solid bioE for elec EJ



    component


    """

    ret =  available_pe_potential_solid_bioe_for_elec_ej() * efficiency_conversion_bioe_to_elec()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xreplacement_res_for_heatxcom_twx():
    """
    Xreplacement RES for heatXcom TWX



    component


    """

    ret =  xwear_res_capacity_for_heatxcom_twx() * xreplacement_res_for_heatxcomx() * (
        1 - xres_heatxcom_tot_overcapacityx())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def max_bioe_twe():
    """
    max BioE TWe



    component


    """

    ret =  available_potential_fe_solid_bioe_for_elec_ej() * twe_per_twh() / ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
@cache('step')
def remaining_potential_tot_res_heat():
    """
    remaining potential tot RES heat



    component


    """

    ret =  functions.if_then_else(single_precision(max_pe_potential_tot_res_heat_ej()) > single_precision(pes_tot_res_for_heat()) + less_thr_func(max_pe_potential_tot_res_heat_ej(), pes_tot_res_for_heat()),
                                  (max_pe_potential_tot_res_heat_ej() - pes_tot_res_for_heat()) /
                                  max_pe_potential_tot_res_heat_ej(), 0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpotential_fes_heatxcom_nuclear_chp_plants_ejx():
    """
    XPotential FES HeatXcom nuclear CHP plants EJX



    component


    """

    ret =  fe_nuclear_elec_generation_twh(
    ) * share_of_heat_production_in_chp_plants_vs_total_nucelar_elec_generation()
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('run')
def share_of_heat_production_in_chp_plants_vs_total_nucelar_elec_generation():
    """
    share of heat production in CHP plants vs total nucelar elec generation



    constant


    """

    ret =  9.150844475380217e-06
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('step')
def pes_waste_for_elec_plants():
    """
    PES waste for elec plants



    component


    """

    ret =  pes_waste_ej() * share_pes_waste_for_elec_plants()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
# RES BIOE - Biogas
# RES BIOE - Biogas
@cache('step')
def losses_chp_biogas():
    """
    Losses CHP biogas



    component


    """

    ret =  pes_biogas_for_chp() - xfes_heatxcom_from_biogas_in_chp_plantsx(
    ) - fes_elec_from_biogas_in_chp_plants()
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
# WASTE - PES & FES
# WASTE - PES & FES
@cache('step')
def losses_chp_waste():
    """
    Losses CHP waste



    component


    """

    ret =  pes_waste_for_chp_plants() - fes_elec_from_waste_in_chp_plants(
    ) - xfes_heatxcom_from_waste_in_chp_plantsx()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
# HEAT RES - Potential
@cache('step')
def share_pes_biogas_for_heat():
    """
    share PES biogas for heat



    component


    """

    ret =  xpes_tot_biogas_for_heatxcomx() / pes_biogas_ej()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def pe_losses_res_for_elec():
    """
    PE losses RES for elec



    component


    """

    ret =  pe_losses_bioe_for_elec_ej() + pe_losses_biogas_for_elec() + pe_losses_waste_for_elec()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
# WASTE - PES & FES
@cache('step')
def pes_tot_waste_for_elec():
    """
    PES tot waste for elec



    component


    """

    ret =  pes_waste_for_elec_plants() + fes_elec_from_waste_in_chp_plants() + losses_chp_waste(
    ) * share_efficiency_waste_for_elec_in_chp_plants()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_tot_waste_for_heatxcomx():
    """
    XPES tot waste for heatXcomX



    component


    """

    ret =  xpes_waste_for_heatxcom_plantsx() + xfes_heatxcom_from_waste_in_chp_plantsx(
    ) + losses_chp_waste() * (1 - share_efficiency_waste_for_elec_in_chp_plants())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xabundance_res_heatxcom2x():
    """
    Xabundance RES heatXcom2X



    component


    """

    ret =  np.sqrt(xabundance_res_heatxcomx())
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
@cache('step')
def max_pe_potential_biogas_for_heat():
    """
    max PE potential biogas for heat



    component


    """

    ret =  max_biogas_ej() * share_pes_biogas_for_heat()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def share_efficiency_biogas_for_elec_in_chp_plants():
    """
    share efficiency biogas for elec in CHP plants



    component


    """

    ret =  efficiency_biogas_for_elec_chp_plants() / (
        efficiency_biogas_for_elec_chp_plants() + efficiency_biogas_for_heat_chp_plants())
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('step')
def share_efficiency_waste_for_elec_in_chp_plants():
    """
    share efficiency waste for elec in CHP plants



    component


    """

    ret =  efficiency_waste_for_elec_chp_plants() / (
        efficiency_waste_for_elec_chp_plants() + efficiency_waste_for_heat_chp_plants())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmax_potential_pe_nonxelectric_resx():
    """
    XMax potential PE nonXelectric RESX



    component


    """

    ret =  max_pe_potential_tot_res_heat_ej() + max_peavail_biofuels_potential()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC - Electricity related losses
# RES BIOE - Biogas
@cache('step')
def pes_tot_biogas_for_elec():
    """
    PES tot biogas for elec



    component


    """

    ret =  pes_biogas_for_elec_plants() + fes_elec_from_biogas_in_chp_plants() + losses_chp_biogas(
    ) * share_efficiency_biogas_for_elec_in_chp_plants()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_tot_biogas_for_heatxcomx():
    """
    XPES tot biogas for heatXcomX



    component


    """

    ret =  xpes_biogas_for_heatxcom_plantsx() + xfes_heatxcom_from_biogas_in_chp_plantsx(
    ) + losses_chp_biogas() * (1 - share_efficiency_biogas_for_elec_in_chp_plants())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def pe_losses_nre_elec_generation():
    """
    PE losses NRE elec generation



    component


    """

    ret =  pe_losses_coal_for_elec_ej() + pe_losses_conv_gas_for_elec_ej(
    ) + pe_losses_oil_for_elec_ej() + pe_losses_uncon_gas_for_elec_ej(
    ) + pe_losses_uranium_for_elec_ej()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def elec_gen_related_losses_ej():
    """
    Elec gen related losses EJ



    component


    """

    ret =  pe_losses_nre_elec_generation() + pe_losses_res_for_elec()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def pe_losses_biogas_for_elec():
    """
    PE losses biogas for elec



    component


    """

    ret =  pes_tot_biogas_for_elec() - fes_elec_from_biogas_ej()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def pe_losses_waste_for_elec():
    """
    PE losses waste for elec



    component


    """

    ret =  pes_tot_waste_for_elec() - fes_elec_from_waste_ej()
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
# RES - PED from non-electric RES
@cache('step')
def max_pe_potential_tot_res_heat_ej():
    """
    max PE potential tot RES heat EJ



    component


    """

    ret =  max_pe_potential_biogas_for_heat() + functions.sum(max_pe_potential_res_for_heat())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
# RES BIOE - Biogas
@cache('step')
def share_pes_biogas_for_elec():
    """
    share PES biogas for elec



    component


    """

    ret =  pes_tot_biogas_for_elec() / pes_biogas_ej()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def p_hydro_growth():
    """
    P hydro growth



    constant


    """

    ret =  0.028
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES - Capacities and generation all techn
@cache('step')
def fe_real_tot_generation_res_elec_twh():
    """
    FE real tot generation RES elec TWh



    component


    """

    ret =  np.minimum(
        np.maximum(total_fe_elec_demand_after_priorities_twh(), 0),
        potential_tot_generation_res_elec_twh())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def p_solar_pv_growth():
    """
    P solar PV growth



    constant


    """

    ret =  0.15
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
@cache('step')
def remaining_potential_constraint_on_new_res_heat_capacity():
    """
    remaining potential constraint on new RES heat capacity



    component


    """

    ret =  functions.if_then_else(
        remaining_potential_res_for_heat() > threshold_remaining_potential_new_capacity(), 1,
        remaining_potential_res_for_heat() * (1 / threshold_remaining_potential_new_capacity()))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
@cache('step')
def total_fe_elec_generation_twh():
    """
    Total FE Elec generation TWh



    component


    """

    ret =  fe_elec_generation_from_nre_twh() + fe_tot_generation_all_res_elec_twh(
    ) + fes_elec_from_waste_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES - Potential
# ELEC - Total FE electricity generation
# TFES by fuel
# COMP- Historical data Elec production
@cache('step')
def fe_tot_generation_all_res_elec_twh():
    """
    FE tot generation all RES elec TWh



    component


    """

    ret =  fe_real_tot_generation_res_elec_twh() + fes_elec_from_res_with_priority_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def p_oceanic_growth():
    """
    P oceanic growth



    constant


    """

    ret =  0.1
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# RES - PES
# TPED by fuel
@cache('step')
def pe_elec_generation_from_res_ej():
    """
    PE Elec generation from RES EJ



    component


    """

    ret =  pe_bioe_for_elec_generation_ej() + xpe_geotxelec_for_elec_generation_ejx(
    ) + pe_hydro_for_elec_generation_ej() + pe_oceanic_for_elec_generation_ej(
    ) + pe_solar_pv_for_elec_generation_ej() + pe_csp_for_elec_generation_ej(
    ) + pe_onshore_wind_for_elec_generation_ej() + pe_offshore_wind_for_elec_generation_ej(
    ) + pes_tot_biogas_for_elec()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def p_res_elec_growth():
    """
    P RES elec growth



    component


    """

    ret =  xr.DataArray(
        data=[p_hydro_growth(),

        p_geot_growth(),

        xp_solid_bioexelec_growthx(),

        p_oceanic_growth(),

        p_wind_onshore_growth(),

        p_wind_offshore_growth(),

        p_solar_pv_growth(),

        p_csp_growth()],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
# ELEC - Total FE electricity generation
@cache('step')
def share_res_electricity_generation():
    """
    share RES electricity generation



    component


    """

    ret =  fe_tot_generation_all_res_elec_twh() / total_fe_elec_generation_twh()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xp_solid_bioexelec_growthx():
    """
    XP solid bioEXelec growthX



    constant


    """

    ret =  0.072
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def p_csp_growth():
    """
    P CSP growth



    constant


    """

    ret =  0.1
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def remaining_potential_tot_res_elec():
    """
    remaining potential tot RES elec



    component


    """

    ret =  functions.if_then_else(single_precision(max_potential_tot_res_elec_twh()) > single_precision(fe_tot_generation_all_res_elec_twh()) + less_thr_func(max_potential_tot_res_elec_twh(), fe_tot_generation_all_res_elec_twh()),
        (max_potential_tot_res_elec_twh() - fe_tot_generation_all_res_elec_twh()) /
        max_potential_tot_res_elec_twh(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def p_wind_offshore_growth():
    """
    P wind offshore growth



    constant


    """

    ret =  0.15
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def p_wind_onshore_growth():
    """
    P wind onshore growth



    constant


    """

    ret =  0.15
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xelec_generation_basexload_from_res_twhx():
    """
    XElec generation baseXload from RES TWhX



    component


    """

    ret =  fe_elec_generation_from_bioe_twh() + xfe_elec_generation_from_geotxelec_twhx(
    ) + fe_elec_generation_from_hydro_twh() + fe_elec_generation_from_oceanic_twh(
    ) + fes_elec_from_biogas_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC - RES elec supply
@cache('step')
def fes_elec_from_res_with_priority_twh():
    """
    FES elec from RES with priority TWh



    component


    """

    ret =  fes_elec_from_biogas_twh()
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_renewables_elec():
    """
    share Renewables Elec



    component


    """

    ret =  fe_tot_generation_all_res_elec_twh() / total_fe_elec_demand_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def abundance_res_elec():
    """
    abundance RES elec



    component


    """

    ret =  functions.if_then_else(
        abs(total_fe_elec_demand_after_priorities_twh() ) < equality_threshold, 0,
        functions.if_then_else(
            total_fe_elec_demand_after_priorities_twh() > fe_real_tot_generation_res_elec_twh(),
            (total_fe_elec_demand_after_priorities_twh() - fe_real_tot_generation_res_elec_twh()) /
            total_fe_elec_demand_after_priorities_twh(), 0))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
@cache('step')
def fes_elec_from_biow():
    """
    FES elec from BioW



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values + fes_elec_from_biogas_twh() + fes_elec_from_waste_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def p_geot_growth():
    """
    P geot growth



    constant


    """

    ret =  0.024
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def max_potential_tot_res_elec_twh():
    """
    max potential tot RES elec TWh



    component


    """

    ret =  functions.sum(max_potential_res_elec_twh()) + max_potential_phs_twe() / twe_per_twh(
    ) + max_biogas_ej() * share_pes_biogas_for_elec() / ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
@cache('run')
def p_solid_bioe_for_heat():
    """
    P solid bioE for heat



    constant


    """

    ret =  0.063
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
@cache('step')
def pe_real_generation_res_elec():
    """
    PE real generation RES elec



    component


    """

    ret =  xr.DataArray(
        data=[real_generation_res_elec_twh().loc[{'res elec':'hydro'}].values * ej_per_twh() * res_to_fossil_accounting(),

        real_generation_res_elec_twh().loc[{'res elec':'xgeotxelecx'}].values * ej_per_twh() * res_to_fossil_accounting(),

        (real_generation_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values / efficiency_conversion_bioe_to_elec()) * ej_per_twh(),

        real_generation_res_elec_twh().loc[{'res elec':'oceanic'}].values * ej_per_twh() * res_to_fossil_accounting(),

        real_generation_res_elec_twh().loc[{'res elec':'wind onshore'}].values * ej_per_twh() * res_to_fossil_accounting(),

        real_generation_res_elec_twh().loc[{'res elec':'wind offshore'}].values * ej_per_twh() * res_to_fossil_accounting(),

        real_generation_res_elec_twh().loc[{'res elec':'solar pv'}].values * ej_per_twh() * res_to_fossil_accounting(),

        real_generation_res_elec_twh().loc[{'res elec':'csp'}].values * ej_per_twh() * res_to_fossil_accounting()],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_renewables_elec_check():
    """
    share Renewables Elec check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (share_renewables_elec() -
         (share_renewables_elec_iea(time()) + share_renewables_elec_chp_iea(time()))) * 100 /
        (share_renewables_elec_iea(time()) + share_renewables_elec_chp_iea(time())), 0)
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_renewables_heat_check():
    """
    share renewables Heat check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (share_renewables_heat_ej() -
         (share_renewables_heat_iea_ej(time()) + share_renewables_fuel_chp_iea_ej(time()))) * 100 /
        (share_renewables_heat_iea_ej(time()) + share_renewables_fuel_chp_iea_ej(time())), 0)
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
@cache('step')
def efficiency_res_heat():
    """
    Efficiency RES heat



    component


    """

    ret =  xr.DataArray(
        data=[efficiency_solar_panels_for_heat() * losses_solar_for_heat(),

        efficiency_geothermal_for_heat(),

        efficiency_conversion_bioe_plants_to_heat()],
        coords={'res heat':['xsolarxheatx','xgeotxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_heatxcom_by_nre_chp_plants_ejx():
    """
    XFED heatXcom by NRE CHP plants EJX



    component


    """

    ret =  xshare_heatxcom_chp_plants_nre_vs_nre_tot_heatxcom_generationx(
        time()) * xfed_heatxcom_nre_ejx()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xconstant_final_energy_intensity_sectorsxhouseholds_tdxx():
    """
    XConstant final energy intensity sectorsXhouseholds TDXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_oil_dem_for_heatxcomx():
    """
    Xshare oil dem for HeatXcomX



    component


    """

    ret =  functions.if_then_else(single_precision(ped_total_oil_ej()) > single_precision(0) + less_thr_func(ped_total_oil_ej(), 0),
                                  ped_oil_for_heat_plants_ej() / ped_total_oil_ej(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
# EROItot - Electricity
@cache('step')
def fe_elec_generation_from_total_oil_twh():
    """
    FE Elec generation from total oil TWh



    component


    """

    ret =  pes_oil_ej() * share_oil_dem_for_elec() * efficiency_liquids_for_electricity(
    ) / ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
@cache('step')
def total_fe_heat_generation_ej():
    """
    Total FE Heat generation EJ



    component


    """

    ret =  fes_res_for_heat_ej() + xfes_heatxcom_from_waste_ejx() + fes_nre_for_heat()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpe_supply_res_nonxelec_ejx():
    """
    XPE supply RES nonXElec EJX



    component


    """

    ret =  xpe_supply_from_res_nonxelec_without_trad_bioe_ejx(
    ) + pe_traditional_biomass_ej_delayed_1yr() + losses_in_charcoal_plants_ej(time())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
# SOLIDS - PED, PES & FES
@cache('step')
def share_coal_dem_for_elec():
    """
    share coal dem for Elec



    component


    """

    ret =  functions.if_then_else(single_precision(ped_coal_ej()) > single_precision(0) + less_thr_func(ped_coal_ej(), 0),
                                  pe_demand_coal_elec_plants_ej() / ped_coal_ej(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def pe_losses_oil_for_elec_ej():
    """
    PE losses oil for Elec EJ



    component


    """

    ret =  pes_oil_ej() * share_oil_dem_for_elec() * (1 - efficiency_liquids_for_electricity())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
# LIQUIDS - PED, PES & FES
@cache('step')
def share_oil_dem_for_elec():
    """
    share oil dem for Elec



    component


    """

    ret =  functions.if_then_else(single_precision(ped_total_oil_ej()) > single_precision(0) + less_thr_func(ped_total_oil_ej(), 0),
                                  pe_demand_oil_elec_plants_ej() / ped_total_oil_ej(), 0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_coal_dem_for_heatxcomx():
    """
    Xshare coal dem for HeatXcomX



    component


    """

    ret =  functions.if_then_else(single_precision(ped_coal_ej()) > single_precision(0) + less_thr_func(ped_coal_ej(), 0),
                                  ped_coal_for_heat_plants_ej() / ped_coal_ej(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
# EROItot - Electricity
@cache('step')
def fe_elec_generation_from_conv_gas_twh():
    """
    FE Elec generation from conv gas TWh



    component


    """

    ret =  real_extraction_conv_gas_ej() * xshare_natx_gas_dem_for_elecx(
    ) * efficiency_gas_for_electricity() / ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
@cache('step')
def share_res_heat_generation():
    """
    share RES heat generation



    component


    """

    ret =  fes_res_for_heat_ej() / total_fe_heat_generation_ej()
    name = 1
    return round_sig(x=ret, name=name)

# GASES - PED, PES & FES
# ABUND - Abundances by sector
@cache('step')
def share_gas_for_transp():
    """
    share gas for Transp



    component


    """

    ret =  functions.if_then_else(single_precision(ped_gases()) > single_precision(0) + less_thr_func(ped_gases(), 0), demand_gas_for_transp_ej() / ped_gases(), 0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_natx_gas_dem_for_elecx():
    """
    Xshare natX gas dem for ElecX



    component


    """

    ret =  functions.if_then_else(single_precision(xped_natx_gas_ejx()) > single_precision(0) + less_thr_func(xped_natx_gas_ejx(), 0),
                                  pe_demand_gas_elec_plants_ej() / xped_natx_gas_ejx(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def pe_losses_conv_gas_for_elec_ej():
    """
    PE losses conv gas for Elec EJ



    component


    """

    ret =  real_extraction_conv_gas_ej() * xshare_natx_gas_dem_for_elecx() * (
        1 - efficiency_gas_for_electricity())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def pe_losses_coal_for_elec_ej():
    """
    PE losses coal for Elec EJ



    component


    """

    ret =  extraction_coal_ej() * share_coal_dem_for_elec() * (
        1 - efficiency_coal_for_electricity())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
# EROItot - Electricity
@cache('step')
def fe_elec_generation_from_coal_twh():
    """
    FE Elec generation from coal TWh



    component


    """

    ret =  extraction_coal_ej() * efficiency_coal_for_electricity() * share_coal_dem_for_elec(
    ) / ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_natx_gas_dem_for_heatxcomx():
    """
    Xshare natX gas dem for HeatXcomX



    component


    """

    ret =  functions.if_then_else(single_precision(xped_natx_gas_ejx()) > single_precision(0) + less_thr_func(xped_natx_gas_ejx(), 0),
                                  ped_gases_for_heat_plants_ej() / xped_natx_gas_ejx(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# HEAT - Total FE heat generation
@cache('step')
def year_scarcity_heat():
    """
    Year scarcity Heat



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_heat()) > single_precision(0.95) + less_thr_func(abundance_heat(), 0.95), 0, time())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_heatxcom_from_biogas_in_chp_plantsx():
    """
    XFES heatXcom from biogas in CHP plantsX



    component


    """

    ret =  pes_biogas_for_chp() * efficiency_biogas_for_heat_chp_plants()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('run')
def efficiency_biogas_for_heat_plants():
    """
    efficiency biogas for heat plants



    constant


    """

    ret =  0.7357016523145766
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def fes_elec_from_biogas_in_chp_plants():
    """
    FES elec from biogas in CHP plants



    component


    """

    ret =  pes_biogas_for_chp() * efficiency_biogas_for_elec_chp_plants()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
# RES BIOE - Biogas
@cache('step')
def fes_elec_from_biogas_ej():
    """
    FES elec from biogas EJ



    component


    """

    ret =  fes_elec_from_biogas_in_chp_plants() + fes_elec_from_biogas_in_elec_plants()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_biogas_for_heatxcom_plantsx():
    """
    XPES biogas for heatXcom plantsX



    component


    """

    ret =  pes_biogas_ej() * xshare_pes_biogas_for_heatxcom_plantsx()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def fes_elec_from_biogas_in_elec_plants():
    """
    FES elec from biogas in elec plants



    component


    """

    ret =  pes_biogas_for_elec_plants() * efficiency_biogas_for_elec_plants()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('run')
def efficiency_biogas_for_elec_plants():
    """
    efficiency biogas for elec plants



    constant


    """

    ret =  0.29585066538416954
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_biogas_for_heatxcom_plantsx():
    """
    XFES biogas for heatXcom plantsX



    component


    """

    ret =  xpes_biogas_for_heatxcom_plantsx() * efficiency_biogas_for_heat_plants()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def pes_biogas_for_elec_plants():
    """
    PES biogas for elec plants



    component


    """

    ret =  pes_biogas_ej() * share_pes_biogas_for_elec_plants()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('run')
def share_pes_biogas_tfc():
    """
    share PES biogas TFC



    constant


    """

    ret =  0.4098230262379001
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def potential_pes_biogas_for_tfc():
    """
    Potential PES biogas for TFC



    component


    """

    ret =  pes_biogas_ej() * share_pes_biogas_tfc()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC RES: Overcapacities due to RES variability a
# ELEC - Total FE electricity generation
# RES BIOE - Biogas
@cache('step')
def fes_elec_from_biogas_twh():
    """
    FES elec from biogas TWh



    component


    """

    ret =  fes_elec_from_biogas_ej() / ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def pes_biogas_for_chp():
    """
    PES biogas for CHP



    component


    """

    ret =  pes_biogas_ej() * share_pes_biogas_for_chp()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xshare_pes_biogas_for_heatxcom_plantsx():
    """
    Xshare PES biogas for heatXcom plantsX



    constant


    """

    ret =  0.007384519814561412
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('run')
def efficiency_biogas_for_elec_chp_plants():
    """
    efficiency biogas for elec CHP plants



    constant


    """

    ret =  0.3901372793044364
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_heatxcom_from_biogas_ejx():
    """
    XFES heatXcom from biogas EJX



    component


    """

    ret =  xfes_biogas_for_heatxcom_plantsx() + xfes_heatxcom_from_biogas_in_chp_plantsx()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('run')
def efficiency_biogas_for_heat_chp_plants():
    """
    efficiency biogas for heat CHP plants



    constant


    """

    ret =  0.07955076327527948
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('run')
def share_pes_biogas_for_elec_plants():
    """
    share PES biogas for elec plants



    constant


    """

    ret =  0.34932337340295677
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('run')
def share_pes_biogas_for_chp():
    """
    share PES biogas for CHP



    constant


    """

    ret =  0.23346908054458176
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('step')
def demand_elec_plants_fossil_fuels_twh():
    """
    demand Elec plants fossil fuels TWh



    component


    """

    ret =  np.maximum(demand_elec_nre_twh() - fe_nuclear_elec_generation_twh() -
                      fes_elec_fossil_fuel_chp_plants_twh(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def adapt_growth_biogas():
    """
    adapt growth biogas



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        past_biogas_growth(),
        functions.if_then_else(time() < 2020,
                               past_biogas_growth() + (growth_biogas() - past_biogas_growth()) *
                               (time_dmnl(time()) - 2015) / 5, growth_biogas()))
    name = 1
    return round_sig(x=ret, name=name)

# TPES
@cache('step')
def total_extraction_nre_ej():
    """
    Total extraction NRE EJ



    component


    """

    ret =  extraction_coal_ej() + real_extraction_conv_gas_ej() + real_extraction_conv_oil_ej(
    ) + real_extraction_unconv_gas_ej() + real_extraction_unconv_oil_ej() + extraction_uranium_ej(
    )
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def new_pes_biogas():
    """
    new PES biogas



    component


    """

    ret =  functions.if_then_else(time() < 2014,
                                  (historic_biogas_pes(time() + 1) - historic_biogas_pes(time())),
                                  ((max_biogas_ej() - pes_biogas_ej()) / max_biogas_ej()) *
                                  adapt_growth_biogas() * pes_biogas_ej())
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def energy_cost_pressure():
    """
    Energy cost pressure



    component


    """

    ret =  1 - abundance_of_energy()
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def pressure_to_improve_energy_intensity_efficiency():
    """
    Pressure to improve energy intensity efficiency



    component


    """

    ret =  energy_cost_pressure() + implementation_policy_to_improve_energy_intensity_effciency()
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def abundance_of_energy():
    """
    abundance of energy



    component


    """

    ret =  xr.DataArray(
        data=[abundance_liquids(),

        abundance_gases(),

        abundance_solids(),
        1.0,
        1.0],
        coords={'final sources':['liquids','gases','solids','electricity','heat']},
        dims=['final sources'])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def energy_market_pressure():
    """
    Energy market pressure



    component


    """

    ret =  0 * relative_abundance_of_energy()
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def relative_abundance_of_energy():
    """
    relative abundance of energy



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def pressure_to_change_energy_technology():
    """
    Pressure to change energy technology



    component


    """

    ret =  np.minimum(
        np.maximum(energy_market_pressure() + implementatio_policy_to_change_final_energy(), 0), 1)
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('step')
def remaining_recycling_rate_improv_for_alt_techn_per_mineral():
    """
    remaining recycling rate improv for alt techn per mineral



    component


    """

    ret =  functions.zidz(max_recycling_rates_minerals() - recycling_rates_minerals_alt_techn(),
                          max_recycling_rates_minerals())
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('step')
def remaining_recycling_rate_improv_rest_per_mineral():
    """
    remaining recycling rate improv Rest per mineral



    component


    """

    ret =  functions.zidz(max_recycling_rates_minerals() - recycling_rates_minerals_rest(),
                          max_recycling_rates_minerals())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xced_oxm_over_lifetime_per_material_res_elec_varx():
    """
    XCED OXM over lifetime per material RES elec varX



    component


    """

    ret =  res_elec_capacity_under_construction_tw(
    ) * xmaterials_for_oxm_per_capacity_installed_res_elecx(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * lifetime_res_elec() * (
        m_per_t() / kg_per_mt()) * (kg_per_mt() / mj_per_ej())
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
# EROI - RES elec
@cache('step')
def energy_cons_per_unit_of_material_cons_for_res_elec():
    """
    Energy cons per unit of material cons for RES elec



    component


    """

    ret =  recycling_rates_minerals_alt_techn(
    ) * xinitial_energy_cons_per_unit_of_material_cons_xrecycledxx() + (
        1 - recycling_rates_minerals_alt_techn()
    ) * xinitial_energy_cons_per_unit_of_material_cons_xvirginxx()
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('step')
def ced_new_cap_per_material_res_elec_var():
    """
    CED new cap per material RES elec var



    component


    """

    ret =  materials_required_for_new_res_elec_mt(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
@cache('step')
def energy_required_for_material_consumption_for_new_res_elec():
    """
    Energy required for material consumption for new RES elec



    component


    """

    ret =  materials_required_for_new_res_elec_mt(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xenergy_required_for_material_consumption_for_oxm_res_elecx():
    """
    XEnergy required for material consumption for OXM RES elecX



    component


    """

    ret =  xmaterials_required_for_oxm_res_elec_mtx(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def share_other_cumulative_demand_to_extract_vs_resources_materials():
    """
    share other cumulative demand to extract vs resources materials



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(current_mineral_resources_mt()) - less_thr_func(0, current_mineral_resources_mt()), 0,
        cum_materials_to_extract_rest_from_2015() / current_mineral_resources_mt())
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('step')
def materials_to_extract_rest_from_2015_mt():
    """
    Materials to extract Rest from 2015 Mt



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, materials_to_extract_rest_mt())
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def total_recycled_materials_for_res_elec_mt():
    """
    Total recycled materials for RES elec Mt



    component


    """

    ret =  total_materials_required_for_res_elec_mt() - total_materials_to_extract_for_res_elec_mt(
    )
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('step')
def cum_materials_to_extract_rest():
    """
    cum materials to extract Rest



    component


    """


    ret =  integ_cum_materials_to_extract_rest()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def cum_materials_to_extract_rest_from_2015():
    """
    cum materials to extract Rest from 2015



    component


    """


    ret =  integ_cum_materials_to_extract_rest_from_2015()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def share_other_cumulative_demand_to_extract_vs_reserves_materials():
    """
    share other cumulative demand to extract vs reserves materials



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(current_mineral_reserves_mt()) - less_thr_func(0, current_mineral_reserves_mt()), 0,
        cum_materials_to_extract_rest_from_2015() / current_mineral_reserves_mt())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmaterials_availability_xannual_extractionxx():
    """
    Xmaterials availability Xannual extractionXX



    component


    """

    ret =  functions.if_then_else(single_precision(maximum_annual_extraction_materials()) > single_precision(total_materials_to_extract_mt()) + less_thr_func(maximum_annual_extraction_materials(), total_materials_to_extract_mt()), 1, 0)
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('run')
def mt_per_tonne():
    """
    Mt per tonne



    constant


    """

    ret =  1e-06
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Rest demand
@cache('run')
def initial_cumulated_material_requirements_for_rest_1995():
    """
    initial cumulated material requirements for Rest 1995



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('run')
def p_recycling_minerals_rest():
    """
    P recycling minerals Rest



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def cum_materials_to_extract_for_res_elec_from_2015():
    """
    cum materials to extract for RES elec from 2015



    component


    """


    ret =  integ_cum_materials_to_extract_for_res_elec_from_2015()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def total_materials_to_extract_for_res_elec_mt():
    """
    Total materials to extract for RES elec Mt



    component


    """

    ret =  total_materials_required_for_res_elec_mt() * (1 - recycling_rates_minerals_alt_techn())
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def cum_materials_to_extract_for_res_elec():
    """
    cum materials to extract for RES elec



    component


    """


    ret =  integ_cum_materials_to_extract_for_res_elec()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def total_materials_to_extract_for_res_elec_from_2015_mt():
    """
    Total materials to extract for RES elec from 2015 Mt



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0, total_materials_to_extract_for_res_elec_mt())
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('run')
def max_recycling_rates_minerals():
    """
    Max recycling rates minerals



    constant


    """

    ret =  0.95
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('run')
def p_recycling_minerals_alt_techn():
    """
    P recycling minerals alt techn



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
# MATERIALS - Rest demand
@cache('run')
def current_recycling_rates_minerals():
    """
    current recycling rates minerals



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[0.56], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.15], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.9], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[0.48], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.005], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.005], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[0.71], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.005], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.39], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.53], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.3], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[0.6], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.735], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.635], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.75], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.005], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.91], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.005], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[0.395], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Recycling & material extraction deman
@cache('run')
def historic_improvement_recycling_rates_minerals():
    """
    Historic improvement recycling rates minerals



    constant


    """

    ret =  xr.DataArray(
        data=[
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.
        ],
        coords={
            'materials': [
                'adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement',
                'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx',
                'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass',
                'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt',
                'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese',
                'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic',
                'neodymium', 'nickel', 'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead',
                'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand',
                'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel',
                'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires',
                'zinc'
            ]
        },
        dims=['materials'])
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def min_energy_intensity_vs_intial():
    """
    min energy intensity vs intial



    constant


    """

    ret =  0.3
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def global_energy_intensity_by_sector():
    """
    Global energy intensity by sector



    component


    """

    ret =  functions.sum(dim='final sources', data=evol_final_energy_intensity_by_sector_and_fuel())
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def historical_mean_rate_energy_intensity():
    """
    historical mean rate energy intensity



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[-0.00594331, -0.12011212, -0.01702233, 0.00649193, -0.04199447]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.01292442, -0.04518246, -0.01034553, -0.00468514, 0.01253506]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.00400537, -0.03138628, -0.01180247, 0.00788062, -0.00678344]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00992662, -0.01949561, -0.04541047, -0.01309037, -0.06194968]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.02023598, -0.04979546, -0.03162812, -0.02367399, -0.04577093]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.01776254, -0.12490597, -0.02823126, 0.01183321, -0.00733869]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00950341, 0.05313765, -0.04045002, -0.01206145, 0.00749482]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.01886605, -0.01847262, -0.03188427, -0.02840818, -0.00295949]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.01281968, -0.02254765, -0.02264326, -0.01543831, -0.04863373]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00156569, -0.04884816, -0.01991446, -0.01145573, -0.01683848]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01540775, -0.01630938, -0.01400116, 0.0066328, -0.00025385]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.00733569, -0.05521067, -0.03842037, -0.01299638, -0.00274809]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.01479467, -0.13612799, -0.05007249, -0.00896218, -0.08810427]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.04147397, -0.14823739, -0.07540414, -0.04217459, -0.12148873]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00904026, 0.02069871, -0.03899174, 0.0007079, -0.03058529]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00569454, -0.04044161, -0.02169462, -0.00191565, -0.05464288]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.00532142, 0.00891843, -0.03679899, -0.02325057, 0.05228512]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00841393, -0.31124022, -0.00453141, 0.00490538, 0.0093747]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.00605911, -0.01422534, -0.02190432, -0.01092198, -0.17006181]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00036395, -0.01905378, -0.04133912, -0.02730321, -0.07895893]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00386347, -0.01325683, -0.02968859, -0.00458373, -0.09655175]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01980806, -0.01257244, -0.03131294, 0.05258562, -0.02234642]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00404563, 0., 0.00290517, -0.00873679, -0.06650353]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., -0.03742717, -0.04770914, -0.07070365]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., -0.00469314, -0.01226672, -0.73407054]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01034127, -0.18672971, 0.01036057, -0.0406507, -0.09713694]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01239715, -0.04082706, -0.04085653, -0.1071439, -0.03762176]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.00447336, -0.1398287, -0.04749345, -0.01013251, -0.34654798]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[-0.01637684, -0.04922435, -0.02508782, -0.01725737, -0.18054948]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.00418362, -0.02366548, -0.03195417, -0.00052349, -0.01660373]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01668083, -0.02455965, -0.02219986, -0.05911978, 0.00105572]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.03481818, -0.00483609, -0.00077546, 0.00180503, -0.07266663]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01151148, -0.02092861, -0.025051, -0.00719733, 0.03300624]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.01105651, -0.05757374, -0.01504249, -0.02016622, -0.01382198]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xinitial_energy_cons_per_unit_of_material_cons_xrecycledxx():
    """
    XInitial energy cons per unit of material cons XrecycledXX



    component


    """

    ret =  functions.if_then_else(
        abs(xinitial_energy_cons_per_unit_of_material_cons_xrecycledx_x_datax() ) < equality_threshold,
        xinitial_energy_cons_per_unit_of_material_cons_xvirginxx(),
        xinitial_energy_cons_per_unit_of_material_cons_xrecycledx_x_datax())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmaterials_required_for_oxm_res_elec_mtx():
    """
    Xmaterials required for OXM RES elec MtX



    component


    """

    ret =  installed_capacity_res_elec_tw() * xmaterials_for_oxm_per_capacity_installed_res_elecx(
    ) * m_per_t() / kg_per_mt()
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('run')
def initial_energy_intensity_by_fuel_and_sector_1995():
    """
    Initial energy intensity by fuel and sector 1995



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[[0.72419715, 0.1779432, 3.32933712, 0.21503569, 0.47086768]],
            coords={
                'sectors': ['agriculture hunting forestry and fishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[1.563491, 0.39615757, 1.34136258, 4.69078138, 0.95068866]],
            coords={
                'sectors': ['mining and quarrying'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.38725897, 0.16496473, 0.51411063, 0.26307121, 1.04794477]],
            coords={
                'sectors': ['food beverages and tobacco'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.80985958, 0.2208637, 0.88069214, 0.22170153, 0.99195523]],
            coords={
                'sectors': ['textiles and textile products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.35139111, 0.11953127, 0.54391221, 0.09354931, 0.36017316]],
            coords={
                'sectors': ['leather leather and footwear'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.87290452, 0.87939526, 0.85318442, 0.16492388, 1.54358467]],
            coords={
                'sectors': ['wood and products of woood and cork'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[1.22637684, 0.10098146, 0.77252486, 0.4924435, 1.41596642]],
            coords={
                'sectors': ['pulp paper printing and publishing'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.91655574, 0.83062554, 11.72311084, 3.00131309, 0.56351885]],
            coords={
                'sectors': ['coke refined petroleum and nuclear fuel'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[1.80355296, 0.85843673, 1.61410459, 1.47202838, 1.69033825]],
            coords={
                'sectors': ['chemicals and chemical products'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[1.75167552, 0.45484858, 2.07496676, 1.02938173, 2.73744036]],
            coords={
                'sectors': ['rubber and plastics'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[1.60100296, 0.11474365, 2.19605512, 1.34343882, 7.9972297]],
            coords={
                'sectors': ['other non metalic mineral'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[1.97761559, 0.4444717, 0.72411951, 1.31080652, 2.8148349]],
            coords={
                'sectors': ['basic metals and fabricated metal'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.38497879, 0.26636943, 0.42382675, 0.12021542, 0.34965201]],
            coords={
                'sectors': ['machinery nec'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.38811218, 0.10697241, 0.47801984, 0.11143834, 0.08366552]],
            coords={
                'sectors': ['electrical and optical equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.29157176, 0.03230247, 0.2833001, 0.0917108, 0.11490861]],
            coords={
                'sectors': ['transport equipment'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.85691698, 0.25247423, 2.14392313, 0.63619042, 2.12600143]],
            coords={
                'sectors': ['manufacturing nec recycling'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[2.62405852, 0.17656446, 0.64790005, 3.92239687, 1.30847508]],
            coords={
                'sectors': ['electricity gas and water supply'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.10596937, 0.08379637, 0.70681406, 0.05766783, 0.08061731]],
            coords={
                'sectors': ['construction'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.29023907, 0.0343743, 0.65161055, 0.12338712, 0.02572129]],
            coords={
                'sectors': [
                    'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.19819158, 0.04237136, 0.79950961, 0.08311736, 0.03537462]],
            coords={
                'sectors':
                ['wholesale trade and commissions trade except of motor vehicles and motorcycles'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.49852024, 0.03854825, 1.44622456, 0.18343785, 0.03731645]],
            coords={
                'sectors': [
                    'retail trade except of motor vehicles and motorcycles repair of household goods'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.63162597, 0.05421857, 1.41963918, 0.22968876, 0.47227027]],
            coords={
                'sectors': ['hotels and restaurants'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.44834521, 0., 5.78459465, 1.76868907, 0.19289284]],
            coords={
                'sectors': ['inland transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.00000000e+00, 0.00000000e+00, 3.79341609e+01, 4.57484716e-06,
                   2.02575499e-02]],
            coords={
                'sectors': ['water transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.00000000e+00, 0.00000000e+00, 2.67157318e+01, 1.98484033e-05,
                   0.00000000e+00]],
            coords={
                'sectors': ['air transport'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.3351251, 0.24766114, 2.12294444, 0.15920033, 0.04933503]],
            coords={
                'sectors': [
                    'other supporting and auxiliary transport activities activities of travel agencies'
                ],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.21424532, 0.03630473, 0.76886647, 0.55711416, 0.00880215]],
            coords={
                'sectors': ['post and telecommunications'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.13571294, 0.03042315, 0.39026415, 0.04635998, 0.01496297]],
            coords={
                'sectors': ['financial intermedation'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.31331707, 0.06748503, 0.14099642, 0.08985637, 0.05228868]],
            coords={
                'sectors': ['real estate activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.18155751, 0.02293191, 0.83315094, 0.0977231, 0.02983079]],
            coords={
                'sectors': ['renting od meq and other business activities'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.40839952, 0.05140637, 1.18805417, 1.53739517, 0.07497031]],
            coords={
                'sectors': ['public admin and defence compulsory social security'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.47589912, 0.07013227, 0.85574009, 0.19922448, 0.19199273]],
            coords={
                'sectors': ['education'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.33605505, 0.04920999, 1.08999354, 0.16141075, 0.05863355]],
            coords={
                'sectors': ['health and social work'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0.38788798, 0.11996459, 1.14939878, 0.2435464, 0.06819527]],
            coords={
                'sectors': ['other community social and persona services'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
        xr.DataArray(
            data=[[0., 0., 0., 0., 0.]],
            coords={
                'sectors': ['private households with employed persons'],
                'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']
            },
            dims=['sectors', 'final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def share_tot_cum_dem_vs_reserves_materials():
    """
    share tot cum dem vs reserves materials



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(current_mineral_reserves_mt()) - less_thr_func(0, current_mineral_reserves_mt()), 0,
        total_cumulative_demand_materials_to_extract_from_2015() / current_mineral_reserves_mt())
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('step')
def share_tot_cum_dem_vs_resources_materials():
    """
    share tot cum dem vs resources materials



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(current_mineral_resources_mt()) - less_thr_func(0, current_mineral_resources_mt()), 0,
        total_cumulative_demand_materials_to_extract_from_2015() / current_mineral_resources_mt())
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
@cache('run')
def maximum_annual_extraction_materials():
    """
    maximum annual extraction materials



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[36.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[32.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.26], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[1.95], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[25.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.28], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[6.2], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[43.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.52], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[2100.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[17.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmaterials_availability_xresourcesxx():
    """
    Xmaterials availability XresourcesXX



    component


    """

    ret =  functions.if_then_else(single_precision(1) > single_precision(share_tot_cum_dem_vs_resources_materials()) + less_thr_func(1, share_tot_cum_dem_vs_resources_materials()), 1, 0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmaterials_availability_xreservesxx():
    """
    Xmaterials availability XreservesXX



    component


    """

    ret =  functions.if_then_else(single_precision(1) > single_precision(share_tot_cum_dem_vs_reserves_materials()) + less_thr_func(1, share_tot_cum_dem_vs_reserves_materials()), 1, 0)
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
# TPED by fuel
# TPES
# INDICATORS - Energy
# COMP - Historical data
@cache('step')
def tpes_ej():
    """
    TPES EJ



    component


    """

    ret =  total_extraction_nre_ej() + tpe_from_res_ej() + pes_waste_ej()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_heatxcom_from_waste_ejx():
    """
    XFES heatXcom from waste EJX



    component


    """

    ret =  xfes_waste_for_heatxcom_plantsx() + xfes_heatxcom_from_waste_in_chp_plantsx()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_waste_for_heatxcom_plantsx():
    """
    XFES waste for heatXcom plantsX



    component


    """

    ret =  xpes_waste_for_heatxcom_plantsx() * efficiency_waste_for_heat_plants()
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('step')
def fes_elec_from_waste_in_chp_plants():
    """
    FES elec from waste in CHP plants



    component


    """

    ret =  pes_waste_for_chp_plants() * efficiency_waste_for_elec_chp_plants()
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('step')
def fes_elec_from_waste_in_elec_plants():
    """
    FES elec from waste in elec plants



    component


    """

    ret =  pes_waste_for_elec_plants() * efficiency_waste_for_elec_plants()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_heatxcom_from_waste_in_chp_plantsx():
    """
    XFES heatXcom from waste in CHP plantsX



    component


    """

    ret =  pes_waste_for_chp_plants() * efficiency_waste_for_heat_chp_plants()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_waste_for_heatxcom_plantsx():
    """
    XPES waste for heatXcom plantsX



    component


    """

    ret =  pes_waste_ej() * xshare_pes_waste_for_heatxcom_plantsx()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xshare_pes_waste_for_heatxcom_plantsx():
    """
    Xshare PES waste for heatXcom plantsX



    constant


    """

    ret =  0.09050587321058197
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def share_pes_waste_tfc():
    """
    share PES waste TFC



    constant


    """

    ret =  0.19153003642106164
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def share_pes_waste_for_chp():
    """
    share PES waste for CHP



    constant


    """

    ret =  0.31858724855960674
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def share_pes_waste_for_elec_plants():
    """
    share PES waste for elec plants



    constant


    """

    ret =  0.39937684180874966
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('step')
def new_waste_supply_ej():
    """
    new waste supply EJ



    component


    """

    ret =  functions.if_then_else(
        time() < 2014, (historic_pes_waste_ej(time() + 1) - historic_pes_waste_ej(time())),
        functions.if_then_else(abs(max_waste() ) < equality_threshold,
                               pes_waste_ej() * p_waste_change(),
                               ((max_waste() - pes_waste_ej()) / max_waste()) *
                               adapt_growth_waste() * pes_waste_ej()))
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('step')
def waste_change():
    """
    waste change



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(annual_gdp_growth_rate()) + less_thr_func(0, annual_gdp_growth_rate()),
                                  annual_gdp_growth_rate(), p_waste_change())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
# WASTE - PES & FES
@cache('step')
def fes_elec_from_waste_ej():
    """
    FES elec from waste EJ



    component


    """

    ret =  fes_elec_from_waste_in_chp_plants() + fes_elec_from_waste_in_elec_plants()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC - Total FE electricity generation
# WASTE - PES & FES
@cache('step')
def fes_elec_from_waste_twh():
    """
    FES elec from waste TWh



    component


    """

    ret =  fes_elec_from_waste_ej() / ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def efficiency_waste_for_elec_plants():
    """
    efficiency waste for elec plants



    constant


    """

    ret =  0.24469672999410882
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
# PES solids biofuels & waste
# TPED by fuel
# TPES
@cache('step')
def pes_waste_ej():
    """
    PES waste EJ



    component


    """


    ret =  integ_pes_waste_ej()
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('step')
def pes_waste_for_chp_plants():
    """
    PES waste for CHP plants



    component


    """

    ret =  pes_waste_ej() * share_pes_waste_for_chp()
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def efficiency_waste_for_elec_chp_plants():
    """
    efficiency waste for elec CHP plants



    constant


    """

    ret =  0.17149829235093653
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def initial_pes_waste():
    """
    initial PES waste



    constant


    """

    ret =  0.94403359
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
# SOLIDS - PED, PES & FES
# SOLIDS - PED, PES & FES
@cache('step')
def pes_waste_for_tfc():
    """
    PES waste for TFC



    component


    """

    ret =  pes_waste_ej() * share_pes_waste_tfc()
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def efficiency_waste_for_heat_chp_plants():
    """
    efficiency waste for heat CHP plants



    constant


    """

    ret =  0.33592792668422766
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('step')
def adapt_growth_waste():
    """
    adapt growth waste



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        past_waste_growth(),
        functions.if_then_else(time() < 2020,
                               past_waste_growth() + (waste_change() - past_waste_growth()) *
                               (time_dmnl(time()) - 2015) / 5, waste_change()))
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def p_waste_change():
    """
    P waste change



    constant


    """

    ret =  0.04436
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def efficiency_waste_for_heat_plants():
    """
    efficiency waste for heat plants



    constant


    """

    ret =  0.7066660881268088
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def max_waste():
    """
    max waste



    constant


    """

    ret =  4
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_solids_bioe_x_waste_ejx():
    """
    XPES solids bioE X waste EJX



    component


    """

    ret =  pes_solids_bioe_ej() - pes_waste_ej()
    name = 1
    return round_sig(x=ret, name=name)

# WASTE - PES & FES
@cache('run')
def past_waste_growth():
    """
    Past waste growth



    constant


    """

    ret =  0.0443634053842744
    name = 1
    return round_sig(x=ret, name=name)
# WASTE - PES & FES

def historic_pes_waste_ej(x):
    """
    Historic PES waste EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.94403359, 0.9254440700000001, 0.96264396, 0.97404579, 1.0843876399999999, 1.20351759,
        1.1960921400000002, 1.24529466, 1.24459098, 1.29713973, 1.34430356, 1.4583756600000002,
        1.48302204, 1.51246124, 1.57780579, 1.8249681400000002, 1.96140481, 2.0159873999999998,
        2.07606288, 2.11879269
    ])
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def ch4_emissions_oil():
    """
    CH4 emissions oil



    component


    """

    ret =  pes_oil_ej() * gch4_per_mj_oil() * mj_per_ej() / g_per_mt()
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gch4_per_mj_coal():
    """
    gCH4 per MJ coal



    constant


    """

    ret =  0.094
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gch4_per_mj_oil():
    """
    gCH4 per MJ oil



    constant


    """

    ret =  0.094
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
@cache('step')
def fe_elec_generation_from_fossil_fuels_twh():
    """
    FE Elec generation from fossil fuels TWh



    component


    """

    ret =  fe_elec_generation_from_coal_twh() + fe_elec_generation_from_conv_gas_twh(
    ) + fe_elec_generation_from_unconv_gas_twh() + fe_elec_generation_from_total_oil_twh(
    ) + fes_elec_fossil_fuel_chp_plants_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
# RESOURCES - Uranium extraction
@cache('step')
def pe_demand_uranium_ej():
    """
    PE demand uranium EJ



    component


    """

    ret =  potential_generation_nuclear_elec_twh() * ej_per_twh(
    ) / efficiency_uranium_for_electricity()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# ELEC - Total FE electricity generation
# COMP- Historical data Elec production
@cache('step')
def fes_elec_fossil_fuel_chp_plants_twh():
    """
    FES Elec fossil fuel CHP plants TWh



    component


    """

    ret =  fes_elec_fossil_fuel_chp_plants_ej() / ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfes_heatxcom_fossil_fuels_chp_plants_ejx():
    """
    XFES heatXcom fossil fuels CHP plants EJX



    component


    """

    ret =  fed_heat_fossil_fuels_chp_plants_ej()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def potential_generation_nuclear_elec_twh():
    """
    potential generation nuclear elec TWh



    component


    """

    ret =  np.minimum(installed_capacity_nuclear_tw() * cp_nuclear() / twe_per_twh(),
                      demand_elec_nre_twh())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ELEC RES - Capacities and generation all techn
# HEAT RES-com - Capacities and generation all tech
@cache('run')
def threshold_remaining_potential_new_capacity():
    """
    threshold remaining potential new capacity



    constant


    """

    ret =  0.5
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# ELEC RES - Capacities and generation all techn
# RES - Employment
# MATERIALS - Demand for RES ELEC
# WATER - demand
# IMPACTS - RES Land-use
@cache('step')
def installed_capacity_res_elec_tw():
    """
    installed capacity RES elec TW



    component


    """


    ret =  integ_installed_capacity_res_elec_tw()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_time_planxconstr_res_elecx():
    """
    Xtotal time planXconstr RES elecX



    component


    """

    ret =  np.maximum(time_construction_res_elec(), time_step()) + np.maximum(time_planification_res_elec(), time_step())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def historic_new_required_capacity_res_elec():
    """
    Historic new required capacity RES elec



    component


    """

    ret =  xr.DataArray(
        data=[table_hist_capacity_res_elec_subhydro(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'hydro'}].values + 1) -
        table_hist_capacity_res_elec_subhydro(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'hydro'}].values),

        table_hist_capacity_res_elec_subxgeotxelecx(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'xgeotxelecx'}].values +
                                                    1) -
        table_hist_capacity_res_elec_subxgeotxelecx(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'xgeotxelecx'}].values),

        table_hist_capacity_res_elec_subxsolidbioexelecx(time() +
                                                         xtotal_time_planxconstr_res_elecx().loc[{'res elec':'xsolid bioexelecx'}].values + 1) -
        table_hist_capacity_res_elec_subxsolidbioexelecx(time() +
                                                         xtotal_time_planxconstr_res_elecx().loc[{'res elec':'xsolid bioexelecx'}].values),

        table_hist_capacity_res_elec_suboceanic(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'oceanic'}].values + 1) -
        table_hist_capacity_res_elec_suboceanic(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'oceanic'}].values),

        table_hist_capacity_res_elec_subwindonshore(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'wind onshore'}].values +
                                                    1) -
        table_hist_capacity_res_elec_subwindonshore(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'wind onshore'}].values),

        table_hist_capacity_res_elec_subwindoffshore(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'wind offshore'}].values +
                                                     1) -
        table_hist_capacity_res_elec_subwindoffshore(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'wind offshore'}].values),

        table_hist_capacity_res_elec_subsolarpv(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'solar pv'}].values + 1) -
        table_hist_capacity_res_elec_subsolarpv(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'solar pv'}].values),

        table_hist_capacity_res_elec_subcsp(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'csp'}].values + 1) -
        table_hist_capacity_res_elec_subcsp(time() + xtotal_time_planxconstr_res_elecx().loc[{'res elec':'csp'}].values)],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def new_required_capacity_res_elec():
    """
    new required capacity RES elec



    component


    """

    ret =  functions.if_then_else(
        time() < (2014 - xtotal_time_planxconstr_res_elecx()),
        historic_new_required_capacity_res_elec(),
        installed_capacity_res_elec_tw() * adapt_growth_res_elec_after_allocation() *
        remaining_potential_constraint_on_new_res_elec_capacity() * abundance_res_elec2())
    name = 1
    return round_sig(x=ret, name=name)

def table_hist_capacity_res_elec_subhydro(x):
    """
    table hist capacity RES elec SUBhydro



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.624490030142469, 0.634070032104449, 0.648500038817058, 0.657040027472228,
        0.674990036363238, 0.688220036952962, 0.698510036448025, 0.71313002660836,
        0.731790042351271, 0.750370028846775, 0.770490041148841, 0.792870048368696,
        0.82019003092096, 0.852160046683773, 0.8859300529104, 0.919750045926543, 0.946530044441139,
        0.976190046399544, 1.0204726000000002, 1.0567436000000001, 1.0915964000000002
    ])
    name = 1
    return round_sig(x=ret, name=name)

def table_hist_capacity_res_elec_subxgeotxelecx(x):
    """
    table hist capacity RES elec SUBXgeotXelecX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.007384619419926325, 0.007564460131224811, 0.007748680578241711, 0.007937387422504466,
        0.008130689923113103, 0.008328700000000001, 0.0080097, 0.0082019, 0.008357799999999999,
        0.0083085, 0.008689700000000002, 0.008921799999999999, 0.0091435, 0.0094635, 0.0099034,
        0.0101248, 0.010015399999999999, 0.010482, 0.010787, 0.0114575, 0.0118479
    ])
    name = 1
    return round_sig(x=ret, name=name)

def table_hist_capacity_res_elec_subxsolidbioexelecx(x):
    """
    table hist capacity RES elec SUBXsolidbioEXelecX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.02075556130242758, 0.022343248327393452, 0.024052384734166706, 0.025892260736819418,
        0.02787287720003564, 0.030005, 0.0318786, 0.033744300000000005, 0.0367185, 0.0392421,
        0.0394749, 0.0430149, 0.0451547, 0.0488584, 0.054362, 0.0588526, 0.062469699999999996,
        0.06715460000000001, 0.07378739999999999, 0.078254, 0.08402219999999999
    ])
    name = 1
    return round_sig(x=ret, name=name)

def table_hist_capacity_res_elec_suboceanic(x):
    """
    table hist capacity RES elec SUBoceanic



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.00020760889860613118, 0.000218011155677108, 0.00022893461850032943,
        0.0002404054021231664, 0.0002524509300890995, 0.00026510000000000005,
        0.00026510000000000005, 0.00026510000000000005, 0.00026510000000000005,
        0.00026510000000000005, 0.00026510000000000005, 0.00026510000000000005, 0.0002674,
        0.0002674, 0.00026910000000000004, 0.00027069999999999997, 0.0005251,
        0.0005277000000000001, 0.0005265, 0.0005273, 0.0005334
    ])
    name = 1
    return round_sig(x=ret, name=name)

def table_hist_capacity_res_elec_subwindonshore(x):
    """
    table hist capacity RES elec SUBwindonshore



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.00455706227539308, 0.005947976066351176, 0.007763426775385634, 0.010132992235415647,
        0.013225800231482635, 0.0172626, 0.0238921, 0.0306214, 0.038160599999999996,
        0.046517699999999995, 0.057833, 0.0723973, 0.09245880000000001, 0.1182237, 0.147983,
        0.1795401, 0.21826189999999998, 0.26633959999999995, 0.2961245, 0.341731, 0.4036673
    ])
    name = 1
    return round_sig(x=ret, name=name)

def table_hist_capacity_res_elec_subwindoffshore(x):
    """
    table hist capacity RES elec SUBwindoffshore



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        4.77710420476937e-06, 8.101151096781961e-06, 1.3738165691962337e-05, 2.329757763125582e-05,
        3.9508704120663585e-05, 6.7e-05, 7.7e-05, 0.000237, 0.0005072, 0.0005974,
        0.0006843999999999999, 0.0008834, 0.0010939, 0.0014419, 0.0021587, 0.0031435, 0.0037905,
        0.005374100000000001, 0.0075502, 0.0084937, 0.011636700000000002
    ])
    name = 1
    return round_sig(x=ret, name=name)

def table_hist_capacity_res_elec_subsolarpv(x):
    """
    table hist capacity RES elec SUBsolarPV



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        3.921043766748761e-05, 7.173827876311545e-05, 0.00013125027278544562,
        0.00024013168984911632, 0.0004393379704745763, 0.0008037999999999999, 0.0010863, 0.0014352,
        0.0019645, 0.0030528, 0.0045056, 0.006083399999999999, 0.0086421, 0.0145796, 0.0224319,
        0.0387925, 0.0693767, 0.098947, 0.1353989, 0.1718955, 0.2192966
    ])
    name = 1
    return round_sig(x=ret, name=name)

def table_hist_capacity_res_elec_subcsp(x):
    """
    table hist capacity RES elec SUBCSP



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.00035380000000000003, 0.00035380000000000003, 0.00035380000000000003,
        0.00035380000000000003, 0.00035380000000000003, 0.00035380000000000003,
        0.00035380000000000003, 0.00035380000000000003, 0.00035380000000000003,
        0.00035380000000000003, 0.00035380000000000003, 0.000341, 0.000424, 0.00048039999999999997,
        0.0008077000000000001, 0.00116082, 0.00167582, 0.00253307, 0.00328092,
        0.004338020000000001, 0.00469302
    ])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def initial_capacity_in_construction_res_elec():
    """
    initial capacity in construction RES elec



    component


    """

    ret =  initial_required_capacity_res_elec()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def remaining_potential_constraint_on_new_res_elec_capacity():
    """
    remaining potential constraint on new RES elec capacity



    component


    """

    ret =  functions.if_then_else(remaining_potential_res_elec_after_intermitt() >
                                  threshold_remaining_potential_new_capacity(), 1,
                                  remaining_potential_res_elec_after_intermitt() *
                                  (1 / threshold_remaining_potential_new_capacity()))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def required_capacity_res_elec_tw():
    """
    required capacity RES elec TW



    component


    """


    ret =  integ_required_capacity_res_elec_tw()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def res_elec_planned_capacity_tw():
    """
    RES elec planned capacity TW



    component


    """


    ret =  integ_res_elec_planned_capacity_tw()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xinitial_energy_cons_per_unit_of_material_cons_xrecycledx_x_datax():
    """
    XInitial energy cons per unit of material cons XrecycledX X dataX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[29.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[16.5], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[9.4], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[10.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[258.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[60.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[3710.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[36.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[72.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ELEC - PHS
# ESOI - PHS
@cache('run')
def cp_phs():
    """
    Cp PHS



    constant


    """

    ret =  0.1
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ELEC RES - Potential
@cache('run')
def max_potential_phs_twe():
    """
    max potential PHS TWe



    constant


    """

    ret =  0.25
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
# ELEC - PHS
# ESOI - PHS
@cache('step')
def max_capacity_potential_phs():
    """
    max capacity potential PHS



    component


    """

    ret =  max_potential_phs_twe() / cp_phs()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_per_new_res_elec_capacity_installed_x_hvdcsx():
    """
    Xmaterials per new RES elec capacity installed X HVDCsX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[100.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[125.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[435.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[1160.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[112.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[15.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[200.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[7.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_per_new_res_elec_capacity_installed_x_material_overgrid_high_powerx():
    """
    Xmaterials per new RES elec capacity installed X material overgrid high powerX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[7362.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[48.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[2044.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[6200.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[1140.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[562.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[29683.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[7500.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[40.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[130000.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[11.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[1390.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[970.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[190.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[160000.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[64.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[2651.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[2544.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[200.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def real_growth_gtl():
    """
    real growth GTL



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_gases()) > single_precision(abundance_liquids()) - less_thr_func(abundance_gases(), abundance_liquids()),
                                  functions.if_then_else(abs(abundance_gases() - 1) < equality_threshold,
                                                         exogenous_growth_gtl(), 0),
                                  0) * abundance_liquids_gtl() * scarcity_conv_oil()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def scarcity_conv_gas_delayed_1yr():
    """
    scarcity conv gas delayed 1yr



    component


    """

    ret =  delay_scarcity_conv_gas_round1__time_step___time_step_02502_1__time_step()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def scarcity_conv_oil_delayed_1yr():
    """
    scarcity conv oil delayed 1yr



    component


    """

    ret =  delay_scarcity_conv_oil_round1__time_step___time_step_03989_1__time_step()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_natx_gasx():
    """
    XPES natX gasX



    component


    """

    ret =  real_extraction_conv_gas_ej() + real_extraction_unconv_gas_ej()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
# EROItot - Electricity
@cache('step')
def fe_elec_generation_from_unconv_gas_twh():
    """
    FE Elec generation from unconv gas TWh



    component


    """

    ret =  real_extraction_unconv_gas_ej() * xshare_natx_gas_dem_for_elecx(
    ) * efficiency_gas_for_electricity() / ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def exponent_availability_conv_oil():
    """
    exponent availability conv oil



    constant


    """

    ret =  1 / 4
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def pe_losses_uncon_gas_for_elec_ej():
    """
    PE losses uncon gas for Elec EJ



    component


    """

    ret =  real_extraction_unconv_gas_ej() * xshare_natx_gas_dem_for_elecx() * (
        1 - efficiency_gas_for_electricity())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xabundance_total_natx_gasx():
    """
    Xabundance total natX gasX



    component


    """

    ret =  functions.if_then_else(single_precision(xpes_natx_gasx()) > single_precision(xped_natx_gas_ejx()) + less_thr_func(xpes_natx_gasx(), xped_natx_gas_ejx()), 1, 1 - functions.zidz(
        (xped_natx_gas_ejx() - xpes_natx_gasx()), xped_natx_gas_ejx()))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def exponent_availability_conv_gas():
    """
    exponent availability conv gas



    constant


    """

    ret =  1 / 4
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
# RESOURCES - CTL & GTL supply
@cache('step')
def scarcity_conv_gas():
    """
    scarcity conv gas



    component


    """

    ret =  functions.if_then_else(
        abs(max_extraction_conv_gas_ej() ) < equality_threshold, 0,
        functions.if_then_else(single_precision(max_extraction_conv_gas_ej()) > single_precision(extraction_conv_gas_ej()) - less_thr_func(max_extraction_conv_gas_ej(), extraction_conv_gas_ej()), 1 -
                               functions.zidz(max_extraction_conv_gas_ej() - extraction_conv_gas_ej(),
                                 max_extraction_conv_gas_ej())**exponent_availability_conv_gas(),
                               0))
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xyear_scarcity_total_natx_gasx():
    """
    XYear scarcity total natX gasX



    component


    """

    ret =  functions.if_then_else(single_precision(xabundance_total_natx_gasx()) > single_precision(0.95) + less_thr_func(xabundance_total_natx_gasx(), 0.95), 0, time())
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def share_unconv_gas_vs_tot_agg_in_2050():
    """
    share unconv gas vs tot agg in 2050



    component


    """

    ret =  functions.if_then_else(abs(choose_extraction_tot_agg_gas_curve() - 1) < equality_threshold,
                                  share_unconv_gas_vs_tot_agg_in_2050_laherrere2010(),
                                  functions.if_then_else(
                                      abs(choose_extraction_tot_agg_gas_curve() - 2) < equality_threshold,
                                      share_unconv_gas_vs_tot_agg_in_2050_mohr12_bg(),
                                      share_unconv_gas_vs_tot_agg_in_2050_user_defined()))
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def share_unconv_gas_vs_tot_agg_in_2050_laherrere2010():
    """
    share unconv gas vs tot agg in 2050 Laherrere2010



    constant


    """

    ret =  0.25
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def share_unconv_gas_vs_tot_agg_in_2050_mohr12_bg():
    """
    share unconv gas vs tot agg in 2050 Mohr12 BG



    constant


    """

    ret =  0.26666666666666666
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def share_unconv_gas_vs_tot_agg_in_2050_user_defined():
    """
    share unconv gas vs tot agg in 2050 User defined



    constant


    """

    ret =  0.5
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def cumulated_tot_agg_gas_extraction_to_1995():
    """
    cumulated tot agg gas extraction to 1995



    component


    """

    ret =  cumulated_conv_gas_extraction_to_1995() + cumulated_unconv_gas_extraction_to_1995()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def share_conv_gas_vs_tot_agg():
    """
    share conv gas vs tot agg



    component


    """

    ret =  1 - share_unconv_gas_vs_tot_agg()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def share_rurr_tot_agg_gas_to_leave_underground():
    """
    share RURR tot agg gas to leave underground



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def share_unconv_gas_vs_tot_agg():
    """
    share unconv gas vs tot agg



    component


    """

    ret =  functions.if_then_else(time() > 2012,
                                  np.minimum(evolution_share_unconv_gas_vs_tot_agg(), 1),
                                  historic_unconv_gas(time()) / xped_natx_gas_ejx())
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def rurr_tot_agg_gas():
    """
    RURR tot agg gas



    component


    """


    ret =  integ_rurr_tot_agg_gas()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def flow_tot_agg_gas_left_in_ground():
    """
    Flow tot agg gas left in ground



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_tot_agg_gas(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_tot_agg_gas() + 1, 0,
                                      tot_agg_gas_to_leave_underground()))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def tot_rurr_tot_agg_gas():
    """
    Tot RURR tot agg gas



    component


    """

    ret =  rurr_tot_agg_gas() + total_agg_gas_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def urr_total_agg_gas_unlimited():
    """
    URR total agg gas unlimited



    component


    """

    ret =  urr_total_gas_mohr12_bg() * 10000
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def start_policy_leave_in_ground_tot_agg_gas():
    """
    Start policy leave in ground tot agg gas



    constant


    """

    ret =  2015
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xextraction_unconv_gas_x_tot_aggx():
    """
    Xextraction unconv gas X tot aggX



    component


    """

    ret =  extraction_tot_agg_gas_ej() * share_unconv_gas_vs_tot_agg()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def total_agg_gas_left_in_ground():
    """
    Total agg gas left in ground



    component


    """


    ret =  integ_total_agg_gas_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def evolution_share_unconv_gas_vs_tot_agg():
    """
    evolution share unconv gas vs tot agg



    component


    """

    ret =  (share_unconv_gas_vs_tot_agg_in_2050() - 0.1268) / (2050 - 2012) * time() + (
        share_unconv_gas_vs_tot_agg_in_2050() - ((share_unconv_gas_vs_tot_agg_in_2050() - 0.1268) /
                                                 (2050 - 2012)) * 2050)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
# RESOURCES - Natural gas extraction
# TPES
@cache('step')
def real_extraction_conv_gas_ej():
    """
    real extraction conv gas EJ



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_gasxx() - 1) < equality_threshold,
                                  extraction_conv_gas_ej(), xextraction_conv_gas_x_tot_aggx())
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def urr_tot_agg_gas():
    """
    URR tot agg gas



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_gasxx() - 1) < equality_threshold, 0,
                                  functions.if_then_else(
                                      abs(xunlimited_nrexx() - 1) < equality_threshold,
                                      urr_total_agg_gas_unlimited(),
                                      functions.if_then_else(
                                          abs(xunlimited_gasxx() - 1) < equality_threshold,
                                          urr_total_agg_gas_unlimited(),
                                          functions.if_then_else(
                                              abs(choose_extraction_tot_agg_gas_curve() - 1) < equality_threshold,
                                              urr_total_gas_laherrere10(),
                                              functions.if_then_else(
                                                  abs(choose_extraction_tot_agg_gas_curve() - 2) < equality_threshold,
                                                  urr_total_gas_mohr12_bg(),
                                                  urr_total_gas_user_defined())))))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def cumulated_tot_agg_gas_extraction():
    """
    cumulated tot agg gas extraction



    component


    """


    ret =  integ_cumulated_tot_agg_gas_extraction()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xextraction_conv_gas_x_tot_aggx():
    """
    Xextraction conv gas X tot aggX



    component


    """

    ret =  extraction_tot_agg_gas_ej() * share_conv_gas_vs_tot_agg()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
# RESOURCES - Natural gas extraction
# TPES
@cache('step')
def real_extraction_unconv_gas_ej():
    """
    real extraction unconv gas EJ



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_gasxx() - 1) < equality_threshold,
                                  extraction_unconv_gas_ej(), xextraction_unconv_gas_x_tot_aggx())
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def max_extraction_tot_agg_gas_ej():
    """
    max extraction tot agg gas EJ



    component


    """

    ret =  functions.if_then_else(
        abs(xseparate_conv_and_unconv_gasxx() ) < equality_threshold,
        functions.if_then_else(abs(choose_extraction_tot_agg_gas_curve() - 1) < equality_threshold,
                               table_max_extraction_total_gas_laherrere10(tot_rurr_tot_agg_gas()),
                               functions.if_then_else(abs(choose_extraction_tot_agg_gas_curve() - 2) < equality_threshold,
                                                      table_max_extraction_total_gas_bg_mohr12(
                                                          tot_rurr_tot_agg_gas()),
                                                      table_max_extraction_total_gas_user_defined(
                                                          tot_rurr_tot_agg_gas()))), 0)
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def extraction_tot_agg_gas_ej():
    """
    extraction tot agg gas EJ



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(rurr_tot_agg_gas()) + less_thr_func(0, rurr_tot_agg_gas()), 0,
        functions.if_then_else(abs(xunlimited_nrexx() - 1) < equality_threshold,
                               xped_natx_gas_ejx(),
                               functions.if_then_else(abs(xunlimited_gasxx() - 1) < equality_threshold,
                                                      xped_natx_gas_ejx(),
                                                      np.minimum(
                                                          xped_natx_gas_ejx(),
                                                          max_extraction_tot_agg_gas_ej()))))
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xreal_extraction_conv_oil_mbxdx():
    """
    Xreal extraction conv oil MbXdX



    component


    """

    ret =  real_extraction_conv_oil_ej() * xmbxd_per_ejxyearx()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def share_unconv_oil_vs_tot_agg():
    """
    share unconv oil vs tot agg



    component


    """

    ret =  functions.if_then_else(time() > 2012,
                                  np.minimum(evolution_share_unconv_oil_vs_tot_agg(), 1),
                                  historic_unconv_oil(time()) / ped_total_oil_ej())
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def evolution_share_unconv_oil_vs_tot_agg():
    """
    evolution share unconv oil vs tot agg



    component


    """

    ret =  (share_unconv_oil_vs_tot_agg_in_2050() - 0.059) / (2050 - 2012) * time() + (
        share_unconv_oil_vs_tot_agg_in_2050() - ((share_unconv_oil_vs_tot_agg_in_2050() - 0.059) /
                                                 (2050 - 2012)) * 2050)
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def share_unconv_oil_vs_tot_agg_in_2050():
    """
    share unconv oil vs tot agg in 2050



    component


    """

    ret =  functions.if_then_else(abs(choose_extraction_curve_tot_agg_oil() - 1) < equality_threshold,
                                  share_unconv_oil_vs_tot_agg_in_2050_laherrere2006(),
                                  share_unconv_oil_vs_tot_agg_in_2050_user_defined())
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def share_unconv_oil_vs_tot_agg_in_2050_laherrere2006():
    """
    share unconv oil vs tot agg in 2050 Laherrere2006



    constant


    """

    ret =  0.5
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def share_unconv_oil_vs_tot_agg_in_2050_user_defined():
    """
    share unconv oil vs tot agg in 2050 User defined



    constant


    """

    ret =  0.5
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xextraction_conv_oil_x_tot_aggx():
    """
    Xextraction conv oil X tot aggX



    component


    """

    ret =  extraction_tot_agg_oil_ej() * share_conv_oil_vs_tot_agg()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xextraction_unconv_oil_x_tot_aggx():
    """
    Xextraction unconv oil X tot aggX



    component


    """

    ret =  extraction_tot_agg_oil_ej() * share_unconv_oil_vs_tot_agg()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def share_conv_oil_vs_tot_agg():
    """
    share conv oil vs tot agg



    component


    """

    ret =  1 - share_unconv_oil_vs_tot_agg()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def rurr_tot_agg_oil():
    """
    RURR tot agg oil



    component


    """


    ret =  integ_rurr_tot_agg_oil()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
# HEAT - Total FE heat generation
# LIQUIDS - PED, PES & FES
# RESOURCES - Oil extraction
# LOSSES
# TPES
# IMPACTS - Total GHG emissions
@cache('step')
def pes_oil_ej():
    """
    PES oil EJ



    component


    """

    ret =  real_extraction_conv_oil_ej() + real_extraction_unconv_oil_ej()
    name = 1
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def check_hist_crude_oil():
    """
    check hist crude oil



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (real_extraction_conv_oil_ej() - historic_crude_oil_extraction(time())) *
        100 / historic_crude_oil_extraction(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def extraction_tot_agg_oil_ej():
    """
    extraction tot agg oil EJ



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(rurr_tot_agg_oil()) + less_thr_func(0, rurr_tot_agg_oil()), 0,
        functions.if_then_else(abs(xunlimited_nrexx() - 1) < equality_threshold,
                               ped_total_oil_ej(),
                               functions.if_then_else(abs(xunlimited_oilxx() - 1) < equality_threshold,
                                                      ped_total_oil_ej(),
                                                      np.minimum(
                                                          ped_total_oil_ej(),
                                                          max_extraction_tot_agg_oil_ej()))))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
# TPES
# COMP - Historical data
@cache('step')
def real_extraction_conv_oil_ej():
    """
    real extraction conv oil EJ



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_oilxx() - 1) < equality_threshold,
                                  extraction_conv_oil_ej(), xextraction_conv_oil_x_tot_aggx())

    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def max_extraction_tot_agg_oil_ej():
    """
    max extraction tot agg oil EJ



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_oilxx() ) < equality_threshold,
                                  functions.if_then_else(
                                      abs(choose_extraction_curve_tot_agg_oil() - 1) < equality_threshold,
                                      table_max_extraction_tot_agg_oil_laherrere_2006(
                                          tot_rurr_tot_agg_oil()),
                                      table_max_extraction_tot_agg_oil_user_defined(
                                          tot_rurr_tot_agg_oil())), 0)
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gch4_per_mj_conv_gas():
    """
    gCH4 per MJ conv gas



    constant


    """

    ret =  0.78
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def total_agg_oil_left_in_ground():
    """
    Total agg oil left in ground



    component


    """


    ret =  integ_total_agg_oil_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def cumulated_tot_agg_oil_extraction():
    """
    cumulated tot agg oil extraction



    component


    """


    ret =  integ_cumulated_tot_agg_oil_extraction()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def cumulated_tot_agg_extraction_to_1995():
    """
    cumulated tot agg extraction to 1995



    component


    """

    ret =  cumulated_conv_oil_extraction_to_1995() + cumulated_unconv_oil_extraction_to_1995()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def urr_tot_agg_oil():
    """
    URR tot agg oil



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_oilxx() - 1) < equality_threshold, 0,
                                  functions.if_then_else(
                                      abs(xunlimited_oilxx() - 1) < equality_threshold,
                                      urr_tot_agg_oil_unlimited(),
                                      functions.if_then_else(
                                          abs(xunlimited_nrexx() - 1) < equality_threshold,
                                          urr_tot_agg_oil_unlimited(),
                                          functions.if_then_else(
                                              abs(choose_extraction_curve_tot_agg_oil() - 1) < equality_threshold,
                                              urr_tot_agg_oil_laherrere_2006(),
                                              urr_tot_agg_oil_user_defined()))))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def urr_tot_agg_oil_unlimited():
    """
    URR tot agg oil unlimited



    component


    """

    ret =  urr_tot_agg_oil_laherrere_2006() * 10000
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def tot_rurr_tot_agg_oil():
    """
    Tot RURR tot agg oil



    component


    """

    ret =  rurr_tot_agg_oil() + total_agg_oil_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def start_policy_leave_in_ground_tot_agg_oil():
    """
    Start policy leave in ground tot agg oil



    constant


    """

    ret =  2015
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def flow_tot_agg_oil_left_in_ground():
    """
    Flow tot agg oil left in ground



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_tot_agg_oil(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_tot_agg_oil() + 1, 0,
                                      tot_agg_oil_to_leave_underground()))
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def g_per_mt():
    """
    g per Mt



    constant


    """

    ret =  1000000000000
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def share_rurr_tot_agg_oil_to_leave_underground():
    """
    share RURR tot agg oil to leave underground



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gch4_per_mj_unconv_gas():
    """
    gCH4 per MJ unconv gas



    constant


    """

    ret =  2.48
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def tot_rurr_unconv_gas():
    """
    Tot RURR unconv gas



    component


    """

    ret =  rurr_unconv_gas() + total_unconv_gas_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('run')
def start_policy_leave_in_ground_coal():
    """
    Start policy leave in ground coal



    constant


    """

    ret =  2015
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def total_unconv_gas_left_in_ground():
    """
    Total unconv gas left in ground



    component


    """


    ret =  integ_total_unconv_gas_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def flow_coal_left_in_ground():
    """
    Flow coal left in ground



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_coal(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_coal() + 1, 0,
                                      coal_to_leave_underground()))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def flow_conv_gas_left_in_ground():
    """
    Flow conv gas left in ground



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_conv_gas(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_conv_gas() + 1, 0,
                                      conv_gas_to_leave_underground()))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def flow_conv_oil_left_in_ground():
    """
    Flow conv oil left in ground



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_conv_oil(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_conv_oil() + 1, 0,
                                      conv_oil_to_leave_underground()))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def flow_unconv_oil_left_in_ground():
    """
    Flow unconv oil left in ground



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_unconv_oil(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_unconv_oil() + 1, 0,
                                      unconv_oil_to_leave_underground()))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def share_rurr_unconv_gas_to_leave_underground():
    """
    share RURR unconv gas to leave underground



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def share_rurr_unconv_oil_to_leave_underground():
    """
    share RURR unconv oil to leave underground



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def extraction_unconv_gas_ej():
    """
    extraction unconv gas EJ



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(rurr_unconv_gas()) + less_thr_func(0, rurr_unconv_gas()), 0,
                                  functions.if_then_else(
                                      time() < 2013,
                                      historic_unconv_gas(time()),
                                      functions.if_then_else(
                                          abs(xseparate_conv_and_unconv_gasxx() - 1) < equality_threshold,
                                          np.minimum(max_extraction_unconv_gas(),
                                                     max_unconv_gas_growth_extraction_ej()), 0)))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def tot_rurr_coal():
    """
    Tot RURR coal



    component


    """

    ret =  rurr_coal() + total_coal_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def tot_rurr_conv_gas():
    """
    Tot RURR conv gas



    component


    """

    ret =  rurr_conv_gas() + total_conv_gas_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def tot_rurr_conv_oil():
    """
    Tot RURR conv oil



    component


    """

    ret =  rurr_conv_oil() + total_conv_oil_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def share_rurr_conv_oil_to_leave_underground():
    """
    share RURR conv oil to leave underground



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def tot_rurr_unconv_oil():
    """
    Tot RURR unconv oil



    component


    """

    ret =  rurr_unconv_oil_ej() + total_unconv_oil_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def total_unconv_oil_left_in_ground():
    """
    Total unconv oil left in ground



    component


    """


    ret =  integ_total_unconv_oil_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def total_coal_left_in_ground():
    """
    Total coal left in ground



    component


    """


    ret =  integ_total_coal_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def total_conv_gas_left_in_ground():
    """
    Total conv gas left in ground



    component


    """


    ret =  integ_total_conv_gas_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def total_conv_oil_left_in_ground():
    """
    Total conv oil left in ground



    component


    """


    ret =  integ_total_conv_oil_left_in_ground()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def extraction_conv_oil_ej():
    """
    extraction conv oil EJ



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(rurr_conv_oil()) + less_thr_func(0, rurr_conv_oil()), 0,
                                  functions.if_then_else(abs(xunlimited_nrexx() - 1) < equality_threshold,
                                                         demand_conv_oil_ej(),
                                                         functions.if_then_else(
                                                             abs(xunlimited_oilxx() - 1) < equality_threshold,
                                                             demand_conv_oil_ej(),
                                                             np.minimum(
                                                                 demand_conv_oil_ej(),
                                                                 max_extraction_conv_oil_ej()))))
    name = 1
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
# HEAT - Total FE heat generation
# SOLIDS - PED, PES & FES
# RESOURCES - Coal extraction
# LOSSES
# TPES
# COMP - Historical data
@cache('step')
def extraction_coal_ej():
    """
    extraction coal EJ



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(rurr_coal()) + less_thr_func(0, rurr_coal()), 0,
                                  functions.if_then_else(abs(xunlimited_nrexx() - 1) < equality_threshold,
                                                         ped_coal_ej(),
                                                         functions.if_then_else(
                                                             abs(xunlimited_coalxx() - 1) < equality_threshold,
                                                             ped_coal_ej(),
                                                             np.minimum(
                                                                 ped_coal_ej(),
                                                                 max_extraction_coal_ej()))))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def rurr_coal():
    """
    RURR coal



    component


    """


    ret =  integ_rurr_coal()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def max_extraction_conv_oil_ej():
    """
    max extraction conv oil EJ



    component


    """

    ret =  functions.if_then_else(
        abs(xseparate_conv_and_unconv_oilxx() - 1) < equality_threshold,
        functions.if_then_else(
            abs(choose_extraction_curve_conv_oil() - 1) < equality_threshold,
            table_max_extraction_maggio12middle_conv_oil_ej(tot_rurr_conv_oil()),
            functions.if_then_else(
                abs(choose_extraction_curve_conv_oil() - 2) < equality_threshold,
                table_max_extraction_maggio12_high_conv_oil_ej(tot_rurr_conv_oil()),
                functions.if_then_else(abs(choose_extraction_curve_conv_oil() - 3) < equality_threshold,
                                       table_max_extraction_maggio12_low_con_oil_ej(
                                           tot_rurr_conv_oil()),
                                       table_max_conv_oil_extraction_user_defined(
                                           tot_rurr_conv_oil())))), 0)
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def start_policy_leave_in_ground_unconv_gas():
    """
    Start policy leave in ground unconv gas



    constant


    """

    ret =  2015
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def flow_unconv_gas_left_in_ground():
    """
    Flow unconv gas left in ground



    component


    """

    ret =  functions.if_then_else(time() < start_policy_leave_in_ground_unconv_gas(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_unconv_gas() + 1, 0,
                                      unconv_gas_to_leave_underground()))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def start_policy_leave_in_ground_unconv_oil():
    """
    Start policy leave in ground unconv oil



    constant


    """

    ret =  2015
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def start_policy_leave_in_ground_conv_oil():
    """
    Start policy leave in ground conv oil



    constant


    """

    ret =  2015
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('run')
def share_rurr_coal_to_leave_underground():
    """
    share RURR coal to leave underground



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpes_oil_mbxdx():
    """
    XPES oil MbXdX



    component


    """

    ret =  pes_oil_ej() * xmbxd_per_ejxyearx()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
# MATERIALS - Total demand to extract vs reserves &
@cache('run')
def current_mineral_reserves_mt():
    """
    current mineral reserves Mt



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[28000.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.5], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[480.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[720.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.0052], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.011], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[160000.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[13.5], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[2400.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[570.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[11.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[81.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[87.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.53], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[4.8], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.01108], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[794.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[15.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[230.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)
# PES solids biofuels & waste
# RES - PED from non-electric RES
# SOLIDS - PED, PES & FES
# TFES by fuel

def losses_in_charcoal_plants_ej(x):
    """
    Losses in charcoal plants EJ



    lookup


    """
    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        2.1227642400000004, 2.1262310400000004, 2.1231064699999997, 2.1130467299999998, 2.1735678,
        2.18822825, 2.2560478, 2.3403727099999996, 2.4201021199999997, 2.54406673, 2.6353136,
        2.7016235600000007, 2.83974232, 2.86682404, 2.78902384, 2.92604662, 3.0703874300000003,
        3.5232924299999997, 3.4736966899999997, 3.4703146300000003
    ])



    name = 1
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def check_hist_tpes():
    """
    check hist TPES



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (tpes_ej() - historic_tpes(time())) * 100 / historic_tpes(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def share_rurr_conv_gas_to_leave_underground():
    """
    share RURR conv gas to leave underground



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def start_policy_leave_in_ground_conv_gas():
    """
    Start policy leave in ground conv gas



    constant


    """

    ret =  2015
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def extraction_conv_gas_ej():
    """
    extraction conv gas EJ



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(rurr_conv_gas()) + less_thr_func(0, rurr_conv_gas()), 0,
                                  functions.if_then_else(abs(xunlimited_nrexx() - 1) < equality_threshold,
                                                         demand_conv_gas(),
                                                         functions.if_then_else(
                                                             abs(xunlimited_gasxx() - 1) < equality_threshold,
                                                             demand_conv_gas(),
                                                             np.minimum(
                                                                 demand_conv_gas(),
                                                                 max_extraction_conv_gas_ej()))))
    name = 1
    return round_sig(x=ret, name=name)

# GASES - PED, PES & FES
# RESOURCES - Natural gas extraction
@cache('step')
def check_gases():
    """
    check gases



    component


    """

    ret =  functions.zidz(ped_gases() - pes_gases(), pes_gases())
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
# FE INTENSITIES by sector and fuel
# ECON- Final demands by sector, households and fuel
# GASES - PED, PES & FES
# TRANSP - Alternative transport
# TRANSP - Households transport
# RESOURCES - CTL & GTL supply
@cache('step')
def abundance_gases():
    """
    abundance gases



    component


    """

    ret =  functions.if_then_else(single_precision(pes_gases()) > single_precision(ped_gases()) + less_thr_func(pes_gases(), ped_gases()), 1,
                                  1 - functions.zidz(ped_gases() - pes_gases(), ped_gases()))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def demand_conv_gas():
    """
    Demand conv gas



    component


    """

    ret =  np.maximum(xped_natx_gas_ejx() - extraction_unconv_gas_ej(), 0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xped_natx_gas_without_gtlx():
    """
    XPED natX gas without GTLX



    component


    """

    ret =  np.maximum(0, xped_natx_gas_ejx() - xped_natx_gas_for_gtl_ejx())
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
# FE INTENSITIES by sector and fuel
# ECON- Final demands by sector, households and fuel
# SOLIDS - PED, PES & FES
@cache('step')
def abundance_solids():
    """
    abundance solids



    component


    """

    ret =  functions.if_then_else(single_precision(pes_solids()) > single_precision(ped_solids()) + less_thr_func(pes_solids(), ped_solids()), 1,
                                  1 - functions.zidz(ped_solids() - pes_solids(), ped_solids()))
    name = 1
    return round_sig(x=ret, name=name)

# ECON: final energy demand by sector, household
# DEM- Non-energy use
# E INTENSITIES - Aggregated indicators
# ECON- Final demands by sector, households and fuel
# ELEC - Electricity related losses
# ELEC RES - Total monetary investments
# TRANSP - Transport energy demand
# TRANSP - Transport energy demand
# MATERIALS - Rest demand
# INDICATORS - Social and Environmental impacts
# COMP - Historical data
# COMP - GDP
@cache('step')
def gdp():
    """
    GDP



    component


    """

    ret =  real_demand() / 1e+006
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('step')
def demand_elec_gas_and_coal_twh():
    """
    demand Elec gas and coal TWh



    component


    """

    ret =  demand_elec_plants_fossil_fuels_twh() - fe_demand_oil_elec_plants_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# COMP- Historical data Elec production
@cache('step')
def fe_demand_coal_elec_plants_twh():
    """
    FE demand coal Elec plants TWh



    component


    """

    ret =  share_coal_for_elec() * demand_elec_gas_and_coal_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# COMP- Historical data Elec production
@cache('step')
def fe_demand_gas_elec_plants_twh():
    """
    FE demand gas Elec plants TWh



    component


    """

    ret =  xshare_gasxxcoal_xgasx_for_elecx() * demand_elec_gas_and_coal_twh()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_heat_gasxcoal_ejx():
    """
    XFED Heat gasXcoal EJX



    component


    """

    ret =  xfed_heatxcom_plants_fossil_fuels_ejx() - fed_heat_liquids_plants_ej()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# LIQUIDS - PED, PES & FES
# COMP- Historical data II
@cache('step')
def pe_demand_oil_elec_plants_ej():
    """
    PE demand oil Elec plants EJ



    component


    """

    ret =  (fe_demand_oil_elec_plants_twh() / efficiency_liquids_for_electricity()) * ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# COMP- Historical data Elec production
@cache('step')
def fe_demand_oil_elec_plants_twh():
    """
    FE demand oil Elec plants TWh



    component


    """

    ret =  share_oil_for_elec() * total_fe_elec_demand_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('step')
def share_coal_for_elec():
    """
    share coal for Elec



    component


    """

    ret =  1 - xshare_gasxxcoal_xgasx_for_elecx()
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_liquids_elec():
    """
    share liquids Elec



    component


    """

    ret =  fe_demand_oil_elec_plants_twh() / total_fe_elec_demand_twh()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Total demand to extract vs reserves &
# MATERIALS - Total demand to extract vs reserves &
@cache('run')
def current_mineral_resources_mt():
    """
    current mineral resources Mt



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[75000.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[6.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[12000.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[2100.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[1.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.0471], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[800000.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[39.5], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[12000.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[1030.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[14.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[130.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[2000.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[1.308], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[76.2], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.025], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[2000.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[63.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[1900.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def share_chp_plants_oil():
    """
    share CHP plants oil



    component


    """

    ret =  np.maximum(
        functions.if_then_else(time() > 2014, -0.002985 * (time()) + 6.04554,
                               historic_share_chp_plants_oil(time())), 0)
    name = 1
    return round_sig(x=ret, name=name)
# CHP plants - FES commercial Heat and Elec

def historic_share_chp_plants_gas(x):
    """
    historic share CHP plants gas



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.542607444618554, 0.5269313193785189, 0.5382532583200468, 0.5297249478665147,
        0.5460827573768788, 0.5224061688803091, 0.5411877199764002, 0.5577070041319593,
        0.562342979935354, 0.5613705825714098, 0.5857040300373348, 0.5890026793248733,
        0.5983201717080272, 0.5900688532905982, 0.5816026263018895, 0.5969505315378193,
        0.6059246889618498, 0.6191394159419781, 0.6468415798082748, 0.6518959474484941
    ])
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def share_chp_plants_coal():
    """
    share CHP plants coal



    component


    """

    ret =  1 - historic_share_chp_plants_gas(time()) - share_chp_plants_oil()
    name = 1
    return round_sig(x=ret, name=name)
# HEAT COM- Heat-com demand covered by fossil fuels

def historic_share_liquids_for_heat_plants(x):
    """
    Historic share liquids for heat plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.18497727967085997, 0.17191646447570985, 0.1330532764569175, 0.15161781088000842,
        0.13531842227897417, 0.12023891330325187, 0.12010733048107938, 0.10499688074615252,
        0.10170168137771297, 0.08909757858316623, 0.08211320924276294, 0.07546615577930145,
        0.07135237951220018, 0.06937841924029578, 0.07213434112645382, 0.07775801992149381,
        0.06902303071848384, 0.06498032662251463, 0.06478078155386392, 0.06752954750266457
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_coalxcoalxgasx_for_heat_plantsx():
    """
    Xshare coalXcoalXgasX for heat plantsX



    component


    """

    ret =  1 - xshare_gasxxcoalxgasx_for_heat_plantsx(time())
    name = 1
    return round_sig(x=ret, name=name)
# CHP plants - FES commercial Heat and Elec

def historic_share_chp_plants_oil(x):
    """
    historic share CHP plants oil



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.09862928248379321, 0.11787701564444898, 0.10608252906539808, 0.10858090585253756,
        0.08882230533385095, 0.07562621902637809, 0.0734600656866448, 0.06506856593351103,
        0.06329937880802863, 0.07127638319113186, 0.07390231230724596, 0.07582583414512277,
        0.06819575385574247, 0.06266601654129239, 0.06998905828978902, 0.05073704681719738,
        0.05157446046900763, 0.04861084829791663, 0.033462519012298, 0.032844869257742916
    ])
    name = 1
    return round_sig(x=ret, name=name)

# EROIgrid - Allocation RES elec
@cache('step')
def output_elec_over_lifetime_res_elec_for_allocation2():
    """
    output elec over lifetime RES elec for allocation2



    component


    """

    ret =  xxstaticx_eroigrid_res_elecx() * cedtot_over_lifetime_res_elec_for_allocation(
    ) * quality_of_electricity()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES - Potential
# ELEC RES - Capacities and generation all techn
# ELEC - Total FE electricity generation
# EROI - RES elec
# EROIgrid - Allocation RES elec
@cache('step')
def real_generation_res_elec_twh():
    """
    real generation RES elec TWh



    component


    """

    ret =  potential_generation_res_elec_twh() * (1 - res_elec_tot_overcapacity())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('step')
def demand_storage_capacity():
    """
    demand storage capacity



    component


    """

    ret =  xshare_capacity_storagexres_elec_varx() * total_installed_capacity_res_elec_var()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_capacity_storagexres_elec_varx():
    """
    Xshare capacity storageXRES elec varX



    component


    """

    ret =  0.099 + 0.1132 * share_elec_demand_covered_by_res()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xstaticxdynamic_quality_of_electricityxx():
    """
    XstaticXdynamic quality of electricityXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xxstaticx_eroigrid_totxeffective_for_allocation_res_elecx():
    """
    XXstaticX EROIgrid totXeffective for allocation RES elecX



    component


    """

    ret =  functions.zidz(
        functions.sum(output_elec_over_lifetime_res_elec_for_allocation2()),
        functions.sum(cedtot_over_lifetime_res_elec_for_allocation()))
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xratio_eroigrid_vs_eroi_xstaticxx():
    """
    Xratio EROIgrid vs EROI XstaticXX



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(xxstaticx_eroi_res_elecx()) - less_thr_func(0, xxstaticx_eroi_res_elecx()), 0,
                                  xxstaticx_eroigrid_res_elecx() / xxstaticx_eroi_res_elecx())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xratio_eroi_per_techn_vs_eroitot_xstaticxx():
    """
    Xratio EROI per techn vs EROItot XstaticXX



    component


    """

    ret =  functions.xidz(xxstaticx_eroigrid_res_elecx(),
                          xxstaticx_eroigrid_totxeffective_for_allocation_res_elecx(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('step')
def total_installed_capacity_res_elec_var():
    """
    Total installed capacity RES elec var



    component


    """
    da = installed_capacity_res_elec_tw()
    ret =  da.loc[{'res elec':'wind onshore'}].values + da.loc[{'res elec':'wind offshore'}].values + da.loc[{'res elec':'solar pv'}].values + da.loc[{'res elec':'csp'}].values
    name = 1
    return round_sig(x=ret, name=name)

# EROIgrid - Allocation RES elec
@cache('run')
def share_res_elec_generation_stored():
    """
    share RES elec generation stored



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.2], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.2], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.2], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.2], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Storage demand and supply
@cache('run')
def rt_storage_efficiency_phs():
    """
    rt storage efficiency PHS



    constant


    """

    ret =  0.75
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# ELEC RES - Capacities and generation all techn
@cache('step')
def cp_baseload_reduction():
    """
    Cp baseload reduction



    component


    """

    ret =  xr.DataArray(
        data=[cp_res_elec().loc[{'res elec':'hydro'}].values / xcpxini_res_elecx_subhydro(time()),

        cp_res_elec().loc[{'res elec':'xgeotxelecx'}].values / xcpxini_res_elecx_subxgeotxelecx(time()),

        cp_res_elec().loc[{'res elec':'xsolid bioexelecx'}].values / xcpxini_res_elecx_subxsolidbioexelecx(time()),

        cp_res_elec().loc[{'res elec':'oceanic'}].values / xcpxini_res_elecx_suboceanic(time()),

        cp_res_elec().loc[{'res elec':'wind onshore'}].values / xcpxini_res_elecx_subwindonshore(time()),

        cp_res_elec().loc[{'res elec':'wind offshore'}].values / xcpxini_res_elecx_subwindoffshore(time()),

        cp_res_elec().loc[{'res elec':'solar pv'}].values / xcpxini_res_elecx_subsolarpv(time()),

        cp_res_elec().loc[{'res elec':'csp'}].values / xcpxini_res_elecx_subcsp(time())],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xratio_x_1x():
    """
    Xratio X 1X



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
# EROIgrid - Allocation RES elec
# EROItot - Electricity
@cache('step')
def cedtot_over_lifetime_res_elec():
    """
    CEDtot over lifetime RES elec



    component


    """

    ret =  xr.DataArray(
        data=[fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'hydro'}].values,

        fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'xgeotxelecx'}].values,

        fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'xsolid bioexelecx'}].values,

        fei_over_lifetime_res_elec_dispatch().loc[{'res elec':'oceanic'}].values,

        fei_over_lifetime_res_elec_var().loc[{'res elec':'wind onshore'}].values,

        fei_over_lifetime_res_elec_var().loc[{'res elec':'wind offshore'}].values,

        fei_over_lifetime_res_elec_var().loc[{'res elec':'solar pv'}].values,

        fei_over_lifetime_res_elec_var().loc[{'res elec':'csp'}].values],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# EROI - RES elec
@cache('step')
def real_cp_res_elec():
    """
    real Cp RES elec



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        cp_res_elec(),
        functions.if_then_else(
            abs(installed_capacity_res_elec_tw() ) < equality_threshold, 0,
            real_generation_res_elec_twh() * twe_per_twh() / installed_capacity_res_elec_tw()))
    name = 1
    return round_sig(x=ret, name=name)

# EROIgrid - Allocation RES elec
@cache('step')
def cedtot_over_lifetime_res_elec_for_allocation():
    """
    CEDtot over lifetime RES elec for allocation



    component


    """

    ret =  cedtot_over_lifetime_res_elec() * remaining_potential_res_elec_switch()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# EROIgrid - Allocation RES elec
@cache('step')
def remaining_potential_res_elec_after_intermitt():
    """
    remaining potential RES elec after intermitt



    component


    """

    ret =  functions.if_then_else(
        potential_res_elec_after_intermitt_twh() > potential_generation_res_elec_twh(),
        (potential_res_elec_after_intermitt_twh() - potential_generation_res_elec_twh()) /
        potential_res_elec_after_intermitt_twh(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# EROIgrid - Allocation RES elec
@cache('step')
def remaining_potential_res_elec_switch():
    """
    remaining potential RES elec switch



    component


    """

    ret =  functions.if_then_else(remaining_potential_res_elec_after_intermitt() < 0.025, 0, 1)
    name = 1
    return round_sig(x=ret, name=name)

# EROIgrid - Allocation RES elec
@cache('step')
def output_elec_over_lifetime_res_elec_for_allocation():
    """
    output elec over lifetime RES elec for allocation



    component


    """

    ret =  output_elec_over_lifetime_res_elec() * remaining_potential_res_elec_switch()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# ELEC RES - Capacities and generation all techn
@cache('step')
def potential_res_elec_after_intermitt_twh():
    """
    potential RES elec after intermitt TWh



    component


    """

    ret =  max_potential_res_elec_twe() * cp_baseload_reduction() / twe_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def replacement_rate_res_elec():
    """
    replacement rate RES elec



    component


    """

    ret =  functions.if_then_else(single_precision(potential_res_elec_after_intermitt_twh()) > single_precision(potential_generation_res_elec_twh()) + less_thr_func(potential_res_elec_after_intermitt_twh(), potential_generation_res_elec_twh()), 1, 0.9)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xxstaticx_eroitot_res_elecx():
    """
    XXstaticX EROItot RES elecX



    component


    """

    ret =  functions.if_then_else(
        functions.sum(cedtot_over_lifetime_res_elec()) < 0, 0,
        functions.sum(output_elec_over_lifetime_res_elec()) / (functions.sum(cedtot_over_lifetime_res_elec())))
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xxstaticx_eroitotxeffective_for_allocation_res_elecx():
    """
    XXstaticX EROItotXeffective for allocation RES elecX



    component


    """

    ret =  functions.zidz(
        functions.sum(output_elec_over_lifetime_res_elec_for_allocation()),
        functions.sum(cedtot_over_lifetime_res_elec_for_allocation()))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def new_capacity_installed_onshore_wind_tw():
    """
    new capacity installed onshore wind TW



    component


    """

    ret =  new_res_elec_capacity_under_planning().loc[{'res elec':'wind onshore'}].values
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xced_oxm_over_lifetime_per_water_res_elec_varx():
    """
    XCED OXM over lifetime per water RES elec varX



    component


    """

    ret =  res_elec_capacity_under_construction_tw() * xwater_for_oxm_x_res_elecx(
    ) * energy_requirements_per_unit_of_water_consumption() * lifetime_res_elec() * (
        m_per_t() / kg_per_mt()) * (kg_per_mt() / mj_per_ej())
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xselfxelectricity_consumption_res_elecx():
    """
    XselfXelectricity consumption RES elecX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.02], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.02], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.005], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('run')
def share_energy_requirements_for_decom_res_elec():
    """
    share energy requirements for decom RES elec



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.1], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.1], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.1], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.1], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xced_oxm_over_lifetime_res_elec_varx():
    """
    XCED OXM over lifetime RES elec varX



    component


    """

    ret =  (functions.sum(dim='materials', data=xced_oxm_over_lifetime_per_material_res_elec_varx()) +
            functions.sum(dim='water', data=xced_oxm_over_lifetime_per_water_res_elec_varx()))
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('run')
def grid_correction_factor_res_elec():
    """
    Grid correction factor RES elec



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.06], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.06], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.06], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.075], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_for_oxm_per_capacity_installed_x_cspx():
    """
    Xmaterials for OXM per capacity installed X CSPX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[0.78], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[3450.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[140.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[11.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[3.2], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[98.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.014], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[2000.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_water_for_oxm_required_by_res_elecx():
    """
    XTotal water for OXM required by RES elecX



    component


    """

    ret =  functions.sum(dim='water', data=xwater_for_oxm_required_for_res_elecx())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xwater_for_oxm_required_for_res_elecx():
    """
    XWater for OXM required for RES elecX



    component


    """

    ret =  xr.DataArray(
        data=np.zeros((8, 2)),
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp'], 'water':['xcleanx pumped waterx','xdistilledx deionized waterx']},
        dims=['res elec','water'])
    data = [installed_capacity_res_elec_tw().loc[{'res elec':'hydro'}].values * xwater_for_oxm_x_res_elecx().loc[{'res elec':'hydro'}].drop('res elec') * m_per_t() / kg_per_mt(),

        installed_capacity_res_elec_tw().loc[{'res elec':'xgeotxelecx'}].values * xwater_for_oxm_x_res_elecx().loc[{'res elec':'xgeotxelecx'}].drop('res elec') * m_per_t() / kg_per_mt(),

        installed_capacity_res_elec_tw().loc[{'res elec':'xsolid bioexelecx'}].values * xwater_for_oxm_x_res_elecx().loc[{'res elec':'xsolid bioexelecx'}].drop('res elec') * m_per_t() / kg_per_mt(),

        installed_capacity_res_elec_tw().loc[{'res elec':'oceanic'}].values * xwater_for_oxm_x_res_elecx().loc[{'res elec':'oceanic'}].drop('res elec') * m_per_t() / kg_per_mt(),

        installed_capacity_res_elec_tw().loc[{'res elec':'wind onshore'}].values * xwater_for_oxm_x_res_elecx().loc[{'res elec':'wind onshore'}].drop('res elec') * m_per_t() / kg_per_mt(),

        installed_capacity_res_elec_tw().loc[{'res elec':'wind offshore'}].values * xwater_for_oxm_x_res_elecx().loc[{'res elec':'wind offshore'}].drop('res elec') * m_per_t() / kg_per_mt(),

        installed_capacity_res_elec_tw().loc[{'res elec':'solar pv'}].values * xwater_for_oxm_x_res_elecx().loc[{'res elec':'solar pv'}].drop('res elec') * m_per_t() / kg_per_mt(),

        installed_capacity_res_elec_tw().loc[{'res elec':'csp'}].values * xwater_for_oxm_x_res_elecx().loc[{'res elec':'csp'}].drop('res elec') * m_per_t() / kg_per_mt()]
    ret.values = list(map(lambda x: x if type(x) != xr.DataArray else xr.align(ret, x, join='left')[1], data))
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_for_oxm_per_capacity_installed_res_elecx():
    """
    Xmaterials for OXM per capacity installed RES elecX



    constant


    """

    ret =  xr.DataArray(
        data=np.zeros((8, 58)),
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp'], 'materials':['adhesive','aluminium','aluminium mirrors','cadmium','carbon fiber','cement','chromium','copper','diesel','dy','xelectricxelectronic componentsx','evacuation lines','fiberglass','foam glass','galium','glass','glass reinforcing plastic','gravel','indium','iron','kno3 mined','asphalt','lime','limestone','lithium','lubricant','magnesium','manganese','heavy equipment','concrete','molybdenum','nano3 mined','nano3 synthetic','neodymium','nickel','xover grid x15xxx','xover grid x5xxx','paint','lead','plastics','polypropylene','rock','rock wool','sand','silicon sand','silicon wafer modules','silver','site preparation','tin','soda ash','steel','synthetic oil','tellurium','titanium','titanium dioxide','vanadium','wires','zinc']},
        dims=['res elec','materials'])
    data = [[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],

        xmaterials_for_oxm_per_capacity_installed_x_wind_onshorex(),

        xmaterials_for_oxm_per_capacity_installed_x_wind_offshorex(),

        xmaterials_for_oxm_per_capacity_installed_x_pvx(),

        xmaterials_for_oxm_per_capacity_installed_x_cspx()]
    ret.values = list(map(lambda x: x if type(x) != xr.DataArray else xr.align(ret, x, join='left')[1], data))
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_energy_requirements_oxm_for_water_consumption_res_elecx():
    """
    XTotal energy requirements OXM for water consumption RES elecX



    component


    """

    ret =  functions.sum(dim='water', data=xenergy_requirements_for_oxm_for_water_consumption_res_elecx())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xenergy_requirements_for_oxm_for_water_consumption_res_elecx():
    """
    XEnergy requirements for OXM for water consumption RES elecX



    component


    """

    ret =  energy_requirements_per_unit_of_water_consumption(
    ) * xwater_for_oxm_required_for_res_elecx() * kg_per_mt() / mj_per_ej()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xwater_for_oxm_x_res_elecx():
    """
    Xwater for OXM X RES elecX



    constant


    """

    ret =  xr.DataArray(
        data=np.zeros((8, 2)),
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp'], 'water':['xcleanx pumped waterx','xdistilledx deionized waterx']},
        dims=['res elec','water'])
    data = [[0.0,0.0],
        [0.0,0.0],
        [0.0,0.0],
        [0.0,0.0],

        xwater_for_oxm_x_wind_onshorex(),

        xwater_for_oxm_x_wind_offshorex(),

        xwater_for_oxm_x_pvx(),

        xwater_for_oxm_x_cspx()]
    ret.values = list(map(lambda x: x if type(x) != xr.DataArray else xr.align(ret, x, join='left')[1], data))
    name = 1
    return round_sig(x=ret, name=name)

# WATER - demand
@cache('run')
def energy_requirements_per_unit_of_water_consumption():
    """
    Energy requirements per unit of water consumption



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.0025], coords={'water': ['xcleanx pumped waterx']}, dims=['water']),
        xr.DataArray(
            data=[0.025], coords={'water': ['xdistilledx deionized waterx']}, dims=['water']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xunlimited_nrexx():
    """
    Xunlimited NREXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
# RESOURCES - Uranium extraction
# TPED by fuel
# TPES
@cache('step')
def extraction_uranium_ej():
    """
    extraction uranium EJ



    component


    """

    ret =  functions.if_then_else(abs(xunlimited_nrexx() - 1) < equality_threshold,
                                  pe_demand_uranium_ej(),
                                  functions.if_then_else(abs(xunlimited_uraniumxx() - 1) < equality_threshold,
                                                         pe_demand_uranium_ej(),
                                                         np.minimum(pe_demand_uranium_ej(),
                                                                    max_extraction_uranium_ej())))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('step')
def urr_uranium():
    """
    URR uranium



    component


    """

    ret =  functions.if_then_else(
        abs(xunlimited_nrexx() - 1) < equality_threshold,
        urr_uranium_unlimited(),
        functions.if_then_else(abs(xunlimited_uraniumxx() - 1) < equality_threshold,
                               urr_uranium_unlimited(),
                               functions.if_then_else(abs(choose_extraction_uranium_curve() - 1) < equality_threshold,
                                                      urr_uranium_ewg13(),
                                                      functions.if_then_else(
                                                          abs(choose_extraction_uranium_curve() - 2) < equality_threshold,
                                                          urr_uranium_zittel12(),
                                                          urr_uranium_user_defined()))))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def urr_conv_oil():
    """
    URR conv oil



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_oilxx() - 1) < equality_threshold,
                                  functions.if_then_else(
                                      abs(xunlimited_nrexx() - 1) < equality_threshold,
                                      urr_conv_oil_unlimited(),
                                      functions.if_then_else(
                                          abs(xunlimited_oilxx() - 1) < equality_threshold,
                                          urr_conv_oil_unlimited(),
                                          functions.if_then_else(
                                              abs(choose_extraction_curve_conv_oil() - 1) < equality_threshold,
                                              urr_conv_oil_maggio12_middle(),
                                              functions.if_then_else(
                                                  abs(choose_extraction_curve_conv_oil() - 2) < equality_threshold,
                                                  urr_conv_oil_maggio12_high(),
                                                  functions.if_then_else(
                                                      abs(choose_extraction_curve_conv_oil() - 3) < equality_threshold,
                                                      urr_conv_oil_maggio12_low(),
                                                      urr_conv_oil_user_defined()))))), 0)
    name = 1
    return round_sig(x=ret, name=name)
# RES BIOE - Biogas

def historic_biogas_pes(x):
    """
    Historic biogas PES



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.13135264000000002, 0.14522501999999998, 0.16244876, 0.17106013, 0.25641764, 0.28492731,
        0.33951272, 0.39872446, 0.41243782, 0.45342303, 0.49843029, 0.53353154, 0.6900384399999999,
        0.79629398, 0.90816046, 0.8367312299999999, 0.98802405, 1.0866051, 1.19932464, 1.26714942
    ])
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def growth_biogas():
    """
    growth biogas



    component


    """

    ret =  biogas_evol() * past_biogas_growth()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('run')
def past_biogas_growth():
    """
    past biogas growth



    constant


    """

    ret =  0.13194015127346587
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('step')
def pes_biogas_ej():
    """
    PES Biogas EJ



    component


    """


    ret =  integ_pes_biogas_ej()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Biogas
@cache('run')
def biogas_evol():
    """
    biogas evol



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
# RES BIOE - Biogas
# HEAT RES - Potential
@cache('run')
def max_biogas_ej():
    """
    max biogas EJ



    constant


    """

    ret =  5
    name = 1
    return round_sig(x=ret, name=name)
# PES solids biofuels & waste
# COMP - Historical data

def historic_tpes_primary_solids_biofuels_ej(x):
    """
    Historic TPES primary solids biofuels EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        39.187146320000004, 39.76702503, 40.11931938, 40.5134706, 40.893591210000004, 41.10535763,
        41.15765571, 41.79125286, 42.83426437, 43.73486414, 44.677431979999994, 45.46331815999999,
        46.33799293, 46.95628118, 47.306617700000004, 49.05395875, 49.46683572, 50.816292020000006,
        51.921499829999995, 52.55214558
    ])
    name = 1
    return round_sig(x=ret, name=name)
# SOLIDS - PED, PES & FES

def historic_pes_peat_ej(x):
    """
    Historic PES peat EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.22106789999999998, 0.23877885999999998, 0.22058298, 0.19908862, 0.17753217000000002,
        0.16850551, 0.18371079999999998, 0.19475629, 0.19557075, 0.17127541000000002, 0.15939893,
        0.18059229999999998, 0.18713522, 0.17082508, 0.15967739, 0.1799199, 0.1704865, 0.15029599,
        0.13919226, 0.13753595999999998
    ])
    name = 1
    return round_sig(x=ret, name=name)

# SOLIDS - PED, PES & FES
# IMPACTS - Total GHG emissions
@cache('step')
def pes_peat_ej():
    """
    PES peat EJ



    component


    """

    ret =  np.maximum(
        functions.if_then_else(time() > 2014, -0.0125 * (time()) + 25.3125,
                               historic_pes_peat_ej(time())), 0)
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
# EROI - Required final energy invested RES
@cache('step')
def fei_over_lifetime_res_elec_dispatch():
    """
    FEI over lifetime RES elec dispatch



    component


    """

    ret =  cedtot_per_tw_over_lifetime_res_elec_dispatch(
    ) * res_elec_capacity_under_construction_tw()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# EROIgrid - Allocation RES elec
@cache('run')
def activate_eroi_allocation_rule():
    """
    activate EROI allocation rule



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# EROIgrid - Allocation RES elec
@cache('step')
def eroi_allocation_rule_per_res_elec():
    """
    EROI allocation rule per RES elec



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, 1,
        functions.if_then_else(
            abs(xratio_eroi_per_techn_vs_eroitot_xstaticxx() ) < equality_threshold, 0,
            functions.if_then_else(single_precision(0.1) > single_precision(xratio_eroi_per_techn_vs_eroitot_xstaticxx()) + less_thr_func(0.1, xratio_eroi_per_techn_vs_eroitot_xstaticxx()), 0,
                0.434294 * np.log(xratio_eroi_per_techn_vs_eroitot_xstaticxx()) + 1)))
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xxdynamicx_eroi_res_elec_varx():
    """
    XXdynamicX EROI RES elec varX



    component


    """

    ret =  functions.if_then_else(abs(fei_res_elec_var() ) < equality_threshold, 0,
                                  real_generation_res_elec_ej() / fei_res_elec_var())
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xeroixini_res_elec_dispatchx():
    """
    XEROIXini RES elec dispatchX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[50.], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[7.], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[1.5], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[3.25], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Heat generation

def share_renewables_fuel_chp_iea_ej(x):
    """
    share renewables fuel CHP IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.007388682, 0.012521189, 0.012854715, 0.013195454, 0.011547294, 0.011753827, 0.01158899,
        0.012564891, 0.014253632, 0.014147948, 0.014746481, 0.016142735, 0.018131825, 0.019864843,
        0.021452481, 0.023580794, 0.023474059, 0.025863853, 0.028955449, 0.030543604
    ])



    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Heat generation

def share_renewables_heat_iea_ej(x):
    """
    share renewables Heat IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.010911371, 0.011618286, 0.011440809, 0.011591345, 0.013006855, 0.012420881, 0.013177455,
        0.013471765, 0.013733987, 0.014226124, 0.01484072, 0.015011934, 0.01560381, 0.016078662,
        0.017549775, 0.017892274, 0.01649884, 0.017476516, 0.018336711, 0.018998007
    ])



    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_coal_heat_check():
    """
    share coal Heat check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (share_coal_heat_ej() -
                        (share_coal_heat_iea_ej(time()))) * 100 / share_coal_heat_iea_ej(time()),
        0)
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_nuclear_heat_check():
    """
    share nuclear Heat check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (share_nuclear_heat_ej() -
         (share_nuclear_heat_iea_ej(time()))) * 100 / share_nuclear_heat_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Heat generation

def share_nuclear_heat_iea_ej(x):
    """
    share nuclear Heat IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.001467247, 0.001793372, 0.001699785, 0.00155326, 0.001625836, 0.001560707, 0.001771427,
        0.001751567, 0.001722431, 0.001776911, 0.001537023, 0.001612582, 0.002115042, 0.002071281,
        0.002152061, 0.001937993, 0.001915221, 0.001883117, 0.001949081, 0.001914596
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Heat generation

def share_gases_heat_iea_ej(x):
    """
    share gases Heat IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.265677389, 0.255892017, 0.273098291, 0.281041979, 0.292601939, 0.269992897, 0.267672033,
        0.267486525, 0.268352067, 0.266451191, 0.249871066, 0.240462503, 0.230559824, 0.226335569,
        0.22389501, 0.219367053, 0.221081987, 0.214417059, 0.181150891, 0.174354169
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Heat generation

def share_coal_heat_iea_ej(x):
    """
    share coal Heat IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.149438446, 0.153211456, 0.158328472, 0.152201922, 0.159692751, 0.169871225, 0.173951402,
        0.179353927, 0.182446615, 0.187789231, 0.208850958, 0.215670113, 0.227609035, 0.229360061,
        0.236148759, 0.240037492, 0.248599252, 0.262116093, 0.282901391, 0.282066169
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Heat generation

def share_liquids_heat_iea_ej(x):
    """
    share liquids Heat IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.094214549, 0.084933005, 0.06621254, 0.077426769, 0.070781899, 0.060117212, 0.060282593,
        0.052420883, 0.051037593, 0.044430359, 0.041036801, 0.037232358, 0.035203276, 0.033972393,
        0.035764826, 0.038734289, 0.034822368, 0.03311725, 0.032143982, 0.033053979
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_fossil_fuel_chp_check():
    """
    share fossil fuel CHP check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (share_fossil_fuel_chp_ej() -
         (share_fossil_fuel_chp_iea_ej(time()))) * 100 / share_fossil_fuel_chp_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_gases_heat_check():
    """
    share gases Heat check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (share_gases_heat_ej() -
                        (share_gases_heat_iea_ej(time()))) * 100 / share_gases_heat_iea_ej(time()),
        0)
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Heat generation

def share_fossil_fuel_chp_iea_ej(x):
    """
    share fossil fuel CHP IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.466584673, 0.476257133, 0.472014375, 0.45773556, 0.44457521, 0.467371814, 0.46474411,
        0.466255719, 0.461610666, 0.464072063, 0.434192087, 0.439381536, 0.437808399, 0.440720844,
        0.43262677, 0.425496855, 0.421427326, 0.412960497, 0.419782763, 0.422534077
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Heat generation
@cache('step')
def share_liquids_heat_check():
    """
    share liquids Heat check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (share_liquids_heat_ej() -
         (share_liquids_heat_iea_ej(time()))) * 100 / share_liquids_heat_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def cp_historic_csp():
    """
    Cp historic CSP



    component


    """

    ret =  functions.if_then_else(time() < 2000, 0.15,
                                  functions.if_then_else(time() < 2014,
                                                         0.00714286 * time() - 14.1357, 0.25))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# ELEC RES - Total monetary investments
# RES - Employment
# MATERIALS - Demand for RES ELEC
# WATER - demand
# EROI - RES elec
# EROI - RES elec
@cache('step')
def res_elec_capacity_under_construction_tw():
    """
    RES elec capacity under construction TW



    component


    """

    ret =  res_elec_planned_capacity_tw() / time_construction_res_elec()
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Elec production

def share_liquids_elec_iea(x):
    """
    share liquids Elec IEA



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.0826, 0.0796, 0.078, 0.0791, 0.0762, 0.0719, 0.0687, 0.0655, 0.0634, 0.0602, 0.0573,
        0.0507, 0.0507, 0.0477, 0.0443, 0.0418, 0.0449, 0.0464, 0.043, 0.0399
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_nuclear_elec():
    """
    share Nuclear Elec



    component


    """

    ret =  fe_nuclear_elec_generation_twh() / total_fe_elec_demand_twh()
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_nuclear_elec_check():
    """
    share Nuclear Elec check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (share_nuclear_elec() -
                        (share_nuclear_elec_iea(time()))) * 100 / share_nuclear_elec_iea(time()),
        0)
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Elec production

def share_nuclear_elec_iea(x):
    """
    share Nuclear Elec IEA



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.1746, 0.1755, 0.1701, 0.1694, 0.1705, 0.1667, 0.1682, 0.1633, 0.156, 0.1549, 0.1501,
        0.1458, 0.1361, 0.1342, 0.1328, 0.1274, 0.1156, 0.1076, 0.1053, 0.1057
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Elec production

def share_renewables_elec_chp_iea(x):
    """
    share renewables Elec CHP IEA



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.0043, 0.0039, 0.0042, 0.0042, 0.0045, 0.0043, 0.0043, 0.0045, 0.0048, 0.0049, 0.0051,
        0.0054, 0.0056, 0.0059, 0.0065, 0.007, 0.0072, 0.0078, 0.008, 0.0083
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Elec production

def share_renewables_elec_iea(x):
    """
    share Renewables Elec IEA



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.1974, 0.1942, 0.193, 0.1896, 0.1858, 0.1825, 0.1778, 0.1769, 0.1727, 0.177, 0.1785, 0.18,
        0.1776, 0.1843, 0.1919, 0.1944, 0.198, 0.2074, 0.2151, 0.2218
    ])



    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data II

def pe_gas_for_chp_plants_iea_ej(x):
    """
    PE gas for CHP plants IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        10.33257557, 10.67989242, 10.84886144, 10.68962976, 11.0816629, 10.49639976, 10.91461632,
        11.43656349, 12.05679544, 12.55013689, 13.00794104, 12.97521867, 13.30313208, 13.29276894,
        12.60497732, 13.91626607, 14.35718883, 14.7060233, 14.93825247, 14.20298496
    ])



    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data II
@cache('step')
def check_coal_primary_chp_plants():
    """
    check coal primary CHP plants



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (ped_coal_for_chp_plants_ej() -
         (pe_coal_for_chp_plants_iea_ej(time()))) * 100 / pe_coal_for_chp_plants_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data II

def pe_demand_coal_for_heat_plants_iea_ej(x):
    """
    PE demand coal for Heat plants IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        2.39888095, 2.369, 2.309, 2.237, 2.316, 2.765, 2.797, 2.808, 3.265, 3.346, 3.724, 3.939,
        4.136, 3.96, 3.951, 4.502, 4.7, 5.443, 5.224, 5.021
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data II

def pe_demand_for_liquids_heat_plants_iea_ej(x):
    """
    PE demand for liquids Heat plants IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        1.41770612, 1.291, 0.966, 1.119, 1.008, 0.905, 0.899, 0.813, 0.794, 0.7, 0.66, 0.608,
        0.564, 0.533, 0.562, 0.658, 0.607, 0.585, 0.556, 0.581
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data II

def pe_liquids_for_chp_plants_iea_ej(x):
    """
    PE liquids for CHP plants IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        1.75700741, 1.817, 1.742, 1.717, 1.816, 1.558, 1.34, 1.413, 1.508, 1.486, 1.427, 1.425,
        1.234, 1.1, 1.092, 0.95, 1.094, 1.081, 0.703, 0.715
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data II
@cache('step')
def check_gas_primary_chp_plants():
    """
    check gas primary CHP plants



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (ped_gas_for_chp_plants_ej() -
         (pe_gas_for_chp_plants_iea_ej(time()))) * 100 / pe_gas_for_chp_plants_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data II
@cache('step')
def check_primary_liquids_elec_plants():
    """
    check primary liquids Elec plants



    component


    """

    ret =  functions.if_then_else(time() < 2015, (pe_demand_oil_elec_plants_ej() -
                                                  (pe_demand_liquids_elec_plants_iea_ej(time()))) *
                                  100 / pe_demand_liquids_elec_plants_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)
# ELEC - Electricity demand covered by fossil fuels
# MATERIALS - Recycling & material extraction deman
# MATERIALS - Recycling & material extraction deman
# COMP- Historical data II

def pe_demand_liquids_elec_plants_iea_ej(x):
    """
    PE demand liquids Elec plants IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        11.29650142, 11.096, 11.734, 11.793, 11.045, 10.921, 10.704, 10.439, 10.435, 10.58, 10.595,
        9.776, 10.022, 9.714, 9.502, 9.487, 10.39, 11.044, 10.722, 10.154
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data II
@cache('step')
def check_primary_liquids_heat_plants():
    """
    check primary liquids Heat plants



    component


    """

    ret =  functions.if_then_else(time() < 2015,
                                  (ped_oil_for_heat_plants_ej() -
                                   (pe_demand_for_liquids_heat_plants_iea_ej(time()))) * 100 /
                                  pe_demand_for_liquids_heat_plants_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Elec production

def share_fossil_fuel_elec_chp_iea(x):
    """
    share fossil fuel Elec CHP IEA



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.1128, 0.1114, 0.1086, 0.1045, 0.1029, 0.1022, 0.1033, 0.1034, 0.1079, 0.1046, 0.1025,
        0.1005, 0.0971, 0.0961, 0.0909, 0.0905, 0.0875, 0.0863, 0.0819, 0.0788
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data II

def pe_coal_for_chp_plants_iea_ej(x):
    """
    PE coal for CHP plants IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        6.86610114, 7.157, 6.621, 6.277, 6.255, 6.765, 6.537, 6.65, 7.228, 7.049, 7.298, 7.67,
        7.36, 7.309, 6.567, 6.922, 6.951, 6.897, 6.809, 6.339
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data II
@cache('step')
def check_primary_gas_elec_gas():
    """
    check primary gas Elec gas



    component


    """

    ret =  functions.if_then_else(time() < 2015, (pe_demand_gas_elec_plants_ej() -
                                                  (pe_demand_gas_elec_plants_iea_ej(time()))) * 100
                                  / pe_demand_gas_elec_plants_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_coal_elec():
    """
    share coal Elec



    component


    """

    ret =  fe_demand_coal_elec_plants_twh() / total_fe_elec_demand_twh()
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_coal_elec_check():
    """
    share coal Elec check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (share_coal_elec() -
                        (share_coal_elec_iea(time()))) * 100 / share_coal_elec_iea(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data II
@cache('step')
def check_liquids_chp_plants():
    """
    check liquids CHP plants



    component


    """

    ret =  functions.if_then_else(time() < 2015, (ped_oil_for_chp_plants_ej() -
                                                  (pe_liquids_for_chp_plants_iea_ej(time()))) * 100
                                  / pe_liquids_for_chp_plants_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data II

def pe_demand_coal_elec_plants_iea_ej(x):
    """
    PE demand coal Elec plants IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        45.9923162, 48.557, 49.265, 50.612, 51.031, 54.597, 56.273, 57.726, 61.743, 64.855, 68.08,
        71.867, 76.053, 75.401, 74.589, 78.796, 82.871, 82.853, 85.869, 86.718
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data II
@cache('step')
def check_primary_coal_elec_plants():
    """
    check primary coal Elec plants



    component


    """

    ret =  functions.if_then_else(time() < 2015, (pe_demand_coal_elec_plants_ej() -
                                                  (pe_demand_coal_elec_plants_iea_ej(time()))) *
                                  100 / pe_demand_coal_elec_plants_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data II

def pe_demand_gases_for_heat_plants_iea_ej(x):
    """
    PE demand gases for Heat plants IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        4.4536318, 4.559, 4.465, 4.383, 4.65, 4.55, 4.587, 4.532, 4.84, 4.598, 4.44, 4.41, 4.184,
        4.032, 3.921, 4.241, 4.421, 4.417, 3.946, 3.761
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_gases_elec():
    """
    share gases Elec



    component


    """

    ret =  fe_demand_gas_elec_plants_twh() / total_fe_elec_demand_twh()
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_gases_elec_check():
    """
    share gases Elec check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (share_gases_elec() -
                        (share_gases_elec_iea(time()))) * 100 / share_gases_elec_iea(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data II

def pe_demand_gas_elec_plants_iea_ej(x):
    """
    PE demand gas Elec plants IEA EJ



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        12.91270427, 13.45145231, 14.39230377, 15.6005989, 16.70930053, 17.75362056, 18.77084025,
        19.96206897, 20.52697026, 22.09406967, 23.60089258, 25.26878797, 27.03100179, 28.0720165,
        28.69614096, 31.26179042, 31.32289903, 32.96342944, 32.17809027, 33.03257109
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_liquids_elec_check():
    """
    share liquids Elec check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, (share_liquids_elec() -
                        (share_liquids_elec_iea(time()))) * 100 / share_liquids_elec_iea(time()),
        0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xcheck_primary_liquids_for_elecx_heat_and_chp_plantsx():
    """
    Xcheck primary liquids for ElecX Heat and CHP plantsX



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        ((pe_demand_oil_elec_plants_ej() + ped_oil_for_chp_plants_ej() +
          ped_oil_for_heat_plants_ej()) -
         (pe_demand_liquids_elec_plants_iea_ej(time()) + pe_liquids_for_chp_plants_iea_ej(time()) +
          pe_demand_for_liquids_heat_plants_iea_ej(time()))) * 100 /
        (pe_demand_for_liquids_heat_plants_iea_ej(time()) + pe_demand_liquids_elec_plants_iea_ej(
            time()) + pe_liquids_for_chp_plants_iea_ej(time())), 0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xcheck_primary_gas_for_elecx_heat_and_chp_plantsx():
    """
    Xcheck primary gas for ElecX Heat and CHP plantsX



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        ((pe_demand_gas_elec_plants_ej() + ped_gases_for_heat_plants_ej() +
          ped_gas_for_chp_plants_ej()) -
         (pe_demand_gas_elec_plants_iea_ej(time()) + pe_demand_gases_for_heat_plants_iea_ej(
             time()) + pe_gas_for_chp_plants_iea_ej(time()))) * 100 /
        (pe_demand_gas_elec_plants_iea_ej(time()) +
         pe_demand_gases_for_heat_plants_iea_ej(time()) + pe_gas_for_chp_plants_iea_ej(time())), 0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xcheck_primary_coal_for_elecx_heat_and_chp_plantsx():
    """
    Xcheck primary coal for ElecX Heat and CHP plantsX



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, ((ped_coal_for_chp_plants_ej() + pe_demand_coal_elec_plants_ej() +
                         ped_coal_for_heat_plants_ej()) -
                        (pe_coal_for_chp_plants_iea_ej(time()) + pe_demand_coal_elec_plants_iea_ej(
                            time()) + pe_demand_coal_for_heat_plants_iea_ej(time()))) * 100 /
        (pe_coal_for_chp_plants_iea_ej(time()) + pe_demand_coal_elec_plants_iea_ej(time()) +
         pe_demand_coal_for_heat_plants_iea_ej(time())), 0)
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data II
@cache('step')
def check_primary_coal_for_heat_plants():
    """
    check primary coal for Heat plants



    component


    """

    ret =  functions.if_then_else(time() < 2015, (ped_coal_for_heat_plants_ej() -
                                                  (pe_demand_coal_for_heat_plants_iea_ej(time())))
                                  * 100 / pe_demand_coal_for_heat_plants_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Elec production

def share_gases_elec_iea(x):
    """
    share gases Elec IEA



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.096, 0.0965, 0.1043, 0.111, 0.1188, 0.1223, 0.1282, 0.133, 0.1335, 0.139, 0.1422, 0.1469,
        0.1542, 0.1591, 0.164, 0.1673, 0.1652, 0.1692, 0.1629, 0.1654
    ])
    name = 1
    return round_sig(x=ret, name=name)
# COMP- Historical data Elec production

def share_coal_elec_iea(x):
    """
    share Coal Elec IEA



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.3322, 0.3388, 0.3418, 0.3423, 0.3414, 0.35, 0.3495, 0.3534, 0.3617, 0.3594, 0.3642,
        0.3707, 0.3786, 0.3727, 0.3696, 0.3716, 0.3817, 0.3753, 0.3837, 0.38
    ])
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_fossil_fuel_chp_elec_check():
    """
    share fossil fuel CHP Elec check



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (share_fossil_fuel_chp_elec() -
         (share_fossil_fuel_elec_chp_iea(time()))) * 100 / share_fossil_fuel_elec_chp_iea(time()),
        0)
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data II
@cache('step')
def check_primary_gases_heat_plants():
    """
    check primary gases Heat plants



    component


    """

    ret =  functions.if_then_else(time() < 2015, (ped_gases_for_heat_plants_ej() -
                                                  (pe_demand_gases_for_heat_plants_iea_ej(time())))
                                  * 100 / pe_demand_gases_for_heat_plants_iea_ej(time()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# COMP- Historical data Elec production
@cache('step')
def share_fossil_fuel_chp_elec():
    """
    share fossil fuel CHP Elec



    component


    """

    ret =  fes_elec_fossil_fuel_chp_plants_twh() / total_fe_elec_demand_twh()
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('step')
def real_generation_solar_pv_ej():
    """
    real generation solar PV EJ



    component


    """

    ret =  real_generation_res_elec_ej().loc[{'res elec':'solar pv'}].values
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def materials_required_for_new_csp_mt():
    """
    materials required for new CSP Mt



    component


    """

    ret =  materials_required_for_new_res_elec_mt().loc[{'res elec':'csp'}].drop('res elec')
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def materials_required_for_new_pv_mt():
    """
    materials required for new PV Mt



    component


    """

    ret =  materials_required_for_new_res_elec_mt().loc[{'res elec':'solar pv'}].drop('res elec')
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def materials_required_for_new_wind_offshore_mt():
    """
    materials required for new wind offshore Mt



    component


    """

    ret =  materials_required_for_new_res_elec_mt().loc[{'res elec':'wind offshore'}].drop('res elec')
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def materials_required_for_new_wind_onshore_mt():
    """
    materials required for new wind onshore Mt



    component


    """

    ret =  materials_required_for_new_res_elec_mt().loc[{'res elec':'wind onshore'}].drop('res elec')
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmaterials_required_for_oxm_csp_mtx():
    """
    Xmaterials required for OXM CSP MtX



    component


    """

    ret =  xmaterials_required_for_oxm_res_elec_mtx().loc[{'res elec':'csp'}].drop('res elec')
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmaterials_required_for_oxm_pv_mtx():
    """
    Xmaterials required for OXM PV MtX



    component


    """

    ret =  xmaterials_required_for_oxm_res_elec_mtx().loc[{'res elec':'solar pv'}].drop('res elec')
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmaterials_required_for_oxm_wind_offshore_mtx():
    """
    Xmaterials required for OXM wind offshore MtX



    component


    """

    ret =  xmaterials_required_for_oxm_res_elec_mtx().loc[{'res elec':'wind offshore'}].drop('res elec')
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmaterials_required_for_oxm_wind_onshore_mtx():
    """
    Xmaterials required for OXM wind onshore MtX



    component


    """

    ret =  xmaterials_required_for_oxm_res_elec_mtx().loc[{'res elec':'wind onshore'}].drop('res elec')
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
# EROI - RES elec
@cache('step')
def real_generation_res_elec_ej():
    """
    real generation RES elec EJ



    component


    """

    ret =  real_generation_res_elec_twh() * ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xenergy_ownxuse_constant_830_mtoe_in_2013x():
    """
    Xenergy ownXuse constant 830 Mtoe in 2013X



    constant


    """

    ret =  830
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xpipeline_transport_constant_2x6_ej_in_2014x():
    """
    Xpipeline transport constant 2X6 EJ in 2014X



    constant


    """

    ret =  2.6
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
@cache('step')
def cedtot_solar_pv():
    """
    CEDtot solar PV



    component


    """

    ret =  fei_res_elec_var().loc[{'res elec':'solar pv'}].values
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def total_materials_required_for_new_res_elec_mt():
    """
    Total materials required for new RES elec Mt



    component


    """

    ret =  functions.sum(dim='res elec', data=materials_required_for_new_res_elec_mt())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_materials_required_for_oxm_res_elec_mtx():
    """
    XTotal materials required for OXM RES elec MtX



    component


    """

    ret =  functions.sum(dim='res elec', data=xmaterials_required_for_oxm_res_elec_mtx())
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xeroix1x():
    """
    XEROIX1X



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
@cache('step')
def electrical_distribution_losses_twh():
    """
    Electrical distribution losses TWh



    component


    """

    ret =  total_fe_elec_demand_twh() * xshare_transmxdistr_elec_lossesx()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmax_share_transmxdistr_elec_lossesx():
    """
    XMax share transmXdistr elec lossesX



    component


    """

    ret =  xshare_transmxdistr_elec_losses_initialx() * (1 + 0.0115 * np.exp(4.2297 * 1) - 0.00251)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xheatxcom_distribution_lossesx():
    """
    XHeatXcom distribution lossesX



    component


    """

    ret =  xfed_heatxcom_ejx() * share_heat_distribution_losses()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# ELEC RES - Capacities and generation all techn
@cache('step')
def potential_generation_res_elec_twh():
    """
    potential generation RES elec TWh



    component


    """

    ret =  installed_capacity_res_elec_tw() * cp_res_elec() / twe_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# EROI - RES elec
# EROIgrid - Allocation RES elec
# EROItot - Electricity
@cache('step')
def output_elec_over_lifetime_res_elec():
    """
    output elec over lifetime RES elec



    component


    """

    ret =  real_cp_res_elec() * res_elec_capacity_under_construction_tw() * (
        1 / twe_per_twh()) * lifetime_res_elec() * ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xvariation_share_transmxdistr_elec_lossesx():
    """
    Xvariation share transmXdistr elec lossesX



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0,
                                  xvariation_share_transmxdistr_losses_elecx() *
                                  xremaining_share_transmxdistr_elec_lossesx())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xremaining_share_transmxdistr_elec_lossesx():
    """
    Xremaining share transmXdistr elec lossesX



    component


    """

    ret =  (xmax_share_transmxdistr_elec_lossesx() - xshare_transmxdistr_elec_lossesx()
            ) / xmax_share_transmxdistr_elec_lossesx()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_transmxdistr_elec_lossesx():
    """
    Xshare transmXdistr elec lossesX



    component


    """


    ret =  integ_xshare_transmxdistr_elec_lossesx()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('step')
def total_gen_losses_demand_for_elec_plants_ej():
    """
    Total gen losses demand for Elec plants EJ



    component


    """

    ret =  pe_demand_gas_elec_plants_ej() * (
        1 - efficiency_gas_for_electricity()) + pe_demand_coal_elec_plants_ej() * (
            1 - efficiency_coal_for_electricity()) + pe_demand_oil_elec_plants_ej() * (
                1 - efficiency_liquids_for_electricity()
            ) + pe_losses_uranium_for_elec_ej() + pe_losses_bioe_for_elec_ej()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC - Electricity demand covered by fossil fuels
# ELEC - Electricity related losses
@cache('step')
def pe_losses_bioe_for_elec_ej():
    """
    PE losses BioE for Elec EJ



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'xsolid bioexelecx'}].values - fe_elec_generation_from_bioe_twh() * ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xwater_for_oxm_x_wind_offshorex():
    """
    Xwater for OXM X wind offshoreX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'water': ['xcleanx pumped waterx']}, dims=['water']),
        xr.DataArray(
            data=[0.], coords={'water': ['xdistilledx deionized waterx']}, dims=['water']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
# MATERIALS - Rest demand
@cache('step')
def total_materials_required_for_res_elec_mt():
    """
    Total materials required for RES elec Mt



    component


    """

    ret =  total_materials_required_for_new_res_elec_mt(
    ) + xtotal_materials_required_for_oxm_res_elec_mtx()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
@cache('step')
def share_energy_for_material_consumption_for_alt_techn_vs_tfec():
    """
    share energy for material consumption for alt techn vs TFEC



    component


    """

    ret =  total_energy_required_for_total_material_consumption_for_alt_techn() / real_tfec()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xwater_for_oxm_x_pvx():
    """
    Xwater for OXM X PVX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'water': ['xcleanx pumped waterx']}, dims=['water']),
        xr.DataArray(
            data=[20000.], coords={'water': ['xdistilledx deionized waterx']}, dims=['water']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xwater_for_oxm_x_wind_onshorex():
    """
    Xwater for OXM X wind onshoreX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'water': ['xcleanx pumped waterx']}, dims=['water']),
        xr.DataArray(
            data=[0.], coords={'water': ['xdistilledx deionized waterx']}, dims=['water']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_for_oxm_per_capacity_installed_x_wind_offshorex():
    """
    Xmaterials for OXM per capacity installed X wind offshoreX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[10.8], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[59.6], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[5.8], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[356.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[122.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[19.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[25.6], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[9.2], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_for_oxm_per_capacity_installed_x_wind_onshorex():
    """
    Xmaterials for OXM per capacity installed X wind onshoreX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[10.8], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[29.8], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[5.8], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[65.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[122.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[19.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[25.6], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[9.2], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_for_oxm_per_capacity_installed_x_pvx():
    """
    Xmaterials for OXM per capacity installed X PVX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[1294.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[2.56], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.04], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xwater_for_oxm_x_cspx():
    """
    Xwater for OXM X CSPX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[12000000.], coords={'water': ['xcleanx pumped waterx']}, dims=['water']),
        xr.DataArray(
            data=[500000.], coords={'water': ['xdistilledx deionized waterx']}, dims=['water']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_per_capacity_installed_x_pv_0x():
    """
    Xmaterials per capacity installed X PV 0X



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[16000.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[6.1], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[75000.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[550.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[2200.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[15600.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[150.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.3], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[640000.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[500000.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[4.5], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[162500.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[53.5], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[500.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[100.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[50.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[235.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[21.2], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[5760.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[3200.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[46.7], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(
            data=[6000000.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[463.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[2000.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[4.7], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[6.25], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.475], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[162.5], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def cum_materials_requirements_for_res_elec():
    """
    cum materials requirements for RES elec



    component


    """


    ret =  integ_cum_materials_requirements_for_res_elec()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Energy consumption
# WATER - demand
# ESOI - EV batteries
# EROI - RES elec
# EROI - RES elec
# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
@cache('run')
def mj_per_ej():
    """
    MJ per EJ



    constant


    """

    ret =  1000000000000
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xinitial_energy_cons_per_unit_of_material_cons_xvirginxx():
    """
    XInitial energy cons per unit of material cons XvirginXX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[100.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[218.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[264.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[200.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[4.5], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[83.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[57.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[38.5], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[384.2], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[2000.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[120.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[28.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[28.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[610000.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[15.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[100.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[0.083], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[3320.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[25.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[21.5], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[3.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[5.3], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.85], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[853.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[100.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[220.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[57.6], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[2000.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.95], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[378.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[21.5], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[384.2], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[164.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[70.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[49.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[80.5], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[95.4], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[1.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[16.6], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.081], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.1], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[2000.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[1580.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(data=[0.45], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[250.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[56.7], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[207.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[589000.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[400.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[60.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[3710.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[36.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[72.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_per_new_capacity_installed_x_cspx():
    """
    Xmaterials per new capacity installed X CSPX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[740.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(
            data=[3280.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[250000.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[2200.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[3200.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[15600.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[150.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[310.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[2500.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[130000.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[500000.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[650000.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[220000.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[11000.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[170000.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[3000.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[2000.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[100.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[200.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[340000.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(
            data=[340000.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[940.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[500.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[1300000.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[4700.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[1900.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[92000.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[13.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(
            data=[18000000.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[18000.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[240000.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[44000.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[25.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[11.5], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[1.9], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[650.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_per_new_capacity_installed_x_wind_offshorex():
    """
    Xmaterials per new capacity installed X wind offshoreX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.74], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[9400.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[3800.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[24000.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[22200.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[18080.88818398], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[14.58], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[450.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[3800.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[950.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[900000.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[640.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(
            data=[115.90312938], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[183.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[111.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[670.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[9200.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[16560.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(
            data=[12000000.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[400000.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[640.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_per_new_capacity_installed_x_wind_onshorex():
    """
    Xmaterials per new capacity installed X wind onshoreX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.74], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[2030.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[1500.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[561600.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[2700.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[5700.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[4.86], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[450.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[6090.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[950.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[11900.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[22000.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[640.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(
            data=[36.53846154], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[61.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[111.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[670.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[1940.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[16560.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(
            data=[1500000.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[126100.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[640.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('run')
def initial_cumulated_material_requirements_for_res_elec_1995():
    """
    initial cumulated material requirements for RES elec 1995



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmaterials_per_new_capacity_installed_x_pvx():
    """
    Xmaterials per new capacity installed X PVX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.], coords={'materials': ['adhesive']}, dims=['materials']),
        xr.DataArray(data=[16000.], coords={'materials': ['aluminium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['aluminium mirrors']}, dims=['materials']),
        xr.DataArray(data=[6.1], coords={'materials': ['cadmium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['carbon fiber']}, dims=['materials']),
        xr.DataArray(data=[75000.], coords={'materials': ['cement']}, dims=['materials']),
        xr.DataArray(data=[550.], coords={'materials': ['chromium']}, dims=['materials']),
        xr.DataArray(data=[2200.], coords={'materials': ['copper']}, dims=['materials']),
        xr.DataArray(data=[15600.], coords={'materials': ['diesel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['dy']}, dims=['materials']),
        xr.DataArray(
            data=[0.],
            coords={'materials': ['xelectricxelectronic componentsx']},
            dims=['materials']),
        xr.DataArray(data=[150.], coords={'materials': ['evacuation lines']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['fiberglass']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['foam glass']}, dims=['materials']),
        xr.DataArray(data=[0.3], coords={'materials': ['galium']}, dims=['materials']),
        xr.DataArray(data=[640000.], coords={'materials': ['glass']}, dims=['materials']),
        xr.DataArray(
            data=[0.], coords={'materials': ['glass reinforcing plastic']}, dims=['materials']),
        xr.DataArray(data=[500000.], coords={'materials': ['gravel']}, dims=['materials']),
        xr.DataArray(data=[4.5], coords={'materials': ['indium']}, dims=['materials']),
        xr.DataArray(data=[162500.], coords={'materials': ['iron']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['kno3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['asphalt']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lime']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['limestone']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lithium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['lubricant']}, dims=['materials']),
        xr.DataArray(data=[53.5], coords={'materials': ['magnesium']}, dims=['materials']),
        xr.DataArray(data=[500.], coords={'materials': ['manganese']}, dims=['materials']),
        xr.DataArray(data=[100.], coords={'materials': ['heavy equipment']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['concrete']}, dims=['materials']),
        xr.DataArray(data=[50.], coords={'materials': ['molybdenum']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 mined']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['nano3 synthetic']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['neodymium']}, dims=['materials']),
        xr.DataArray(data=[235.], coords={'materials': ['nickel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x15xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['xover grid x5xxx']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['paint']}, dims=['materials']),
        xr.DataArray(data=[21.2], coords={'materials': ['lead']}, dims=['materials']),
        xr.DataArray(data=[5760.], coords={'materials': ['plastics']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['polypropylene']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['rock wool']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['sand']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['silicon sand']}, dims=['materials']),
        xr.DataArray(
            data=[3200.], coords={'materials': ['silicon wafer modules']}, dims=['materials']),
        xr.DataArray(data=[46.7], coords={'materials': ['silver']}, dims=['materials']),
        xr.DataArray(
            data=[6000000.], coords={'materials': ['site preparation']}, dims=['materials']),
        xr.DataArray(data=[463.], coords={'materials': ['tin']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['soda ash']}, dims=['materials']),
        xr.DataArray(data=[2000.], coords={'materials': ['steel']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['synthetic oil']}, dims=['materials']),
        xr.DataArray(data=[4.7], coords={'materials': ['tellurium']}, dims=['materials']),
        xr.DataArray(data=[6.25], coords={'materials': ['titanium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['titanium dioxide']}, dims=['materials']),
        xr.DataArray(data=[0.475], coords={'materials': ['vanadium']}, dims=['materials']),
        xr.DataArray(data=[0.], coords={'materials': ['wires']}, dims=['materials']),
        xr.DataArray(data=[162.5], coords={'materials': ['zinc']}, dims=['materials']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
# MATERIALS - Demand for EV batteries
# MATERIALS - Energy consumption
# WATER - demand
# EROI - RES elec
# EROI - RES elec
@cache('run')
def kg_per_mt():
    """
    kg per Mt



    constant


    """

    ret =  1000000000
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
# RES - Employment
# RES - Employment
# MATERIALS - Demand for RES ELEC
# MATERIALS - Demand for EV batteries
# WATER - demand
# ESOI - EV batteries
# EROI - RES elec
@cache('run')
def m_per_t():
    """
    M per T



    constant


    """

    ret =  1000000
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def elec_generation_variable_from_res_twh():
    """
    Elec generation variable from RES TWh



    component


    """

    ret =  fe_elec_generation_from_solar_pv_twh() + fe_elec_generation_from_csp_twh(
    ) + fe_elec_generation_from_onshore_wind_twh() + fe_elec_generation_from_offshore_wind_twh()
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('run')
def copper_for_solar_thermal():
    """
    copper for solar thermal



    constant


    """

    ret =  2
    name = 1
    return round_sig(x=ret, name=name)

# MATERIALS - Demand for RES ELEC
@cache('step')
def copper_demand_for_solar_thermal():
    """
    Copper demand for solar thermal



    component


    """

    ret =  area_solar_collectors() * copper_for_solar_thermal() / 1e+009
    name = 1
    return round_sig(x=ret, name=name)

# HEAT COM- Heat-com demand covered by fossil fuels
@cache('step')
def total_gen_losses_demand_for_heat_plants_ej():
    """
    Total gen losses demand for Heat plants EJ



    component


    """

    ret =  ped_gases_for_heat_plants_ej() * (
        1 - efficiency_gases_for_heat_plants(time())) + ped_oil_for_heat_plants_ej() * (
            1 - efficiency_liquids_for_heat_plants(time())) + ped_coal_for_heat_plants_ej() * (
                1 - efficiency_coal_for_heat_plants(time()))
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def total_gen_losses_demand_for_chp_plants_ej():
    """
    Total gen losses demand for CHP plants EJ



    component


    """

    ret =  ped_gas_for_chp_plants_ej() * (1 - efficiency_elec_gas_chp_plants(
        time()) - efficiency_heat_gas_chp_plants(time())) + ped_oil_for_chp_plants_ej() * (
            1 - efficiency_elec_oil_chp_plants(time()) - efficiency_heat_oil_chp_plants(time())
        ) + ped_coal_for_chp_plants_ej() * (
            1 - efficiency_heat_coal_chp_plants(time()) - efficiency_elec_coal_chp_plants(time()))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def new_required_capacity_nuclear():
    """
    new required capacity nuclear



    component


    """

    ret =  np.maximum(0,
                      functions.if_then_else(time() < 2014, 0,
                                             functions.if_then_else(abs(demand_elec_nre_twh() ) < equality_threshold, 0,
                                                                    installed_capacity_nuclear_tw(
                                                                    ) * p_nuclear_elec_gen()))
                      ) * effects_shortage_uranium() * cp_limit_nuclear()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xreplacement_res_for_heatxcomx():
    """
    Xreplacement RES for heatXcomX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[1.], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[1.], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[1.], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
@cache('run')
def losses_solar_for_heat():
    """
    Losses solar for heat



    constant


    """

    ret =  0.37
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpotential_fes_res_for_heatxcom_ejx():
    """
    Xpotential FES RES for heatXcom EJX



    component


    """

    ret =  xpotential_fes_res_for_heatxcom_twhx() * ej_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpotential_fes_res_for_heatxcom_twhx():
    """
    Xpotential FES RES for heatXcom TWhX



    component


    """

    ret =  xinstalled_capacity_res_heatxcom_twx() * efficiency_res_heat() * cp_res_for_heat(
    ) / twe_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
@cache('run')
def cp_res_for_heat():
    """
    Cp RES for heat



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[1.], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.265], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.5], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
@cache('run')
def efficiency_geothermal_for_heat():
    """
    Efficiency geothermal for heat



    constant


    """

    ret =  0.6960561017716173
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
@cache('run')
def efficiency_solar_panels_for_heat():
    """
    Efficiency solar panels for heat



    constant


    """

    ret =  0.095
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
# HEAT RES-com - Capacities and generation all tech
@cache('step')
def max_fe_potential_res_for_heat():
    """
    Max FE potential RES for heat



    component


    """

    ret =  xr.DataArray(
        data=[max_pe_potential_res_for_heat().loc[{'res heat':'xsolarxheatx'}].values + efficiency_res_heat().loc[{'res heat':'xsolarxheatx'}].values * 0,

        max_pe_potential_res_for_heat().loc[{'res heat':'xgeotxheatx'}].values * efficiency_res_heat().loc[{'res heat':'xgeotxheatx'}].values,

        max_pe_potential_res_for_heat().loc[{'res heat':'xsolid bioexheatx'}].values * efficiency_res_heat().loc[{'res heat':'xsolid bioexheatx'}].values],
        coords={'res heat':['xsolarxheatx','xgeotxheatx','xsolid bioexheatx']},
        dims=['res heat'])
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
@cache('run')
def p_geothermal_for_heat():
    """
    P geothermal for heat



    constant


    """

    ret =  0.077
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xpast_res_growth_for_heatxcomx():
    """
    Xpast RES growth for heatXcomX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.2771226], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.03571556], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(
            data=[0.06278667], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xinstalled_capacity_res_heatxcom_twx():
    """
    Xinstalled capacity RES heatXcom TWX



    component


    """


    ret =  integ_xinstalled_capacity_res_heatxcom_twx()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xinitial_value_res_for_heatxcomx():
    """
    Xinitial value RES for heatXcomX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[2.03033268e-06], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[0.00199927], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(
            data=[0.01167364], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
@cache('run')
def p_solar_for_heat():
    """
    P solar for heat



    constant


    """

    ret =  0.1
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
@cache('run')
def life_time_res_for_heat():
    """
    life time RES for heat



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[25.], coords={'res heat': ['xsolarxheatx']}, dims=['res heat']),
        xr.DataArray(data=[25.], coords={'res heat': ['xgeotxheatx']}, dims=['res heat']),
        xr.DataArray(data=[30.], coords={'res heat': ['xsolid bioexheatx']}, dims=['res heat']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xwear_res_capacity_for_heatxcom_twx():
    """
    Xwear RES capacity for heatXcom TWX



    component


    """

    ret =  xinstalled_capacity_res_heatxcom_twx() / life_time_res_for_heat()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# CHP plants - FES commercial Heat and Elec
@cache('step')
def potential_fe_gen_elec_fossil_fuel_chp_plants_ej():
    """
    Potential FE gen Elec fossil fuel CHP plants EJ



    component


    """

    ret =  (potential_fe_gen_elec_coal_chp_plants_ej() + potential_fe_gen_elec_gas_chp_plants_ej()
            + potential_fe_gen_elec_liquids_chp_plants_ej())
    name = 1
    return round_sig(x=ret, name=name)
# CHP plants - FES commercial Heat and Elec

def efficiency_heat_oil_chp_plants(x):
    """
    efficiency Heat oil CHP plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.3583416873580516, 0.3864604322008381, 0.3427001131428596, 0.3426092425053806,
        0.2559985362733478, 0.27189087245527377, 0.30978899425880957, 0.2622427885462967,
        0.24249576844589016, 0.2814351729755998, 0.29350979736335797, 0.3169617343470768,
        0.3231319029553005, 0.32732942509102475, 0.3520069392931584, 0.31166079425216436,
        0.2791179408861372, 0.26613174359788516, 0.27197080023843734, 0.2580529463443723
    ])
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def fed_heat_liquids_chp_plants_ej():
    """
    FED heat liquids CHP plants EJ



    component


    """

    ret =  fed_heat_fossil_fuels_chp_plants_ej() * share_chp_plants_oil()
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def potential_fe_gen_elec_liquids_chp_plants_ej():
    """
    Potential FE gen Elec liquids CHP plants EJ



    component


    """

    ret =  ped_oil_for_chp_plants_ej() * efficiency_elec_oil_chp_plants(time())
    name = 1
    return round_sig(x=ret, name=name)
# CHP plants - FES commercial Heat and Elec

def efficiency_elec_coal_chp_plants(x):
    """
    efficiency Elec coal CHP plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.2720349999945255, 0.26306888518464144, 0.27545884097647594, 0.2789107682797072,
        0.27789150432842147, 0.27881150074145433, 0.27529653704292867, 0.2751896064517859,
        0.28277916069267756, 0.2832216768494038, 0.2842686028283118, 0.28068849559240433,
        0.28915271943220666, 0.30241506318388284, 0.30167536009026713, 0.2979348686134844,
        0.2926626262217007, 0.29278659243706345, 0.29829300985708207, 0.30761772840986873
    ])
    name = 1
    return round_sig(x=ret, name=name)
# CHP plants - FES commercial Heat and Elec

def efficiency_elec_gas_chp_plants(x):
    """
    efficiency Elec gas CHP plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.26782295944824347, 0.2666572241731881, 0.26830320759173404, 0.2738131863521139,
        0.2780196471160715, 0.3055205882931438, 0.30845101322725493, 0.3086933636483027,
        0.316047593938253, 0.31760571304542445, 0.31705352904069145, 0.3240026232130847,
        0.3279642160865447, 0.32862834898119897, 0.3336097590206972, 0.3351515677147325,
        0.3243356439411598, 0.32476042746440104, 0.31797643498814804, 0.33154405356432026
    ])
    name = 1
    return round_sig(x=ret, name=name)
# CHP plants - FES commercial Heat and Elec

def efficiency_elec_oil_chp_plants(x):
    """
    efficiency Elec oil CHP plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.3605926738806412, 0.3348333610288783, 0.3466476498304344, 0.3531299339587378,
        0.29737398934546155, 0.31012580512156696, 0.32513162171457316, 0.3418424504153948,
        0.3283762635470366, 0.31172847731284714, 0.30438653224080414, 0.29767818828797493,
        0.29991121932367, 0.2993764831229446, 0.300032598019261, 0.29242527982890654,
        0.2893579004394849, 0.29758405532679544, 0.3107288107229254, 0.32993276951104356
    ])
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def potential_fe_gen_elec_coal_chp_plants_ej():
    """
    Potential FE gen Elec coal CHP plants EJ



    component


    """

    ret =  ped_coal_for_chp_plants_ej() * efficiency_elec_coal_chp_plants(time())
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def potential_fe_gen_elec_gas_chp_plants_ej():
    """
    Potential FE gen Elec gas CHP plants EJ



    component


    """

    ret =  ped_gas_for_chp_plants_ej() * efficiency_elec_gas_chp_plants(time())
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def fed_heat_gas_chp_plants_ej():
    """
    FED heat gas CHP plants EJ



    component


    """

    ret =  fed_heat_fossil_fuels_chp_plants_ej() * historic_share_chp_plants_gas(time())
    name = 1
    return round_sig(x=ret, name=name)
# CHP plants - FES commercial Heat and Elec

def efficiency_heat_gas_chp_plants(x):
    """
    efficiency Heat gas CHP plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.34818197812029744, 0.30658370588876055, 0.29071150494278963, 0.27992388078634095,
        0.2689321316194951, 0.2916261332435117, 0.29242955417479993, 0.289381470807263,
        0.2814273774503354, 0.27495237777396736, 0.26980106264673176, 0.2871855759202209,
        0.27965735655873114, 0.2711157820652094, 0.26852977690961327, 0.26853864364145363,
        0.26936552781846224, 0.26897430848954873, 0.27004646141898336, 0.2843726348639607
    ])
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
# LIQUIDS - PED, PES & FES
# COMP- Historical data II
@cache('step')
def ped_oil_for_chp_plants_ej():
    """
    PED oil for CHP plants EJ



    component


    """

    ret =  fed_heat_liquids_chp_plants_ej() / efficiency_heat_oil_chp_plants(time())
    name = 1
    return round_sig(x=ret, name=name)

def xshare_heatxcom_chp_plants_nre_vs_nre_tot_heatxcom_generationx(x):
    """
    XShare heatXcom CHP plants NRE vs NRE tot heatXcom generationX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.47494035235993737, 0.48784484337985057, 0.48319453732010703, 0.4689138621366646,
        0.45504898204107314, 0.4787622080599851, 0.4767869618571136, 0.47874893888354547,
        0.47529703503778065, 0.477544401524363, 0.4599838057623242, 0.4659217428645132,
        0.4655488531999236, 0.46823780955770167, 0.4600254701073876, 0.45474576456496724,
        0.44887567948184176, 0.4410346921325672, 0.44994734894394395, 0.4547662603919192
    ])
    name = 1
    return round_sig(x=ret, name=name)
# CHP plants - FES commercial Heat and Elec

def efficiency_heat_coal_chp_plants(x):
    """
    efficiency Heat coal CHP plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.31110748223005136, 0.27146068584931377, 0.2800501070449751, 0.2908126586324929,
        0.2831297492346078, 0.30852271504871565, 0.30629958279337455, 0.2992513065181298,
        0.2780742636093701, 0.2829681290854686, 0.24566650306280705, 0.2406045127607476,
        0.2438696178536268, 0.25122846960658485, 0.2689306835894877, 0.27212307003403136,
        0.26753189523710946, 0.2645720057578548, 0.24924363461171214, 0.25692298720979917
    ])
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
@cache('step')
def fed_heat_coal_chp_plants_ej():
    """
    FED heat coal CHP plants EJ



    component


    """

    ret =  fed_heat_fossil_fuels_chp_plants_ej() * share_chp_plants_coal()
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
# GASES - PED, PES & FES
# COMP- Historical data II
@cache('step')
def ped_gas_for_chp_plants_ej():
    """
    PED gas for CHP plants EJ



    component


    """

    ret =  fed_heat_gas_chp_plants_ej() / efficiency_heat_gas_chp_plants(time())
    name = 1
    return round_sig(x=ret, name=name)

# CHP plants - FES commercial Heat and Elec
# SOLIDS - PED, PES & FES
# COMP- Historical data II
@cache('step')
def ped_coal_for_chp_plants_ej():
    """
    PED coal for CHP plants EJ



    component


    """

    ret =  fed_heat_coal_chp_plants_ej() / efficiency_heat_coal_chp_plants(time())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def remaining_potential_csp():
    """
    remaining potential CSP



    component


    """

    ret =  functions.if_then_else(single_precision(max_potential_res_elec_twh().loc[{'res elec':'csp'}].values) > single_precision(real_generation_res_elec_twh().loc[{'res elec':'csp'}].values) + less_thr_func(max_potential_res_elec_twh().loc[{'res elec':'csp'}].values, real_generation_res_elec_twh().loc[{'res elec':'csp'}].values),
                                  (max_potential_res_elec_twh().loc[{'res elec':'csp'}].values - real_generation_res_elec_twh().loc[{'res elec':'csp'}].values) /
                                  max_potential_res_elec_twh().loc[{'res elec':'csp'}].values, 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
@cache('step')
def pe_csp_for_elec_generation_ej():
    """
    PE CSP for Elec generation EJ



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'csp'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def invest_res_for_elec():
    """
    Invest RES for Elec



    component


    """

    ret =  np.maximum(
        invest_biow_tdolar() + xinvest_geotxelec_tdolarx() + invest_hydro_tdolar() +
        invest_oceanic_tdolar() + invest_solar_tdolar() + invest_onshore_wind_tdolar() +
        invest_offshore_wind_tdolar() + invest_csp_tdolar(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def power_density_csp():
    """
    power density CSP



    component


    """

    ret =  xpower_density_res_elec_twexmhax().loc[{'res elec':'csp'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
# ELEC RES - Capacities and generation all techn
@cache('step')
def max_potential_res_elec_twe():
    """
    max potential RES elec TWe



    component


    """

    ret =  xr.DataArray(
        data=[max_hydro_twe(),

        xmax_geotxelec_twex(),

        max_bioe_twe(),

        max_oceanic_twe(),

        max_onshore_wind_twe(),

        max_offshore_wind_twe(),

        max_solar_pv_on_land_twe(),

        max_csp_twe()],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
# IMPACTS - RES Land-use
@cache('step')
def surface_csp_mha():
    """
    surface CSP Mha



    component


    """

    ret =  surface_res_elec().loc[{'res elec':'csp'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def fe_elec_generation_from_csp_twh():
    """
    FE Elec generation from CSP TWh



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'csp'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def invest_csp_tdolar():
    """
    invest CSP Tdolar



    component


    """

    ret =  invest_res_elec_tdolar().loc[{'res elec':'csp'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def cp_limit_nuclear():
    """
    Cp limit nuclear



    component


    """

    ret =  functions.if_then_else(single_precision(cp_nuclear()) > single_precision(min_cp_nuclear()) + less_thr_func(cp_nuclear(), min_cp_nuclear()), 1, 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def invest_nuclear_tdolar():
    """
    invest nuclear Tdolar



    component


    """

    ret =  np.maximum(0,
                      functions.if_then_else(single_precision(0) > single_precision(nuclear_capacity_under_construction()) + less_thr_func(0, nuclear_capacity_under_construction()), 0,
                          (nuclear_capacity_under_construction() + replacement_nuclear_capacity())
                          * invest_cost_nuclear(time()) / 1000))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
# ELEC - Demand of electricity from nuclear
@cache('step')
def cp_nuclear():
    """
    Cp nuclear



    component


    """

    ret =  cp_nuclear_initial() * cp_exogenous_res_elec_dispatch_reduction()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def installed_capacity_nuclear_tw():
    """
    installed capacity nuclear TW



    component


    """


    ret =  integ_installed_capacity_nuclear_tw()
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xnuclear_capacity_phasexoutx():
    """
    Xnuclear capacity phaseXoutX



    component


    """

    ret =  functions.if_then_else(abs(selection_of_nuclear_scenario() - 4) < equality_threshold,
                                  functions.if_then_else(
                                      time() < xstart_year_nuclear_growth_scen3x4x(), 0,
                                      xp_nuclear_scen3x4x() * installed_capacity_nuclear_tw()), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('run')
def min_cp_nuclear():
    """
    min Cp nuclear



    constant


    """

    ret =  0.6
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def replacement_nuclear_capacity():
    """
    replacement nuclear capacity



    component


    """

    ret =  functions.if_then_else(
        time() < 2013,
        nuclear_capacity_under_construction(),
        functions.if_then_else(
            abs(selection_of_nuclear_scenario() - 2) < equality_threshold, 0,
            functions.if_then_else(abs(selection_of_nuclear_scenario() - 4) < equality_threshold, 0,
                                   replacement_rate_nuclear() * wear_nuclear() *
                                   (1 - nuclear_overcapacity())))) * cp_limit_nuclear()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def min_cp_baseload_res():
    """
    min Cp baseload RES



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.15], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.15], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.15], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.15], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.0525], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.07875], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.0375], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.0625], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
# RESOURCES - Uranium extraction
@cache('step')
def abundance_uranium():
    """
    abundance uranium



    component


    """

    ret =  functions.if_then_else(
        abs(pe_demand_uranium_ej() ) < equality_threshold, 1,
        functions.if_then_else(single_precision(extraction_uranium_ej()) > single_precision(pe_demand_uranium_ej()) + less_thr_func(extraction_uranium_ej(), pe_demand_uranium_ej()), 1,
            1 - ((pe_demand_uranium_ej() - extraction_uranium_ej()) / pe_demand_uranium_ej())))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def nuclear_overcapacity():
    """
    nuclear overcapacity



    component


    """

    ret =  functions.if_then_else(
        abs(potential_generation_nuclear_elec_twh() ) < equality_threshold, 0,
        (potential_generation_nuclear_elec_twh() - fe_nuclear_elec_generation_twh()) /
        potential_generation_nuclear_elec_twh())
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xp_nuclear_scen3x4x():
    """
    XP nuclear scen3X4X



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xstart_year_nuclear_growth_scen3x4x():
    """
    Xstart year nuclear growth scen3X4X



    constant


    """

    ret =  2020
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def p_nuclear_elec_gen():
    """
    P nuclear elec gen



    component


    """

    ret =  functions.if_then_else(
        abs(selection_of_nuclear_scenario() - 1) < equality_threshold, 0,
        functions.if_then_else(abs(selection_of_nuclear_scenario() - 2) < equality_threshold, 0,
                               functions.if_then_else(
                                   abs(selection_of_nuclear_scenario() - 4) < equality_threshold, 0,
                                   functions.if_then_else(
                                       abs(selection_of_nuclear_scenario() - 3) < equality_threshold,
                                       functions.if_then_else(
                                           time() < xstart_year_nuclear_growth_scen3x4x(), 0,
                                           xp_nuclear_scen3x4x()), 0))))
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def required_capacity_nuclear_tw():
    """
    required capacity nuclear TW



    component


    """


    ret =  integ_required_capacity_nuclear_tw()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('run')
def time_planification_nuclear():
    """
    time planification nuclear



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('run')
def time_construction_nuclear():
    """
    time construction nuclear



    constant


    """

    ret =  8
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def initial_capacity_installed_nuclear():
    """
    initial capacity installed nuclear



    component


    """

    ret =  initial_gen_nuclear() * twe_per_twh() / cp_nuclear_initial()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def nuclear_capacity_under_construction():
    """
    Nuclear capacity under construction



    component


    """

    ret =  functions.if_then_else(
        time() < 2013,
        (historic_nuclear_generation_twh(time() + 1) - historic_nuclear_generation_twh(time())) *
        twe_per_twh() / cp_nuclear(), planned_nuclear_capacity_tw() / time_construction_nuclear())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def new_nuclear_capacity_under_planning():
    """
    new nuclear capacity under planning



    component


    """

    ret =  np.maximum(0, required_capacity_nuclear_tw() / time_planification_nuclear())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
# ELEC - Demand of electricity from nuclear
@cache('run')
def selection_of_nuclear_scenario():
    """
    selection of nuclear scenario



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('run')
def initial_gen_nuclear():
    """
    initial gen nuclear



    constant


    """

    ret =  2210
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('run')
def initial_capacity_in_construction_nuclear():
    """
    initial capacity in construction nuclear



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('run')
def initial_required_capacity_nuclear():
    """
    initial required capacity nuclear



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def planned_nuclear_capacity_tw():
    """
    Planned nuclear capacity TW



    component


    """


    ret =  integ_planned_nuclear_capacity_tw()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def wear_nuclear():
    """
    wear nuclear



    component


    """

    ret =  functions.if_then_else(time() < 2012, 0,
                                  installed_capacity_nuclear_tw() / life_time_nuclear())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('step')
def effects_shortage_uranium():
    """
    effects shortage uranium



    component


    """

    ret =  functions.if_then_else(abs(extraction_uranium_ej() ) < equality_threshold, 0,
                                  functions.if_then_else(single_precision(abundance_uranium()) > single_precision(0.8) + less_thr_func(abundance_uranium(), 0.8),
                                                         ((abundance_uranium() - 0.8) * 5)**2, 0))
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def tpes_intensity_ej_tdolar():
    """
    TPES intensity EJ Tdolar



    component


    """

    ret =  functions.zidz(tpes_ej(), gdp())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def real_ped_intensity_of_electricity():
    """
    real PED intensity of Electricity



    component


    """

    ret =  (total_fe_elec_demand_ej() + elec_gen_related_losses_ej()) / gdp()
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# ECON- Final demands by sector, households and fuel
# WASTE - PES & FES
@cache('step')
def annual_gdp_growth_rate():
    """
    Annual GDP growth rate



    component


    """

    ret =  -1 + gdp() / gdp_delayed_1yr()
    name = 1
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def historic_energy_intensity():
    """
    Historic energy intensity



    component


    """

    ret =  functions.if_then_else(time() < 2015, historic_tpes(time()) / gdp(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# DEM- Non-energy use
# ECON- Final demands by sector, households and fuel
# MATERIALS - Rest demand
@cache('step')
def gdp_delayed_1yr():
    """
    GDP delayed 1yr



    component


    """

    ret =  delay_gdp_round1__time_step___time_step_2916_1__time_step()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def share_tot_monet_invest_elec_res_vs_gdp():
    """
    share tot monet invest Elec RES vs GDP



    component


    """

    ret =  total_monet_invest_res_for_elec_tdolar() / gdp()
    name = 1
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def real_ped_intensity_of_transport():
    """
    real PED intensity of Transport



    component


    """

    ret =  pe_demand_transport_after_policies_ej() / gdp()
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def climate_feedback_param():
    """
    Climate Feedback Param



    component


    """

    ret =  co2_rad_force_coeff() / climate_sensitivity()
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def feedback_cooling():
    """
    Feedback Cooling



    component


    """

    ret =  atmos_uocean_temp() * climate_feedback_param()
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def gdppc():
    """
    GDPpc



    component


    """

    ret =  gdp() * dollars_to_tdollars() / population()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xunlimited_coalxx():
    """
    Xunlimited coalXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# E INTENSITIES - Aggregated indicators
# ECON- Final demands by sector, households and fuel
# TFES by final fuel
# MATERIALS - Energy consumption
# TFES by fuel
# TPES
# EROI - Required final energy invested RES
# EROI system
# INDICATORS - Energy
@cache('step')
def real_tfec():
    """
    Real TFEC



    component


    """

    ret =  functions.sum(real_fe_consumption_by_fuel())
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xunlimited_uraniumxx():
    """
    Xunlimited uraniumXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def urr_coal_unlimited():
    """
    URR coal unlimited



    component


    """

    ret =  urr_coal_high15() * 10000
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('step')
def urr_uranium_unlimited():
    """
    URR uranium unlimited



    component


    """

    ret =  urr_uranium_ewg13() * 10000
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def urr_conv_gas_unlimited():
    """
    URR conv gas unlimited



    component


    """

    ret =  urr_conv_gas_high_mohr15() * 10000
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xunlimited_gasxx():
    """
    Xunlimited gasXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xunlimited_oilxx():
    """
    Xunlimited oilXX



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS CC - Energy losses function
@cache('step')
def elf_concentrations_logistic():
    """
    ELF concentrations logistic



    component


    """

    ret =  1 - 1 / (1 + np.exp((co2_ppm_concentrations() - a_logistic()) / b_logistic()))
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def urr_conv_oil_unlimited():
    """
    URR conv oil unlimited



    component


    """

    ret =  urr_conv_oil_maggio12_high() * 10000
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS CC - Energy losses function
@cache('run')
def b_logistic():
    """
    b logistic



    constant


    """

    ret =  50
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS CC - Energy losses function
@cache('run')
def a_logistic():
    """
    a logistic



    constant


    """

    ret =  600
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS CC - Energy losses function
@cache('run')
def selection_shape_elf():
    """
    selection shape ELF



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xactivate_elf_by_scenxx():
    """
    Xactivate ELF by scenXX



    constant


    """

    ret =  1
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# ELEC - Total FE electricity generation
# ELEC - Total FE electricity generation
# HEAT - Total FE heat generation
# IMPACTS CC - Energy losses function
@cache('run')
def method_cc_impacts():
    """
    Method CC impacts



    constant


    """

    ret =  0
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_demand_liquids_mbxdx():
    """
    XTotal demand liquids mbXdX



    component


    """

    ret =  ped_liquids_ej() * xmbxd_per_ejxyearx()
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def ped_coal_without_ctl():
    """
    PED coal without CTL



    component


    """

    ret =  ped_coal_ej() - ped_coal_for_ctl_ej()
    name = 1
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
# ABUND - Abundances by sector
@cache('step')
def share_liquids_for_transp():
    """
    share liquids for Transp



    component


    """

    ret =  functions.if_then_else(single_precision(ped_liquids_ej()) > single_precision(0) + less_thr_func(ped_liquids_ej(), 0),
                                  real_pe_demand_of_liquids_for_transp_ej() / ped_liquids_ej(), 0)
    name = 1
    return round_sig(x=ret, name=name)

# ABUND - Abundances by sector
@cache('step')
def pes_for_transp_ej():
    """
    PES for Transp EJ



    component


    """

    ret =  pe_gas_extraction_for_transp_ej() + pe_liquids_supply_for_transp_ej(
    ) + current_trends_demand_elec_for_transport()
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('step')
def surface_hydro_mha():
    """
    surface hydro Mha



    component


    """

    ret =  surface_res_elec().loc[{'res elec':'hydro'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def remaining_potential_hydro():
    """
    remaining potential hydro



    component


    """

    ret =  functions.if_then_else(single_precision(max_potential_res_elec_twh().loc[{'res elec':'hydro'}].values) > single_precision(real_generation_res_elec_twh().loc[{'res elec':'hydro'}].values) + less_thr_func(max_potential_res_elec_twh().loc[{'res elec':'hydro'}].values, real_generation_res_elec_twh().loc[{'res elec':'hydro'}].values),
                                  (max_potential_res_elec_twh().loc[{'res elec':'hydro'}].values - real_generation_res_elec_twh().loc[{'res elec':'hydro'}].values) /
                                  max_potential_res_elec_twh().loc[{'res elec':'hydro'}].values, 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def grid_reinforcement_costs_tdollar():
    """
    Grid reinforcement costs Tdollar



    component


    """

    ret =  grid_reinforcement_costs() * new_capacity_installed_onshore_wind_tw() / g_per_t()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def remaining_potential_onshore_wind():
    """
    remaining potential onshore wind



    component


    """

    ret =  functions.if_then_else(single_precision(max_potential_res_elec_twh().loc[{'res elec':'wind onshore'}].values) > single_precision(real_generation_res_elec_twh().loc[{'res elec':'wind onshore'}].values) + less_thr_func(max_potential_res_elec_twh().loc[{'res elec':'wind onshore'}].values, real_generation_res_elec_twh().loc[{'res elec':'wind onshore'}].values),
                                  (max_potential_res_elec_twh().loc[{'res elec':'wind onshore'}].values - real_generation_res_elec_twh().loc[{'res elec':'wind onshore'}].values) /
                                  max_potential_res_elec_twh().loc[{'res elec':'wind onshore'}].values, 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def remaining_potential_offshore_wind():
    """
    remaining potential offshore wind



    component


    """

    ret =  functions.if_then_else(max_potential_res_elec_twh().loc[{'res elec':'wind offshore'}].values > real_generation_res_elec_twh().loc[{'res elec':'wind offshore'}].values,
                                  (max_potential_res_elec_twh().loc[{'res elec':'wind offshore'}].values - real_generation_res_elec_twh().loc[{'res elec':'wind offshore'}].values) /
                                  max_potential_res_elec_twh().loc[{'res elec':'wind offshore'}].values, 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def remaining_potential_bioe():
    """
    remaining potential BioE



    component


    """

    ret =  functions.if_then_else(single_precision(max_potential_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values) > single_precision(real_generation_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values) + less_thr_func(max_potential_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values, real_generation_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values),
                                  (max_potential_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values - real_generation_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values) /
                                  max_potential_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values, 0)
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def remaining_potential_oceanic():
    """
    remaining potential oceanic



    component


    """

    ret =  functions.if_then_else(single_precision(max_potential_res_elec_twh().loc[{'res elec':'oceanic'}].values) > single_precision(real_generation_res_elec_twh().loc[{'res elec':'oceanic'}].values) + less_thr_func(max_potential_res_elec_twh().loc[{'res elec':'oceanic'}].values, real_generation_res_elec_twh().loc[{'res elec':'oceanic'}].values),
                                  (max_potential_res_elec_twh().loc[{'res elec':'oceanic'}].values - real_generation_res_elec_twh().loc[{'res elec':'oceanic'}].values) /
                                  max_potential_res_elec_twh().loc[{'res elec':'oceanic'}].values, 0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xremaining_potential_geotxelecx():
    """
    Xremaining potential geotXelecX



    component


    """

    ret =  functions.if_then_else(single_precision(max_potential_res_elec_twh().loc[{'res elec':'xgeotxelecx'}].values) > single_precision(real_generation_res_elec_twh().loc[{'res elec':'xgeotxelecx'}].values) + less_thr_func(max_potential_res_elec_twh().loc[{'res elec':'xgeotxelecx'}].values, real_generation_res_elec_twh().loc[{'res elec':'xgeotxelecx'}].values),
                                  (max_potential_res_elec_twh().loc[{'res elec':'xgeotxelecx'}].values - real_generation_res_elec_twh().loc[{'res elec':'xgeotxelecx'}].values) /
                                  max_potential_res_elec_twh().loc[{'res elec':'xgeotxelecx'}].values, 0)
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xremaining_potential_solarxelec_pvx():
    """
    Xremaining potential solarXelec PVX



    component


    """

    ret =  functions.if_then_else(single_precision(max_potential_res_elec_twh().loc[{'res elec':'solar pv'}].values) > single_precision(real_generation_res_elec_twh().loc[{'res elec':'solar pv'}].values) + less_thr_func(max_potential_res_elec_twh().loc[{'res elec':'solar pv'}].values, real_generation_res_elec_twh().loc[{'res elec':'solar pv'}].values),
                                  (max_potential_res_elec_twh().loc[{'res elec':'solar pv'}].values - real_generation_res_elec_twh().loc[{'res elec':'solar pv'}].values) /
                                  max_potential_res_elec_twh().loc[{'res elec':'solar pv'}].values, 0)
    name = 1
    return round_sig(x=ret, name=name)

def xhistoric_nonxenergy_usex_subliquids(x):
    """
    XHistoric nonXenergy useX SUBliquids



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            17.86474451, 18.642216100000002, 19.669279510000003, 19.482172170000002, 20.12592009,
            20.309394920000003, 20.619444329999997, 21.34137057, 22.0610557, 23.32966305,
            23.355337600000002, 24.137153670000004, 24.616445300000002, 23.63162834,
            23.839961539999997
        ])
    name = 1
    return round_sig(x=ret, name=name)

def xhistoric_nonxenergy_usex_subsolids(x):
    """
    XHistoric nonXenergy useX SUBsolids



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.86732782, 0.9033509599999999, 0.81759914, 0.74229678, 0.68570774, 1.03101072,
            1.01775427, 1.08015287, 1.18593028, 1.29226541, 1.4364099399999999, 1.6726974399999999,
            1.7306662800000001, 1.74080181, 1.78348197
        ])
    name = 1
    return round_sig(x=ret, name=name)

def xhistoric_nonxenergy_usex_subgases(x):
    """
    XHistoric nonXenergy useX SUBgases



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3.74572862, 3.75295893, 4.03143292, 4.05466539, 4.17667242, 4.47201851,
            4.3420209199999995, 4.3756594699999996, 4.45160149, 4.78613611, 5.14473235, 5.00779145,
            5.3063370899999995, 5.338537059999999, 5.4843512
        ])
    name = 1
    return round_sig(x=ret, name=name)

def xhistoric_nonxenergy_usex_subelectricity(x):
    """
    XHistoric nonXenergy useX SUBelectricity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.001995, 0.001996, 0.001997, 0.001998, 0.001999, 0.002, 0.002001, 0.002002, 0.002003,
            0.002004, 0.002005, 0.002006, 0.002007, 0.002008, 0.002009
        ])
    name = 1
    return round_sig(x=ret, name=name)

def xhistoric_nonxenergy_usex_subheat(x):
    """
    XHistoric nonXenergy useX SUBheat



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.001995, 0.001996, 0.001997, 0.001998, 0.001999, 0.002, 0.002001, 0.002002, 0.002003,
            0.002004, 0.002005, 0.002006, 0.002007, 0.002008, 0.002009
        ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xinitial_nonxenergy_usex():
    """
    Xinitial nonXenergy useX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(
            data=[0.86732782], coords={'final sources': ['solids']}, dims=['final sources']),
        xr.DataArray(
            data=[17.86474451], coords={'final sources': ['liquids']}, dims=['final sources']),
        xr.DataArray(
            data=[3.74572862], coords={'final sources': ['gases']}, dims=['final sources']),
        xr.DataArray(
            data=[0.001995], coords={'final sources': ['electricity']}, dims=['final sources']),
        xr.DataArray(data=[0.001995], coords={'final sources': ['heat']}, dims=['final sources']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def real_total_output():
    """
    Real total output



    component


    """

    ret =  functions.sum(real_total_output_by_sector())
    name = 1
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# ECON- Final demands by sector, households and fuel
@cache('step')
def real_demand():
    """
    Real demand



    component


    """

    ret =  functions.sum(real_demand_by_sector())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def max_potential_res_elec_twh():
    """
    max potential RES elec TWh



    component


    """

    ret =  max_potential_res_elec_twe() / twe_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
@cache('step')
def pe_onshore_wind_for_elec_generation_ej():
    """
    PE onshore wind for Elec generation EJ



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'wind onshore'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def fe_elec_generation_from_solar_pv_twh():
    """
    FE Elec generation from solar PV TWh



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'solar pv'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('step')
def power_density_solar_pv():
    """
    power density solar PV



    component


    """

    ret =  xpower_density_res_elec_twexmhax().loc[{'res elec':'solar pv'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
@cache('step')
def pe_hydro_for_elec_generation_ej():
    """
    PE hydro for Elec generation EJ



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'hydro'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def invest_biow_tdolar():
    """
    invest bioW Tdolar



    component


    """

    ret =  invest_res_elec_tdolar().loc[{'res elec':'xsolid bioexelecx'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def fe_elec_generation_from_hydro_twh():
    """
    FE Elec generation from hydro TWh



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'hydro'}].values
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xinvest_geotxelec_tdolarx():
    """
    Xinvest geotXelec TdolarX



    component


    """

    ret =  invest_res_elec_tdolar().loc[{'res elec':'xgeotxelecx'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def invest_hydro_tdolar():
    """
    invest hydro Tdolar



    component


    """

    ret =  invest_res_elec_tdolar().loc[{'res elec':'hydro'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES: Overcapacities due to RES variability a
# ELEC RES - Total monetary investments
@cache('step')
def fe_elec_generation_from_onshore_wind_twh():
    """
    FE Elec generation from onshore wind TWh



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'wind onshore'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def invest_oceanic_tdolar():
    """
    invest oceanic Tdolar



    component


    """

    ret =  invest_res_elec_tdolar().loc[{'res elec':'oceanic'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def invest_onshore_wind_tdolar():
    """
    invest onshore wind Tdolar



    component


    """

    ret =  invest_res_elec_tdolar().loc[{'res elec':'wind onshore'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def invest_offshore_wind_tdolar():
    """
    invest offshore wind Tdolar



    component


    """

    ret =  invest_res_elec_tdolar().loc[{'res elec':'wind offshore'}].values
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xfe_elec_generation_from_geotxelec_twhx():
    """
    XFE Elec generation from geotXelec TWhX



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'xgeotxelecx'}].values
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpe_geotxelec_for_elec_generation_ejx():
    """
    XPE geotXelec for Elec generation EJX



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'xgeotxelecx'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
@cache('step')
def pe_solar_pv_for_elec_generation_ej():
    """
    PE solar PV for Elec generation EJ



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'solar pv'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def fe_elec_generation_from_bioe_twh():
    """
    FE Elec generation from bioE TWh



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'xsolid bioexelecx'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
@cache('step')
def pe_oceanic_for_elec_generation_ej():
    """
    PE oceanic for Elec generation EJ



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'oceanic'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# RES BIOE - Elec&Heat potential
# PES solids biofuels & waste
# IMPACTS - Total GHG emissions
@cache('step')
def pe_bioe_for_elec_generation_ej():
    """
    PE bioE for Elec generation EJ



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'xsolid bioexelecx'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES: Overcapacities due to RES variability a
# ELEC RES - Total monetary investments
@cache('step')
def fe_elec_generation_from_offshore_wind_twh():
    """
    FE Elec generation from offshore wind TWh



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'wind offshore'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def fe_elec_generation_from_oceanic_twh():
    """
    FE Elec generation from oceanic TWh



    component


    """

    ret =  real_generation_res_elec_twh().loc[{'res elec':'oceanic'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
@cache('step')
def pe_offshore_wind_for_elec_generation_ej():
    """
    PE offshore wind for Elec generation EJ



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'wind offshore'}].values
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def invest_solar_tdolar():
    """
    invest solar Tdolar



    component


    """

    ret =  invest_res_elec_tdolar().loc[{'res elec':'solar pv'}].values
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def initial_value_land_compet_biofuels_2gen_mha():
    """
    initial value land compet biofuels 2gen Mha



    component


    """

    ret =  initial_value_land_compet_biofuels_2gen_ktoe() * ej_per_ktoe()
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('run')
def initial_value_land_compet_biofuels_2gen_ktoe():
    """
    initial value land compet biofuels 2gen ktoe



    constant


    """

    ret =  9052.844108405054
    name = 1
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
# RESOURCES - CTL & GTL supply
@cache('step')
def abundance_coal():
    """
    abundance coal



    component


    """

    ret =  functions.if_then_else(single_precision(extraction_coal_ej()) > single_precision(ped_coal_ej()) + less_thr_func(extraction_coal_ej(), ped_coal_ej()), 1, 1 - functions.zidz(
        (ped_coal_ej() - extraction_coal_ej()), ped_coal_ej()))
    name = 1
    return round_sig(x=ret, name=name)

# FE INTENSITY households
# FE INTENSITIES by sector and fuel
# ECON- Final demands by sector, households and fuel
# LIQUIDS - PED, PES & FES
# RESOURCES - CTL & GTL supply
# RESOURCES - CTL & GTL supply
@cache('step')
def abundance_liquids():
    """
    abundance liquids



    component


    """

    ret =  functions.if_then_else(single_precision(pes_liquids_ej()) > single_precision(ped_liquids_ej()) + less_thr_func(pes_liquids_ej(), ped_liquids_ej()), 1,
        1 - functions.zidz(ped_liquids_ej() - pes_liquids_ej(), ped_liquids_ej()))
    name = 1
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Dedicated crops for biofuels (marg)
# RES BIOE - Crop and forest residues
# LIQUIDS - PED, PES & FES
# RESOURCES - Oil extraction
# RESOURCES - CTL & GTL supply
# RESOURCES - CTL & GTL supply
# RESOURCES - CTL & GTL supply
# RESOURCES - CTL & GTL supply
@cache('step')
def check_liquids():
    """
    check liquids



    component


    """

    ret =  functions.zidz((ped_liquids_ej() - pes_liquids_ej()), pes_liquids_ej())
    name = 1
    return round_sig(x=ret, name=name)
# DEM - Adjustment non-commercial heat demand
# HEAT - Heat demand
# HEAT COM- Heat-com demand covered by fossil fuels
# HEAT - Total FE heat generation

def efficiency_liquids_for_heat_plants(x):
    """
    efficiency liquids for heat plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.923745042449277, 0.8363533334993, 0.8318533392156451, 0.8343775097644487,
        0.8449037487785805, 0.8134700348415264, 0.8315091058175587, 0.8036477473316663,
        0.8199626156663135, 0.8201391471763312, 0.830453125951232, 0.8510030575113863,
        0.8534313700543216, 0.8527114298309938, 0.8308203086018318, 0.8312887451419698,
        0.831416361961128, 0.8371918262743113, 0.8177248627892834, 0.7866970642224647
    ])


    name = 1
    return round_sig(x=ret, name=name)

def xshare_gasxxcoalxgasx_for_heat_plantsx(x):
    """
    Xshare gasXXcoalXgasX for heat plantsX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.6400078403045726, 0.625494609565936, 0.633011936961359, 0.6486922923750881,
        0.6469276462210707, 0.6138097737209008, 0.606109214138782, 0.5986175242634422,
        0.595281391596198, 0.586586261482702, 0.5447112921280477, 0.52717673519315,
        0.5032201982790275, 0.49668145545839515, 0.48668197474167624, 0.4775030096269512,
        0.47070644665043565, 0.44995202983813604, 0.3903674184032734, 0.382003505923756
    ])


    name = 1
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
# HEAT - Heat demand
# HEAT - Total FE heat generation
@cache('run')
def share_heat_distribution_losses():
    """
    Share heat distribution losses



    constant


    """

    ret =  0.0615
    name = 1
    return round_sig(x=ret, name=name)
# DEM - Adjustment non-commercial heat demand
# HEAT - Heat demand
# HEAT COM- Heat-com demand covered by fossil fuels
# HEAT - Total FE heat generation

def efficiency_coal_for_heat_plants(x):
    """
    efficiency coal for heat plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.8347276195018529, 0.7939170408846971, 0.8075329657040736, 0.7985649421904794,
        0.8053657541377044, 0.733731426164544, 0.7535839368782363, 0.7799860116948372,
        0.69841143160776, 0.7051976957278331, 0.7227781675786922, 0.7195061580692983,
        0.7293959273179971, 0.7378841063284353, 0.7364423437396291, 0.7004937762675217,
        0.7112918920827271, 0.665985886065352, 0.7079178145750791, 0.7145485392606492
    ])
    name = 1
    return round_sig(x=ret, name=name)
# DEM - Adjustment non-commercial heat demand
# HEAT - Heat demand
# HEAT COM- Heat-com demand covered by fossil fuels
# HEAT - Total FE heat generation

def efficiency_gases_for_heat_plants(x):
    """
    efficiency gases for heat plants



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.9002705165704274, 0.798015478374507, 0.8265508524517888, 0.8513851809893292,
        0.8366292510284139, 0.8071606916492732, 0.8075260241858225, 0.8116671100413593,
        0.7803729904585283, 0.8291140201011076, 0.8331308600977195, 0.8581487641371625,
        0.8638647108245096, 0.8652630305943033, 0.8470379015860756, 0.8354220854903446,
        0.8235722289925171, 0.7992649323212949, 0.7349857411765203, 0.7302838157453234
    ])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def potential_tot_res_elec_after_intermitt():
    """
    potential tot RES elec after intermitt



    component


    """

    ret =  functions.sum(potential_res_elec_after_intermitt_twh())
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def invest_res_elec_tdolar():
    """
    invest RES elec Tdolar



    component


    """

    ret =  xr.DataArray(
        data=[np.maximum(
            0, res_elec_capacity_under_construction_tw().loc[{'res elec':'hydro'}].values * invest_cost_res_elec_subhydro(time())),

        np.maximum(0,
                   res_elec_capacity_under_construction_tw().loc[{'res elec':'xgeotxelecx'}].values * invest_cost_res_elec_subxgeotxelecx(
                       time())),

        np.maximum(0,
                   res_elec_capacity_under_construction_tw().loc[{'res elec':'xsolid bioexelecx'}].values *
                   invest_cost_res_elec_subxsolidbioexelecx(time())),

        np.maximum(
            0,
            res_elec_capacity_under_construction_tw().loc[{'res elec':'oceanic'}].values * invest_cost_res_elec_suboceanic(time())),

        np.maximum(0,
                   res_elec_capacity_under_construction_tw().loc[{'res elec':'wind onshore'}].values * invest_cost_res_elec_subwindonshore(
                       time())),

        np.maximum(0,
                   res_elec_capacity_under_construction_tw().loc[{'res elec':'wind offshore'}].values *
                   invest_cost_res_elec_subwindoffshore(time())),

        np.maximum(
            0,
            res_elec_capacity_under_construction_tw().loc[{'res elec':'solar pv'}].values * invest_cost_res_elec_subsolarpv(time())),

        np.maximum(
            0, res_elec_capacity_under_construction_tw().loc[{'res elec':'csp'}].values * invest_cost_res_elec_subcsp(time()))],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
@cache('step')
def pe_biow_for_elec_generation_mtoe():
    """
    PE BioW for Elec generation Mtoe



    component


    """

    ret =  pe_real_generation_res_elec().loc[{'res elec':'xsolid bioexelecx'}].values * mtoe_per_ej()
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def remaining_potential_tot_res_elec_after_intermitt():
    """
    remaining potential tot RES elec after intermitt



    component


    """

    ret =  functions.if_then_else(
        potential_tot_res_elec_after_intermitt() > fe_real_tot_generation_res_elec_twh(),
        (potential_tot_res_elec_after_intermitt() - fe_real_tot_generation_res_elec_twh()) /
        potential_tot_res_elec_after_intermitt(), 0)
    name = 1
    return round_sig(x=ret, name=name)

def invest_cost_res_elec_subhydro(x):
    """
    invest cost RES elec SUBhydro



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            2.214812479617349, 2.214812479617349, 2.344999239047722, 2.4170522882922056,
            2.5259506468094357, 2.6168357430155447, 2.697076638765082, 2.697076638765082,
            2.697076638765082, 2.697076638765082, 2.697076638765082, 2.697076638765082
        ])
    name = 1
    return round_sig(x=ret, name=name)

def invest_cost_res_elec_subxgeotxelecx(x):
    """
    invest cost RES elec SUBXgeotXelecX



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            20.381187520382646, 10.190593760191323, 8.9042830742472, 7.519718230242417,
            5.9361887161648, 4.947096858354169, 4.254405044026525, 4.254405044026525,
            4.254405044026525, 4.254405044026525, 4.254405044026525, 4.254405044026525
        ])
    name = 1
    return round_sig(x=ret, name=name)

def invest_cost_res_elec_subxsolidbioexelecx(x):
    """
    invest cost RES elec SUBXsolidbioEXelecX



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            4.614670290248939, 2.3073351451244695, 2.0076599630394605, 1.9937406239808673,
            1.9462511142515488, 1.923325144037395, 1.904493097075769, 1.904493097075769,
            1.904493097075769, 1.904493097075769, 1.904493097075769, 1.904493097075769
        ])
    name = 1
    return round_sig(x=ret, name=name)

def invest_cost_res_elec_suboceanic(x):
    """
    invest cost RES elec SUBoceanic



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            11.816700076095225, 5.908350038047613, 3.1867098597673653, 2.297509729318404,
            1.7669372757908466, 1.4754499402108923, 1.3141493640613107, 1.3141493640613107,
            1.3141493640613107, 1.3141493640613107, 1.3141493640613107, 1.3141493640613107
        ])
    name = 1
    return round_sig(x=ret, name=name)

def invest_cost_res_elec_subwindonshore(x):
    """
    invest cost RES elec SUBwindonshore



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            2.4727296445265785, 1.2363648222632893, 1.0275747363843895, 0.8171470812044787,
            0.7794829872812261, 0.7418188933579735, 0.7319934775519076, 0.7319934775519076,
            0.7319934775519076, 0.7319934775519076, 0.7319934775519076, 0.7319934775519076
        ])
    name = 1
    return round_sig(x=ret, name=name)

def invest_cost_res_elec_subwindoffshore(x):
    """
    invest cost RES elec SUBwindoffshore



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            4.74895097293184, 2.37447548646592, 1.801326231112077, 1.2609283617784537,
            1.1954255897380146, 1.088983585172301, 1.0685139689096637, 1.0685139689096637,
            1.0685139689096637, 1.0685139689096637, 1.0685139689096637, 1.0685139689096637
        ])
    name = 1
    return round_sig(x=ret, name=name)

def invest_cost_res_elec_subsolarpv(x):
    """
    invest cost RES elec SUBsolarPV



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            6.134334601587128, 3.067167300793564, 2.137027937819328, 1.4541615392977496,
            0.8408918360691379, 0.6427459506468093, 0.6230951190346776, 0.6230951190346776,
            0.6230951190346776, 0.6230951190346776, 0.6230951190346776, 0.6230951190346776
        ])
    name = 1
    return round_sig(x=ret, name=name)

def invest_cost_res_elec_subcsp(x):
    """
    invest cost RES elec SUBCSP



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            11.872377432329598, 5.936188716164799, 4.56554321121861, 4.129949777149689,
            3.4904789651049017, 3.438895532123056, 3.406144146102837, 3.406144146102837,
            3.406144146102837, 3.406144146102837, 3.406144146102837, 3.406144146102837
        ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xpower_density_res_elec_twexmhax():
    """
    Xpower density RES elec TWeXMhaX



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.04], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.5], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.025], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.01], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.033], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.033], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xpower_density_res_elec_twxmhax():
    """
    Xpower density RES elec TWXMhaX



    component


    """

    ret =  xr.DataArray(
        data=[xpower_density_res_elec_twexmhax().loc[{'res elec':'hydro'}].values / xcpxini_res_elecx_subhydro(time()),

        xpower_density_res_elec_twexmhax().loc[{'res elec':'xgeotxelecx'}].values / xcpxini_res_elecx_subxgeotxelecx(time()),

        xpower_density_res_elec_twexmhax().loc[{'res elec':'xsolid bioexelecx'}].values / xcpxini_res_elecx_subxsolidbioexelecx(time()),

        xpower_density_res_elec_twexmhax().loc[{'res elec':'oceanic'}].values / xcpxini_res_elecx_suboceanic(time()),

        xpower_density_res_elec_twexmhax().loc[{'res elec':'wind onshore'}].values / xcpxini_res_elecx_subwindonshore(time()),

        xpower_density_res_elec_twexmhax().loc[{'res elec':'wind offshore'}].values / xcpxini_res_elecx_subwindoffshore(time()),

        xpower_density_res_elec_twexmhax().loc[{'res elec':'solar pv'}].values / xcpxini_res_elecx_subsolarpv(time()),

        xpower_density_res_elec_twexmhax().loc[{'res elec':'csp'}].values / xcpxini_res_elecx_subcsp(time())],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def time_95pc_ts_potential_res_elec():
    """
    Time 95pc TS potential RES elec



    component


    """

    ret =  functions.if_then_else(remaining_potential_res_elec_after_intermitt() > 0.05, 0, time())
    name = 1
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('step')
def surface_res_elec():
    """
    surface RES elec



    component


    """

    ret =  functions.if_then_else(
        abs(xpower_density_res_elec_twxmhax() ) < equality_threshold, 0,
        installed_capacity_res_elec_tw() / xpower_density_res_elec_twxmhax())
    name = 1
    return round_sig(x=ret, name=name)

@cache('step')
def xmax_geotxelec_twex():
    """
    Xmax geotXelec TWeX



    component


    """

    ret =  xmax_pe_geotxelec_twthx() * efficiency_conversion_geot_pe_to_elec()
    name = 1
    return round_sig(x=ret, name=name)

@cache('run')
def xmax_pe_geotxelec_twthx():
    """
    Xmax PE geotXelec TWthX



    constant


    """

    ret =  0.3
    name = 1
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('run')
def efficiency_conversion_geot_pe_to_elec():
    """
    Efficiency conversion geot PE to Elec



    constant


    """

    ret =  0.3333333333333333
    name = 1
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
@cache('step')
def geot_pe_potential_for_heat_ej():
    """
    Geot PE potential for heat EJ



    component


    """

    ret =  geot_pe_potential_for_heat_twth() * ej_per_twh() / twe_per_twh()
    name = 1
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculturehuntingforestryandfishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.7241971461530861, 0.708925072756311, 0.6998144878200208, 0.6785107560564287,
            0.6552600882169791, 0.6315664469531413, 0.6474383043815678, 0.6462555924915394,
            0.6362194771534165, 0.6269526899631803, 0.6287197169177513, 0.649448507837442,
            0.6773391674440454, 0.6565191729265971, 0.6750516890263
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subminingandquarrying(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.5634910007468006, 1.5112702035759604, 1.5517158938879816, 1.575438640288508,
            1.4696896640657946, 1.3928937451492078, 1.5000345353002407, 1.5238317127411405,
            1.4923776372944797, 1.4713179539606795, 1.4377547101284724, 1.335297206200341,
            1.3719554839627304, 1.204262170936298, 1.3283417503967039
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfoodbeveragesandtobacco(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.3872589683473662, 0.38991463229558976, 0.39101143513263503, 0.39505465054488725,
            0.3933419255112612, 0.3999444285581746, 0.4126345462420507, 0.43200313279180014,
            0.4299559197047555, 0.4285299876839093, 0.42113855610917716, 0.41406466418672366,
            0.4119767790339999, 0.41251967360302194, 0.41162621691153284
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextilesandtextileproducts(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8098595818123617, 0.7809054314995713, 0.8129769521621621, 0.8081881969977484,
            0.784174020547582, 0.7971238777782144, 0.7883099437214302, 0.7795792702076949,
            0.7823568161223905, 0.8096815563070188, 0.7777888964711069, 0.7702650014841644,
            0.7564683224583868, 0.7000523728228109, 0.7024078973512976
        ])
    name = 1
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleatherleatherandfootwear(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.35139111305783044, 0.33316073109134436, 0.33808935659610473, 0.3301911458791342,
            0.32742257172175443, 0.3383774266000901, 0.3320929088833885, 0.33770250309350436,
            0.3328079754567813, 0.3515074970928847, 0.32595095501665117, 0.3104903316024953,
            0.29027356586922193, 0.259192861625059, 0.25757048727115656
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwoodandproductsofwooodandcork(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8729045156287489, 0.8803194465366069, 0.8585216005427833, 0.8732729973240205,
            0.8661234191346798, 0.8644024652870175, 0.8579514779557791, 0.8304499897127946,
            0.816855955036405, 0.8115243628395454, 0.6828909225936747, 0.6526672333479513,
            0.6585095983831069, 0.6816773281639819, 0.6777837932313371
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulppaperprintingandpublishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.2263768378705506, 1.2144155296983012, 1.197934329105796, 1.184390669935025,
            1.1712966433885488, 1.169997471480713, 1.1966292361936643, 1.1999943104244262,
            1.2203385654272842, 1.2225810264380397, 1.1766825924595492, 1.1650724058722093,
            1.1133423207226474, 1.0869767100370342, 1.0676209273986548
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcokerefinedpetroleumandnuclearfuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.9165557410684781, 0.9358000116019126, 0.9243841164204407, 0.8953306635622691,
            0.8664440465758191, 0.9135066449439855, 0.9412770882258273, 0.9360752874602274,
            0.8827707875290411, 0.8565953567127532, 0.7948033487166073, 0.7413171643399493,
            0.7637328942332169, 0.6064451932608534, 0.7069123870027182
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicalsandchemicalproducts(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.803552955165638, 1.8308315315621162, 1.7496041241839269, 1.6452475708012788,
            1.6101830348922872, 1.666450341608775, 1.6611052910033846, 1.6204107698126002,
            1.644341474418466, 1.6483042729285202, 1.6666473381010591, 1.6106763308171923,
            1.558536120192618, 1.5213063144943828, 1.514599748285027
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubberandplastics(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.7516755152937544, 1.824574493937183, 1.7920601389627309, 1.8067289091982912,
            1.8795825516807279, 1.8435548709997704, 1.7842275613004739, 1.845080435538212,
            1.6730108764548752, 1.6424496581101529, 1.7862606136200592, 1.706408159426469,
            1.6649464780881742, 1.6942052186029162, 1.7361101430096244
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothernonmetalicmineral(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.6010029593093376, 1.5932256091723684, 1.5699087462270658, 1.5672520333522502,
            1.5615298108178137, 1.576358038509453, 1.6431319189067426, 1.6958319558465424,
            1.7255318447745254, 1.7461744142710622, 1.7654612366537779, 1.791817576209471,
            1.770131164494374, 1.784319158537765, 1.9519753682288368
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasicmetalsandfabricatedmetal(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.9776155903879076, 1.9392195301196662, 1.9215450030205303, 1.9122783844372382,
            1.929118205864779, 1.9815693703488377, 2.009944479295412, 2.075131535173117,
            2.1513151629091998, 2.218317981076714, 2.199945744072929, 2.18378513331959,
            2.2290229109356408, 2.230707958542277, 2.19480244602311
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinerynec(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.38497878559139365, 0.3561548057898775, 0.3415291275971809, 0.3427851610972068,
            0.3437487022843571, 0.3419753064945743, 0.351691064073765, 0.3675386907038594,
            0.3554827336970448, 0.3381230832880707, 0.3132625954950186, 0.2968054254027654,
            0.28813036805786, 0.2886452636675849, 0.32409542919292395
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricalandopticalequipment(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.3881121835801443, 0.37575844467622455, 0.35774634718219034, 0.34943354468767224,
            0.34381343331070224, 0.3112718982059209, 0.30223301921643114, 0.2984296037773112,
            0.2794400085359985, 0.2601078257061289, 0.24895235432425467, 0.23047562719826115,
            0.2196316918765295, 0.2080015981423694, 0.21658242383320356
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransportequipment(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.29157175769801413, 0.2932733502579648, 0.2865009021457406, 0.28318178927285437,
            0.2708520350396004, 0.3056591045379025, 0.30542284485819216, 0.2939825969509754,
            0.2861779021729641, 0.2932579238288578, 0.2660253656864477, 0.2523719403305147,
            0.23683425921684204, 0.2378185951613391, 0.26199150808256366
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturingnecrecycling(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8569169761465573, 0.8694348704950727, 0.8818025593648565, 0.9096502751261356,
            0.8725065440725464, 0.8044003060524386, 0.7687321308791215, 0.8306392887445839,
            0.7500852931749243, 0.7215141505939332, 0.7922587022642156, 0.7822512856775996,
            0.7153254959608057, 0.814964939710505, 0.8170739106271032
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricitygasandwatersupply(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.6240585151440134, 2.5827547952978684, 2.6980829723166853, 2.6792657225987164,
            2.6552380668911906, 2.6264867297696766, 2.333247621523986, 2.5314112882795596,
            2.679668653346034, 2.6745633298685774, 2.6329365375141487, 2.7085624351709545,
            2.6564387303091666, 2.581028066820715, 2.8574728837212615
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.10596936611626993, 0.11752002197397564, 0.10637076833546745, 0.11940080040308412,
            0.1244365089556424, 0.11581265842963812, 0.09767988013188698, 0.10694419521117998,
            0.07564017908634847, 0.07964715523270356, 0.07981840107155418, 0.08144162509328723,
            0.08640696882604974, 0.0957116405787128, 0.10473826892065874
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.29023907436985996, 0.29546092453914086, 0.29982259435817693, 0.2985275103903131,
            0.29392740007325285, 0.27164651402310047, 0.29919715620305803, 0.2914474478372432,
            0.30414212771064614, 0.29680981845439275, 0.27965689364716184, 0.2851604486781958,
            0.2876977938199097, 0.3028584015902629, 0.3180725318361754
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.19819158488103156, 0.1988984695251443, 0.1902452549651085, 0.19114253279138624,
            0.20230726292600698, 0.20364517363751936, 0.20459780243338735, 0.21415082222077816,
            0.2131461017021144, 0.2073036112635289, 0.20399321708074358, 0.20639263406148797,
            0.20941044634889908, 0.22317011916600307, 0.19985183045602822
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.49852024433364905, 0.49076619915037845, 0.4876463096311946, 0.48853974638903147,
            0.5091177800892925, 0.4873674900783807, 0.48477778575155234, 0.49413782382381477,
            0.5021773483646942, 0.5071092493982671, 0.47008904095609366, 0.46117943887494817,
            0.48198387270778725, 0.5049321158097608, 0.47628496738666654
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotelsandrestaurants(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.6316259743579479, 0.6228922759356975, 0.6409051875756983, 0.6349520690961216,
            0.6313261976802199, 0.651029329488104, 0.6736644901263359, 0.7027213918199248,
            0.7192748013416302, 0.6926817749424463, 0.6975865146363074, 0.7203197000945538,
            0.7392532435237646, 0.7956355255567737, 0.8162340685124673
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinlandtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.4483452062377024, 0.45286590216598044, 0.4451464125518518, 0.4250024788940665,
            0.41169122950481013, 0.4262861053691009, 0.4192292284879077, 0.4322862683192847,
            0.44932075861220566, 0.44433341086508094, 0.42814418035587165, 0.4182680008927667,
            0.417896236209432, 0.40666224043170296, 0.426985238182552
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwatertransport(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
            1.796435777226835e-05
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subairtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.3351250982416472, 0.3172017860190052, 0.31166535903931797, 0.3122641457070816,
            0.31651534903017065, 0.31346968682389725, 0.3367378703234646, 0.3585816344470207,
            0.362608758443261, 0.3258887564484779, 0.32369825391559337, 0.3429122781162016,
            0.3442326706594128, 0.36049764467512, 0.385387817272461
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpostandtelecommunications(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.2142453248427915, 0.21499985706883995, 0.22267555471748215, 0.21380570808085655,
            0.21358389938913094, 0.22572066945040173, 0.23543921322671155, 0.25414285942284653,
            0.27141218788839566, 0.26605874009786207, 0.2603283205745559, 0.26302026795246414,
            0.25757215250376087, 0.2678675035433738, 0.260025663572746
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancialintermedation(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.1357129444444695, 0.1335025986264839, 0.13347355787905524, 0.13530342504336876,
            0.1332398516261878, 0.13077685809779627, 0.11743851754411798, 0.1160974931940181,
            0.12211180403401836, 0.12151574900706145, 0.1160599929351302, 0.10825026748583594,
            0.1273302306890166, 0.13839558030319435, 0.1314090560744755
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrealestateactivities(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.31331706620363864, 0.3265824867186806, 0.3271085731423654, 0.3272119796580536,
            0.3049079343857554, 0.3466851022546009, 0.34590615723305, 0.3448615879622379,
            0.34449325697322003, 0.4120247607578594, 0.4612870638020768, 0.4424942059188907,
            0.3033982102598411, 0.23581865395159607, 0.27833046224886804
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrentingodmeqandotherbusinessactivities(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.18155751478579876, 0.18895369439380297, 0.1917845876674018, 0.19816334008250425,
            0.19857039813316948, 0.1913312708049842, 0.18326721091232706, 0.17244465134086473,
            0.17894389359752372, 0.17355814915740955, 0.17303117423193054, 0.1690768138532979,
            0.18005979606460235, 0.18671243674251325, 0.19378581229435143
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublicadminanddefencecompulsorysocialsecurity(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.40839951722917894, 0.41521087071191465, 0.428197889891699, 0.4285825726564223,
            0.4201005975115219, 0.41268027268550234, 0.41211831677424504, 0.4760328358825686,
            0.4927290785345975, 0.4784253547244492, 0.45089277838492575, 0.46473727616096494,
            0.5449616826567257, 0.5187732225130455, 0.5261113823348827
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.47589911961923304, 0.48810317329631847, 0.5010530845658214, 0.5051103590478777,
            0.4922273153673258, 0.5185140868132246, 0.5384892892118762, 0.5537962899548067,
            0.5845769195084838, 0.58478878910022, 0.5987198528777099, 0.6291527202343252,
            0.6913116423440042, 0.7187108804786225, 0.7413783491439477
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealthandsocialwork(x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.3360550520069014, 0.3319971823881577, 0.3409749985532885, 0.3457400169586935,
            0.3412237146366023, 0.3511642658000085, 0.36708612847876365, 0.36523506999109473,
            0.35815461591948927, 0.3366687390387799, 0.33902602916666025, 0.34971618344751637,
            0.35870597911304736, 0.37563059560620543, 0.3927018157309736
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothercommunitysocialandpersonaservices(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.3878879751275172, 0.3708698504162547, 0.38061467555067985, 0.39955675996821305,
            0.37762190742153284, 0.39091172102920724, 0.41155004608034385, 0.40927380105050193,
            0.4264253858524306, 0.40805678697158126, 0.4090799499066392, 0.41687604647378884,
            0.4272759179642891, 0.4509644858352208, 0.4522371698072469
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivatehouseholdswithemployedpersons(
        x):
    """
    Historic final energy intensity by sector and fuel SUBelectricity SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subagriculturehuntingforestryandfishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.1779432025330486, 0.13870566072139295, 0.12082321976272868, 0.10766561537513031,
            0.09597024560745618, 0.09182242973574563, 0.08539721418532935, 0.0787292046287861,
            0.07030452612026813, 0.06269616690801379, 0.06307201187077975, 0.06065281467706288,
            0.058431192052661673, 0.05410842896927197, 0.0510942878679188
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subminingandquarrying(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.39615757379217825, 0.32878335690737376, 0.3362121913754084, 0.34030321699873817,
            0.3253708905799867, 0.30352427099269813, 0.3190183924402979, 0.31766176005583435,
            0.29892007500937756, 0.2821306927832654, 0.31331686569242123, 0.3033988711082321,
            0.24857012656222557, 0.20262862281637783, 0.2166198242522173
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subfoodbeveragesandtobacco(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.1649647280255494, 0.13923080611190244, 0.1263507918573515, 0.1277449510541924,
            0.12716106154203174, 0.11618948111680057, 0.1142084176253826, 0.1168006772309306,
            0.12312181692947147, 0.12095713287918461, 0.11552633351364591, 0.12292787322827238,
            0.11748827453727254, 0.11982507444312444, 0.12010630655390092
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextilesandtextileproducts(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.22086369963658975, 0.17406359415856623, 0.16224183065657455, 0.16693572325962322,
            0.14221222136468667, 0.16381674657242878, 0.16831927283258077, 0.16035462473979567,
            0.1699009243369815, 0.19637915352632893, 0.20538678486804926, 0.2042770960192051,
            0.20725324806843504, 0.19506434139198559, 0.18757549668212872
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subleatherleatherandfootwear(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.11953127415721884, 0.09474854527802631, 0.08764839600586773, 0.09160695304828317,
            0.07675379152382777, 0.08074204323896955, 0.0765580679520774, 0.07607364176815419,
            0.08070889556854588, 0.08007746298837058, 0.08548371486588444, 0.08390383293604126,
            0.0784175115061452, 0.07111850342724534, 0.07220969773204038
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subwoodandproductsofwooodandcork(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8793952550681494, 0.6058544020694085, 0.5334020352587955, 0.5153793131116994,
            0.5871102874575236, 0.6203211143932192, 0.6313759985796766, 0.6439431104796545,
            0.6336153779824492, 0.6116323831845747, 0.16268003128165623, 0.1751541793275517,
            0.17069373931579726, 0.17493073288251104, 0.17875505194786806
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulppaperprintingandpublishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.10098146124858108, 0.11673687052005599, 0.1065235889017262, 0.10299216552866995,
            0.1043649214113851, 0.09959438674903104, 0.10092439867341914, 0.1148304535785617,
            0.16569911801129128, 0.18942512330308361, 0.2771436820991434, 0.30306283049683014,
            0.27831023548626566, 0.26327814535913896, 0.27018420717455527
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subcokerefinedpetroleumandnuclearfuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8306255389091878, 0.6867102923723846, 0.6398357709177271, 0.6697612735545528,
            0.670925349487634, 0.69573506267167, 0.6784280946749888, 0.6479712802547783,
            0.6406490162492339, 0.6332790793123346, 0.7910253303240071, 0.8147667435369287,
            0.8170829812904382, 0.6381195823715806, 0.7219641210498305
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicalsandchemicalproducts(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8584367275321372, 0.7397852128919687, 0.66724787357296, 0.6203922862106979,
            0.6560905475232538, 0.6819427269570311, 0.6776761148129726, 0.686752258681959,
            0.7212402945801206, 0.7175417810760435, 0.7353749926714853, 0.7432752979283427,
            0.708328418290724, 0.6675915872266838, 0.6697430685859749
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubberandplastics(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.45484858353019414, 0.4169535131947727, 0.34233489349153934, 0.2979234015680134,
            0.30308181830473974, 0.3345339758495406, 0.35123840932746825, 0.35016180267711977,
            0.3665131168273066, 0.3592264431339666, 0.35219533350061194, 0.29932860571513153,
            0.27516433128575574, 0.2608095184633447, 0.24392179773689737
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subothernonmetalicmineral(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.11474364843841015, 0.1015156829539818, 0.2177311622717772, 0.19260112324974013,
            0.18604035907517102, 0.1777772888849531, 0.1742377266923505, 0.16987213626105857,
            0.17492400397096394, 0.16052398993714187, 0.14392398218341645, 0.1410041738280214,
            0.12626025452818337, 0.12437860569413856, 0.1287548346990869
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasicmetalsandfabricatedmetal(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.44447170498746297, 0.3740641211103763, 0.3443018955107859, 0.3497366773559753,
            0.34266848220142215, 0.33541431019446977, 0.34253539572158626, 0.3375977700887783,
            0.3578912584020226, 0.3332142470071909, 0.2401490143563816, 0.2341118588902698,
            0.22060202401252918, 0.2110469999353281, 0.22324946339634189
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinerynec(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.2663694279283176, 0.22834274038650992, 0.19136133527590732, 0.1567241483829092,
            0.13627395112125568, 0.12207404682352825, 0.13086844378406912, 0.12703448169356632,
            0.13194301068066228, 0.11608788997022071, 0.0448765141282129, 0.04099402725968257,
            0.03927945818919396, 0.04207330253739464, 0.04949099593229013
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricalandopticalequipment(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.10697241151915922, 0.08110770964139015, 0.06775288279050015, 0.05798327242155274,
            0.04813151588123405, 0.04356663913750887, 0.04610646694846743, 0.049813041729922375,
            0.05209170202061897, 0.0462007411792655, 0.030935037153437074, 0.026442576907990945,
            0.021417439255133085, 0.01861581660746257, 0.01916486673716104
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransportequipment(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03230247009715763, 0.03365905354505978, 0.027996316297390002, 0.02328166049397064,
            0.016752929925473788, 0.020291977858137314, 0.0213013464890133, 0.0228524453077512,
            0.033935283609928715, 0.031736018731842015, 0.05254221452168104, 0.04936251156866958,
            0.04186236037475353, 0.04464775304925383, 0.04994551000737595
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturingnecrecycling(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.2524742276675683, 0.2770783096876561, 0.2892463743676505, 0.2726238912320256,
            0.20248756829133865, 0.23533342968400625, 0.23530529422681773, 0.247149639979618,
            0.19238247594215038, 0.198232738346037, 0.36604614345951936, 0.29344140426572635,
            0.2457290709739713, 0.20145663662856353, 0.195326355095624
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricitygasandwatersupply(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.17656446185316851, 0.19723181996517808, 0.18347204565651665, 0.19266562232704065,
            0.1947154183161916, 0.1618948285114104, 0.1630513827742791, 0.21828024089209305,
            0.22847274873040754, 0.22888674862368502, 0.25089241522740796, 0.3033529090412146,
            0.257287023612773, 0.21861331518289215, 0.23420824672911003
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.08379636901874359, 0.05912016941951932, 0.026342470308196652, 0.021830112391916626,
            0.0177824602263644, 0.016762388622089782, 0.017243465798204186, 0.01683030360991656,
            0.017512769640440632, 0.01567768728733379, 0.012056202327930386, 0.011630321576189033,
            0.01092204974650565, 0.009948286060340346, 0.009952056661525526
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03437430257370856, 0.03248580789445889, 0.0270797425335092, 0.02779519490656119,
            0.03217994573339533, 0.026495515235646395, 0.026497911781729094, 0.024298545909817323,
            0.03498663904464091, 0.035730772189885705, 0.03929017979841124, 0.03797602390213039,
            0.033945298554452795, 0.03266934670788762, 0.03173735550499495
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.042371359971517655, 0.05596603289438831, 0.04526361343825246, 0.04894881026177591,
            0.058810711388934206, 0.04360433804948141, 0.036269972299407226, 0.03565722771814744,
            0.03291479330176479, 0.037072826274117766, 0.04706699112577181, 0.044500239569127034,
            0.04317299982273536, 0.04155025444429505, 0.039743093028138735
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.038548253304305724, 0.04543492604717159, 0.03797619948818973, 0.038324294809383125,
            0.04647061847730034, 0.03601597769139513, 0.03304018345041006, 0.031854062061745764,
            0.03309240552870057, 0.035111994469669665, 0.040624476649071505, 0.04005480654467303,
            0.036903519610464, 0.0361959235250515, 0.03583695395061817
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotelsandrestaurants(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.05421856516261138, 0.06721093729720135, 0.05909171500281141, 0.06368378192849475,
            0.05173149970655956, 0.041638870297647956, 0.04098533755316226, 0.04087420088691245,
            0.04173290319937963, 0.04143908662406254, 0.05174077473653161, 0.05231967402658624,
            0.05148103706063099, 0.05301062747112252, 0.0511994977092531
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subinlandtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subwatertransport(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subairtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.24766113572697843, 0.1433899454448171, 0.12316507729787073, 0.13796832107982962,
            0.12429463594134163, 0.08049699793469801, 0.07487965639843151, 0.08137504711026855,
            0.08083071239759364, 0.05079584375909763, 0.07694141579440167, 0.08051152446751678,
            0.07359313523684147, 0.08391793791851684, 0.08941672292834295
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subpostandtelecommunications(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03630472699066584, 0.053799886422011715, 0.04635085848000384, 0.04920255698920714,
            0.04471989075620264, 0.028167882760934248, 0.026121487945967616, 0.02833649680492858,
            0.02612764861482731, 0.024237634423599087, 0.028316188776702115, 0.028717708371679678,
            0.02677222898860627, 0.02554407567267204, 0.02464708011310464
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancialintermedation(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.030423145843399658, 0.01665834393959276, 0.013724581645353938, 0.013857964113641258,
            0.016876689956649222, 0.014085984707775272, 0.01454129496610741, 0.015620240851815416,
            0.01145640746873261, 0.011402731070671244, 0.01498877870550671, 0.013780569855714568,
            0.013772255699667259, 0.014489989788855498, 0.013042316407139366
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subrealestateactivities(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.06748503295992885, 0.052712807858196084, 0.04472599605940692, 0.046509173529932196,
            0.0504066297130954, 0.03849224135054137, 0.0360052279616092, 0.03715113372307803,
            0.032936513579305195, 0.032487350660944145, 0.043093058601061336, 0.04439623389273437,
            0.04488968655828661, 0.04391661441880346, 0.04894407742230376
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subrentingodmeqandotherbusinessactivities(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.02293190808390416, 0.02262243635032486, 0.02096783126347895, 0.021200116426919404,
            0.022831201389171465, 0.017908038621137316, 0.016612837616866733, 0.015609073846054325,
            0.017273430847816965, 0.016751498672525022, 0.021446576044583145, 0.019672326657890368,
            0.019251023600707035, 0.017985852686757987, 0.018232748381631827
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublicadminanddefencecompulsorysocialsecurity(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.05140637380793385, 0.07822085710082032, 0.07133029723763389, 0.06891092689533442,
            0.06304826454808282, 0.05149168732541779, 0.0418730835245787, 0.04009143398935635,
            0.04430766757577621, 0.04199334476524284, 0.05229449894692725, 0.051383289960517796,
            0.049446733721309644, 0.045065768676422174, 0.042066680726921296
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.07013227324556913, 0.08104441544299763, 0.08272705449847896, 0.07810516179875657,
            0.06825900240482347, 0.06472244536514574, 0.0637069998901736, 0.06583975893681501,
            0.07683234666824508, 0.07429560990252872, 0.086218901772325, 0.08224517387718157,
            0.07588073999609178, 0.07548265200675518, 0.06914624772308771
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealthandsocialwork(x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.04920998564494497, 0.05141091949811415, 0.04964417414865848, 0.047972326347685064,
            0.04261668454896623, 0.038718065794993886, 0.036951395118389394, 0.033341191049112706,
            0.04012472109111849, 0.03942842881355761, 0.048905735335520575, 0.04624978583898568,
            0.04175277696184828, 0.04141778786473526, 0.0392434145450208
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subothercommunitysocialandpersonaservices(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.11996458634785939, 0.08592983866536506, 0.07838672881777277, 0.08568639893345602,
            0.07897091899630367, 0.0683711232726671, 0.0631457274135446, 0.060414224408576064,
            0.08316301095814016, 0.07813626750086931, 0.09398520749492556, 0.08550533473728855,
            0.0850786293713881, 0.08126627081667989, 0.07499198502816463
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivatehouseholdswithemployedpersons(
        x):
    """
    Historic final energy intensity by sector and fuel SUBheat SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subagriculturehuntingforestryandfishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3.329337118908045, 3.330675684050808, 3.3056876138324642, 3.1199531685254867,
            3.0241068932783115, 2.913297719495583, 2.9134123728860097, 2.8418511504522566,
            2.864071301035634, 2.9121439587818303, 2.8766896473028623, 2.8180790690079416,
            2.7650218349970377, 2.716188095273943, 2.6291186340338735
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subminingandquarrying(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.341362582961613, 1.3031888788382189, 1.430442507336978, 1.3586221542833017,
            1.2683375830030548, 1.1904792850612582, 1.3060180798221765, 1.3125273787385903,
            1.2362762669247982, 1.2020515629065311, 1.243794594549829, 1.1279633790764654,
            1.1888499764514466, 1.1056716925600512, 1.1957022090658815
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfoodbeveragesandtobacco(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.5141106312905513, 0.603598529324621, 0.5875630312879945, 0.5580747247388643,
            0.5415400295426829, 0.5264542344804909, 0.5176174853298501, 0.5357280165325976,
            0.5053332256371244, 0.5050467405535006, 0.4624651436890585, 0.43955110647063045,
            0.4211821670459373, 0.43365936067900906, 0.43695490528792386
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextilesandtextileproducts(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8806921388281855, 0.8756214954360507, 0.8493598989798039, 0.849401584399211,
            0.8177730902051842, 0.7683661837902472, 0.7385580752297171, 0.7266538185593405,
            0.685886679576849, 0.6750174885763368, 0.5772313619964377, 0.4942478870481967,
            0.4380942904533427, 0.3984498823683011, 0.39044188652557815
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleatherleatherandfootwear(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.543912206399191, 0.5340187940829365, 0.5116496072212218, 0.5283172564053502,
            0.52680124791453, 0.5160920116238285, 0.4839876013888461, 0.4903718807301217,
            0.47278823455177815, 0.48650732038204203, 0.4456619609063785, 0.38300531633542345,
            0.34493136665614327, 0.33242252290944396, 0.32373966589948894
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwoodandproductsofwooodandcork(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8531844182955877, 0.8718623558860619, 0.8991039249103806, 0.8887123996273485,
            0.8400586168150834, 0.7360387172883827, 0.7629195554611713, 0.7178919576542717,
            0.674971768143922, 0.6969505117942627, 0.628846884172979, 0.5627407956041065,
            0.5205881484627026, 0.5945847854329732, 0.5734254205861374
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulppaperprintingandpublishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.7725248579617263, 0.7829920825601699, 0.71588470498281, 0.6658319712890726,
            0.6109643718413529, 0.6275512605419433, 0.6274433630557608, 0.6317304071588181,
            0.6232452688814072, 0.6055655728152876, 0.5941347693276107, 0.5273776248355037,
            0.4675760180751636, 0.44067615705964414, 0.4127060249099261
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcokerefinedpetroleumandnuclearfuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            11.7231108444282, 11.564060149591148, 11.340352092537033, 11.563164910084271,
            10.864766657178281, 10.78734421352214, 10.422426170714616, 10.13479762834825,
            10.646299993259092, 9.876892067841245, 9.28312948240861, 8.218957060695121,
            8.251613356505768, 6.446731934830551, 7.192514001861613
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicalsandchemicalproducts(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.6141045942860075, 1.6909110512942793, 1.572229862535497, 1.5122922448899447,
            1.444844708751819, 1.6245028626437592, 1.6194445026822364, 1.5954316796075378,
            1.5077553991312074, 1.4466087946389843, 1.4173573214290192, 1.3253081564925795,
            1.1517646687971657, 1.1532075599936515, 1.147877898612611
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubberandplastics(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.07496675931583, 2.0471013607839823, 2.1060098875889803, 2.0250070773864786,
            1.9256772798330721, 1.7744313808070888, 1.8326844748437345, 1.8302390102895039,
            1.7403618599700157, 1.7490444809705308, 1.6296472792826477, 1.5410729153480742,
            1.523797656583321, 1.4965650486610333, 1.5970725518602833
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothernonmetalicmineral(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.1960551224028317, 2.209369279652865, 2.201003613289866, 2.2289873773480102,
            2.182166433525797, 2.125796639185664, 2.135941163866843, 2.1895788315277644,
            2.0588368623674085, 2.149389932401747, 2.044124977828056, 1.9443217660087966,
            1.7825558242556516, 1.796657802289093, 1.784985510455616
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasicmetalsandfabricatedmetal(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.7241195062411848, 0.7375635494333631, 0.7032144025710197, 0.6719289119030403,
            0.6670717802584506, 0.6116986317199548, 0.5740454082643064, 0.5817006667890227,
            0.5871598157245603, 0.5681759516759873, 0.5279323510489545, 0.48096034933659615,
            0.4313959142384465, 0.42793579021142536, 0.3892597105315791
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinerynec(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.42382675349627796, 0.43122964165918515, 0.4024212807620231, 0.3864226855321285,
            0.3651561958518691, 0.32735529906108074, 0.3280616894183554, 0.33543565446199,
            0.3015467633221359, 0.26741820406262334, 0.24156298078554952, 0.21311040005375667,
            0.1874255971936674, 0.18467487948687933, 0.19970663635058034
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricalandopticalequipment(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.4780198382356848, 0.4656733928129168, 0.42951861678564707, 0.40692686532943895,
            0.3501942085262995, 0.2948534837509099, 0.3024074848311568, 0.2965072485549675,
            0.2596846206533788, 0.24143762875863975, 0.2025018460671438, 0.17409890423093224,
            0.15556729617604012, 0.15031456794290005, 0.16111787204701797
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransportequipment(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.28330009685739516, 0.28893704651021057, 0.2773930693280514, 0.28170554050566016,
            0.2640356328569094, 0.26617313675337534, 0.26283685026058545, 0.2564223272621323,
            0.230220430382433, 0.20440510127256015, 0.18441103774116221, 0.164381507678766,
            0.13916350423569543, 0.13915089157188737, 0.14861014064157796
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturingnecrecycling(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.1439231260394123, 2.1822559556740875, 2.1474962473193537, 2.07879094938377,
            1.972911085545422, 1.8619188110702514, 1.9425922821998154, 1.9697176408084243,
            2.0376165598787592, 1.9143895352629738, 1.9587050999351112, 1.828934743240394,
            1.7881466556469001, 1.7799236844600743, 1.536870590861347
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricitygasandwatersupply(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.6479000510051925, 0.7888352005533726, 0.4910380502554513, 0.762775895251622,
            0.8909714193880099, 0.7699091727081196, 0.6930243309961277, 0.7477250122028472,
            0.5764056024097187, 0.6500826340495188, 0.596504106990596, 0.6436601191645978,
            0.6877840796316987, 0.5217922438419683, 0.4799829085953009
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.7068140570746776, 0.7227705762434552, 0.723014885894517, 0.7203116642657159,
            0.7079479952465305, 0.6676336719201289, 0.6906288420159953, 0.707442981637406,
            0.6912606246613766, 0.7418413135750408, 0.7067213863020272, 0.6910126127831341,
            0.6757742886564271, 0.6529588055898928, 0.6673692259969749
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.6516105473732899, 0.7067331641687331, 0.7052237313922668, 0.7032266073227602,
            0.6584109402587034, 0.5688049730923108, 0.5965215562557896, 0.5980643302533083,
            0.6041771342169036, 0.5388948338236673, 0.4964532411883088, 0.4691391675535737,
            0.4521656701888638, 0.4559478455247383, 0.4802210060653841
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.7995096107996469, 0.8252303745364935, 0.7798495117539251, 0.7444435424859365,
            0.72262496148166, 0.6588557039169136, 0.6603278340733744, 0.6678129315618117,
            0.6274976663337006, 0.5761831477921462, 0.5227289399539211, 0.4870441351630275,
            0.4405275466176488, 0.4212832080799799, 0.41068203325280145
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.446224555441523, 1.4643807252621894, 1.4226249680023841, 1.401696186693699,
            1.3626051781347446, 1.2573436676006837, 1.26578256600391, 1.2930398860925554,
            1.2494822721142014, 1.152885617016206, 1.018537732900704, 0.9540939035898561,
            0.9361325265437349, 0.9200090487527909, 0.9301466718050068
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotelsandrestaurants(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.4196391790703387, 1.3406818361044612, 1.2874351683880576, 1.2413950506121258,
            1.2026812023761946, 1.1181780600825535, 1.1472675222149598, 1.1918494467876615,
            1.172418278082716, 1.1060507580712087, 1.0084245382646344, 0.9600751466958125,
            0.9295922641309979, 0.9201921195598188, 0.9410191119656305
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinlandtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            5.784594651904558, 5.841882003967407, 5.839617640577593, 5.7234514917421215,
            5.657425050322958, 5.805799163252883, 5.663783687522083, 5.750384611283826,
            5.571938823049814, 5.823024459391582, 5.905534741704541, 5.866070650980922,
            5.986263477165527, 5.783156576760606, 6.039976824724781
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwatertransport(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            37.934160872041296, 36.99229763627306, 35.857425977728354, 32.96034180361318,
            31.445947900920526, 29.763612221163157, 26.048593193983724, 25.861329225695492,
            25.45709435265915, 25.418232855341213, 25.38070613356822, 25.818633973914036,
            25.508559284206285, 23.604923335035554, 23.628451979670505
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subairtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            26.715731790170462, 26.56880019698801, 26.328612139720242, 26.223829302523733,
            26.197997404918336, 26.067380154584512, 26.7898761094474, 27.715378037620063,
            26.232774917101356, 26.350842653034938, 27.148378588319208, 25.86595493563425,
            25.117966478688643, 24.98087409172215, 25.085688402244163
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.1229444350330606, 2.1858031309710717, 2.1320401162060647, 2.055422526573931,
            2.374243138577253, 2.370472430671425, 2.283326869757632, 2.380527825919046,
            2.310300684442889, 2.3597546052719696, 2.478585956426773, 2.4043233217469235,
            2.516730364247273, 2.433282842599178, 2.4998329347889965
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpostandtelecommunications(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.768866469814047, 0.7045417318922991, 0.7194975287130991, 0.686119642007903,
            0.7092963917543954, 0.654541940164107, 0.6347548762949851, 0.5634405956580067,
            0.5496258714368878, 0.5691231126948875, 0.5039055491247418, 0.4743621645845341,
            0.4544160511189765, 0.4363370772707986, 0.42985268631370466
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancialintermedation(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.3902641535503575, 0.39105057361588785, 0.36928625271596555, 0.3590196832786975,
            0.3311330705068184, 0.2928685985146136, 0.2917779755835705, 0.29868082636364557,
            0.2902083684045119, 0.26600477549860785, 0.23489360829412367, 0.21865593642713602,
            0.2028196283561895, 0.19794825830942292, 0.18964237442788068
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrealestateactivities(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.1409964245790585, 0.14439320564947894, 0.14063164520472649, 0.13509166535454392,
            0.1288958348609235, 0.11690815269898912, 0.11608242310919194, 0.11967779600337494,
            0.12119155409114757, 0.11743942535448604, 0.12178078876475229, 0.12048154605461525,
            0.11657170991065724, 0.09798634547528803, 0.09797353805266568
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrentingodmeqandotherbusinessactivities(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8331509374789908, 0.829491361635125, 0.8049435201879306, 0.7970443962992084,
            0.7720927405758682, 0.7200246935712061, 0.709344111552235, 0.7133550252479071,
            0.6943767987662407, 0.6622543272614511, 0.595351967333654, 0.5644181014297519,
            0.5320028562634156, 0.5176487865353288, 0.5129037303887946
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublicadminanddefencecompulsorysocialsecurity(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.188054173776091, 1.2133749217514826, 1.2293234237135575, 1.2591481738533437,
            1.23382876392151, 1.1449859659635278, 1.177309625404061, 1.0967878132962667,
            1.0769636298652006, 1.0096199030034558, 0.9373540473198093, 0.8892619382789926,
            0.8470477962264747, 0.8214546226563212, 0.8399946748583467
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.8557400891731884, 0.8946873392113956, 0.8930235708093455, 0.863600839879686,
            0.8440303344467589, 0.8331950040741682, 0.8489841047121085, 0.8783471403593619,
            0.9017617765416281, 0.8783858725603914, 0.8578588702110131, 0.84812637305127,
            0.8251310688591592, 0.8420228859112069, 0.8500830847935721
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealthandsocialwork(x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.0899935382020292, 1.0794120603730522, 1.0621939904932913, 1.08098506648033,
            1.0309042029489681, 0.9656209501278881, 0.9759940916694444, 1.0121111492679689,
            0.990627177098384, 0.9236746612908852, 0.8266047105195358, 0.7931182690161734,
            0.753621853534988, 0.7451840021094017, 0.7538561418115993
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothercommunitysocialandpersonaservices(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.1493987832688457, 1.1737860075312971, 1.1936830991238718, 1.166987638888675,
            1.1358084209938066, 1.0397880554918142, 1.0555258409763975, 1.0882492817835265,
            1.0919568464681755, 1.0506014179378114, 0.9811287055745656, 0.947410230772165,
            0.9201585543705517, 0.9257677200705718, 0.9278880445636408
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivatehouseholdswithemployedpersons(
        x):
    """
    Historic final energy intensity by sector and fuel SUBliquids SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subagriculturehuntingforestryandfishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.2150356857860538, 0.23808543700535903, 0.2249828547013219, 0.21602014550226922,
            0.25889017338562903, 0.3259500278002751, 0.30655078994607454, 0.29941530067408895,
            0.2930062742460066, 0.2847165229116941, 0.28312899573704253, 0.2654783961694343,
            0.27608551847790697, 0.26682611690254887, 0.2602340811751907
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subminingandquarrying(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            4.69078137659984, 4.674680491324511, 5.587114925579444, 6.1914402689816,
            5.643124554829877, 4.7890750283167876, 5.175069902516305, 5.305256393623547,
            5.391187388350845, 5.160359018472073, 4.822047105186946, 4.677087534518218,
            4.783909980508121, 4.2016196787038345, 4.66277570791695
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subfoodbeveragesandtobacco(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.2630712135501656, 0.27482351735400984, 0.27176046941280213, 0.2497142239689453,
            0.22989966178122906, 0.3600879841515737, 0.3413860103813444, 0.3547360068253931,
            0.36246315037251453, 0.3521274036529059, 0.32192202882420695, 0.3346373730931401,
            0.33396681864562816, 0.34726851971209743, 0.3339346560772236
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextilesandtextileproducts(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.22170152798157228, 0.2237974799341098, 0.22194372593124323, 0.2088333392472178,
            0.2078195342464195, 0.37449650739686546, 0.3586887970392119, 0.35716516554601063,
            0.3522560996360123, 0.3372593373609412, 0.2703351189299814, 0.2807123319032572,
            0.23938080094397174, 0.22992445335417921, 0.23320030953411783
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subleatherleatherandfootwear(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.09354931025043114, 0.08569200202644255, 0.07304451191760464, 0.07136564883324445,
            0.07142556681890348, 0.13274807570637964, 0.12685016213423064, 0.1330872415275701,
            0.1352147717183019, 0.13128698330208274, 0.11008525329131127, 0.11462176247199041,
            0.08361741992091336, 0.08401051515893582, 0.08239602053061558
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subwoodandproductsofwooodandcork(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.16492387728107027, 0.16491848452458757, 0.17214935197061923, 0.15540987441108312,
            0.1865545041278503, 0.29532803353441145, 0.2736965354434537, 0.27654305363553544,
            0.25044299075958315, 0.2658742371032976, 0.24614252504824943, 0.2469193165307909,
            0.2211512839987132, 0.24082680160621636, 0.24657379451326117
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulppaperprintingandpublishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.4924434997862261, 0.5035167282353104, 0.4653216859317899, 0.416591668347514,
            0.4086323731686502, 0.613317658719408, 0.5591768596748578, 0.5785423929231583,
            0.5518005251519551, 0.5041909188261511, 0.46497936329519063, 0.48430099601694454,
            0.465593045195173, 0.4675388580899925, 0.4605113243815074
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subcokerefinedpetroleumandnuclearfuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3.001313091203896, 3.0136097520997716, 3.0187830548072707, 3.0860044083105795,
            2.7827190716347494, 2.8832010213385604, 2.5713728213822455, 2.55786837768559,
            2.58934072303863, 2.3350796608400692, 2.3657057558749415, 2.127615858724935,
            2.2773583161358864, 1.801941232901524, 2.0365279736233033
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicalsandchemicalproducts(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.4720283824991145, 1.5052003901403945, 1.227394683360337, 1.0698578280829782,
            0.9930600957911819, 1.7935253915292202, 1.6220540746157186, 1.6494410859171633,
            1.5954078706124257, 1.4787828774506468, 1.4343100769756831, 1.3652166052244967,
            1.372267493452264, 1.443397904881445, 1.4393643503734865
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubberandplastics(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.0293817265575258, 0.9634590176256144, 0.8084872172503001, 0.7447988463130435,
            0.7574406019185975, 1.1443065304809965, 1.1112397447609332, 1.1054866360049462,
            1.0595606692803112, 0.8905151550364396, 0.916813834866961, 0.9561836159085504,
            1.026366866964444, 1.020582712688694, 0.988506771535562
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subothernonmetalicmineral(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.3434388215425517, 1.3321895549088403, 1.2478116049949717, 1.1401006559973619,
            1.0459696896128106, 1.6702388734857427, 1.6313622184199437, 1.616916337047098,
            1.6330751267939065, 1.6658242098361133, 1.7203220587904575, 1.7494487873793174,
            1.5969640730489476, 1.7302326094995535, 1.6461143264900595
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasicmetalsandfabricatedmetal(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.3108065204731356, 1.2970726797702823, 1.223246411855429, 1.1573621173483866,
            1.0881253181024007, 1.3578255707345974, 1.2821081524399458, 1.3184894219974421,
            1.2981874351653735, 1.3748165507613535, 1.344758899655073, 1.2838653063511276,
            1.2020320290082778, 1.2277708817077104, 1.1200065077483736
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinerynec(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.12021541503389496, 0.125859005200925, 0.11693052615703729, 0.10233769702234845,
            0.10751502036095155, 0.16047071251418615, 0.16435763359568387, 0.17573359055047366,
            0.13896859663809286, 0.12862940828249575, 0.13033325871434107, 0.12927142408537162,
            0.12075807835585903, 0.11131003163056657, 0.1228715703315172
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricalandopticalequipment(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.11143834233419135, 0.10446346973068762, 0.08630328151060919, 0.07603249406013288,
            0.07347651364949342, 0.10598282339115185, 0.10454045158487307, 0.10568435481581273,
            0.09458777586901591, 0.08087214110403276, 0.0779417468457911, 0.0773382557605845,
            0.07139255213764506, 0.06749995759716539, 0.0696880791985734
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransportequipment(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.09171080390030475, 0.09674881322946037, 0.09240595736273877, 0.08242188677387538,
            0.07992416080483881, 0.1317976929305965, 0.12381969027745805, 0.12022025094727935,
            0.11739312755701244, 0.11216561087233788, 0.10393196609250903, 0.11603835704046012,
            0.0995192919370611, 0.10302801213369138, 0.11017171508322544
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturingnecrecycling(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.6361904215163183, 0.5768012848405862, 0.49606560248822923, 0.4791899670272098,
            0.4655241319213859, 0.7072263003043257, 0.6779642867657234, 0.7087227274542818,
            0.7253802448834674, 0.6980047177438519, 0.6557236985272097, 0.7780231182652662,
            0.6918367741023013, 0.7164942889097418, 0.6922034067991938
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricitygasandwatersupply(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            3.922396867876367, 3.8908805002198408, 3.960153185544289, 3.83014763381815,
            3.9460710609040603, 3.774894877799922, 3.626795439041342, 3.6828147873277035,
            3.6692242539163367, 3.316258634594196, 3.1406659378887456, 3.125622734727022,
            3.0467002692300236, 2.719112027570859, 2.731191712620855
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0576678281130595, 0.05759797955978964, 0.054849216142609805, 0.0552265188459029,
            0.05300736053494253, 0.05613113180406631, 0.05618862193148195, 0.06099338784248549,
            0.05319371454310903, 0.05768052438354068, 0.0628364091240574, 0.0717340226442291,
            0.06750868575973777, 0.06593784904217394, 0.06379826136672472
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.12338711663998722, 0.14156085039019767, 0.13373724297590503, 0.13413682440414731,
            0.12139209363720632, 0.10245712863372658, 0.11672220331164067, 0.10247692561851296,
            0.1206959374120446, 0.11796345098172378, 0.10455470303966023, 0.11496503227562352,
            0.10979747716211825, 0.1152407626679457, 0.11355533230389225
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.08311735701445912, 0.08264550858150513, 0.07434285189320597, 0.06999926533913718,
            0.07061226806353163, 0.0716951075551002, 0.07334576145974414, 0.05856954203692449,
            0.06790223122416063, 0.06816169525522552, 0.07017975751497046, 0.07055662220598093,
            0.06856935642669208, 0.06884983984819669, 0.059109336003401826
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.18343784780840566, 0.17571147382553004, 0.18011471116892394, 0.17822724063783765,
            0.18225544144230094, 0.18442893092936644, 0.19182988574399967, 0.16835349960126303,
            0.1890527790350421, 0.2018756033970964, 0.177562069213027, 0.18024065768526512,
            0.1817884994152512, 0.18541308729004072, 0.17702047126196366
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotelsandrestaurants(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.2296887595143569, 0.24545601899441238, 0.23698316507221934, 0.22727014618673852,
            0.21806857343434236, 0.21680620576486417, 0.23817525049187382, 0.1975478906533931,
            0.23029168731264182, 0.27893354750435984, 0.28591236839346135, 0.31385282829780764,
            0.3331125790232874, 0.36885491612934235, 0.4056754915204717
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subinlandtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.768689072259116, 1.7917046309320366, 1.6435054662707695, 1.7138042008227812,
            1.6384909990256893, 1.5335878589335115, 1.5632852095573495, 1.6604083471917965,
            1.7233067613550759, 1.706074719516254, 1.7138893365378558, 1.662875867498881,
            1.6927180417809842, 1.7030920439173627, 1.5852983930085915
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subwatertransport(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            4.574847164421825e-06, 3.5083317099837725e-06, 2.2543503767533515e-06,
            3.17598044517727e-06, 3.0609406155923296e-06, 4.179549733011051e-06,
            4.791593519426679e-06, 5.593053392243618e-06, 6.2620557715434685e-06,
            5.3582774971173225e-06, 5.569080572749993e-06, 4.6301136778513e-06,
            2.8986228116751754e-06, 2.3681513939620776e-06, 2.6750270062826567e-06
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subairtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.98484033210949e-05, 1.7923726095086716e-05, 1.605546819730422e-05,
            1.4401849284183056e-05, 1.2420497571094495e-05, 1.7167074083988788e-05,
            2.034013678030099e-05, 2.1669727520137526e-05, 2.1755735666445098e-05,
            2.460157375607659e-05, 2.572173276240859e-05, 2.4062924673815406e-05,
            1.828053757134078e-05, 1.700543257260789e-05, 1.8955822151764034e-05
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.15920033013924226, 0.1462653412657254, 0.13750798057984082, 0.12679160517105934,
            0.11254947017031008, 0.0992373868312057, 0.10003573898651195, 0.09807307747323457,
            0.098822003010246, 0.0958235680501819, 0.1026264908600997, 0.11059539187753116,
            0.10686343674581243, 0.10442853400674075, 0.10464609635555286
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subpostandtelecommunications(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.557114158189985, 0.5472730387659961, 0.5197418530500941, 0.48040480838419714,
            0.463796507686431, 0.19502066451585254, 0.2289077295900135, 0.17540867590327613,
            0.1928580587552419, 0.19487285803924426, 0.18958196691492568, 0.17019557521741102,
            0.16233097499278262, 0.175412440542782, 0.19572299833922094
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancialintermedation(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.04635997696533132, 0.046744865231611685, 0.0447351069202782, 0.04268199059411815,
            0.04019711180265761, 0.04058553299336325, 0.04038838748973608, 0.03749500409530591,
            0.041477197913039505, 0.04384957478665145, 0.04314727817008194, 0.03987437403323019,
            0.04170324486339083, 0.04301901300787914, 0.04127577120538779
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subrealestateactivities(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.08985636789862816, 0.08216238775043112, 0.07728302303128676, 0.07620240719739438,
            0.08203065255604013, 0.08084037631457545, 0.07381013913588647, 0.07785242082615067,
            0.07619088399937314, 0.07697874791338906, 0.07792588444866497, 0.07842749110707001,
            0.08563504721208058, 0.07975016800990146, 0.07296546325240949
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subrentingodmeqandotherbusinessactivities(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.09772309728227796, 0.11115803062770635, 0.10580241841803352, 0.10681667576834784,
            0.10522864842044777, 0.09467934401641757, 0.10213827710360436, 0.07833873328067115,
            0.09223915867588768, 0.09732601885051016, 0.09079701149562185, 0.08828570175789931,
            0.09002758585564463, 0.09515527299978106, 0.10356363499920908
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublicadminanddefencecompulsorysocialsecurity(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.5373951654266507, 1.5773355485211076, 1.6069214033563568, 1.5581521796463662,
            1.5924836438541974, 0.8946457984749805, 0.7860677446553317, 0.995986989631449,
            0.9270969437849531, 0.920655350885897, 0.8808926606088301, 0.8257451964081314,
            0.8606127743772815, 0.8316784911656734, 0.7942589878807084
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.19922448291807177, 0.23531238535474305, 0.2231415545653652, 0.22317933842060514,
            0.21438305202709548, 0.20330385315308613, 0.20638467532306168, 0.20166091895972965,
            0.21881425132286253, 0.23146364374082085, 0.22149602406457594, 0.21929110510209326,
            0.21676984237117738, 0.2211725111125796, 0.21082169303149165
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealthandsocialwork(x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.16141074562287921, 0.1732020915342724, 0.1657377513264254, 0.15836996028351982,
            0.14672777699115497, 0.1409050090986358, 0.1482148290757661, 0.11775767279203651,
            0.13644975758106215, 0.14151464690529184, 0.1413414886206319, 0.1476061657474311,
            0.14650608522130928, 0.15076300318769348, 0.1530470898697172
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subothercommunitysocialandpersonaservices(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.24354639533712438, 0.22410397697909837, 0.2181196495176965, 0.21745540669238772,
            0.20644271381163165, 0.20161633674608678, 0.19625973853525425, 0.1554216693170553,
            0.1772699128910812, 0.1885584053327525, 0.19696970570613992, 0.19897657587795384,
            0.20464005931279272, 0.20326798159486942, 0.1974065126411919
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivatehouseholdswithemployedpersons(
        x):
    """
    Historic final energy intensity by sector and fuel SUBgases SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subagriculturehuntingforestryandfishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBAgricultureHuntingForestryandFishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.47086767577643307, 0.4689734098478973, 0.46425486815666095, 0.44534418746398285,
            0.3761325320228421, 0.3551956503752216, 0.3365821032904595, 0.34446052697705376,
            0.3369613067779433, 0.3777420374226403, 0.37787484702445184, 0.3437689486231854,
            0.33878148476263686, 0.2577659589029426, 0.254453630249139
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subminingandquarrying(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBMiningandQuarrying



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.9506886637716337, 1.086630598111202, 1.01667597588606, 1.0660191521961055,
            0.7868691739470693, 0.6526677911564813, 0.7204889529796701, 0.7089010456405578,
            0.7794095503774782, 0.9704793933712182, 0.9823892491919919, 0.9278313661875002,
            1.0833070867907952, 1.1106331881813756, 1.1870484516524065
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfoodbeveragesandtobacco(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBFoodBeveragesandTobacco



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.047944772544057, 1.0360568758730864, 1.014018726524927, 1.037066032050261,
            0.9815967965758055, 0.8843014622198658, 0.8913773668345029, 0.9258706928918075,
            0.9257269945592518, 0.9338086140710012, 0.9319479467345817, 0.9313754300283074,
            0.9057922750003461, 0.9381713242043601, 0.9704476184444321
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextilesandtextileproducts(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBTextilesandTextileProducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.9919552339550126, 0.847724044256255, 0.8186147769677025, 0.81062296508303,
            0.679675887864844, 0.5982756067962458, 0.576790088518787, 0.5340863948470222,
            0.5622607319189838, 0.5959511163236807, 0.5801706727339709, 0.5566949002161229,
            0.624951990189146, 0.5381622642204315, 0.49724291289232403
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleatherleatherandfootwear(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBLeatherLeatherandFootwear



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.36017316114149805, 0.30773968477634617, 0.2849058765356348, 0.28389198839865315,
            0.2758287894515175, 0.2705123915880374, 0.28642317546372925, 0.2592775000171222,
            0.25206032689756724, 0.2709501807246148, 0.2663224865185713, 0.2287097852786095,
            0.2384944850405983, 0.21190362815894584, 0.20248687709043212
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwoodandproductsofwooodandcork(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBWoodandProductsofWooodandCork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.543584669525891, 1.5713183395016395, 1.646331221104433, 1.795836426642832,
            1.7003984320277263, 1.6450077961022413, 1.5214839862171476, 1.460250568308033,
            1.5076450931321594, 1.490623203594703, 1.4882085700158616, 1.507689366543973,
            1.4345046259573466, 1.340937777405623, 1.4033225078918963
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulppaperprintingandpublishing(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBPulpPaperPrintingandPublishing



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.4159664231910054, 1.304235051194609, 1.281397330192007, 1.2932604822767144,
            1.7279845074651576, 1.79209690458253, 1.6591041409759142, 1.6661724225835628,
            1.6987801938092408, 1.7486162038976443, 1.7029419945596787, 1.7735393640433785,
            1.6853550268307096, 1.672671289783725, 1.6711461492308324
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcokerefinedpetroleumandnuclearfuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBCokeRefinedPetroleumandNuclearFuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.5635188463251101, 0.5851743616352155, 0.644313184666882, 0.5956834437835982,
            0.49924952325982, 0.43173984930216935, 0.43175921769540554, 0.4496436343012353,
            0.5259551145590667, 0.5010401981173834, 0.5140242524355785, 0.4598391323689899,
            0.5220070922514527, 0.4924544615127271, 0.5813028667048447
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicalsandchemicalproducts(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBChemicalsandChemicalproducts



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.6903382545784063, 1.71695887393953, 1.469956704294383, 1.4895419256445153,
            1.2626857708375467, 1.1688347660484393, 1.0871156670944135, 0.9774906791790546,
            0.9984063908017585, 0.8767153852524797, 0.8983008834549645, 0.9166930108086413,
            0.893563172950253, 1.0391958069307874, 1.0175599074126027
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubberandplastics(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBRubberandPlastics



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.737440360502975, 2.653168017704157, 2.3746478495781074, 2.301560235678491,
            2.1343633621673237, 2.2543433691103147, 2.362199596156166, 2.2395715789595965,
            2.2315149801129435, 2.5658474287552555, 2.3535271983433366, 2.1753693103690255,
            1.9161742444311836, 2.0402872988069793, 2.339955454982643
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothernonmetalicmineral(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBOtherNonMetalicMineral



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            7.997229698191975, 7.898914325545515, 7.359390241247763, 7.592310249488967,
            6.762910078485197, 6.149652094708481, 5.861348640263628, 5.853044728617025,
            6.3261226753336075, 7.468790661464871, 7.079585118618744, 6.69047655316014,
            6.401360089133524, 7.701962886349043, 8.312149498212824
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasicmetalsandfabricatedmetal(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBBasicMetalsandFabricatedMetal



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.8148348986360774, 2.7453111311363547, 2.62872708237162, 2.659907145169566,
            2.5387101246366677, 2.4665722660034266, 2.469370038397535, 2.500267643907507,
            2.576380342176518, 2.6695802148310936, 2.7472797340606054, 2.5883819547704148,
            2.5827382350380548, 2.5709281525712395, 2.740962862203819
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinerynec(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBMachineryNec



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.34965201090783354, 0.3485096842068839, 0.279054906117444, 0.2722305779495829,
            0.21136922436306088, 0.15806235724746237, 0.1484902355772645, 0.14741882829007508,
            0.1384083188884344, 0.13466797467401953, 0.13256471647883097, 0.12398225671433248,
            0.11652380828759189, 0.13134038990693306, 0.16728426817745956
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricalandopticalequipment(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBElectricalandOpticalEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.08366552295776288, 0.07699981553840246, 0.06369621210588272, 0.058340254239730174,
            0.042325918786184684, 0.0320595402306469, 0.02807478003124616, 0.027376385420920114,
            0.02927390039219211, 0.024577044028479374, 0.025339117161324965, 0.025543833065601596,
            0.02342428699557163, 0.023911288618159554, 0.026689358813394036
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransportequipment(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBTransportEquipment



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.11490861193939238, 0.1081115484591803, 0.09782318939953644, 0.09946326835018271,
            0.08621479823797218, 0.06788637710209482, 0.058334866072842516, 0.05555683566291642,
            0.048682890764563565, 0.059841856401587394, 0.06601679929344707, 0.06934989494004637,
            0.06723161358908392, 0.07580187120558012, 0.09639910951746274
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturingnecrecycling(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBManufacturingNecRecycling



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            2.1260014275788457, 2.057377428407371, 2.125381313616282, 1.9796899098853875,
            1.8710438855509501, 1.4687557873678239, 1.4589131498308834, 1.4739066155637164,
            1.700472171062823, 1.130894749465082, 1.2406698372753777, 1.1490739423944445,
            1.2665482132346975, 1.2983931028176303, 1.0499525592289427
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricitygasandwatersupply(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBElectricityGasandWaterSupply



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            1.3084750756044377, 0.8104891430027502, 1.4617037707221665, 1.4740996259925323,
            1.4532688376916307, 0.7903411399721437, 0.6739217324429169, 0.6471981342381012,
            0.5778798448053956, 1.1868744116093666, 1.2377147803870046, 1.493035799268893,
            1.4076289630531469, 1.9610455757353344, 2.473662882947921
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBConstruction



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0806173098110524, 0.08594746821812554, 0.08227770261546485, 0.08790092227513241,
            0.082592651911312, 0.07803037174132649, 0.07631604424840002, 0.07855546769792085,
            0.08064242615211846, 0.09077341072803267, 0.0935716874675732, 0.09678117644493296,
            0.09358490192627616, 0.09520594589281828, 0.09338286988860646
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.025721291024246726, 0.016083306734592066, 0.014490815894267404, 0.011709581795349921,
            0.011843814344314, 0.006437423982575871, 0.006038090062378964, 0.006924385237484932,
            0.005840258138144695, 0.005757907828806607, 0.006912255105327055, 0.006751246171271431,
            0.006345110988541257, 0.016012871441109883, 0.014688616766985556
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.03537462347797025, 0.027669882015807064, 0.019705301927951002, 0.017788346538106767,
            0.017375946586304866, 0.01627434845823217, 0.01727836710245665, 0.017746088363663474,
            0.01741101783195977, 0.01560001702399118, 0.01671682497154882, 0.015361347240106649,
            0.015576581908921505, 0.023605606963863403, 0.02326698711777169
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.037316453987752074, 0.03091046696966835, 0.024031385122695918, 0.019608921509661207,
            0.02168496086862959, 0.015667218303222808, 0.014353440929141027, 0.01681562607212218,
            0.01606014700682523, 0.017763624047269853, 0.017737844896048568, 0.01563874466158938,
            0.015034232682292148, 0.02157103204247284, 0.019494781005226815
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotelsandrestaurants(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBHotelsandRestaurants



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.4722702682067137, 0.47460825688986324, 0.47383107688548043, 0.3703955333570004,
            0.37102085338660346, 0.35307411559939295, 0.33593866092970665, 0.3338866063826925,
            0.3216559203857432, 0.3142122903875573, 0.2967407036504295, 0.2814820577560224,
            0.27497346761202845, 0.30930676588570116, 0.4039358176250789
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinlandtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBInlandTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.19289283581954494, 0.16574545130667276, 0.19524776114800166, 0.18703942896412112,
            0.16542357718750053, 0.14393693859557874, 0.1314425924974032, 0.13094386215497808,
            0.1269476862589329, 0.10145820290830995, 0.09604043673468537, 0.08083495467828675,
            0.07216014172557603, 0.06770309824135796, 0.06913765881252633
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwatertransport(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBWaterTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.020257549859789746, 0.019448944791747077, 0.019280458523735264, 0.01762807010271867,
            0.016233261604574407, 0.015221325496242827, 0.017148001015014576, 0.009232557504386006,
            0.008250926485561576, 0.008534341220736591, 0.011991602081475282, 0.010329416483131851,
            0.010122791173485413, 0.008017104407259123, 0.009792350532875372
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subairtransport(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBAirTransport



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0, 3.7320077908385635e-09, 2.2663659904102352e-08, 1.8129530502928767e-09,
            8.158887940594626e-10, 1.247536137308539e-08, 1.8922947444433446e-08,
            5.832519874703696e-08, 4.1394275468854056e-08, 0.0, 9.204725637811062e-08, 0.0, 0.0,
            0.0, 1.4927273666771165e-06
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.0493350265929109, 0.029880114079265312, 0.021769099877484484, 0.020264698559728334,
            0.017857791999022384, 0.014372833892743186, 0.011835549572757174, 0.01408806635254586,
            0.01675283806156665, 0.01713560079288289, 0.023635420648333905, 0.025905630959936757,
            0.028382091125843972, 0.035127368310808114, 0.03583322404994238
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpostandtelecommunications(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBPostandTelecommunications



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.008802154580489769, 0.006849775384173932, 0.006373370813379648, 0.006583077487266303,
            0.008034321437860639, 0.0073363880967828735, 0.007681985254192262,
            0.008404733845272091, 0.007886118279475168, 0.007305669706869622, 0.006415787721319419,
            0.005497382571308583, 0.004051243085745235, 0.005356114156355983, 0.00632167139261299
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancialintermedation(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBFinancialIntermedation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.014962966037717462, 0.006999623262715352, 0.0036871410062793044,
            0.0033891821903891508, 0.0034949956176403957, 0.0034969511236112604,
            0.0034045480124904604, 0.004572338604714762, 0.003151566761297772,
            0.002744336513450184, 0.003656388017117741, 0.003261585586787987, 0.002924517223465735,
            0.004112842705433558, 0.0035234857215375435
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrealestateactivities(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBRealEstateActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.05228868249614063, 0.03642181330610362, 0.022236844264576505, 0.020547510389879235,
            0.02261782112835534, 0.021273067203708175, 0.019078580107547097, 0.0203027669092925,
            0.01799919562097267, 0.017683685455269195, 0.018537828954760974, 0.01700557316702225,
            0.012207164859604358, 0.014836353679504581, 0.014944020301411355
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrentingodmeqandotherbusinessactivities(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBRentingodMEqandOtherBusinessActivities



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.029830788986078677, 0.029845876171776614, 0.02113408923374507, 0.019246162349765873,
            0.01892142091938055, 0.017096721854260245, 0.01609687839592735, 0.015985049597349442,
            0.017093125327429747, 0.01745848677226531, 0.016628019140136795, 0.015934870158277582,
            0.015999333840099075, 0.024502755541621134, 0.029755588523413663
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublicadminanddefencecompulsorysocialsecurity(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBPublicAdminandDefenceCompulsorySocialSecurity



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.07497030892925732, 0.07517673240762333, 0.05697302605983134, 0.0518955977723396,
            0.05837898098001085, 0.059572159062379205, 0.05560159332740479, 0.06700976660980495,
            0.07550049342595831, 0.08251023861209508, 0.08367537095400508, 0.07903576498544398,
            0.07918587821401651, 0.09242157665491013, 0.08212842751731904
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBEducation



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.1919927279978261, 0.19636605203264995, 0.13655203248504122, 0.13531732732379906,
            0.12183360291423968, 0.11352346481802379, 0.10736468368656304, 0.09991975953695116,
            0.0969676728724115, 0.08726517762316098, 0.07728313027204972, 0.07702355401757097,
            0.06538195677412448, 0.11079695373357354, 0.1145460059306778
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealthandsocialwork(x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBHealthandSocialWork



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.05863354785334826, 0.04942708233077913, 0.040557906871152904, 0.03822182114509443,
            0.034823706400789224, 0.030804019895711035, 0.02692162160782073, 0.02454673774799809,
            0.02701696026679188, 0.0301557121060672, 0.033701894426510906, 0.040421025987261654,
            0.047543910250616266, 0.07589495099367287, 0.0789741172107411
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothercommunitysocialandpersonaservices(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBOtherCommunitySocialandPersonaServices



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [
            0.06819527064185982, 0.04535426311241609, 0.0293864371017948, 0.026153967022636167,
            0.04590856884905141, 0.04269116062911706, 0.033670880404642486, 0.04238727073336363,
            0.0704906307634303, 0.06313010917804748, 0.05951784163038456, 0.055528082109709026,
            0.052090285210014645, 0.08120230568637982, 0.08137642990063773
        ])
    name = 2
    return round_sig(x=ret, name=name)

def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivatehouseholdswithemployedpersons(
        x):
    """
    Historic final energy intensity by sector and fuel SUBsolids SUBPrivateHouseholdswithEmployedPersons



    lookup


    """

    ret =  functions.lookup(
        x,
        [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009],
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def initial_instal_cap_res_elec():
    """
    initial instal cap RES elec



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.62449003], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.00738462], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(
            data=[0.02075556], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.00020761], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.00455706], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(
            data=[4.77710420e-06], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[3.92104377e-05], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.0003538], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def abundance_res_elec2():
    """
    abundance RES elec2



    component


    """

    ret =  np.sqrt(abundance_res_elec())
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def potential_tot_generation_res_elec_twh():
    """
    potential tot generation RES elec TWh



    component


    """

    ret =  functions.sum(potential_generation_res_elec_twh())
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('step')
def res_elec_tot_overcapacity():
    """
    RES elec tot overcapacity



    component


    """

    ret =  functions.if_then_else(
        abs(potential_tot_generation_res_elec_twh() ) < equality_threshold, 0,
        (potential_tot_generation_res_elec_twh() - fe_real_tot_generation_res_elec_twh()) /
        potential_tot_generation_res_elec_twh())
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# EROI - RES elec
@cache('step')
def wear_res_elec():
    """
    wear RES elec



    component


    """

    ret =  functions.if_then_else(time() < 2015, 0,
                                  installed_capacity_res_elec_tw() / lifetime_res_elec())
    name = 2
    return round_sig(x=ret, name=name)

def xcpxini_res_elecx_subhydro(x):
    """
    XCpXini RES elecX SUBhydro



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            0.42302360615086493, 0.42302360615086493, 0.42302360615086493, 0.42302360615086493,
            0.35794305135842425, 0.332, 0.332, 0.332, 0.332, 0.332, 0.332, 0.332
        ])
    name = 2
    return round_sig(x=ret, name=name)

def xcpxini_res_elecx_subxgeotxelecx(x):
    """
    XCpXini RES elecX SUBXgeotXelecX



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100],
        [0.65, 0.65, 0.65, 0.65, 0.65, 0.65, 0.65, 0.65, 0.65, 0.65, 0.65, 0.65])
    name = 2
    return round_sig(x=ret, name=name)

def xcpxini_res_elecx_subxsolidbioexelecx(x):
    """
    XCpXini RES elecX SUBXsolidbioEXelecX



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100],
        [0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5])
    name = 2
    return round_sig(x=ret, name=name)

def xcpxini_res_elecx_suboceanic(x):
    """
    XCpXini RES elecX SUBoceanic



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100],
        [0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.2])
    name = 2
    return round_sig(x=ret, name=name)

def xcpxini_res_elecx_subwindonshore(x):
    """
    XCpXini RES elecX SUBwindonshore



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100],
        [0.21, 0.21, 0.21, 0.21, 0.21, 0.21, 0.21, 0.21, 0.21, 0.21, 0.21, 0.21])
    name = 2
    return round_sig(x=ret, name=name)

def xcpxini_res_elecx_subwindoffshore(x):
    """
    XCpXini RES elecX SUBwindoffshore



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100],
        [0.315, 0.315, 0.315, 0.315, 0.315, 0.315, 0.315, 0.315, 0.315, 0.315, 0.315, 0.315])
    name = 2
    return round_sig(x=ret, name=name)

def xcpxini_res_elecx_subsolarpv(x):
    """
    XCpXini RES elecX SUBsolarPV



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100],
        [0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15])
    name = 2
    return round_sig(x=ret, name=name)

def xcpxini_res_elecx_subcsp(x):
    """
    XCpXini RES elecX SUBCSP



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100],
        [0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('run')
def g_per_t():
    """
    G per T



    constant


    """

    ret =  1000
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ELEC RES - Capacities and generation all techn
@cache('run')
def time_construction_res_elec():
    """
    time construction RES elec



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[2.], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[2.], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[2.], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[2.], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[2.], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[4.], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[1.], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[2.], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ELEC RES - Capacities and generation all techn
@cache('run')
def time_planification_res_elec():
    """
    time planification RES elec



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[2.], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[3.5], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.5], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.5], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[1.], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.5], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.08333333], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.5], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def initial_required_capacity_res_elec():
    """
    initial required capacity RES elec



    constant


    """

    ret =  xr.DataArray(
        data=[table_hist_capacity_res_elec_subhydro(1996) - table_hist_capacity_res_elec_subhydro(1995),

        table_hist_capacity_res_elec_subxgeotxelecx(1996) -
        table_hist_capacity_res_elec_subxgeotxelecx(1995),

        table_hist_capacity_res_elec_subxsolidbioexelecx(1996) -
        table_hist_capacity_res_elec_subxsolidbioexelecx(1995),

        table_hist_capacity_res_elec_suboceanic(1996) -
        table_hist_capacity_res_elec_suboceanic(1995),

        table_hist_capacity_res_elec_subwindonshore(1996) -
        table_hist_capacity_res_elec_subwindonshore(1995),

        table_hist_capacity_res_elec_subwindoffshore(1996) -
        table_hist_capacity_res_elec_subwindoffshore(1995),

        table_hist_capacity_res_elec_subsolarpv(1996) -
        table_hist_capacity_res_elec_subsolarpv(1995),

        table_hist_capacity_res_elec_subcsp(1996) - table_hist_capacity_res_elec_subcsp(1995)],
        coords={'res elec':['hydro','xgeotxelecx','xsolid bioexelecx','oceanic','wind onshore','wind offshore','solar pv','csp']},
        dims=['res elec'])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
# ELEC RES - Total monetary investments
@cache('step')
def new_res_elec_capacity_under_planning():
    """
    new RES elec capacity under planning



    component


    """

    ret =  np.maximum(0, required_capacity_res_elec_tw() / time_planification_res_elec())
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - PHS
# ELEC RES - Capacities and generation all techn
# WATER - demand
# ESOI - PHS
# EROI - RES elec
# EROI - RES elec
@cache('run')
def lifetime_res_elec():
    """
    lifetime RES elec



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[80.], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[30.], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(data=[30.], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[40.], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[20.], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[20.], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[25.], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[25.], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Capacities and generation all techn
@cache('run')
def past_res_elec_capacity_growth():
    """
    past RES elec capacity growth



    constant


    """

    ret =  utils.xrmerge([
        xr.DataArray(data=[0.02816056], coords={'res elec': ['hydro']}, dims=['res elec']),
        xr.DataArray(data=[0.02391914], coords={'res elec': ['xgeotxelecx']}, dims=['res elec']),
        xr.DataArray(
            data=[0.07241524], coords={'res elec': ['xsolid bioexelecx']}, dims=['res elec']),
        xr.DataArray(data=[0.04771433], coords={'res elec': ['oceanic']}, dims=['res elec']),
        xr.DataArray(data=[0.25131581], coords={'res elec': ['wind onshore']}, dims=['res elec']),
        xr.DataArray(data=[0.41031785], coords={'res elec': ['wind offshore']}, dims=['res elec']),
        xr.DataArray(data=[0.45342377], coords={'res elec': ['solar pv']}, dims=['res elec']),
        xr.DataArray(data=[0.295], coords={'res elec': ['csp']}, dims=['res elec']),
    ])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def afforestation_program_2020_gtco2():
    """
    Afforestation program 2020 GtCO2



    component


    """

    ret =  afforestation_program_2020(time()) * activate_affores_program() / (
        c_per_co2() * mt_per_gt())
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def mt_per_gt():
    """
    Mt per Gt



    constant


    """

    ret =  1000
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def gtl_efficiency():
    """
    GTL efficiency



    constant


    """

    ret =  0.52
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def ctl_efficiency():
    """
    CTL efficiency



    constant


    """

    ret =  0.31
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xctlxgtl_gbx():
    """
    XCTLXGTL GbX



    component


    """

    ret =  xfes_ctlxgtl_ejx() / gboe_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xmbxd_per_ejxyearx():
    """
    XMbXd per EJXyearX



    constant


    """

    ret =  0.479726
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('step')
def max_extraction_uranium_ej():
    """
    max extraction uranium EJ



    component


    """

    ret =  functions.if_then_else(abs(choose_extraction_uranium_curve() - 1) < equality_threshold,
                                  table_max_extraction_uranium_ewg13_ej(rurr_uranium()),
                                  functions.if_then_else(abs(choose_extraction_uranium_curve() - 2) < equality_threshold,
                                                         table_max_extraction_uranium_zittel12(
                                                             rurr_uranium()),
                                                         table_max_extraction_uranium_user_defined(
                                                             rurr_uranium())))
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def hist_growth_ctl():
    """
    Hist growth CTL



    constant


    """

    ret =  0.021
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def gboe_per_ej():
    """
    Gboe per EJ



    constant


    """

    ret =  5.582
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Uranium extraction

def table_max_extraction_uranium_user_defined(x):
    """
    table max extraction uranium user defined



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xuserxdefined_extraction_growth_unconv_gasx(x):
    """
    XUserXdefined extraction growth unconv gasX



    lookup


    """

    ret =  functions.lookup(x, [
        2015, 2020, 2025, 2030, 2035, 2040, 2045, 2050, 2055, 2060, 2065, 2070, 2075, 2080, 2085,
        2090, 2095, 2100
    ], [
        0.19, 0.18, 0.17, 0.16, 0.15, 0.14, 0.13, 0.12, 0.11, 0.1, 0.09, 0.08, 0.07, 0.06, 0.05,
        0.04, 0.03, 0.02
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xuserxdefined_extraction_growth_unconv_oilx(x):
    """
    XUserXdefined extraction growth unconv oilX



    lookup


    """

    ret =  functions.lookup(x, [
        2015, 2020, 2025, 2030, 2035, 2040, 2045, 2050, 2055, 2060, 2065, 2070, 2075, 2080, 2085,
        2090, 2095, 2100
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def hist_growth_gtl():
    """
    Hist growth GTL



    constant


    """

    ret =  0.163
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('run')
def urr_uranium_user_defined():
    """
    URR uranium User defined



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('run')
def replacement_rate_nuclear():
    """
    replacement rate nuclear



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def demand_gas_for_oil_refinery_gains():
    """
    demand gas for oil refinery gains



    component


    """

    ret =  oil_refinery_gains_ej() * efficiency_gas_for_oil_refinery_gains()
    name = 2
    return round_sig(x=ret, name=name)

# TPES
# INDICATORS - Energy
@cache('step')
def share_res_vs_tpes():
    """
    share RES vs TPES



    component


    """

    ret =  tpe_from_res_ej() / tpes_ej()
    name = 2
    return round_sig(x=ret, name=name)

# TPES
@cache('step')
def tpes_mtoe():
    """
    TPES Mtoe



    component


    """

    ret =  tpes_ej() * mtoe_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES: Overcapacities due to RES variability a
# ELEC - Total FE electricity generation
# EROItot - Electricity
@cache('step')
def fe_elec_generation_from_nre_twh():
    """
    FE Elec generation from NRE TWh



    component


    """

    ret =  fe_elec_generation_from_fossil_fuels_twh() + fe_nuclear_elec_generation_twh()
    name = 2
    return round_sig(x=ret, name=name)

# ABUND - Abundances by sector
@cache('step')
def check_transp():
    """
    check Transp



    component


    """

    ret =  (pe_demand_transport_after_policies_ej() - pes_for_transp_ej()) / pes_for_transp_ej()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Traditional biomass
@cache('step')
def pepc_consumption_people_depending_on_trad_biomass():
    """
    PEpc consumption people depending on trad biomass



    component


    """

    ret =  pe_consumption_trad_biomass_ref() / people_relying_trad_biomass_ref()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def share_gas_for_oil_refinery_gains():
    """
    share gas for oil refinery gains



    component


    """

    ret =  functions.if_then_else(single_precision(xped_natx_gas_without_gtlx()) > single_precision(0) + less_thr_func(xped_natx_gas_without_gtlx(), 0),
        demand_gas_for_oil_refinery_gains() / xped_natx_gas_without_gtlx(), 0.5)
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Traditional biomass
@cache('run')
def people_relying_trad_biomass_ref():
    """
    People relying trad biomass ref



    constant


    """

    ret =  2500000000
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Traditional biomass
@cache('run')
def pe_consumption_trad_biomass_ref():
    """
    PE consumption trad biomass ref



    constant


    """

    ret =  39.626
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('run')
def tpefpc_threshold_high_development():
    """
    TPEFpc threshold high development



    constant


    """

    ret =  106
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('run')
def tped_acceptable_standard_living():
    """
    TPED acceptable standard living



    constant


    """

    ret =  40
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def pop_not_dependent_on_trad_biomass():
    """
    Pop not dependent on trad biomass



    component


    """

    ret =  population() - population_dependent_on_trad_biomass()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xcarbon_footprint_tco2xpersonx():
    """
    XCarbon footprint tCO2XpersonX



    component


    """

    ret =  total_co2_emissions_gtco2() * t_per_gt() / population()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xaverage_tpespc_xwithout_trad_biomassxx():
    """
    XAverage TPESpc Xwithout trad biomassXX



    component


    """

    ret =  xtpes_xwithout_trad_biomassxx() * gj_per_ej() / pop_not_dependent_on_trad_biomass()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xtpes_xwithout_trad_biomassxx():
    """
    XTPES Xwithout trad biomassXX



    component


    """

    ret =  tpes_ej() - pe_traditional_biomass_ej_delayed_1yr()
    name = 2
    return round_sig(x=ret, name=name)
# COMP - Historical data

def historic_electricity_consumption(x):
    """
    Historic electricity consumption



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
    ], [
        10395.0, 10600.0, 10664.0, 10871.0, 11122.0, 11486.0, 11817.0, 12132.0, 12443.0, 12707.0,
        13246.0, 13506.0, 13950.0, 14472.0, 15136.0, 15717.7218623, 16402.931668, 17186.6705223,
        17452.3423379, 17396.8323423, 18645.4399641, 19332.7421049, 19696.3720442, 20314.169915,
        20715.762225
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('step')
def max_npp_potential_bioe_residues_for_heat_and_elec():
    """
    Max NPP potential BioE residues for heat and elec



    component


    """

    ret =  max_npp_potential_bioe_residues() * (1 - share_cellulosic_biofuels_vs_bioe_residues())
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('step')
def max_npp_potential_bioe_residues_for_cellulosic_biofuels():
    """
    Max NPP potential BioE residues for cellulosic biofuels



    component


    """

    ret =  max_npp_potential_bioe_residues() * share_cellulosic_biofuels_vs_bioe_residues()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def total_carbon_emisions():
    """
    Total Carbon Emisions



    component


    """

    ret =  total_co2_emissions_gtco2() * c_per_co2() * t_per_gt()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def init_co2_in_atmos_ppm():
    """
    init CO2 in Atmos ppm



    constant


    """

    ret =  360.3
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def transfer_coefficient_for_lower_level():
    """
    Transfer coefficient for lower level



    constant


    """

    ret =  0.025
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def total_cumulative_emissions_gtco2():
    """
    Total cumulative emissions GtCO2



    component


    """

    ret =  total_cumulative_emissions_gtc() / c_per_co2()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def a_uo_heat_cap():
    """
    A UO Heat Cap



    component


    """

    ret =  1 / climate_equation_coefficient_for_upper_level()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def init_co2_in_atmos():
    """
    init CO2 in Atmos



    component


    """

    ret =  init_co2_in_atmos_ppm() * ppm_per_gtc() * t_per_gt()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def gtco2_historic_emissions_rcps():
    """
    GtCO2 historic emissions RCPs



    component


    """

    ret =  gtc_historic_emissions_rcps(time()) / c_per_co2()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def heat_trans_coeff():
    """
    Heat Trans Coeff



    component


    """

    ret =  1 / transfer_coefficient_for_lower_level()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def carbon_emissions_gtc():
    """
    Carbon emissions GtC



    component


    """

    ret =  total_co2_emissions_gtco2() * c_per_co2()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def climate_equation_coefficient_for_upper_level():
    """
    Climate equation coefficient for upper level



    constant


    """

    ret =  0.098
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction

def historic_unconv_oil(x):
    """
    Historic unconv oil



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
    ], [
        0.8042547447027982, 0.8410083449405368, 0.882574107072071, 0.9251795264521996,
        0.9725192749523464, 1.0602575075341525, 1.1056776788632727, 1.2903463301151419,
        1.4172025337438179, 1.4737016614421428, 2.6459843451365654, 2.7048717825313453,
        3.129671545010238, 3.4697200909205494, 3.9555142957287224, 3.9560890176996315,
        4.200681334685477, 4.568217289793923, 5.212758598196126, 5.557878988219476,
        6.383971244410107, 7.882661254058065, 10.105037981577686
    ])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gco2_per_mj_unconv_gas():
    """
    gCO2 per MJ unconv gas



    constant


    """

    ret =  56.1
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gco2_per_mj_unconv_oil():
    """
    gCO2 per MJ unconv oil



    constant


    """

    ret =  91.4
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def co2_emissions_ctl():
    """
    CO2 emissions CTL



    component


    """

    ret =  gco2_per_mj_ctl() * extraction_coal_for_ctl_ej() * mj_per_ej() / g_per_gt()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
@cache('run')
def gco2_per_mj_conv_gas():
    """
    gCO2 per MJ conv gas



    constant


    """

    ret =  56.1
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gco2_per_mj_ctl():
    """
    gCO2 per MJ CTL



    constant


    """

    ret =  165.2
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gco2_per_mj_gtl():
    """
    gCO2 per MJ GTL



    constant


    """

    ret =  103.3
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction
# RESOURCES - Natural gas extraction

def historic_unconv_gas(x):
    """
    Historic unconv gas



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
    ], [
        0.3529964919526898, 0.5158656374975538, 0.7149245438358863, 0.9159055601403124,
        1.0378448477267395, 1.1591242358421463, 1.2288537238849544, 1.3240656171041556,
        1.4884079785240143, 1.5300788574753446, 1.5807465736347022, 1.7830034338888128,
        1.8830435171379032, 2.005767665679057, 2.2213013634395504, 2.395469289580111,
        2.7316041027828772, 3.3313660906676272, 4.438876784314759, 5.42400344639208,
        7.802258325161671, 10.281798451970328, 12.20978412863096
    ])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gco2_per_mj_coal():
    """
    gCO2 per MJ coal



    constant


    """

    ret =  94.6
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gco2_per_mj_conv_oil():
    """
    gCO2 per MJ conv oil



    constant


    """

    ret =  73.3
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def gco2_per_mj_shale_oil():
    """
    gCO2 per MJ shale oil



    constant


    """

    ret =  146.1
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('run')
def technological_max_pct_of_change():
    """
    technological max pct of change



    constant


    """

    ret =  0.47
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('run')
def k_liquid_elec_bev_twh_ej():
    """
    K liquid Elec BEV TWh EJ



    constant


    """

    ret =  95
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def hev_fraction_of_share_available():
    """
    HEV fraction of share available



    component


    """

    ret =  functions.if_then_else(
        technological_max_pct_of_change() > total_share_ev(),
        (technological_max_pct_of_change() - total_share_ev()) / technological_max_pct_of_change(),
        0)
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def bev_fraction_of_share_available():
    """
    BEV fraction of share available



    component


    """

    ret =  functions.if_then_else(
        technological_max_pct_of_change() > total_share_ev(),
        (technological_max_pct_of_change() - total_share_ev()) / technological_max_pct_of_change(),
        0)
    name = 2
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# ECON- Final demands by sector, households and fuel
@cache('run')
def dollars_to_tdollars():
    """
    dollars to Tdollars



    constant


    """

    ret =  1000000000000
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def variation_historic_pop():
    """
    variation historic pop



    component


    """

    ret =  functions.if_then_else(time() < 2014, historic_pop(time() + 1) - historic_pop(time()),
                                  0)
    name = 2
    return round_sig(x=ret, name=name)

def historic_pop(x):
    """
    Historic pop



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        5705843054, 5788596142, 5871549366, 5953672500, 6034911639, 6115444311, 6195589560,
        6274734084, 6353976427, 6433748714, 6513959904, 6594722462, 6675832678, 6758302523,
        6840955706, 6923684085, 7006907989, 7089451551, 7176092192, 7260780278, 7346633037
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def land_compet_2gen_vs_total_land_compet():
    """
    land compet 2gen vs total land compet



    component


    """

    ret =  land_compet_biofuels_2gen_mha() / land_compet_required_dedicated_crops_for_biofuels()
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
# INDICATORS - Social and Environmental impacts
@cache('run')
def gj_per_ej():
    """
    GJ per EJ



    constant


    """

    ret =  1000000000
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def land_shifted_to_biofuels_3gen():
    """
    Land shifted to biofuels 3gen



    component


    """

    ret =  functions.if_then_else(
        time() < start_year_3gen(), 0,
        functions.if_then_else(
            time() < (start_year_3gen() + 5),
            annual_shift_from_2gen_to_3gen() * land_compet_biofuels_2gen_mha() *
            biofuels_3gen_land_compet_available() * land_compet_2gen_vs_total_land_compet(),
            p_biofuels_3gen() * land_compet_biofuels_3gen_mha() *
            biofuels_3gen_land_compet_available() * land_compet_2gen_vs_total_land_compet()))
    name = 2
    return round_sig(x=ret, name=name)

# INDICATORS - Energy
@cache('step')
def average_tpes_per_capita():
    """
    Average TPES per capita



    component


    """

    ret =  tpes_ej() * gj_per_ej() / population()
    name = 2
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
@cache('run')
def geot_pe_potential_for_heat_twth():
    """
    Geot PE potential for heat TWth



    constant


    """

    ret =  0.3
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# ELEC - Electricity related losses
@cache('step')
def pe_losses_uranium_for_elec_ej():
    """
    PE losses uranium for Elec EJ



    component


    """

    ret =  extraction_uranium_ej() * (1 - efficiency_uranium_for_electricity())
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def pe_demand_transport_after_policies_mtoe():
    """
    PE demand Transport after policies Mtoe



    component


    """

    ret =  pe_demand_transport_after_policies_ej() * mtoe_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

# RES - PES
@cache('step')
def share_res_for_elec_vs_tpe_res():
    """
    share RES for Elec vs TPE RES



    component


    """

    ret =  pe_elec_generation_from_res_ej() / (
        xpe_supply_res_nonxelec_ejx() + pe_elec_generation_from_res_ej())
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xtpes_de_castro_phd_x_scen_xmadcoalx_ejx():
    """
    XTPES de Castro PhD X Scen XmadcoalX EJX



    component


    """

    ret =  xtpes_de_castro_phd_x_scen_xmadcoalxx(time()) / mtoe_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def extraction_coal_mtoe():
    """
    extraction coal Mtoe



    component


    """

    ret =  extraction_coal_ej() * mtoe_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def max_extraction_coal_mtoe():
    """
    max extraction coal Mtoe



    component


    """

    ret =  max_extraction_coal_ej() * mtoe_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

# RES - PES
# TPES
# INDICATORS - Energy
# COMP - Historical data
@cache('step')
def tpe_from_res_ej():
    """
    TPE from RES EJ



    component


    """

    ret =  pe_elec_generation_from_res_ej() + xpe_supply_res_nonxelec_ejx()
    name = 2
    return round_sig(x=ret, name=name)

# RES - PES
@cache('step')
def tpe_from_res_mtoe():
    """
    TPE from RES Mtoe



    component


    """

    ret =  tpe_from_res_ej() * mtoe_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xtpes_de_castro_phd_x_scen_ii_ejx():
    """
    XTPES de Castro PhD X Scen II EJX



    component


    """

    ret =  xtpes_de_castro_phd_x_scen_iix(time()) / mtoe_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
# ELEC - Electricity related losses
# ENERGY DISTR LOSSES
@cache('step')
def electrical_distribution_losses_ej():
    """
    Electrical distribution losses EJ



    component


    """

    ret =  electrical_distribution_losses_twh() * ej_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
# HEAT RES-com - Capacities and generation all tech
@cache('run')
def efficiency_conversion_bioe_plants_to_heat():
    """
    Efficiency conversion BioE plants to heat



    constant


    """

    ret =  0.55
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('run')
def efficiency_improvement_biofuels_3gen():
    """
    Efficiency improvement biofuels 3gen



    constant


    """

    ret =  0.15
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Total biofuels liquids
@cache('step')
def potential_peavail_biofuels_2gen_land_compet_ej():
    """
    Potential PEavail biofuels 2gen land compet EJ



    component


    """

    ret =  land_compet_biofuels_2gen_mha() * land_productivity_biofuels_2gen_ej_mha()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def annual_additional_historic_land_use_biofuels_2gen():
    """
    Annual additional historic land use biofuels 2gen



    component


    """

    ret =  annual_additional_historic_product_biofuels_2gen() * ej_per_ktoe(
    ) / land_productivity_biofuels_2gen_ej_mha()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('run')
def max_npp_potential_bioe_residues():
    """
    Max NPP potential bioE residues



    constant


    """

    ret =  25
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('run')
def grid_reinforcement_costs():
    """
    Grid reinforcement costs



    constant


    """

    ret =  238.32929015926584
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def balancing_costs():
    """
    Balancing costs



    component


    """

    ret =  (balancing_costs_ref(share_variable_res_elec_generation_vs_total())) / m_per_t()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def historic_land_compet_available_for_biofuels_2gen():
    """
    Historic land compet available for biofuels 2gen



    component


    """

    ret =  74847.7 * ej_per_ktoe() / land_productivity_biofuels_2gen_ej_mha()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('run')
def land_productivity_biofuels_2gen_ej_mha():
    """
    Land productivity biofuels 2gen EJ MHa



    constant


    """

    ret =  0.02
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xmax_peavail_potential_biofuels_2x3genx():
    """
    XMax PEavail potential biofuels 2X3genX



    component


    """

    ret =  functions.if_then_else(
        time() < start_year_3gen(),
        max_land_compet_biofuels_2gen() * land_productivity_biofuels_2gen_ej_mha(),
        max_land_compet_biofuels_2gen() * land_productivity_biofuels_2gen_ej_mha() *
        (1 + efficiency_improvement_biofuels_3gen()))
    name = 2
    return round_sig(x=ret, name=name)
# ELEC RES - Total monetary investments

def balancing_costs_ref(x):
    """
    Balancing costs ref



    lookup


    """

    ret =  functions.lookup(x, [
        0.0, 0.1, 0.2, 0.30000000000000004, 0.4, 0.5, 0.6, 0.7, 0.7999999999999999,
        0.8999999999999999, 0.9999999999999999
    ], [
        0.0, 1.516461136385635, 3.03292227277127, 4.549383409156905, 4.549383409156905,
        4.549383409156905, 4.549383409156905, 4.549383409156905, 4.549383409156905,
        4.549383409156905, 4.549383409156905
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def land_compet_biofuels_2gen_mha():
    """
    Land compet biofuels 2gen Mha



    component


    """


    ret =  integ_land_compet_biofuels_2gen_mha()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# RES BIOE - Elec&Heat potential
@cache('run')
def efficiency_conversion_bioe_to_elec():
    """
    efficiency conversion bioE to Elec



    constant


    """

    ret =  0.35
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Total biofuels liquids
@cache('step')
def potential_peavail_biofuels_prod_3gen_ej():
    """
    Potential PEavail biofuels prod 3gen EJ



    component


    """

    ret =  land_compet_biofuels_3gen_mha() * land_productivity_biofuels_2gen_ej_mha() * (
        1 + efficiency_improvement_biofuels_3gen())
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
# RES BIOE - Total biofuels liquids
@cache('step')
def max_peavail_potential_bioe_residues_for_cellulosic_biofuels():
    """
    Max PEavail potential bioE residues for cellulosic biofuels



    component


    """

    ret =  max_npp_potential_bioe_residues_for_cellulosic_biofuels(
    ) * efficiency_bioe_residues_to_cellulosic_liquids()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('run')
def p_biofuels_3gen():
    """
    P biofuels 3gen



    constant


    """

    ret =  0.05
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xconstrain_gas_exogenous_growthx_delayed_1yrx():
    """
    Xconstrain gas exogenous growthX delayed 1yrX



    component


    """

    ret =  delay_xconstrain_gas_exogenous_growthxx_round1__time_step___time_step_1_1__time_step()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xconstrain_gas_exogenous_growthxx():
    """
    Xconstrain gas exogenous growthXX



    component


    """

    ret =  functions.if_then_else(single_precision(check_gases()) > single_precision(-0.01) + less_thr_func(check_gases(), -0.01), 1, check_gases())
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xconstrain_liquids_exogenous_growthx_delayed_1yrx():
    """
    Xconstrain liquids exogenous growthX delayed 1yrX



    component


    """

    ret =  delay_xconstrain_liquids_exogenous_growthxx_round1__time_step___time_step_1_1__time_step(
    )
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def check_liquids_delayed_1yr():
    """
    check liquids delayed 1yr



    component


    """

    ret =  delay_check_liquids_round1__time_step___time_step_1_1__time_step()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def check_gas_delayed_1yr():
    """
    check gas delayed 1yr



    component


    """

    ret =  delay_check_gases_round1__time_step___time_step_1_1__time_step()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xconstrain_liquids_exogenous_growthxx():
    """
    Xconstrain liquids exogenous growthXX



    component


    """

    ret =  functions.if_then_else(single_precision(check_liquids()) > single_precision(0.00121114) + less_thr_func(check_liquids(), 0.00121114), 1, check_liquids())
    name = 2
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def check_hist_res():
    """
    check hist RES



    component


    """

    ret =  functions.if_then_else(time() < 2015,
                                  (tpe_from_res_ej() - historic_total_res_extraction(time())) * 100
                                  / historic_total_res_extraction(time()), 0)
    name = 2
    return round_sig(x=ret, name=name)
# COMP - Historical data

def historic_total_res_extraction(x):
    """
    Historic total RES extraction



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
    ], [
        46.944247770000004, 47.749732359999996, 48.48870097, 49.09046492, 49.60896682,
        50.726245590000005, 51.55587169, 52.09233801, 52.67941864, 53.41530317, 54.14254115999999,
        54.09579433, 55.02510847, 56.379325789999996, 58.27609373, 60.03002464, 61.697667020000004,
        63.54365126, 65.52607556, 66.93695557, 69.9659654, 71.50804720000001, 74.53703940999999,
        77.32016644, 79.29877121
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Elec&Heat potential
# HEAT RES - Potential
@cache('run')
def max_potential_npp_bioe_conventional_for_heat():
    """
    Max potential NPP bioE conventional for heat



    constant


    """

    ret =  25
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES: Overcapacities due to RES variability a
@cache('step')
def share_variable_res_elec_vs_total_generation_delayed_1yr():
    """
    Share variable RES elec vs total generation delayed 1yr



    component


    """

    ret =  delay_share_variable_res_elec_generation_vs_total_round1__time_step___time_step_00071_1__time_step(
    )
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xovercapacity_vsx_intermittent_res_penetration_0x():
    """
    Xovercapacity vsX intermittent RES penetration 0X



    component


    """

    ret =  np.maximum(
        1, 0.9599 * np.exp(0.8938 * share_variable_res_elec_vs_total_generation_delayed_1yr()))
    name = 2
    return round_sig(x=ret, name=name)

def xtotal_ghg_emissions_excluding_landxuse_change_and_forestryx(x):
    """
    XTotal GHG Emissions Excluding LandXUse Change and ForestryX



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
    ], [
        30.42374529, 30.648470539999998, 30.5871651, 30.69306597, 30.90004943, 31.558604589999998,
        32.22670639, 32.50772787, 32.70006585, 32.970280429999995, 33.88661512, 34.25715087,
        34.85206885, 36.15210262, 37.60734796, 38.78192774, 39.88996925, 41.13507125, 41.52499632,
        41.23689253, 42.96809055, 44.19065258, 44.81553514, 0.0, 0.0
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xtotal_ghg_emissions_including_landxuse_change_and_forestryx(x):
    """
    XTotal GHG Emissions Including LandXUse Change and ForestryX



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
    ], [
        33.93721432, 33.280597629999995, 33.29526431, 33.408575479999996, 33.62893115, 34.30997387,
        34.98005164, 35.41270247, 35.68394669, 35.828676019999996, 36.679717360000005,
        37.36603727999999, 38.12323356, 39.449471089999996, 40.84106103, 42.00097215, 42.77940812,
        43.91581001, 44.37242347, 44.00255874, 45.747709900000004, 46.90571681, 47.59855416, 0.0,
        0.0
    ])
    name = 2
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def check_hist_emissions():
    """
    check hist emissions



    component


    """

    ret =  functions.if_then_else(
        time() < 2014,
        ((total_co2_emissions_gtco2() - xco2_landxuse_change_emissionsx(time())) -
         total_co2_emissions_from_fossil_fuel_consumption_and_cement_production(time())) * 100 /
        total_co2_emissions_from_fossil_fuel_consumption_and_cement_production(time()), 0)
    name = 2
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def check_hist_liquids():
    """
    check hist liquids



    component


    """

    ret =  functions.if_then_else(time() < 2015,
                                  (pes_liquids_ej() - historic_liquids_extraction(time())) * 100 /
                                  historic_liquids_extraction(time()), 0)
    name = 2
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def check_hist_natural_gas():
    """
    check hist natural gas



    component


    """

    ret =  functions.if_then_else(time() < 2015,
                                  (xpes_natx_gasx() - historic_natural_gas_extraction(time())) *
                                  100 / historic_natural_gas_extraction(time()), 0)
    name = 2
    return round_sig(x=ret, name=name)
# COMP - Historical data

def total_co2_emissions_from_fossil_fuel_consumption_and_cement_production(x):
    """
    Total CO2 emissions from fossil fuel consumption and cement production



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
    ], [
        22.352, 22.627, 22.403333333333332, 22.381333333333334, 22.762666666666668,
        23.261333333333333, 23.800333333333334, 24.159666666666666, 24.093666666666667,
        24.049666666666667, 24.665666666666667, 25.248666666666665, 25.468666666666667,
        27.012333333333334, 28.361666666666668, 29.425, 30.459, 31.122666666666667,
        32.03933333333333, 31.683666666666667, 33.50233333333333, 34.86266666666667,
        35.46033333333333, 35.845333333333336
    ])
    name = 2
    return round_sig(x=ret, name=name)

# COMP - Historical data
@cache('step')
def check_hist_coal():
    """
    check hist coal



    component


    """

    ret =  functions.if_then_else(time() < 2015,
                                  (extraction_coal_ej() - historic_coal_extraction(time())) * 100 /
                                  historic_coal_extraction(time()), 0)
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
# ELEC - Electricity related losses
@cache('step')
def total_electrical_losses_ej():
    """
    Total electrical losses EJ



    component


    """

    ret =  elec_gen_related_losses_ej() + electrical_distribution_losses_ej()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
@cache('step')
def gen_losses_vs_pe_for_elec():
    """
    Gen losses vs PE for elec



    component


    """

    ret =  elec_gen_related_losses_ej() / total_pe_for_electricity_consumption_ej()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def initial_gtl_production():
    """
    initial GTL production



    constant


    """

    ret =  0.064206
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def variation_ctl():
    """
    variation CTL



    component


    """

    ret =  functions.if_then_else(
        time() < 2013,
        historic_ctl_production(time() + 1) - historic_ctl_production(time()),
        functions.if_then_else(
            check_liquids() > equality_threshold,
            xconstrain_liquids_exogenous_growthxx() * ctl_potential_production(),
            ctl_potential_production() * real_growth_ctl()))
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('step')
def variation_gtl():
    """
    variation GTL



    component


    """

    ret =  functions.if_then_else(
        time() < 2013,
        historic_gtl_production(time() + 1) - historic_gtl_production(time()),
        functions.if_then_else(
            check_liquids() > equality_threshold,
            xconstrain_liquids_exogenous_growthxx() * gtl_potential_production(),
            gtl_potential_production() * real_growth_gtl()))
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - CTL & GTL supply

def historic_gtl_production(x):
    """
    Historic GTL production



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.06346222, 0.058381449999999994, 0.06496829, 0.06506953, 0.06767736, 0.064206, 0.05606117,
        0.04747241000000001, 0.025892259999999997, 0.033874730000000006, 0.03802368, 0.06179034,
        0.058625540000000004, 0.042497019999999996, 0.08330459, 0.10185614, 0.10121208, 0.13211313,
        0.15293263, 0.14861201999999998, 0.15202329, 0.14950447, 0.24690301, 0.29162875,
        0.31618224, 0.0
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def initial_ctl_production():
    """
    initial CTL production



    constant


    """

    ret =  0.26293196
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - CTL & GTL supply

def historic_ctl_production(x):
    """
    Historic CTL production



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.16499057, 0.16768147, 0.21825412, 0.24450376000000001, 0.24462182999999998, 0.26293196,
        0.27309483, 0.27772338, 0.26207404, 0.26722632, 0.30062303999999995, 0.15679415,
        0.16186449, 0.16539379999999998, 0.13935132, 0.10021173, 0.14281665, 0.11917675999999999,
        0.09999644, 0.11247093, 0.14480621, 0.2200387, 0.22243912, 0.2406164, 0.27127102000000003,
        0.0
    ])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('step')
def improvement_efficiency_gas_for_electricity():
    """
    improvement efficiency gas for electricity



    component


    """

    ret =  functions.if_then_else(
        time() < 2013, (historic_efficiency_gas_for_electricity(time() + 1) -
                        historic_efficiency_gas_for_electricity(time())) * percent_to_share(),
        efficiency_gas_for_electricity() * remaining_efficiency_improv_gas_for_electricity() *
        efficiency_improv_gas_for_electricity())
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# GASES - PED, PES & FES
# COMP- Historical data II
@cache('step')
def pe_demand_gas_elec_plants_ej():
    """
    PE demand gas Elec plants EJ



    component


    """

    ret =  (fe_demand_gas_elec_plants_twh() / efficiency_gas_for_electricity()) * ej_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# SOLIDS - PED, PES & FES
# COMP- Historical data II
@cache('step')
def pe_demand_coal_elec_plants_ej():
    """
    PE demand coal Elec plants EJ



    component


    """

    ret =  (fe_demand_coal_elec_plants_twh() / efficiency_coal_for_electricity()) * ej_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('run')
def max_efficiency_gas_power_plants():
    """
    Max efficiency gas power plants



    constant


    """

    ret =  0.6
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity related losses
# TPED by sector
@cache('step')
def total_pe_for_electricity_consumption_ej():
    """
    Total PE for electricity consumption EJ



    component


    """

    ret =  total_fe_elec_demand_ej() + elec_gen_related_losses_ej()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('run')
def initial_efficiency_gas_for_electricity():
    """
    initial efficiency gas for electricity



    constant


    """

    ret =  37.813
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('step')
def remaining_efficiency_improv_gas_for_electricity():
    """
    remaining efficiency improv gas for electricity



    component


    """

    ret =  (max_efficiency_gas_power_plants() - efficiency_gas_for_electricity()
            ) / max_efficiency_gas_power_plants()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('run')
def efficiency_improv_gas_for_electricity():
    """
    Efficiency improv gas for electricity



    constant


    """

    ret =  0.05
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# ELEC - Electricity related losses
# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
# ELEC - Total FE electricity generation
@cache('step')
def efficiency_gas_for_electricity():
    """
    efficiency gas for electricity



    component


    """


    ret =  integ_efficiency_gas_for_electricity()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
@cache('run')
def percent_to_share():
    """
    percent to share



    constant


    """

    ret =  0.01
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xadditional_pe_production_of_ctlxgtl_for_liquidsx():
    """
    XAdditional PE production of CTLXGTL for liquidsX



    component


    """

    ret =  ped_coal_for_ctl_ej() + xped_natx_gas_for_gtl_ejx() - xfes_ctlxgtl_ejx()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def annual_additional_historic_product_biofuels_2gen():
    """
    Annual additional historic product biofuels 2gen



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        historic_produc_biofuels_2gen(time() + 1) - historic_produc_biofuels_2gen(time()), 0)
    name = 2
    return round_sig(x=ret, name=name)
# RES BIOE - Dedicated crops for biofuels (compet)

def historic_produc_biofuels_2gen(x):
    """
    Historic produc biofuels 2gen



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        7096.883524626101, 7971.694528739449, 7707.278339633647, 7846.760877745217,
        8705.079406958428, 9052.844108405054, 9171.927311788611, 10472.882598545473,
        9937.08737060958, 9619.477252460889, 9178.36595626787, 10024.948550755016,
        11835.252519246962, 14686.710467070612, 16445.720935966816, 19650.71469184889,
        25666.356544617964, 34701.5349030663, 46454.13782417379, 51905.07940132504,
        59604.69080832337, 61162.8030389672, 62278.46999420314, 67260.47281761839,
        74207.51498164036, 74847.36659353007
    ])
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def ngv_adapt_growth():
    """
    NGV adapt growth



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        ngv_past_share_growth(),
        functions.if_then_else(
            time() < 2020,
            ngv_past_share_growth() +
            (p_ngv_growth() * effects_shortage_gas() - ngv_past_share_growth()) *
            (time_dmnl(time()) - 2015) / 5, p_ngv_growth() * effects_shortage_gas()))
    name = 2
    return round_sig(x=ret, name=name)
# TRANSP - Alternative transport

def historical_share_ngvs(x):
    """
    Historical share NGVs



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0045651,
        0.005616333333333339, 0.00696102702702704, 0.008811342105263183, 0.010107012820512858,
        0.01110156249999998, 0.012934378048780476, 0.013861333333333335, 0.0, 0.0, 0.0
    ])
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def variation_share_ngv():
    """
    variation share NGV



    component


    """

    ret =  functions.if_then_else(
        time() < 2011, (historical_share_ngvs(time() + 1) - historical_share_ngvs(time())),
        ngv_adapt_growth() * ngv_fraction_of_change_available() * ngv_share())
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def hev_adapt_growth():
    """
    HEV adapt growth



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        xbevxhev_past_share_growthx(),
        functions.if_then_else(
            time() < 2020,
            xbevxhev_past_share_growthx() +
            (p_hev_growth() * xeffects_shortage_elec_on_bevxhevx() - xbevxhev_past_share_growthx())
            * (time_dmnl(time()) - 2015) / 5,
            p_hev_growth() * xeffects_shortage_elec_on_bevxhevx()))
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def bev_adapt_growth():
    """
    BEV adapt growth



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        xbevxhev_past_share_growthx(),
        functions.if_then_else(
            time() < 2020,
            xbevxhev_past_share_growthx() +
            (p_bev_growth() * xeffects_shortage_elec_on_bevxhevx() - xbevxhev_past_share_growthx())
            * (time_dmnl(time()) - 2015) / 5,
            p_bev_growth() * xeffects_shortage_elec_on_bevxhevx()))
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def carbon_budget():
    """
    carbon budget



    constant


    """

    ret =  1000
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def adapt_co2_emissions_unconv_gas():
    """
    Adapt CO2 emissions unconv gas



    component


    """

    ret =  functions.if_then_else(time() < 2050,
                                  0.01 + (0.22 - 0.01) * (time_dmnl(time()) - 2000) / 50,
                                  functions.if_then_else(time() < 2100, 0.22 + (0.6 - 0.22) *
                                                         (time_dmnl(time()) - 2050) / 50, 0.6))
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def cumulative_emissions_to_1995():
    """
    Cumulative emissions to 1995



    constant


    """

    ret =  378.66
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def variation_share_bev():
    """
    variation share BEV



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (xhistorical_share_bevxhevx(time() + 1) - xhistorical_share_bevxhevx(time())) / 2,
        bev_adapt_growth() * bev_fraction_of_share_available() * bev_share())
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def variation_share_hev():
    """
    variation share HEV



    component


    """

    ret =  functions.if_then_else(
        time() < 2015,
        (xhistorical_share_bevxhevx(time() + 1) - xhistorical_share_bevxhevx(time())) / 2,
        hev_share() * hev_fraction_of_share_available() * hev_adapt_growth())
    name = 2
    return round_sig(x=ret, name=name)

def xhistorical_share_bevxhevx(x):
    """
    XHistorical share BEVXHEVX



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.67e-06,
        1.7305555555555575e-06, 1.6932432432432467e-06, 3.721052631578958e-06,
        5.285897435897455e-06, 1.091999999999998e-05, 5.177439024390239e-05,
        0.00014935833333333334, 0.0003082333333333328, 0.0005497099999999982, 0.000940860215053759
    ])
    name = 2
    return round_sig(x=ret, name=name)

# HEAT RES - Potential
@cache('run')
def fe_solar_potential_for_heat():
    """
    FE solar potential for heat



    constant


    """

    ret =  22
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xstart_year_bioe_residues_for_heatxelecx():
    """
    Xstart year BioE residues for heatXelecX



    constant


    """

    ret =  2020
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def co2_fossil_fuel_emissions():
    """
    CO2 fossil fuel emissions



    component


    """

    ret =  co2_emissions_conv_gas_without_gtl() + co2_emissions_unconv_gas() + co2_emissions_gtl(
    ) + co2_emissions_conv_oil() + co2_emissions_unconv_oil() + co2_emissions_coal_without_ctl(
    ) + co2_emissions_ctl()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
# INDICATORS - Social and Environmental impacts
@cache('run')
def t_per_gt():
    """
    t per Gt



    constant


    """

    ret =  1000000000
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
# TRANSP - Alternative transport
@cache('step')
def additional_increase_pe_demand_for_transp_elec_ej():
    """
    Additional increase PE demand for Transp Elec EJ



    component


    """

    ret =  increase_elec_demand_for_transp_ej() + pe_losses_related_to_elec_transp()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def temp_change_2c():
    """
    Temp change 2C



    constant


    """

    ret =  2
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
# ABUND - Abundances by sector
@cache('step')
def pe_demand_transport_after_policies_ej():
    """
    PE demand Transport after policies EJ



    component


    """

    ret =  demand_gas_for_transp_ej() + real_pe_demand_of_liquids_for_transp_ej()
    name = 2
    return round_sig(x=ret, name=name)

# ABUND - Abundances by sector
@cache('step')
def pe_liquids_supply_for_transp_ej():
    """
    PE liquids supply for Transp EJ



    component


    """

    ret =  pes_liquids_ej() * share_liquids_for_transp()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('step')
def share_land_compet_biofuels():
    """
    Share land compet biofuels



    component


    """

    ret =  land_compet_required_dedicated_crops_for_biofuels() / global_arable_land()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('step')
def share_land_total_res_vs_arable():
    """
    share land total RES vs arable



    component


    """

    ret =  total_land_requirements_renew_mha() / global_arable_land()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('step')
def share_land_res_land_compet_vs_arable():
    """
    share land RES land compet vs arable



    component


    """

    ret =  (land_compet_required_dedicated_crops_for_biofuels() + surface_solar_pv_mha()
            ) / global_arable_land()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def year_scarcity_oil():
    """
    Year scarcity oil



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_total_oil()) > single_precision(0.95) + less_thr_func(abundance_total_oil(), 0.95), 0, time())
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def co2_net_emiss():
    """
    CO2 Net Emiss



    component


    """

    ret =  marginal_atmos_retention() * total_carbon_emisions()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xshare_gasxxcoal_xgasx_for_elecx():
    """
    Xshare gasXXcoal XgasX for ElecX



    component


    """

    ret =  functions.if_then_else(time() > 2014,
                                  xfuture_share_gasxxcoalxgasx_for_elecx(),
                                  xhist_share_gasxxcoal_xgasx_elecx(time()))
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xfuture_share_gasxxcoalxgasx_for_elecx():
    """
    Xfuture share gasXXcoalXgasX for ElecX



    constant


    """

    ret =  0.3
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def cumulated_conv_gas_extraction():
    """
    cumulated conv gas extraction



    component


    """


    ret =  integ_cumulated_conv_gas_extraction()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
# RESOURCES - Natural gas extraction
@cache('run')
def cumulated_conv_gas_extraction_to_1995():
    """
    cumulated conv gas extraction to 1995



    constant


    """

    ret =  1470
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def cumulated_unconv_gas_extraction():
    """
    Cumulated unconv gas extraction



    component


    """


    ret =  integ_cumulated_unconv_gas_extraction()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
# RESOURCES - Natural gas extraction
@cache('run')
def cumulated_unconv_gas_extraction_to_1995():
    """
    cumulated unconv gas extraction to 1995



    constant


    """

    ret =  6
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def marginal_atmos_retention():
    """
    Marginal Atmos Retention



    constant


    """

    ret =  0.64
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('step')
def rurr_uranium():
    """
    RURR uranium



    component


    """


    ret =  integ_rurr_uranium()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('run')
def cumulated_uranium_extraction_to_1995():
    """
    cumulated uranium extraction to 1995



    constant


    """

    ret =  814
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def cumulated_conv_oil_extraction():
    """
    cumulated conv oil extraction



    component


    """


    ret =  integ_cumulated_conv_oil_extraction()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
@cache('run')
def cumulated_conv_oil_extraction_to_1995():
    """
    cumulated conv oil extraction to 1995



    constant


    """

    ret =  4636
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def cumulated_unconv_oil_extraction():
    """
    cumulated unconv oil extraction



    component


    """


    ret =  integ_cumulated_unconv_oil_extraction()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def cumulated_coal_extraction():
    """
    Cumulated coal extraction



    component


    """


    ret =  integ_cumulated_coal_extraction()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('run')
def cumulated_coal_extraction_to_1995():
    """
    cumulated coal extraction to 1995



    constant


    """

    ret =  5018
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
@cache('run')
def cumulated_unconv_oil_extraction_to_1995():
    """
    cumulated unconv oil extraction to 1995



    constant


    """

    ret =  13.5
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('step')
def cumulated_uranium_extraction():
    """
    Cumulated uranium extraction



    component


    """


    ret =  integ_cumulated_uranium_extraction()
    name = 2
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
# LIQUIDS - PED, PES & FES
# RESOURCES - Oil extraction
# RESOURCES - Natural gas extraction
@cache('step')
def oil_refinery_gains_ej():
    """
    Oil refinery gains EJ



    component


    """

    ret =  oil_refinery_gains_share() * pes_oil_ej_delayed()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def oil_refinery_gains_share():
    """
    Oil refinery gains share



    constant


    """

    ret =  0.027
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def efficiency_gas_for_oil_refinery_gains():
    """
    Efficiency gas for oil refinery gains



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# DEM - Adjustment non-commercial heat demand
# HEAT - Total FE heat generation
# LIQUIDS - PED, PES & FES
# LIQUIDS - PED, PES & FES
# ABUND - Abundances by sector
# COMP - Historical data
@cache('step')
def pes_liquids_ej():
    """
    PES Liquids EJ



    component


    """

    ret =  pes_oil_ej() + other_liquids_supply_ej()
    name = 2
    return round_sig(x=ret, name=name)

def xco2_landxuse_change_emissionsx(x):
    """
    XCO2 landXuse change emissionsX



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3
    ])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
# IMPACTS CC - Energy losses function
@cache('step')
def temperature_change():
    """
    Temperature change



    component


    """

    ret =  atmos_uocean_temp()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def temp_diff():
    """
    Temp Diff



    component


    """

    ret =  atmos_uocean_temp() - deep_ocean_temp()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def co2_rad_forcing():
    """
    CO2 Rad Forcing



    component


    """

    ret =  co2_rad_force_coeff() * functions.log(co2_in_atmos() / preindustrial_co2(), 2)
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def atmos_uocean_temp():
    """
    Atmos UOcean Temp



    component


    """


    ret =  integ_atmos_uocean_temp()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def chg_a_uo_temp():
    """
    Chg A UO Temp



    component


    """

    ret =  (total_radiative_forcing() - feedback_cooling() - heat_transfer()) / a_uo_heat_cap()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def heat_transfer():
    """
    Heat Transfer



    component


    """

    ret =  temp_diff() * do_heat_cap() / heat_trans_coeff()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def deep_ocean_temp():
    """
    Deep Ocean Temp



    component


    """


    ret =  integ_deep_ocean_temp()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def chg_do_temp():
    """
    Chg DO Temp



    component


    """

    ret =  heat_transfer() / do_heat_cap()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def climate_sensitivity():
    """
    Climate Sensitivity



    constant


    """

    ret =  2.9
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def do_heat_cap():
    """
    DO Heat Cap



    component


    """

    ret =  heat_capacity_ratio() * heat_trans_coeff()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def init_atmos_uocean_temp():
    """
    init Atmos UOcean Temp



    constant


    """

    ret =  0.35
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def co2_rad_force_coeff():
    """
    CO2 Rad Force Coeff



    constant


    """

    ret =  3.8
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def heat_capacity_ratio():
    """
    Heat Capacity Ratio



    constant


    """

    ret =  0.088
    name = 2
    return round_sig(x=ret, name=name)
# IMPACTS - Carbon cycle & climate

def other_ghg_rad_forcing(x):
    """
    Other GHG Rad Forcing



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        0.011904761904761904, 0.023809523809523808, 0.03571428571428571, 0.047619047619047616,
        0.05952380952380952, 0.07142857142857142, 0.08333333333333333, 0.09523809523809523,
        0.10714285714285714, 0.11904761904761904, 0.13095238095238093, 0.14285714285714285,
        0.15476190476190477, 0.16666666666666669, 0.1785714285714286, 0.19047619047619052,
        0.20238095238095244, 0.21428571428571436, 0.22619047619047628, 0.2380952380952382, 0.25,
        0.25481121912286414, 0.25971502956352116, 0.2647132132304461, 0.2698075863247226,
        0.2749999999999999, 0.2798275087232865, 0.2847397623210218, 0.28973824845364465,
        0.2948244808968437, 0.29999999999999993, 0.3048412094224786, 0.3097605432071983,
        0.31475926207548766, 0.3198386470933953, 0.32499999999999996, 0.3298528931572529,
        0.3347782496129542, 0.3397771513875506, 0.34485069665819534, 0.34999999999999987,
        0.35486297490513247, 0.3597935170243449, 0.36479256515097, 0.36986107112213606,
        0.37500000000000006, 0.3798717630840887, 0.384806817036304, 0.389805984091619,
        0.39487009716696403, 0.4000000000000001, 0.4048794916999771, 0.40981850699807953,
        0.41481777200656483, 0.41987802169533733, 0.42500000000000027, 0.4298863415159332,
        0.4348288626398904, 0.4398282092842273, 0.44488503478753066, 0.45000000000000023,
        0.45489245438265813, 0.4598381001206191, 0.4648375155167262, 0.4698912851612009,
        0.47500000000000014, 0.4798979431760198, 0.48484639129384055, 0.4898458651331151,
        0.4948968908434944, 0.5000000000000002, 0.5049028988367429, 0.5098538745074922,
        0.5148533984439553, 0.5199019467006031, 0.5250000000000004, 0.5299073954525241,
        0.5348606623910048, 0.5398602295954837, 0.544906529853989, 0.5500000000000003,
        0.5549114939895983, 0.5598668475668505, 0.5648664523973954, 0.5699107036444414, 0.575,
        0.5799152450747611, 0.5848725069045569, 0.5898721446592106, 0.5949145205788192, 0.6,
        0.6049186910982999, 0.6098777047334673, 0.6148773714623407, 0.6199180245516037,
        0.6249999999999998, 0.6299218678236823, 0.634882495299963, 0.639882187660661,
        0.6449212525412924, 0.6499999999999997, 0.6549248056339129, 0.6598869246686443,
        0.664886639813804, 0.6699242359209866, 0.6749999999999999, 0.6799275305067678,
        0.6848910322089359, 0.6898907676978563, 0.6949270014818099, 0.7
    ])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def one_year():
    """
    one year



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def init_deep_ocean_temp():
    """
    init Deep Ocean Temp



    constant


    """

    ret =  0.0068
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def co2_storage():
    """
    CO2 Storage



    component


    """

    ret =  (co2_in_atmos() - preindustrial_co2()) * rate_of_co2_transfer()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def preindustrial_co2():
    """
    Preindustrial CO2



    constant


    """

    ret =  588000000000
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def average_atmos_retention():
    """
    Average Atmos Retention



    component


    """

    ret =  (co2_net_emiss() - co2_storage()) / total_carbon_emisions()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('step')
def co2_in_atmos():
    """
    CO2 in Atmos



    component


    """


    ret =  integ_co2_in_atmos()
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
# IMPACTS CC - Energy losses function
@cache('step')
def co2_ppm_concentrations():
    """
    CO2 ppm concentrations



    component


    """

    ret =  co2_in_atmos() / (2.13 * 1e+009)
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def rate_of_co2_transfer():
    """
    Rate of CO2 Transfer



    constant


    """

    ret =  0.008333
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# IMPACTS - RES Land-use
@cache('step')
def land_compet_required_dedicated_crops_for_biofuels():
    """
    Land compet required dedicated crops for biofuels



    component


    """

    ret =  land_compet_biofuels_2gen_mha() + land_compet_biofuels_3gen_mha()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def new_biofuels_2gen_land_compet():
    """
    new biofuels 2gen land compet



    component


    """

    ret =  functions.if_then_else(
        check_liquids() > equality_threshold,
        xconstrain_liquids_exogenous_growthxx() * land_compet_biofuels_2gen_mha(),
        np.maximum(
            annual_additional_historic_land_use_biofuels_2gen() + adapt_growth_biofuels_2gen() *
            land_compet_biofuels_2gen_mha() * biofuels_land_compet_available(), 0))
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('run')
def annual_shift_from_2gen_to_3gen():
    """
    Annual shift from 2gen to 3gen



    constant


    """

    ret =  0.1
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def land_compet_biofuels_3gen_mha():
    """
    Land compet biofuels 3gen Mha



    component


    """


    ret =  integ_land_compet_biofuels_3gen_mha()
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xp_bioe_residues_for_heatxelecx():
    """
    XP bioE residues for heatXelecX



    constant


    """

    ret =  0.05
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('step')
def potential_pe_cellulosic_biofuel_ej():
    """
    Potential PE cellulosic biofuel EJ



    component


    """


    ret =  integ_potential_pe_cellulosic_biofuel_ej()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('step')
def cellulosic_biofuels_available():
    """
    Cellulosic biofuels available



    component


    """

    #ret =  (max_npp_potential_bioe_residues_for_cellulosic_biofuels() -
    #        potential_pe_cellulosic_biofuel_ej()
    #        ) / max_npp_potential_bioe_residues_for_cellulosic_biofuels()
    ret = functions.zidz(max_npp_potential_bioe_residues_for_cellulosic_biofuels() -
            potential_pe_cellulosic_biofuel_ej(), max_npp_potential_bioe_residues_for_cellulosic_biofuels())

    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xbioe_residues_for_heatxelec_availablex():
    """
    XBioE residues for heatXelec availableX



    component


    """

    ret =  (max_npp_potential_bioe_residues_for_heat_and_elec() -
            xpe_bioe_residues_for_heatxelec_ejx()
            ) / max_npp_potential_bioe_residues_for_heat_and_elec()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xpe_bioe_residues_for_heatxelec_ejx():
    """
    XPE bioE residues for heatXelec EJX



    component


    """


    ret =  integ_xpe_bioe_residues_for_heatxelec_ejx()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('run')
def p_cellulosic_biofuels():
    """
    P cellulosic biofuels



    constant


    """

    ret =  0.08
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('run')
def share_cellulosic_biofuels_vs_bioe_residues():
    """
    share cellulosic biofuels vs BioE residues



    constant


    """

    ret =  0.25
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Crop and forest residues
@cache('run')
def start_year_cellulosic_biofuels():
    """
    start year cellulosic biofuels



    constant


    """

    ret =  2025
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('step')
def share_land_total_res_vs_urban_surface():
    """
    share land total RES vs urban surface



    component


    """

    ret =  total_land_requirements_renew_mha() / urban_surface_2008()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def max_land_compet_biofuels_2gen():
    """
    Max land compet biofuels 2gen



    component


    """

    ret =  additional_land_compet_available_for_biofuels(
    ) + historic_land_compet_available_for_biofuels_2gen()
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('run')
def p_biofuels_2gen():
    """
    P biofuels 2gen



    constant


    """

    ret =  0.05
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('run')
def past_biofuels_2gen():
    """
    past biofuels 2gen



    constant


    """

    ret =  0.11139790590538201
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Dedicated crops for biofuels (marg)
@cache('step')
def adapt_growth_biofuels_2gen():
    """
    adapt growth biofuels 2gen



    component


    """

    ret =  functions.if_then_else(
        time() < 2015, 0,
        functions.if_then_else(time() < 2020,
                               past_biofuels_2gen() + (p_biofuels_2gen() - past_biofuels_2gen()) *
                               (time_dmnl(time()) - 2015) / 5, p_biofuels_2gen()))
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Dedicated crops for biofuels (marg)
# RES BIOE - Crop and forest residues
# RES BIOE - Crop and forest residues
@cache('run')
def ej_per_ktoe():
    """
    EJ per ktoe



    constant


    """

    ret =  4.1868e-05
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def abundance_total_oil():
    """
    abundance total oil



    component


    """

    ret =  functions.if_then_else(single_precision(pes_oil_ej()) > single_precision(ped_total_oil_ej()) + less_thr_func(pes_oil_ej(), ped_total_oil_ej()), 1, 1 - functions.zidz(
        (ped_total_oil_ej() - pes_oil_ej()), ped_total_oil_ej()))
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('run')
def cp_nuclear_initial():
    """
    Cp nuclear initial



    constant


    """

    ret =  0.7652
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def extraction_coal_without_ctl_ej():
    """
    extraction coal without CTL EJ



    component


    """

    ret =  np.maximum(extraction_coal_ej() - extraction_coal_for_ctl_ej(), 0)
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
# IMPACTS - Total GHG emissions
@cache('step')
def extraction_coal_for_ctl_ej():
    """
    extraction coal for CTL EJ



    component


    """

    ret =  ped_coal_for_ctl_ej()
    name = 2
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
@cache('step')
def other_liquids_supply_ej():
    """
    Other liquids supply EJ



    component


    """

    ret =  oil_refinery_gains_ej() + xfes_ctlxgtl_ejx() + fes_total_biofuels_production_ej()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('run')
def max_offshore_wind_twe():
    """
    max offshore wind TWe



    constant


    """

    ret =  0.25
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_unconv_oil_high_mohr15():
    """
    URR unconv oil High Mohr15



    constant


    """

    ret =  22000
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_unconv_oil_low_mohr15():
    """
    URR unconv oil Low Mohr15



    constant


    """

    ret =  6000
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_unconv_oil_user_defined():
    """
    URR unconv oil User defined



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('run')
def urr_uranium_zittel12():
    """
    URR uranium Zittel12



    constant


    """

    ret =  3700
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_conv_oil_maggio12_high():
    """
    URR conv oil Maggio12 High



    constant


    """

    ret =  16700
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_conv_oil_maggio12_low():
    """
    URR conv oil Maggio12 Low



    constant


    """

    ret =  12500
    name = 2
    return round_sig(x=ret, name=name)

def table_max_extraction_tot_agg_oil_laherrere_2006(x):
    """
    table max extraction tot agg oil Laherrere 2006



    lookup


    """

    ret =  functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200, 13800, 14400, 15000, 15600, 16200, 16800
    ], [
        0.0, 54.326237263, 77.92436275200001, 98.78275612, 115.24967286600001, 130.06932745699999,
        140.497709169, 151.475382009, 163.45282174999997, 171.0, 177.460153, 182.146242,
        187.686421656, 189.07350399999999, 189.324922862, 189.324922862, 189.324922862,
        189.324922862, 189.324922862, 189.324922862, 189.324922862, 189.324922862, 189.324922862,
        189.324922862, 189.324922862, 189.324922862, 189.324922862, 189.324922862, 189.324922862
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_extraction_maggio12_high_conv_oil_ej(x):
    """
    table max extraction Maggio12 High conv oil EJ



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500, 15000, 15500, 16000, 16500, 17000
    ], [
        0.0, 15.6384, 43.7316, 66.2483, 87.3472, 103.416, 119.408, 131.57, 140.057, 150.257,
        156.932, 162.683, 167.462, 171.239, 173.998, 175.733, 176.452, 176.452, 176.452, 176.452,
        176.452, 176.452, 176.452, 176.452, 176.452, 176.452, 176.452, 176.452, 176.452, 176.452,
        176.452, 176.452, 176.452, 176.452, 176.452
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_extraction_maggio12_low_con_oil_ej(x):
    """
    table max extraction Maggio12 Low con oil EJ



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500
    ], [
        0.0, 31.19257370050454, 55.93036702223348, 78.9742737866738, 98.99541997370072,
        112.90238993902808, 129.7869030062217, 139.05477860121135, 149.65885530140235,
        155.85226158136214, 160.213586322676, 162.5352378599971, 162.89303662558495,
        162.89303662558495, 162.89303662558495, 162.89303662558495, 162.89303662558495,
        162.89303662558495, 162.89303662558495, 162.89303662558495, 162.89303662558495,
        162.89303662558495, 162.89303662558495, 162.89303662558495, 162.89303662558495,
        162.89303662558495
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_extraction_maggio12middle_conv_oil_ej(x):
    """
    table max extraction Maggio12middle conv oil EJ



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xseparate_conv_and_unconv_gasxx():
    """
    Xseparate conv and unconv gasXX



    constant


    """

    ret =  0
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def urr_tot_agg_oil_laherrere_2006():
    """
    URR tot agg oil Laherrere 2006



    constant


    """

    ret =  16800
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_extraction_unconv_oil_user_defined(x):
    """
    table max extraction unconv oil User defined



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Uranium extraction

def table_max_extraction_uranium_zittel12(x):
    """
    table max extraction uranium Zittel12



    lookup


    """

    ret =  functions.lookup(
        x, [0, 300, 600, 900, 1200, 1500, 1800, 2100, 2400, 2700, 3000, 3300, 3600, 3900], [
            0.0, 10.03061064, 23.76488412, 33.486863160000006, 39.968210400000004, 44.0, 44.0,
            44.0, 44.0, 44.0, 44.0, 44.0, 44.0, 44.0
        ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_tot_agg_oil_user_defined():
    """
    URR tot agg oil User defined



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def choose_extraction_curve_tot_agg_oil():
    """
    choose extraction curve tot agg oil



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_conv_oil_maggio12_middle():
    """
    URR conv oil Maggio12 middle



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_conv_oil_user_defined():
    """
    URR conv oil User defined



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def choose_extraction_tot_agg_gas_curve():
    """
    choose extraction tot agg gas curve



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_extraction_unconv_oil_low_mohr15(x):
    """
    table max extraction unconv oil Low Mohr15



    lookup


    """

    ret =  functions.lookup(
        x, [0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000], [
            0.0, 43.7805, 76.0897, 88.8414, 94.8748, 97.1327, 97.1327, 97.1327, 97.1327, 97.1327,
            97.1327, 97.1327, 97.1327
        ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_unconv_oil_bg_mohr15():
    """
    URR unconv oil BG Mohr15



    constant


    """

    ret =  10500
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_extraction_unconv_oil_high_mohr15(x):
    """
    table max extraction unconv oil High Mohr15



    lookup


    """

    ret =  functions.lookup(x, [
        0, 1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000,
        15000, 16000, 17000, 18000, 19000, 20000, 21000, 22000
    ], [
        0.0, 24.283, 74.1838, 112.563, 141.537, 163.822, 181.819, 196.629, 208.171, 221.49,
        231.626, 241.856, 246.539, 246.539, 246.539, 246.539, 246.539, 246.539, 246.539, 246.539,
        246.539, 246.539, 246.539
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_extraction_tot_agg_oil_user_defined(x):
    """
    table max extraction tot agg oil User defined



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_conv_oil_extraction_user_defined(x):
    """
    table max conv oil extraction User defined



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xseparate_conv_and_unconv_oilxx():
    """
    Xseparate conv and unconv oilXX



    constant


    """

    ret =  0
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('run')
def urr_uranium_ewg13():
    """
    URR uranium EWG13



    constant


    """

    ret =  4000
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Uranium extraction

def table_max_extraction_uranium_ewg13_ej(x):
    """
    table max extraction uranium EWG13 EJ



    lookup


    """

    ret =  functions.lookup(
        x, [0, 300, 600, 900, 1200, 1500, 1800, 2100, 2400, 2700, 3000, 3300, 3600, 3900], [
            0.0, 17.2759, 31.6577, 41.2069, 48.093, 51.4386, 52.2, 52.2, 52.2, 52.2, 52.2, 52.2,
            52.2, 52.2
        ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('run')
def kt_uranium_per_ej():
    """
    kt uranium per EJ



    constant


    """

    ret =  2.386634845
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
@cache('run')
def life_time_nuclear():
    """
    life time nuclear



    constant


    """

    ret =  40
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
# IMPACTS - RES Land-use
@cache('step')
def surface_solar_pv_mha():
    """
    surface solar PV Mha



    component


    """

    ret =  surface_res_elec().loc[{'res elec':'solar pv'}].values
    name = 2
    return round_sig(x=ret, name=name)
# ELEC - Electricity demand covered by fossil fuels

def hist_share_oil_elec(x):
    """
    Hist share oil Elec



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010
    ], [
        0.08179045518574861, 0.07839861446722847, 0.07794585253287176, 0.07934861054352757,
        0.07492787387164873, 0.07004498230243206, 0.06754355131189571, 0.06449449910798712,
        0.06163245529496515, 0.05952908873934008, 0.05601852360426078, 0.04972132065506773,
        0.04758264742818795, 0.04532315701504618, 0.041656283956667574, 0.037650367975256026
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('step')
def cumulated_uranium_extraction_kt():
    """
    Cumulated uranium extraction kt



    component


    """

    ret =  cumulated_uranium_extraction() * kt_uranium_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('step')
def extraction_uranium_kt():
    """
    extraction uranium kt



    component


    """

    ret =  extraction_uranium_ej() * kt_uranium_per_ej()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
@cache('run')
def efficiency_uranium_for_electricity():
    """
    efficiency uranium for electricity



    constant


    """

    ret =  0.3333333333
    name = 2
    return round_sig(x=ret, name=name)

def xhist_share_gasxxcoal_xgasx_elecx(x):
    """
    XHist share gasXXcoal XgasX ElecX



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.22413023824294928, 0.2217251120457687, 0.23386523401297685, 0.24495146040651222,
        0.25808112961507557, 0.2589533680662218, 0.2683927992823794, 0.2734259444963716,
        0.26962043633910304, 0.2788310171125217, 0.2808384825803754, 0.28373606863189066,
        0.28937632319005824, 0.299165260107698, 0.3072979704663346, 0.3104639543550776,
        0.30204108495613075, 0.31079123544643, 0.2980520456011999, 0.3032674271146121
    ])
    name = 2
    return round_sig(x=ret, name=name)
# ELEC - Demand of electricity from nuclear

def historic_nuclear_generation_twh(x):
    """
    Historic nuclear generation TWh



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        1908, 1996, 2015, 2081, 2125, 2210, 2291, 2271, 2316, 2393, 2449, 2516, 2545, 2517, 2617,
        2639, 2659, 2597, 2602, 2568, 2620, 2507, 2345, 2364, 2408, 0
    ])
    name = 2
    return round_sig(x=ret, name=name)
# ELEC - Demand of electricity from nuclear

def invest_cost_nuclear(x):
    """
    invest cost nuclear



    lookup


    """

    ret =  functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            5.536708628135624, 5.536708628135624, 5.536708628135624, 5.536708628135624,
            5.536708628135624, 5.536708628135624, 5.536708628135624, 5.536708628135624,
            5.536708628135624, 5.536708628135624, 5.536708628135624, 5.536708628135624
        ])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# ELEC - Demand of electricity from nuclear
# ELEC - Total FE electricity generation
# CHP plants - FES commercial Heat and Elec
# EROItot - Electricity
# COMP- Historical data Elec production
@cache('step')
def fe_nuclear_elec_generation_twh():
    """
    FE nuclear Elec generation TWh



    component


    """

    ret =  extraction_uranium_ej() * efficiency_uranium_for_electricity() / ej_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Electric batteries
# INDICATORS - Energy
@cache('run')
def kwh_per_twh():
    """
    kWh per TWh



    constant


    """

    ret =  1000000000
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
# ELEC - Electricity related losses
@cache('step')
def total_fe_elec_demand_ej():
    """
    Total FE Elec demand EJ



    component


    """

    ret =  total_fe_elec_demand_twh() * ej_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def gco2e_per_kwh_biomass():
    """
    gCO2e per KWh biomass



    constant


    """

    ret =  0
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
# RES BIOE - Total biofuels liquids
# RES - PES
# TRANSP - Transport energy demand
# RESOURCES - Oil extraction
# RESOURCES - Coal extraction
# TPES
# EROI system
@cache('run')
def mtoe_per_ej():
    """
    MToe per EJ



    constant


    """

    ret =  23.88459
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xshare_transmxdistr_elec_losses_initialx():
    """
    Xshare transmXdistr elec losses initialX



    constant


    """

    ret =  0.095
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
# ELEC - Electricity demand
# ELEC - Electricity demand
# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC - Electricity demand covered by fossil fuels
# ELEC - Electricity demand covered by fossil fuels
# ELEC - Demand of electricity from nuclear
# ELEC - PHS
# ELEC RES - Potential
# ELEC RES - Potential
# ELEC RES - Capacities and generation all techn
# ELEC - Total FE electricity generation
# ELEC - Total FE electricity generation
# ELEC - Total FE electricity generation
# RES BIOE - Biogas
# WASTE - PES & FES
# CHP plants - FES commercial Heat and Elec
# HEAT RES - Potential
# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
# TRANSP - Alternative transport
# TFES by fuel
# ESOI - PHS
# ESOI - EV batteries
# EROI - RES elec
# EROI - RES elec
# EROItot - Electricity
@cache('run')
def ej_per_twh():
    """
    EJ per TWh



    constant


    """

    ret =  0.0036
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('step')
def surface_onshore_wind_mha():
    """
    surface onshore wind Mha



    component


    """

    ret =  surface_res_elec().loc[{'res elec':'wind onshore'}].values
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Demand of electricity from nuclear
# ELEC - Demand of electricity from nuclear
# ELEC - Storage demand and supply
# ELEC - PHS
# ELEC - PHS
# ELEC RES - Potential
# ELEC RES - Potential
# ELEC RES - Capacities and generation all techn
# ELEC RES - Capacities and generation all techn
# ELEC RES - Capacities and generation all techn
# ELEC RES - Capacities and generation all techn
# HEAT RES - Potential
# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
# ESOI - PHS
# ESOI - EV batteries
# EROI - RES elec
@cache('run')
def twe_per_twh():
    """
    TWe per TWh



    constant


    """

    ret =  0.0001141552511
    name = 2
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('run')
def initial_population():
    """
    initial population



    constant


    """

    ret =  5705843054
    name = 2
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# ECON- Final demands by sector, households and fuel
# RES BIOE - Traditional biomass
# INDICATORS - Energy
# INDICATORS - Energy
# INDICATORS - Social and Environmental impacts
@cache('step')
def population():
    """
    Population



    component


    """


    ret =  integ_population()
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def ve_objetive_ue2020_extrap():
    """
    VE objetive UE2020 extrap



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def ve_objetive_ue2020():
    """
    VE objetive UE2020



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('run')
def choose_extraction_uranium_curve():
    """
    Choose extraction uranium curve



    constant


    """

    ret =  2
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('run')
def p_ngv_growth():
    """
    P NGV growth



    constant


    """

    ret =  0.07
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('run')
def ngv_past_share_growth():
    """
    NGV past share growth



    constant


    """

    ret =  0.17194658004023933
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xbevxhev_past_share_growthx():
    """
    XBEVXHEV past share growthX



    constant


    """

    ret =  0.884
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def ngv_share():
    """
    NGV share



    component


    """


    ret =  integ_ngv_share()
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def hev_share():
    """
    HEV share



    component


    """


    ret =  integ_hev_share()
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def bev_share():
    """
    BEV share



    component


    """


    ret =  integ_bev_share()
    name = 2
    return round_sig(x=ret, name=name)

# TPES
@cache('step')
def year_scarcity_tpe():
    """
    Year scarcity TPE



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_tpe()) > single_precision(0.95) + less_thr_func(abundance_tpe(), 0.95), 0, time())
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Uranium extraction
@cache('step')
def year_scarcity_uranium():
    """
    Year scarcity uranium



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_uranium()) > single_precision(0.95) + less_thr_func(abundance_uranium(), 0.95), 0, time())
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def year_scarcity_coal():
    """
    Year scarcity coal



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_coal()) > single_precision(0.95) + less_thr_func(abundance_coal(), 0.95), 0, time())
    name = 2
    return round_sig(x=ret, name=name)

# ABUND - Abundances by sector
@cache('step')
def abundance_transport():
    """
    abundance transport



    component


    """

    ret =  functions.if_then_else(pes_for_transp_ej() > pe_demand_transport_after_policies_ej(), 1,
                                  1 -
                                  ((pe_demand_transport_after_policies_ej() - pes_for_transp_ej())
                                   / pe_demand_transport_after_policies_ej()))
    name = 2
    return round_sig(x=ret, name=name)

# ABUND - Abundances by sector
@cache('step')
def year_scarcity_transp():
    """
    Year scarcity Transp



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_transport()) > single_precision(0.95) + less_thr_func(abundance_transport(), 0.95), 0, time())
    name = 2
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
@cache('step')
def year_scarcity_liquids():
    """
    Year scarcity liquids



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_liquids()) > single_precision(0.95) + less_thr_func(abundance_liquids(), 0.95), 0, time())
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xeffects_shortage_elec_on_bevxhevx():
    """
    Xeffects shortage elec on BEVXHEVX



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_electricity()) > single_precision(0.8) + less_thr_func(abundance_electricity(), 0.8),
                                  ((abundance_electricity() - 0.8) * 5)**2, 0)
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
# TRANSP - Inland transport sector
# RESOURCES - CTL & GTL supply
@cache('step')
def effects_shortage_gas():
    """
    effects shortage gas



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_gases()) > single_precision(0.8) + less_thr_func(abundance_gases(), 0.8), ((abundance_gases() - 0.8) * 5)**2, 0)
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def ngv_fraction_of_change_available():
    """
    NGV fraction of change available



    component


    """

    ret =  ((max_ngv_percent_of_change() - ngv_share()) / max_ngv_percent_of_change())
    name = 2
    return round_sig(x=ret, name=name)

# GASES - PED, PES & FES
@cache('step')
def year_scarcity_gases():
    """
    Year scarcity gases



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_gases()) > single_precision(0.95) + less_thr_func(abundance_gases(), 0.95), 0, time())
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('run')
def p_bev_growth():
    """
    P BEV growth



    constant


    """

    ret =  0.09
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def max_ngv_percent_of_change():
    """
    max NGV percent of change



    component


    """

    ret =  (1 - total_share_ev())
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def max_percent_of_change():
    """
    max percent of change



    constant


    """

    ret =  0.448
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('run')
def choose_extraction_coal_curve():
    """
    choose extraction coal curve



    constant


    """

    ret =  3
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def selection_constraint_extraction_unconv_gas():
    """
    Selection constraint extraction unconv gas



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Total FE electricity generation
@cache('step')
def year_scarcity_elec():
    """
    Year scarcity Elec



    component


    """

    ret =  functions.if_then_else(single_precision(abundance_electricity()) > single_precision(0.95) + less_thr_func(abundance_electricity(), 0.95), 0, time())
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def extraction_unconv_gas_delayed():
    """
    extraction unconv gas delayed



    component


    """

    ret =  delay_extraction_unconv_gas_ej_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def max_unconv_gas_growth_extraction_ej():
    """
    max unconv gas growth extraction EJ



    component


    """

    ret =  functions.if_then_else(single_precision(-0.01) > single_precision(check_gas_delayed_1yr()) + less_thr_func(-0.01, check_gas_delayed_1yr()),
        (1 + xconstrain_gas_exogenous_growthx_delayed_1yrx()) * extraction_unconv_gas_delayed(),
        extraction_unconv_gas_delayed() * max_unconv_gas_growth_extraction())
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def p_constraint_growth_extraction_unconv_gas():
    """
    P constraint growth extraction unconv gas



    constant


    """

    ret =  0.08
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def choose_extraction_curve_unconv_gas():
    """
    choose extraction curve unconv gas



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def choose_extraction_curve_unconv_oil():
    """
    choose extraction curve unconv oil



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def total_share_ev():
    """
    total share EV



    component


    """

    ret =  bev_share() + hev_share()
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('run')
def p_hev_growth():
    """
    P HEV growth



    constant


    """

    ret =  0.09
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('step')
def biofuels_land_compet_available():
    """
    Biofuels land compet available



    component


    """

    ret =  (max_land_compet_biofuels_2gen() - land_compet_biofuels_2gen_mha() -
            land_compet_biofuels_3gen_mha()) / max_land_compet_biofuels_2gen()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def urr_oil_aspo():
    """
    URR oil ASPO



    constant


    """

    ret =  12800
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_gas_mohr_high2013():
    """
    URR gas Mohr High2013



    constant


    """

    ret =  28500
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def urr_gas_leherrere2010():
    """
    URR gas Leherrere2010



    constant


    """

    ret =  13600
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_gas_mohr_bg2012():
    """
    URR gas Mohr BG2012



    constant


    """

    ret =  19900
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - RES elec supply
@cache('run')
def res_to_fossil_accounting():
    """
    RES to fossil accounting



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def new_c_gtc():
    """
    new C GtC



    component


    """

    ret =  carbon_emissions_gtc()
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def year_adjust():
    """
    year adjust



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

def xtpes_de_castro_phd_x_scen_xmadcoalxx(x):
    """
    XTPES de Castro PhD X Scen XmadcoalXX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        7727.25, 7904.71, 8090.31, 8272.83, 8453.11, 8632.09, 8810.72, 8989.94, 9170.63, 9353.55,
        9539.38, 9728.58, 9921.49, 10118.2, 10318.7, 10522.5, 10729.3, 10938.4, 11148.8, 11359.5,
        11569.5, 11786.3, 11999.6, 12211.4, 12420.8, 12626.9, 12829.2, 13027.3, 13221, 13410.5,
        13596.1, 13774, 13924.7, 14047.6, 14141.5, 14208.3, 14251.8, 14277.8, 14292.7, 14303.7,
        14317.3, 14343.2, 14382.9, 14441.4, 14521.6, 14625.6, 14754.7, 14909.2, 15089.5, 15295.1,
        15525.7, 15768.7, 16021.4, 16282.5, 16551.3, 16827.5, 17111.3, 17402.7, 17701.8, 18008.4,
        18322.2, 18642.6, 18968.8, 19299.8, 19634.2, 19970.7, 20218.4, 20503.9, 20792.3, 21082,
        21370.6, 21655.5, 21933.9, 22202.8, 22459.9, 22702.5, 22928.4, 23135.7, 23322.3, 23486.7,
        23627.4, 23743.2, 23833.1, 23896.3, 23932.3, 23940.6, 23921.3, 23874.3, 23810.4, 23774.2,
        23753, 23746.1, 23752.7, 23771.8, 23802.3, 23843.3, 23893.7, 23952.6, 24018.9, 24091.6,
        24169.7, 24252.2, 24338.1, 24426.4, 24516.1, 24606.1, 24695.4, 24783, 24868, 24949.3,
        25026, 25097.2, 25162, 25219.5, 25268.9, 25309.5
    ])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('run')
def urban_surface_2008():
    """
    urban surface 2008



    constant


    """

    ret =  300
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def pre_industrial_value_ppm():
    """
    pre industrial value ppm



    constant


    """

    ret =  275
    name = 2
    return round_sig(x=ret, name=name)
# COMP - Historical data

def historic_tpes(x):
    """
    Historic TPES



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
    ], [
        367.27824232999996, 369.80021808, 370.10188359, 373.85445558, 376.49107106, 386.32755368,
        395.99083282, 400.11296999, 402.05673523, 410.72494535000004, 420.22447686000004,
        424.76838970999995, 433.61047764, 448.95249227, 469.50612458999996, 482.86261257999996,
        497.09219364999996, 510.9159773, 517.61014845, 513.61450861, 542.2700547000001,
        550.58678017, 559.9193788099999, 567.27428163, 573.55505552
    ])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
# IMPACTS - Carbon cycle & climate
# INDICATORS - Social and Environmental impacts
# COMP - Historical data
@cache('run')
def c_per_co2():
    """
    C per CO2



    constant


    """

    ret =  0.272727272
    name = 2
    return round_sig(x=ret, name=name)
# COMP - Historical data

def historic_coal_extraction(x):
    """
    Historic coal extraction



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
    ], [
        92.95971952, 90.52395622, 88.93634092, 89.26750847, 90.03556969, 92.45324945, 93.94172036,
        93.28011565, 92.94987548, 93.37931848000001, 96.96404212, 98.54193956, 101.84921095,
        109.64058717, 118.38527423000001, 125.31717954000001, 132.88631276, 139.9969369,
        141.69452609, 142.03754203, 153.09457012000001, 159.30506497, 161.13494472, 163.10375116,
        164.05938973
    ])
    name = 2
    return round_sig(x=ret, name=name)
# COMP - Historical data

def historic_natural_gas_extraction(x):
    """
    Historic natural gas extraction



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
    ], [
        69.63541319, 72.0449446, 72.12715572, 73.36381595, 73.33112109999999, 75.66591356999999,
        78.36687037, 79.21943373, 79.97846641, 83.56539577, 86.71736648000001, 87.34085883,
        90.28420176, 93.60664527, 95.94604267, 98.80492587, 101.3465432, 105.81270529000001,
        108.54881217, 106.19365766, 114.56974331999999, 116.8379315, 119.01788815, 121.63895194,
        121.4414386
    ])
    name = 2
    return round_sig(x=ret, name=name)
# COMP - Historical data

def historic_liquids_extraction(x):
    """
    Historic liquids extraction



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
    ], [
        135.34532599000002, 135.99321097, 136.70658668000002, 137.64694317, 138.39015611000002,
        141.33118687, 145.00410076, 148.65399012999998, 149.01796758, 151.92545035, 153.21595617,
        155.10573028, 156.46599387999998, 159.69866319, 166.13078964, 167.60699336000002,
        169.70260508, 170.88399325999998, 170.99991855000002, 167.93633169, 173.24536827,
        173.30416877000002, 176.88721836000002, 176.50335453, 179.41696216999998
    ])
    name = 2
    return round_sig(x=ret, name=name)
# COMP - Historical data

def historic_crude_oil_extraction(x):
    """
    Historic crude oil extraction



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014
    ], [
        127.83615043, 127.91110152, 128.20885903, 129.51449347, 131.32835549, 132.80839306000001,
        137.36404604, 141.79097772999998, 141.76275527, 141.88197549, 145.06952113999998,
        145.86612486, 145.76086142, 150.04828162, 155.18872345, 156.55082523, 157.51828895,
        157.66610619, 158.66583971, 155.5972526, 158.7730987, 158.73186157, 162.40275298,
        160.97799775, 164.03661609
    ])
    name = 2
    return round_sig(x=ret, name=name)
# COMP - Historical data

def gtc_historic_emissions_rcps(x):
    """
    GtC historic emissions RCPs



    lookup


    """

    ret =  functions.lookup(x, [1990, 1995, 2000, 2005], [6.144, 6.4395, 6.735, 7.971])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('run')
def activate_affores_program():
    """
    activate Affores program



    constant


    """

    ret =  0
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def adapt_emissions_shale_oil():
    """
    Adapt emissions shale oil



    component


    """

    ret =  functions.if_then_else(time() < 2050,
                                  0.001 + (0.15 - 0.001) * (time_dmnl(time()) - 2000) / 50,
                                  functions.if_then_else(time() < 2100, 0.15 + (0.72 - 0.15) *
                                                         (time_dmnl(time()) - 2050) / 50, 0.72))
    name = 2
    return round_sig(x=ret, name=name)
# IMPACTS - Total GHG emissions

def afforestation_program_2020(x):
    """
    Afforestation program 2020



    lookup


    """

    ret =  functions.lookup(x, [
        1980, 2020, 2025, 2030, 2035, 2040, 2045, 2050, 2055, 2060, 2065, 2070, 2075, 2080, 2085,
        2090, 2095, 2100, 2105, 2110, 2115, 2120
    ], [
        0.0, 0.0, 138.2727041328326, 299.56044274475425, 452.9353423630819, 597.8798562521115,
        733.8764376947656, 860.4075399609283, 976.9556163242087, 1083.0031200675294,
        1178.0325044589117, 1261.5262227738276, 1332.966728293337, 1391.8364742817357,
        1437.6179140219465, 1469.7935007894412, 1487.8456878541037, 1491.256928496994,
        1479.5096759861335, 1452.086383602582, 1408.469504618086, 1348.1414923118427
    ])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - RES Land-use
@cache('run')
def global_arable_land():
    """
    Global arable land



    constant


    """

    ret =  1526
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def choose_extraction_curve_conv_oil():
    """
    choose extraction curve conv oil



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def choose_extraction_conv_gas_curve():
    """
    choose extraction conv gas curve



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Total monetary investments
@cache('step')
def cumulated_total_monet_invest_res_for_elec():
    """
    Cumulated total monet invest RES for Elec



    component


    """


    ret =  integ_cumulated_total_monet_invest_res_for_elec()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
@cache('run')
def efficiency_coal_for_electricity():
    """
    efficiency coal for electricity



    constant


    """

    ret =  0.353
    name = 2
    return round_sig(x=ret, name=name)
# ELEC - Electricity demand covered by fossil fuels

def historic_efficiency_gas_for_electricity(x):
    """
    Historic efficiency gas for electricity



    lookup


    """

    ret =  functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        37.813, 38.008, 38.708, 38.843, 39.84, 39.903, 39.938, 40.541, 41.535, 41.946, 42.082,
        42.187, 42.885, 43.196, 43.382, 43.393, 43.951, 43.855, 44.466, 44.268
    ])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand covered by fossil fuels
# ELEC - Electricity related losses
# ELEC - Total FE electricity generation
@cache('run')
def efficiency_liquids_for_electricity():
    """
    efficiency liquids for electricity



    constant


    """

    ret =  0.361
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('run')
def additional_land_compet_available_for_biofuels():
    """
    Additional land compet available for biofuels



    constant


    """

    ret =  100
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('run')
def max_hydro_twe():
    """
    max hydro TWe



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('run')
def max_oceanic_twe():
    """
    max oceanic TWe



    constant


    """

    ret =  0.05
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES - Potential
@cache('run')
def max_onshore_wind_twe():
    """
    max onshore wind TWe



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

def p_timeseries_pop_growth_rate(x):
    """
    P timeseries pop growth rate



    lookup


    """

    ret =  functions.lookup(x, [
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100, 2101
    ], [
        0.0105, 0.0102, 0.01, 0.0098, 0.0096, 0.0093, 0.0091, 0.0089, 0.0087, 0.0085, 0.0082,
        0.008, 0.0078, 0.0076, 0.0074, 0.0073, 0.0071, 0.0069, 0.0067, 0.0065, 0.0064, 0.0062,
        0.006, 0.0058, 0.0057, 0.0055, 0.0053, 0.0052, 0.005, 0.0048, 0.0047, 0.0045, 0.0044,
        0.0042, 0.004, 0.0039, 0.0037, 0.0036, 0.0035, 0.0033, 0.0032, 0.0031, 0.0029, 0.0028,
        0.0027, 0.0026, 0.0025, 0.0024, 0.0023, 0.0022, 0.0021, 0.002, 0.002, 0.0019, 0.0018,
        0.0017, 0.0017, 0.0016, 0.0015, 0.0015, 0.0014, 0.0013, 0.0013, 0.0012, 0.0012, 0.0011,
        0.0011, 0.001, 0.001, 0.0009, 0.0009, 0.0009, 0.0008, 0.0008, 0.0008, 0.0008, 0.0007,
        0.0007, 0.0007, 0.0006, 0.0006, 0.0006, 0.0006, 0.0005, 0.0005, 0.0, 0.0
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def p_ctl():
    """
    P CTL



    constant


    """

    ret =  0.15
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - CTL & GTL supply
@cache('run')
def p_gtl():
    """
    P GTL



    constant


    """

    ret =  0.2
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Carbon cycle & climate
@cache('run')
def ppm_per_gtc():
    """
    ppm per GtC



    constant


    """

    ret =  2.12
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Dedicated crops for biofuels (compet)
@cache('run')
def start_year_3gen():
    """
    start year 3gen



    constant


    """

    ret =  2025
    name = 2
    return round_sig(x=ret, name=name)
# ELEC - Electricity demand covered by fossil fuels
# ELEC - PHS
# ELEC RES - Capacities and generation all techn
# RES BIOE - Dedicated crops for biofuels (compet)
# RES BIOE - Biogas
# RES BIOE - Biogas
# WASTE - PES & FES
# HEAT RES-com - Capacities and generation all tech
# HEAT RES-com - Capacities and generation all tech
# HEAT RES-nc - Capacities and generation all techn
# HEAT RES-nc - Capacities and generation all techn
# TRANSP - Alternative transport
# TRANSP - Alternative transport
# TRANSP - Alternative transport
# IMPACTS - Total GHG emissions

def time_dmnl(x):
    """
    Time dmnl



    lookup


    """

    ret =  functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ])
    name = 2
    return round_sig(x=ret, name=name)

# ELEC RES: Overcapacities due to RES variability a
# ELEC RES - Total monetary investments
@cache('step')
def share_variable_res_elec_generation_vs_total():
    """
    Share variable RES elec generation vs total



    component


    """

    ret =  functions.if_then_else(
        (fe_elec_generation_from_nre_twh() + elec_generation_variable_from_res_twh() +
         xelec_generation_basexload_from_res_twhx()) > 0,
        elec_generation_variable_from_res_twh() /
        (fe_elec_generation_from_nre_twh() + elec_generation_variable_from_res_twh() +
         xelec_generation_basexload_from_res_twhx()), 0.5)
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_extraction_aspo_oil_ej(x):
    """
    table max extraction ASPO oil EJ



    lookup


    """

    ret =  functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0, 29.97834, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537,
        137.018, 145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044, 171.044
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_gas_laherrere2010(x):
    """
    table max extraction gas Laherrere2010



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000
    ], [
        0, 23.8621, 44.631, 62.3067, 77.7729, 93.8891, 107.821, 119.752, 129.033, 137.87, 144.16,
        148.917, 151.78, 152.895, 152.895, 152.895, 152.895, 152.895, 152.895, 152.895, 152.895,
        152.895, 152.895, 152.895, 152.895, 152.895, 152.895
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_gas_mohr_high2012(x):
    """
    table max extraction gas Mohr High2012



    lookup


    """

    ret =  functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500, 24000, 25500, 27000, 28500
    ], [
        0, 36.4477, 58.3656, 72.8198, 98.5, 130.439, 159.979, 177.268, 190.383, 194.052, 194.197,
        194.197, 194.197, 194.197, 194.197, 194.197, 194.197, 194.197, 194.197, 194.197
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_gas_mohr_bg2012(x):
    """
    table max extraction gas Mohr BG2012



    lookup


    """

    ret =  functions.lookup(x, [
        0, 1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000,
        15000, 16000, 17000, 18000, 19000, 20000
    ], [
        0, 27.4739, 42.1041, 51.2745, 58.1215, 69.3726, 79.5334, 99.1636, 115.575, 128.798,
        138.387, 147.465, 150.753, 157.008, 159.247, 159.247, 159.247, 159.247, 159.247, 159.247,
        159.247
    ])
    name = 2
    return round_sig(x=ret, name=name)

# IMPACTS - Total GHG emissions
@cache('step')
def total_cumulative_emissions_gtc():
    """
    Total cumulative emissions GtC



    component


    """


    ret =  integ_total_cumulative_emissions_gtc()
    name = 2
    return round_sig(x=ret, name=name)

def xtnes_de_castro_phd_x_scen_ix(x):
    """
    XTNES de Castro PhD X Scen IX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        7173.13, 7368.39, 7560.95, 7750.95, 7938.54, 8123.85, 8306.94, 8487.87, 8666.6, 8843.01,
        9016.91, 9188, 9355.9, 9520.11, 9680.06, 9835.12, 9984.57, 10127.7, 10263.7, 10391.9,
        10511.6, 10622.1, 10723.1, 10814.1, 10894.8, 10965.4, 11025.8, 11076.5, 11117.8, 11150.3,
        11174.8, 11187.2, 11171.3, 11125.6, 11048.7, 10941.3, 10806.3, 10648.5, 10473.6, 10288,
        10098.3, 9914.3, 9737.27, 9572.49, 9422.99, 9290.79, 9176.91, 9081.61, 9004.54, 8944.92,
        8901.71, 8868.15, 8842.36, 8822.52, 8807.44, 8796.45, 8789.34, 8786.33, 8787.93, 8794.91,
        8800.9, 8805.38, 8813.47, 8826.89, 8847.13, 8844.22, 8829.3, 8822.62, 8829.47, 8853.12,
        8895.31, 8956.7, 9037.27, 9136.56, 9253.94, 9388.68, 9540.13, 9707.65, 9890.73, 10089,
        10302, 10529.6, 10771.5, 11027.7, 11298, 11582.2, 11880.5, 12192.5, 12518.4, 12857.9,
        13211, 13577.5, 13957.4, 14350.4, 14756.5, 15175.4, 15606.9, 16051, 16507.3, 16975.7,
        17456, 17947.9, 18451.3, 18965.9, 19491.6, 20028, 20575.1, 21132.7, 21700.6, 22278.8,
        22867.1, 23465.6, 24074.2, 24693.1, 25322.3, 25962.1
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xnet_oil_extraction_de_castro_phd_x_scen_ix(x):
    """
    XNet oil extraction de Castro PhD X Scen IX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2586.21, 2657.24, 2727.03, 2795.64, 2863.08, 2929.39, 2994.56, 3058.58, 3121.38, 3182.87,
        3242.92, 3301.35, 3357.93, 3412.41, 3464.47, 3513.79, 3559.99, 3602.72, 3641.58, 3676.21,
        3706.25, 3731.41, 3751.42, 3766.07, 3775.24, 3778.86, 3776.96, 3769.61, 3756.99, 3739.31,
        3716.84, 3688.03, 3646.76, 3592.8, 3525.96, 3446.96, 3357.36, 3259.3, 3155.32, 3048.04,
        2940, 2833.9, 2731.45, 2633.99, 2542.41, 2457.18, 2378.41, 2305.92, 2239.3, 2178.03,
        2121.51, 2067.41, 2015.01, 1963.69, 1913.01, 1862.72, 1812.75, 1763.12, 1713.95, 1665.41,
        1617.7, 1569.24, 1521.33, 1474.43, 1428.88, 1378.91, 1326.44, 1276.28, 1229.49, 1186.64,
        1147.86, 1113.07, 1082.06, 1054.53, 1030.2, 1008.79, 990.086, 973.882, 960.031, 948.419,
        938.964, 931.608, 926.306, 923.025, 921.731, 922.385, 924.939, 929.328, 935.469, 943.259,
        952.569, 963.248, 975.123, 987.996, 1001.65, 1015.86, 1030.38, 1044.96, 1059.33, 1073.23,
        1086.43, 1098.65, 1109.67, 1119.27, 1127.24, 1133.4, 1137.59, 1139.68, 1139.56, 1137.17,
        1132.46, 1125.41, 1116.04, 1104.39, 1090.55, 1074.6
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xpe_oil_extraction_de_castro_phd_x_scen_iix(x):
    """
    XPE oil extraction de Castro PhD X Scen IIX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2853.6, 2932.69, 3010.55, 3087.22, 3162.74, 3237.16, 3310.47, 3382.67, 3453.7, 3523.47,
        3591.83, 3658.6, 3723.55, 3786.39, 3846.79, 3904.4, 3958.81, 4009.62, 4056.42, 4098.78,
        4136.27, 4169.57, 4198.24, 4222.15, 4241.19, 4255.33, 4264.62, 4269.17, 4269.15, 4264.78,
        4256.34, 4242.11, 4215.12, 4174.81, 4120.76, 4053.66, 3975.16, 3887.67, 3794.08, 3697.45,
        3600.71, 3506.78, 3417.66, 3334.64, 3258.44, 3189.17, 3126.44, 3069.5, 3017.29, 2968.65,
        2922.37, 2873.74, 2821.48, 2764.72, 2703.17, 2637.04, 2566.93, 2493.69, 2418.33, 2341.94,
        2265.58, 2190.28, 2116.99, 2046.54, 1979.65, 1909.2, 1839.5, 1776.01, 1719.36, 1669.55,
        1626.19, 1588.74, 1556.59, 1529.08, 1505.57, 1485.41, 1467.99, 1452.74, 1439.12, 1426.63,
        1414.83, 1403.34, 1391.81, 1379.96, 1367.57, 1354.44, 1340.44, 1325.47, 1309.44, 1293.54,
        1278.69, 1264.85, 1251.89, 1239.63, 1227.89, 1216.49, 1205.3, 1194.17, 1182.97, 1171.6,
        1159.95, 1147.95, 1135.52, 1122.6, 1109.14, 1095.1, 1080.45, 1065.16, 1049.24, 1032.67,
        1015.47, 997.651, 979.238, 960.264, 940.765, 920.786
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xnet_oil_extraction_de_castro_phd_x_scen_iiix(x):
    """
    XNet oil extraction de Castro PhD X Scen IIIX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2586.21, 2657.24, 2727.03, 2795.64, 2863.08, 2929.39, 2994.56, 3058.58, 3121.38, 3182.87,
        3242.92, 3301.35, 3357.93, 3412.41, 3464.47, 3513.79, 3559.99, 3602.72, 3641.58, 3676.21,
        3706.25, 3731.41, 3751.42, 3766.07, 3775.24, 3778.86, 3776.96, 3769.56, 3756.02, 3735.73,
        3708.45, 3672.29, 3621.03, 3554.53, 3472.85, 3377.15, 3269.46, 3152.43, 3029.07, 2902.39,
        2775.22, 2649.98, 2528.6, 2412.5, 2302.57, 2199.24, 2102.59, 2012.4, 1928.26, 1849.64,
        1775.94, 1705.9, 1639, 1574.79, 1512.89, 1453.04, 1395.04, 1338.77, 1284.12, 1231.05,
        1179.51, 1129.49, 1080.95, 1033.89, 988.283, 939.916, 888.983, 838.921, 790.425, 743.889,
        699.498, 657.297, 617.242, 579.24, 543.177, 508.93, 476.381, 445.423, 415.961, 387.915,
        361.216, 335.808, 311.643, 288.683, 266.895, 246.251, 226.726, 208.297, 190.943, 174.638,
        159.36, 145.082, 131.775, 119.409, 107.95, 97.3636, 87.6112, 78.6537, 70.4503, 62.9592,
        56.1382, 49.9448, 44.3368, 39.2727, 34.7122, 30.6158, 26.9459, 23.6663, 20.743, 18.1435,
        15.8376, 13.797, 11.9953, 10.4082, 9.01332, 7.79015
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xpe_coal_extraction_de_castro_phd_x_scen_iix(x):
    """
    XPE coal extraction de Castro PhD X Scen IIX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2081.44, 2135.51, 2188.27, 2239.78, 2290.12, 2339.37, 2387.58, 2434.81, 2481.08, 2526.43,
        2570.82, 2614.21, 2656.54, 2697.73, 2737.64, 2776.16, 2813.12, 2848.39, 2881.79, 2913.19,
        2942.43, 2970.14, 2996.14, 3020.47, 3043.2, 3064.45, 3084.36, 3103.16, 3121.08, 3138.41,
        3155.49, 3171.11, 3180.04, 3181.73, 3175.59, 3161.8, 3141.21, 3115.28, 3085.91, 3055.2,
        3025.33, 2998.72, 2977.04, 2961.66, 2953.52, 2953.18, 2960.78, 2976.17, 2998.94, 3028.5,
        3064.18, 3101.34, 3138.56, 3174.51, 3208.26, 3239.24, 3267.14, 3291.87, 3313.47, 3332.03,
        3347.68, 3360.5, 3370.53, 3377.77, 3382.15, 3369.61, 3346.49, 3321.23, 3294.99, 3268.1,
        3240.42, 3211.63, 3181.37, 3149.3, 3115.17, 3078.78, 3040.04, 2998.94, 2955.52, 2909.85,
        2862.1, 2812.42, 2761.01, 2708.07, 2653.84, 2598.53, 2542.37, 2485.58, 2428.5, 2374.08,
        2323.46, 2276.45, 2232.72, 2191.9, 2153.67, 2117.71, 2083.74, 2051.52, 2020.82, 1991.45,
        1963.21, 1935.96, 1909.56, 1883.85, 1858.74, 1834.13, 1809.91, 1786.01, 1762.35, 1738.88,
        1715.54, 1692.29, 1669.09, 1645.91, 1622.73, 1599.53
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xnet_gas_extraction_de_castro_phd_x_scen_ix(x):
    """
    XNet gas extraction de Castro PhD X Scen IX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1378.15, 1422.43, 1466.37, 1509.97, 1553.27, 1596.27, 1639, 1681.45, 1723.6, 1765.43,
        1806.88, 1847.87, 1888.31, 1928.08, 1967.04, 2005.02, 2041.85, 2077.35, 2111.34, 2143.61,
        2174.01, 2202.36, 2228.55, 2252.45, 2274, 2293.18, 2310, 2324.5, 2336.78, 2346.97, 2355.23,
        2360.56, 2358.93, 2349.91, 2333.02, 2308.36, 2276.56, 2238.66, 2196.02, 2150.19, 2102.72,
        2055.42, 2009.48, 1965.96, 1925.6, 1888.94, 1856.22, 1827.53, 1802.76, 1781.69, 1764.05,
        1748.05, 1733.15, 1718.83, 1704.69, 1690.47, 1676.02, 1661.3, 1646.32, 1631.17, 1615.96,
        1598.98, 1581.36, 1563.41, 1545.42, 1521.06, 1491.54, 1462.02, 1433.79, 1407.58, 1383.71,
        1362.23, 1342.98, 1325.69, 1310.03, 1295.68, 1282.28, 1269.52, 1257.12, 1244.82, 1232.39,
        1219.62, 1206.36, 1192.45, 1177.78, 1162.23, 1145.74, 1128.25, 1109.71, 1090.12, 1069.47,
        1047.78, 1025.09, 1001.45, 976.929, 951.611, 925.587, 898.96, 871.837, 844.332, 816.56,
        788.637, 760.675, 732.784, 705.067, 677.624, 650.543, 623.906, 597.787, 572.25, 547.351,
        523.135, 499.642, 476.902, 454.936, 433.76
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xpe_gas_extraction_de_castro_phd_x_scen_iix(x):
    """
    XPE gas extraction de Castro PhD X Scen IIX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1511.25, 1560.28, 1609, 1657.46, 1705.67, 1753.67, 1801.48, 1849.09, 1896.51, 1943.7,
        1990.62, 2037.18, 2083.29, 2128.84, 2173.66, 2217.6, 2260.46, 2302.05, 2342.18, 2380.64,
        2417.18, 2452.25, 2485.65, 2517.3, 2547.21, 2575.39, 2601.9, 2626.83, 2650.34, 2672.6,
        2693.85, 2713.04, 2725.69, 2731.16, 2728.82, 2718.75, 2701.64, 2678.7, 2651.58, 2622.12,
        2592.25, 2564.02, 2538.98, 2518.29, 2502.75, 2492.79, 2488.48, 2489.57, 2495.59, 2505.9,
        2519.76, 2533.25, 2544.98, 2553.75, 2558.7, 2559.31, 2555.33, 2546.69, 2533.45, 2515.76,
        2493.79, 2467.71, 2437.67, 2403.82, 2366.26, 2315.78, 2256.66, 2195.62, 2133.79, 2071.61,
        2009.2, 1946.54, 1883.58, 1820.32, 1756.81, 1693.17, 1629.56, 1566.17, 1503.21, 1440.92,
        1379.52, 1319.21, 1260.2, 1202.66, 1146.75, 1092.58, 1040.26, 989.855, 941.438, 895.994,
        853.858, 814.762, 778.406, 744.501, 712.786, 683.028, 655.029, 628.61, 603.618, 579.92,
        557.398, 535.948, 515.482, 495.919, 477.189, 459.233, 441.995, 425.428, 409.491, 394.147,
        379.364, 365.113, 351.369, 338.109, 325.314, 312.966
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xnet_gas_extraction_de_castro_phd_x_scen_iiix(x):
    """
    XNet gas extraction de Castro PhD X Scen IIIX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1378.15, 1422.43, 1466.37, 1509.97, 1553.27, 1596.27, 1639, 1681.45, 1723.6, 1765.43,
        1806.88, 1847.87, 1888.31, 1928.08, 1967.04, 2005.02, 2041.85, 2077.35, 2111.34, 2143.61,
        2174.01, 2202.36, 2228.55, 2252.45, 2274, 2293.18, 2310, 2324.47, 2336.18, 2344.72,
        2349.88, 2350.38, 2342.04, 2324.36, 2296.96, 2260.1, 2214.64, 2161.88, 2103.47, 2041.17,
        1976.72, 1911.72, 1847.53, 1785.2, 1725.49, 1668.87, 1615.53, 1565.49, 1518.59, 1474.56,
        1433.08, 1393.24, 1354.7, 1317.14, 1280.29, 1243.98, 1208.07, 1172.48, 1137.16, 1102.09,
        1067.27, 1032.71, 998.441, 964.476, 930.845, 893.625, 852.817, 811.701, 771.01, 731.211,
        692.579, 655.254, 619.284, 584.664, 551.358, 519.318, 488.491, 458.831, 430.297, 402.855,
        376.484, 351.166, 326.892, 303.655, 281.454, 260.289, 240.158, 221.061, 202.995, 185.952,
        169.922, 154.893, 140.845, 127.756, 115.6, 104.347, 93.9612, 84.4072, 75.6455, 67.6349,
        60.3331, 53.697, 47.6834, 42.2492, 37.3523, 32.9516, 29.0072, 25.481, 22.3367, 19.5399,
        17.0583, 14.8618, 12.922, 11.2131, 9.71091, 8.39349
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xnet_coal_extraction_de_castro_phd_x_scen_iiix(x):
    """
    XNet coal extraction de Castro PhD X Scen IIIX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1945.63, 1995.96, 2045.03, 2092.89, 2139.62, 2185.28, 2229.93, 2273.61, 2316.36, 2358.19,
        2399.06, 2438.96, 2477.8, 2515.5, 2551.95, 2587.03, 2620.58, 2652.48, 2682.56, 2710.7,
        2736.76, 2760.66, 2782.31, 2801.7, 2818.82, 2833.73, 2846.52, 2857.31, 2865.65, 2871.1,
        2873.47, 2871.23, 2859.31, 2837.16, 2804.26, 2760.86, 2707.91, 2646.88, 2579.6, 2508.07,
        2434.29, 2360.07, 2286.98, 2216.23, 2148.71, 2084.94, 2025.17, 1969.39, 1917.4, 1868.88,
        1823.41, 1779.85, 1737.75, 1696.69, 1656.32, 1616.38, 1576.69, 1537.11, 1497.56, 1458,
        1418.41, 1378.78, 1339.14, 1299.51, 1259.91, 1215.03, 1164.71, 1113.39, 1062.05, 1011.37,
        961.747, 913.409, 866.46, 820.93, 776.805, 734.051, 692.63, 652.506, 613.653, 576.054,
        539.706, 504.611, 470.782, 438.236, 406.994, 377.077, 348.507, 321.3, 295.471, 271.027,
        247.969, 226.292, 205.981, 187.015, 169.365, 152.995, 137.864, 123.924, 111.123, 99.4056,
        88.714, 78.9881, 70.1672, 62.1905, 54.9978, 48.5302, 42.7303, 37.5432, 32.916, 28.7988,
        25.1446, 21.9093, 19.0516, 16.5334, 14.3195, 12.3777
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xnet_coal_extraction_de_castro_phd_x_scen_ix(x):
    """
    XNet coal extraction de Castro PhD X Scen IX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1378.15, 1422.43, 1466.37, 1509.97, 1553.27, 1596.27, 1639, 1681.45, 1723.6, 1765.43,
        1806.88, 1847.87, 1888.31, 1928.08, 1967.04, 2005.02, 2041.85, 2077.35, 2111.34, 2143.61,
        2174.01, 2202.36, 2228.55, 2252.45, 2274, 2293.18, 2310, 2324.5, 2336.78, 2346.97, 2355.23,
        2360.56, 2358.93, 2349.91, 2333.02, 2308.36, 2276.56, 2238.66, 2196.02, 2150.19, 2102.72,
        2055.42, 2009.48, 1965.96, 1925.6, 1888.94, 1856.22, 1827.53, 1802.76, 1781.69, 1764.05,
        1748.05, 1733.15, 1718.83, 1704.69, 1690.47, 1676.02, 1661.3, 1646.32, 1631.17, 1615.96,
        1598.98, 1581.36, 1563.41, 1545.42, 1521.06, 1491.54, 1462.02, 1433.79, 1407.58, 1383.71,
        1362.23, 1342.98, 1325.69, 1310.03, 1295.68, 1282.28, 1269.52, 1257.12, 1244.82, 1232.39,
        1219.62, 1206.36, 1192.45, 1177.78, 1162.23, 1145.74, 1128.25, 1109.71, 1090.12, 1069.47,
        1047.78, 1025.09, 1001.45, 976.929, 951.611, 925.587, 898.96, 871.837, 844.332, 816.56,
        788.637, 760.675, 732.784, 705.067, 677.624, 650.543, 623.906, 597.787, 572.25, 547.351,
        523.135, 499.642, 476.902, 454.936, 433.76
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xtnes_de_castro_phd_x_scen_iiix(x):
    """
    XTNES de Castro PhD X Scen IIIX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        7173.13, 7368.39, 7560.95, 7750.95, 7938.54, 8123.85, 8306.94, 8487.87, 8666.6, 8843.01,
        9016.91, 9188, 9355.9, 9520.11, 9680.06, 9835.12, 9984.57, 10127.7, 10263.7, 10391.9,
        10511.6, 10622.1, 10723.1, 10814.1, 10894.8, 10965.4, 11025.8, 11075, 11109.9, 11131.5,
        11139.1, 11127.4, 11079.5, 10993.9, 10869.3, 10707.1, 10511.3, 10287.5, 10042.6, 9783.87,
        9518.5, 9252.88, 8992.4, 8741.21, 8502.22, 8277.14, 8066.69, 7870.72, 7688.5, 7518.87,
        7360.42, 7209.45, 7064.52, 6924.25, 6787.58, 6653.67, 6521.96, 6392.02, 6263.61, 6136.56,
        6010.8, 5886.31, 5763.1, 5641.19, 5520.62, 5379.15, 5222.66, 5064.42, 4907.35, 4753.31,
        4603.4, 4458.15, 4317.76, 4182.18, 4051.27, 3924.82, 3802.6, 3684.43, 3570.15, 3459.62,
        3352.78, 3249.57, 3149.98, 3054.03, 2961.73, 2873.14, 2788.3, 2707.25, 2630.04, 2556.68,
        2487.21, 2421.6, 2359.84, 2301.9, 2247.7, 2197.16, 2150.2, 2106.69, 2066.5, 2029.49,
        1995.52, 1964.42, 1936.03, 1910.19, 1886.73, 1865.5, 1846.32, 1829.04, 1813.51, 1799.59,
        1787.14, 1776.03, 1766.14, 1757.35, 1749.56, 1742.67
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xprimary_coal_extraction_de_castro_phd_x_scen_xmadcoalxx(x):
    """
    XPrimary coal extraction de Castro PhD X Scen XmadcoalXX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2058.92, 2092.27, 2125.72, 2159.47, 2193.76, 2228.81, 2264.86, 2302.13, 2340.84, 2381.18,
        2423.34, 2467.47, 2513.71, 2562.15, 2612.87, 2665.92, 2721.31, 2779.03, 2839.06, 2901.33,
        2965.74, 3032.57, 3101.67, 3172.97, 3246.4, 3321.9, 3399.43, 3478.96, 3560.51, 3644.12,
        3729.86, 3816.11, 3897.75, 3974.81, 4046.62, 4112.97, 4174.06, 4230.44, 4282.94, 4332.57,
        4380.4, 4427.69, 4475.38, 4524.25, 4574.94, 4627.89, 4683.41, 4741.63, 4802.57, 4866.15,
        4932.23, 4997.04, 5059.9, 5120.05, 5176.96, 5230.21, 5279.47, 5324.46, 5364.9, 5400.52,
        5431.05, 5456.18, 5475.61, 5489.04, 5496.15, 5485.53, 5462.41, 5433, 5397.99, 5357.58,
        5311.76, 5260.45, 5203.57, 5141.1, 5073.11, 4999.72, 4921.11, 4837.56, 4749.39, 4656.95,
        4560.64, 4460.9, 4358.16, 4252.9, 4145.55, 4036.59, 3926.47, 3815.6, 3704.47, 3595.32,
        3489.22, 3386.16, 3286.01, 3188.63, 3093.87, 3001.58, 2911.63, 2823.91, 2738.31, 2654.74,
        2573.11, 2493.35, 2415.41, 2339.22, 2264.73, 2191.92, 2120.74, 2051.16, 1983.17, 1916.73,
        1851.84, 1788.49, 1726.66, 1666.35, 1607.54, 1550.24
    ])
    name = 2
    return round_sig(x=ret, name=name)

def xtpes_de_castro_phd_x_scen_iix(x):
    """
    XTPES de Castro PhD X Scen IIX



    lookup


    """

    ret =  functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        7749.13, 7949.4, 8160.47, 8369.05, 8575.33, 8779.44, 8981.51, 9181.61, 9379.72, 9575.75,
        9769.52, 9960.74, 10149, 10333.8, 10514.5, 10690.6, 10861.1, 11025.4, 11182.6, 11331.9,
        11472.6, 11613.1, 11744, 11868.2, 11985.7, 12096.8, 12202, 12302.1, 12398, 12490.6,
        12581.3, 12667.2, 12730, 12767.9, 12779.1, 12764.7, 12728.4, 12676, 12615, 12553.4,
        12499.7, 12465.2, 12453.3, 12470.3, 12519.7, 12603.9, 12723.6, 12878.5, 13067.2, 13287.8,
        13538.2, 13805, 14082.5, 14367, 14656.3, 14949, 15244.7, 15543.8, 15846.7, 16154.3,
        16467.1, 16785.7, 17110, 17439.8, 17774.5, 18113.2, 18354.1, 18638.3, 18929.8, 19227.6,
        19529.3, 19832.1, 20132.6, 20427.7, 20714.2, 20989.2, 21250.3, 21494.8, 21720.8, 21926.3,
        22109.7, 22269.4, 22404.4, 22513.5, 22596.2, 22651.7, 22679.9, 22680.6, 22664.5, 22677.2,
        22706.2, 22750.8, 22809.7, 22881.7, 22965.5, 23059.8, 23163.4, 23275.1, 23393.7, 23518,
        23647, 23779.6, 23914.5, 24050.7, 24187.2, 24322.8, 24456.4, 24587, 24713.6, 24835.1,
        24950.6, 25059.1, 25159.7, 25251.6, 25333.9, 25405.8
    ])
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def p_nuclear_scen_1():
    """
    P nuclear scen 1



    constant


    """

    ret =  0
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def xp_nuclear_2x3x():
    """
    XP nuclear 2X3X



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def gco2e_per_gtco2e_4():
    """
    gCO2e per GTCO2e 4



    constant


    """

    ret =  1 / 1000
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def gco2e_per_gtco2e_3():
    """
    gCO2e per GTCO2e 3



    constant


    """

    ret =  1 / 1000
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def gco2e_per_gtco2e_0():
    """
    gCO2e per GTCO2e 0



    constant


    """

    ret =  1 / 1000
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def gco2e_per_gtco2e_1():
    """
    gCO2e per GTCO2e 1



    constant


    """

    ret =  1 / 1000
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def twh_per_gco2e_0():
    """
    TWh per gCO2e 0



    constant


    """

    ret =  0.02
    name = 2
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
# ECON- Final demands by sector, households and fuel
@cache('step')
def real_demand_by_sector():
    """
    Real demand by sector

    dollars

    component

    Real demand by sector (35 WIOD sectors). US$1995
    """

    ret =  np.maximum(
        0,
        ia_matrix().loc[{'sectors1':'agriculture hunting forestry and fishing'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'agriculture hunting forestry and fishing'}].values + ia_matrix().loc[{'sectors1':'mining and quarrying'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'mining and quarrying'}].values +
        ia_matrix().loc[{'sectors1':'food beverages and tobacco'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'food beverages and tobacco'}].values + ia_matrix().loc[{'sectors1':'textiles and textile products'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'textiles and textile products'}].values +
        ia_matrix().loc[{'sectors1':'leather leather and footwear'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'leather leather and footwear'}].values + ia_matrix().loc[{'sectors1':'wood and products of woood and cork'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'wood and products of woood and cork'}].values +
        ia_matrix().loc[{'sectors1':'pulp paper printing and publishing'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'pulp paper printing and publishing'}].values + ia_matrix().loc[{'sectors1':'coke refined petroleum and nuclear fuel'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'coke refined petroleum and nuclear fuel'}].values +
        ia_matrix().loc[{'sectors1':'chemicals and chemical products'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'chemicals and chemical products'}].values + ia_matrix().loc[{'sectors1':'rubber and plastics'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'rubber and plastics'}].values +
        ia_matrix().loc[{'sectors1':'other non metalic mineral'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'other non metalic mineral'}].values + ia_matrix().loc[{'sectors1':'basic metals and fabricated metal'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'basic metals and fabricated metal'}].values +
        ia_matrix().loc[{'sectors1':'machinery nec'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'machinery nec'}].values + ia_matrix().loc[{'sectors1':'electrical and optical equipment'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'electrical and optical equipment'}].values +
        ia_matrix().loc[{'sectors1':'transport equipment'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'transport equipment'}].values + ia_matrix().loc[{'sectors1':'manufacturing nec recycling'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'manufacturing nec recycling'}].values +
        ia_matrix().loc[{'sectors1':'electricity gas and water supply'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'electricity gas and water supply'}].values + ia_matrix().loc[{'sectors1':'construction'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'construction'}].values +
        ia_matrix().loc[{'sectors1':'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'}].values + ia_matrix().loc[{'sectors1':'wholesale trade and commissions trade except of motor vehicles and motorcycles'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'wholesale trade and commissions trade except of motor vehicles and motorcycles'}].values +
        ia_matrix().loc[{'sectors1':'retail trade except of motor vehicles and motorcycles repair of household goods'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'retail trade except of motor vehicles and motorcycles repair of household goods'}].values + ia_matrix().loc[{'sectors1':'hotels and restaurants'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'hotels and restaurants'}].values +
        ia_matrix().loc[{'sectors1':'inland transport'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'inland transport'}].values + ia_matrix().loc[{'sectors1':'water transport'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'water transport'}].values +
        ia_matrix().loc[{'sectors1':'air transport'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'air transport'}].values + ia_matrix().loc[{'sectors1':'other supporting and auxiliary transport activities activities of travel agencies'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'other supporting and auxiliary transport activities activities of travel agencies'}].values +
        ia_matrix().loc[{'sectors1':'post and telecommunications'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'post and telecommunications'}].values + ia_matrix().loc[{'sectors1':'financial intermedation'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'financial intermedation'}].values +
        ia_matrix().loc[{'sectors1':'real estate activities'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'real estate activities'}].values + ia_matrix().loc[{'sectors1':'renting od meq and other business activities'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'renting od meq and other business activities'}].values +
        ia_matrix().loc[{'sectors1':'public admin and defence compulsory social security'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'public admin and defence compulsory social security'}].values + ia_matrix().loc[{'sectors1':'education'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'education'}].values +
        ia_matrix().loc[{'sectors1':'health and social work'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'health and social work'}].values + ia_matrix().loc[{'sectors1':'other community social and persona services'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'other community social and persona services'}].values +
        ia_matrix().loc[{'sectors1':'private households with employed persons'}].drop('sectors1') * real_total_output_by_sector().loc[{'sectors':'private households with employed persons'}].values)
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def required_total_output_by_sector():
    """
    Required total output by sector



    component


    """

    ret =  leontief_matrix().loc[{'sectors1':'agriculture hunting forestry and fishing'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'agriculture hunting forestry and fishing'}].values + leontief_matrix().loc[{'sectors1':'mining and quarrying'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'mining and quarrying'}].values + leontief_matrix().loc[{'sectors1':'food beverages and tobacco'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'food beverages and tobacco'}].values + leontief_matrix().loc[{'sectors1':'textiles and textile products'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'textiles and textile products'}].values + leontief_matrix().loc[{'sectors1':'leather leather and footwear'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'leather leather and footwear'}].values + leontief_matrix().loc[{'sectors1':'wood and products of woood and cork'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'wood and products of woood and cork'}].values + leontief_matrix().loc[{'sectors1':'pulp paper printing and publishing'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'pulp paper printing and publishing'}].values + leontief_matrix().loc[{'sectors1':'coke refined petroleum and nuclear fuel'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'coke refined petroleum and nuclear fuel'}].values + leontief_matrix().loc[{'sectors1':'chemicals and chemical products'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'chemicals and chemical products'}].values + leontief_matrix().loc[{'sectors1':'rubber and plastics'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'rubber and plastics'}].values + leontief_matrix().loc[{'sectors1':'other non metalic mineral'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'other non metalic mineral'}].values + leontief_matrix().loc[{'sectors1':'basic metals and fabricated metal'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'basic metals and fabricated metal'}].values + leontief_matrix().loc[{'sectors1':'machinery nec'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'machinery nec'}].values + leontief_matrix().loc[{'sectors1':'electrical and optical equipment'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'electrical and optical equipment'}].values + leontief_matrix().loc[{'sectors1':'transport equipment'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'transport equipment'}].values + leontief_matrix().loc[{'sectors1':'manufacturing nec recycling'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'manufacturing nec recycling'}].values + leontief_matrix().loc[{'sectors1':'electricity gas and water supply'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'electricity gas and water supply'}].values + leontief_matrix().loc[{'sectors1':'construction'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'construction'}].values + leontief_matrix().loc[{'sectors1':'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'}].values + leontief_matrix().loc[{'sectors1':'wholesale trade and commissions trade except of motor vehicles and motorcycles'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'wholesale trade and commissions trade except of motor vehicles and motorcycles'}].values + leontief_matrix().loc[{'sectors1':'retail trade except of motor vehicles and motorcycles repair of household goods'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'retail trade except of motor vehicles and motorcycles repair of household goods'}].values + leontief_matrix().loc[{'sectors1':'hotels and restaurants'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'hotels and restaurants'}].values + leontief_matrix().loc[{'sectors1':'inland transport'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'inland transport'}].values + leontief_matrix().loc[{'sectors1':'water transport'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'water transport'}].values + leontief_matrix().loc[{'sectors1':'air transport'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'air transport'}].values + leontief_matrix().loc[{'sectors1':'other supporting and auxiliary transport activities activities of travel agencies'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'other supporting and auxiliary transport activities activities of travel agencies'}].values + leontief_matrix().loc[{'sectors1':'post and telecommunications'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'post and telecommunications'}].values + leontief_matrix().loc[{'sectors1':'financial intermedation'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'financial intermedation'}].values + leontief_matrix().loc[{'sectors1':'real estate activities'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'real estate activities'}].values + leontief_matrix().loc[{'sectors1':'renting od meq and other business activities'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'renting od meq and other business activities'}].values + leontief_matrix().loc[{'sectors1':'public admin and defence compulsory social security'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'public admin and defence compulsory social security'}].values + leontief_matrix().loc[{'sectors1':'education'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'education'}].values + leontief_matrix().loc[{'sectors1':'health and social work'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'health and social work'}].values + leontief_matrix().loc[{'sectors1':'other community social and persona services'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'other community social and persona services'}].values + leontief_matrix().loc[{'sectors1':'private households with employed persons'}].drop('sectors1') * demand_by_sector().loc[{'sectors':'private households with employed persons'}].values
    name = 2
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def rate_change_intensity_bottom_up():
    """
    rate change intensity BOTTOM UP



    constant


    """

    ret =  xr.DataArray(
        data=np.zeros((35, 5)),
        coords={'sectors':['agriculture hunting forestry and fishing','mining and quarrying','food beverages and tobacco','textiles and textile products','leather leather and footwear','wood and products of woood and cork','pulp paper printing and publishing','coke refined petroleum and nuclear fuel','chemicals and chemical products','rubber and plastics','other non metalic mineral','basic metals and fabricated metal','machinery nec','electrical and optical equipment','transport equipment','manufacturing nec recycling','electricity gas and water supply','construction','sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','wholesale trade and commissions trade except of motor vehicles and motorcycles','retail trade except of motor vehicles and motorcycles repair of household goods','hotels and restaurants','inland transport','water transport','air transport','other supporting and auxiliary transport activities activities of travel agencies','post and telecommunications','financial intermedation','real estate activities','renting od meq and other business activities','public admin and defence compulsory social security','education','health and social work','other community social and persona services','private households with employed persons'], 'final sources':['electricity','heat','liquids','gases','solids']},
        dims=['sectors','final sources'])
    data = [[0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],

        functions.if_then_else(abs(activate_bottom_up_method().loc[{'sectors':'inland transport'}].values - 1) < equality_threshold,
                               inland_transport_variation_intensity(), 0),
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0],
        [0.0,0.0,0.0,0.0,0.0]]
    ret.values = list(map(lambda x: x if type(x) != xr.DataArray else xr.align(ret, x, join='left')[1], data))
    name = 2
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
@cache('step')
def evol_final_energy_intensity_by_sector_and_fuel():
    """
    Evol final energy intensity by sector and fuel



    component


    """


    ret =  integ_evol_final_energy_intensity_by_sector_and_fuel()

    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def rurr_unconv_oil_ej():
    """
    RURR unconv oil EJ



    component


    """


    ret =  integ_rurr_unconv_oil_ej()
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Alternative transport
@cache('step')
def pe_elec_for_transp_ej():
    """
    PE Elec for Transp EJ



    component


    """

    ret =  current_trends_demand_elec_for_transport() + pe_losses_related_to_elec_transp(
    ) + increase_elec_demand_for_transp_ej()
    name = 2
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
# TRANSP - Transport energy demand
# TRANSP - Alternative transport
@cache('step')
def oil_liquids_saved_by_evs_ej():
    """
    Oil liquids saved by EVs EJ



    component


    """

    ret =  functions.if_then_else(abs(activate_transport_h_bottom_up_method() ) < equality_threshold,
                                  liquid_demand_transport_ej() * (bev_share() +
                                                                  (1 / 3) * hev_share()), 0)
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
# TRANSP - Transport energy demand
# TRANSP - Alternative transport
@cache('step')
def oil_liquids_saved_by_ngvs():
    """
    Oil liquids saved by NGVs



    component


    """

    ret =  functions.if_then_else(abs(activate_transport_h_bottom_up_method() ) < equality_threshold,
                                  ngv_share() * liquid_demand_transport_ej(), 0)
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
# ELEC - RES elec supply
# ELEC - RES elec supply
# ELEC - Electricity demand covered by fossil fuels
# ELEC - Total FE electricity generation
# COMP- Historical data Elec production
@cache('step')
def total_fe_elec_demand_twh():
    """
    Total FE Elec demand TWh



    component


    """

    ret =  (fe_demand_elec_consum_twh() + increase_elec_demand_for_transp_twh()) * (
        1 + xshare_transmxdistr_elec_lossesx())
    name = 2
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
@cache('step')
def share_liquids_for_final_energy():
    """
    share liquids for final energy



    component


    """

    ret =  functions.zidz(
        required_fed_by_liquids_ej(),
        (oil_liquids_saved_by_evs_ej() + ped_liquids_ej() -
         xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'liquids'}].values - other_liquids_required_ej()))
    name = 2
    return round_sig(x=ret, name=name)

# RES BIOE - Total biofuels liquids
# LIQUIDS - PED, PES & FES
# LIQUIDS - PED, PES & FES
# RESOURCES - CTL & GTL supply
@cache('step')
def ped_liquids_ej():
    """
    PED liquids EJ



    component


    """

    ret =  np.maximum(0,
                      required_fed_by_liquids_ej() + other_liquids_required_ej() +
                      pe_demand_oil_elec_plants_ej() + ped_oil_for_heat_plants_ej() +
                      ped_oil_for_chp_plants_ej() + xped_liquids_heatxncx() +
                      xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'liquids'}].values - oil_liquids_saved_by_evs_ej())
    name = 2
    return round_sig(x=ret, name=name)

# HEAT COM- Heat-com demand covered by fossil fuels
# RESOURCES - Oil extraction
# COMP- Historical data Heat generation
@cache('step')
def fed_heat_liquids_plants_ej():
    """
    FED Heat liquids plants EJ



    component


    """

    ret =  xfed_heatxcom_plants_fossil_fuels_ejx() * share_liquids_fot_heat_plants()
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def real_final_energy_by_sector_and_fuel():
    """
    Real final energy by sector and fuel



    component


    """

    ret =  required_final_energy_by_sector_and_fuel() * energy_scarcity_feedback_shortage_coeff(
    ) * cc_impacts_feedback_shortage_coeff()
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Inland transport sector
@cache('run')
def hist_var_inlandt():
    """
    hist var inlandT



    constant


    """

    ret =  xr.DataArray(
        data=[0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
        coords={
            'vehiclet': [
                'hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq',
                'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec'
            ]
        },
        dims=['vehiclet'])
    name = 2
    return round_sig(x=ret, name=name)

# FE INTENSITIES by sector and fuel
# TRANSP - Inland transport sector
@cache('step')
def inland_transport_variation_intensity():
    """
    inland transport variation intensity



    component


    """

    ret =  xr.DataArray(
        data=[var_i_inland_elec(),
        0.0,

        var_i_inlandt_liq(),
        0.0,

        var_i_inlandt_gas()],
        coords={'final sources':['electricity','heat','liquids','solids','gases']},
        dims=['final sources'])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def max_unconv_oil_growth_extraction():
    """
    max unconv oil growth extraction



    component


    """

    ret =  1 + (functions.if_then_else(
        abs(selection_constraint_extraction_unconv_oil() - 1) < equality_threshold,
        (p_constraint_growth_extraction_unconv_oil()) * time_step() * scarcity_conv_oil_stock(),
        (xuserxdefined_extraction_growth_unconv_oilx(time()) * time_step()))
                ) * abundance_unconv_oil2()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_heatxcom_nre_ejx():
    """
    XFED HeatXcom NRE EJX



    component


    """

    ret =  np.maximum(
        xfed_heatxcom_after_priorities_ejx() - xtotal_fe_real_supply_res_for_heatxcom_ejx(), 0)
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# TFES by final fuel
@cache('step')
def real_fe_consumption_by_fuel():
    """
    real FE consumption by fuel



    component


    """
    ret =  xr.DataArray(
        data=[test2() + 0 * total_fe_elec_consumption_ej(),

        total_fe_heat_consumption_ej(),

        real_fe_consumption_liquids_ej(),

        real_fe_consumption_solids_ej(),

        real_fe_consumption_gases_ej()],
        coords={'final sources':['electricity','heat','liquids','solids','gases']},
        dims=['final sources'])
    name = 2
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
@cache('step')
def other_liquids_required_ej():
    """
    Other liquids required EJ



    component


    """

    ret =  losses_ej().loc[{'final sources':'liquids'}].values + transformation_losses_ej().loc[{'final sources':'liquids'}].values + xnonxenergy_use_demand_by_final_fuel_ejx().loc[{'final sources':'liquids'}].values
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
@cache('step')
def fe_elec_demand_consum_after_cc_impacts_ej():
    """
    FE Elec demand consum after CC impacts EJ



    component


    """

    ret =  required_fed_by_fuel().loc[{'final sources':'electricity'}].values + xexlosses_by_final_fuel_due_to_cc_impactsx().loc[{'final sources':'electricity'}].values
    name = 2
    return round_sig(x=ret, name=name)

# LIQUIDS - PED, PES & FES
@cache('step')
def required_fed_by_liquids_ej():
    """
    Required FED by liquids EJ



    component


    """

    ret =  required_fed_by_fuel().loc[{'final sources':'liquids'}].values
    name = 2
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def real_gfcf():
    """
    Real GFCF



    component


    """

    ret =  real_demand_by_sector() * (
        1 - share_consum_goverment_and_inventories()) * xpct_gfcf_vs_gfcfxhdx()
    name = 2
    return round_sig(x=ret, name=name)

# ECON DEM - WIOD sectors and households
@cache('step')
def real_household_demand():
    """
    Real Household demand



    component


    """

    ret =  real_demand_by_sector() * (1 - share_consum_goverment_and_inventories()) * (
        1 - xpct_gfcf_vs_gfcfxhdx())
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def extraction_unconv_oil_ej():
    """
    extraction unconv oil EJ



    component


    """

    ret =  np.minimum(
        functions.if_then_else(single_precision(0) > single_precision(rurr_unconv_oil_ej()) + less_thr_func(0, rurr_unconv_oil_ej()), 0,
                               functions.if_then_else(
                                   time() > 2012,
                                   functions.if_then_else(
                                       abs(xseparate_conv_and_unconv_oilxx() - 1) < equality_threshold,
                                       np.minimum(max_extraction_unconv_oil(),
                                                  max_unconv_oil_growth_extraction_ej()), 0),
                                   historic_unconv_oil(time()))), ped_total_oil_ej())
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def demand_conv_oil_ej():
    """
    Demand conv oil EJ



    component


    """

    ret =  np.maximum(ped_total_oil_ej() - extraction_unconv_oil_ej(), 0)
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def max_unconv_oil_growth_extraction_ej():
    """
    max unconv oil growth extraction EJ



    component


    """

    ret =  functions.if_then_else(single_precision(0) > single_precision(check_liquids_delayed_1yr()) + less_thr_func(0, check_liquids_delayed_1yr()), (1 + xconstrain_liquids_exogenous_growthx_delayed_1yrx())
        * extraction_unconv_oil_delayed(),
        extraction_unconv_oil_delayed() * max_unconv_oil_growth_extraction())
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xfed_heatxcom_ejx():
    """
    XFED HeatXcom EJX



    component


    """

    ret =  xrequired_heatxcom_after_cc_impactsx()
    name = 2
    return round_sig(x=ret, name=name)

# ELEC - Electricity demand
@cache('step')
def fe_demand_elec_consum_twh():
    """
    FE demand Elec consum TWh



    component


    """

    ret =  fe_elec_demand_consum_after_cc_impacts_ej() / ej_per_twh()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def xtotal_fed_heatxcom_ejx():
    """
    XTotal FED HeatXcom EJX



    component


    """

    ret =  xfed_heatxcom_ejx() * (1 + share_heat_distribution_losses())
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def real_total_output_by_fuel_and_sector():
    """
    Real total output by fuel and sector



    component


    """

    ret =  functions.xidz(real_final_energy_by_sector_and_fuel(),
                          final_energy_intensity_by_sector_and_fuel(),
                          required_total_output_by_sector() / 1e+006) * 1e+006
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# TRANSP - Transport energy demand
@cache('step')
def required_final_energy_by_sector_and_fuel():
    """
    Required final energy by sector and fuel



    component


    """

    ret =  required_total_output_by_sector() * final_energy_intensity_by_sector_and_fuel() / 1e+006
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def urr_unconv_oil():
    """
    URR unconv oil



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_oilxx() - 1) < equality_threshold,
                                  functions.if_then_else(
                                      abs(choose_extraction_curve_unconv_oil() - 1) < equality_threshold,
                                      urr_unconv_oil_bg_mohr15(),
                                      functions.if_then_else(
                                          abs(choose_extraction_curve_unconv_oil() - 2) < equality_threshold,
                                          urr_unconv_oil_low_mohr15(),
                                          functions.if_then_else(
                                              abs(choose_extraction_curve_unconv_oil() - 3) < equality_threshold,
                                              urr_unconv_oil_high_mohr15(),
                                              urr_unconv_oil_user_defined()))), 0)
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def max_extraction_unconv_gas():
    """
    max extraction unconv gas



    component


    """

    ret =  functions.if_then_else(
        abs(choose_extraction_curve_unconv_gas() - 1) < equality_threshold,
        table_max_extraction_unconv_gas_bg_mohr15(tot_rurr_unconv_gas()),
        functions.if_then_else(abs(choose_extraction_curve_unconv_gas() - 2) < equality_threshold,
                               table_max_extraction_unconv_gas_low_mohr15(tot_rurr_unconv_gas()),
                               functions.if_then_else(abs(choose_extraction_curve_unconv_gas() - 3) < equality_threshold,
                                                      table_max_extraction_unconv_gas_high_mohr15(
                                                          tot_rurr_unconv_gas()),
                                                      table_max_extraction_unconv_gas_user_defined(
                                                          tot_rurr_unconv_gas()))))
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def max_extraction_unconv_oil():
    """
    max extraction unconv oil



    component


    """

    ret =  functions.if_then_else(
        abs(choose_extraction_curve_unconv_oil() - 1) < equality_threshold,
        table_max_extraction_unconv_oil_bg_mohr15(tot_rurr_unconv_oil()),
        functions.if_then_else(abs(choose_extraction_curve_unconv_oil() - 2) < equality_threshold,
                               table_max_extraction_unconv_oil_low_mohr15(tot_rurr_unconv_oil()),
                               functions.if_then_else(abs(choose_extraction_curve_unconv_oil() - 3) < equality_threshold,
                                                      table_max_extraction_unconv_oil_high_mohr15(
                                                          tot_rurr_unconv_oil()),
                                                      table_max_extraction_unconv_oil_user_defined(
                                                          tot_rurr_unconv_oil()))))
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def max_extraction_coal_ej():
    """
    max extraction coal EJ



    component


    """

    ret =  functions.if_then_else(
        abs(choose_extraction_coal_curve() - 1) < equality_threshold,
        table_max_extraction_coal_mohr2012_ej(tot_rurr_coal()),
        functions.if_then_else(abs(choose_extraction_coal_curve() - 2) < equality_threshold,
                               table_max_extraction_coal_low_mohr15(tot_rurr_coal()),
                               functions.if_then_else(abs(choose_extraction_coal_curve() - 3) < equality_threshold,
                                                      table_max_extraction_coal_bg_mohr15(
                                                          tot_rurr_coal()),
                                                      functions.if_then_else(
                                                          abs(choose_extraction_coal_curve() - 4) < equality_threshold,
                                                          table_max_extraction_coal_high_mohr15(
                                                              tot_rurr_coal()),
                                                          table_max_extraction_coal_user_defined(
                                                              tot_rurr_coal())))))
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def max_extraction_conv_gas_ej():
    """
    max extraction conv gas EJ



    component


    """

    ret =  functions.if_then_else(
        abs(xseparate_conv_and_unconv_gasxx() - 1) < equality_threshold,
        functions.if_then_else(abs(choose_extraction_conv_gas_curve() - 1) < equality_threshold,
                               table_max_extraction_conv_gas_bg_mohr15(tot_rurr_conv_gas()),
                               functions.if_then_else(
                                   abs(choose_extraction_conv_gas_curve() - 2) < equality_threshold,
                                   table_max_extraction_conv_gas_low_mohr15(tot_rurr_conv_gas()),
                                   functions.if_then_else(
                                       abs(choose_extraction_conv_gas_curve() - 3) < equality_threshold,
                                       table_max_extraction_conv_gas_high_mohr15(
                                           tot_rurr_conv_gas()),
                                       table_max_extraction_conv_gas_user_defined(
                                           tot_rurr_conv_gas())))), 0)
    name = 2
    return round_sig(x=ret, name=name)

# HEAT COM- Heat-com demand covered by fossil fuels
# RESOURCES - Natural gas extraction
# COMP- Historical data Heat generation
@cache('step')
def fed_heat_gas_plants_ej():
    """
    FED Heat gas plants EJ



    component


    """

    ret =  xfed_heat_gasxcoal_ejx() * xshare_gasxxcoalxgasx_for_heat_plantsx(time())
    name = 2
    return round_sig(x=ret, name=name)

# HEAT COM- Heat-com demand covered by fossil fuels
# RESOURCES - Coal extraction
# COMP- Historical data Heat generation
@cache('step')
def fed_heat_coal_plants_ej():
    """
    FED Heat coal plants EJ



    component


    """

    ret =  xfed_heat_gasxcoal_ejx() * xshare_coalxcoalxgasx_for_heat_plantsx()
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('step')
def urr_coal():
    """
    URR coal



    component


    """

    ret =  functions.if_then_else(abs(xunlimited_nrexx() - 1) < equality_threshold,
                                  urr_coal_unlimited(),
                                  functions.if_then_else(
                                      abs(xunlimited_coalxx() - 1) < equality_threshold,
                                      urr_coal_unlimited(),
                                      functions.if_then_else(
                                          abs(choose_extraction_coal_curve() - 1) < equality_threshold,
                                          urr_coal_mohr2012_ej(),
                                          functions.if_then_else(
                                              abs(choose_extraction_coal_curve() - 2) < equality_threshold,
                                              urr_coal_low_mohr15(),
                                              functions.if_then_else(
                                                  abs(choose_extraction_coal_curve() - 3) < equality_threshold,
                                                  urr_coal_bg_mohr15(),
                                                  functions.if_then_else(
                                                      abs(choose_extraction_coal_curve() - 4) < equality_threshold,
                                                      urr_coal_high15(),
                                                      urr_coal_user_defined_ej()))))))
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def urr_conv_gas():
    """
    URR conv gas



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_gasxx() - 1) < equality_threshold,
                                  functions.if_then_else(
                                      abs(xunlimited_nrexx() - 1) < equality_threshold,
                                      urr_conv_gas_unlimited(),
                                      functions.if_then_else(
                                          abs(xunlimited_gasxx() - 1) < equality_threshold,
                                          urr_conv_gas_unlimited(),
                                          functions.if_then_else(
                                              abs(choose_extraction_conv_gas_curve() - 1) < equality_threshold,
                                              urr_conv_gas_bg_mohr15(),
                                              functions.if_then_else(
                                                  abs(choose_extraction_conv_gas_curve() - 2) < equality_threshold,
                                                  urr_conv_gas_low_mohr15(),
                                                  functions.if_then_else(
                                                      abs(choose_extraction_conv_gas_curve() - 3) < equality_threshold,
                                                      urr_conv_gas_high_mohr15(),
                                                      urr_conv_gas_user_defined()))))), 0)
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
@cache('step')
def real_demand_by_sector_delayed():
    """
    Real demand by sector delayed



    component


    """
    data = [delay_real_demand_by_sector_agriculture_hunting_forestry_and_fishing(),
        delay_real_demand_by_sector_mining_and_quarrying(),
        delay_real_demand_by_sector_food_beverages_and_tobacco(),
        delay_real_demand_by_sector_textiles_and_textile_products(),
        delay_real_demand_by_sector_leather_leather_and_footwear(),
        delay_real_demand_by_sector_wood_and_products_of_woood_and_cork(),
        delay_real_demand_by_sector_pulp_paper_printing_and_publishing(),
        delay_real_demand_by_sector_coke_refined_petroleum_and_nuclear_fuel(),
        delay_real_demand_by_sector_chemicals_and_chemical_products(),
        delay_real_demand_by_sector_rubber_and_plastics(),
        delay_real_demand_by_sector_other_non_metalic_mineral(),
        delay_real_demand_by_sector_basic_metals_and_fabricated_metal(),
        delay_real_demand_by_sector_machinery_nec(),
        delay_real_demand_by_sector_electrical_and_optical_equipment(),
        delay_real_demand_by_sector_transport_equipment(),
        delay_real_demand_by_sector_manufacturing_nec_recycling(),
        delay_real_demand_by_sector_electricity_gas_and_water_supply(),
        delay_real_demand_by_sector_construction(),
        delay_real_demand_by_sector_sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(),
        delay_real_demand_by_sector_wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(),
        delay_real_demand_by_sector_retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(),
        delay_real_demand_by_sector_hotels_and_restaurants(),
        delay_real_demand_by_sector_inland_transport(),
        delay_real_demand_by_sector_water_transport(),
        delay_real_demand_by_sector_air_transport(),
        delay_real_demand_by_sector_other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(),
        delay_real_demand_by_sector_post_and_telecommunications(),
        delay_real_demand_by_sector_financial_intermedation(),
        delay_real_demand_by_sector_real_estate_activities(),
        delay_real_demand_by_sector_renting_od_meq_and_other_business_activities(),
        delay_real_demand_by_sector_public_admin_and_defence_compulsory_social_security(),
        delay_real_demand_by_sector_education(),
        delay_real_demand_by_sector_health_and_social_work(),
        delay_real_demand_by_sector_other_community_social_and_persona_services(),
        delay_real_demand_by_sector_private_households_with_employed_persons()]
    ret =  xr.DataArray(data=data, coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']}, dims=['sectors'], name ='init_delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step')
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def required_final_energy_other_transport():
    """
    Required final energy other transport



    component


    """

    ret =  required_final_energy_by_sector_and_fuel().loc[{'sectors':'other supporting and auxiliary transport activities activities of travel agencies'}].drop('sectors')
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
# TRANSP - Alternative transport
# ABUND - Abundances by sector
@cache('step')
def current_trends_demand_elec_for_transport():
    """
    Current trends demand Elec for transport



    component


    """

    ret =  required_final_energy_air_transport().loc[{'final sources':'electricity'}].values + required_final_energy_inland_transport().loc[{'final sources':'electricity'}].values + required_final_energy_other_transport().loc[{'final sources':'electricity'}].values + required_final_energy_water_transport().loc[{'final sources':'electricity'}].values
    name = 2
    return round_sig(x=ret, name=name)

# HEAT COM- Heat-com demand covered by fossil fuels
# GASES - PED, PES & FES
# COMP- Historical data II
@cache('step')
def ped_gases_for_heat_plants_ej():
    """
    PED gases for Heat plants EJ



    component


    """

    ret =  fed_heat_gas_plants_ej() / efficiency_gases_for_heat_plants(time())
    name = 2
    return round_sig(x=ret, name=name)

# HEAT COM- Heat-com demand covered by fossil fuels
# LIQUIDS - PED, PES & FES
# COMP- Historical data II
@cache('step')
def ped_oil_for_heat_plants_ej():
    """
    PED oil for Heat plants EJ



    component


    """

    ret =  fed_heat_liquids_plants_ej() / efficiency_liquids_for_heat_plants(time())
    name = 2
    return round_sig(x=ret, name=name)

# HEAT COM- Heat-com demand covered by fossil fuels
# SOLIDS - PED, PES & FES
# COMP- Historical data II
@cache('step')
def ped_coal_for_heat_plants_ej():
    """
    PED coal for Heat plants EJ



    component


    """

    ret =  fed_heat_coal_plants_ej() / efficiency_coal_for_heat_plants(time())
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# TRANSP - Inland transport sector
@cache('step')
def real_total_output_by_sector():
    """
    Real total output by sector



    component


    """

    ret =  np.minimum(real_total_output_by_fuel_and_sector().loc[{'final sources':'electricity'}].drop('final sources'),
                      np.minimum(real_total_output_by_fuel_and_sector().loc[{'final sources':'heat'}].drop('final sources'),
                                 np.minimum(real_total_output_by_fuel_and_sector().loc[{'final sources':'liquids'}].drop('final sources'),
                                            np.minimum(real_total_output_by_fuel_and_sector().loc[{'final sources':'gases'}].drop('final sources'),
                                                       real_total_output_by_fuel_and_sector().loc[{'final sources':'solids'}].drop('final sources')))))
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def required_final_energy_air_transport():
    """
    Required final energy air transport



    component


    """

    ret =  required_final_energy_by_sector_and_fuel().loc[{'sectors':'air transport'}].drop('sectors')
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def required_final_energy_inland_transport():
    """
    Required final energy inland transport



    component


    """

    ret =  required_final_energy_by_sector_and_fuel().loc[{'sectors':'inland transport'}].drop('sectors')
    name = 2
    return round_sig(x=ret, name=name)

# TRANSP - Transport energy demand
@cache('step')
def required_final_energy_water_transport():
    """
    Required final energy water transport



    component


    """

    ret =  required_final_energy_by_sector_and_fuel().loc[{'sectors':'water transport'}].drop('sectors')
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_conv_gas_low_mohr15(x):
    """
    table max extraction conv gas Low Mohr15



    lookup


    """

    ret =  functions.lookup(x, [0, 1100, 2200, 3300, 4400, 5500, 6600, 7700, 8800, 9900, 11000], [
        0.0, 49.9516317103108, 93.2254158071833, 118.25322742687901, 122.34766617760398,
        124.03503097023999, 124.03503097023999, 124.03503097023999, 124.03503097023999,
        124.03503097023999, 124.03503097023999
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_conv_gas_user_defined(x):
    """
    table max extraction conv gas User defined



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_total_gas_bg_mohr12(x):
    """
    table max extraction total gas BG Mohr12



    lookup


    """

    ret =  functions.lookup(x, [
        0, 1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000,
        15000, 16000, 17000, 18000, 19000, 20000
    ], [
        0.0, 27.4739, 42.1041, 51.2745, 58.1215, 69.3726, 79.5334, 99.1636, 115.575, 128.798,
        138.387, 147.465, 150.753, 157.008, 159.247, 159.247, 159.247, 159.247, 159.247, 159.247,
        159.247
    ])
    name = 2
    return round_sig(x=ret, name=name)

def table_max_extraction_total_gas_laherrere10(x):
    """
    table max extraction total gas Laherrere10



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000
    ], [
        0.0, 23.86208767023, 44.63101020582, 62.30666199473999, 77.77291011605999, 93.88909467,
        107.8214336676, 119.75242469669998, 129.03255377280001, 137.87016844320001, 144.16042095,
        148.91718678119997, 151.78032891450002, 152.89453583099998, 152.89453583099998,
        152.89453583099998, 152.89453583099998, 152.89453583099998, 152.89453583099998,
        152.89453583099998, 152.89453583099998, 152.89453583099998, 152.89453583099998,
        152.89453583099998, 152.89453583099998, 152.89453583099998, 152.89453583099998
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_total_gas_user_defined(x):
    """
    table max extraction total gas User defined



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_unconv_gas_bg_mohr15(x):
    """
    table max extraction unconv gas BG Mohr15



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 36.281, 59.2482, 79.7156, 92.6562, 104.84, 114.916, 121.97, 128.037, 132.221, 134.519,
        134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974,
        134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_unconv_gas_high_mohr15(x):
    """
    table max extraction unconv gas High Mohr15



    lookup


    """

    ret =  functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500, 24000
    ], [
        0.0, 83.3997, 135.51, 170.831, 191.804, 202.817, 205.615, 205.615, 205.615, 205.615,
        205.615, 205.615, 205.615, 205.615, 205.615, 205.615, 205.615
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_unconv_gas_low_mohr15(x):
    """
    table max extraction unconv gas Low Mohr15



    lookup


    """

    ret =  functions.lookup(x, [0, 400, 800, 1200, 1600, 2000, 2400, 2800],
                            [0.0, 0.0, 0.0, 0.0, 35.9832, 35.9832, 35.9832, 35.9832])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_unconv_gas_user_defined(x):
    """
    table max extraction unconv gas User defined



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Coal extraction

def table_max_extraction_coal_bg_mohr15(x):
    """
    table max extraction coal BG Mohr15



    lookup


    """

    ret =  functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500
    ], [
        0.0, 45.8067598, 101.45721979999999, 129.918137083, 129.918137083, 129.918137083,
        140.66041522999998, 172.65528881379996, 210.75409942999997, 245.94171830000005,
        245.94171830000005, 245.94171830000005, 245.94171830000005, 245.94171830000005,
        245.94171830000005, 245.94171830000005
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Coal extraction

def table_max_extraction_coal_high_mohr15(x):
    """
    table max extraction coal High Mohr15



    lookup


    """

    ret =  functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500, 24000, 25500, 27000, 28500, 30000, 31500
    ], [
        0.0, 49.7354913, 91.20222852385001, 93.95328151, 120.98636085000001, 142.6042858061,
        157.275271969, 170.16619920999997, 187.20167642, 198.71934311599998, 213.61074611700002,
        228.50740735999997, 252.196010948, 264.94099527149996, 272.123730091, 274.87091482169996,
        274.87091482169996, 274.87091482169996, 274.87091482169996, 274.87091482169996,
        274.87091482169996, 274.87091482169996
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('run')
def urr_coal_bg_mohr15():
    """
    URR coal BG Mohr15



    constant


    """

    ret =  22500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('run')
def urr_coal_user_defined_ej():
    """
    URR coal User defined EJ



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_conv_gas_bg_mohr15():
    """
    URR conv gas BG Mohr15



    constant


    """

    ret =  13200
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_conv_gas_high_mohr15():
    """
    URR conv gas High Mohr15



    constant


    """

    ret =  22500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_conv_gas_low_mohr15():
    """
    URR conv gas Low Mohr15



    constant


    """

    ret =  11000
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_conv_gas_user_defined():
    """
    URR conv gas User defined



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def urr_total_gas_laherrere10():
    """
    URR total gas Laherrere10



    constant


    """

    ret =  13000
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_total_gas_mohr12_bg():
    """
    URR total gas Mohr12 BG



    constant


    """

    ret =  20000
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_total_gas_user_defined():
    """
    URR total gas User defined



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('step')
def urr_unconv_gas():
    """
    URR unconv gas



    component


    """

    ret =  functions.if_then_else(abs(xseparate_conv_and_unconv_gasxx() - 1) < equality_threshold,
                                  functions.if_then_else(
                                      abs(choose_extraction_curve_unconv_gas() - 1) < equality_threshold,
                                      urr_unconv_gas_bg_mohr15(),
                                      functions.if_then_else(
                                          abs(choose_extraction_curve_unconv_gas() - 2) < equality_threshold,
                                          urr_unconv_gas_low_mohr15(),
                                          functions.if_then_else(
                                              abs(choose_extraction_curve_unconv_gas() - 3) < equality_threshold,
                                              urr_unconv_gas_high_mohr15(),
                                              urr_unconv_gas_user_defined()))), 0)
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_unconv_gas_bg_mohr15():
    """
    URR unconv gas BG Mohr15



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Coal extraction

def table_max_extraction_coal_low_mohr15(x):
    """
    table max extraction coal Low Mohr15



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 10.7961939, 41.4887626012, 69.58980656000001, 94.81256219, 106.66908508867104,
        119.606988, 140.56757997980003, 158.59254590300003, 179.93148888900004, 202.8910104403,
        214.784730211, 224.11729660400007, 224.4862273680001, 224.4862273680001, 224.4862273680001,
        224.4862273680001, 224.4862273680001, 224.4862273680001, 224.4862273680001,
        224.4862273680001, 224.4862273680001, 224.4862273680001, 224.4862273680001,
        224.4862273680001, 224.4862273680001, 224.4862273680001, 224.4862273680001,
        224.4862273680001, 224.4862273680001
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Coal extraction

def table_max_extraction_coal_mohr2012_ej(x):
    """
    table max extraction coal Mohr2012 EJ



    lookup


    """

    ret =  functions.lookup(x, [
        0, 2000, 4000, 6000, 8000, 10000, 12000, 14000, 16000, 18000, 20000, 22000, 24000, 26000,
        28000
    ], [
        0.0, 50.9743, 81.21, 91.3165, 96.3823, 109.46, 123.422, 148.171, 204.223, 216.291, 216.291,
        216.291, 216.291, 216.291, 216.291
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Coal extraction

def table_max_extraction_coal_user_defined(x):
    """
    table max extraction coal User defined



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_conv_gas_bg_mohr15(x):
    """
    table max extraction conv gas BG Mohr15



    lookup


    """

    ret =  functions.lookup(
        x, [0, 1200, 2400, 3600, 4800, 6000, 7200, 8400, 9600, 10800, 12000, 13200], [
            0.0, 53.7105, 94.1488, 124.386, 132.465, 134.77, 134.77, 134.77, 134.77, 134.77,
            134.77, 134.77
        ])
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Natural gas extraction

def table_max_extraction_conv_gas_high_mohr15(x):
    """
    table max extraction conv gas High Mohr15



    lookup


    """

    ret =  functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500
    ], [
        0.0, 85.11063547168709, 136.8937293509019, 179.4938495500182, 203.86285809423433,
        216.27987364278962, 218.76844181844524, 218.76844181844524, 218.76844181844524,
        218.76844181844524, 218.76844181844524, 218.76844181844524, 218.76844181844524,
        218.76844181844524, 218.76844181844524, 218.76844181844524
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_unconv_gas_low_mohr15():
    """
    URR unconv gas Low Mohr15



    constant


    """

    ret =  2800
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('run')
def urr_coal_high15():
    """
    URR coal High15



    constant


    """

    ret =  31500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('run')
def urr_coal_low_mohr15():
    """
    URR coal Low Mohr15



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_unconv_gas_high_mohr15():
    """
    URR unconv gas High Mohr15



    constant


    """

    ret =  24000
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Natural gas extraction
@cache('run')
def urr_unconv_gas_user_defined():
    """
    URR unconv gas User defined



    constant


    """

    ret =  14500
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Coal extraction
@cache('run')
def urr_coal_mohr2012_ej():
    """
    URR coal Mohr2012 EJ



    constant


    """

    ret =  27800
    name = 2
    return round_sig(x=ret, name=name)
# RESOURCES - Oil extraction

def table_max_extraction_unconv_oil_bg_mohr15(x):
    """
    table max extraction unconv oil BG Mohr15



    lookup


    """

    ret =  functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500
    ], [
        0.0, 11.1694, 20.7262, 46.1364, 67.4689, 93.0394, 111.958, 122.983, 129.802, 134.906,
        138.735, 138.735, 138.735, 138.735, 138.735, 138.735, 138.735, 138.735, 138.735, 138.735,
        138.735, 138.735
    ])
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def selection_constraint_extraction_unconv_oil():
    """
    Selection constraint extraction unconv oil



    constant


    """

    ret =  1
    name = 2
    return round_sig(x=ret, name=name)

# ECON- Final demands by sector, households and fuel
# ELEC - Total FE electricity generation
# TRANSP - Alternative transport
# TRANSP - Households transport
# TRANSP - Households transport
@cache('step')
def abundance_electricity():
    """
    Abundance electricity



    component


    """

    ret =  functions.if_then_else(
        total_fe_elec_generation_twh() > total_fe_elec_demand_twh(), 1,
        1 - functions.zidz(total_fe_elec_demand_twh() - total_fe_elec_generation_twh(),
                           total_fe_elec_demand_twh()))
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('run')
def p_constraint_growth_extraction_unconv_oil():
    """
    P constraint growth extraction unconv oil



    constant


    """

    ret =  0.02
    name = 2
    return round_sig(x=ret, name=name)

# RESOURCES - Oil extraction
@cache('step')
def extraction_unconv_oil_delayed():
    """
    extraction unconv oil delayed



    component


    """
    print(str(time()) + ' ({0:.2f}%)                '.format(100.*(time() - initial_time())/(final_time() - initial_time())), end='\r')

    ret =  delay_extraction_unconv_oil_ej_roundtime_step__time_step___time_step_109_time_step__time_step()
    name = 2
    return round_sig(x=ret, name=name)

@cache('run')
def final_time():
    """
    FINAL TIME



    constant


    """
    ret = 2050
    name = 2
    return ret

@cache('run')
def initial_time():
    """
    INITIAL TIME



    constant


    """

    ret =  1995
    name = 2
    return ret

@cache('step')
def saveper():
    """
    SAVEPER



    component


    """

    ret =  time_step()
    name = 2
    return ret

# DEM - Adjustment non-commercial heat demand
# E INTENSITIES - Aggregated indicators
# E INTENSITIES - Aggregated indicators
# E INTENSITIES - Aggregated indicators
# ELEC RES - Total monetary investments
# TRANSP - Households transport
# TRANSP - Households transport
# TRANSP - Households transport
# TRANSP - Households transport
# TRANSP - Inland transport sector
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Oil extraction
# RESOURCES - Natural gas extraction
# RESOURCES - Natural gas extraction
# RESOURCES - Natural gas extraction
# RESOURCES - Natural gas extraction
# RESOURCES - Coal extraction
# RESOURCES - Coal extraction
# RESOURCES - Coal extraction
# RESOURCES - Coal extraction
# LOSSES
# TPES
# EROItot - Electricity
# EROI system
# EROI system
# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
# IMPACTS - Total GHG emissions
@cache('run')
def time_step():
    """
    TIME STEP



    constant


    """

    ret =  0.125
    name = 2
    return ret

# Delay definitions

delay_tfec_intensity_until_2009_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: tfec_intensity_until_2009(), lambda: time_step(),
    lambda: 0, lambda: 1)

delay_annual_gdp_growth_rate_round3__time_step___time_step_0_3__time_step = functions.Delay(
    lambda: annual_gdp_growth_rate(), lambda: round(3 / time_step()) * time_step(), lambda: 0,
    lambda: 3 / time_step())

delay_annual_gdp_growth_rate_round1__time_step___time_step_0_1__time_step = functions.Delay(
    lambda: annual_gdp_growth_rate(), lambda: round(1 / time_step()) * time_step(), lambda: 0,
    lambda: 1 / time_step())

delay_annual_gdp_growth_rate_round2__time_step___time_step_0_2__time_step = functions.Delay(
    lambda: annual_gdp_growth_rate(), lambda: round(2 / time_step()) * time_step(), lambda: 0,
    lambda: 2 / time_step())

delay_fe_tot_generation_all_res_elec_twh_round1__time_step___time_step_2464_1__time_step = functions.Delay(
    lambda: fe_tot_generation_all_res_elec_twh(), lambda: round(1 / time_step()) * time_step(),
    lambda: 2464, lambda: 1 / time_step())

delay_fes_res_for_heat_ej_round1__time_step___time_step_3488_1__time_step = functions.Delay(
    lambda: fes_res_for_heat_ej(), lambda: round(1 / time_step()) * time_step(), lambda: 3.488,
    lambda: 1 / time_step())

delay_desired_gdp_round1__time_step___time_step_2916_1__time_step = functions.Delay(
    lambda: desired_gdp(), lambda: round(1 / time_step()) * time_step(), lambda: 29.16,
    lambda: 1 / time_step())

integ_desired_gdppc = functions.Integ(lambda: desired_variation_gdppc(),
                                      lambda: gdppc_initial_year())

delay_tfec_intensity_until_2009_without_eroi_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: tfec_intensity_until_2009_without_eroi(),
    lambda: time_step(), lambda: 0,
    lambda: 1)

delay_tpes_intensity_until_2009_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: tpes_intensity_until_2009(), lambda: time_step(),
    lambda: 0, lambda: 1)

delay_tfes_intensity_ej_tdolar_without_eroi_round1__time_step___time_step_8827_1__time_step = functions.Delay(
    lambda: tfes_intensity_ej_tdolar_without_eroi(), lambda: round(1 / time_step()) * time_step(),
    lambda: 8.827, lambda: 1 / time_step())

integ_cumulative_co2e_ghg_emissions = functions.Integ(lambda: total_ce_all_ghg(), lambda: 0)


def _init_evol_final_energy_intensity_h():
    """
    Implicit

    See docs for evol_final_energy_intensity_h

    setup

    Provides initial conditions for evol_final_energy_intensity_h function
    """

    ret =  initial_energy_intensity_1995_h()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _devol_final_energy_intensity_h_dt():
    """
    Implicit

    See docs for evol_final_energy_intensity_h

    component

    Provides derivative for evol_final_energy_intensity_h function
    """

    ret =  increase_of_intensity_due_to_energy_a_technology_change_h_top_down(
    ) + inertial_rate_energy_intensity_h_top_down(
    ) - decrease_of_intensity_due_to_energy_a_technology_change_h_top_down()
    name = 2
    return round_sig(x=ret, name=name)

# Integration definitions

integ_evol_final_energy_intensity_h = functions.Integ(
    lambda: _devol_final_energy_intensity_h_dt(), lambda: _init_evol_final_energy_intensity_h())

integ_installed_capacity_phs_tw = functions.Integ(
    lambda: phs_capacity_under_construction() - wear_phs(), lambda: initial_instal_cap_phs())


integ_phs_planned_capacity = functions.Integ(lambda: new_phs_capacity_under_planning()+replacement_capacity_phs()-phs_capacity_under_construction(), lambda: initial_capacity_in_construction_phs())

integ_required_capacity_phs = functions.Integ(
    lambda: new_required_phs_capacity() - new_phs_capacity_under_planning(),
    lambda: initial_required_capacity_phs())

integ_xbatteries_evxhibx2wex = functions.Integ(
    lambda: new_batteries() + replacement_batteries() - discarded_batteries(), lambda: 1)

integ_xbatteries_evxhibx2wex2 = functions.Integ(
    lambda: np.exp(5*(time()-initial_time()))/required_number_standard_batteries(), lambda: 0)

'''
delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step = functions.Delay(
    lambda: installed_capacity_res_elec_tw(), lambda: round(1 / time_step()) * time_step(),
    lambda: xr.DataArray(data=[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], coords={'res elec': ['hydro', 'xgeotxelecx', 'xsolid bioexelecx', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']}, dims=['res elec'], name ='init_delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step'), lambda: 1 / time_step())
'''
delay_installed_capacity_res_elec_tw_hydro = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['hydro'], lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())
delay_installed_capacity_res_elec_tw_xgeotxelecx = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['xgeotxelecx'], lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())
delay_installed_capacity_res_elec_tw_xsolid_bioexelecx = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['xsolid bioexelecx'], lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())
delay_installed_capacity_res_elec_tw_oceanic = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['oceanic'], lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())
delay_installed_capacity_res_elec_tw_wind_onshore = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['wind onshore'], lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())
delay_installed_capacity_res_elec_tw_wind_offshore = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['wind offshore'], lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())
delay_installed_capacity_res_elec_tw_solar_pv = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['solar pv'], lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())
delay_installed_capacity_res_elec_tw_csp = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['csp'], lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())
    #for x in ['hydro', 'xgeotxelecx', 'xsolid bioexelecx', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']


delay_rurr_coal_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_coal_start_year_plg(), lambda:  time_step(),
    lambda: 0, lambda: 1)

delay_rurr_tot_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_tot_gas_until_start_year_plg(),
    lambda:  time_step(), lambda: 0,
    lambda: 1)

delay_rurr_conv_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_conv_oil_until_start_year_plg(),
    lambda:  time_step(), lambda: 0,
    lambda: 1)

delay_rurr_unconv_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_unconv_oil_until_start_year_plg(),
    lambda:  time_step(), lambda: 0,
    lambda: 1)

delay_rurr_tot_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_tot_oil_until_start_year_plg(),
    lambda:  time_step(), lambda: 0,
    lambda: 1)

delay_rurr_conv_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_conv_gas_until_start_year_plg(),
    lambda:  time_step(), lambda: 0,
    lambda: 1)

delay_rurr_unconv_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_unconv_gas_until_start_year_plg(),
    lambda:  time_step(), lambda: 0,
    lambda: 1)

delay_overdemand_eroist_tot_until_2015_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: overdemand_eroist_tot_until_2015(),
    lambda:  time_step(), lambda: 0,
    lambda: 1)


integ_rurr_conv_gas = functions.Integ(lambda: -extraction_conv_gas_ej()-flow_conv_gas_left_in_ground(), lambda: urr_conv_gas()-cumulated_conv_gas_extraction_to_1995()*xseparate_conv_and_unconv_gasxx())


integ_rurr_unconv_gas = functions.Integ(lambda: -extraction_unconv_gas_ej()-flow_unconv_gas_left_in_ground(), lambda: urr_unconv_gas()-cumulated_unconv_gas_extraction_to_1995()*xseparate_conv_and_unconv_gasxx())


integ_rurr_conv_oil = functions.Integ(lambda: -extraction_conv_oil_ej()-flow_conv_oil_left_in_ground(), lambda: urr_conv_oil()-cumulated_conv_oil_extraction_to_1995()*xseparate_conv_and_unconv_oilxx())

delay_eroist_tot_round1__time_step___time_step_xeroistx1995xx_1__time_step = functions.Delay(
    lambda: eroist_tot(), lambda: round(1 / time_step()) * time_step(), lambda: xeroistx1995xx(),
    lambda: 1 / time_step())

delay_eroist_tot_roundtime_step__time_step___time_step_xeroistx1995xx_time_step__time_step = functions.Delay(
    lambda: eroist_tot(), lambda:  time_step(),
    lambda: xeroistx1995xx(), lambda: 1)

delay_eroist_until_2015_roundtime_step__time_step___time_step_xeroistx1995xx_time_step__time_step = functions.Delay(
    lambda: eroist_until_2015(), lambda:  time_step(),
    lambda: xeroistx1995xx(), lambda: 1)


def _init_recycling_rates_minerals_rest():
    """
    Implicit

    See docs for recycling_rates_minerals_rest

    setup

    Provides initial conditions for recycling_rates_minerals_rest function
    """

    ret =  current_recycling_rates_minerals() * xall_minerals_virginxx()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _drecycling_rates_minerals_rest_dt():
    """
    Implicit

    See docs for recycling_rates_minerals_rest

    component

    Provides derivative for recycling_rates_minerals_rest function
    """

    ret =  improvement_recycling_rates_minerals_rest()
    name = 2
    return round_sig(x=ret, name=name)

integ_recycling_rates_minerals_rest = functions.Integ(
    lambda: _drecycling_rates_minerals_rest_dt(), lambda: _init_recycling_rates_minerals_rest())


def _init_recycling_rates_minerals_alt_techn():
    """
    Implicit

    See docs for recycling_rates_minerals_alt_techn

    setup

    Provides initial conditions for recycling_rates_minerals_alt_techn function
    """

    ret =  current_recycling_rates_minerals() * xeolxrr_minerals_alt_techn_res_vsx_total_economyx(
    ) * xall_minerals_virginxx()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _drecycling_rates_minerals_alt_techn_dt():
    """
    Implicit

    See docs for recycling_rates_minerals_alt_techn

    component

    Provides derivative for recycling_rates_minerals_alt_techn function
    """

    ret =  improvement_recycling_rates_minerals_alt_techn()
    name = 2
    return round_sig(x=ret, name=name)

integ_recycling_rates_minerals_alt_techn = functions.Integ(
    lambda: _drecycling_rates_minerals_alt_techn_dt(),
    lambda: _init_recycling_rates_minerals_alt_techn())

@cache('run')
def _init_percents_h_vehicles():
    """
    Implicit

    See docs for percents_h_vehicles

    setup

    Provides initial conditions for percents_h_vehicles function
    """

    ret =  xr.DataArray(
            data=[1 - initial_2w_percent(), 0, 0, 0, initial_2w_percent(), 0],
            coords={'households vehicles': ['liq 4wheels','elec 4wheels','hib 4wheels','gas 4wheels','liq 2wheels','elec 2wheels']},
            dims=['households vehicles'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dpercents_h_vehicles_dt():
    """
    Implicit

    See docs for percents_h_vehicles

    component

    Provides derivative for percents_h_vehicles function
    """

    ret =  var_percents_h_vehicles()
    name = 2
    return round_sig(x=ret, name=name)

integ_percents_h_vehicles = functions.Integ(lambda: _dpercents_h_vehicles_dt(),
                                            lambda: _init_percents_h_vehicles())


def _init_percent_t_vehicles():
    """
    Implicit

    See docs for percent_t_vehicles

    setup

    Provides initial conditions for percent_t_vehicles function
    """

    ret = initial_percent_t_vehicles()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dpercent_t_vehicles_dt():
    """
    Implicit

    See docs for percent_t_vehicles

    component

    Provides derivative for percent_t_vehicles function
    """

    ret =  var_percent_t_vehicles()
    name = 2
    return round_sig(x=ret, name=name)

integ_percent_t_vehicles = functions.Integ(lambda: _dpercent_t_vehicles_dt(),
                                           lambda: _init_percent_t_vehicles())


def _init_energy_intensity_of_households_transport():
    """
    Implicit

    See docs for energy_intensity_of_households_transport

    setup

    Provides initial conditions for energy_intensity_of_households_transport function
    """

    ret =  initial_energy_intensity_of_households_transport_2009()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _denergy_intensity_of_households_transport_dt():
    """
    Implicit

    See docs for energy_intensity_of_households_transport

    component

    Provides derivative for energy_intensity_of_households_transport function
    """

    ret =  variation_energy_intensity_of_households_transport()
    name = 2
    return round_sig(x=ret, name=name)

integ_energy_intensity_of_households_transport = functions.Integ(
    lambda: _denergy_intensity_of_households_transport_dt(),
    lambda: _init_energy_intensity_of_households_transport())

integ_max_percent_2_wheels = functions.Integ(lambda: rate_4w_to_2w(), lambda: initial_2w_percent())

integ_max_percent_4_wheels = functions.Integ(lambda: -rate_4w_to_2w(),
                                             lambda: 1 - initial_2w_percent())

delay_pes_oil_ej_round01__time_step___time_step_1395_01__time_step = functions.Delay(
    lambda: pes_oil_ej(), lambda: time_step(), lambda: 139.5,
    lambda: 1)


def _init_minerals_extraction_projection_rest_cte_rr():
    """
    Implicit

    See docs for minerals_extraction_projection_rest_cte_rr

    setup

    Provides initial conditions for minerals_extraction_projection_rest_cte_rr function
    """

    ret =  initial_minerals_extraction_rest() * mt_per_tonne()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dminerals_extraction_projection_rest_cte_rr_dt():
    """
    Implicit

    See docs for minerals_extraction_projection_rest_cte_rr

    component

    Provides derivative for minerals_extraction_projection_rest_cte_rr function
    """

    ret =  variation_minerals_extraction_rest()
    name = 2
    return round_sig(x=ret, name=name)

integ_minerals_extraction_projection_rest_cte_rr = functions.Integ(
    lambda: _dminerals_extraction_projection_rest_cte_rr_dt(),
    lambda: _init_minerals_extraction_projection_rest_cte_rr())

integ_potential_peavail_biofuels_land_marg_ej = functions.Integ(lambda: new_biofuels_land_marg(),
                                                                lambda: 0)

delay_tfec_res_ej_round1__time_step___time_step_43_1__time_step = functions.Delay(
    lambda: tfec_res_ej(), lambda: round(1 / time_step()) * time_step(), lambda: 43,
    lambda: 1 / time_step())

delay_tpe_from_res_ej_round1__time_step___time_step_4803_1__time_step = functions.Delay(
    lambda: tpe_from_res_ej(), lambda: round(1 / time_step()) * time_step(), lambda: 48.03,
    lambda: 1 / time_step())

delay_share_res_vs_tpes_round1__time_step___time_step_0123_1__time_step = functions.Delay(
    lambda: share_res_vs_tpes(), lambda: round(1 / time_step()) * time_step(), lambda: 0.123,
    lambda: 1 / time_step())

delay_share_res_vs_tfec_round1__time_step___time_step_01614_1__time_step = functions.Delay(
    lambda: share_res_vs_tfec(), lambda: round(1 / time_step()) * time_step(), lambda: 0.1614,
    lambda: 1 / time_step())


def _init_xinstalled_capacity_res_heatxnc_twx():
    """
    Implicit

    See docs for xinstalled_capacity_res_heatxnc_twx

    setup

    Provides initial conditions for xinstalled_capacity_res_heatxnc_twx function
    """

    ret =  xinitial_value_res_for_heatxncx() * xdeactivate_heat_dem_correctionxx()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dxinstalled_capacity_res_heatxnc_twx_dt():
    """
    Implicit

    See docs for xinstalled_capacity_res_heatxnc_twx

    component

    Provides derivative for xinstalled_capacity_res_heatxnc_twx function
    """

    ret =  xnew_res_capacity_for_heatxnc_twx() + xreplacement_res_for_heatxnc_twx(
    ) - xwear_res_capacity_for_heatxnc_twx()
    name = 2
    return round_sig(x=ret, name=name)

integ_xinstalled_capacity_res_heatxnc_twx = functions.Integ(
    lambda: _dxinstalled_capacity_res_heatxnc_twx_dt(),
    lambda: _init_xinstalled_capacity_res_heatxnc_twx())


def _init_installed_capacity_res_heat_tw_0():
    """
    Implicit

    See docs for installed_capacity_res_heat_tw_0

    setup

    Provides initial conditions for installed_capacity_res_heat_tw_0 function
    """

    ret =  initial_value_res_for_heat_0()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dinstalled_capacity_res_heat_tw_0_dt():
    """
    Implicit

    See docs for installed_capacity_res_heat_tw_0

    component

    Provides derivative for installed_capacity_res_heat_tw_0 function
    """

    ret =  xnew_res_capacity_for_heatxcom_tw_0x() + replacement_res_for_heat_tw_0(
    ) - wear_res_capacity_for_heat_tw_0()
    name = 2
    return round_sig(x=ret, name=name)

integ_installed_capacity_res_heat_tw_0 = functions.Integ(
    lambda: _dinstalled_capacity_res_heat_tw_0_dt(),
    lambda: _init_installed_capacity_res_heat_tw_0())


def _init_xnonxenergy_use_demand_by_final_fuel_ejx():
    """
    Implicit

    See docs for xnonxenergy_use_demand_by_final_fuel_ejx

    setup

    Provides initial conditions for xnonxenergy_use_demand_by_final_fuel_ejx function
    """

    ret =  xinitial_nonxenergy_usex()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dxnonxenergy_use_demand_by_final_fuel_ejx_dt():
    """
    Implicit

    See docs for xnonxenergy_use_demand_by_final_fuel_ejx

    component

    Provides derivative for xnonxenergy_use_demand_by_final_fuel_ejx function
    """

    ret =  xannual_variation_nonxenergy_usex()
    name = 2
    return round_sig(x=ret, name=name)

integ_xnonxenergy_use_demand_by_final_fuel_ejx = functions.Integ(
    lambda: _dxnonxenergy_use_demand_by_final_fuel_ejx_dt(),
    lambda: _init_xnonxenergy_use_demand_by_final_fuel_ejx())


def _init_cum_materials_to_extract_for_ev_batteries():
    """
    Implicit

    See docs for cum_materials_to_extract_for_ev_batteries

    setup

    Provides initial conditions for cum_materials_to_extract_for_ev_batteries function
    """

    ret = xr.DataArray(data=[initial_cumulated_material_requirements_for_ev_batteries_1995()]*58,
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dcum_materials_to_extract_for_ev_batteries_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_for_ev_batteries

    component

    Provides derivative for cum_materials_to_extract_for_ev_batteries function
    """

    ret =  total_materials_to_extract_for_ev_batteries_mt()
    name = 2
    return round_sig(x=ret, name=name)

integ_cum_materials_to_extract_for_ev_batteries = functions.Integ(
    lambda: _dcum_materials_to_extract_for_ev_batteries_dt(),
    lambda: _init_cum_materials_to_extract_for_ev_batteries())


def _init_cum_materials_requirements_for_ev_batteries():
    """
    Implicit

    See docs for cum_materials_requirements_for_ev_batteries

    setup

    Provides initial conditions for cum_materials_requirements_for_ev_batteries function
    """

    ret = xr.DataArray(data=[initial_cumulated_material_requirements_for_ev_batteries_1995()]*58,
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dcum_materials_requirements_for_ev_batteries_dt():
    """
    Implicit

    See docs for cum_materials_requirements_for_ev_batteries

    component

    Provides derivative for cum_materials_requirements_for_ev_batteries function
    """

    ret =  total_materials_required_for_ev_batteries()
    name = 2
    return round_sig(x=ret, name=name)

integ_cum_materials_requirements_for_ev_batteries = functions.Integ(
    lambda: _dcum_materials_requirements_for_ev_batteries_dt(),
    lambda: _init_cum_materials_requirements_for_ev_batteries())


def _init_cum_materials_to_extract_for_ev_batteries_from_2015():
    """
    Implicit

    See docs for cum_materials_to_extract_for_ev_batteries_from_2015

    setup

    Provides initial conditions for cum_materials_to_extract_for_ev_batteries_from_2015 function
    """

    ret = xr.DataArray(data=[initial_cumulated_material_requirements_for_ev_batteries_1995()]*58,
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dcum_materials_to_extract_for_ev_batteries_from_2015_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_for_ev_batteries_from_2015

    component

    Provides derivative for cum_materials_to_extract_for_ev_batteries_from_2015 function
    """

    ret =  total_materials_to_extract_for_ev_batteries_from_2015_mt()
    name = 2
    return round_sig(x=ret, name=name)

integ_cum_materials_to_extract_for_ev_batteries_from_2015 = functions.Integ(
    lambda: _dcum_materials_to_extract_for_ev_batteries_from_2015_dt(),
    lambda: _init_cum_materials_to_extract_for_ev_batteries_from_2015())
'''
delay_pes_fossil_fuel_extraction_by_hand = functions.Delay(
    lambda: pes_fossil_fuel_extraction(), lambda:  time_step(),
    lambda: xr.DataArray(data=[139, 101, 79, 0, 0], coords={'final sources': ['liquids','solids','gases','electricity','heat']}, dims=['final sources']), lambda: 1)
'''

delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_139_time_step__time_step = functions.Delay(
    lambda: pes_fossil_fuel_extraction().loc['liquids'].values, lambda:  time_step(),
    lambda: 139, lambda: 1)

delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_101_time_step__time_step = functions.Delay(
    lambda: pes_fossil_fuel_extraction().loc['solids'].values, lambda:  time_step(),
    lambda: 101, lambda: 1)

delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_79_time_step__time_step = functions.Delay(
    lambda: pes_fossil_fuel_extraction().loc['gases'].values, lambda:  time_step(),
    lambda: 79, lambda: 1)

delay_feh_coal_roundtime_step__time_step___time_step_187_time_step__time_step = functions.Delay(
    lambda: feh_coal(), lambda:  time_step(), lambda: 18.7,
    lambda: 1)

delay_tfes_intensity_ej_tdolar_round1__time_step___time_step_8827_1__time_step = functions.Delay(
    lambda: tfes_intensity_ej_tdolar(), lambda: round(1 / time_step()) * time_step(),
    lambda: 8.827, lambda: 1 / time_step())

delay_tpes_intensity_ej_tdolar_round1__time_step___time_step_1314_1__time_step = functions.Delay(
    lambda: tpes_intensity_ej_tdolar(), lambda: round(1 / time_step()) * time_step(),
    lambda: 13.14, lambda: 1 / time_step())

delay_feh_gas_roundtime_step__time_step___time_step_3409_time_step__time_step = functions.Delay(
    lambda: feh_gas(), lambda:  time_step(), lambda: 34.09,
    lambda: 1)

integ_scarcity_conv_gas_stock = functions.Integ(lambda: increase_scarcity_conv_gas(),
                                                lambda: 0.2502)

integ_ctl_potential_production = functions.Integ(
    lambda: replacement_ctl() + variation_ctl() - wear_ctl(), lambda: initial_ctl_production())

integ_gtl_potential_production = functions.Integ(
    lambda: replacement_gtl() + variation_gtl() - wear_gtl(), lambda: initial_gtl_production())

integ_abundance_unconv_oil_stock = functions.Integ(lambda: increase_abundance_unconv_oil(),
                                                   lambda: 1)

delay_abundance_unconv_oil_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: abundance_unconv_oil(), lambda: round(1 / time_step()) * time_step(), lambda: 1,
    lambda: 1 / time_step())

integ_scarcity_conv_oil_stock = functions.Integ(lambda: increase_scarcity_conv_oil(),
                                                lambda: 0.3989)

integ_share_variable_res_elec_generation_vs_total_gen = functions.Integ(
    lambda: increase_variable_res_share_elec_vs_total_generation(),
    lambda: initial_share_variable_res_elec_gen_vs_total())

integ_cumulated_invest_e_grid = functions.Integ(
    lambda: extra_monet_invest_to_cope_with_variable_elec_res(), lambda: 0)

delay_feh_oil_roundtime_step__time_step___time_step_3409_time_step__time_step = functions.Delay(
    lambda: feh_oil(), lambda:  time_step(), lambda: 34.09,
    lambda: 1)

delay_pe_traditional_biomass_ej_round1__time_step___time_step_30_1__time_step = functions.Delay(
    lambda: pe_traditional_biomass_ej(), lambda: round(1 / time_step()) * time_step(), lambda: 30,
    lambda: 1 / time_step())

integ_labour_share = functions.Integ(lambda: variation_labour_share(), lambda: 0.6374)


def _init_demand_by_sector_fd():
    """
    Implicit

    See docs for demand_by_sector_fd

    setup

    Provides initial conditions for demand_by_sector_fd function
    """

    ret =  initial_demand_by_sectot()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _ddemand_by_sector_fd_dt():
    """
    Implicit

    See docs for demand_by_sector_fd

    component

    Provides derivative for demand_by_sector_fd function
    """

    ret =  variation_demand_flow_fd() - demand_not_covered_by_sector_fd()
    name = 2
    return round_sig(x=ret, name=name)

integ_demand_by_sector_fd = functions.Integ(lambda: _ddemand_by_sector_fd_dt(),
                                            lambda: _init_demand_by_sector_fd())


def _init_gross_fixed_capital_formation():
    """
    Implicit

    See docs for gross_fixed_capital_formation

    setup

    Provides initial conditions for gross_fixed_capital_formation function
    """

    ret =  initial_gfcf()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dgross_fixed_capital_formation_dt():
    """
    Implicit

    See docs for gross_fixed_capital_formation

    component

    Provides derivative for gross_fixed_capital_formation function
    """

    ret =  variation_gfcf() - gfcf_not_covered()
    name = 2
    return round_sig(x=ret, name=name)

integ_gross_fixed_capital_formation = functions.Integ(
    lambda: _dgross_fixed_capital_formation_dt(), lambda: _init_gross_fixed_capital_formation())


def _init_household_demand():
    """
    Implicit

    See docs for household_demand

    setup

    Provides initial conditions for household_demand function
    """

    ret =  initial_household_demand()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dhousehold_demand_dt():
    """
    Implicit

    See docs for household_demand

    component

    Provides derivative for household_demand function
    """

    ret =  variation_household_demand() - household_demand_not_covered()
    name = 2
    return round_sig(x=ret, name=name)

integ_household_demand = functions.Integ(lambda: _dhousehold_demand_dt(),
                                         lambda: _init_household_demand())

delay_quality_of_electricity_2015_round003125__time_step___time_step_075_003125__time_step = functions.Delay(
    lambda: quality_of_electricity_2015(), lambda: time_step(),
    lambda: 0.75, lambda: 1)

delay_overdemand_elec_tot_until_2015_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: overdemand_elec_tot_until_2015(),
    lambda: time_step(), lambda: 0,
    lambda: 1)

delay_xsharexe_losses_cc_until_2015x_round003125__time_step___time_step_0_003125__time_step = functions.Delay(
    lambda: xsharexe_losses_cc_until_2015x(), lambda: time_step(),
    lambda: 0, lambda: 1)

delay_overdemand_elec_eroi_from_2015_round1__time_step___time_step_0_1__time_step = functions.Delay(
    lambda: overdemand_elec_eroi_from_2015(), lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())

integ_cc_total = functions.Integ(lambda: variation_cc() - cc_total_not_covered(),
                                 lambda: initial_cc_total())

integ_lc = functions.Integ(lambda: variation_lc() - lc_not_covered(), lambda: initial_lc_total())


def _init_cum_materials_to_extract_rest():
    """
    Implicit

    See docs for cum_materials_to_extract_rest

    setup

    Provides initial conditions for cum_materials_to_extract_rest function
    """

    ret = xr.DataArray(data=[initial_cumulated_material_requirements_for_rest_1995()]*58,
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dcum_materials_to_extract_rest_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_rest

    component

    Provides derivative for cum_materials_to_extract_rest function
    """

    ret =  materials_to_extract_rest_mt()
    name = 2
    return round_sig(x=ret, name=name)

integ_cum_materials_to_extract_rest = functions.Integ(
    lambda: _dcum_materials_to_extract_rest_dt(), lambda: _init_cum_materials_to_extract_rest())


def _init_cum_materials_to_extract_rest_from_2015():
    """
    Implicit

    See docs for cum_materials_to_extract_rest_from_2015

    setup

    Provides initial conditions for cum_materials_to_extract_rest_from_2015 function
    """

    ret = xr.DataArray(data=[initial_cumulated_material_requirements_for_rest_1995()]*58,
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dcum_materials_to_extract_rest_from_2015_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_rest_from_2015

    component

    Provides derivative for cum_materials_to_extract_rest_from_2015 function
    """

    ret =  materials_to_extract_rest_from_2015_mt()
    name = 2
    return round_sig(x=ret, name=name)

integ_cum_materials_to_extract_rest_from_2015 = functions.Integ(
    lambda: _dcum_materials_to_extract_rest_from_2015_dt(),
    lambda: _init_cum_materials_to_extract_rest_from_2015())


def _init_cum_materials_to_extract_for_res_elec_from_2015():
    """
    Implicit

    See docs for cum_materials_to_extract_for_res_elec_from_2015

    setup

    Provides initial conditions for cum_materials_to_extract_for_res_elec_from_2015 function
    """

    ret = xr.DataArray(data=[initial_cumulated_material_requirements_for_res_elec_1995()]*58,
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dcum_materials_to_extract_for_res_elec_from_2015_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_for_res_elec_from_2015

    component

    Provides derivative for cum_materials_to_extract_for_res_elec_from_2015 function
    """

    ret =  total_materials_to_extract_for_res_elec_from_2015_mt()
    name = 2
    return round_sig(x=ret, name=name)

integ_cum_materials_to_extract_for_res_elec_from_2015 = functions.Integ(
    lambda: _dcum_materials_to_extract_for_res_elec_from_2015_dt(),
    lambda: _init_cum_materials_to_extract_for_res_elec_from_2015())


def _init_cum_materials_to_extract_for_res_elec():
    """
    Implicit

    See docs for cum_materials_to_extract_for_res_elec

    setup

    Provides initial conditions for cum_materials_to_extract_for_res_elec function
    """

    ret = xr.DataArray(data=[initial_cumulated_material_requirements_for_res_elec_1995()]*58,
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dcum_materials_to_extract_for_res_elec_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_for_res_elec

    component

    Provides derivative for cum_materials_to_extract_for_res_elec function
    """

    ret =  total_materials_to_extract_for_res_elec_mt()
    name = 2
    return round_sig(x=ret, name=name)

integ_cum_materials_to_extract_for_res_elec = functions.Integ(
    lambda: _dcum_materials_to_extract_for_res_elec_dt(),
    lambda: _init_cum_materials_to_extract_for_res_elec())

integ_pes_waste_ej = functions.Integ(lambda: new_waste_supply_ej(), lambda: initial_pes_waste())


def _init_installed_capacity_res_elec_tw():
    """
    Implicit

    See docs for installed_capacity_res_elec_tw

    setup

    Provides initial conditions for installed_capacity_res_elec_tw function
    """

    ret =  initial_instal_cap_res_elec()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dinstalled_capacity_res_elec_tw_dt():
    """
    Implicit

    See docs for installed_capacity_res_elec_tw

    component

    Provides derivative for installed_capacity_res_elec_tw function
    """

    ret =  res_elec_capacity_under_construction_tw() - wear_res_elec()
    name = 2
    return round_sig(x=ret, name=name)

integ_installed_capacity_res_elec_tw = functions.Integ(
    lambda: _dinstalled_capacity_res_elec_tw_dt(), lambda: _init_installed_capacity_res_elec_tw())


def _init_required_capacity_res_elec_tw():
    """
    Implicit

    See docs for required_capacity_res_elec_tw

    setup

    Provides initial conditions for required_capacity_res_elec_tw function
    """

    ret =  initial_required_capacity_res_elec()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _drequired_capacity_res_elec_tw_dt():
    """
    Implicit

    See docs for required_capacity_res_elec_tw

    component

    Provides derivative for required_capacity_res_elec_tw function
    """

    ret =  new_required_capacity_res_elec() - new_res_elec_capacity_under_planning()
    name = 2
    return round_sig(x=ret, name=name)

integ_required_capacity_res_elec_tw = functions.Integ(
    lambda: _drequired_capacity_res_elec_tw_dt(), lambda: _init_required_capacity_res_elec_tw())


def _init_res_elec_planned_capacity_tw():
    """
    Implicit

    See docs for res_elec_planned_capacity_tw

    setup

    Provides initial conditions for res_elec_planned_capacity_tw function
    """

    ret =  initial_capacity_in_construction_res_elec()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dres_elec_planned_capacity_tw_dt():
    """
    Implicit

    See docs for res_elec_planned_capacity_tw

    component

    Provides derivative for res_elec_planned_capacity_tw function
    """

    ret =  new_res_elec_capacity_under_planning() + replacement_capacity_res_elec(
    ) - res_elec_capacity_under_construction_tw()
    name = 2
    return round_sig(x=ret, name=name)

integ_res_elec_planned_capacity_tw = functions.Integ(lambda: _dres_elec_planned_capacity_tw_dt(),
                                                     lambda: _init_res_elec_planned_capacity_tw())

delay_scarcity_conv_gas_round1__time_step___time_step_02502_1__time_step = functions.Delay(
    lambda: scarcity_conv_gas(), lambda: round(1 / time_step()) * time_step(), lambda: 0.2502,
    lambda: 1 / time_step())

delay_scarcity_conv_oil_round1__time_step___time_step_03989_1__time_step = functions.Delay(
    lambda: scarcity_conv_oil(), lambda: round(1 / time_step()) * time_step(), lambda: 0.3989,
    lambda: 1 / time_step())

integ_rurr_tot_agg_gas = functions.Integ(
    lambda: -extraction_tot_agg_gas_ej() - flow_tot_agg_gas_left_in_ground(),
    lambda: functions.if_then_else(abs(xseparate_conv_and_unconv_gasxx()) < equality_threshold, urr_tot_agg_gas() - cumulated_tot_agg_gas_extraction_to_1995(), 0)
    )

integ_total_agg_gas_left_in_ground = functions.Integ(lambda: flow_tot_agg_gas_left_in_ground(),
                                                     lambda: 0)

integ_cumulated_tot_agg_gas_extraction = functions.Integ(
    lambda: extraction_tot_agg_gas_ej(), lambda: cumulated_tot_agg_gas_extraction_to_1995())

integ_rurr_tot_agg_oil = functions.Integ(
    lambda: -extraction_tot_agg_oil_ej() - flow_tot_agg_oil_left_in_ground(),
    lambda: functions.if_then_else(abs(xseparate_conv_and_unconv_oilxx()) < equality_threshold, urr_tot_agg_oil() - cumulated_tot_agg_extraction_to_1995(), 0)
    )

integ_total_agg_oil_left_in_ground = functions.Integ(lambda: flow_tot_agg_oil_left_in_ground(),
                                                     lambda: 0)

integ_cumulated_tot_agg_oil_extraction = functions.Integ(
    lambda: extraction_tot_agg_oil_ej(), lambda: cumulated_tot_agg_extraction_to_1995())

integ_total_unconv_gas_left_in_ground = functions.Integ(lambda: flow_unconv_gas_left_in_ground(),
                                                        lambda: 0)

integ_total_unconv_oil_left_in_ground = functions.Integ(lambda: flow_unconv_oil_left_in_ground(),
                                                        lambda: 0)

integ_total_coal_left_in_ground = functions.Integ(lambda: flow_coal_left_in_ground(), lambda: 0)

integ_total_conv_gas_left_in_ground = functions.Integ(lambda: flow_conv_gas_left_in_ground(),
                                                      lambda: 0)

integ_total_conv_oil_left_in_ground = functions.Integ(lambda: flow_conv_oil_left_in_ground(),
                                                      lambda: 0)

integ_rurr_coal = functions.Integ(lambda: -extraction_coal_ej() - flow_coal_left_in_ground(),
                                  lambda: urr_coal() - cumulated_coal_extraction_to_1995())

integ_pes_biogas_ej = functions.Integ(lambda: new_pes_biogas(), lambda: 0.13135)

integ_xshare_transmxdistr_elec_lossesx = functions.Integ(
    lambda: xvariation_share_transmxdistr_elec_lossesx(),
    lambda: xshare_transmxdistr_elec_losses_initialx())


def _init_cum_materials_requirements_for_res_elec():
    """
    Implicit

    See docs for cum_materials_requirements_for_res_elec

    setup

    Provides initial conditions for cum_materials_requirements_for_res_elec function
    """

    ret = xr.DataArray(data=[initial_cumulated_material_requirements_for_res_elec_1995()]*58,
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'xelectricxelectronic componentsx', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'xover grid x15xxx', 'xover grid x5xxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dcum_materials_requirements_for_res_elec_dt():
    """
    Implicit

    See docs for cum_materials_requirements_for_res_elec

    component

    Provides derivative for cum_materials_requirements_for_res_elec function
    """

    ret =  total_materials_required_for_res_elec_mt()
    name = 2
    return round_sig(x=ret, name=name)

integ_cum_materials_requirements_for_res_elec = functions.Integ(
    lambda: _dcum_materials_requirements_for_res_elec_dt(),
    lambda: _init_cum_materials_requirements_for_res_elec())


def _init_xinstalled_capacity_res_heatxcom_twx():
    """
    Implicit

    See docs for xinstalled_capacity_res_heatxcom_twx

    setup

    Provides initial conditions for xinstalled_capacity_res_heatxcom_twx function
    """

    ret =  xinitial_value_res_for_heatxcomx()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _dxinstalled_capacity_res_heatxcom_twx_dt():
    """
    Implicit

    See docs for xinstalled_capacity_res_heatxcom_twx

    component

    Provides derivative for xinstalled_capacity_res_heatxcom_twx function
    """

    ret =  xnew_res_capacity_for_heatxcom_twx() + xreplacement_res_for_heatxcom_twx(
    ) - xwear_res_capacity_for_heatxcom_twx()
    name = 2
    return round_sig(x=ret, name=name)

integ_xinstalled_capacity_res_heatxcom_twx = functions.Integ(
    lambda: _dxinstalled_capacity_res_heatxcom_twx_dt(),
    lambda: _init_xinstalled_capacity_res_heatxcom_twx())


integ_installed_capacity_nuclear_tw = functions.Integ(lambda: nuclear_capacity_under_construction()-xnuclear_capacity_phasexoutx()-wear_nuclear(), lambda: initial_capacity_installed_nuclear())

integ_required_capacity_nuclear_tw = functions.Integ(
    lambda: new_required_capacity_nuclear() - new_nuclear_capacity_under_planning(),
    lambda: initial_required_capacity_nuclear())


integ_planned_nuclear_capacity_tw = functions.Integ(lambda: new_nuclear_capacity_under_planning()+replacement_nuclear_capacity()-nuclear_capacity_under_construction(), lambda: initial_capacity_in_construction_nuclear())

delay_gdp_round1__time_step___time_step_2916_1__time_step = functions.Delay(
    lambda: gdp(), lambda: round(1 / time_step()) * time_step(), lambda: 29.16,
    lambda: 1 / time_step())


integ_land_compet_biofuels_2gen_mha = functions.Integ(lambda: new_biofuels_2gen_land_compet()-land_shifted_to_biofuels_3gen(), lambda: initial_value_land_compet_biofuels_2gen_mha()*land_productivity_biofuels_2gen_ej_mha())

delay_xconstrain_gas_exogenous_growthxx_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: xconstrain_gas_exogenous_growthxx(), lambda: round(1 / time_step()) * time_step(),
    lambda: 1, lambda: 1 / time_step())

delay_xconstrain_liquids_exogenous_growthxx_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: xconstrain_liquids_exogenous_growthxx(), lambda: round(1 / time_step()) * time_step(),
    lambda: 1, lambda: 1 / time_step())

delay_check_liquids_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: check_liquids(), lambda: round(1 / time_step()) * time_step(), lambda: 1,
    lambda: 1 / time_step())

delay_check_gases_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: check_gases(), lambda: round(1 / time_step()) * time_step(), lambda: 1,
    lambda: 1 / time_step())

delay_share_variable_res_elec_generation_vs_total_round1__time_step___time_step_00071_1__time_step = functions.Delay(
    lambda: share_variable_res_elec_generation_vs_total(),
    lambda: round(1 / time_step()) * time_step(), lambda: 0.0071, lambda: 1 / time_step())

integ_efficiency_gas_for_electricity = functions.Integ(
    lambda: improvement_efficiency_gas_for_electricity(),
    lambda: initial_efficiency_gas_for_electricity() * percent_to_share())

integ_cumulated_conv_gas_extraction = functions.Integ(
    lambda: extraction_conv_gas_ej(), lambda: cumulated_conv_gas_extraction_to_1995())

integ_cumulated_unconv_gas_extraction = functions.Integ(
    lambda: extraction_unconv_gas_ej(), lambda: cumulated_unconv_gas_extraction_to_1995())

integ_rurr_uranium = functions.Integ(
    lambda: -extraction_uranium_ej(),
    lambda: urr_uranium() - cumulated_uranium_extraction_to_1995())

integ_cumulated_conv_oil_extraction = functions.Integ(
    lambda: extraction_conv_oil_ej(), lambda: cumulated_conv_oil_extraction_to_1995())

integ_cumulated_unconv_oil_extraction = functions.Integ(
    lambda: extraction_unconv_oil_ej(), lambda: cumulated_unconv_oil_extraction_to_1995())

integ_cumulated_coal_extraction = functions.Integ(lambda: extraction_coal_ej(),
                                                  lambda: cumulated_coal_extraction_to_1995())

integ_cumulated_uranium_extraction = functions.Integ(
    lambda: extraction_uranium_ej(), lambda: cumulated_uranium_extraction_to_1995())

integ_atmos_uocean_temp = functions.Integ(lambda: chg_a_uo_temp(),
                                          lambda: init_atmos_uocean_temp())

integ_deep_ocean_temp = functions.Integ(lambda: chg_do_temp(), lambda: init_deep_ocean_temp())

integ_co2_in_atmos = functions.Integ(lambda: co2_net_emiss() - co2_storage(),
                                     lambda: init_co2_in_atmos())

integ_land_compet_biofuels_3gen_mha = functions.Integ(lambda: land_shifted_to_biofuels_3gen(),
                                                      lambda: 0)

integ_potential_pe_cellulosic_biofuel_ej = functions.Integ(lambda: new_cellulosic_biofuels(),
                                                           lambda: 0)

integ_xpe_bioe_residues_for_heatxelec_ejx = functions.Integ(
    lambda: xnew_bioe_residues_for_heatxelecx(), lambda: 0)

integ_population = functions.Integ(lambda: pop_variation(), lambda: initial_population())

integ_ngv_share = functions.Integ(lambda: variation_share_ngv(), lambda: 0)

integ_hev_share = functions.Integ(lambda: variation_share_hev(), lambda: 0)

integ_bev_share = functions.Integ(lambda: variation_share_bev(), lambda: 0)

delay_extraction_unconv_gas_ej_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: extraction_unconv_gas_ej(), lambda:  time_step(),
    lambda: 0, lambda: 1)

integ_cumulated_total_monet_invest_res_for_elec = functions.Integ(
    lambda: total_monet_invest_res_for_elec_tdolar(), lambda: 0)

integ_total_cumulative_emissions_gtc = functions.Integ(lambda: new_c_gtc(),
                                                       lambda: cumulative_emissions_to_1995())


def _init_evol_final_energy_intensity_by_sector_and_fuel():
    """
    Implicit

    See docs for evol_final_energy_intensity_by_sector_and_fuel

    setup

    Provides initial conditions for evol_final_energy_intensity_by_sector_and_fuel function
    """

    ret =  initial_energy_intensity_by_fuel_and_sector_1995()
    name = 2
    return round_sig(x=ret, name=name)

@cache('step')
def _devol_final_energy_intensity_by_sector_and_fuel_dt():
    """
    Implicit

    See docs for evol_final_energy_intensity_by_sector_and_fuel

    component

    Provides derivative for evol_final_energy_intensity_by_sector_and_fuel function
    """

    ret =  increase_of_intensity_due_to_energy_a_technology_change_top_down(
    ) + inertial_rate_energy_intensity_top_down() + rate_change_intensity_bottom_up(
    ) - decrease_of_intensity_due_to_energy_a_technology_change_top_down()
    name = 2
    return round_sig(x=ret, name=name)

integ_evol_final_energy_intensity_by_sector_and_fuel = functions.Integ(
    lambda: _devol_final_energy_intensity_by_sector_and_fuel_dt(),
    lambda: _init_evol_final_energy_intensity_by_sector_and_fuel())


integ_rurr_unconv_oil_ej = functions.Integ(lambda: -extraction_unconv_oil_ej()-flow_unconv_oil_left_in_ground(), lambda: urr_unconv_oil()-cumulated_unconv_oil_extraction_to_1995()*xseparate_conv_and_unconv_oilxx())

delay_real_demand_by_sector_agriculture_hunting_forestry_and_fishing = functions.Delay(
    lambda: real_demand_by_sector().loc['agriculture hunting forestry and fishing'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_mining_and_quarrying = functions.Delay(
    lambda: real_demand_by_sector().loc['mining and quarrying'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_food_beverages_and_tobacco = functions.Delay(
    lambda: real_demand_by_sector().loc['food beverages and tobacco'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_textiles_and_textile_products = functions.Delay(
    lambda: real_demand_by_sector().loc['textiles and textile products'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_leather_leather_and_footwear = functions.Delay(
    lambda: real_demand_by_sector().loc['leather leather and footwear'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_wood_and_products_of_woood_and_cork = functions.Delay(
    lambda: real_demand_by_sector().loc['wood and products of woood and cork'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_pulp_paper_printing_and_publishing = functions.Delay(
    lambda: real_demand_by_sector().loc['pulp paper printing and publishing'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_coke_refined_petroleum_and_nuclear_fuel = functions.Delay(
    lambda: real_demand_by_sector().loc['coke refined petroleum and nuclear fuel'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_chemicals_and_chemical_products = functions.Delay(
    lambda: real_demand_by_sector().loc['chemicals and chemical products'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_rubber_and_plastics = functions.Delay(
    lambda: real_demand_by_sector().loc['rubber and plastics'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_other_non_metalic_mineral = functions.Delay(
    lambda: real_demand_by_sector().loc['other non metalic mineral'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_basic_metals_and_fabricated_metal = functions.Delay(
    lambda: real_demand_by_sector().loc['basic metals and fabricated metal'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_machinery_nec = functions.Delay(
    lambda: real_demand_by_sector().loc['machinery nec'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_electrical_and_optical_equipment = functions.Delay(
    lambda: real_demand_by_sector().loc['electrical and optical equipment'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_transport_equipment = functions.Delay(
    lambda: real_demand_by_sector().loc['transport equipment'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_manufacturing_nec_recycling = functions.Delay(
    lambda: real_demand_by_sector().loc['manufacturing nec recycling'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_electricity_gas_and_water_supply = functions.Delay(
    lambda: real_demand_by_sector().loc['electricity gas and water supply'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_construction = functions.Delay(
    lambda: real_demand_by_sector().loc['construction'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel = functions.Delay(
    lambda: real_demand_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles = functions.Delay(
    lambda: real_demand_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods = functions.Delay(
    lambda: real_demand_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_hotels_and_restaurants = functions.Delay(
    lambda: real_demand_by_sector().loc['hotels and restaurants'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_inland_transport = functions.Delay(
    lambda: real_demand_by_sector().loc['inland transport'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_water_transport = functions.Delay(
    lambda: real_demand_by_sector().loc['water transport'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_air_transport = functions.Delay(
    lambda: real_demand_by_sector().loc['air transport'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies = functions.Delay(
    lambda: real_demand_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_post_and_telecommunications = functions.Delay(
    lambda: real_demand_by_sector().loc['post and telecommunications'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_financial_intermedation = functions.Delay(
    lambda: real_demand_by_sector().loc['financial intermedation'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_real_estate_activities = functions.Delay(
    lambda: real_demand_by_sector().loc['real estate activities'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_renting_od_meq_and_other_business_activities = functions.Delay(
    lambda: real_demand_by_sector().loc['renting od meq and other business activities'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_public_admin_and_defence_compulsory_social_security = functions.Delay(
    lambda: real_demand_by_sector().loc['public admin and defence compulsory social security'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_education = functions.Delay(
    lambda: real_demand_by_sector().loc['education'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_health_and_social_work = functions.Delay(
    lambda: real_demand_by_sector().loc['health and social work'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_other_community_social_and_persona_services = functions.Delay(
    lambda: real_demand_by_sector().loc['other community social and persona services'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_real_demand_by_sector_private_households_with_employed_persons = functions.Delay(
    lambda: real_demand_by_sector().loc['private households with employed persons'], lambda: round(1 / time_step()) * time_step(), lambda: 10.,
    lambda: 1 / time_step())

delay_extraction_unconv_oil_ej_roundtime_step__time_step___time_step_109_time_step__time_step = functions.Delay(
    lambda: extraction_unconv_oil_ej(), lambda:  time_step(),
    lambda: 1.09, lambda: 1)


# Playground

integ_my_random = functions.Integ(lambda: population(), lambda: initial_pes_waste())

@cache('step')   # updated on every time step
def my_random_function():
    return gdppc() / gdppc_annual_growth_ssp2(time()) + integ_my_random()

#@
