import re
import sys
import matplotlib.pyplot as plt
import numpy as np
from scipy import interpolate
import math
import os
import xarray as xr
import getopt
import pandas as pd
import xlsxwriter as xlw
from shutil import copyfile, move

import warnings
warnings.filterwarnings("ignore")

import pysd
import initglobals

# initialization of the global vars
try:
    initglobals.init()
except:
    print("Unexpected error:", sys.exc_info()[0])
    raise


_namespace = {
    'TIME':
    'time',
    'Time':
    'time',
    'Leontief Matrix':
    'leontief_matrix',
    'IA Matrix':
    'ia_matrix',
    'IA matrix 2002':
    'ia_matrix_2002',
    'IA matrix 2003':
    'ia_matrix_2003',
    'IA matrix 2004':
    'ia_matrix_2004',
    'IA matrix 2005':
    'ia_matrix_2005',
    'IA matrix 2006':
    'ia_matrix_2006',
    'IA matrix 2007':
    'ia_matrix_2007',
    'IA matrix 2008':
    'ia_matrix_2008',
    'IA matrix for Python':
    'ia_matrix_for_python',
    'IA matrix 1995':
    'ia_matrix_1995',
    'IA matrix 1996':
    'ia_matrix_1996',
    'IA matrix 1997':
    'ia_matrix_1997',
    'IA matrix 1998':
    'ia_matrix_1998',
    'IA matrix 1999':
    'ia_matrix_1999',
    'IA matrix 2000':
    'ia_matrix_2000',
    'IA matrix 2001':
    'ia_matrix_2001',
    'IA matrix 2009':
    'ia_matrix_2009',
    'Leontief matrix for Python':
    'leontief_matrix_for_python',
    'Leontief matrix 1996':
    'leontief_matrix_1996',
    'Leontief matrix 1997':
    'leontief_matrix_1997',
    'Leontief matrix 1998':
    'leontief_matrix_1998',
    'Leontief matrix 1999':
    'leontief_matrix_1999',
    'Leontief matrix 2000':
    'leontief_matrix_2000',
    'Leontief matrix 2001':
    'leontief_matrix_2001',
    'Leontief matrix 2002':
    'leontief_matrix_2002',
    'Leontief matrix 2003':
    'leontief_matrix_2003',
    'Leontief matrix 2004':
    'leontief_matrix_2004',
    'Leontief matrix 2005':
    'leontief_matrix_2005',
    'Leontief matrix 2006':
    'leontief_matrix_2006',
    'Leontief matrix 2007':
    'leontief_matrix_2007',
    'Leontief matrix 2008':
    'leontief_matrix_2008',
    'Leontief matrix 2009':
    'leontief_matrix_2009',
    'Leontief matrix 1995':
    'leontief_matrix_1995',
    'replacement batteries':
    'replacement_batteries',
    'share of electric light vehicles':
    'share_of_electric_light_vehicles',
    'Xshare elecXhyb light vehiclesX':
    'xshare_elecxhyb_light_vehiclesx',
    'aux P inland transp':
    'aux_p_inland_transp',
    'T fin inlandT':
    't_fin_inlandt',
    'H elec initial growth':
    'h_elec_initial_growth',
    'T fin H veh':
    't_fin_h_veh',
    'H gas initial growth':
    'h_gas_initial_growth',
    'T ini H veh':
    't_ini_h_veh',
    'H hyb initial growth':
    'h_hyb_initial_growth',
    'policy 2wheels':
    'policy_2wheels',
    'H 2w initial growth':
    'h_2w_initial_growth',
    'var percents H vehicles':
    'var_percents_h_vehicles',
    'increase Households energy final demand for Transp':
    'increase_households_energy_final_demand_for_transp',
    'adapt growth RES elec':
    'adapt_growth_res_elec',
    'GDPpc annual growth SSP2':
    'gdppc_annual_growth_ssp2',
    'Annual GDPpc growth rate':
    'annual_gdppc_growth_rate',
    'variation pop SSP4':
    'variation_pop_ssp4',
    'variation pop SSP5':
    'variation_pop_ssp5',
    'start year P recycling minerals alt techn':
    'start_year_p_recycling_minerals_alt_techn',
    'start year P recycling minerals Rest':
    'start_year_p_recycling_minerals_rest',
    'select pop SSPs':
    'select_pop_ssps',
    'variation pop SSP1':
    'variation_pop_ssp1',
    'variation pop SSP2':
    'variation_pop_ssp2',
    'variation pop SSP3':
    'variation_pop_ssp3',
    'Annual population growth rate':
    'annual_population_growth_rate',
    'improvement recycling rates minerals Rest':
    'improvement_recycling_rates_minerals_rest',
    'variation pop SSPs':
    'variation_pop_ssps',
    'Pop SSP2':
    'pop_ssp2',
    'Pop SSP4':
    'pop_ssp4',
    'Pop SSP1':
    'pop_ssp1',
    'improvement recycling rates minerals alt techn':
    'improvement_recycling_rates_minerals_alt_techn',
    'Pop SSP5':
    'pop_ssp5',
    'Pop SSP3':
    'pop_ssp3',
    'EROImet':
    'eroimet',
    'EROImet FC tot from 2015 1':
    'eroimet_fc_tot_from_2015_1',
    'XActivate EROImet methodXX':
    'xactivate_eroimet_methodxx',
    'EROI FC tot from 2015':
    'eroi_fc_tot_from_2015',
    'XDesired GDP growth affected by real GDPXX':
    'xdesired_gdp_growth_affected_by_real_gdpxx',
    'Dynamic low range FEC good standard of living':
    'dynamic_low_range_fec_good_standard_of_living',
    'Cp EV batteries for elec storage':
    'cp_ev_batteries_for_elec_storage',
    'Dynamic threshold Xhigh developmentX':
    'dynamic_threshold_xhigh_developmentx',
    'max Cp EV batteries for elec storage':
    'max_cp_ev_batteries_for_elec_storage',
    'TFEC intensity until 2009':
    'tfec_intensity_until_2009',
    'Desired annual GDP growth rate':
    'desired_annual_gdp_growth_rate',
    'auxiliary metabolic EROI':
    'auxiliary_metabolic_eroi',
    'Cumulative TFEC intensity change from 2009':
    'cumulative_tfec_intensity_change_from_2009',
    'lifetime EV batteries':
    'lifetime_ev_batteries',
    'Dynamic high range FEC good standard of living':
    'dynamic_high_range_fec_good_standard_of_living',
    'EROI FC tot from 2015 1':
    'eroi_fc_tot_from_2015_1',
    'TFEC per capita':
    'tfec_per_capita',
    'aux15':
    'aux15',
    'Cp EV batteries for Transp':
    'cp_ev_batteries_for_transp',
    'annual GDP growth rate delayed 3yr':
    'annual_gdp_growth_rate_delayed_3yr',
    'annual GDP growth rate delayed 1yr':
    'annual_gdp_growth_rate_delayed_1yr',
    'annual GDP growth rate delayed 2yr':
    'annual_gdp_growth_rate_delayed_2yr',
    'P customized cte GDPpc variation':
    'p_customized_cte_gdppc_variation',
    'Annual growth rate electricity generation RES elec tot':
    'annual_growth_rate_electricity_generation_res_elec_tot',
    'Annual growth rate RES for heat':
    'annual_growth_rate_res_for_heat',
    'P customized year pop evolution':
    'p_customized_year_pop_evolution',
    'Implementatio policy to change final energy':
    'implementatio_policy_to_change_final_energy',
    'Implementatio policy to change final energy H':
    'implementatio_policy_to_change_final_energy_h',
    'Implementation policy to improve energy intensity effciency':
    'implementation_policy_to_improve_energy_intensity_effciency',
    'P timeseries GDPpc growth rate':
    'p_timeseries_gdppc_growth_rate',
    'Desired variation GDPpc':
    'desired_variation_gdppc',
    'select GDPpc evolution input':
    'select_gdppc_evolution_input',
    'select Population evolution input':
    'select_population_evolution_input',
    'P customized year GDPpc evolution':
    'p_customized_year_gdppc_evolution',
    'pop variation':
    'pop_variation',
    'Implementation policy to improve energy intensity effciency H':
    'implementation_policy_to_improve_energy_intensity_effciency_h',
    'FE tot generation all RES elec TWh delayed 1yr':
    'fe_tot_generation_all_res_elec_twh_delayed_1yr',
    'P customized cte pop variation':
    'p_customized_cte_pop_variation',
    'FES RES for heat delayed 1yr':
    'fes_res_for_heat_delayed_1yr',
    'exp slow evol change energy':
    'exp_slow_evol_change_energy',
    'Year to finish policy improve efficiency':
    'year_to_finish_policy_improve_efficiency',
    'exp slow evolution improve efficiency':
    'exp_slow_evolution_improve_efficiency',
    'exp rapid evolution improve efficiency':
    'exp_rapid_evolution_improve_efficiency',
    'exp rapid evol change energy':
    'exp_rapid_evol_change_energy',
    'Year to finish policy change energy':
    'year_to_finish_policy_change_energy',
    'Year to finish policy improve efficiency H':
    'year_to_finish_policy_improve_efficiency_h',
    'Year to finish policy change energy H':
    'year_to_finish_policy_change_energy_h',
    'exp rapid evolution change energy H':
    'exp_rapid_evolution_change_energy_h',
    'Pressure to improve energy intensity efficiency H':
    'pressure_to_improve_energy_intensity_efficiency_h',
    'exp slow evolution change energy H':
    'exp_slow_evolution_change_energy_h',
    'exp rapid evolution improve efficiency H':
    'exp_rapid_evolution_improve_efficiency_h',
    'exp slow evolution improve efficiency H':
    'exp_slow_evolution_improve_efficiency_h',
    'Desired GDP':
    'desired_gdp',
    'Desired GDP delayed 1yr':
    'desired_gdp_delayed_1yr',
    'Exogenous GDP growth rate corrected':
    'exogenous_gdp_growth_rate_corrected',
    'diff annual GDP growth rate':
    'diff_annual_gdp_growth_rate',
    'variation historic GDPpc':
    'variation_historic_gdppc',
    'GDPpc initial year':
    'gdppc_initial_year',
    'Historic GDPpc':
    'historic_gdppc',
    'Desired GDPpc':
    'desired_gdppc',
    'XSIAF feedbackXX':
    'xsiaf_feedbackxx',
    'Energy intensity of households rest':
    'energy_intensity_of_households_rest',
    'Total Radiative Forcing':
    'total_radiative_forcing',
    'Evol final energy intensity by sector and fuel two':
    'evol_final_energy_intensity_by_sector_and_fuel_two',
    'TPES intensity until 2009':
    'tpes_intensity_until_2009',
    'XshareXE losses CC until 2015X':
    'xsharexe_losses_cc_until_2015x',
    'aux13':
    'aux13',
    'aux14':
    'aux14',
    'Xshare EXlosses CC from 2015X':
    'xshare_exlosses_cc_from_2015x',
    'ELF':
    'elf',
    'Cumulative TPES intensity change from 2009':
    'cumulative_tpes_intensity_change_from_2009',
    'Cumulative TFEC intensity change from 2009 without EROI':
    'cumulative_tfec_intensity_change_from_2009_without_eroi',
    'XELF DietzXStern X2015XX':
    'xelf_dietzxstern_x2015xx',
    'Arctic September free of sea ice':
    'arctic_september_free_of_sea_ice',
    'TFEC intensity until 2009 without EROI':
    'tfec_intensity_until_2009_without_eroi',
    'TFEI sectors':
    'tfei_sectors',
    'Annual TFES intensity growth rate without EROI':
    'annual_tfes_intensity_growth_rate_without_eroi',
    'Final energy intensity by sector and fuel':
    'final_energy_intensity_by_sector_and_fuel',
    'Energy intensity of households':
    'energy_intensity_of_households',
    'TFES intensity without EROI delayed 1yr':
    'tfes_intensity_without_eroi_delayed_1yr',
    'Households total final energy demand':
    'households_total_final_energy_demand',
    'required TFED sectors':
    'required_tfed_sectors',
    'share FED households vs sectors':
    'share_fed_households_vs_sectors',
    'TFES intensity EJ Tdolar without EROI':
    'tfes_intensity_ej_tdolar_without_eroi',
    'GHG emissions 2050 MLT2030':
    'ghg_emissions_2050_mlt2030',
    'GHG emissions 2050 MLT2020':
    'ghg_emissions_2050_mlt2020',
    'Cumulative CO2e GHG emissions':
    'cumulative_co2e_ghg_emissions',
    'Xadapt growth RES for heatXcomX':
    'xadapt_growth_res_for_heatxcomx',
    'Xadapt growth RES for heatXncX':
    'xadapt_growth_res_for_heatxncx',
    'Start year P growth RES elec':
    'start_year_p_growth_res_elec',
    'Start year P growth RES heat':
    'start_year_p_growth_res_heat',
    'Total Ce all GHG':
    'total_ce_all_ghg',
    'Labour share growth':
    'labour_share_growth',
    'Year Initial Labour share':
    'year_initial_labour_share',
    'Year Final Labour share':
    'year_final_labour_share',
    'P labour share 2050':
    'p_labour_share_2050',
    'growth labour share':
    'growth_labour_share',
    'Initial Labour share':
    'initial_labour_share',
    'XTotal GHG emissions OLT MEDEAS D3X2X':
    'xtotal_ghg_emissions_olt_medeas_d3x2x',
    'XTotal GHG emissions BAUXCAT MEDEAS D3X2X':
    'xtotal_ghg_emissions_bauxcat_medeas_d3x2x',
    'XTotal GHG emissions MLT2030 MEDEAS D3X2X':
    'xtotal_ghg_emissions_mlt2030_medeas_d3x2x',
    'XTotal GHG emissions MLT2020 MEDEAS D3X2X':
    'xtotal_ghg_emissions_mlt2020_medeas_d3x2x',
    'Low range FEC good standard of living':
    'low_range_fec_good_standard_of_living',
    'Threshold FEC Xhigh developmentX':
    'threshold_fec_xhigh_developmentx',
    'High range FEC good standard of living':
    'high_range_fec_good_standard_of_living',
    'Decrease of intensity due to energy a technology change TOP DOWN':
    'decrease_of_intensity_due_to_energy_a_technology_change_top_down',
    'Maximun yearly aceleration of intensity improvement pct':
    'maximun_yearly_aceleration_of_intensity_improvement_pct',
    'Policy change energy speed':
    'policy_change_energy_speed',
    'Maximun yearly aceleratuin of intensity improvement':
    'maximun_yearly_aceleratuin_of_intensity_improvement',
    'Policy to improve efficiency speed':
    'policy_to_improve_efficiency_speed',
    'inertial rate energy intensity TOP DOWN':
    'inertial_rate_energy_intensity_top_down',
    'minimum fraction':
    'minimum_fraction',
    'Increase of intensity due to energy a technology change TOP DOWN':
    'increase_of_intensity_due_to_energy_a_technology_change_top_down',
    'Year policy change energy':
    'year_policy_change_energy',
    'Max yearly change':
    'max_yearly_change',
    'efficiency rate of substitution':
    'efficiency_rate_of_substitution',
    'Activate BOTTOM UP method':
    'activate_bottom_up_method',
    'Year policy to improve efficiency':
    'year_policy_to_improve_efficiency',
    'Decrease of intensity due to energy a technology change H TOP DOWN':
    'decrease_of_intensity_due_to_energy_a_technology_change_h_top_down',
    'Increase of intensity due to energy a technology change H TOP DOWN':
    'increase_of_intensity_due_to_energy_a_technology_change_h_top_down',
    'Maximun yearly aceleratuin of intensity improvement H':
    'maximun_yearly_aceleratuin_of_intensity_improvement_h',
    'Evol final energy intensity H':
    'evol_final_energy_intensity_h',
    'inertial rate energy intensity H TOP DOWN':
    'inertial_rate_energy_intensity_h_top_down',
    'XTotal CO2e XGWPX100 yearsXX':
    'xtotal_co2e_xgwpx100_yearsxx',
    'Total CO2e all GHG':
    'total_co2e_all_ghg',
    'Net TFEC per capita':
    'net_tfec_per_capita',
    'GWP 100 years CH4':
    'gwp_100_years_ch4',
    'correction factor all GHGs':
    'correction_factor_all_ghgs',
    'real generation RES elec var TWh':
    'real_generation_res_elec_var_twh',
    'XActivate energy scarcity feedbackXX':
    'xactivate_energy_scarcity_feedbackxx',
    'max potential PHS TWh':
    'max_potential_phs_twh',
    'replacement rate PHS':
    'replacement_rate_phs',
    'Energy scarcity feedback shortage coeff':
    'energy_scarcity_feedback_shortage_coeff',
    'real FE elec stored EV batteries TWh':
    'real_fe_elec_stored_ev_batteries_twh',
    'share RES elec generation stored ENDOGENOUS':
    'share_res_elec_generation_stored_endogenous',
    'share tot FEI RES elec var':
    'share_tot_fei_res_elec_var',
    'Cp EV batteries required':
    'cp_ev_batteries_required',
    'output EV bateries for storage over lifetime':
    'output_ev_bateries_for_storage_over_lifetime',
    'discarded batteries':
    'discarded_batteries',
    'ESOI elec storage':
    'esoi_elec_storage',
    'ESOI EV batteries':
    'esoi_ev_batteries',
    'rt elec storage efficiency':
    'rt_elec_storage_efficiency',
    'XXXabundanceXX storageX':
    'xxxabundancexx_storagex',
    'Used EV batteries for elec storage':
    'used_ev_batteries_for_elec_storage',
    'max Cp EV batteries':
    'max_cp_ev_batteries',
    'Total capacity elec storage TW':
    'total_capacity_elec_storage_tw',
    'adapt growth PHS':
    'adapt_growth_phs',
    'demand EV batteries for elec storage':
    'demand_ev_batteries_for_elec_storage',
    'CED per TW over lifetime PHS':
    'ced_per_tw_over_lifetime_phs',
    'remaining potential constraint on new PHS capacity':
    'remaining_potential_constraint_on_new_phs_capacity',
    'new required PHS capacity':
    'new_required_phs_capacity',
    'share FEI for RES vs TFEC':
    'share_fei_for_res_vs_tfec',
    'FEI EV batteries':
    'fei_ev_batteries',
    'output PHS over lifetime':
    'output_phs_over_lifetime',
    'ESOI static PHS':
    'esoi_static_phs',
    'Total FEI RES':
    'total_fei_res',
    'adapt growth RES elec after allocation':
    'adapt_growth_res_elec_after_allocation',
    'replacement capacity RES elec':
    'replacement_capacity_res_elec',
    'ESOI PHS':
    'esoi_phs',
    'P PHS growth':
    'p_phs_growth',
    'constraint elec storage availability':
    'constraint_elec_storage_availability',
    'EROIst tot':
    'eroist_tot',
    'initial instal cap PHS':
    'initial_instal_cap_phs',
    'CEDtot over lifetime PHS':
    'cedtot_over_lifetime_phs',
    'installed capacity PHS TW':
    'installed_capacity_phs_tw',
    'PHS capacity under construction':
    'phs_capacity_under_construction',
    'past PHS capacity growth':
    'past_phs_capacity_growth',
    'new PHS capacity under planning':
    'new_phs_capacity_under_planning',
    'wear PHS':
    'wear_phs',
    'replacement capacity PHS':
    'replacement_capacity_phs',
    'real FE elec stored PHS TWh':
    'real_fe_elec_stored_phs_twh',
    'PHS planned capacity':
    'phs_planned_capacity',
    'Final energy invested PHS':
    'final_energy_invested_phs',
    'remaining potential PHS':
    'remaining_potential_phs',
    'Historic new required capacity PHS':
    'historic_new_required_capacity_phs',
    'table hist capacity PHS':
    'table_hist_capacity_phs',
    'initial capacity in construction PHS':
    'initial_capacity_in_construction_phs',
    'required capacity PHS':
    'required_capacity_phs',
    'initial required capacity PHS':
    'initial_required_capacity_phs',
    'Total FEI over lifetime RES elec dispatch':
    'total_fei_over_lifetime_res_elec_dispatch',
    'min lifetime EV batteries':
    'min_lifetime_ev_batteries',
    'Total transport FED by fuel':
    'total_transport_fed_by_fuel',
    'Total required final energy transport IOT sectors':
    'total_required_final_energy_transport_iot_sectors',
    'Total final energy invested RES elec var':
    'total_final_energy_invested_res_elec_var',
    'materials required for EV batteries Mt':
    'materials_required_for_ev_batteries_mt',
    'XnewXreplaced batteries TWX':
    'xnewxreplaced_batteries_twx',
    'XEnergy intensity construction EV batteries MJXMWX':
    'xenergy_intensity_construction_ev_batteries_mjxmwx',
    'new batteries':
    'new_batteries',
    'Xbatteries EVXhibX2wEX':
    'xbatteries_evxhibx2wex',
    'Xbatteries EVXhibX2wEX2':
    'xbatteries_evxhibx2wex2',
    'kW per MW':
    'kw_per_mw',
    'Net stored energy EV battery over lifetime':
    'net_stored_energy_ev_battery_over_lifetime',
    'Grid correction factor EV batteries':
    'grid_correction_factor_ev_batteries',
    'Share energy requirements for decom EV batteries':
    'share_energy_requirements_for_decom_ev_batteries',
    'Working hours per year':
    'working_hours_per_year',
    'days per year':
    'days_per_year',
    'Total energy required for material consumption for RES elec':
    'total_energy_required_for_material_consumption_for_res_elec',
    'Total energy required for material consumption per RES elec':
    'total_energy_required_for_material_consumption_per_res_elec',
    'MW in 1 year to MJ':
    'mw_in_1_year_to_mj',
    'Working hours per day':
    'working_hours_per_day',
    'Annual work hours for RES':
    'annual_work_hours_for_res',
    'Hours work per GJ RES delivered':
    'hours_work_per_gj_res_delivered',
    'Total energy required for total material consumption for EV batteries':
    'total_energy_required_for_total_material_consumption_for_ev_batteries',
    'Total energy required for total material consumption for alt techn':
    'total_energy_required_for_total_material_consumption_for_alt_techn',
    'Energy required for material consumption per RES elec':
    'energy_required_for_material_consumption_per_res_elec',
    'Total energy required per material for alt techn':
    'total_energy_required_per_material_for_alt_techn',
    'Total jobs RES elec':
    'total_jobs_res_elec',
    'total jobs biofuels':
    'total_jobs_biofuels',
    'Total jobs RES':
    'total_jobs_res',
    'XTotal DXI jobs RES heat per technX':
    'xtotal_dxi_jobs_res_heat_per_technx',
    'Total jobs RES heat':
    'total_jobs_res_heat',
    'XTotal DXI jobs RES elec per technX':
    'xtotal_dxi_jobs_res_elec_per_technx',
    'Ratio total vs D jobs RES heat':
    'ratio_total_vs_d_jobs_res_heat',
    'Ratio total vs D jobs RES elec':
    'ratio_total_vs_d_jobs_res_elec',
    'Employment factor biofuels':
    'employment_factor_biofuels',
    'installed capacity RES elec delayed 1yr':
    'installed_capacity_res_elec_delayed_1yr',
    'XJobs OXM RES heat per technX':
    'xjobs_oxm_res_heat_per_technx',
    'D jobs new installed RES heat per techn':
    'd_jobs_new_installed_res_heat_per_techn',
    'D jobs fuel supply solids bioE':
    'd_jobs_fuel_supply_solids_bioe',
    'new capacity installed growth rate RES elec':
    'new_capacity_installed_growth_rate_res_elec',
    'Employment factor fuel supply solids bioE':
    'employment_factor_fuel_supply_solids_bioe',
    'XActivate EROI tot FC feedback through intensitiesXX':
    'xactivate_eroi_tot_fc_feedback_through_intensitiesxx',
    'test7':
    'test7',
    'Exogenous growth GTL':
    'exogenous_growth_gtl',
    'replacement GTL':
    'replacement_gtl',
    'XCrash programme GTLXX':
    'xcrash_programme_gtlxx',
    'XActivate EROI tot FC feedbackXX':
    'xactivate_eroi_tot_fc_feedbackxx',
    'Exogenous growth CTL':
    'exogenous_growth_ctl',
    'replacement CTL':
    'replacement_ctl',
    'table max extraction ASPO oil EJ010':
    'table_max_extraction_aspo_oil_ej010',
    'aux12':
    'aux12',
    'coal to leave underground':
    'coal_to_leave_underground',
    'RURR coal start year PLG':
    'rurr_coal_start_year_plg',
    'RURR conv oil until start year PLG':
    'rurr_conv_oil_until_start_year_plg',
    'conv gas to leave underground':
    'conv_gas_to_leave_underground',
    'conv oil to leave underground':
    'conv_oil_to_leave_underground',
    'URR oil ASPO 0':
    'urr_oil_aspo_0',
    'URR oil ASPO 0 0':
    'urr_oil_aspo_0_0',
    'URR oil ASPO 0 1':
    'urr_oil_aspo_0_1',
    'RURR unconv oil until start year PLG':
    'rurr_unconv_oil_until_start_year_plg',
    'RURR unconv gas until start year PLG':
    'rurr_unconv_gas_until_start_year_plg',
    'unconv gas to leave underground':
    'unconv_gas_to_leave_underground',
    'tot agg gas to leave underground':
    'tot_agg_gas_to_leave_underground',
    'overdemand EROIst tot':
    'overdemand_eroist_tot',
    'unconv oil to leave underground':
    'unconv_oil_to_leave_underground',
    'aux11':
    'aux11',
    'tot agg oil to leave underground':
    'tot_agg_oil_to_leave_underground',
    'aux6':
    'aux6',
    'aux7':
    'aux7',
    'aux8':
    'aux8',
    'aux9':
    'aux9',
    'RURR conv gas until start year PLG':
    'rurr_conv_gas_until_start_year_plg',
    'aux10':
    'aux10',
    'test ir restando al EROIst tot':
    'test_ir_restando_al_eroist_tot',
    'table max extraction ASPO oil EJ01':
    'table_max_extraction_aspo_oil_ej01',
    'table max extraction ASPO oil EJ0':
    'table_max_extraction_aspo_oil_ej0',
    'table max extraction ASPO oil EJ00':
    'table_max_extraction_aspo_oil_ej00',
    'RURR tot gas until start year PLG':
    'rurr_tot_gas_until_start_year_plg',
    'RURR tot oil until start year PLG':
    'rurr_tot_oil_until_start_year_plg',
    'overdemand EROIst tot from 2015':
    'overdemand_eroist_tot_from_2015',
    'overdemand EROIst tot until 2015':
    'overdemand_eroist_tot_until_2015',
    'Required FED by fuel before heat correction':
    'required_fed_by_fuel_before_heat_correction',
    'aux5':
    'aux5',
    'EROI FC tot from 2015 2':
    'eroi_fc_tot_from_2015_2',
    'share Elec demand covered by RES':
    'share_elec_demand_covered_by_res',
    'RURR conv gas':
    'rurr_conv_gas',
    'RURR unconv gas':
    'rurr_unconv_gas',
    'RURR conv oil':
    'rurr_conv_oil',
    'Xvariation share transmXdistr losses elecX':
    'xvariation_share_transmxdistr_losses_elecx',
    'real PE demand of liquids for Transp EJ':
    'real_pe_demand_of_liquids_for_transp_ej',
    'Share demand solids in transport':
    'share_demand_solids_in_transport',
    'Share demand gas in transport':
    'share_demand_gas_in_transport',
    'Share demand by fuel in transport':
    'share_demand_by_fuel_in_transport',
    'Share demand electricity in transport':
    'share_demand_electricity_in_transport',
    'demand gas for Transp EJ':
    'demand_gas_for_transp_ej',
    'Share demand heat in transport':
    'share_demand_heat_in_transport',
    'Share demand liquids in transport':
    'share_demand_liquids_in_transport',
    'demand liquids for Transp EJ':
    'demand_liquids_for_transp_ej',
    'CO2 emissions per value added':
    'co2_emissions_per_value_added',
    'liquids per X bus':
    'liquids_per_x_bus',
    'liquids per X HV':
    'liquids_per_x_hv',
    'liquids per X LV':
    'liquids_per_x_lv',
    'energy per X train':
    'energy_per_x_train',
    'adjust energy for transport to inland transport':
    'adjust_energy_for_transport_to_inland_transport',
    'Xeffects shortage elec on EVXhibX':
    'xeffects_shortage_elec_on_evxhibx',
    'hist var percent H':
    'hist_var_percent_h',
    'required number standard batteries':
    'required_number_standard_batteries',
    'adjust exogenous GDP growth rate':
    'adjust_exogenous_gdp_growth_rate',
    'Transport TFED energy intensity':
    'transport_tfed_energy_intensity',
    'total number light vehicles':
    'total_number_light_vehicles',
    'Transport TFED':
    'transport_tfed',
    'Transport households final energy demand':
    'transport_households_final_energy_demand',
    'Liquid demand transport EJ':
    'liquid_demand_transport_ej',
    'PE losses related to Elec Transp':
    'pe_losses_related_to_elec_transp',
    'variation LC':
    'variation_lc',
    'variation CC':
    'variation_cc',
    'Increase Elec demand for Transp EJ':
    'increase_elec_demand_for_transp_ej',
    'Total liquid saved by Elec EJ':
    'total_liquid_saved_by_elec_ej',
    'test':
    'test',
    'real FE consumption liquids EJ':
    'real_fe_consumption_liquids_ej',
    'Oil liquids saved by EVs kbd':
    'oil_liquids_saved_by_evs_kbd',
    'Savings PE Transport EJ':
    'savings_pe_transport_ej',
    'increase Elec demand EVs TWh':
    'increase_elec_demand_evs_twh',
    'increase Elec demand for Transp TWh':
    'increase_elec_demand_for_transp_twh',
    'share oil for Elec':
    'share_oil_for_elec',
    'CO2 emissions unconv gas':
    'co2_emissions_unconv_gas',
    'Xreduce oil share for electricityXX':
    'xreduce_oil_share_for_electricityxx',
    'max solar on land Mha':
    'max_solar_on_land_mha',
    'max solar PV on land MHa':
    'max_solar_pv_on_land_mha',
    'max solar PV on land TWe':
    'max_solar_pv_on_land_twe',
    'Total CH4 emissions fossil fuels':
    'total_ch4_emissions_fossil_fuels',
    'max CSP on land MHa':
    'max_csp_on_land_mha',
    'max CSP TWe':
    'max_csp_twe',
    'gCH4 per MJ GTL':
    'gch4_per_mj_gtl',
    'share liquids fot heat plants':
    'share_liquids_fot_heat_plants',
    'CH4 emissions GTL':
    'ch4_emissions_gtl',
    'CH4 emissions CTL':
    'ch4_emissions_ctl',
    'Xreduce oil share for heatXcomXX':
    'xreduce_oil_share_for_heatxcomxx',
    'gCH4 per MJ CTL':
    'gch4_per_mj_ctl',
    'g per Gt':
    'g_per_gt',
    'BioE CO2 emissions':
    'bioe_co2_emissions',
    'maximum 2w':
    'maximum_2w',
    'P H vehicle':
    'p_h_vehicle',
    'EROIst delayed 1yr':
    'eroist_delayed_1yr',
    'EROIst delayed':
    'eroist_delayed',
    'solid biofuels emissions relevant EJ':
    'solid_biofuels_emissions_relevant_ej',
    'XEROIstX1995XX':
    'xeroistx1995xx',
    'EROIst until 2015':
    'eroist_until_2015',
    'aux4':
    'aux4',
    'remaining potential elec storage by RES techn':
    'remaining_potential_elec_storage_by_res_techn',
    'Cp hydro 2015':
    'cp_hydro_2015',
    'ESOI PHS full potential':
    'esoi_phs_full_potential',
    'max capacity elec storage':
    'max_capacity_elec_storage',
    'CC impacts feedback shortage coeff':
    'cc_impacts_feedback_shortage_coeff',
    'XEOLXRR minerals alt techn RES vsX total economyX':
    'xeolxrr_minerals_alt_techn_res_vsx_total_economyx',
    'XCrash programme CTLXX':
    'xcrash_programme_ctlxx',
    'recycling rates minerals Rest':
    'recycling_rates_minerals_rest',
    'recycling rates minerals alt techn':
    'recycling_rates_minerals_alt_techn',
    'P elec':
    'p_elec',
    'percents H vehicles':
    'percents_h_vehicles',
    'T hist H transp':
    't_hist_h_transp',
    'bat number 2w':
    'bat_number_2w',
    'bateries ratio 2w E':
    'bateries_ratio_2w_e',
    'bat number EV':
    'bat_number_ev',
    'bat number hib':
    'bat_number_hib',
    'bateries ratio bus E':
    'bateries_ratio_bus_e',
    'bateries ratio hib bus':
    'bateries_ratio_hib_bus',
    'bateries ratio hib HV':
    'bateries_ratio_hib_hv',
    'bateries ratio hib LV':
    'bateries_ratio_hib_lv',
    'total number hybrid light vehicles':
    'total_number_hybrid_light_vehicles',
    'total number elec light vehicles':
    'total_number_elec_light_vehicles',
    'total number gas light vehicles':
    'total_number_gas_light_vehicles',
    'EV batteries TW':
    'ev_batteries_tw',
    'var percent T vehicles':
    'var_percent_t_vehicles',
    'P train elec':
    'p_train_elec',
    'P HV gas':
    'p_hv_gas',
    'T ini inlandT':
    't_ini_inlandt',
    'P inlandT':
    'p_inlandt',
    'Activate policy inlandT':
    'activate_policy_inlandt',
    'P LV elec':
    'p_lv_elec',
    'P LV hyb':
    'p_lv_hyb',
    'P bus elec':
    'p_bus_elec',
    'P bus gas':
    'p_bus_gas',
    'P bus hyb':
    'p_bus_hyb',
    'percents bus':
    'percents_bus',
    'percents train':
    'percents_train',
    'P HV hyb':
    'p_hv_hyb',
    'adapt var inlandT':
    'adapt_var_inlandt',
    'percents LV':
    'percents_lv',
    'var I inland Elec':
    'var_i_inland_elec',
    'var I inlandT Gas':
    'var_i_inlandt_gas',
    'var I inlandT liq':
    'var_i_inlandt_liq',
    'shares available T':
    'shares_available_t',
    'vehicles inlandT':
    'vehicles_inlandt',
    'percent T vehicles':
    'percent_t_vehicles',
    'Efects shortage inlandT':
    'efects_shortage_inlandt',
    'P LV gas':
    'p_lv_gas',
    'percents HV':
    'percents_hv',
    'initial Xt inland':
    'initial_xt_inland',
    'NX bus inlandT':
    'nx_bus_inlandt',
    'NX HV inland T':
    'nx_hv_inland_t',
    'NX LV inland T':
    'nx_lv_inland_t',
    'N vehicles inland t 0':
    'n_vehicles_inland_t_0',
    'Energy initial inland transport':
    'energy_initial_inland_transport',
    'energy per X t':
    'energy_per_x_t',
    'NX train inland T':
    'nx_train_inland_t',
    'A2 coef tH':
    'a2_coef_th',
    'H gas adapt growth':
    'h_gas_adapt_growth',
    'H hyb adapt growth':
    'h_hyb_adapt_growth',
    'Activate policy H transp':
    'activate_policy_h_transp',
    'Initial energy intensity of households transport 2009':
    'initial_energy_intensity_of_households_transport_2009',
    'H EV adapt growth':
    'h_ev_adapt_growth',
    'var IH E2':
    'var_ih_e2',
    'var IH gas2':
    'var_ih_gas2',
    'A1 coef tH':
    'a1_coef_th',
    'var IH liq2':
    'var_ih_liq2',
    'percent H vehicles initial':
    'percent_h_vehicles_initial',
    'Energy intensity of households transport':
    'energy_intensity_of_households_transport',
    'H 2wE adapt growth':
    'h_2we_adapt_growth',
    'Liq 4w 0':
    'liq_4w_0',
    'percent 2w liq 0':
    'percent_2w_liq_0',
    'percent 4w liq 0':
    'percent_4w_liq_0',
    'Electricity 2wE 0':
    'electricity_2we_0',
    'N vehicles H 0':
    'n_vehicles_h_0',
    'Demand H 0':
    'demand_h_0',
    'ratio N veh Demand H':
    'ratio_n_veh_demand_h',
    'percents 2w H vehicles':
    'percents_2w_h_vehicles',
    'percents 4w H vehicles':
    'percents_4w_h_vehicles',
    'P 2wE':
    'p_2we',
    'max percent 2 wheels':
    'max_percent_2_wheels',
    'max percent 4 wheels':
    'max_percent_4_wheels',
    'aux hist H':
    'aux_hist_h',
    'P gas':
    'p_gas',
    'P share 2 wheelers':
    'p_share_2_wheelers',
    'P hyb':
    'p_hyb',
    'initial 2w percent':
    'initial_2w_percent',
    'share available 2w':
    'share_available_2w',
    'rate 4w to 2w':
    'rate_4w_to_2w',
    'share FEH over FED by final fuel':
    'share_feh_over_fed_by_final_fuel',
    'share FEH over FED oil':
    'share_feh_over_fed_oil',
    'share FEH over FED coal':
    'share_feh_over_fed_coal',
    'Xshare FEH over FED natX gasX':
    'xshare_feh_over_fed_natx_gasx',
    'XFED coal for heatXncX':
    'xfed_coal_for_heatxncx',
    'PES oil EJ delayed':
    'pes_oil_ej_delayed',
    'Energy required for material consumption for EV batteries':
    'energy_required_for_material_consumption_for_ev_batteries',
    'area solar collectors':
    'area_solar_collectors',
    'Households final energy demand':
    'households_final_energy_demand',
    'change total intensity to rest':
    'change_total_intensity_to_rest',
    'Activate transport H BOTTOM UP method':
    'activate_transport_h_bottom_up_method',
    'variation energy intensity of households transport':
    'variation_energy_intensity_of_households_transport',
    'Number vehicles H':
    'number_vehicles_h',
    'effects shortage elec on EV':
    'effects_shortage_elec_on_ev',
    'effects shortage gas H veh':
    'effects_shortage_gas_h_veh',
    'XXaX extraction projection mineralsX':
    'xxax_extraction_projection_mineralsx',
    'XXbX extraction projection mineralsX':
    'xxbx_extraction_projection_mineralsx',
    'variation minerals extraction Rest':
    'variation_minerals_extraction_rest',
    'Total recycled materials for other Mt':
    'total_recycled_materials_for_other_mt',
    'share minerals consumption alt techn vs total economy':
    'share_minerals_consumption_alt_techn_vs_total_economy',
    'XLabor share cteXX':
    'xlabor_share_ctexx',
    'initial minerals extraction Rest':
    'initial_minerals_extraction_rest',
    'Minerals extraction projection Rest cte rr':
    'minerals_extraction_projection_rest_cte_rr',
    'Materials to extract Rest Mt':
    'materials_to_extract_rest_mt',
    'Historical extraction minerals Rest SUBAdhesive':
    'historical_extraction_minerals_rest_subadhesive',
    'Historical extraction minerals Rest SUBAluminium':
    'historical_extraction_minerals_rest_subaluminium',
    'Historical extraction minerals Rest SUBAluminiummirrors':
    'historical_extraction_minerals_rest_subaluminiummirrors',
    'Historical extraction minerals Rest SUBCadmium':
    'historical_extraction_minerals_rest_subcadmium',
    'Historical extraction minerals Rest SUBCarbonfiber':
    'historical_extraction_minerals_rest_subcarbonfiber',
    'Historical extraction minerals Rest SUBCement':
    'historical_extraction_minerals_rest_subcement',
    'Historical extraction minerals Rest SUBChromium':
    'historical_extraction_minerals_rest_subchromium',
    'Historical extraction minerals Rest SUBCopper':
    'historical_extraction_minerals_rest_subcopper',
    'Historical extraction minerals Rest SUBdiesel':
    'historical_extraction_minerals_rest_subdiesel',
    'Historical extraction minerals Rest SUBDy':
    'historical_extraction_minerals_rest_subdy',
    'Historical extraction minerals Rest SUBXElectricXelectroniccomponentsX':
    'historical_extraction_minerals_rest_subxelectricxelectroniccomponentsx',
    'Historical extraction minerals Rest SUBEvacuationlines':
    'historical_extraction_minerals_rest_subevacuationlines',
    'Historical extraction minerals Rest SUBFiberglass':
    'historical_extraction_minerals_rest_subfiberglass',
    'Historical extraction minerals Rest SUBFoamglass':
    'historical_extraction_minerals_rest_subfoamglass',
    'Historical extraction minerals Rest SUBGalium':
    'historical_extraction_minerals_rest_subgalium',
    'Historical extraction minerals Rest SUBGlass':
    'historical_extraction_minerals_rest_subglass',
    'Historical extraction minerals Rest SUBGlassreinforcingplastic':
    'historical_extraction_minerals_rest_subglassreinforcingplastic',
    'Historical extraction minerals Rest SUBgravel':
    'historical_extraction_minerals_rest_subgravel',
    'Historical extraction minerals Rest SUBIndium':
    'historical_extraction_minerals_rest_subindium',
    'Historical extraction minerals Rest SUBIron':
    'historical_extraction_minerals_rest_subiron',
    'Historical extraction minerals Rest SUBKNO3mined':
    'historical_extraction_minerals_rest_subkno3mined',
    'Historical extraction minerals Rest SUBAsphalt':
    'historical_extraction_minerals_rest_subasphalt',
    'Historical extraction minerals Rest SUBLime':
    'historical_extraction_minerals_rest_sublime',
    'Historical extraction minerals Rest SUBLimestone':
    'historical_extraction_minerals_rest_sublimestone',
    'Historical extraction minerals Rest SUBLithium':
    'historical_extraction_minerals_rest_sublithium',
    'Historical extraction minerals Rest SUBLubricant':
    'historical_extraction_minerals_rest_sublubricant',
    'Historical extraction minerals Rest SUBMagnesium':
    'historical_extraction_minerals_rest_submagnesium',
    'Historical extraction minerals Rest SUBManganese':
    'historical_extraction_minerals_rest_submanganese',
    'Historical extraction minerals Rest SUBHeavyequipment':
    'historical_extraction_minerals_rest_subheavyequipment',
    'Historical extraction minerals Rest SUBConcrete':
    'historical_extraction_minerals_rest_subconcrete',
    'Historical extraction minerals Rest SUBMolybdenum':
    'historical_extraction_minerals_rest_submolybdenum',
    'Historical extraction minerals Rest SUBNaNO3mined':
    'historical_extraction_minerals_rest_subnano3mined',
    'Historical extraction minerals Rest SUBNaNO3synthetic':
    'historical_extraction_minerals_rest_subnano3synthetic',
    'Historical extraction minerals Rest SUBNeodymium':
    'historical_extraction_minerals_rest_subneodymium',
    'Historical extraction minerals Rest SUBNickel':
    'historical_extraction_minerals_rest_subnickel',
    'Historical extraction minerals Rest SUBXOvergridX15XXX':
    'historical_extraction_minerals_rest_subxovergridx15xxx',
    'Historical extraction minerals Rest SUBXOvergridX5XXX':
    'historical_extraction_minerals_rest_subxovergridx5xxx',
    'Historical extraction minerals Rest SUBPaint':
    'historical_extraction_minerals_rest_subpaint',
    'Historical extraction minerals Rest SUBLead':
    'historical_extraction_minerals_rest_sublead',
    'Historical extraction minerals Rest SUBPlastics':
    'historical_extraction_minerals_rest_subplastics',
    'Historical extraction minerals Rest SUBPolypropylene':
    'historical_extraction_minerals_rest_subpolypropylene',
    'Historical extraction minerals Rest SUBRock':
    'historical_extraction_minerals_rest_subrock',
    'Historical extraction minerals Rest SUBRockwool':
    'historical_extraction_minerals_rest_subrockwool',
    'Historical extraction minerals Rest SUBSand':
    'historical_extraction_minerals_rest_subsand',
    'Historical extraction minerals Rest SUBSiliconsand':
    'historical_extraction_minerals_rest_subsiliconsand',
    'Historical extraction minerals Rest SUBSiliconwafermodules':
    'historical_extraction_minerals_rest_subsiliconwafermodules',
    'Historical extraction minerals Rest SUBSilver':
    'historical_extraction_minerals_rest_subsilver',
    'Historical extraction minerals Rest SUBSitepreparation':
    'historical_extraction_minerals_rest_subsitepreparation',
    'Historical extraction minerals Rest SUBTin':
    'historical_extraction_minerals_rest_subtin',
    'Historical extraction minerals Rest SUBsodaash':
    'historical_extraction_minerals_rest_subsodaash',
    'Historical extraction minerals Rest SUBsteel':
    'historical_extraction_minerals_rest_substeel',
    'Historical extraction minerals Rest SUBsyntheticoil':
    'historical_extraction_minerals_rest_subsyntheticoil',
    'Historical extraction minerals Rest SUBtellurium':
    'historical_extraction_minerals_rest_subtellurium',
    'Historical extraction minerals Rest SUBtitanium':
    'historical_extraction_minerals_rest_subtitanium',
    'Historical extraction minerals Rest SUBtitaniumdioxide':
    'historical_extraction_minerals_rest_subtitaniumdioxide',
    'Historical extraction minerals Rest SUBvanadium':
    'historical_extraction_minerals_rest_subvanadium',
    'Historical extraction minerals Rest SUBwires':
    'historical_extraction_minerals_rest_subwires',
    'Historical extraction minerals Rest SUBzinc':
    'historical_extraction_minerals_rest_subzinc',
    'Historical variation minerals extraction Rest':
    'historical_variation_minerals_extraction_rest',
    'Minerals extraction projection Rest with rr':
    'minerals_extraction_projection_rest_with_rr',
    'Minerals consumption estimation Rest cte rr':
    'minerals_consumption_estimation_rest_cte_rr',
    'XTotal materials required for RES elec X EV batteries MtX':
    'xtotal_materials_required_for_res_elec_x_ev_batteries_mtx',
    'Xvariation nonXenergy useX':
    'xvariation_nonxenergy_usex',
    'Number 2w':
    'number_2w',
    'Number 4w':
    'number_4w',
    'Number all':
    'number_all',
    'sum 4w shares':
    'sum_4w_shares',
    'share available 4w':
    'share_available_4w',
    'percent 4w':
    'percent_4w',
    'saving ratio 2wE':
    'saving_ratio_2we',
    'percent all':
    'percent_all',
    'percent 2w':
    'percent_2w',
    'Xshare FED coal vs NRE heatXncX':
    'xshare_fed_coal_vs_nre_heatxncx',
    'Xshare FED gas vs NRE heatXncX':
    'xshare_fed_gas_vs_nre_heatxncx',
    'XFED NRE for heatXncX':
    'xfed_nre_for_heatxncx',
    'Xshare FED liquids vs NRE heatXncX':
    'xshare_fed_liquids_vs_nre_heatxncx',
    'Required FED by fuel':
    'required_fed_by_fuel',
    'BioE gen land marg available':
    'bioe_gen_land_marg_available',
    'Potential PEavail cellulosic biofuel EJ':
    'potential_peavail_cellulosic_biofuel_ej',
    'Potential PEavail total biofuels':
    'potential_peavail_total_biofuels',
    'Biofuels 3gen land compet available':
    'biofuels_3gen_land_compet_available',
    'FES total biofuels production EJ':
    'fes_total_biofuels_production_ej',
    'FES total biofuels production EJ 2':
    'fes_total_biofuels_production_ej_2',
    'XFES total biofuels production MbXdX':
    'xfes_total_biofuels_production_mbxdx',
    'PEavail biofuels 2gen land compet EJ':
    'peavail_biofuels_2gen_land_compet_ej',
    'PEavail biofuels 3gen land compet EJ':
    'peavail_biofuels_3gen_land_compet_ej',
    'PEavail biofuels land marg EJ':
    'peavail_biofuels_land_marg_ej',
    'PE biofuels land marg EJ':
    'pe_biofuels_land_marg_ej',
    'XPE biofuels prod 2genX3gen EJX':
    'xpe_biofuels_prod_2genx3gen_ejx',
    'PE biomass for biofuels production EJ':
    'pe_biomass_for_biofuels_production_ej',
    'Xnew BioE residues for heatXelecX':
    'xnew_bioe_residues_for_heatxelecx',
    'PE cellulosic biofuel EJ':
    'pe_cellulosic_biofuel_ej',
    'new biofuels land marg':
    'new_biofuels_land_marg',
    'Max PEavail biofuels potential':
    'max_peavail_biofuels_potential',
    'start year biofuels land marg':
    'start_year_biofuels_land_marg',
    'Land required biofuels land marg':
    'land_required_biofuels_land_marg',
    'Max PEavail potential biofuels marginal lands':
    'max_peavail_potential_biofuels_marginal_lands',
    'share biofuels overcapacity':
    'share_biofuels_overcapacity',
    'Total land requirements renew Mha':
    'total_land_requirements_renew_mha',
    'Land occupation ratio biofuels marg land':
    'land_occupation_ratio_biofuels_marg_land',
    'new cellulosic biofuels':
    'new_cellulosic_biofuels',
    'PEavail cellulosic biofuel EJ':
    'peavail_cellulosic_biofuel_ej',
    'Potential PEavail biofuels land marg EJ':
    'potential_peavail_biofuels_land_marg_ej',
    'BioE potential NPP marginal lands':
    'bioe_potential_npp_marginal_lands',
    'Oil saved by biofuels Mtoe':
    'oil_saved_by_biofuels_mtoe',
    'start production biofuels':
    'start_production_biofuels',
    'Additional PE production of bioenergy for biofuels':
    'additional_pe_production_of_bioenergy_for_biofuels',
    'Oil liquids saved by biofuels EJ':
    'oil_liquids_saved_by_biofuels_ej',
    'Efficiency bioE residues to cellulosic liquids':
    'efficiency_bioe_residues_to_cellulosic_liquids',
    'Conv efficiency from NPP to biofuels':
    'conv_efficiency_from_npp_to_biofuels',
    'PES heat RES':
    'pes_heat_res',
    'PED NRE Liquids':
    'ped_nre_liquids',
    'XPotential FES CTLXGTL EJX':
    'xpotential_fes_ctlxgtl_ejx',
    'TFEC RES delayed 1yr':
    'tfec_res_delayed_1yr',
    'TFEC RES EJ':
    'tfec_res_ej',
    'XPE supply from RES nonXelec without trad bioE EJX':
    'xpe_supply_from_res_nonxelec_without_trad_bioe_ejx',
    'PES gases':
    'pes_gases',
    'XFES CTLXGTL EJX':
    'xfes_ctlxgtl_ejx',
    'Xactivate ELF all scenXX':
    'xactivate_elf_all_scenxx',
    'TPES RES delayed 1yr':
    'tpes_res_delayed_1yr',
    'Annual TPES RES growth rate':
    'annual_tpes_res_growth_rate',
    'XPED natX gas EJX':
    'xped_natx_gas_ejx',
    'Annual share RES vs TFEC growth rate':
    'annual_share_res_vs_tfec_growth_rate',
    'Annual share RES vs TPES growth rate':
    'annual_share_res_vs_tpes_growth_rate',
    'share RES vs TPES delayed 1yr':
    'share_res_vs_tpes_delayed_1yr',
    'GTL production':
    'gtl_production',
    'PES biogas for TFC':
    'pes_biogas_for_tfc',
    'Xshare CTLXGTL overcapacityX':
    'xshare_ctlxgtl_overcapacityx',
    'share RES vs TFEC delayed 1yr':
    'share_res_vs_tfec_delayed_1yr',
    'Xshare EXlosses CCX':
    'xshare_exlosses_ccx',
    'XPED natX gas for GTL EJX':
    'xped_natx_gas_for_gtl_ejx',
    'max unconv gas growth extraction':
    'max_unconv_gas_growth_extraction',
    'PED coal for CTL EJ':
    'ped_coal_for_ctl_ej',
    'Annual TFEC RES growth rate':
    'annual_tfec_res_growth_rate',
    'PED total oil EJ':
    'ped_total_oil_ej',
    'CTL production':
    'ctl_production',
    'CH4 emissions coal without CTL':
    'ch4_emissions_coal_without_ctl',
    'PES tot RES for heat':
    'pes_tot_res_for_heat',
    'XTotal FE real supply RES for heatXnc EJX':
    'xtotal_fe_real_supply_res_for_heatxnc_ejx',
    'real extraction conv oil emissions relevant EJ':
    'real_extraction_conv_oil_emissions_relevant_ej',
    'Xnew RES capacity for heatXnc TWX':
    'xnew_res_capacity_for_heatxnc_twx',
    'CO2 emissions coal without CTL':
    'co2_emissions_coal_without_ctl',
    'XAnnual variation nonXenergy useX':
    'xannual_variation_nonxenergy_usex',
    'real extraction conv gas emissions relevant EJ':
    'real_extraction_conv_gas_emissions_relevant_ej',
    'XPES RES for heatXnc by technX':
    'xpes_res_for_heatxnc_by_technx',
    'FES NRE for heat':
    'fes_nre_for_heat',
    'Xinstalled capacity RES heatXnc TWX':
    'xinstalled_capacity_res_heatxnc_twx',
    'PES solids bioE EJ':
    'pes_solids_bioe_ej',
    'XFED HeatXcom plants fossil fuels EJX':
    'xfed_heatxcom_plants_fossil_fuels_ejx',
    'real extraction unconv gas emissions relevant EJ':
    'real_extraction_unconv_gas_emissions_relevant_ej',
    'real extraction unconv oil emissions relevant EJ':
    'real_extraction_unconv_oil_emissions_relevant_ej',
    'available PE potential solid bioE for elec EJ':
    'available_pe_potential_solid_bioe_for_elec_ej',
    'remaining potential RES for heat':
    'remaining_potential_res_for_heat',
    'extraction coal emissions relevant EJ':
    'extraction_coal_emissions_relevant_ej',
    'Xwear RES capacity for heatXnc TWX':
    'xwear_res_capacity_for_heatxnc_twx',
    'Xpotential FES RES for heatXnc TWhX':
    'xpotential_fes_res_for_heatxnc_twhx',
    'abundance RES heat2 0':
    'abundance_res_heat2_0',
    'XFE real generation RES heatXnc EJX':
    'xfe_real_generation_res_heatxnc_ejx',
    'XTotal FE real supply RES for heatXcom EJX':
    'xtotal_fe_real_supply_res_for_heatxcom_ejx',
    'Xnew RES capacity for heatXcom TW 0X':
    'xnew_res_capacity_for_heatxcom_tw_0x',
    'initial value RES for heat 0':
    'initial_value_res_for_heat_0',
    'share coal Heat EJ':
    'share_coal_heat_ej',
    'Xpast RES growth for heatXcom 0X':
    'xpast_res_growth_for_heatxcom_0x',
    'PES RES for heat by techn 0':
    'pes_res_for_heat_by_techn_0',
    'installed capacity RES heat TW 0':
    'installed_capacity_res_heat_tw_0',
    'share renewables Heat EJ':
    'share_renewables_heat_ej',
    'XFE real generation RES heatXcom EJ 0X':
    'xfe_real_generation_res_heatxcom_ej_0x',
    'remaining potential constraint on new RES heat capacity 0':
    'remaining_potential_constraint_on_new_res_heat_capacity_0',
    'Efficiency conversion BioE plants to heat 0':
    'efficiency_conversion_bioe_plants_to_heat_0',
    'XFED HeatXcom after priorities EJX':
    'xfed_heatxcom_after_priorities_ejx',
    'FES heat from BioW':
    'fes_heat_from_biow',
    'share fossil fuel CHP EJ':
    'share_fossil_fuel_chp_ej',
    'adapt growth RES for heat 0':
    'adapt_growth_res_for_heat_0',
    'Xabundance RES heatXncX':
    'xabundance_res_heatxncx',
    'remaining potential RES for heat 0':
    'remaining_potential_res_for_heat_0',
    'Efficiency geothermal for heat 0':
    'efficiency_geothermal_for_heat_0',
    'XFES HeatXcom nuclear CHP plants EJX':
    'xfes_heatxcom_nuclear_chp_plants_ejx',
    'XFE real supply RES for heatXnc tot EJX':
    'xfe_real_supply_res_for_heatxnc_tot_ejx',
    'RES heat tot overcapacity 0':
    'res_heat_tot_overcapacity_0',
    'Xinitial value RES for heatXncX':
    'xinitial_value_res_for_heatxncx',
    'share gases Heat EJ':
    'share_gases_heat_ej',
    'Xpast RES growth for heatXncX':
    'xpast_res_growth_for_heatxncx',
    'wear RES capacity for heat TW 0':
    'wear_res_capacity_for_heat_tw_0',
    'P solid bioE for heat 0':
    'p_solid_bioe_for_heat_0',
    'FED heat fossil fuels CHP plants EJ':
    'fed_heat_fossil_fuels_chp_plants_ej',
    'Xreplacement RES for heatXncX':
    'xreplacement_res_for_heatxncx',
    'potential FES RES for heat EJ 0':
    'potential_fes_res_for_heat_ej_0',
    'Efficiency solar panels for heat 0':
    'efficiency_solar_panels_for_heat_0',
    'Cp RES for heat 0':
    'cp_res_for_heat_0',
    'Xreplacement RES for heatXnc TWX':
    'xreplacement_res_for_heatxnc_twx',
    'P RES for heat 0':
    'p_res_for_heat_0',
    'abundance RES heat 0':
    'abundance_res_heat_0',
    'P solar for heat 0':
    'p_solar_for_heat_0',
    'Xabundance RES heatXcomX':
    'xabundance_res_heatxcomx',
    'Xabundance RES heatXnc2X':
    'xabundance_res_heatxnc2x',
    'Xpotential FES tot RES for heatXnc EJX':
    'xpotential_fes_tot_res_for_heatxnc_ejx',
    'life time RES for heat 0':
    'life_time_res_for_heat_0',
    'Efficiency RES heat 0':
    'efficiency_res_heat_0',
    'XHistoric RES capacity for heatXcom 0X SUBXsolarXheatX':
    'xhistoric_res_capacity_for_heatxcom_0x_subxsolarxheatx',
    'XHistoric RES capacity for heatXcom 0X SUBXgeotXheatX':
    'xhistoric_res_capacity_for_heatxcom_0x_subxgeotxheatx',
    'XHistoric RES capacity for heatXcom 0X SUBXsolidbioEXheatX':
    'xhistoric_res_capacity_for_heatxcom_0x_subxsolidbioexheatx',
    'replacement RES for heat 0':
    'replacement_res_for_heat_0',
    'P geothermal for heat 0':
    'p_geothermal_for_heat_0',
    'FES RES for heat EJ':
    'fes_res_for_heat_ej',
    'XFE real supply RES for heatXcom tot EJX':
    'xfe_real_supply_res_for_heatxcom_tot_ejx',
    'share liquids Heat EJ':
    'share_liquids_heat_ej',
    'Losses solar for heat 0':
    'losses_solar_for_heat_0',
    'potential FES tot RES for heat EJ 0':
    'potential_fes_tot_res_for_heat_ej_0',
    'potential FES RES for heat TWh 0':
    'potential_fes_res_for_heat_twh_0',
    'FE real supply RES for heat tot EJ 0':
    'fe_real_supply_res_for_heat_tot_ej_0',
    'share nuclear Heat EJ':
    'share_nuclear_heat_ej',
    'XRES heatXnc tot overcapacityX':
    'xres_heatxnc_tot_overcapacityx',
    'Xpotential FES RES for heatXnc EJX':
    'xpotential_fes_res_for_heatxnc_ejx',
    'XHistoric RES capacity for heatXncX SUBXsolarXheatX':
    'xhistoric_res_capacity_for_heatxncx_subxsolarxheatx',
    'XHistoric RES capacity for heatXncX SUBXgeotXheatX':
    'xhistoric_res_capacity_for_heatxncx_subxgeotxheatx',
    'XHistoric RES capacity for heatXncX SUBXsolidbioEXheatX':
    'xhistoric_res_capacity_for_heatxncx_subxsolidbioexheatx',
    'replacement RES for heat TW 0':
    'replacement_res_for_heat_tw_0',
    'Historic share of transformation losses vs extraction SUBliquids':
    'historic_share_of_transformation_losses_vs_extraction_subliquids',
    'Historic share of transformation losses vs extraction SUBsolids':
    'historic_share_of_transformation_losses_vs_extraction_subsolids',
    'Other solids required':
    'other_solids_required',
    'Ratio gain gas vs lose solids in tranf processes':
    'ratio_gain_gas_vs_lose_solids_in_tranf_processes',
    'Other gases required':
    'other_gases_required',
    'Transformation Losses EJ':
    'transformation_losses_ej',
    'FES Elec fossil fuel CHP plants EJ':
    'fes_elec_fossil_fuel_chp_plants_ej',
    'Historic GDP WIOD':
    'historic_gdp_wiod',
    'check GDP':
    'check_gdp',
    'Total FE Elec demand after priorities TWh':
    'total_fe_elec_demand_after_priorities_twh',
    'Demand Elec NRE TWh':
    'demand_elec_nre_twh',
    'XNonXenergy use demand by final fuel EJX':
    'xnonxenergy_use_demand_by_final_fuel_ejx',
    'share solids for final energy':
    'share_solids_for_final_energy',
    'real FE consumption gases EJ':
    'real_fe_consumption_gases_ej',
    'real FE consumption solids EJ':
    'real_fe_consumption_solids_ej',
    'XTotal real nonXenergy use consumption EJX':
    'xtotal_real_nonxenergy_use_consumption_ejx',
    'share gases for final energy':
    'share_gases_for_final_energy',
    'Xratio FED for heatXnc vs FED for heatXcomX':
    'xratio_fed_for_heatxnc_vs_fed_for_heatxcomx',
    'real FE consumption by fuel before heat correction':
    'real_fe_consumption_by_fuel_before_heat_correction',
    'Required TFED before heat dem corr':
    'required_tfed_before_heat_dem_corr',
    'cum materials to extract for EV batteries':
    'cum_materials_to_extract_for_ev_batteries',
    'share cum dem materials to extract alt techn vs total':
    'share_cum_dem_materials_to_extract_alt_techn_vs_total',
    'Total recycled materials for EV batteries Mt':
    'total_recycled_materials_for_ev_batteries_mt',
    'share RES vs TFEC':
    'share_res_vs_tfec',
    'cum materials to extract for alt techn from 2015':
    'cum_materials_to_extract_for_alt_techn_from_2015',
    'Total materials to extract for EV batteries from 2015 Mt':
    'total_materials_to_extract_for_ev_batteries_from_2015_mt',
    'Xmaterials per new capacity installed X EV batteriesX':
    'xmaterials_per_new_capacity_installed_x_ev_batteriesx',
    'Total materials to extract for EV batteries Mt':
    'total_materials_to_extract_for_ev_batteries_mt',
    'Total materials to extract Mt':
    'total_materials_to_extract_mt',
    'cum materials requirements for EV batteries':
    'cum_materials_requirements_for_ev_batteries',
    'XXstaticX EROIgrid RES elecX':
    'xxstaticx_eroigrid_res_elecx',
    'T per x':
    't_per_x',
    'Total materials required for EV batteries':
    'total_materials_required_for_ev_batteries',
    'TFEC from RES per capita':
    'tfec_from_res_per_capita',
    'share materials cum demand to extract vs reserves for RES elec':
    'share_materials_cum_demand_to_extract_vs_reserves_for_res_elec',
    'rt storage efficiency EV batteries':
    'rt_storage_efficiency_ev_batteries',
    'cum materials to extract for EV batteries from 2015':
    'cum_materials_to_extract_for_ev_batteries_from_2015',
    'total cumulative demand materials to extract from 2015':
    'total_cumulative_demand_materials_to_extract_from_2015',
    'initial cumulated material requirements for EV batteries 1995':
    'initial_cumulated_material_requirements_for_ev_batteries_1995',
    'share materials cum demand to extract vs resources for RES elec':
    'share_materials_cum_demand_to_extract_vs_resources_for_res_elec',
    'FE tot generation all RES elec EJ':
    'fe_tot_generation_all_res_elec_ej',
    'XCarbon footprint tonnesCXpersonX':
    'xcarbon_footprint_tonnescxpersonx',
    'Potential max HDI':
    'potential_max_hdi',
    'kW per battery EV':
    'kw_per_battery_ev',
    'PES fossil fuel extraction delayed':
    'pes_fossil_fuel_extraction_delayed',
    'PE traditional biomass EJ':
    'pe_traditional_biomass_ej',
    'Historic share of losses vs extraction SUBliquids':
    'historic_share_of_losses_vs_extraction_subliquids',
    'Historic share of losses vs extraction SUBsolids':
    'historic_share_of_losses_vs_extraction_subsolids',
    'Historic share of losses vs extraction SUBgases':
    'historic_share_of_losses_vs_extraction_subgases',
    'Losses EJ':
    'losses_ej',
    'PES fossil fuel extraction':
    'pes_fossil_fuel_extraction',
    'modern BioE in households':
    'modern_bioe_in_households',
    'Historic demand by sector SUBAgricultureHuntingForestryandFishing':
    'historic_demand_by_sector_subagriculturehuntingforestryandfishing',
    'Historic demand by sector SUBMiningandQuarrying':
    'historic_demand_by_sector_subminingandquarrying',
    'Historic demand by sector SUBFoodBeveragesandTobacco':
    'historic_demand_by_sector_subfoodbeveragesandtobacco',
    'Historic demand by sector SUBTextilesandTextileProducts':
    'historic_demand_by_sector_subtextilesandtextileproducts',
    'Historic demand by sector SUBLeatherLeatherandFootwear':
    'historic_demand_by_sector_subleatherleatherandfootwear',
    'Historic demand by sector SUBWoodandProductsofWooodandCork':
    'historic_demand_by_sector_subwoodandproductsofwooodandcork',
    'Historic demand by sector SUBPulpPaperPrintingandPublishing':
    'historic_demand_by_sector_subpulppaperprintingandpublishing',
    'Historic demand by sector SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_demand_by_sector_subcokerefinedpetroleumandnuclearfuel',
    'Historic demand by sector SUBChemicalsandChemicalproducts':
    'historic_demand_by_sector_subchemicalsandchemicalproducts',
    'Historic demand by sector SUBRubberandPlastics':
    'historic_demand_by_sector_subrubberandplastics',
    'Historic demand by sector SUBOtherNonMetalicMineral':
    'historic_demand_by_sector_subothernonmetalicmineral',
    'Historic demand by sector SUBBasicMetalsandFabricatedMetal':
    'historic_demand_by_sector_subbasicmetalsandfabricatedmetal',
    'Historic demand by sector SUBMachineryNec':
    'historic_demand_by_sector_submachinerynec',
    'Historic demand by sector SUBElectricalandOpticalEquipment':
    'historic_demand_by_sector_subelectricalandopticalequipment',
    'Historic demand by sector SUBTransportEquipment':
    'historic_demand_by_sector_subtransportequipment',
    'Historic demand by sector SUBManufacturingNecRecycling':
    'historic_demand_by_sector_submanufacturingnecrecycling',
    'Historic demand by sector SUBElectricityGasandWaterSupply':
    'historic_demand_by_sector_subelectricitygasandwatersupply',
    'Historic demand by sector SUBConstruction':
    'historic_demand_by_sector_subconstruction',
    'Historic demand by sector SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_demand_by_sector_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic demand by sector SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_demand_by_sector_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic demand by sector SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_demand_by_sector_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic demand by sector SUBHotelsandRestaurants':
    'historic_demand_by_sector_subhotelsandrestaurants',
    'Historic demand by sector SUBInlandTransport':
    'historic_demand_by_sector_subinlandtransport',
    'Historic demand by sector SUBWaterTransport':
    'historic_demand_by_sector_subwatertransport',
    'Historic demand by sector SUBAirTransport':
    'historic_demand_by_sector_subairtransport',
    'Historic demand by sector SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_demand_by_sector_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic demand by sector SUBPostandTelecommunications':
    'historic_demand_by_sector_subpostandtelecommunications',
    'Historic demand by sector SUBFinancialIntermedation':
    'historic_demand_by_sector_subfinancialintermedation',
    'Historic demand by sector SUBRealEstateActivities':
    'historic_demand_by_sector_subrealestateactivities',
    'Historic demand by sector SUBRentingodMEqandOtherBusinessActivities':
    'historic_demand_by_sector_subrentingodmeqandotherbusinessactivities',
    'Historic demand by sector SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_demand_by_sector_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic demand by sector SUBEducation':
    'historic_demand_by_sector_subeducation',
    'Historic demand by sector SUBHealthandSocialWork':
    'historic_demand_by_sector_subhealthandsocialwork',
    'Historic demand by sector SUBOtherCommunitySocialandPersonaServices':
    'historic_demand_by_sector_subothercommunitysocialandpersonaservices',
    'Historic demand by sector SUBPrivateHouseholdswithEmployedPersons':
    'historic_demand_by_sector_subprivatehouseholdswithemployedpersons',
    'variation demand flow FD':
    'variation_demand_flow_fd',
    'historic variation demand':
    'historic_variation_demand',
    'Initial energy intensity 1995 H':
    'initial_energy_intensity_1995_h',
    'min energy intensity vs intial H':
    'min_energy_intensity_vs_intial_h',
    'minimum fraction H':
    'minimum_fraction_h',
    'Energy market pressure H':
    'energy_market_pressure_h',
    'Pressure to change energy technology H':
    'pressure_to_change_energy_technology_h',
    'Policy to improve efficiency speed H':
    'policy_to_improve_efficiency_speed_h',
    'Max yearly change H':
    'max_yearly_change_h',
    'available improvement efficiency H':
    'available_improvement_efficiency_h',
    'Year policy change energy H':
    'year_policy_change_energy_h',
    'Efficiency energy aceleration H':
    'efficiency_energy_aceleration_h',
    'efficiency rate of substitution H':
    'efficiency_rate_of_substitution_h',
    'Energy cost pressure H':
    'energy_cost_pressure_h',
    'abundance of energy H':
    'abundance_of_energy_h',
    'Global energy intensity H':
    'global_energy_intensity_h',
    'historic rate final energy intensity H':
    'historic_rate_final_energy_intensity_h',
    'Maximun yearly aceleration of intensity improvement pct H':
    'maximun_yearly_aceleration_of_intensity_improvement_pct_h',
    'historical mean rate energy intensity H':
    'historical_mean_rate_energy_intensity_h',
    'relative abundance of energy H':
    'relative_abundance_of_energy_h',
    'Policy change energy speed H':
    'policy_change_energy_speed_h',
    'Historic final energy intensity H SUBelectricity':
    'historic_final_energy_intensity_h_subelectricity',
    'Historic final energy intensity H SUBheat':
    'historic_final_energy_intensity_h_subheat',
    'Historic final energy intensity H SUBliquids':
    'historic_final_energy_intensity_h_subliquids',
    'Historic final energy intensity H SUBsolids':
    'historic_final_energy_intensity_h_subsolids',
    'Historic final energy intensity H SUBgases':
    'historic_final_energy_intensity_h_subgases',
    'Initial global energy intensity 2009 H':
    'initial_global_energy_intensity_2009_h',
    'Year policy to improve efficiency H':
    'year_policy_to_improve_efficiency_h',
    'share conv vs total oil extraction':
    'share_conv_vs_total_oil_extraction',
    'XPES natX gas without GTLX':
    'xpes_natx_gas_without_gtlx',
    'Tot FEH NRE':
    'tot_feh_nre',
    'CO2 emissions conv gas without GTL':
    'co2_emissions_conv_gas_without_gtl',
    'CO2 emissions conv oil':
    'co2_emissions_conv_oil',
    'CO2 emissions GTL':
    'co2_emissions_gtl',
    'PE gas extraction for Transp EJ':
    'pe_gas_extraction_for_transp_ej',
    'CH4 emissions conv gas without GTL':
    'ch4_emissions_conv_gas_without_gtl',
    'CO2 emissions unconv oil':
    'co2_emissions_unconv_oil',
    'XShortage coef without MIN without EXlossesX':
    'xshortage_coef_without_min_without_exlossesx',
    'share conv vs total gas extraction':
    'share_conv_vs_total_gas_extraction',
    'share FEH over PES coal solids':
    'share_feh_over_pes_coal_solids',
    'CH4 emissions unconv gas':
    'ch4_emissions_unconv_gas',
    'FEH coal delayed':
    'feh_coal_delayed',
    'FEH coal':
    'feh_coal',
    'FE heat demand consum after CC impacts EJ':
    'fe_heat_demand_consum_after_cc_impacts_ej',
    'share electricity vs TFES':
    'share_electricity_vs_tfes',
    'CO2 emissions peat':
    'co2_emissions_peat',
    'XTotal EXlosses due to CC impactsX':
    'xtotal_exlosses_due_to_cc_impactsx',
    'Required FED by gas':
    'required_fed_by_gas',
    'XEXlosses by final fuel due to CC impactsX':
    'xexlosses_by_final_fuel_due_to_cc_impactsx',
    'XEXlosses heatXcom due to CC impactsX':
    'xexlosses_heatxcom_due_to_cc_impactsx',
    'Required TFED':
    'required_tfed',
    'Total FE Elec consumption EJ':
    'total_fe_elec_consumption_ej',
    'Total FE Elec consumption TWh':
    'total_fe_elec_consumption_twh',
    'PED gases':
    'ped_gases',
    'Total FE Heat consumption EJ':
    'total_fe_heat_consumption_ej',
    'share solids vs TFES':
    'share_solids_vs_tfes',
    'Total CO2 emissions GTCO2':
    'total_co2_emissions_gtco2',
    'share gases vs TFES':
    'share_gases_vs_tfes',
    'Required FED solids':
    'required_fed_solids',
    'XFED HeatXnc EJX':
    'xfed_heatxnc_ejx',
    'share heat vs TFES':
    'share_heat_vs_tfes',
    'PED solids':
    'ped_solids',
    'scarcity conv oil':
    'scarcity_conv_oil',
    'test2':
    'test2',
    'share liquids vs TFES':
    'share_liquids_vs_tfes',
    'GFCF not covered':
    'gfcf_not_covered',
    'LC not covered':
    'lc_not_covered',
    'CC total not covered':
    'cc_total_not_covered',
    'Household demand not covered':
    'household_demand_not_covered',
    'Real total output inland transport':
    'real_total_output_inland_transport',
    'TFES intensity EJ Tdolar':
    'tfes_intensity_ej_tdolar',
    'TFES intensity EJ Tdolar delayed 1yr':
    'tfes_intensity_ej_tdolar_delayed_1yr',
    'TPES intensity EJ Tdolar delayed 1yr':
    'tpes_intensity_ej_tdolar_delayed_1yr',
    'Annual TPES intensity growth rate':
    'annual_tpes_intensity_growth_rate',
    'Annual TFES intensity growth rate':
    'annual_tfes_intensity_growth_rate',
    'Total D jobs RES elec per techn':
    'total_d_jobs_res_elec_per_techn',
    'Total D jobs RES heat per techn':
    'total_d_jobs_res_heat_per_techn',
    'Employment factors new RES elec':
    'employment_factors_new_res_elec',
    'Employment factors new RES heat':
    'employment_factors_new_res_heat',
    'XEmployment factors OXM RES elecX':
    'xemployment_factors_oxm_res_elecx',
    'XEmployment factors OXM RES heatX':
    'xemployment_factors_oxm_res_heatx',
    'D jobs new installed RES elec per techn':
    'd_jobs_new_installed_res_elec_per_techn',
    'XJobs OXM RES elec per technX':
    'xjobs_oxm_res_elec_per_technx',
    'NX0 vehicles per Xinland T':
    'nx0_vehicles_per_xinland_t',
    'initial percent T vehicles':
    'initial_percent_t_vehicles',
    'saving ratios V':
    'saving_ratios_v',
    'FEH gas delayed':
    'feh_gas_delayed',
    'share FEH over PES gases':
    'share_feh_over_pes_gases',
    'Xshare gases dem for HeatXncX':
    'xshare_gases_dem_for_heatxncx',
    'FEH gas':
    'feh_gas',
    'increase scarcity conv gas':
    'increase_scarcity_conv_gas',
    'scarcity conv gas stock':
    'scarcity_conv_gas_stock',
    'XPES natX gas for HeatXnc plantsX':
    'xpes_natx_gas_for_heatxnc_plantsx',
    'real growth CTL':
    'real_growth_ctl',
    'CTL potential production':
    'ctl_potential_production',
    'abundance liquids GTL':
    'abundance_liquids_gtl',
    'GTL potential production':
    'gtl_potential_production',
    'abundance liquids CTL':
    'abundance_liquids_ctl',
    'wear GTL':
    'wear_gtl',
    'lifetime GTL':
    'lifetime_gtl',
    'lifetime CTL':
    'lifetime_ctl',
    'wear CTL':
    'wear_ctl',
    'abundance unconv oil stock':
    'abundance_unconv_oil_stock',
    'increase abundance unconv oil':
    'increase_abundance_unconv_oil',
    'abundance unconv oil':
    'abundance_unconv_oil',
    'abundance unconv oil delayed 1yr':
    'abundance_unconv_oil_delayed_1yr',
    'abundance unconv oil2':
    'abundance_unconv_oil2',
    'real extraction unconv oil EJ':
    'real_extraction_unconv_oil_ej',
    'scarcity conv oil stock':
    'scarcity_conv_oil_stock',
    'Share variable RES elec generation vs total gen':
    'share_variable_res_elec_generation_vs_total_gen',
    'extra monet invest to cope with variable Elec RES':
    'extra_monet_invest_to_cope_with_variable_elec_res',
    'increase scarcity conv oil':
    'increase_scarcity_conv_oil',
    'share extra monet invest to cope with variable Elec RES':
    'share_extra_monet_invest_to_cope_with_variable_elec_res',
    'initial share variable RES elec gen vs total':
    'initial_share_variable_res_elec_gen_vs_total',
    'XPES oil for HeatXnc plantsX':
    'xpes_oil_for_heatxnc_plantsx',
    'cumulated invest E grid':
    'cumulated_invest_e_grid',
    'Total monet invest RES for elec Tdolar':
    'total_monet_invest_res_for_elec_tdolar',
    'Cp exogenous RES elec var reduction':
    'cp_exogenous_res_elec_var_reduction',
    'Cp exogenous RES elec dispatch reduction':
    'cp_exogenous_res_elec_dispatch_reduction',
    'increase variable RES share elec vs total generation':
    'increase_variable_res_share_elec_vs_total_generation',
    'available improvement efficiency':
    'available_improvement_efficiency',
    'XPES NRE HeatXncX':
    'xpes_nre_heatxncx',
    'share FEH over PES liquids':
    'share_feh_over_pes_liquids',
    'XFED natX gas for heatXncX':
    'xfed_natx_gas_for_heatxncx',
    'TPES heat':
    'tpes_heat',
    'XFED oil for heatXncX':
    'xfed_oil_for_heatxncx',
    'FEH oil':
    'feh_oil',
    'FEH oil delayed':
    'feh_oil_delayed',
    'PES NRE heat':
    'pes_nre_heat',
    'Xshare liquids dem for HeatXncX':
    'xshare_liquids_dem_for_heatxncx',
    'XPES NRE HeatXcomX':
    'xpes_nre_heatxcomx',
    'check TPE':
    'check_tpe',
    'XFED by fuel for heatXncX':
    'xfed_by_fuel_for_heatxncx',
    'XShare FED heatXcom vs total heatX':
    'xshare_fed_heatxcom_vs_total_heatx',
    'check hist Elec':
    'check_hist_elec',
    'Average elec consumption per capita':
    'average_elec_consumption_per_capita',
    'PES solids':
    'pes_solids',
    'XFED solid bioE for heatXncX':
    'xfed_solid_bioe_for_heatxncx',
    'Total FED Heat EJ':
    'total_fed_heat_ej',
    'XHeatXnc distribution lossesX':
    'xheatxnc_distribution_lossesx',
    'XTotal FED HeatXnc EJX':
    'xtotal_fed_heatxnc_ejx',
    'XTotal FED NRE HeatXncX':
    'xtotal_fed_nre_heatxncx',
    'XPED gas HeatXncX':
    'xped_gas_heatxncx',
    'XPED coal HeatXncX':
    'xped_coal_heatxncx',
    'XDeactivate heat dem correctionXX':
    'xdeactivate_heat_dem_correctionxx',
    'XPED liquids HeatXncX':
    'xped_liquids_heatxncx',
    'PED coal EJ':
    'ped_coal_ej',
    'share FEH over FED solid bioE':
    'share_feh_over_fed_solid_bioe',
    'FES Heat from coal':
    'fes_heat_from_coal',
    'XFES Heat from natX gasX':
    'xfes_heat_from_natx_gasx',
    'FES Heat from oil':
    'fes_heat_from_oil',
    'Total distribution losses':
    'total_distribution_losses',
    'Abundance heat':
    'abundance_heat',
    'XPES coal for HeatXnc plantsX':
    'xpes_coal_for_heatxnc_plantsx',
    'Population dependent on trad biomass':
    'population_dependent_on_trad_biomass',
    'required FED sectors by fuel':
    'required_fed_sectors_by_fuel',
    'share trad biomass vs solids in households':
    'share_trad_biomass_vs_solids_in_households',
    'PE traditional biomass EJ delayed 1yr':
    'pe_traditional_biomass_ej_delayed_1yr',
    'Xnew RES capacity for heatXcom TWX':
    'xnew_res_capacity_for_heatxcom_twx',
    'share global pop dependent on trad biomass':
    'share_global_pop_dependent_on_trad_biomass',
    'XHistoric RES capacity for heatXcomX SUBXsolarXheatX':
    'xhistoric_res_capacity_for_heatxcomx_subxsolarxheatx',
    'XHistoric RES capacity for heatXcomX SUBXgeotXheatX':
    'xhistoric_res_capacity_for_heatxcomx_subxgeotxheatx',
    'XHistoric RES capacity for heatXcomX SUBXsolidbioEXheatX':
    'xhistoric_res_capacity_for_heatxcomx_subxsolidbioexheatx',
    'XPES oil for HeatXcom plantsX':
    'xpes_oil_for_heatxcom_plantsx',
    'Xshare coal dem for HeatXncX':
    'xshare_coal_dem_for_heatxncx',
    'XRES heatXcom tot overcapacityX':
    'xres_heatxcom_tot_overcapacityx',
    'XPES natX gas for HeatXcom plantsX':
    'xpes_natx_gas_for_heatxcom_plantsx',
    'XPES coal for HeatXcom plantsX':
    'xpes_coal_for_heatxcom_plantsx',
    'check hist solids bioE':
    'check_hist_solids_bioe',
    'XRequired heatXcom after CC impactsX':
    'xrequired_heatxcom_after_cc_impactsx',
    'variation labour share':
    'variation_labour_share',
    'share inventories next step SUBAgricultureHuntingForestryandFishing':
    'share_inventories_next_step_subagriculturehuntingforestryandfishing',
    'share inventories next step SUBMiningandQuarrying':
    'share_inventories_next_step_subminingandquarrying',
    'share inventories next step SUBFoodBeveragesandTobacco':
    'share_inventories_next_step_subfoodbeveragesandtobacco',
    'share inventories next step SUBTextilesandTextileProducts':
    'share_inventories_next_step_subtextilesandtextileproducts',
    'share inventories next step SUBLeatherLeatherandFootwear':
    'share_inventories_next_step_subleatherleatherandfootwear',
    'share inventories next step SUBWoodandProductsofWooodandCork':
    'share_inventories_next_step_subwoodandproductsofwooodandcork',
    'share inventories next step SUBPulpPaperPrintingandPublishing':
    'share_inventories_next_step_subpulppaperprintingandpublishing',
    'share inventories next step SUBCokeRefinedPetroleumandNuclearFuel':
    'share_inventories_next_step_subcokerefinedpetroleumandnuclearfuel',
    'share inventories next step SUBChemicalsandChemicalproducts':
    'share_inventories_next_step_subchemicalsandchemicalproducts',
    'share inventories next step SUBRubberandPlastics':
    'share_inventories_next_step_subrubberandplastics',
    'share inventories next step SUBOtherNonMetalicMineral':
    'share_inventories_next_step_subothernonmetalicmineral',
    'share inventories next step SUBBasicMetalsandFabricatedMetal':
    'share_inventories_next_step_subbasicmetalsandfabricatedmetal',
    'share inventories next step SUBMachineryNec':
    'share_inventories_next_step_submachinerynec',
    'share inventories next step SUBElectricalandOpticalEquipment':
    'share_inventories_next_step_subelectricalandopticalequipment',
    'share inventories next step SUBTransportEquipment':
    'share_inventories_next_step_subtransportequipment',
    'share inventories next step SUBManufacturingNecRecycling':
    'share_inventories_next_step_submanufacturingnecrecycling',
    'share inventories next step SUBElectricityGasandWaterSupply':
    'share_inventories_next_step_subelectricitygasandwatersupply',
    'share inventories next step SUBConstruction':
    'share_inventories_next_step_subconstruction',
    'share inventories next step SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_inventories_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share inventories next step SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_inventories_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share inventories next step SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_inventories_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share inventories next step SUBHotelsandRestaurants':
    'share_inventories_next_step_subhotelsandrestaurants',
    'share inventories next step SUBInlandTransport':
    'share_inventories_next_step_subinlandtransport',
    'share inventories next step SUBWaterTransport':
    'share_inventories_next_step_subwatertransport',
    'share inventories next step SUBAirTransport':
    'share_inventories_next_step_subairtransport',
    'share inventories next step SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_inventories_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share inventories next step SUBPostandTelecommunications':
    'share_inventories_next_step_subpostandtelecommunications',
    'share inventories next step SUBFinancialIntermedation':
    'share_inventories_next_step_subfinancialintermedation',
    'share inventories next step SUBRealEstateActivities':
    'share_inventories_next_step_subrealestateactivities',
    'share inventories next step SUBRentingodMEqandOtherBusinessActivities':
    'share_inventories_next_step_subrentingodmeqandotherbusinessactivities',
    'share inventories next step SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_inventories_next_step_subpublicadminanddefencecompulsorysocialsecurity',
    'share inventories next step SUBEducation':
    'share_inventories_next_step_subeducation',
    'share inventories next step SUBHealthandSocialWork':
    'share_inventories_next_step_subhealthandsocialwork',
    'share inventories next step SUBOtherCommunitySocialandPersonaServices':
    'share_inventories_next_step_subothercommunitysocialandpersonaservices',
    'share inventories next step SUBPrivateHouseholdswithEmployedPersons':
    'share_inventories_next_step_subprivatehouseholdswithemployedpersons',
    'variation historic demand':
    'variation_historic_demand',
    'share consum goverments SUBAgricultureHuntingForestryandFishing':
    'share_consum_goverments_subagriculturehuntingforestryandfishing',
    'share consum goverments SUBMiningandQuarrying':
    'share_consum_goverments_subminingandquarrying',
    'share consum goverments SUBFoodBeveragesandTobacco':
    'share_consum_goverments_subfoodbeveragesandtobacco',
    'share consum goverments SUBTextilesandTextileProducts':
    'share_consum_goverments_subtextilesandtextileproducts',
    'share consum goverments SUBLeatherLeatherandFootwear':
    'share_consum_goverments_subleatherleatherandfootwear',
    'share consum goverments SUBWoodandProductsofWooodandCork':
    'share_consum_goverments_subwoodandproductsofwooodandcork',
    'share consum goverments SUBPulpPaperPrintingandPublishing':
    'share_consum_goverments_subpulppaperprintingandpublishing',
    'share consum goverments SUBCokeRefinedPetroleumandNuclearFuel':
    'share_consum_goverments_subcokerefinedpetroleumandnuclearfuel',
    'share consum goverments SUBChemicalsandChemicalproducts':
    'share_consum_goverments_subchemicalsandchemicalproducts',
    'share consum goverments SUBRubberandPlastics':
    'share_consum_goverments_subrubberandplastics',
    'share consum goverments SUBOtherNonMetalicMineral':
    'share_consum_goverments_subothernonmetalicmineral',
    'share consum goverments SUBBasicMetalsandFabricatedMetal':
    'share_consum_goverments_subbasicmetalsandfabricatedmetal',
    'share consum goverments SUBMachineryNec':
    'share_consum_goverments_submachinerynec',
    'share consum goverments SUBElectricalandOpticalEquipment':
    'share_consum_goverments_subelectricalandopticalequipment',
    'share consum goverments SUBTransportEquipment':
    'share_consum_goverments_subtransportequipment',
    'share consum goverments SUBManufacturingNecRecycling':
    'share_consum_goverments_submanufacturingnecrecycling',
    'share consum goverments SUBElectricityGasandWaterSupply':
    'share_consum_goverments_subelectricitygasandwatersupply',
    'share consum goverments SUBConstruction':
    'share_consum_goverments_subconstruction',
    'share consum goverments SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_consum_goverments_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share consum goverments SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_consum_goverments_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share consum goverments SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_consum_goverments_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share consum goverments SUBHotelsandRestaurants':
    'share_consum_goverments_subhotelsandrestaurants',
    'share consum goverments SUBInlandTransport':
    'share_consum_goverments_subinlandtransport',
    'share consum goverments SUBWaterTransport':
    'share_consum_goverments_subwatertransport',
    'share consum goverments SUBAirTransport':
    'share_consum_goverments_subairtransport',
    'share consum goverments SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_consum_goverments_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share consum goverments SUBPostandTelecommunications':
    'share_consum_goverments_subpostandtelecommunications',
    'share consum goverments SUBFinancialIntermedation':
    'share_consum_goverments_subfinancialintermedation',
    'share consum goverments SUBRealEstateActivities':
    'share_consum_goverments_subrealestateactivities',
    'share consum goverments SUBRentingodMEqandOtherBusinessActivities':
    'share_consum_goverments_subrentingodmeqandotherbusinessactivities',
    'share consum goverments SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_consum_goverments_subpublicadminanddefencecompulsorysocialsecurity',
    'share consum goverments SUBEducation':
    'share_consum_goverments_subeducation',
    'share consum goverments SUBHealthandSocialWork':
    'share_consum_goverments_subhealthandsocialwork',
    'share consum goverments SUBOtherCommunitySocialandPersonaServices':
    'share_consum_goverments_subothercommunitysocialandpersonaservices',
    'share consum goverments SUBPrivateHouseholdswithEmployedPersons':
    'share_consum_goverments_subprivatehouseholdswithemployedpersons',
    'variation GFCF':
    'variation_gfcf',
    'variation historic GFCF':
    'variation_historic_gfcf',
    'variation household demand':
    'variation_household_demand',
    'labour share':
    'labour_share',
    'share inventories SUBAgricultureHuntingForestryandFishing':
    'share_inventories_subagriculturehuntingforestryandfishing',
    'share inventories SUBMiningandQuarrying':
    'share_inventories_subminingandquarrying',
    'share inventories SUBFoodBeveragesandTobacco':
    'share_inventories_subfoodbeveragesandtobacco',
    'share inventories SUBTextilesandTextileProducts':
    'share_inventories_subtextilesandtextileproducts',
    'share inventories SUBLeatherLeatherandFootwear':
    'share_inventories_subleatherleatherandfootwear',
    'share inventories SUBWoodandProductsofWooodandCork':
    'share_inventories_subwoodandproductsofwooodandcork',
    'share inventories SUBPulpPaperPrintingandPublishing':
    'share_inventories_subpulppaperprintingandpublishing',
    'share inventories SUBCokeRefinedPetroleumandNuclearFuel':
    'share_inventories_subcokerefinedpetroleumandnuclearfuel',
    'share inventories SUBChemicalsandChemicalproducts':
    'share_inventories_subchemicalsandchemicalproducts',
    'share inventories SUBRubberandPlastics':
    'share_inventories_subrubberandplastics',
    'share inventories SUBOtherNonMetalicMineral':
    'share_inventories_subothernonmetalicmineral',
    'share inventories SUBBasicMetalsandFabricatedMetal':
    'share_inventories_subbasicmetalsandfabricatedmetal',
    'share inventories SUBMachineryNec':
    'share_inventories_submachinerynec',
    'share inventories SUBElectricalandOpticalEquipment':
    'share_inventories_subelectricalandopticalequipment',
    'share inventories SUBTransportEquipment':
    'share_inventories_subtransportequipment',
    'share inventories SUBManufacturingNecRecycling':
    'share_inventories_submanufacturingnecrecycling',
    'share inventories SUBElectricityGasandWaterSupply':
    'share_inventories_subelectricitygasandwatersupply',
    'share inventories SUBConstruction':
    'share_inventories_subconstruction',
    'share inventories SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_inventories_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share inventories SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_inventories_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share inventories SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_inventories_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share inventories SUBHotelsandRestaurants':
    'share_inventories_subhotelsandrestaurants',
    'share inventories SUBInlandTransport':
    'share_inventories_subinlandtransport',
    'share inventories SUBWaterTransport':
    'share_inventories_subwatertransport',
    'share inventories SUBAirTransport':
    'share_inventories_subairtransport',
    'share inventories SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_inventories_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share inventories SUBPostandTelecommunications':
    'share_inventories_subpostandtelecommunications',
    'share inventories SUBFinancialIntermedation':
    'share_inventories_subfinancialintermedation',
    'share inventories SUBRealEstateActivities':
    'share_inventories_subrealestateactivities',
    'share inventories SUBRentingodMEqandOtherBusinessActivities':
    'share_inventories_subrentingodmeqandotherbusinessactivities',
    'share inventories SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_inventories_subpublicadminanddefencecompulsorysocialsecurity',
    'share inventories SUBEducation':
    'share_inventories_subeducation',
    'share inventories SUBHealthandSocialWork':
    'share_inventories_subhealthandsocialwork',
    'share inventories SUBOtherCommunitySocialandPersonaServices':
    'share_inventories_subothercommunitysocialandpersonaservices',
    'share inventories SUBPrivateHouseholdswithEmployedPersons':
    'share_inventories_subprivatehouseholdswithemployedpersons',
    'share consum goverments next step SUBAgricultureHuntingForestryandFishing':
    'share_consum_goverments_next_step_subagriculturehuntingforestryandfishing',
    'share consum goverments next step SUBMiningandQuarrying':
    'share_consum_goverments_next_step_subminingandquarrying',
    'share consum goverments next step SUBFoodBeveragesandTobacco':
    'share_consum_goverments_next_step_subfoodbeveragesandtobacco',
    'share consum goverments next step SUBTextilesandTextileProducts':
    'share_consum_goverments_next_step_subtextilesandtextileproducts',
    'share consum goverments next step SUBLeatherLeatherandFootwear':
    'share_consum_goverments_next_step_subleatherleatherandfootwear',
    'share consum goverments next step SUBWoodandProductsofWooodandCork':
    'share_consum_goverments_next_step_subwoodandproductsofwooodandcork',
    'share consum goverments next step SUBPulpPaperPrintingandPublishing':
    'share_consum_goverments_next_step_subpulppaperprintingandpublishing',
    'share consum goverments next step SUBCokeRefinedPetroleumandNuclearFuel':
    'share_consum_goverments_next_step_subcokerefinedpetroleumandnuclearfuel',
    'share consum goverments next step SUBChemicalsandChemicalproducts':
    'share_consum_goverments_next_step_subchemicalsandchemicalproducts',
    'share consum goverments next step SUBRubberandPlastics':
    'share_consum_goverments_next_step_subrubberandplastics',
    'share consum goverments next step SUBOtherNonMetalicMineral':
    'share_consum_goverments_next_step_subothernonmetalicmineral',
    'share consum goverments next step SUBBasicMetalsandFabricatedMetal':
    'share_consum_goverments_next_step_subbasicmetalsandfabricatedmetal',
    'share consum goverments next step SUBMachineryNec':
    'share_consum_goverments_next_step_submachinerynec',
    'share consum goverments next step SUBElectricalandOpticalEquipment':
    'share_consum_goverments_next_step_subelectricalandopticalequipment',
    'share consum goverments next step SUBTransportEquipment':
    'share_consum_goverments_next_step_subtransportequipment',
    'share consum goverments next step SUBManufacturingNecRecycling':
    'share_consum_goverments_next_step_submanufacturingnecrecycling',
    'share consum goverments next step SUBElectricityGasandWaterSupply':
    'share_consum_goverments_next_step_subelectricitygasandwatersupply',
    'share consum goverments next step SUBConstruction':
    'share_consum_goverments_next_step_subconstruction',
    'share consum goverments next step SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_consum_goverments_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share consum goverments next step SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_consum_goverments_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share consum goverments next step SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_consum_goverments_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share consum goverments next step SUBHotelsandRestaurants':
    'share_consum_goverments_next_step_subhotelsandrestaurants',
    'share consum goverments next step SUBInlandTransport':
    'share_consum_goverments_next_step_subinlandtransport',
    'share consum goverments next step SUBWaterTransport':
    'share_consum_goverments_next_step_subwatertransport',
    'share consum goverments next step SUBAirTransport':
    'share_consum_goverments_next_step_subairtransport',
    'share consum goverments next step SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_consum_goverments_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share consum goverments next step SUBPostandTelecommunications':
    'share_consum_goverments_next_step_subpostandtelecommunications',
    'share consum goverments next step SUBFinancialIntermedation':
    'share_consum_goverments_next_step_subfinancialintermedation',
    'share consum goverments next step SUBRealEstateActivities':
    'share_consum_goverments_next_step_subrealestateactivities',
    'share consum goverments next step SUBRentingodMEqandOtherBusinessActivities':
    'share_consum_goverments_next_step_subrentingodmeqandotherbusinessactivities',
    'share consum goverments next step SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_consum_goverments_next_step_subpublicadminanddefencecompulsorysocialsecurity',
    'share consum goverments next step SUBEducation':
    'share_consum_goverments_next_step_subeducation',
    'share consum goverments next step SUBHealthandSocialWork':
    'share_consum_goverments_next_step_subhealthandsocialwork',
    'share consum goverments next step SUBOtherCommunitySocialandPersonaServices':
    'share_consum_goverments_next_step_subothercommunitysocialandpersonaservices',
    'share consum goverments next step SUBPrivateHouseholdswithEmployedPersons':
    'share_consum_goverments_next_step_subprivatehouseholdswithemployedpersons',
    'share consum goverment and inventories':
    'share_consum_goverment_and_inventories',
    'share consum goverments and inventories next step':
    'share_consum_goverments_and_inventories_next_step',
    'capital share':
    'capital_share',
    'historic growth capital share':
    'historic_growth_capital_share',
    'Historic GFCF SUBAgricultureHuntingForestryandFishing':
    'historic_gfcf_subagriculturehuntingforestryandfishing',
    'Historic GFCF SUBMiningandQuarrying':
    'historic_gfcf_subminingandquarrying',
    'Historic GFCF SUBFoodBeveragesandTobacco':
    'historic_gfcf_subfoodbeveragesandtobacco',
    'Historic GFCF SUBTextilesandTextileProducts':
    'historic_gfcf_subtextilesandtextileproducts',
    'Historic GFCF SUBLeatherLeatherandFootwear':
    'historic_gfcf_subleatherleatherandfootwear',
    'Historic GFCF SUBWoodandProductsofWooodandCork':
    'historic_gfcf_subwoodandproductsofwooodandcork',
    'Historic GFCF SUBPulpPaperPrintingandPublishing':
    'historic_gfcf_subpulppaperprintingandpublishing',
    'Historic GFCF SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_gfcf_subcokerefinedpetroleumandnuclearfuel',
    'Historic GFCF SUBChemicalsandChemicalproducts':
    'historic_gfcf_subchemicalsandchemicalproducts',
    'Historic GFCF SUBRubberandPlastics':
    'historic_gfcf_subrubberandplastics',
    'Historic GFCF SUBOtherNonMetalicMineral':
    'historic_gfcf_subothernonmetalicmineral',
    'Historic GFCF SUBBasicMetalsandFabricatedMetal':
    'historic_gfcf_subbasicmetalsandfabricatedmetal',
    'Historic GFCF SUBMachineryNec':
    'historic_gfcf_submachinerynec',
    'Historic GFCF SUBElectricalandOpticalEquipment':
    'historic_gfcf_subelectricalandopticalequipment',
    'Historic GFCF SUBTransportEquipment':
    'historic_gfcf_subtransportequipment',
    'Historic GFCF SUBManufacturingNecRecycling':
    'historic_gfcf_submanufacturingnecrecycling',
    'Historic GFCF SUBElectricityGasandWaterSupply':
    'historic_gfcf_subelectricitygasandwatersupply',
    'Historic GFCF SUBConstruction':
    'historic_gfcf_subconstruction',
    'Historic GFCF SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_gfcf_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic GFCF SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_gfcf_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic GFCF SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_gfcf_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic GFCF SUBHotelsandRestaurants':
    'historic_gfcf_subhotelsandrestaurants',
    'Historic GFCF SUBInlandTransport':
    'historic_gfcf_subinlandtransport',
    'Historic GFCF SUBWaterTransport':
    'historic_gfcf_subwatertransport',
    'Historic GFCF SUBAirTransport':
    'historic_gfcf_subairtransport',
    'Historic GFCF SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_gfcf_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic GFCF SUBPostandTelecommunications':
    'historic_gfcf_subpostandtelecommunications',
    'Historic GFCF SUBFinancialIntermedation':
    'historic_gfcf_subfinancialintermedation',
    'Historic GFCF SUBRealEstateActivities':
    'historic_gfcf_subrealestateactivities',
    'Historic GFCF SUBRentingodMEqandOtherBusinessActivities':
    'historic_gfcf_subrentingodmeqandotherbusinessactivities',
    'Historic GFCF SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_gfcf_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic GFCF SUBEducation':
    'historic_gfcf_subeducation',
    'Historic GFCF SUBHealthandSocialWork':
    'historic_gfcf_subhealthandsocialwork',
    'Historic GFCF SUBOtherCommunitySocialandPersonaServices':
    'historic_gfcf_subothercommunitysocialandpersonaservices',
    'Historic GFCF SUBPrivateHouseholdswithEmployedPersons':
    'historic_gfcf_subprivatehouseholdswithemployedpersons',
    'Demand by sector':
    'demand_by_sector',
    'Historic HD SUBAgricultureHuntingForestryandFishing':
    'historic_hd_subagriculturehuntingforestryandfishing',
    'Historic HD SUBMiningandQuarrying':
    'historic_hd_subminingandquarrying',
    'Historic HD SUBFoodBeveragesandTobacco':
    'historic_hd_subfoodbeveragesandtobacco',
    'Historic HD SUBTextilesandTextileProducts':
    'historic_hd_subtextilesandtextileproducts',
    'Historic HD SUBLeatherLeatherandFootwear':
    'historic_hd_subleatherleatherandfootwear',
    'Historic HD SUBWoodandProductsofWooodandCork':
    'historic_hd_subwoodandproductsofwooodandcork',
    'Historic HD SUBPulpPaperPrintingandPublishing':
    'historic_hd_subpulppaperprintingandpublishing',
    'Historic HD SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_hd_subcokerefinedpetroleumandnuclearfuel',
    'Historic HD SUBChemicalsandChemicalproducts':
    'historic_hd_subchemicalsandchemicalproducts',
    'Historic HD SUBRubberandPlastics':
    'historic_hd_subrubberandplastics',
    'Historic HD SUBOtherNonMetalicMineral':
    'historic_hd_subothernonmetalicmineral',
    'Historic HD SUBBasicMetalsandFabricatedMetal':
    'historic_hd_subbasicmetalsandfabricatedmetal',
    'Historic HD SUBMachineryNec':
    'historic_hd_submachinerynec',
    'Historic HD SUBElectricalandOpticalEquipment':
    'historic_hd_subelectricalandopticalequipment',
    'Historic HD SUBTransportEquipment':
    'historic_hd_subtransportequipment',
    'Historic HD SUBManufacturingNecRecycling':
    'historic_hd_submanufacturingnecrecycling',
    'Historic HD SUBElectricityGasandWaterSupply':
    'historic_hd_subelectricitygasandwatersupply',
    'Historic HD SUBConstruction':
    'historic_hd_subconstruction',
    'Historic HD SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_hd_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic HD SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_hd_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic HD SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_hd_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic HD SUBHotelsandRestaurants':
    'historic_hd_subhotelsandrestaurants',
    'Historic HD SUBInlandTransport':
    'historic_hd_subinlandtransport',
    'Historic HD SUBWaterTransport':
    'historic_hd_subwatertransport',
    'Historic HD SUBAirTransport':
    'historic_hd_subairtransport',
    'Historic HD SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_hd_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic HD SUBPostandTelecommunications':
    'historic_hd_subpostandtelecommunications',
    'Historic HD SUBFinancialIntermedation':
    'historic_hd_subfinancialintermedation',
    'Historic HD SUBRealEstateActivities':
    'historic_hd_subrealestateactivities',
    'Historic HD SUBRentingodMEqandOtherBusinessActivities':
    'historic_hd_subrentingodmeqandotherbusinessactivities',
    'Historic HD SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_hd_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic HD SUBEducation':
    'historic_hd_subeducation',
    'Historic HD SUBHealthandSocialWork':
    'historic_hd_subhealthandsocialwork',
    'Historic HD SUBOtherCommunitySocialandPersonaServices':
    'historic_hd_subothercommunitysocialandpersonaservices',
    'Historic HD SUBPrivateHouseholdswithEmployedPersons':
    'historic_hd_subprivatehouseholdswithemployedpersons',
    'Xinclude materials for overgridsXX':
    'xinclude_materials_for_overgridsxx',
    'XAll minerals virginXX':
    'xall_minerals_virginxx',
    'materials for new RES elec per capacity installed':
    'materials_for_new_res_elec_per_capacity_installed',
    'CEDtot per material RES elec var':
    'cedtot_per_material_res_elec_var',
    'CEDtot per TW RES elec var':
    'cedtot_per_tw_res_elec_var',
    'CEDtot per TW per material RES elec var':
    'cedtot_per_tw_per_material_res_elec_var',
    'Demand by sector FD':
    'demand_by_sector_fd',
    'sum variation':
    'sum_variation',
    'CC sectoral':
    'cc_sectoral',
    'CC sectoral next step':
    'cc_sectoral_next_step',
    'Gross fixed capital formation':
    'gross_fixed_capital_formation',
    'variation CC sectoral':
    'variation_cc_sectoral',
    'share CC next step SUBAgricultureHuntingForestryandFishing':
    'share_cc_next_step_subagriculturehuntingforestryandfishing',
    'share CC next step SUBMiningandQuarrying':
    'share_cc_next_step_subminingandquarrying',
    'share CC next step SUBFoodBeveragesandTobacco':
    'share_cc_next_step_subfoodbeveragesandtobacco',
    'share CC next step SUBTextilesandTextileProducts':
    'share_cc_next_step_subtextilesandtextileproducts',
    'share CC next step SUBLeatherLeatherandFootwear':
    'share_cc_next_step_subleatherleatherandfootwear',
    'share CC next step SUBWoodandProductsofWooodandCork':
    'share_cc_next_step_subwoodandproductsofwooodandcork',
    'share CC next step SUBPulpPaperPrintingandPublishing':
    'share_cc_next_step_subpulppaperprintingandpublishing',
    'share CC next step SUBCokeRefinedPetroleumandNuclearFuel':
    'share_cc_next_step_subcokerefinedpetroleumandnuclearfuel',
    'share CC next step SUBChemicalsandChemicalproducts':
    'share_cc_next_step_subchemicalsandchemicalproducts',
    'share CC next step SUBRubberandPlastics':
    'share_cc_next_step_subrubberandplastics',
    'share CC next step SUBOtherNonMetalicMineral':
    'share_cc_next_step_subothernonmetalicmineral',
    'share CC next step SUBBasicMetalsandFabricatedMetal':
    'share_cc_next_step_subbasicmetalsandfabricatedmetal',
    'share CC next step SUBMachineryNec':
    'share_cc_next_step_submachinerynec',
    'share CC next step SUBElectricalandOpticalEquipment':
    'share_cc_next_step_subelectricalandopticalequipment',
    'share CC next step SUBTransportEquipment':
    'share_cc_next_step_subtransportequipment',
    'share CC next step SUBManufacturingNecRecycling':
    'share_cc_next_step_submanufacturingnecrecycling',
    'share CC next step SUBElectricityGasandWaterSupply':
    'share_cc_next_step_subelectricitygasandwatersupply',
    'share CC next step SUBConstruction':
    'share_cc_next_step_subconstruction',
    'share CC next step SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_cc_next_step_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share CC next step SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_cc_next_step_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share CC next step SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_cc_next_step_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share CC next step SUBHotelsandRestaurants':
    'share_cc_next_step_subhotelsandrestaurants',
    'share CC next step SUBInlandTransport':
    'share_cc_next_step_subinlandtransport',
    'share CC next step SUBWaterTransport':
    'share_cc_next_step_subwatertransport',
    'share CC next step SUBAirTransport':
    'share_cc_next_step_subairtransport',
    'share CC next step SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_cc_next_step_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share CC next step SUBPostandTelecommunications':
    'share_cc_next_step_subpostandtelecommunications',
    'share CC next step SUBFinancialIntermedation':
    'share_cc_next_step_subfinancialintermedation',
    'share CC next step SUBRealEstateActivities':
    'share_cc_next_step_subrealestateactivities',
    'share CC next step SUBRentingodMEqandOtherBusinessActivities':
    'share_cc_next_step_subrentingodmeqandotherbusinessactivities',
    'share CC next step SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_cc_next_step_subpublicadminanddefencecompulsorysocialsecurity',
    'share CC next step SUBEducation':
    'share_cc_next_step_subeducation',
    'share CC next step SUBHealthandSocialWork':
    'share_cc_next_step_subhealthandsocialwork',
    'share CC next step SUBOtherCommunitySocialandPersonaServices':
    'share_cc_next_step_subothercommunitysocialandpersonaservices',
    'share CC next step SUBPrivateHouseholdswithEmployedPersons':
    'share_cc_next_step_subprivatehouseholdswithemployedpersons',
    'Xpct GFCF vs GFCFXHDX':
    'xpct_gfcf_vs_gfcfxhdx',
    'Household demand':
    'household_demand',
    'LC next step':
    'lc_next_step',
    'Household demand total':
    'household_demand_total',
    'CEDtot per TW over lifetime RES elec dispatch':
    'cedtot_per_tw_over_lifetime_res_elec_dispatch',
    'quality of electricity 2015':
    'quality_of_electricity_2015',
    'Dynamic quality of electricity':
    'dynamic_quality_of_electricity',
    'quality of electricity':
    'quality_of_electricity',
    'Cp exogenous RES elec reduction':
    'cp_exogenous_res_elec_reduction',
    'Cp RES elec':
    'cp_res_elec',
    'XRES elec variablesXX':
    'xres_elec_variablesxx',
    'aux3':
    'aux3',
    'overdemand elec EROI from 2015':
    'overdemand_elec_eroi_from_2015',
    'FEI RES elec var':
    'fei_res_elec_var',
    'XXstaticX EROI RES elecX':
    'xxstaticx_eroi_res_elecx',
    'materials required for new RES elec Mt':
    'materials_required_for_new_res_elec_mt',
    'EROIst tot elec':
    'eroist_tot_elec',
    'FEI over lifetime RES elec var':
    'fei_over_lifetime_res_elec_var',
    'CED decom RES elec capacity':
    'ced_decom_res_elec_capacity',
    'test ir restando al EROIst tot elec':
    'test_ir_restando_al_eroist_tot_elec',
    'EROIst tot elec 2':
    'eroist_tot_elec_2',
    'XCED OXM per material RES elec varX':
    'xced_oxm_per_material_res_elec_varx',
    'CEDtot new cap RES elec var':
    'cedtot_new_cap_res_elec_var',
    'XCEDtot OXM RES elec varX':
    'xcedtot_oxm_res_elec_varx',
    'overdemand elec tot until 2015':
    'overdemand_elec_tot_until_2015',
    'aux2':
    'aux2',
    'aux1':
    'aux1',
    'overdemand elec tot':
    'overdemand_elec_tot',
    'overdemand elec EROI delayed 1yr':
    'overdemand_elec_eroi_delayed_1yr',
    'XActivate EROI elec feedbackXX':
    'xactivate_eroi_elec_feedbackxx',
    'CED unconv gas for elec':
    'ced_unconv_gas_for_elec',
    'CED uranium for elec':
    'ced_uranium_for_elec',
    'CEDtot over lifetime NRE elec':
    'cedtot_over_lifetime_nre_elec',
    'b lineal regr':
    'b_lineal_regr',
    'EROI uranium':
    'eroi_uranium',
    'EROI coal':
    'eroi_coal',
    'EROI conv gas':
    'eroi_conv_gas',
    'EROI total oil':
    'eroi_total_oil',
    'EROI unconv gas':
    'eroi_unconv_gas',
    'CED coal for elec':
    'ced_coal_for_elec',
    'CED conv gas for elec':
    'ced_conv_gas_for_elec',
    'a lineal regr':
    'a_lineal_regr',
    'CED total oil for elec':
    'ced_total_oil_for_elec',
    'ESOI PHS depleted potential':
    'esoi_phs_depleted_potential',
    'Historic labour share':
    'historic_labour_share',
    'CC total':
    'cc_total',
    'initial household demand':
    'initial_household_demand',
    'total demand':
    'total_demand',
    'demand not covered by sector FD':
    'demand_not_covered_by_sector_fd',
    'initial demand by sectot':
    'initial_demand_by_sectot',
    'share CC sectoral SUBAgricultureHuntingForestryandFishing':
    'share_cc_sectoral_subagriculturehuntingforestryandfishing',
    'share CC sectoral SUBMiningandQuarrying':
    'share_cc_sectoral_subminingandquarrying',
    'share CC sectoral SUBFoodBeveragesandTobacco':
    'share_cc_sectoral_subfoodbeveragesandtobacco',
    'share CC sectoral SUBTextilesandTextileProducts':
    'share_cc_sectoral_subtextilesandtextileproducts',
    'share CC sectoral SUBLeatherLeatherandFootwear':
    'share_cc_sectoral_subleatherleatherandfootwear',
    'share CC sectoral SUBWoodandProductsofWooodandCork':
    'share_cc_sectoral_subwoodandproductsofwooodandcork',
    'share CC sectoral SUBPulpPaperPrintingandPublishing':
    'share_cc_sectoral_subpulppaperprintingandpublishing',
    'share CC sectoral SUBCokeRefinedPetroleumandNuclearFuel':
    'share_cc_sectoral_subcokerefinedpetroleumandnuclearfuel',
    'share CC sectoral SUBChemicalsandChemicalproducts':
    'share_cc_sectoral_subchemicalsandchemicalproducts',
    'share CC sectoral SUBRubberandPlastics':
    'share_cc_sectoral_subrubberandplastics',
    'share CC sectoral SUBOtherNonMetalicMineral':
    'share_cc_sectoral_subothernonmetalicmineral',
    'share CC sectoral SUBBasicMetalsandFabricatedMetal':
    'share_cc_sectoral_subbasicmetalsandfabricatedmetal',
    'share CC sectoral SUBMachineryNec':
    'share_cc_sectoral_submachinerynec',
    'share CC sectoral SUBElectricalandOpticalEquipment':
    'share_cc_sectoral_subelectricalandopticalequipment',
    'share CC sectoral SUBTransportEquipment':
    'share_cc_sectoral_subtransportequipment',
    'share CC sectoral SUBManufacturingNecRecycling':
    'share_cc_sectoral_submanufacturingnecrecycling',
    'share CC sectoral SUBElectricityGasandWaterSupply':
    'share_cc_sectoral_subelectricitygasandwatersupply',
    'share CC sectoral SUBConstruction':
    'share_cc_sectoral_subconstruction',
    'share CC sectoral SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'share_cc_sectoral_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'share CC sectoral SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'share_cc_sectoral_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'share CC sectoral SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'share_cc_sectoral_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'share CC sectoral SUBHotelsandRestaurants':
    'share_cc_sectoral_subhotelsandrestaurants',
    'share CC sectoral SUBInlandTransport':
    'share_cc_sectoral_subinlandtransport',
    'share CC sectoral SUBWaterTransport':
    'share_cc_sectoral_subwatertransport',
    'share CC sectoral SUBAirTransport':
    'share_cc_sectoral_subairtransport',
    'share CC sectoral SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'share_cc_sectoral_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'share CC sectoral SUBPostandTelecommunications':
    'share_cc_sectoral_subpostandtelecommunications',
    'share CC sectoral SUBFinancialIntermedation':
    'share_cc_sectoral_subfinancialintermedation',
    'share CC sectoral SUBRealEstateActivities':
    'share_cc_sectoral_subrealestateactivities',
    'share CC sectoral SUBRentingodMEqandOtherBusinessActivities':
    'share_cc_sectoral_subrentingodmeqandotherbusinessactivities',
    'share CC sectoral SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'share_cc_sectoral_subpublicadminanddefencecompulsorysocialsecurity',
    'share CC sectoral SUBEducation':
    'share_cc_sectoral_subeducation',
    'share CC sectoral SUBHealthandSocialWork':
    'share_cc_sectoral_subhealthandsocialwork',
    'share CC sectoral SUBOtherCommunitySocialandPersonaServices':
    'share_cc_sectoral_subothercommunitysocialandpersonaservices',
    'share CC sectoral SUBPrivateHouseholdswithEmployedPersons':
    'share_cc_sectoral_subprivatehouseholdswithemployedpersons',
    'Beta 1 lab':
    'beta_1_lab',
    'initial LC total':
    'initial_lc_total',
    'initial GFCF':
    'initial_gfcf',
    'beta 0 cap':
    'beta_0_cap',
    'initial CC total':
    'initial_cc_total',
    'beta 2 cap':
    'beta_2_cap',
    'demand not covered total FD':
    'demand_not_covered_total_fd',
    'beta 1 cap':
    'beta_1_cap',
    'LC':
    'lc',
    'Bet 0 lab':
    'bet_0_lab',
    'TPED by fuel':
    'tped_by_fuel',
    'abundance TPE':
    'abundance_tpe',
    'beta 2 lab':
    'beta_2_lab',
    'historic rate final energy intensity':
    'historic_rate_final_energy_intensity',
    'Initial global energy intensity by sector 2009':
    'initial_global_energy_intensity_by_sector_2009',
    'Efficiency energy aceleration':
    'efficiency_energy_aceleration',
    'available PE potential solid bioE for heat EJ':
    'available_pe_potential_solid_bioe_for_heat_ej',
    'Max PE potential RES for heat':
    'max_pe_potential_res_for_heat',
    'XPES RES for heatXcom by technX':
    'xpes_res_for_heatxcom_by_technx',
    'P RES for heat':
    'p_res_for_heat',
    'XFE real generation RES heatXcom EJX':
    'xfe_real_generation_res_heatxcom_ejx',
    'Xpotential FES tot RES for heatXcom EJX':
    'xpotential_fes_tot_res_for_heatxcom_ejx',
    'XTotal PE solid bioE potential heatXelec EJX':
    'xtotal_pe_solid_bioe_potential_heatxelec_ejx',
    'available potential FE solid bioE for elec EJ':
    'available_potential_fe_solid_bioe_for_elec_ej',
    'Xreplacement RES for heatXcom TWX':
    'xreplacement_res_for_heatxcom_twx',
    'max BioE TWe':
    'max_bioe_twe',
    'remaining potential tot RES heat':
    'remaining_potential_tot_res_heat',
    'XPotential FES HeatXcom nuclear CHP plants EJX':
    'xpotential_fes_heatxcom_nuclear_chp_plants_ejx',
    'share of heat production in CHP plants vs total nucelar elec generation':
    'share_of_heat_production_in_chp_plants_vs_total_nucelar_elec_generation',
    'PES waste for elec plants':
    'pes_waste_for_elec_plants',
    'Losses CHP biogas':
    'losses_chp_biogas',
    'Losses CHP waste':
    'losses_chp_waste',
    'share PES biogas for heat':
    'share_pes_biogas_for_heat',
    'PE losses RES for elec':
    'pe_losses_res_for_elec',
    'PES tot waste for elec':
    'pes_tot_waste_for_elec',
    'XPES tot waste for heatXcomX':
    'xpes_tot_waste_for_heatxcomx',
    'Xabundance RES heatXcom2X':
    'xabundance_res_heatxcom2x',
    'max PE potential biogas for heat':
    'max_pe_potential_biogas_for_heat',
    'share efficiency biogas for elec in CHP plants':
    'share_efficiency_biogas_for_elec_in_chp_plants',
    'share efficiency waste for elec in CHP plants':
    'share_efficiency_waste_for_elec_in_chp_plants',
    'XMax potential PE nonXelectric RESX':
    'xmax_potential_pe_nonxelectric_resx',
    'PES tot biogas for elec':
    'pes_tot_biogas_for_elec',
    'XPES tot biogas for heatXcomX':
    'xpes_tot_biogas_for_heatxcomx',
    'PE losses NRE elec generation':
    'pe_losses_nre_elec_generation',
    'Elec gen related losses EJ':
    'elec_gen_related_losses_ej',
    'PE losses biogas for elec':
    'pe_losses_biogas_for_elec',
    'PE losses waste for elec':
    'pe_losses_waste_for_elec',
    'max PE potential tot RES heat EJ':
    'max_pe_potential_tot_res_heat_ej',
    'share PES biogas for elec':
    'share_pes_biogas_for_elec',
    'P hydro growth':
    'p_hydro_growth',
    'FE real tot generation RES elec TWh':
    'fe_real_tot_generation_res_elec_twh',
    'P solar PV growth':
    'p_solar_pv_growth',
    'remaining potential constraint on new RES heat capacity':
    'remaining_potential_constraint_on_new_res_heat_capacity',
    'Total FE Elec generation TWh':
    'total_fe_elec_generation_twh',
    'FE tot generation all RES elec TWh':
    'fe_tot_generation_all_res_elec_twh',
    'P oceanic growth':
    'p_oceanic_growth',
    'PE Elec generation from RES EJ':
    'pe_elec_generation_from_res_ej',
    'P RES elec growth':
    'p_res_elec_growth',
    'share RES electricity generation':
    'share_res_electricity_generation',
    'XP solid bioEXelec growthX':
    'xp_solid_bioexelec_growthx',
    'P CSP growth':
    'p_csp_growth',
    'remaining potential tot RES elec':
    'remaining_potential_tot_res_elec',
    'P wind offshore growth':
    'p_wind_offshore_growth',
    'P wind onshore growth':
    'p_wind_onshore_growth',
    'XElec generation baseXload from RES TWhX':
    'xelec_generation_basexload_from_res_twhx',
    'FES elec from RES with priority TWh':
    'fes_elec_from_res_with_priority_twh',
    'share Renewables Elec':
    'share_renewables_elec',
    'abundance RES elec':
    'abundance_res_elec',
    'FES elec from BioW':
    'fes_elec_from_biow',
    'P geot growth':
    'p_geot_growth',
    'max potential tot RES elec TWh':
    'max_potential_tot_res_elec_twh',
    'P solid bioE for heat':
    'p_solid_bioe_for_heat',
    'PE real generation RES elec':
    'pe_real_generation_res_elec',
    'share Renewables Elec check':
    'share_renewables_elec_check',
    'share renewables Heat check':
    'share_renewables_heat_check',
    'Efficiency RES heat':
    'efficiency_res_heat',
    'XFED heatXcom by NRE CHP plants EJX':
    'xfed_heatxcom_by_nre_chp_plants_ejx',
    'XConstant final energy intensity sectorsXhouseholds TDXX':
    'xconstant_final_energy_intensity_sectorsxhouseholds_tdxx',
    'Xshare oil dem for HeatXcomX':
    'xshare_oil_dem_for_heatxcomx',
    'FE Elec generation from total oil TWh':
    'fe_elec_generation_from_total_oil_twh',
    'Total FE Heat generation EJ':
    'total_fe_heat_generation_ej',
    'XPE supply RES nonXElec EJX':
    'xpe_supply_res_nonxelec_ejx',
    'share coal dem for Elec':
    'share_coal_dem_for_elec',
    'PE losses oil for Elec EJ':
    'pe_losses_oil_for_elec_ej',
    'share oil dem for Elec':
    'share_oil_dem_for_elec',
    'Xshare coal dem for HeatXcomX':
    'xshare_coal_dem_for_heatxcomx',
    'FE Elec generation from conv gas TWh':
    'fe_elec_generation_from_conv_gas_twh',
    'share RES heat generation':
    'share_res_heat_generation',
    'share gas for Transp':
    'share_gas_for_transp',
    'Xshare natX gas dem for ElecX':
    'xshare_natx_gas_dem_for_elecx',
    'PE losses conv gas for Elec EJ':
    'pe_losses_conv_gas_for_elec_ej',
    'PE losses coal for Elec EJ':
    'pe_losses_coal_for_elec_ej',
    'FE Elec generation from coal TWh':
    'fe_elec_generation_from_coal_twh',
    'Xshare natX gas dem for HeatXcomX':
    'xshare_natx_gas_dem_for_heatxcomx',
    'Year scarcity Heat':
    'year_scarcity_heat',
    'XFES heatXcom from biogas in CHP plantsX':
    'xfes_heatxcom_from_biogas_in_chp_plantsx',
    'efficiency biogas for heat plants':
    'efficiency_biogas_for_heat_plants',
    'FES elec from biogas in CHP plants':
    'fes_elec_from_biogas_in_chp_plants',
    'FES elec from biogas EJ':
    'fes_elec_from_biogas_ej',
    'XPES biogas for heatXcom plantsX':
    'xpes_biogas_for_heatxcom_plantsx',
    'FES elec from biogas in elec plants':
    'fes_elec_from_biogas_in_elec_plants',
    'efficiency biogas for elec plants':
    'efficiency_biogas_for_elec_plants',
    'XFES biogas for heatXcom plantsX':
    'xfes_biogas_for_heatxcom_plantsx',
    'PES biogas for elec plants':
    'pes_biogas_for_elec_plants',
    'share PES biogas TFC':
    'share_pes_biogas_tfc',
    'Potential PES biogas for TFC':
    'potential_pes_biogas_for_tfc',
    'FES elec from biogas TWh':
    'fes_elec_from_biogas_twh',
    'PES biogas for CHP':
    'pes_biogas_for_chp',
    'Xshare PES biogas for heatXcom plantsX':
    'xshare_pes_biogas_for_heatxcom_plantsx',
    'efficiency biogas for elec CHP plants':
    'efficiency_biogas_for_elec_chp_plants',
    'XFES heatXcom from biogas EJX':
    'xfes_heatxcom_from_biogas_ejx',
    'efficiency biogas for heat CHP plants':
    'efficiency_biogas_for_heat_chp_plants',
    'share PES biogas for elec plants':
    'share_pes_biogas_for_elec_plants',
    'share PES biogas for CHP':
    'share_pes_biogas_for_chp',
    'demand Elec plants fossil fuels TWh':
    'demand_elec_plants_fossil_fuels_twh',
    'adapt growth biogas':
    'adapt_growth_biogas',
    'Total extraction NRE EJ':
    'total_extraction_nre_ej',
    'new PES biogas':
    'new_pes_biogas',
    'Energy cost pressure':
    'energy_cost_pressure',
    'Pressure to improve energy intensity efficiency':
    'pressure_to_improve_energy_intensity_efficiency',
    'abundance of energy':
    'abundance_of_energy',
    'Energy market pressure':
    'energy_market_pressure',
    'relative abundance of energy':
    'relative_abundance_of_energy',
    'Pressure to change energy technology':
    'pressure_to_change_energy_technology',
    'remaining recycling rate improv for alt techn per mineral':
    'remaining_recycling_rate_improv_for_alt_techn_per_mineral',
    'remaining recycling rate improv Rest per mineral':
    'remaining_recycling_rate_improv_rest_per_mineral',
    'XCED OXM over lifetime per material RES elec varX':
    'xced_oxm_over_lifetime_per_material_res_elec_varx',
    'Energy cons per unit of material cons for RES elec':
    'energy_cons_per_unit_of_material_cons_for_res_elec',
    'CED new cap per material RES elec var':
    'ced_new_cap_per_material_res_elec_var',
    'Energy required for material consumption for new RES elec':
    'energy_required_for_material_consumption_for_new_res_elec',
    'XEnergy required for material consumption for OXM RES elecX':
    'xenergy_required_for_material_consumption_for_oxm_res_elecx',
    'share other cumulative demand to extract vs resources materials':
    'share_other_cumulative_demand_to_extract_vs_resources_materials',
    'Materials to extract Rest from 2015 Mt':
    'materials_to_extract_rest_from_2015_mt',
    'Total recycled materials for RES elec Mt':
    'total_recycled_materials_for_res_elec_mt',
    'cum materials to extract Rest':
    'cum_materials_to_extract_rest',
    'cum materials to extract Rest from 2015':
    'cum_materials_to_extract_rest_from_2015',
    'share other cumulative demand to extract vs reserves materials':
    'share_other_cumulative_demand_to_extract_vs_reserves_materials',
    'Xmaterials availability Xannual extractionXX':
    'xmaterials_availability_xannual_extractionxx',
    'Mt per tonne':
    'mt_per_tonne',
    'initial cumulated material requirements for Rest 1995':
    'initial_cumulated_material_requirements_for_rest_1995',
    'P recycling minerals Rest':
    'p_recycling_minerals_rest',
    'cum materials to extract for RES elec from 2015':
    'cum_materials_to_extract_for_res_elec_from_2015',
    'Total materials to extract for RES elec Mt':
    'total_materials_to_extract_for_res_elec_mt',
    'cum materials to extract for RES elec':
    'cum_materials_to_extract_for_res_elec',
    'Total materials to extract for RES elec from 2015 Mt':
    'total_materials_to_extract_for_res_elec_from_2015_mt',
    'Max recycling rates minerals':
    'max_recycling_rates_minerals',
    'P recycling minerals alt techn':
    'p_recycling_minerals_alt_techn',
    'current recycling rates minerals':
    'current_recycling_rates_minerals',
    'Historic improvement recycling rates minerals':
    'historic_improvement_recycling_rates_minerals',
    'min energy intensity vs intial':
    'min_energy_intensity_vs_intial',
    'Global energy intensity by sector':
    'global_energy_intensity_by_sector',
    'historical mean rate energy intensity':
    'historical_mean_rate_energy_intensity',
    'XInitial energy cons per unit of material cons XrecycledXX':
    'xinitial_energy_cons_per_unit_of_material_cons_xrecycledxx',
    'Xmaterials required for OXM RES elec MtX':
    'xmaterials_required_for_oxm_res_elec_mtx',
    'Initial energy intensity by fuel and sector 1995':
    'initial_energy_intensity_by_fuel_and_sector_1995',
    'share tot cum dem vs reserves materials':
    'share_tot_cum_dem_vs_reserves_materials',
    'share tot cum dem vs resources materials':
    'share_tot_cum_dem_vs_resources_materials',
    'maximum annual extraction materials':
    'maximum_annual_extraction_materials',
    'Xmaterials availability XresourcesXX':
    'xmaterials_availability_xresourcesxx',
    'Xmaterials availability XreservesXX':
    'xmaterials_availability_xreservesxx',
    'TPES EJ':
    'tpes_ej',
    'XFES heatXcom from waste EJX':
    'xfes_heatxcom_from_waste_ejx',
    'XFES waste for heatXcom plantsX':
    'xfes_waste_for_heatxcom_plantsx',
    'FES elec from waste in CHP plants':
    'fes_elec_from_waste_in_chp_plants',
    'FES elec from waste in elec plants':
    'fes_elec_from_waste_in_elec_plants',
    'XFES heatXcom from waste in CHP plantsX':
    'xfes_heatxcom_from_waste_in_chp_plantsx',
    'XPES waste for heatXcom plantsX':
    'xpes_waste_for_heatxcom_plantsx',
    'Xshare PES waste for heatXcom plantsX':
    'xshare_pes_waste_for_heatxcom_plantsx',
    'share PES waste TFC':
    'share_pes_waste_tfc',
    'share PES waste for CHP':
    'share_pes_waste_for_chp',
    'share PES waste for elec plants':
    'share_pes_waste_for_elec_plants',
    'new waste supply EJ':
    'new_waste_supply_ej',
    'waste change':
    'waste_change',
    'FES elec from waste EJ':
    'fes_elec_from_waste_ej',
    'FES elec from waste TWh':
    'fes_elec_from_waste_twh',
    'efficiency waste for elec plants':
    'efficiency_waste_for_elec_plants',
    'PES waste EJ':
    'pes_waste_ej',
    'PES waste for CHP plants':
    'pes_waste_for_chp_plants',
    'efficiency waste for elec CHP plants':
    'efficiency_waste_for_elec_chp_plants',
    'initial PES waste':
    'initial_pes_waste',
    'PES waste for TFC':
    'pes_waste_for_tfc',
    'efficiency waste for heat CHP plants':
    'efficiency_waste_for_heat_chp_plants',
    'adapt growth waste':
    'adapt_growth_waste',
    'P waste change':
    'p_waste_change',
    'efficiency waste for heat plants':
    'efficiency_waste_for_heat_plants',
    'max waste':
    'max_waste',
    'XPES solids bioE X waste EJX':
    'xpes_solids_bioe_x_waste_ejx',
    'Past waste growth':
    'past_waste_growth',
    'Historic PES waste EJ':
    'historic_pes_waste_ej',
    'CH4 emissions oil':
    'ch4_emissions_oil',
    'gCH4 per MJ coal':
    'gch4_per_mj_coal',
    'gCH4 per MJ oil':
    'gch4_per_mj_oil',
    'FE Elec generation from fossil fuels TWh':
    'fe_elec_generation_from_fossil_fuels_twh',
    'PE demand uranium EJ':
    'pe_demand_uranium_ej',
    'FES Elec fossil fuel CHP plants TWh':
    'fes_elec_fossil_fuel_chp_plants_twh',
    'XFES heatXcom fossil fuels CHP plants EJX':
    'xfes_heatxcom_fossil_fuels_chp_plants_ejx',
    'potential generation nuclear elec TWh':
    'potential_generation_nuclear_elec_twh',
    'threshold remaining potential new capacity':
    'threshold_remaining_potential_new_capacity',
    'installed capacity RES elec TW':
    'installed_capacity_res_elec_tw',
    'Xtotal time planXconstr RES elecX':
    'xtotal_time_planxconstr_res_elecx',
    'Historic new required capacity RES elec':
    'historic_new_required_capacity_res_elec',
    'new required capacity RES elec':
    'new_required_capacity_res_elec',
    'table hist capacity RES elec SUBhydro':
    'table_hist_capacity_res_elec_subhydro',
    'table hist capacity RES elec SUBXgeotXelecX':
    'table_hist_capacity_res_elec_subxgeotxelecx',
    'table hist capacity RES elec SUBXsolidbioEXelecX':
    'table_hist_capacity_res_elec_subxsolidbioexelecx',
    'table hist capacity RES elec SUBoceanic':
    'table_hist_capacity_res_elec_suboceanic',
    'table hist capacity RES elec SUBwindonshore':
    'table_hist_capacity_res_elec_subwindonshore',
    'table hist capacity RES elec SUBwindoffshore':
    'table_hist_capacity_res_elec_subwindoffshore',
    'table hist capacity RES elec SUBsolarPV':
    'table_hist_capacity_res_elec_subsolarpv',
    'table hist capacity RES elec SUBCSP':
    'table_hist_capacity_res_elec_subcsp',
    'initial capacity in construction RES elec':
    'initial_capacity_in_construction_res_elec',
    'remaining potential constraint on new RES elec capacity':
    'remaining_potential_constraint_on_new_res_elec_capacity',
    'required capacity RES elec TW':
    'required_capacity_res_elec_tw',
    'RES elec planned capacity TW':
    'res_elec_planned_capacity_tw',
    'XInitial energy cons per unit of material cons XrecycledX X dataX':
    'xinitial_energy_cons_per_unit_of_material_cons_xrecycledx_x_datax',
    'Cp PHS':
    'cp_phs',
    'max potential PHS TWe':
    'max_potential_phs_twe',
    'max capacity potential PHS':
    'max_capacity_potential_phs',
    'Xmaterials per new RES elec capacity installed X HVDCsX':
    'xmaterials_per_new_res_elec_capacity_installed_x_hvdcsx',
    'Xmaterials per new RES elec capacity installed X material overgrid high powerX':
    'xmaterials_per_new_res_elec_capacity_installed_x_material_overgrid_high_powerx',
    'real growth GTL':
    'real_growth_gtl',
    'scarcity conv gas delayed 1yr':
    'scarcity_conv_gas_delayed_1yr',
    'scarcity conv oil delayed 1yr':
    'scarcity_conv_oil_delayed_1yr',
    'XPES natX gasX':
    'xpes_natx_gasx',
    'FE Elec generation from unconv gas TWh':
    'fe_elec_generation_from_unconv_gas_twh',
    'exponent availability conv oil':
    'exponent_availability_conv_oil',
    'PE losses uncon gas for Elec EJ':
    'pe_losses_uncon_gas_for_elec_ej',
    'Xabundance total natX gasX':
    'xabundance_total_natx_gasx',
    'exponent availability conv gas':
    'exponent_availability_conv_gas',
    'scarcity conv gas':
    'scarcity_conv_gas',
    'XYear scarcity total natX gasX':
    'xyear_scarcity_total_natx_gasx',
    'share unconv gas vs tot agg in 2050':
    'share_unconv_gas_vs_tot_agg_in_2050',
    'share unconv gas vs tot agg in 2050 Laherrere2010':
    'share_unconv_gas_vs_tot_agg_in_2050_laherrere2010',
    'share unconv gas vs tot agg in 2050 Mohr12 BG':
    'share_unconv_gas_vs_tot_agg_in_2050_mohr12_bg',
    'share unconv gas vs tot agg in 2050 User defined':
    'share_unconv_gas_vs_tot_agg_in_2050_user_defined',
    'cumulated tot agg gas extraction to 1995':
    'cumulated_tot_agg_gas_extraction_to_1995',
    'share conv gas vs tot agg':
    'share_conv_gas_vs_tot_agg',
    'share RURR tot agg gas to leave underground':
    'share_rurr_tot_agg_gas_to_leave_underground',
    'share unconv gas vs tot agg':
    'share_unconv_gas_vs_tot_agg',
    'RURR tot agg gas':
    'rurr_tot_agg_gas',
    'Flow tot agg gas left in ground':
    'flow_tot_agg_gas_left_in_ground',
    'Tot RURR tot agg gas':
    'tot_rurr_tot_agg_gas',
    'URR total agg gas unlimited':
    'urr_total_agg_gas_unlimited',
    'Start policy leave in ground tot agg gas':
    'start_policy_leave_in_ground_tot_agg_gas',
    'Xextraction unconv gas X tot aggX':
    'xextraction_unconv_gas_x_tot_aggx',
    'Total agg gas left in ground':
    'total_agg_gas_left_in_ground',
    'evolution share unconv gas vs tot agg':
    'evolution_share_unconv_gas_vs_tot_agg',
    'real extraction conv gas EJ':
    'real_extraction_conv_gas_ej',
    'URR tot agg gas':
    'urr_tot_agg_gas',
    'cumulated tot agg gas extraction':
    'cumulated_tot_agg_gas_extraction',
    'Xextraction conv gas X tot aggX':
    'xextraction_conv_gas_x_tot_aggx',
    'real extraction unconv gas EJ':
    'real_extraction_unconv_gas_ej',
    'max extraction tot agg gas EJ':
    'max_extraction_tot_agg_gas_ej',
    'extraction tot agg gas EJ':
    'extraction_tot_agg_gas_ej',
    'Xreal extraction conv oil MbXdX':
    'xreal_extraction_conv_oil_mbxdx',
    'share unconv oil vs tot agg':
    'share_unconv_oil_vs_tot_agg',
    'evolution share unconv oil vs tot agg':
    'evolution_share_unconv_oil_vs_tot_agg',
    'share unconv oil vs tot agg in 2050':
    'share_unconv_oil_vs_tot_agg_in_2050',
    'share unconv oil vs tot agg in 2050 Laherrere2006':
    'share_unconv_oil_vs_tot_agg_in_2050_laherrere2006',
    'share unconv oil vs tot agg in 2050 User defined':
    'share_unconv_oil_vs_tot_agg_in_2050_user_defined',
    'Xextraction conv oil X tot aggX':
    'xextraction_conv_oil_x_tot_aggx',
    'Xextraction unconv oil X tot aggX':
    'xextraction_unconv_oil_x_tot_aggx',
    'share conv oil vs tot agg':
    'share_conv_oil_vs_tot_agg',
    'RURR tot agg oil':
    'rurr_tot_agg_oil',
    'PES oil EJ':
    'pes_oil_ej',
    'check hist crude oil':
    'check_hist_crude_oil',
    'extraction tot agg oil EJ':
    'extraction_tot_agg_oil_ej',
    'real extraction conv oil EJ':
    'real_extraction_conv_oil_ej',
    'max extraction tot agg oil EJ':
    'max_extraction_tot_agg_oil_ej',
    'gCH4 per MJ conv gas':
    'gch4_per_mj_conv_gas',
    'Total agg oil left in ground':
    'total_agg_oil_left_in_ground',
    'cumulated tot agg oil extraction':
    'cumulated_tot_agg_oil_extraction',
    'cumulated tot agg extraction to 1995':
    'cumulated_tot_agg_extraction_to_1995',
    'URR tot agg oil':
    'urr_tot_agg_oil',
    'URR tot agg oil unlimited':
    'urr_tot_agg_oil_unlimited',
    'Tot RURR tot agg oil':
    'tot_rurr_tot_agg_oil',
    'Start policy leave in ground tot agg oil':
    'start_policy_leave_in_ground_tot_agg_oil',
    'Flow tot agg oil left in ground':
    'flow_tot_agg_oil_left_in_ground',
    'g per Mt':
    'g_per_mt',
    'share RURR tot agg oil to leave underground':
    'share_rurr_tot_agg_oil_to_leave_underground',
    'gCH4 per MJ unconv gas':
    'gch4_per_mj_unconv_gas',
    'Tot RURR unconv gas':
    'tot_rurr_unconv_gas',
    'Start policy leave in ground coal':
    'start_policy_leave_in_ground_coal',
    'Total unconv gas left in ground':
    'total_unconv_gas_left_in_ground',
    'Flow coal left in ground':
    'flow_coal_left_in_ground',
    'Flow conv gas left in ground':
    'flow_conv_gas_left_in_ground',
    'Flow conv oil left in ground':
    'flow_conv_oil_left_in_ground',
    'Flow unconv oil left in ground':
    'flow_unconv_oil_left_in_ground',
    'share RURR unconv gas to leave underground':
    'share_rurr_unconv_gas_to_leave_underground',
    'share RURR unconv oil to leave underground':
    'share_rurr_unconv_oil_to_leave_underground',
    'extraction unconv gas EJ':
    'extraction_unconv_gas_ej',
    'Tot RURR coal':
    'tot_rurr_coal',
    'Tot RURR conv gas':
    'tot_rurr_conv_gas',
    'Tot RURR conv oil':
    'tot_rurr_conv_oil',
    'share RURR conv oil to leave underground':
    'share_rurr_conv_oil_to_leave_underground',
    'Tot RURR unconv oil':
    'tot_rurr_unconv_oil',
    'Total unconv oil left in ground':
    'total_unconv_oil_left_in_ground',
    'Total coal left in ground':
    'total_coal_left_in_ground',
    'Total conv gas left in ground':
    'total_conv_gas_left_in_ground',
    'Total conv oil left in ground':
    'total_conv_oil_left_in_ground',
    'extraction conv oil EJ':
    'extraction_conv_oil_ej',
    'extraction coal EJ':
    'extraction_coal_ej',
    'RURR coal':
    'rurr_coal',
    'max extraction conv oil EJ':
    'max_extraction_conv_oil_ej',
    'Start policy leave in ground unconv gas':
    'start_policy_leave_in_ground_unconv_gas',
    'Flow unconv gas left in ground':
    'flow_unconv_gas_left_in_ground',
    'Start policy leave in ground unconv oil':
    'start_policy_leave_in_ground_unconv_oil',
    'Start policy leave in ground conv oil':
    'start_policy_leave_in_ground_conv_oil',
    'share RURR coal to leave underground':
    'share_rurr_coal_to_leave_underground',
    'XPES oil MbXdX':
    'xpes_oil_mbxdx',
    'current mineral reserves Mt':
    'current_mineral_reserves_mt',
    'Losses in charcoal plants EJ':
    'losses_in_charcoal_plants_ej',
    'check hist TPES':
    'check_hist_tpes',
    'share RURR conv gas to leave underground':
    'share_rurr_conv_gas_to_leave_underground',
    'Start policy leave in ground conv gas':
    'start_policy_leave_in_ground_conv_gas',
    'extraction conv gas EJ':
    'extraction_conv_gas_ej',
    'check gases':
    'check_gases',
    'abundance gases':
    'abundance_gases',
    'Demand conv gas':
    'demand_conv_gas',
    'XPED natX gas without GTLX':
    'xped_natx_gas_without_gtlx',
    'abundance solids':
    'abundance_solids',
    'GDP':
    'gdp',
    'demand Elec gas and coal TWh':
    'demand_elec_gas_and_coal_twh',
    'FE demand coal Elec plants TWh':
    'fe_demand_coal_elec_plants_twh',
    'FE demand gas Elec plants TWh':
    'fe_demand_gas_elec_plants_twh',
    'XFED Heat gasXcoal EJX':
    'xfed_heat_gasxcoal_ejx',
    'PE demand oil Elec plants EJ':
    'pe_demand_oil_elec_plants_ej',
    'FE demand oil Elec plants TWh':
    'fe_demand_oil_elec_plants_twh',
    'share coal for Elec':
    'share_coal_for_elec',
    'share liquids Elec':
    'share_liquids_elec',
    'current mineral resources Mt':
    'current_mineral_resources_mt',
    'share CHP plants oil':
    'share_chp_plants_oil',
    'historic share CHP plants gas':
    'historic_share_chp_plants_gas',
    'share CHP plants coal':
    'share_chp_plants_coal',
    'Historic share liquids for heat plants':
    'historic_share_liquids_for_heat_plants',
    'Xshare coalXcoalXgasX for heat plantsX':
    'xshare_coalxcoalxgasx_for_heat_plantsx',
    'historic share CHP plants oil':
    'historic_share_chp_plants_oil',
    'output elec over lifetime RES elec for allocation2':
    'output_elec_over_lifetime_res_elec_for_allocation2',
    'real generation RES elec TWh':
    'real_generation_res_elec_twh',
    'demand storage capacity':
    'demand_storage_capacity',
    'Xshare capacity storageXRES elec varX':
    'xshare_capacity_storagexres_elec_varx',
    'XstaticXdynamic quality of electricityXX':
    'xstaticxdynamic_quality_of_electricityxx',
    'XXstaticX EROIgrid totXeffective for allocation RES elecX':
    'xxstaticx_eroigrid_totxeffective_for_allocation_res_elecx',
    'Xratio EROIgrid vs EROI XstaticXX':
    'xratio_eroigrid_vs_eroi_xstaticxx',
    'Xratio EROI per techn vs EROItot XstaticXX':
    'xratio_eroi_per_techn_vs_eroitot_xstaticxx',
    'Total installed capacity RES elec var':
    'total_installed_capacity_res_elec_var',
    'share RES elec generation stored':
    'share_res_elec_generation_stored',
    'rt storage efficiency PHS':
    'rt_storage_efficiency_phs',
    'Cp baseload reduction':
    'cp_baseload_reduction',
    'Xratio X 1X':
    'xratio_x_1x',
    'CEDtot over lifetime RES elec':
    'cedtot_over_lifetime_res_elec',
    'real Cp RES elec':
    'real_cp_res_elec',
    'CEDtot over lifetime RES elec for allocation':
    'cedtot_over_lifetime_res_elec_for_allocation',
    'remaining potential RES elec after intermitt':
    'remaining_potential_res_elec_after_intermitt',
    'remaining potential RES elec switch':
    'remaining_potential_res_elec_switch',
    'output elec over lifetime RES elec for allocation':
    'output_elec_over_lifetime_res_elec_for_allocation',
    'potential RES elec after intermitt TWh':
    'potential_res_elec_after_intermitt_twh',
    'replacement rate RES elec':
    'replacement_rate_res_elec',
    'XXstaticX EROItot RES elecX':
    'xxstaticx_eroitot_res_elecx',
    'XXstaticX EROItotXeffective for allocation RES elecX':
    'xxstaticx_eroitotxeffective_for_allocation_res_elecx',
    'new capacity installed onshore wind TW':
    'new_capacity_installed_onshore_wind_tw',
    'XCED OXM over lifetime per water RES elec varX':
    'xced_oxm_over_lifetime_per_water_res_elec_varx',
    'XselfXelectricity consumption RES elecX':
    'xselfxelectricity_consumption_res_elecx',
    'share energy requirements for decom RES elec':
    'share_energy_requirements_for_decom_res_elec',
    'XCED OXM over lifetime RES elec varX':
    'xced_oxm_over_lifetime_res_elec_varx',
    'Grid correction factor RES elec':
    'grid_correction_factor_res_elec',
    'Xmaterials for OXM per capacity installed X CSPX':
    'xmaterials_for_oxm_per_capacity_installed_x_cspx',
    'XTotal water for OXM required by RES elecX':
    'xtotal_water_for_oxm_required_by_res_elecx',
    'XWater for OXM required for RES elecX':
    'xwater_for_oxm_required_for_res_elecx',
    'Xmaterials for OXM per capacity installed RES elecX':
    'xmaterials_for_oxm_per_capacity_installed_res_elecx',
    'XTotal energy requirements OXM for water consumption RES elecX':
    'xtotal_energy_requirements_oxm_for_water_consumption_res_elecx',
    'XEnergy requirements for OXM for water consumption RES elecX':
    'xenergy_requirements_for_oxm_for_water_consumption_res_elecx',
    'Xwater for OXM X RES elecX':
    'xwater_for_oxm_x_res_elecx',
    'Energy requirements per unit of water consumption':
    'energy_requirements_per_unit_of_water_consumption',
    'Xunlimited NREXX':
    'xunlimited_nrexx',
    'extraction uranium EJ':
    'extraction_uranium_ej',
    'URR uranium':
    'urr_uranium',
    'URR conv oil':
    'urr_conv_oil',
    'Historic biogas PES':
    'historic_biogas_pes',
    'growth biogas':
    'growth_biogas',
    'past biogas growth':
    'past_biogas_growth',
    'PES Biogas EJ':
    'pes_biogas_ej',
    'biogas evol':
    'biogas_evol',
    'max biogas EJ':
    'max_biogas_ej',
    'Historic TPES primary solids biofuels EJ':
    'historic_tpes_primary_solids_biofuels_ej',
    'Historic PES peat EJ':
    'historic_pes_peat_ej',
    'PES peat EJ':
    'pes_peat_ej',
    'FEI over lifetime RES elec dispatch':
    'fei_over_lifetime_res_elec_dispatch',
    'activate EROI allocation rule':
    'activate_eroi_allocation_rule',
    'EROI allocation rule per RES elec':
    'eroi_allocation_rule_per_res_elec',
    'XXdynamicX EROI RES elec varX':
    'xxdynamicx_eroi_res_elec_varx',
    'XEROIXini RES elec dispatchX':
    'xeroixini_res_elec_dispatchx',
    'share renewables fuel CHP IEA EJ':
    'share_renewables_fuel_chp_iea_ej',
    'share renewables Heat IEA EJ':
    'share_renewables_heat_iea_ej',
    'share coal Heat check':
    'share_coal_heat_check',
    'share nuclear Heat check':
    'share_nuclear_heat_check',
    'share nuclear Heat IEA EJ':
    'share_nuclear_heat_iea_ej',
    'share gases Heat IEA EJ':
    'share_gases_heat_iea_ej',
    'share coal Heat IEA EJ':
    'share_coal_heat_iea_ej',
    'share liquids Heat IEA EJ':
    'share_liquids_heat_iea_ej',
    'share fossil fuel CHP check':
    'share_fossil_fuel_chp_check',
    'share gases Heat check':
    'share_gases_heat_check',
    'share fossil fuel CHP IEA EJ':
    'share_fossil_fuel_chp_iea_ej',
    'share liquids Heat check':
    'share_liquids_heat_check',
    'Cp historic CSP':
    'cp_historic_csp',
    'RES elec capacity under construction TW':
    'res_elec_capacity_under_construction_tw',
    'share liquids Elec IEA':
    'share_liquids_elec_iea',
    'share Nuclear Elec':
    'share_nuclear_elec',
    'share Nuclear Elec check':
    'share_nuclear_elec_check',
    'share Nuclear Elec IEA':
    'share_nuclear_elec_iea',
    'share renewables Elec CHP IEA':
    'share_renewables_elec_chp_iea',
    'share Renewables Elec IEA':
    'share_renewables_elec_iea',
    'PE gas for CHP plants IEA EJ':
    'pe_gas_for_chp_plants_iea_ej',
    'check coal primary CHP plants':
    'check_coal_primary_chp_plants',
    'PE demand coal for Heat plants IEA EJ':
    'pe_demand_coal_for_heat_plants_iea_ej',
    'PE demand for liquids Heat plants IEA EJ':
    'pe_demand_for_liquids_heat_plants_iea_ej',
    'PE liquids for CHP plants IEA EJ':
    'pe_liquids_for_chp_plants_iea_ej',
    'check gas primary CHP plants':
    'check_gas_primary_chp_plants',
    'check primary liquids Elec plants':
    'check_primary_liquids_elec_plants',
    'PE demand liquids Elec plants IEA EJ':
    'pe_demand_liquids_elec_plants_iea_ej',
    'check primary liquids Heat plants':
    'check_primary_liquids_heat_plants',
    'share fossil fuel Elec CHP IEA':
    'share_fossil_fuel_elec_chp_iea',
    'PE coal for CHP plants IEA EJ':
    'pe_coal_for_chp_plants_iea_ej',
    'check primary gas Elec gas':
    'check_primary_gas_elec_gas',
    'share coal Elec':
    'share_coal_elec',
    'share coal Elec check':
    'share_coal_elec_check',
    'check liquids CHP plants':
    'check_liquids_chp_plants',
    'PE demand coal Elec plants IEA EJ':
    'pe_demand_coal_elec_plants_iea_ej',
    'check primary coal Elec plants':
    'check_primary_coal_elec_plants',
    'PE demand gases for Heat plants IEA EJ':
    'pe_demand_gases_for_heat_plants_iea_ej',
    'share gases Elec':
    'share_gases_elec',
    'share gases Elec check':
    'share_gases_elec_check',
    'PE demand gas Elec plants IEA EJ':
    'pe_demand_gas_elec_plants_iea_ej',
    'share liquids Elec check':
    'share_liquids_elec_check',
    'Xcheck primary liquids for ElecX Heat and CHP plantsX':
    'xcheck_primary_liquids_for_elecx_heat_and_chp_plantsx',
    'Xcheck primary gas for ElecX Heat and CHP plantsX':
    'xcheck_primary_gas_for_elecx_heat_and_chp_plantsx',
    'Xcheck primary coal for ElecX Heat and CHP plantsX':
    'xcheck_primary_coal_for_elecx_heat_and_chp_plantsx',
    'check primary coal for Heat plants':
    'check_primary_coal_for_heat_plants',
    'share gases Elec IEA':
    'share_gases_elec_iea',
    'share Coal Elec IEA':
    'share_coal_elec_iea',
    'share fossil fuel CHP Elec check':
    'share_fossil_fuel_chp_elec_check',
    'check primary gases Heat plants':
    'check_primary_gases_heat_plants',
    'share fossil fuel CHP Elec':
    'share_fossil_fuel_chp_elec',
    'real generation solar PV EJ':
    'real_generation_solar_pv_ej',
    'materials required for new CSP Mt':
    'materials_required_for_new_csp_mt',
    'materials required for new PV Mt':
    'materials_required_for_new_pv_mt',
    'materials required for new wind offshore Mt':
    'materials_required_for_new_wind_offshore_mt',
    'materials required for new wind onshore Mt':
    'materials_required_for_new_wind_onshore_mt',
    'Xmaterials required for OXM CSP MtX':
    'xmaterials_required_for_oxm_csp_mtx',
    'Xmaterials required for OXM PV MtX':
    'xmaterials_required_for_oxm_pv_mtx',
    'Xmaterials required for OXM wind offshore MtX':
    'xmaterials_required_for_oxm_wind_offshore_mtx',
    'Xmaterials required for OXM wind onshore MtX':
    'xmaterials_required_for_oxm_wind_onshore_mtx',
    'real generation RES elec EJ':
    'real_generation_res_elec_ej',
    'Xenergy ownXuse constant 830 Mtoe in 2013X':
    'xenergy_ownxuse_constant_830_mtoe_in_2013x',
    'Xpipeline transport constant 2X6 EJ in 2014X':
    'xpipeline_transport_constant_2x6_ej_in_2014x',
    'CEDtot solar PV':
    'cedtot_solar_pv',
    'Total materials required for new RES elec Mt':
    'total_materials_required_for_new_res_elec_mt',
    'XTotal materials required for OXM RES elec MtX':
    'xtotal_materials_required_for_oxm_res_elec_mtx',
    'XEROIX1X':
    'xeroix1x',
    'Electrical distribution losses TWh':
    'electrical_distribution_losses_twh',
    'XMax share transmXdistr elec lossesX':
    'xmax_share_transmxdistr_elec_lossesx',
    'XHeatXcom distribution lossesX':
    'xheatxcom_distribution_lossesx',
    'potential generation RES elec TWh':
    'potential_generation_res_elec_twh',
    'output elec over lifetime RES elec':
    'output_elec_over_lifetime_res_elec',
    'Xvariation share transmXdistr elec lossesX':
    'xvariation_share_transmxdistr_elec_lossesx',
    'Xremaining share transmXdistr elec lossesX':
    'xremaining_share_transmxdistr_elec_lossesx',
    'Xshare transmXdistr elec lossesX':
    'xshare_transmxdistr_elec_lossesx',
    'Total gen losses demand for Elec plants EJ':
    'total_gen_losses_demand_for_elec_plants_ej',
    'PE losses BioE for Elec EJ':
    'pe_losses_bioe_for_elec_ej',
    'Xwater for OXM X wind offshoreX':
    'xwater_for_oxm_x_wind_offshorex',
    'Total materials required for RES elec Mt':
    'total_materials_required_for_res_elec_mt',
    'share energy for material consumption for alt techn vs TFEC':
    'share_energy_for_material_consumption_for_alt_techn_vs_tfec',
    'Xwater for OXM X PVX':
    'xwater_for_oxm_x_pvx',
    'Xwater for OXM X wind onshoreX':
    'xwater_for_oxm_x_wind_onshorex',
    'Xmaterials for OXM per capacity installed X wind offshoreX':
    'xmaterials_for_oxm_per_capacity_installed_x_wind_offshorex',
    'Xmaterials for OXM per capacity installed X wind onshoreX':
    'xmaterials_for_oxm_per_capacity_installed_x_wind_onshorex',
    'Xmaterials for OXM per capacity installed X PVX':
    'xmaterials_for_oxm_per_capacity_installed_x_pvx',
    'Xwater for OXM X CSPX':
    'xwater_for_oxm_x_cspx',
    'Xmaterials per capacity installed X PV 0X':
    'xmaterials_per_capacity_installed_x_pv_0x',
    'cum materials requirements for RES elec':
    'cum_materials_requirements_for_res_elec',
    'MJ per EJ':
    'mj_per_ej',
    'XInitial energy cons per unit of material cons XvirginXX':
    'xinitial_energy_cons_per_unit_of_material_cons_xvirginxx',
    'Xmaterials per new capacity installed X CSPX':
    'xmaterials_per_new_capacity_installed_x_cspx',
    'Xmaterials per new capacity installed X wind offshoreX':
    'xmaterials_per_new_capacity_installed_x_wind_offshorex',
    'Xmaterials per new capacity installed X wind onshoreX':
    'xmaterials_per_new_capacity_installed_x_wind_onshorex',
    'initial cumulated material requirements for RES elec 1995':
    'initial_cumulated_material_requirements_for_res_elec_1995',
    'Xmaterials per new capacity installed X PVX':
    'xmaterials_per_new_capacity_installed_x_pvx',
    'kg per Mt':
    'kg_per_mt',
    'M per T':
    'm_per_t',
    'Elec generation variable from RES TWh':
    'elec_generation_variable_from_res_twh',
    'copper for solar thermal':
    'copper_for_solar_thermal',
    'Copper demand for solar thermal':
    'copper_demand_for_solar_thermal',
    'Total gen losses demand for Heat plants EJ':
    'total_gen_losses_demand_for_heat_plants_ej',
    'Total gen losses demand for CHP plants EJ':
    'total_gen_losses_demand_for_chp_plants_ej',
    'new required capacity nuclear':
    'new_required_capacity_nuclear',
    'Xreplacement RES for heatXcomX':
    'xreplacement_res_for_heatxcomx',
    'Losses solar for heat':
    'losses_solar_for_heat',
    'Xpotential FES RES for heatXcom EJX':
    'xpotential_fes_res_for_heatxcom_ejx',
    'Xpotential FES RES for heatXcom TWhX':
    'xpotential_fes_res_for_heatxcom_twhx',
    'Cp RES for heat':
    'cp_res_for_heat',
    'Efficiency geothermal for heat':
    'efficiency_geothermal_for_heat',
    'Efficiency solar panels for heat':
    'efficiency_solar_panels_for_heat',
    'Max FE potential RES for heat':
    'max_fe_potential_res_for_heat',
    'P geothermal for heat':
    'p_geothermal_for_heat',
    'Xpast RES growth for heatXcomX':
    'xpast_res_growth_for_heatxcomx',
    'Xinstalled capacity RES heatXcom TWX':
    'xinstalled_capacity_res_heatxcom_twx',
    'Xinitial value RES for heatXcomX':
    'xinitial_value_res_for_heatxcomx',
    'P solar for heat':
    'p_solar_for_heat',
    'life time RES for heat':
    'life_time_res_for_heat',
    'Xwear RES capacity for heatXcom TWX':
    'xwear_res_capacity_for_heatxcom_twx',
    'Potential FE gen Elec fossil fuel CHP plants EJ':
    'potential_fe_gen_elec_fossil_fuel_chp_plants_ej',
    'efficiency Heat oil CHP plants':
    'efficiency_heat_oil_chp_plants',
    'FED heat liquids CHP plants EJ':
    'fed_heat_liquids_chp_plants_ej',
    'Potential FE gen Elec liquids CHP plants EJ':
    'potential_fe_gen_elec_liquids_chp_plants_ej',
    'efficiency Elec coal CHP plants':
    'efficiency_elec_coal_chp_plants',
    'efficiency Elec gas CHP plants':
    'efficiency_elec_gas_chp_plants',
    'efficiency Elec oil CHP plants':
    'efficiency_elec_oil_chp_plants',
    'Potential FE gen Elec coal CHP plants EJ':
    'potential_fe_gen_elec_coal_chp_plants_ej',
    'Potential FE gen Elec gas CHP plants EJ':
    'potential_fe_gen_elec_gas_chp_plants_ej',
    'FED heat gas CHP plants EJ':
    'fed_heat_gas_chp_plants_ej',
    'efficiency Heat gas CHP plants':
    'efficiency_heat_gas_chp_plants',
    'PED oil for CHP plants EJ':
    'ped_oil_for_chp_plants_ej',
    'XShare heatXcom CHP plants NRE vs NRE tot heatXcom generationX':
    'xshare_heatxcom_chp_plants_nre_vs_nre_tot_heatxcom_generationx',
    'efficiency Heat coal CHP plants':
    'efficiency_heat_coal_chp_plants',
    'FED heat coal CHP plants EJ':
    'fed_heat_coal_chp_plants_ej',
    'PED gas for CHP plants EJ':
    'ped_gas_for_chp_plants_ej',
    'PED coal for CHP plants EJ':
    'ped_coal_for_chp_plants_ej',
    'remaining potential CSP':
    'remaining_potential_csp',
    'PE CSP for Elec generation EJ':
    'pe_csp_for_elec_generation_ej',
    'Invest RES for Elec':
    'invest_res_for_elec',
    'power density CSP':
    'power_density_csp',
    'max potential RES elec TWe':
    'max_potential_res_elec_twe',
    'surface CSP Mha':
    'surface_csp_mha',
    'FE Elec generation from CSP TWh':
    'fe_elec_generation_from_csp_twh',
    'invest CSP Tdolar':
    'invest_csp_tdolar',
    'Cp limit nuclear':
    'cp_limit_nuclear',
    'invest nuclear Tdolar':
    'invest_nuclear_tdolar',
    'Cp nuclear':
    'cp_nuclear',
    'installed capacity nuclear TW':
    'installed_capacity_nuclear_tw',
    'Xnuclear capacity phaseXoutX':
    'xnuclear_capacity_phasexoutx',
    'min Cp nuclear':
    'min_cp_nuclear',
    'replacement nuclear capacity':
    'replacement_nuclear_capacity',
    'min Cp baseload RES':
    'min_cp_baseload_res',
    'abundance uranium':
    'abundance_uranium',
    'nuclear overcapacity':
    'nuclear_overcapacity',
    'XP nuclear scen3X4X':
    'xp_nuclear_scen3x4x',
    'Xstart year nuclear growth scen3X4X':
    'xstart_year_nuclear_growth_scen3x4x',
    'P nuclear elec gen':
    'p_nuclear_elec_gen',
    'required capacity nuclear TW':
    'required_capacity_nuclear_tw',
    'time planification nuclear':
    'time_planification_nuclear',
    'time construction nuclear':
    'time_construction_nuclear',
    'initial capacity installed nuclear':
    'initial_capacity_installed_nuclear',
    'Nuclear capacity under construction':
    'nuclear_capacity_under_construction',
    'new nuclear capacity under planning':
    'new_nuclear_capacity_under_planning',
    'selection of nuclear scenario':
    'selection_of_nuclear_scenario',
    'initial gen nuclear':
    'initial_gen_nuclear',
    'initial capacity in construction nuclear':
    'initial_capacity_in_construction_nuclear',
    'initial required capacity nuclear':
    'initial_required_capacity_nuclear',
    'Planned nuclear capacity TW':
    'planned_nuclear_capacity_tw',
    'wear nuclear':
    'wear_nuclear',
    'effects shortage uranium':
    'effects_shortage_uranium',
    'TPES intensity EJ Tdolar':
    'tpes_intensity_ej_tdolar',
    'real PED intensity of Electricity':
    'real_ped_intensity_of_electricity',
    'Annual GDP growth rate':
    'annual_gdp_growth_rate',
    'Historic energy intensity':
    'historic_energy_intensity',
    'GDP delayed 1yr':
    'gdp_delayed_1yr',
    'share tot monet invest Elec RES vs GDP':
    'share_tot_monet_invest_elec_res_vs_gdp',
    'real PED intensity of Transport':
    'real_ped_intensity_of_transport',
    'Climate Feedback Param':
    'climate_feedback_param',
    'Feedback Cooling':
    'feedback_cooling',
    'GDPpc':
    'gdppc',
    'Xunlimited coalXX':
    'xunlimited_coalxx',
    'Real TFEC':
    'real_tfec',
    'Xunlimited uraniumXX':
    'xunlimited_uraniumxx',
    'URR coal unlimited':
    'urr_coal_unlimited',
    'URR uranium unlimited':
    'urr_uranium_unlimited',
    'URR conv gas unlimited':
    'urr_conv_gas_unlimited',
    'Xunlimited gasXX':
    'xunlimited_gasxx',
    'Xunlimited oilXX':
    'xunlimited_oilxx',
    'ELF concentrations logistic':
    'elf_concentrations_logistic',
    'URR conv oil unlimited':
    'urr_conv_oil_unlimited',
    'b logistic':
    'b_logistic',
    'a logistic':
    'a_logistic',
    'selection shape ELF':
    'selection_shape_elf',
    'Xactivate ELF by scenXX':
    'xactivate_elf_by_scenxx',
    'Method CC impacts':
    'method_cc_impacts',
    'XTotal demand liquids mbXdX':
    'xtotal_demand_liquids_mbxdx',
    'PED coal without CTL':
    'ped_coal_without_ctl',
    'share liquids for Transp':
    'share_liquids_for_transp',
    'PES for Transp EJ':
    'pes_for_transp_ej',
    'surface hydro Mha':
    'surface_hydro_mha',
    'remaining potential hydro':
    'remaining_potential_hydro',
    'Grid reinforcement costs Tdollar':
    'grid_reinforcement_costs_tdollar',
    'remaining potential onshore wind':
    'remaining_potential_onshore_wind',
    'remaining potential offshore wind':
    'remaining_potential_offshore_wind',
    'remaining potential BioE':
    'remaining_potential_bioe',
    'remaining potential oceanic':
    'remaining_potential_oceanic',
    'Xremaining potential geotXelecX':
    'xremaining_potential_geotxelecx',
    'Xremaining potential solarXelec PVX':
    'xremaining_potential_solarxelec_pvx',
    'XHistoric nonXenergy useX SUBliquids':
    'xhistoric_nonxenergy_usex_subliquids',
    'XHistoric nonXenergy useX SUBsolids':
    'xhistoric_nonxenergy_usex_subsolids',
    'XHistoric nonXenergy useX SUBgases':
    'xhistoric_nonxenergy_usex_subgases',
    'XHistoric nonXenergy useX SUBelectricity':
    'xhistoric_nonxenergy_usex_subelectricity',
    'XHistoric nonXenergy useX SUBheat':
    'xhistoric_nonxenergy_usex_subheat',
    'Xinitial nonXenergy useX':
    'xinitial_nonxenergy_usex',
    'Real total output':
    'real_total_output',
    'Real demand':
    'real_demand',
    'max potential RES elec TWh':
    'max_potential_res_elec_twh',
    'PE onshore wind for Elec generation EJ':
    'pe_onshore_wind_for_elec_generation_ej',
    'FE Elec generation from solar PV TWh':
    'fe_elec_generation_from_solar_pv_twh',
    'power density solar PV':
    'power_density_solar_pv',
    'PE hydro for Elec generation EJ':
    'pe_hydro_for_elec_generation_ej',
    'invest bioW Tdolar':
    'invest_biow_tdolar',
    'FE Elec generation from hydro TWh':
    'fe_elec_generation_from_hydro_twh',
    'Xinvest geotXelec TdolarX':
    'xinvest_geotxelec_tdolarx',
    'invest hydro Tdolar':
    'invest_hydro_tdolar',
    'FE Elec generation from onshore wind TWh':
    'fe_elec_generation_from_onshore_wind_twh',
    'invest oceanic Tdolar':
    'invest_oceanic_tdolar',
    'invest onshore wind Tdolar':
    'invest_onshore_wind_tdolar',
    'invest offshore wind Tdolar':
    'invest_offshore_wind_tdolar',
    'XFE Elec generation from geotXelec TWhX':
    'xfe_elec_generation_from_geotxelec_twhx',
    'XPE geotXelec for Elec generation EJX':
    'xpe_geotxelec_for_elec_generation_ejx',
    'PE solar PV for Elec generation EJ':
    'pe_solar_pv_for_elec_generation_ej',
    'FE Elec generation from bioE TWh':
    'fe_elec_generation_from_bioe_twh',
    'PE oceanic for Elec generation EJ':
    'pe_oceanic_for_elec_generation_ej',
    'PE bioE for Elec generation EJ':
    'pe_bioe_for_elec_generation_ej',
    'FE Elec generation from offshore wind TWh':
    'fe_elec_generation_from_offshore_wind_twh',
    'FE Elec generation from oceanic TWh':
    'fe_elec_generation_from_oceanic_twh',
    'PE offshore wind for Elec generation EJ':
    'pe_offshore_wind_for_elec_generation_ej',
    'invest solar Tdolar':
    'invest_solar_tdolar',
    'initial value land compet biofuels 2gen Mha':
    'initial_value_land_compet_biofuels_2gen_mha',
    'initial value land compet biofuels 2gen ktoe':
    'initial_value_land_compet_biofuels_2gen_ktoe',
    'abundance coal':
    'abundance_coal',
    'abundance liquids':
    'abundance_liquids',
    'check liquids':
    'check_liquids',
    'efficiency liquids for heat plants':
    'efficiency_liquids_for_heat_plants',
    'Xshare gasXXcoalXgasX for heat plantsX':
    'xshare_gasxxcoalxgasx_for_heat_plantsx',
    'Share heat distribution losses':
    'share_heat_distribution_losses',
    'efficiency coal for heat plants':
    'efficiency_coal_for_heat_plants',
    'efficiency gases for heat plants':
    'efficiency_gases_for_heat_plants',
    'potential tot RES elec after intermitt':
    'potential_tot_res_elec_after_intermitt',
    'invest RES elec Tdolar':
    'invest_res_elec_tdolar',
    'PE BioW for Elec generation Mtoe':
    'pe_biow_for_elec_generation_mtoe',
    'remaining potential tot RES elec after intermitt':
    'remaining_potential_tot_res_elec_after_intermitt',
    'invest cost RES elec SUBhydro':
    'invest_cost_res_elec_subhydro',
    'invest cost RES elec SUBXgeotXelecX':
    'invest_cost_res_elec_subxgeotxelecx',
    'invest cost RES elec SUBXsolidbioEXelecX':
    'invest_cost_res_elec_subxsolidbioexelecx',
    'invest cost RES elec SUBoceanic':
    'invest_cost_res_elec_suboceanic',
    'invest cost RES elec SUBwindonshore':
    'invest_cost_res_elec_subwindonshore',
    'invest cost RES elec SUBwindoffshore':
    'invest_cost_res_elec_subwindoffshore',
    'invest cost RES elec SUBsolarPV':
    'invest_cost_res_elec_subsolarpv',
    'invest cost RES elec SUBCSP':
    'invest_cost_res_elec_subcsp',
    'Xpower density RES elec TWeXMhaX':
    'xpower_density_res_elec_twexmhax',
    'Xpower density RES elec TWXMhaX':
    'xpower_density_res_elec_twxmhax',
    'Time 95pc TS potential RES elec':
    'time_95pc_ts_potential_res_elec',
    'surface RES elec':
    'surface_res_elec',
    'Xmax geotXelec TWeX':
    'xmax_geotxelec_twex',
    'Xmax PE geotXelec TWthX':
    'xmax_pe_geotxelec_twthx',
    'Efficiency conversion geot PE to Elec':
    'efficiency_conversion_geot_pe_to_elec',
    'Geot PE potential for heat EJ':
    'geot_pe_potential_for_heat_ej',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subairtransport',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBelectricity SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivatehouseholdswithemployedpersons',
    'Historic final energy intensity by sector and fuel SUBheat SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBheat SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBheat SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBheat SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBheat SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBheat SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBheat SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBheat SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBheat SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBheat SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBheat SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBheat SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBheat SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBheat SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBheat SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBheat SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBheat SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBheat SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction',
    'Historic final energy intensity by sector and fuel SUBheat SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBheat SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBheat SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBheat SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBheat SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBheat SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBheat SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subairtransport',
    'Historic final energy intensity by sector and fuel SUBheat SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBheat SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBheat SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBheat SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBheat SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBheat SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBheat SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation',
    'Historic final energy intensity by sector and fuel SUBheat SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBheat SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBheat SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivatehouseholdswithemployedpersons',
    'Historic final energy intensity by sector and fuel SUBliquids SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBliquids SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBliquids SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBliquids SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBliquids SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBliquids SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBliquids SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBliquids SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBliquids SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBliquids SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBliquids SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBliquids SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBliquids SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBliquids SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBliquids SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBliquids SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBliquids SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBliquids SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction',
    'Historic final energy intensity by sector and fuel SUBliquids SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBliquids SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBliquids SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBliquids SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBliquids SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBliquids SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBliquids SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subairtransport',
    'Historic final energy intensity by sector and fuel SUBliquids SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBliquids SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBliquids SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBliquids SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBliquids SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBliquids SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBliquids SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation',
    'Historic final energy intensity by sector and fuel SUBliquids SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBliquids SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBliquids SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivatehouseholdswithemployedpersons',
    'Historic final energy intensity by sector and fuel SUBgases SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBgases SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBgases SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBgases SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBgases SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBgases SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBgases SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBgases SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBgases SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBgases SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBgases SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBgases SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBgases SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBgases SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBgases SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBgases SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBgases SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBgases SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction',
    'Historic final energy intensity by sector and fuel SUBgases SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBgases SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBgases SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBgases SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBgases SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBgases SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBgases SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subairtransport',
    'Historic final energy intensity by sector and fuel SUBgases SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBgases SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBgases SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBgases SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBgases SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBgases SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBgases SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation',
    'Historic final energy intensity by sector and fuel SUBgases SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBgases SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBgases SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivatehouseholdswithemployedpersons',
    'Historic final energy intensity by sector and fuel SUBsolids SUBAgricultureHuntingForestryandFishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subagriculturehuntingforestryandfishing',
    'Historic final energy intensity by sector and fuel SUBsolids SUBMiningandQuarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subminingandquarrying',
    'Historic final energy intensity by sector and fuel SUBsolids SUBFoodBeveragesandTobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfoodbeveragesandtobacco',
    'Historic final energy intensity by sector and fuel SUBsolids SUBTextilesandTextileProducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextilesandtextileproducts',
    'Historic final energy intensity by sector and fuel SUBsolids SUBLeatherLeatherandFootwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleatherleatherandfootwear',
    'Historic final energy intensity by sector and fuel SUBsolids SUBWoodandProductsofWooodandCork':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwoodandproductsofwooodandcork',
    'Historic final energy intensity by sector and fuel SUBsolids SUBPulpPaperPrintingandPublishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulppaperprintingandpublishing',
    'Historic final energy intensity by sector and fuel SUBsolids SUBCokeRefinedPetroleumandNuclearFuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcokerefinedpetroleumandnuclearfuel',
    'Historic final energy intensity by sector and fuel SUBsolids SUBChemicalsandChemicalproducts':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicalsandchemicalproducts',
    'Historic final energy intensity by sector and fuel SUBsolids SUBRubberandPlastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubberandplastics',
    'Historic final energy intensity by sector and fuel SUBsolids SUBOtherNonMetalicMineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothernonmetalicmineral',
    'Historic final energy intensity by sector and fuel SUBsolids SUBBasicMetalsandFabricatedMetal':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasicmetalsandfabricatedmetal',
    'Historic final energy intensity by sector and fuel SUBsolids SUBMachineryNec':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinerynec',
    'Historic final energy intensity by sector and fuel SUBsolids SUBElectricalandOpticalEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricalandopticalequipment',
    'Historic final energy intensity by sector and fuel SUBsolids SUBTransportEquipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransportequipment',
    'Historic final energy intensity by sector and fuel SUBsolids SUBManufacturingNecRecycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturingnecrecycling',
    'Historic final energy intensity by sector and fuel SUBsolids SUBElectricityGasandWaterSupply':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricitygasandwatersupply',
    'Historic final energy intensity by sector and fuel SUBsolids SUBConstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction',
    'Historic final energy intensity by sector and fuel SUBsolids SUBSaleMaintenanceandRepairofMotorVehiclesandaMotorcyclesRetailSaleoffuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsalemaintenanceandrepairofmotorvehiclesandamotorcyclesretailsaleoffuel',
    'Historic final energy intensity by sector and fuel SUBsolids SUBWholesaleTradeandCommissionsTradeExceptofMotorvehiclesandMotorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesaletradeandcommissionstradeexceptofmotorvehiclesandmotorcycles',
    'Historic final energy intensity by sector and fuel SUBsolids SUBRetailTradeExceptofMotorVehiclesandMotorcyclesRepairofHouseholdgoods':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretailtradeexceptofmotorvehiclesandmotorcyclesrepairofhouseholdgoods',
    'Historic final energy intensity by sector and fuel SUBsolids SUBHotelsandRestaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotelsandrestaurants',
    'Historic final energy intensity by sector and fuel SUBsolids SUBInlandTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinlandtransport',
    'Historic final energy intensity by sector and fuel SUBsolids SUBWaterTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwatertransport',
    'Historic final energy intensity by sector and fuel SUBsolids SUBAirTransport':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subairtransport',
    'Historic final energy intensity by sector and fuel SUBsolids SUBOtherSupportingandAuxiliaryTransportActivitiesActivitiesofTravelAgencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothersupportingandauxiliarytransportactivitiesactivitiesoftravelagencies',
    'Historic final energy intensity by sector and fuel SUBsolids SUBPostandTelecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpostandtelecommunications',
    'Historic final energy intensity by sector and fuel SUBsolids SUBFinancialIntermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancialintermedation',
    'Historic final energy intensity by sector and fuel SUBsolids SUBRealEstateActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrealestateactivities',
    'Historic final energy intensity by sector and fuel SUBsolids SUBRentingodMEqandOtherBusinessActivities':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrentingodmeqandotherbusinessactivities',
    'Historic final energy intensity by sector and fuel SUBsolids SUBPublicAdminandDefenceCompulsorySocialSecurity':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublicadminanddefencecompulsorysocialsecurity',
    'Historic final energy intensity by sector and fuel SUBsolids SUBEducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation',
    'Historic final energy intensity by sector and fuel SUBsolids SUBHealthandSocialWork':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealthandsocialwork',
    'Historic final energy intensity by sector and fuel SUBsolids SUBOtherCommunitySocialandPersonaServices':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subothercommunitysocialandpersonaservices',
    'Historic final energy intensity by sector and fuel SUBsolids SUBPrivateHouseholdswithEmployedPersons':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivatehouseholdswithemployedpersons',
    'initial instal cap RES elec':
    'initial_instal_cap_res_elec',
    'abundance RES elec2':
    'abundance_res_elec2',
    'potential tot generation RES elec TWh':
    'potential_tot_generation_res_elec_twh',
    'RES elec tot overcapacity':
    'res_elec_tot_overcapacity',
    'wear RES elec':
    'wear_res_elec',
    'XCpXini RES elecX SUBhydro':
    'xcpxini_res_elecx_subhydro',
    'XCpXini RES elecX SUBXgeotXelecX':
    'xcpxini_res_elecx_subxgeotxelecx',
    'XCpXini RES elecX SUBXsolidbioEXelecX':
    'xcpxini_res_elecx_subxsolidbioexelecx',
    'XCpXini RES elecX SUBoceanic':
    'xcpxini_res_elecx_suboceanic',
    'XCpXini RES elecX SUBwindonshore':
    'xcpxini_res_elecx_subwindonshore',
    'XCpXini RES elecX SUBwindoffshore':
    'xcpxini_res_elecx_subwindoffshore',
    'XCpXini RES elecX SUBsolarPV':
    'xcpxini_res_elecx_subsolarpv',
    'XCpXini RES elecX SUBCSP':
    'xcpxini_res_elecx_subcsp',
    'G per T':
    'g_per_t',
    'time construction RES elec':
    'time_construction_res_elec',
    'time planification RES elec':
    'time_planification_res_elec',
    'initial required capacity RES elec':
    'initial_required_capacity_res_elec',
    'new RES elec capacity under planning':
    'new_res_elec_capacity_under_planning',
    'lifetime RES elec':
    'lifetime_res_elec',
    'past RES elec capacity growth':
    'past_res_elec_capacity_growth',
    'Afforestation program 2020 GtCO2':
    'afforestation_program_2020_gtco2',
    'Mt per Gt':
    'mt_per_gt',
    'GTL efficiency':
    'gtl_efficiency',
    'CTL efficiency':
    'ctl_efficiency',
    'XCTLXGTL GbX':
    'xctlxgtl_gbx',
    'XMbXd per EJXyearX':
    'xmbxd_per_ejxyearx',
    'max extraction uranium EJ':
    'max_extraction_uranium_ej',
    'Hist growth CTL':
    'hist_growth_ctl',
    'Gboe per EJ':
    'gboe_per_ej',
    'table max extraction uranium user defined':
    'table_max_extraction_uranium_user_defined',
    'XUserXdefined extraction growth unconv gasX':
    'xuserxdefined_extraction_growth_unconv_gasx',
    'XUserXdefined extraction growth unconv oilX':
    'xuserxdefined_extraction_growth_unconv_oilx',
    'Hist growth GTL':
    'hist_growth_gtl',
    'URR uranium User defined':
    'urr_uranium_user_defined',
    'replacement rate nuclear':
    'replacement_rate_nuclear',
    'demand gas for oil refinery gains':
    'demand_gas_for_oil_refinery_gains',
    'share RES vs TPES':
    'share_res_vs_tpes',
    'TPES Mtoe':
    'tpes_mtoe',
    'FE Elec generation from NRE TWh':
    'fe_elec_generation_from_nre_twh',
    'check Transp':
    'check_transp',
    'PEpc consumption people depending on trad biomass':
    'pepc_consumption_people_depending_on_trad_biomass',
    'share gas for oil refinery gains':
    'share_gas_for_oil_refinery_gains',
    'People relying trad biomass ref':
    'people_relying_trad_biomass_ref',
    'PE consumption trad biomass ref':
    'pe_consumption_trad_biomass_ref',
    'TPEFpc threshold high development':
    'tpefpc_threshold_high_development',
    'TPED acceptable standard living':
    'tped_acceptable_standard_living',
    'Pop not dependent on trad biomass':
    'pop_not_dependent_on_trad_biomass',
    'XCarbon footprint tCO2XpersonX':
    'xcarbon_footprint_tco2xpersonx',
    'XAverage TPESpc Xwithout trad biomassXX':
    'xaverage_tpespc_xwithout_trad_biomassxx',
    'XTPES Xwithout trad biomassXX':
    'xtpes_xwithout_trad_biomassxx',
    'Historic electricity consumption':
    'historic_electricity_consumption',
    'Max NPP potential BioE residues for heat and elec':
    'max_npp_potential_bioe_residues_for_heat_and_elec',
    'Max NPP potential BioE residues for cellulosic biofuels':
    'max_npp_potential_bioe_residues_for_cellulosic_biofuels',
    'Total Carbon Emisions':
    'total_carbon_emisions',
    'init CO2 in Atmos ppm':
    'init_co2_in_atmos_ppm',
    'Transfer coefficient for lower level':
    'transfer_coefficient_for_lower_level',
    'Total cumulative emissions GtCO2':
    'total_cumulative_emissions_gtco2',
    'A UO Heat Cap':
    'a_uo_heat_cap',
    'init CO2 in Atmos':
    'init_co2_in_atmos',
    'GtCO2 historic emissions RCPs':
    'gtco2_historic_emissions_rcps',
    'Heat Trans Coeff':
    'heat_trans_coeff',
    'Carbon emissions GtC':
    'carbon_emissions_gtc',
    'Climate equation coefficient for upper level':
    'climate_equation_coefficient_for_upper_level',
    'Historic unconv oil':
    'historic_unconv_oil',
    'gCO2 per MJ unconv gas':
    'gco2_per_mj_unconv_gas',
    'gCO2 per MJ unconv oil':
    'gco2_per_mj_unconv_oil',
    'CO2 emissions CTL':
    'co2_emissions_ctl',
    'gCO2 per MJ conv gas':
    'gco2_per_mj_conv_gas',
    'gCO2 per MJ CTL':
    'gco2_per_mj_ctl',
    'gCO2 per MJ GTL':
    'gco2_per_mj_gtl',
    'Historic unconv gas':
    'historic_unconv_gas',
    'gCO2 per MJ coal':
    'gco2_per_mj_coal',
    'gCO2 per MJ conv oil':
    'gco2_per_mj_conv_oil',
    'gCO2 per MJ shale oil':
    'gco2_per_mj_shale_oil',
    'technological max pct of change':
    'technological_max_pct_of_change',
    'K liquid Elec BEV TWh EJ':
    'k_liquid_elec_bev_twh_ej',
    'HEV fraction of share available':
    'hev_fraction_of_share_available',
    'BEV fraction of share available':
    'bev_fraction_of_share_available',
    'dollars to Tdollars':
    'dollars_to_tdollars',
    'variation historic pop':
    'variation_historic_pop',
    'Historic pop':
    'historic_pop',
    'land compet 2gen vs total land compet':
    'land_compet_2gen_vs_total_land_compet',
    'GJ per EJ':
    'gj_per_ej',
    'Land shifted to biofuels 3gen':
    'land_shifted_to_biofuels_3gen',
    'Average TPES per capita':
    'average_tpes_per_capita',
    'Geot PE potential for heat TWth':
    'geot_pe_potential_for_heat_twth',
    'PE losses uranium for Elec EJ':
    'pe_losses_uranium_for_elec_ej',
    'PE demand Transport after policies Mtoe':
    'pe_demand_transport_after_policies_mtoe',
    'share RES for Elec vs TPE RES':
    'share_res_for_elec_vs_tpe_res',
    'XTPES de Castro PhD X Scen XmadcoalX EJX':
    'xtpes_de_castro_phd_x_scen_xmadcoalx_ejx',
    'extraction coal Mtoe':
    'extraction_coal_mtoe',
    'max extraction coal Mtoe':
    'max_extraction_coal_mtoe',
    'TPE from RES EJ':
    'tpe_from_res_ej',
    'TPE from RES Mtoe':
    'tpe_from_res_mtoe',
    'XTPES de Castro PhD X Scen II EJX':
    'xtpes_de_castro_phd_x_scen_ii_ejx',
    'Electrical distribution losses EJ':
    'electrical_distribution_losses_ej',
    'Efficiency conversion BioE plants to heat':
    'efficiency_conversion_bioe_plants_to_heat',
    'Efficiency improvement biofuels 3gen':
    'efficiency_improvement_biofuels_3gen',
    'Potential PEavail biofuels 2gen land compet EJ':
    'potential_peavail_biofuels_2gen_land_compet_ej',
    'Annual additional historic land use biofuels 2gen':
    'annual_additional_historic_land_use_biofuels_2gen',
    'Max NPP potential bioE residues':
    'max_npp_potential_bioe_residues',
    'Grid reinforcement costs':
    'grid_reinforcement_costs',
    'Balancing costs':
    'balancing_costs',
    'Historic land compet available for biofuels 2gen':
    'historic_land_compet_available_for_biofuels_2gen',
    'Land productivity biofuels 2gen EJ MHa':
    'land_productivity_biofuels_2gen_ej_mha',
    'XMax PEavail potential biofuels 2X3genX':
    'xmax_peavail_potential_biofuels_2x3genx',
    'Balancing costs ref':
    'balancing_costs_ref',
    'Land compet biofuels 2gen Mha':
    'land_compet_biofuels_2gen_mha',
    'efficiency conversion bioE to Elec':
    'efficiency_conversion_bioe_to_elec',
    'Potential PEavail biofuels prod 3gen EJ':
    'potential_peavail_biofuels_prod_3gen_ej',
    'Max PEavail potential bioE residues for cellulosic biofuels':
    'max_peavail_potential_bioe_residues_for_cellulosic_biofuels',
    'P biofuels 3gen':
    'p_biofuels_3gen',
    'Xconstrain gas exogenous growthX delayed 1yrX':
    'xconstrain_gas_exogenous_growthx_delayed_1yrx',
    'Xconstrain gas exogenous growthXX':
    'xconstrain_gas_exogenous_growthxx',
    'Xconstrain liquids exogenous growthX delayed 1yrX':
    'xconstrain_liquids_exogenous_growthx_delayed_1yrx',
    'check liquids delayed 1yr':
    'check_liquids_delayed_1yr',
    'check gas delayed 1yr':
    'check_gas_delayed_1yr',
    'Xconstrain liquids exogenous growthXX':
    'xconstrain_liquids_exogenous_growthxx',
    'check hist RES':
    'check_hist_res',
    'Historic total RES extraction':
    'historic_total_res_extraction',
    'Max potential NPP bioE conventional for heat':
    'max_potential_npp_bioe_conventional_for_heat',
    'Share variable RES elec vs total generation delayed 1yr':
    'share_variable_res_elec_vs_total_generation_delayed_1yr',
    'Xovercapacity vsX intermittent RES penetration 0X':
    'xovercapacity_vsx_intermittent_res_penetration_0x',
    'XTotal GHG Emissions Excluding LandXUse Change and ForestryX':
    'xtotal_ghg_emissions_excluding_landxuse_change_and_forestryx',
    'XTotal GHG Emissions Including LandXUse Change and ForestryX':
    'xtotal_ghg_emissions_including_landxuse_change_and_forestryx',
    'check hist emissions':
    'check_hist_emissions',
    'check hist liquids':
    'check_hist_liquids',
    'check hist natural gas':
    'check_hist_natural_gas',
    'Total CO2 emissions from fossil fuel consumption and cement production':
    'total_co2_emissions_from_fossil_fuel_consumption_and_cement_production',
    'check hist coal':
    'check_hist_coal',
    'Total electrical losses EJ':
    'total_electrical_losses_ej',
    'Gen losses vs PE for elec':
    'gen_losses_vs_pe_for_elec',
    'initial GTL production':
    'initial_gtl_production',
    'variation CTL':
    'variation_ctl',
    'variation GTL':
    'variation_gtl',
    'Historic GTL production':
    'historic_gtl_production',
    'initial CTL production':
    'initial_ctl_production',
    'Historic CTL production':
    'historic_ctl_production',
    'improvement efficiency gas for electricity':
    'improvement_efficiency_gas_for_electricity',
    'PE demand gas Elec plants EJ':
    'pe_demand_gas_elec_plants_ej',
    'PE demand coal Elec plants EJ':
    'pe_demand_coal_elec_plants_ej',
    'Max efficiency gas power plants':
    'max_efficiency_gas_power_plants',
    'Total PE for electricity consumption EJ':
    'total_pe_for_electricity_consumption_ej',
    'initial efficiency gas for electricity':
    'initial_efficiency_gas_for_electricity',
    'remaining efficiency improv gas for electricity':
    'remaining_efficiency_improv_gas_for_electricity',
    'Efficiency improv gas for electricity':
    'efficiency_improv_gas_for_electricity',
    'efficiency gas for electricity':
    'efficiency_gas_for_electricity',
    'percent to share':
    'percent_to_share',
    'XAdditional PE production of CTLXGTL for liquidsX':
    'xadditional_pe_production_of_ctlxgtl_for_liquidsx',
    'Annual additional historic product biofuels 2gen':
    'annual_additional_historic_product_biofuels_2gen',
    'Historic produc biofuels 2gen':
    'historic_produc_biofuels_2gen',
    'NGV adapt growth':
    'ngv_adapt_growth',
    'Historical share NGVs':
    'historical_share_ngvs',
    'variation share NGV':
    'variation_share_ngv',
    'HEV adapt growth':
    'hev_adapt_growth',
    'BEV adapt growth':
    'bev_adapt_growth',
    'carbon budget':
    'carbon_budget',
    'Adapt CO2 emissions unconv gas':
    'adapt_co2_emissions_unconv_gas',
    'Cumulative emissions to 1995':
    'cumulative_emissions_to_1995',
    'variation share BEV':
    'variation_share_bev',
    'variation share HEV':
    'variation_share_hev',
    'XHistorical share BEVXHEVX':
    'xhistorical_share_bevxhevx',
    'FE solar potential for heat':
    'fe_solar_potential_for_heat',
    'Xstart year BioE residues for heatXelecX':
    'xstart_year_bioe_residues_for_heatxelecx',
    'CO2 fossil fuel emissions':
    'co2_fossil_fuel_emissions',
    't per Gt':
    't_per_gt',
    'Additional increase PE demand for Transp Elec EJ':
    'additional_increase_pe_demand_for_transp_elec_ej',
    'Temp change 2C':
    'temp_change_2c',
    'PE demand Transport after policies EJ':
    'pe_demand_transport_after_policies_ej',
    'PE liquids supply for Transp EJ':
    'pe_liquids_supply_for_transp_ej',
    'Share land compet biofuels':
    'share_land_compet_biofuels',
    'share land total RES vs arable':
    'share_land_total_res_vs_arable',
    'share land RES land compet vs arable':
    'share_land_res_land_compet_vs_arable',
    'Year scarcity oil':
    'year_scarcity_oil',
    'CO2 Net Emiss':
    'co2_net_emiss',
    'Xshare gasXXcoal XgasX for ElecX':
    'xshare_gasxxcoal_xgasx_for_elecx',
    'Xfuture share gasXXcoalXgasX for ElecX':
    'xfuture_share_gasxxcoalxgasx_for_elecx',
    'cumulated conv gas extraction':
    'cumulated_conv_gas_extraction',
    'cumulated conv gas extraction to 1995':
    'cumulated_conv_gas_extraction_to_1995',
    'Cumulated unconv gas extraction':
    'cumulated_unconv_gas_extraction',
    'cumulated unconv gas extraction to 1995':
    'cumulated_unconv_gas_extraction_to_1995',
    'Marginal Atmos Retention':
    'marginal_atmos_retention',
    'RURR uranium':
    'rurr_uranium',
    'cumulated uranium extraction to 1995':
    'cumulated_uranium_extraction_to_1995',
    'cumulated conv oil extraction':
    'cumulated_conv_oil_extraction',
    'cumulated conv oil extraction to 1995':
    'cumulated_conv_oil_extraction_to_1995',
    'cumulated unconv oil extraction':
    'cumulated_unconv_oil_extraction',
    'Cumulated coal extraction':
    'cumulated_coal_extraction',
    'cumulated coal extraction to 1995':
    'cumulated_coal_extraction_to_1995',
    'cumulated unconv oil extraction to 1995':
    'cumulated_unconv_oil_extraction_to_1995',
    'Cumulated uranium extraction':
    'cumulated_uranium_extraction',
    'Oil refinery gains EJ':
    'oil_refinery_gains_ej',
    'Oil refinery gains share':
    'oil_refinery_gains_share',
    'Efficiency gas for oil refinery gains':
    'efficiency_gas_for_oil_refinery_gains',
    'PES Liquids EJ':
    'pes_liquids_ej',
    'XCO2 landXuse change emissionsX':
    'xco2_landxuse_change_emissionsx',
    'Temperature change':
    'temperature_change',
    'Temp Diff':
    'temp_diff',
    'CO2 Rad Forcing':
    'co2_rad_forcing',
    'Atmos UOcean Temp':
    'atmos_uocean_temp',
    'Chg A UO Temp':
    'chg_a_uo_temp',
    'Heat Transfer':
    'heat_transfer',
    'Deep Ocean Temp':
    'deep_ocean_temp',
    'Chg DO Temp':
    'chg_do_temp',
    'Climate Sensitivity':
    'climate_sensitivity',
    'DO Heat Cap':
    'do_heat_cap',
    'init Atmos UOcean Temp':
    'init_atmos_uocean_temp',
    'CO2 Rad Force Coeff':
    'co2_rad_force_coeff',
    'Heat Capacity Ratio':
    'heat_capacity_ratio',
    'Other GHG Rad Forcing':
    'other_ghg_rad_forcing',
    'one year':
    'one_year',
    'init Deep Ocean Temp':
    'init_deep_ocean_temp',
    'CO2 Storage':
    'co2_storage',
    'Preindustrial CO2':
    'preindustrial_co2',
    'Average Atmos Retention':
    'average_atmos_retention',
    'CO2 in Atmos':
    'co2_in_atmos',
    'CO2 ppm concentrations':
    'co2_ppm_concentrations',
    'Rate of CO2 Transfer':
    'rate_of_co2_transfer',
    'Land compet required dedicated crops for biofuels':
    'land_compet_required_dedicated_crops_for_biofuels',
    'new biofuels 2gen land compet':
    'new_biofuels_2gen_land_compet',
    'Annual shift from 2gen to 3gen':
    'annual_shift_from_2gen_to_3gen',
    'Land compet biofuels 3gen Mha':
    'land_compet_biofuels_3gen_mha',
    'XP bioE residues for heatXelecX':
    'xp_bioe_residues_for_heatxelecx',
    'Potential PE cellulosic biofuel EJ':
    'potential_pe_cellulosic_biofuel_ej',
    'Cellulosic biofuels available':
    'cellulosic_biofuels_available',
    'XBioE residues for heatXelec availableX':
    'xbioe_residues_for_heatxelec_availablex',
    'XPE bioE residues for heatXelec EJX':
    'xpe_bioe_residues_for_heatxelec_ejx',
    'P cellulosic biofuels':
    'p_cellulosic_biofuels',
    'share cellulosic biofuels vs BioE residues':
    'share_cellulosic_biofuels_vs_bioe_residues',
    'start year cellulosic biofuels':
    'start_year_cellulosic_biofuels',
    'share land total RES vs urban surface':
    'share_land_total_res_vs_urban_surface',
    'Max land compet biofuels 2gen':
    'max_land_compet_biofuels_2gen',
    'P biofuels 2gen':
    'p_biofuels_2gen',
    'past biofuels 2gen':
    'past_biofuels_2gen',
    'adapt growth biofuels 2gen':
    'adapt_growth_biofuels_2gen',
    'EJ per ktoe':
    'ej_per_ktoe',
    'abundance total oil':
    'abundance_total_oil',
    'Cp nuclear initial':
    'cp_nuclear_initial',
    'extraction coal without CTL EJ':
    'extraction_coal_without_ctl_ej',
    'extraction coal for CTL EJ':
    'extraction_coal_for_ctl_ej',
    'Other liquids supply EJ':
    'other_liquids_supply_ej',
    'max offshore wind TWe':
    'max_offshore_wind_twe',
    'URR unconv oil High Mohr15':
    'urr_unconv_oil_high_mohr15',
    'URR unconv oil Low Mohr15':
    'urr_unconv_oil_low_mohr15',
    'URR unconv oil User defined':
    'urr_unconv_oil_user_defined',
    'URR uranium Zittel12':
    'urr_uranium_zittel12',
    'URR conv oil Maggio12 High':
    'urr_conv_oil_maggio12_high',
    'URR conv oil Maggio12 Low':
    'urr_conv_oil_maggio12_low',
    'table max extraction tot agg oil Laherrere 2006':
    'table_max_extraction_tot_agg_oil_laherrere_2006',
    'table max extraction Maggio12 High conv oil EJ':
    'table_max_extraction_maggio12_high_conv_oil_ej',
    'table max extraction Maggio12 Low con oil EJ':
    'table_max_extraction_maggio12_low_con_oil_ej',
    'table max extraction Maggio12middle conv oil EJ':
    'table_max_extraction_maggio12middle_conv_oil_ej',
    'Xseparate conv and unconv gasXX':
    'xseparate_conv_and_unconv_gasxx',
    'URR tot agg oil Laherrere 2006':
    'urr_tot_agg_oil_laherrere_2006',
    'table max extraction unconv oil User defined':
    'table_max_extraction_unconv_oil_user_defined',
    'table max extraction uranium Zittel12':
    'table_max_extraction_uranium_zittel12',
    'URR tot agg oil User defined':
    'urr_tot_agg_oil_user_defined',
    'choose extraction curve tot agg oil':
    'choose_extraction_curve_tot_agg_oil',
    'URR conv oil Maggio12 middle':
    'urr_conv_oil_maggio12_middle',
    'URR conv oil User defined':
    'urr_conv_oil_user_defined',
    'choose extraction tot agg gas curve':
    'choose_extraction_tot_agg_gas_curve',
    'table max extraction unconv oil Low Mohr15':
    'table_max_extraction_unconv_oil_low_mohr15',
    'URR unconv oil BG Mohr15':
    'urr_unconv_oil_bg_mohr15',
    'table max extraction unconv oil High Mohr15':
    'table_max_extraction_unconv_oil_high_mohr15',
    'table max extraction tot agg oil User defined':
    'table_max_extraction_tot_agg_oil_user_defined',
    'table max conv oil extraction User defined':
    'table_max_conv_oil_extraction_user_defined',
    'Xseparate conv and unconv oilXX':
    'xseparate_conv_and_unconv_oilxx',
    'URR uranium EWG13':
    'urr_uranium_ewg13',
    'table max extraction uranium EWG13 EJ':
    'table_max_extraction_uranium_ewg13_ej',
    'kt uranium per EJ':
    'kt_uranium_per_ej',
    'life time nuclear':
    'life_time_nuclear',
    'surface solar PV Mha':
    'surface_solar_pv_mha',
    'Hist share oil Elec':
    'hist_share_oil_elec',
    'Cumulated uranium extraction kt':
    'cumulated_uranium_extraction_kt',
    'extraction uranium kt':
    'extraction_uranium_kt',
    'efficiency uranium for electricity':
    'efficiency_uranium_for_electricity',
    'XHist share gasXXcoal XgasX ElecX':
    'xhist_share_gasxxcoal_xgasx_elecx',
    'Historic nuclear generation TWh':
    'historic_nuclear_generation_twh',
    'invest cost nuclear':
    'invest_cost_nuclear',
    'FE nuclear Elec generation TWh':
    'fe_nuclear_elec_generation_twh',
    'kWh per TWh':
    'kwh_per_twh',
    'Total FE Elec demand EJ':
    'total_fe_elec_demand_ej',
    'gCO2e per KWh biomass':
    'gco2e_per_kwh_biomass',
    'MToe per EJ':
    'mtoe_per_ej',
    'Xshare transmXdistr elec losses initialX':
    'xshare_transmxdistr_elec_losses_initialx',
    'EJ per TWh':
    'ej_per_twh',
    'surface onshore wind Mha':
    'surface_onshore_wind_mha',
    'TWe per TWh':
    'twe_per_twh',
    'initial population':
    'initial_population',
    'Population':
    'population',
    'VE objetive UE2020 extrap':
    've_objetive_ue2020_extrap',
    'VE objetive UE2020':
    've_objetive_ue2020',
    'Choose extraction uranium curve':
    'choose_extraction_uranium_curve',
    'P NGV growth':
    'p_ngv_growth',
    'NGV past share growth':
    'ngv_past_share_growth',
    'XBEVXHEV past share growthX':
    'xbevxhev_past_share_growthx',
    'NGV share':
    'ngv_share',
    'HEV share':
    'hev_share',
    'BEV share':
    'bev_share',
    'Year scarcity TPE':
    'year_scarcity_tpe',
    'Year scarcity uranium':
    'year_scarcity_uranium',
    'Year scarcity coal':
    'year_scarcity_coal',
    'abundance transport':
    'abundance_transport',
    'Year scarcity Transp':
    'year_scarcity_transp',
    'Year scarcity liquids':
    'year_scarcity_liquids',
    'Xeffects shortage elec on BEVXHEVX':
    'xeffects_shortage_elec_on_bevxhevx',
    'effects shortage gas':
    'effects_shortage_gas',
    'NGV fraction of change available':
    'ngv_fraction_of_change_available',
    'Year scarcity gases':
    'year_scarcity_gases',
    'P BEV growth':
    'p_bev_growth',
    'max NGV percent of change':
    'max_ngv_percent_of_change',
    'max percent of change':
    'max_percent_of_change',
    'choose extraction coal curve':
    'choose_extraction_coal_curve',
    'Selection constraint extraction unconv gas':
    'selection_constraint_extraction_unconv_gas',
    'Year scarcity Elec':
    'year_scarcity_elec',
    'extraction unconv gas delayed':
    'extraction_unconv_gas_delayed',
    'max unconv gas growth extraction EJ':
    'max_unconv_gas_growth_extraction_ej',
    'P constraint growth extraction unconv gas':
    'p_constraint_growth_extraction_unconv_gas',
    'choose extraction curve unconv gas':
    'choose_extraction_curve_unconv_gas',
    'choose extraction curve unconv oil':
    'choose_extraction_curve_unconv_oil',
    'total share EV':
    'total_share_ev',
    'P HEV growth':
    'p_hev_growth',
    'Biofuels land compet available':
    'biofuels_land_compet_available',
    'URR oil ASPO':
    'urr_oil_aspo',
    'URR gas Mohr High2013':
    'urr_gas_mohr_high2013',
    'URR gas Leherrere2010':
    'urr_gas_leherrere2010',
    'URR gas Mohr BG2012':
    'urr_gas_mohr_bg2012',
    'RES to fossil accounting':
    'res_to_fossil_accounting',
    'new C GtC':
    'new_c_gtc',
    'year adjust':
    'year_adjust',
    'XTPES de Castro PhD X Scen XmadcoalXX':
    'xtpes_de_castro_phd_x_scen_xmadcoalxx',
    'urban surface 2008':
    'urban_surface_2008',
    'pre industrial value ppm':
    'pre_industrial_value_ppm',
    'Historic TPES':
    'historic_tpes',
    'C per CO2':
    'c_per_co2',
    'Historic coal extraction':
    'historic_coal_extraction',
    'Historic natural gas extraction':
    'historic_natural_gas_extraction',
    'Historic liquids extraction':
    'historic_liquids_extraction',
    'Historic crude oil extraction':
    'historic_crude_oil_extraction',
    'GtC historic emissions RCPs':
    'gtc_historic_emissions_rcps',
    'activate Affores program':
    'activate_affores_program',
    'Adapt emissions shale oil':
    'adapt_emissions_shale_oil',
    'Afforestation program 2020':
    'afforestation_program_2020',
    'Global arable land':
    'global_arable_land',
    'choose extraction curve conv oil':
    'choose_extraction_curve_conv_oil',
    'choose extraction conv gas curve':
    'choose_extraction_conv_gas_curve',
    'Cumulated total monet invest RES for Elec':
    'cumulated_total_monet_invest_res_for_elec',
    'efficiency coal for electricity':
    'efficiency_coal_for_electricity',
    'Historic efficiency gas for electricity':
    'historic_efficiency_gas_for_electricity',
    'efficiency liquids for electricity':
    'efficiency_liquids_for_electricity',
    'Additional land compet available for biofuels':
    'additional_land_compet_available_for_biofuels',
    'max hydro TWe':
    'max_hydro_twe',
    'max oceanic TWe':
    'max_oceanic_twe',
    'max onshore wind TWe':
    'max_onshore_wind_twe',
    'P timeseries pop growth rate':
    'p_timeseries_pop_growth_rate',
    'P CTL':
    'p_ctl',
    'P GTL':
    'p_gtl',
    'ppm per GtC':
    'ppm_per_gtc',
    'start year 3gen':
    'start_year_3gen',
    'Time dmnl':
    'time_dmnl',
    'Share variable RES elec generation vs total':
    'share_variable_res_elec_generation_vs_total',
    'table max extraction ASPO oil EJ':
    'table_max_extraction_aspo_oil_ej',
    'table max extraction gas Laherrere2010':
    'table_max_extraction_gas_laherrere2010',
    'table max extraction gas Mohr High2012':
    'table_max_extraction_gas_mohr_high2012',
    'table max extraction gas Mohr BG2012':
    'table_max_extraction_gas_mohr_bg2012',
    'Total cumulative emissions GtC':
    'total_cumulative_emissions_gtc',
    'XTNES de Castro PhD X Scen IX':
    'xtnes_de_castro_phd_x_scen_ix',
    'XNet oil extraction de Castro PhD X Scen IX':
    'xnet_oil_extraction_de_castro_phd_x_scen_ix',
    'XPE oil extraction de Castro PhD X Scen IIX':
    'xpe_oil_extraction_de_castro_phd_x_scen_iix',
    'XNet oil extraction de Castro PhD X Scen IIIX':
    'xnet_oil_extraction_de_castro_phd_x_scen_iiix',
    'XPE coal extraction de Castro PhD X Scen IIX':
    'xpe_coal_extraction_de_castro_phd_x_scen_iix',
    'XNet gas extraction de Castro PhD X Scen IX':
    'xnet_gas_extraction_de_castro_phd_x_scen_ix',
    'XPE gas extraction de Castro PhD X Scen IIX':
    'xpe_gas_extraction_de_castro_phd_x_scen_iix',
    'XNet gas extraction de Castro PhD X Scen IIIX':
    'xnet_gas_extraction_de_castro_phd_x_scen_iiix',
    'XNet coal extraction de Castro PhD X Scen IIIX':
    'xnet_coal_extraction_de_castro_phd_x_scen_iiix',
    'XNet coal extraction de Castro PhD X Scen IX':
    'xnet_coal_extraction_de_castro_phd_x_scen_ix',
    'XTNES de Castro PhD X Scen IIIX':
    'xtnes_de_castro_phd_x_scen_iiix',
    'XPrimary coal extraction de Castro PhD X Scen XmadcoalXX':
    'xprimary_coal_extraction_de_castro_phd_x_scen_xmadcoalxx',
    'XTPES de Castro PhD X Scen IIX':
    'xtpes_de_castro_phd_x_scen_iix',
    'P nuclear scen 1':
    'p_nuclear_scen_1',
    'XP nuclear 2X3X':
    'xp_nuclear_2x3x',
    'gCO2e per GTCO2e 4':
    'gco2e_per_gtco2e_4',
    'gCO2e per GTCO2e 3':
    'gco2e_per_gtco2e_3',
    'gCO2e per GTCO2e 0':
    'gco2e_per_gtco2e_0',
    'gCO2e per GTCO2e 1':
    'gco2e_per_gtco2e_1',
    'TWh per gCO2e 0':
    'twh_per_gco2e_0',
    'Real demand by sector':
    'real_demand_by_sector',
    'Required total output by sector':
    'required_total_output_by_sector',
    'rate change intensity BOTTOM UP':
    'rate_change_intensity_bottom_up',
    'Evol final energy intensity by sector and fuel':
    'evol_final_energy_intensity_by_sector_and_fuel',
    'RURR unconv oil EJ':
    'rurr_unconv_oil_ej',
    'PE Elec for Transp EJ':
    'pe_elec_for_transp_ej',
    'Oil liquids saved by EVs EJ':
    'oil_liquids_saved_by_evs_ej',
    'Oil liquids saved by NGVs':
    'oil_liquids_saved_by_ngvs',
    'Total FE Elec demand TWh':
    'total_fe_elec_demand_twh',
    'share liquids for final energy':
    'share_liquids_for_final_energy',
    'PED liquids EJ':
    'ped_liquids_ej',
    'FED Heat liquids plants EJ':
    'fed_heat_liquids_plants_ej',
    'Real final energy by sector and fuel':
    'real_final_energy_by_sector_and_fuel',
    'hist var inlandT':
    'hist_var_inlandt',
    'inland transport variation intensity':
    'inland_transport_variation_intensity',
    'max unconv oil growth extraction':
    'max_unconv_oil_growth_extraction',
    'XFED HeatXcom NRE EJX':
    'xfed_heatxcom_nre_ejx',
    'real FE consumption by fuel':
    'real_fe_consumption_by_fuel',
    'Other liquids required EJ':
    'other_liquids_required_ej',
    'FE Elec demand consum after CC impacts EJ':
    'fe_elec_demand_consum_after_cc_impacts_ej',
    'Required FED by liquids EJ':
    'required_fed_by_liquids_ej',
    'Real GFCF':
    'real_gfcf',
    'Real Household demand':
    'real_household_demand',
    'extraction unconv oil EJ':
    'extraction_unconv_oil_ej',
    'Demand conv oil EJ':
    'demand_conv_oil_ej',
    'max unconv oil growth extraction EJ':
    'max_unconv_oil_growth_extraction_ej',
    'XFED HeatXcom EJX':
    'xfed_heatxcom_ejx',
    'FE demand Elec consum TWh':
    'fe_demand_elec_consum_twh',
    'XTotal FED HeatXcom EJX':
    'xtotal_fed_heatxcom_ejx',
    'Real total output by fuel and sector':
    'real_total_output_by_fuel_and_sector',
    'Required final energy by sector and fuel':
    'required_final_energy_by_sector_and_fuel',
    'URR unconv oil':
    'urr_unconv_oil',
    'max extraction unconv gas':
    'max_extraction_unconv_gas',
    'max extraction unconv oil':
    'max_extraction_unconv_oil',
    'max extraction coal EJ':
    'max_extraction_coal_ej',
    'max extraction conv gas EJ':
    'max_extraction_conv_gas_ej',
    'FED Heat gas plants EJ':
    'fed_heat_gas_plants_ej',
    'FED Heat coal plants EJ':
    'fed_heat_coal_plants_ej',
    'URR coal':
    'urr_coal',
    'URR conv gas':
    'urr_conv_gas',
    'Real demand by sector delayed':
    'real_demand_by_sector_delayed',
    'Required final energy other transport':
    'required_final_energy_other_transport',
    'Current trends demand Elec for transport':
    'current_trends_demand_elec_for_transport',
    'PED gases for Heat plants EJ':
    'ped_gases_for_heat_plants_ej',
    'PED oil for Heat plants EJ':
    'ped_oil_for_heat_plants_ej',
    'PED coal for Heat plants EJ':
    'ped_coal_for_heat_plants_ej',
    'Real total output by sector':
    'real_total_output_by_sector',
    'Required final energy air transport':
    'required_final_energy_air_transport',
    'Required final energy inland transport':
    'required_final_energy_inland_transport',
    'Required final energy water transport':
    'required_final_energy_water_transport',
    'table max extraction conv gas Low Mohr15':
    'table_max_extraction_conv_gas_low_mohr15',
    'table max extraction conv gas User defined':
    'table_max_extraction_conv_gas_user_defined',
    'table max extraction total gas BG Mohr12':
    'table_max_extraction_total_gas_bg_mohr12',
    'table max extraction total gas Laherrere10':
    'table_max_extraction_total_gas_laherrere10',
    'table max extraction total gas User defined':
    'table_max_extraction_total_gas_user_defined',
    'table max extraction unconv gas BG Mohr15':
    'table_max_extraction_unconv_gas_bg_mohr15',
    'table max extraction unconv gas High Mohr15':
    'table_max_extraction_unconv_gas_high_mohr15',
    'table max extraction unconv gas Low Mohr15':
    'table_max_extraction_unconv_gas_low_mohr15',
    'table max extraction unconv gas User defined':
    'table_max_extraction_unconv_gas_user_defined',
    'table max extraction coal BG Mohr15':
    'table_max_extraction_coal_bg_mohr15',
    'table max extraction coal High Mohr15':
    'table_max_extraction_coal_high_mohr15',
    'URR coal BG Mohr15':
    'urr_coal_bg_mohr15',
    'URR coal User defined EJ':
    'urr_coal_user_defined_ej',
    'URR conv gas BG Mohr15':
    'urr_conv_gas_bg_mohr15',
    'URR conv gas High Mohr15':
    'urr_conv_gas_high_mohr15',
    'URR conv gas Low Mohr15':
    'urr_conv_gas_low_mohr15',
    'URR conv gas User defined':
    'urr_conv_gas_user_defined',
    'URR total gas Laherrere10':
    'urr_total_gas_laherrere10',
    'URR total gas Mohr12 BG':
    'urr_total_gas_mohr12_bg',
    'URR total gas User defined':
    'urr_total_gas_user_defined',
    'URR unconv gas':
    'urr_unconv_gas',
    'URR unconv gas BG Mohr15':
    'urr_unconv_gas_bg_mohr15',
    'table max extraction coal Low Mohr15':
    'table_max_extraction_coal_low_mohr15',
    'table max extraction coal Mohr2012 EJ':
    'table_max_extraction_coal_mohr2012_ej',
    'table max extraction coal User defined':
    'table_max_extraction_coal_user_defined',
    'table max extraction conv gas BG Mohr15':
    'table_max_extraction_conv_gas_bg_mohr15',
    'table max extraction conv gas High Mohr15':
    'table_max_extraction_conv_gas_high_mohr15',
    'URR unconv gas Low Mohr15':
    'urr_unconv_gas_low_mohr15',
    'URR coal High15':
    'urr_coal_high15',
    'URR coal Low Mohr15':
    'urr_coal_low_mohr15',
    'URR unconv gas High Mohr15':
    'urr_unconv_gas_high_mohr15',
    'URR unconv gas User defined':
    'urr_unconv_gas_user_defined',
    'URR coal Mohr2012 EJ':
    'urr_coal_mohr2012_ej',
    'table max extraction unconv oil BG Mohr15':
    'table_max_extraction_unconv_oil_bg_mohr15',
    'Selection constraint extraction unconv oil':
    'selection_constraint_extraction_unconv_oil',
    'Abundance electricity':
    'abundance_electricity',
    'P constraint growth extraction unconv oil':
    'p_constraint_growth_extraction_unconv_oil',
    'extraction unconv oil delayed':
    'extraction_unconv_oil_delayed',
    'FINAL TIME':
    'final_time',
    'INITIAL TIME':
    'initial_time',
    'SAVEPER':
    'saveper',
    'TIME STEP':
    'time_step'
}


#######################################
model_name = 'model'

# default values
time_step = 0.125       
initial_time = 1995 
final_time = 2050
return_timestep = None
silent = False
run_params = {}
vensim_data_folder = 'vensim_data'
scenario_sheet = 'PythonScenario'  #default

# adds parameter from command line to pysd.run()
def update_params(param_name, param):
    if param is not None:
        run_params[param_name] = param

def name_for_run_params(pyname):
    for key, value in _namespace.items():
        if value == pyname:
            return key
    else:
        print('Not found in _namespace function name for ', pyname)
        return ''

# get command line parameters: 
# -h (help), -s (silent mode), -o MODEL (open model), -i (initial time), -f (final time), -t (time step)
try:
    opts, args = getopt.getopt(sys.argv[1:],"hsf:t:i:o:u:r:")

    for opt, arg in opts:
        if opt == '-h':
            print('run.py -i <initial_time> -f <final_time> -t <time_step> -r <return_timestep>\n        -o <open model> -s <silent mode> -u <update scenario to sheet name in Excel>')
            sys.exit()
        elif opt in ("-s", "--silent"):
            silent = True
        elif opt in ("-i", "--initial_time"):
            initial_time = float(arg)
            update_params('INITIAL TIME', initial_time)
        elif opt in ("-f", "--final_time"):
            final_time = float(arg)        
            update_params('FINAL TIME', final_time)
        elif opt in ("-t", "--time_step"):
            time_step = float(arg)
            update_params('TIME STEP', time_step)        
        elif opt in ("-o", "--open"):
            model_name = arg if arg[-3:].strip() != '.py' else arg[:-3]
        elif opt in ("-u", "--update_scenario"):
            scenario_sheet = arg
        elif opt in ("-r", "--return_timestep"):
            return_timestep = arg
        else:
            pass
except getopt.GetoptError:
    print('default config')


# check if Excel file was updated

# auxiliary vars for Excel update
error_texts = []
model_py_file = model_name + '.py'
updated_lookups = set()

# the first (0) row skipped
# rows and colomns 0-based: [0, 1, 2, 3....]
df1 = pd.read_excel(io='inputs.xlsx', sheet_name=scenario_sheet, header=None, skiprows=[0])
df2 = pd.read_excel(io='inputs_default.xlsx', sheet_name='PythonScenario', header=None, skiprows=[0])


#NaN if unchanged, or new value if changed
df = df1[df1.ne(df2)]

def make_py_name(stri):
    # replaces non-standard symbols->X, space->_
    replace_symbols = r'-%?()+\"'
    new_symbol = 'X'
    stri = stri.strip().lower().replace(' ', '_')
    for sym in replace_symbols:
        stri = stri.replace(sym, new_symbol)
    return stri

def make_py_index(stri):
    # replaces non-standard symbols->X
    replace_symbols = r'-%?()+\"\''
    new_symbol = 'X'
    for sym in replace_symbols:
        stri = stri.replace(sym, new_symbol)
    return stri


with open('model_default.txt', 'r', encoding='utf-8') as f:
    model_mdl = f.read()

with open(model_py_file, 'r', encoding='utf-8') as f:
    model_py = f.read()

# make the dictionary if indices {index_name:[index_values],...}
# might be needed if we input vector/matrix GET XLS CONSTANT
# not always working correctly!!!
pattern = re.compile(r'\|\s*(?P<index_name>[^~:\|=]*?):(?P<index_values>[^~:=]*?)~\s*~')
indices = {}
for m in re.finditer(pattern, model_mdl):    
    index_values = [make_py_index(x.strip(' \n\t\r\\')) for x in m.group('index_values').lower().split(',')]
    indices[make_py_index(m.group('index_name').lower().strip(' \n\t\r\\'))] = index_values

# makes the list of updated cells in the form:
# elem_id+2=row_index_in_xlsx, when row[elem_id]=[idx_1, idx_2,...]
lst = []
for _, row in df.iterrows():
    mask = row.notnull()
    lst += [row[mask].index.tolist()]


cells_updated = []
for idx_row, value in enumerate(lst):
    if value:  # if value =! [], i.e. there are updated cells in the row
        for idx_col in value:#idx_col = int(value[0].split(' ')[-1]) - 1  # gives the column index, since value='Something: column_index'. -1 since it's zero-based
            cell_str = str(xlw.utility.xl_col_to_name(idx_col)) + str(idx_row + 2)
            cell_contains = df1.iloc[idx_row, idx_col]
            try: 
                _ = float(cell_contains) # if it's not a number, proceed to the next 
            except:
                continue
            cells_updated += [(cell_str, cell_contains)]
            #inp = input(str(cells_updated[-1]))

pattern_gen = lambda x: re.compile(r"\|(?P<funcname>[^\|=~]*?)\[(?P<index>[^\]=~]*)\][^~]*?'BAU',[\s\\]*'{}'".format(x))

if cells_updated:
    print('Updating scenario...')
old_params = {}
run_params1 = {}
for cell in cells_updated:
    cell_str, value = cell
    idx_row, idx_col = xlw.utility.xl_cell_to_rowcol(cell_str) # 
    pattern = pattern_gen(cell_str)
    found_cell_by_pattern = False
    for m in re.finditer(pattern, model_mdl):
        found_cell_by_pattern = True
        index_str = m.group('index').strip().lower()
        if index_str == 'bau':
            funcname_py = make_py_name(m.group('funcname'))
            run_params[name_for_run_params(funcname_py.lower())] = float(value)
            run_params1[funcname_py.lower()] = float(value)
        elif 'bau' in index_str:
            inp = input('input of vector/matrix constant, pattern group:', m.group(0))
            indices_list = [x.strip() for x in index_str.split(',')]
            dims = []
            for given_idx in indices_list:
                if given_idx in list(indices.keys()):
                    dims.append(indices[given_idx])
        else:
            error_msg = 'WARNING: Cell {} in inputs.xlsx has not been updated to {}, m.group(0)={}, due to the internal error, report this.'.format(cell_str, str(value), m.group(0))
            print(error_msg)
            error_texts.append(error_msg)
            inp = input('Press any key to continue...')

    if not found_cell_by_pattern:
        for idx_seek in range(idx_col - 1, -1, -1):
            try:
                _ = float(df1.iloc[idx_row - 1, idx_seek])
            except:
                # here we find where the data row starts, given some cell in the middle
                cell_str_lookup = xlw.utility.xl_col_to_name(idx_seek + 1) + str(idx_row + 1)
                if cell_str_lookup in updated_lookups:
                    found_cell_by_pattern = True
                    break
                else:
                    updated_lookups.add(cell_str_lookup)
                pattern_lookup = re.compile(r"\|(?P<funcname>[^\|=~]*?)\[(?P<index>[^\]=~,]*)\][^~]*?'BAU',[\s\\]*'(?P<arg_row>\d+)',[\s\\]*'{}'".format(cell_str_lookup))
                for ml in re.finditer(pattern_lookup, model_mdl):
                    found_cell_by_pattern = True
                    ml_funcname = make_py_name(ml.group('funcname').strip().lower())
                    pattern_lookup_in_py = re.compile(r"def {}[\s\S]*?\[[^\]]*\][\s\S]*?\[\s*(?P<data>[^\]]*?)\s*\]".format(ml_funcname))
                    for mp in re.finditer(pattern_lookup_in_py, model_py):
                        old_data = mp.group('data')
                        old_data_len = len(old_data.split(','))
                        new_data = ',\n        '.join([str(x) for x in list(df1.iloc[idx_row - 1, idx_seek + 1 : idx_seek + 1 + old_data_len])])
                        new_group = mp.group(0).replace(old_data, new_data) 
                        model_py = model_py.replace(mp.group(0), new_group)
                break
        if not found_cell_by_pattern:        
            error_msg = 'WARNING: Cell {} in inputs.xlsx cannot be updated to {}, report this.'.format(cell_str, str(value))
            print(error_msg)
            error_texts.append(error_msg)
        #inp = input('Press any key to continue...')                 

# direct inline replacement just in case
for func, val in run_params1.items():
    model_py = re.sub(r'(def {}[\s\S]*?=\s*)\S*'.format(func), r'\1' + ' ' + str(val), model_py)


if cells_updated:
    print('Updated values:')
    for key, value in run_params.items():
        print(key, value)

    # save error log
    with open('excel_updater_error_log.txt', 'a', encoding='utf-8') as f:
        f.write('\n==========\n' + '\n'.join(error_texts))


    print('Done')

# save updated model or make a copy if not updated
with open(model_py_file[:-3] + '_updated.py', 'w', encoding='utf-8') as f:
    f.write(model_py)


# end of Excel checking
#inp = input(run_params)

# update name not to interfere with the original file
model_name = model_name + '_updated'
model_py_file = model_name + '.py'

'''
# check if Excel file was updated
df1 = pd.read_excel(io='inputs.xlsx', sheet_name='PythonScenario')
df2 = pd.read_excel(io='inputs_default.xlsx', sheet_name='PythonScenario')

df = df1[df1 != df2]

def make_py_name(stri):
    replace_symbols = r'-%?()+'
    new_symbol = 'X'
    stri = stri.strip().lower().replace(' ', '_')
    for sym in replace_symbols:
        stri = stri.replace(sym, new_symbol)
    return stri

with open('model_default.txt', 'r', encoding='utf-8') as f:
    model_mdl = f.read()

lst = []
for _, row in df.iterrows():
    mask = row.notnull()
    lst += [row[mask].index.tolist()]

excel_updated = False
cells_updated = []
for idx, value in enumerate(lst):
    if value:
        idx2 = int(value[0].split(' ')[-1])
        cell_str = str(xlw.utility.xl_col_to_name(idx2)) + str(idx + 2)
        cells_updated += [(cell_str, df1.iloc[idx, idx2])]
        excel_updated = True


if excel_updated:
    for cell in cells_updated:
        cell_str = cell[0]
        value = cell[1]
        pattern = re.compile(r"\|(?P<funcname>[^\|=]*?)\[(?P<index>[^\]=]*)\]=[^=]*'BAU', '{}'".format(cell_str))
        for m in re.finditer(pattern, model_mdl):
            if m.group('index').strip().lower() != 'bau':
                print('Cell {} in inputs.xlsx has not been updated to {} due to the internal error, report this.'.format(cell_str, str(value)))
                inp = input('Press any key to continue...')
            else:
                try:
                    funcname_py = make_py_name(m.group('funcname'))
                    run_params[funcname_py] = float(value)
                    inp = input('----------\nValue in cell {} is updated to {}'.format(cell_str, str(value)))
                except:
                    print('Cell {} in inputs.xlsx cannot be updated to {}, report this.'.format(cell_str, str(value)))
                    inp = input('Press any key to continue...')                 
# end of Excel checking
'''

'''
# keeps flush data from vensim simulations in its own format
if vensim_considered
vensim_files = [x for x in os.listdir(
    vensim_data_folder) if x.endswith(".txt")]

VENSIM_COLUMNS = [x[:-4].lower() for x in vensim_files]  # make param from param.txt
COLUMNS = VENSIM_COLUMNS
'''

# auxiliary vensim data tuple (x, y) generator
def vensim_tuple1(file=None, omit=1):
    with open(file, 'r', encoding='utf-8') as f:
        s = f.readline().split('\t')
        xs = [float(x) for x in s[1:-1]]
        for i in range(omit):
            f.readline()
        ys = [float(x) for x in f.readline().split('\t')[1:1 + len(xs)]]
    return (xs, ys)


def test_vensim_tuple():
    assert type(vensim_tuple1(file='run.py')) == tuple

#######################################
# here we choose the output

print('=================================', flush=True)
available_columns_file = model_name + '.py'
try:
    with open(available_columns_file, 'r', encoding='utf-8') as f:
        modelpy = f.read()
except:
    modelpy = 'VARIABLES NOT ACCESSIBLE'

all_columns = []
pattern = re.compile(r"@cache\('step'\)\s*def ([\w\_]*)\(\)")
pattern_lookups = re.compile(r"[\w\_]+\(x\)")

lookups = []
lookups = [x[:-3] for x in pattern_lookups.findall(modelpy)]
    
# make the list of 'step'-cached (i.e. functions of time) vars and offer to pick
i = 0
for m in re.finditer(pattern, modelpy):
    funcname = m.group(1).strip()
    if not funcname.startswith('_'):
        i += 1
        all_columns.append(funcname)
        print(i, ': ', m.group(1), flush=True)
if silent:
    col_ind = 'r'
else:
    print("================\nEnter column indices (ex: 1,4,5), 0 for all\nr to repeat the last\nyour vars (comma-separated)\n+var1,var2 to add to last conf")

    while True:
        col_ind = input("\t>")
        if col_ind.strip() == '':
            continue

        col_ind_split = [x.strip(' \t\n\b+') for x in col_ind.split(',')]
        if any(x in col_ind_split for x in ['r', '0']):
            break
        col_indices = [int(x) - 1 for x in col_ind_split if x.isdigit()]
        col_vars = [x for x in col_ind_split if not x.isdigit()]


        try:
            return_columns_set = set(list(map(all_columns.__getitem__, col_indices)) + col_vars)
        except IndexError:
            print('\t\tWrong numerical index, try again...')
            continue

        # check if the variable names are correct
        if return_columns_set.issubset(all_columns):
            break
        else:
            print('\t\tWrong variable name, try again...')


col_ind_split = [x.strip(' \t\n\b+') for x in col_ind.split(',')]

col_indices = [int(x) - 1 for x in col_ind_split if x.isdigit()]
col_vars = [x for x in col_ind_split if not x.isdigit()]

return_columns_set = set(list(map(all_columns.__getitem__, col_indices)) + col_vars)

if -1 in col_indices:
    return_columns = all_columns
elif 'r' in col_vars:
    try:
        with open('last_output_conf.txt', 'r', encoding='utf-8') as f:
            return_columns = [x.strip() for x in f.readline().split(",")]
    except:
        print("Warning: no last configuration, take all vars")
        return_columns = all_columns
elif col_ind.lstrip().startswith('+'):    
    try:
        with open('last_output_conf.txt', 'r', encoding='utf-8') as f:
            return_columns_set.update([x.strip() for x in f.readline().split(",")])
    except:
        print("Warning: no last configuration, take the given vars only")
    return_columns = sorted(list(return_columns_set))
else:
    return_columns = sorted(list(return_columns_set))

COLUMNS = all_columns

print('------------\nTo return:\n------------')
print('\t' + '\n\t'.join(return_columns), flush=True)

with open('last_output_conf.txt', 'w', encoding='utf-8') as f:
    f.write(",".join(return_columns))

print('------------\nThese columns are saved as the last configuration to last_output_conf.txt\n------------')

#######################################

'''
# this option is temporarily turned off
while True:
    if silent:
        toparse = 'n'
    else:
        toparse = input("Parse? [y/n]")
        
    if toparse.lower() == 'n':
        pysdfunc = pysd.load
        suffix = '.py'
        print("Running existing model: ", model_name+suffix)
        break
    elif toparse.lower() == 'y':
        pysdfunc = pysd.read_vensim
        suffix = '.mdl'
        print("Parsing->running Vensim model: ", model_name+suffix)
        break
#################################
'''
# uncomment this if the above is commented
pysdfunc = pysd.load
suffix = '.py'
print("================\nRunning existing model: ", model_name+suffix)
#################################
 

model = pysdfunc(model_name+suffix)


# saving aux info for the plot legend
if cells_updated:
    scenario_info = scenario_sheet
else:
    scenario_info = 'BAU'
return_timestep_info = return_timestep or str(time_step)  
header = '#{},{},{}'.format(scenario_info, str(time_step), str(return_timestep_info))


if not silent:
    inp = input('Model parsed and loaded with header\n{}\npress any key to run...'.format(header))
'''
if not return_timestep is None:
    model.set_components({'return_timestamps':range(1995,2020)})

if return_columns[0] != '':
    stocks = model.run(return_columns=return_columns, params=run_params)
else:
    stocks = model.run(params=run_params)
'''
# running parameters
kwargs = {'params' : run_params}
if return_columns[0] != '':
    kwargs['return_columns'] = return_columns
if not return_timestep is None:
    kwargs['return_timestamps'] = np.arange(initial_time, final_time + 0.01, float(return_timestep))


stocks = model.run(**kwargs)
stocks.index.name = 'time'


def criterion(a, b):
    return 0. if b==0. else (a/b-1.)     


datalog_access_mode = 'w'
if os.path.isfile('data.log'):
    print('Copy data1.log to data2.log')
    try:
        move('data1.log', 'data2.log')
    except:
        print('Unknown error occurred while move(data1.log, data2.log)')

    print('Copy data.log to data1.log')
    try:
        copyfile('data.log', 'data1.log')
    except IOError:
        print('Unable to make a copy, appending to data.log. Destination not writeable?')
        datalog_access_mode = 'a'
    except:
        print('Unknown error occurred while copyfile(data.log, data1.log)')
        raise

with open('data.log', datalog_access_mode, encoding='utf-8') as f:

    f.write(header + '\ntime\t'+'\t'.join([str(x) for x in stocks.index.values]))
    for column in stocks:
        sty = stocks[column].values
        try:
            new_stock = xr.concat(stocks[column].values, stocks[column].index)
            dim = new_stock.dims[1]  # works only if the stock is 1d
            for coord in new_stock.coords[dim].values:            
                f.write('\n' + '{}[{}]'.format(column, coord) + '\t'+'\t'.join([str(x) for x in new_stock.loc[:, coord].values]))
        except:
            f.write('\n' + column + '\t'+'\t'.join([str(x) for x in stocks[column]]))


import subprocess
cmd = 'python plot_tool.py'

p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
